package ServiceFactory;

use strict;
use warnings;
use MicroService;
use IO::Socket qw(AF_INET SOCK_STREAM);
use Server;


# Method to create a microservice instance
# Arguments:
#   port       - The port number for the server to listen on (optional, default: 3401).
#   host       - The host address the server will bind to (optional, default: '0.0.0.0').
#   reuse_port - Whether the server should reuse the port (optional, default: 1).
#   listen     - The maximum number of pending connections (optional, default: 5).
#   domain     - The socket domain (optional, default: AF_INET for IPv4).
#   type       - The socket type (optional, default: SOCK_STREAM for TCP).
# Returns:
#   A Microservice object configured with the specified server settings.
sub create_microservice {
    my ($class, %args) = @_;


    #my $server;
    #if ($args{type} eq 'http') {
    #    $server = HTTPServer->new(%args);
    #} elsif ($args{type} eq 'tcp') {
    #    $server = TCPServer->new(%args);
    #} elsif ($args{type} eq 'udp') {
    #    $server = UDPServer->new(%args);
    #} else {
    #    die "Unknown server type: $args{type}";
    #}

    my $server = Server->new(
        port       => $args{port},
        host       => $args{host} || '0.0.0.0',    # Default host if not specified
        reuse_port => $args{reuse_port} || 1,      # Default reuse port setting
        listen     => $args{listen} || 5,          # Default listen queue size
        domain     => $args{domain} || AF_INET,    # Default domain (IPv4)
        type       => $args{type} || SOCK_STREAM,  # Default socket type (TCP)
    );
    
    return MicroService->new(server => $server);
}

1;
