import { ALERT_CONST, ALERT_ITEM_TYPE, ALERT_LEVEL } from '@const/alerting.constants';
import { IMetric, Item } from './architecture.dto';
import { BasicResponseDTO } from './basicResponse.dto';
import { ItemGroup, ItemGroupWithLabels } from './customGroup.dto';
import { MailGroup } from './mailGroup.dto';
import { UserDTO } from './usersResponse.dto';
import { AlertIntegrations } from './elements/alert.elements';
export type AlertConst = typeof ALERT_CONST[keyof typeof ALERT_CONST];
export type AlertLevel = typeof ALERT_LEVEL[keyof typeof ALERT_LEVEL];
export type AlertItemType = typeof ALERT_ITEM_TYPE[keyof typeof ALERT_ITEM_TYPE];
interface IAlertGroup {
    alerting_group_id?: string;
    class?: string;
    hw_type?: string;
    subsystem?: string;
    custom_group_id?: string;
    label: string;
    disabled: boolean;
    created_by?: UserDTO;
}
export declare class CreateAlertGroupDTO extends ItemGroup implements IAlertGroup {
    alerting_group_id?: string;
    class: string;
    hw_type?: string;
    subsystem?: string;
    custom_group_id?: string;
    label: string;
    alerts: SaveAlert[];
    disabled: boolean;
    group_id?: number;
}
export declare class AlertGroup extends ItemGroupWithLabels implements IAlertGroup {
    alerting_group_id: string;
    class?: string;
    hw_type?: string;
    subsystem?: string;
    custom_group_id?: string;
    label: string;
    alerts: GetAlert[];
    disabled: boolean;
    created_by?: UserDTO;
    group?: {
        group_id: number;
        name: string;
    };
}
export declare class AlertBase {
    limit: number;
    peak: number;
    repeat: number;
    exclude_start: number;
    exclude_end: number;
    mail_groups: MailGroup[];
    level?: AlertLevel | string;
    integrations: AlertIntegrations;
}
export declare class Alert extends AlertBase {
    alerting_group_id: string;
    alerting_id: string;
    triggered: number;
}
export declare class SaveAlert extends Alert {
    metric: string;
}
export declare class GetAlert extends Alert {
    metric: IMetric;
}
export declare class AlertsResponseDTO extends BasicResponseDTO {
    data: AlertGroup[];
}
export declare class AlertResponseDTO extends BasicResponseDTO {
    data: Alert;
}
export declare class AlertHistory {
    alerting_history_id: string;
    alerting_group_id: string;
    alerting_group_label: string;
    group_label?: string;
    group_id?: number;
    metric: IMetric;
    limit: number;
    level: string;
    targets: string[];
    items: Item[];
    triggered: string;
}
export declare class AlertHistoryDTO extends BasicResponseDTO {
    data: AlertHistory[];
}
export {};
