import { BasicResponseDTO } from './basicResponse.dto';
import { ItemGroup, ParentRegex, ParentTag } from './customGroup.dto';
import { IMetricType, UnitPrefix } from './pageResponse.dto';
import { Unit } from './subsystemMetricsResponse.dto';
export declare class DeleteArchitecture {
    hostcfg_id: string;
}
export declare class Architecture {
    item_id: string;
    label: string;
    object_id: string;
    hw_type: string;
    subsystem: string;
    parents?: string[];
    children?: string[];
    agent_mapping?: string;
    hostcfg_id?: string;
    wwn?: string;
    wwns?: string[];
    target_wwn: string;
    target_wwns: string[];
}
export interface IHwTypeBase {
    class?: string;
    hw_type: string;
    label: string;
    active?: boolean;
    allowed?: boolean;
}
export declare class Subsystem {
    subsystem: string;
    label: string;
    hw_type?: string;
    class?: string;
    items?: Item[];
}
export declare class Class {
    label: string;
    class: string;
    active?: boolean;
}
export declare class ItemIds {
    item_ids: string[];
}
export declare class Labels {
    [key: string]: string;
}
export declare class Item {
    item_id: string;
    label: string;
    hw_type?: string;
    _hw_type?: {
        label: string;
    };
    subsystem?: string;
    _subsystem?: {
        label: string;
    };
    class?: string;
    _class?: {
        label: string;
    };
    parents?: Item[];
    root_parent?: Item;
    url?: string;
    hostcfg_id?: string;
    subsystem_detail?: Subsystem;
}
export declare class RegexPreviewDTO {
    class?: string;
    hw_type?: string;
    device?: string;
    regex?: string | string[];
    subsystem: string;
    tags?: string[];
    parents?: {
        intersection: boolean;
        tags?: ParentTag[];
        item_ids?: string[];
        regex?: ParentRegex[];
    };
    linuxExclude?: boolean;
}
export declare class ItemGroupPreview extends ItemGroup {
    class?: string;
    hw_type?: string;
    subsystem: string;
    linuxExclude?: boolean;
}
export interface SearchResult {
    metadata: {
        start: number;
        end: number;
        max: number;
        page: number;
        info?: string;
    };
    items: Item[];
}
export interface IMetric extends IMetricType {
    metric: string;
    label: string;
    longLabel?: string;
    unit: Unit;
    shortcut: string;
    defaultPrefix?: UnitPrefix;
    defaultPrecision?: number;
    metricType?: string;
    unitLabel?: string;
}
export interface IDefaultSubsystem {
    label?: string;
    excluded?: {
        all?: string[];
        alerting?: string[];
        customGroup?: string[];
        reporter?: string[];
        exporter?: string[];
    };
    metrics?: {
        [metric: string]: IMetric;
    };
}
export interface IStatusDefinition {
    explicit?: string;
    calculated?: string[];
}
export interface ITechnology {
    [hw_type: string]: {
        order?: number;
        label?: string;
        subsystems?: {
            [subsystem: string]: IDefaultSubsystem;
        };
        status?: IStatusDefinition;
    };
}
export interface IClass {
    class: string;
    hwTypes: ITechnology;
}
export interface Property {
    property_name: string;
    label: string;
}
export declare class ItemGroupMetricRequest extends ItemGroup {
    class?: string;
    hw_type?: string;
    subsystem: string;
}
export declare class ItemGroupConfigurationRequest extends ItemGroupMetricRequest {
}
export declare class LabelsResponseDTO extends BasicResponseDTO {
    data: Labels;
}
export declare class PropsResponseDTO extends BasicResponseDTO {
    data: Property[];
}
export declare class ItemsResponseDTO extends BasicResponseDTO {
    data: Item[];
}
export declare class ItemAliasesResponseDTO extends BasicResponseDTO {
    data: Array<Item & {
        alias: string;
    }>;
}
export declare class ItemsSearchResponseDTO extends BasicResponseDTO {
    data: SearchResult;
}
export declare class HwTypesResponseDTO extends BasicResponseDTO {
    data: IHwTypeBase[];
}
export declare class SubsystemsResponseDTO extends BasicResponseDTO {
    data: Subsystem[];
}
export declare class ClassesResponseDTO extends BasicResponseDTO {
    data: Class[];
}
export declare class MetricsResponseDTO extends BasicResponseDTO {
    data: IMetric[];
}
export declare class ArchitectureResponseDTO extends BasicResponseDTO {
    data: IClass[];
}
export declare class VendorKeysResponseDTO extends BasicResponseDTO {
    data: string[];
}
