import { BasicResponseDTO } from './basicResponse.dto';
export declare class BackendService {
    label: string;
    microservice: string;
    active: boolean;
    version: string;
    script?: string;
    pid?: number;
    monitor?: {
        memory_byte: number;
        cpu_percent: number;
    };
}
interface Tables {
    total: number;
    hw_types: {
        [key: string]: {
            total: number;
            subsystems: {
                [key: string]: {
                    total: number;
                    tables: {
                        [key: string]: number;
                    };
                };
            };
        };
    };
}
export declare class ILicenseTechnology {
    technology: string;
    expirationDate: string;
    limit: number;
}
export declare class BackendDTO {
    backend: {
        free: boolean;
        version: string;
        node_version: string;
        demo?: boolean;
        production?: boolean;
        docker?: boolean;
        appliance?: string;
        premium?: {
            valid: boolean;
            expirationDate?: string;
            technologies: ILicenseTechnology[];
            isTrial?: boolean;
        };
        limits?: HostcfgLimits;
        cert?: {
            crt: string;
            key: string;
        };
        firstLogin?: boolean;
    };
    database: {
        server: string;
        port: number;
        database: string;
        version: string;
        tables?: Tables;
    };
}
export declare class Devices {
    Server: {
        [key: string]: {
            VM: number;
        };
    };
    Storage: {
        [key: string]: {
            Device: number;
            Volume: number;
            Pool: number;
        };
    };
    SAN: {
        [key: string]: {
            Device: number;
            Port: number;
        };
    };
    LAN: {
        [key: string]: {
            Device: number;
            Port: number;
        };
    };
    Database: {
        [key: string]: {
            DB: number;
        };
    };
    Container: {
        [key: string]: {
            Cluster?: number;
            Pod?: number;
            Container?: number;
        };
    };
    Cloud: {
        [key: string]: {
            VM: number;
        };
    };
}
export declare class HostcfgLimits {
    [_class: string]: {
        limit?: number;
        hwTypes?: {
            [hwType: string]: {
                limit?: number;
                types?: {
                    [type: string]: number;
                };
            };
        };
    };
}
export declare class LicenseLimit {
    technology: string;
    label: string;
    limit: number;
    publicAPI: boolean;
    subsystem: string;
    expiration: string;
    expired: boolean;
}
export declare class BackendServiceResponseDTO extends BasicResponseDTO {
    data: BackendService;
}
export declare class BackendServicesResponseDTO extends BasicResponseDTO {
    data: BackendService[];
}
export declare class BackendResponseDTO extends BasicResponseDTO {
    data: BackendDTO;
}
export declare class DevicesResponseDTO extends BasicResponseDTO {
    data: Devices;
}
export declare class ServicesResponseDTO extends BasicResponseDTO {
    data: BackendService[];
}
export declare class TablesResponseDTO extends BasicResponseDTO {
    data: Tables;
}
export declare class HostcfgLimitsResponseDTO extends BasicResponseDTO {
    data: HostcfgLimits;
}
export declare class LicenseLimitsResponseDTO extends BasicResponseDTO {
    data: LicenseLimit[];
}
export {};
