import type { MetricValueTypeField, TopInterval, TopOrderByField } from '../constants/reportItem.constants';
import { Box } from '../pageResponse.dto';
export declare class BaseDataElement {
    interval?: TopInterval | null;
    time_range?: TimeRange | null;
}
export declare class TimeRange {
    date_from: Date;
    date_to: Date;
}
export declare class SingleMetricGraphDataElement extends BaseDataElement {
    metric: string;
    stacked?: boolean;
    maxItems?: number;
}
export declare class MetricGraphDataElement extends BaseDataElement {
    metrics: string[];
    stacked?: boolean;
}
export declare class MetricTableDataElement extends BaseDataElement {
    metrics: string[];
    order_by: MetricValueTypeField;
}
export declare class ConfigurationTableDataElement extends BaseDataElement {
    properties_key?: string[];
    property_keys?: string[];
    tablePerItem: boolean;
}
export declare class CustomGroupTableDataElement {
    metric: string;
    custom_group_id: string;
}
export declare class HealtStatusTableDataElement {
    subsystem: string;
    item_ids?: string[];
    regex?: string[];
    parent_ids?: string[];
}
export declare class SingleTopTableDataElement extends BaseDataElement {
    metric: string;
    order_by?: TopOrderByField;
}
export declare class TopTableDataElement extends BaseDataElement {
    metrics: string[];
    order_by?: TopOrderByField;
}
export declare class TextDataElement {
    text: string;
    style?: TextStyle;
}
export type textAlignment = 'left' | 'center' | 'right' | 'justify';
export type textDecoration = 'underline' | 'lineThrough' | 'overline';
export type textDecorationStyle = 'dashed' | 'dotted' | 'double' | 'wavy';
export type colorString = string;
export type TextStyle = TextStyleElement | "h1" | "h2" | "h3" | "h4" | null;
export declare class TextStyleElement {
    fontSize?: number | null;
    lineHeight?: number | null;
    bold?: boolean | null;
    italics?: boolean | null;
    alignment?: textAlignment | null;
    characterSpacing?: number | null;
    color?: colorString | null;
    background?: colorString | null;
    decoration?: textDecoration | null;
    decorationStyle?: textDecorationStyle | null;
    decorationColor?: colorString | null;
    marginTop?: number | null;
    marginRight?: number | null;
    marginBottom?: number | null;
    marginLeft?: number | null;
}
export type ReportItemDataElement = TextDataElement | MetricGraphDataElement | MetricTableDataElement | ConfigurationTableDataElement | HealtStatusTableDataElement | TopTableDataElement | Box;
export type ReportItemDataSingleElement = TextDataElement | SingleMetricGraphDataElement | MetricTableDataElement | ConfigurationTableDataElement | HealtStatusTableDataElement | TopTableDataElement | Box;
