import { EXPORT_FORMAT, EXPORT_TYPE, MAIL_FREQUENCY, OPERATOR, TOP_INTERVAL } from '@const/exporter.constants';
import { BasicResponseDTO } from './basicResponse.dto';
import { ItemGroup, ItemGroupWithLabels } from './customGroup.dto';
export type ExportFormat = typeof EXPORT_FORMAT[keyof typeof EXPORT_FORMAT];
export type ExportType = typeof EXPORT_TYPE[keyof typeof EXPORT_TYPE];
export type TopInterval = typeof TOP_INTERVAL[keyof typeof TOP_INTERVAL];
export type Operator = typeof OPERATOR[keyof typeof OPERATOR];
export type MailFrequency = typeof MAIL_FREQUENCY[keyof typeof MAIL_FREQUENCY];
export declare class MetricFilter {
    operator: Operator;
    value: number;
    metric?: string;
}
export declare class TypeFilter {
    hw_type: string;
    subsystem: string;
}
export declare class ConfigurationFilter {
    [key: string]: string;
}
export declare class JsonResponse {
    metadata: {
        total: number;
        interval: number;
    };
    items: any[];
}
export declare class SaveExport extends ItemGroup {
    class?: string;
    hw_type?: string;
    subsystem?: string;
    custom_group_id?: string;
    label: string;
    format: ExportFormat;
    type: ExportType;
    data?: ExportTimeseriesRequestDTO | ExportConfigurationRequestDTO;
    recurrence?: Recurrence;
}
export declare class Export extends ItemGroupWithLabels {
    class?: string;
    hw_type?: string;
    subsystem?: string;
    custom_group_id?: string;
    export_id: string;
    last_run: string;
    created: string;
    label: string;
    format: ExportFormat;
    type: ExportType;
    data?: ExportTimeseriesRequestDTO | ExportConfigurationRequestDTO;
    recurrence?: Recurrence;
}
export declare class Recurrence {
    rrule?: string;
    freq: MailFrequency;
    weekday?: string[];
    month?: number[];
    setpos?: number;
    month_day?: number;
    year_day?: number;
    week_number?: number;
    mail?: string;
    file?: boolean;
}
export declare class ExporterHistoryResponseDTO extends BasicResponseDTO {
    data: Export[];
}
export declare class ExporterDetailResponseDTO extends BasicResponseDTO {
    data: Export | {
        hw_type?: string;
        subsystem?: string;
    };
}
export declare class ExportBase extends ItemGroup {
    class?: string;
    hw_type?: string;
    subsystem?: string;
    custom_group_id?: string;
    uuids?: string[];
}
export declare class PublicExportTimeseriesRequestDTO extends ExportBase {
    start: number;
    end: number;
    metric: string[];
    format?: ExportFormat;
}
export declare class PublicExportMappingRequestDTO extends ExportBase {
    format?: ExportFormat;
}
export declare class PublicExportConfigurationRequestDTO extends ExportBase {
    properties: string[];
    format?: ExportFormat;
}
export declare class PublicExportCapacityRequestDTO extends ExportBase {
    format?: ExportFormat;
}
export declare class ExportTimeseriesRequestDTO extends ExportBase {
    interval?: boolean;
    start?: number;
    end?: number;
    metric?: string[];
    metricFilter?: MetricFilter;
    configuration?: ConfigurationFilter;
}
export declare class ExportMappingRequestDTO extends ExportBase {
}
export declare class ExportCapacityRequestDTO extends ExportBase {
}
export declare class ExportConfigurationRequestDTO extends ExportBase {
    properties?: string[];
}
export declare class ExportValuesRequestDTO {
    headers: string[];
    values: Array<string | number>[];
}
export declare class ExportTopRequestDTO {
    uuids: string[];
    metric: string;
    interval: TopInterval;
}
export declare class ExportMetricsRequestDTO {
    hw_type: string;
    subsystem: string;
}
