"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MailTestDTO = exports.SettingsResponseDTO = exports.SettingKey = void 0;
const openapi = require("@nestjs/swagger");
const basicResponse_dto_1 = require("./basicResponse.dto");
const util_1 = require("./util");
class GlobKeyTypes {
    constructor() {
        this.SNMP_TRAP_HOST = '';
        this.SNMP_COMMUNITY = '';
        this.TOKEN_EXPIRATION = 0;
        this.TOP_ITEMS = 0;
        this.VMWARE_SDK = 0;
        this.PUBLIC_URL = '';
        this.DATA_RETENTION = 365;
        this.REMOVE_FROM_MENU = 30;
    }
    static _OPENAPI_METADATA_FACTORY() {
        return { SNMP_TRAP_HOST: { required: true, type: () => Object, default: "" }, SNMP_COMMUNITY: { required: true, type: () => Object, default: "" }, TOKEN_EXPIRATION: { required: true, type: () => Object, default: 0 }, TOP_ITEMS: { required: true, type: () => Object, default: 0 }, VMWARE_SDK: { required: true, type: () => Object, default: 0 }, PUBLIC_URL: { required: true, type: () => Object, default: "" }, DATA_RETENTION: { required: true, type: () => Object, default: 365 }, REMOVE_FROM_MENU: { required: true, type: () => Object, default: 30 }, LKEY: { required: true, type: () => String } };
    }
}
class AlertingKeyTypes {
    constructor() {
        this.NAGIOS = false;
        this.IS_SCRIPT = false;
        this.SCRIPT = '';
        this.INCLUDE_GRAPHS = 0;
        this.REPEAT_TIME = 0;
        this.PEAK_TIME = 0;
        this.XORMON_URL = '';
        this.ALARMONE_URL = '';
        this.SEND_PER_ITEM = false;
    }
    static _OPENAPI_METADATA_FACTORY() {
        return { NAGIOS: { required: true, type: () => Object, default: false }, IS_SCRIPT: { required: true, type: () => Object, default: false }, SCRIPT: { required: true, type: () => Object, default: "" }, INCLUDE_GRAPHS: { required: true, type: () => Object, default: 0 }, REPEAT_TIME: { required: true, type: () => Object, default: 0 }, PEAK_TIME: { required: true, type: () => Object, default: 0 }, XORMON_URL: { required: true, type: () => Object, default: "" }, ALARMONE_URL: { required: true, type: () => Object, default: "" }, SEND_PER_ITEM: { required: true, type: () => Object, default: false } };
    }
}
class SmtpKeyTypes {
    constructor() {
        this.HOST = '';
        this.USER = '';
        this.PASS = '';
        this.PORT = '';
        this.ENCRYPTION = '';
        this.MAIL_FROM = '';
    }
    static _OPENAPI_METADATA_FACTORY() {
        return { HOST: { required: true, type: () => Object, default: "" }, USER: { required: true, type: () => Object, default: "" }, PASS: { required: true, type: () => Object, default: "" }, PORT: { required: true, type: () => Object, default: "" }, ENCRYPTION: { required: true, type: () => Object, default: "" }, MAIL_FROM: { required: true, type: () => Object, default: "" } };
    }
}
class ServicenowKeyTypes {
    constructor() {
        this.INSTANCE = '';
        this.USER = '';
        this.PASS = '';
        this.TYPE = '';
        this.CLASS = '';
    }
    static _OPENAPI_METADATA_FACTORY() {
        return { INSTANCE: { required: true, type: () => Object, default: "" }, USER: { required: true, type: () => Object, default: "" }, PASS: { required: true, type: () => Object, default: "" }, TYPE: { required: true, type: () => Object, default: "" }, CLASS: { required: true, type: () => Object, default: "" } };
    }
}
class CertKeyTypes {
    constructor() {
        this.KEY = '';
        this.CRT = '';
    }
    static _OPENAPI_METADATA_FACTORY() {
        return { KEY: { required: true, type: () => Object, default: "" }, CRT: { required: true, type: () => Object, default: "" } };
    }
}
class LdapKeyTypes {
    constructor() {
        this.TIMEOUT_SEC = 0;
        this.PAGE_SIZE = 0;
    }
    static _OPENAPI_METADATA_FACTORY() {
        return { TIMEOUT_SEC: { required: true, type: () => Object, default: 0 }, PAGE_SIZE: { required: true, type: () => Object, default: 0 } };
    }
}
const ALERTING_ = 'ALERTING_';
const GLOB_ = 'GLOB_';
const SMTP_ = 'SMTP_';
const CERT_ = 'CERT_';
const LDAP_ = 'LDAP_';
const SERVICENOW_ = 'SERVICENOW_';
class SettingKey {
    constructor(prefix, keyTypes) {
        this.PREFIX = prefix;
        this.keyTypes = keyTypes;
    }
    getType(propertyName) {
        return typeof this.keyTypes[propertyName.substring(propertyName.startsWith(this.PREFIX) ? this.PREFIX.length : 0)];
    }
    get(propertyName) {
        return this.PREFIX + propertyName;
    }
    reType(settings) {
        Object.entries(settings).forEach(([key, value]) => {
            if (value != null) {
                switch (this.getType(key)) {
                    case 'boolean':
                        settings[key] = (0, util_1.parseBool)(value);
                        break;
                    case 'number':
                        settings[key] = parseInt(value);
                        break;
                }
            }
        });
    }
    static _OPENAPI_METADATA_FACTORY() {
        return { PREFIX: { required: true, type: () => String }, keyTypes: { required: true } };
    }
}
exports.SettingKey = SettingKey;
SettingKey.ALERTING = new SettingKey(ALERTING_, new AlertingKeyTypes());
SettingKey.GLOB = new SettingKey(GLOB_, new GlobKeyTypes());
SettingKey.SMTP = new SettingKey(SMTP_, new SmtpKeyTypes());
SettingKey.LDAP = new SettingKey(LDAP_, new LdapKeyTypes());
SettingKey.SERVICENOW = new SettingKey(SERVICENOW_, new ServicenowKeyTypes());
class SettingsResponseDTO extends basicResponse_dto_1.BasicResponseDTO {
    static _OPENAPI_METADATA_FACTORY() {
        return { data: { required: true, type: () => Object } };
    }
}
exports.SettingsResponseDTO = SettingsResponseDTO;
class MailTestDTO {
    static _OPENAPI_METADATA_FACTORY() {
        return { mail: { required: true, type: () => String } };
    }
}
exports.MailTestDTO = MailTestDTO;
//# sourceMappingURL=settings.dto.js.map