import { HostcfgLimits } from "@xormon/common/backendResponse.dto";
import { HostCfg } from "@xormon/entity/hostcfg.entity";
import { HostCfgDTO } from "@xormon/common/hostCfgResponse.dto";
import { HwTypeKey } from "@xormon/common/constants/hwTypeKey.constants";
interface ILicenseTechnology {
    technology: string;
    expirationDate: string;
    limit: number;
    maxDeviceCount?: number;
    publicAPI: boolean;
}
export interface ILicenseData {
    sign: string;
    verified: boolean;
    isTrial?: boolean;
    expirationDate?: string;
    technologies: ILicenseTechnology[];
}
export interface ILicenseInfo {
    valid: boolean;
    expirationDate?: string;
    isTrial: boolean;
    technologies: ILicenseTechnology[];
}
export interface CheckedTechnologies {
    [hwType: string]: boolean;
}
export declare class License {
    private static crypto;
    private static fs;
    private static PUB_KEY_PATH;
    static LICENSE_PATH: string;
    static DAYS_AFTER_EXPIRATION: number;
    static licenseTypes: {
        storage: {
            hard: boolean;
            subsystem: string;
        };
        san: {
            hard: boolean;
            subsystem: string;
        };
        lan: {
            hard: boolean;
            subsystem: string;
        };
        virtualization: {
            hard: boolean;
            subsystem: string;
        };
        database: {
            hard: boolean;
            subsystem: string;
        };
        cloud: {
            hard: boolean;
            subsystem: string;
        };
        container: {
            hard: boolean;
            subsystem: string;
        };
    };
    static check(licenseFile: string): Promise<ILicenseData>;
    static data(): Promise<ILicenseData>;
    private static getLicenseFile;
    private static getPubKeyFile;
    static getTechnologies(apiCheck?: boolean): Promise<string[]>;
    static getTechnologiesWithLimits(): Promise<[HostcfgLimits, boolean]>;
    static checkSubTypesLimit(licenseObject: HostcfgLimits): HostcfgLimits;
    static rewriteTypes(licenseObject: HostcfgLimits): HostcfgLimits;
    static addToLicenseObject(licenseObject: HostcfgLimits, _class: string, limit: number, hwType?: string, subType?: string): HostcfgLimits;
    static checkTechnology(hwType: HwTypeKey | string, apiCheck?: boolean): Promise<boolean>;
    static checkTechnologies(hwTypes: string[], apiCheck?: boolean): Promise<CheckedTechnologies>;
    static hasAnyValid(): Promise<boolean>;
    static getHostcfgLimits(): HostcfgLimits;
    static getLimits(): Promise<HostcfgLimits>;
    static checkHostcfgLimits(hostcfgs: HostCfg[]): Promise<HostCfgDTO[]>;
    static isSubType(technology: string): boolean;
    static applyDemoLicense(): Promise<void>;
}
export {};
