"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArchitectureInterceptor = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const aclResponse_constants_1 = require("../common/constants/aclResponse.constants");
const user_entity_1 = require("../entity/user.entity");
const ACLCheck_1 = require("../service/acl/ACLCheck");
const User_1 = require("../service/users/User");
const ApiKey_1 = require("../service/public/ApiKey");
const operators_1 = require("rxjs/operators");
const typeorm_2 = require("typeorm");
const dataSource_1 = require("../config/dataSource");
const adminGroup_entity_1 = require("../entity/adminGroup.entity");
const adminGroups_constants_1 = require("../common/constants/adminGroups.constants");
let ArchitectureInterceptor = class ArchitectureInterceptor {
    constructor(usersRepository) {
        this.usersRepository = usersRepository;
    }
    async intercept(context, next) {
        const request = context.switchToHttp().getRequest();
        const response = context.switchToHttp().getResponse();
        const caller = context.getHandler().name;
        let user;
        const url = response.req.url;
        const tmpPath = url.split('/');
        if (tmpPath && tmpPath.length > 2 && tmpPath[2] === 'public') {
            user = await ApiKey_1.ApiKey.getUserByRequest(request, true);
        }
        else {
            user = await User_1.User.getUserByRequest(request, true);
        }
        const groupIds = user.groups.map(group => group.group_id);
        return next
            .handle()
            .pipe((0, operators_1.map)(async (data) => {
            if (groupIds.includes(1)) {
                for (let item of data.data) {
                    item.allowed = true;
                }
                return data;
            }
            const items = data.data;
            let filteredItems = [];
            const tmpUrl = new URL('https://do.not.care' + url);
            if (caller === 'getClasses') {
                const skipAcl = tmpUrl.searchParams.get('skip_acl');
                const classes = items.map((item) => item.class);
                const filteredClasses = skipAcl ? classes : await ACLCheck_1.ACLCheck.filterClasses(groupIds, classes, false);
                for (const item of items) {
                    if (filteredClasses.includes(item.class)) {
                        filteredItems.push(item);
                    }
                }
            }
            else if (caller === 'hw_types') {
                const adminGroups = tmpUrl.searchParams.get('admin_groups');
                if (adminGroups === 'true') {
                    const rulesRepository = (0, dataSource_1.getDataSource)().getRepository(adminGroup_entity_1.AdminGroup);
                    for (let groupId of groupIds) {
                        for (let item of data.data) {
                            if (item.allowed) {
                                continue;
                            }
                            let found = false;
                            const classRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: groupId }, type: adminGroups_constants_1.RULE_TYPE.CLASS } });
                            for (let rule of classRules) {
                                if (rule.value == item.class) {
                                    item.allowed = true;
                                    found = true;
                                    break;
                                }
                            }
                            if (found) {
                                continue;
                            }
                            const hwTypeRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: groupId }, type: adminGroups_constants_1.RULE_TYPE.HWTYPE } });
                            for (let rule of hwTypeRules) {
                                if (rule.value == item.hw_type) {
                                    item.allowed = true;
                                    break;
                                }
                            }
                        }
                    }
                }
                const skipAcl = tmpUrl.searchParams.get('skip_acl');
                const hwTypes = items.map((item) => item.hw_type);
                const filteredHwTypes = skipAcl ? hwTypes : await ACLCheck_1.ACLCheck.filterHwTypes(groupIds, hwTypes, false);
                for (const item of items) {
                    if (filteredHwTypes.includes(item.hw_type)) {
                        filteredItems.push(item);
                    }
                }
            }
            else if ((caller === 'getSubsystems') || (caller === 'subsystems')) {
                const skipAcl = tmpUrl.searchParams.get('skip_acl');
                const classes = items.map((item) => item.class);
                const filteredClasses = skipAcl ? classes : await ACLCheck_1.ACLCheck.filterClasses(groupIds, classes, false);
                const hwTypes = items.map((item) => item.hw_type);
                const filteredHwTypes = skipAcl ? hwTypes : await ACLCheck_1.ACLCheck.filterHwTypes(groupIds, hwTypes, false);
                for (const item of items) {
                    if (filteredClasses.includes(item.class) || filteredHwTypes.includes(item.hw_type)) {
                        filteredItems.push(item);
                    }
                }
            }
            else if (caller === 'items') {
                const _class = tmpUrl.searchParams.get('cls');
                const hwType = ((_class !== 'san') && (_class !== 'lan')) ? tmpUrl.searchParams.get('hw_type') : undefined;
                const subsys = tmpUrl.searchParams.get('subsystem');
                const strictAcl = tmpUrl.searchParams.get('strict_acl');
                const strict = strictAcl ? (strictAcl === 'true') : true;
                const itemIds = items.map((item) => item.item_id);
                const filteredIds = await ACLCheck_1.ACLCheck.filterItemsByRulesTopDown(groupIds, itemIds);
                filteredItems = items.filter((item) => filteredIds.includes(item.item_id));
                if (!strict) {
                    for (let item of items) {
                        if (filteredIds.includes(item.item_id)) {
                            continue;
                        }
                        const itemStatus = await ACLCheck_1.ACLCheck.getItemStatus(groupIds, item.item_id, false);
                        if (itemStatus !== aclResponse_constants_1.ITEM_STATUS.denied) {
                            filteredItems.push(item);
                        }
                    }
                }
            }
            else if (caller === 'parentsByHwTypeAndSubsystem') {
                const hwTypeParam = tmpUrl.searchParams.get('hw_type');
                const hwType = ((hwTypeParam !== 'san') && (hwTypeParam !== 'lan')) ? hwTypeParam : undefined;
                const hwTypeStatus = await ACLCheck_1.ACLCheck.getHwTypeStatus(groupIds, hwType, false);
                if (hwTypeStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                    filteredItems = items;
                }
                else {
                    for (let item of items) {
                        const itemStatus = item.item_id
                            ? await ACLCheck_1.ACLCheck.getItemStatus(groupIds, item.item_id, false)
                            : await ACLCheck_1.ACLCheck.isAnySubsysItemGranted(request, item.subsystem, item.hw_type, undefined, false);
                        if (itemStatus !== aclResponse_constants_1.ITEM_STATUS.denied) {
                            filteredItems.push(item);
                        }
                    }
                }
            }
            else if (caller === 'regex' || caller === 'multiRegex' || caller === "preview" || caller === "multiPreview") {
                const strictAcl = tmpUrl.searchParams.get('strict_acl');
                const strict = strictAcl ? (strictAcl === 'true') : true;
                const itemIds = items.map((item) => item.item_id);
                const filteredIds = await ACLCheck_1.ACLCheck.filterItemsByRulesTopDown(groupIds, itemIds);
                filteredItems = items.filter((item) => filteredIds.includes(item.item_id));
                if (!strict) {
                    for (let item of items) {
                        if (filteredIds.includes(item.item_id)) {
                            continue;
                        }
                        const itemStatus = await ACLCheck_1.ACLCheck.getItemStatus(groupIds, item.item_id, false);
                        if (itemStatus !== aclResponse_constants_1.ITEM_STATUS.denied) {
                            filteredItems.push(item);
                        }
                    }
                }
            }
            return {
                statusCode: data.statusCode,
                data: filteredItems
            };
        }));
    }
};
exports.ArchitectureInterceptor = ArchitectureInterceptor;
exports.ArchitectureInterceptor = ArchitectureInterceptor = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(user_entity_1.User)),
    __metadata("design:paramtypes", [typeorm_2.Repository])
], ArchitectureInterceptor);
//# sourceMappingURL=architecture.interceptor.js.map