"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HealthStatusInterceptor = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const aclResponse_constants_1 = require("../common/constants/aclResponse.constants");
const user_entity_1 = require("../entity/user.entity");
const ACLCheck_1 = require("../service/acl/ACLCheck");
const User_1 = require("../service/users/User");
const ApiKey_1 = require("../service/public/ApiKey");
const operators_1 = require("rxjs/operators");
const typeorm_2 = require("typeorm");
let HealthStatusInterceptor = class HealthStatusInterceptor {
    constructor(usersRepository) {
        this.usersRepository = usersRepository;
    }
    async intercept(context, next) {
        const request = context.switchToHttp().getRequest();
        const response = context.switchToHttp().getResponse();
        const caller = context.getHandler().name;
        let user;
        const url = response.req.url;
        const tmpPath = url.split('/');
        if (tmpPath && tmpPath.length > 2 && tmpPath[2] === 'public') {
            user = await ApiKey_1.ApiKey.getUserByRequest(request, true);
        }
        else {
            user = await User_1.User.getUserByRequest(request, true);
        }
        const groupIds = user.groups.map(group => group.group_id);
        return next
            .handle()
            .pipe((0, operators_1.map)(async (data) => {
            if (groupIds.includes(1)) {
                return data;
            }
            const items = data.data;
            let filteredItems = [];
            for (let item of items) {
                let filteredDevices = [];
                for (let device of item.devices) {
                    let itemStatus;
                    if (device.item_id) {
                        itemStatus = await ACLCheck_1.ACLCheck.getItemStatus(groupIds, device.item_id, false);
                    }
                    else {
                        itemStatus = await ACLCheck_1.ACLCheck.getHwTypeStatus(groupIds, device.hw_type, true);
                    }
                    if (itemStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                        filteredDevices.push(device);
                    }
                }
                if (filteredDevices.length > 0) {
                    let tmpItem = Object.assign({}, item);
                    tmpItem.devices = filteredDevices;
                    filteredItems.push(tmpItem);
                }
            }
            return {
                statusCode: data.statusCode,
                data: filteredItems
            };
        }));
    }
};
exports.HealthStatusInterceptor = HealthStatusInterceptor;
exports.HealthStatusInterceptor = HealthStatusInterceptor = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(user_entity_1.User)),
    __metadata("design:paramtypes", [typeorm_2.Repository])
], HealthStatusInterceptor);
//# sourceMappingURL=healthStatus.interceptor.js.map