"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var MailService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MailService = void 0;
const common_1 = require("@nestjs/common");
const settings_dto_1 = require("../common/settings.dto");
const PM2_1 = require("../service/configuration/PM2");
const Settings_1 = require("../service/configuration/Settings");
const Format_1 = require("../service/log/Format");
const winston = require("winston");
let MailService = MailService_1 = class MailService {
    constructor() {
        this.backendLogger = new common_1.Logger(MailService_1.name);
        this.winstonLogger = winston.createLogger({
            format: winston.format.combine(winston.format.timestamp(), Format_1.logFormat),
            transports: [
                new winston.transports.File({ filename: 'logs/mailer-error.log', level: 'error' }),
                new winston.transports.File({ filename: 'logs/mailer.log' }),
            ],
        });
        this.fs = require('fs');
        this.path = require('path');
        this.nodemailer = require('nodemailer');
    }
    async handleTransportStatus() {
        const settings = await Settings_1.Settings.get({
            prefix: 'SMTP'
        });
        if (this.isTransportSetUp()) {
            if (settings['SMTP_HOST'] === undefined || settings['SMTP_HOST'].length <= 1) {
                this.destroyTransport();
            }
            else if (this.transporter.host !== settings['SMTP_HOST']) {
                this.refreshTransport();
            }
        }
        else {
            if (settings['SMTP_HOST'] !== undefined || settings['SMTP_HOST'].length > 1) {
                this.refreshTransport();
            }
        }
    }
    isTransportSetUp() {
        if (this.transporter !== undefined) {
            return true;
        }
        return false;
    }
    async onModuleInit() {
        if (this.transporter === undefined) {
            await this.refreshTransport();
        }
        process.on('message', (data) => {
            const parsed = PM2_1.PM2.parseMessage(data);
            if (parsed.action === 'refresh-transport') {
                this.refreshTransport(true);
            }
        });
    }
    async destroyTransport() {
        this.transporter = undefined;
    }
    async verifyTransport() {
        const settings = await Settings_1.Settings.get({
            prefix: 'SMTP'
        });
        if (this.transporter === undefined) {
            if (global.transporter) {
                this.transporter = global.transporter;
            }
            else if (settings['SMTP_HOST'] && settings['SMTP_HOST'].length >= 1) {
                this.transporter = this.nodemailer.createTransport({
                    host: settings['SMTP_HOST'],
                    port: +settings['SMTP_PORT'] || 465,
                    secure: settings['SMTP_ENCRYPTION'] && settings['SMTP_ENCRYPTION'].length >= 1
                        ? settings['SMTP_ENCRYPTION'] === 'tls'
                            ? false
                            : settings['SMTP_ENCRYPTION']
                        : false,
                    auth: settings['SMTP_USER'] && settings['SMTP_PASS'] && settings['SMTP_PASS'].length >= 1 ? {
                        user: settings['SMTP_USER'],
                        pass: settings['SMTP_PASS'],
                    } : undefined,
                    tls: {
                        rejectUnauthorized: false,
                    }
                });
                global.transporter = this.transporter;
            }
            else {
                throw new Error('SMTP is not configured! \nConfigure in Settings -> Application -> SMTP');
            }
        }
        return new Promise((resolve, reject) => {
            this.transporter.verify((error, success) => {
                if (error) {
                    if (error.toString().includes("ECONNREFUSED")) {
                        reject({
                            message: `Connection refused: ${settings['SMTP_HOST']}:${settings['SMTP_PORT']}`,
                            error: "Connection refused"
                        });
                    }
                    else {
                        reject(error);
                    }
                }
                else {
                    resolve();
                }
            });
        });
    }
    async refreshTransport(change = false) {
        const settings = await Settings_1.Settings.get({
            prefix: 'SMTP'
        });
        if (!change && global.transporter) {
            this.transporter = global.transporter;
            return;
        }
        try {
            if (settings['SMTP_HOST'] && settings['SMTP_HOST'].length >= 1) {
                this.transporter = this.nodemailer.createTransport({
                    host: settings['SMTP_HOST'],
                    port: +settings['SMTP_PORT'] || 465,
                    secure: settings['SMTP_ENCRYPTION'] && settings['SMTP_ENCRYPTION'].length >= 1
                        ? settings['SMTP_ENCRYPTION'] === 'tls'
                            ? false
                            : settings['SMTP_ENCRYPTION']
                        : false,
                    auth: settings['SMTP_USER'] && settings['SMTP_PASS'] && settings['SMTP_PASS'].length >= 1 ? {
                        user: settings['SMTP_USER'],
                        pass: settings['SMTP_PASS'],
                    } : undefined,
                    tls: {
                        rejectUnauthorized: false
                    }
                });
                global.transporter = this.transporter;
                this.transporter.verify((error, success) => {
                    if (error) {
                        this.backendLogger.error("[MailVerify] error: " + error);
                        this.transporter = undefined;
                    }
                    else {
                        this.backendLogger.log('[MailInit] connected to SMTP');
                    }
                });
            }
            else {
                this.transporter = undefined;
            }
        }
        catch (e) {
            this.backendLogger.error("[MailInit] error: " + e);
        }
    }
    async send(mails, subject, data, files = undefined, repeated = false, plainText) {
        if (mails.length < 1) {
            throw new Error('SMTP no recipients defined');
        }
        if (this.transporter !== undefined) {
            let attachments = [];
            if (files) {
                for (const file of files) {
                    attachments.push({
                        path: file,
                        contentDisposition: "attachment"
                    });
                }
            }
            try {
                const mailFromKey = settings_dto_1.SettingKey.SMTP.get('MAIL_FROM');
                const settings = await Settings_1.Settings.get({ keys: [mailFromKey] });
                const mailFrom = settings[mailFromKey] ? settings[mailFromKey] : undefined;
                await this.transporter.sendMail({
                    from: mailFrom,
                    to: mails,
                    subject: subject,
                    html: data,
                    text: plainText ? plainText : undefined,
                    attachments: attachments.length >= 1 ? attachments : undefined
                });
                this.winstonLogger.log('info', "[" + subject + "] to " + mails.toString());
            }
            catch (e) {
                console.log(e);
                this.winstonLogger.log('error', e);
                throw new Error('SMTP is not configured!');
            }
        }
        else {
            if (!repeated) {
                await this.refreshTransport();
                return this.send(mails, subject, data, files, true);
            }
            throw new Error('SMTP is not configured!');
        }
    }
};
exports.MailService = MailService;
exports.MailService = MailService = MailService_1 = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [])
], MailService);
//# sourceMappingURL=mail.service.js.map