"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDataSource = exports.createDataSource = exports.databaseConfiguration = void 0;
const typeorm_1 = require("typeorm");
const process_1 = require("process");
const Crypto_1 = require("../service/configuration/Crypto");
const databaseConfiguration = async () => {
    require('dotenv').config({ override: true });
    return {
        type: 'postgres',
        host: process_1.env['DB_HOST'],
        port: +process_1.env['DB_PORT'],
        username: process_1.env['DB_USERNAME'],
        password: await Crypto_1.Crypto.decrypt(process_1.env['DB_PASSWORD']),
        database: process_1.env['DB_DATABASE'],
        entities: ['dist/entity/**/*.entity.js'],
        subscribers: ['dist/subscriber/objectItems.subscriber.js', 'dist/subscriber/healthStatus.subscriber.js'],
        synchronize: false,
    };
};
exports.databaseConfiguration = databaseConfiguration;
const createDataSource = async () => {
    global.dataSource = await (new typeorm_1.DataSource({
        ...await (0, exports.databaseConfiguration)(),
        type: 'postgres',
    })).initialize();
};
exports.createDataSource = createDataSource;
const getDataSource = () => { return global.dataSource; };
exports.getDataSource = getDataSource;
//# sourceMappingURL=dataSource.js.map