"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addServerGroupPrimaryColumn1718878382856 = void 0;
const itemRelation_entity_1 = require("../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const serverGroup_entity_1 = require("../../entity/serverGroup.entity");
const serverGroupItems_entity_1 = require("../../entity/serverGroupItems.entity");
const typeorm_1 = require("typeorm");
class addServerGroupPrimaryColumn1718878382856 {
    constructor() {
        this.name = 'addServerGroupPrimaryColumn1718878382856';
    }
    async up(queryRunner) {
        await queryRunner.addColumn("server_group_items", new typeorm_1.TableColumn({
            name: "is_primary",
            type: "boolean",
            default: true
        }));
        const items = await queryRunner.manager.getRepository(serverGroupItems_entity_1.ServerGroupItems).find();
        const itemsWithChildren = await queryRunner.manager.getRepository(objectItem_entity_1.ObjectItem).find({
            select: {
                item_id: true,
                children: true
            },
            where: {
                item_id: (0, typeorm_1.In)(items.map(i => i.item_id))
            },
            relations: {
                children: true
            },
            withDeleted: true
        });
        const map = {};
        for (const item of itemsWithChildren) {
            map[item.item_id] = item;
        }
        for (const item of items) {
            item._item = map[item.item_id];
        }
        const serverGroupAsOitToSave = [];
        const serverGroups = await queryRunner.manager.getRepository(serverGroup_entity_1.ServerGroup).find();
        for (const serverGroup of serverGroups) {
            if (serverGroup.hw_type === "linux") {
                serverGroupAsOitToSave.push({
                    item_id: serverGroup.server_group_id,
                    hw_type: "linux",
                    subsystem: "group",
                    object_id: "linux",
                    label: serverGroup.label
                });
            }
            else {
                serverGroupAsOitToSave.push({
                    item_id: serverGroup.server_group_id,
                    hw_type: "lancisco",
                    subsystem: "group",
                    object_id: "lancisco",
                    label: serverGroup.label
                });
            }
        }
        await queryRunner.manager.getRepository(objectItem_entity_1.ObjectItem).save(serverGroupAsOitToSave);
        const itemsToAdd = [];
        const relationsToAdd = [];
        for (const sgItem of items) {
            for (const child of sgItem._item.children) {
                itemsToAdd.push({
                    item_id: child.child,
                    server_group_id: sgItem.server_group_id,
                    isPrimary: false
                });
                relationsToAdd.push({
                    child: child.child,
                    parent: sgItem.server_group_id,
                    origin: itemRelation_entity_1.RelationOrigin.parent
                });
            }
        }
        await queryRunner.manager.getRepository(serverGroupItems_entity_1.ServerGroupItems).insert(itemsToAdd);
        await queryRunner.manager.getRepository(itemRelation_entity_1.ItemRelation).save(relationsToAdd);
    }
    async down(queryRunner) {
        const itemsToDrop = await queryRunner.manager.getRepository(serverGroupItems_entity_1.ServerGroupItems).findBy({ isPrimary: false });
        await queryRunner.manager.getRepository(serverGroupItems_entity_1.ServerGroupItems).delete({ isPrimary: false });
        await queryRunner.manager.getRepository(itemRelation_entity_1.ItemRelation).remove(itemsToDrop.map(it => {
            return {
                child: it.item_id,
                parent: it.server_group_id,
                origin: itemRelation_entity_1.RelationOrigin.parent
            };
        }));
        await queryRunner.dropColumn("server_group_items", "is_primary");
    }
}
exports.addServerGroupPrimaryColumn1718878382856 = addServerGroupPrimaryColumn1718878382856;
//# sourceMappingURL=1718878382856-addServerGroupPrimaryColumn.migration.js.map