"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ACLController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const aclResponse_dto_1 = require("../../common/aclResponse.dto");
const aclResponse_constants_1 = require("../../common/constants/aclResponse.constants");
const menuResponse_dto_1 = require("../../common/menuResponse.dto");
const Architecture_1 = require("../architecture/Architecture");
const menu_service_1 = require("../menu/menu.service");
const ACLCheck_1 = require("./ACLCheck");
const acl_service_1 = require("./acl.service");
const xormon_exception_1 = require("../../exception/xormon.exception");
let ACLController = class ACLController {
    constructor(menuService, ACLService) {
        this.menuService = menuService;
        this.ACLService = ACLService;
    }
    async getRules() {
        const rules = await this.ACLService.getRules();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: rules
        };
    }
    async getGroupRules(group_id) {
        const rules = await this.ACLService.getRules(group_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: rules
        };
    }
    async createRules(request, headers, group_id, AclRule) {
        const createdRules = await this.ACLService.saveRules(group_id, AclRule);
        if (createdRules) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'rules created',
                data: createdRules
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'error during rule creation',
                data: null
            };
        }
    }
    async clearRules(request, headers) {
        await this.ACLService.clearRules();
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'request to clear rules accepted'
        };
    }
    async deleteRules(request, headers, group_id, strict = false) {
        await this.ACLService.deleteRules(group_id);
        if (!strict) {
            await this.ACLService.createAdminRolesRules(group_id);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: 'request to delete rules accepted'
        };
    }
    async updateRules(request, headers, group_id, AclTree) {
        const update = await this.ACLService.updateRules(group_id, AclTree);
        if (update) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'rules updated'
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'error during rule update',
                error: 'Rules not updated'
            };
        }
    }
    async updateRulesChanges(request, headers, group_id, changes) {
        const update = await this.ACLService.updateRuleChanges(group_id, changes);
        if (update) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'rules updated'
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'error during rule update',
                error: 'Rules not updated'
            };
        }
    }
    async getRoot(group_id) {
        let menuItems = [];
        let classes = await Architecture_1.Architecture.getClasses();
        for (let technology_class of classes) {
            const classStatus = await ACLCheck_1.ACLCheck.getClassStatus([group_id], technology_class.class, false);
            let classFolder = new aclResponse_dto_1.AclMenuFolder(technology_class.label, '', classStatus, technology_class.class);
            classFolder.url = '';
            classFolder.disabled = await this.ACLService.isDisabled(group_id, technology_class.class);
            let classChildrenItems = [];
            if (!classFolder.disabled) {
                if (technology_class.class === 'san' || technology_class.class === 'storage' || technology_class.class === 'lan') {
                    let items = [];
                    if (technology_class.class === 'san') {
                        items = await this.menuService.getSanRoot();
                    }
                    else if (technology_class.class === 'lan') {
                        items = await this.menuService.getLanRoot(null);
                    }
                    else {
                        const devices = await this.menuService.getDevices('storage');
                        for (const device of devices) {
                            let path = device.hw_type.toLowerCase() + '/device?ids=' + device.item_id;
                            items.push(new menuResponse_dto_1.MenuFolder(device.label, path, device.hw_type.toLowerCase(), 'device', device.item_id));
                        }
                    }
                    items = await this.ACLService.filterFolder(items);
                    items = await this.ACLService.convertItemPaths(items);
                    items = await this.ACLService.assignStatusDeviceFolder(items, group_id, technology_class.class);
                    classChildrenItems = items;
                }
                else {
                    let items = await this.menuService.getRoot(technology_class.class);
                    for (const item of items) {
                        if (await this.menuService.checkItem(item.hw_type)) {
                            let hwType = item.hw_type.toLowerCase();
                            let hwTypeStatus = await ACLCheck_1.ACLCheck.getHwTypeStatus([group_id], hwType, false);
                            let hwTypeFolder = (technology_class.class !== 'cloud')
                                ? new aclResponse_dto_1.AclMenuFolder(item.label, hwType, hwTypeStatus, undefined, hwType)
                                : new aclResponse_dto_1.AclMenuLink(item.label, '', hwTypeStatus);
                            if (classStatus === aclResponse_constants_1.ITEM_STATUS.granted) {
                                hwTypeFolder.disabled = true;
                            }
                            else {
                                hwTypeFolder.disabled = await this.ACLService.isDisabled(group_id, technology_class.class, hwType);
                            }
                            classChildrenItems.push(hwTypeFolder);
                        }
                    }
                }
            }
            classFolder.children = classChildrenItems;
            menuItems.push(classFolder);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getHwType(hw_type, group_id) {
        let menuItems = [];
        const hwTypeClass = await this.ACLService.getHwTypeClass(hw_type);
        if (hwTypeClass === 'virtualization' && hw_type === 'power') {
            menuItems = await this.ACLService.getVirtualizationSubsys(hw_type);
        }
        else {
            menuItems = await this.menuService.getHwTypeItems(hw_type, true);
        }
        menuItems = await this.ACLService.filterFolder(menuItems);
        menuItems = await this.ACLService.convertItemPaths(menuItems);
        menuItems = await this.ACLService.assignStatusSubsysFolder(menuItems, group_id, hw_type);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getSubsystem(hw_type, subsystem, group_id, item_ids) {
        let menuItems = [];
        const hwTypeClass = await this.ACLService.getHwTypeClass(hw_type);
        if (hwTypeClass === 'virtualization' && hw_type === 'power') {
            menuItems = await this.ACLService.getVirtualizationSubsysItems(hw_type, subsystem);
        }
        else if (hwTypeClass === 'storage' && subsystem === 'device') {
            menuItems = await this.ACLService.getStorageDeviceSubsysItems(hw_type, item_ids);
        }
        else {
            menuItems = await this.menuService.getSubsystemItems(hw_type, subsystem, item_ids, true);
        }
        menuItems.forEach((mi) => {
            if (Array.isArray(mi.itemId)) {
                if (mi.itemId.length !== 1) {
                    throw new xormon_exception_1.XormonException(`Multiple itemIds (${mi.itemId})!`, 500, "Internal server error");
                }
                mi.itemId = mi.itemId[0];
            }
        });
        menuItems = await this.ACLService.filterFolder(menuItems);
        menuItems = await this.ACLService.convertItemPaths(menuItems);
        if ((hwTypeClass.includes('san') || hwTypeClass.includes('lan')) && subsystem === 'fabric') {
            menuItems = await this.ACLService.assignStatusDeviceFolder(menuItems, group_id, hwTypeClass);
        }
        else {
            menuItems = await this.ACLService.assignStatusSubsysFolder(menuItems, group_id, hw_type, subsystem, item_ids);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async parents(uuid) {
        let parents = await ACLCheck_1.ACLCheck.findParents2([uuid]);
        return parents;
    }
    async parent(uuid) {
        let parents = await ACLCheck_1.ACLCheck.findParents(uuid);
        return parents;
    }
    async children(uuid) {
        let children = await ACLCheck_1.ACLCheck.findChildren(uuid);
        return children;
    }
};
exports.ACLController = ACLController;
__decorate([
    openapi.ApiOperation({ description: "return all rules" }),
    (0, common_1.Get)('v1/rules/'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/aclResponse.dto").AclResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "getRules", null);
__decorate([
    openapi.ApiOperation({ description: "return rules per group" }),
    (0, common_1.Get)('v1/rules/:group_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/aclResponse.dto").AclResponseDTO }),
    __param(0, (0, common_1.Param)('group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "getGroupRules", null);
__decorate([
    openapi.ApiOperation({ description: "create rules" }),
    (0, common_1.Post)('v1/rules/:group_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 201, type: require("../../common/aclResponse.dto").AclResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('group_id')),
    __param(3, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, aclResponse_dto_1.AclResponseDTO]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "createRules", null);
__decorate([
    openapi.ApiOperation({ description: "delete all rules" }),
    (0, common_1.Delete)('v1/rules/'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "clearRules", null);
__decorate([
    openapi.ApiOperation({ description: "delete rules" }),
    (0, common_1.Delete)('v1/rules/:group_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('group_id')),
    __param(3, (0, common_1.Query)('strict')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, Boolean]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "deleteRules", null);
__decorate([
    openapi.ApiOperation({ description: "Save" }),
    (0, common_1.Put)('v1/tree/:group_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('group_id')),
    __param(3, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, aclResponse_dto_1.AclMenuResponseDTO]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "updateRules", null);
__decorate([
    openapi.ApiOperation({ description: "Save (only) changes" }),
    (0, common_1.Put)('v1/tree/changes/:group_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('group_id')),
    __param(3, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, aclResponse_dto_1.AclMenuChangesResponseDTO]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "updateRulesChanges", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu-like tree for class and hw_type" }),
    (0, common_1.Get)('v1/tree'),
    openapi.ApiResponse({ status: 200, type: require("../../common/aclResponse.dto").AclMenuResponseDTO }),
    __param(0, (0, common_1.Query)('group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "getRoot", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu tree of hw_type" }),
    (0, common_1.Get)('v1/tree/hw_type/:hw_type'),
    openapi.ApiResponse({ status: 200, type: require("../../common/aclResponse.dto").AclMenuResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Query)('group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Number]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "getHwType", null);
__decorate([
    openapi.ApiOperation({ description: "Returns items in folder" }),
    (0, common_1.Get)('v1/tree/hw_type/:hw_type/:subsystem'),
    openapi.ApiResponse({ status: 200, type: require("../../common/aclResponse.dto").AclMenuResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Param)('subsystem')),
    __param(2, (0, common_1.Query)('group_id')),
    __param(3, (0, common_1.Query)('ids')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Number, String]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "getSubsystem", null);
__decorate([
    openapi.ApiOperation({ description: "test" }),
    (0, common_1.Get)('v1/parents/:uuid'),
    openapi.ApiResponse({ status: 200, type: Object }),
    __param(0, (0, common_1.Param)('uuid')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "parents", null);
__decorate([
    openapi.ApiOperation({ description: "test" }),
    (0, common_1.Get)('v1/parent/:uuid'),
    openapi.ApiResponse({ status: 200, type: Object }),
    __param(0, (0, common_1.Param)('uuid')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "parent", null);
__decorate([
    openapi.ApiOperation({ description: "test" }),
    (0, common_1.Get)('v1/children/:uuid'),
    openapi.ApiResponse({ status: 200, type: Object }),
    __param(0, (0, common_1.Param)('uuid')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ACLController.prototype, "children", null);
exports.ACLController = ACLController = __decorate([
    (0, swagger_1.ApiTags)('ACL'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/acl'),
    __metadata("design:paramtypes", [menu_service_1.MenuService, acl_service_1.ACLService])
], ACLController);
//# sourceMappingURL=acl.controller.js.map