"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlertingController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const xormon_exception_1 = require("../../exception/xormon.exception");
const mail_service_1 = require("../../mail/mail.service");
const alerting_dto_1 = require("../../common/alerting.dto");
const User_1 = require("../users/User");
const alerting_service_1 = require("./alerting.service");
const deviceGroupHwAlerting_dto_1 = require("../../common/deviceGroupHwAlerting.dto");
const typeorm_transactional_1 = require("typeorm-transactional");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const Definitions_1 = require("../metrics/Definitions");
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
let AlertingController = class AlertingController {
    constructor(alertingService, mailService) {
        this.alertingService = alertingService;
        this.mailService = mailService;
        this.log = alerting_service_1.AlertingService.logger;
    }
    async getAlerts(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        const alertGroups = await this.alertingService.getAlertingGroups(user.user_id, user.groups.map(g => g.group_id));
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await this.alertingService.transformAlertGroup(alertGroups, user.user_id)
        };
    }
    async saveAlertGroups(request, alertGroupsDto) {
        const user = await User_1.User.getUserByRequest(request, true);
        const filtered = await this.alertingService.filterAlertGroups(alertGroupsDto, user.user_id, user.groups.map(g => g.group_id));
        let message;
        if (filtered.length !== alertGroupsDto.length) {
            message = "Some alert groups were not saved based on the license limit";
            alertGroupsDto = filtered;
        }
        let savedIds = [];
        try {
            savedIds = await this.alertingService.saveAlertingGroups(alertGroupsDto, user);
        }
        catch (e) {
            this.log.error('[saveAlertGroups] error:', e);
            throw new xormon_exception_1.XormonException("an error occurred while saving the alert", common_1.HttpStatus.BAD_REQUEST);
        }
        const alertingGroups = await this.alertingService.getAlertingGroupsByIds(savedIds);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await this.alertingService.transformAlertGroup(alertingGroups, user.user_id),
            message: message
        };
    }
    async updateAlertGroup(request, alert_group_id, alert_group) {
        const user = await User_1.User.getUserByRequest(request, true);
        const result = await this.alertingService.saveAlertingGroups([alert_group], user);
        return {
            data: result,
            statusCode: common_1.HttpStatus.OK
        };
    }
    async deleteAlertGroups(alertGroupIds) {
        await this.alertingService.deleteAlertingGroups(alertGroupIds);
        return {
            statusCode: common_1.HttpStatus.OK
        };
    }
    async getHwAlertingEmails() {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: (await this.alertingService.getHwAlertingEmails()).map(mailGroup => {
                return mailGroup._mail_group;
            })
        };
    }
    async updateHwAlertingEmails(mailGroupIds) {
        const saved = await this.alertingService.updateHwAlertingEmails(mailGroupIds);
        if (saved.length !== mailGroupIds.length) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'error during mail saving',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: "mail groups updated"
        };
    }
    async testHwAlertingEmails() {
        const mailGroups = await this.alertingService.getHwAlertingEmails();
        if (mailGroups.length === 0) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'no mail groups configured',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        let mails = [];
        mailGroups.forEach(mailGroup => {
            mails = mails.concat(mailGroup._mail_group.mails);
        });
        try {
            await this.mailService.verifyTransport();
            await this.mailService.send(mails, "Test e-mail", "Test email from the XORMON monitoring system");
        }
        catch (e) {
            this.log.error("[MailTest]", e);
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: e.message,
                error: e.message,
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: "test email sent"
        };
    }
    async getDeviceGroups(request) {
        const userId = (await User_1.User.getUserByRequest(request)).user_id;
        const deviceGroups = await this.alertingService.getDeviceGroups(userId);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: deviceGroups
        };
    }
    async createDeviceGroups(request, deviceGroupHwAlertingBody) {
        const user = await User_1.User.getUserByRequest(request);
        const savedHwEventDeviceGroup = await this.alertingService.createDeviceGroups(deviceGroupHwAlertingBody, user.user_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: savedHwEventDeviceGroup
        };
    }
    async updateDeviceGroup(request, deviceGroupHwAlertingBody) {
        const userId = (await User_1.User.getUserByRequest(request)).user_id;
        if (userId !== deviceGroupHwAlertingBody.user_id) {
            throw new xormon_exception_1.XormonException("You are not authorized to update this device group", common_1.HttpStatus.UNAUTHORIZED);
        }
        const updated = await this.alertingService.updateDeviceGroup(deviceGroupHwAlertingBody);
        if (updated.success) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'device group updated',
                data: updated.device_group
            };
        }
        else {
            throw new xormon_exception_1.XormonException('An error occurred while updating the device group', common_1.HttpStatus.NOT_MODIFIED);
        }
    }
    async deleteDeviceGroupById(request, deviceGroupId) {
        const deleted = await this.alertingService.deleteDeviceGroup(deviceGroupId);
        if (deleted) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: "device group deleted"
            };
        }
        throw new xormon_exception_1.XormonException("no device group deleted", common_1.HttpStatus.BAD_REQUEST);
    }
    async history(request, from, to) {
        const user = await User_1.User.getUserByRequest(request, true);
        const history = await this.alertingService.getAlertingHistory(user.user_id, user.groups.map(g => g.group_id), from, to);
        let itemIds = [];
        const metricsMapping = {};
        const alertingGroupMapping = {};
        for (const historyItem of history) {
            alertingGroupMapping[historyItem.alerting_group_id] = historyItem._alerting_group;
        }
        for (const alertingGroupId in alertingGroupMapping) {
            const alertingGroup = alertingGroupMapping[alertingGroupId];
            metricsMapping[alertingGroup.alerting_group_id] = await ItemGroupWrapper_1.ItemGroupWrapper.getMetrics(alertingGroup.item_group_id, 'alerting');
        }
        for (const hItem of history) {
            itemIds.push(...hItem._items.map(i => i.item_id));
        }
        itemIds = [...new Set(itemIds)];
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            relations: {
                parents: {
                    _parent: true
                }
            }, where: {
                item_id: (0, typeorm_1.In)(itemIds)
            }
        });
        return {
            statusCode: common_1.HttpStatus.OK,
            data: history.map(h => {
                const metric = metricsMapping[h._alerting_group.alerting_group_id].find(m => m.metric === h.metric);
                const historyItemIds = h._items.map(i => i.item_id);
                const itemsWithParents = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(items.filter(i => historyItemIds.includes(i.item_id))));
                return {
                    alerting_history_id: h.alerting_history_id,
                    alerting_group_id: h.alerting_group_id,
                    alerting_group_label: h._alerting_group.label,
                    group_label: h._alerting_group._group
                        ? h._alerting_group._group.name
                        : null,
                    group_id: h._alerting_group._group
                        ? h._alerting_group._group.group_id
                        : null,
                    metric: metric || {
                        metric: h.metric,
                        label: h.metric,
                        shortcut: null,
                        unit: null,
                        defaultPrefix: null,
                        defaultPrecision: 0
                    },
                    limit: h.limit,
                    level: h.level,
                    targets: h.targets,
                    triggered: h.triggeredAt.toISOString(),
                    items: itemsWithParents.map(i => {
                        return {
                            item_id: i.item_id,
                            label: i.label,
                            root_parent: i.root_parent
                        };
                    })
                };
            })
        };
    }
};
exports.AlertingController = AlertingController;
__decorate([
    openapi.ApiOperation({ description: "return alerts" }),
    (0, common_1.Get)('v1/alertGroups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/alerting.dto").AlertsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "getAlerts", null);
__decorate([
    (0, common_1.Post)('v1/alertGroups'),
    openapi.ApiResponse({ status: 201, type: require("../../common/alerting.dto").AlertsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Array]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "saveAlertGroups", null);
__decorate([
    (0, typeorm_transactional_1.Transactional)(),
    (0, common_1.Put)('v1/alert_group/:alert_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('alert_group_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, alerting_dto_1.CreateAlertGroupDTO]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "updateAlertGroup", null);
__decorate([
    (0, common_1.Delete)('v1/alertGroups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "deleteAlertGroups", null);
__decorate([
    openapi.ApiOperation({ description: "return mail groups from hw alerting" }),
    (0, common_1.Get)('v1/hw/emails'),
    openapi.ApiResponse({ status: 200, type: require("../../common/mailGroup.dto").MailGroupsResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "getHwAlertingEmails", null);
__decorate([
    openapi.ApiOperation({ description: "update hw alerting email groups" }),
    (0, common_1.Put)('v1/hw/emails'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "updateHwAlertingEmails", null);
__decorate([
    openapi.ApiOperation({ description: "test email" }),
    (0, common_1.Get)('v1/hw/test'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "testHwAlertingEmails", null);
__decorate([
    (0, common_1.Get)('v1/hw/deviceGroups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/deviceGroupHwAlerting.dto").DeviceGroupsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "getDeviceGroups", null);
__decorate([
    (0, common_1.Post)('v1/hw/deviceGroups'),
    openapi.ApiResponse({ status: 201, type: require("../../common/deviceGroupHwAlerting.dto").DeviceGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "createDeviceGroups", null);
__decorate([
    (0, common_1.Put)('v1/hw/deviceGroups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/deviceGroupHwAlerting.dto").DeviceGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, deviceGroupHwAlerting_dto_1.UpdateDeviceGroupHwAlertingBody]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "updateDeviceGroup", null);
__decorate([
    (0, common_1.Delete)('v1/hw/deviceGroups/:deviceGroupId'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('deviceGroupId')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "deleteDeviceGroupById", null);
__decorate([
    (0, common_1.Get)('v1/history'),
    openapi.ApiResponse({ status: 200, type: require("../../common/alerting.dto").AlertHistoryDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('from')),
    __param(2, (0, common_1.Query)('to')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Date, Date]),
    __metadata("design:returntype", Promise)
], AlertingController.prototype, "history", null);
exports.AlertingController = AlertingController = __decorate([
    (0, swagger_1.ApiTags)('Alerting'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/alerting'),
    __metadata("design:paramtypes", [alerting_service_1.AlertingService,
        mail_service_1.MailService])
], AlertingController);
//# sourceMappingURL=alerting.controller.js.map