"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlertingService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../../config/dataSource");
const alertingGroup_entity_1 = require("../../entity/alertingGroup.entity");
const alertingHwEmail_entity_1 = require("../../entity/alertingHwEmail.entity");
const hwType_entity_1 = require("../../entity/hwType.entity");
const mailGroup_entity_1 = require("../../entity/mailGroup.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const user_entity_1 = require("../../entity/user.entity");
const typeorm_2 = require("typeorm");
const uuid_1 = require("uuid");
const CustomGroupWrapper_1 = require("../custom_group/CustomGroupWrapper");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const License_1 = require("../../config/License");
const xormon_exception_1 = require("../../exception/xormon.exception");
const deviceGroupHwAlerting_entity_1 = require("../../entity/deviceGroupHwAlerting.entity");
const itemGroup_entity_1 = require("../../entity/itemGroup.entity");
const customGroup_dto_1 = require("../../common/customGroup.dto");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const Definitions_1 = require("../metrics/Definitions");
const WinstonLogger_1 = require("../log/WinstonLogger");
const alertingHistory_entity_1 = require("../../entity/alertingHistory.entity");
let AlertingService = class AlertingService {
    constructor(hwTypesRepository, objectItemsRepository, alertingGroupRepository, mailGroupsRepository, alertingHwEmailsRepository, hwEventDeviceGroupRepository, itemGroupRepository, userRepository, hostcfgRepository, alertingHistoryRepository) {
        this.hwTypesRepository = hwTypesRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.alertingGroupRepository = alertingGroupRepository;
        this.mailGroupsRepository = mailGroupsRepository;
        this.alertingHwEmailsRepository = alertingHwEmailsRepository;
        this.hwEventDeviceGroupRepository = hwEventDeviceGroupRepository;
        this.itemGroupRepository = itemGroupRepository;
        this.userRepository = userRepository;
        this.hostcfgRepository = hostcfgRepository;
        this.alertingHistoryRepository = alertingHistoryRepository;
    }
    async getMailGroups() {
        return this.mailGroupsRepository.find({ order: { label: 'ASC' } });
    }
    async saveMailGroup(mail) {
        return this.mailGroupsRepository.save(mail);
    }
    async deleteMailGroup(mail_group_id) {
        const deleted = await this.mailGroupsRepository.delete({ mail_group_id: mail_group_id });
        return deleted.affected === 1;
    }
    async updateMailGroup(mail_group_id, mail) {
        const update = await this.mailGroupsRepository
            .createQueryBuilder()
            .update(mailGroup_entity_1.MailGroup)
            .set({ label: mail.label, mails: mail.mails })
            .where('mail_group_id = :id', { id: mail_group_id })
            .execute();
        return update.affected === 1 ? true : false;
    }
    async getAlertingGroups(userId, groupIds) {
        return this.alertingGroupRepository.find({
            relations: {
                alerts: {
                    mail_groups: true
                },
                _item_group: {
                    items: {
                        _item: true
                    },
                    tags: {
                        _tag: true
                    },
                    parents: {
                        _item: true
                    },
                    regex_parents: true,
                    parent_tags: {
                        _tag: true
                    }
                },
                _group: true
            }, where: [{
                    user_id: userId,
                    group_id: (0, typeorm_2.IsNull)()
                }, {
                    group_id: (0, typeorm_2.In)(groupIds)
                }]
        });
    }
    async getAlertingGroupsByIds(agIds) {
        return this.alertingGroupRepository.find({
            where: {
                alerting_group_id: (0, typeorm_2.In)(agIds)
            },
            relations: {
                alerts: { mail_groups: true },
                _item_group: {
                    items: {
                        _item: true
                    },
                    tags: {
                        _tag: true
                    },
                    parents: {
                        _item: true
                    },
                    regex_parents: true,
                    parent_tags: {
                        _tag: true
                    }
                },
                _group: true
            }
        });
    }
    async saveAlertingGroups(alertGroups, user) {
        const savedIds = [];
        await (0, dataSource_1.getDataSource)().transaction(async (transactionalEntityManager) => {
            for (const alertGroup of alertGroups) {
                let oldItemGroupId = undefined;
                if (alertGroup.alerting_group_id) {
                    const beforeUpdate = await this.alertingGroupRepository.findOneBy({ alerting_group_id: alertGroup.alerting_group_id });
                    if (beforeUpdate?.item_group_id) {
                        oldItemGroupId = beforeUpdate.item_group_id;
                    }
                }
                const itemGroupId = !alertGroup.custom_group_id ? await ItemGroupWrapper_1.ItemGroupWrapper.saveItemGroup(alertGroup, oldItemGroupId) : null;
                const id = alertGroup.alerting_group_id ? alertGroup.alerting_group_id : (0, uuid_1.v4)();
                await transactionalEntityManager.getRepository(alertingGroup_entity_1.AlertingGroup).save([{
                        alerting_group_id: id,
                        label: alertGroup.label,
                        item_group_id: itemGroupId || null,
                        custom_group_id: alertGroup.custom_group_id ? alertGroup.custom_group_id : null,
                        alerts: alertGroup.alerts,
                        class: alertGroup.class,
                        hw_type: alertGroup.hw_type,
                        subsystem: alertGroup.subsystem,
                        disabled: alertGroup.disabled,
                        user_id: user.user_id,
                        group_id: alertGroup.group_id ? alertGroup.group_id : null
                    }]);
                savedIds.push(id);
            }
        });
        return savedIds;
    }
    async deleteAlertingGroups(alertGroupIds) {
        return this.alertingGroupRepository.delete(alertGroupIds);
    }
    async getLabel(item_id) {
        const device = await this.objectItemsRepository.findOneBy({ item_id: item_id });
        return device ? device.label : "undefined";
    }
    async getHwTypeLabel(hw_type) {
        const hwType = await this.hwTypesRepository.findOneBy({ hw_type: hw_type });
        return hwType ? hwType.label : hw_type;
    }
    async getItem(item_id) {
        return this.objectItemsRepository.findOneBy({ item_id: item_id });
    }
    async getLabels(item_ids) {
        const items = await this.objectItemsRepository.findBy({ item_id: (0, typeorm_2.In)(item_ids) });
        return items.map(item => {
            return {
                uuid: item.item_id,
                label: item.label
            };
        });
    }
    async getHwAlertingEmails() {
        return this.alertingHwEmailsRepository.find({
            relations: {
                _mail_group: true
            }
        });
    }
    async updateHwAlertingEmails(mailGroupIds) {
        await this.alertingHwEmailsRepository.delete({
            mail_group_id: (0, typeorm_2.Not)((0, typeorm_2.In)(mailGroupIds))
        });
        return await this.alertingHwEmailsRepository.save(mailGroupIds.map(mailGroupId => {
            return {
                mail_group_id: mailGroupId
            };
        }));
    }
    async getDeviceGroups(userId) {
        let deviceGroups;
        if (userId === 1 || this.isUserInAdminGroup(userId)) {
            deviceGroups = await this.hwEventDeviceGroupRepository.find({
                relations: {
                    mail_groups: true,
                    _hw_type: true,
                    _class: true,
                    _item_group: {
                        items: {
                            _item: true
                        }
                    }
                },
                order: {
                    created_at: 'ASC'
                }
            });
        }
        else {
            deviceGroups = await this.hwEventDeviceGroupRepository.find({
                relations: {
                    mail_groups: true,
                    _hw_type: true,
                    _class: true,
                    _item_group: {
                        items: {
                            _item: true
                        }
                    }
                },
                where: {
                    user_id: userId
                },
                order: {
                    created_at: 'ASC'
                }
            });
        }
        return deviceGroups.map(deviceGroup => {
            return {
                device_group_id: deviceGroup.device_group_id,
                label: deviceGroup.label,
                class: deviceGroup.class,
                hw_type: deviceGroup.hw_type,
                subsystem: deviceGroup.subsystem,
                enabled: deviceGroup.enabled,
                items: deviceGroup._item_group.items.map(item => {
                    return {
                        item_id: item.item_id,
                        label: item._item.label
                    };
                }),
                regex: deviceGroup._item_group.regex,
                mailGroups: deviceGroup.mail_groups.map(mailGroup => {
                    return {
                        mail_group_id: mailGroup.mail_group_id,
                        label: mailGroup.label,
                        mails: mailGroup.mails.map(mail => mail)
                    };
                }),
                integrations: !deviceGroup.integrations || Object.keys(deviceGroup.integrations).length === 0
                    ? {
                        snmp: false,
                        servicenow: false
                    }
                    : deviceGroup.integrations
            };
        });
    }
    async isUserInAdminGroup(userId) {
        const user = await this.userRepository.findOne({
            relations: {
                groups: true
            },
            where: {
                user_id: userId
            }
        });
        if (user) {
            return user.groups.some(group => group.name === 'Admin');
        }
        return false;
    }
    async createDeviceGroups(deviceGroupObject, userId) {
        const itemGroupDataObject = new customGroup_dto_1.ItemGroup();
        if (deviceGroupObject.regex) {
            itemGroupDataObject.regex = deviceGroupObject.regex;
        }
        else {
            itemGroupDataObject.item_ids = deviceGroupObject.items;
        }
        const itemGroupId = await ItemGroupWrapper_1.ItemGroupWrapper.saveItemGroup(itemGroupDataObject);
        const deviceGroup = await this.hwEventDeviceGroupRepository.create({
            label: deviceGroupObject.label,
            class: deviceGroupObject.class,
            hw_type: deviceGroupObject.hw_type,
            subsystem: (0, Definitions_1.findRootSubsystem)(deviceGroupObject.class, deviceGroupObject.hw_type),
            enabled: true,
            user_id: userId,
            item_group_id: itemGroupId,
            mail_groups: await this.mailGroupsRepository.findBy({ mail_group_id: (0, typeorm_2.In)(deviceGroupObject.mail_groups) }),
            integrations: deviceGroupObject.integrations
        });
        await this.hwEventDeviceGroupRepository.save(deviceGroup);
        return {
            device_group_id: deviceGroup.device_group_id,
            label: deviceGroup.label,
            class: deviceGroup.class,
            hw_type: deviceGroup.hw_type,
            subsystem: deviceGroup.subsystem,
            enabled: deviceGroup.enabled,
            mailGroups: deviceGroup.mail_groups.map(mailGroup => {
                return {
                    mail_group_id: mailGroup.mail_group_id,
                    label: mailGroup.label,
                    mails: mailGroup.mails
                };
            }),
            integrations: !deviceGroup.integrations || Object.keys(deviceGroup.integrations).length === 0
                ? {
                    snmp: false,
                    servicenow: false
                }
                : deviceGroup.integrations
        };
    }
    async deleteDeviceGroup(deviceGroupId) {
        const result = await (0, dataSource_1.getDataSource)().transaction(async (transactionalEntityManager) => {
            const hwEventDeviceGroup = await transactionalEntityManager.getRepository(deviceGroupHwAlerting_entity_1.HwEventDeviceGroup).findOneBy({ device_group_id: deviceGroupId });
            if (!hwEventDeviceGroup) {
                return false;
            }
            await transactionalEntityManager.getRepository(deviceGroupHwAlerting_entity_1.HwEventDeviceGroup).delete(deviceGroupId);
            await transactionalEntityManager.getRepository(itemGroup_entity_1.ItemGroup).delete(hwEventDeviceGroup.item_group_id);
            return true;
        });
        return result;
    }
    async updateDeviceGroup(deviceGroupHwAlerting) {
        const itemGroupDataObject = new customGroup_dto_1.ItemGroup();
        if (deviceGroupHwAlerting.regex) {
            itemGroupDataObject.regex = deviceGroupHwAlerting.regex;
        }
        else {
            itemGroupDataObject.item_ids = deviceGroupHwAlerting.items;
        }
        const hwEventDeviceGroup = await this.hwEventDeviceGroupRepository.findOneBy({ device_group_id: deviceGroupHwAlerting.device_group_id });
        const itemGroupId = await ItemGroupWrapper_1.ItemGroupWrapper.saveItemGroup(itemGroupDataObject);
        const deviceGroup = await this.hwEventDeviceGroupRepository.create({
            label: deviceGroupHwAlerting.label,
            class: deviceGroupHwAlerting.class,
            hw_type: deviceGroupHwAlerting.hw_type,
            enabled: deviceGroupHwAlerting.enabled,
            user_id: deviceGroupHwAlerting.user_id,
            item_group_id: itemGroupId,
            mail_groups: await this.mailGroupsRepository.findBy({ mail_group_id: (0, typeorm_2.In)(deviceGroupHwAlerting.mail_groups) }),
            integrations: deviceGroupHwAlerting.integrations
        });
        let transactionSuccess = false;
        let deviceGroupUpdatedWithRelations;
        await (0, dataSource_1.getDataSource)().transaction(async (transactionalEntityManager) => {
            await transactionalEntityManager.getRepository(deviceGroupHwAlerting_entity_1.HwEventDeviceGroup).delete(deviceGroupHwAlerting.device_group_id);
            await transactionalEntityManager.getRepository(itemGroup_entity_1.ItemGroup).delete(hwEventDeviceGroup.item_group_id);
            const savedDeviceGroup = await this.hwEventDeviceGroupRepository.save(deviceGroup);
            if (savedDeviceGroup) {
                transactionSuccess = true;
                deviceGroupUpdatedWithRelations = await this.hwEventDeviceGroupRepository.findOne({
                    relations: {
                        mail_groups: true,
                        _item_group: {
                            items: {
                                _item: true
                            }
                        }
                    },
                    where: { device_group_id: deviceGroup.device_group_id }
                });
            }
        });
        deviceGroupUpdatedWithRelations.mail_groups = deviceGroupHwAlerting.mail_groups.map(id => {
            return deviceGroupUpdatedWithRelations.mail_groups.find(group => group.mail_group_id === id);
        });
        return {
            success: transactionSuccess,
            device_group: {
                device_group_id: deviceGroupUpdatedWithRelations.device_group_id,
                label: deviceGroupUpdatedWithRelations.label,
                class: deviceGroupUpdatedWithRelations.class,
                hw_type: deviceGroupUpdatedWithRelations.hw_type,
                enabled: deviceGroupUpdatedWithRelations.enabled,
                mailGroups: deviceGroupUpdatedWithRelations.mail_groups.map(mailGroup => {
                    return {
                        mail_group_id: mailGroup.mail_group_id,
                        label: mailGroup.label,
                        mails: mailGroup.mails
                    };
                }),
                regex: deviceGroupUpdatedWithRelations._item_group.regex,
                items: deviceGroupUpdatedWithRelations._item_group.items.map(item => {
                    return {
                        item_id: item.item_id,
                        label: item._item.label
                    };
                }),
                integrations: !deviceGroupUpdatedWithRelations.integrations || Object.keys(deviceGroupUpdatedWithRelations.integrations).length === 0
                    ? {
                        snmp: false,
                        servicenow: false
                    }
                    : deviceGroupUpdatedWithRelations.integrations
            }
        };
    }
    async filterAlertGroups(alertGroups, userId, groupIds) {
        const isFree = !(await License_1.License.hasAnyValid());
        if (isFree) {
            const alerts = await this.getAlertingGroups(userId, groupIds);
            let newAlerts = alertGroups.filter(alert => !alert.alerting_group_id);
            if (alerts.length >= 3 && newAlerts.length >= 1) {
                throw new xormon_exception_1.XormonException("You can only have 3 alert groups in free version", common_1.HttpStatus.PAYMENT_REQUIRED);
            }
            const maxNew = 3 - alerts.length;
            if (maxNew <= 0) {
                newAlerts = [];
            }
            else {
                newAlerts = newAlerts.slice(0, maxNew);
            }
            alertGroups = alertGroups
                .filter(alertGroup => alertGroup.alerting_group_id)
                .concat(newAlerts);
        }
        else {
            let technologies = [];
            const alerts = await this.getAlertingGroups(userId, groupIds);
            for (const alert of [...alerts, ...alertGroups]) {
                if (alert.class && !technologies.includes(alert.class)) {
                    technologies.push(alert.class);
                }
                else if (alert.hw_type && !technologies.includes(alert.hw_type)) {
                    technologies.push(alert.hw_type);
                }
            }
            let isLicensed = {};
            for (const technology of technologies) {
                isLicensed[technology] = await License_1.License.checkTechnology(technology);
            }
            const unlicensedSaved = alerts.filter(alert => !(isLicensed[alert.class] || isLicensed[alert.hw_type]));
            const newAlerts = alertGroups.filter(alert => !alert.alerting_group_id);
            const licensedNew = newAlerts.filter(alert => isLicensed[alert.class] || isLicensed[alert.hw_type]);
            let unlicensedNew = newAlerts.filter(alert => !(isLicensed[alert.class] || isLicensed[alert.hw_type]));
            const maxNew = 3 - unlicensedSaved.length;
            if (maxNew <= 0) {
                unlicensedNew = [];
            }
            else {
                unlicensedNew = unlicensedNew.slice(0, maxNew);
            }
            alertGroups = alertGroups
                .filter(alertGroup => alertGroup.alerting_group_id)
                .concat(licensedNew)
                .concat(unlicensedNew);
        }
        return alertGroups;
    }
    transformMailGroups(mailGroups, users, userId) {
        for (const mailGroup of mailGroups) {
            if (mailGroup.user_id !== userId) {
                const user = users.find(u => u.user_id === mailGroup.user_id);
                mailGroup.label = `${mailGroup.label} (${user.username})`;
            }
        }
        return mailGroups;
    }
    async transformAlertGroup(alertGroups, userId) {
        const users = await (0, dataSource_1.getDataSource)().getRepository(user_entity_1.User).find();
        const metricsMapping = {};
        for (const alertGroup of alertGroups) {
            metricsMapping[alertGroup.alerting_group_id] = alertGroup.item_group_id
                ? await ItemGroupWrapper_1.ItemGroupWrapper.getMetrics(alertGroup.item_group_id, 'alerting')
                : alertGroup.custom_group_id
                    ? await CustomGroupWrapper_1.CustomGroupWrapper.getMetrics(alertGroup.custom_group_id)
                    : [];
        }
        return alertGroups.map(alertGroup => {
            return {
                ...alertGroup,
                alerting_group_id: alertGroup.alerting_group_id,
                label: alertGroup.label,
                class: alertGroup.class,
                hw_type: alertGroup.hw_type,
                subsystem: alertGroup.subsystem,
                alerts: alertGroup.alerts.map(alert => {
                    return {
                        alerting_group_id: alert.alerting_group_id,
                        alerting_id: alert.alerting_id,
                        triggered: alert.triggered,
                        metric: metricsMapping[alertGroup.alerting_group_id].find(metric => metric.metric === alert.metric)
                            ? metricsMapping[alertGroup.alerting_group_id].find(metric => metric.metric === alert.metric)
                            : {
                                metric: alert.metric,
                                label: alert.metric,
                                shortcut: null,
                                unit: null,
                                defaultPrefix: null,
                                defaultPrecision: 0
                            },
                        limit: Math.round(alert.limit * 100) / 100,
                        peak: alert.peak,
                        repeat: alert.repeat,
                        exclude_start: alert.exclude_start,
                        exclude_end: alert.exclude_end,
                        mail_groups: alertGroups !== null
                            ? this.transformMailGroups(alert.mail_groups, users, userId)
                            : alert.mail_groups,
                        level: alert.level,
                        integrations: !alert.integrations || Object.keys(alert.integrations).length === 0
                            ? {
                                snmp: false,
                                servicenow: false
                            }
                            : alert.integrations
                    };
                }),
                items: alertGroup._item_group?.items && alertGroup._item_group.items.length >= 1
                    ? alertGroup._item_group.items.map(item => {
                        return {
                            item_id: item.item_id,
                            label: item._item.label
                        };
                    })
                    : null,
                regex: alertGroup._item_group?.regex && alertGroup._item_group.regex.length >= 1
                    ? alertGroup._item_group.regex
                    : null,
                tags: alertGroup._item_group?.tags && alertGroup._item_group.tags.length >= 1
                    ? alertGroup._item_group.tags.map(tag => {
                        return {
                            tag_id: tag.tag_value_id,
                            tag: tag._tag.value
                        };
                    })
                    : null,
                parents: alertGroup._item_group
                    ? {
                        intersection: alertGroup._item_group.intersection,
                        items: alertGroup._item_group.parents && alertGroup._item_group.parents.length >= 1
                            ? alertGroup._item_group.parents.map(item => {
                                return {
                                    item_id: item.item_id,
                                    label: item._item.label
                                };
                            })
                            : null,
                        regex: alertGroup._item_group.regex_parents && alertGroup._item_group.regex_parents.length >= 1
                            ? alertGroup._item_group.regex_parents.map(regex => {
                                return {
                                    subsystem: regex.subsystem,
                                    regex: regex.regex
                                };
                            })
                            : null,
                        tags: alertGroup._item_group.parent_tags && alertGroup._item_group.parent_tags.length >= 1
                            ? alertGroup._item_group.parent_tags.map(tag => {
                                return {
                                    tag_id: tag.tag_value_id,
                                    tag: tag._tag.value,
                                    subsystem: tag.subsystem
                                };
                            })
                            : null,
                    }
                    : null,
                custom_group_id: alertGroup.custom_group_id ? alertGroup.custom_group_id : null,
                disabled: alertGroup.disabled,
                group: alertGroup._group ? {
                    group_id: alertGroup._group.group_id,
                    name: alertGroup._group.name
                } : null
            };
        });
    }
    async getAlertingHistory(userId, groupIds, from, to) {
        return await this.alertingHistoryRepository.find({
            relations: {
                _alerting_group: {
                    _group: true
                },
                _items: {
                    _item: true
                }
            }, where: [{
                    _alerting_group: {
                        user_id: userId,
                        group_id: (0, typeorm_2.IsNull)()
                    },
                    triggeredAt: (0, typeorm_2.And)((0, typeorm_2.LessThan)(to), (0, typeorm_2.MoreThan)(from))
                }, {
                    _alerting_group: {
                        group_id: (0, typeorm_2.In)(groupIds)
                    },
                    triggeredAt: (0, typeorm_2.And)((0, typeorm_2.LessThan)(to), (0, typeorm_2.MoreThan)(from))
                }]
        });
    }
};
exports.AlertingService = AlertingService;
AlertingService.logger = (0, WinstonLogger_1.LoggerFactory)('alerting-service');
exports.AlertingService = AlertingService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(hwType_entity_1.HwType)),
    __param(1, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(2, (0, typeorm_1.InjectRepository)(alertingGroup_entity_1.AlertingGroup)),
    __param(3, (0, typeorm_1.InjectRepository)(mailGroup_entity_1.MailGroup)),
    __param(4, (0, typeorm_1.InjectRepository)(alertingHwEmail_entity_1.AlertingHwEmail)),
    __param(5, (0, typeorm_1.InjectRepository)(deviceGroupHwAlerting_entity_1.HwEventDeviceGroup)),
    __param(6, (0, typeorm_1.InjectRepository)(itemGroup_entity_1.ItemGroup)),
    __param(7, (0, typeorm_1.InjectRepository)(user_entity_1.User)),
    __param(8, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(9, (0, typeorm_1.InjectRepository)(alertingHistory_entity_1.AlertingHistory)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], AlertingService);
//# sourceMappingURL=alerting.service.js.map