"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Architecture = void 0;
const dataSource_1 = require("../../config/dataSource");
const class_entity_1 = require("../../entity/class.entity");
const hwType_entity_1 = require("../../entity/hwType.entity");
const subsystem_entity_1 = require("../../entity/subsystem.entity");
const _Class = require("../metrics/Classes");
const App_1 = require("../configuration/App");
const typeorm_1 = require("typeorm");
const class_constants_1 = require("../../common/constants/class.constants");
class Architecture {
    static allowedHwTypes() {
        return App_1.App.allowedHwTypes();
    }
    static getInternalMonitoringSubsystems() {
        return [{
                label: 'Task',
                subsystem: 'task',
                hw_type: 'application',
                properties: {}
            }];
    }
    static async getClasses(withHwTypes = false) {
        let allowed = App_1.App.isProduction() ? this.allowedHwTypes() : undefined;
        let iclass = [];
        const classes = await (0, dataSource_1.getDataSource)()
            .getRepository(class_entity_1.Class)
            .find({
            relations: {
                hw_types: true
            }, order: {
                class_order: 'ASC'
            },
            where: {
                _class: (0, typeorm_1.Not)((0, typeorm_1.In)(['xormon']))
            }
        });
        for (const _class of classes) {
            if (!withHwTypes) {
                iclass.push({
                    label: _class.label,
                    class: _class._class,
                });
            }
            else {
                let iHwTypes = [];
                for (const hwType of _class.hw_types) {
                    if (allowed !== undefined && !allowed.includes(hwType.hw_type)) {
                        continue;
                    }
                    iHwTypes.push({
                        class: hwType.class,
                        hw_type: hwType.hw_type,
                        label: hwType.label,
                    });
                }
                iclass.push({
                    label: _class.label,
                    class: _class._class,
                    hw_types: iHwTypes,
                });
            }
        }
        return iclass;
    }
    static async getHwTypes(withSubsystems = false, _class) {
        let allowed = App_1.App.isProduction() ? this.allowedHwTypes() : undefined;
        const hwTypes = await (0, dataSource_1.getDataSource)()
            .getRepository(hwType_entity_1.HwType)
            .find({
            where: {
                class: Array.isArray(_class) ? (0, typeorm_1.In)(_class) : _class,
                hw_type: (0, typeorm_1.Not)((0, typeorm_1.In)(['application', 'database', 'timescaledb', 'as400']))
            }, relations: {
                subsystems: true
            }, order: {
                hw_type_order: 'ASC'
            }
        });
        let iHwTypes = [];
        for (const hwType of hwTypes) {
            if (allowed !== undefined && !allowed.includes(hwType.hw_type)) {
                continue;
            }
            if (!withSubsystems) {
                iHwTypes.push({
                    class: hwType.class.toLowerCase(),
                    hw_type: hwType.hw_type,
                    label: hwType.label,
                });
            }
            else {
                let iSubsystems = [];
                for (const subsystem of hwType.subsystems) {
                    iSubsystems.push({
                        subsystem: subsystem.subsystem,
                        label: subsystem.label,
                    });
                }
                iHwTypes.push({
                    class: hwType.class,
                    hw_type: hwType.hw_type,
                    label: hwType.label,
                    subsystems: iSubsystems,
                });
            }
        }
        return iHwTypes;
    }
    static async getHwType(findHwType, withSubsystems = false) {
        const hwType = await (0, dataSource_1.getDataSource)()
            .getRepository(hwType_entity_1.HwType)
            .findOne({ where: { hw_type: findHwType }, relations: { subsystems: true } });
        if (!hwType)
            return null;
        if (!withSubsystems) {
            return {
                class: hwType.class,
                hw_type: hwType.hw_type,
                label: hwType.label,
            };
        }
        else {
            let iSubsystems = [];
            for (const subsystem of hwType.subsystems) {
                iSubsystems.push({
                    subsystem: subsystem.subsystem,
                    label: subsystem.label,
                });
            }
            return {
                class: hwType.class,
                hw_type: hwType.hw_type,
                label: hwType.label,
                subsystems: iSubsystems,
            };
        }
    }
    static async getSubsystems(byHwType, type) {
        let allowed = App_1.App.isProduction() ? this.allowedHwTypes() : undefined;
        let subsys = byHwType
            ? await (0, dataSource_1.getDataSource)()
                .getRepository(subsystem_entity_1.Subsystem)
                .find({ where: { hw_type: byHwType }, relations: { hw_type_entity: true } })
            : await (0, dataSource_1.getDataSource)()
                .getRepository(subsystem_entity_1.Subsystem)
                .find({ where: { hw_type: (0, typeorm_1.Not)((0, typeorm_1.In)(['application', 'database'])) }, relations: { hw_type_entity: true } });
        let subsystems = [];
        for (const subsystem of subsys) {
            if (allowed !== undefined && !allowed.includes(subsystem.hw_type)) {
                continue;
            }
            if (type && type === 'customGroup' && (subsystem.properties && subsystem.properties.customGroup === false)) {
                continue;
            }
            subsystems.push({
                label: subsystem.label ? subsystem.label : subsystem.subsystem,
                subsystem: subsystem.subsystem,
                hw_type: subsystem.hw_type,
                hw_type_label: subsystem.hw_type_entity.label,
                class: subsystem.hw_type_entity.class,
            });
        }
        return subsystems;
    }
    static getClassesFromMetrics(withHwTypes = false) {
        let iclass = [];
        const classes = Object.keys(_Class);
        for (const _class of classes) {
            if (!withHwTypes) {
                iclass.push({
                    label: _Class[_class].label ? _Class[_class].label : _class,
                    order: _Class[_class].order !== undefined ? _Class[_class].order : null,
                    class: _class.toLowerCase(),
                });
            }
            else {
                let iHwTypes = [];
                const hwTypes = Object.keys(_Class[_class]['technology']);
                for (const hwType of hwTypes) {
                    iHwTypes.push({
                        class: _class.toLowerCase(),
                        hw_type: hwType,
                        label: _Class[_class]['technology'][hwType]['label'] ? _Class[_class]['technology'][hwType]['label'] : hwType,
                        order: _Class[_class]['technology'][hwType]['order'] ? _Class[_class]['technology'][hwType]['order'] : null
                    });
                }
                iclass.push({
                    label: _Class[_class].label ? _Class[_class].label : _class,
                    order: _Class[_class].order !== undefined ? _Class[_class].order : null,
                    class: _class.toLowerCase(),
                    hw_types: iHwTypes,
                });
            }
        }
        return iclass;
    }
    static getHwTypesFromMetrics(withSubsystems = false, _class) {
        let _classes = [];
        if (_class) {
            _classes.push(_class);
        }
        else {
            _classes = Object.keys(_Class);
        }
        let iHwTypes = [];
        for (const _classItem of _classes) {
            const className = _classItem.charAt(0).toUpperCase() + _classItem.slice(1).toLowerCase();
            const hwTypes = Object.keys(_Class[className]['technology']);
            for (const hwType of hwTypes) {
                if (!withSubsystems) {
                    iHwTypes.push({
                        class: className.toLowerCase(),
                        hw_type: hwType,
                        label: _Class[className]['technology'][hwType]['label'] ? _Class[className]['technology'][hwType]['label'] : hwType,
                    });
                }
                else {
                    if (!_Class[className]['technology'][hwType]['subsystems']) {
                        continue;
                    }
                    const iSubsystems = [];
                    const subsystems = Object.keys(_Class[className]['technology'][hwType]['subsystems']);
                    for (const subsystem of subsystems) {
                        const subsystemDef = _Class[className]['technology'][hwType]['subsystems'][subsystem];
                        iSubsystems.push({
                            subsystem: subsystem,
                            label: _Class[className]['technology'][hwType]['subsystems'][subsystem]['label']
                                ? _Class[className]['technology'][hwType]['subsystems'][subsystem]['label']
                                : subsystem,
                            top: _Class[className]['technology'][hwType]['subsystems'][subsystem]['top']
                                ? _Class[className]['technology'][hwType]['subsystems'][subsystem]['top']
                                : undefined,
                            properties: {
                                top: _Class[className]['technology'][hwType]['subsystems'][subsystem]['top'] !== undefined,
                                customGroup: subsystemDef['excludedTools'] === undefined || (!subsystemDef['excludedTools'].includes('customGroup') && !subsystemDef['excludedTools'].includes('all')),
                                alerting: subsystemDef['excludedTools'] === undefined || (!subsystemDef['excludedTools'].includes('alerting') && !subsystemDef['excludedTools'].includes('all')),
                                exporter: subsystemDef['excludedTools'] === undefined || (!subsystemDef['excludedTools'].includes('exporter') && !subsystemDef['excludedTools'].includes('all')),
                                reporter: subsystemDef['excludedTools'] === undefined || (!subsystemDef['excludedTools'].includes('reporter') && !subsystemDef['excludedTools'].includes('all')),
                                aliases: _Class[className]['technology'][hwType]['subsystems'][subsystem]['aliases'] === true
                                    || (className == class_constants_1.classDefinitionLabel[class_constants_1.ClassKey.STORAGE] && ['pool', 'volume', 'host'].includes(subsystem))
                                    || (className == class_constants_1.classDefinitionLabel[class_constants_1.ClassKey.SAN] && subsystem === 'sanport')
                            }
                        });
                    }
                    iHwTypes.push({
                        class: className.toLowerCase(),
                        hw_type: hwType,
                        label: _Class[className]['technology'][hwType]['label'] ? _Class[className]['technology'][hwType]['label'] : hwType,
                        subsystems: iSubsystems,
                    });
                }
            }
        }
        return iHwTypes;
    }
    static getHwTypeFromMetrics(findHwType, withSubsystems = false) {
        let _classes = Object.keys(_Class);
        for (const _classItem of _classes) {
            const className = _classItem.charAt(0).toUpperCase() + _classItem.slice(1).toLowerCase();
            const hwTypes = Object.keys(_Class[className]['technology']);
            for (const hwType of hwTypes) {
                if (hwType === findHwType) {
                    if (!withSubsystems) {
                        return {
                            class: className.toLowerCase(),
                            hw_type: hwType,
                            label: _Class[className]['technology'][hwType]['label'] ? _Class[className]['technology'][hwType]['label'] : hwType,
                        };
                    }
                    else {
                        if (!_Class[className]['technology'][hwType]['subsystems']) {
                            continue;
                        }
                        const iSubsystems = [];
                        const subsystems = Object.keys(_Class[className]['technology'][hwType]['subsystems']);
                        for (const subsystem of subsystems) {
                            iSubsystems.push({
                                subsystem: subsystem,
                                label: _Class[className]['technology'][hwType]['subsystems'][subsystem]['label']
                                    ? _Class[className]['technology'][hwType]['subsystems'][subsystem]['label']
                                    : subsystem,
                                top: _Class[className]['technology'][hwType]['subsystems'][subsystem]['top']
                                    ? _Class[className]['technology'][hwType]['subsystems'][subsystem]['top']
                                    : undefined,
                            });
                        }
                        return {
                            class: className.toLowerCase(),
                            hw_type: hwType,
                            label: _Class[className]['technology'][hwType]['label'] ? _Class[className]['technology'][hwType]['label'] : hwType,
                            subsystems: iSubsystems,
                        };
                    }
                }
            }
        }
        return null;
    }
    static getSubsystemsFromMetrics(byHwType) {
        let subsystems = [];
        let hwTypes = Architecture.getHwTypesFromMetrics(true);
        for (const hwType of hwTypes) {
            if (byHwType && byHwType !== hwType.hw_type) {
                continue;
            }
            for (const subsys of hwType.subsystems) {
                subsystems.push({
                    label: subsys.label,
                    subsystem: subsys.subsystem,
                    hw_type: hwType.hw_type,
                    hw_type_label: hwType.label,
                    class: hwType.class,
                    top: subsys.top ? subsys.top : undefined,
                    properties: subsys.properties
                        ? subsys.properties
                        : {}
                });
            }
        }
        return subsystems;
    }
    static sortByLabel(items) {
        if (Array.isArray(items) && items.length >= 1 && items[0].label !== undefined) {
            return items.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        }
        else {
            return items;
        }
    }
}
exports.Architecture = Architecture;
//# sourceMappingURL=Architecture.js.map