"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AliasService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const xormon_exception_1 = require("../../exception/xormon.exception");
const typeorm_2 = require("typeorm");
const Definitions_1 = require("../metrics/Definitions");
const WinstonLogger_1 = require("../log/WinstonLogger");
let AliasService = class AliasService {
    constructor(objectItemRepository) {
        this.objectItemRepository = objectItemRepository;
    }
    async getItemsWithAlias() {
        let items = [];
        let mappedOriginalLabel = {};
        const itemsWithOriginalLabels = await this.objectItemRepository.createQueryBuilder("oi")
            .select('oi.label as label, oi.item_id, oi.subsystem, oi.hw_type, oi.alias')
            .where('oi.alias is not null')
            .getRawMany();
        for (const itemsWithOriginalLabel of itemsWithOriginalLabels) {
            mappedOriginalLabel[itemsWithOriginalLabel.item_id] = itemsWithOriginalLabel.label;
        }
        const itemsWithRelations = await this.objectItemRepository.find({
            where: {
                item_id: (0, typeorm_2.In)(itemsWithOriginalLabels.map(item => item.item_id))
            }, relations: {
                parents: {
                    _parent: true
                }
            }
        });
        for (const itemsWithRelation of itemsWithRelations) {
            itemsWithRelation.label = mappedOriginalLabel[itemsWithRelation.item_id];
        }
        items = (0, Definitions_1.addRootParent)(itemsWithRelations.map(item => {
            return {
                item_id: item.item_id,
                label: item.label,
                subsystem: item.subsystem,
                hw_type: item.hw_type,
                alias: item.alias,
                parents: (0, Definitions_1.ObjectItemToArchItem)(item.parents.map(parent => parent._parent))
            };
        }));
        items.forEach(item => {
            item.parents = undefined;
        });
        return items;
    }
    async saveAliases(aliases) {
        const items = await this.objectItemRepository.findBy({ item_id: (0, typeorm_2.In)(aliases.map(item => item.item_id)) });
        if (items.length !== aliases.length) {
            let itemIds = items.map(i => i.item_id);
            let notFound = [];
            const logger = (0, WinstonLogger_1.LoggerFactory)('alias-import');
            for (const alias of aliases) {
                if (!itemIds.includes(alias.item_id)) {
                    notFound.push(alias);
                }
            }
            logger.error("Error during import - id not found");
            for (const alias of notFound) {
                logger.error(`ID: ${alias.item_id}, Alias: ${alias.alias}`);
            }
            throw new xormon_exception_1.XormonException("An error occurred. Find the output in the logs.", common_1.HttpStatus.BAD_REQUEST);
        }
        items.forEach(item => {
            item.alias = aliases.find(alias => alias.item_id === item.item_id).alias;
        });
        await this.objectItemRepository.save(items);
    }
    async deleteAlias(itemId) {
        const item = await this.objectItemRepository.findOneBy({ item_id: itemId });
        if (!item) {
            throw new xormon_exception_1.XormonException("Item not found", common_1.HttpStatus.NOT_FOUND);
        }
        item.alias = null;
        return (await this.objectItemRepository.save(item)).alias === null;
    }
    async deleteAliases() {
        const deleted = (await this.objectItemRepository.createQueryBuilder().update(objectItem_entity_1.ObjectItem).set({ alias: null }).execute());
        return true;
    }
};
exports.AliasService = AliasService;
exports.AliasService = AliasService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __metadata("design:paramtypes", [typeorm_2.Repository])
], AliasService);
//# sourceMappingURL=alias.service.js.map