"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArchitectureController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const architecture_dto_1 = require("../../common/architecture.dto");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const Definitions_1 = require("../metrics/Definitions");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const Architecture_1 = require("./Architecture");
const architecture_service_1 = require("./architecture.service");
const alias_dto_1 = require("../../common/alias.dto");
const architecture_interceptor_1 = require("../../interceptor/architecture.interceptor");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const ItemRegexWrapper_1 = require("./ItemRegexWrapper");
const alias_service_1 = require("./alias.service");
const xormon_exception_1 = require("../../exception/xormon.exception");
const platform_express_1 = require("@nestjs/platform-express");
const dataSource_1 = require("../../config/dataSource");
const typeorm_1 = require("typeorm");
const Linux_1 = require("../menu/menu/Linux");
const User_1 = require("../users/User");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
let ArchitectureController = class ArchitectureController {
    constructor(architectureService, aliasService) {
        this.architectureService = architectureService;
        this.aliasService = aliasService;
    }
    async getClasses(active, skip_acl = false) {
        let classes = await this.architectureService.getClasses();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(classes),
        };
    }
    async hw_types(active, className, type, skip_acl = false, admin_groups = false) {
        let hTypes = [];
        if (type && type === 'top') {
            const hwTypes = className ? Architecture_1.Architecture.getHwTypesFromMetrics(true, className) : Architecture_1.Architecture.getHwTypesFromMetrics(true);
            for (const hwType of hwTypes) {
                if (hwType.subsystems.find((subsystem) => subsystem.top)) {
                    hTypes.push({
                        hw_type: hwType.hw_type,
                        label: hwType.label,
                    });
                }
            }
        }
        else {
            const showActive = active ? active : false;
            const hwTypes = className
                ? await this.architectureService.getHwTypes(showActive, className)
                : await this.architectureService.getHwTypes(showActive);
            for (const hwType of hwTypes) {
                hTypes.push({
                    hw_type: hwType.hw_type,
                    label: hwType.label,
                });
            }
        }
        const exludedHwTypes = [hwTypeKey_constants_1.HwTypeKey.WINDOWS, hwTypeKey_constants_1.HwTypeKey.SOLARIS, hwTypeKey_constants_1.HwTypeKey.LINUX, hwTypeKey_constants_1.HwTypeKey.AS400];
        hTypes = hTypes.filter(hType => !exludedHwTypes.includes(hType.hw_type));
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(hTypes),
        };
    }
    async getSubsystems(active = false, className, hwType, type, skip_acl = false) {
        const subsystems = await this.architectureService.getSubsystems(active, className, hwType, type);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(subsystems),
        };
    }
    async subsystems(hwType, active, type, skip_acl = false) {
        let subsys = [];
        if (type && type === 'top') {
            const subsystems = Architecture_1.Architecture.getSubsystemsFromMetrics(hwType);
            for (const subsystem of subsystems) {
                if (subsystem.top) {
                    subsys.push({
                        subsystem: subsystem.subsystem,
                        label: subsystem.label,
                        hw_type: subsystem.hw_type
                    });
                }
            }
        }
        else if (type && type === 'configuration') {
            const subsystems = Architecture_1.Architecture.getSubsystemsFromMetrics(hwType);
            for (const subsystem of subsystems) {
                const configurations = MetricsWrapper_1.MetricsWrapper.getConfiguration(subsystem.class, hwType, subsystem.subsystem);
                if (Object.keys(configurations).length >= 1 && subsystem.properties.exporter) {
                    subsys.push({
                        subsystem: subsystem.subsystem,
                        label: subsystem.label,
                        hw_type: subsystem.hw_type
                    });
                }
            }
        }
        else if (type && type === 'custom_group') {
            const subsystems = await this.architectureService.getSubsystems(active, undefined, hwType, type);
            for (const subsystem of subsystems) {
                subsys.push({
                    subsystem: subsystem.subsystem,
                    label: subsystem.label,
                    hw_type: subsystem.hw_type
                });
            }
        }
        else {
            const showActive = active ? active : false;
            const subsystems = await this.architectureService.getSubsystems(showActive, undefined, hwType, type ? type : undefined);
            for (const subsystem of subsystems) {
                subsys.push({
                    subsystem: subsystem.subsystem,
                    label: subsystem.label,
                    hw_type: subsystem.hw_type
                });
            }
        }
        if ((await Architecture_1.Architecture.getHwTypes(false, 'storage')).map(hwType => hwType.hw_type).includes(hwType) && subsys.find(subs => subs.subsystem === 'device') === undefined) {
            subsys.push({
                subsystem: 'device',
                label: 'Device'
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(subsys),
        };
    }
    async itemsByHwTypeAndSubsystem(hw_type, subsystem) {
        let itemsWithName = [];
        let items = await this.architectureService.getByHwTypeAndSubsystem(hw_type, subsystem);
        for (const item of items) {
            const parents = this.getParentLabels(item, []);
            parents.sort((a, b) => a.subsystem > b.subsystem ? 1 : -1);
            itemsWithName.push({
                label: item.label,
                item_id: item.item_id,
                parents: parents
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(itemsWithName),
        };
    }
    getParentLabels(item, parentLabels) {
        if (item.parents) {
            for (const parent of item.parents) {
                const parentObj = parent._parent;
                parentLabels.push({ item_id: parentObj.item_id, label: parentObj.label, hw_type: parentObj.hw_type, subsystem: parentObj.subsystem });
            }
        }
        return parentLabels;
    }
    async parentsByHwTypeAndSubsystem(hw_type, subsystem) {
        if (hw_type === 'virtualization' && subsystem === 'vm') {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: this.architectureService.getVirtCrossPlatformParents()
            };
        }
        let parents = [];
        let subsystems = [];
        let subsysItems = {};
        let items = await this.architectureService.getByHwTypeAndSubsystem(hw_type, subsystem);
        items.forEach(item => {
            item.parents.forEach(parent => {
                if (!subsystems.includes(parent._parent.subsystem)) {
                    subsystems.push(parent._parent.subsystem);
                }
                if (subsysItems[parent._parent.subsystem] && !subsysItems[parent._parent.subsystem].find(item => item.item_id === parent._parent.item_id)) {
                    subsysItems[parent._parent.subsystem].push({
                        item_id: parent._parent.item_id,
                        label: parent._parent.label
                    });
                }
                else {
                    subsysItems[parent._parent.subsystem] = [{
                            item_id: parent._parent.item_id,
                            label: parent._parent.label
                        }];
                }
            });
        });
        const subsystemsDef = await this.architectureService.getSubsystems(false, undefined, hw_type);
        for (const subsystem of subsystemsDef) {
            if (subsystems.includes(subsystem.subsystem)) {
                parents.push({
                    subsystem: subsystem.subsystem,
                    hw_type: hw_type,
                    label: subsystem.label,
                    items: subsysItems[subsystem.subsystem] ? subsysItems[subsystem.subsystem] : []
                });
            }
        }
        if (subsystems.includes("device")) {
            parents.push({
                subsystem: 'device',
                label: 'Device',
                hw_type: hw_type
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(parents),
        };
    }
    async items(subsystem, hw_type, _class, parents, withParents, withRootParent, childSubsystem, strict_acl = true, withoutMapping = false, withDeleted = false) {
        let itemsWithName = [];
        if (parents !== undefined) {
            if (!Array.isArray(parents)) {
                parents = parents.split(",");
            }
        }
        let items = await this.architectureService.getItems({
            subsystem: subsystem,
            hw_type: hw_type,
            _class: _class,
            parents: parents,
            withParents: withParents || withRootParent,
            childSubsystem: childSubsystem,
            withDeleted: withDeleted
        });
        if (withoutMapping) {
            items = await this.architectureService.filterAgentMappings(items);
        }
        for (const item of items) {
            const itemParents = [];
            if (item.parents && item.parents[0] && item.parents[0]._parent) {
                for (const itemParent of item.parents) {
                    itemParents.push({
                        label: itemParent._parent.label,
                        item_id: itemParent._parent.item_id,
                        subsystem: itemParent._parent.subsystem,
                        hw_type: itemParent._parent.hw_type
                    });
                }
            }
            itemsWithName.push({
                label: item.label,
                item_id: item.item_id,
                hw_type: item.hw_type,
                subsystem: item.subsystem,
                parents: itemParents.length >= 1 ? itemParents : undefined
            });
            if (withRootParent) {
                itemsWithName = (0, Definitions_1.addRootParent)(itemsWithName);
                if (!withParents) {
                    itemsWithName.forEach(i => delete i.parents);
                }
            }
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(itemsWithName),
        };
    }
    async search(subsystem, hw_type, _class, parents, search, page, limit) {
        limit = limit ? limit : 10;
        if (parents !== undefined) {
            if (!Array.isArray(parents)) {
                parents = parents.split(",");
            }
        }
        let itemsWithName = [];
        let items = await this.architectureService.getItems({
            subsystem: subsystem,
            hw_type: hw_type,
            _class: _class,
            parents: parents,
            withParents: false,
            search: search
        });
        const metadata = {
            start: page ? page * limit : 0,
            end: page ? (page * limit + limit - 1 >= items.length ? items.length : page * limit + limit - 1) : limit - 1,
            max: items.length >= 1 ? items.length - 1 : 0,
            page: page ? +page : 0,
        };
        if (metadata.start > metadata.max) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    metadata: {
                        start: metadata.start,
                        end: metadata.end,
                        max: metadata.max,
                        page: metadata.page,
                        info: 'the page number has a higher start index than the max index',
                    },
                    items: [],
                },
            };
        }
        if (items.length >= 1) {
            items = items.slice(metadata.start, metadata.end + 1);
            for (const item of items) {
                const itemParents = [];
                if (item.parents && item.parents[0]._parent) {
                    for (const itemParent of item.parents) {
                        itemParents.push({
                            label: itemParent._parent.label,
                            item_id: itemParent._parent.item_id,
                            subsystem: itemParent._parent.subsystem,
                            hw_type: itemParent._parent.hw_type
                        });
                    }
                }
                itemsWithName.push({
                    label: item.label,
                    item_id: item.item_id,
                    hw_type: item.hw_type,
                    subsystem: item.subsystem,
                    parents: itemParents.length >= 1 ? itemParents : undefined
                });
            }
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                metadata: metadata,
                items: Architecture_1.Architecture.sortByLabel(itemsWithName),
            },
        };
    }
    async itemsByIds(items) {
        let labels = {};
        let objectItems = await this.architectureService.getItemsByIds(items.item_ids);
        for (const objectItem of objectItems) {
            labels[objectItem.item_id] = objectItem.label;
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: labels,
        };
    }
    async getMetrics() {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: this.architectureService.getClassesDef()
        };
    }
    async metricsByHwTypeAndSubsystem(hw_type, subsystem, type) {
        let metrics = [];
        const rows = await MetricsWrapper_1.MetricsWrapper.getMetrics(hw_type, subsystem);
        for (const row of rows) {
            if (type && type === 'top') {
                if (row.top) {
                    metrics.push({
                        longLabel: 'longLabel' in row ? row.longLabel : undefined,
                        label: row.label,
                        metric: row.metric,
                        unit: row.unit,
                        shortcut: row.shortcut,
                        defaultPrefix: row.defaultPrefix,
                        defaultPrecision: row.defaultPrecision
                    });
                }
            }
            else {
                metrics.push({
                    longLabel: 'longLabel' in row ? row.longLabel : undefined,
                    label: row.label,
                    metric: row.metric,
                    unit: row.unit,
                    shortcut: row.shortcut,
                    defaultPrefix: row.defaultPrefix,
                    defaultPrecision: row.defaultPrecision
                });
            }
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(metrics),
        };
    }
    async regex(regex, withParents, withRootParent, strict_acl = true, withDeleted = false) {
        if (!regex.regex || !regex.subsystem || (!regex.class && !regex.hw_type && !regex.device)) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'regex or subsystem is missing in request',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        const hwTypeOrClass = regex.hw_type ? regex.hw_type : regex.class;
        const reg = Array.isArray(regex.regex) ? regex.regex : [regex.regex];
        let items = [];
        let objectItems = [];
        if (hwTypeOrClass && regex.parents) {
            if (regex.parents.regex) {
                objectItems = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegexParentsRegex(regex.parents.regex, hwTypeOrClass, regex.subsystem, reg, regex.parents.intersection, withDeleted);
            }
            else if (regex.parents.item_ids) {
                objectItems = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegexParents(regex.parents.item_ids, hwTypeOrClass, regex.subsystem, reg, regex.parents.intersection, withDeleted);
            }
            else if (regex.parents.tags) {
                objectItems = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegexParentsTag(regex.parents.tags, hwTypeOrClass, regex.subsystem, reg, regex.parents.intersection, withDeleted);
            }
        }
        else if (hwTypeOrClass && regex.device) {
            objectItems = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegexDevice(regex.device, regex.subsystem, reg, withDeleted);
        }
        else if (hwTypeOrClass) {
            objectItems = await ItemRegexWrapper_1.ItemRegexWrapper.getItemsByRegex(hwTypeOrClass, regex.subsystem, reg, withDeleted);
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'hw_type or parent is missing in request',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        if (objectItems.length === 0) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: [],
            };
        }
        if (withParents) {
            let subsysFilter = undefined;
            if (regex.parents) {
                if (regex.parents.item_ids) {
                    subsysFilter = (await this.architectureService.getItemsByIds(regex.parents.item_ids)).map(item => item.subsystem);
                }
                else if (regex.parents.regex) {
                    subsysFilter = regex.parents.regex.map(parent => parent.subsystem);
                }
            }
            const parents = await this.architectureService.findParents(objectItems.map((obj) => {
                return obj.item_id;
            }), subsysFilter);
            for (const item of objectItems) {
                items.push({
                    item_id: item.item_id,
                    label: item.label,
                    subsystem: item.subsystem,
                    parents: parents[item.item_id] ? parents[item.item_id] : [],
                });
            }
        }
        else if (withRootParent) {
            const oItems = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                }, where: {
                    item_id: (0, typeorm_1.In)(objectItems.map((obj) => {
                        return obj.item_id;
                    }))
                },
                withDeleted: withDeleted ? withDeleted : undefined
            });
            items = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(oItems));
            items.forEach(item => item.parents = undefined);
        }
        else {
            items = objectItems.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    subsystem: item.subsystem
                };
            });
        }
        if (regex.linuxExclude) {
            const excludeIds = await (0, Linux_1.getLinuxExcludedIds)();
            items = items.filter((item => !excludeIds.includes(item.item_id)));
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(items),
        };
    }
    async multiRegex(regexes, withParents, strict_acl = true) {
        let items = [];
        for (const regex of regexes) {
            const data = await this.regex(regex, withParents);
            items = items.concat(data.data.filter(item => items.findIndex(t => t.item_id === item.item_id) === -1));
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(items),
        };
    }
    async getAliases() {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(await this.aliasService.getItemsWithAlias()),
        };
    }
    async saveAliases(aliases) {
        await this.aliasService.saveAliases(aliases);
        const items = await this.architectureService.getItemsByIds(aliases.map(item => item.item_id));
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(items.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    subsystem: item.subsystem,
                    hw_type: item.hw_type
                };
            })),
        };
    }
    async importAliases(file) {
        const csv = file.buffer.toString();
        const rows = csv.split("\n");
        let aliases = [];
        let firstRow = true;
        let header = [];
        let c = 0;
        for (const row of rows) {
            const columns = row.split(";");
            let data = {};
            let i = 0;
            for (const column of columns) {
                if (firstRow) {
                    header[i] = column;
                }
                else {
                    data[header[i]] = column;
                }
                i++;
            }
            if (firstRow) {
                firstRow = false;
            }
            c++;
            if (data['item_id'] && data['alias']) {
                aliases.push({
                    item_id: data['item_id'],
                    alias: data['alias']
                });
            }
        }
        if (aliases.length === 0) {
            throw new xormon_exception_1.XormonException("No aliases were imported", common_1.HttpStatus.BAD_REQUEST);
        }
        return await this.saveAliases(aliases);
    }
    async exportAliases() {
        const aliases = Architecture_1.Architecture.sortByLabel(await this.aliasService.getItemsWithAlias());
        let csv = 'item_id;label;alias;';
        for (const alias of aliases) {
            csv += `\n${alias.item_id};${alias.label};${alias.alias};`;
        }
        return csv;
    }
    async deleteAliases() {
        try {
            await this.aliasService.deleteAliases();
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Aliases successfully deleted'
            };
        }
        catch (error) {
            console.error(error);
            throw new xormon_exception_1.XormonException("An error occurred while deleting aliases", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteAlias(itemId) {
        if (await this.aliasService.deleteAlias(itemId)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Alias successfully deleted'
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while deleting aliases", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async itemGroupMetrics(itemGroupMetricRequest, type) {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await ItemGroupWrapper_1.ItemGroupWrapper.getMetricsByData(itemGroupMetricRequest, itemGroupMetricRequest.subsystem, itemGroupMetricRequest.hw_type, itemGroupMetricRequest.class, type),
        };
    }
    async preview(preview, withParents, withRootParent) {
        if (!preview.subsystem || (!preview.regex && !preview.item_ids && !preview.tags) || (!preview.class && !preview.hw_type)) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'request does not contain the necessary data',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        let items = [];
        let objectItems = await ItemGroupWrapper_1.ItemGroupWrapper.getItemsByData(preview, preview.subsystem, preview.hw_type, preview.class);
        if (objectItems.length === 0) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: [],
            };
        }
        if (withParents) {
            let subsysFilter = undefined;
            if (preview.parents) {
                if (preview.parents.item_ids) {
                    subsysFilter = (await this.architectureService.getItemsByIds(preview.parents.item_ids)).map(item => item.subsystem);
                }
                else if (preview.parents.regex) {
                    subsysFilter = preview.parents.regex.map(parent => parent.subsystem);
                }
                else if (preview.parents.tags) {
                    subsysFilter = preview.parents.tags.map(parent => parent.subsystem);
                }
            }
            const parents = await this.architectureService.findParents(objectItems.map((obj) => {
                return obj.item_id;
            }), subsysFilter);
            for (const item of objectItems) {
                items.push({
                    item_id: item.item_id,
                    label: item.label,
                    subsystem: item.subsystem,
                    parents: parents[item.item_id] ? parents[item.item_id] : [],
                });
            }
        }
        else if (withRootParent) {
            const oItems = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    parents: {
                        _parent: true
                    }
                }, where: {
                    item_id: (0, typeorm_1.In)(objectItems.map((obj) => {
                        return obj.item_id;
                    }))
                }
            });
            items = (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(oItems));
            items.forEach(item => item.parents = undefined);
        }
        else {
            items = objectItems.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    subsystem: item.subsystem
                };
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(items),
        };
    }
    async multiPreview(previews, withParents, withRootParent) {
        let items = [];
        for (const preview of previews) {
            const data = await this.preview(preview, withParents, withRootParent);
            items = items.concat(data.data.filter(item => items.findIndex(t => t.item_id === item.item_id) === -1));
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: Architecture_1.Architecture.sortByLabel(items),
        };
    }
    async getDeviceGroupItems(request, _class, hw_type, regex) {
        const user = await User_1.User.getUserByRequest(request, true);
        const groupIds = user.groups.map(group => group.group_id);
        let deviceGroupItems = await this.architectureService.getDeviceGroupItems(_class, hw_type, groupIds);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: deviceGroupItems
        };
    }
    async getDeviceGroupItemsRegex(request, regexPreview) {
        let objectItems = await this.architectureService.getDeviceGroupItemsWithRegex(regexPreview);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: objectItems
        };
    }
};
exports.ArchitectureController = ArchitectureController;
__decorate([
    openapi.ApiOperation({ description: "return classes" }),
    (0, common_1.Get)('v1/classes'),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ClassesResponseDTO }),
    __param(0, (0, common_1.Query)('active')),
    __param(1, (0, common_1.Query)('skip_acl')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean, Object]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "getClasses", null);
__decorate([
    openapi.ApiOperation({ description: "return hw_types in array" }),
    (0, common_1.Get)('v1/hw_types'),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").HwTypesResponseDTO }),
    __param(0, (0, common_1.Query)('active')),
    __param(1, (0, common_1.Query)('class')),
    __param(2, (0, common_1.Query)('type')),
    __param(3, (0, common_1.Query)('skip_acl')),
    __param(4, (0, common_1.Query)('admin_groups')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean, String, String, Object, Object]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "hw_types", null);
__decorate([
    openapi.ApiOperation({ description: "return subsystems" }),
    (0, common_1.Get)('v1/subsystems'),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").SubsystemsResponseDTO }),
    __param(0, (0, common_1.Query)('active')),
    __param(1, (0, common_1.Query)('class')),
    __param(2, (0, common_1.Query)('hw_type')),
    __param(3, (0, common_1.Query)('type')),
    __param(4, (0, common_1.Query)('skip_acl')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Boolean, String, String, String, Object]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "getSubsystems", null);
__decorate([
    openapi.ApiOperation({ description: "return subsystems in array" }),
    (0, common_1.Get)('v1/hw_types/:hw_type/subsystems'),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").SubsystemsResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Query)('active')),
    __param(2, (0, common_1.Query)('type')),
    __param(3, (0, common_1.Query)('skip_acl')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Boolean, String, Object]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "subsystems", null);
__decorate([
    openapi.ApiOperation({ description: "return items by hw_type and subsytem\nDELETE THIS ENDPOINT", deprecated: true }),
    (0, common_1.Get)('v1/hw_types/:hw_type/subsystems/:subsystem/items'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Param)('subsystem')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "itemsByHwTypeAndSubsystem", null);
__decorate([
    openapi.ApiOperation({ description: "return parents subsystems by hw_type and subsytem" }),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    (0, common_1.Get)('v1/hw_types/:hw_type/subsystems/:subsystem/parents'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").SubsystemsResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Param)('subsystem')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "parentsByHwTypeAndSubsystem", null);
__decorate([
    openapi.ApiOperation({ description: "return all items, in the case of multiple parents, separate them with a comma" }),
    (0, common_1.Get)('v1/items'),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Query)('subsystem')),
    __param(1, (0, common_1.Query)('hw_type')),
    __param(2, (0, common_1.Query)('class')),
    __param(3, (0, common_1.Query)('parents')),
    __param(4, (0, common_1.Query)('withParents')),
    __param(5, (0, common_1.Query)('withRootParent')),
    __param(6, (0, common_1.Query)('childSubsystem')),
    __param(7, (0, common_1.Query)('strict_acl')),
    __param(8, (0, common_1.Query)('without_mapping')),
    __param(9, (0, common_1.Query)('withDeleted')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String, Array, Boolean, Boolean, String, Object, Object, Boolean]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "items", null);
__decorate([
    openapi.ApiOperation({ description: "search in items, in the case of multiple parents, separate them with a comma" }),
    (0, common_1.Get)('v1/search'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsSearchResponseDTO }),
    __param(0, (0, common_1.Query)('subsystem')),
    __param(1, (0, common_1.Query)('hw_type')),
    __param(2, (0, common_1.Query)('class')),
    __param(3, (0, common_1.Query)('parents')),
    __param(4, (0, common_1.Query)('search')),
    __param(5, (0, common_1.Query)('page')),
    __param(6, (0, common_1.Query)('limit')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String, Array, String, Number, Number]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "search", null);
__decorate([
    openapi.ApiOperation({ description: "return item labels" }),
    (0, common_1.Post)('v1/labels'),
    openapi.ApiResponse({ status: 201, type: require("../../common/architecture.dto").LabelsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [architecture_dto_1.ItemIds]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "itemsByIds", null);
__decorate([
    (0, common_1.Get)('v1/metrics'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ArchitectureResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "getMetrics", null);
__decorate([
    openapi.ApiOperation({ description: "return metrics by hw_type and subsystem" }),
    (0, common_1.Get)('v1/hw_types/:hw_type/subsystems/:subsystem/metrics'),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").MetricsResponseDTO }),
    __param(0, (0, common_1.Param)('hw_type')),
    __param(1, (0, common_1.Param)('subsystem')),
    __param(2, (0, common_1.Query)('type')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "metricsByHwTypeAndSubsystem", null);
__decorate([
    openapi.ApiOperation({ description: "regex preview" }),
    (0, common_1.Post)('v1/regex'),
    (0, common_1.HttpCode)(200),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Query)('parents')),
    __param(2, (0, common_1.Query)('rootParent')),
    __param(3, (0, common_1.Query)('strict_acl')),
    __param(4, (0, common_1.Query)('withDeleted')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [architecture_dto_1.RegexPreviewDTO, Boolean, Boolean, Object, Boolean]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "regex", null);
__decorate([
    openapi.ApiOperation({ description: "multi regex preview" }),
    (0, swagger_1.ApiBody)({
        isArray: true,
        type: architecture_dto_1.RegexPreviewDTO,
    }),
    (0, common_1.Post)('v1/multiregex'),
    (0, common_1.HttpCode)(200),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Query)('parents')),
    __param(2, (0, common_1.Query)('strict_acl')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, Boolean, Object]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "multiRegex", null);
__decorate([
    (0, common_1.Get)('v1/aliases'),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemAliasesResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "getAliases", null);
__decorate([
    openapi.ApiOperation({ description: "update aliases" }),
    (0, swagger_1.ApiBody)({
        isArray: true,
        type: alias_dto_1.Alias,
    }),
    (0, common_1.Post)('v1/aliases'),
    (0, common_1.HttpCode)(201),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 201, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "saveAliases", null);
__decorate([
    openapi.ApiOperation({ description: "import aliases" }),
    (0, common_1.Post)('v1/aliases/import'),
    (0, common_1.HttpCode)(201),
    (0, common_1.UseInterceptors)((0, platform_express_1.FileInterceptor)('file')),
    openapi.ApiResponse({ status: 201, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.UploadedFile)(new common_1.ParseFilePipe({
        validators: [
            new common_1.FileTypeValidator({ fileType: 'csv' }),
        ]
    }))),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "importAliases", null);
__decorate([
    openapi.ApiOperation({ description: "export aliases" }),
    (0, common_1.Get)('v1/aliases/export'),
    (0, common_1.HttpCode)(200),
    openapi.ApiResponse({ status: 200, type: String }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "exportAliases", null);
__decorate([
    openapi.ApiOperation({ description: "delete all aliases" }),
    (0, common_1.Delete)('v1/alias/'),
    (0, common_1.HttpCode)(201),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "deleteAliases", null);
__decorate([
    openapi.ApiOperation({ description: "delete alias" }),
    (0, common_1.Delete)('v1/alias/:item_id'),
    (0, common_1.HttpCode)(201),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('item_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "deleteAlias", null);
__decorate([
    openapi.ApiOperation({ description: "item group metrics" }),
    (0, common_1.Post)('v1/metrics'),
    (0, common_1.HttpCode)(200),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").MetricsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Query)('type')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [architecture_dto_1.ItemGroupMetricRequest, String]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "itemGroupMetrics", null);
__decorate([
    openapi.ApiOperation({ description: "item group preview" }),
    (0, common_1.Post)('v1/preview'),
    (0, common_1.HttpCode)(200),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Query)('parents')),
    __param(2, (0, common_1.Query)('rootParent')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [architecture_dto_1.ItemGroupPreview, Boolean, Boolean]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "preview", null);
__decorate([
    openapi.ApiOperation({ description: "multi item group preview" }),
    (0, swagger_1.ApiBody)({
        isArray: true,
        type: architecture_dto_1.ItemGroupPreview,
    }),
    (0, common_1.Post)('v1/multipreview'),
    (0, common_1.HttpCode)(200),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Query)('parents')),
    __param(2, (0, common_1.Query)('rootParent')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array, Boolean, Boolean]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "multiPreview", null);
__decorate([
    openapi.ApiOperation({ description: "return only root parents of items" }),
    (0, common_1.Get)('device_group_items'),
    openapi.ApiResponse({ status: 200, type: Object }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('class')),
    __param(2, (0, common_1.Query)('hw_type')),
    __param(3, (0, common_1.Query)('regex')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, Boolean]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "getDeviceGroupItems", null);
__decorate([
    openapi.ApiOperation({ description: "for regex preview" }),
    (0, common_1.Post)('device_group_items/regex'),
    openapi.ApiResponse({ status: 201, type: Object }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, architecture_dto_1.RegexPreviewDTO]),
    __metadata("design:returntype", Promise)
], ArchitectureController.prototype, "getDeviceGroupItemsRegex", null);
exports.ArchitectureController = ArchitectureController = __decorate([
    (0, swagger_1.ApiTags)('Architecture'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/architecture'),
    __metadata("design:paramtypes", [architecture_service_1.ArchitectureService,
        alias_service_1.AliasService])
], ArchitectureController);
//# sourceMappingURL=architecture.controller.js.map