"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArchitectureService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const class_entity_1 = require("../../entity/class.entity");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const hwType_entity_1 = require("../../entity/hwType.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const subsystem_entity_1 = require("../../entity/subsystem.entity");
const ClassesDef = require("../metrics/Classes");
const Classes_1 = require("../metrics/Classes");
const typeorm_2 = require("typeorm");
const App_1 = require("../configuration/App");
const PM2_1 = require("../configuration/PM2");
const Architecture_1 = require("./Architecture");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const Definitions_1 = require("../metrics/Definitions");
const User_1 = require("../users/User");
;
;
const agentMapping_entity_1 = require("../../entity/agentMapping.entity");
const publicApiMap_1 = require("../public/hostcfg/publicApiMap");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
let ArchitectureService = class ArchitectureService {
    constructor(classesRepository, hwTypesRepository, subsystemsRepository, hostCfgRepository, objectItemsRepository, agentMappingRepository, userRepository) {
        this.classesRepository = classesRepository;
        this.hwTypesRepository = hwTypesRepository;
        this.subsystemsRepository = subsystemsRepository;
        this.hostCfgRepository = hostCfgRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.agentMappingRepository = agentMappingRepository;
        this.userRepository = userRepository;
    }
    async onModuleInit() {
        if (!PM2_1.PM2.isRunning() || !PM2_1.PM2.isMasterBackend()) {
            return;
        }
        const classes = Architecture_1.Architecture.getClassesFromMetrics(true);
        for (const _class of classes) {
            await this.classesRepository.save({
                _class: _class.class,
                label: _class.label,
                class_order: _class.order
            });
            let hwTypes = [];
            let subsystems = [];
            for (const hwType of _class.hw_types) {
                hwTypes.push({
                    hw_type: hwType.hw_type,
                    label: hwType.label,
                    class: _class.class,
                    hw_type_order: hwType.order
                });
                const subsys = Architecture_1.Architecture.getSubsystemsFromMetrics(hwType.hw_type);
                for (const sub of subsys) {
                    subsystems.push({
                        hw_type: hwType.hw_type,
                        subsystem: sub.subsystem,
                        label: sub.label,
                        properties: sub.properties
                    });
                }
            }
            await this.classesRepository.save({
                _class: 'xormon',
                label: 'Internal monitoring',
                class_order: 100
            });
            hwTypes = hwTypes.concat([{
                    hw_type: 'application',
                    class: 'xormon',
                    label: 'Internal monitoring'
                }]);
            subsystems = subsystems.concat(Architecture_1.Architecture.getInternalMonitoringSubsystems());
            await this.hwTypesRepository.save(hwTypes);
            await this.subsystemsRepository.save(subsystems);
        }
    }
    async getClasses() {
        const classes = [];
        const activeHwTypes = await App_1.App.getActiveHwTypes();
        for (const _class of await Architecture_1.Architecture.getClasses(true)) {
            classes.push({
                label: _class.label,
                class: _class.class,
                active: _class.hw_types.some(hwType => activeHwTypes.includes(hwType.hw_type))
            });
        }
        return classes;
    }
    async getHwTypes(active, className, list_models) {
        let dict = [];
        const hwTypes = className ? await Architecture_1.Architecture.getHwTypes(false, className) : await Architecture_1.Architecture.getHwTypes();
        if (hwTypes.length === 0) {
            return [];
        }
        if (list_models) {
            const vendorMap = (0, publicApiMap_1.createVendorMapFromObject)();
            for (const hwType of hwTypes) {
                if (vendorMap[hwType.hw_type]) {
                    hwType.matching_models = Object.keys(vendorMap[hwType.hw_type].labels);
                }
            }
        }
        if (!active) {
            return hwTypes;
        }
        for (const hwType of hwTypes) {
            dict.push(hwType.hw_type);
        }
        const hostcfgs = (await this.hostCfgRepository.createQueryBuilder().select('hw_type').distinct(true).getRawMany()).map((obj) => {
            return obj.hw_type;
        });
        const linuxCheck = await this.objectItemsRepository.findOneBy({ hw_type: 'linux' });
        if (linuxCheck) {
            hostcfgs.push('linux');
        }
        const windowsCheck = await this.objectItemsRepository.findOneBy({ hw_type: 'windows' });
        if (windowsCheck) {
            hostcfgs.push('windows');
        }
        const dockerCheck = await this.objectItemsRepository.findOneBy({ hw_type: 'docker' });
        if (dockerCheck) {
            hostcfgs.push('docker');
        }
        const solarisCheck = await this.objectItemsRepository.findOneBy({ hw_type: 'solaris' });
        if (solarisCheck) {
            hostcfgs.push('solaris');
        }
        return hwTypes.filter((hwType) => hostcfgs.includes(hwType.hw_type));
    }
    async getHwType(hwType) {
        return await Architecture_1.Architecture.getHwType(hwType, false);
    }
    async getSubsystem(hwType, subsystem) {
        return await this.subsystemsRepository.findOneBy({ hw_type: hwType, subsystem: subsystem });
    }
    async getSubsystems(active, _class, hw_type, type) {
        let subsystems = [];
        let hwTypes = hw_type ? [hw_type] : [];
        let activeHwTypes = undefined;
        if (active && !['linux', 'windows', 'docker', 'solaris'].includes(hw_type)) {
            activeHwTypes = (await this.hostCfgRepository.createQueryBuilder().select('hw_type').distinct(true).getRawMany()).map((obj) => {
                return obj.hw_type;
            });
        }
        if (_class) {
            const classHwTypes = await this.hwTypesRepository.findBy({ class: _class });
            for (const classHwType of classHwTypes) {
                hwTypes.push(classHwType.hw_type);
            }
        }
        if (activeHwTypes !== undefined) {
            hwTypes = hwTypes.filter(hwType => activeHwTypes.includes(hwType));
        }
        if (!_class && !hw_type) {
            subsystems = await this.subsystemsRepository.find({
                relations: {
                    hw_type_entity: true
                }
            });
            hwTypes = subsystems.map(subsystems => subsystems.hw_type);
        }
        else {
            subsystems = await this.subsystemsRepository.find({
                where: {
                    hw_type: (0, typeorm_2.In)(hwTypes)
                }, relations: {
                    hw_type_entity: true
                }
            });
            hwTypes = subsystems.map(subsystems => subsystems.hw_type);
        }
        if (hwTypes.length === 0) {
            return [];
        }
        if (type && type === 'custom_group') {
            subsystems = subsystems.filter(subsystem => subsystem.properties && subsystem.properties.customGroup === true);
        }
        else if (type && type === 'configuration') {
            subsystems = subsystems.filter(subsystem => (Object.keys(MetricsWrapper_1.MetricsWrapper.getConfiguration(subsystem.hw_type_entity.class, subsystem.hw_type, subsystem.subsystem))).length >= 1 && subsystem.properties.exporter === true);
        }
        else if (type && type === 'aliases') {
            subsystems = subsystems.filter(subsystem => subsystem.properties && subsystem.properties.aliases === true);
        }
        else if (type && type === 'mapping') {
            subsystems = subsystems.filter(subsystem => ['device', 'volume', 'host'].includes(subsystem.subsystem));
        }
        if (type && type === 'alerting' || type === 'metric') {
            subsystems = subsystems.filter(subsystem => subsystem.properties && subsystem.properties.alerting === true);
        }
        else if (type && (type === 'exporter' || type === 'timeseries')) {
            subsystems = subsystems.filter(subsystem => subsystem.properties && subsystem.properties.exporter === true);
        }
        else if (type && (type === 'reporter')) {
            subsystems = subsystems.filter(subsystem => subsystem.properties && subsystem.properties.reporter === true);
        }
        if (active) {
            const activeSubsystems = (await this.objectItemsRepository.createQueryBuilder("object_item")
                .select(['subsystem', 'hw_type'])
                .distinctOn(['subsystem'])
                .where("object_item.hw_type IN (:...hwTypes)", { hwTypes: hwTypes })
                .getRawMany()).map(item => item.subsystem);
            subsystems = subsystems.filter(subsystem => activeSubsystems.includes(subsystem.subsystem));
        }
        if (_class) {
            subsystems = subsystems.filter((value, index, self) => index === self.findIndex((t) => (t.subsystem === value.subsystem)));
        }
        return subsystems.map(subsystem => {
            return {
                label: subsystem.label ? subsystem.label : subsystem.subsystem,
                subsystem: subsystem.subsystem,
                hw_type: !_class ? subsystem.hw_type : undefined,
                hw_type_label: !_class ? subsystem.hw_type_entity.label : undefined,
                class: subsystem.hw_type_entity.class,
            };
        });
    }
    getVirtCrossPlatformParents() {
        return [{
                label: 'ESXi (VMware)',
                subsystem: 'esxi',
                hw_type: 'vmware'
            }, {
                label: 'Server (IBM Power)',
                subsystem: 'server',
                hw_type: 'power'
            }, {
                label: 'Host (Nutanix)',
                subsystem: 'host',
                hw_type: 'nutanix'
            }, {
                label: 'Node (Proxmox)',
                subsystem: 'node',
                hw_type: 'proxmox'
            }, {
                label: 'Server (OracleVM)',
                subsystem: 'server',
                hw_type: 'oraclevm'
            }];
    }
    async getByHwTypeAndSubsystem(hw_type, subsystem) {
        return this.objectItemsRepository.find({ where: { subsystem, hw_type }, relations: { parents: { _parent: true } }, order: { label: 'ASC' } });
    }
    async getItems(attributes) {
        let where = {};
        let relations = {
            'hw_type_entity': true
        };
        if (attributes._class) {
            where['hw_type_entity'] = {
                class: attributes._class
            };
        }
        if (attributes.parents) {
            relations['parents'] = true;
            where['parents'] = {
                parent: (0, typeorm_2.In)(attributes.parents)
            };
        }
        if (attributes.withParents) {
            relations['parents'] = {
                '_parent': true
            };
        }
        if (attributes.childSubsystem) {
            relations['children'] = {
                '_child': true
            };
            where['children'] = {
                '_child': {
                    'subsystem': attributes.childSubsystem
                }
            };
        }
        if (attributes.subsystem) {
            where['subsystem'] = attributes.subsystem;
        }
        if (attributes.hw_type) {
            if (attributes.hw_type === 'storage') {
                const hwTypes = Object.keys(Classes_1.Storage.technology);
                where['hw_type'] = (0, typeorm_2.In)(hwTypes);
            }
            else if (attributes.hw_type === 'san') {
                const hwTypes = Object.keys(Classes_1.San.technology);
                where['hw_type'] = (0, typeorm_2.In)(hwTypes);
            }
            else {
                where['hw_type'] = attributes.hw_type;
            }
        }
        if (attributes.search) {
            where['label'] = (0, typeorm_2.ILike)("%" + attributes.search + "%");
        }
        return this.objectItemsRepository.find({
            withDeleted: attributes.withDeleted ? true : undefined,
            where: where,
            relations: relations,
            order: {
                label: 'ASC'
            }
        });
    }
    async getRootItems(hostcfgRelation = false) {
        return await this.objectItemsRepository.find({
            relations: {
                parents: true,
                hw_type_entity: true,
                hostcfg_relations: hostcfgRelation ? {
                    _hostcfg: true
                } : undefined
            },
            where: [{
                    subsystem: 'device',
                    hw_type_entity: {
                        class: (0, typeorm_2.In)(['storage', 'san', 'lan'])
                    }
                }, {
                    parents: {
                        parent: (0, typeorm_2.IsNull)()
                    },
                    hw_type: (0, typeorm_2.Not)((0, typeorm_2.In)(['timescaledb', 'linux'])),
                    subsystem: (0, typeorm_2.In)(['device', 'server', 'cluster', 'db', 'vcenter', 'host'])
                }]
        });
    }
    async getItem(item_id) {
        return await this.objectItemsRepository.findOneBy({ item_id: item_id });
    }
    async getItemWithParents(item_id) {
        return await this.objectItemsRepository.findOne({
            where: {
                item_id: item_id
            },
            relations: {
                parents: {
                    _parent: true
                },
                hw_type_entity: true
            },
            order: {
                label: 'ASC'
            }
        });
    }
    async getParentsByIds(itemIds) {
        let parents = {};
        const items = await this.objectItemsRepository.find({
            where: {
                children: {
                    child: (0, typeorm_2.In)(itemIds)
                }
            },
            relations: {
                children: true
            },
            order: {
                subsystem: 'ASC',
                label: 'ASC'
            }
        });
        for (const item of items) {
            for (const child of item.children) {
                if (parents[child.child]) {
                    parents[child.child].push({
                        label: item.label,
                        item_id: item.item_id,
                        subsystem: item.subsystem
                    });
                }
                else {
                    parents[child.child] = [{
                            label: item.label,
                            item_id: item.item_id,
                            subsystem: item.subsystem
                        }];
                }
            }
        }
        return parents;
    }
    async getItemsByIds(item_ids, withParents = false) {
        return await this.objectItemsRepository.find({
            where: {
                item_id: (0, typeorm_2.In)(item_ids)
            },
            order: {
                label: 'ASC'
            },
            relations: withParents ? {
                parents: {
                    _parent: true
                }
            } : undefined
        });
    }
    async findParents(itemIds, subsystems) {
        const itemsWithParents = await this.objectItemsRepository.find({
            relations: {
                parents: {
                    _parent: true
                }
            },
            where: {
                item_id: (0, typeorm_2.In)(itemIds),
                parents: subsystems && subsystems.length >= 1 ? {
                    _parent: {
                        subsystem: (0, typeorm_2.In)(subsystems)
                    }
                } : undefined
            }
        });
        let parentDict = {};
        for (const item of itemsWithParents) {
            parentDict[item.item_id] = item.parents.map(parent => {
                return {
                    item_id: parent._parent.item_id,
                    label: parent._parent.label,
                    subsystem: parent._parent.subsystem
                };
            });
        }
        return parentDict;
    }
    getClassesDef() {
        let classes = [];
        for (const _class of Object.keys(ClassesDef)) {
            if (_class === 'Xormon') {
                continue;
            }
            if (_class === 'Storage' || _class === 'San') {
                let hwTypes = {};
                const hwTypesDef = ClassesDef[_class]['technology'];
                for (const hwType in hwTypesDef) {
                    hwTypes[hwType] = {
                        label: hwTypesDef[hwType].label ? hwTypesDef[hwType].label : hwType,
                        order: hwTypesDef[hwType].order ? hwTypesDef[hwType].order : undefined,
                        subsystems: {}
                    };
                    for (const subsystem in hwTypesDef[hwType].subsystems) {
                        let metrics = {};
                        if (hwTypesDef[hwType].subsystems[subsystem]['metrics']) {
                            for (const metric in hwTypesDef[hwType].subsystems[subsystem]['metrics']) {
                                const metricDef = hwTypesDef[hwType].subsystems[subsystem]['metrics'][metric];
                                if (metricDef === undefined) {
                                    console.warn(`[${hwType}] [${subsystem}] [${metric}] Metric definition not found`);
                                    continue;
                                }
                                metrics[metric] = {
                                    metric: metric,
                                    label: metricDef['label'],
                                    unit: metricDef['unit'],
                                    shortcut: metricDef['shortcut'],
                                    defaultPrefix: metricDef['defaultPrefix']
                                        ? metricDef['defaultPrefix']
                                        : undefined
                                };
                            }
                        }
                        else if (hwTypesDef[hwType].subsystems[subsystem]['single']) {
                            const singlePage = hwTypesDef[hwType].subsystems[subsystem]['single'];
                            for (const tab in singlePage) {
                                for (const metricsDef in singlePage[tab]) {
                                    for (const metric in singlePage[tab][metricsDef]) {
                                        if (singlePage[tab][metricsDef][metric]['metric']) {
                                            metrics[singlePage[tab][metricsDef][metric]['metric']] = {
                                                metric: singlePage[tab][metricsDef][metric]['metric'],
                                                label: singlePage[tab][metricsDef][metric]['label'],
                                                unit: singlePage[tab][metricsDef][metric]['unit'],
                                                shortcut: singlePage[tab][metricsDef][metric]['shortcut'],
                                                defaultPrefix: singlePage[tab][metricsDef][metric]['defaultPrefix']
                                                    ? singlePage[tab][metricsDef][metric]['defaultPrefix']
                                                    : undefined
                                            };
                                        }
                                    }
                                }
                            }
                        }
                        hwTypes[hwType]['subsystems'][subsystem] = {
                            label: hwTypesDef[hwType]['subsystems'][subsystem].label ? hwTypesDef[hwType]['subsystems'][subsystem].label : undefined,
                            metrics: metrics
                        };
                    }
                }
                classes.push({
                    class: _class.toLowerCase(),
                    hwTypes: hwTypes
                });
            }
            else {
                classes.push({
                    class: _class.toLowerCase(),
                    hwTypes: ClassesDef[_class]['technology']
                });
            }
        }
        return classes;
    }
    async findSubsystem(subsystem, hwType, _class) {
        return await this.subsystemsRepository.findOne({
            relations: _class ? {
                hw_type_entity: true
            } : undefined,
            where: {
                subsystem: subsystem,
                hw_type: hwType ? hwType : undefined,
                hw_type_entity: _class ? {
                    class: _class
                } : undefined
            }
        });
    }
    async filterAgentMappings(items) {
        const server_ids = items.map((item) => item.item_id);
        const mappings = await this.agentMappingRepository.findBy({ agent_id: (0, typeorm_2.In)(server_ids) });
        const mapped_server_ids = mappings.map((mapping) => mapping.agent_id);
        const servers = items.filter((item) => !mapped_server_ids.includes(item.item_id));
        return servers;
    }
    async getDeviceGroupItems(_class, hw_type, groupIds) {
        let objectItems = await this.getObjectItemsForDeviceGroup();
        if (_class === 'all') {
            return objectItems
                .filter(objectItem => (0, Definitions_1.findRootSubsystem)(objectItem.hw_type_entity.class, objectItem.hw_type))
                .map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label
                };
            });
        }
        else {
            if (hw_type === 'all') {
                return objectItems
                    .filter(objectItem => {
                    return ((0, Definitions_1.findRootSubsystem)(objectItem.hw_type_entity.class, objectItem.hw_type) &&
                        objectItem.hw_type_entity.class === _class);
                })
                    .map(item => {
                    return {
                        item_id: item.item_id,
                        label: item.label
                    };
                });
            }
            else {
                let filteredObjectItems = objectItems
                    .filter(objectItem => {
                    return ((0, Definitions_1.findRootSubsystem)(objectItem.hw_type_entity.class, objectItem.hw_type) &&
                        objectItem.hw_type_entity.class === _class &&
                        objectItem.hw_type === hw_type);
                })
                    .map(item => {
                    return {
                        item_id: item.item_id,
                        label: item.label
                    };
                });
                return filteredObjectItems;
            }
        }
    }
    async getObjectItemsForDeviceGroup() {
        return await this.objectItemsRepository.find({
            relations: {
                parents: true,
                hw_type_entity: true
            },
            where: {
                parents: {
                    parent: (0, typeorm_2.IsNull)()
                },
                hw_type: (0, typeorm_2.Not)((0, typeorm_2.In)([hwTypeKey_constants_1.HwTypeKey.TIMESCALEDB, hwTypeKey_constants_1.HwTypeKey.LINUX, hwTypeKey_constants_1.HwTypeKey.AS400, hwTypeKey_constants_1.HwTypeKey.WINDOWS, hwTypeKey_constants_1.HwTypeKey.SOLARIS])),
                subsystem: (0, typeorm_2.In)([...Object.values(Definitions_1.rootSubsystems), ...['device']])
            },
            order: {
                hw_type: 'ASC'
            }
        });
    }
    async getDeviceGroupItemsWithRegex(regexPreview) {
        let objectItems = await this.getObjectItemsForDeviceGroup();
        let filteredObjectItems = objectItems
            .filter(objectItem => {
            return ((0, Definitions_1.findRootSubsystem)(objectItem.hw_type_entity.class, objectItem.hw_type) &&
                objectItem.hw_type_entity.class === regexPreview.class &&
                objectItem.hw_type === regexPreview.hw_type);
        })
            .map(item => {
            return {
                item_id: item.item_id,
                label: item.label
            };
        });
        return filteredObjectItems.filter(item => new RegExp(regexPreview.regex[0]).test(item.label));
    }
    getVendors() {
        return (0, publicApiMap_1.getVendorKeys)();
    }
    getVendorDeviceLabel(vendor) {
        return (0, publicApiMap_1.getVendorDevice)(vendor);
    }
};
exports.ArchitectureService = ArchitectureService;
exports.ArchitectureService = ArchitectureService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(class_entity_1.Class)),
    __param(1, (0, typeorm_1.InjectRepository)(hwType_entity_1.HwType)),
    __param(2, (0, typeorm_1.InjectRepository)(subsystem_entity_1.Subsystem)),
    __param(3, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(4, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(5, (0, typeorm_1.InjectRepository)(agentMapping_entity_1.AgentMapping)),
    __param(6, (0, typeorm_1.InjectRepository)(User_1.User)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], ArchitectureService);
//# sourceMappingURL=architecture.service.js.map