"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dump = void 0;
const dataSource_1 = require("../../config/dataSource");
const App_1 = require("./App");
const version_entity_1 = require("../../entity/version.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const itemRelation_entity_1 = require("../../entity/itemRelation.entity");
const itemProperty_entity_1 = require("../../entity/itemProperty.entity");
const heatmap_entity_1 = require("../../entity/heatmap.entity");
const microserviceJobsStatus_entity_1 = require("../../entity/microserviceJobsStatus.entity");
const setting_entity_1 = require("../../entity/setting.entity");
const backend_entity_1 = require("../../entity/backend.entity");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const Crypto_1 = require("./Crypto");
const healthStatus_entity_1 = require("../../entity/healthStatus.entity");
const fs = require("fs");
const ldap_entity_1 = require("../../entity/ldap.entity");
const ldapGroup_entity_1 = require("../../entity/ldapGroup.entity");
const acl_entity_1 = require("../../entity/acl.entity");
const storageMapping_entity_1 = require("../../entity/storageMapping.entity");
const audit_entity_1 = require("../../entity/audit.entity");
const Linux_1 = require("../menu/menu/Linux");
const Reporter_1 = require("../reporter/Reporter");
const hostcfgRelation_entity_1 = require("../../entity/hostcfgRelation.entity");
const storageGroup_entity_1 = require("../../entity/storageGroup.entity");
const storageGroupDevice_entity_1 = require("../../entity/storageGroupDevice.entity");
const agentMapping_entity_1 = require("../../entity/agentMapping.entity");
const customGroup_entity_1 = require("../../entity/customGroup.entity");
const log_task_1 = require("../../tasks-server/log.task");
const taskQueue_entity_1 = require("../../entity/taskQueue.entity");
const typeorm_transactional_1 = require("typeorm-transactional");
class Dump {
    constructor(cli = false) {
        this.cli = cli;
        this.ROOT_PATH = App_1.App.ROOT_PATH;
        this.LOGS_PATH = App_1.App.LOGS_PATH;
        this.TMP_PATH = this.ROOT_PATH + "/server-nest/files/tmp";
        this.TMP_FILE_PATH = App_1.App.TMP_FILES_PATH;
        this.DESTINATION_PATH = this.TMP_PATH + "/xormon-logs";
        this.SQL_OUTPUT = null;
        this.filesToDelete = [];
        this.fs = require('fs');
    }
    async sql(sql) {
        let output = "";
        try {
            const forbidden = ['DELETE', 'UPDATE', 'DROP', 'INSERT', 'CREATE', 'ALTER', 'TRUNCATE', 'SET'];
            const regex = new RegExp(forbidden.join("|"), "i");
            if (regex.test(sql)) {
                console.error("[Dump] [SQL] An illegal character was entered when generating the log");
                output = "An illegal character was entered when generating the log. SQL query: " + sql;
            }
            else {
                output = JSON.stringify(await (0, dataSource_1.getDataSource)().query(sql), null, 2);
            }
        }
        catch (e) {
            console.error("[Dump] [SQL] Error: " + e);
            output = e.toString();
        }
        this.SQL_OUTPUT = output;
    }
    async generate() {
        if (this.cli) {
            await (0, dataSource_1.createDataSource)();
            (0, typeorm_transactional_1.initializeTransactionalContext)();
        }
        if (await this.fs.existsSync(this.LOGS_PATH + "/logs.tar.gz")) {
            await this.fs.rmSync(this.LOGS_PATH + "/logs.tar.gz", { recursive: true, force: true });
        }
        try {
            this.trimLogs();
            this.log("[Logs]: trimming logs");
        }
        catch (error) {
            this.log("[Logs]: failed at trimming logs, error: " + error);
        }
        try {
            await this.dumpMigrationTable();
            this.filesToDelete.push(this.LOGS_PATH + "/db_migrations.json");
            this.log("[MigrationTable] saved to " + this.LOGS_PATH + "/db_migrations.json");
        }
        catch (e) {
            this.log("[MigrationTable] error: " + e, true);
        }
        try {
            await this.dumpDbConf();
            this.filesToDelete.push(this.LOGS_PATH + "/db_databaseconf.json");
            this.log("[DatabaseConfTable] saved to " + this.LOGS_PATH + "/db_databaseconf.json");
        }
        catch (e) {
            this.log("[DatabaseConfTable] error: " + e, true);
        }
        try {
            await this.dumpHostcfgTable();
            this.filesToDelete.push(this.LOGS_PATH + "/db_hostcfg.json");
            this.log("[HostcfgTable] saved to " + this.LOGS_PATH + "/db_hostcfg.json");
        }
        catch (e) {
            this.log("[HostcfgTable] error: " + e, true);
        }
        try {
            await this.dumpTaskQueueTable();
            this.filesToDelete.push(this.LOGS_PATH + "/db_task_queue.json");
            this.log("[TaskQueueTable] saved to " + this.LOGS_PATH + "/db_task_queue.json");
        }
        catch (e) {
            this.log("[TaskQueueTable] error: " + e, true);
        }
        for (const table of this.getTables()) {
            try {
                let data = await (0, dataSource_1.getDataSource)().getRepository(table.entity).find();
                if (table.tablename === 'ldaps') {
                    data = this.hidePass(data);
                }
                else if (table.tablename === "audits") {
                    data = data.map(row => { return { ...row, ip: null }; });
                }
                await this.fs.writeFileSync(this.LOGS_PATH + "/db_" + table.tablename + ".json", JSON.stringify(data));
                this.filesToDelete.push(this.LOGS_PATH + "/db_" + table.tablename + ".json");
                this.log("[" + table.tablename + "]: saved to " + this.LOGS_PATH + "/db_" + table.tablename + ".json");
            }
            catch (e) {
                this.log("[" + table.tablename + "]: skipped, error: " + e, true);
            }
        }
        try {
            await this.dumpPM2List();
            this.filesToDelete.push(this.LOGS_PATH + "/pm2_list.json");
            await this.dumpPM2Config();
            this.filesToDelete.push(this.LOGS_PATH + "/pm2_config.json");
            this.log("[PM2] saved to " + this.LOGS_PATH + "/pm2_config.json && " + this.LOGS_PATH + "/pm2_list.json");
        }
        catch (e) {
            this.log("[PM2]: skipped, error: " + e, true);
        }
        try {
            await this.dumpTasks();
            this.filesToDelete.push(this.LOGS_PATH + "/db_tasks.json");
            await this.dumpJobs();
            this.filesToDelete.push(this.LOGS_PATH + "/db_jobs.json");
            this.log("[Tasks & Jobs] saved to " + this.LOGS_PATH + "/db_tasks.json && " + this.LOGS_PATH + "/db_jobs.json");
        }
        catch (e) {
            this.log("[Tasks & Jobs]: skipped, error: " + e, true);
        }
        try {
            await this.dumpLicense();
            this.log("[License]: saved to " + this.LOGS_PATH + "/license.json");
            this.filesToDelete.push(this.LOGS_PATH + "/license.json");
        }
        catch (e) {
            this.log("[License]: skipped, error: " + e, true);
        }
        try {
            await this.dumpSysLog();
            this.filesToDelete.push(this.LOGS_PATH + "/sys.json");
            this.filesToDelete.push(this.LOGS_PATH + "/ps.txt");
            this.log("[Sys]: saved to " + this.LOGS_PATH + "/sys.json");
            this.log("[Ps]: saved to " + this.LOGS_PATH + "/ps.txt");
        }
        catch (e) {
            this.log("[Sys & Ps]: skipped, error: " + e, true);
        }
        try {
            await this.dumpPostgresConf();
            this.log("[PostgresConf]: saved to " + this.LOGS_PATH + "/postgres.conf");
        }
        catch (e) {
            this.log("[PostgresConf]: skipped, error: " + e, true);
        }
        try {
            await this.generateReport();
            this.filesToDelete.push(this.LOGS_PATH + "/report.pdf");
            this.log("[Report]: saved to " + this.LOGS_PATH + "/report.pdf");
        }
        catch (e) {
            this.log("[Report]: skipped, error: " + e, true);
        }
        if (this.cli) {
            const tar = require('tar');
            if (!fs.existsSync(this.TMP_FILE_PATH)) {
                fs.mkdirSync(this.TMP_FILE_PATH);
            }
            try {
                await tar.create({
                    gzip: true,
                    file: this.TMP_FILE_PATH + "/logs.tar.gz",
                    cwd: this.LOGS_PATH,
                }, fs.readdirSync(this.LOGS_PATH));
                this.filesToDelete.forEach(file => {
                    if (fs.existsSync(file)) {
                        fs.rmSync(file);
                    }
                });
            }
            catch (error) {
                console.log(error);
            }
        }
    }
    hidePass(rows) {
        for (const index in rows) {
            const row = rows[index];
            for (const key in row) {
                if (key === 'password' || key === 'pass') {
                    row[key] = null;
                }
            }
        }
        return rows;
    }
    async gzip() {
        const tar = require('tar');
        try {
            fs.cpSync(this.LOGS_PATH, this.DESTINATION_PATH, { recursive: true });
            if (this.SQL_OUTPUT) {
                await this.fs.writeFileSync(this.DESTINATION_PATH + "/sql.txt", this.SQL_OUTPUT);
                this.filesToDelete.push(this.LOGS_PATH + "/sys.json");
            }
            if (fs.existsSync(this.TMP_PATH + "/logs.tar.gz")) {
                fs.rmSync(this.TMP_PATH + "/logs.tar.gz");
            }
            await tar.create({
                gzip: true,
                file: this.TMP_PATH + "/logs.tar.gz",
                cwd: this.TMP_PATH,
            }, ["xormon-logs"]);
            fs.rmSync(this.DESTINATION_PATH, { recursive: true });
            this.filesToDelete.forEach(file => { console.log(file); fs.rmSync(file); });
        }
        catch (error) {
            console.log(error);
        }
        return this.TMP_PATH + "/logs.tar.gz";
    }
    log(message, error = false) {
        if (error) {
            console.error("[Dump] " + message);
        }
        else if (this.cli) {
            console.log("[Dump] " + message);
        }
    }
    trimLogs() {
        (0, log_task_1.trimLogs)();
    }
    async dumpMigrationTable() {
        const data = await (0, dataSource_1.getDataSource)().query("SELECT * FROM public.migrations");
        await this.fs.writeFileSync(this.LOGS_PATH + "/db_migrations.json", JSON.stringify(data, null, 2));
    }
    async dumpDbConf() {
        const data = await (0, dataSource_1.getDataSource)().query("SELECT * FROM public.item_properties WHERE item_id = 'selfmonitdb'");
        await this.fs.writeFileSync(this.LOGS_PATH + "/db_atabaseconf.json", JSON.stringify(data, null, 2));
    }
    async dumpHostcfgTable() {
        const data = await (0, dataSource_1.getDataSource)().getRepository(hostcfg_entity_1.HostCfg).find();
        for (const cfg of data) {
            cfg.data = Crypto_1.Crypto.hidePasswords(cfg.data);
            cfg.data = this.findAndHide(cfg.data);
        }
        await this.fs.writeFileSync(this.LOGS_PATH + "/db_hostcfg.json", JSON.stringify(data, null, 2));
    }
    async dumpTaskQueueTable() {
        const data = await (0, dataSource_1.getDataSource)().getRepository(taskQueue_entity_1.TaskQueue).find();
        await this.fs.writeFileSync(this.LOGS_PATH + "/db_task_queue.json", JSON.stringify(data, null, 2));
    }
    async dumpPM2Config() {
        await this.fs.writeFileSync(this.LOGS_PATH + "/pm2_config.json", JSON.stringify(require(this.ROOT_PATH + "/server-nest/production.config.js")), null, 2);
    }
    async dumpLicense() {
        if (this.fs.existsSync(this.ROOT_PATH + "/server-nest/files/license/xormon.lic")) {
            await this.fs.copyFileSync(this.ROOT_PATH + "/server-nest/files/license/xormon.lic", this.LOGS_PATH + "/license.json");
        }
    }
    async dumpPM2List() {
        const { promisify } = require('util');
        const exec = promisify(require('child_process').exec);
        const command = await exec('npx pm2 list');
        await this.fs.writeFileSync(this.LOGS_PATH + "/pm2_list.txt", command.stdout);
    }
    execStdout() {
        const { promisify } = require('util');
        const exec = promisify(require('child_process').exec);
        return async function e(command) {
            try {
                return (await exec(command)).stdout;
            }
            catch (error) {
                if (error.code === 1)
                    return error.stdout;
                return "command failed";
            }
        };
    }
    async dumpPostgresConf() {
        const { promisify } = require('util');
        const exec = promisify(require('child_process').exec);
        const query = await (0, dataSource_1.getDataSource)().query("SHOW config_file;");
        const command = (await exec('cat ' + query[0].config_file)).stdout;
        await this.fs.writeFileSync(this.LOGS_PATH + "/postgres.conf", command);
    }
    async dumpSysLog() {
        const exec = this.execStdout();
        const command = await exec('df -h');
        const os = require('os');
        const systemRelease = await exec("cat /etc/system-release");
        let filesystem = [];
        const rows = command.split("\n");
        let isHeader = true;
        let header = [];
        for (const row of rows) {
            const columns = row.split(" ");
            let i = 0;
            let filesystemRow = {};
            for (const column of columns) {
                if (column == "") {
                    continue;
                }
                if (isHeader) {
                    header.push(column);
                }
                else {
                    filesystemRow[header[i]] = column;
                }
                i++;
            }
            filesystem.push(filesystemRow);
            if (isHeader) {
                isHeader = false;
            }
        }
        const id = (await exec("id")).trimEnd();
        const microservices = (await exec("ls -lR " + App_1.App.ROOT_PATH + "/server-nest/microservices" + " | grep '\\.pm\\|.\\pl' | grep -v Xormon")).split("\n");
        const perlVersion = (await exec("perl --version")).match(/[a-zA-Z]{1}\d{1,2}\.\d{1,2}\.\d{1,3}/);
        const opensslVersion = await exec("openssl version");
        let moduleVersions = await exec("dpkg --list liblwp-protocol-https-perl libcrypt-ssleay-perl libio-socket-ssl-perl libmozilla-ldap-perl");
        if (moduleVersions === "command failed") {
            moduleVersions = await exec("rpm -q perl-LWP-Protocol-https perl-Crypt-SSLeay perl-Mozilla-CA perl-libwww-perl");
        }
        let awsVersion = await exec("aws --version");
        const awsPath = awsVersion ? await exec("which aws") : "No path to AWS executable";
        awsVersion = awsVersion ? awsVersion : "AWS not installed";
        let gcloudVersion = await exec("gcloud version");
        const gcloudPath = awsVersion ? await exec("which gcloud") : "No path to GCloud executable";
        gcloudVersion = gcloudVersion ? gcloudVersion : "Gcloud not installed";
        const perlModules = moduleVersions.split("\n").filter(v => v !== "");
        const dbConf = await (0, dataSource_1.databaseConfiguration)();
        let data = {
            arch: os.arch(),
            platform: os.platform(),
            cpu: os.cpus().map(cpu => {
                return {
                    model: cpu.model,
                    speedInMhz: cpu.speed
                };
            }),
            mem_total: os.totalmem(),
            mem_free: os.freemem(),
            uptime: this.parseUptime(os.uptime()),
            date: new Date(Date.now()),
            version: os.version(),
            release: (systemRelease === "command failed" ? "unknown" : systemRelease),
            filesystem: filesystem,
            microservices: microservices,
            node: process.version.match(/^v(\d+\.\d+)/)[1],
            user: os.userInfo().username,
            id: id,
            database: {
                server: dbConf.host,
                port: dbConf.port,
                database: dbConf.database,
                version: await App_1.App.getTimescaleVersion(),
            },
            perl: (perlVersion ? perlVersion[0] : "No perl version found"),
            perlModules: perlModules,
            openssl: opensslVersion.trimEnd(),
            gcloudVersion: gcloudVersion,
            gcloudPath: gcloudPath,
            awsVersion: awsVersion,
            awsPath: awsPath,
            ng_version: App_1.App.getVersion()
        };
        await this.fs.writeFileSync(this.LOGS_PATH + "/sys.json", JSON.stringify(data, null, 2));
        const ps = await exec('ps uxf');
        await this.fs.writeFileSync(this.LOGS_PATH + "/ps.txt", ps);
    }
    parseUptime(seconds) {
        const ut_seconds = Math.floor(seconds % 60);
        let ut_minutes = Math.floor(seconds / 60);
        let ut_hours = Math.floor(ut_minutes / 60);
        const ut_days = Math.floor(ut_hours / 24);
        ut_minutes %= 60;
        ut_hours %= 24;
        return `Uptime: ${ut_days} days, ${ut_hours} hours, ${ut_minutes} minutes, ${ut_seconds} seconds`;
    }
    async dumpTasks() {
        const data = await (0, dataSource_1.getDataSource)().query(`select * from metrics.metrics_application_task where "time" > '${new Date(Date.now() - 7 * 24 * 60 * 60 * 1000).toISOString()}'`);
        await this.fs.writeFileSync(this.LOGS_PATH + "/db_tasks.json", JSON.stringify(data, null, 2));
    }
    async dumpJobs() {
        const data = await (0, dataSource_1.getDataSource)().query(`SELECT ca.view_name, ca.view_definition, j.proc_name, j.schedule_interval, j.config, j.next_start, js.last_run_duration, js.total_runs, js.total_successes, js.total_failures  FROM timescaledb_information.continuous_aggregates ca INNER JOIN timescaledb_information.jobs j ON ca.materialization_hypertable_name = j.hypertable_name INNER JOIN timescaledb_information.job_stats js on j.job_id = js.job_id order by ca.view_name`);
        await this.fs.writeFileSync(this.LOGS_PATH + "/db_jobs.json", JSON.stringify(data, null, 2));
    }
    findAndHide(data) {
        const propertiesToHide = ['host', 'apiHost', 'sshHost', 'managementUrl'];
        for (const key in data) {
            if (typeof data[key] === 'object') {
                data[key] = this.findAndHide(data[key]);
            }
            else {
                if (propertiesToHide.includes(key) || this.isValidIp(data[key])) {
                    data[key] = null;
                }
            }
        }
        return data;
    }
    isValidIp(ip) {
        const regexExp = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/gi;
        return regexExp.test(ip);
    }
    async generateReport() {
        const item = await (0, Linux_1.getSelfMonitoringAgent)();
        if (!item) {
            return;
        }
        let disks = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                hw_type: 'linux',
                subsystem: 'disk_total',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        let jobs = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                hw_type: 'linux',
                subsystem: 'job',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        const now = Date.now();
        const report = await Reporter_1.Reporter.generateReport({
            label: "Xormon VM",
            report_items: [
                {
                    item_index: 0,
                    format: 'graph',
                    type: 'metric',
                    class: 'pdf_export',
                    hw_type: 'linux',
                    subsystem: 'server',
                    data: {
                        metrics: ['cpu_sys', 'cpu_usr', 'cpu_wait', 'cpu_idle', 'mem_free', 'mem4', 'mem_used', 'mem_fs_cache'],
                        item_ids: [item.item_id],
                        stacked: false,
                        time_range: {
                            date_from: new Date(now - 24 * 60 * 60 * 1000),
                            date_to: new Date(now)
                        }
                    }
                },
                {
                    item_index: 1,
                    format: 'graph',
                    type: 'metric',
                    class: 'pdf_export',
                    hw_type: 'linux',
                    subsystem: 'server',
                    data: {
                        metrics: ['read_iops', 'write_iops', 'read_data', 'write_data', 'read_latency', 'write_latency'],
                        item_ids: disks.map(disk => disk.item_id),
                        stacked: true,
                        time_range: {
                            date_from: new Date(now - 24 * 60 * 60 * 1000),
                            date_to: new Date(now)
                        }
                    }
                },
                {
                    item_index: 2,
                    format: 'graph',
                    type: 'metric',
                    class: 'pdf_export',
                    hw_type: 'application',
                    subsystem: 'task',
                    data: {
                        metrics: ['time_difference', 'rss'],
                        item_ids: jobs.map(job => job.item_id),
                        stacked: true,
                        time_range: {
                            date_from: new Date(now - 7 * 24 * 60 * 60 * 1000),
                            date_to: new Date(now)
                        }
                    }
                },
                {
                    item_index: 3,
                    format: 'graph',
                    type: 'metric',
                    class: 'pdf_export',
                    hw_type: 'timescaledb',
                    subsystem: 'db',
                    data: {
                        metrics: ['numbackends', 'deadlocks_max', 'size', 'relation', 'extend', 'page', 'tuple', 'object', 'userlock'],
                        item_ids: ["selfmonitdb"],
                        stacked: false,
                        time_range: {
                            date_from: new Date(now - 7 * 24 * 60 * 60 * 1000),
                            date_to: new Date(now)
                        }
                    }
                }
            ]
        });
        if (report.relative_path) {
            await this.fs.copyFileSync(App_1.App.ROOT_PATH + "/server-nest/files/reporter/" + report.relative_path, this.LOGS_PATH + "/report.pdf");
            await this.fs.unlinkSync(App_1.App.ROOT_PATH + "/server-nest/files/reporter/" + report.relative_path);
        }
    }
    getTables() {
        return [
            {
                "entity": version_entity_1.Version,
                "tablename": "versions"
            },
            {
                "entity": objectItem_entity_1.ObjectItem,
                "tablename": "object_items"
            },
            {
                "entity": itemRelation_entity_1.ItemRelation,
                "tablename": "item_relations"
            },
            {
                "entity": itemProperty_entity_1.ItemProperty,
                "tablename": "item_properties"
            },
            {
                "entity": heatmap_entity_1.Heatmap,
                "tablename": "heatmaps"
            },
            {
                "entity": microserviceJobsStatus_entity_1.MicroserviceJobStatus,
                "tablename": "microservice_job_status"
            },
            {
                "entity": setting_entity_1.Setting,
                "tablename": "settings"
            },
            {
                "entity": healthStatus_entity_1.HealthStatus,
                "tablename": "health_status"
            },
            {
                "entity": backend_entity_1.Backend,
                "tablename": "backend"
            },
            {
                "entity": ldap_entity_1.Ldap,
                "tablename": "ldaps"
            },
            {
                "entity": ldapGroup_entity_1.LdapGroup,
                "tablename": "ldap_groups"
            },
            {
                "entity": acl_entity_1.ACL,
                "tablename": "acl"
            },
            {
                "entity": storageMapping_entity_1.StorageMapping,
                "tablename": "storage_mappings"
            },
            {
                "entity": storageGroup_entity_1.StorageGroup,
                "tablename": "storage_groups"
            },
            {
                "entity": storageGroupDevice_entity_1.StorageGroupDevice,
                "tablename": "storage_group_devices"
            },
            {
                "entity": audit_entity_1.Audit,
                "tablename": "audits"
            },
            {
                "entity": hostcfgRelation_entity_1.HostcfgRelation,
                "tablename": "hostcfg_relations"
            },
            {
                "entity": agentMapping_entity_1.AgentMapping,
                "tablename": "agent_mappings"
            },
            {
                "entity": customGroup_entity_1.CustomGroup,
                "tablename": "custom_groups"
            }
        ];
    }
}
exports.Dump = Dump;
//# sourceMappingURL=Dump.js.map