"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Item = void 0;
const dataSource_1 = require("../../config/dataSource");
const itemRelation_entity_1 = require("../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
const Linux_1 = require("../menu/menu/Linux");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const hostcfgRelation_entity_1 = require("../../entity/hostcfgRelation.entity");
const storageGroupDevice_entity_1 = require("../../entity/storageGroupDevice.entity");
const top_entity_1 = require("../../entity/top.entity");
const heatmap_entity_1 = require("../../entity/heatmap.entity");
const WinstonLogger_1 = require("../log/WinstonLogger");
const class_constants_1 = require("../../common/constants/class.constants");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
const TagWrapper_1 = require("../tag/TagWrapper");
class Item {
    constructor(item_id) {
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.item_id = item_id;
    }
    async getItem() {
        if (await this.checkItem()) {
            return this.item;
        }
        else {
            return null;
        }
    }
    async getLabel() {
        if (await this.checkItem()) {
            return this.item.label;
        }
        else {
            return this.item_id;
        }
    }
    async findParent(parentHwType, parentSubsystem) {
        const parent = await (0, dataSource_1.getDataSource)()
            .createQueryBuilder(itemRelation_entity_1.ItemRelation, 'ir')
            .select('ir.child', 'item_id')
            .addSelect('oi.item_id', 'parent_id')
            .addSelect('oi.label', 'parent_label')
            .innerJoin(objectItem_entity_1.ObjectItem, 'oi', 'ir.parent = oi.item_id')
            .where('ir.child = :child', { child: this.item_id })
            .andWhere('oi.hw_type = :hw_type', { hw_type: parentHwType })
            .andWhere('oi.subsystem = :subsystem', { subsystem: parentSubsystem })
            .getRawOne();
        if (parent) {
            return parent;
        }
        else {
            return {
                parent_id: null,
                item_id: null,
                parent_label: null,
            };
        }
    }
    async findChildren(childrenHwType, childrenSubsystem) {
        const children = await (0, dataSource_1.getDataSource)()
            .createQueryBuilder(itemRelation_entity_1.ItemRelation, 'ir')
            .select('ir.parent', 'item_id')
            .addSelect('oi.item_id', 'child_id')
            .addSelect('oi.label', 'child_label')
            .innerJoin(objectItem_entity_1.ObjectItem, 'oi', 'ir.child = oi.item_id')
            .where('ir.parent = :parent', { parent: this.item_id })
            .andWhere('oi.hw_type = :hw_type', { hw_type: childrenHwType })
            .andWhere('oi.subsystem = :subsystem', { subsystem: childrenSubsystem })
            .getRawMany();
        if (children) {
            return children;
        }
        else {
            return [];
        }
    }
    async checkItem() {
        if (!this.item) {
            const item = await this.objectItemsRepository.findOneBy({ item_id: this.item_id });
            if (item) {
                this.item = item;
                return true;
            }
            else {
                return false;
            }
        }
        else {
            return false;
        }
    }
    static async search(word, regex = false, tag = false) {
        const objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        const excludedIds = await (0, Linux_1.getLinuxExcludedIds)();
        const excludedHwTypes = ['timescaledb', 'application', 'database'];
        const exclude = {
            [class_constants_1.ClassKey.STORAGE]: {
                subsystems: [
                    'cache',
                    'cpu',
                    'ddboost',
                    'drive',
                    'lan',
                    'mdisk',
                    'mtree',
                    'port_mapping',
                    'raid_group',
                    'replication_group_link',
                    'san',
                    'system',
                    'vvol'
                ]
            },
            [class_constants_1.ClassKey.VIRTUALIZATION]: {
                subsystems: ['vm_folder'],
                hwTypes: {
                    [hwTypeKey_constants_1.HwTypeKey.LINUX]: ['disk_total', 'job', 'lan', 'lan_error', 'san', 'san_error', 'san_resp', 'sea'],
                    [hwTypeKey_constants_1.HwTypeKey.NUTANIX]: ['disk', 'vdisk'],
                    [hwTypeKey_constants_1.HwTypeKey.ORACLEVM]: ['server_disk', 'vm_disk'],
                    [hwTypeKey_constants_1.HwTypeKey.POWER]: [
                        'backingsea',
                        'fiber',
                        'freelinkaggregation',
                        'generic',
                        'genericphysical',
                        'genericv',
                        'genericvirtual',
                        'hea',
                        'ioadapter',
                        'iobus',
                        'ioslot',
                        'physicalvolume',
                        'profileioslot',
                        'sea',
                        'sriovlogical',
                        'sriovphysical',
                        'trunk',
                        'virtualethernet',
                    ],
                    [hwTypeKey_constants_1.HwTypeKey.VMWARE]: ['datastore_vm', 'esxi_vm'],
                    [hwTypeKey_constants_1.HwTypeKey.WINDOWS]: ['fixed_disk', 'job'],
                    [hwTypeKey_constants_1.HwTypeKey.KUBERNETES]: ['pod', 'container'],
                    [hwTypeKey_constants_1.HwTypeKey.OPENSHIFT]: ['pod', 'container'],
                    [hwTypeKey_constants_1.HwTypeKey.SOLARIS]: ['sanldom', 'netldom'],
                },
            },
        };
        function getAllSubsystems(exclude) {
            const allSubsystems = [];
            for (const className of Object.keys(exclude)) {
                const classData = exclude[className];
                if (classData.subsystems) {
                    allSubsystems.push(...classData.subsystems);
                }
                if (classData.hwTypes) {
                    for (const hwType of Object.keys(classData.hwTypes)) {
                        const subsystems = classData.hwTypes[hwType];
                        allSubsystems.push(...subsystems);
                    }
                }
            }
            return [...new Set(allSubsystems)];
        }
        const baseWhereClause = {
            alias: (0, typeorm_1.IsNull)(),
            hw_type: (0, typeorm_1.Not)((0, typeorm_1.In)(excludedHwTypes)),
            subsystem: (0, typeorm_1.Not)((0, typeorm_1.In)(getAllSubsystems(exclude))),
            item_id: (0, typeorm_1.Not)((0, typeorm_1.In)(excludedIds))
        };
        let whereClause;
        if (regex) {
            const regexPattern = `.*${word}.*`;
            whereClause = [
                {
                    ...baseWhereClause,
                    label: (0, typeorm_1.Raw)(label => `${label} ~ '${regexPattern}'`)
                }, {
                    ...baseWhereClause,
                    alias: (0, typeorm_1.Raw)(alias => `${alias} ~ '${regexPattern}'`),
                }
            ];
        }
        else if (tag) {
            let [tag, value] = word.split(':');
            if (!value) {
                value = 'self';
            }
            let items = await TagWrapper_1.TagWrapper.tagSearch(tag, value);
            return items.sort((a, b) => {
                const aMatch = a.label === word;
                const bMatch = b.label === word;
                const res = +bMatch - +aMatch;
                if (res === 0) {
                    const aStarts = a.label.startsWith(word);
                    const bStarts = b.label.startsWith(word);
                    return +bStarts - +aStarts;
                }
                else {
                    return res;
                }
            });
        }
        else {
            whereClause = [
                {
                    ...baseWhereClause,
                    label: (0, typeorm_1.ILike)(`%${word}%`),
                }, {
                    ...baseWhereClause,
                    alias: (0, typeorm_1.ILike)(`%${word}%`),
                }
            ];
        }
        return (await objectItemsRepository.find({
            where: whereClause,
            relations: {
                hw_type_entity: true,
                parents: {
                    _parent: true
                }
            }
        })).sort((a, b) => {
            const aMatch = a.label === word;
            const bMatch = b.label === word;
            const res = +bMatch - +aMatch;
            if (res === 0) {
                const aStarts = a.label.startsWith(word);
                const bStarts = b.label.startsWith(word);
                return +bStarts - +aStarts;
            }
            else {
                return res;
            }
        });
    }
    static async getMetrics(itemIds, exclude = undefined) {
        let metrics = [];
        const items = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findBy({ item_id: (0, typeorm_1.In)(itemIds) });
        let hwTypes = [];
        let subsystem;
        for (const item of items) {
            if (!subsystem) {
                subsystem = item.subsystem;
            }
            if (!hwTypes.includes(item.hw_type)) {
                hwTypes.push(item.hw_type);
            }
        }
        for (const hwType of hwTypes) {
            const hMetrics = await MetricsWrapper_1.MetricsWrapper.getMetrics(hwType, subsystem, exclude);
            if (metrics.length === 0) {
                metrics = hMetrics;
                continue;
            }
            else {
                for (const hMetric of hMetrics) {
                    if (metrics.find(metric => metric.metric === hMetric.metric) === undefined) {
                        metrics.push(hMetric);
                    }
                }
            }
        }
        return metrics;
    }
    static async removeDeviceInfo(id) {
        const item = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOneBy({ item_id: id });
        const label = item ? item.label : id;
        const logger = (0, WinstonLogger_1.LoggerFactory)('cleanup');
        let hostsIds = [];
        let hcfgrels = await (0, dataSource_1.getDataSource)().getRepository(hostcfgRelation_entity_1.HostcfgRelation).find({ select: ["item_id"], where: { hostcfg_id: id } });
        let ids = hcfgrels && hcfgrels.length > 0 ? hcfgrels.map(hcfgrel => hcfgrel.item_id) : [];
        hostsIds.push(...ids);
        let deviceId = hostsIds && hostsIds.length === 1 ? hostsIds[0] : id;
        let children = await (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation).find({ select: ["child"], where: { parent: deviceId } });
        let childrenIds = children && children.length > 0 ? children.map(relation => relation.child) : [];
        hostsIds.push(...childrenIds);
        hostsIds.push(deviceId);
        try {
            (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).delete({ item_id: (0, typeorm_1.In)(hostsIds) });
            (0, dataSource_1.getDataSource)().getRepository(storageGroupDevice_entity_1.StorageGroupDevice).delete({ item_id: (0, typeorm_1.In)(hostsIds) });
            (0, dataSource_1.getDataSource)().getRepository(top_entity_1.Top).delete({ item_id: (0, typeorm_1.In)(hostsIds) });
            (0, dataSource_1.getDataSource)().getRepository(heatmap_entity_1.Heatmap).delete({ item_id: (0, typeorm_1.In)(hostsIds) });
            logger.log('info', `[removeDeviceInfo] Removed info for: ${label}`);
        }
        catch (error) {
            logger.log('error', `[removeDeviceInfo] Error removing info for: ${label}, Error: ${error.message}`);
        }
    }
}
exports.Item = Item;
//# sourceMappingURL=Item.js.map