"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PM2 = void 0;
class PM2 {
    static async describeService(name) {
        const service = await this.describeServiceCall(name);
        if (service && service[0]) {
            return {
                active: service[0].pm2_env.status === 'online' ? true : false,
                pid: service[0].pid,
                version: service[0].pm2_env.version,
                monitor: {
                    memory_byte: service[0].monit.memory,
                    cpu_percent: service[0].monit.cpu,
                }
            };
        }
        else {
            return {
                active: false,
                pid: null,
                version: null,
                monitor: {
                    memory_byte: null,
                    cpu_percent: null,
                }
            };
        }
    }
    static async startService(service) {
        return new Promise((resolve, reject) => {
            this.pm2.start(service, (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
    static async stopService(service) {
        return new Promise((resolve, reject) => {
            this.pm2.stop(service, (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
    static async restartService(service) {
        return new Promise((resolve, reject) => {
            this.pm2.start(service, (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
    static async listServices() {
        return new Promise((resolve, reject) => {
            this.pm2.list((err, res) => {
                if (err) {
                    reject(err);
                }
                resolve(res);
            });
        });
    }
    static async describeServiceCall(name) {
        return new Promise((resolve, reject) => {
            this.pm2.describe(name, (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
    static async registerService(data) {
        return new Promise((resolve, reject) => {
            this.pm2.start(data, (err, proc) => {
                if (err) {
                    reject(err);
                }
                resolve(proc);
            });
        });
    }
    static async getProcessId(name) {
        const services = await this.describeServiceCall(name);
        return services.map(service => service.pm_id);
    }
    static isRunning() {
        return process.env.pm_id !== undefined;
    }
    static isMasterBackend() {
        return process.env.pm_id === '0';
    }
    static async shouldILive() {
        const process = require('process');
        const self = await this.describeService(process.env.pm_id);
        return self.pid === process.pid;
    }
    static async commitSuicide() {
        const process = require('process');
        process.kill(process.pid, 15);
    }
    static async sendMessage(processName, data) {
        if (!PM2.isRunning()) {
            return;
        }
        const processIds = await this.getProcessId(processName);
        for (const processId of processIds) {
            this.pm2.sendDataToProcessId(processId, {
                id: processId,
                type: 'process:msg',
                data: data,
                topic: 'message'
            }, (err) => {
                if (err) {
                    console.error(`[PM2 sendMessage] Error: ${err}`);
                }
            });
        }
    }
    static parseMessage(message) {
        return message.data;
    }
    static parseMessageTask(message) {
        return message.data;
    }
    static async scale(service, count) {
        const cp = require('child_process');
        const output = await cp.spawnSync('npx', ['pm2', 'scale', service, `${count}`], { encoding: 'utf8' });
    }
}
exports.PM2 = PM2;
PM2.pm2 = require('pm2');
//# sourceMappingURL=PM2.js.map