"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VmWareSDK = void 0;
const path_1 = require("path");
const Microservice_1 = require("../microservices/Microservice");
const App_1 = require("./App");
const Settings_1 = require("./Settings");
class VmWareSDK {
    static async install() {
        try {
            const UPGRADE_PATH = App_1.App.isDockerVersion() ? this.DOCKER_SDK_PATH : this.SDK_PATH;
            if (this.fs.existsSync(UPGRADE_PATH)) {
                await this.fs.rmSync(UPGRADE_PATH, { recursive: true, force: true });
            }
            await this.fs.mkdirSync(UPGRADE_PATH);
            await this.fs.mkdirSync(UPGRADE_PATH + '/apps');
            await this.fs.mkdirSync(UPGRADE_PATH + '/VMware');
            const tarFile = this.TMP_PATH + '/vmware-sdk.tar.gz';
            await this.tar.extract({
                file: tarFile,
                cwd: this.TMP_PATH,
                sync: true
            });
            const archiveFolder = this.TMP_PATH + '/' + 'vmware-vsphere-cli-distrib';
            const vmwareFolder = archiveFolder + '/lib/VMware/share/VMware';
            if (this.fs.existsSync(vmwareFolder)) {
                await this.fse.copySync(vmwareFolder, UPGRADE_PATH + '/VMware', { overwrite: true });
            }
            if (this.fs.existsSync(UPGRADE_PATH + '/VMware/pyexe')) {
                await this.fs.rmSync(UPGRADE_PATH + '/VMware/pyexe', { recursive: true, force: true });
            }
            await this.fse.copySync(archiveFolder + '/apps/general', UPGRADE_PATH + '/apps');
            await this.fs.chmodSync(UPGRADE_PATH + '/VMware/VICommon.pm', 0o755);
            let VICommonFile = await this.fs.readFileSync(UPGRADE_PATH + '/VMware/VICommon.pm', { encoding: 'utf8', flag: 'r' });
            VICommonFile = VICommonFile.replace("use VMware::SSOConnection;", "#use VMware::SSOConnection;");
            VICommonFile = VICommonFile.replace("$ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;", "eval {\n# required for new IO::Socket::SSL versions\nrequire IO::Socket::SSL;\nIO::Socket::SSL->import();\nIO::Socket::SSL::set_ctx_defaults( SSL_verify_mode => 0 );\n};\n");
            await this.fs.writeFileSync(UPGRADE_PATH + '/VMware/VICommon.pm', VICommonFile, 'utf8');
            if (App_1.App.isDockerVersion()) {
                await this.createDockerPathSymlink();
            }
            const ms = new Microservice_1.Microservice();
            await ms.init('vmware');
            if (ms.isActive()) {
                ms.start();
            }
            return true;
        }
        catch (e) {
            console.error(e);
            return false;
        }
    }
    static async createDockerPathSymlink() {
        if (this.fs.existsSync(this.DOCKER_SDK_PATH) && !this.fs.existsSync(this.SDK_PATH)) {
            await this.fs.symlinkSync(this.DOCKER_SDK_PATH, this.SDK_PATH, 'dir');
        }
    }
    static async checkSdk() {
        const setting = await Settings_1.Settings.get({
            keys: ['GLOB_VMWARE_SDK']
        });
        if (!await this.fs.existsSync(this.SDK_PATH) && setting['GLOB_VMWARE_SDK'] == '1') {
            await Settings_1.Settings.save({
                'GLOB_VMWARE_SDK': "0"
            });
        }
        else if (await this.fs.existsSync(this.SDK_PATH) && (!setting['GLOB_VMWARE_SDK'] || setting['GLOB_VMWARE_SDK'] == '0')) {
            await Settings_1.Settings.save({
                'GLOB_VMWARE_SDK': "1"
            });
        }
    }
}
exports.VmWareSDK = VmWareSDK;
VmWareSDK.SDK_PATH = (0, path_1.resolve)('../perl-modules/vmware-lib');
VmWareSDK.TMP_PATH = (0, path_1.resolve)('./files/tmp');
VmWareSDK.DOCKER_SDK_PATH = (0, path_1.resolve)('./files/vmware-lib');
VmWareSDK.fs = require('fs');
VmWareSDK.fse = require('fs-extra');
VmWareSDK.tar = require('tar');
//# sourceMappingURL=VmWareSDK.js.map