"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HostCfgService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const hostcfgRelation_entity_1 = require("../../entity/hostcfgRelation.entity");
const typeorm_2 = require("typeorm");
const Crypto_1 = require("./Crypto");
const dataSource_1 = require("../../config/dataSource");
const microserviceJobsStatus_entity_1 = require("../../entity/microserviceJobsStatus.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const Definitions_1 = require("../metrics/Definitions");
const License_1 = require("../../config/License");
const Architecture_1 = require("../architecture/Architecture");
const xormon_exception_1 = require("../../exception/xormon.exception");
const uuid_1 = require("uuid");
const fabric_service_1 = require("./fabric.service");
const menuGroup_service_1 = require("./menuGroup.service");
const metricsTable_service_1 = require("../metrics/metricsTable.service");
const Microservice_1 = require("../microservices/Microservice");
const event_emitter_1 = require("@nestjs/event-emitter");
const publicApiMap_1 = require("../public/hostcfg/publicApiMap");
let HostCfgService = class HostCfgService {
    constructor(objectItemRepository, hostCfgRepository, hostCfgRelationsRepository, microservicesJobsStatusRepository, fabricService, menugroupService, metricsTableService, eventEmitter) {
        this.objectItemRepository = objectItemRepository;
        this.hostCfgRepository = hostCfgRepository;
        this.hostCfgRelationsRepository = hostCfgRelationsRepository;
        this.microservicesJobsStatusRepository = microservicesJobsStatusRepository;
        this.fabricService = fabricService;
        this.menugroupService = menugroupService;
        this.metricsTableService = metricsTableService;
        this.eventEmitter = eventEmitter;
    }
    async getHostCfgData(_class, hw_type) {
        let hostcfg = [];
        if (hw_type) {
            hostcfg = await this.getHostCfgs(undefined, hw_type);
        }
        else if (_class) {
            hostcfg = await this.getHostCfgs(_class);
        }
        else {
            hostcfg = await this.getHostCfgs();
        }
        hostcfg.forEach((cfg) => {
            cfg.data = Crypto_1.Crypto.hidePasswords(cfg.data);
            cfg.hw_type_label = cfg.hw_type_entity ? cfg.hw_type_entity.label : cfg.hw_type;
        });
        return hostcfg;
    }
    async saveHostcfgData(hostcfg, source) {
        const isFree = !(await License_1.License.checkTechnology(hostcfg.hw_type));
        const text = isFree ? "in free version" : "with your license";
        const hwTypeDef = await Architecture_1.Architecture.getHwType(hostcfg.hw_type);
        const limits = await License_1.License.getLimits();
        if (limits[hwTypeDef.class]) {
            if (limits[hwTypeDef.class].limit) {
                const hostcfgsByClass = (await this.getHostCfgs(hwTypeDef.class)).filter(hostcfg => hostcfg.disabled === false);
                if (limits[hwTypeDef.class].limit !== undefined && limits[hwTypeDef.class].limit !== -1 && hostcfgsByClass.length >= limits[hwTypeDef.class].limit) {
                    throw new xormon_exception_1.XormonException("You can only have " + limits[hwTypeDef.class].limit + " devices " + text, common_1.HttpStatus.PAYMENT_REQUIRED);
                }
            }
            else if (limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]) {
                const hostcfgs = (await this.getHostCfgs(undefined, hwTypeDef.hw_type)).filter(hostcfg => hostcfg.disabled === false);
                if (limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['limit'] !== undefined && limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['limit'] !== -1 && hostcfgs.length >= limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['limit']) {
                    throw new xormon_exception_1.XormonException("You can only have " + limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type].limit + " devices " + text, common_1.HttpStatus.PAYMENT_REQUIRED);
                }
                if (limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['types']) {
                    const type = hostcfg.data.licType;
                    if (type && limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['types'][type]) {
                        const hostcfgsByType = hostcfgs.filter(hostcfg => hostcfg.data['licType'] && hostcfg.data['licType'].toLowerCase() == type.toLowerCase());
                        if (limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['types'][type] !== -1 && hostcfgsByType.length >= limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['types'][type]) {
                            throw new xormon_exception_1.XormonException("You can only have " + limits[hwTypeDef.class]['hwTypes'][hwTypeDef.hw_type]['types'][type] + " " + type + " devices " + text, common_1.HttpStatus.PAYMENT_REQUIRED);
                        }
                    }
                }
            }
        }
        if (!hostcfg.hostcfg_id) {
            hostcfg.hostcfg_id = (0, uuid_1.v4)();
        }
        if (source) {
            const sourceHostcfg = await this.getHostCfg(source);
            if (sourceHostcfg) {
                hostcfg = await this.copyPasswords(sourceHostcfg, hostcfg);
            }
        }
        if (hostcfg.data['fabric_id']) {
            const fabric = await this.fabricService.getFabric(hostcfg.data['fabric_id']);
            hostcfg.data['fabric_label'] = fabric ? fabric.label : null;
        }
        if (hostcfg.data['main_group_id']) {
            const main_group = await this.menugroupService.getMenuGroup(hostcfg.data['menu_group_id']);
            hostcfg.data['main_group'] = main_group ? main_group.label : null;
        }
        if (hostcfg.data['sub_group_id']) {
            const sub_group = await this.menugroupService.getMenuGroup(hostcfg.data['sub_group_id']);
            hostcfg.data['sub_group'] = sub_group ? sub_group.label : null;
        }
        const saved = await this.saveHostCfg(hostcfg);
        if (saved) {
            saved.data = Crypto_1.Crypto.hidePasswords(saved.data);
            this.metricsTableService.generate();
            Microservice_1.Microservice.checkAndStart(saved.hw_type);
            return {
                ...saved,
                hw_type_label: saved.hw_type_entity ? saved.hw_type_entity.label : saved.hw_type
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while saving hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateHostcfgData(hostcfg_id, hostcfg) {
        hostcfg.hw_type = hostcfg.hw_type.toLowerCase();
        if (hostcfg.data['fabric_id']) {
            const fabric = await this.fabricService.getFabric(hostcfg.data['fabric_id']);
            hostcfg.data['fabric_label'] = fabric ? fabric.label : null;
        }
        if (hostcfg.data['main_group_id']) {
            const main_group = await this.menugroupService.getMenuGroup(hostcfg.data['menu_group_id']);
            hostcfg.data['main_group'] = main_group ? main_group.label : null;
        }
        if (hostcfg.data['sub_group_id']) {
            const sub_group = await this.menugroupService.getMenuGroup(hostcfg.data['sub_group_id']);
            hostcfg.data['sub_group'] = sub_group ? sub_group.label : null;
        }
        const [updated, old] = await this.updateHostCfg(hostcfg_id, hostcfg);
        if (updated.hostcfg_id) {
            this.renameItem(updated.hostcfg_id, updated.label);
            this.eventEmitter.emit('reload', {
                event: 'reload',
                message: {
                    reload: 'health-status'
                },
            });
            return [true, old];
        }
        else {
            return [false, null];
        }
    }
    async updateHostcfgLabel(hostcfgs) {
        return this.hostCfgRepository.save(hostcfgs);
    }
    async getHostCfg(uuid) {
        return this.hostCfgRepository.findOneBy({ hostcfg_id: uuid });
    }
    async getHostCfgs(_class, hwType) {
        let conditions = {};
        if (_class) {
            conditions['hw_type_entity'] = {
                class: _class
            };
        }
        if (hwType) {
            conditions['hw_type'] = hwType;
        }
        return this.hostCfgRepository.find({
            where: conditions,
            relations: {
                hw_type_entity: true
            },
            order: {
                createdAt: 'ASC'
            }
        });
    }
    async getHostsCfg() {
        return this.getHostCfgs();
    }
    async getHostCfgByHWType(hw_type = null) {
        return this.getHostCfgs(undefined, hw_type);
    }
    async getHostCfgByClass(_class) {
        return this.getHostCfgs(_class);
    }
    async copyPasswords(sourceHostCfg, hostcfg) {
        hostcfg.data = this.findInHostcfgObject(sourceHostCfg.data, hostcfg.data);
        return hostcfg;
    }
    findInHostcfgObject(source, obj) {
        if (Object.keys(obj).length >= 1) {
            for (const key in obj) {
                if (typeof obj[key] === 'object' && obj[key] !== null && source[key]) {
                    obj[key] = this.findInHostcfgObject(source[key], obj[key]);
                }
                else if (obj[key] === null && Crypto_1.Crypto.passKeys.includes(key) && source[key]) {
                    obj[key] = source[key];
                }
            }
        }
        return obj;
    }
    async getHostcfgRelations(hostcfg_id) {
        return await this.hostCfgRelationsRepository.findBy({ hostcfg_id: hostcfg_id });
    }
    async saveHostCfg(hostcfg) {
        return await this.hostCfgRepository.save(hostcfg);
    }
    async saveHostcfgRelations(hostcfgRelations) {
        const insert = await (0, dataSource_1.getDataSource)().createQueryBuilder().insert().into(hostcfgRelation_entity_1.HostcfgRelation).values(hostcfgRelations).orIgnore().execute();
        return insert.generatedMaps.length;
    }
    async updateHostCfg(hostcfg_id, hostcfg) {
        const row = await this.hostCfgRepository.findOneBy({ hostcfg_id: hostcfg_id });
        if (!row) {
            return null;
        }
        const updated = await this.hostCfgRepository.save({ hostcfg_id: hostcfg_id, ...hostcfg });
        return [updated, row];
    }
    async deleteHostCfg(hostcfg) {
        const deleted = await this.hostCfgRepository.remove(hostcfg);
        if (deleted) {
            await this.microservicesJobsStatusRepository.delete({ identification: deleted.label });
            return true;
        }
        else {
            return false;
        }
    }
    async disableHostCfgs(hostcfgs) {
        const cfgsToSave = hostcfgs.map(cfg => {
            return ({
                ...cfg,
                disabled: true
            });
        });
        return await this.hostCfgRepository.save(cfgsToSave);
    }
    async renameItem(hostcfgId, label) {
        const subsystemsToRename = {
            'kubernetes': 'cluster',
            'openshift': 'cluster'
        };
        const item = await this.objectItemRepository.findOne({
            relations: {
                hostcfg_relations: true,
                hw_type_entity: true
            },
            where: {
                hostcfg_relations: {
                    hostcfg_id: hostcfgId
                },
                subsystem: (0, typeorm_2.In)(Object.values(Definitions_1.rootSubsystems).concat('device'))
            }
        });
        if (!item) {
            return;
        }
        if ((['storage', 'san'].includes(item.hw_type_entity.class) && item.subsystem === 'device') || (subsystemsToRename[item.hw_type] && subsystemsToRename[item.hw_type][item.subsystem])) {
            item.label = label;
            await this.objectItemRepository.upsert(item, ['item_id']);
            return;
        }
    }
    getHostcfgDefinition(hwType) {
        return (0, publicApiMap_1.describeMetaData)(hwType);
    }
    getHostcfgDefinitionByLabel(device) {
        return (0, publicApiMap_1.getHwTypeByVendorDevice)(device);
    }
};
exports.HostCfgService = HostCfgService;
exports.HostCfgService = HostCfgService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(1, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(2, (0, typeorm_1.InjectRepository)(hostcfgRelation_entity_1.HostcfgRelation)),
    __param(3, (0, typeorm_1.InjectRepository)(microserviceJobsStatus_entity_1.MicroserviceJobStatus)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        fabric_service_1.FabricService,
        menuGroup_service_1.MenuGroupService,
        metricsTable_service_1.MetricsTableService,
        event_emitter_1.EventEmitter2])
], HostCfgService);
//# sourceMappingURL=hostCfg.service.js.map