"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomTaskController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const platform_express_1 = require("@nestjs/platform-express");
const swagger_1 = require("@nestjs/swagger");
const customTask_dto_1 = require("../../common/customTask.dto");
const xormon_exception_1 = require("../../exception/xormon.exception");
const custom_task_service_1 = require("./custom_task.service");
let CustomTaskController = class CustomTaskController {
    constructor(customTaskService) {
        this.customTaskService = customTaskService;
    }
    ;
    async uploadCustomTaskFile(file, customTaskDTO) {
        const fullPath = await this.customTaskService.saveFileToFiles(file);
        const customTask = await this.customTaskService.saveCustomTask({
            ...customTaskDTO,
            filePath: fullPath
        });
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: "Successfuly uploaded custom task",
            data: {
                ...customTask,
                mailGroups: customTask.mailGroups.map(mg => mg.mail_group_id),
                _mailGroups: customTask.mailGroups
            }
        };
    }
    async getAllCustomTasks() {
        try {
            const customTasks = await this.customTaskService.getAllCustomTasks();
            return {
                message: "Successfuly fetched all custom tasks",
                statusCode: common_1.HttpStatus.OK,
                data: customTasks.map(customTask => {
                    return {
                        ...customTask,
                        mailGroups: customTask.mailGroups.map(mg => mg.mail_group_id),
                        _mailGroups: customTask.mailGroups
                    };
                })
            };
        }
        catch (error) {
            console.error(error);
            throw new xormon_exception_1.XormonException("Error happened while trying to fetch custom tasks", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    async updateCustomTask(customTaskDTO) {
        try {
            const customTask = await this.customTaskService.saveCustomTask(customTaskDTO);
            return {
                message: "Successfuly updated custom task with id: " + customTask.custom_task_id,
                statusCode: common_1.HttpStatus.OK,
                data: {
                    ...customTask,
                    mailGroups: customTask.mailGroups.map(mg => mg.mail_group_id),
                    _mailGroups: customTask.mailGroups
                }
            };
        }
        catch (error) {
            console.error(error);
            throw new xormon_exception_1.XormonException("Updating custom task with id: " + customTaskDTO.custom_task_id + " failed.", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    async deleteCustomTask(id) {
        try {
            const deleted = await this.customTaskService.deleteCustomTask(id);
            if (!deleted.affected) {
                return {
                    message: "Custom task with id: " + id + " does not exist.",
                    statusCode: common_1.HttpStatus.BAD_REQUEST
                };
            }
            return {
                message: "Successfuly deleted custom task with id: " + id,
                statusCode: common_1.HttpStatus.OK
            };
        }
        catch (error) {
            console.error(error);
            throw new xormon_exception_1.XormonException("Deleting custom task with id: " + id + " failed.", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
};
exports.CustomTaskController = CustomTaskController;
__decorate([
    (0, common_1.Post)("/v1/custom_task"),
    (0, common_1.UseInterceptors)((0, platform_express_1.FileInterceptor)('file')),
    openapi.ApiResponse({ status: 201, type: require("../../common/customTask.dto").SingleCustomTaskResponseDTO }),
    __param(0, (0, common_1.UploadedFile)(new common_1.ParseFilePipe({
        validators: [
            new common_1.FileTypeValidator({ fileType: '.gz' }),
        ]
    }))),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, customTask_dto_1.CustomTaskDTO]),
    __metadata("design:returntype", Promise)
], CustomTaskController.prototype, "uploadCustomTaskFile", null);
__decorate([
    (0, common_1.Get)("/v1/custom_task"),
    openapi.ApiResponse({ status: 200, type: require("../../common/customTask.dto").CustomTaskResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], CustomTaskController.prototype, "getAllCustomTasks", null);
__decorate([
    (0, common_1.Put)("/v1/custom_task"),
    openapi.ApiResponse({ status: 200, type: require("../../common/customTask.dto").SingleCustomTaskResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [customTask_dto_1.CustomTaskDTO]),
    __metadata("design:returntype", Promise)
], CustomTaskController.prototype, "updateCustomTask", null);
__decorate([
    (0, common_1.Delete)("/v1/custom_task/:id"),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)("id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], CustomTaskController.prototype, "deleteCustomTask", null);
exports.CustomTaskController = CustomTaskController = __decorate([
    (0, swagger_1.ApiTags)('Custom Task'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/custom_task'),
    __metadata("design:paramtypes", [custom_task_service_1.CustomTaskService])
], CustomTaskController);
//# sourceMappingURL=custom_task.controller.js.map