"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomTaskService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const customTask_entity_1 = require("../../entity/customTask.entity");
const customTaskMailGroupRelations_entity_1 = require("../../entity/customTaskMailGroupRelations.entity");
const xormon_exception_1 = require("../../exception/xormon.exception");
const node_fs_1 = require("node:fs");
const promises_1 = require("node:fs/promises");
const typeorm_2 = require("typeorm");
const zlib_1 = require("zlib");
let CustomTaskService = class CustomTaskService {
    constructor(customTaskRepository, customTaskRelationsRepository) {
        this.customTaskRepository = customTaskRepository;
        this.customTaskRelationsRepository = customTaskRelationsRepository;
    }
    async saveFileToFiles(file) {
        const directory = __dirname.replace("/dist/service/custom_task", "/files/custom");
        const fullPath = directory + "/" + file.originalname.replace(".gz", "");
        if (!(0, node_fs_1.existsSync)(directory)) {
            try {
                await (0, promises_1.mkdir)(directory);
            }
            catch (error) {
                console.error(error);
                throw new xormon_exception_1.XormonException(error, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
            }
        }
        try {
            const buffer = (0, zlib_1.gunzipSync)(file.buffer);
            (0, node_fs_1.writeFileSync)(fullPath, buffer);
        }
        catch (error) {
            console.error(error);
            throw new xormon_exception_1.XormonException(error, common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        return fullPath;
    }
    async saveCustomTask(customTaskDTO) {
        const customTask = await this.customTaskRepository.save(this.customTaskRepository.create({
            ...customTaskDTO,
            mailGroups: []
        }));
        if (typeof customTaskDTO.mailGroups === "string") {
            customTaskDTO.mailGroups = JSON.parse(customTaskDTO.mailGroups);
        }
        await this.customTaskRelationsRepository.delete({
            custom_task_id: customTask.custom_task_id,
            mail_group_id: (0, typeorm_2.Not)((0, typeorm_2.In)(customTaskDTO.mailGroups))
        });
        let groups = [];
        if (typeof customTaskDTO.mailGroups === "string") {
            for (const group of JSON.parse(customTaskDTO.mailGroups)) {
                groups.push(this.customTaskRelationsRepository.create({ custom_task_id: customTask.custom_task_id, mail_group_id: group }));
            }
        }
        else {
            for (const group of customTaskDTO.mailGroups) {
                groups.push(this.customTaskRelationsRepository.create({ custom_task_id: customTask.custom_task_id, mail_group_id: group }));
            }
        }
        const relations = await this.customTaskRelationsRepository.save(groups);
        const customTaskWithMailGroups = await this.customTaskRepository.findOne({
            where: {
                custom_task_id: customTask.custom_task_id
            },
            relations: {
                mailGroups: true
            }
        });
        return customTaskWithMailGroups;
    }
    async getAllCustomTasks() {
        return await this.customTaskRepository.find({ relations: { mailGroups: true } });
    }
    async deleteCustomTask(id) {
        return await this.customTaskRepository.delete({ custom_task_id: id });
    }
};
exports.CustomTaskService = CustomTaskService;
exports.CustomTaskService = CustomTaskService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(customTask_entity_1.CustomTask)),
    __param(1, (0, typeorm_1.InjectRepository)(customTaskMailGroupRelations_entity_1.CustomTaskMailGroupRelations)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository])
], CustomTaskService);
//# sourceMappingURL=custom_task.service.js.map