import { BasicResponseDTO } from '@xormon/common/basicResponse.dto';
import { CreateDTO, CreateDashboardGraphDTO, CreateResponseDTO, DashboardDTO, DashboardGroupsDTO, DefaultDashboardDTO, FindDashboardGraphDTO, FindGraphsDTO, GroupDashboardFoldersDTO, UpdateDashboardDTO, UpdateDashboardFolderDTO, UpdateDashboardGraphDTO, UpdateDashboardGroupDTO } from '@xormon/common/dashboard.dto';
import { MenuItem, MenuResponseDTO } from '@xormon/common/menuResponse.dto';
import { PageResponseDTO } from '@xormon/common/pageResponse.dto';
import { DashboardFolder } from '@xormon/entity/dashboardFolder.entity';
import { ArchitectureService } from '@xormon/service/architecture/architecture.service';
import { MenuService } from '@xormon/service/menu/menu.service';
import { Request } from 'express';
import { DashboardService } from './dashboard.service';
import { CustomGroupService } from '../custom_group/custom_group.service';
export declare class DashboardController {
    private readonly dashboardService;
    private readonly menuService;
    private readonly architectureService;
    private readonly customGroupService;
    constructor(dashboardService: DashboardService, menuService: MenuService, architectureService: ArchitectureService, customGroupService: CustomGroupService);
    getDefault(request: Request): Promise<DefaultDashboardDTO>;
    getDashboards(request: Request, parent?: string): Promise<DashboardDTO[]>;
    getDashboardDefault(request: Request): Promise<DashboardDTO>;
    getGlobalStatus(request: Request): Promise<PageResponseDTO>;
    getGlobalHeatmap(request: Request): Promise<PageResponseDTO>;
    getDashboard(request: Request, dashboard_id?: string): Promise<DashboardDTO>;
    setDefault(request: Request, dashboard_id: string): Promise<BasicResponseDTO>;
    deleteDashboard(request: Request, dashboard_id: string): Promise<BasicResponseDTO>;
    saveFolder(request: Request, folder: CreateDTO): Promise<CreateResponseDTO>;
    deleteDashboardFolder(request: Request, dashboard_folder_id: string): Promise<BasicResponseDTO>;
    updateDashboardFolder(request: Request, folder: UpdateDashboardFolderDTO, dashboard_folder_id: string): Promise<BasicResponseDTO>;
    saveDashboard(request: Request, dashboard: CreateDTO): Promise<CreateResponseDTO>;
    updateDashboard(request: Request, dashboard: UpdateDashboardDTO, dashboard_id: string): Promise<BasicResponseDTO>;
    saveGraph(request: Request, graph: CreateDashboardGraphDTO): Promise<CreateResponseDTO>;
    findGraphs(request: Request, graph: FindDashboardGraphDTO): Promise<FindGraphsDTO>;
    updateDashboardGraph(request: Request, graph: UpdateDashboardGraphDTO, dashboard_graph_id: string): Promise<BasicResponseDTO>;
    deleteDashboardGraph(request: Request, dashboard_graph_id: string): Promise<BasicResponseDTO>;
    getGroups(request: Request, dashboard_id?: string): Promise<DashboardGroupsDTO>;
    saveGroup(request: Request, group: CreateDTO): Promise<CreateResponseDTO>;
    updateDashboardGroup(request: Request, group: UpdateDashboardGroupDTO, dashboard_group_id: string): Promise<BasicResponseDTO>;
    deleteDashboardGroup(request: Request, dashboard_group_id: string): Promise<BasicResponseDTO>;
    getDashboardFolders(request: Request): Promise<DashboardFolder[]>;
    getDashboardFolder(request: Request, dashboard_folder_id: string): Promise<DashboardFolder>;
    getMenu(request: Request): Promise<MenuResponseDTO>;
    getGroupsMenu(request: Request): Promise<GroupDashboardFoldersDTO>;
    getMenuFolder(request: Request, dashboard_folder_id: string): Promise<MenuResponseDTO>;
    getPathDefault(request: Request): Promise<MenuResponseDTO>;
    getPath(request: Request, dashboard_id: string): Promise<MenuResponseDTO>;
    findDashboard(request: any, menuItems: MenuItem[], parents: string[]): Promise<MenuItem[]>;
    getMenuDashboard(request: Request, dashboard_id: string): Promise<PageResponseDTO>;
}
