"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DashboardController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const dashboard_constants_1 = require("../../common/constants/dashboard.constants");
const healthStatus_constants_1 = require("../../common/constants/healthStatus.constants");
const menuResponse_constants_1 = require("../../common/constants/menuResponse.constants");
const pageResponse_constants_1 = require("../../common/constants/pageResponse.constants");
const dashboard_dto_1 = require("../../common/dashboard.dto");
const pageResponse_dto_1 = require("../../common/pageResponse.dto");
const License_1 = require("../../config/License");
const architecture_service_1 = require("../architecture/architecture.service");
const menu_service_1 = require("../menu/menu.service");
const Architecture_1 = require("../architecture/Architecture");
const CustomGroupWrapper_1 = require("../custom_group/CustomGroupWrapper");
const HealthStatus_1 = require("../menu/menu/HealthStatus");
const User_1 = require("../users/User");
const dashboard_service_1 = require("./dashboard.service");
const MenuHandler_1 = require("../menu/MenuHandler");
const Heatmap_1 = require("../menu/menu/Heatmap");
const Stor_1 = require("../menu/menu/Stor");
const dataSource_1 = require("../../config/dataSource");
const storageGroup_entity_1 = require("../../entity/storageGroup.entity");
const class_constants_1 = require("../../common/constants/class.constants");
const custom_group_service_1 = require("../custom_group/custom_group.service");
let DashboardController = class DashboardController {
    constructor(dashboardService, menuService, architectureService, customGroupService) {
        this.dashboardService = dashboardService;
        this.menuService = menuService;
        this.architectureService = architectureService;
        this.customGroupService = customGroupService;
    }
    async getDefault(request) {
        const user = await User_1.User.getUserByRequest(request);
        const dashboard = await this.dashboardService.getDashboardDefault(user.user_id);
        if (dashboard) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    dashboard: await this.getDashboard(request, dashboard.dashboard_id),
                    path: (await this.getPath(request, dashboard.dashboard_id)).data
                }
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'default dashboard not found',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getDashboards(request, parent) {
        let dashboardDTO = [];
        const user = await User_1.User.getUserByRequest(request);
        const dashboards = parent ? await this.dashboardService.getDashboards(parent, user.user_id) : await this.dashboardService.getAllDashboards(user.user_id);
        for (const dashboard of dashboards) {
            dashboardDTO.push(await this.getDashboard(request, dashboard.dashboard_id));
        }
        return dashboardDTO;
    }
    async getDashboardDefault(request) {
        const user = await User_1.User.getUserByRequest(request);
        const dashboard = await this.dashboardService.getDashboardDefault(user.user_id);
        if (dashboard) {
            return this.getDashboard(request, dashboard.dashboard_id);
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'default dashboard not found',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getGlobalStatus(request) {
        const statusPage = await new HealthStatus_1.HealthStatus().setAcl(request).getGlobalStatusPage();
        if (statusPage) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: statusPage,
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Global status page not found',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getGlobalHeatmap(request) {
        const globalHeatmap = await new Heatmap_1.CrossHeatmap(request).gen();
        if (globalHeatmap) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: globalHeatmap,
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Global heatmap page not found',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getDashboard(request, dashboard_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardOwner(dashboard_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const dashboard = await this.dashboardService.getDashboardWithGroups(dashboard_id, user.user_id);
        const folder = dashboard.parent ? await this.dashboardService.getPath(dashboard.parent) : '/';
        let dashboardDTO = {
            dashboard_id: dashboard.dashboard_id,
            label: dashboard.label,
            type: dashboard.type,
            groups: [],
            folder: folder,
            default: dashboard.default,
            width: dashboard.width
        };
        for (const group of dashboard.dashboard_groups) {
            dashboardDTO.groups.push({
                dashboard_group_id: group.dashboard_group_id,
                label: group.label,
            });
        }
        return dashboardDTO;
    }
    async setDefault(request, dashboard_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardOwner(dashboard_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const success = await this.dashboardService.setDefault(dashboard_id, user.user_id);
        if (success) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'default has been set',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'the default cannot be set',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteDashboard(request, dashboard_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardOwner(dashboard_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const dashboard = await this.dashboardService.getDashboard(dashboard_id);
        if (dashboard.default === true) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'the default dashboard cannot be deleted',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        if (await this.dashboardService.deleteDashboard(dashboard_id)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Dashboard deleted',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'dashboard was not deleted',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveFolder(request, folder) {
        const user = await User_1.User.getUserByRequest(request);
        const saved = await this.dashboardService.saveDashboardFolder({
            ...folder,
            type: dashboard_constants_1.DASHBOARD_TYPE.global,
        }, user.user_id);
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Dashboard folder successfully saved',
                data: {
                    label: saved.label,
                    parent: saved.parent ? saved.parent.dashboard_folder_id : null,
                    id: saved.dashboard_folder_id,
                },
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'dashboard folder was not saved',
                error: 'bad request',
                data: null,
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteDashboardFolder(request, dashboard_folder_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardFolderOwner(dashboard_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        if (await this.dashboardService.deleteDashboardFolder(dashboard_folder_id)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Dashboard folder deleted',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'dashboard folder was not deleted (does it include a default dashboard?)',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateDashboardFolder(request, folder, dashboard_folder_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardFolderOwner(dashboard_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const updated = await this.dashboardService.updateDashboardFolder(dashboard_folder_id, folder);
        if (updated) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'dashboard folder updated',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'dashboard folder was not updated',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveDashboard(request, dashboard) {
        const user = await User_1.User.getUserByRequest(request);
        const saved = await this.dashboardService.saveDashboard({
            ...dashboard,
            type: dashboard_constants_1.DASHBOARD_TYPE.global,
            parent: dashboard.parent ? await this.dashboardService.getFolder(dashboard.parent) : null,
        }, user.user_id);
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Dashboard successfully saved',
                data: {
                    id: saved.dashboard_id,
                    label: saved.label,
                    parent: dashboard.parent,
                },
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'The dashboard was not saved',
                data: null,
            };
        }
    }
    async updateDashboard(request, dashboard, dashboard_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardOwner(dashboard_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const updated = await this.dashboardService.updateDashboard(dashboard_id, dashboard);
        if (updated) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'dashboard updated',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'dashboard was not updated',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveGraph(request, graph) {
        const saved = await this.dashboardService.saveGraph({ ...graph, width: 2, height: 2 });
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Dashboard graph successfully saved',
                data: {
                    id: saved.dashboard_graph_id,
                },
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'The dashboard graph was not saved',
                data: null,
            };
        }
    }
    async findGraphs(request, graph) {
        const user = await User_1.User.getUserByRequest(request, true);
        const find = await this.dashboardService.findGraphs(graph.url, graph.tab, user.user_id, graph.interval ? graph.interval : undefined, user.groups.map(g => g.group_id));
        let graphs = find.map((graph) => {
            return {
                dashboard_graph_id: graph.dashboard_graph_id,
                dashboard_group: {
                    label: graph.dashboard_group.label,
                    dashboard_group_id: graph.dashboard_group.dashboard_group_id,
                },
                dashboard: {
                    label: graph.dashboard_group.dashboard.label,
                    dashboard_id: graph.dashboard_group.dashboard.dashboard_id,
                },
                interval: graph.interval,
                group_id: graph.dashboard_group.dashboard.group_id
            };
        });
        return {
            statusCode: common_1.HttpStatus.OK,
            data: graphs,
        };
    }
    async updateDashboardGraph(request, graph, dashboard_graph_id) {
        const updated = await this.dashboardService.updateDashboardGraph(dashboard_graph_id, graph);
        if (updated) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Dashboard graph updated',
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Updating graph failed',
            };
        }
    }
    async deleteDashboardGraph(request, dashboard_graph_id) {
        if (await this.dashboardService.deleteDashboardGraph(dashboard_graph_id)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Dashboard graph deleted',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'dashboard graph was not deleted',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getGroups(request, dashboard_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (dashboard_id && !await this.dashboardService.checkDashboardOwner(dashboard_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const groups = dashboard_id ? await this.dashboardService.getDashboardGroups(dashboard_id) : await this.dashboardService.getAllGroups();
        let reformattedGroups = groups.map((group) => {
            return {
                label: group.label,
                dashboard_id: group.dashboard_id,
                dashboard_group_id: group.dashboard_group_id,
            };
        });
        return {
            statusCode: common_1.HttpStatus.CREATED,
            data: reformattedGroups,
        };
    }
    async saveGroup(request, group) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (group.parent && !await this.dashboardService.checkDashboardOwner(group.parent, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const saved = await this.dashboardService.saveGroup({
            label: group.label,
            dashboard_id: group.parent,
            width: 6,
            height: 2,
        });
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Dashboard group successfully saved',
                data: {
                    id: saved.dashboard_group_id,
                    label: saved.label,
                    parent: saved.dashboard_id,
                },
            };
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'The dashboard group was not saved',
                data: null,
            };
        }
    }
    async updateDashboardGroup(request, group, dashboard_group_id) {
        const sorted = await this.dashboardService.resortGroup(dashboard_group_id, group.dashboard_id, group.x, group.y);
        if (sorted) {
            const updated = await this.dashboardService.updateDashboardGroup(dashboard_group_id, group);
            if (updated) {
                return {
                    statusCode: common_1.HttpStatus.OK,
                    message: 'Dashboard group updated',
                };
            }
            else {
                return {
                    statusCode: common_1.HttpStatus.BAD_REQUEST,
                    message: 'Updating group failed',
                };
            }
        }
        else {
            return {
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Resorting groups failed',
            };
        }
    }
    async deleteDashboardGroup(request, dashboard_group_id) {
        if (await this.dashboardService.deleteDashboardGroup(dashboard_group_id)) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Dashboard group deleted',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'dashboard group was not deleted',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getDashboardFolders(request) {
        const user = await User_1.User.getUserByRequest(request);
        return await this.dashboardService.getAllFolders(user.user_id);
    }
    async getDashboardFolder(request, dashboard_folder_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardFolderOwner(dashboard_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const folder = await this.dashboardService.getFolder(dashboard_folder_id);
        return await this.dashboardService.getFolders(folder);
    }
    async getMenu(request) {
        const user = await User_1.User.getUserByRequest(request);
        let items = [];
        items.push({
            isLeaf: true,
            itemId: healthStatus_constants_1.HEALTH_STATUS_ITEM_ID,
            pageType: 'health-status',
            url: '/api/dashboard/v1/health-status',
            title: 'Health Status'
        });
        items.push({
            isLeaf: false,
            itemId: dashboard_constants_1.HEATMAP_FOLDER_ITEM_ID,
            pageType: dashboard_constants_1.HEATMAP_FOLDER_ITEM_ID,
            url: '/api/custom_group/v1/menu/dashboard',
            title: 'Heatmap Cust. Grp.',
        });
        items.push({
            isLeaf: true,
            itemId: healthStatus_constants_1.HEATMAP_ITEM_ID,
            pageType: 'heatmap',
            url: '/api/dashboard/v1/heatmap',
            title: 'Heatmap'
        });
        const dashboards = await this.dashboardService.getDashboards(null, user.user_id);
        dashboards.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const dashboard of dashboards) {
            items.push({
                isLeaf: true,
                itemId: dashboard.dashboard_id,
                pageType: 'dashboard',
                url: '/api/dashboard/v1/menu/dashboard/' + dashboard.dashboard_id,
                title: dashboard.label,
                styleClass: dashboard.default ? menuResponse_constants_1.MENU_STYLE_CLASS.star : undefined
            });
        }
        const dashboard_folders = await this.dashboardService.getDashboardFolders(null, user.user_id);
        dashboard_folders.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const dashboard_folder of dashboard_folders) {
            items.push({
                isLeaf: false,
                itemId: dashboard_folder.dashboard_folder_id,
                pageType: 'dashboard_folder',
                url: '/api/dashboard/v1/menu/folder/' + dashboard_folder.dashboard_folder_id,
                title: dashboard_folder.label,
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items,
        };
    }
    async getGroupsMenu(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        let groupDashboardFolders = [];
        const dashboards = await this.dashboardService.getRootGroupDashboards(user.groups.map(g => g.group_id));
        dashboards.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        const dashboard_folders = await this.dashboardService.getDashboardFolders(null, undefined, user.groups.map(g => g.group_id));
        dashboard_folders.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const group of user.groups) {
            groupDashboardFolders.push({
                group_id: group.group_id,
                title: group.name,
                children: dashboards.filter(d => d.group_id === group.group_id).map(dashboard => {
                    return {
                        isLeaf: true,
                        itemId: dashboard.dashboard_id,
                        pageType: 'dashboard',
                        url: '/api/dashboard/v1/menu/dashboard/' + dashboard.dashboard_id,
                        title: dashboard.label,
                        styleClass: dashboard.default ? menuResponse_constants_1.MENU_STYLE_CLASS.star : undefined
                    };
                }).concat(dashboard_folders.filter(d => d.group_id === group.group_id).map(dashboard_folder => {
                    return {
                        isLeaf: false,
                        itemId: dashboard_folder.dashboard_folder_id,
                        pageType: 'dashboard_folder',
                        url: '/api/dashboard/v1/menu/folder/' + dashboard_folder.dashboard_folder_id,
                        title: dashboard_folder.label,
                    };
                }))
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: groupDashboardFolders,
        };
    }
    async getMenuFolder(request, dashboard_folder_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardFolderOwner(dashboard_folder_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const folder = await this.dashboardService.getFolder(dashboard_folder_id);
        let items = [];
        const dashboards = await this.dashboardService.getDashboards(dashboard_folder_id, !folder.group_id ? user.user_id : undefined, folder.group_id ? [folder.group_id] : undefined);
        dashboards.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const dashboard of dashboards) {
            items.push({
                isLeaf: true,
                itemId: dashboard.dashboard_id,
                pageType: 'dashboard',
                url: '/api/dashboard/v1/menu/dashboard/' + dashboard.dashboard_id,
                title: dashboard.label,
                styleClass: dashboard.default ? menuResponse_constants_1.MENU_STYLE_CLASS.star : undefined
            });
        }
        const dashboard_folders = await this.dashboardService.getDashboardFolders(folder, !folder.group_id ? user.user_id : undefined, folder.group_id ? [folder.group_id] : undefined);
        dashboard_folders.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const dashboard_folder of dashboard_folders) {
            items.push({
                isLeaf: false,
                itemId: dashboard_folder.dashboard_folder_id,
                pageType: 'dashboard_folder',
                url: '/api/dashboard/v1/menu/folder/' + dashboard_folder.dashboard_folder_id,
                title: dashboard_folder.label,
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items,
        };
    }
    async getPathDefault(request) {
        const user = await User_1.User.getUserByRequest(request);
        const dashboard = await this.dashboardService.getDashboardDefault(user.user_id);
        if (dashboard) {
            return this.getPath(request, dashboard.dashboard_id);
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'default dashboard not found',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getPath(request, dashboard_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        const cgCheck = await this.customGroupService.getCustomGroup(dashboard_id);
        if (cgCheck) {
            const menuItems = await this.getMenu(request);
            const cgFolder = menuItems.data.find(item => item.pageType === 'heatmap_folder');
            cgFolder.children = await this.customGroupService.getCustomGroupDashboardFolder(user.user_id);
            return menuItems;
        }
        if (!await this.dashboardService.checkDashboardOwner(dashboard_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        let menuItems = [];
        const dashboard = await this.dashboardService.getDashboard(dashboard_id);
        if (!dashboard.parent) {
            return this.getMenu(request);
        }
        const dashboardFolder = await this.dashboardService.getFolder(dashboard.parent.dashboard_folder_id);
        const parentsIds = await this.dashboardService.getParentsIds(dashboardFolder);
        if (dashboard.group_id) {
            const menu = (await this.getGroupsMenu(request)).data;
            const group = menu.find(m => m.group_id === dashboard.group_id);
            const groupItems = group?.children;
            menuItems = [{ ...group, children: await this.findDashboard(request, groupItems, parentsIds) }];
        }
        else {
            menuItems = await this.findDashboard(request, (await this.getMenu(request)).data, parentsIds);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async findDashboard(request, menuItems, parents) {
        for (const menuItem of menuItems) {
            if (menuItem.pageType === "dashboard_folder" && parents.includes(menuItem.itemId)) {
                menuItem.children = await this.findDashboard(request, (await this.getMenuFolder(request, menuItem.itemId)).data, parents);
            }
        }
        return menuItems;
    }
    async getMenuDashboard(request, dashboard_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (!await this.dashboardService.checkDashboardOwner(dashboard_id, user.user_id, user.groups.map(g => g.group_id))) {
            throw new common_1.ForbiddenException();
        }
        const dashboard = await this.dashboardService.getDashboard(dashboard_id);
        let dashboardGroups = [];
        for (const dashboardGroup of dashboard.dashboard_groups) {
            let dashboardGraphs = [];
            for (const dashboardGraph of dashboardGroup.dashboard_graphs) {
                try {
                    let item_id, hw_type, type;
                    if (dashboardGraph.url.includes('custom_group')) {
                        const url = dashboardGraph.url.split('/');
                        const customGroupId = url[6];
                        const customGroup = await CustomGroupWrapper_1.CustomGroupWrapper.getCustomGroup(customGroupId);
                        if (!customGroup) {
                            console.warn("[Dashboard] CG " + customGroupId + " not found in dashboard " + dashboard.label + " [" + dashboard.dashboard_id + "]");
                            continue;
                        }
                        const customGroupInfo = await CustomGroupWrapper_1.CustomGroupWrapper.getInfoAndParent(customGroup);
                        let isFree = true;
                        let licensedTechnologies = undefined;
                        if (customGroupInfo.isClassDefined && customGroup.class == 'virtualization') {
                            licensedTechnologies = await License_1.License.getTechnologies();
                        }
                        else {
                            isFree = customGroupInfo.isClassDefined
                                ? !(await License_1.License.checkTechnology(customGroup.class))
                                : !(await License_1.License.checkTechnology(customGroup.hw_type));
                        }
                        let metrics = await CustomGroupWrapper_1.CustomGroupWrapper.getMetrics(customGroup.custom_group_id);
                        for (const metric of metrics) {
                            if (metric.label === dashboardGraph.tab) {
                                let items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(customGroupId, customGroup.class == class_constants_1.ClassKey.VIRTUALIZATION && !customGroup.hw_type);
                                const tab = await CustomGroupWrapper_1.CustomGroupWrapper.genTab(items, metric, customGroupInfo.parentType || undefined, customGroupInfo.subsystemLabel, isFree, undefined, undefined, licensedTechnologies);
                                const customGroupGraph = tab.content[0];
                                dashboardGraphs.push({
                                    ...customGroupGraph,
                                    label: customGroupGraph.label || customGroup.subsystem,
                                    graphLabel: 'Custom group : ' + customGroup.label + ' : ' + metric.label,
                                    width: dashboardGraph.width,
                                    height: dashboardGraph.height,
                                    interval: dashboardGraph.interval,
                                    url: dashboardGraph.url,
                                    x: dashboardGraph.x,
                                    y: dashboardGraph.y,
                                    dashboard_graph_id: dashboardGraph.dashboard_graph_id,
                                    tab: dashboardGraph.tab
                                });
                            }
                        }
                    }
                    else if (dashboardGraph.url.includes('sgroup') && !dashboardGraph.url.includes('resgroup')) {
                        const url = dashboardGraph.url.split('?item_id=');
                        const itemId = url[1];
                        const sgroup = await (0, dataSource_1.getDataSource)().getRepository(storageGroup_entity_1.StorageGroup).findOneBy({
                            storage_group_id: itemId
                        });
                        if (!sgroup) {
                            console.warn("[GetDashboardGraph] error: storage group not found " + itemId);
                            continue;
                        }
                        const sgroupPage = await (new Stor_1.StorTotalPage(itemId, request)).getPage();
                        const tab = sgroupPage.tabs.find(tab => tab.title === dashboardGraph.tab);
                        if (tab) {
                            const box = tab.content.find(con => con.type === 'FourGraphBox');
                            dashboardGraphs.push({
                                metrics: box.metrics,
                                uuids: box.uuids,
                                aggregated: true,
                                graphLabel: 'Storage Group : ' + sgroup.label + ' : ' + dashboardGraph.tab,
                                label: box.label,
                                width: dashboardGraph.width,
                                height: dashboardGraph.height,
                                interval: dashboardGraph.interval,
                                url: dashboardGraph.url,
                                x: dashboardGraph.x,
                                y: dashboardGraph.y,
                                dashboard_graph_id: dashboardGraph.dashboard_graph_id,
                                tab: dashboardGraph.tab
                            });
                        }
                    }
                    else {
                        let start = 0;
                        const url = dashboardGraph.url.split('/');
                        if (url[0] === '') {
                            start = 1;
                        }
                        hw_type = url[4 + start];
                        if (url[5 + start].includes('?item_id=')) {
                            const item_split = url[5 + start].split('?item_id=');
                            item_id = item_split[1];
                            type = item_split[0];
                        }
                        else {
                            type = url[5 + start];
                        }
                        const menuHandler = new MenuHandler_1.MenuHandler(hw_type, request);
                        let page;
                        try {
                            page = await menuHandler.getPage(type, item_id ? item_id : undefined);
                        }
                        catch (e) {
                            console.warn("[GetDashboardGraph] error: graph not found for hw_type " + hw_type + ", type " + type + " & item_id " + item_id);
                            continue;
                        }
                        if (!page || !page.tabs) {
                            console.warn("[GetDashboardGraph] error: graph not found for hw_type " + hw_type + ", type " + type + " & item_id " + item_id);
                            continue;
                        }
                        for (const tab in page.tabs) {
                            if (page.tabs[tab].title === dashboardGraph.tab) {
                                let label = page.title + ': ' + page.tabs[tab].title + ' - ' + dashboardGraph.interval;
                                if (item_id) {
                                    const item = await this.architectureService.getItemWithParents(item_id);
                                    if (!item) {
                                        continue;
                                    }
                                    const storageHwTypes = await Architecture_1.Architecture.getHwTypes(false, 'storage');
                                    if (storageHwTypes.map(hwType => hwType.hw_type).includes(item.hw_type)) {
                                        if (item.subsystem == 'device') {
                                            label = item.label + " : " + type + ' : ' + page.tabs[tab].title + ' - ' + dashboardGraph.interval;
                                        }
                                        else {
                                            const device = item.parents.find(parent => parent._parent.subsystem === 'device');
                                            if (device) {
                                                label = device._parent.label + " : " + item.label + ' : ' + page.tabs[tab].title + ' - ' + dashboardGraph.interval;
                                            }
                                            else {
                                                label = item.hw_type_entity.label + " : " + item.label + ' : ' + page.tabs[tab].title + ' - ' + dashboardGraph.interval;
                                            }
                                        }
                                    }
                                    else {
                                        label = item.hw_type_entity.label + " : " + item.label + ' : ' + page.tabs[tab].title + ' - ' + dashboardGraph.interval;
                                    }
                                }
                                const fgBox = page.tabs[tab].content.find(box => box.type === 'FourGraphBox');
                                if (!fgBox) {
                                    console.error("[Dashboard] Page " + url[6] + " no box found");
                                    continue;
                                }
                                dashboardGraphs.push({
                                    metrics: fgBox.metrics ? fgBox.metrics : [],
                                    uuids: fgBox.uuids ? fgBox.uuids : [],
                                    aggregated: fgBox.aggregated ? fgBox.aggregated : undefined,
                                    parent: fgBox.parent ? fgBox.parent : undefined,
                                    graphLabel: label,
                                    label: fgBox.label,
                                    width: dashboardGraph.width,
                                    height: dashboardGraph.height,
                                    interval: dashboardGraph.interval,
                                    url: dashboardGraph.url,
                                    x: dashboardGraph.x,
                                    y: dashboardGraph.y,
                                    dashboard_graph_id: dashboardGraph.dashboard_graph_id,
                                    tab: dashboardGraph.tab
                                });
                            }
                        }
                    }
                }
                catch (e) {
                    console.warn(`[Dashboard] Failed to add graph to dashboard. Dashboard: ${dashboard.label} Graph URL: ${dashboardGraph.url}`);
                }
            }
            dashboardGroups.push({
                label: dashboardGroup.label,
                width: dashboardGroup.width,
                height: dashboardGroup.height,
                graphs: dashboardGraphs,
                x: dashboardGroup.x,
                y: dashboardGroup.y,
                dashboard_group_id: dashboardGroup.dashboard_group_id,
            });
        }
        let tabs = [];
        const dashboardBox = {
            type: pageResponse_constants_1.BOX_TYPE.DashboardBox,
            groups: dashboardGroups,
            width: dashboard.width
        };
        tabs.push({
            title: dashboard.label,
            content: [dashboardBox],
        });
        const page = new pageResponse_dto_1.Page(dashboard.label, tabs);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: page,
        };
    }
};
exports.DashboardController = DashboardController;
__decorate([
    (0, common_1.Get)('v1/default'),
    openapi.ApiResponse({ status: 200, type: require("../../common/dashboard.dto").DefaultDashboardDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getDefault", null);
__decorate([
    (0, common_1.Get)('v1/dashboard'),
    openapi.ApiResponse({ status: 200, type: [require("../../common/dashboard.dto").DashboardDTO] }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('parent')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getDashboards", null);
__decorate([
    (0, common_1.Get)('v1/dashboard/default'),
    openapi.ApiResponse({ status: 200, type: require("../../common/dashboard.dto").DashboardDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getDashboardDefault", null);
__decorate([
    (0, common_1.Get)('v1/health-status'),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").PageResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getGlobalStatus", null);
__decorate([
    (0, common_1.Get)('v1/heatmap'),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").PageResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getGlobalHeatmap", null);
__decorate([
    (0, common_1.Get)('v1/dashboard/:dashboard_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/dashboard.dto").DashboardDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getDashboard", null);
__decorate([
    (0, common_1.Put)('v1/dashboard/:dashboard_id/default'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "setDefault", null);
__decorate([
    (0, common_1.Delete)('v1/dashboard/:dashboard_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "deleteDashboard", null);
__decorate([
    (0, common_1.Post)('v1/folder'),
    openapi.ApiResponse({ status: 201, type: require("../../common/dashboard.dto").CreateResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.CreateDTO]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "saveFolder", null);
__decorate([
    (0, common_1.Delete)('v1/folder/:dashboard_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_folder_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "deleteDashboardFolder", null);
__decorate([
    (0, common_1.Put)('v1/folder/:dashboard_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Param)('dashboard_folder_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.UpdateDashboardFolderDTO, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "updateDashboardFolder", null);
__decorate([
    (0, common_1.Post)('v1/dashboard'),
    openapi.ApiResponse({ status: 201, type: require("../../common/dashboard.dto").CreateResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.CreateDTO]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "saveDashboard", null);
__decorate([
    (0, common_1.Put)('v1/dashboard/:dashboard_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Param)('dashboard_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.UpdateDashboardDTO, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "updateDashboard", null);
__decorate([
    (0, common_1.Post)('v1/graph'),
    openapi.ApiResponse({ status: 201, type: require("../../common/dashboard.dto").CreateResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.CreateDashboardGraphDTO]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "saveGraph", null);
__decorate([
    (0, common_1.Post)('v1/graph/find'),
    openapi.ApiResponse({ status: 201, type: require("../../common/dashboard.dto").FindGraphsDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.FindDashboardGraphDTO]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "findGraphs", null);
__decorate([
    (0, common_1.Put)('v1/graph/:dashboard_graph_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Param)('dashboard_graph_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.UpdateDashboardGraphDTO, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "updateDashboardGraph", null);
__decorate([
    (0, common_1.Delete)('v1/graph/:dashboard_graph_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_graph_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "deleteDashboardGraph", null);
__decorate([
    (0, common_1.Get)('v1/group'),
    openapi.ApiResponse({ status: 200, type: require("../../common/dashboard.dto").DashboardGroupsDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('dashboard_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getGroups", null);
__decorate([
    (0, common_1.Post)('v1/group'),
    openapi.ApiResponse({ status: 201, type: require("../../common/dashboard.dto").CreateResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.CreateDTO]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "saveGroup", null);
__decorate([
    (0, common_1.Put)('v1/group/:dashboard_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Param)('dashboard_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, dashboard_dto_1.UpdateDashboardGroupDTO, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "updateDashboardGroup", null);
__decorate([
    (0, common_1.Delete)('v1/group/:dashboard_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "deleteDashboardGroup", null);
__decorate([
    (0, common_1.Get)('v1/folder'),
    openapi.ApiResponse({ status: 200, type: [require("../../entity/dashboardFolder.entity").DashboardFolder] }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getDashboardFolders", null);
__decorate([
    (0, common_1.Get)('v1/folder/:dashboard_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../entity/dashboardFolder.entity").DashboardFolder }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_folder_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getDashboardFolder", null);
__decorate([
    (0, common_1.Get)('v1/menu'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getMenu", null);
__decorate([
    (0, common_1.Get)('v1/menu/groups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/dashboard.dto").GroupDashboardFoldersDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getGroupsMenu", null);
__decorate([
    (0, common_1.Get)('v1/menu/folder/:dashboard_folder_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_folder_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getMenuFolder", null);
__decorate([
    openapi.ApiOperation({ description: "getPath default dashboard" }),
    (0, common_1.Get)('v1/menu/getpath/default'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getPathDefault", null);
__decorate([
    openapi.ApiOperation({ description: "getPath" }),
    (0, common_1.Get)('v1/menu/getpath/:dashboard_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getPath", null);
__decorate([
    (0, common_1.Get)('v1/menu/dashboard/:dashboard_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").PageResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('dashboard_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], DashboardController.prototype, "getMenuDashboard", null);
exports.DashboardController = DashboardController = __decorate([
    (0, swagger_1.ApiTags)('Dashboard'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/dashboard'),
    __metadata("design:paramtypes", [dashboard_service_1.DashboardService,
        menu_service_1.MenuService,
        architecture_service_1.ArchitectureService,
        custom_group_service_1.CustomGroupService])
], DashboardController);
//# sourceMappingURL=dashboard.controller.js.map