import { DashboardType, UpdateDashboardDTO, UpdateDashboardFolderDTO, UpdateDashboardGraphDTO, UpdateDashboardGroupDTO } from '@xormon/common/dashboard.dto';
import { GraphInterval } from '@xormon/common/pageResponse.dto';
import { Dashboard } from '@xormon/entity/dashboard.entity';
import { DashboardFolder } from '@xormon/entity/dashboardFolder.entity';
import { DashboardGraph as DashboardGraphEntity } from '@xormon/entity/dashboardGraph.entity';
import { DashboardGroup } from '@xormon/entity/dashboardGroup.entity';
import { Repository, TreeRepository } from 'typeorm';
declare class StoreGraph {
    width: number;
    height: number;
    dashboard_group_id: string;
    tab: string;
    interval: GraphInterval;
    url: string;
}
declare class StoreGroup {
    label: string;
    width: number;
    height: number;
    dashboard_id: string;
}
declare class StoreDashboard {
    label: string;
    parent: DashboardFolder;
    type: DashboardType;
}
declare class StoreFolder {
    label: string;
    type: DashboardType;
    parent: string;
    group_id?: number;
}
export declare class DashboardService {
    private dashboardsRepository;
    private dashboardFoldersRepository;
    private dashboardGroupsRepository;
    private dashboardGraphsRepository;
    constructor(dashboardsRepository: Repository<Dashboard>, dashboardFoldersRepository: TreeRepository<DashboardFolder>, dashboardGroupsRepository: TreeRepository<DashboardGroup>, dashboardGraphsRepository: TreeRepository<DashboardGraphEntity>);
    getDashboards(parent: string, user_id?: number, group_ids?: number[]): Promise<Dashboard[]>;
    getRootDashboardFolders(user_id?: number, group_ids?: number[]): Promise<DashboardFolder[]>;
    getRootDashboards(user_id: number): Promise<Dashboard[]>;
    getRootGroupDashboards(group_ids: number[]): Promise<Dashboard[]>;
    getAllDashboards(user_id?: number): Promise<Dashboard[]>;
    getAllFolders(user_id?: number): Promise<DashboardFolder[]>;
    getFolders(parent: DashboardFolder): Promise<DashboardFolder>;
    getParentsFolders(parent: DashboardFolder): Promise<DashboardFolder>;
    getParentsIds(parent: DashboardFolder): Promise<string[]>;
    findInParent(parent: DashboardFolder): string[];
    getPath(dashboard_folder: DashboardFolder): Promise<string>;
    findNextPath(folder: DashboardFolder): Promise<string>;
    getFolder(dashboard_folder_id: string): Promise<DashboardFolder>;
    getAllGroups(): Promise<DashboardGroup[]>;
    getDashboardFolders(parent: DashboardFolder, user_id?: number, group_ids?: number[]): Promise<DashboardFolder[]>;
    getDashboardDefault(user_id?: number): Promise<Dashboard>;
    setDefault(dashboard_id: string, user_id: number): Promise<boolean>;
    getDashboard(dashboard_id: string): Promise<Dashboard>;
    getDashboardWithGroups(dashboard_id: string, user_id?: number): Promise<Dashboard>;
    getDashboardGroup(dashboard_group_id: string): Promise<DashboardGroup>;
    getDashboardGroups(dashboard_id?: string): Promise<DashboardGroup[]>;
    findGraphs(url: string, tab: string, user_id: number, interval?: GraphInterval, group_ids?: number[]): Promise<DashboardGraphEntity[]>;
    saveGraph(graph: StoreGraph): Promise<DashboardGraphEntity>;
    saveGroup(group: StoreGroup): Promise<DashboardGroup>;
    saveDashboardFolder(folder: StoreFolder, user_id?: number): Promise<DashboardFolder>;
    findDashboardsInFolder(dashboardFolder: DashboardFolder, ids: string[]): Promise<string[]>;
    deleteDashboardFolder(dashboard_folder_id: string): Promise<boolean>;
    saveDashboard(dashboard: StoreDashboard, user_id?: number): Promise<Dashboard>;
    deleteDashboard(dashboard_id: string): Promise<boolean>;
    getDashboardGraph(dashboard_graph_id: string): Promise<DashboardGraphEntity>;
    updateDashboardGraph(dashboard_graph_id: string, dashboard_graph: UpdateDashboardGraphDTO): Promise<Boolean>;
    updateDashboard(dashboard_id: string, dashboard: UpdateDashboardDTO): Promise<boolean>;
    updateDashboardFolder(dashboard_folder_id: string, dashboard_folder: UpdateDashboardFolderDTO): Promise<Boolean>;
    deleteDashboardGraph(dashboard_graph_id: string): Promise<boolean>;
    updateDashboardGroup(dashboard_group_id: string, dashboard_group: UpdateDashboardGroupDTO): Promise<Boolean>;
    deleteDashboardGroup(dashboard_group_id: string): Promise<boolean>;
    resortGroupHorizontal(dashboard_id: string, y: number, start_x: number): Promise<boolean>;
    resortGroup(dashboard_group_id: string, dashboard_id: string, x: number, y: number): Promise<boolean>;
    checkDashboardOwner(dashboard_id: string, user_id: number, group_ids?: number[]): Promise<boolean>;
    checkDashboardFolderOwner(dashboard_folder_id: string, user_id: number, group_ids: number[]): Promise<boolean>;
}
export {};
