"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Exporter = void 0;
const exporter_constants_1 = require("../../common/constants/exporter.constants");
const dataSource_1 = require("../../config/dataSource");
const itemProperty_entity_1 = require("../../entity/itemProperty.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const Metric_1 = require("../metrics/Metric");
const typeorm_1 = require("typeorm");
const CustomGroupWrapper_1 = require("../custom_group/CustomGroupWrapper");
const ItemGroupWrapper_1 = require("../custom_group/ItemGroupWrapper");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const ACLCheck_1 = require("../acl/ACLCheck");
const class_constants_1 = require("../../common/constants/class.constants");
;
const Definitions_1 = require("../metrics/Definitions");
class Exporter {
    static async loadItems(request) {
        let items = [];
        if (request.uuids) {
            items = (await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findBy({ item_id: (0, typeorm_1.In)(request.uuids) })).map(item => {
                return {
                    label: item.label,
                    item_id: item.item_id
                };
            });
        }
        else if (request.custom_group_id) {
            items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(request.custom_group_id);
        }
        else {
            items = await ItemGroupWrapper_1.ItemGroupWrapper.getItemsByData(request, request.subsystem, request.hw_type, request.class);
        }
        return this.addRootParent(items);
    }
    static async addRootParent(items) {
        return (0, Definitions_1.addRootParent)((0, Definitions_1.ObjectItemToArchItem)(await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(items.map(i => i.item_id))
            },
            relations: {
                parents: {
                    _parent: true
                }
            }
        })));
    }
    static async applyFilter(items, filter, start, end) {
        let itemsArray = items.map(item => item.item_id);
        const filteredItems = await (new Metric_1.Metric(filter.metric, itemsArray)).getWithOperator(filter.operator, filter.value, start, end);
        itemsArray = Object.keys(filteredItems);
        return items.filter(item => itemsArray.includes(item.item_id));
    }
    static async mapping(exportRequest, request, format, res) {
        let items = await this.loadItems(exportRequest);
        if (request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(request, items.map(item => item.item_id));
            items = items.filter(item => ids.includes(item.item_id));
        }
        let hosts = items.length >= 1
            ? await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).find({
                relations: {
                    children: {
                        _child: true
                    },
                    parents: {
                        _parent: true
                    },
                    hw_type_entity: true
                },
                where: {
                    hw_type_entity: {
                        class: class_constants_1.ClassKey.STORAGE
                    },
                    subsystem: 'host',
                    item_id: exportRequest.subsystem === 'host'
                        ? (0, typeorm_1.In)(items.map(i => i.item_id))
                        : undefined,
                    parents: {
                        _parent: {
                            subsystem: 'device',
                            item_id: exportRequest.subsystem === 'device'
                                ? (0, typeorm_1.In)(items.map(i => i.item_id))
                                : undefined
                        }
                    },
                    children: {
                        child: exportRequest.subsystem === 'volume'
                            ? (0, typeorm_1.In)(items.map(i => i.item_id))
                            : undefined,
                        _child: {
                            subsystem: 'volume'
                        }
                    }
                }
            })
            : [];
        let hostData = [];
        for (const host of hosts) {
            hostData.push({
                item_id: host.item_id,
                label: host.label,
                device: host.parents.find(p => p._parent.subsystem === 'device')?._parent?.label,
                volumes: host.children.map(ch => ch._child?.label)
            });
        }
        if (format === 'csv') {
            let csv = 'host;device;volumes\n';
            for (const host of hostData) {
                csv += `${host.label};${host.device};${host.volumes.toString()}\n`;
            }
            return csv;
        }
        else if (!format || format === 'json') {
            return hostData;
        }
        else if (format === 'xlsx') {
            const ExcelJS = require('exceljs');
            const workbook = new ExcelJS.Workbook();
            const sheet = workbook.addWorksheet("Export");
            sheet.addRow(['host', 'device', 'volums']);
            for (const host of hostData) {
                sheet.addRow([host.label, host.device, host.volumes.toString()]);
            }
            const fileName = 'export.xlsx';
            res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
            res.setHeader('Content-Disposition', 'attachment; filename=' + fileName);
            await workbook.xlsx.write(res);
            res.end();
        }
    }
    static async timeseries(exportRequest, request) {
        if (!this.isValid(exportRequest)) {
            return null;
        }
        let items = await this.loadItems(exportRequest);
        if (request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(request, items.map(item => item.item_id));
            items = items.filter(item => ids.includes(item.item_id));
        }
        if (!Array.isArray(exportRequest.metric)) {
            exportRequest.metric = [exportRequest.metric];
        }
        if (exportRequest.metricFilter && exportRequest.metricFilter.operator) {
            if (exportRequest.metricFilter.metric === undefined && exportRequest.metric.length === 1) {
                exportRequest.metricFilter.metric = exportRequest.metric[0];
            }
            items = await this.applyFilter(items, exportRequest.metricFilter, exportRequest.start, exportRequest.end);
        }
        let interval = 1440;
        if ((Date.now() / 1000) - exportRequest.start < this.LIMIT_1M) {
            interval = 1;
        }
        else if ((Date.now() / 1000) - exportRequest.start < this.LIMIT_5M) {
            interval = 5;
        }
        else if ((Date.now() / 1000) - exportRequest.start < this.LIMIT_1H) {
            interval = 60;
        }
        else if ((Date.now() / 1000) - exportRequest.start < this.LIMIT_6H) {
            interval = 360;
        }
        const metrics = await new MetricsWrapper_1.MetricsWrapper(exportRequest.metric, items.map(item => item.item_id)).getDataFromTable(exportRequest.start, exportRequest.end, undefined, undefined, interval);
        for (const itemId in metrics) {
            for (const metric in metrics[itemId]) {
                for (const time in metrics[itemId][metric]) {
                    const timestamp = new Date(time).getTime() / 1000;
                    if (timestamp < exportRequest.start || timestamp > exportRequest.end) {
                        delete metrics[itemId][metric][time];
                    }
                }
            }
        }
        return {
            items: items,
            metricsItems: metrics
        };
    }
    static async configuration(exportRequest, request, format, res) {
        if (!this.isValid(exportRequest)) {
            return null;
        }
        let items = await this.loadItems(exportRequest);
        if (items.length === 0) {
            return null;
        }
        if (request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(request, items.map(item => item.item_id));
            items = items.filter(item => ids.includes(item.item_id));
        }
        const definitions = items[0] ? await MetricsWrapper_1.MetricsWrapper.getConfigurationFindClass(items[0].hw_type, items[0].subsystem) : {};
        const itemProperties = await (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty).findBy({
            item_id: (0, typeorm_1.In)(items.map(item => item.item_id)),
            property_name: exportRequest.properties !== undefined
                ? (0, typeorm_1.In)(exportRequest.properties)
                : undefined
        });
        const headers = itemProperties.reduce((found, next) => {
            if (!found.includes(next.property_name)) {
                found.push(next.property_name);
            }
            return found;
        }, []);
        let configurationItems = {};
        for (const itemProperty of itemProperties) {
            if (configurationItems[itemProperty.item_id]) {
                configurationItems[itemProperty.item_id][itemProperty.property_name] = itemProperty.property_value;
            }
            else {
                configurationItems[itemProperty.item_id] = {
                    [itemProperty.property_name]: itemProperty.property_value
                };
            }
        }
        if (format === 'csv') {
            let csv = 'item_id;label;parent';
            headers.forEach(header => {
                csv += `;${header}${definitions[header]?.shortcut ? ` [${definitions[header].shortcut}]` : ''}`;
            });
            items.forEach(item => {
                csv += `\n${item.item_id};${item.label};${item.root_parent ? item.root_parent.label : ''}`;
                headers.forEach(header => {
                    if (configurationItems[item.item_id] && configurationItems[item.item_id][header]) {
                        csv += `;${configurationItems[item.item_id][header]}`;
                    }
                    else {
                        csv += ";";
                    }
                });
            });
            return csv;
        }
        else if (!format || format === 'json') {
            let data = [];
            for (const item of items) {
                data.push({
                    item_id: item.item_id,
                    label: item.label,
                    parent: item.root_parent ? item.root_parent.label : undefined,
                    configuration: configurationItems[item.item_id]
                });
            }
            return data;
        }
        else if (format === 'xlsx') {
            const ExcelJS = require('exceljs');
            const workbook = new ExcelJS.Workbook();
            const sheet = workbook.addWorksheet("Export");
            let header = ['item_id', 'label', 'parent'];
            headers.forEach(h => {
                header.push(`${h}${definitions[h]?.shortcut ? ` [${definitions[h].shortcut}]` : ''}`);
            });
            sheet.addRow(header);
            for (const item of items) {
                let row = [item.item_id, item.label, item.root_parent ? item.root_parent.label : ''];
                headers.forEach(header => {
                    if (configurationItems[item.item_id] && configurationItems[item.item_id][header]) {
                        row.push(configurationItems[item.item_id][header]);
                    }
                    else {
                        row.push(null);
                    }
                });
                sheet.addRow(row);
            }
            const fileName = 'export.xlsx';
            res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
            res.setHeader('Content-Disposition', 'attachment; filename=' + fileName);
            await workbook.xlsx.write(res);
            res.end();
        }
    }
    static async capacity(exportRequest, request, format, res) {
        let items = await this.loadItems(exportRequest);
        if (items.length === 0) {
            return null;
        }
        if (request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(request, items.map(item => item.item_id));
            items = items.filter(item => ids.includes(item.item_id));
        }
        const metrics = ['capacity_free', 'capacity_used', 'capacity_total'];
        const metricsWrapper = new MetricsWrapper_1.MetricsWrapper(metrics, items.map(i => i.item_id));
        const data = await metricsWrapper.getLastValuesFromTable();
        if (format === 'csv') {
            let csv = 'item_id;label;parent';
            metrics.forEach(metric => {
                csv += `;${metric}`;
            });
            items.forEach(item => {
                csv += `\n${item.item_id};${item.label};${item.root_parent ? item.root_parent.label : ''}`;
                metrics.forEach(metric => {
                    if (data[item.item_id] && data[item.item_id][metric]) {
                        csv += `;${data[item.item_id][metric]}`;
                    }
                    else {
                        csv += ";";
                    }
                });
            });
            return csv;
        }
        else if (!format || format === 'json') {
            let jData = [];
            for (const item of items) {
                jData.push({
                    item_id: item.item_id,
                    label: item.label,
                    parent: item.root_parent ? item.root_parent.label : undefined,
                    data: data[item.item_id]
                });
            }
            return jData;
        }
        else if (format === 'xlsx') {
            const ExcelJS = require('exceljs');
            const workbook = new ExcelJS.Workbook();
            const sheet = workbook.addWorksheet("Export");
            let header = ['item_id', 'label', 'parent'];
            metrics.forEach(m => {
                header.push(m);
            });
            sheet.addRow(header);
            for (const item of items) {
                let row = [item.item_id, item.label, item.root_parent ? item.root_parent.label : ''];
                metrics.forEach(metric => {
                    if (data[item.item_id] && data[item.item_id][metric]) {
                        row.push(data[item.item_id][metric]);
                    }
                    else {
                        row.push(null);
                    }
                });
                sheet.addRow(row);
            }
            const fileName = 'export.xlsx';
            res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
            res.setHeader('Content-Disposition', 'attachment; filename=' + fileName);
            await workbook.xlsx.write(res);
            res.end();
        }
    }
    static async valuesToCsv(exportRequest, request) {
        let csv = '';
        exportRequest.headers.forEach(header => {
            csv += `${header};`;
        });
        csv += `\n`;
        exportRequest.values.forEach(row => {
            row.forEach(column => {
                csv += `${column};`;
            });
            csv += `\n`;
        });
        return csv;
    }
    static isValid(request) {
        return true;
    }
    static getTimeseriesInterval(data, freq) {
        let start;
        let end;
        if (data.interval !== undefined) {
            const d = new Date();
            if (data.interval) {
                d.setHours(0, 0, 0, 0);
                end = d.getTime();
            }
            else {
                end = d.getTime();
            }
            switch (freq) {
                case exporter_constants_1.MAIL_FREQUENCY.daily:
                    if (data.interval) {
                        d.setDate(d.getDate() - 1);
                        start = d.getTime();
                    }
                    else {
                        start = end - (1 * 24 * 60 * 60 * 1000);
                    }
                    break;
                case exporter_constants_1.MAIL_FREQUENCY.weekly:
                    if (data.interval) {
                        d.setDate(d.getDate() - 7);
                        start = d.getTime();
                    }
                    else {
                        start = end - (7 * 24 * 60 * 60 * 1000);
                    }
                    break;
                case exporter_constants_1.MAIL_FREQUENCY.monthly:
                    if (data.interval) {
                        d.setMonth(d.getMonth() - 1);
                        start = d.getTime();
                    }
                    else {
                        start = end - (31 * 24 * 60 * 60 * 1000);
                    }
                    break;
                case exporter_constants_1.MAIL_FREQUENCY.yearly:
                    if (data.interval) {
                        d.setFullYear(d.getFullYear() - 1);
                        start = d.getTime();
                    }
                    else {
                        start = end - (365 * 24 * 60 * 60 * 1000);
                    }
                    break;
            }
            start = start / 1000;
            end = end / 1000;
        }
        return {
            start: start,
            end: end
        };
    }
}
exports.Exporter = Exporter;
Exporter.LIMIT_1M = 60 * 60 * 24 * 30;
Exporter.LIMIT_5M = 60 * 60 * 24 * 30 * 3;
Exporter.LIMIT_1H = 60 * 60 * 24 * 30 * 6;
Exporter.LIMIT_6H = 60 * 60 * 24 * 30 * 12;
Exporter.LIMIT_1D = 60 * 60 * 24 * 30 * 12 * 3;
//# sourceMappingURL=Exporter.js.map