"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExporterFormatter = void 0;
const exporter_constants_1 = require("../../common/constants/exporter.constants");
const Metric_1 = require("../metrics/Metric");
const Item_1 = require("../configuration/Item");
class ExporterFormatter {
    constructor(data) {
        this.metrics = [];
        this.data = data;
        for (const item of data.items) {
            if (data.metricsItems[item.item_id]) {
                for (const metric in this.data.metricsItems[item.item_id]) {
                    this.metrics.push(metric);
                }
                if (Object.keys(data.metricsItems[item.item_id]).length >= 1) {
                    break;
                }
            }
        }
    }
    setStartEnd(data) {
        this.start = data.start;
        this.end = data.end;
    }
    formatNumber(num) {
        if (num === null) {
            return null;
        }
        return Math.round(num * (100)) / 100;
    }
    async exportTimeseriesToResponse(format, res) {
        const metricDefinitions = await Item_1.Item.getMetrics(this.data.items.map(item => item.item_id));
        if (!format || format === exporter_constants_1.EXPORT_FORMAT.json) {
            let response = {
                metadata: {
                    total: this.data.items.length,
                    interval: Metric_1.Metric.getInterval(this.start, this.end),
                },
                items: [],
            };
            for (const item of this.data.items) {
                for (const metric in this.data.metricsItems[item.item_id]) {
                    const def = metricDefinitions.find(metricObj => metricObj.metric === metric);
                    response.items.push({
                        item_id: item.item_id,
                        label: item.label,
                        parent: item.root_parent ? item.root_parent.label : undefined,
                        metric: def && def.shortcut
                            ? `${metric} [${def.shortcut}]`
                            : metric,
                        timeseries: this.data.metricsItems[item.item_id][metric],
                    });
                }
            }
            res.send(response);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.csv) {
            let response = 'time;item;parent';
            this.metrics.forEach(metric => {
                response += `;${metric}`;
                const def = metricDefinitions.find(metricObj => metricObj.metric === metric);
                if (def && def.shortcut) {
                    response += ` [${def.shortcut}]`;
                }
            });
            response += "\n";
            for (const item of this.data.items) {
                const tmpData = {};
                for (const metric in this.data.metricsItems[item.item_id]) {
                    for (const time in this.data.metricsItems[item.item_id][metric]) {
                        if (!tmpData[time]) {
                            tmpData[time] = {};
                        }
                        tmpData[time][metric] = this.formatNumber(this.data.metricsItems[item.item_id][metric][time]);
                    }
                }
                for (const time in tmpData) {
                    response += `${time};${item.label};${item.root_parent ? item.root_parent.label : ''}`;
                    for (const metric in tmpData[time]) {
                        response += `;${tmpData[time][metric]}`;
                    }
                    response += "\n";
                }
            }
            res.send(response);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.xlsx) {
            const ExcelJS = require('exceljs');
            const workbook = new ExcelJS.Workbook();
            const sheet = workbook.addWorksheet("Export");
            let header = ['time', 'item', 'parent'];
            this.metrics.forEach(metric => {
                const def = metricDefinitions.find(metricObj => metricObj.metric === metric);
                if (def && def.shortcut) {
                    header.push(`${metric} [${def.shortcut}]`);
                }
                else {
                    header.push(metric);
                }
            });
            sheet.addRow(header);
            for (const item of this.data.items) {
                const tmpData = {};
                for (const metric in this.data.metricsItems[item.item_id]) {
                    for (const time in this.data.metricsItems[item.item_id][metric]) {
                        if (!tmpData[time]) {
                            tmpData[time] = {};
                        }
                        tmpData[time][metric] = this.formatNumber(this.data.metricsItems[item.item_id][metric][time]);
                    }
                }
                for (const time in tmpData) {
                    const row = [time, item.label, item.root_parent ? item.root_parent.label : ''];
                    for (const metric in tmpData[time]) {
                        row.push(tmpData[time][metric]);
                    }
                    sheet.addRow(row);
                }
            }
            const fileName = 'export.xlsx';
            res.setHeader('Content-Type', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
            res.setHeader('Content-Disposition', 'attachment; filename=' + fileName);
            await workbook.xlsx.write(res);
            res.end();
        }
        else {
            res.send(null);
        }
    }
    async exportTimeseriesToFile(format, file) {
        const fs = require('fs');
        if (!format || format === exporter_constants_1.EXPORT_FORMAT.csv) {
            let data = 'time;item';
            this.metrics.forEach(metric => {
                data += `;${metric}`;
            });
            data += "\n";
            for (const item of this.data.items) {
                const tmpData = {};
                for (const metric in this.data.metricsItems[item.item_id]) {
                    for (const time in this.data.metricsItems[item.item_id][metric]) {
                        if (!tmpData[time]) {
                            tmpData[time] = {};
                        }
                        tmpData[time][metric] = this.formatNumber(this.data.metricsItems[item.item_id][metric][time]);
                    }
                }
                for (const time in tmpData) {
                    data += `${time};${item.label}`;
                    for (const metric in tmpData[time]) {
                        data += `;${tmpData[time][metric]}`;
                    }
                    data += "\n";
                }
            }
            await fs.writeFileSync(file + '.csv', data);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.json) {
            let data = {
                metadata: {
                    total: this.data.items.length,
                    interval: Metric_1.Metric.getInterval(this.start, this.end),
                },
                items: [],
            };
            for (const item of this.data.items) {
                for (const metric in this.data.metricsItems[item.item_id]) {
                    data.items.push({
                        item_id: item.item_id,
                        label: item.label,
                        metric: metric,
                        timeseries: this.data.metricsItems[item.item_id][metric],
                    });
                }
            }
            let json = JSON.stringify(data);
            await fs.writeFileSync(file + '.json', json);
        }
        else if (format === exporter_constants_1.EXPORT_FORMAT.xlsx) {
            const ExcelJS = require('exceljs');
            const workbook = new ExcelJS.Workbook();
            const sheet = workbook.addWorksheet("Export");
            let header = ['time', 'item'];
            this.metrics.forEach(metric => {
                header.push(metric);
            });
            sheet.addRow(header);
            for (const item of this.data.items) {
                const tmpData = {};
                for (const metric in this.data.metricsItems[item.item_id]) {
                    for (const time in this.data.metricsItems[item.item_id][metric]) {
                        if (!tmpData[time]) {
                            tmpData[time] = {};
                        }
                        tmpData[time][metric] = this.formatNumber(this.data.metricsItems[item.item_id][metric][time]);
                    }
                }
                for (const time in tmpData) {
                    const row = [time, item.label];
                    for (const metric in tmpData[time]) {
                        row.push(tmpData[time][metric]);
                    }
                    sheet.addRow(row);
                }
            }
            await workbook.xlsx.writeFile(file + '.xlsx');
        }
    }
}
exports.ExporterFormatter = ExporterFormatter;
//# sourceMappingURL=ExporterFormatter.js.map