"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HeatmapController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const heatmap_dto_1 = require("../../common/heatmap.dto");
const heatmap_service_1 = require("./heatmap.service");
let HeatmapController = class HeatmapController {
    constructor(heatmapService) {
        this.heatmapService = heatmapService;
    }
    async tiers() {
        let tiers = [];
        const heatmapTiers = await this.heatmapService.getHeatmapTiers();
        for (const heatmapTier of heatmapTiers) {
            tiers.push(heatmapTier);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: tiers,
        };
    }
    async tier(heatmap_tier_id) {
        const heatmapTier = await this.heatmapService.getHeatmapTier(heatmap_tier_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: heatmapTier,
        };
    }
    async updateTier(heatmap_tier_id, heatmap_tier) {
        const saved = await this.heatmapService.saveHeatmapTier({
            ...heatmap_tier,
            heatmap_tier_id: heatmap_tier_id,
        });
        if (saved.heatmap_tier_id) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Heatmap tier successfully updated',
                data: saved,
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Heatmap tier was not updated',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteTier(heatmap_tier_id) {
        const deleted = await this.heatmapService.deleteHeatmapTier(heatmap_tier_id);
        if (deleted.affected === 1) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Heatmap tier successfully deleted'
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Heatmap tier was not deleted',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveTier(heatmap_tier) {
        const saved = await this.heatmapService.saveHeatmapTier(heatmap_tier);
        if (saved.heatmap_tier_id) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Heatmap tier successfully created',
                data: saved,
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Heatmap tier was not saved',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async devices() {
        let devices = [];
        const items = await this.heatmapService.getDevices();
        for (const item of items) {
            let pools = [];
            for (const child of item.children) {
                if (child._child.subsystem === 'pool') {
                    pools.push({
                        label: child._child.label,
                        item_id: child._child.item_id,
                        heatmap_tier_id: child._child.heatmap_tier_item
                            ? child._child.heatmap_tier_item.heatmap_tier_id
                            : null
                    });
                }
            }
            devices.push({
                label: item.label,
                pools: pools
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: devices,
        };
    }
    async saveDevices(devices) {
        let heatmapTierItems = [];
        let deletedTiers = [];
        for (const device of devices) {
            for (const pool of device.pools) {
                if (pool.heatmap_tier_id) {
                    heatmapTierItems.push({
                        item_id: pool.item_id,
                        heatmap_tier_id: pool.heatmap_tier_id,
                    });
                }
                else {
                    deletedTiers.push(pool.item_id);
                }
            }
        }
        const saved = await this.heatmapService.saveHeatmapTierItems(heatmapTierItems);
        await this.heatmapService.deleteHeatmapTierItemsByIds(deletedTiers);
        if (saved.length >= 1) {
            const devices = await this.devices();
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Heatmap devices successfully saved',
                data: devices.data,
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Heatmap devices was not updated',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async saveHeatmapTierItem(heatmapTierItems) {
        const saved = await this.heatmapService.saveHeatmapTierItems(heatmapTierItems);
        if (saved.length >= 1) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'Heatmap tier items successfully saved',
            };
        }
        else {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.BAD_REQUEST,
                message: 'Heatmap tier items was not saved',
                error: 'bad request',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
    }
};
exports.HeatmapController = HeatmapController;
__decorate([
    (0, common_1.Get)('v1/tiers'),
    openapi.ApiResponse({ status: 200, type: require("../../common/heatmap.dto").TiersResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "tiers", null);
__decorate([
    (0, common_1.Get)('v1/tiers/:heatmap_tier_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/heatmap.dto").TierResponseDTO }),
    __param(0, (0, common_1.Param)('heatmap_tier_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "tier", null);
__decorate([
    (0, common_1.Put)('v1/tiers/:heatmap_tier_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/heatmap.dto").TierResponseDTO }),
    __param(0, (0, common_1.Param)('heatmap_tier_id')),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, heatmap_dto_1.TierBase]),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "updateTier", null);
__decorate([
    (0, common_1.Delete)('v1/tiers/:heatmap_tier_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('heatmap_tier_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "deleteTier", null);
__decorate([
    (0, common_1.Post)('v1/tiers'),
    openapi.ApiResponse({ status: 201, type: require("../../common/heatmap.dto").TierResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [heatmap_dto_1.TierBase]),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "saveTier", null);
__decorate([
    (0, common_1.Get)('v1/devices'),
    openapi.ApiResponse({ status: 200, type: require("../../common/heatmap.dto").DevicesResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "devices", null);
__decorate([
    (0, swagger_1.ApiBody)({
        isArray: true,
        type: heatmap_dto_1.Device,
    }),
    (0, common_1.Post)('v1/devices'),
    openapi.ApiResponse({ status: 201, type: require("../../common/heatmap.dto").DevicesResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "saveDevices", null);
__decorate([
    (0, common_1.Post)('v1/items'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], HeatmapController.prototype, "saveHeatmapTierItem", null);
exports.HeatmapController = HeatmapController = __decorate([
    (0, swagger_1.ApiTags)('Heatmap'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/heatmap'),
    __metadata("design:paramtypes", [heatmap_service_1.HeatmapService])
], HeatmapController);
//# sourceMappingURL=heatmap.controller.js.map