"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var LdapService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.LdapService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const util_1 = require("../../common/util");
const groupLdapGroup_entity_1 = require("../../entity/groupLdapGroup.entity");
const ldap_entity_1 = require("../../entity/ldap.entity");
const ldapGroup_entity_1 = require("../../entity/ldapGroup.entity");
const fs = require("fs");
const ldapts_1 = require("ldapts");
const tls_1 = require("tls");
const typeorm_2 = require("typeorm");
const Settings_1 = require("../configuration/Settings");
const WinstonLogger_1 = require("../log/WinstonLogger");
const ldap_cert_service_1 = require("./ldap.cert.service");
const ldap_util_1 = require("./ldap.util");
let LdapService = LdapService_1 = class LdapService {
    constructor(ldapCertService, ldapRepository, ldapGroupsRepository, groupLdapGroupsRepository) {
        this.ldapCertService = ldapCertService;
        this.ldapRepository = ldapRepository;
        this.ldapGroupsRepository = ldapGroupsRepository;
        this.groupLdapGroupsRepository = groupLdapGroupsRepository;
        this.log = LdapService_1.logger;
        this.settings = {};
    }
    async getLdaps() {
        return await this.ldapRepository.find({
            relations: {
                _groups: true
            }
        });
    }
    async getLdap(ldap_id) {
        return await this.ldapRepository.findOne({
            where: {
                ldap_id: ldap_id
            },
            relations: {
                _groups: true
            }
        });
    }
    async saveLdap(ldapOrSave) {
        const ldap = ldapOrSave;
        if (ldap._groups) {
            const result = await this.ldapGroupsRepository.upsert(ldap._groups, ['ldap_group', 'ldap_id']);
            const groups = (result).generatedMaps;
            for (let i = 0; i < groups.length; i++) {
                groups[i] = { ...groups[i], ...result.identifiers[i] };
            }
            ldap._groups = groups;
        }
        const saved = await this.ldapRepository.save(ldap);
        if (saved?.ldap_id) {
            return saved;
        }
        else {
            return null;
        }
    }
    async deleteLdap(ldap_id) {
        const deleted = await this.ldapRepository.delete({ ldap_id: ldap_id });
        return deleted.affected === 1;
    }
    async getLdapGroups() {
        return await this.ldapGroupsRepository.find({
            relations: {
                _ldap: true
            }
        });
    }
    async getGroupsByLdapGroups(ldap_id, ldap_groups) {
        return (await this.groupLdapGroupsRepository.find({
            where: {
                ldap_id: ldap_id,
                ldap_group: (0, typeorm_2.In)(ldap_groups)
            },
            relations: {
                _group: true
            }
        })).map(ldap_group => {
            return ldap_group._group;
        });
    }
    async updateGroupLdapGroups(group_id, ldap_groups) {
        await this.groupLdapGroupsRepository.delete({ group_id: group_id });
        return await this.groupLdapGroupsRepository.save(ldap_groups.map(ldap_group => {
            return {
                group_id: group_id,
                ldap_id: ldap_group.ldap_id,
                ldap_group: ldap_group.ldap_group
            };
        }));
    }
    async saveLdapGroups(ldapGroups) {
        return (await this.ldapGroupsRepository.save(ldapGroups, { chunk: 1000 })).length === ldapGroups.length;
    }
    async deleteLdapGroups(ldap_id) {
        await this.groupLdapGroupsRepository.delete({
            ldap_id: ldap_id
        });
        await this.ldapGroupsRepository.delete({
            ldap_id: ldap_id
        });
    }
    async testLdapAllGroups(ldapData) {
        const client = await this.createClient(ldapData);
        try {
            await this.handleLogin(client, ldapData);
            const groupBaseDN = (ldapData.group_base?.trim() ? (ldapData.group_base + ',') : '') + ldapData.base_dn;
            const allGroups = await this.search({
                ...ldapData,
                base_dn: groupBaseDN
            }, {
                filter: '(objectClass=' + ldapData.group_object_class + ')',
                scope: 'sub'
            }, client);
            return allGroups.filter((0, util_1.uniqueArrayFilterFce)((a, b) => a.dn === b.dn));
        }
        finally {
            client.unbind();
        }
    }
    search(ldapData, opts, client, attr) {
        this.log.info(`[Search]: ${ldapData.base_dn}\n\t${JSON.stringify(opts)}`);
        return new Promise((resolve, reject) => {
            client.search(ldapData.base_dn, {
                paged: { pageSize: +this.settings.LDAP_PAGE_SIZE || 500 }, timeLimit: +this.settings.LDAP_TIMEOUT_SEC || 3600,
                attributes: ['cn', attr ?? ''], ...opts
            }).then(result => {
                let data = [];
                for (const entry of result.searchEntries) {
                    data.push({ cn: entry.cn, dn: ldap_util_1.LdapUtil.trimDN(entry.dn).toLowerCase(), [attr]: entry[attr] });
                }
                this.log.debug(`[Search results]:`, data);
                resolve(data);
            }, reason => {
                reject(reason);
            });
        });
    }
    async createClient(ldapData) {
        Settings_1.Settings.get({ keys: ['LDAP_PAGE_SIZE', 'LDAP_TIMEOUT_SEC'] }).then(values => {
            this.settings = values;
        }, reason => this.log.error('Failed to get settings', reason));
        this.log.info(`[Client]: url: ${ldapData.url.map(url => ldapData.protocol + '://' + url)}`);
        const certs = await this.ldapCertService.getLdapCertificates();
        return new ldapts_1.Client({
            url: ldapData.url.map(url => ldapData.protocol + '://' + url)[0],
            tlsOptions: ldapData.protocol === 'ldaps' && {
                ca: [...tls_1.rootCertificates, ...(certs?.length ? certs.map(lc => fs.readFileSync(lc.path)) : [])],
                checkServerIdentity: () => undefined,
            }
        });
    }
    getBindDN(ldapData) {
        let bindDN;
        const fullLogin = (ldapData.user_prefix || '') + ldapData.login + (ldapData.user_postfix || '');
        if (ldapData.technical) {
            bindDN = ldapData.login;
        }
        else if (ldapData.full_dn_auth) {
            bindDN = `${ldapData.user_filter_attribute}=${fullLogin},`;
            if (ldapData.user_base) {
                bindDN += ldapData.user_base + ',';
            }
            bindDN += ldapData.base_dn;
        }
        else
            bindDN = fullLogin;
        this.log.info(`[Bind]: ${bindDN}`);
        return bindDN;
    }
    async getUserGroups(ldapData, client, bindDN) {
        let userDN = bindDN;
        if (!ldapData.full_dn_auth) {
            const usernameNormalized = bindDN.substring(bindDN.lastIndexOf('\\') + 1).toLowerCase();
            const userProps = await this.search(ldapData, { filter: `(&(objectClass=${ldapData.user_object_class})(${ldapData.user_filter_attribute}=${usernameNormalized}))`, scope: 'sub' }, client, ldapData.group_filter_attribute);
            if (userProps.length != 1) {
                throw new Error(`${userProps.length} objects found for user ${bindDN}\nUser props: ` + JSON.stringify(userProps));
            }
            if (ldapData.group_filter_type === 'user') {
                const groups = userProps[0][ldapData.group_filter_attribute];
                return Array.isArray(groups) ? groups : [groups];
            }
            userDN = userProps[0].dn;
        }
        else if (ldapData.group_filter_type === 'user') {
            return (await this.search({ ...ldapData, base_dn: bindDN }, { filter: '(objectClass=*)', scope: 'base' }, client, ldapData.group_filter_attribute)).map(g => g[ldapData.group_filter_attribute]);
        }
        userDN = ldap_util_1.LdapUtil.escapeFilter(userDN);
        return (await this.search(ldapData, {
            filter: `(&(objectClass=${ldapData.group_object_class})(${ldapData.group_filter_attribute}=${userDN}))`,
            scope: 'sub'
        }, client)).map(group => group.dn);
    }
    async login(username, password) {
        const ldaps = await this.getLdaps();
        const errors = [];
        for (const ldapData of ldaps) {
            const client = await this.createClient(ldapData);
            let groups;
            try {
                groups = await this.handleLogin(client, { ...ldapData, login: username, password });
                return {
                    ldap_id: ldapData.ldap_id,
                    groups: groups
                };
            }
            catch (e) {
                errors.push(e);
            }
            finally {
                client.unbind();
            }
        }
        this.log.error(errors);
        return null;
    }
    async handleLogin(client, ldapData) {
        const bindDN = this.getBindDN(ldapData);
        await new Promise((resolve, reject) => {
            client.bind(bindDN, ldapData.password).then(() => resolve(), reason => reject(reason));
        });
        const groups = (await this.getUserGroups({ ...ldapData }, client, bindDN)).map(s => s.toLocaleLowerCase());
        if (ldapData.required_user_group) {
            const requiredGroup = ldap_util_1.LdapUtil.trimDN(ldapData.required_user_group).toLowerCase();
            if (!groups.some(g => requiredGroup === g)) {
                this.log.warn(`User ${bindDN} authenticated but doesn't have required group ${requiredGroup}`);
                throw new common_1.UnauthorizedException({
                    statusCode: common_1.HttpStatus.UNAUTHORIZED,
                    message: 'login failed',
                    error: 'group required'
                });
            }
        }
        return groups;
    }
};
exports.LdapService = LdapService;
LdapService.logger = (0, WinstonLogger_1.LoggerFactory)('ldap');
exports.LdapService = LdapService = LdapService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, typeorm_1.InjectRepository)(ldap_entity_1.Ldap)),
    __param(2, (0, typeorm_1.InjectRepository)(ldapGroup_entity_1.LdapGroup)),
    __param(3, (0, typeorm_1.InjectRepository)(groupLdapGroup_entity_1.GroupLdapGroup)),
    __metadata("design:paramtypes", [ldap_cert_service_1.LdapCertService,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], LdapService);
//# sourceMappingURL=ldap.service.js.map