"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LogController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const log_dto_1 = require("../../common/log.dto");
const log_service_1 = require("./log.service");
const fs_1 = require("fs");
const Dump_1 = require("../configuration/Dump");
const xormon_exception_1 = require("../../exception/xormon.exception");
const WinstonLogger_1 = require("./WinstonLogger");
const User_1 = require("../users/User");
let LogController = class LogController {
    constructor(logService) {
        this.logService = logService;
    }
    async list() {
        const logs = await this.logService.listLogs();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: logs,
        };
    }
    async genLogs(res, logRequest) {
        let tarFile;
        const dump = new Dump_1.Dump(false);
        try {
            if (logRequest.sql) {
                await dump.sql(logRequest.sql).catch(e => {
                    console.error("[Dump] [SQL] Error: " + e);
                });
            }
            await dump.generate();
            tarFile = await dump.gzip();
        }
        catch (e) {
            console.error("Failed to generate logs. Error: " + e);
            throw new xormon_exception_1.XormonException("Failed to generate logs", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        const file = (0, fs_1.createReadStream)(tarFile);
        res.set({
            'Content-Type': 'application/x-tar',
            'Content-Disposition': `attachment; filename="logs.tar.gz"`,
        });
        res.on("finish", () => {
            try {
                const fs = require('fs');
                fs.rmSync(tarFile);
            }
            catch (error) {
                console.log(error);
            }
        });
        return new common_1.StreamableFile(file);
    }
    async backendLogs(query) {
        const logs = await this.logService.openLog('backend', query.lines);
        if (logs === null) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.NOT_FOUND,
                message: 'logs not found',
                error: 'logs not found',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: logs
        };
    }
    async alertingLogs(query) {
        const logs = await this.logService.openLog('alerting', query.lines);
        if (logs === null) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.NOT_FOUND,
                message: 'logs not found',
                error: 'logs not found',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: logs
        };
    }
    async getLogs(file, query) {
        const logs = await this.logService.openLog(file, query.lines);
        if (logs === null) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.NOT_FOUND,
                message: 'logs not found',
                error: 'logs not found',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: logs
        };
    }
    async microserviceLogs(microservice, query) {
        const logs = await this.logService.openLog(microservice, query.lines);
        if (logs === null) {
            throw new common_1.HttpException({
                statusCode: common_1.HttpStatus.NOT_FOUND,
                message: 'logs not found',
                error: 'logs not found',
            }, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: logs
        };
    }
    async logUI(request, log) {
        const user = await User_1.User.getUserByRequest(request);
        const logger = (0, WinstonLogger_1.LoggerFactory)('ui');
        if (typeof log.log === 'object') {
            logger.log(log.level, `${user ? '[' + user.username + ']' : ''} ${JSON.stringify(log.log, null, 2)}`);
        }
        else {
            logger.log(log.level, `${user ? '[' + user.username + ']' : ''} ${log.log}`);
        }
        logger.close();
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: "Log saved successfully"
        };
    }
};
exports.LogController = LogController;
__decorate([
    (0, common_1.Get)('/'),
    openapi.ApiResponse({ status: 200, type: require("../../common/log.dto").LogsResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], LogController.prototype, "list", null);
__decorate([
    (0, common_1.Post)('/'),
    openapi.ApiResponse({ status: 201 }),
    __param(0, (0, common_1.Res)({ passthrough: true })),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, log_dto_1.LogRequest]),
    __metadata("design:returntype", Promise)
], LogController.prototype, "genLogs", null);
__decorate([
    openapi.ApiOperation({ description: "backend logs" }),
    (0, swagger_1.ApiQuery)({ name: 'lines' }),
    (0, common_1.Get)('backend'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").TextResponseDTO }),
    __param(0, (0, common_1.Query)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], LogController.prototype, "backendLogs", null);
__decorate([
    openapi.ApiOperation({ description: "alerting logs" }),
    (0, swagger_1.ApiQuery)({ name: 'lines' }),
    (0, common_1.Get)('alerting'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").TextResponseDTO }),
    __param(0, (0, common_1.Query)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], LogController.prototype, "alertingLogs", null);
__decorate([
    (0, swagger_1.ApiQuery)({ name: 'lines' }),
    (0, common_1.Get)(':file'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").TextResponseDTO }),
    __param(0, (0, common_1.Param)('file')),
    __param(1, (0, common_1.Query)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Promise)
], LogController.prototype, "getLogs", null);
__decorate([
    openapi.ApiOperation({ description: "microservices logs" }),
    (0, swagger_1.ApiQuery)({ name: 'lines' }),
    (0, common_1.Get)('microservice/:microservice'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").TextResponseDTO }),
    __param(0, (0, common_1.Param)('microservice')),
    __param(1, (0, common_1.Query)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object]),
    __metadata("design:returntype", Promise)
], LogController.prototype, "microserviceLogs", null);
__decorate([
    (0, common_1.Post)('/ui'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, log_dto_1.UILog]),
    __metadata("design:returntype", Promise)
], LogController.prototype, "logUI", null);
exports.LogController = LogController = __decorate([
    (0, swagger_1.ApiTags)('Log'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/log'),
    __metadata("design:paramtypes", [log_service_1.LogService])
], LogController);
//# sourceMappingURL=log.controller.js.map