"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MenuController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const menuResponse_dto_1 = require("../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../common/pageResponse.dto");
const Architecture_1 = require("../architecture/Architecture");
const storage_group_service_1 = require("../storage_group/storage_group.service");
const menu_service_1 = require("./menu.service");
const Stor_1 = require("./menu/Stor");
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const menuAccess_interceptor_1 = require("../../interceptor/menuAccess.interceptor");
const User_1 = require("../users/User");
const MenuHandler_1 = require("./MenuHandler");
const CustomTop_1 = require("./menu/CustomTop");
const Overview_1 = require("./menu/Overview");
const Topology_1 = require("./menu/Topology");
const serverGroup_service_1 = require("../server_group/serverGroup.service");
let MenuController = class MenuController {
    constructor(menuService, storageGroupService, serverGroupService) {
        this.menuService = menuService;
        this.storageGroupService = storageGroupService;
        this.serverGroupService = serverGroupService;
    }
    async getPage(request, hw_type, type, item_id) {
        const menuHandler = new MenuHandler_1.MenuHandler(hw_type, request);
        let page = await menuHandler.getPage(type, item_id ? item_id : undefined);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: page,
        };
    }
    async getTab(request, hw_type, type, tab, item_id) {
        const menuHandler = new MenuHandler_1.MenuHandler(hw_type, request);
        let content = await menuHandler.getTab(type, item_id ? item_id : undefined, tab);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: content,
        };
    }
    async getRoot(request, technology_class) {
        let menuItems = [];
        let classes = await Architecture_1.Architecture.getClasses();
        let class_flag = false;
        if (technology_class) {
            classes = [{ class: technology_class, label: technology_class }];
            class_flag = true;
        }
        for (let technology_class of classes) {
            let classChildrenItems = [];
            if (technology_class.class === 'san') {
                let items = await this.menuService.getSanRoot();
                classChildrenItems = items;
            }
            else if (technology_class.class === 'lan') {
                let items = await this.menuService.getLanRoot(request);
                classChildrenItems = items;
            }
            else {
                let items = await this.menuService.getRoot(technology_class.class);
                for (let item of items) {
                    if (await this.menuService.checkItem(item.hw_type)) {
                        let path = item.hw_type.toLowerCase();
                        classChildrenItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, item.hw_type.toLowerCase()));
                    }
                }
            }
            let classFolder = new menuResponse_dto_1.MenuClassFolder(technology_class.label, technology_class.class, false);
            if (classChildrenItems.length > 0) {
                if (class_flag) {
                    return {
                        statusCode: common_1.HttpStatus.OK,
                        data: classChildrenItems,
                    };
                }
                else {
                    classFolder.isActive = true;
                    classFolder.children = classChildrenItems;
                }
            }
            menuItems.push(classFolder);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getHwType(request, hw_type) {
        let menuHandler = new MenuHandler_1.MenuHandler(hw_type, request);
        let menuItems = await menuHandler.getRootFolder();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getStorageGroups(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        let storageGroups = await this.storageGroupService.getStorageGroups(user.user_id);
        storageGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        let menuItems = [];
        for (const storageGroup of storageGroups) {
            menuItems.push({
                title: storageGroup.label,
                url: '/api/menu/v1/folder/sgroup/' + storageGroup.storage_group_id,
                itemId: storageGroup.storage_group_id,
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getGroupStorageGroups(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        let groupFolders = [];
        let storageGroups = await this.storageGroupService.getGroupStorageGroups(user.groups.map(g => g.group_id));
        for (const group of user.groups) {
            groupFolders.push({
                group_id: group.group_id,
                title: group.name,
                children: storageGroups.filter(storageGroup => storageGroup.group_id === group.group_id).map(storageGroup => {
                    return {
                        title: storageGroup.label,
                        url: '/api/menu/v1/folder/sgroup/' + storageGroup.storage_group_id,
                        itemId: storageGroup.storage_group_id,
                    };
                })
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: groupFolders,
        };
    }
    async getStorageGroup(request, storage_group_id) {
        const user = await User_1.User.getUserByRequest(request, true);
        let storageGroup = await this.storageGroupService.getStorageGroup(storage_group_id);
        if (storageGroup.user_id !== user.user_id && !(user.groups.map(g => g.group_id).includes(storageGroup.group_id))) {
            throw new common_1.ForbiddenException();
        }
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Total', 'sgroup/total?item_id=' + storageGroup.storage_group_id, 'sgroup'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Capacity', 'sgroup/capacity?item_id=' + storageGroup.storage_group_id, 'capacity'));
        storageGroup.storage_group_devices.sort((a, b) => a._item.label.localeCompare(b._item.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const device of storageGroup.storage_group_devices) {
            menuItems.push({
                title: device._item.label,
                url: '/api/menu/v1/folder/hw_type/' + device._item.hw_type.toLowerCase() + '/' + device._item.subsystem.toLowerCase() + '?ids=' + device.item_id,
                hwType: device._item.hw_type.toLowerCase(),
                subsystem: device._item.subsystem.toLowerCase(),
                itemId: device.item_id,
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getServerGroups(request) {
        const user = await User_1.User.getUserByRequest(request);
        let serverGroups = await this.serverGroupService.getServerGroups(user.user_id);
        serverGroups.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        let menuItems = [];
        for (const serverGroup of serverGroups) {
            menuItems.push({
                title: serverGroup.label,
                url: '/api/menu/v1/folder/servergroup/' + serverGroup.server_group_id,
                itemId: serverGroup.server_group_id,
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getStorages(request) {
        let devices = await this.menuService.getDevices('storage');
        let menuItems = (0, Stor_1.storageGlobalPages)('storage');
        devices.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (const device of devices) {
            let path = device.hw_type.toLowerCase() + '/device?ids=' + device.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(device.label, path, device.hw_type.toLowerCase(), 'device', device.item_id));
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getSubsystem(request, hw_type, subsystemOrFolderType, itemIds) {
        const menuHandler = new MenuHandler_1.MenuHandler(hw_type, request);
        let menuItems = await menuHandler.getFolder(subsystemOrFolderType, itemIds);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems
        };
    }
    async getPath(request, hw_type, type, item_id, sgroup_id) {
        let menuItems = [];
        if (hw_type === 'sgroup' || sgroup_id) {
            if (!sgroup_id && item_id) {
                sgroup_id = item_id;
                item_id = undefined;
            }
            const getPath = async (sgroup_id, item_id, storageGroups) => {
                let menuItems = [];
                for (const storageGroup of storageGroups) {
                    if (storageGroup.itemId == sgroup_id) {
                        storageGroup.children = (await this.getStorageGroup(request, storageGroup.itemId)).data;
                        if (item_id) {
                            const item = await (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findOneBy({ item_id: item_id });
                            if (item) {
                                const menuHandler = new MenuHandler_1.MenuHandler(item.hw_type, request);
                                const path = await menuHandler.getPath(type, item_id);
                                const deviceId = path.find(item => item.subsystem === 'device')?.itemId;
                                for (const sgItem of storageGroup.children) {
                                    if (sgItem.subsystem === 'device' && sgItem.itemId === deviceId) {
                                        sgItem.children = path && path[0] ? path[0].children : [];
                                    }
                                }
                            }
                        }
                    }
                    menuItems.push(storageGroup);
                }
                return menuItems;
            };
            const storageGroup = await this.storageGroupService.getStorageGroup(sgroup_id);
            if (storageGroup && storageGroup.group_id) {
                const groupStorageGroups = await this.getGroupStorageGroups(request);
                for (const groupStorageGroup of groupStorageGroups.data) {
                    if (groupStorageGroup.group_id === storageGroup.group_id) {
                        groupStorageGroup.children = await getPath(sgroup_id, item_id, groupStorageGroup.children);
                    }
                    menuItems.push(groupStorageGroup);
                }
            }
            else {
                const storageGroups = await this.getStorageGroups(request);
                menuItems = await getPath(sgroup_id, item_id, storageGroups.data);
            }
        }
        else {
            const menuHandler = new MenuHandler_1.MenuHandler(hw_type, request);
            menuItems = await menuHandler.getPath(type, item_id);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getPathSgroup(request, hw_type, type, item_id, sgroup_id) {
        let menuItems = [];
        const menuHandler = new MenuHandler_1.MenuHandler(hw_type, request);
        const path = await menuHandler.getPath(type, item_id);
        const deviceId = path.find(item => item.subsystem === 'device')?.itemId;
        const getPath = async (sgroupId, storageGroups) => {
            let menuItems = [];
            for (const menuItem of storageGroups) {
                if (menuItem.itemId === sgroupId) {
                    menuItem.children = (await this.getStorageGroup(request, menuItem.itemId)).data;
                    for (const sgChild of menuItem.children) {
                        if (sgChild.subsystem === 'device' && sgChild.itemId === deviceId) {
                            sgChild.children = path && path[0] ? path[0].children : [];
                        }
                    }
                }
                menuItems.push(menuItem);
            }
            return menuItems;
        };
        const storageGroup = await this.storageGroupService.getStorageGroup(sgroup_id);
        if (!storageGroup) {
            throw new common_1.NotFoundException();
        }
        if (storageGroup && storageGroup.group_id) {
            const groupStorageGroups = await this.getGroupStorageGroups(request);
            for (const groupStorageGroup of groupStorageGroups.data) {
                if (groupStorageGroup.group_id === storageGroup.group_id) {
                    groupStorageGroup.children = await getPath(sgroup_id, groupStorageGroup.children);
                }
                menuItems.push(groupStorageGroup);
            }
        }
        else {
            const storageGroups = await this.getStorageGroups(request);
            menuItems = await getPath(sgroup_id, storageGroups.data);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: menuItems,
        };
    }
    async getOverviewData(request, req) {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await Overview_1.Overview.getPage(request, req),
        };
    }
    async getCustomTopData(req, request) {
        const customTop = new CustomTop_1.CustomTop(request.hw_type, request.subsystem, request.metrics, request.start, request.end, request.type, request.parentsSubsystem ? request.parentsSubsystem : undefined, req);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                title: request.pageLabel ? request.pageLabel : "TOP",
                content: [
                    await customTop.genTable()
                ]
            },
        };
    }
    async getTopologyData(request, topologyRequest) {
        return {
            statusCode: common_1.HttpStatus.OK,
            data: await Topology_1.Topology.getPage(topologyRequest, request),
        };
    }
};
exports.MenuController = MenuController;
__decorate([
    openapi.ApiOperation({ description: "Returns menu page" }),
    (0, common_1.Get)('v1/page/:hw_type/:type'),
    (0, common_1.UseInterceptors)(menuAccess_interceptor_1.MenuAccessInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").PageResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('type')),
    __param(3, (0, common_1.Query)('item_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getPage", null);
__decorate([
    openapi.ApiOperation({ description: "lazy load tab content" }),
    (0, common_1.Get)('v1/page/:hw_type/:type/:tab'),
    (0, common_1.UseInterceptors)(menuAccess_interceptor_1.MenuAccessInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/pageResponse.dto").TabResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('type')),
    __param(3, (0, common_1.Param)('tab')),
    __param(4, (0, common_1.Query)('item_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, String, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getTab", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu tree of storages per hw_type" }),
    (0, common_1.Get)('v1/folder'),
    (0, common_1.UseInterceptors)(menuAccess_interceptor_1.MenuAccessInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('class')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getRoot", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu tree of hw_type" }),
    (0, common_1.Get)('v1/folder/hw_type/:hw_type'),
    (0, common_1.UseInterceptors)(menuAccess_interceptor_1.MenuAccessInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hw_type')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getHwType", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu tree of storage groups" }),
    (0, common_1.Get)('v1/folder/sgroups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getStorageGroups", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu tree of group storage groups" }),
    (0, common_1.Get)('v1/folder/group_sgroups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/storageGroups.dto").GroupStorageGroupsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getGroupStorageGroups", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu tree of storage group" }),
    (0, common_1.Get)('v1/folder/sgroup/:storage_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('storage_group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getStorageGroup", null);
__decorate([
    (0, common_1.Get)('v1/folder/servergroups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getServerGroups", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu tree of storages" }),
    (0, common_1.Get)('v1/folder/class/storage'),
    (0, common_1.UseInterceptors)(menuAccess_interceptor_1.MenuAccessInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getStorages", null);
__decorate([
    openapi.ApiOperation({ description: "Returns items in folder" }),
    (0, common_1.Get)('v1/folder/hw_type/:hw_type/:subsystemOrFolderType'),
    (0, common_1.UseInterceptors)(menuAccess_interceptor_1.MenuAccessInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('subsystemOrFolderType')),
    __param(3, (0, common_1.Query)('ids')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getSubsystem", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu path to item" }),
    (0, common_1.Get)('v1/getpath/:hw_type/:type'),
    (0, common_1.UseInterceptors)(menuAccess_interceptor_1.MenuAccessInterceptor),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('type')),
    __param(3, (0, common_1.Query)('item_id')),
    __param(4, (0, common_1.Query)('sgroup_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, String, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getPath", null);
__decorate([
    openapi.ApiOperation({ description: "Returns menu path to item" }),
    (0, common_1.Get)('v1/getpath/sgroup/:hw_type/:type'),
    openapi.ApiResponse({ status: 200, type: require("../../common/menuResponse.dto").MenuResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('type')),
    __param(3, (0, common_1.Query)('item_id')),
    __param(4, (0, common_1.Query)('sgroup_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, String, String]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getPathSgroup", null);
__decorate([
    (0, common_1.Post)('v1/overview'),
    openapi.ApiResponse({ status: 201, type: require("../../common/pageResponse.dto").OverviewResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [pageResponse_dto_1.OverviewRequest, Object]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getOverviewData", null);
__decorate([
    (0, common_1.Post)('v1/top'),
    openapi.ApiResponse({ status: 201, type: require("../../common/pageResponse.dto").CustomTopResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, pageResponse_dto_1.CustomTopRequest]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getCustomTopData", null);
__decorate([
    (0, common_1.Post)('v1/topology'),
    openapi.ApiResponse({ status: 201, type: require("../../common/pageResponse.dto").OverviewResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, pageResponse_dto_1.TopologyRequest]),
    __metadata("design:returntype", Promise)
], MenuController.prototype, "getTopologyData", null);
exports.MenuController = MenuController = __decorate([
    (0, swagger_1.ApiTags)('Menu'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/menu'),
    __metadata("design:paramtypes", [menu_service_1.MenuService, storage_group_service_1.StorageGroupService, serverGroup_service_1.ServerGroupService])
], MenuController);
//# sourceMappingURL=menu.controller.js.map