"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UniversalGroupAbstractFolder = exports.StorUniversalGroupAbstractFolder = exports.UniversalGroupStorAbstractPage = exports.UniversalGroupAbstractPage = exports.StorAbstractPage = exports.AbstractPage = exports.StorAbstractFolder = exports.AbstractFolder = void 0;
const dataSource_1 = require("../../../config/dataSource");
const hostcfg_entity_1 = require("../../../entity/hostcfg.entity");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
class AbstractFolder {
    constructor(itemId, request) {
        this.subRootFolders = [];
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        this.itemId = itemId;
        this.request = request;
    }
    setItemId(itemId) {
        this.itemId = itemId;
    }
    setRequest(request) {
        this.request = request;
    }
    async getPath(parentIds) {
        const children = await this.getChildren();
        for (const child of children) {
            if (child._new) {
                const child_id = parentIds.find((v) => v === child.itemId);
                if (child_id || child.itemId === "true") {
                    child.children = await new child._new(child_id)["getPath"](parentIds);
                }
                else if (this.subRootFolders.includes(child._new)) {
                    child.children = await new child._new(child_id)["getPath"](parentIds);
                }
            }
        }
        return children;
    }
    async getChildren(request, useFirst) {
        return null;
    }
}
exports.AbstractFolder = AbstractFolder;
class StorAbstractFolder extends AbstractFolder {
    constructor(hwType, subsystem, itemId, request) {
        super(itemId, request);
        this.hostCfgRepository = (0, dataSource_1.getDataSource)().getRepository(hostcfg_entity_1.HostCfg);
        this.hwType = hwType;
        this.subsystem = subsystem;
    }
    async getPath(parentIds) {
        if (!this.hwType) {
            const item = await this.objectItemsRepository.findOneBy({ item_id: this.itemId });
            this.hwType = item.hw_type;
        }
        const children = await this.getChildren();
        for (const child of children) {
            if (child._new) {
                const child_id = parentIds.find((v) => v === child.itemId);
                if (child_id || child.itemId === "true") {
                    let subsys = child.url.includes("-items") ? child.subsystem + "-items" : child.subsystem;
                    child.children = await new child._new(this.hwType, subsys, child_id)["getPath"](parentIds);
                }
            }
        }
        return children;
    }
}
exports.StorAbstractFolder = StorAbstractFolder;
class AbstractPage {
    constructor(itemId, request) {
        this.itemId = itemId;
        this.request = request;
        this.withItemId = false;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        this.itemPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    }
    ;
    async getPath() {
        const parents = await this.objectItemsRepository.find({
            relations: {
                children: true
            },
            where: {
                children: {
                    child: this.itemId
                }
            }
        });
        const parentIds = parents.map(item => item.item_id);
        if (this.withItemId) {
            parentIds.push(this.itemId);
        }
        const folder = new this.rootFolder(this.hwType, null, this.itemId);
        folder.setItemId(this.itemId);
        folder.setRequest(this.request);
        const path = await folder.getPath(parentIds);
        return path;
    }
}
exports.AbstractPage = AbstractPage;
class StorAbstractPage extends AbstractPage {
    constructor(hwType, type, itemId, request) {
        super(itemId, request);
        this.withItemId = true;
        this.hostCfgRepository = (0, dataSource_1.getDataSource)().getRepository(hostcfg_entity_1.HostCfg);
        this.hwType = hwType ? hwType.toLowerCase() : undefined;
        this.type = type;
        this.itemId = itemId;
        this.request = request;
    }
    ;
    async getPath() {
        const parents = await this.objectItemsRepository.find({
            relations: {
                children: true
            },
            where: {
                children: {
                    child: this.itemId
                }
            }
        });
        const parentIds = parents.map(item => item.item_id);
        if (this.withItemId) {
            parentIds.push(this.itemId);
        }
        const folder = new this.rootFolder(this.hwType, null, this.itemId);
        folder.setItemId(this.itemId);
        folder.setRequest(this.request);
        const path = await folder.getPath(parentIds);
        return path;
    }
}
exports.StorAbstractPage = StorAbstractPage;
class UniversalGroupAbstractPage extends AbstractPage {
    async getPath() {
        const split = this.groupId ? [this.itemId, this.groupId] : this.itemId ? this.itemId.split(":::") : [];
        const parents = await this.objectItemsRepository.find({
            relations: {
                children: true
            },
            where: {
                children: {
                    child: split.length > 1 ? split[0] : this.itemId
                }
            }
        });
        let parentIds = parents.map(item => item.item_id);
        if (split.length > 1) {
            parentIds = parentIds.concat(parentIds.map(id => id + ":::" + split[1]));
        }
        if (this.groupId) {
            parentIds.push(this.groupId);
        }
        if (this.withItemId) {
            parentIds.push(split.length > 1 ? this.itemId + ":::" + split[1] : this.itemId);
        }
        const folder = new this.rootFolder(this.hwType, null, this.itemId);
        folder.setItemId(this.itemId);
        folder.setRequest(this.request);
        const path = await folder.getPath(parentIds, split.length > 1 ? false : true);
        return path;
    }
}
exports.UniversalGroupAbstractPage = UniversalGroupAbstractPage;
class UniversalGroupStorAbstractPage extends StorAbstractPage {
    async getPath() {
        const split = this.groupId ? [this.itemId, this.groupId] : this.itemId ? this.itemId.split(":::") : [];
        const parents = await this.objectItemsRepository.find({
            relations: {
                children: true
            },
            where: {
                children: {
                    child: split.length > 1 ? split[0] : this.itemId
                }
            }
        });
        let parentIds = parents.map(item => item.item_id);
        if (split.length > 1) {
            parentIds = parentIds.concat(parentIds.map(id => id + ":::" + split[1]));
        }
        if (this.groupId) {
            parentIds.push(this.groupId);
        }
        if (this.withItemId) {
            parentIds.push(split.length > 1 ? this.itemId + ":::" + split[1] : this.itemId);
        }
        const folder = new this.rootFolder(this.hwType, null, this.itemId, this.request);
        folder.setItemId(this.itemId);
        folder.setRequest(this.request);
        const path = await folder.getPath(parentIds, split.length > 1 ? false : true);
        return path;
    }
}
exports.UniversalGroupStorAbstractPage = UniversalGroupStorAbstractPage;
class StorUniversalGroupAbstractFolder extends StorAbstractFolder {
    async getPath(parentIds, useFirst) {
        if (!this.hwType) {
            const item = await this.objectItemsRepository.findOneBy({ item_id: this.itemId });
            this.hwType = item.hw_type;
        }
        const children = await this.getChildren(this.request, useFirst);
        for (const child of children) {
            if (child._new) {
                let itId = new URL('https://do.not.care' + child.url).searchParams.get("ids");
                itId = itId ? itId : child.itemId;
                const child_id = parentIds.find((v) => v === itId);
                if (child_id) {
                    let subsys = child.url.includes("-items") ? child.subsystem + "-items" : child.subsystem;
                    child.children = await new child._new(this.hwType, subsys, child_id)["getPath"](parentIds, useFirst);
                    if (child.subsystem === "group" && child.children?.length > 0 && useFirst)
                        break;
                }
            }
        }
        return children;
    }
}
exports.StorUniversalGroupAbstractFolder = StorUniversalGroupAbstractFolder;
class UniversalGroupAbstractFolder extends AbstractFolder {
    async getPath(parentIds, useFirst) {
        if (!this.hwType) {
            const item = await this.objectItemsRepository.findOneBy({ item_id: this.itemId });
            this.hwType = item.hw_type;
        }
        const children = await this.getChildren(this.request, useFirst);
        for (const child of children) {
            if (child._new) {
                let itId = new URL('https://do.not.care' + child.url).searchParams.get("ids");
                itId = itId ? itId : child.itemId;
                const child_id = parentIds.find((v) => v === itId);
                if (child_id) {
                    child.children = await new child._new(child_id, this.request)["getPath"](parentIds, useFirst);
                    if (child.subsystem === "group" && child.children?.length > 0 && useFirst)
                        break;
                }
            }
        }
        return children;
    }
}
exports.UniversalGroupAbstractFolder = UniversalGroupAbstractFolder;
//# sourceMappingURL=AbstractMenu.js.map