"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AzureHeatmapPage = exports.AzureStatusesPage = exports.AzureVmrgrouptotalsPage = exports.AzureVmsubscriptiontotalsPage = exports.AzureVmmgrouptotalsPage = exports.AzureVmtotalsPage = exports.AzureTablePage = exports.AzureQueuePage = exports.AzureFilePage = exports.AzureBlobPage = exports.AzureAccountPage = exports.AbstractAzureAccountPage = exports.AzureAccountfolderFolder = exports.AzureVmPage = exports.AzureMetrics = exports.AzureAccountFolder = exports.AzureVmFolder = exports.AzureRgroupFolder = exports.AzureSubscriptionFolder = exports.AzureMgroupFolder = exports.AzureConfigurationPage = exports.AzureRootFolder = exports.AzureSingleItemPage = exports.AzureSubsystemFolder = exports.AzurePage = exports.AzureFolder = void 0;
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const ConfigurationTable_1 = require("./ConfigurationTable");
const ACLCheck_1 = require("../../acl/ACLCheck");
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const Heatmap_1 = require("./Heatmap");
const Table_1 = require("./Table");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const URL_1 = require("./URL");
const AbstractMenu_1 = require("./AbstractMenu");
const Color_1 = require("../../metrics/Color");
const TabFactory_1 = require("./TabFactory");
class AzureFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'azure';
    }
}
exports.AzureFolder = AzureFolder;
class AzurePage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'azure';
        this.rootFolder = AzureRootFolder;
    }
}
exports.AzurePage = AzurePage;
class AzureSubsystemFolder extends AzureFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.AzureSubsystemFolder = AzureSubsystemFolder;
class AzureSingleItemPage extends AzurePage {
}
exports.AzureSingleItemPage = AzureSingleItemPage;
class AzureRootFolder extends AzureFolder {
    async _get() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration', 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Statuses', this.hwType + '/statuses', 'statuses'));
        menuItems.push(new menuResponse_dto_1.MenuLink('VM Totals', this.hwType + '/vm-totals', 'vm-totals'));
        let managementGroups = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'mgroup'
            },
            relations: {
                children: true
            }
        });
        for (let mgroup of managementGroups) {
            if (mgroup.children.length >= 1) {
                let path = this.hwType + '/mgroup?ids=' + mgroup.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(mgroup.label, path, this.hwType, 'mgroup', mgroup.item_id, AzureMgroupFolder));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(...(await this._get()));
        return menuItems;
    }
    async getChildrenAcl() {
        return this._get();
    }
}
exports.AzureRootFolder = AzureRootFolder;
class AzureConfigurationPage extends AzurePage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        let vms = await this.objectItemsRepository.findBy({ hw_type: 'azure', subsystem: 'vm' });
        vms = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, vms);
        let vmTable = new ConfigurationTable_1.ConfigurationTable(vms.map(vm => vm.item_id), ["location", "ip_adress", "instance_size", "disk_size", "os", "state"], ['mgroup', 'subscription', "rgroup"]);
        vmTable.setItemLabel("VM");
        vmTable.setAcl(this.request);
        tabs.push({
            title: 'Azure VM Configuration',
            content: [await vmTable.generate()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureConfigurationPage = AzureConfigurationPage;
class AzureMgroupFolder extends AzureSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = "mgroup";
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('VM Totals', this.hwType + '/vm-mgroup-totals?item_id=' + this.itemId, 'vm-mgroup-totals', undefined, true));
        const items = await this.objectItemsRepository.find({
            where: {
                subsystem: 'subscription',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (let item of items) {
            let path = this.hwType + '/subscription?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, 'azure', 'subscription', item.item_id, AzureSubscriptionFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AzureMgroupFolder = AzureMgroupFolder;
class AzureSubscriptionFolder extends AzureSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = "subscription";
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('VM Totals', this.hwType + '/vm-subscription-totals?item_id=' + this.itemId, 'vm-subscription-totals', undefined, true));
        const items = await this.objectItemsRepository.find({
            where: {
                subsystem: 'rgroup',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (let item of items) {
            let path = this.hwType + '/rgroup?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, 'azure', 'rgroup', item.item_id, AzureRgroupFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AzureSubscriptionFolder = AzureSubscriptionFolder;
class AzureRgroupFolder extends AzureSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = "rgroup";
    }
    async getChildren() {
        let menuItems = [];
        const vms = await this.objectItemsRepository.find({
            where: {
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        if (vms.length > 0) {
            menuItems.push(new menuResponse_dto_1.MenuLink('VM Totals', this.hwType + '/vm-rgroup-totals?item_id=' + this.itemId, 'vm-rgroup-totals', undefined, true));
            let path = this.hwType + '/vm?ids=' + this.itemId;
            menuItems.push(new menuResponse_dto_1.MenuFolder("VM", path, this.hwType, this.subsystem, this.itemId, AzureVmFolder));
        }
        const accounts = await this.objectItemsRepository.find({
            where: {
                subsystem: 'account',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        if (accounts.length > 0) {
            let path = this.hwType + '/account?ids=' + this.itemId;
            menuItems.push(new menuResponse_dto_1.MenuFolder("Storage Accounts", path, this.hwType, this.subsystem, this.itemId, AzureAccountFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AzureRgroupFolder = AzureRgroupFolder;
class AzureVmFolder extends AzureSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = "rgroup";
    }
    async getChildren() {
        let menuItems = [];
        const items = await this.objectItemsRepository.find({
            where: {
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (let item of items) {
            let path = this.hwType + '/vm?item_id=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, 'vm', item.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AzureVmFolder = AzureVmFolder;
class AzureAccountFolder extends AzureSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = "account";
    }
    async getChildren() {
        let menuItems = [];
        const items = await this.objectItemsRepository.find({
            where: {
                subsystem: 'account',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (let item of items) {
            let path = this.hwType + '/account-folder?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, "account", item.item_id, AzureAccountfolderFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.AzureAccountFolder = AzureAccountFolder;
class AzureMetrics {
}
exports.AzureMetrics = AzureMetrics;
AzureMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.AZURE, subsystem, metric);
        if (definition) {
            Object.assign(this, definition);
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
    getAggrGraph(type, stacked, order, aggregate) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.aggregate = aggregate ? aggregate : pageResponse_constants_1.METRIC_AGGREGATE.items;
        return this;
    }
};
class AzureVmPage extends AzureSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.AZURE, "vm", [this.itemId]);
        tabFactory.addTab("CPU", { metric: "cpu_util", default: TabFactory_1.MPDefault.LINE });
        tabFactory.addTab("Disk", [
            { metric: "read_bytes", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "write_bytes", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        tabFactory.addTab("LAN", [
            { metric: "received_bytes", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "sent_bytes", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        tabFactory.addTab("IO", [
            { metric: "read_ops", default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "write_ops", default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        tabFactory.addTab("Memory", [
            { metric: "usedMemory", order: 0, color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.READ_LINE },
            { metric: "freeMemory", order: 1, color: Color_1.Color.pallete.green, default: TabFactory_1.MPDefault.WRITE_LINE }
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabFactory.getTabs());
        return this.page;
    }
}
exports.AzureVmPage = AzureVmPage;
class AzureAccountfolderFolder extends AzureSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let path = [
            this.hwType + '/account?item_id=' + this.itemId,
            this.hwType + '/blob?item_id=' + this.itemId,
            this.hwType + '/file?item_id=' + this.itemId,
            this.hwType + '/queue?item_id=' + this.itemId,
            this.hwType + '/table?item_id=' + this.itemId,
        ];
        menuItems.push(new menuResponse_dto_1.MenuLink("Account", path[0], 'account', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink("Blob", path[1], 'blob', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink("File", path[2], 'file', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink("Queue", path[3], 'queue', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink("Table", path[4], 'table', this.itemId));
        this.children = menuItems;
        return menuItems;
    }
}
exports.AzureAccountfolderFolder = AzureAccountfolderFolder;
class AbstractAzureAccountPage extends AzureSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'account';
        this.subPart = 'account';
        this.withItemId = true;
    }
    async getTabs() {
        this.item = await this.objectItemsRepository.findOneBy({
            item_id: this.itemId,
        });
        this.tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.AZURE, "account", [this.itemId]);
        let tabs = [];
        tabs.push(this.tabFactory.createTab("Ingress", { metric: this.subPart + "_ingress", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab("Egress", { metric: this.subPart + "_egress", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab("Transactions", { metric: this.subPart + "_transactions", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab("Success Server Latency", { metric: this.subPart + "_suc_server_lat", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab("Success E2E Latency", { metric: this.subPart + "_suc_e2e_lat", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab("Availability %", { metric: this.subPart + "_availibility", default: TabFactory_1.MPDefault.LINE }));
        return tabs;
    }
}
exports.AbstractAzureAccountPage = AbstractAzureAccountPage;
class AzureAccountPage extends AbstractAzureAccountPage {
    constructor() {
        super(...arguments);
        this.subPart = 'account';
    }
    async getPage() {
        let tabs = await super.getTabs();
        tabs.push(this.tabFactory.createTab('Used Capacity', { metric: "used_capacity", default: TabFactory_1.MPDefault.AREA }));
        let title = 'Storage Account ' + (this.item !== null ? this.item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureAccountPage = AzureAccountPage;
class AzureBlobPage extends AbstractAzureAccountPage {
    constructor() {
        super(...arguments);
        this.subPart = 'blob';
    }
    async getPage() {
        let tabs = await super.getTabs();
        tabs.push(this.tabFactory.createTab('Blob Capacity', { metric: "blob_capacity", default: TabFactory_1.MPDefault.AREA }), this.tabFactory.createTab('Blob Count', { metric: "blob_count", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab('Blob Container Count', { metric: "container_count", default: TabFactory_1.MPDefault.LINE }));
        let title = 'Storage Account ' + (this.item !== null ? this.item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureBlobPage = AzureBlobPage;
class AzureFilePage extends AbstractAzureAccountPage {
    constructor() {
        super(...arguments);
        this.subPart = 'file';
    }
    async getPage() {
        let tabs = await super.getTabs();
        tabs.push(this.tabFactory.createTab('File Capacity', { metric: "file_capacity", default: TabFactory_1.MPDefault.AREA }), this.tabFactory.createTab('File Count', { metric: "file_count", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab('File Share Capacity Quota', { metric: "file_share_capacity_quota", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab('File Share Count', { metric: "file_share_count", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab('File Share Snapshot Count', { metric: "file_share_snapshot_count", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab('File Share Snapshot Size', { metric: "file_share_snapshot_size", default: TabFactory_1.MPDefault.LINE }));
        let title = 'Storage Account ' + (this.item !== null ? this.item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureFilePage = AzureFilePage;
class AzureQueuePage extends AbstractAzureAccountPage {
    constructor() {
        super(...arguments);
        this.subPart = 'queue';
    }
    async getPage() {
        let tabs = await super.getTabs();
        tabs.push(this.tabFactory.createTab('Queue Capacity', { metric: "queue_capacity", default: TabFactory_1.MPDefault.AREA }), this.tabFactory.createTab('Queue Count', { metric: "queue_count", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab('Queue Message Count', { metric: "queue_message_count", default: TabFactory_1.MPDefault.LINE }));
        let title = 'Storage Account ' + (this.item !== null ? this.item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureQueuePage = AzureQueuePage;
class AzureTablePage extends AbstractAzureAccountPage {
    constructor() {
        super(...arguments);
        this.subPart = 'table';
    }
    async getPage() {
        let tabs = await super.getTabs();
        tabs.push(this.tabFactory.createTab('Table Capacity', { metric: "table_capacity", default: TabFactory_1.MPDefault.AREA }), this.tabFactory.createTab('Table Count', { metric: "table_count", default: TabFactory_1.MPDefault.LINE }), this.tabFactory.createTab('Table Entity Count', { metric: "table_entity_count", default: TabFactory_1.MPDefault.LINE }));
        let title = 'Storage Account ' + (this.item !== null ? this.item.label : this.itemId);
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureTablePage = AzureTablePage;
const getVMTotalsTabs = (vms) => {
    const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.CLOUD, hwTypeKey_constants_1.HwTypeKey.AZURE, "vm", vms);
    tabFactory.addTab("CPU", { metric: "cpu_util", default: TabFactory_1.MPDefault.LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }, undefined, undefined, true);
    tabFactory.addTab("Disk", [
        { metric: "read_bytes", default: TabFactory_1.MPDefault.READ_LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
        { metric: "write_bytes", default: TabFactory_1.MPDefault.WRITE_LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
    ], undefined, undefined, true);
    tabFactory.addTab("LAN", [
        { metric: "received_bytes", default: TabFactory_1.MPDefault.READ_LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
        { metric: "sent_bytes", default: TabFactory_1.MPDefault.WRITE_LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
    ], undefined, undefined, true);
    tabFactory.addTab("IO", [
        { metric: "read_ops", default: TabFactory_1.MPDefault.READ_LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
        { metric: "write_ops", default: TabFactory_1.MPDefault.WRITE_LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
    ], undefined, undefined, true);
    return tabFactory.getTabs();
};
class AzureVmtotalsPage extends AzureSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'mgroup';
    }
    async getPage() {
        let vms = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
            },
            relations: {
                parents: true
            }
        })).map(item => item.item_id);
        let title = 'VM';
        this.page = new pageResponse_dto_1.Page(title, getVMTotalsTabs(vms));
        return this.page;
    }
}
exports.AzureVmtotalsPage = AzureVmtotalsPage;
class AzureVmmgrouptotalsPage extends AzureSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'mgroup';
        this.withItemId = true;
    }
    async getPage() {
        let vms = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }
        })).map(item => item.item_id);
        let title = 'VM';
        this.page = new pageResponse_dto_1.Page(title, getVMTotalsTabs(vms));
        return this.page;
    }
}
exports.AzureVmmgrouptotalsPage = AzureVmmgrouptotalsPage;
class AzureVmsubscriptiontotalsPage extends AzureSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'subscribe';
        this.withItemId = true;
    }
    async getPage() {
        let vms = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }
        })).map(item => item.item_id);
        let title = 'VM';
        this.page = new pageResponse_dto_1.Page(title, getVMTotalsTabs(vms));
        return this.page;
    }
}
exports.AzureVmsubscriptiontotalsPage = AzureVmsubscriptiontotalsPage;
class AzureVmrgrouptotalsPage extends AzureSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'rgroup';
        this.withItemId = true;
    }
    async getPage() {
        let vms = (await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }
        })).map(item => item.item_id);
        let title = 'VM';
        this.page = new pageResponse_dto_1.Page(title, getVMTotalsTabs(vms));
        return this.page;
    }
}
exports.AzureVmrgrouptotalsPage = AzureVmrgrouptotalsPage;
class AzureStatusesPage extends AzurePage {
    constructor() {
        super(...arguments);
        this.title = 'Statuses';
    }
    async getPage() {
        let title = 'Statuses';
        let tabs = [];
        let vmStatuses = new Table_1.Table()
            .addHeader({ value: 'VM' })
            .addHeader({ value: "Level" })
            .addHeader({ value: 'Code' })
            .addHeader({ value: 'Status' });
        const vms = await this.objectItemsRepository.find({
            where: {
                subsystem: "vm",
                hw_type: "azure"
            },
            relations: {
                properties: true
            }
        });
        for (const vm of vms) {
            const stateProperties = vm.properties.filter((property => property.property_name.includes("State")));
            if (stateProperties.find(property => property.property_name === "powerStateCode")) {
                vmStatuses.addRow([
                    { order: 0, value: vm.label, url: URL_1.URL.generate('azure', 'vm', vm.item_id) },
                    { order: 1, value: stateProperties.find(property => property.property_name === "powerStateLevel").property_value },
                    { order: 2, value: stateProperties.find(property => property.property_name === "powerStateCode").property_value },
                    { order: 3, value: stateProperties.find(property => property.property_name === "powerStateMessage").property_value },
                ]);
            }
            if (stateProperties.find(property => property.property_name === "provisioningStateCode")) {
                vmStatuses.addRow([
                    { order: 0, value: vm.label, url: URL_1.URL.generate('azure', 'vm', vm.item_id) },
                    { order: 1, value: stateProperties.find(property => property.property_name === "provisioningStateLevel").property_value },
                    { order: 2, value: stateProperties.find(property => property.property_name === "provisioningStateCode").property_value },
                    { order: 3, value: stateProperties.find(property => property.property_name === "provisioningStateMessage").property_value },
                ]);
            }
        }
        vmStatuses.sortByIndex(0);
        tabs.push({
            title: 'VM Statuses',
            content: [vmStatuses.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureStatusesPage = AzureStatusesPage;
class AzureHeatmapPage extends AzurePage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'VM',
            content: Object.values(await new Heatmap_1.Heatmap('azure', 'vm', true)
                .computedHeatmapBox(class_constants_1.ClassKey.CLOUD, this.request)),
        });
        let computeTable = new Table_1.Table()
            .addHeader({ value: "Managment Group" })
            .addHeader({ value: 'Subscription' })
            .addHeader({ value: 'Resource Group' })
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: 'azure',
                subsystem: 'vm',
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                },
                hostcfg_relations: {
                    _hostcfg: true
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, vms.map(item => item.item_id));
            vms = vms.filter(item => ids.includes(item.item_id));
        }
        for (const vm of vms) {
            const managementGroup = vm.hostcfg_relations[0]._hostcfg;
            const subscription = vm.parents.find(parent => parent._parent.subsystem === 'subscription');
            const resourceGroup = vm.parents.find(parent => parent._parent.subsystem === 'rgroup');
            const cpu = vm.heatmaps.find(heatmap => heatmap.metric === 'cpu_util');
            computeTable.addRow([
                { order: 0, value: managementGroup.label },
                { order: 1, value: subscription._parent.label },
                { order: 2, value: resourceGroup._parent.label },
                { order: 3, value: vm.label, url: URL_1.URL.generate('azure', 'vm', vm.item_id) },
                { order: 4, value: cpu ? cpu.value : null },
            ]);
        }
        computeTable.sortByIndex(4);
        tabs.push({
            title: 'VM table',
            content: [computeTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.AzureHeatmapPage = AzureHeatmapPage;
//# sourceMappingURL=Azure.js.map