import { MenuItem, MenuLink } from "@xormon/common/menuResponse.dto";
import { ChartType, FourGraphBox, Metric, MetricAggregate, Page, Tab } from '@xormon/common/pageResponse.dto';
import { ItemRelation } from "@xormon/entity/itemRelation.entity";
import { ObjectItem } from "@xormon/entity/objectItem.entity";
import { StorHealthstatusPage } from "./Stor";
import { ItemProperty } from "@xormon/entity/itemProperty.entity";
import { Request } from "express";
declare class EcsFolder {
    hwType: string;
    children?: Array<MenuItem>;
    itemId: string;
    objectItemsRepository: import("typeorm").Repository<ObjectItem>;
    itemRelationsRepository: import("typeorm").Repository<ItemRelation>;
    constructor(itemId?: string);
    protected makeMenuLinks(items: {
        key: string;
        label: string;
    }[], prefix?: string): MenuLink[];
}
export declare class EcsVdcFolder extends EcsFolder {
    getChildren(): Promise<MenuItem[]>;
    getPath(): Promise<void>;
}
export declare class EcsVdcitemFolder extends EcsFolder {
    getChildren(): Promise<MenuItem[]>;
}
export declare class EcsNodeFolder extends EcsFolder {
    getChildren(): Promise<MenuItem[]>;
}
export declare class EcsSubsystemItemsFolder extends EcsFolder {
    subsystem: string;
    constructor(itemId: string, subsystem: string);
    getChildren(): Promise<any[]>;
}
export declare class EcsNodeitemsFolder extends EcsSubsystemItemsFolder {
    constructor(itemId: string);
}
export declare class EcsPage {
    itemId: string;
    request: Request;
    hwType: string;
    page: Page;
    objectItemsRepository: import("typeorm").Repository<ObjectItem>;
    itemRelationsRepository: import("typeorm").Repository<ItemRelation>;
    constructor(itemId: string, request: Request);
    getPath(): Promise<MenuItem[]>;
}
declare abstract class EcsSingleItemPage extends EcsPage {
    subsystem: string;
    constructor(itemId: string, request: Request, subsystem?: string);
    getPath(subsystem?: string, items?: boolean): Promise<MenuItem[]>;
}
declare class EcsMetrics extends Metric {
    constructor(subsystem: string, metric: string);
    getGraph(type: ChartType, stacked?: boolean, order?: number, color?: string, aggregate?: MetricAggregate): this;
}
declare abstract class EcsVdcItemPage extends EcsSingleItemPage {
    getIds(): Promise<string[]>;
    getPath(subsystem?: string, items?: boolean): Promise<MenuItem[]>;
    protected getMetricByIds({ uuids, aggregated }: Pick<FourGraphBox, 'uuids' | 'aggregated'>): (title: string, metric: string, type: ChartType, stacked?: boolean, order?: number, color?: string, aggregate?: MetricAggregate) => Tab;
    protected getMetricsByIds({ uuids }: Pick<FourGraphBox, 'uuids'>): (title: string, data: [metric: string, type: ChartType, stacked?: boolean, order?: number, color?: string, aggregate?: MetricAggregate][]) => Tab;
}
declare class EcsTabs extends EcsVdcItemPage {
    pageTitle: string;
    singleMetricGraphs?: Parameters<ReturnType<typeof this.getMetricByIds>>[];
    multipleMetricGraphs?: [tabPosition: number, ...data: Parameters<ReturnType<typeof this.getMetricsByIds>>][];
    constructor(itemId: string, request: Request, subsystem?: string);
    getPage(): Promise<Page>;
}
declare class EcsVdcTabs extends EcsTabs {
    constructor(itemId: string, request: Request);
}
declare class EcsNodeTabs extends EcsTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdcdataPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdctpsPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdcdiskPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdccpuPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdcmemoryPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdclatencyPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdcnetworkPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsVdccapacityPage extends EcsVdcTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodedataPage extends EcsNodeTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodediskPage extends EcsNodeTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodetpsPage extends EcsNodeTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodecpuPage extends EcsNodeTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodememoryPage extends EcsNodeTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodelatencyPage extends EcsNodeTabs {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodenetworkPage extends EcsNodeTabs {
    constructor(itemId: string, request: Request);
}
declare class SubsystemItemPage extends EcsVdcItemPage {
    singleMetricGraphs: Parameters<ReturnType<typeof this.getMetricByIds>>[];
    multipleMetricGraphs?: [tabPosition: number, ...data: Parameters<ReturnType<typeof this.getMetricsByIds>>][];
    getPage(): Promise<Page>;
    getPath(): Promise<MenuItem[]>;
}
export declare class EcsVdcPage extends SubsystemItemPage {
    constructor(itemId: string, request: Request);
}
export declare class EcsNodePage extends SubsystemItemPage {
    constructor(itemId: string, request: Request);
}
export declare class EcsConfigurationPage extends EcsPage {
    itemPropertiesRepository: import("typeorm").Repository<ItemProperty>;
    getPage(): Promise<Page>;
}
export declare class EcsCapacityPage extends EcsPage {
    getPage(): Promise<Page>;
}
export declare class EcsCapacityvdcPage extends EcsSingleItemPage {
    getPage(): Promise<Page>;
}
export declare class EcsHealthstatusPage extends StorHealthstatusPage {
    getPage(): Promise<Page>;
}
export declare class EcsConfigurationvdcPage extends EcsSingleItemPage {
    itemPropertiesRepository: import("typeorm").Repository<ItemProperty>;
    getPage(): Promise<Page>;
}
export declare class EcsHealthstatusvdcPage extends EcsSingleItemPage {
    itemPropertiesRepository: import("typeorm").Repository<ItemProperty>;
    getPage(): Promise<Page>;
}
export {};
