import { HeatmapBox, HeatmapContent, Page, TableBox } from '@xormon/common/pageResponse.dto';
import { ObjectItem } from '@xormon/entity/objectItem.entity';
import { MetricSimpleItem } from '@xormon/service/metrics/Metric';
import { Request } from 'express';
import { HeatmapTierInterval } from '@xormon/common/elements/heatmapTier.elements';
import { ClassKey } from '@xormon/common/constants/class.constants';
export declare class HeatmapMetric {
    label: string;
    metric?: string;
    metricUsed?: string;
    metricTotal?: string;
    metricFree?: string;
}
export declare class CrossHeatmap {
    private request;
    private objectItemsRepository;
    private lastUpdateAt;
    constructor(request: Request);
    gen(): Promise<Page>;
}
export declare class HeatmapSan {
    request?: Request;
    itemIds?: string[];
    private objectItemsRepository;
    private generatedBox;
    private lastUpdateAt;
    constructor(request?: Request, itemIds?: string[]);
    table(metric?: 'data_in_util' | 'data_out_util'): Promise<TableBox>;
    gen(): Promise<HeatmapContent>;
}
export declare class HeatmapLan {
    request?: Request;
    itemIds?: string[];
    private objectItemsRepository;
    private generatedBox;
    private lastUpdateAt;
    constructor(request?: Request, itemIds?: string[]);
    table(metric?: 'data_in_util' | 'data_out_util'): Promise<TableBox>;
    gen(): Promise<HeatmapContent>;
}
export declare class HeatmapStorageCapacity {
    private request?;
    private objectItemsRepository;
    private userId;
    private userGroupIds;
    private lastUpdateAt;
    constructor(request?: Request);
    gen(deviceId?: string): Promise<HeatmapContent>;
}
export declare class HeatmapStorage {
    request?: Request;
    itemIds?: string[];
    private heatmapRepository;
    private heatmapTierRepository;
    private heatmapTierItemRepository;
    private itemRelationsRepository;
    private objectItemsRepository;
    private volumes;
    private pools;
    private volumeData;
    private poolData;
    private deviceId;
    private lastUpdateAt;
    constructor(request?: Request, itemIds?: string[]);
    loadItems(subsystem: string, parentId?: string): Promise<void>;
    getColor(intervals: HeatmapTierInterval[], value: number): string;
    tableBox(subsystem: string): Promise<TableBox>;
    heatmapBox(subsystem: string, deviceId?: string): Promise<HeatmapContent>;
    heatmapStorageBox(subsystem: string): Promise<HeatmapContent>;
    private getTierColor;
}
export declare class Heatmap {
    private hw_type;
    private subsystem;
    private table;
    private itemIds;
    private metrics;
    private box;
    private objectItemsRepository;
    private parentSubsystem;
    private itemsWithParents;
    private lastUpdateAt;
    constructor(hw_type: string, subsystem: string, table?: boolean, itemIds?: string[]);
    addMetric(metric: HeatmapMetric): this;
    generateUrl(it: ObjectItem, request: Request, tabLabel?: string): Promise<string>;
    computedHeatmapBox(_class: ClassKey, request?: Request): Promise<HeatmapContent>;
    addParent(subsystem: string): void;
    genTable(): TableBox;
    virtHeatmapBox(request?: Request): Promise<HeatmapContent>;
    heatmapBox(): Promise<HeatmapBox>;
    getData(isProxy?: boolean): Promise<any[]>;
    getList(): Promise<{}>;
    getMetricByUsedAndTotal(metricUsed: string, metricTotal: string, isProxy?: boolean): Promise<any[]>;
    getMetricByFreeAndTotal(metricFree: string, metricTotal: string, isProxy?: boolean): Promise<any[]>;
    getMetricByUsedAndTotalSimpleItem(metricUsed: string, metricTotal: string): Promise<MetricSimpleItem>;
    getMetric(metric: string, isProxy?: boolean): Promise<any[]>;
    getMetricSimpleItem(metric: string): Promise<MetricSimpleItem>;
}
