import { MenuItem, Newable } from '@xormon/common/menuResponse.dto';
import { ChartType, Page, MetricAggregate } from '@xormon/common/pageResponse.dto';
import { ItemProperty } from '@xormon/entity/itemProperty.entity';
import { ObjectItem } from '@xormon/entity/objectItem.entity';
import { AgentMapping } from '@xormon/entity/agentMapping.entity';
import { Request } from 'express';
import { IMetric, Item } from '@xormon/common/architecture.dto';
import { ISubsystem } from '@xormon/service/architecture/Architecture';
import { AbstractFolder, AbstractPage } from './AbstractMenu';
import { ServerGroup } from '@xormon/entity/serverGroup.entity';
import { ServerGroupItems } from '@xormon/entity/serverGroupItems.entity';
export declare const getLinuxMapping: () => Promise<AgentMapping[]>;
export declare const getLinuxMappedIds: (linuxId?: boolean) => Promise<string[]>;
export declare const getLinuxExcludedIds: () => Promise<string[]>;
export declare const getLinuxCustomGroupMetrics: () => {
    [subsystem: string]: Partial<IMetric>[];
};
export declare const getLinuxCustomGroupSubsystems: () => ISubsystem[];
export declare const getLinuxMappedChildren: (itemIds: string[], subsystem: string) => Promise<Item[]>;
export declare const getLinuxServers: (itemIds: string[]) => Promise<Item[]>;
export declare const getSelfMonitoringAgent: () => Promise<ObjectItem>;
export declare class LinuxFolder extends AbstractFolder {
    hwType: string;
    children?: Array<MenuItem>;
    agentMappingRepository: import("typeorm").Repository<AgentMapping>;
}
export declare class LinuxSubsystemFolder extends LinuxFolder {
    subsystem: string;
    itemId: string;
    constructor(itemId?: string);
}
export declare class LinuxRootFolder extends LinuxFolder {
    serverGroupsRepository: import("typeorm").Repository<ServerGroup>;
    serverGroupItemsRepository: import("typeorm").Repository<ServerGroupItems>;
    getChildren(request?: Request): Promise<any[]>;
    getChildrenAcl(): Promise<any[]>;
}
export declare class LinuxPage extends AbstractPage {
    hwType: string;
    rootFolder: Newable<AbstractFolder>;
    agentMappingRepository: import("typeorm").Repository<AgentMapping>;
    getPath(): Promise<MenuItem[]>;
}
export declare abstract class LinuxSingleItemPage extends LinuxPage {
    subsystem: string;
    itemId: string;
    getPath(): Promise<MenuItem[]>;
}
export declare class LinuxStatusPage extends LinuxPage {
    getPage(): Promise<Page>;
}
export declare class LinuxTotalsPage extends LinuxPage {
    getPage(): Promise<Page>;
}
export declare class LinuxGroupconfigurationPage extends LinuxPage {
    getPage(): Promise<Page>;
}
export declare class LinuxHeatmapPage extends LinuxPage {
    private lgItemsRepository;
    getPage(): Promise<Page>;
}
export declare class LinuxTopPage extends LinuxPage {
    title: string;
    withItemId: boolean;
    getPage(): Promise<Page>;
}
export declare class LinuxServerPage extends LinuxSingleItemPage {
    itemId: string;
    request: Request;
    subsystem: string;
    serverGroupItemsRepository: import("typeorm").Repository<ServerGroupItems>;
    private parentId;
    constructor(itemId: string, request: Request);
    getPage(): Promise<Page>;
    getPath(): Promise<MenuItem[]>;
}
export declare class LinuxMetrics {
    static Metric: {
        new (subsystem: string, metric: string): {
            getGraph(type: ChartType, stacked?: boolean, order?: number, color?: string): any;
            getAggrGraph(type: ChartType, stacked?: boolean, order?: number, aggregate?: MetricAggregate): any;
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            color?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
}
export declare class LinuxOldMetrics {
    static Metric: {
        new (): {
            subsystem: string;
            init(): any;
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            color?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static CpuSysMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static CpuUsrMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static CpuWaitMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static CpuStolenMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static CpuIdleMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static SendBytesMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static ReceivedBytesMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static MemoryTotalMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static Memory2Metric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static MemoryFreeMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static Memory4Metric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static MemoryUsedMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static MemoryFSCacheMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static MemoryWTSizeMetric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static Paging1Metric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static Paging2Metric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static Paging3Metric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
    static Paging4Metric: {
        new (): {
            type: ChartType;
            stacked: boolean;
            metric: string;
            order: number;
            color: string;
            subsystem: string;
            init(): any;
            aggregate?: MetricAggregate;
            unit: import("../../../common/subsystemMetricsResponse.dto").Unit;
            unitLabel?: string;
            shortcut: string;
            label?: string;
            defaultPrefix?: import("@xormon/common/pageResponse.dto").UnitPrefix;
            defaultPrecision?: number;
            minY?: number;
            legendTop?: boolean;
            legendOnly?: boolean;
        };
    };
}
export declare class LinuxgroupsFolder {
    children?: Array<MenuItem>;
    objectItemsRepository: import("typeorm").Repository<ObjectItem>;
    serverGroupRepository: import("typeorm").Repository<ServerGroup>;
}
export declare class LinuxgroupsSubsystemFolder extends LinuxgroupsFolder {
    subsystem: string;
    itemId: string;
    constructor(itemId?: string);
}
export declare class LinuxgroupsPage {
    itemId: string;
    request: Request;
    page: Page;
    constructor(itemId: string, request: Request);
    objectItemsRepository: import("typeorm").Repository<ObjectItem>;
    serverGroupRepository: import("typeorm").Repository<ServerGroup>;
    itemPropertiesRepository: import("typeorm").Repository<ItemProperty>;
    getPath(): Promise<MenuItem[]>;
}
export declare class LinuxGroupFolder extends LinuxgroupsSubsystemFolder {
    subsystem: string;
    getChildren(useFirst?: boolean): Promise<any[]>;
}
export declare class LinuxServergrouptotalsPage extends LinuxgroupsPage {
    getPage(): Promise<Page>;
    getPath(): Promise<MenuItem[]>;
}
export declare class LinuxServergroupPage extends LinuxgroupsPage {
    itemId: string;
    request: Request;
    subsystem: string;
    private parentId;
    constructor(itemId: string, request: Request);
    getPath(): Promise<MenuItem[]>;
    getPage(): Promise<Page>;
}
