"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLazyContentURL = exports.DefaultPage = exports.DefaultFolder = void 0;
const common_1 = require("@nestjs/common");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
class DefaultFolder {
    constructor(itemId, request) {
        this.itemId = itemId;
        this.request = request;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        this.itemPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    }
    async getChildren() {
        return [];
    }
    async getChildrenAcl() {
        return this.getChildren();
    }
    async getPath(parentIds) {
        const children = await this.getChildren();
        for (const child of children) {
            const child_id = parentIds.find((v) => v === child.itemId);
            if (child_id || child.itemId === "true") {
                if (child._new) {
                    child.children = await new child._new(child_id)["getPath"](parentIds);
                }
            }
        }
        return children;
    }
}
exports.DefaultFolder = DefaultFolder;
class DefaultPage {
    constructor(itemId, request) {
        this.itemId = itemId;
        this.request = request;
        this.withItemId = false;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
        this.itemPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    }
    async getPage() {
        return new pageResponse_dto_1.Page("Undefined page", []);
    }
    async getPath() {
        const parentIds = (await this.itemRelationsRepository.findBy({
            child: this.itemId
        })).map(r => r.parent);
        if (this.withItemId) {
            parentIds.push(this.itemId);
        }
        const path = await new this.rootFolder().getPath(parentIds);
        return path;
    }
    async getTab(tab) {
        throw new common_1.NotFoundException;
    }
}
exports.DefaultPage = DefaultPage;
const createLazyContentURL = (hwType, pageType, tab, itemId) => {
    return `/api/menu/v1/page/${hwType}/${pageType}/${tab}${itemId ? "?item_id=" + itemId : ""}`;
};
exports.createLazyContentURL = createLazyContentURL;
//# sourceMappingURL=MenuDefinitions.js.map