"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OraclevmMetrics = exports.OraclevmVmPage = exports.OraclevmServerPage = exports.OraclevmServerpooltotalsPage = exports.OraclevmManagertotalsPage = exports.OraclevmVmFolder = exports.OraclevmServerFolder = exports.OraclevmServerpoolFolder = exports.OraclevmManagerFolder = exports.OraclevmRootFolder = exports.OraclevmConfigurationPage = exports.OraclevmTopPage = exports.OraclevmHeatmapPage = exports.OraclevmSingleItemPage = exports.OraclevmPage = exports.OraclevmSubsystemFolder = exports.OraclevmFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const ACLCheck_1 = require("../../acl/ACLCheck");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const Top_1 = require("./Top");
const URL_1 = require("./URL");
const Heatmap_1 = require("./Heatmap");
const Table_1 = require("./Table");
const ConfigurationTable_1 = require("./ConfigurationTable");
const Prediction_1 = require("../../metrics/Prediction");
const AbstractMenu_1 = require("./AbstractMenu");
const cpu_peak_help = {
    url: "https://xormon.com/max_vrs_average.php",
    text: "How it works?",
};
class OraclevmFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'oraclevm';
    }
}
exports.OraclevmFolder = OraclevmFolder;
class OraclevmSubsystemFolder extends OraclevmFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId.toString();
        }
    }
}
exports.OraclevmSubsystemFolder = OraclevmSubsystemFolder;
class OraclevmPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'oraclevm';
        this.rootFolder = OraclevmRootFolder;
    }
}
exports.OraclevmPage = OraclevmPage;
class OraclevmSingleItemPage extends OraclevmPage {
}
exports.OraclevmSingleItemPage = OraclevmSingleItemPage;
class OraclevmHeatmapPage extends OraclevmPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'VM',
            content: Object.values(await new Heatmap_1.Heatmap('oraclevm', 'vm', true)
                .virtHeatmapBox(this.request)),
        });
        let vmTable = new Table_1.Table()
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'Manager' })
            .addHeader({ value: 'Server Pool' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: 'oraclevm',
                subsystem: 'vm'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        vms = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, vms);
        for (const vm of vms) {
            const serverPool = vm.parents.find(parent => parent._parent.subsystem === 'server_pool');
            const manager = vm.parents.find(parent => parent._parent.subsystem === 'manager');
            const cpu = vm.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            vmTable.addRow([
                { order: 0, value: vm.label, url: URL_1.URL.generate('oraclevm', 'vm', vm.item_id) },
                { order: 1, value: manager ? manager._parent.label : 'undefined' },
                { order: 2, value: serverPool ? serverPool._parent.label : 'undefined' },
                { order: 3, value: cpu ? cpu.value : null },
            ]);
        }
        vmTable.sortByIndex(3);
        tabs.push({
            title: 'VM table',
            content: [vmTable.generateBox()],
        });
        tabs.push({
            title: 'Server',
            content: Object.values(await new Heatmap_1.Heatmap('oraclevm', 'server', true)
                .virtHeatmapBox(this.request)),
        });
        let serverTable = new Table_1.Table()
            .addHeader({ value: 'Server' })
            .addHeader({ value: 'Manager' })
            .addHeader({ value: 'Server Pool' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: 'oraclevm',
                subsystem: 'server'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        servers = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, servers);
        for (const server of servers) {
            const serverPool = server.parents.find(parent => parent._parent.subsystem === 'server_pool');
            const manager = server.parents.find(parent => parent._parent.subsystem === 'manager');
            const cpu = server.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            const memory = server.heatmaps.find(heatmap => heatmap.metric === 'mem_perc');
            serverTable.addRow([
                { order: 0, value: server.label, url: URL_1.URL.generate('oraclevm', 'server', server.item_id) },
                { order: 1, value: manager ? manager._parent.label : 'undefined' },
                { order: 2, value: serverPool ? serverPool._parent.label : 'undefined' },
                { order: 3, value: cpu ? cpu.value : null },
                { order: 4, value: memory ? memory.value : null },
            ]);
        }
        serverTable.sortByIndex(4);
        tabs.push({
            title: 'Server table',
            content: [serverTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OraclevmHeatmapPage = OraclevmHeatmapPage;
class OraclevmTopPage extends OraclevmPage {
    constructor() {
        super(...arguments);
        this.title = 'VM TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('oraclevm', 'vm');
        top.setParentSubsystem('manager');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.OraclevmTopPage = OraclevmTopPage;
class OraclevmConfigurationPage extends OraclevmPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let tabs = [];
        const serverPools = await this.objectItemsRepository.findBy({
            hw_type: hwTypeKey_constants_1.HwTypeKey.ORACLEVM,
            subsystem: 'server'
        });
        const serverPoolTable = new ConfigurationTable_1.ConfigurationTable(serverPools.map(item => item.item_id), ['serverpool_name', 'hostname', 'ip_address', 'total_memory', 'cpu_sockets', 'cpu_type', 'hypervisor_type', 'hypervisor_version', 'product_name', 'bios_version']);
        serverPoolTable.setAcl(this.request);
        serverPoolTable.setItemLabel("Server");
        tabs.push({
            title: 'Server Pool',
            content: [await serverPoolTable.generate()],
        });
        const vms = await this.objectItemsRepository.findBy({
            hw_type: hwTypeKey_constants_1.HwTypeKey.ORACLEVM,
            subsystem: 'vm'
        });
        const vmTable = new ConfigurationTable_1.ConfigurationTable(vms.map(item => item.item_id), ['serverpool_name', 'server_name', 'total_memory', 'cpu_count', 'os_type', 'domain_type']);
        vmTable.setAcl(this.request);
        vmTable.setItemLabel("VM");
        tabs.push({
            title: 'VM',
            content: [await vmTable.generate()],
        });
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.OraclevmConfigurationPage = OraclevmConfigurationPage;
class OraclevmRootFolder extends OraclevmFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration', 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('VM TOP', this.hwType + '/top', 'top'));
        const managers = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'manager'
        });
        for (let manager of managers) {
            let path = this.hwType + '/manager?ids=' + manager.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(manager.label, path, this.hwType, 'manager', manager.item_id, OraclevmManagerFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        const managers = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'manager'
        });
        for (const manager of managers) {
            let path = this.hwType + '/manager?ids=' + manager.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(manager.label, path, this.hwType, 'manager', manager.item_id, OraclevmManagerFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OraclevmRootFolder = OraclevmRootFolder;
class OraclevmManagerFolder extends OraclevmSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'manager';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/' + this.subsystem + '-totals' + '?item_id=' + this.itemId, this.subsystem + '-totals', this.itemId));
        const pools = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'server_pool',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (const pool of pools) {
            let path = this.hwType + '/server-pool?ids=' + pool.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(pool.label, path, this.hwType, 'server_pool', pool.item_id, OraclevmServerpoolFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        const pools = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'server_pool',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (const pool of pools) {
            let path = this.hwType + '/server-pool?ids=' + pool.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(pool.label, path, this.hwType, 'server_pool', pool.item_id, OraclevmServerpoolFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OraclevmManagerFolder = OraclevmManagerFolder;
class OraclevmServerpoolFolder extends OraclevmSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'server_pool';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/server-pool-totals' + '?item_id=' + this.itemId, this.subsystem + '-totals', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuFolder("Server", 'oraclevm/server?ids=' + this.itemId, this.hwType, 'server', this.itemId, OraclevmServerFolder));
        menuItems.push(new menuResponse_dto_1.MenuFolder("VM", 'oraclevm/vm?ids=' + this.itemId, this.hwType, 'vm', this.itemId, OraclevmVmFolder));
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuFolder("VM", 'oraclevm/vm?ids=' + this.itemId, this.hwType, 'vm', this.itemId, OraclevmVmFolder));
        this.children = menuItems;
        return menuItems;
    }
}
exports.OraclevmServerpoolFolder = OraclevmServerpoolFolder;
class OraclevmServerFolder extends OraclevmSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'server';
    }
    async getChildren() {
        let menuItems = [];
        const servers = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'server',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (const server of servers) {
            menuItems.push(new menuResponse_dto_1.MenuLink(server.label, this.hwType + '/server' + '?item_id=' + server.item_id, 'server', server.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OraclevmServerFolder = OraclevmServerFolder;
class OraclevmVmFolder extends OraclevmSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getChildren() {
        let menuItems = [];
        const vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (const vm of vms) {
            menuItems.push(new menuResponse_dto_1.MenuLink(vm.label, this.hwType + '/vm' + '?item_id=' + vm.item_id, 'vm', vm.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        const vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        });
        for (const vm of vms) {
            menuItems.push(new menuResponse_dto_1.MenuLink(vm.label, this.hwType + '/vm' + '?item_id=' + vm.item_id, 'vm', vm.item_id));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.OraclevmVmFolder = OraclevmVmFolder;
class OraclevmManagertotalsPage extends OraclevmSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'manager';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Manager ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'server',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: servers.map(server => server.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: servers.map(server => server.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'used_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: servers.map(server => server.item_id),
                    aggregated: true,
                },
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'VM aggregated',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Server',
                        subsystem: 'server'
                    }
                },
            ],
        });
        tabs.push({
            title: 'VM %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Server',
                        subsystem: 'server'
                    }
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OraclevmManagertotalsPage = OraclevmManagertotalsPage;
class OraclevmServerpooltotalsPage extends OraclevmSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'server_pool';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Server pool ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'server',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: servers.map(server => server.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: servers.map(server => server.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'used_mem')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: servers.map(server => server.item_id),
                    aggregated: true,
                },
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'VM aggregated',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Server',
                        subsystem: 'server'
                    }
                },
            ],
        });
        tabs.push({
            title: 'VM %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0)
                    ],
                    uuids: vms.map(vm => vm.item_id),
                    aggregated: true,
                    label: 'VM',
                    parent: {
                        label: 'Server',
                        subsystem: 'server'
                    }
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OraclevmServerpooltotalsPage = OraclevmServerpooltotalsPage;
class OraclevmServerPage extends OraclevmSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'server';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Server ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        tabs.push({
            title: 'CPU max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_cores_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_percent')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OraclevmMetrics.Metric('server', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_cpu',
                    interval: 'month',
                    max: 100
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OraclevmMetrics.Metric('server', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_cpu',
                    interval: 'year',
                    max: 100
                })
            ],
        });
        tabs.push({
            title: 'CPU % max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'cpu_utilization_percent_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'total_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OraclevmMetrics.Metric('server', 'used_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, Color_1.Color.pallete.red),
                        new OraclevmMetrics.Metric('server', 'free_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 2, Color_1.Color.pallete.green),
                        new OraclevmMetrics.Metric('server', 'free_swap')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 3, Color_1.Color.pallete.blue)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        tabs.push({
            title: 'Memory %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('server', 'mem_perc')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        const interfaces = await this.objectItemsRepository.find({
            where: {
                subsystem: 'server_interface',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        const metricsWrapperInterface = new MetricsWrapper_1.MetricsWrapper(['network_received'], interfaces.map(item => item.item_id));
        const lastValuesInterface = await metricsWrapperInterface.getLastValuesFromTable();
        if (lastValuesInterface && Object.keys(lastValuesInterface).length >= 1) {
            tabs.push({
                title: 'LAN',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new OraclevmMetrics.Metric('server_interface', 'network_received')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1),
                            new OraclevmMetrics.Metric('server_interface', 'network_transmitted')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1)
                        ],
                        uuids: interfaces.map(item => item.item_id),
                        aggregated: true
                    },
                ],
            });
        }
        const disks = await this.objectItemsRepository.find({
            where: {
                subsystem: 'server_disk',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        const metricsWrapperDisk = new MetricsWrapper_1.MetricsWrapper(['disk_read'], disks.map(item => item.item_id));
        const lastValuesDisk = await metricsWrapperDisk.getLastValuesFromTable();
        if (lastValuesDisk && Object.keys(lastValuesDisk).length >= 1) {
            tabs.push({
                title: 'Disk',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new OraclevmMetrics.Metric('server_disk', 'disk_read')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1),
                            new OraclevmMetrics.Metric('server_disk', 'disk_write')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1)
                        ],
                        uuids: disks.map(item => item.item_id),
                        aggregated: true
                    },
                ],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OraclevmServerPage = OraclevmServerPage;
class OraclevmVmPage extends OraclevmSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        tabs.push({
            title: 'CPU max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black),
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_cores_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'CPU %',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_percent')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OraclevmMetrics.Metric('vm', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_vm',
                    interval: 'month',
                    max: 100
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new OraclevmMetrics.Metric('vm', 'cpu_utilization_percent')
                        .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_vm',
                    interval: 'year',
                    max: 100
                }),
            ],
        });
        tabs.push({
            title: 'CPU % max',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'cpu_utilization_percent_max')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [item.item_id],
                },
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new OraclevmMetrics.Metric('vm', 'used_mem')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [item.item_id],
                },
            ],
        });
        const interfaces = await this.objectItemsRepository.find({
            where: {
                subsystem: 'vm_interface',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        const metricsWrapperInterface = new MetricsWrapper_1.MetricsWrapper(['network_received'], interfaces.map(item => item.item_id));
        const lastValuesInterface = await metricsWrapperInterface.getLastValuesFromTable();
        if (lastValuesInterface && Object.keys(lastValuesInterface).length >= 1) {
            tabs.push({
                title: 'LAN',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new OraclevmMetrics.Metric('vm_interface', 'network_received')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1),
                            new OraclevmMetrics.Metric('vm_interface', 'network_transmitted')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1)
                        ],
                        uuids: interfaces.map(item => item.item_id),
                        aggregated: true
                    },
                ],
            });
        }
        const disks = await this.objectItemsRepository.find({
            where: {
                subsystem: 'vm_disk',
                parents: {
                    parent: item.item_id
                }
            },
            relations: {
                parents: true
            }
        });
        const metricsWrapperDisk = new MetricsWrapper_1.MetricsWrapper(['disk_read'], disks.map(item => item.item_id));
        const lastValuesDisk = await metricsWrapperDisk.getLastValuesFromTable();
        if (lastValuesDisk && Object.keys(lastValuesDisk).length >= 1) {
            tabs.push({
                title: 'Disk',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new OraclevmMetrics.Metric('vm_disk', 'disk_read')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1),
                            new OraclevmMetrics.Metric('vm_disk', 'disk_write')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, -1)
                        ],
                        uuids: disks.map(item => item.item_id),
                        aggregated: true
                    },
                ],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.OraclevmVmPage = OraclevmVmPage;
class OraclevmMetrics {
}
exports.OraclevmMetrics = OraclevmMetrics;
OraclevmMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.ORACLEVM, subsystem, metric);
        if (definition) {
            Object.assign(this, definition);
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
    getAggrGraph(type, stacked, order, aggregate) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.aggregate = aggregate ? aggregate : pageResponse_constants_1.METRIC_AGGREGATE.items;
        return this;
    }
};
//# sourceMappingURL=Oraclevm.js.map