"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PostgresMetrics = exports.PostgresRatioPage = exports.PostgresHealthstatusPage = exports.PostgresTopPage = exports.PostgresWaiteventsPage = exports.PostgresSqlqueryPage = exports.PostgresLocksPage = exports.PostgresSessionsPage = exports.PostgresThroughputPage = exports.PostgresSizePage = exports.PostgresBgwriterPage = exports.PostgresConfigurationPage = exports.PostgresOverviewPage = exports.PostgresSingleItemPage = exports.PostgresPage = exports.PostgresDbFolder = exports.PostgresDbsFolder = exports.PostgresDbfoldersFolder = exports.PostgresRootFolder = exports.PostgresSubsystemFolder = exports.PostgresFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemProperty_entity_1 = require("../../../entity/itemProperty.entity");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const HealthStatus_1 = require("./HealthStatus");
const Table_1 = require("./Table");
const Top_1 = require("./Top");
const typeorm_1 = require("typeorm");
const AbstractMenu_1 = require("./AbstractMenu");
const Color_1 = require("../../metrics/Color");
const TabFactory_1 = require("./TabFactory");
class PostgresFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = 'postgres';
    }
}
exports.PostgresFolder = PostgresFolder;
class PostgresSubsystemFolder extends PostgresFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.PostgresSubsystemFolder = PostgresSubsystemFolder;
class PostgresRootFolder extends PostgresFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Health Status', this.hwType + '/health-status', 'health-status'));
        menuItems.push(new menuResponse_dto_1.MenuLink('DB TOP', this.hwType + '/top', 'top'));
        let subsystem_server = 'dbfolders';
        let servers = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: subsystem_server,
        });
        servers.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let item of servers) {
            let path = this.hwType + '/' + subsystem_server + '?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, subsystem_server, item.item_id, PostgresDbfoldersFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.PostgresRootFolder = PostgresRootFolder;
class PostgresDbfoldersFolder extends PostgresSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'dbfolders';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Overview', this.hwType + '/overview?item_id=' + this.itemId, 'overview', this.itemId, true));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration?item_id=' + this.itemId, 'configuration', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink('BG writer', this.hwType + '/bg-writer?item_id=' + this.itemId, 'bg-writer', this.itemId));
        let subsystems = [{
                subsystem: 'db',
                folder: PostgresDbsFolder
            }];
        for (let childSubsystem of subsystems) {
            let childSubsystemItems = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: childSubsystem.subsystem,
                    updatedAt: (0, typeorm_1.MoreThanOrEqual)(new Date(new Date().valueOf() - ((1000 * 60 * 60) / 2)))
                }
            });
            if (childSubsystemItems !== null) {
                let path = this.hwType + '/dbs' + (this.itemId !== null ? '?ids=' + this.itemId : '');
                menuItems.push(new menuResponse_dto_1.MenuFolder("Instances", path, this.hwType, childSubsystem.subsystem, this.itemId, childSubsystem.folder));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.PostgresDbfoldersFolder = PostgresDbfoldersFolder;
class PostgresDbsFolder extends PostgresSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
    }
    async getChildren() {
        let menuItems = [];
        let items = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: this.subsystem,
                updatedAt: (0, typeorm_1.MoreThanOrEqual)(new Date(new Date().valueOf() - ((1000 * 60 * 60) / 2)))
            }
        });
        if (this.itemId) {
            let parentChildren = await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } });
            let childrenIds = parentChildren.map((relation) => relation.child);
            items.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
            for (let item of items) {
                if (childrenIds.includes(item.item_id)) {
                    let path = this.hwType + '/' + this.subsystem + '?ids=' + item.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, this.subsystem, item.item_id, PostgresDbFolder));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.PostgresDbsFolder = PostgresDbsFolder;
class PostgresDbFolder extends PostgresSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Size', this.hwType + '/size?item_id=' + this.itemId, 'size', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink('Throughput', this.hwType + '/throughput?item_id=' + this.itemId, 'throughput', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink('Sessions', this.hwType + '/sessions?item_id=' + this.itemId, 'sessions', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink('Locks', this.hwType + '/locks?item_id=' + this.itemId, 'locks', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink('SQL query', this.hwType + '/sql-query?item_id=' + this.itemId, 'sql-query', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink('Wait events', this.hwType + '/wait-events?item_id=' + this.itemId, 'wait-events', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuLink('Ratio', this.hwType + '/ratio?item_id=' + this.itemId, 'ratio', this.itemId));
        this.children = menuItems;
        return menuItems;
    }
}
exports.PostgresDbFolder = PostgresDbFolder;
class PostgresPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = 'postgres';
        this.rootFolder = PostgresRootFolder;
    }
}
exports.PostgresPage = PostgresPage;
class PostgresSingleItemPage extends PostgresPage {
    constructor() {
        super(...arguments);
        this.parentSubsystem = 'dbfolders';
        this.withItemId = true;
    }
}
exports.PostgresSingleItemPage = PostgresSingleItemPage;
class PostgresOverviewPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Overview';
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        let uuids = await getInstances(this.itemId);
        if (uuids === undefined) {
            uuids[0] = this.itemId;
        }
        uuids.sort();
        tabs.push({
            title: 'Overview',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                    graphs: [
                        {
                            metrics: [
                                await new PostgresMetrics.BasicDbMetric('tupreturned', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init()
                            ],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                            metrics: [
                                await new PostgresMetrics.BasicDbMetric('blksread', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init(),
                            ],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        },
                        {
                            type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                            metrics: [
                                await new PostgresMetrics.BasicDbMetric('active', pageResponse_constants_1.CHART_TYPE.line, this.subsystem, false, 0).init()
                            ],
                            uuids: uuids,
                            interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                            aggregated: true,
                        }
                    ],
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.PostgresOverviewPage = PostgresOverviewPage;
class PostgresConfigurationPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        let tableType = 'main';
        let id = this.itemId;
        let serverTable = await gen_table('main', id, {
            'shared_buffers': 'Shared buffers',
            'work_mem': 'Work memory ',
            'max_connections': 'Max connections',
            'temp_buffers': 'Temp buffers',
            'maintenance_work_mem': 'Maintenance work mem',
            'autovacuum_work_mem': 'Autovacuum work mem',
            'wal_buffers': 'WAL buffers',
            'effective_cache_size': 'Effective cache size',
            'random_page_cost': 'Random page cost',
            'min_wal_size': 'Min WAL size',
            'max_wal_size': 'Max WAL size',
            'wal_sync_method': 'WAL sync method',
        });
        tabs.push({
            title: 'Configuration',
            content: [serverTable.generatePivotBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.PostgresConfigurationPage = PostgresConfigurationPage;
class PostgresBgwriterPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'dbfolders';
        this.title = 'BG Writer';
    }
    async getPage() {
        let title = 'BG Writer';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "dbfolders", [this.itemId]);
        tabFactory.addTab("Buffers", [
            { metric: 'bufferscheckpoint', default: TabFactory_1.MPDefault.LINE },
            { metric: 'buffersbackend', default: TabFactory_1.MPDefault.LINE },
            { metric: 'buffersclean', default: TabFactory_1.MPDefault.LINE }
        ]);
        tabFactory.addTab("Checkpoints", [
            { metric: "checkpointsreq", default: TabFactory_1.MPDefault.LINE },
            { metric: "checkpointstimed", default: TabFactory_1.MPDefault.LINE }
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresBgwriterPage = PostgresBgwriterPage;
class PostgresSizePage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'DB Size';
    }
    async getPage() {
        let title = 'DB Size';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "db", [this.itemId]);
        tabFactory.addTab(title, [
            { metric: 'size', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        let tableType = 'relations';
        let id = this.itemId;
        let serverTable = await gen_table(tableType, id, {
            'Namespace': 'Namespace',
            'Name': 'Name',
            'Type': 'Type',
            'Size': 'Size\n[GiB]',
            'Pages': 'Pages',
            'Rows': 'Rows'
        });
        tabs.push({
            title: 'Relations',
            content: [serverTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresSizePage = PostgresSizePage;
class PostgresThroughputPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Throughput';
    }
    async getPage() {
        let title = 'Throughput';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "db", [this.itemId]);
        tabFactory.addTab('Blocks', [
            { metric: 'blkshit', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'blksread', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Tuples', [
            { metric: 'tupupdated', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'tupinserted', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'tupdeleted', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'tupreturned', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'tupfetched', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Temp', [
            { metric: 'tempfiles', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'tempbytes', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresThroughputPage = PostgresThroughputPage;
class PostgresSessionsPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Sessions';
    }
    async getPage() {
        let title = 'Sessions';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "db", [this.itemId]);
        tabFactory.addTab('Sessions', [
            { metric: 'active', default: TabFactory_1.MPDefault.LINE },
            { metric: 'idle', default: TabFactory_1.MPDefault.LINE },
            { metric: 'idleintransaction', default: TabFactory_1.MPDefault.LINE },
            { metric: 'idleintransactionaborted', default: TabFactory_1.MPDefault.LINE },
            { metric: 'fastpathfunctioncall', default: TabFactory_1.MPDefault.LINE },
            { metric: 'disabled', default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresSessionsPage = PostgresSessionsPage;
class PostgresLocksPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Locks';
    }
    async getPage() {
        let title = 'Locks';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "db", [this.itemId]);
        tabFactory.addTab('Held', [
            { metric: 'hrelation', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'hextend', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'hpage', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'htuple', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'hobject', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'huserlock', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Awaited', [
            { metric: 'arelation', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'aextend', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'apage', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'atuple', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'aobject', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'auserlock', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Deadlocks', [
            { metric: 'deadlocks', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresLocksPage = PostgresLocksPage;
class PostgresSqlqueryPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'SQL query';
    }
    async getPage() {
        let title = 'SQL query';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "db", [this.itemId]);
        tabFactory.addTab('User', [
            { metric: 'xactcommit', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.AREA_STACKED },
            { metric: 'xactrollback', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.AREA_STACKED },
        ]);
        tabFactory.addTab('Cursors', [
            { metric: 'cursors', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresSqlqueryPage = PostgresSqlqueryPage;
class PostgresWaiteventsPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Wait events';
    }
    async getPage() {
        let title = 'Wait events';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "db", [this.itemId]);
        tabFactory.addTab('Type', [
            { metric: 'LWLockNamed', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'LWLockTranche', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'Lock', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'BufferPin', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('LWLockNamed', [
            { metric: 'ShmemIndexLock', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'XidGenLock', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ProcArrayLock', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'WALBufMappingLock', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'WALWriteLock', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ControlFileLock', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'CheckpointLock', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'RelCacheInitLock', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'CheckpointerCommLock', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'TablespaceCreateLock', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'AddinShmemInitLock', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'AutovacuumLock', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'AutovacuumScheduleLock', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'SyncScanLock', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'RelationMappingLock', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'AsyncCtlLock', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'SerializableFinishedListLock', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'SerializablePredicateLockListLock', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'OldSerXidLock', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'BackgroundWorkerLock', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ReplicationSlotAllocationLock', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'CommitTsLock', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ReplicationOriginLock', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'MultiXactTruncationLock', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'OldSnapshotTimeMapLock', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'WrapLimitsVacuumLock', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('LWLockTranche', [
            { metric: 'clog', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'committimestamp', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'subtrans', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'multixactoffset', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'multixactmember', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'async', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'oldserxid', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'walinsert', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'buffercontent', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'bufferio', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'replicationorigin', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'replicationslotio', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'proc', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'buffermapping', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'lockmanager', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'predicatelockmanager', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Lock', [
            { metric: 'relation', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'extend', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'frozenid', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'page', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'tuple', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'transactionid', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'virtualxid', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'speculativetoken', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'object', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'userlock', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'advisory', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'AutovacuumLock', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'AutovacuumScheduleLock', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'SyncScanLock', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'RelationMappingLock', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'AsyncCtlLock', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'SerializableFinishedListLock', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'SerializablePredicateLockListLock', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'OldSerXidLock', color: Color_1.Color.pallete.blueDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'BackgroundWorkerLock', color: Color_1.Color.pallete.orangeDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ReplicationSlotAllocationLock', color: Color_1.Color.pallete.yellowDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'CommitTsLock', color: Color_1.Color.pallete.brownDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'ReplicationOriginLock', color: Color_1.Color.pallete.purpleDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'MultiXactTruncationLock', color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
            { metric: 'OldSnapshotTimeMapLock', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
            { metric: 'WrapLimitsVacuumLock', color: Color_1.Color.pallete.greenDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresWaiteventsPage = PostgresWaiteventsPage;
class PostgresTopPage extends PostgresPage {
    constructor() {
        super(...arguments);
        this.title = 'DB TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('postgres', 'db');
        top.setParentSubsystem('dbfolders');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.PostgresTopPage = PostgresTopPage;
class PostgresHealthstatusPage extends PostgresPage {
    constructor() {
        super(...arguments);
        this.title = 'Health Status';
    }
    async getPage() {
        let title = 'Health Status';
        let tabs = [];
        let healthStatus = await new HealthStatus_1.HealthStatus().setAcl(this.request).useDeviceAliasColumn(true).useLinkToItem(true).useClickableAlias(false).getStatusByParameters("db", "database", "Instance", "postgres");
        tabs.push({
            title: 'Health Status',
            content: [healthStatus],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.PostgresHealthstatusPage = PostgresHealthstatusPage;
class PostgresRatioPage extends PostgresSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'db';
        this.title = 'Ratio';
    }
    async getPage() {
        let title = 'Ratio';
        let tabs = [];
        const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, "db", [this.itemId]);
        tabFactory.addTab('Cache', [
            { metric: 'cachehitratio', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        tabFactory.addTab('Commit', [
            { metric: 'commitratio', color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
        ]);
        this.page = new pageResponse_dto_1.Page(title, tabs.concat(...tabFactory.getTabs()));
        return this.page;
    }
}
exports.PostgresRatioPage = PostgresRatioPage;
class PostgresMetrics {
}
exports.PostgresMetrics = PostgresMetrics;
PostgresMetrics.Metric = class extends pageResponse_dto_1.Metric {
    async init() {
        const metric = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.DATABASE, hwTypeKey_constants_1.HwTypeKey.POSTGRES, this.subsystem, this.metric);
        if (metric) {
            this.unit = metric.unit;
            this.shortcut = metric.shortcut;
            this.label = metric.label;
            this.defaultPrefix = metric.defaultPrefix ? metric.defaultPrefix : undefined;
            this.defaultPrecision = metric.defaultPrecision ? metric.defaultPrecision : undefined;
        }
        return this;
    }
};
PostgresMetrics.BasicDbMetric = class extends PostgresMetrics.Metric {
    constructor(metric, type, subsystem, stacked, order, color) {
        super();
        this.metric = metric;
        this.type = type;
        this.subsystem = subsystem;
        this.stacked = stacked;
        this.order = order;
        this.color = color;
    }
};
PostgresMetrics.CheckpointMetric = class extends PostgresMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'DB';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'buffers_checkpoint';
        this.order = 0;
    }
};
async function gen_table(tableType, id, dict) {
    const itemsPropertiesRepository = (0, dataSource_1.getDataSource)().getRepository(itemProperty_entity_1.ItemProperty);
    let serverTable = new Table_1.Table();
    for (const item in dict) {
        let value = dict[item];
        let properties = {};
        if (value.includes("[GiB]")) {
            value = value.replace('\n[GiB]', '');
            properties = {
                prefix: "Gi",
                unit: "bytes",
                shortcut: "GiB",
                number: true,
            };
        }
        serverTable.addHeader({ property: value, value: value, ...properties });
    }
    let rawProperties = Object.keys(dict);
    let tableRes = await itemsPropertiesRepository.findOneBy({ item_id: id, property_name: tableType });
    if (tableRes) {
        let wholeStuff = JSON.parse(tableRes.property_value);
        for (const dbRow of wholeStuff) {
            let row = [];
            for (const rawHeader of rawProperties) {
                let value = dbRow[rawHeader] ? dbRow[rawHeader] : "";
                row.push({
                    value: value,
                });
            }
            serverTable.addRow(row);
        }
        serverTable.sortByIndex(0, true);
        return serverTable;
    }
    else {
        let row = [];
        row.push({
            order: 1,
            value: "value",
        });
        return serverTable.addRow(row);
    }
}
async function getInstances(item_id) {
    let itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    let items = [];
    let itemRelations = await itemRelationsRepository.find({
        where: {
            parent: item_id,
            updatedAt: (0, typeorm_1.MoreThanOrEqual)(new Date(new Date().valueOf() - ((1000 * 60 * 60) / 2)))
        }
    });
    for (let relation of itemRelations) {
        items.push(relation.child);
    }
    return items;
}
//# sourceMappingURL=Postgres.js.map