"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProxmoxHealthstatusPage = exports.ProxmoxStoragePage = exports.ProxmoxLxcPage = exports.ProxmoxNodePage = exports.ProxmoxVmPage = exports.ProxmoxSingleItemPage = exports.ProxmoxClusterstoragetotalsPage = exports.ProxmoxClusternodetotalsPage = exports.ProxmoxClusterlxctotalsPage = exports.ProxmoxClustervmtotalsPage = exports.ProxmoxClusterTotalsPage = exports.ProxmoxVmtopPage = exports.ProxmoxArchitecturePage = exports.ProxmoxConfigurationPage = exports.ProxmoxOverviewPage = exports.ProxmoxTopPage = exports.ProxmoxHeatmapPage = exports.ProxmoxTotalsPage = exports.ProxmoxPage = exports.ProxmoxMetrics = exports.ProxmoxMetric = exports.ProxmoxStorageFolder = exports.ProxmoxLxcFolder = exports.ProxmoxVmFolder = exports.ProxmoxNodeFolder = exports.ProxmoxChildSubsystemFolder = exports.ProxmoxClusterFolder = exports.ProxmoxRootFolder = exports.ProxmoxSubsystemFolder = exports.ProxmoxFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const dataSource_1 = require("../../../config/dataSource");
const itemRelation_entity_1 = require("../../../entity/itemRelation.entity");
const objectItem_entity_1 = require("../../../entity/objectItem.entity");
const ACLCheck_1 = require("../../acl/ACLCheck");
const Item_1 = require("../../configuration/Item");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const Metric_1 = require("../../metrics/Metric");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const typeorm_1 = require("typeorm");
const HealthStatus_1 = require("./HealthStatus");
const Heatmap_1 = require("./Heatmap");
const Mapping_1 = require("./Mapping");
const Overview_1 = require("./Overview");
const Table_1 = require("./Table");
const Top_1 = require("./Top");
const URL_1 = require("./URL");
const ConfigurationTable_1 = require("./ConfigurationTable");
const Prediction_1 = require("../../metrics/Prediction");
const cpu_peak_help = {
    url: "https://xormon.com/max_vrs_average.php",
    text: "How it works?",
};
class ProxmoxFolder {
    constructor() {
        this.hwType = 'proxmox';
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPathFolder(parentIds) {
        const children = await this.getChildren();
        for (const child of children) {
            const child_id = parentIds.find((v) => v === child.itemId);
            if (child_id) {
                if (child._new) {
                    child.children = await new child._new(child_id)["getPathFolder"](parentIds);
                }
            }
        }
        return children;
    }
    async getChildren() {
        return null;
    }
}
exports.ProxmoxFolder = ProxmoxFolder;
class ProxmoxSubsystemFolder extends ProxmoxFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.ProxmoxSubsystemFolder = ProxmoxSubsystemFolder;
class ProxmoxRootFolder extends ProxmoxFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Configuration', this.hwType + '/configuration', 'configuration'));
        menuItems.push(new menuResponse_dto_1.MenuLink('VM TOP', this.hwType + '/top', 'top'));
        menuItems.push(new menuResponse_dto_1.MenuLink('Overview', this.hwType + '/overview', 'overview'));
        let subsystem_cluster = 'cluster';
        let clusters = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: subsystem_cluster,
        });
        for (let item of clusters) {
            let path = this.hwType + '/' + subsystem_cluster + '?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, subsystem_cluster, item.item_id, ProxmoxClusterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        let subsystem_cluster = 'cluster';
        let clusters = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: subsystem_cluster,
        });
        for (let item of clusters) {
            let path = this.hwType + '/' + subsystem_cluster + '?ids=' + item.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(item.label, path, this.hwType, subsystem_cluster, item.item_id, ProxmoxClusterFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.ProxmoxRootFolder = ProxmoxRootFolder;
class ProxmoxClusterFolder extends ProxmoxSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'cluster';
    }
    async getChildren() {
        let menuItems = [];
        let subsystems = ['Node', 'VM', 'LXC', 'Storage'];
        let subsystemFolders = {
            "Node": ProxmoxNodeFolder,
            "VM": ProxmoxVmFolder,
            "LXC": ProxmoxLxcFolder,
            "Storage": ProxmoxStorageFolder
        };
        for (let childSubsystem of subsystems) {
            let childSubsystemCheck = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: childSubsystem.toLowerCase(),
            });
            if (childSubsystemCheck) {
                menuItems.push(new menuResponse_dto_1.MenuLink(childSubsystem + ' Totals', this.hwType + '/' + this.subsystem + '-' + childSubsystem.toLowerCase() + '-totals' + '?item_id=' + this.itemId, childSubsystem.toLowerCase() + '-totals', this.itemId, childSubsystem === 'Node' ? true : undefined));
            }
        }
        for (let childSubsystem of subsystems) {
            let childSubsystemCheck = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: childSubsystem.toLowerCase(),
            });
            if (childSubsystemCheck) {
                let path = this.hwType + '/' + childSubsystem.toLowerCase() + (this.itemId !== null ? '?ids=' + this.itemId : '');
                menuItems.push(new menuResponse_dto_1.MenuFolder(childSubsystem, path, this.hwType, childSubsystem.toLowerCase(), this.itemId, subsystemFolders[childSubsystem]));
            }
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        let childSubsystem = 'VM';
        let childSubsystemItems = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: childSubsystem.toLowerCase(),
        });
        if (childSubsystemItems !== null) {
            let path = this.hwType + '/' + childSubsystem.toLowerCase() + (this.itemId !== null ? '?ids=' + this.itemId : '');
            menuItems.push(new menuResponse_dto_1.MenuFolder(childSubsystem, path, this.hwType, childSubsystem.toLowerCase(), this.itemId, ProxmoxVmFolder));
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.ProxmoxClusterFolder = ProxmoxClusterFolder;
class ProxmoxChildSubsystemFolder extends ProxmoxSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let items = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: this.subsystem });
        if (this.itemId) {
            let parentChildren = await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } });
            let childrenIds = parentChildren.map((relation) => relation.child);
            for (let item of items) {
                if (childrenIds.includes(item.item_id)) {
                    let path = this.hwType + '/' + this.subsystem + '?item_id=' + item.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, this.subsystem, item.item_id));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
    async getChildrenAcl() {
        let menuItems = [];
        let items = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: this.subsystem });
        if (this.itemId) {
            let parentChildren = await this.itemRelationsRepository.find({ where: { parent: this.itemId.toString() } });
            let childrenIds = parentChildren.map((relation) => relation.child);
            for (let item of items) {
                if (childrenIds.includes(item.item_id)) {
                    let path = this.hwType + '/' + this.subsystem + '?item_id=' + item.item_id;
                    menuItems.push(new menuResponse_dto_1.MenuLink(item.label, path, this.subsystem, item.item_id));
                }
            }
        }
        this.children = menuItems;
        return menuItems;
    }
}
exports.ProxmoxChildSubsystemFolder = ProxmoxChildSubsystemFolder;
class ProxmoxNodeFolder extends ProxmoxChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
    }
}
exports.ProxmoxNodeFolder = ProxmoxNodeFolder;
class ProxmoxVmFolder extends ProxmoxChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
}
exports.ProxmoxVmFolder = ProxmoxVmFolder;
class ProxmoxLxcFolder extends ProxmoxChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
    }
}
exports.ProxmoxLxcFolder = ProxmoxLxcFolder;
class ProxmoxStorageFolder extends ProxmoxChildSubsystemFolder {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage';
    }
}
exports.ProxmoxStorageFolder = ProxmoxStorageFolder;
class ProxmoxMetric extends pageResponse_dto_1.Metric {
    constructor(metric, type, stacked, order) {
        super();
        this.metric = metric;
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
    }
}
exports.ProxmoxMetric = ProxmoxMetric;
class ProxmoxMetrics {
}
exports.ProxmoxMetrics = ProxmoxMetrics;
ProxmoxMetrics.Metric = class extends pageResponse_dto_1.Metric {
    init() {
        const metric = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.PROXMOX, this.subsystem, this.metric);
        if (metric) {
            Object.assign(this, metric);
        }
        return this;
    }
};
ProxmoxMetrics.AggrMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.aggregate = pageResponse_constants_1.METRIC_AGGREGATE.items;
    }
};
ProxmoxMetrics.VmCpuAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_percent';
        this.order = 0;
    }
};
ProxmoxMetrics.VmCpuMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_percent';
        this.order = 0;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.VmCpuMaxMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_percent_max';
        this.order = 1;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
ProxmoxMetrics.VmCpuCoresMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_cores';
        this.order = 0;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.VmMaxCpuCoresMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'maxcpu';
        this.order = 1;
        this.color = Color_1.Color.metric.total;
    }
};
ProxmoxMetrics.VmCpuCoresAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu_utilization_cores';
        this.order = 0;
    }
};
ProxmoxMetrics.VmMemusedAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'mem';
        this.order = 0;
    }
};
ProxmoxMetrics.VmMemfreeAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'memfree';
        this.order = 0;
    }
};
ProxmoxMetrics.VmMemMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.subsystem = 'vm';
        this.stacked = true;
        this.metric = 'mem';
        this.color = Color_1.Color.pallete.red;
        this.order = 0;
    }
};
ProxmoxMetrics.VmMemFreeMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.subsystem = 'vm';
        this.stacked = true;
        this.metric = 'memfree';
        this.color = Color_1.Color.pallete.green;
        this.order = 1;
    }
};
ProxmoxMetrics.VmMaxmemMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'maxmem';
        this.color = Color_1.Color.metric.total;
        this.order = 2;
    }
};
ProxmoxMetrics.VmDiskreadAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'diskread';
        this.order = -1;
    }
};
ProxmoxMetrics.VmDiskreadMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'diskread';
        this.order = -1;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.VmDiskwriteAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'diskwrite';
        this.order = 0;
    }
};
ProxmoxMetrics.VmDiskwriteMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'diskwrite';
        this.order = 0;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
ProxmoxMetrics.VmNetinAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'netin';
        this.order = -1;
    }
};
ProxmoxMetrics.VmNetinMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'netin';
        this.order = -1;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.VmNetoutAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'netout';
        this.order = 0;
    }
};
ProxmoxMetrics.VmNetoutMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'netout';
        this.order = 0;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
ProxmoxMetrics.LxcCpuAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcCpuMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcMemusedAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'mem';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcMemfreeAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'memfree';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcMemMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.subsystem = 'lxc';
        this.stacked = false;
        this.metric = 'mem';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcMaxmemMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'maxmem';
        this.order = 1;
    }
};
ProxmoxMetrics.LxcDiskreadAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'diskread';
        this.order = -1;
    }
};
ProxmoxMetrics.LxcDiskreadMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'diskread';
        this.order = -1;
    }
};
ProxmoxMetrics.LxcDiskwriteAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'diskwrite';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcDiskwriteMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'diskwrite';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcNetinAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'netin';
        this.order = -1;
    }
};
ProxmoxMetrics.LxcNetinMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'netin';
        this.order = -1;
    }
};
ProxmoxMetrics.LxcNetoutAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'netout';
        this.order = 0;
    }
};
ProxmoxMetrics.LxcNetoutMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'netout';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeCpuMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_percent';
        this.order = 0;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.NodeCpuMaxMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_percent_max';
        this.order = 1;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
ProxmoxMetrics.NodeCpuCoresMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_cores';
        this.order = 0;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.NodeMaxCpuCoresMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'maxcpu';
        this.order = 0;
        this.color = Color_1.Color.metric.total;
    }
};
ProxmoxMetrics.NodeCpuAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'cpu_utilization_percent';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeCpuCoresAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'cpu_utilization_cores';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeMemtotalMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'memtotal';
        this.color = Color_1.Color.metric.total;
        this.order = 1;
    }
};
ProxmoxMetrics.NodeMemusedMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'memused';
        this.color = Color_1.Color.pallete.red;
        this.order = 0;
    }
};
ProxmoxMetrics.NodeMemfreeMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'memfree';
        this.color = Color_1.Color.pallete.green;
        this.order = 1;
    }
};
ProxmoxMetrics.NodeMemusedAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'memused';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeMemfreeAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'memfree';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeSwaptotalMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'swaptotal';
        this.color = Color_1.Color.metric.total;
        this.order = 1;
    }
};
ProxmoxMetrics.NodeSwapusedMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'swapused';
        this.color = Color_1.Color.pallete.red;
        this.order = 0;
    }
};
ProxmoxMetrics.NodeSwapfreeMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'swapfree';
        this.color = Color_1.Color.pallete.green;
        this.order = 0;
    }
};
ProxmoxMetrics.NodeSwapusedAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'swapused';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeSwapfreeAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'swapfree';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeIowaitMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'iowait';
        this.order = 0;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.NodeIowaitAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'iowait';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeNetinMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'netin';
        this.order = -1;
        this.color = Color_1.Color.pallete.redDark;
    }
};
ProxmoxMetrics.NodeNetinAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'netin';
        this.order = -1;
    }
};
ProxmoxMetrics.NodeNetoutMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'netout';
        this.order = 0;
        this.color = Color_1.Color.pallete.blueDark;
    }
};
ProxmoxMetrics.NodeNetoutAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'netout';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeRoottotalMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'roottotal';
        this.color = Color_1.Color.metric.total;
        this.order = 1;
    }
};
ProxmoxMetrics.NodeRootusedMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'rootused';
        this.color = Color_1.Color.pallete.red;
        this.order = 0;
    }
};
ProxmoxMetrics.NodeRootfreeMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'rootfree';
        this.color = Color_1.Color.pallete.green;
        this.order = 0;
    }
};
ProxmoxMetrics.NodeRootusedAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'rootused';
        this.order = 0;
    }
};
ProxmoxMetrics.NodeRootfreeAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'rootfree';
        this.order = 0;
    }
};
ProxmoxMetrics.StorageTotalMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage';
        this.type = pageResponse_constants_1.CHART_TYPE.line;
        this.stacked = false;
        this.metric = 'total';
        this.color = Color_1.Color.metric.total;
        this.order = 1;
    }
};
ProxmoxMetrics.StorageUsedMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'used';
        this.color = Color_1.Color.pallete.red;
        this.order = 0;
    }
};
ProxmoxMetrics.StorageFreeMetric = class extends ProxmoxMetrics.Metric {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'free';
        this.color = Color_1.Color.pallete.green;
        this.order = 0;
    }
};
ProxmoxMetrics.StorageUsedAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'used';
        this.order = 0;
    }
};
ProxmoxMetrics.StorageFreeAggrMetric = class extends ProxmoxMetrics.AggrMetric {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage';
        this.type = pageResponse_constants_1.CHART_TYPE.area;
        this.stacked = true;
        this.metric = 'free';
        this.order = 0;
    }
};
class ProxmoxPage {
    constructor(itemId, request) {
        this.itemId = itemId;
        this.request = request;
        this.hwType = 'proxmox';
        this.withItemId = false;
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
    async getPath() {
        const parents = await this.objectItemsRepository.find({
            relations: {
                children: true
            },
            where: {
                children: {
                    child: this.itemId
                }
            }
        });
        const parentIds = parents.map(item => item.item_id);
        if (this.withItemId) {
            parentIds.push(this.itemId);
        }
        const path = await new ProxmoxRootFolder().getPathFolder(parentIds);
        return path;
    }
}
exports.ProxmoxPage = ProxmoxPage;
class ProxmoxTotalsPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.objectItemsRepository = (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem);
        this.itemRelationsRepository = (0, dataSource_1.getDataSource)().getRepository(itemRelation_entity_1.ItemRelation);
    }
}
exports.ProxmoxTotalsPage = ProxmoxTotalsPage;
class ProxmoxHeatmapPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'Node',
            content: Object.values(await new Heatmap_1.Heatmap('proxmox', 'node', true)
                .virtHeatmapBox(this.request)),
        });
        let nodeTable = new Table_1.Table()
            .addHeader({ value: 'Node' })
            .addHeader({ value: 'Cluster' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let nodes = await this.objectItemsRepository.find({
            where: {
                hw_type: 'proxmox',
                subsystem: 'node'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        nodes = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, nodes);
        for (const node of nodes) {
            const cluster = node.parents.find(parent => parent._parent.subsystem === 'cluster');
            const cpu = node.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            const memory = node.heatmaps.find(heatmap => heatmap.metric === 'memory');
            nodeTable.addRow([
                { order: 0, value: node.label, url: URL_1.URL.generate('proxmox', 'node', node.item_id) },
                { order: 1, value: cluster ? cluster._parent.label : 'undefined' },
                { order: 2, value: cpu ? cpu.value : null },
                { order: 3, value: memory ? memory.value : null },
            ]);
        }
        nodeTable.sortByIndex(2);
        nodeTable.setBoxExport("proxmox-node-heatmap");
        tabs.push({
            title: 'Node table',
            content: [nodeTable.generateBox()],
        });
        tabs.push({
            title: 'VM',
            content: Object.values(await new Heatmap_1.Heatmap('proxmox', 'vm', true)
                .virtHeatmapBox(this.request)),
        });
        let vmTable = new Table_1.Table()
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'Cluster' })
            .addHeader({ value: 'Host' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: 'proxmox',
                subsystem: 'vm'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        vms = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, vms);
        for (const vm of vms) {
            const cluster = vm.parents.find(parent => parent._parent.subsystem === 'cluster');
            const host = vm.parents.find(parent => parent._parent.subsystem === 'node');
            const cpu = vm.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization_percent');
            const memory = vm.heatmaps.find(heatmap => heatmap.metric === 'memory');
            vmTable.addRow([
                { order: 0, value: vm.label, url: URL_1.URL.generate('proxmox', 'vm', vm.item_id) },
                { order: 1, value: cluster ? cluster._parent.label : 'undefined' },
                { order: 2, value: host ? host._parent.label : 'undefined', url: host ? URL_1.URL.generate('proxmox', 'node', host.parent) : undefined },
                { order: 3, value: cpu ? cpu.value : null },
                { order: 4, value: memory ? memory.value : null },
            ]);
        }
        vmTable.sortByIndex(3);
        vmTable.setBoxExport("proxmox-vm-heatmap");
        tabs.push({
            title: 'VM table',
            content: [vmTable.generateBox()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxHeatmapPage = ProxmoxHeatmapPage;
class ProxmoxTopPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.title = 'VM TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('proxmox', 'vm');
        top.setParentSubsystem('cluster');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        this.page = new pageResponse_dto_1.Page(this.title, tabs);
        return this.page;
    }
}
exports.ProxmoxTopPage = ProxmoxTopPage;
class ProxmoxOverviewPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.title = 'Overview';
    }
    async getPage() {
        let title = 'Overview';
        let tabs = [];
        const overview = new Overview_1.Overview('proxmox');
        let nodes = await this.objectItemsRepository.findBy({ hw_type: hwTypeKey_constants_1.HwTypeKey.PROXMOX, subsystem: 'node' });
        nodes = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, nodes);
        overview.addCategory('Node', nodes.map(item => {
            return {
                label: item.label,
                item_id: item.item_id
            };
        }));
        tabs.push({
            title: 'Node',
            content: [
                overview.form()
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxOverviewPage = ProxmoxOverviewPage;
class ProxmoxConfigurationPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.title = 'Configuration';
    }
    async getPage() {
        let title = 'Configuration';
        let tabs = [];
        let nodes = await this.objectItemsRepository.findBy({ hw_type: 'proxmox', subsystem: 'node' });
        const nodeTable = new ConfigurationTable_1.ConfigurationTable(nodes.map(node => node.item_id), ['status', 'uptime', 'maxcpu', 'maxdisk', 'maxmem'], ['cluster']);
        nodeTable.setItemLabel("Node");
        nodeTable.setAcl(this.request);
        tabs.push({
            title: 'Node',
            content: [await nodeTable.generate()],
        });
        let vms = await this.objectItemsRepository.findBy({ hw_type: 'proxmox', subsystem: 'vm' });
        const vmTable = new ConfigurationTable_1.ConfigurationTable(vms.map(vm => vm.item_id), ['status', 'cpus', 'maxdisk', 'maxmem'], ['cluster', 'node']);
        vmTable.setItemLabel("VM");
        vmTable.setAcl(this.request);
        tabs.push({
            title: 'VM',
            content: [await vmTable.generate()],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxConfigurationPage = ProxmoxConfigurationPage;
class ProxmoxArchitecturePage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.title = 'Architecture';
    }
    async getPage() {
        let title = 'Architecture';
        let tabs = [];
        let vmSunburst = [];
        let clusters = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'cluster' });
        for (const cluster of clusters) {
            let nodeTotal = 0;
            let clusterNodes = await new Item_1.Item(cluster.item_id).findChildren('proxmox', 'node');
            let nodeIds = clusterNodes.map((clusterNode) => clusterNode.child_id);
            let nodeUsedMem = await new Metric_1.Metric('memused', nodeIds).getAverage(60);
            let nodeTotalMem = await new Metric_1.Metric('memtotal', nodeIds).getAverage(60);
            for (const node of clusterNodes) {
                nodeTotal += +nodeTotalMem[node.child_id];
                vmSunburst.push({
                    item_id: node.child_id,
                    label: node.child_label,
                    parent_id: cluster.item_id,
                    parent_label: cluster.label,
                    value: nodeUsedMem[node.child_id] ? +nodeUsedMem[node.child_id] : 0,
                });
                let nodeVMs = await new Item_1.Item(node.child_id).findChildren('proxmox', 'vm');
                let vmIds = nodeVMs.map((nodeVM) => nodeVM.child_id);
                let vmUsedMem = await new Metric_1.Metric('mem', vmIds).getAverage(60);
                for (const vm of nodeVMs) {
                    vmSunburst.push({
                        item_id: vm.child_id,
                        label: vm.child_label,
                        parent_id: node.child_id,
                        parent_label: node.child_label,
                        value: vmUsedMem[vm.child_id] ? +vmUsedMem[vm.child_id] : 0,
                    });
                }
            }
            vmSunburst.push({
                item_id: cluster.item_id,
                label: cluster.label,
                parent_id: '',
                parent_label: '',
                value: nodeTotal ? nodeTotal : 0,
            });
        }
        let sunburstBox = {
            type: pageResponse_constants_1.BOX_TYPE.SunburstBox,
            items: vmSunburst,
        };
        tabs.push({
            title: 'VM',
            content: [sunburstBox],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxArchitecturePage = ProxmoxArchitecturePage;
class ProxmoxVmtopPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.title = 'VM Top';
    }
}
exports.ProxmoxVmtopPage = ProxmoxVmtopPage;
class ProxmoxClusterTotalsPage extends ProxmoxTotalsPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'cluster';
        this.withItemId = true;
    }
}
exports.ProxmoxClusterTotalsPage = ProxmoxClusterTotalsPage;
class ProxmoxClustervmtotalsPage extends ProxmoxClusterTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM Totals ' + (item !== null ? item.label : this.itemId);
        let child_vms = [];
        let vms = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'vm' });
        for (let vm of vms) {
            child_vms.push(vm.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_vms) });
        for (let relation of itemRelations) {
            items.push(relation.child);
        }
        const parent = MetricsWrapper_1.MetricsWrapper.getParent(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.PROXMOX, 'vm');
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmCpuCoresAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmCpuAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'Memory used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmMemusedAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'Memory free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmMemfreeAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmDiskreadAggrMetric().init(), await new ProxmoxMetrics.VmDiskwriteAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'VM'
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmNetinAggrMetric().init(), await new ProxmoxMetrics.VmNetoutAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'VM'
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxClustervmtotalsPage = ProxmoxClustervmtotalsPage;
class ProxmoxClusterlxctotalsPage extends ProxmoxClusterTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'LXC Totals ' + (item !== null ? item.label : this.itemId);
        let child_lxcs = [];
        let lxcs = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'lxc' });
        for (let lxc of lxcs) {
            child_lxcs.push(lxc.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_lxcs) });
        for (let relation of itemRelations) {
            items.push(relation.child);
        }
        const parent = MetricsWrapper_1.MetricsWrapper.getParent(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.PROXMOX, 'lxc');
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcCpuAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'LXC'
                },
            ],
        });
        tabs.push({
            title: 'Memory used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcMemusedAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'LXC'
                },
            ],
        });
        tabs.push({
            title: 'Memory free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcMemfreeAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'LXC'
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcDiskreadAggrMetric().init(), await new ProxmoxMetrics.LxcDiskwriteAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'LXC'
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcNetinAggrMetric().init(), await new ProxmoxMetrics.LxcNetoutAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'LXC'
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxClusterlxctotalsPage = ProxmoxClusterlxctotalsPage;
class ProxmoxClusternodetotalsPage extends ProxmoxClusterTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Node Totals ' + (item !== null ? item.label : this.itemId);
        let child_nodes = [];
        let nodes = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'node' });
        for (let node of nodes) {
            child_nodes.push(node.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_nodes) });
        for (let relation of itemRelations) {
            items.push(relation.child);
        }
        const parent = MetricsWrapper_1.MetricsWrapper.getParent(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.PROXMOX, 'node');
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeCpuCoresAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node',
                },
            ],
        });
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeCpuAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node',
                },
            ],
        });
        tabs.push({
            title: 'Memory used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeMemusedAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        tabs.push({
            title: 'Memory free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeMemfreeAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        tabs.push({
            title: 'Swap used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeSwapusedAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        tabs.push({
            title: 'Swap free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeSwapfreeAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        tabs.push({
            title: 'CPU IO wait',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeIowaitAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeNetinAggrMetric().init(), await new ProxmoxMetrics.NodeNetoutAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        tabs.push({
            title: 'OS disk used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeRootusedAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        tabs.push({
            title: 'OS disk free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeRootfreeAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Node'
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxClusternodetotalsPage = ProxmoxClusternodetotalsPage;
class ProxmoxClusterstoragetotalsPage extends ProxmoxClusterTotalsPage {
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Node Totals ' + (item !== null ? item.label : this.itemId);
        let child_storages = [];
        let storages = await this.objectItemsRepository.findBy({ hw_type: this.hwType, subsystem: 'storage' });
        for (let storage of storages) {
            child_storages.push(storage.item_id);
        }
        let items = [];
        let itemRelations = await this.itemRelationsRepository.findBy({ parent: this.itemId, child: (0, typeorm_1.In)(child_storages) });
        for (let relation of itemRelations) {
            items.push(relation.child);
        }
        const parent = MetricsWrapper_1.MetricsWrapper.getParent(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.PROXMOX, 'storage');
        let tabs = [];
        tabs.push({
            title: 'Free',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.StorageFreeAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Storage'
                },
            ],
        });
        tabs.push({
            title: 'Used',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.StorageUsedAggrMetric().init()],
                    uuids: items,
                    aggregated: true,
                    parent: parent
                        ? {
                            label: parent.label,
                            subsystem: parent.subsystem
                        } : undefined,
                    label: 'Storage'
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxClusterstoragetotalsPage = ProxmoxClusterstoragetotalsPage;
class ProxmoxSingleItemPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.parentSubsystem = 'cluster';
    }
}
exports.ProxmoxSingleItemPage = ProxmoxSingleItemPage;
class ProxmoxVmPage extends ProxmoxSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        const metricsWrapper = await (new MetricsWrapper_1.MetricsWrapper(['maxcpu'], [this.itemId])).getLastDailyValuesFromTable();
        const maxCpuCores = metricsWrapper[this.itemId]?.['maxcpu'];
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.VmCpuCoresMetric().init(),
                        new ProxmoxMetrics.VmMaxCpuCoresMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.VmCpuCoresMetric().init(),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_cpu',
                    interval: 'month',
                    max: maxCpuCores
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.VmCpuCoresMetric().init(),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_cpu',
                    interval: 'year',
                    max: maxCpuCores
                }),
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmCpuMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.VmCpuMetric().init(),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_vm',
                    interval: 'month',
                    max: 100
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.VmCpuMetric().init(),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_vm',
                    interval: 'year',
                    max: 100
                }),
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.VmMemMetric().init(),
                        new ProxmoxMetrics.VmMemFreeMetric().init(),
                        new ProxmoxMetrics.VmMaxmemMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmDiskreadMetric().init(), await new ProxmoxMetrics.VmDiskwriteMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.VmNetinMetric().init(), await new ProxmoxMetrics.VmNetoutMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxVmPage = ProxmoxVmPage;
class ProxmoxNodePage extends ProxmoxSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'node';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Node ' + (item !== null ? item.label : this.itemId);
        const metricsWrapper = await (new MetricsWrapper_1.MetricsWrapper(['maxcpu'], [this.itemId])).getLastDailyValuesFromTable();
        const maxCpuCores = metricsWrapper[this.itemId]?.['maxcpu'];
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.NodeCpuCoresMetric().init(),
                        new ProxmoxMetrics.NodeMaxCpuCoresMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.NodeCpuCoresMetric().init(),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_cpu',
                    interval: 'month',
                    max: maxCpuCores
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.NodeCpuCoresMetric().init(),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_cpu',
                    interval: 'year',
                    max: maxCpuCores
                }),
            ],
            help: cpu_peak_help,
        });
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.NodeCpuMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.NodeCpuMetric().init(),
                    other_metrics: [],
                    frequency: '5m',
                    command: 'predict_cpu',
                    interval: 'month',
                    max: 100
                }),
                await Prediction_1.Prediction.generatePredictionBox({
                    item_id: this.itemId,
                    metric: new ProxmoxMetrics.NodeCpuMetric().init(),
                    other_metrics: [],
                    frequency: '6h',
                    command: 'predict_cpu',
                    interval: 'year',
                    max: 100
                }),
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.NodeMemusedMetric().init(),
                        new ProxmoxMetrics.NodeMemfreeMetric().init(),
                        new ProxmoxMetrics.NodeMemtotalMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Swap',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.NodeSwapusedMetric().init(),
                        new ProxmoxMetrics.NodeSwapfreeMetric().init(),
                        new ProxmoxMetrics.NodeSwaptotalMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'CPU IO wait',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.NodeIowaitMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.NodeNetinMetric().init(),
                        new ProxmoxMetrics.NodeNetoutMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'OS disk',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.NodeRootusedMetric().init(),
                        new ProxmoxMetrics.NodeRootfreeMetric().init(),
                        new ProxmoxMetrics.NodeRoottotalMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let mapping = new Mapping_1.Mapping(this.itemId, Mapping_1.MappingType.storage);
        if (await mapping.isExists()) {
            tabs.push({
                title: 'Volumes',
                content: [await mapping.genTable()],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxNodePage = ProxmoxNodePage;
class ProxmoxLxcPage extends ProxmoxSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'lxc';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'LXC ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcCpuMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcMemMetric().init(), await new ProxmoxMetrics.LxcMaxmemMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcDiskreadMetric().init(), await new ProxmoxMetrics.LxcDiskwriteMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [await new ProxmoxMetrics.LxcNetinMetric().init(), await new ProxmoxMetrics.LxcNetoutMetric().init()],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxLxcPage = ProxmoxLxcPage;
class ProxmoxStoragePage extends ProxmoxSingleItemPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'storage';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Storage ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'Capacity',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new ProxmoxMetrics.StorageUsedMetric().init(),
                        new ProxmoxMetrics.StorageFreeMetric().init(),
                        new ProxmoxMetrics.StorageTotalMetric().init()
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxStoragePage = ProxmoxStoragePage;
class ProxmoxHealthstatusPage extends ProxmoxPage {
    constructor() {
        super(...arguments);
        this.title = 'Health Status';
        this.withItemId = true;
    }
    async getPage() {
        let title = 'Health Status';
        let tabs = [];
        if (this.itemId) {
            let healthStatus = new HealthStatus_1.HealthStatus().getStatusByIds([this.itemId]);
            tabs.push({
                title: 'Health Status',
                content: [await healthStatus],
            });
        }
        else {
            let healthStatus = new HealthStatus_1.HealthStatus().getStatusByParameters("cluster", "virtualization", "Cluster", "proxmox");
            tabs.push({
                title: 'Health Status',
                content: [await healthStatus],
            });
        }
        this.page = new pageResponse_dto_1.Page(title, tabs);
        return this.page;
    }
}
exports.ProxmoxHealthstatusPage = ProxmoxHealthstatusPage;
//# sourceMappingURL=Proxmox.js.map