"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Solaris = void 0;
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const AbstractMenu_1 = require("./AbstractMenu");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const TabFactory_1 = require("./TabFactory");
const class_constants_1 = require("../../../common/constants/class.constants");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const Color_1 = require("../../metrics/Color");
const ACLCheck_1 = require("../../acl/ACLCheck");
const URL_1 = require("./URL");
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const Table_1 = require("./Table");
const Heatmap_1 = require("./Heatmap");
const typeorm_1 = require("typeorm");
const Top_1 = require("./Top");
var Solaris;
(function (Solaris) {
    class Folder extends AbstractMenu_1.AbstractFolder {
        constructor() {
            super(...arguments);
            this.hwType = 'solaris';
        }
    }
    Solaris.Folder = Folder;
    class SPage extends AbstractMenu_1.AbstractPage {
        constructor() {
            super(...arguments);
            this.hwType = 'solaris';
            this.withItemId = true;
            this.rootFolder = RootFolder;
        }
    }
    Solaris.SPage = SPage;
    class RootFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
            menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/ldom-totals', 'ldom-totals'));
            menuItems.push(new menuResponse_dto_1.MenuLink('CDOM TOP', this.hwType + '/cdom-top', 'cdom-tops'));
            menuItems.push(new menuResponse_dto_1.MenuLink('LDOM TOP', this.hwType + '/ldom-top', 'ldom-top'));
            let cdoms = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'cdom'
                }, relations: {
                    parents: true
                }
            });
            let ldoms = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'ldom'
                }, relations: {
                    parents: true
                }
            });
            const os_agent_map = {};
            cdoms.forEach(cdom => {
                os_agent_map[cdom.item_id + "_osagent"] = true;
            });
            ldoms.forEach(ldom => {
                os_agent_map[ldom.item_id + "_osagent"] = true;
            });
            ldoms = ldoms.filter(ldom => !ldom.parents || ldom.parents.length === 0);
            for (let cdom of cdoms) {
                let path = this.hwType + '/cdom?ids=' + cdom.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(cdom.label, path, this.hwType, 'cdom', cdom.item_id, CdomFolder));
            }
            let osagents = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'osagent',
                }, relations: {
                    parents: true
                }
            });
            osagents = osagents.filter(osagent => !os_agent_map[osagent.item_id]);
            for (let osagent of osagents) {
                let path = this.hwType + '/standalone?ids=' + osagent.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(osagent.label, path, this.hwType, 'osagent', osagent.item_id, StandaloneFolder));
            }
            return menuItems;
        }
    }
    Solaris.RootFolder = RootFolder;
    class LdomFolder extends Folder {
        constructor() {
            super(...arguments);
            this.subsystem = 'ldom';
        }
        async getChildren() {
            let menuItems = [];
            let ldoms = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'ldom',
                    parents: {
                        parent: this.itemId
                    }
                }, relations: {
                    parents: true
                }
            });
            menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/' + this.subsystem + '?item_id=' + this.itemId, this.subsystem, this.itemId));
            for (let ldom of ldoms) {
                let path = this.hwType + '/ldom?ids=' + ldom.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(ldom.label, path, this.hwType, 'ldom', ldom.item_id, LdomFolder));
            }
            let zones = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'zone',
                    parents: {
                        parent: this.itemId
                    }
                }
            });
            if (zones.length > 0) {
                menuItems.push(new menuResponse_dto_1.MenuFolder("ZONE", this.hwType + '/zone?ids=' + this.itemId, this.hwType, 'zone', this.itemId, ZoneFolder));
            }
            return menuItems;
        }
    }
    Solaris.LdomFolder = LdomFolder;
    class StandaloneFolder extends Folder {
        constructor() {
            super(...arguments);
            this.subsystem = 'standalone';
        }
        async getChildren() {
            let menuItems = [];
            let ldoms = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'osagent',
                    parents: {
                        parent: this.itemId
                    }
                }, relations: {
                    parents: true
                }
            });
            menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/' + this.subsystem + '?item_id=' + this.itemId, this.subsystem, this.itemId));
            for (let ldom of ldoms) {
                let path = this.hwType + '/standalone?ids=' + ldom.item_id;
                menuItems.push(new menuResponse_dto_1.MenuFolder(ldom.label, path, this.hwType, 'osagent', ldom.item_id, LdomFolder));
            }
            return menuItems;
        }
    }
    Solaris.StandaloneFolder = StandaloneFolder;
    class ZoneFolder extends Folder {
        async getChildren() {
            let menuItems = [];
            let zones = await this.objectItemsRepository.find({
                where: {
                    hw_type: this.hwType,
                    subsystem: 'zone',
                    parents: {
                        parent: this.itemId
                    }
                }, relations: {
                    parents: true
                }
            });
            for (let zone of zones) {
                let path = this.hwType + '/zone?ids=' + zone.item_id;
                menuItems.push(new menuResponse_dto_1.MenuLink(zone.label, this.hwType + '/' + 'zone' + '?item_id=' + zone.item_id, 'zone', zone.item_id));
            }
            return menuItems;
        }
    }
    Solaris.ZoneFolder = ZoneFolder;
    class CdomFolder extends LdomFolder {
        constructor() {
            super(...arguments);
            this.subsystem = 'cdom';
        }
    }
    Solaris.CdomFolder = CdomFolder;
    class CdomPage extends SPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'cdom';
        }
        async getPage() {
            let item = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            });
            let title = 'CDOM ' + (item !== null ? item.label : this.itemId);
            const tabFactoryLdom = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "ldom", [this.itemId]);
            tabFactoryLdom.addTab("CPU", { metric: "cpu_utilization", default: TabFactory_1.MPDefault.LINE });
            tabFactoryLdom.addTab("Memory", { metric: "memory_allocated", default: TabFactory_1.MPDefault.LINE });
            const tabFactoryOsAgent = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "osagent", [this.itemId + "_osagent"]);
            tabFactoryOsAgent.addTab("CPU OS", [
                { metric: "cpu_idle", color: Color_1.Color.pallete.green, default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "cpu_system", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "cpu_user", color: Color_1.Color.pallete.yellow, default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "cpu_wait", color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.AREA_STACKED }
            ]);
            tabFactoryOsAgent.addTab("CPU queue", [
                { metric: "virtual_p", color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
                { metric: "load", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
                { metric: "blocked_p", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.LINE }
            ]);
            tabFactoryOsAgent.addTab("Memory OS", [
                { metric: "mem_free", default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "mem_in_use_client", default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "mem_used", default: TabFactory_1.MPDefault.AREA_STACKED },
            ]);
            tabFactoryOsAgent.addTab("Paging 1", [
                { metric: "pgs1", default: TabFactory_1.MPDefault.LINE },
                { metric: "pgs2", default: TabFactory_1.MPDefault.LINE }
            ]);
            tabFactoryOsAgent.addTab("Paging 2", [
                { metric: "pgs4", default: TabFactory_1.MPDefault.LINE },
            ]);
            let sans = await this.objectItemsRepository.find({
                where: {
                    hw_type: "solaris",
                    subsystem: 'sanldom',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const tabFactorySan = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "sanldom", sans.map(san => san.item_id));
            let osAgentSanItem = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: 'sanldom',
                item_id: this.itemId,
            });
            if (sans.length >= 1) {
                tabFactorySan.addTab("SAN", [
                    { metric: "bytes_read", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "bytes_write", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
                tabFactorySan.addTab("SAN IO", [
                    { metric: "io_read", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "io_write", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
                tabFactorySan.addTab("SAN RESP", [
                    { metric: "resp_time", default: TabFactory_1.MPDefault.LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                ], undefined, undefined, true);
            }
            let nets = await this.objectItemsRepository.find({
                where: {
                    hw_type: "solaris",
                    subsystem: 'netldom',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const tabFactoryNet = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "netldom", nets.map(net => net.item_id));
            if (nets.length >= 1) {
                tabFactoryNet.addTab("NET", [
                    { metric: "read_bytes", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "write_bytes", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
            }
            let pools = await this.objectItemsRepository.find({
                where: {
                    hw_type: "solaris",
                    subsystem: 'pool',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const tabFactoryPool = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "pool", pools.map(pool => pool.item_id));
            tabFactoryPool.addTab("Pool", [
                { metric: "size", color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
                { metric: "used", color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.LINE },
            ]);
            const allTabs = [...tabFactoryLdom.getTabs(), ...tabFactoryOsAgent.getTabs(), ...tabFactorySan.getTabs(), ...tabFactoryNet.getTabs(), ...tabFactoryPool.getTabs()];
            const itemPropertiesFS = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'FS_SOL' } });
            if (itemPropertiesFS) {
                let fsTable = new Table_1.Table()
                    .addHeader({ value: 'Filesystem' })
                    .addHeader({ value: 'Total[GB]', number: true })
                    .addHeader({ value: 'Used[GB]', number: true })
                    .addHeader({ value: 'Available[GB]', number: true })
                    .addHeader({ value: 'Usage[%]', number: true })
                    .addHeader({ value: 'Mounted on' });
                let fsLines = JSON.parse(itemPropertiesFS.property_value);
                for (const itemId in fsLines) {
                    const line = fsLines[itemId];
                    fsTable.addRow([
                        { order: 0, value: line.label },
                        { order: 1, value: line.total ? +line.total.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 2, value: line.used ? +line.used.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 3, value: line.available ? +line.available.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 4, value: line.usage ? +line.usage : "" },
                        { order: 5, value: line.mounted_on },
                    ]);
                }
                fsTable.sortByIndex(4);
                allTabs.push({
                    title: 'FS',
                    content: [
                        fsTable.generateBox()
                    ]
                });
            }
            const itemPropertiesMultiSol = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'PATHSOL' } });
            if (itemPropertiesMultiSol) {
                let SolarisMultiTable = new Table_1.Table()
                    .addHeader({ value: 'Disk name', trim: false })
                    .addHeader({ value: 'Disk Alias', trim: false })
                    .addHeader({ value: 'Path properties', trim: false })
                    .addHeader({ value: 'Path info', trim: false, newLines: true });
                let MultipathLines = JSON.parse(itemPropertiesMultiSol.property_value);
                for (const itemId in MultipathLines) {
                    const line = MultipathLines[itemId];
                    const split = line.path_info.trim().split(" ");
                    if (split.length > 1) {
                        const splitParts = split.map(sp => sp.split(","));
                        let curren_path = "";
                        for (let i = 0; i < splitParts[0].length; i++) {
                            for (let j = 0; j < splitParts.length; j++) {
                                curren_path += j === 0 ? splitParts[j][i] : ", " + splitParts[j][i];
                            }
                            curren_path += "\n";
                        }
                        line.path_info = curren_path.replaceAll(":", ": ").trim();
                    }
                    SolarisMultiTable.addRow([
                        { order: 0, value: line.disk_name },
                        { order: 1, value: line.disk_alias },
                        { order: 2, value: line.path_properties },
                        { order: 3, value: line.path_info },
                    ]);
                }
                allTabs.push({
                    title: 'Multipath',
                    content: [
                        SolarisMultiTable.generateBox()
                    ]
                });
            }
            return new pageResponse_dto_1.Page(title, allTabs);
        }
    }
    Solaris.CdomPage = CdomPage;
    class LdomtotalsPage extends SPage {
        async getPage() {
            let items = await this.objectItemsRepository.findBy({
                hw_type: this.hwType,
                subsystem: (0, typeorm_1.In)(['ldom', 'cdom']),
            });
            let title = 'Totals';
            const tabFactoryLdom = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "ldom", items.map(i => i.item_id));
            tabFactoryLdom.addTab("CPU", { metric: "cpu_utilization", default: TabFactory_1.MPDefault.LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }, undefined, undefined, true);
            tabFactoryLdom.addTab("Memory", { metric: "memory_allocated", default: TabFactory_1.MPDefault.LINE_STACKED, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }, undefined, undefined, true);
            return new pageResponse_dto_1.Page(title, tabFactoryLdom.getTabs());
        }
    }
    Solaris.LdomtotalsPage = LdomtotalsPage;
    class LdomPage extends SPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'ldom';
        }
        async getPage() {
            let item = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            });
            let title = 'LDOM ' + (item !== null ? item.label : this.itemId);
            const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "ldom", [this.itemId]);
            tabFactory.addTab("CPU", { metric: "cpu_utilization", default: TabFactory_1.MPDefault.LINE });
            tabFactory.addTab("Memory", { metric: "memory_allocated", default: TabFactory_1.MPDefault.LINE });
            let osAgentItem = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: 'osagent',
                item_id: this.itemId + '_osagent',
            });
            const tabFactoryOsAgent = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "osagent", [this.itemId + "_osagent"]);
            if (osAgentItem) {
                tabFactoryOsAgent.addTab("CPU OS", [
                    { metric: "cpu_idle", color: Color_1.Color.pallete.green, default: TabFactory_1.MPDefault.AREA_STACKED },
                    { metric: "cpu_system", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.AREA_STACKED },
                    { metric: "cpu_user", color: Color_1.Color.pallete.yellow, default: TabFactory_1.MPDefault.AREA_STACKED },
                    { metric: "cpu_wait", color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.AREA_STACKED }
                ]);
                tabFactoryOsAgent.addTab("CPU queue", [
                    { metric: "virtual_p", color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
                    { metric: "load", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
                    { metric: "blocked_p", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.LINE }
                ]);
                tabFactoryOsAgent.addTab("Memory OS", [
                    { metric: "mem_used", default: TabFactory_1.MPDefault.AREA_STACKED },
                    { metric: "mem_in_use_client", default: TabFactory_1.MPDefault.AREA_STACKED },
                    { metric: "mem_free", default: TabFactory_1.MPDefault.AREA_STACKED },
                ]);
                tabFactoryOsAgent.addTab("Paging 1", [
                    { metric: "pgs1", default: TabFactory_1.MPDefault.LINE },
                    { metric: "pgs2", default: TabFactory_1.MPDefault.LINE }
                ]);
                tabFactoryOsAgent.addTab("Paging 2", [
                    { metric: "pgs4", default: TabFactory_1.MPDefault.LINE },
                ]);
            }
            let sans = await this.objectItemsRepository.find({
                where: {
                    hw_type: "solaris",
                    subsystem: 'sanldom',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const tabFactorySan = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "sanldom", sans.map(san => san.item_id));
            let osAgentSanItem = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: 'sanldom',
                item_id: this.itemId + 'sanldom',
            });
            if (sans.length >= 1) {
                tabFactorySan.addTab("SAN", [
                    { metric: "bytes_read", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "bytes_write", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
                tabFactorySan.addTab("SAN IO", [
                    { metric: "io_read", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "io_write", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
                tabFactorySan.addTab("SAN RESP", [
                    { metric: "resp_time", default: TabFactory_1.MPDefault.LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                ], undefined, undefined, true);
            }
            let nets = await this.objectItemsRepository.find({
                where: {
                    hw_type: "solaris",
                    subsystem: 'netldom',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const tabFactoryNet = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "netldom", nets.map(net => net.item_id));
            if (nets.length >= 1) {
                tabFactoryNet.addTab("NET", [
                    { metric: "read_bytes", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "write_bytes", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
            }
            const allTabs = [...tabFactory.getTabs(), ...tabFactoryOsAgent.getTabs(), ...tabFactorySan.getTabs(), ...tabFactoryNet.getTabs()];
            const itemPropertiesFS = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'FS_SOL' } });
            if (itemPropertiesFS) {
                let fsTable = new Table_1.Table()
                    .addHeader({ value: 'Filesystem' })
                    .addHeader({ value: 'Total[GB]', number: true })
                    .addHeader({ value: 'Used[GB]', number: true })
                    .addHeader({ value: 'Available[GB]', number: true })
                    .addHeader({ value: 'Usage[%]', number: true })
                    .addHeader({ value: 'Mounted on' });
                let fsLines = JSON.parse(itemPropertiesFS.property_value);
                for (const itemId in fsLines) {
                    const line = fsLines[itemId];
                    fsTable.addRow([
                        { order: 0, value: line.label },
                        { order: 1, value: line.total ? +line.total.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 2, value: line.used ? +line.used.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 3, value: line.available ? +line.available.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 4, value: line.usage ? +line.usage : "" },
                        { order: 5, value: line.mounted_on },
                    ]);
                }
                fsTable.sortByIndex(4);
                allTabs.push({
                    title: 'FS',
                    content: [
                        fsTable.generateBox()
                    ]
                });
            }
            const itemPropertiesMultiSol = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'PATHSOL' } });
            if (itemPropertiesMultiSol) {
                let SolarisMultiTable = new Table_1.Table()
                    .addHeader({ value: 'Disk name', trim: false })
                    .addHeader({ value: 'Disk Alias', trim: false })
                    .addHeader({ value: 'Path properties', trim: false })
                    .addHeader({ value: 'Path info', trim: false });
                let MultipathLines = JSON.parse(itemPropertiesMultiSol.property_value);
                for (const itemId in MultipathLines) {
                    const line = MultipathLines[itemId];
                    SolarisMultiTable.addRow([
                        { order: 0, value: line.disk_name },
                        { order: 1, value: line.disk_alias },
                        { order: 2, value: line.path_properties },
                        { order: 3, value: line.path_info },
                    ]);
                }
                allTabs.push({
                    title: 'Multipath',
                    content: [
                        SolarisMultiTable.generateBox()
                    ]
                });
            }
            return new pageResponse_dto_1.Page(title, allTabs);
        }
    }
    Solaris.LdomPage = LdomPage;
    class StandalonePage extends SPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'osagent';
        }
        async getPage() {
            let item = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            });
            let title = 'STANDALONE ' + (item !== null ? item.label : this.itemId);
            const tabFactoryOsAgent = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "osagent", [this.itemId]);
            tabFactoryOsAgent.addTab("CPU OS", [
                { metric: "cpu_idle", color: Color_1.Color.pallete.green, default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "cpu_system", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "cpu_user", color: Color_1.Color.pallete.yellow, default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "cpu_wait", color: Color_1.Color.pallete.red, default: TabFactory_1.MPDefault.AREA_STACKED }
            ]);
            tabFactoryOsAgent.addTab("CPU queue", [
                { metric: "virtual_p", color: Color_1.Color.pallete.black, default: TabFactory_1.MPDefault.LINE },
                { metric: "load", color: Color_1.Color.pallete.redDark, default: TabFactory_1.MPDefault.LINE },
                { metric: "blocked_p", color: Color_1.Color.pallete.blue, default: TabFactory_1.MPDefault.LINE }
            ]);
            tabFactoryOsAgent.addTab("Memory OS", [
                { metric: "mem_used", default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "mem_in_use_client", default: TabFactory_1.MPDefault.AREA_STACKED },
                { metric: "mem_free", default: TabFactory_1.MPDefault.AREA_STACKED },
            ]);
            tabFactoryOsAgent.addTab("Paging 1", [
                { metric: "pgs1", default: TabFactory_1.MPDefault.LINE },
                { metric: "pgs2", default: TabFactory_1.MPDefault.LINE }
            ]);
            tabFactoryOsAgent.addTab("Paging 2", [
                { metric: "pgs4", default: TabFactory_1.MPDefault.LINE },
            ]);
            let sans = await this.objectItemsRepository.find({
                where: {
                    hw_type: "solaris",
                    subsystem: 'sanldom',
                    parents: {
                        parent: this.itemId
                    }
                },
                relations: {
                    parents: true
                }
            });
            const tabFactorySan = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "sanldom", sans.map(san => san.item_id));
            let osAgentSanItem = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: 'sanldom',
                item_id: this.itemId + 'sanldom',
            });
            if (sans.length >= 1) {
                tabFactorySan.addTab("SAN", [
                    { metric: "bytes_read", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "bytes_write", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
                tabFactorySan.addTab("SAN IO", [
                    { metric: "io_read", default: TabFactory_1.MPDefault.READ_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                    { metric: "io_write", default: TabFactory_1.MPDefault.WRITE_LINE_AGGR, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items }
                ], undefined, undefined, true);
                tabFactorySan.addTab("SAN RESP", [
                    { metric: "resp_time", default: TabFactory_1.MPDefault.LINE, aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items },
                ], undefined, undefined, true);
            }
            const allTabs = [...tabFactoryOsAgent.getTabs(), ...tabFactorySan.getTabs()];
            const itemPropertiesFS = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'FS_SOL' } });
            if (itemPropertiesFS) {
                let fsTable = new Table_1.Table()
                    .addHeader({ value: 'Filesystem' })
                    .addHeader({ value: 'Total[GB]', number: true })
                    .addHeader({ value: 'Used[GB]', number: true })
                    .addHeader({ value: 'Available[GB]', number: true })
                    .addHeader({ value: 'Usage[%]', number: true })
                    .addHeader({ value: 'Mounted on' });
                let fsLines = JSON.parse(itemPropertiesFS.property_value);
                for (const itemId in fsLines) {
                    const line = fsLines[itemId];
                    fsTable.addRow([
                        { order: 0, value: line.label },
                        { order: 1, value: line.total ? +line.total.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 2, value: line.used ? +line.used.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 3, value: line.available ? +line.available.replace(/^(\d+\.\d).*$/, '$1') : "" },
                        { order: 4, value: line.usage ? +line.usage : "" },
                        { order: 5, value: line.mounted_on },
                    ]);
                }
                fsTable.sortByIndex(4);
                allTabs.push({
                    title: 'FS',
                    content: [
                        fsTable.generateBox()
                    ]
                });
            }
            return new pageResponse_dto_1.Page(title, allTabs);
        }
    }
    Solaris.StandalonePage = StandalonePage;
    class ZonePage extends SPage {
        constructor() {
            super(...arguments);
            this.subsystem = 'zone';
        }
        async getPage() {
            let item = await this.objectItemsRepository.findOneBy({
                hw_type: this.hwType,
                subsystem: this.subsystem,
                item_id: this.itemId,
            });
            let title = 'ZONE ' + (item !== null ? item.label : this.itemId);
            const tabFactory = new TabFactory_1.TabFactory(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.SOLARIS, "zone", [this.itemId]);
            tabFactory.addTab("CPU", { metric: "cpu_used_in_cores", default: TabFactory_1.MPDefault.LINE });
            tabFactory.addTab("CPU percent", { metric: "cpu_used_in_perc", default: TabFactory_1.MPDefault.LINE });
            tabFactory.addTab("Memory", [
                { metric: "used_memory", default: TabFactory_1.MPDefault.AREA_AGGR },
            ]);
            tabFactory.addTab("Net", { metric: "net_used_in_perc", default: TabFactory_1.MPDefault.LINE });
            const allTabs = tabFactory.getTabs();
            return new pageResponse_dto_1.Page(title, allTabs);
        }
    }
    Solaris.ZonePage = ZonePage;
    class CdomtopPage extends SPage {
        constructor() {
            super(...arguments);
            this.title = 'CDOM TOP';
            this.subsystem = 'cdom';
            this.withItemId = true;
        }
        async getPage() {
            const top = new Top_1.Top();
            top.setSubsystem('solaris', this.subsystem);
            if (this.subsystem === 'ldom') {
                top.setParentSubsystem('cdom');
            }
            top.setAcl(this.request);
            const tabs = await top.genTabs();
            this.page = new pageResponse_dto_1.Page(this.title, tabs);
            return this.page;
        }
    }
    Solaris.CdomtopPage = CdomtopPage;
    class LdomtopPage extends CdomtopPage {
        constructor() {
            super(...arguments);
            this.title = 'LDOM TOP';
            this.subsystem = 'ldom';
        }
    }
    Solaris.LdomtopPage = LdomtopPage;
    class HeatmapPage extends SPage {
        constructor() {
            super(...arguments);
            this.title = 'Heatmap';
        }
        async getPage() {
            let title = 'Heatmap';
            let tabs = [];
            tabs.push({
                title: 'VM',
                content: Object.values(await new Heatmap_1.Heatmap('solaris', 'ldom', true)
                    .virtHeatmapBox(this.request)),
            });
            let vmTable = new Table_1.Table()
                .addHeader({ value: 'LDOM' })
                .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
            let vms = await this.objectItemsRepository.find({
                where: {
                    hw_type: 'solaris',
                    subsystem: 'ldom'
                },
                relations: {
                    heatmaps: true,
                    parents: {
                        _parent: true
                    }
                }
            });
            vms = vms.filter(vm => vm.parents && vm.parents.length > 0);
            vms = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, vms);
            for (const vm of vms) {
                const cpu = vm.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization');
                vmTable.addRow([
                    { order: 0, value: vm ? vm.label : 'undefined', url: URL_1.URL.generate('solaris', 'ldom', vm.item_id) },
                    { order: 1, value: cpu ? cpu.value : null },
                ]);
            }
            vmTable.sortByIndex(1);
            tabs.push({
                title: 'VM table',
                content: [vmTable.generateBox()],
            });
            tabs.push({
                title: 'Server',
                content: Object.values(await new Heatmap_1.Heatmap('solaris', 'cdom', true)
                    .virtHeatmapBox(this.request)),
            });
            let serverTable = new Table_1.Table()
                .addHeader({ value: 'CDOM' })
                .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
            let servers = await this.objectItemsRepository.find({
                where: {
                    hw_type: 'solaris',
                    subsystem: 'cdom'
                },
                relations: {
                    heatmaps: true,
                    parents: {
                        _parent: true
                    }
                }
            });
            servers = await ACLCheck_1.ACLCheck.filterObjectItemsWithRequest(this.request, servers);
            for (const server of servers) {
                const cpu = server.heatmaps.find(heatmap => heatmap.metric === 'cpu_utilization');
                serverTable.addRow([
                    { order: 0, value: server ? server.label : 'undefined', url: URL_1.URL.generate('solaris', 'cdom', server.item_id) },
                    { order: 1, value: cpu ? cpu.value : null },
                ]);
            }
            serverTable.sortByIndex(1);
            tabs.push({
                title: 'Server table',
                content: [serverTable.generateBox()],
            });
            return new pageResponse_dto_1.Page(title, tabs);
        }
    }
    Solaris.HeatmapPage = HeatmapPage;
})(Solaris || (exports.Solaris = Solaris = {}));
//# sourceMappingURL=Solaris.js.map