import { TableBox, TableColumn, TableHeader, TablePivotBox, TableStyleClass, UnitPrefix } from '@xormon/common/pageResponse.dto';
import { Unit } from '@xormon/common/subsystemMetricsResponse.dto';
export declare class PropertiesTableDefinition {
    order: number;
    header: string;
    property: string;
    url?: string;
    number?: boolean;
    unit?: Unit;
    shortcut?: string;
}
export declare class HeaderDefinition {
    value: string;
    number?: boolean;
    unit?: Unit;
    shortcut?: string;
    headerIndex?: number;
    property?: string;
    url?: string;
    styleClass?: TableStyleClass;
    decimal?: number;
    prefix?: UnitPrefix;
    multiplier?: 1024 | 1000;
    sort?: 'desc' | 'asc';
    trim?: boolean;
    newLines?: boolean;
    linuxMpath?: boolean;
    trimLength?: number;
    disable?: {
        search?: boolean;
        sort?: boolean;
    };
}
export declare class Properties {
    headerIndex?: number;
    property: string;
    url?: string;
}
export interface ITableColumn extends Omit<TableColumn, "value" | "url"> {
    value: (string | number) | (string | number)[];
    url?: string | string[];
}
export declare class Table {
    private header;
    private rows;
    private properties;
    private definition;
    private exportProperties;
    private exportIds;
    private exportLabel;
    private exportBox;
    private itemProperties;
    private objectItemsRepository;
    private lastIndex;
    private fixedCols;
    private orientation;
    constructor();
    setFixedCols(fixedCols: number): void;
    setOrientation(orientation: 'vertical' | 'horizontal'): void;
    setBoxExport(fileName?: string): void;
    addHeader(header: HeaderDefinition): this;
    addExport(itemIds: string[], properties: string[], exportLabel?: string): void;
    sortByIndex(i: number, rev?: boolean, isString?: boolean): void;
    sortByValues(i: number, values: string[], rev?: boolean): void;
    propertiesTableSimple(item_id: string, properties: string[]): Promise<void>;
    addPropertiesItems(itemIds: string[]): Promise<void>;
    addPropertiesItem(item_id: string): Promise<void>;
    addRow(data: ITableColumn[]): this;
    addRows(data: ITableColumn[][]): this;
    generate(): {
        header: TableHeader[];
        rows: TableColumn[][];
        orientation: "vertical" | "horizontal";
    };
    generateBox(): TableBox;
    generateBoxWithManagement(): TableBox;
    generatePivotBox(): TablePivotBox;
}
