"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WindowsHeatmapPage = exports.WindowsVmtopPage = exports.WindowsMetrics = exports.WindowsServerPage = exports.WindowsClusterPage = exports.WindowsVmPage = exports.WindowsStoragePage = exports.WindowsVmFolder = exports.WindowsStorageFolder = exports.WindowsServerFolder = exports.WindowsClusterFolder = exports.WindowsDomainFolder = exports.WindowsRootFolder = exports.WindowsPage = exports.WindowsSubsystemFolder = exports.WindowsFolder = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const Color_1 = require("../../metrics/Color");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const class_constants_1 = require("../../../common/constants/class.constants");
const menuResponse_dto_1 = require("../../../common/menuResponse.dto");
const pageResponse_dto_1 = require("../../../common/pageResponse.dto");
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const ItemPropertiesWrapper_1 = require("../../configuration/ItemPropertiesWrapper");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const ACLCheck_1 = require("../../acl/ACLCheck");
const Heatmap_1 = require("./Heatmap");
const Table_1 = require("./Table");
const URL_1 = require("./URL");
const Top_1 = require("./Top");
const typeorm_1 = require("typeorm");
const AbstractMenu_1 = require("./AbstractMenu");
class WindowsFolder extends AbstractMenu_1.AbstractFolder {
    constructor() {
        super(...arguments);
        this.hwType = hwTypeKey_constants_1.HwTypeKey.WINDOWS;
    }
}
exports.WindowsFolder = WindowsFolder;
class WindowsSubsystemFolder extends WindowsFolder {
    constructor(itemId) {
        super();
        if (itemId !== null) {
            this.itemId = itemId;
        }
    }
}
exports.WindowsSubsystemFolder = WindowsSubsystemFolder;
class WindowsPage extends AbstractMenu_1.AbstractPage {
    constructor() {
        super(...arguments);
        this.hwType = hwTypeKey_constants_1.HwTypeKey.WINDOWS;
        this.rootFolder = WindowsRootFolder;
    }
}
exports.WindowsPage = WindowsPage;
class WindowsRootFolder extends WindowsFolder {
    constructor() {
        super(...arguments);
        this.hwType = hwTypeKey_constants_1.HwTypeKey.WINDOWS;
    }
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Heatmap', this.hwType + '/heatmap', 'heatmap'));
        menuItems.push(new menuResponse_dto_1.MenuLink('VM TOP', this.hwType + '/vm-top', 'vm-top'));
        let domains = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'domain'
        });
        domains.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let domain of domains) {
            let path = this.hwType + '/domain?ids=' + domain.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(domain.label, path, this.hwType, 'domain', domain.item_id, WindowsDomainFolder));
        }
        let clusters = await this.objectItemsRepository.findBy({
            hw_type: this.hwType,
            subsystem: 'cluster'
        });
        for (let cluster of clusters) {
            let path = this.hwType + '/cluster?ids=' + cluster.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(cluster.label, path, this.hwType, 'cluster', cluster.item_id, WindowsClusterFolder));
        }
        return menuItems;
    }
}
exports.WindowsRootFolder = WindowsRootFolder;
class WindowsDomainFolder extends WindowsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'server',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        servers.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let server of servers) {
            let path = this.hwType + '/server?ids=' + server.item_id;
            menuItems.push(new menuResponse_dto_1.MenuFolder(server.label, path, this.hwType, 'server', server.item_id, WindowsServerFolder));
        }
        return menuItems;
    }
}
exports.WindowsDomainFolder = WindowsDomainFolder;
class WindowsClusterFolder extends WindowsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/cluster?item_id=' + this.itemId, 'cluster', this.itemId));
        if (this.objectItemsRepository.findOne({
            where: {
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            }, relations: {
                parents: true
            }
        }) !== null) {
            menuItems.push(new menuResponse_dto_1.MenuFolder('VM', this.hwType + '/vm?ids=' + this.itemId, this.hwType, 'vm', this.itemId, WindowsVmFolder));
        }
        return menuItems;
    }
}
exports.WindowsClusterFolder = WindowsClusterFolder;
class WindowsServerFolder extends WindowsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        menuItems.push(new menuResponse_dto_1.MenuLink('Totals', this.hwType + '/server?item_id=' + this.itemId, 'server', this.itemId));
        menuItems.push(new menuResponse_dto_1.MenuFolder('Storage', this.hwType + '/storage?ids=' + this.itemId, this.hwType, 'storage', this.itemId, WindowsStorageFolder));
        const relations = await this.itemRelationsRepository.findBy({ parent: this.itemId });
        let vms = await this.objectItemsRepository.find({
            where: {
                subsystem: 'vm',
                item_id: (0, typeorm_1.In)(relations.map(relation => relation.child))
            }, relations: {
                parents: {
                    _parent: true
                }
            }
        });
        vms = vms.filter(vm => vm.parents.find(relation => relation._parent.subsystem === 'cluster') === undefined);
        if (vms.length >= 1) {
            menuItems.push(new menuResponse_dto_1.MenuFolder('VM', this.hwType + '/vm?ids=' + this.itemId, this.hwType, 'vm', this.itemId, WindowsVmFolder));
        }
        return menuItems;
    }
}
exports.WindowsServerFolder = WindowsServerFolder;
class WindowsStorageFolder extends WindowsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        let storages = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'fixed_disk',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        storages.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let storage of storages) {
            menuItems.push(new menuResponse_dto_1.MenuLink(storage.label, this.hwType + '/storage?item_id=' + storage.item_id, 'storage', storage.item_id));
        }
        return menuItems;
    }
}
exports.WindowsStorageFolder = WindowsStorageFolder;
class WindowsVmFolder extends WindowsSubsystemFolder {
    async getChildren() {
        let menuItems = [];
        const parent = await this.objectItemsRepository.findOneBy({
            item_id: this.itemId
        });
        const relations = await this.itemRelationsRepository.findBy({ parent: this.itemId });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                item_id: (0, typeorm_1.In)(relations.map(relation => relation.child))
            },
            relations: {
                parents: parent.subsystem === 'cluster'
                    ? true
                    : {
                        _parent: true
                    }
            }
        });
        if (parent.subsystem === 'server') {
            vms = vms.filter(vm => vm.parents.find(relation => relation._parent.subsystem === 'cluster') === undefined);
        }
        vms.sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }));
        for (let vm of vms) {
            menuItems.push(new menuResponse_dto_1.MenuLink(vm.label, this.hwType + '/vm?item_id=' + vm.item_id, 'vm', vm.item_id));
        }
        return menuItems;
    }
}
exports.WindowsVmFolder = WindowsVmFolder;
class WindowsStoragePage extends WindowsPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'fixed_disk';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Storage ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'Capacity',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('fixed_disk', 'Size')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.greenDark),
                        new WindowsMetrics.Metric('fixed_disk', 'UsedSpace')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.blueDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('fixed_disk', 'DiskReadBytesPersec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('fixed_disk', 'DiskWriteBytesPerse')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('fixed_disk', 'DiskReadsPersec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('fixed_disk', 'DiskWritesPersec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Latency',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('fixed_disk', 'AvgDisksecPerRead')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('fixed_disk', 'AvgDisksecPerWrite')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        const page = new pageResponse_dto_1.Page(title, tabs);
        return page;
    }
}
exports.WindowsStoragePage = WindowsStoragePage;
class WindowsVmPage extends WindowsPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'vm';
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'VM ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('vm', 'vmCPU')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('vm', 'cpu_usage_perc')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.red)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('vm', 'memory_total')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.greenDark),
                        new WindowsMetrics.Metric('vm', 'memory_used')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Disk',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('vm', 'disk_read_bytes_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('vm', 'disk_write_bytes_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('vm', 'bytes_received_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('vm', 'bytes_sent_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        const page = new pageResponse_dto_1.Page(title, tabs);
        return page;
    }
}
exports.WindowsVmPage = WindowsVmPage;
class WindowsClusterPage extends WindowsPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'cluster';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Cluster ' + (item !== null ? item.label : this.itemId);
        const itemProperties = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'NODE_CONF' } });
        let nodesTable = new Table_1.Table()
            .addHeader({ value: 'Cluster' })
            .addHeader({ value: 'Domain' })
            .addHeader({ value: 'Node' })
            .addHeader({ value: 'RoleOfNode' })
            .addHeader({ value: 'StateOfNode' });
        if (itemProperties) {
            let nodeLines = JSON.parse(itemProperties.property_value);
            for (const itemId in nodeLines) {
                const line = nodeLines[itemId];
                nodesTable.addRow([
                    { order: 0, value: line.Cluster },
                    { order: 1, value: line.Domain },
                    { order: 2, value: line.Node },
                    { order: 3, value: line.RoleOfNode },
                    { order: 4, value: line.StateOfNode },
                ]);
            }
        }
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('cluster', 'cpu_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('cluster', 'vCPU')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.black)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'VM aggregated',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('vm', 'cpu_utilization_cores')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                    ],
                    uuids: vms.map(item => item.item_id),
                    aggregated: true,
                }
            ],
        });
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('cluster', 'used_bytes')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.red),
                        new WindowsMetrics.Metric('cluster', 'cache_bytes')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, Color_1.Color.pallete.blue),
                        new WindowsMetrics.Metric('cluster', 'avail_m_bytes')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 2, Color_1.Color.pallete.green),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'server',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'Nodes',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'cpu_usage')
                            .getAggrGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, pageResponse_constants_1.METRIC_AGGREGATE.items)
                    ],
                    uuids: servers.map(server => server.item_id),
                    aggregated: true,
                },
            ],
        });
        if (itemProperties) {
            tabs.push({
                title: 'Configuration',
                content: [
                    nodesTable.generateBox(),
                ],
            });
        }
        const page = new pageResponse_dto_1.Page(title, tabs);
        return page;
    }
}
exports.WindowsClusterPage = WindowsClusterPage;
class WindowsServerPage extends WindowsPage {
    constructor() {
        super(...arguments);
        this.subsystem = 'server';
        this.withItemId = true;
    }
    async getPage() {
        let item = await this.objectItemsRepository.findOneBy({
            hw_type: this.hwType,
            subsystem: this.subsystem,
            item_id: this.itemId,
        });
        let title = 'Server ' + (item !== null ? item.label : this.itemId);
        let tabs = [];
        tabs.push({
            title: 'CPU',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'cpu_usage')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('server', 'vCPU')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.black)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'CPU%',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'cpu_usage_perc')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'CPU queue',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'ProcessorQueueLength')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Process',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'processes')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('server', 'threads')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 0, Color_1.Color.pallete.blueDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let jobs = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'job',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        const jobCommands = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'command');
        const jobUsers = await ItemPropertiesWrapper_1.ItemPropertiesWrapper.getProperty(jobs.map(job => job.item_id), 'user');
        if (jobs.length >= 1) {
            tabs.push({
                title: 'JOB',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "JOB CPU"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                        graphs: [
                            {
                                metrics: [
                                    new WindowsMetrics.Metric('job', 'time_difference')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                            {
                                metrics: [
                                    new WindowsMetrics.Metric('job', 'time_difference')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                        ],
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.TextBox,
                        styleClass: pageResponse_constants_1.TEXT_STYLE_CLASS.h2,
                        text: "JOB MEM"
                    },
                    {
                        type: pageResponse_constants_1.BOX_TYPE.MultiGraphBox,
                        graphs: [
                            {
                                metrics: [
                                    new WindowsMetrics.Metric('job', 'rss')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.day,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                            {
                                metrics: [
                                    new WindowsMetrics.Metric('job', 'rss')
                                        .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1)
                                ],
                                uuids: jobs.map(job => job.item_id),
                                aggregated: true,
                                interval: pageResponse_constants_1.GRAPH_INTERVAL.week,
                                label: 'PID',
                                properties: [
                                    {
                                        label: "User",
                                        values: jobUsers
                                    },
                                    {
                                        label: "Command",
                                        values: jobCommands
                                    }
                                ]
                            },
                        ],
                    },
                ],
            });
        }
        let licenceTable;
        let itemPropertiesL;
        const itemPropertiesMax = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'LICENCE_MAX' } });
        if (itemPropertiesMax) {
            itemPropertiesL = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'LICENCE_MAX' } });
            licenceTable = new Table_1.Table()
                .addHeader({ value: 'InstanceID' })
                .addHeader({ value: 'OS' })
                .addHeader({ value: 'OS-version' })
                .addHeader({ value: 'LastBoot' })
                .addHeader({ value: 'License' })
                .addHeader({ value: 'Cores' })
                .addHeader({ value: 'Min_MacAddress' })
                .addHeader({ value: 'Max_MacAddress' })
                .addHeader({ value: 'Min_WWPNAddress' })
                .addHeader({ value: 'Max_WWPNAddress' })
                .addHeader({ value: 'Numa_Spanning_Enabled' })
                .addHeader({ value: 'IdentifyingNumber' });
            if (itemPropertiesL) {
                let licenceLines = JSON.parse(itemPropertiesL.property_value);
                for (const itemId in licenceLines) {
                    const line = licenceLines[itemId];
                    licenceTable.addRow([
                        { order: 0, value: line.InstanceID },
                        { order: 1, value: line.OS },
                        { order: 2, value: line.OS_version },
                        { order: 3, value: line.LastBoot },
                        { order: 4, value: line.License },
                        { order: 5, value: line.Cores },
                        { order: 6, value: line.Min_MacAddress },
                        { order: 7, value: line.Max_MacAddress },
                        { order: 8, value: line.Min_WWPNAddress },
                        { order: 9, value: line.Max_WWPNAddress },
                        { order: 10, value: line.Numa_Spanning_Enabled },
                        { order: 11, value: line.IdentifyingNumber },
                    ]);
                }
            }
        }
        else {
            itemPropertiesL = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'LICENCE' } });
            licenceTable = new Table_1.Table()
                .addHeader({ value: 'InstanceID' })
                .addHeader({ value: 'OS' })
                .addHeader({ value: 'OS-version' })
                .addHeader({ value: 'LastBoot' })
                .addHeader({ value: 'License' })
                .addHeader({ value: 'Cores' })
                .addHeader({ value: 'Log.processors' })
                .addHeader({ value: 'Proc.status' })
                .addHeader({ value: 'IdentifyingNumber' });
            if (itemPropertiesL) {
                let licenceLines = JSON.parse(itemPropertiesL.property_value);
                for (const itemId in licenceLines) {
                    const line = licenceLines[itemId];
                    licenceTable.addRow([
                        { order: 0, value: line.InstanceID },
                        { order: 1, value: line.OS },
                        { order: 2, value: line.OS_version },
                        { order: 3, value: line.LastBoot },
                        { order: 4, value: line.License },
                        { order: 5, value: line.Cores },
                        { order: 6, value: line.Log_processors },
                        { order: 7, value: line.Proc_status },
                        { order: 8, value: line.IdentifyingNumber },
                    ]);
                }
            }
        }
        const itemProperties = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'ADAPTERS' } });
        let adaptersTable = new Table_1.Table()
            .addHeader({ value: 'Description' })
            .addHeader({ value: 'DHCPServer' })
            .addHeader({ value: 'DNSHostName' })
            .addHeader({ value: 'IPAddress' })
            .addHeader({ value: 'MACAddress' })
            .addHeader({ value: 'ServiceName' })
            .addHeader({ value: 'SettingID' });
        if (itemProperties) {
            let adaptersLines = JSON.parse(itemProperties.property_value);
            for (const itemId in adaptersLines) {
                const line = adaptersLines[itemId];
                adaptersTable.addRow([
                    { order: 0, value: line.Description },
                    { order: 1, value: line.DHCPServer },
                    { order: 2, value: line.DNSHostName },
                    { order: 3, value: line.IPAddress },
                    { order: 4, value: line.MACAddress },
                    { order: 5, value: line.ServiceName },
                    { order: 6, value: line.SettingID },
                ]);
            }
        }
        const itemPropertiesV = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'VM_CONF' } });
        let vm_confTable = new Table_1.Table()
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'vCPU' })
            .addHeader({ value: 'Reservation' })
            .addHeader({ value: 'Max_Limit' })
            .addHeader({ value: 'Weight' })
            .addHeader({ value: 'Enabled_state_or_Uptime' })
            .addHeader({ value: 'HeartBeat' })
            .addHeader({ value: 'MacAddr' })
            .addHeader({ value: 'DynamicMem' })
            .addHeader({ value: 'TimeSync' })
            .addHeader({ value: 'Vss_Status' });
        if (itemPropertiesV) {
            let vm_confLines = JSON.parse(itemPropertiesV.property_value);
            for (const itemId in vm_confLines) {
                const line = vm_confLines[itemId];
                vm_confTable.addRow([
                    { order: 0, value: line.VM },
                    { order: 1, value: line.vCPU },
                    { order: 2, value: line.Reservation },
                    { order: 3, value: line.Max_Limit },
                    { order: 4, value: line.Weight },
                    { order: 5, value: line.Enabled_state_or_Uptime },
                    { order: 6, value: line.HeartBeat },
                    { order: 6, value: line.MacAddr },
                    { order: 6, value: line.DynamicMem },
                    { order: 6, value: line.TimeSync },
                    { order: 6, value: line.Vss_Status },
                ]);
            }
        }
        const itemPropertiesD = await this.itemPropertiesRepository.findOne({ where: { item_id: this.itemId, property_name: 'VM_VHD_CONF' } });
        let vm_vhd_confTable = new Table_1.Table()
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'Controller' })
            .addHeader({ value: 'Location' })
            .addHeader({ value: 'Path' });
        if (itemPropertiesV) {
            let vm_vhd_confLines = JSON.parse(itemPropertiesD.property_value);
            for (const itemId in vm_vhd_confLines) {
                const line = vm_vhd_confLines[itemId];
                vm_vhd_confTable.addRow([
                    { order: 0, value: line.VM },
                    { order: 1, value: line.Controller },
                    { order: 2, value: line.Location },
                    { order: 3, value: line.Path },
                ]);
            }
        }
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'vm',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        if (vms.length !== 0) {
            tabs.push({
                title: 'VM aggregated',
                content: [
                    {
                        type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                        metrics: [
                            new WindowsMetrics.Metric('vm', 'cpu_utilization_cores')
                                .getAggrGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0)
                        ],
                        uuids: vms.map(item => item.item_id),
                        aggregated: true,
                    }
                ],
            });
        }
        tabs.push({
            title: 'Memory',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'used_bytes')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 0, Color_1.Color.pallete.red),
                        new WindowsMetrics.Metric('server', 'cache_bytes')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 1, Color_1.Color.pallete.blue),
                        new WindowsMetrics.Metric('server', 'avail_m_bytes')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.area, true, 2, Color_1.Color.pallete.green),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Paging 1',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'pages_output_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                        new WindowsMetrics.Metric('server', 'pages_input_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Paging 2',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'disk_tranfers_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark),
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'IO',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'disk_reads_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('server', 'disk_writes_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        tabs.push({
            title: 'Data',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'disk_read_bytes_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('server', 'disk_write_bytes_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        let fixed_disk = await this.objectItemsRepository.find({
            where: {
                hw_type: this.hwType,
                subsystem: 'fixed_disk',
                parents: {
                    parent: this.itemId
                }
            },
            relations: {
                parents: true
            }
        });
        tabs.push({
            title: 'Latency',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('fixed_disk', 'AvgDisksecPerRead')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('fixed_disk', 'AvgDisksecPerWrite')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: fixed_disk.map(fixed_disk => fixed_disk.item_id),
                    aggregated: true,
                },
            ],
        });
        tabs.push({
            title: 'LAN',
            content: [
                {
                    type: pageResponse_constants_1.BOX_TYPE.FourGraphBox,
                    metrics: [
                        new WindowsMetrics.Metric('server', 'bytes_received_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, -1, Color_1.Color.pallete.redDark),
                        new WindowsMetrics.Metric('server', 'bytes_sent_persec')
                            .getGraph(pageResponse_constants_1.CHART_TYPE.line, false, 1, Color_1.Color.pallete.blueDark)
                    ],
                    uuids: [this.itemId],
                    aggregated: false,
                },
            ],
        });
        if (itemProperties && itemPropertiesL) {
            tabs.push({
                title: 'Configuration',
                content: [
                    licenceTable.generateBox(),
                    adaptersTable.generateBox(),
                    vm_confTable.generateBox(),
                    vm_vhd_confTable.generateBox(),
                ],
            });
        }
        const page = new pageResponse_dto_1.Page(title, tabs);
        return page;
    }
}
exports.WindowsServerPage = WindowsServerPage;
class WindowsMetrics {
}
exports.WindowsMetrics = WindowsMetrics;
WindowsMetrics.Metric = class extends pageResponse_dto_1.Metric {
    constructor(subsystem, metric) {
        super();
        this.metric = metric;
        const definition = MetricsWrapper_1.MetricsWrapper.getMetric(class_constants_1.ClassKey.VIRTUALIZATION, hwTypeKey_constants_1.HwTypeKey.WINDOWS, subsystem, metric);
        if (definition) {
            Object.assign(this, definition);
        }
    }
    getGraph(type, stacked, order, color) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.color = color ? color : undefined;
        return this;
    }
    getAggrGraph(type, stacked, order, aggregate) {
        this.type = type;
        this.stacked = stacked ? stacked : false;
        this.order = order ? order : 0;
        this.aggregate = aggregate ? aggregate : pageResponse_constants_1.METRIC_AGGREGATE.items;
        return this;
    }
};
class WindowsVmtopPage extends WindowsPage {
    constructor() {
        super(...arguments);
        this.title = 'VM TOP';
    }
    async getPage() {
        const top = new Top_1.Top();
        top.setSubsystem('windows', 'vm');
        top.setParentSubsystem('server');
        top.setAcl(this.request);
        const tabs = await top.genTabs();
        const page = new pageResponse_dto_1.Page(this.title, tabs);
        return page;
    }
}
exports.WindowsVmtopPage = WindowsVmtopPage;
class WindowsHeatmapPage extends WindowsPage {
    constructor() {
        super(...arguments);
        this.title = 'Heatmap';
    }
    async getPage() {
        let title = 'Heatmap';
        let tabs = [];
        tabs.push({
            title: 'VM',
            content: Object.values(await new Heatmap_1.Heatmap('windows', 'vm', true)
                .virtHeatmapBox(this.request))
        });
        let vmTable = new Table_1.Table()
            .addHeader({ value: 'Server' })
            .addHeader({ value: 'VM' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true })
            .addHeader({ value: 'Memory', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let vms = await this.objectItemsRepository.find({
            where: {
                hw_type: 'windows',
                subsystem: 'vm',
                parents: {
                    _parent: {
                        subsystem: 'server'
                    }
                }
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, vms.map(item => item.item_id));
            vms = vms.filter(item => ids.includes(item.item_id));
        }
        for (const vm of vms) {
            const server = vm.parents.find(parent => parent._parent.subsystem === 'server');
            const cpu = vm.heatmaps.find(heatmap => heatmap.metric === 'cpu_usage_proc');
            const memory = vm.heatmaps.find(heatmap => heatmap.metric === 'memory_proc');
            vmTable.addRow([
                { order: 0, value: server._parent.label },
                { order: 1, value: vm.label, url: URL_1.URL.generate('windows', 'vm', vm.item_id) },
                { order: 2, value: cpu ? cpu.value : null },
                { order: 3, value: memory ? memory.value : null },
            ]);
        }
        vmTable.sortByIndex(2);
        tabs.push({
            title: 'VM table',
            content: [vmTable.generateBox()],
        });
        tabs.push({
            title: 'Server',
            content: Object.values(await new Heatmap_1.Heatmap('windows', 'server', true)
                .virtHeatmapBox(this.request))
        });
        let serverTable = new Table_1.Table()
            .addHeader({ value: 'Server' })
            .addHeader({ value: 'CPU', unit: subsystemMetricsResponse_constants_1.UNIT.percent, number: true });
        let servers = await this.objectItemsRepository.find({
            where: {
                hw_type: 'windows',
                subsystem: 'server'
            },
            relations: {
                heatmaps: true,
                parents: {
                    _parent: true
                }
            }
        });
        if (this.request) {
            const ids = await ACLCheck_1.ACLCheck.filterItemsWithRequest(this.request, servers.map(item => item.item_id));
            servers = servers.filter(item => ids.includes(item.item_id));
        }
        for (const server of servers) {
            const cpu = server.heatmaps.find(heatmap => heatmap.metric === 'cpu_usage_perc');
            serverTable.addRow([
                { order: 0, value: server.label ? server.label : null, url: server ? URL_1.URL.generate('windows', 'server', server.item_id) : undefined },
                { order: 1, value: cpu ? cpu.value : null },
            ]);
        }
        serverTable.sortByIndex(1);
        tabs.push({
            title: 'Server table',
            content: [serverTable.generateBox()],
        });
        const page = new pageResponse_dto_1.Page(title, tabs);
        return page;
    }
}
exports.WindowsHeatmapPage = WindowsHeatmapPage;
//# sourceMappingURL=Windows.js.map