"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Heartbeat = void 0;
const Metric_1 = require("./Metric");
class Heartbeat {
    static apply(timeseries, locf_counter, maxLocf) {
        if (locf_counter === undefined || +locf_counter <= maxLocf) {
            const t_length = timeseries.length;
            if (t_length >= 1) {
                return {
                    value: timeseries[t_length - 1],
                    locf: locf_counter === undefined ? 1 : locf_counter + 1,
                };
            }
            else {
                return {
                    value: null,
                    locf: 0,
                };
            }
        }
        else {
            return {
                value: null,
                locf: 0,
            };
        }
    }
    static apply2(lastValue, currenctLOCF, maxLOCF) {
        if (currenctLOCF === undefined || +currenctLOCF <= maxLOCF) {
            lastValue = lastValue === undefined ? null : Number(lastValue.toPrecision(5));
            return {
                value: [lastValue],
                locf: currenctLOCF === undefined ? 1 : currenctLOCF + 1,
                reset: 0,
            };
        }
        else {
            let array = new Array(currenctLOCF);
            for (let i = 0; i < array.length; i++) {
                array[i] = null;
            }
            return {
                value: array,
                locf: 0,
                reset: currenctLOCF,
            };
        }
    }
    static processValue2(actValue, lastValue, currenctLOCF, maxLOCF) {
        if (lastValue !== null && actValue === null) {
            return Heartbeat.apply2(lastValue, currenctLOCF, maxLOCF);
        }
        else {
            return {
                value: [actValue],
                locf: 0,
                reset: 0,
            };
        }
    }
    static processValue(timeseries, locf_counter, maxLocf, partialArray) {
        let hb = false;
        let length = partialArray.length;
        let lookAhead = maxLocf;
        for (let laIndex = 0; laIndex <= lookAhead; laIndex++) {
            if (laIndex < length) {
                if (partialArray[laIndex].value !== null) {
                    hb = true;
                    break;
                }
            }
        }
        if (hb) {
            let response = Heartbeat.apply(timeseries, locf_counter, maxLocf);
            return {
                value: response.value,
                locf: response.locf,
            };
        }
        else {
            return {
                value: null,
                locf: 0,
            };
        }
    }
    static apply3(lastValue, currenctLOCF, maxLOCF) {
        if (currenctLOCF === undefined || currenctLOCF.length <= maxLOCF) {
            lastValue = lastValue === undefined ? null : +Number(lastValue).toPrecision(5);
            return {
                value: lastValue,
                locf: currenctLOCF,
                reset: [],
            };
        }
        else {
            return {
                value: null,
                locf: [],
                reset: currenctLOCF,
            };
        }
    }
    static processValue3(actValue, lastValue, currenctLOCF, maxLOCF) {
        if (lastValue !== null && actValue === null) {
            return Heartbeat.apply3(lastValue, currenctLOCF, maxLOCF);
        }
        else {
            return {
                value: actValue,
                locf: [],
                reset: [],
            };
        }
    }
    static checkData(separately, metricsItems, interval, min5minutesInterval, itemSpecification) {
        if (separately) {
            for (const itemId in metricsItems) {
                const hwType = itemSpecification[itemId].hw_type ? itemSpecification[itemId].hw_type : undefined;
                const subsystem = itemSpecification[itemId].subsystem ? itemSpecification[itemId].subsystem : undefined;
                const is5min = hwType !== undefined ? min5minutesInterval.includes(hwType) : false;
                for (const metric in metricsItems[itemId]) {
                    let locf = [];
                    let maxLOCF = Metric_1.Metric.getLocf(metric, hwType, subsystem);
                    if (interval > 5) {
                        maxLOCF = 0;
                    }
                    let lastValue = null;
                    for (const time in metricsItems[itemId][metric]) {
                        let actValue = metricsItems[itemId][metric][time];
                        locf.push(time);
                        let response = Heartbeat.processValue3(actValue, lastValue, locf, maxLOCF);
                        lastValue = metricsItems[itemId][metric][time] = response.value;
                        locf = response.locf;
                        if (response.reset.length >= 1) {
                            let lastDataPointTime;
                            for (const newTime of response.reset) {
                                lastDataPointTime = newTime;
                                metricsItems[itemId][metric][newTime] = null;
                            }
                        }
                    }
                    if (locf.length >= 1) {
                        for (const newTime of locf) {
                            metricsItems[itemId][metric][newTime] = null;
                        }
                    }
                }
            }
        }
        else {
        }
        return metricsItems;
    }
}
exports.Heartbeat = Heartbeat;
//# sourceMappingURL=Heartbeat.js.map