"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Metric = void 0;
const bucket_config_1 = require("../../config/bucket.config");
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
const Classes_1 = require("./Classes");
;
;
const MetricsWrapper_1 = require("./MetricsWrapper");
const TableGenerator_1 = require("./TableGenerator");
class Metric {
    constructor(metric, items) {
        this.uuids = [];
        this.itemGroups = {};
        this.initialized = false;
        this.exists = true;
        this.entityManager = (0, dataSource_1.getDataSource)();
        this.metric = metric;
        this.uuids = items ? items : [];
    }
    isMultiStorage() {
        return Object.keys(this.itemGroups).length >= 2;
    }
    isStorage() {
        return this.class !== undefined && this.class === 'storage';
    }
    getHwType() {
        return this.hw_type;
    }
    getSubsystem() {
        return this.subsystem;
    }
    getClass() {
        return this.class;
    }
    async init() {
        this.initialized = true;
        const find = await this.findHwTypeAndSubsystem();
        if (!find) {
            this.exists = false;
            return false;
        }
        if (Object.keys(this.itemGroups).length === 0) {
            try {
                const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_' + this.hw_type + '_' + this.subsystem);
                const columns = repository.metadata.columns.map((column) => {
                    return column.propertyName;
                });
                if (!columns.includes(this.metric)) {
                    this.exists = false;
                    return false;
                }
            }
            catch (e) {
                console.error('No repository for hw type: ' + this.hw_type + ', subsystem: ' + this.subsystem);
                return false;
            }
        }
        return true;
    }
    addItem(item_id) {
        this.uuids.push(item_id);
    }
    async findHwTypeAndSubsystem() {
        let item;
        for (const uuid of this.uuids) {
            item = await this.entityManager.getRepository(objectItem_entity_1.ObjectItem).findOne({
                where: {
                    item_id: uuid
                },
                relations: {
                    hw_type_entity: true
                }
            });
            if (item) {
                this.hw_type = item.hw_type;
                this.subsystem = item.subsystem;
                this.class = item.hw_type_entity.class;
                break;
            }
        }
        if (item) {
            if (this.class == 'storage' || this.class == 'san') {
                const items = await this.entityManager.getRepository(objectItem_entity_1.ObjectItem).findBy({ item_id: (0, typeorm_1.In)(this.uuids) });
                for (const oItem of items) {
                    if (!this.itemGroups[oItem.hw_type]) {
                        this.itemGroups[oItem.hw_type] = [];
                    }
                    this.itemGroups[oItem.hw_type].push(oItem.item_id);
                }
            }
            else {
                if (item.hasAgent && Object.keys(Classes_1.Virtualization.technology.linux.subsystems.server.metrics).includes(this.metric)) {
                    this.hw_type = 'linux';
                    this.subsystem = 'server';
                }
            }
            return true;
        }
        else {
            return false;
        }
    }
    async getDataFromTable(start, end) {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let interval = Metric.getInterval(start, end);
            let min5minutesInterval = await MetricsWrapper_1.MetricsWrapper.getMin5MinutesInterval();
            const maxMetrics = await TableGenerator_1.TableGenerator.getMaxMetrics();
            if (Object.keys(this.itemGroups).length >= 1) {
                let unionQuery = '';
                for (const hwType in this.itemGroups) {
                    try {
                        const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_' + hwType + '_' + this.subsystem);
                        const columns = repository.metadata.columns.map((column) => {
                            return column.propertyName;
                        });
                        if (!columns.includes(this.metric)) {
                            continue;
                        }
                    }
                    catch (e) {
                        console.error('No repository for hw type: ' + hwType + ', subsystem: ' + this.subsystem);
                        continue;
                    }
                    const hasMax = maxMetrics[hwType] && maxMetrics[hwType][this.subsystem] ? true : false;
                    let table;
                    let timeColumn = 'bucket';
                    if (interval === Metric.INTERVAL_1_M) {
                        if (min5minutesInterval.includes(hwType)) {
                            table = Metric.SCHEMA + 'metrics_' + hwType + '_' + this.subsystem + '_5m';
                            interval = Metric.INTERVAL_5_M;
                        }
                        else {
                            table = Metric.SCHEMA + 'metrics_' + hwType + '_' + this.subsystem + '_1m';
                            interval = Metric.INTERVAL_1_M;
                        }
                    }
                    else if (interval === Metric.INTERVAL_5_M) {
                        table = Metric.SCHEMA + 'metrics_' + hwType + '_' + this.subsystem + '_5m';
                    }
                    else if (interval === Metric.INTERVAL_1_H) {
                        table = Metric.SCHEMA + 'metrics_' + hwType + '_' + this.subsystem + '_1h';
                    }
                    else if (interval === Metric.INTERVAL_6_H) {
                        table = Metric.SCHEMA + 'metrics_' + hwType + '_' + this.subsystem + '_6h';
                    }
                    else {
                        table = Metric.SCHEMA + 'metrics_' + hwType + '_' + this.subsystem + '_1d';
                    }
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.itemGroups[hwType].entries()) {
                        if (i === this.itemGroups[hwType].length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let fn = hasMax && maxMetrics[hwType][this.subsystem].includes(this.metric) ? 'max' : 'avg';
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            "(SELECT time_bucket_gapfill('" +
                                interval +
                                ' minute\', ' + timeColumn + ') AS newTime, ' + fn + '(value) as value, item_id FROM ( SELECT ' + timeColumn + ', "' +
                                this.metric +
                                '" as value, item_id FROM ' +
                                table +
                                " WHERE " + timeColumn + " >= '" +
                                new Date(start * 1000).toISOString() +
                                "' AND " + timeColumn + " < '" +
                                new Date(end * 1000).toISOString() +
                                "' AND item_id IN " +
                                dbArray +
                                ') AS query GROUP BY item_id, newTime ORDER BY newTime ASC) ';
                    }
                    else {
                        unionQuery +=
                            "UNION (SELECT time_bucket_gapfill('" +
                                interval +
                                ' minute\', ' + timeColumn + ') AS newTime, ' + fn + '(value) as value, item_id FROM ( SELECT ' + timeColumn + ', "' +
                                this.metric +
                                '" as value, item_id FROM ' +
                                table +
                                " WHERE " + timeColumn + " >= '" +
                                new Date(start * 1000).toISOString() +
                                "' AND " + timeColumn + " < '" +
                                new Date(end * 1000).toISOString() +
                                "' AND item_id IN " +
                                dbArray +
                                ') AS query GROUP BY item_id, newTime ORDER BY newTime ASC) ';
                    }
                }
                let result = [];
                if (process.env['APP_DEBUG'] && process.env['APP_DEBUG'] === 'true' ? true : false) {
                    const started = new Date().getTime();
                    result = await this.entityManager.query(unionQuery);
                    console.debug("[UnionMetricsSelect] : " + (new Date().getTime() - started) / 1000 + " sec.");
                }
                else {
                    result = await this.entityManager.query(unionQuery);
                }
                result.sort((a, b) => (a.newtime > b.newtime ? 1 : -1));
                return result;
            }
            else {
                let table;
                let timeColumn = 'bucket';
                if (interval === Metric.INTERVAL_1_M) {
                    if (min5minutesInterval.includes(this.hw_type)) {
                        table = Metric.SCHEMA + 'metrics_' + this.hw_type + '_' + this.subsystem + '_5m';
                        interval = Metric.INTERVAL_5_M;
                    }
                    else {
                        table = Metric.SCHEMA + 'metrics_' + this.hw_type + '_' + this.subsystem + '_1m';
                        interval = Metric.INTERVAL_1_M;
                    }
                }
                else if (interval === Metric.INTERVAL_5_M) {
                    table = Metric.SCHEMA + 'metrics_' + this.hw_type + '_' + this.subsystem + '_5m';
                }
                else if (interval === Metric.INTERVAL_1_H) {
                    table = Metric.SCHEMA + 'metrics_' + this.hw_type + '_' + this.subsystem + '_1h';
                }
                else if (interval === Metric.INTERVAL_6_H) {
                    table = Metric.SCHEMA + 'metrics_' + this.hw_type + '_' + this.subsystem + '_6h';
                }
                else {
                    table = Metric.SCHEMA + 'metrics_' + this.hw_type + '_' + this.subsystem + '_1d';
                }
                const hasMax = maxMetrics[this.hw_type] && maxMetrics[this.hw_type][this.subsystem] ? true : false;
                let dbArray;
                dbArray = this.uuids.length > 0 ? '(\'\', ' : '(';
                for (const [i, value] of this.uuids.entries()) {
                    if (i === this.uuids.length - 1) {
                        dbArray += "'" + value + "')";
                    }
                    else {
                        dbArray += "'" + value + "',";
                    }
                }
                let fn = hasMax && maxMetrics[this.hw_type][this.subsystem].includes(this.metric) ? 'max' : 'avg';
                const selectQuery = 'SELECT ' + timeColumn + ' as newTime, ' +
                    this.metric +
                    ' as value, item_id FROM ' +
                    table +
                    " WHERE " + timeColumn + " >= '" +
                    new Date(start * 1000).toISOString() +
                    "' AND " + timeColumn + " < '" +
                    new Date(end * 1000).toISOString() +
                    "' AND item_id IN " +
                    dbArray;
                const selectQuery2 = "SELECT time_bucket_gapfill('" +
                    interval +
                    ' minute\', ' + timeColumn + ') AS newTime, ' + fn + '(value) as value, item_id FROM ( SELECT ' + timeColumn + ', "' +
                    this.metric +
                    '" as value, item_id FROM ' +
                    table +
                    " WHERE " + timeColumn + " >= '" +
                    new Date(start * 1000).toISOString() +
                    "' AND " + timeColumn + " < '" +
                    new Date(end * 1000).toISOString() +
                    "' AND item_id IN " +
                    dbArray +
                    ') AS query GROUP BY item_id, newTime ORDER BY newTime ASC';
                if (process.env['APP_DEBUG'] && process.env['APP_DEBUG'] === 'true' ? true : false) {
                    const started = new Date().getTime();
                    const result = await this.entityManager.query(selectQuery2);
                    console.debug("[SingleMetricsSelect] " + table + " : " + (new Date().getTime() - started) / 1000 + " sec.");
                    return result;
                }
                return await this.entityManager.query(selectQuery2);
            }
        }
        else {
            return [];
        }
    }
    async getDataFromBucket(start, end) {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            return await this.getDataFromTable(start, end);
        }
        else {
            return [];
        }
        const now = new Date().getTime();
        const config = (0, bucket_config_1.default)();
        let interval = Metric.getInterval(start, end);
        let table;
        let table_interval;
        if (now - (config.bucketing_interval + 2) * 60 * 60 * 1000 < start * 1000) {
            return await this.getGapFill('avg', 5, new Date(start * 1000).toISOString(), new Date(end * 1000).toISOString());
        }
        else {
            if (now - config.bucket_5m_retention * 30 * 24 * 60 * 60 * 1000 < start * 1000) {
                table_interval = Metric.INTERVAL_5_M;
            }
            else if (now - config.bucket_1h_retention * 30 * 24 * 60 * 60 * 1000 < start * 1000) {
                table_interval = Metric.INTERVAL_1_H;
            }
            else if (now - config.bucket_6h_retention * 30 * 24 * 60 * 60 * 1000 < start * 1000) {
                table_interval = Metric.INTERVAL_6_H;
            }
            else {
                table_interval = Metric.INTERVAL_1_D;
            }
        }
        if (table_interval > interval) {
            interval = table_interval;
        }
        if (interval === Metric.INTERVAL_5_M) {
            table = 'metrics_5m';
        }
        else if (interval === Metric.INTERVAL_1_H) {
            table = 'metrics_1h';
        }
        else if (interval === Metric.INTERVAL_6_H) {
            table = 'metrics_6h';
        }
        else {
            table = 'metrics_1d';
        }
        let dbArray = '(\'\', ';
        for (const [i, value] of this.uuids.entries()) {
            if (i === this.uuids.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        const selectQuery = "SELECT time AS newTime, (data->>'" +
            this.metric +
            "')::numeric as value, item_id FROM " +
            table +
            " WHERE time >= '" +
            new Date(start * 1000).toISOString() +
            "' AND time < '" +
            new Date(end * 1000).toISOString() +
            "' AND item_id IN " +
            dbArray +
            ' GROUP BY item_id, newTime ORDER BY newTime ASC';
        if (now - (config.bucketing_interval + 2) * 60 * 60 * 1000 < end * 1000) {
            const pure_data = this.getGapFill('avg', interval, new Date(now - config.bucketing_interval * 60 * 60 * 1000).toISOString(), new Date(end * 1000).toISOString());
            let data = await this.entityManager.query(selectQuery);
            return data.concat(await pure_data);
        }
        else {
            return await this.entityManager.query(selectQuery);
        }
    }
    async getGapFill(func, interval, start, end) {
        let dbArray = '(\'\', ';
        for (const [i, value] of this.uuids.entries()) {
            if (i === this.uuids.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        const selectQuery = "SELECT time_bucket_gapfill('" +
            interval +
            " minute', time) AS newTime, " +
            func +
            "((data->>'" +
            this.metric +
            "')::numeric) as value, item_id FROM metrics WHERE time >= '" +
            start +
            "' AND time < '" +
            end +
            "' AND item_id IN " +
            dbArray +
            ' GROUP BY item_id, newTime';
        return await this.entityManager.query(selectQuery);
    }
    async getMetricTimeseriesItem(func, interval, start, end, locale = false) {
        const metrics = await this.getGapFill(func, interval, start, end);
        let timeseriesItems = {};
        for (const metric of metrics) {
            if (!timeseriesItems[metric.item_id]) {
                timeseriesItems[metric.item_id] = {};
            }
            if (locale) {
                timeseriesItems[metric.item_id][this.toLocaleDateTime(metric.newtime)] = parseInt(metric.value);
            }
            else {
                timeseriesItems[metric.item_id][new Date(metric.newtime).toISOString()] = parseInt(metric.value);
            }
        }
        return timeseriesItems;
    }
    async getWithOperator(operator, value, start, end) {
        if (!this.initialized) {
            await this.init();
        }
        if (!this.exists) {
            return {};
        }
        const startTime = new Date(start * 1000).toISOString();
        const endTime = new Date(end * 1000).toISOString();
        let dbArray = '(\'\', ';
        for (const [i, value] of this.uuids.entries()) {
            if (i === this.uuids.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        let op;
        let fn;
        switch (operator) {
            case "gt":
                op = ">";
                fn = 'min';
                break;
            case "lt":
                op = "<";
                fn = 'max';
                break;
            case "ge":
                op = ">=";
                fn = 'min';
                break;
            case "le":
                op = "<=";
                fn = 'max';
                break;
            case "eq":
                op = "=";
                fn = 'avg';
                break;
        }
        let items = {};
        let unionQuery = '';
        if (Object.keys(this.itemGroups).length >= 1) {
            for (const hwType in this.itemGroups) {
                const table = await MetricsWrapper_1.MetricsWrapper.getTable(hwType, this.subsystem, start, end);
                const repository = (0, dataSource_1.getDataSource)().getRepository(`metrics.metrics_${hwType}_${this.subsystem}`);
                const columns = repository.metadata.columns.map((column) => {
                    return column.propertyName;
                });
                if (!columns.includes(this.metric)) {
                    continue;
                }
                let dbArray = '(\'\', ';
                for (const [i, value] of this.itemGroups[hwType].entries()) {
                    if (i === this.itemGroups[hwType].length - 1) {
                        dbArray += "'" + value + "')";
                    }
                    else {
                        dbArray += "'" + value + "',";
                    }
                }
                if (unionQuery.length === 0) {
                    unionQuery += '(select distinct(item_id), ' + fn + '("' + this.metric + '") as mValue from ' + table + ' where item_id IN ' + dbArray + ' and bucket >= \'' + startTime + '\' and bucket <= \'' + endTime + '\' group by item_id)';
                }
                else {
                    unionQuery += 'UNION (select distinct(item_id), ' + fn + '("' + this.metric + '") as mValue from ' + table + ' where item_id IN ' + dbArray + ' and bucket >= \'' + startTime + '\' and bucket <= \'' + endTime + '\' group by item_id)';
                }
            }
        }
        else {
            const table = await MetricsWrapper_1.MetricsWrapper.getTable(this.hw_type, this.subsystem, start, end);
            let dbArray = '(\'\', ';
            for (const [i, value] of this.uuids.entries()) {
                if (i === this.uuids.length - 1) {
                    dbArray += "'" + value + "')";
                }
                else {
                    dbArray += "'" + value + "',";
                }
            }
            if (unionQuery.length === 0) {
                unionQuery += '(select distinct(item_id), ' + fn + '("' + this.metric + '") as mValue from ' + table + ' where item_id IN ' + dbArray + ' and bucket >= \'' + startTime + '\' and bucket <= \'' + endTime + '\' group by item_id)';
            }
            else {
                unionQuery += 'UNION (select distinct(item_id), ' + fn + '("' + this.metric + '") as mValue from ' + table + ' where item_id IN ' + dbArray + ' and bucket >= \'' + startTime + '\' and bucket <= \'' + endTime + '\' group by item_id)';
            }
        }
        if (unionQuery === '') {
            return {};
        }
        const fnQuery = "select * from (" + unionQuery + ") as subquery where subquery.mValue " + op + " '" + value + "'";
        const values = await this.entityManager.query(fnQuery);
        for (const value of values) {
            items[value.item_id] = value.mValue;
        }
        return items;
    }
    async getAverage(interval) {
        let dbArray = '(\'\', ';
        for (const [i, value] of this.uuids.entries()) {
            if (i === this.uuids.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        let items = {};
        const selectQuery = "select distinct on (item_id) item_id, time_bucket_gapfill('" +
            interval +
            " minute', time) AS newTime, avg((data->>'" +
            this.metric +
            "')::numeric) as value FROM metrics WHERE item_id in " +
            dbArray +
            " and time >= NOW() - INTERVAL '" +
            interval +
            " MINUTE' and time <= NOW() GROUP BY item_id, newTime";
        const averages = await this.entityManager.query(selectQuery);
        for (const average of averages) {
            items[average.item_id] = average.value;
        }
        return items;
    }
    async getAverageFromTable(interval) {
        if (!this.initialized) {
            await this.init();
        }
        if (!this.exists) {
            return {};
        }
        let items = {};
        let averages;
        if (Object.keys(this.itemGroups).length >= 1) {
            let unionQuery = '';
            for (const hwType in this.itemGroups) {
                const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_' + hwType + '_' + this.subsystem);
                const columns = repository.metadata.columns.map((column) => {
                    return column.propertyName;
                });
                if (!columns.includes(this.metric)) {
                    continue;
                }
                let dbArray = '(\'\', ';
                for (const [i, value] of this.itemGroups[hwType].entries()) {
                    if (i === this.itemGroups[hwType].length - 1) {
                        dbArray += "'" + value + "')";
                    }
                    else {
                        dbArray += "'" + value + "',";
                    }
                }
                if (unionQuery.length === 0) {
                    unionQuery +=
                        "(SELECT item_id, time_bucket('" +
                            interval +
                            ' minute\', bucket) AS interval, last("' +
                            this.metric +
                            '", bucket) FROM ' +
                            Metric.SCHEMA +
                            'metrics_' +
                            hwType +
                            '_' +
                            this.subsystem +
                            "_1h WHERE bucket > now () - INTERVAL '" +
                            (interval + 60) +
                            " minute' GROUP BY item_id, bucket ORDER BY interval DESC) ";
                }
                else {
                    unionQuery +=
                        "UNION (SELECT item_id, time_bucket('" +
                            interval +
                            ' minute\', bucket) AS interval, last("' +
                            this.metric +
                            '", bucket) FROM ' +
                            Metric.SCHEMA +
                            'metrics_' +
                            hwType +
                            '_' +
                            this.subsystem +
                            "_1h WHERE bucket > now () - INTERVAL '" +
                            (interval + 60) +
                            " minute' GROUP BY item_id, bucket ORDER BY interval DESC) ";
                }
            }
            averages = await this.entityManager.query(unionQuery);
        }
        else {
            let dbArray = '(\'\', ';
            for (const [i, value] of this.uuids.entries()) {
                if (i === this.uuids.length - 1) {
                    dbArray += "'" + value + "')";
                }
                else {
                    dbArray += "'" + value + "',";
                }
            }
            const selectQuery = "SELECT item_id, time_bucket('" +
                interval +
                ' minute\', bucket) AS interval, last("' +
                this.metric +
                '", bucket) FROM ' +
                Metric.SCHEMA +
                'metrics_' +
                this.hw_type +
                '_' +
                this.subsystem +
                "_1h WHERE bucket > now () - INTERVAL '" +
                (interval + 60) +
                " minute' GROUP BY item_id, bucket ORDER BY interval DESC;";
            averages = await this.entityManager.query(selectQuery);
        }
        for (const average of averages) {
            items[average.item_id] = average.last;
        }
        return items;
    }
    async getAverageAndMaxFromTable(interval) {
        if (!this.initialized) {
            await this.init();
        }
        if (!this.exists) {
            return {};
        }
        const bucket = interval >= 129600 ? '6h' : '1h';
        let items = {};
        let values;
        if (Object.keys(this.itemGroups).length >= 1) {
            let unionQuery = '';
            for (const hwType in this.itemGroups) {
                const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_' + hwType + '_' + this.subsystem);
                const columns = repository.metadata.columns.map((column) => {
                    return column.propertyName;
                });
                if (!columns.includes(this.metric)) {
                    continue;
                }
                let dbArray = '(\'\', ';
                for (const [i, value] of this.itemGroups[hwType].entries()) {
                    if (i === this.itemGroups[hwType].length - 1) {
                        dbArray += "'" + value + "')";
                    }
                    else {
                        dbArray += "'" + value + "',";
                    }
                }
                if (unionQuery.length === 0) {
                    unionQuery +=
                        '(SELECT item_id, avg("' +
                            this.metric +
                            '") as avg, max("' +
                            this.metric +
                            '") as max FROM ' +
                            Metric.SCHEMA +
                            'metrics_' +
                            hwType +
                            '_' +
                            this.subsystem +
                            '_' +
                            bucket +
                            " WHERE bucket > now () - INTERVAL '" +
                            (interval + 60) +
                            " minute' and item_id in " +
                            dbArray +
                            ' GROUP BY item_id, bucket ORDER BY interval DESC) ';
                }
                else {
                    unionQuery +=
                        'UNION (SELECT item_id, avg("' +
                            this.metric +
                            '") as avg, max("' +
                            this.metric +
                            '") as max FROM ' +
                            Metric.SCHEMA +
                            'metrics_' +
                            hwType +
                            '_' +
                            this.subsystem +
                            '_' +
                            bucket +
                            " WHERE bucket > now () - INTERVAL '" +
                            (interval + 60) +
                            " minute' and item_id in " +
                            dbArray +
                            ' GROUP BY item_id, bucket ORDER BY interval DESC) ';
                }
            }
            values = await this.entityManager.query(unionQuery);
        }
        else {
            let dbArray = '(\'\', ';
            for (const [i, value] of this.uuids.entries()) {
                if (i === this.uuids.length - 1) {
                    dbArray += "'" + value + "')";
                }
                else {
                    dbArray += "'" + value + "',";
                }
            }
            const selectQuery = 'SELECT item_id, avg("' +
                this.metric +
                '") as avg, max("' +
                this.metric +
                '") as max FROM ' +
                Metric.SCHEMA +
                'metrics_' +
                this.hw_type +
                '_' +
                this.subsystem +
                '_' +
                bucket +
                " WHERE bucket > now () - INTERVAL '" +
                (interval + 60) +
                " minute' and item_id in " +
                dbArray +
                ' GROUP BY item_id ORDER BY item_id desc;';
            values = await this.entityManager.query(selectQuery);
        }
        for (const value of values) {
            items[value.item_id] = {
                avg: Math.round(+value.avg * 100) / 100,
                max: Math.round(+value.max * 100) / 100,
            };
        }
        return items;
    }
    async getLastValue() {
        let dbArray = '(\'\', ';
        for (const [i, value] of this.uuids.entries()) {
            if (i === this.uuids.length - 1) {
                dbArray += "'" + value + "')";
            }
            else {
                dbArray += "'" + value + "',";
            }
        }
        let items = {};
        const selectQuery = "SELECT DISTINCT ON (item_id) item_id, time, data->>'" +
            this.metric +
            "' as value FROM metrics WHERE item_id IN " +
            dbArray +
            ' order by item_id, time DESC ';
        const lastValues = await this.entityManager.query(selectQuery);
        for (const lastValue of lastValues) {
            items[lastValue.item_id] = lastValue.value;
        }
        return items;
    }
    async getLastValueFromTable() {
        if (!this.initialized) {
            await this.init();
        }
        if (!this.exists) {
            return {};
        }
        let values;
        let items = {};
        if (Object.keys(this.itemGroups).length >= 1) {
            let unionQuery = '';
            for (const hwType in this.itemGroups) {
                try {
                    const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_' + hwType + '_' + this.subsystem);
                    const columns = repository.metadata.columns.map((column) => {
                        return column.propertyName;
                    });
                    if (!columns.includes(this.metric)) {
                        continue;
                    }
                }
                catch (e) {
                    console.error('No repository for hw type: ' + hwType + ', subsystem: ' + this.subsystem);
                    continue;
                }
                let dbArray = '(\'\', ';
                for (const [i, value] of this.itemGroups[hwType].entries()) {
                    if (i === this.itemGroups[hwType].length - 1) {
                        dbArray += "'" + value + "')";
                    }
                    else {
                        dbArray += "'" + value + "',";
                    }
                }
                if (unionQuery.length === 0) {
                    unionQuery +=
                        '(SELECT distinct on (item_id) item_id, time, ' +
                            this.metric +
                            ' as last FROM ' +
                            Metric.SCHEMA +
                            'metrics_' +
                            hwType +
                            '_' +
                            this.subsystem +
                            " WHERE time > now () - INTERVAL '6 hours' and item_id in " +
                            dbArray +
                            ' ORDER BY item_id, time DESC) ';
                }
                else {
                    unionQuery +=
                        'UNION (SELECT distinct on (item_id) item_id, time, ' +
                            this.metric +
                            ' as last FROM ' +
                            Metric.SCHEMA +
                            'metrics_' +
                            hwType +
                            '_' +
                            this.subsystem +
                            " WHERE time > now () - INTERVAL '6 hours' and item_id in " +
                            dbArray +
                            ' ORDER BY item_id, time DESC) ';
                }
            }
            values = await this.entityManager.query(unionQuery);
        }
        else {
            let dbArray = '(\'\', ';
            for (const [i, value] of this.uuids.entries()) {
                if (i === this.uuids.length - 1) {
                    dbArray += "'" + value + "')";
                }
                else {
                    dbArray += "'" + value + "',";
                }
            }
            const selectQuery = 'SELECT distinct on (item_id) item_id, time, ' +
                this.metric +
                ' as last FROM ' +
                Metric.SCHEMA +
                'metrics_' +
                this.hw_type +
                '_' +
                this.subsystem +
                " WHERE time > now () - INTERVAL '6 hours' and item_id in " +
                dbArray +
                ' ORDER BY item_id, time desc;';
            values = await this.entityManager.query(selectQuery);
        }
        for (const lastValue of values) {
            items[lastValue.item_id] = lastValue.last;
        }
        return items;
    }
    static getInterval(start, end) {
        let interval = Metric.INTERVAL_1_M;
        const length = end - start;
        if (length >= 60 * 60 * 24 * 90) {
            interval = Metric.INTERVAL_1_D;
        }
        else if (length >= 60 * 60 * 24 * 14) {
            interval = Metric.INTERVAL_6_H;
        }
        else if (length >= 60 * 60 * 24 * 1.5) {
            interval = Metric.INTERVAL_1_H;
        }
        else if (length >= 60 * 60 * 6) {
            interval = Metric.INTERVAL_5_M;
        }
        let table_interval;
        const now = new Date().getTime();
        const config = (0, bucket_config_1.default)();
        if (now - (1 * 30 * 24 * 60 * 60 * 1000) < start * 1000) {
            table_interval = Metric.INTERVAL_1_M;
        }
        else {
            if (now - config.bucket_5m_retention * 30 * 24 * 60 * 60 * 1000 < start * 1000) {
                table_interval = Metric.INTERVAL_5_M;
            }
            else if (now - config.bucket_1h_retention * 30 * 24 * 60 * 60 * 1000 < start * 1000) {
                table_interval = Metric.INTERVAL_1_H;
            }
            else if (now - config.bucket_6h_retention * 30 * 24 * 60 * 60 * 1000 < start * 1000) {
                table_interval = Metric.INTERVAL_6_H;
            }
            else {
                table_interval = Metric.INTERVAL_1_D;
            }
        }
        if (table_interval > interval) {
            interval = table_interval;
        }
        return interval;
    }
    toLocaleDateTime(datetime) {
        const tzoffset = new Date(datetime).getTimezoneOffset() * 60000;
        return new Date(datetime - tzoffset).toISOString().slice(0, -1);
    }
    static getLocf(metric, hwtype, subsystem) {
        let reverseExeptions = {
            ecs: {
                node: ['resp_t_r', 'resp_t_w', 'read', 'write', 'data_rate', 'read_tps', 'write_tps', 'tps_rate', 'error_tps'],
                vdc: ['resp_t_r', 'resp_t_w', 'read', 'write', 'data_rate', 'read_tps', 'write_tps', 'tps_rate', 'error_tps'],
                device: ['read', 'write', 'data_rate', 'resp_t_r', 'resp_t_w']
            },
            hcp: {
                tenant: [''],
                namespace: ['']
            }
        };
        let hourExeptions = {
            vmware: {
                datastore: ['disk_capacity', 'disk_used', 'disk_provisioned'],
            },
            hcp: {
                device: ['object_total', 'ingested'],
            }
        };
        if (hwtype && subsystem && reverseExeptions[hwtype]) {
            const metricList = reverseExeptions[hwtype][subsystem] ? reverseExeptions[hwtype][subsystem] : [];
            if (metricList.includes(metric)) {
                return Metric.LOCF;
            }
            else {
                return 15;
            }
        }
        else {
            if (hwtype && subsystem && hourExeptions[hwtype] && hourExeptions[hwtype][subsystem] && hourExeptions[hwtype][subsystem].includes(metric)) {
                return 15;
            }
            else if (metric == "time_difference" || metric == "rss") {
                return 12;
            }
            else {
                return Metric.LOCF;
            }
        }
    }
    static async getMetrics(hwType, subsystem, exclude = false) {
        return MetricsWrapper_1.MetricsWrapper.getMetrics(hwType, subsystem, 'customGroup');
    }
    static getHeatmapLabels(_class, hwType, subsystem) {
        return MetricsWrapper_1.MetricsWrapper.getHeatmapLabels(_class, hwType, subsystem);
    }
    static getMetric(_class, hwType, subsystem, metric) {
        return MetricsWrapper_1.MetricsWrapper.getMetric(_class, hwType, subsystem, metric);
    }
    static async getMetricFindClass(hwType, subsystem, metric) {
        return MetricsWrapper_1.MetricsWrapper.getMetricFindClass(hwType, subsystem, metric);
    }
    static getParent(_class, hwType, subsystem) {
        return MetricsWrapper_1.MetricsWrapper.getParent(_class, hwType, subsystem);
    }
    static getParentInfo(_class, hwType, parent) {
        return MetricsWrapper_1.MetricsWrapper.getParentInfo(_class, hwType, parent);
    }
}
exports.Metric = Metric;
Metric.INTERVAL_1_M = 1;
Metric.INTERVAL_5_M = 5;
Metric.INTERVAL_1_H = 60;
Metric.INTERVAL_6_H = 360;
Metric.INTERVAL_1_D = 1440;
Metric.SCHEMA = 'metrics.';
Metric.LOCF = 2;
//# sourceMappingURL=Metric.js.map