"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsWrapper = void 0;
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const Architecture_1 = require("../architecture/Architecture");
const typeorm_1 = require("typeorm");
const _Class = require("./Classes");
const Classes_1 = require("./Classes");
const class_constants_1 = require("../../common/constants/class.constants");
;
;
const Heartbeat_1 = require("./Heartbeat");
const Metric_1 = require("./Metric");
const subsystemMetricsResponse_constants_1 = require("../../common/constants/subsystemMetricsResponse.constants");
class MetricsWrapper {
    constructor(metrics, items) {
        this.uuids = [];
        this.structure = {};
        this.initialized = false;
        this.exists = true;
        this.entityManager = (0, dataSource_1.getDataSource)();
        this.excluded = {};
        this.debug = false;
        this.metrics = metrics;
        this.uuids = items ? items : [];
    }
    setDebug() {
        this.debug = true;
    }
    debugLog(text) {
        if (this.debug) {
            console.log(`[MetricsDebugger] [${new Date().toISOString()}]: ${text}`);
        }
    }
    isStorage() {
        return this.class !== undefined && this.class === 'storage';
    }
    getClass() {
        return this.class;
    }
    getHwTypes() {
        return Object.keys(this.structure);
    }
    async init() {
        this.initialized = true;
        const find = await this.findItemsProps();
        if (!find) {
            this.exists = false;
            return false;
        }
        for (const hwType in this.structure) {
            for (const subsystem in this.structure[hwType]) {
                try {
                    const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_' + hwType + '_' + subsystem);
                    const columns = repository.metadata.columns.map((column) => {
                        return column.propertyName;
                    });
                    for (const metric of this.metrics) {
                        if (columns.includes(metric) || metric.endsWith("_max")) {
                            if (!this.structure[hwType][subsystem].metrics) {
                                this.structure[hwType][subsystem].metrics = [metric];
                            }
                            else {
                                this.structure[hwType][subsystem].metrics.push(metric);
                            }
                        }
                        else {
                            if (this.excluded[hwType]) {
                                if (!this.excluded[hwType].includes(subsystem)) {
                                    this.excluded[hwType].push(subsystem);
                                }
                            }
                            else {
                                this.excluded[hwType] = [subsystem];
                            }
                        }
                    }
                }
                catch (e) {
                    console.error('No repository for hw type: ' + hwType + ', subsystem: ' + subsystem);
                }
            }
        }
        return true;
    }
    addItem(item_id) {
        this.uuids.push(item_id);
    }
    async findItemsProps() {
        const items = await this.entityManager.getRepository(objectItem_entity_1.ObjectItem).find({
            where: {
                item_id: (0, typeorm_1.In)(this.uuids)
            }, relations: {
                hw_type_entity: true
            }
        });
        if (items.length === 0) {
            return false;
        }
        this.class = items[0].hw_type_entity.class;
        for (const item of items) {
            if (!this.structure) {
                this.structure = {
                    [item.hw_type]: {
                        [item.subsystem]: {
                            'itemIds': [item.item_id]
                        }
                    }
                };
            }
            else {
                if (!this.structure[item.hw_type]) {
                    this.structure[item.hw_type] = {
                        [item.subsystem]: {
                            'itemIds': [item.item_id]
                        }
                    };
                }
                else {
                    if (!this.structure[item.hw_type][item.subsystem]) {
                        this.structure[item.hw_type][item.subsystem] = {
                            'itemIds': [item.item_id]
                        };
                    }
                    else {
                        this.structure[item.hw_type][item.subsystem]['itemIds'].push(item.item_id);
                    }
                }
            }
        }
        return true;
    }
    async getDataFromTable(start, end, separately = true, heartbeat = false, overrideInterval) {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let interval = Metric_1.Metric.getInterval(start, end);
            let unionQuery = '';
            let min5minutesInterval = await MetricsWrapper.getMin5MinutesInterval();
            let itemSpecification = {};
            if (overrideInterval !== undefined) {
                interval = overrideInterval;
            }
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (!this.structure[hwType][subsystem].itemIds || !this.structure[hwType][subsystem].metrics) {
                        continue;
                    }
                    let table;
                    let timeColumn = 'bucket';
                    if (interval === Metric_1.Metric.INTERVAL_1_M) {
                        if (min5minutesInterval.includes(hwType)) {
                            table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_5m';
                            interval = Metric_1.Metric.INTERVAL_5_M;
                        }
                        else {
                            table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_1m';
                            interval = Metric_1.Metric.INTERVAL_1_M;
                        }
                    }
                    else if (interval === Metric_1.Metric.INTERVAL_5_M) {
                        table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_5m';
                    }
                    else if (interval === Metric_1.Metric.INTERVAL_1_H) {
                        table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_1h';
                    }
                    else if (interval === Metric_1.Metric.INTERVAL_6_H) {
                        table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_6h';
                    }
                    else {
                        table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_1d';
                    }
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        itemSpecification[value] = {
                            hw_type: hwType,
                            subsystem: subsystem,
                        };
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let metrics = '';
                    for (const [i, value] of this.structure[hwType][subsystem].metrics.entries()) {
                        if (i === this.structure[hwType][subsystem].metrics.length - 1) {
                            metrics += '"' + value + '" as "' + value + '"';
                        }
                        else {
                            metrics += '"' + value + '" as "' + value + '", ';
                        }
                    }
                    const nonExistMetrics = this.metrics.filter(metric => !this.structure[hwType][subsystem].metrics.includes(metric));
                    for (const [i, value] of nonExistMetrics.entries()) {
                        if (i === 0 && this.structure[hwType][subsystem].metrics.length === 0) {
                            metrics += 'CAST(null AS float4) as "' + value + '"';
                        }
                        else {
                            metrics += ',CAST(null AS float4) as "' + value + '"';
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            '(SELECT ' + timeColumn + ' as newTime, ' +
                                metrics +
                                ', item_id FROM ' +
                                table +
                                " WHERE " + timeColumn + " >= '" +
                                new Date(start * 1000).toISOString() +
                                "' AND " + timeColumn + " < '" +
                                new Date(end * 1000).toISOString() +
                                "' AND item_id IN " +
                                dbArray +
                                ')';
                    }
                    else {
                        unionQuery +=
                            'UNION (SELECT ' + timeColumn + ' as newTime, ' +
                                metrics +
                                ', item_id FROM ' +
                                table +
                                " WHERE " + timeColumn + " >= '" +
                                new Date(start * 1000).toISOString() +
                                "' AND " + timeColumn + " < '" +
                                new Date(end * 1000).toISOString() +
                                "' AND item_id IN " +
                                dbArray +
                                ')';
                    }
                }
            }
            let startTime = Date.now();
            const query = await this.entityManager.query(unionQuery);
            this.debugLog(`Query lasted: ${(Date.now() - startTime) / 1000} secs.`);
            startTime = Date.now();
            let timeObject = {};
            const startTimestamp = Math.floor(start - (start % (60 * interval)));
            for (let timestamp = startTimestamp; timestamp < end; timestamp += (interval * 60)) {
                timeObject[new Date(timestamp * 1000).toISOString()] = null;
            }
            let metricsItems = {};
            if (separately) {
                for (const queryItem of query) {
                    if (queryItem['item_id'] === '') {
                        continue;
                    }
                    const time = new Date(queryItem['newtime']).toISOString();
                    if (!metricsItems[queryItem['item_id']]) {
                        metricsItems[queryItem['item_id']] = {};
                    }
                    for (const queryItemMetric in queryItem) {
                        if (!['item_id', 'newtime'].includes(queryItemMetric)) {
                            if (!metricsItems[queryItem['item_id']][queryItemMetric]) {
                                metricsItems[queryItem['item_id']][queryItemMetric] = Object.assign({}, timeObject);
                            }
                            metricsItems[queryItem['item_id']][queryItemMetric][time] = queryItem[queryItemMetric];
                        }
                    }
                }
            }
            else {
                metricsItems = {};
                for (const queryItem of query) {
                    if (queryItem['item_id'] === '') {
                        continue;
                    }
                    let timePoint = {};
                    const time = new Date(queryItem['newtime']).toISOString();
                    for (const queryItemMetric in queryItem) {
                        if (!['item_id', 'newtime'].includes(queryItemMetric)) {
                            timePoint[queryItemMetric] = +queryItem[queryItemMetric];
                        }
                    }
                    if (!metricsItems[queryItem['item_id']]) {
                        metricsItems[queryItem['item_id']] = {
                            [time]: timePoint
                        };
                    }
                    else {
                        metricsItems[queryItem['item_id']][time] = timePoint;
                    }
                }
            }
            this.debugLog(`Structuring lasted: ${(Date.now() - startTime) / 1000} secs.`);
            const startTimestampISO = new Date(startTimestamp * 1000).toISOString();
            for (const itemId in metricsItems) {
                if (separately) {
                    for (const metric in metricsItems[itemId]) {
                        if (metricsItems[itemId][metric][startTimestampISO] === null) {
                            delete metricsItems[itemId][metric][startTimestampISO];
                        }
                    }
                }
                else {
                    if (metricsItems[itemId][startTimestampISO] === null) {
                        delete metricsItems[itemId][startTimestampISO];
                    }
                }
            }
            if (heartbeat) {
                startTime = Date.now();
                Heartbeat_1.Heartbeat.checkData(separately, metricsItems, interval, min5minutesInterval, itemSpecification);
                this.debugLog(`Heartbeat lasted: ${(Date.now() - startTime) / 1000} secs.`);
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    async getLastHourValueFromTable() {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let unionQuery = '';
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (!this.structure[hwType][subsystem].itemIds || !this.structure[hwType][subsystem].metrics) {
                        continue;
                    }
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let metrics = '';
                    for (const [i, value] of this.structure[hwType][subsystem].metrics.entries()) {
                        if (i === this.structure[hwType][subsystem].metrics.length - 1) {
                            metrics += 'avg("' + value + '") as "' + value + '"';
                        }
                        else {
                            metrics += 'avg("' + value + '") as "' + value + '", ';
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            '(SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_5m WHERE bucket > now () - INTERVAL '1 hour' and item_id in " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                    else {
                        unionQuery +=
                            'UNION (SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_5m WHERE bucket > now () - INTERVAL '1 hour' and item_id in " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                }
            }
            const query = await this.entityManager.query(unionQuery);
            let metricsItems = {};
            for (const queryItem of query) {
                if (!metricsItems[queryItem['item_id']]) {
                    metricsItems[queryItem['item_id']] = {};
                }
                for (const queryItemMetric in queryItem) {
                    if (!['item_id', 'time', 'bucket'].includes(queryItemMetric)) {
                        if (!metricsItems[queryItem['item_id']][queryItemMetric]) {
                            metricsItems[queryItem['item_id']][queryItemMetric] = queryItem[queryItemMetric];
                        }
                    }
                }
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    async getAvgAndMax(interval, bucket) {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            bucket = bucket === undefined
                ? interval >= 129600 ? '1d' : '1h'
                : bucket;
            const days = interval / (60 * 24);
            let unionQuery = '';
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (!this.structure[hwType][subsystem].itemIds || !this.structure[hwType][subsystem].metrics) {
                        continue;
                    }
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let metrics = '';
                    for (const [i, value] of this.structure[hwType][subsystem].metrics.entries()) {
                        if (i === this.structure[hwType][subsystem].metrics.length - 1) {
                            metrics += 'avg("' + value + '") as "' + value + '_avg", ';
                            metrics += 'max("' + value + '") as "' + value + '_max"';
                        }
                        else {
                            metrics += 'avg("' + value + '") as "' + value + '_avg", ';
                            metrics += 'max("' + value + '") as "' + value + '_max", ';
                        }
                    }
                    const nonExistMetrics = this.metrics.filter(metric => !this.structure[hwType][subsystem].metrics.includes(metric));
                    for (const [i, value] of nonExistMetrics.entries()) {
                        if (i === 0 && this.structure[hwType][subsystem].metrics.length === 0) {
                            metrics += 'CAST(null AS float4) as "' + value + '_avg"';
                            metrics += ',CAST(null AS float4) as "' + value + '_max"';
                        }
                        else {
                            metrics += ',CAST(null AS float4) as "' + value + '_avg"';
                            metrics += ',CAST(null AS float4) as "' + value + '_max"';
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            '(SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_" + bucket + " WHERE bucket > now () - INTERVAL '" + days + " days' and item_id in " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                    else {
                        unionQuery +=
                            'UNION (SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_" + bucket + " WHERE bucket > now () - INTERVAL '" + days + " days' and item_id in " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                }
            }
            const query = await this.entityManager.query(unionQuery);
            let metricsItems = {};
            for (const queryItem of query) {
                if (!metricsItems[queryItem['item_id']]) {
                    metricsItems[queryItem['item_id']] = {};
                }
                for (const queryItemMetric in queryItem) {
                    if (!['item_id', 'time', 'bucket'].includes(queryItemMetric)) {
                        const realMetric = queryItemMetric.replace("_avg", "").replace("_max", "");
                        const type = queryItemMetric.replace(realMetric + "_", "");
                        if (!metricsItems[queryItem['item_id']][realMetric]) {
                            metricsItems[queryItem['item_id']][realMetric] = {
                                avg: type === 'avg' ? queryItem[queryItemMetric] : null,
                                max: type === 'max' ? queryItem[queryItemMetric] : null,
                            };
                        }
                        else {
                            metricsItems[queryItem['item_id']][realMetric][type] = queryItem[queryItemMetric];
                        }
                    }
                }
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    async getAvgAndMaxByTime(start, end) {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let unionQuery = '';
            const startTs = new Date(start * 1000).toISOString();
            const endTs = new Date(end * 1000).toISOString();
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (!this.structure[hwType][subsystem].itemIds || !this.structure[hwType][subsystem].metrics) {
                        continue;
                    }
                    const table = await MetricsWrapper.getTable(hwType, subsystem, start, end);
                    const days = Math.ceil((end - start) / 60 / 60 / 24);
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let metrics = '';
                    for (const [i, value] of this.structure[hwType][subsystem].metrics.entries()) {
                        if (i === this.structure[hwType][subsystem].metrics.length - 1) {
                            metrics += 'avg("' + value + '") as "' + value + '_q_avg", ';
                            metrics += 'max("' + value + '") as "' + value + '_q_max"';
                        }
                        else {
                            metrics += 'avg("' + value + '") as "' + value + '_q_avg", ';
                            metrics += 'max("' + value + '") as "' + value + '_q_max", ';
                        }
                    }
                    const nonExistMetrics = this.metrics.filter(metric => !this.structure[hwType][subsystem].metrics.includes(metric));
                    for (const [i, value] of nonExistMetrics.entries()) {
                        if (i === 0 && this.structure[hwType][subsystem].metrics.length === 0) {
                            metrics += 'CAST(null AS float4) as "' + value + '_q_avg"';
                            metrics += ',CAST(null AS float4) as "' + value + '_q_max"';
                        }
                        else {
                            metrics += ',CAST(null AS float4) as "' + value + '_q_avg"';
                            metrics += ',CAST(null AS float4) as "' + value + '_q_max"';
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            '(SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                table +
                                " WHERE bucket > '" + startTs + "' AND bucket < '" + endTs + "' AND item_id IN " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                    else {
                        unionQuery +=
                            'UNION (SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                table +
                                " WHERE bucket > '" + startTs + "' AND bucket < '" + endTs + "' AND item_id IN " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                }
            }
            const query = await this.entityManager.query(unionQuery);
            let metricsItems = {};
            for (const queryItem of query) {
                if (!metricsItems[queryItem['item_id']]) {
                    metricsItems[queryItem['item_id']] = {};
                }
                for (const queryItemMetric in queryItem) {
                    if (!['item_id', 'time', 'bucket'].includes(queryItemMetric)) {
                        const realMetric = queryItemMetric.replace("_q_avg", "").replace("_q_max", "");
                        const type = queryItemMetric.replace(realMetric + "_q_", "");
                        if (!metricsItems[queryItem['item_id']][realMetric]) {
                            metricsItems[queryItem['item_id']][realMetric] = {
                                avg: type === 'avg' ? queryItem[queryItemMetric] : null,
                                max: type === 'max' ? queryItem[queryItemMetric] : null,
                            };
                        }
                        else {
                            metricsItems[queryItem['item_id']][realMetric][type] = queryItem[queryItemMetric];
                        }
                    }
                }
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    async getSumByTime(start, end) {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let unionQuery = '';
            const startTs = new Date(start * 1000).toISOString();
            const endTs = new Date(end * 1000).toISOString();
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (!this.structure[hwType][subsystem].itemIds || !this.structure[hwType][subsystem].metrics) {
                        continue;
                    }
                    const table = await MetricsWrapper.getTable(hwType, subsystem, start, end);
                    const days = Math.ceil((end - start) / 60 / 60 / 24);
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let metrics = '';
                    for (const [i, value] of this.structure[hwType][subsystem].metrics.entries()) {
                        if (i === this.structure[hwType][subsystem].metrics.length - 1) {
                            metrics += 'sum("' + value + '") as "' + value + '_sum"';
                        }
                        else {
                            metrics += 'sum("' + value + '") as "' + value + '_sum", ';
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            '(SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                table +
                                " WHERE bucket > '" + startTs + "' AND bucket < '" + endTs + "' AND item_id IN " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                    else {
                        unionQuery +=
                            'UNION (SELECT distinct on (item_id) item_id, ' +
                                metrics +
                                ' FROM ' +
                                table +
                                " WHERE bucket > '" + startTs + "' AND bucket < '" + endTs + "' AND item_id IN " +
                                dbArray +
                                ' GROUP BY item_id ORDER BY item_id DESC) ';
                    }
                }
            }
            const query = await this.entityManager.query(unionQuery);
            let metricsItems = {};
            for (const queryItem of query) {
                if (!metricsItems[queryItem['item_id']]) {
                    metricsItems[queryItem['item_id']] = {};
                }
                for (const queryItemMetric in queryItem) {
                    if (!['item_id', 'time', 'bucket'].includes(queryItemMetric)) {
                        const realMetric = queryItemMetric.replace("_sum", "");
                        metricsItems[queryItem['item_id']][realMetric] = queryItem[queryItemMetric];
                    }
                }
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    async getLastDailyValuesFromTable() {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let unionQuery = '';
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (this.excluded[hwType] && this.excluded[hwType].includes(subsystem)) {
                        continue;
                    }
                    if (!this.structure[hwType][subsystem].itemIds || !this.structure[hwType][subsystem].metrics) {
                        continue;
                    }
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let metrics = '';
                    for (const [i, value] of this.structure[hwType][subsystem].metrics.entries()) {
                        if (i === this.structure[hwType][subsystem].metrics.length - 1) {
                            metrics += 'last("' + value + '", bucket) as "' + value + '"';
                        }
                        else {
                            metrics += 'last("' + value + '", bucket) as "' + value + '", ';
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            '(SELECT item_id, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_1d WHERE item_id in " +
                                dbArray +
                                ' GROUP BY item_id) ';
                    }
                    else {
                        unionQuery +=
                            'UNION (SELECT item_id, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_1d WHERE item_id in " +
                                dbArray +
                                ' GROUP BY item_id) ';
                    }
                }
            }
            const query = await this.entityManager.query(unionQuery);
            let metricsItems = {};
            for (const queryItem of query) {
                if (!metricsItems[queryItem['item_id']]) {
                    metricsItems[queryItem['item_id']] = {};
                }
                for (const queryItemMetric in queryItem) {
                    if (!['item_id'].includes(queryItemMetric)) {
                        if (!metricsItems[queryItem['item_id']][queryItemMetric]) {
                            if (queryItemMetric === 'bucket') {
                                metricsItems[queryItem['item_id']][queryItemMetric] = new Date(queryItem[queryItemMetric]).getTime() / 1000;
                            }
                            else {
                                metricsItems[queryItem['item_id']][queryItemMetric] = queryItem[queryItemMetric];
                            }
                        }
                    }
                }
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    async getLastValuesFromTable() {
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let unionQuery = '';
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (this.excluded[hwType] && this.excluded[hwType].includes(subsystem)) {
                        continue;
                    }
                    if (!this.structure[hwType][subsystem].itemIds || !this.structure[hwType][subsystem].metrics) {
                        continue;
                    }
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    let metrics = '';
                    for (const [i, value] of this.structure[hwType][subsystem].metrics.entries()) {
                        if (i === this.structure[hwType][subsystem].metrics.length - 1) {
                            metrics += '"' + value + '" as "' + value + '"';
                        }
                        else {
                            metrics += '"' + value + '" as "' + value + '", ';
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            '(SELECT distinct on (item_id) item_id, bucket, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_5m WHERE bucket > now () - INTERVAL '6 hours' and item_id in " +
                                dbArray +
                                ' ORDER BY item_id, bucket DESC) ';
                    }
                    else {
                        unionQuery +=
                            'UNION (SELECT distinct on (item_id) item_id, bucket, ' +
                                metrics +
                                ' FROM ' +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem +
                                "_5m WHERE bucket > now () - INTERVAL '6 hours' and item_id in " +
                                dbArray +
                                ' ORDER BY item_id, bucket DESC) ';
                    }
                }
            }
            const query = await this.entityManager.query(unionQuery);
            let metricsItems = {};
            for (const queryItem of query) {
                if (!metricsItems[queryItem['item_id']]) {
                    metricsItems[queryItem['item_id']] = {};
                }
                for (const queryItemMetric in queryItem) {
                    if (!['item_id'].includes(queryItemMetric)) {
                        if (!metricsItems[queryItem['item_id']][queryItemMetric]) {
                            if (queryItemMetric === 'bucket') {
                                metricsItems[queryItem['item_id']][queryItemMetric] = new Date(queryItem[queryItemMetric]).getTime() / 1000;
                            }
                            else {
                                metricsItems[queryItem['item_id']][queryItemMetric] = queryItem[queryItemMetric];
                            }
                        }
                    }
                }
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    async getLastTimestampFromTable(chosenAggregate) {
        let dateType = chosenAggregate ? 'bucket' : 'time';
        let tableType = chosenAggregate ? `_${chosenAggregate}` : '';
        if (!this.initialized) {
            await this.init();
        }
        if (this.exists) {
            let unionQuery = '';
            for (const hwType in this.structure) {
                for (const subsystem in this.structure[hwType]) {
                    if (!this.structure[hwType][subsystem].itemIds) {
                        continue;
                    }
                    try {
                        const repository = (0, dataSource_1.getDataSource)().getRepository('metrics.metrics_' + hwType + '_' + subsystem);
                        if (!repository.metadata.columns) {
                            continue;
                        }
                    }
                    catch (e) {
                        continue;
                    }
                    let dbArray = '(\'\', ';
                    for (const [i, value] of this.structure[hwType][subsystem].itemIds.entries()) {
                        if (i === this.structure[hwType][subsystem].itemIds.length - 1) {
                            dbArray += "'" + value + "')";
                        }
                        else {
                            dbArray += "'" + value + "',";
                        }
                    }
                    if (unionQuery.length === 0) {
                        unionQuery +=
                            `(SELECT item_id, last(${dateType}, ${dateType}) as time FROM ` +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem + tableType +
                                " WHERE item_id IN " +
                                dbArray +
                                " group by item_id) ";
                    }
                    else {
                        unionQuery +=
                            `UNION (SELECT item_id, last(${dateType}, ${dateType}) as time  FROM ` +
                                Metric_1.Metric.SCHEMA +
                                'metrics_' +
                                hwType +
                                '_' +
                                subsystem + tableType +
                                " WHERE item_id IN " +
                                dbArray +
                                " group by item_id) ";
                    }
                }
            }
            const query = await this.entityManager.query(unionQuery);
            let metricsItems = {};
            for (const queryItem of query) {
                if (!metricsItems[queryItem['item_id']]) {
                    metricsItems[queryItem['item_id']] = queryItem['time'];
                }
            }
            return metricsItems;
        }
        else {
            return {};
        }
    }
    static getMin5MinutesIntervalFromDefinitions() {
        let min5minutesHwTypes = [];
        min5minutesHwTypes = min5minutesHwTypes.concat(Object.keys(Classes_1.Storage.technology));
        min5minutesHwTypes = min5minutesHwTypes.concat(Object.keys(Classes_1.San.technology));
        min5minutesHwTypes = min5minutesHwTypes.concat(Object.keys(Classes_1.Database.technology));
        min5minutesHwTypes = min5minutesHwTypes.concat(Object.keys(Classes_1.Lan.technology));
        return min5minutesHwTypes;
    }
    static async getMin5MinutesInterval() {
        let min5minutesHwTypes = [];
        if ((0, dataSource_1.getDataSource)() === undefined) {
            return this.getMin5MinutesIntervalFromDefinitions();
        }
        const min5MinutesHwTypes = await Architecture_1.Architecture.getHwTypes(false, ['storage', 'san', 'database', 'lan']);
        min5minutesHwTypes.push(...min5MinutesHwTypes.map(hwType => hwType.hw_type));
        return min5minutesHwTypes;
    }
    static async getTable(hwType, subsystem, start, end) {
        let interval = Metric_1.Metric.getInterval(start, end);
        const min5minutesInterval = await MetricsWrapper.getMin5MinutesInterval();
        let table;
        if (interval === Metric_1.Metric.INTERVAL_1_M) {
            if (min5minutesInterval.includes(hwType)) {
                table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_5m';
                interval = Metric_1.Metric.INTERVAL_5_M;
            }
            else {
                table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_1m';
                interval = Metric_1.Metric.INTERVAL_1_M;
            }
        }
        else if (interval === Metric_1.Metric.INTERVAL_5_M) {
            table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_5m';
        }
        else if (interval === Metric_1.Metric.INTERVAL_1_H) {
            table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_1h';
        }
        else if (interval === Metric_1.Metric.INTERVAL_6_H) {
            table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_6h';
        }
        else {
            table = Metric_1.Metric.SCHEMA + 'metrics_' + hwType + '_' + subsystem + '_1d';
        }
        return table;
    }
    static async getMetrics(hwType, subsystem, exclude = undefined) {
        if (Array.isArray(hwType)) {
            let metrics = [];
            for (const hwT of hwType) {
                const hMetrics = await MetricsWrapper.getMetrics(hwT, subsystem, exclude);
                if (hMetrics === undefined) {
                    continue;
                }
                if (metrics.length === 0) {
                    metrics = hMetrics;
                    continue;
                }
                else {
                    for (const hMetric of hMetrics) {
                        if (metrics.find(metric => metric.metric === hMetric.metric) === undefined) {
                            metrics.push(hMetric);
                        }
                    }
                }
            }
            return metrics;
        }
        else {
            let hwTypeDefinition;
            const classes = await Architecture_1.Architecture.getClasses();
            let classLabel;
            for (const _class of classes) {
                hwTypeDefinition = this.getHwTypeDefinition(_class.class, hwType);
                if (hwTypeDefinition) {
                    classLabel = _class.class;
                    break;
                }
            }
            if (hwTypeDefinition !== undefined && classLabel !== undefined) {
                const subsystemDefinition = hwTypeDefinition['subsystems'] ? hwTypeDefinition['subsystems'][subsystem] : undefined;
                if (subsystemDefinition) {
                    let metricsArray = [];
                    if (subsystemDefinition['metrics']) {
                        const metrics = subsystemDefinition['metrics'];
                        let top = subsystemDefinition['top']
                            ? subsystemDefinition['top']
                            : {};
                        for (const index in metrics) {
                            if (exclude && subsystemDefinition['excluded'] && ((subsystemDefinition['excluded'][exclude] && subsystemDefinition['excluded'][exclude].includes(metrics[index]['metric'])) || (subsystemDefinition['excluded']['all'] && subsystemDefinition['excluded']['all'].includes(metrics[index]['metric'])))) {
                                continue;
                            }
                            if (metrics[index]) {
                                metricsArray.push({
                                    ...metrics[index],
                                    longLabel: 'longLabel' in metrics[index]
                                        ? metrics[index]['longLabel']
                                        : metrics[index]['label'],
                                    top: this.findTopMetric(top, metrics[index]['metric']),
                                    defaultPrefix: metrics[index]['defaultPrefix'] !== undefined
                                        ? metrics[index]['defaultPrefix']
                                        : null,
                                    defaultPrecision: metrics[index]['defaultPrecision'] !== undefined
                                        ? metrics[index]['defaultPrecision']
                                        : 0
                                });
                            }
                        }
                    }
                    if ([class_constants_1.ClassKey.STORAGE, class_constants_1.ClassKey.SAN, class_constants_1.ClassKey.LAN].includes(class_constants_1.ClassKey[classLabel.toUpperCase()])) {
                        const metricsFromPages = this.getMetricsFromPages(classLabel, hwType, subsystem);
                        if (metricsFromPages && metricsFromPages.length >= 1) {
                            metricsFromPages.forEach(metric => {
                                if (metricsArray.find(m => m.metric === metric.metric) === undefined) {
                                    metricsArray.push(metric);
                                }
                            });
                        }
                    }
                    return metricsArray;
                }
                else {
                    console.warn("[getMetrics] error: " + hwType + " " + subsystem + " not found");
                    return [];
                }
            }
        }
    }
    static hasMetrics(classKey, hwType, subsystem) {
        let hwTypeDefinition = this.getHwTypeDefinition(classKey, hwType);
        if (hwTypeDefinition !== undefined) {
            const subsystemDefinition = hwTypeDefinition['subsystems'] ? hwTypeDefinition['subsystems'][subsystem] : undefined;
            return subsystemDefinition !== undefined && (subsystemDefinition['metrics'] !== undefined || subsystemDefinition['single'] !== undefined);
        }
        else {
            console.warn("[hasMetrics] error: " + hwType + " " + subsystem + " not found");
            return false;
        }
    }
    static getHeatmapLabels(_class, hwType, subsystem) {
        const subsystemDefinition = this.getSubsystemDefinition(_class, hwType, subsystem);
        if (subsystemDefinition) {
            if (subsystemDefinition['heatmap']) {
                const heatmap = {};
                for (const heatmapItem of subsystemDefinition['heatmap']) {
                    heatmap[heatmapItem['key']] = heatmapItem['label'];
                }
                return heatmap;
            }
            else {
                return {};
            }
        }
        else {
            console.warn("[getHeatmapLabels] " + _class + " " + hwType + " " + subsystem + " not found");
            return {};
        }
    }
    static getMetricsFromPages(_class, hwType, subsystem) {
        let metricsArray = [];
        const subsystemDefinition = this.getSubsystemDefinition(_class, hwType, subsystem);
        if (subsystemDefinition && subsystemDefinition['single']) {
            for (const tab in subsystemDefinition['single']) {
                for (const metrics in subsystemDefinition['single'][tab]) {
                    for (const metricIndex in subsystemDefinition['single'][tab][metrics]) {
                        if (subsystemDefinition['single'][tab][metrics][metricIndex]['metric']) {
                            const definition = subsystemDefinition['single'][tab][metrics][metricIndex];
                            const metric = definition['metric'];
                            metricsArray.push({
                                metric: metric,
                                label: definition['label'],
                                longLabel: definition['longLabel']
                                    ? definition['longLabel']
                                    : definition['label'],
                                unit: definition['unit'],
                                shortcut: definition['shortcut'],
                                defaultPrefix: definition['defaultPrefix']
                                    ? definition['defaultPrefix']
                                    : null,
                                defaultPrecision: definition['defaultPrecision'] !== undefined
                                    ? definition['defaultPrecision']
                                    : 0,
                                top: subsystemDefinition['top'] && this.findTopMetric(subsystemDefinition['top'], metric),
                                metricType: definition['metricType']
                                    ? definition['metricType']
                                    : undefined
                            });
                        }
                    }
                }
            }
        }
        return metricsArray;
    }
    static getMetric(_class, hwType, subsystem, metric) {
        const subsystemDefinition = this.getSubsystemDefinition(_class, hwType, subsystem);
        if (subsystemDefinition) {
            if ([class_constants_1.ClassKey.STORAGE, class_constants_1.ClassKey.SAN, class_constants_1.ClassKey.LAN].includes(_class)) {
                if (subsystemDefinition['metrics'] && subsystemDefinition['metrics'][metric]) {
                    return {
                        ...subsystemDefinition['metrics'][metric],
                        defaultPrefix: subsystemDefinition['metrics'][metric]['defaultPrefix'] !== undefined
                            ? subsystemDefinition['metrics'][metric]['defaultPrefix']
                            : null,
                        defaultPrecision: subsystemDefinition['metrics'][metric]['defaultPrecision'] !== undefined
                            ? subsystemDefinition['metrics'][metric]['defaultPrecision']
                            : 0
                    };
                }
                else {
                    const metrics = this.getMetricsFromPages(_class, hwType, subsystem);
                    const metricFromPages = metrics.find(metricDef => metricDef.metric === metric);
                    return metricFromPages ? metricFromPages : {};
                }
            }
            else {
                if (!subsystemDefinition['metrics']) {
                    console.warn("[getMetric] not metrics found in " + _class + " " + hwType + " " + subsystem + " " + metric);
                }
                return subsystemDefinition['metrics'][metric]
                    ? {
                        ...subsystemDefinition['metrics'][metric],
                        defaultPrefix: subsystemDefinition['metrics'][metric]['defaultPrefix'] !== undefined
                            ? subsystemDefinition['metrics'][metric]['defaultPrefix']
                            : null,
                        defaultPrecision: subsystemDefinition['metrics'][metric]['defaultPrecision'] !== undefined
                            ? subsystemDefinition['metrics'][metric]['defaultPrecision']
                            : 0
                    }
                    : {};
            }
        }
        else {
            console.error("[getMetric] error: " + _class + " " + hwType + " " + subsystem + " " + metric);
            return {};
        }
    }
    static async findClassKey(hwType) {
        let hwTypeDefinition;
        const classes = await Architecture_1.Architecture.getClasses();
        let classKey;
        for (const _class of classes) {
            hwTypeDefinition = this.getHwTypeDefinition(_class.class, hwType);
            if (hwTypeDefinition) {
                classKey = _class.class;
                return classKey;
            }
        }
        console.warn(`[findClassKey] error: hwType '${hwType}' not found`);
        return undefined;
    }
    static async getMetricFindClass(hwType, subsystem, metric) {
        if (hwType && subsystem) {
            const classKey = await MetricsWrapper.findClassKey(hwType);
            const metricDefinition = classKey ? MetricsWrapper.getMetric(classKey, hwType, subsystem, metric) : {};
            if ((typeof metricDefinition === "object") && Object.keys(metricDefinition).length) {
                return metricDefinition;
            }
        }
        console.warn("[getMetricFindClass] error: " + hwType + " " + subsystem + " not found");
        return {};
    }
    static getClassDefinition(_class) {
        const className = _class.charAt(0).toUpperCase() + _class.slice(1);
        try {
            const Class = _Class[className];
            return Class['technology'];
        }
        catch (e) {
            console.error("[getClassDefinition] error: " + className + " not found");
            return undefined;
        }
    }
    static getHwTypeDefinition(_class, hwType) {
        const classDefinition = this.getClassDefinition(_class);
        if (classDefinition && classDefinition[hwType]) {
            return classDefinition[hwType];
        }
        else {
            return undefined;
        }
    }
    static getSubsystemDefinition(_class, hwType, subsystem) {
        const hwTypeDefinition = this.getHwTypeDefinition(_class, hwType);
        if (!hwTypeDefinition || !hwTypeDefinition['subsystems'] || !hwTypeDefinition['subsystems'][subsystem]) {
            return undefined;
        }
        else {
            return hwTypeDefinition['subsystems'][subsystem];
        }
    }
    static findTabLabelByMetric(_class, hwType, subsystem, metric) {
        const subsystemDefinition = this.getSubsystemDefinition(_class, hwType, subsystem);
        if (!subsystemDefinition) {
            return undefined;
        }
        if ([class_constants_1.ClassKey.STORAGE, class_constants_1.ClassKey.SAN, class_constants_1.ClassKey.LAN].includes(_class)) {
            const storageSubsystem = subsystemDefinition;
            if (!storageSubsystem.single) {
                return undefined;
            }
            for (const tabKey in storageSubsystem.single) {
                const tab = storageSubsystem.single[tabKey];
                if (!tab) {
                    continue;
                }
                const hasMetric = tab.metrics.find(tabMetric => tabMetric.metric == metric);
                if (hasMetric) {
                    return tab.label;
                }
            }
        }
        else {
            return undefined;
        }
        return undefined;
    }
    static getParent(_class, hwType, subsystem) {
        const subsystemDefinition = this.getSubsystemDefinition(_class, hwType, subsystem);
        if (subsystemDefinition) {
            if (subsystemDefinition['parent']) {
                const parent = subsystemDefinition['parent'];
                const parentSubsystemDefinition = this.getSubsystemDefinition(_class, hwType, parent);
                if (parentSubsystemDefinition) {
                    return {
                        label: parentSubsystemDefinition['label'] ? parentSubsystemDefinition['label'] : parent,
                        subsystem: parent,
                        hasPage: parentSubsystemDefinition['metrics'] !== undefined || parentSubsystemDefinition['single'] !== undefined
                    };
                }
                else {
                    return {
                        label: parent,
                        subsystem: parent,
                        hasPage: false
                    };
                }
            }
        }
        else {
            console.warn("[getParent] error: " + _class + " " + hwType + " " + subsystem);
            return undefined;
        }
    }
    static getParentInfo(_class, hwType, parent) {
        const subsystemDefinition = this.getSubsystemDefinition(_class, hwType, parent);
        if (subsystemDefinition) {
            return {
                label: parent,
                subsystem: parent,
                hasPage: subsystemDefinition['metrics'] !== undefined || subsystemDefinition['single'] !== undefined
            };
        }
        else {
            return {
                label: parent.charAt(0).toUpperCase() + parent.slice(1),
                subsystem: parent,
                hasPage: false
            };
        }
    }
    static getConfiguration(_class, hwType, subsystem) {
        const subsystemDefinition = this.getSubsystemDefinition(_class, hwType, subsystem);
        if (subsystemDefinition && subsystemDefinition['configuration']) {
            return subsystemDefinition['configuration'];
        }
        else {
            return {};
        }
    }
    static async getConfigurationFindClass(hwType, subsystem) {
        let hwTypeDefinition;
        const classes = await Architecture_1.Architecture.getClasses();
        let classKey;
        for (const _class of classes) {
            hwTypeDefinition = this.getHwTypeDefinition(_class.class, hwType);
            if (hwTypeDefinition) {
                classKey = _class.class;
                return this.getConfiguration(classKey, hwType, subsystem);
            }
        }
        console.warn("[getMetricFindClass] error: " + hwType + " " + subsystem + " not found");
        return {};
    }
    static async getTopMetrics(hwType, subsystem) {
        let hwTypeDefinition;
        const classes = await Architecture_1.Architecture.getClasses();
        let classKey;
        for (const _class of classes) {
            hwTypeDefinition = this.getHwTypeDefinition(_class.class, hwType);
            if (hwTypeDefinition) {
                classKey = _class.class;
                const subsystemDefinition = this.getSubsystemDefinition(classKey, hwType, subsystem);
                let topMetrics = [];
                if (subsystemDefinition.top) {
                    for (const group in subsystemDefinition.top) {
                        topMetrics.push(...subsystemDefinition.top[group]);
                    }
                }
                return topMetrics;
            }
        }
        return [];
    }
    static getMetricsFromEntity(hwType, subsystem) {
        try {
            const metricsRepository = (0, dataSource_1.getDataSource)().getRepository(`metrics.metrics_${hwType}_${subsystem}`);
            const columns = metricsRepository.metadata.columns.map((column) => {
                return column.propertyName;
            });
            return columns;
        }
        catch (e) {
            return [];
        }
    }
    static skipOthers(_class, hwType, subsystem, metric) {
        let metricDefinition;
        if (_class === 'storage') {
            metricDefinition = Classes_1.Storage.metrics[metric];
        }
        else if (_class === 'san') {
            metricDefinition = Classes_1.San.metrics[metric];
        }
        else if (_class === 'lan') {
            metricDefinition = Classes_1.Lan.metrics[metric];
        }
        else {
            metricDefinition = MetricsWrapper.getMetric(_class, hwType, subsystem, metric);
        }
        if (metricDefinition && metricDefinition.unit && [subsystemMetricsResponse_constants_1.UNIT.percent, subsystemMetricsResponse_constants_1.UNIT.millisecond].includes(metricDefinition.unit)) {
            return true;
        }
        return false;
    }
    static handleLongLabel(metrics) {
        metrics.forEach(metric => {
            metric.label = metric.longLabel
                ? metric.longLabel
                : metric.label;
        });
        return metrics;
    }
    static findTopMetric(top, metric) {
        for (const group in top) {
            if (top[group].includes(metric)) {
                return true;
            }
        }
        return false;
    }
}
exports.MetricsWrapper = MetricsWrapper;
//# sourceMappingURL=MetricsWrapper.js.map