"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Virtualization = void 0;
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const Definitions_1 = require("../Definitions");
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const Color_1 = require("../Color");
class Virtualization {
}
exports.Virtualization = Virtualization;
Virtualization.label = "Server";
Virtualization.order = 0;
Virtualization.technology = {
    proxmox: {
        label: 'Proxmox',
        order: 4,
        subsystems: {
            cluster: {
                label: "Cluster",
                excludedTools: [
                    'alerting'
                ]
            },
            vm: {
                label: 'VM',
                aliases: true,
                parent: 'node',
                top: {
                    'CPU': ['cpu_utilization_percent'],
                    'Data': ['disk_read', 'disk_write'],
                    'LAN': ['netin', 'netout']
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    },
                    {
                        label: 'Memory',
                        key: 'memory',
                        total: 'maxmem',
                        free: 'memfree'
                    }],
                configuration: {
                    name: {
                        label: 'VM Label'
                    },
                    cluster: {
                        label: 'Cluster'
                    },
                    status: {
                        label: 'Status'
                    },
                    uptime: {
                        label: 'Uptime'
                    },
                    cpus: {
                        label: 'CPUs',
                        unit: subsystemMetricsResponse_constants_1.UNIT.cpu_core,
                        decimal: 0
                    },
                    maxdisk: {
                        label: 'Disk size',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    maxmem: {
                        label: 'Memory',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    }
                },
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'CPU',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'CPU (Peak)',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        label: 'CPU %',
                        longLabel: 'CPU %',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU (Peak)',
                        longLabel: 'CPU % (Peak)',
                    },
                    maxcpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'maxcpu',
                        longLabel: 'CPU Virtual',
                        label: 'Virtual',
                    },
                    mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem',
                        longLabel: 'Memory Used',
                        label: 'Used'
                    },
                    memfree: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memfree',
                        longLabel: 'Memory Free',
                        label: 'Free'
                    },
                    maxmem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'maxmem',
                        longLabel: 'Memory Total',
                        label: 'Total'
                    },
                    diskread: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'diskread',
                        longLabel: 'Disk Read',
                        label: 'Read',
                    },
                    diskwrite: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'diskwrite',
                        longLabel: 'Disk Write',
                        label: 'Write',
                    },
                    netin: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'netin',
                        label: 'In',
                        longLabel: 'LAN In'
                    },
                    netout: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'netout',
                        label: 'Out',
                        longLabel: 'LAN Out'
                    },
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'maxcpu',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ]
                }
            },
            lxc: {
                label: 'LXC',
                configuration: {
                    name: {
                        label: 'LXC Label'
                    },
                    cluster: {
                        label: 'Cluster'
                    },
                    status: {
                        label: 'Status'
                    },
                    uptime: {
                        label: 'Uptime'
                    }
                },
                parent: 'node',
                metrics: {
                    cpu: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu',
                        label: 'CPU %',
                        longLabel: 'CPU %',
                    },
                    diskread: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'diskread',
                        longLabel: 'Disk Read',
                        label: 'Read',
                    },
                    diskwrite: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'diskwrite',
                        longLabel: 'Disk Write',
                        label: 'Write',
                    },
                    netin: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'netin',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    netout: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'netout',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                    mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem',
                        longLabel: 'Memory Used',
                        label: 'Used',
                    },
                    memfree: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memfree',
                        longLabel: 'Memory Free',
                        label: 'Free'
                    },
                    memused: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memused',
                        longLabel: 'Memory Used',
                        label: 'Used',
                    },
                },
            },
            node: {
                label: 'Node',
                parent: 'cluster',
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    },
                    {
                        label: 'Memory',
                        key: 'memory',
                        total: 'memtotal',
                        used: 'memused'
                    }],
                configuration: {
                    name: {
                        label: 'Node Label'
                    },
                    cluster: {
                        label: 'Cluster'
                    },
                    status: {
                        label: 'Status'
                    },
                    uptime: {
                        label: 'Uptime',
                        unit: subsystemMetricsResponse_constants_1.UNIT.timestamp
                    },
                    maxcpu: {
                        label: 'CPUs',
                        unit: subsystemMetricsResponse_constants_1.UNIT.cpu_core,
                        decimal: 0
                    },
                    maxdisk: {
                        label: 'Disk size',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    maxmem: {
                        label: 'Memory',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    }
                },
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'CPU',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        label: 'CPU %',
                        longLabel: 'CPU %',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU (Peak)',
                        longLabel: 'CPU % (Peak)',
                    },
                    maxcpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'maxcpu',
                        longLabel: 'CPU Virtual',
                        label: 'Virtual',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'CPU (Peak)',
                    },
                    iowait: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'iowait',
                        longLabel: 'IO Delay',
                        label: 'Delay',
                    },
                    rootfree: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'rootfree',
                        longLabel: 'OS Disk Free',
                        label: 'Free',
                    },
                    roottotal: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'roottotal',
                        longLabel: 'OS Disk Total',
                        label: 'Total',
                    },
                    rootused: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'rootused',
                        longLabel: 'OS Disk Used',
                        label: 'Used',
                    },
                    swapfree: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'swapfree',
                        longLabel: 'Swap Free',
                        label: 'Free',
                    },
                    swaptotal: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'swaptotal',
                        longLabel: 'Swap Total',
                        label: 'Total',
                    },
                    swapused: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        metric: 'swapused',
                        longLabel: 'Swap Used',
                        label: 'Used',
                    },
                    netin: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'netin',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    netout: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'netout',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                    memtotal: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memtotal',
                        longLabel: 'Memory Total',
                        label: 'Total'
                    },
                    memfree: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memfree',
                        longLabel: 'Memory Free',
                        label: 'Free'
                    },
                    memused: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memused',
                        longLabel: 'Memory Used',
                        label: 'Used'
                    },
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ]
                }
            },
            storage: {
                label: 'Storage',
                parent: 'node',
                configuration: {
                    name: {
                        label: 'Storage Label'
                    },
                    cluster: {
                        label: 'Cluster'
                    },
                    active: {
                        label: 'Active'
                    },
                    shared: {
                        label: 'Shared'
                    },
                    type: {
                        label: 'Type'
                    },
                    enabled: {
                        label: 'Enabled'
                    }
                },
                metrics: {
                    free: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'free',
                        longLabel: 'Free Capacity',
                        label: 'Free',
                    },
                    total: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'total',
                        longLabel: 'Total Capacity',
                        label: 'Total',
                    },
                    used: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'used',
                        longLabel: 'Used Capacity',
                        label: 'Used',
                    },
                },
            },
        },
    },
    nutanix: {
        label: 'Nutanix',
        order: 3,
        subsystems: {
            cluster: {
                label: "Cluster",
                excludedTools: [
                    "all"
                ]
            },
            host: {
                label: "Host",
                parent: 'cluster',
                top: {
                    "CPU": ['cpu_utilization_percent'],
                    "Memory": ['memory_usage']
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    },
                    {
                        label: 'Memory',
                        key: 'memory_usage',
                        percent: 'memory_usage'
                    }],
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'CPU',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'CPU (Peak)',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        label: 'CPU %',
                        longLabel: 'CPU %',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU',
                        longLabel: 'CPU % (Peak)',
                    },
                    cpu_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores',
                        longLabel: 'CPU Cores',
                        label: 'Cores',
                    },
                    memory_free: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_free',
                        longLabel: 'Memory Free',
                        label: 'Free',
                    },
                    memory_usage: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'memory_usage',
                        longLabel: 'Memory Usage',
                        label: 'Usage',
                    },
                    memory_used: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_used',
                        longLabel: 'Memory Used',
                        label: 'Used',
                    },
                },
                configuration: {
                    address: {
                        label: "IP Adress"
                    },
                    cpu_count: {
                        label: "CPU Count"
                    },
                    socket_count: {
                        label: "Socker Count"
                    },
                    cpu_model: {
                        label: "CPU Model"
                    },
                    version: {
                        label: "Version"
                    }
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ]
                }
            },
            vm: {
                label: "VM",
                aliases: true,
                parent: 'cluster',
                top: {
                    'CPU': ['cpu_utilization_percent', 'cpu_utilization_cores'],
                    'IO': ['total_io'],
                    'Data': ['total_io_bandwidth']
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    },
                    {
                        label: 'Memory',
                        key: 'memory_usage',
                        total: 'memory_total',
                        used: 'memory_used'
                    }],
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'memory_total',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    all: [
                        'memory_usage_percent',
                        'cpu_cores',
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                },
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'CPU',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'CPU (Peak)',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        longLabel: 'CPU % (Peak)',
                        label: 'CPU (Peak)',
                    },
                    cpu_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores',
                        longLabel: 'CPU Virtual',
                        label: 'Virtual',
                    },
                    memory_free: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_free',
                        longLabel: 'Memory Free',
                        label: 'Free',
                    },
                    memory_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_total',
                        longLabel: 'Memory Total',
                        label: 'Total',
                    },
                    memory_used: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_used',
                        longLabel: 'Memory Used',
                        label: 'Used',
                    },
                    memory_usage_percent: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'memory_usage_percent',
                        longLabel: 'Memory Usage',
                        label: 'Usage',
                    },
                    total_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'total_io',
                        longLabel: 'IO',
                        label: 'Total',
                    },
                    read_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_io',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    write_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_io',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    total_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'total_io_bandwidth',
                        longLabel: 'Data',
                        label: 'Total',
                    },
                    read_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_io_bandwidth',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    write_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_io_bandwidth',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    total_io_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'total_io_latency',
                        longLabel: 'Latency',
                        label: 'Total',
                    },
                    read_io_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'read_io_latency',
                        longLabel: 'Latency Read',
                        label: 'Read',
                    },
                    write_io_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'write_io_latency',
                        longLabel: 'Latency Write',
                        label: 'Write',
                    },
                    received: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'received',
                        label: 'In',
                        longLabel: 'LAN In'
                    },
                    transmitted: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'transmitted',
                        label: 'Out',
                        longLabel: 'LAN Out'
                    },
                },
                configuration: {
                    cpu_count: {
                        label: "CPU Count"
                    },
                    hypervisor: {
                        label: "Hypervisor"
                    },
                    os: {
                        label: "OS"
                    },
                    memory: {
                        label: "Memory",
                        ...Definitions_1.metricTypes.memory
                    }
                }
            },
            disk: {
                label: 'Disk',
                parent: 'cluster',
                excludedTools: [
                    'customGroup'
                ],
                configuration: {
                    type: {
                        label: "Type"
                    },
                    physical_size: {
                        label: "Size",
                        ...Definitions_1.metricTypes.capacity_volume
                    }
                },
                metrics: {
                    read_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_io_bandwidth',
                        label: 'Read',
                    },
                    write_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_io_bandwidth',
                        label: 'Write',
                    },
                    read_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_io',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    write_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_io',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    total_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'total_io',
                        longLabel: 'IO Total',
                        label: 'Total',
                    },
                    total_io_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'total_io_latency',
                        longLabel: 'Latency Total',
                        label: 'Total',
                    }
                }
            },
            vdisk: {
                label: 'Virtual disk',
                parent: 'cluster',
                excludedTools: [
                    'customGroup'
                ],
                configuration: {
                    capacity: {
                        label: "Capacity",
                        ...Definitions_1.metricTypes.capacity_volume
                    }
                },
                metrics: {
                    read_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_io_bandwidth',
                        label: 'Read',
                    },
                    write_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_io_bandwidth',
                        label: 'Write',
                    },
                    read_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_io',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    write_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_io',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    total_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'total_io',
                        longLabel: 'IO Total',
                        label: 'Total',
                    },
                    total_io_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'total_io_latency',
                        longLabel: 'Latency Total',
                        label: 'Total',
                    }
                }
            },
            storage_pool: {
                label: 'Storage pool',
                parent: 'cluster',
                configuration: {
                    capacity_size: {
                        label: "Capacity",
                        ...Definitions_1.metricTypes.capacity_volume
                    },
                    capacity_used: {
                        label: "Used",
                        ...Definitions_1.metricTypes.capacity_volume
                    },
                    capacity_free: {
                        label: "Free",
                        ...Definitions_1.metricTypes.capacity_volume
                    }
                },
                metrics: {
                    read_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_io_bandwidth',
                        label: 'Read',
                    },
                    write_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_io_bandwidth',
                        label: 'Write',
                    },
                    read_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_io',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    write_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_io',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    total_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'total_io',
                        longLabel: 'IO Total',
                        label: 'Read',
                    },
                    total_io_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'total_io_latency',
                        longLabel: 'Latency Total',
                        label: 'Total',
                    }
                }
            },
            storage_container: {
                label: 'Storage container',
                parent: 'cluster',
                configuration: {
                    capacity_size: {
                        label: "Capacity",
                        ...Definitions_1.metricTypes.capacity_volume
                    },
                    capacity_used: {
                        label: "Used",
                        ...Definitions_1.metricTypes.capacity_volume
                    },
                    capacity_free: {
                        label: "Free",
                        ...Definitions_1.metricTypes.capacity_volume
                    },
                    compression_enabled: {
                        label: "Compression"
                    },
                    is_nutanix_managed: {
                        label: "Nutanix Management"
                    }
                },
                metrics: {
                    read_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_io_bandwidth',
                        label: 'Read',
                    },
                    write_io_bandwidth: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_io_bandwidth',
                        label: 'Write',
                    },
                    read_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_io',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    write_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_io',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    total_io: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'total_io',
                        longLabel: 'IO Total',
                        label: 'Total',
                    },
                    total_io_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'total_io_latency',
                        longLabel: 'Latency Total',
                        label: 'Total',
                    }
                }
            }
        },
    },
    linux: {
        label: 'Linux',
        order: 12,
        subsystems: {
            group: {
                label: 'Linux Group',
                excludedTools: [
                    'all'
                ],
            },
            server: {
                label: 'Server',
                aliases: true,
                top: {
                    'CPU': ['cpu_util'],
                    'Memory': ['mem_used']
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_util',
                        percent: 'cpu_util'
                    },
                    {
                        label: 'Memory',
                        key: 'mem_util',
                        total: 'mem_total',
                        used: 'mem_used'
                    }],
                configuration: {
                    version: {
                        label: "version"
                    },
                },
                excluded: {
                    all: [
                        'mem2',
                        'mem4',
                        'mem_fs_cache',
                        'pgs3',
                        'blocked_processes',
                        'blocked_raw',
                        'blocked_direct',
                        'pgs4',
                        'logical_processors',
                        'wtsize',
                        'send_bytes',
                        'received_bytes',
                        'san_send_bytes',
                        'san_received_bytes'
                    ]
                },
                metrics: {
                    cpu_util: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_util',
                        longLabel: 'CPU',
                        label: 'Usage',
                    },
                    cpu_usr: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_usr',
                        longLabel: 'CPU User',
                        label: 'User',
                    },
                    cpu_sys: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_sys',
                        longLabel: 'CPU System',
                        label: 'System',
                    },
                    cpu_wait: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_wait',
                        longLabel: 'CPU IO Wait',
                        label: 'IO Wait',
                    },
                    cpu_idle: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_idle',
                        longLabel: 'CPU Idle',
                        label: 'Idle',
                    },
                    load_avrg: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'load_avrg',
                        longLabel: 'Load Average',
                        label: 'Load Average',
                        color: Color_1.Color.pallete.brownDark,
                    },
                    wtsize: {
                        metric: 'wtsize',
                        label: 'WT Size',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
                    },
                    send_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'send_bytes',
                        longLabel: 'Bytes Out',
                        label: 'Out',
                    },
                    received_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'received_bytes',
                        longLabel: 'Bytes In',
                        label: 'In',
                    },
                    mem_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_total',
                        longLabel: 'Memory Total',
                        label: 'Total',
                        defaultPrecision: 1,
                    },
                    mem2: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem2',
                        label: 'Pinned',
                        defaultPrecision: 1,
                    },
                    mem_free: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_free',
                        longLabel: 'Memory Free',
                        label: 'Free',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.green
                    },
                    mem4: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem4',
                        label: 'mem4',
                        defaultPrecision: 1,
                    },
                    mem_used: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_used',
                        longLabel: 'Memory Used',
                        label: 'Used',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.red
                    },
                    mem_fs_cache: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_fs_cache',
                        label: 'FS Cache',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.blue
                    },
                    mem_util: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'mem_util',
                        longLabel: 'Memory Used %',
                        label: 'Memory %',
                    },
                    pgs1: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'pgs1',
                        label: 'Page In',
                    },
                    pgs2: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'pgs2',
                        label: 'Page Out',
                    },
                    pgs3: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'pgs3',
                        label: 'Paging 3',
                    },
                    pgs4: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'pgs4',
                        label: 'Usage',
                    },
                    logical_processors: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'logical_processors',
                        label: 'Logical Processors',
                        color: Color_1.Color.pallete.redDark,
                    },
                    blocked_processes: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'blocked_processes',
                        label: 'Blocked Processes',
                        color: Color_1.Color.pallete.blueDark,
                    },
                    blocked_raw: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'blocked_raw',
                        label: 'Blocked Raw',
                        color: Color_1.Color.pallete.greenDark,
                    },
                    blocked_direct: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'blocked_direct',
                        label: 'Blocked Direct IO',
                        color: Color_1.Color.pallete.orangeDark,
                    },
                    read_iops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_iops',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    write_iops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_iops',
                        longLabel: 'IO Write',
                        label: 'Write',
                        defaultPrecision: 0
                    },
                    read_data: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_data',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    write_data: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_data',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    read_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'read_latency',
                        longLabel: 'Latency Read',
                        label: 'Read',
                        defaultPrecision: 1
                    },
                    write_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'write_latency',
                        longLabel: 'Latency Write',
                        label: 'Write',
                        defaultPrecision: 1
                    },
                    cpu_stolen: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'st_cpu',
                        longLabel: 'Stolen',
                        label: 'Stolen',
                    },
                    ame_compres: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'ame_compres',
                        longLabel: 'Compressed Memory',
                        label: 'Compressed Memory',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.green,
                    },
                    ame_deficit: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'ame_deficit',
                        label: 'Deficit Memory',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.red,
                    },
                    ame_ratio: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'ame_ratio',
                        label: 'Compression Ratio',
                        color: Color_1.Color.pallete.black,
                    },
                    cpu_core: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_core',
                        longLabel: 'CPU',
                        label: 'Utilisation',
                    },
                    san_send_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'send_bytes',
                        longLabel: 'Bytes Out',
                        label: 'Out',
                    },
                    san_received_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'received_bytes',
                        longLabel: 'Bytes In',
                        label: 'In',
                    },
                },
            },
            job: {
                label: 'Job',
                excludedTools: [
                    'all'
                ],
                configuration: {
                    ip: {
                        label: "IP"
                    },
                },
                metrics: {
                    time_1_sec: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'time_1_sec',
                        label: 'time_1_sec',
                    },
                    time_difference: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'time_difference',
                        label: 'CPU',
                    },
                    rss: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 2,
                        metric: 'rss',
                        label: 'rss'
                    },
                    vzs: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 2,
                        metric: 'vzs',
                        label: 'vzs'
                    },
                },
            },
            wlm: {
                label: 'Wlm',
                excludedTools: [
                    'all'
                ],
                configuration: {
                    ip: {
                        label: "IP"
                    },
                },
                metrics: {
                    wlm_cpu: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        defaultPrecision: 2,
                        metric: 'wlm_cpu',
                        label: 'Class',
                    },
                    wlm_mem: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 2,
                        metric: 'wlm_mem',
                        label: 'Class',
                    },
                    wlm_dkio: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        defaultPrecision: 2,
                        metric: 'wlm_dkio',
                        minY: undefined,
                        label: 'Class'
                    },
                },
            },
            lan: {
                label: 'LAN interface',
                excludedTools: [
                    'alerting',
                    'customGroup'
                ],
                configuration: {
                    ip: {
                        label: "IP"
                    },
                },
                metrics: {
                    recv_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'recv_bytes',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    trans_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'trans_bytes',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                    recv_packets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'recv_packets',
                        longLabel: 'Packets In',
                        label: 'In',
                        defaultPrecision: 0
                    },
                    trans_packets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'trans_packets',
                        longLabel: 'Packets Out',
                        label: 'Out',
                        defaultPrecision: 0
                    },
                    lan_error: {
                        metric: 'lan_error',
                        label: 'Error',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                    },
                },
            },
            san: {
                label: 'SAN interface',
                excludedTools: [
                    'alerting',
                    'customGroup'
                ],
                configuration: {
                    ip: {
                        label: "IP"
                    },
                },
                metrics: {
                    recv_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'recv_bytes',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    trans_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'trans_bytes',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    iops_in: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.frames,
                        shortcut: subsystemMetricsResponse_constants_1.UNIT.frames,
                        metric: 'iops_read',
                        longLabel: 'Frames Read',
                        label: 'Read',
                        defaultPrecision: 0
                    },
                    iops_out: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.frames,
                        shortcut: subsystemMetricsResponse_constants_1.UNIT.frames,
                        metric: 'iops_write',
                        longLabel: 'Frames Write',
                        label: 'Write',
                        defaultPrecision: 0
                    },
                    san_error: {
                        metric: 'san_error',
                        label: 'Error',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                    },
                    resp_t_r: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'resp_t_r',
                        longLabel: 'Latency Read',
                        label: 'Read',
                    },
                    resp_t_w: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'resp_t_w',
                        longLabel: 'Latency Write',
                        label: 'Write',
                    },
                    rx_power: {
                        metric: 'rx_power',
                        label: 'RX Power',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sfp_power_out,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sfp_power_out,
                        defaultPrecision: 2
                    },
                    tx_power: {
                        metric: 'tx_power',
                        label: 'TX Power',
                        unit: subsystemMetricsResponse_constants_1.UNIT.sfp_power_in,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sfp_power_in,
                        defaultPrecision: 2
                    },
                },
            },
            sea: {
                label: 'SEA',
                excludedTools: [
                    'alerting',
                    'customGroup'
                ],
                configuration: {
                    LocalPartitionLabel: {
                        label: 'LPAR'
                    },
                    VirtualSwitchID: {
                        label: 'Virtual Switch'
                    },
                    ip: {
                        label: "IP"
                    },
                },
                metrics: {
                    recv_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'recv_bytes',
                        longLabel: 'Bytes In',
                        label: 'In',
                    },
                    trans_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'trans_bytes',
                        longLabel: 'Bytes Out',
                        label: 'Out',
                    },
                    recv_packets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'recv_packets',
                        longLabel: 'Packets In',
                        label: 'In',
                        defaultPrecision: 0
                    },
                    trans_packets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'trans_packets',
                        longLabel: 'Packets Out',
                        label: 'Out',
                        defaultPrecision: 0
                    },
                },
            },
        },
    },
    power: {
        label: 'IBM Power Systems',
        order: 1,
        subsystems: {
            server: {
                label: 'Server',
                heatmap: [
                    {
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    },
                ],
                configuration: {
                    ServertotalProcUnitsMetric: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: "CPU Configured",
                    },
                    assignedMemToLpars: {
                        ...Definitions_1.metricTypes.memory,
                        label: "Used Mem",
                    },
                    availableMem: {
                        ...Definitions_1.metricTypes.memory,
                        label: "Free Mem",
                    },
                    systemFirmwareUtilAssignedMem: {
                        ...Definitions_1.metricTypes.memory,
                        label: "Firmware Mem",
                    },
                    State: {
                        label: "State"
                    },
                    MachineType: {
                        label: "Machine Type",
                        type: 'string'
                    },
                    SerialNumber: {
                        label: "Serial",
                        type: 'string'
                    },
                    InstalledSystemProcessorUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: "CPU Installed",
                        decimal: 0
                    },
                    CurrentAvailableSystemProcessorUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: "CPU Free",
                    },
                    InstalledSystemMemory: {
                        ...Definitions_1.metricTypes.memory,
                        label: "Memory Installed",
                        decimal: 0
                    },
                    CurrentAvailableSystemMemory: {
                        ...Definitions_1.metricTypes.memory,
                        label: "Memory Free",
                        decimal: 0
                    },
                    ConfigurableSystemMemory: {
                        ...Definitions_1.metricTypes.memory,
                        label: "Memory Configurable",
                        decimal: 0
                    },
                    ConfigurableSystemProcessorUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: "CPU Configurable",
                        decimal: 0
                    },
                    SystemTime: {
                        label: "Time",
                        unit: subsystemMetricsResponse_constants_1.UNIT.timestamp
                    },
                    SystemFirmware: {
                        label: "Firmware"
                    }
                },
                metrics: {
                    assignedMinusBorrowed: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'assignedMinusBorrowed',
                        longLabel: 'CPU Pool Configured',
                        label: 'Configured'
                    },
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'Usage'
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'Usage (Peak)'
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        longLabel: 'CPU %',
                        label: 'Usage'
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        longLabel: 'CPU % (Peak)',
                        label: 'Usage (Peak)'
                    },
                    cpu_physical_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_physical_cores',
                        longLabel: 'CPU Phys',
                        label: 'Phys'
                    },
                    cpu_physical_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_physical_cores_max',
                        longLabel: 'CPU Phys (Peak)',
                        label: 'Phys (Peak)'
                    },
                    cpu_physical_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_physical_percent',
                        longLabel: 'CPU Phys %',
                        label: 'Phys'
                    },
                    cpu_physical_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_physical_percent_max',
                        longLabel: 'CPU Phys % (Peak)',
                        label: 'Phys'
                    },
                    assignedProcUnitsPhysicalProcessor: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'assignedProcUnitsPhysicalProcessor',
                        longLabel: 'CPU Pool Assigned',
                        label: 'Assigned'
                    },
                    utilizedProcUnitsPhysicalProcessor: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'utilizedProcUnitsPhysicalProcessor',
                        longLabel: 'CPU Pool Phys',
                        label: 'Phys'
                    },
                    configuredProcUnitsPhysicalProcessor: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'configuredProcUnitsPhysicalProcessor',
                        longLabel: 'CPU Pool Configured',
                        label: 'Pool Configured'
                    },
                    borrowedProcUnitsPhysicalProcessor: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'borrowedProcUnitsPhysicalProcessor',
                        longLabel: 'CPU Pool Borrowed',
                        label: 'Borrowed'
                    },
                    availableProcUnitsPhysicalProcessor: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'availableProcUnitsPhysicalProcessor',
                        longLabel: 'CPU Pool Available',
                        label: 'Available'
                    },
                    totalProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'totalProcUnits',
                        longLabel: 'CPU Configured',
                        label: 'Configured'
                    },
                    totalProcUnits_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'totalProcUnits_max',
                        longLabel: 'CPU Configured (max)',
                        label: 'Configured (max)'
                    },
                    assignedProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'assignedProcUnits',
                        longLabel: 'CPU Entitled',
                        label: 'Entitled'
                    },
                    configurableProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'configurableProcUnits',
                        longLabel: 'CPU Configured',
                        label: 'Configured'
                    },
                    availableProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'availableProcUnits',
                        longLabel: 'CPU Available',
                        label: 'Available'
                    },
                    powerReading: {
                        ...Definitions_1.metricTypes.power,
                        metric: 'powerReading',
                        longLabel: 'Power Consumption',
                        label: 'Consumption'
                    },
                    totalMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'totalMem',
                        longLabel: 'Total Memory',
                        label: 'Total'
                    },
                    availableMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'availableMem',
                        longLabel: 'Free Memory',
                        label: 'Free'
                    },
                    configurableMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'configurableMem',
                        longLabel: 'Configurable Memory',
                        label: 'Configurable'
                    },
                    assignedMemToLpars: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'assignedMemToLpars',
                        longLabel: 'Used Memory',
                        label: 'Used'
                    },
                    systemFirmwareUtilAssignedMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'systemFirmwareUtilAssignedMem',
                        longLabel: 'Firmware Memory',
                        label: 'Firmware'
                    },
                },
                excluded: {
                    reporter: [
                        'cpu_physical_cores_max',
                        'cpu_physical_percent_max',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'totalProcUnits_max'
                    ],
                    all: [
                        'assignedMinusBorrowed',
                        'configuredProcUnitsPhysicalProcessor',
                    ],
                    alerting: [
                        'cpu_physical_cores_max',
                        'cpu_physical_percent_max',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'totalProcUnits_max'
                    ],
                    customGroup: [
                        'cpu_physical_cores_max',
                        'cpu_physical_percent_max',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'assignedMinusBorrowed',
                        'assignedProcUnitsPhysicalProcessor',
                        'utilizedProcUnitsPhysicalProcessor',
                        'borrowedProcUnitsPhysicalProcessor',
                        'configuredProcUnitsPhysicalProcessor',
                        'availableProcUnitsPhysicalProcessor',
                        'totalProcUnits',
                        'totalProcUnits_max',
                        'assignedProcUnits',
                        'configurableProcUnits',
                        'availableProcUnits',
                        'totalProcUnits_max',
                    ]
                },
            },
            cmc: {
                label: 'CMC',
                excludedTools: [
                    'all'
                ],
            },
            cmcpool: {
                label: 'Pool',
                excludedTools: [
                    'all'
                ],
                configuration: {
                    hmc_uuid: {
                        label: 'HMC UUID'
                    },
                    hmc_names: {
                        label: 'HMC'
                    },
                    CurrentRemainingCreditBalance: {
                        decimal: 2,
                        label: 'Remaining Credits'
                    },
                    base_anyoscores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Base'
                    },
                    system_uuid: {
                        label: 'System UUID'
                    },
                    system_name: {
                        label: 'System'
                    },
                    pool_id: {
                        label: 'PEP2 ID'
                    },
                    base_memory: {
                        ...Definitions_1.metricTypes.memory,
                        decimal: 0,
                        label: 'Base'
                    },
                    pool_name: {
                        label: 'PEP2'
                    },
                    pool_name_href: {
                        label: 'PEP2'
                    },
                    tag_id: {
                        label: 'Tag ID'
                    },
                    tag_name: {
                        label: 'Tag'
                    },
                    number_of_lpars: {
                        label: 'Number of LPARs'
                    },
                    proc_installed: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Installed'
                    },
                    proc_available: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Available Entitled'
                    },
                    mem_installed: {
                        ...Definitions_1.metricTypes.memory,
                        decimal: 0,
                        label: 'Installed'
                    },
                    mem_available: {
                        ...Definitions_1.metricTypes.memory,
                        decimal: 0,
                        label: 'Available'
                    },
                    number_of_partitions: {
                        label: 'Number of Partitions'
                    },
                    base_core_any_os: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Base Any OS'
                    },
                    base_core_linuxvios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Base Linux/VIOS'
                    },
                    base_core_rhel: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'RHEL'
                    },
                    base_core_rhcos: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'RHCOS'
                    },
                    base_core_sles: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'SLES'
                    },
                    base_core_aix: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'AIX'
                    },
                    base_core_ibmi: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'IBMi'
                    },
                    Name: {
                        label: "Name"
                    },
                },
                metrics: {
                    utilizedProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'utilizedProcUnits',
                        longLabel: 'CPU Utilization',
                        label: 'Utilized'
                    },
                    cm_total: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_total',
                        longLabel: 'CPU Utilization',
                        label: 'Utilized'
                    },
                    base_core_any_os: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_any_os',
                        longLabel: 'Base Any OS Cores',
                        label: 'Base'
                    },
                    baseAnyOS: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_anyoscores',
                        longLabel: 'Base Any OS Cores',
                        label: 'Base'
                    },
                    mem_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_total',
                        longLabel: 'Memory Total',
                        label: 'Memory'
                    },
                    base_memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'base_memory',
                        longLabel: 'Base Memory',
                        label: 'Base'
                    },
                    mm_minutes: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mm_minutes',
                        longLabel: 'Metered Memory Minutes',
                        label: 'Metered Memory Minutes'
                    },
                    mm_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mm_total',
                        longLabel: 'Memory Minute Usage',
                        label: 'Memory Usage'
                    },
                    cm_aix: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_aix',
                        longLabel: 'AIX',
                        label: 'AIX'
                    },
                    base_core_aix: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_aix',
                        longLabel: 'Base AIX',
                        label: 'Base'
                    },
                    cmm_aix: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_aix',
                        longLabel: 'AIX Metered Core Minutes',
                        label: 'AIX'
                    },
                    mmm_aix: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mmm_aix',
                        longLabel: 'AIX Metered Memory Minutes',
                        label: 'AIX'
                    },
                    cm_ibmi: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_ibmi',
                        longLabel: 'IBMi',
                        label: 'IBMi'
                    },
                    base_core_ibmi: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_ibmi',
                        longLabel: 'Base IBMi',
                        label: 'Base'
                    },
                    cmm_ibmi: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_ibmi',
                        longLabel: 'IBMi Metered Core Minutes',
                        label: 'IBMi'
                    },
                    mmm_ibmi: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mmm_ibmi',
                        longLabel: 'IBMi Metered Memory Minutes',
                        label: 'IBMi'
                    },
                    cm_otherlinux: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_otherlinux',
                        longLabel: 'Other Linux',
                        label: 'Other Linux'
                    },
                    base_core_linuxvios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_linuxvios',
                        longLabel: 'Base Linux/VIOS',
                        label: 'Base'
                    },
                    cmm_linux: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_linux',
                        longLabel: 'Linux Metered Core Minutes',
                        label: 'Linux'
                    },
                    mmm_linux: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mmm_linux',
                        longLabel: 'Linux Metered Memory Minutes',
                        label: 'Linux'
                    },
                    cmm_linuxvios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_linuxvios',
                        longLabel: 'Linux/VIOS Metered Core Minutes',
                        label: 'Linux/VIOS'
                    },
                    cm_vios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_vios',
                        longLabel: 'VIOS',
                        label: 'VIOS'
                    },
                    cmm_vios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_vios',
                        longLabel: 'VIOS Metered Core Minutes',
                        label: 'VIOS'
                    },
                    mmm_vios: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mmm_vios',
                        longLabel: 'VIOS Metered Memory Minutes',
                        label: 'VIOS'
                    },
                    cm_rhel: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_rhel',
                        longLabel: 'RHEL',
                        label: 'RHEL'
                    },
                    base_core_rhel: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_rhel',
                        longLabel: 'Base RHEL',
                        label: 'Base'
                    },
                    cmm_rhel: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_rhel',
                        longLabel: 'RHEL Metered Core Minutes',
                        label: 'RHEL'
                    },
                    mmm_rhel: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mmm_rhel',
                        longLabel: 'RHEL Metered Memory Minutes',
                        label: 'RHEL'
                    },
                    cm_rhelcoreos: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_rhelcoreos',
                        longLabel: 'RHCOS',
                        label: 'RHCOS'
                    },
                    base_core_rhcos: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_rhcos',
                        longLabel: 'Base RHCOS',
                        label: 'Base'
                    },
                    cmm_rhelcoreos: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_rhelcoreos',
                        longLabel: 'RHCOS Metered Core Minutes',
                        label: 'RHCOS'
                    },
                    mmm_rhcos: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mmm_rhcos',
                        longLabel: 'RHCOS Metered Memory Minutes',
                        label: 'RHCOS'
                    },
                    cm_sles: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cm_sles',
                        longLabel: 'SLES',
                        label: 'SLES'
                    },
                    base_core_sles: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_sles',
                        longLabel: 'Base Core SLES',
                        label: 'Base'
                    },
                    cmm_sles: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cmm_sles',
                        longLabel: 'SLES Metered Core Minutes',
                        label: 'SLES'
                    },
                    mmm_sles: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mmm_sles',
                        longLabel: 'SLES Metered Memory Minutes',
                        label: 'SLES'
                    },
                }
            },
            cmcserver: {
                label: 'Server',
                excludedTools: [
                    'all'
                ],
                configuration: {
                    Name: {
                        label: "Name"
                    },
                    State: {
                        label: "State"
                    },
                    hmc_uuid: {
                        label: 'HMC UUID'
                    },
                    hmc_names: {
                        label: 'HMC'
                    },
                    system_uuid: {
                        label: 'System UUID'
                    },
                    CurrentRemainingCreditBalance: {
                        decimal: 2,
                        label: 'Current Remaining Credit Balance'
                    },
                    system_name: {
                        label: 'System'
                    },
                    pool_id: {
                        label: 'PEP2 ID'
                    },
                    pool_name: {
                        label: 'PEP2'
                    },
                    pool_name_href: {
                        label: 'PEP2'
                    },
                    tag_id: {
                        label: 'Tag ID'
                    },
                    tag_name: {
                        label: 'Tag'
                    },
                    proc_installed: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Installed'
                    },
                    proc_available: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Available Entitled'
                    },
                    mem_installed: {
                        ...Definitions_1.metricTypes.memory,
                        decimal: 0,
                        label: 'Installed'
                    },
                    mem_available: {
                        ...Definitions_1.metricTypes.memory,
                        decimal: 0,
                        label: 'Available'
                    },
                    base_memory: {
                        ...Definitions_1.metricTypes.memory,
                        decimal: 0,
                        label: 'Base'
                    },
                    number_of_partitions: {
                        label: 'Number of Partitions'
                    },
                    number_of_lpars: {
                        label: 'Number of LPARs'
                    },
                    base_anyoscores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Base'
                    },
                    base_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Base',
                    },
                    base_core_any_os: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Any OS Base'
                    },
                    base_core_linuxvios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        decimal: 0,
                        label: 'Linux/VIOS Base'
                    },
                },
                metrics: {
                    utilizedProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'utilizedProcUnits',
                        longLabel: 'CPU Pool Configured',
                        label: 'Utilized'
                    },
                    baseAnyOS: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_anyoscores',
                        longLabel: 'Base Any OS Cores',
                        label: 'Base'
                    },
                    proc_installed: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'proc_installed',
                        longLabel: 'CPU Installed',
                        label: 'Installed'
                    },
                    base_memory: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'base_memory',
                        longLabel: 'Base Memory',
                        label: 'Base'
                    },
                    mem_installed: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_installed',
                        longLabel: 'Installed Memory',
                        label: 'Installed'
                    },
                    mem_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_total',
                        longLabel: 'Memory Total',
                        label: 'Memory'
                    },
                    core_aix: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'core_aix',
                        longLabel: 'AIX',
                        label: 'AIX'
                    },
                    base_core_aix: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_aix',
                        longLabel: 'Base AIX',
                        label: 'Base'
                    },
                    core_ibmi: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'core_ibmi',
                        longLabel: 'IBMi',
                        label: 'IBMi'
                    },
                    base_core_ibmi: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_ibmi',
                        longLabel: 'Base IBMi',
                        label: 'Base'
                    },
                    core_other_linux: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'core_other_linux',
                        longLabel: 'Other Linux',
                        label: 'Other Linux'
                    },
                    core_vios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'core_vios',
                        longLabel: 'VIOS',
                        label: 'VIOS'
                    },
                    base_core_linuxvios: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_linuxvios',
                        longLabel: 'Base Linux/VIOS',
                        label: 'Base'
                    },
                    core_rhel: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'core_rhel',
                        longLabel: 'RHEL',
                        label: 'RHEL'
                    },
                    base_core_rhel: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_rhel',
                        longLabel: 'Base Core RHEL',
                        label: 'Base'
                    },
                    core_rhelcoreos: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'core_rhelcoreos',
                        longLabel: 'RHCOS',
                        label: 'RHCOS'
                    },
                    base_core_rhcos: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_rhcos',
                        longLabel: 'Base Core RHCOS',
                        label: 'Base'
                    },
                    core_sles: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'core_sles',
                        longLabel: 'SLES',
                        label: 'SLES'
                    },
                    base_core_sles: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'base_core_sles',
                        longLabel: 'Base Core SLES',
                        label: 'Base'
                    },
                }
            },
            vm: {
                parent: 'server',
                aliases: true,
                configuration: {
                    SharedPoolLabel: {
                        label: 'Shared CPU Pool'
                    },
                    CurrentProcessingUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'CPU Entitled'
                    },
                    CurrentProcessors: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Processors',
                        decimal: 0
                    },
                    DesiredProcessors: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Desired',
                        decimal: 0
                    },
                    DesiredVirtualProcessors: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Desired Virtual',
                        decimal: 0
                    },
                    AllocatedVirtualProcessors: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Allocated Virtual',
                        decimal: 0
                    },
                    RunProcessors: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Run',
                        decimal: 0
                    },
                    CurrentSharingMode: {
                        label: 'Sharing Mode'
                    },
                    CurrentUncappedWeight: {
                        label: 'Uncapped Weight'
                    },
                    CurrentSharedProcessorPoolID: {
                        label: 'Pool ID'
                    },
                    PartitionState: {
                        label: 'State'
                    },
                    SharingMode: {
                        label: 'Mode'
                    },
                    OperatingSystemVersion: {
                        label: 'OS'
                    },
                    CurrentMemory: {
                        ...Definitions_1.metricTypes.memory,
                        label: 'Current Memory',
                        decimal: 0
                    },
                    MinimumMemory: {
                        ...Definitions_1.metricTypes.memory,
                        label: 'Min Memory',
                        decimal: 0
                    },
                    MaximumMemory: {
                        ...Definitions_1.metricTypes.memory,
                        label: 'Max Memory',
                        decimal: 0
                    },
                    SystemTime: {
                        label: 'System Time',
                        unit: subsystemMetricsResponse_constants_1.UNIT.timestamp
                    },
                    SerialNumber: {
                        label: 'Serial',
                        type: 'string'
                    },
                    ResourceMonitoringControlState: {
                        label: 'RMC State'
                    },
                    ResourceMonitoringIPAddress: {
                        label: 'RMC IP'
                    },
                    ResourceMonitoringControlOperatingSystemShutdownCapable: {
                        label: 'RMC OS Shutdown Capable'
                    }
                },
                label: 'LPAR',
                heatmap: [
                    {
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    },
                    {
                        label: 'Memory',
                        key: 'mem_util',
                    },
                ],
                top: {
                    'CPU': ['cpu_utilization_cores', 'cpu_utilization_percent']
                },
                excluded: {
                    reporter: [
                        'cpu_physical_cores_max',
                        'cpu_physical_percent_max',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    all: [
                        'entitledProcUnitsPercent',
                        'logicalMem',
                        'backedPhysicalMem',
                        'currentVirtualProcessors',
                        'entitledProcUnits',
                        'maxProcUnits',
                        'maxVirtualProcessors',
                        'assignedMem',
                        'utilizedMem',
                        'virtualPersistentMem',
                    ],
                    alerting: [
                        'cpu_physical_cores_max',
                        'cpu_physical_percent_max',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_physical_cores_max',
                        'cpu_physical_percent_max',
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ]
                },
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'Usage'
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'Usage (Peak)'
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        longLabel: 'CPU %',
                        label: 'Usage'
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        longLabel: 'CPU % (Peak)',
                        label: 'Usage (Peak)'
                    },
                    cpu_physical_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_physical_cores',
                        longLabel: 'CPU Phys',
                        label: 'Phys'
                    },
                    cpu_physical_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_physical_cores_max',
                        longLabel: 'CPU Phys (Peak)',
                        label: 'Phys (Peak)'
                    },
                    cpu_physical_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_physical_percent',
                        longLabel: 'CPU Phys %',
                        label: 'Phys'
                    },
                    cpu_physical_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_physical_percent_max',
                        longLabel: 'CPU Phys % (Peak)',
                        label: 'Phys (Peak)'
                    },
                    entitledProcUnitsPercent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'entitledProcUnitsPercent',
                        longLabel: 'CPU Entitled %',
                        label: 'Entitled'
                    },
                    logicalMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'logicalMem',
                        longLabel: 'Allocated Memory',
                        label: 'Allocated'
                    },
                    backedPhysicalMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'backedPhysicalMem',
                        longLabel: 'Phys Memory',
                        label: 'Phys'
                    },
                    currentVirtualProcessors: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'currentVirtualProcessors',
                        longLabel: 'CPU Virtual',
                        label: 'Virtual'
                    },
                    entitledProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'entitledProcUnits',
                        longLabel: 'CPU Entitled',
                        label: 'Entitled'
                    },
                    maxProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'maxProcUnits',
                        longLabel: 'CPU Max',
                        label: 'Max'
                    },
                    maxVirtualProcessors: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'maxVirtualProcessors',
                        longLabel: 'CPU Virtual max',
                        label: 'Virtual Max'
                    },
                    assignedMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'assignedMem',
                        longLabel: 'Assigned Memory',
                        label: 'Assigned'
                    },
                    utilizedMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'utilizedMem',
                        longLabel: 'Utilized Memory',
                        label: 'Utilized'
                    },
                    virtualPersistentMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'virtualPersistentMem',
                        longLabel: 'Virtual Persistent Memory',
                        label: 'Virtual Persistent'
                    },
                },
            },
            pool: {
                label: 'Shared CPU Pool',
                configuration: {
                    PoolName: {
                        label: 'Name',
                    },
                    PoolID: {
                        label: 'ID',
                    },
                    PendingReservedProcessingUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Pending Reserved',
                    },
                    MaximumProcessingUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Max',
                    },
                    CurrentReservedProcessingUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Reserved',
                    },
                    AvailableProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        label: 'Available',
                    },
                },
                metrics: {
                    utilizedProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'utilizedProcUnits',
                        longLabel: 'CPU Phys',
                        label: 'Phys'
                    },
                    totalUtilizedProcUnitsPercent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'totalUtilizedProcUnitsPercent',
                        longLabel: 'CPU Phys %',
                        label: 'Phys'
                    },
                    totalUtilizedProcUnitsPercent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'totalUtilizedProcUnitsPercent_max',
                        longLabel: 'CPU Phys % (Peak)',
                        label: 'Phys (Peak)'
                    },
                    utilizedProcUnits_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'utilizedProcUnits_max',
                        longLabel: 'CPU Phys (Peak)',
                        label: 'Phys (Peak)'
                    },
                    assignedProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'assignedProcUnits',
                        longLabel: 'CPU Allocated',
                        label: 'Allocated'
                    },
                    availableProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'availableProcUnits',
                        longLabel: 'CPU Free',
                        label: 'Free'
                    },
                    configuredProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'configuredProcUnits',
                        longLabel: 'CPU Configured',
                        label: 'Configured'
                    },
                    borrowedProcUnits: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'borrowedProcUnits',
                        longLabel: 'CPU Borrowed',
                        label: 'Borrowed'
                    },
                },
                excluded: {
                    reporter: [
                        'totalUtilizedProcUnitsPercent_max',
                        'utilizedProcUnits_max'
                    ],
                    alerting: [
                        'totalUtilizedProcUnitsPercent_max',
                        'utilizedProcUnits_max'
                    ],
                    customGroup: [
                        'totalUtilizedProcUnitsPercent_max',
                        'utilizedProcUnits_max',
                        'borrowedProcUnits',
                        'configuredProcUnits',
                        'availableProcUnits',
                        'assignedProcUnits',
                    ]
                }
            },
            mempool: {
                label: 'Shared Memory Pool',
                excludedTools: [
                    'all'
                ],
                metrics: {
                    mappedIOMemToLpars: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mappedIOMemToLpars',
                        longLabel: 'Memory Mapped To LPAR',
                        label: 'LPAR mem'
                    },
                    totalMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'totalMem',
                        longLabel: 'Memory Total',
                        label: 'Total'
                    },
                    totalIOMem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'totalIOMem',
                        longLabel: 'Memory IO Total',
                        label: 'IO total'
                    },
                    assignedMemToLpars: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'assignedMemToLpars',
                        longLabel: 'Memory Assigned To LPAR',
                        label: 'Assigned LPAR'
                    },
                    assignedMemToSysFirmware: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'assignedMemToSysFirmware',
                        longLabel: 'Memory Assigned To Firmware',
                        label: 'Assigned FW'
                    },
                },
            },
            sea: {
                label: 'SEA',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    receivedBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedBytes',
                        label: 'In',
                        longLabel: 'Data In',
                    },
                    sentBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentBytes',
                        label: 'Out',
                        longLabel: 'Data Out',
                    },
                    receivedPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPackets',
                        longLabel: 'Packets In',
                        label: 'In',
                    },
                    sentPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPackets',
                        label: 'Out',
                        longLabel: 'Packets Out',
                    },
                },
            },
            hea: {
                label: 'HEA',
                excludedTools: [
                    'all'
                ],
                metrics: {
                    receivedBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedBytes',
                        label: 'In',
                        longLabel: 'Data In',
                    },
                    sentBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentBytes',
                        label: 'Out',
                        longLabel: 'Data Out',
                    },
                    receivedPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPackets',
                        label: 'In',
                        longLabel: 'Packets In',
                    },
                    sentPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPackets',
                        label: 'Out',
                        longLabel: 'Packets Out',
                    },
                },
            },
            generic: {
                label: 'LAN',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    receivedBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedBytes',
                        label: 'In',
                        longLabel: 'Data In',
                    },
                    sentBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentBytes',
                        label: 'Out',
                        longLabel: 'Data Out',
                    },
                    receivedPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPackets',
                        label: 'In',
                        longLabel: 'Packets In',
                    },
                    sentPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPackets',
                        label: 'Out',
                        longLabel: 'Packets Out',
                    },
                },
            },
            genericv: {
                label: 'vLAN',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    receivedBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedBytes',
                        label: 'In',
                        longLabel: 'Data In',
                    },
                    sentBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentBytes',
                        label: 'Out',
                        longLabel: 'Data Out',
                    },
                    receivedPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPackets',
                        label: 'In',
                        longLabel: 'Packets In',
                    },
                    sentPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPackets',
                        label: 'Out',
                        longLabel: 'Packets Out',
                    },
                },
            },
            fiber: {
                label: 'SAN',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    readBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'readBytes',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    writeBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'writeBytes',
                        label: 'Write',
                        longLabel: 'Data Received'
                    },
                    numOfReads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfReads',
                        label: 'Read',
                        longLabel: 'IO Read',
                    },
                    numOfWrites: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfWrites',
                        label: 'Write',
                        longLabel: 'IO Write',
                    },
                },
            },
            genericphysical: {
                label: 'SAS',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    readBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'readBytes',
                        label: 'Read',
                        longLabel: 'Bytes Read',
                    },
                    writeBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'writeBytes',
                        label: 'Write',
                        longLabel: 'Bytes Write',
                    },
                    numOfReads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfReads',
                        label: 'Read',
                        longLabel: 'IO Read',
                    },
                    numOfWrites: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfWrites',
                        label: 'Write',
                        longLabel: 'IO Write',
                    },
                },
            },
            genericvirtual: {
                label: 'vSCSI',
                excludedTools: [
                    'customGroup'
                ],
                configuration: {
                    BackingDevice: {
                        label: 'Backing Device'
                    },
                    BackingDeviceName: {
                        label: 'Backing Device'
                    },
                    AdapterName: {
                        label: 'Adapter'
                    },
                    VirtualSlotNumber: {
                        label: 'Virtual Slot',
                        type: 'number'
                    },
                    DiskCapacity: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        label: 'Disk Capacity',
                    },
                    DiskLabel: {
                        label: 'Disk Label'
                    },
                    DiskName: {
                        label: 'Disk Name'
                    },
                    LocalPartitionID: {
                        label: 'LPAR ID'
                    },
                    LocationCode: {
                        label: 'Location Code'
                    },
                    RemoteLogicalPartitionID: {
                        label: 'Remote LPAR ID',
                    },
                    RemoteSlotNumber: {
                        label: 'Remote Slot',
                        type: 'number'
                    },
                    RequiredAdapter: {
                        label: 'Req. Adapter'
                    },
                    ServerLocationCode: {
                        label: 'Server Location Code'
                    },
                    TargetName: {
                        label: 'Target Name'
                    },
                    PartitionName: {
                        label: 'LPAR'
                    }
                },
                metrics: {
                    readBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'readBytes',
                        label: 'Read',
                        longLabel: 'Bytes Read',
                    },
                    writeBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'writeBytes',
                        label: 'Write',
                        longLabel: 'Bytes Write',
                    },
                    numOfReads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfReads',
                        label: 'Read',
                        longLabel: 'IO Read',
                    },
                    numOfWrites: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfWrites',
                        label: 'Write',
                        longLabel: 'IO Write',
                    },
                },
            },
            virtualfibrenpiv: {
                label: 'NPIV',
                excludedTools: [
                    'all'
                ],
                configuration: {
                    AdapterName: {
                        label: 'Adapter'
                    },
                    ConnectingPartitionID: {
                        label: 'LPAR ID'
                    },
                    RequiredAdapter: {
                        label: 'Required Adapter'
                    },
                    AdapterType: {
                        label: 'Adapter Type'
                    },
                    LocationCode: {
                        label: 'Location Code'
                    },
                    MapPort: {
                        label: 'Port'
                    },
                    DynamicReconfigurationConnectorName: {
                        label: "Dynamic reconfiguration connector"
                    },
                    VariedOn: {
                        label: 'Varied On'
                    },
                    LocalPartitionID: {
                        label: 'Local LPAR'
                    },
                    UniqueDeviceID: {
                        label: 'Device ID'
                    },
                    VirtualSlotNumber: {
                        label: 'Virtual Slot',
                        type: 'number'
                    },
                    ConnectingVirtualSlotNumber: {
                        label: 'Connecting Slot'
                    },
                    AvailablePorts: {
                        label: 'Available Ports'
                    },
                    TotalPorts: {
                        label: 'Total Ports'
                    },
                    PortName: {
                        label: 'Port'
                    },
                    PartitionName: {
                        label: "LPAR"
                    },
                    WWPNs: {
                        label: 'NPIV WWPNs'
                    },
                    WWPN: {
                        label: 'Physical WWPN'
                    },
                    viosName: {
                        label: 'VIOS'
                    },
                },
            },
            sriovphysical: {
                label: 'SR-IOV Physical',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    receivedBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedBytes',
                        label: 'In',
                        longLabel: 'Data In',
                    },
                    sentBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentBytes',
                        label: 'Out',
                        longLabel: 'Data Out',
                    },
                    receivedPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPackets',
                        label: 'In',
                        longLabel: 'Packets In',
                    },
                    sentPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPackets',
                        label: 'Out',
                        longLabel: 'Packets Out',
                    },
                },
            },
            sriovlogical: {
                label: 'SR-IOV Logical',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    receivedBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedBytes',
                        label: 'In',
                        longLabel: 'Data In',
                    },
                    sentBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentBytes',
                        label: 'Out',
                        longLabel: 'Data Out',
                    },
                    receivedPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPackets',
                        label: 'In',
                        longLabel: 'Packets Out',
                    },
                    sentPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPackets',
                        label: 'In',
                        longLabel: 'Packets Out',
                    },
                },
            },
            virtualethernet: {
                label: 'Virtual Ethernet',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    receivedBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedBytes',
                        label: 'In',
                        longLabel: 'Data In',
                    },
                    sentBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentBytes',
                        label: 'Out',
                        longLabel: 'Data Out',
                    },
                    receivedPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPackets',
                        label: 'In',
                        longLabel: 'Packets In',
                    },
                    sentPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPackets',
                        label: 'Out',
                        longLabel: 'Packets Out',
                    },
                    receivedPhysicalBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'receivedPhysicalBytes',
                        label: 'In',
                        longLabel: 'Bytes Physical In',
                    },
                    sentPhysicalBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'sentPhysicalBytes',
                        label: 'Out',
                        longLabel: 'Bytes Physical Sent',
                    },
                    receivedPhysicalPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'receivedPhysicalPackets',
                        label: 'In',
                        longLabel: 'Packets Physical In',
                    },
                    sentPhysicalPackets: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'sentPhysicalPackets',
                        label: 'Out',
                        longLabel: 'Packets Physical Out',
                    },
                },
            },
            sharedstoragepool: {
                label: "Shared Storage Pool",
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    readBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'readBytes',
                        label: 'Read',
                        longLabel: 'Bytes Read',
                    },
                    writeBytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'writeBytes',
                        label: 'Write',
                        longLabel: 'Bytes Write',
                    },
                    numOfReads: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfReads',
                        label: 'Read',
                        longLabel: 'IO Read',
                    },
                    numOfWrites: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'numOfWrites',
                        label: 'Write',
                        longLabel: 'IO Write',
                    },
                },
            },
            physicalvolume: {
                excludedTools: [
                    'all'
                ],
                label: 'Physical Volume',
                configuration: {
                    VolumeName: {
                        label: "Disk name"
                    },
                    VolumeState: {
                        label: "Disk state"
                    },
                    VolumeCapacity: {
                        ...Definitions_1.metricTypes.capacity_volume,
                        label: "Disk capacity",
                    },
                    LocationCode: {
                        label: "Location"
                    },
                    Description: {
                        label: "Description"
                    },
                }
            },
            trunk: {
                excludedTools: [
                    'all'
                ],
                label: 'Trunk',
                configuration: {
                    DeviceName: {
                        label: "Device name"
                    },
                    LocationCode: {
                        label: "Location code"
                    },
                    LocalPartitionID: {
                        label: "Local partition"
                    },
                    VirtualSlotNumber: {
                        label: "Virtual slot",
                        type: 'number'
                    },
                    PortVLANID: {
                        label: "VLAN"
                    },
                    VirtualSwitchID: {
                        label: "Switch ID"
                    },
                    MACAddress: {
                        label: "MAC address"
                    },
                    AllowedOperatingSystemMACAddresses: {
                        label: "Allowed MAC addresses"
                    },
                    LocalPartitionLabel: {
                        label: "LPAR"
                    },
                }
            },
            backingsea: {
                excludedTools: [
                    'all'
                ],
                label: 'Backing SEA',
                configuration: {
                    DeviceName: {
                        label: "Device name"
                    },
                    InterfaceName: {
                        label: "Interface name"
                    },
                    AdapterID: {
                        label: "Adapter ID"
                    },
                    State: {
                        label: "State"
                    },
                    DeviceType: {
                        label: "Device type"
                    },
                    PhysicalLocation: {
                        label: "Physical location"
                    },
                    Description: {
                        label: "Description"
                    },
                    UniqueDeviceID: {
                        label: "Device ID"
                    },
                }
            },
            freelinkaggregation: {
                excludedTools: [
                    'all'
                ],
                label: 'Link Aggregation',
                configuration: {
                    DeviceName: {
                        label: "Device name"
                    },
                    AdapterID: {
                        label: "Adapter ID"
                    },
                    DeviceType: {
                        label: "Device type"
                    },
                    DynamicReconfigurationConnectorName: {
                        label: "Dynamic reconfiguration connector"
                    },
                    Description: {
                        label: "Description"
                    },
                    UniqueDeviceID: {
                        label: "Device ID"
                    },
                    PhysicalLocation: {
                        label: "Physical location"
                    },
                }
            },
            ioadapter: {
                excludedTools: [
                    'all'
                ],
                label: 'IO Adapter',
                configuration: {
                    DeviceName: {
                        label: "Device name"
                    },
                    AdapterID: {
                        label: "Adapter ID"
                    },
                    DeviceType: {
                        label: "Device type"
                    },
                    DynamicReconfigurationConnectorName: {
                        label: "Dynamic reconfiguration connector"
                    },
                    Description: {
                        label: "Description"
                    },
                    UniqueDeviceID: {
                        label: "Device ID"
                    },
                    PhysicalLocation: {
                        label: "Physical location"
                    },
                }
            },
            ioslot: {
                excludedTools: [
                    'all'
                ],
                label: 'IO slot',
                configuration: {
                    Description: {
                        label: "Description"
                    },
                    IOUnitPhysicalLocation: {
                        label: "IO physical location"
                    },
                    SlotPhysicalLocationCode: {
                        label: "Slot physical location"
                    },
                    PCAdapterID: {
                        label: "Adapter ID",
                    },
                    PCIDeviceID: {
                        label: "Device ID"
                    },
                    PartitionName: {
                        label: "LPAR"
                    },
                    PartitionType: {
                        label: "LPAR Type"
                    },
                    SlotDynamicReconfigurationConnectorName: {
                        label: "Connector name"
                    },
                }
            },
            iobus: {
                excludedTools: [
                    'all'
                ],
                label: 'IO Bus',
                configuration: {
                    BackplanePhysicalLocation: {
                        label: "Physical location"
                    },
                    BusDynamicReconfigurationConnectorIndex: {
                        label: "Connector index"
                    },
                    BusDynamicReconfigurationConnectorName: {
                        label: "Connector name"
                    },
                    IOBusID: {
                        label: "Bus ID"
                    },
                }
            },
            profileioslot: {
                excludedTools: [
                    'all'
                ],
                label: 'Profile IO slot',
                configuration: {
                    IOUnitPhysicalLocation: {
                        label: "Physical location"
                    },
                    SlotPhysicalLocationCode: {
                        label: "Location code"
                    },
                    PCAdapterID: {
                        label: "Adapter ID"
                    },
                    PartitionName: {
                        label: "LPAR"
                    },
                    SlotDynamicReconfigurationConnectorIndex: {
                        label: "Connector index"
                    },
                    SlotDynamicReconfigurationConnectorName: {
                        label: "Connector name"
                    },
                    Description: {
                        label: "Description"
                    }
                }
            },
        },
    },
    vmware: {
        label: 'VMware',
        order: 2,
        subsystems: {
            vm: {
                label: 'VM',
                aliases: true,
                top: {
                    'CPU': ['cpu_usage', 'cpu_utilization_percent'],
                    'Memory': ['memory_active', 'memory_granted'],
                    'Data': ['disk_usage'],
                    'Net': ['network_usage'],
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    },
                    {
                        label: 'Memory',
                        key: 'memory_proc',
                        total: 'memory_granted',
                        used: 'memory_active'
                    }],
                configuration: {
                    IpAddress: {
                        label: 'IP Adress'
                    },
                    vCPU: {
                        label: 'vCPU'
                    },
                    Reserved_MHz: {
                        label: 'Reserved',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
                        decimal: 0
                    },
                    Limit_MHz: {
                        label: 'Limit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
                        decimal: 0
                    },
                    Shares: {
                        label: 'Shares'
                    },
                    Shares_value: {
                        label: 'Shares value'
                    },
                    Provisioned_Space_GB: {
                        label: 'Provisioned Space',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 1
                    },
                    Used_Space_GB: {
                        label: 'Used Space',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 1
                    },
                    powerState: {
                        label: 'Power State'
                    },
                    toolsStatus: {
                        label: 'Tools Status'
                    },
                    OS: {
                        label: 'OS'
                    }
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_usage_max',
                    ],
                    all: [
                        'cpu_alloc',
                        'host_hz',
                        'memory_compres',
                        'memory_decompres',
                        'vcpu'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_usage_max',
                        'memory_granted'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_usage_max',
                        'memory_granted'
                    ]
                },
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'CPU',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'CPU (Peak)',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        label: 'CPU %',
                        longLabel: 'CPU %',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU (Peak)',
                        longLabel: 'CPU % (Peak)',
                    },
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_usage',
                        label: 'CPU Hz',
                        longLabel: 'CPU Hz',
                    },
                    cpu_usage_max: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_usage_max',
                        label: 'CPU (Peak)',
                        longLabel: 'CPU Hz (Peak)',
                    },
                    vcpu: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'vcpu',
                        label: 'vCPU',
                    },
                    vm_cpu_ready: {
                        metric: 'vm_cpu_ready',
                        longLabel: 'CPU Ready/VM',
                        label: 'Ready/VM',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    cpu_ready: {
                        metric: 'cpu_ready',
                        longLabel: 'CPU Ready/vCPU',
                        label: 'Ready/vCPU',
                        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
                    },
                    cpu_alloc: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_alloc',
                        longLabel: 'CPU Reserved',
                        label: 'Reserved',
                    },
                    host_hz: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'host_hz',
                        label: 'Host',
                    },
                    memory_active: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_active',
                        label: 'Active',
                        longLabel: 'Memory Active'
                    },
                    memory_granted: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_granted',
                        label: 'Granted',
                        longLabel: 'Memory Granted'
                    },
                    memory_baloon: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_baloon',
                        label: 'Baloon',
                        longLabel: 'Memory Baloon'
                    },
                    disk_usage: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_usage',
                        longLabel: 'Data Rate',
                        label: 'Data Rate',
                    },
                    disk_read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_read',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    disk_write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_write',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    network_usage: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_usage',
                        longLabel: 'LAN',
                        label: 'Usage',
                    },
                    network_received: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_received',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    network_transmitted: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_transmitted',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                    memory_swapin: {
                        metric: 'memory_swapin',
                        longLabel: 'Swap In',
                        label: 'In',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    memory_swapout: {
                        metric: 'memory_swapout',
                        longLabel: 'Swap Out',
                        label: 'Out',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    memory_compres: {
                        metric: 'memory_compres',
                        longLabel: 'Memory Compress',
                        label: 'Compress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                    memory_decompres: {
                        metric: 'memory_decompres',
                        longLabel: 'Memory Decompress',
                        label: 'Decompress',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
                    },
                }
            },
            vm_folder: {
                label: "VM Folder",
                excludedTools: [
                    'all'
                ],
            },
            datastore_vm: {
                label: 'Datastore VM',
                configuration: {
                    provisioned: {
                        label: 'Provisioned Space',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    used: {
                        label: 'Used Space',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                },
                excludedTools: [
                    'all'
                ],
                metrics: {
                    iops_read: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'iops_read',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    iops_write: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'iops_write',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    provisioned_space: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'provisioned_space',
                        label: 'Provisioned',
                        longLabel: 'Provisioned Space'
                    },
                    used_space: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_space',
                        label: 'Used',
                        longLabel: 'Used Space'
                    }
                }
            },
            esxi_vm: {
                label: 'ESXI VM',
                excludedTools: [
                    'all'
                ],
                metrics: {
                    cpu_usage_proc: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_usage_proc',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    start_or_vmotion: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'start_or_vmotion',
                        label: 'start_or_vmotion',
                    }
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_percent_max',
                        'cpu_usage_max',
                    ],
                    alerting: [
                        'cpu_utilization_percent_max',
                        'cpu_usage_max',
                    ],
                    customGroup: [
                        'cpu_utilization_percent_max',
                        'cpu_usage_max',
                    ]
                }
            },
            vcenter: {
                label: 'vCenter',
                excludedTools: [
                    'customGroup',
                    'alerting'
                ],
                configuration: {
                    alias: {
                        label: 'Alias'
                    },
                    clusters: {
                        label: 'Clusters'
                    },
                    esxis: {
                        label: 'ESXi'
                    },
                    fullName_top: {
                        label: 'Version'
                    },
                    host: {
                        label: 'Host'
                    },
                    vcenter_ghz: {
                        label: 'CPU total',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.giga,
                        decimal: 0
                    },
                    vcenter_last_update: {
                        label: 'Last updated'
                    },
                    vcenter_memorySize_gb: {
                        label: 'Memory total',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    vcenter_overallCpuUsage_ghz: {
                        label: 'CPU used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.giga,
                        decimal: 0
                    },
                    vcenter_vms_count: {
                        label: 'VM'
                    },
                    vcenter_overallMemoryUsage_gb: {
                        label: 'Memory used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    }
                },
            },
            datastore: {
                label: 'Datastore',
                aliases: true,
                top: {
                    'IO': ['read', 'write'],
                    'Data': ['read_avg', 'write_avg']
                },
                configuration: {
                    ds_used: {
                        label: 'DS used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    ds_freeSpace: {
                        label: 'DS free space',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    ds_provisioned: {
                        label: 'DS Provisioned',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    disk_uids: {
                        label: 'Volume Id'
                    }
                },
                metrics: {
                    read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    read_avg: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'read_avg',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    write_avg: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'write_avg',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    read_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'read_latency',
                        longLabel: 'Latency Read',
                        label: 'Read',
                    },
                    write_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'write_latency',
                        longLabel: 'Latency Write',
                        label: 'Write',
                    },
                    disk_capacity: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'disk_capacity',
                        longLabel: 'Disk Total',
                        label: 'Total',
                    },
                    disk_used: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'disk_used',
                        longLabel: 'Disk Used',
                        label: 'Used',
                    },
                    disk_provisioned: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'disk_provisioned',
                        longLabel: 'Disk Provisioned',
                        label: 'Provisioned',
                    },
                }
            },
            esxi: {
                label: 'ESXi',
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    }],
                configuration: {
                    vcente_name: {
                        label: 'vCenter'
                    },
                    cluste_name: {
                        label: 'Cluster'
                    },
                    host_ghz: {
                        label: 'CPU total',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.giga,
                        decimal: 0
                    },
                    overallCpuUsage_ghz: {
                        label: 'CPU used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.giga,
                        decimal: 0
                    },
                    host_memorySize_gb: {
                        label: 'Memory total',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    overallMemoryUsage_gb: {
                        label: 'Memory used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 0
                    },
                    hw_numCpuCores: {
                        label: 'Cores'
                    },
                    hw_numCpuThreads: {
                        label: 'Threads'
                    },
                    sw_fullName: {
                        label: 'Version'
                    },
                    esxi_uptime_days: {
                        label: 'Uptime',
                        decimal: 0,
                        unit: subsystemMetricsResponse_constants_1.UNIT.days
                    },
                    hw_vendor: {
                        label: 'Vendor'
                    },
                    hw_model: {
                        label: 'Model'
                    },
                    hw_cpuModel: {
                        label: 'Processor'
                    }
                },
                excluded: {
                    all: [
                        'cpu_alloc',
                        'host_hz',
                        'memory_host_size',
                        'memory_granted'
                    ]
                },
                metrics: {
                    cpu_alloc: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        defaultPrecision: 0,
                        metric: 'cpu_alloc',
                        longLabel: 'CPU Total',
                        label: 'Total',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        longLabel: 'CPU %',
                        label: 'Usage',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        longLabel: 'CPU % (Peak)',
                        label: 'CPU (Peak)',
                    },
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        defaultPrecision: 0,
                        metric: 'cpu_usage',
                        longLabel: 'CPU Hz',
                        label: 'CPU Hz',
                    },
                    cpu_usage_max: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        defaultPrecision: 0,
                        metric: 'cpu_usage_max',
                        longLabel: 'CPU (Peak)',
                        label: 'CPU (Peak)',
                    },
                    host_hz: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        defaultPrecision: 0,
                        metric: 'host_hz',
                        longLabel: 'CPU Host',
                        label: 'Host',
                    },
                    memory_active: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_active',
                        longLabel: 'Memory Active',
                        label: 'Active',
                    },
                    memory_host_size: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_host_size',
                        longLabel: 'Memory Host Size',
                        label: 'Host Size',
                    },
                    memory_granted: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_granted',
                        label: 'Granted',
                        longLabel: 'Memory Granted',
                    },
                    memory_baloon: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_baloon',
                        label: 'Baloon',
                        longLabel: 'Memory Baloon',
                    },
                    disk_read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_read',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    disk_write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_write',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    network_received: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_received',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    network_transmitted: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_transmitted',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                    power_usage: {
                        ...Definitions_1.metricTypes.power,
                        metric: 'power_usage',
                        longLabel: 'Power Usage',
                        label: 'Usage',
                    }
                }
            },
            cluster: {
                label: 'Cluster',
                excluded: {
                    all: [
                        'cpu_total',
                        'memory_total',
                        'total_cluster_cores'
                    ],
                },
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'CPU',
                    },
                    total_cluster_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'total_cluster_cores',
                        longLabel: 'Total',
                        label: 'Total',
                    },
                    cpu_usage_proc: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_usage_proc',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_usage',
                        longLabel: 'CPU Hz',
                        label: 'CPU Hz',
                    },
                    cpu_total_eff: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cluster_eff_cpu',
                        longLabel: 'CPU Total Effective',
                        label: 'Total Effective',
                    },
                    cpu_total: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_total',
                        longLabel: 'CPU Total',
                        label: 'Total',
                    },
                    memory_consumed: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_consumed',
                        longLabel: 'Memory Consumed',
                        label: 'Consumed',
                    },
                    memory_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_total',
                        label: 'Total',
                        longLabel: 'Memory Total'
                    },
                    memory_active: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_active',
                        label: 'Active',
                        longLabel: 'Memory Active'
                    },
                    memory_granted: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_granted',
                        label: 'Granted',
                        longLabel: 'Memory Granted'
                    },
                    memory_baloon: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_baloon',
                        label: 'Baloon',
                        longLabel: 'Memory Baloon'
                    },
                    memory_swap: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_swap',
                        label: 'Swap Out',
                        longLabel: 'Memory Swap Out'
                    },
                }
            },
            respool: {
                label: 'Resource pool',
                parent: 'cluster',
                configuration: {
                    cpu_reservation: {
                        label: 'CPU Reservation',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
                        decimal: 0
                    },
                    cpu_limit: {
                        label: 'CPU Limit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
                        decimal: 0
                    },
                    cpu_alloc_type: {
                        label: 'CPU Allocation Type'
                    },
                    cpu_shares: {
                        label: 'CPU Shares'
                    },
                    cpu_value: {
                        label: 'CPU Shares Value'
                    },
                    mem_reservation: {
                        label: 'Memory Reservation',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
                        decimal: 0
                    },
                    mem_limit: {
                        label: 'Memory Limit',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
                        decimal: 0
                    },
                    mem_alloc_type: {
                        label: 'Memory Allocation Type'
                    },
                    mem_shares: {
                        label: 'Memory Shares'
                    },
                    mem_value: {
                        label: 'Memory Shares Value'
                    }
                },
                excluded: {
                    all: [
                        'cpu_reservation',
                        'cpu_limit',
                        'memory_reservation',
                        'memory_limit',
                        'memory_granted'
                    ]
                },
                metrics: {
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_usage',
                        longLabel: 'CPU usage',
                        label: 'Usage',
                    },
                    cpu_reservation: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_reservation',
                        longLabel: 'CPU reservation',
                        label: 'Reservation',
                    },
                    cpu_limit: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'cpu_limit',
                        longLabel: 'CPU limit',
                        label: 'Limit',
                    },
                    memory_reservation: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_reservation',
                        label: 'Reservation',
                        longLabel: 'Memory Reservation',
                    },
                    memory_limit: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_limit',
                        label: 'Limit',
                        longLabel: 'Memory Limit',
                    },
                    memory_consumed: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_consumed',
                        label: 'Consumed',
                        longLabel: 'Memory Consumed',
                    },
                    memory_active: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_active',
                        label: 'Active',
                        longLabel: 'Memory Active'
                    },
                    memory_granted: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_granted',
                        label: 'Granted',
                        longLabel: 'Memory Granted',
                    },
                    memory_baloon: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_baloon',
                        label: 'Baloon',
                        longLabel: 'Memory Baloon',
                    },
                    memory_swap: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_swap',
                        label: 'Swap Out',
                        longLabel: 'Memory Swap out',
                    }
                }
            }
        }
    },
    ovirt: {
        label: 'oVirt',
        order: 6,
        subsystems: {
            datacenter: {
                label: 'Datacenter',
                excludedTools: [
                    'all'
                ],
            },
            cluster: {
                label: 'Cluster',
                excludedTools: [
                    'all'
                ],
            },
            host: {
                label: 'Host',
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    }, {
                        label: 'Memory',
                        key: 'mem_percent',
                        percent: 'mem_percent'
                    }],
                metrics: {
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU usage',
                        label: 'Usage',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU usage (Peak)',
                        label: 'Usage (Peak)',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU (Peak)',
                        longLabel: 'CPU % (Peak)',
                    },
                    user_cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'user_cpu_utilization_percent',
                        longLabel: 'CPU user',
                        label: 'User',
                    },
                    system_cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'system_cpu_utilization_percent',
                        longLabel: 'CPU system',
                        label: 'System',
                    },
                    ksm_cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'ksm_cpu_utilization_percent',
                        longLabel: 'CPU KSM',
                        label: 'KSM',
                    },
                    cpu_load: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_load',
                        longLabel: 'CPU load',
                        label: 'Load',
                    },
                    totals_vms_vcpus: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'totals_vms_vcpus',
                        longLabel: 'Total VM vCPU',
                        label: 'VM vCPU',
                    },
                    cpu_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores',
                        longLabel: 'CPU total',
                        label: 'Total',
                    },
                    mem_percent: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'mem_percent',
                        longLabel: 'Memory Used %',
                        label: 'Used %',
                    },
                    used_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_mem',
                        longLabel: 'Memory used',
                        label: 'Used',
                    },
                    free_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'free_mem',
                        longLabel: 'Memory free',
                        label: 'Free',
                    },
                    ksm_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'ksm_mem',
                        longLabel: 'Memory KSM',
                        label: 'KSM',
                    },
                    used_swap: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_swap',
                        longLabel: 'Swap used',
                        label: 'Swap used',
                    }
                },
                configuration: {
                    fqdn_or_ip: {
                        label: 'FQDN or IP'
                    },
                    memory_size: {
                        label: 'Memory',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 1
                    },
                    swap_size: {
                        label: 'Swap',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi,
                        decimal: 1
                    },
                    cpu_model: {
                        label: 'CPU model'
                    },
                    cpu_speed: {
                        label: 'CPU speed',
                        unit: subsystemMetricsResponse_constants_1.UNIT.hertz,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.mega,
                        decimal: 0
                    },
                    number_of_cores: {
                        label: 'Number of cores',
                        decimal: 1
                    },
                    number_of_sockets: {
                        label: 'Number of sockets',
                        decimal: 1
                    },
                    host_os: {
                        label: 'OS'
                    },
                    kernel_version: {
                        label: 'Kernel version'
                    },
                    kvm_version: {
                        label: 'KVM version'
                    },
                    threads_per_core: {
                        label: 'Threads per core'
                    },
                    hardware_product_name: {
                        label: 'Hardware product name'
                    },
                    hardware_serial_number: {
                        label: 'Hardware serial number'
                    }
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_cores'
                    ]
                }
            },
            host_nic: {
                label: 'Host NIC',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    network_received: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_received',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    network_transmitted: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_transmitted',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                }
            },
            vm: {
                label: 'VM',
                aliases: true,
                top: {
                    'CPU': ['cpu_utilization_cores', 'cpu_utilization_percent'],
                    'Memory': ['used_mem'],
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    }, {
                        label: 'Memory',
                        key: 'mem_percent',
                        percent: 'mem_percent'
                    }],
                metrics: {
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU usage',
                        label: 'Usage',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU usage (Peak)',
                        label: 'Usage (Peak)',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU (Peak)',
                        longLabel: 'CPU % (Peak)',
                    },
                    user_cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'user_cpu_utilization_percent',
                        longLabel: 'CPU user',
                        label: 'User',
                    },
                    system_cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'system_cpu_utilization_percent',
                        longLabel: 'CPU System',
                        label: 'System',
                    },
                    cpu_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores',
                        longLabel: 'CPU total',
                        label: 'Total',
                    },
                    mem_percent: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'mem_percent',
                        label: 'Used %',
                        longLabel: 'Memory Used %',
                    },
                    used_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_mem',
                        longLabel: 'Memory used',
                        label: 'Used',
                    },
                    free_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'free_mem',
                        longLabel: 'Memory free',
                        label: 'Free',
                    },
                    buffered_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'buffered_mem',
                        longLabel: 'Memory buffered',
                        label: 'Buffered',
                    },
                    cached_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'cached_mem',
                        longLabel: 'Memory cached',
                        label: 'Cached',
                    },
                },
                configuration: {
                    vm_type: {
                        label: 'VM type'
                    },
                    memory_size: {
                        label: 'Memory',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi
                    },
                    cpu_per_socket: {
                        label: 'CPU per socket'
                    },
                    number_of_sockets: {
                        label: 'Number of sockets'
                    },
                    operating_system: {
                        label: 'OS'
                    }
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_cores'
                    ]
                }
            },
            vm_nic: {
                label: 'VM NIC',
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    network_received: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_received',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    network_transmitted: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_transmitted',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                }
            },
            disk: {
                label: 'Disk',
                metrics: {
                    disk_write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_write',
                        longLabel: 'Disk write',
                        label: 'Write',
                    },
                    disk_read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_read',
                        longLabel: 'Disk read',
                        label: 'Read',
                    },
                    disk_write_iops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'disk_write_iops',
                        longLabel: 'Disk write IOPS',
                        label: 'Write IOPS',
                    },
                    disk_read_iops: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'disk_read_iops',
                        longLabel: 'Disk read IOPS',
                        label: 'Read IOPS',
                    },
                    disk_write_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'disk_write_latency',
                        longLabel: 'Disk write latency',
                        label: 'Write latency',
                    },
                    disk_read_latency: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'disk_read_latency',
                        longLabel: 'Disk read latency',
                        label: 'Read latency',
                    },
                    size: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'size',
                        longLabel: 'Disk size',
                        label: 'Size',
                    }
                },
                configuration: {
                    vm_disk_size: {
                        label: 'Size',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi
                    }
                }
            },
            storage_domain: {
                label: 'Storage domain',
                metrics: {
                    used: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'used',
                        longLabel: 'Capacity used',
                        label: 'Used',
                    },
                    total: {
                        ...Definitions_1.metricTypes.capacity,
                        metric: 'total',
                        longLabel: 'Capacity total',
                        label: 'Total',
                    },
                },
                configuration: {
                    free_disk_size: {
                        label: 'Free',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi
                    },
                    used_disk_size: {
                        label: 'Used',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi
                    },
                    total_disk_size: {
                        label: 'Size',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi
                    },
                }
            }
        }
    },
    xenserver: {
        label: 'XenServer',
        order: 7,
    },
    windows: {
        label: 'Windows / Hyper-V',
        order: 8,
        subsystems: {
            server: {
                label: 'Server',
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_usage_perc',
                        total: 'vCPU',
                        used: 'cpu_usage'
                    },
                    {
                        label: 'Memory',
                        key: 'memory_percent',
                        percent: 'memory_percent'
                    }],
                configuration: {
                    version: {
                        label: "version"
                    },
                },
                metrics: {
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_usage',
                        longLabel: 'CPU',
                        label: 'Usage',
                    },
                    cpu_usage_perc: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_usage_perc',
                        longLabel: 'CPU %',
                        label: 'Usage',
                    },
                    host_hz: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'host_hz',
                        longLabel: 'CPU Host Usage Hz',
                        label: 'Host Usage',
                    },
                    memory_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_total',
                        longLabel: 'Memory Host Size',
                        label: 'Host Size',
                    },
                    avail_m_bytes: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'avail_m_bytes',
                        longLabel: 'Memory Free',
                        label: 'Free',
                        defaultPrecision: 2,
                    },
                    memory_available: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_available',
                        longLabel: 'Memory Available',
                        label: 'Available',
                    },
                    memory_percent: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'memory_percent',
                        longLabel: 'Memory %',
                        label: 'Usage',
                    },
                    cache_bytes: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'cache_bytes',
                        longLabel: 'Memory Cache',
                        label: 'Cache',
                        defaultPrecision: 2,
                    },
                    used_bytes: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_bytes',
                        longLabel: 'Memory Used',
                        label: 'Used',
                        defaultPrecision: 1,
                    },
                    disk_bytes_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_bytes_persec',
                        longLabel: 'Data Rate',
                        label: 'Data Rate',
                    },
                    disk_read_bytes_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_read_bytes_persec',
                        defaultPrecision: 2,
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    disk_write_bytes_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_write_bytes_persec',
                        defaultPrecision: 2,
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    disk_reads_persec: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'disk_reads_persec',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    disk_writes_persec: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'disk_writes_persec',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    bytes_total_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_total_persec',
                        longLabel: 'LAN',
                        label: 'Usage',
                    },
                    bytes_received_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_received_persec',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    bytes_sent_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_sent_persec',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                    disk_tranfers_persec: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'disk_tranfers_persec',
                        longLabel: 'Disk Usage %',
                        label: 'Usage',
                    },
                    pages_input_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'pages_input_persec',
                        longLabel: 'Page in',
                        label: 'In',
                    },
                    pages_output_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'pages_output_persec',
                        longLabel: 'Page out',
                        label: 'Out',
                    },
                    vCPU: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'vCPU',
                        label: 'Logical Processors',
                    },
                    ProcessorQueueLength: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'ProcessorQueueLength',
                        label: 'Processor Queue Length',
                    },
                    processes: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'processes',
                        label: 'Processes',
                    },
                    threads: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'threads',
                        label: 'Threads',
                    },
                    SystemCallsPersec: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'SystemCallsPersec',
                        label: 'SystemCallsPersec',
                    },
                },
                excluded: {
                    reporter: [
                        'cpu_usage_max',
                        'cpu_usage_perc_max'
                    ],
                    alerting: [
                        'cpu_usage_max',
                        'cpu_usage_perc_max'
                    ],
                    customGroup: [
                        'cpu_usage_max',
                        'cpu_usage_perc_max',
                        'vCPU',
                        'ProcessorQueueLength',
                        'processes',
                        'threads',
                        'SystemCallsPersec'
                    ],
                }
            },
            vm: {
                label: 'VM',
                heatmap: [
                    {
                        label: 'CPU',
                        key: 'cpu_usage_proc',
                        total: 'vmCPU',
                        used: 'cpu_utilization_cores'
                    },
                    {
                        label: 'Memory',
                        key: 'memory_proc',
                        total: 'memory_total',
                        used: 'memory_used'
                    }
                ],
                configuration: {
                    version: {
                        label: "version"
                    },
                },
                top: {
                    'CPU': ['cpu_utilization_cores'],
                    'Memory': ['memory_used', 'memory_total'],
                    'Data': ['disk_bytes_persec'],
                    'LAN': ['bytes_total_persec']
                },
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        longLabel: 'CPU',
                        label: 'Usage',
                    },
                    cpu_usage_perc: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_usage_perc',
                        longLabel: 'CPU %',
                        label: 'CPU %',
                    },
                    host_hz: {
                        ...Definitions_1.metricTypes.cpu_ghz,
                        metric: 'host_hz',
                        longLabel: 'CPU Host Hz',
                        label: 'Host Usage',
                    },
                    memory_total: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_total',
                        longLabel: 'Memory Total',
                        label: 'Total',
                        defaultPrecision: 1,
                    },
                    memory_used: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'memory_used',
                        longLabel: 'Memory Used',
                        label: 'Used',
                        defaultPrecision: 1,
                    },
                    disk_bytes_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_bytes_persec',
                        longLabel: 'Data Rate',
                        label: 'Data Rate',
                    },
                    disk_read_bytes_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_read_bytes_persec',
                        defaultPrecision: 2,
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    disk_write_bytes_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_write_bytes_persec',
                        defaultPrecision: 2,
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    disk_reads_persec: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'disk_reads_persec',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    disk_writes_persec: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'disk_writes_persec',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    bytes_total_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_total_persec',
                        longLabel: 'LAN',
                        label: 'Usage',
                    },
                    bytes_received_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_received_persec',
                        longLabel: 'LAN In',
                        label: 'In',
                    },
                    bytes_sent_persec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_sent_persec',
                        longLabel: 'LAN Out',
                        label: 'Out',
                    },
                    disk_tranfers_persec: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'disk_tranfers_persec',
                        longLabel: 'Disk Usage %',
                        label: 'Usage',
                    },
                    vmCPU: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'vmCPU',
                        label: 'Logical Processors',
                    },
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'vmCPU'
                    ]
                }
            },
            job: {
                label: 'Job',
                excludedTools: [
                    'all'
                ],
                configuration: {
                    ip: {
                        label: "IP"
                    },
                },
                metrics: {
                    time_1_sec: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'time_1_sec',
                        label: 'time_1_sec',
                    },
                    time_difference: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'time_difference',
                        label: 'CPU',
                    },
                    rss: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'rss',
                        label: 'rss'
                    },
                    vzs: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'vzs',
                        label: 'vzs'
                    },
                },
            },
            fixed_disk: {
                label: 'Disk',
                excludedTools: [
                    'all',
                ],
                configuration: {
                    ip: {
                        label: "IP"
                    },
                },
                metrics: {
                    UsedSpace: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'UsedSpace',
                        label: 'Used',
                    },
                    Size: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'Size',
                        label: 'Total',
                    },
                    DiskReadBytesPersec: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'DiskReadBytesPersec',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    DiskWriteBytesPerse: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'DiskWriteBytesPerse',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    DiskReadsPersec: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'DiskReadsPersec',
                        longLabel: 'IO Read',
                        label: 'Read',
                    },
                    DiskWritesPersec: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'DiskWritesPersec',
                        longLabel: 'IO Write',
                        label: 'Write',
                    },
                    AvgDisksecPerRead: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'AvgDisksecPerRead',
                        longLabel: 'Read Latency',
                        label: 'Read',
                    },
                    AvgDisksecPerReadB: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'AvgDisksecPerReadB',
                        label: 'AvgDisksecPerReadB'
                    },
                    AvgDisksecPerWrite: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'AvgDisksecPerWrite',
                        longLabel: 'Write Latency',
                        label: 'Write',
                    },
                    AvgDisksecPerWriteB: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'AvgDisksecPerWriteB',
                        label: 'AvgDisksecPerWriteB'
                    },
                },
            },
            cluster: {
                label: 'Cluster',
                configuration: {
                    version: {
                        label: "version"
                    },
                },
                excluded: {
                    'customGroup': [
                        'vCPU'
                    ]
                },
                metrics: {
                    cpu_usage: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_usage',
                        longLabel: 'CPU',
                        label: 'Usage',
                    },
                    used_bytes: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_bytes',
                        longLabel: 'Memory Used',
                        label: 'Used',
                        defaultPrecision: 1,
                    },
                    avail_m_bytes: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'avail_m_bytes',
                        longLabel: 'Memory Free',
                        label: 'Free',
                        defaultPrecision: 2,
                    },
                    cache_bytes: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'cache_bytes',
                        longLabel: 'Memory Cache',
                        label: 'Cache',
                        defaultPrecision: 2,
                    },
                    vCPU: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'vCPU',
                        longLabel: 'CPUs total available',
                        label: 'Total available',
                    },
                },
            },
        },
    },
    solaris: {
        label: 'Solaris',
        order: 15,
        subsystems: {
            cdom: {
                label: 'CDOM',
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization',
                        percent: 'cpu_utilization'
                    }],
                top: {
                    "CPU": ["cpu_utilization"]
                },
                configuration: {
                    version: {
                        label: "version"
                    },
                },
                metrics: {
                    cpu_utilization: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_utilization',
                        label: 'Usage',
                    },
                    memory_allocated: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 1,
                        metric: 'memory_allocated',
                        label: 'Allocated memory',
                    },
                },
            },
            ldom: {
                label: 'LDOM',
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization',
                        percent: 'cpu_utilization'
                    }],
                top: {
                    "CPU": ["cpu_utilization"]
                },
                configuration: {
                    version: {
                        label: "version"
                    },
                },
                metrics: {
                    cpu_utilization: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_utilization',
                        label: 'Usage',
                    },
                    memory_allocated: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 1,
                        metric: 'memory_allocated',
                        label: 'Allocated memory',
                    },
                },
            },
            sanldom: {
                label: 'SAN',
                metrics: {
                    bytes_read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_read',
                        longLabel: 'Data Read',
                        label: 'Read',
                    },
                    bytes_write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'bytes_write',
                        longLabel: 'Data Write',
                        label: 'Write',
                    },
                    io_read: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.frames,
                        shortcut: subsystemMetricsResponse_constants_1.UNIT.frames,
                        metric: 'io_read',
                        longLabel: 'Frames Read',
                        label: 'Read',
                        defaultPrecision: 0
                    },
                    io_write: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.frames,
                        shortcut: subsystemMetricsResponse_constants_1.UNIT.frames,
                        metric: 'io_write',
                        longLabel: 'Frames Write',
                        label: 'Write',
                        defaultPrecision: 0
                    },
                    resp_time: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'resp_time',
                        label: 'Latency',
                    },
                },
            },
            osagent: {
                label: 'OS agent',
                metrics: {
                    cpu_idle: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_idle',
                        longLabel: 'CPU Idle',
                        label: 'Idle',
                    },
                    cpu_system: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_system',
                        longLabel: 'CPU System',
                        label: 'System',
                    },
                    cpu_user: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_user',
                        longLabel: 'CPU User',
                        label: 'User',
                    },
                    cpu_wait: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_wait',
                        longLabel: 'CPU IO Wait',
                        label: 'IO Wait',
                    },
                    mem_free: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_free',
                        longLabel: 'Memory Free',
                        label: 'Free',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.green
                    },
                    mem_used: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_used',
                        longLabel: 'Memory Used',
                        label: 'Used',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.red
                    },
                    mem_in_use_client: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_in_use_client',
                        label: 'FS Cache',
                        defaultPrecision: 1,
                        color: Color_1.Color.pallete.blue
                    },
                    pgs1: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'pgs1',
                        label: 'Page In',
                    },
                    pgs2: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'pgs2',
                        label: 'Page Out',
                    },
                    pgs4: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'pgs4',
                        label: 'Usage',
                    },
                    virtual_p: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'virtual_p',
                        label: 'Logical Processors',
                        color: Color_1.Color.pallete.black,
                    },
                    load: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'load',
                        longLabel: 'Load Average',
                        label: 'Load Average',
                        color: Color_1.Color.pallete.red,
                    },
                    blocked_p: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 1,
                        metric: 'blocked_p',
                        label: 'Blocked Processes',
                        color: Color_1.Color.pallete.blueDark,
                    },
                    cpu_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores',
                        longLabel: 'CPU Virtual',
                        label: 'Virtual',
                    },
                },
            },
            netldom: {
                label: 'NET',
                metrics: {
                    read_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'read_bytes',
                        label: 'Read',
                    },
                    write_bytes: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'write_bytes',
                        label: 'Write',
                    },
                },
            },
            pool: {
                label: 'Pool',
                metrics: {
                    size: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'size',
                        label: 'Size CPU',
                    },
                    used: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'used',
                        label: 'Used CPU',
                    },
                },
            },
            zone: {
                label: 'Zone',
                metrics: {
                    cpu_used_in_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_used_in_cores',
                        label: 'Usage',
                    },
                    cpu_used_in_perc: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_used_in_perc',
                        label: 'Usage',
                        longLabel: 'CPU %',
                    },
                    total_memory: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 1,
                        metric: 'total_memory',
                        label: 'Allocated memory',
                    },
                    used_memory: {
                        ...Definitions_1.metricTypes.memory,
                        defaultPrecision: 1,
                        metric: 'used_memory',
                        label: 'Used memory',
                    },
                    net_used_in_perc: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'net_used_in_perc',
                        label: 'Usage',
                    },
                },
            },
        },
    },
    oraclevm: {
        label: 'OracleVM',
        order: 10,
        subsystems: {
            server: {
                label: 'Server',
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    }, {
                        label: 'Memory',
                        key: 'mem_perc',
                        percent: 'mem_perc'
                    }],
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        label: 'Usage',
                        longLabel: 'CPU',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_utilization_percent',
                        label: 'Usage',
                        longLabel: 'CPU %',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        longLabel: 'CPU (Peak)',
                        label: 'CPU (Peak)',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU (Peak)',
                        longLabel: 'CPU % (Peak)',
                    },
                    cpu_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores',
                        longLabel: 'CPU Total',
                        label: 'Total',
                    },
                    free_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'free_mem',
                        longLabel: 'Memory Free',
                        label: 'Free',
                    },
                    used_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_mem',
                        longLabel: 'Memory Used',
                        label: 'Used',
                    },
                    free_swap: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'free_swap',
                        longLabel: 'Memory Swap',
                        label: 'Swap',
                    },
                    total_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'total_mem',
                        longLabel: 'Memory Total',
                        label: 'Total',
                    },
                    mem_perc: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'mem_perc',
                        label: 'Used',
                        longLabel: 'Memory Used %',
                    },
                },
                configuration: {
                    serverpool_name: {
                        label: 'Server Pool'
                    },
                    server_name: {
                        label: 'Server'
                    },
                    hostname: {
                        label: 'Hostname'
                    },
                    ip_address: {
                        label: 'Address'
                    },
                    total_memory: {
                        label: 'Total Memory',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi
                    },
                    cpu_sockets: {
                        label: 'Socket count'
                    },
                    cpu_type: {
                        label: 'CPU model'
                    },
                    hypervisor_type: {
                        label: 'Hypervisor type'
                    },
                    hypervisor_version: {
                        label: 'Hypervisor name',
                        type: 'string'
                    },
                    product_name: {
                        label: 'Product name'
                    },
                    bios_version: {
                        label: 'Bios version',
                        type: 'string'
                    }
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_cores'
                    ]
                }
            },
            vm: {
                label: 'VM',
                aliases: true,
                top: {
                    'CPU': ['cpu_utilization_percent']
                },
                heatmap: [{
                        label: 'CPU',
                        key: 'cpu_utilization_percent',
                        percent: 'cpu_utilization_percent'
                    }],
                metrics: {
                    cpu_utilization_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores',
                        label: 'CPU',
                        longLabel: 'CPU',
                    },
                    cpu_utilization_percent: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'cpu_utilization_percent',
                        label: 'CPU %',
                        longLabel: 'CPU %',
                    },
                    cpu_cores: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_cores',
                        longLabel: 'CPU Virtual',
                        label: 'Virtual',
                    },
                    cpu_utilization_percent_max: {
                        ...Definitions_1.metricTypes.cpu_percent,
                        metric: 'cpu_utilization_percent_max',
                        label: 'CPU % (Peak)',
                        longLabel: 'CPU % (Peak)',
                    },
                    cpu_utilization_cores_max: {
                        ...Definitions_1.metricTypes.cpu_core,
                        metric: 'cpu_utilization_cores_max',
                        label: 'Usage (Peak)',
                        longLabel: 'CPU (Peak)',
                    },
                    used_mem: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'used_mem',
                        longLabel: 'Memory Used',
                        label: 'Used',
                    },
                },
                configuration: {
                    serverpool_name: {
                        label: 'Server Pool'
                    },
                    server_name: {
                        label: 'Server'
                    },
                    vm_name: {
                        label: 'VM'
                    },
                    total_memory: {
                        label: 'Memory',
                        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.gibi
                    },
                    cpu_count: {
                        label: 'CPU count'
                    },
                    os_type: {
                        label: 'Operating system'
                    },
                    domain_type: {
                        label: 'Domain type'
                    }
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_cores'
                    ]
                }
            },
            server_interface: {
                label: 'Server Interface',
                parent: `server`,
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    network_received: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_received',
                        label: 'Read',
                    },
                    network_transmitted: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_transmitted',
                        label: 'Write',
                    },
                }
            },
            server_disk: {
                label: 'Server Disk',
                parent: `server`,
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    disk_read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_read',
                        label: 'Read',
                    },
                    disk_write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_write',
                        label: 'Write',
                    },
                }
            },
            vm_interface: {
                label: 'VM Interface',
                parent: `vm`,
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    network_received: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_received',
                        label: 'Read',
                    },
                    network_transmitted: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'network_transmitted',
                        label: 'Write',
                    },
                }
            },
            vm_disk: {
                label: 'VM Disk',
                parent: `vm`,
                excludedTools: [
                    'customGroup'
                ],
                metrics: {
                    disk_read: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_read',
                        label: 'Read',
                    },
                    disk_write: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'disk_write',
                        label: 'Write',
                    },
                }
            },
            manager: {
                label: "Manager",
                excludedTools: [
                    'all'
                ],
            }
        }
    },
    cloudstack: {
        label: 'Apache CloudStack',
        order: 11
    },
    fusioncompute: {
        label: 'FusionCompute',
        order: 5
    },
    as400: {
        label: 'IBM i OS agent',
        order: 13,
        subsystems: {
            server: {
                label: 'Server',
                metrics: {
                    jobs_in_system: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'jobs_in_system',
                        label: 'JOBS total',
                        longLabel: 'JOBS total',
                    },
                    active_jobs_in_system: {
                        metric: 'active_jobs_in_system',
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        label: 'JOBS active',
                        longLabel: 'JOBS active',
                    },
                    active_threads_in_system: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'active_threads_in_system',
                        longLabel: 'THREADS',
                        label: 'THREADS',
                    },
                },
                configuration: {
                    version: {
                        label: 'version'
                    },
                    version_date: {
                        label: 'version_date'
                    },
                    ip_address: {
                        label: 'Address'
                    },
                },
                excluded: {
                    reporter: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    alerting: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max'
                    ],
                    customGroup: [
                        'cpu_utilization_cores_max',
                        'cpu_utilization_percent_max',
                        'cpu_cores'
                    ]
                }
            },
            shrpool: {
                label: 'Shrpool',
                parent: `server`,
                metrics: {
                    threads: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'threads',
                        longLabel: 'Threads',
                        label: 'Threads',
                    },
                    database_pages: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'database_pages',
                        longLabel: 'Database Pages',
                        label: 'Database Pages',
                    },
                    non_database_pages: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'non_database_pages',
                        longLabel: 'Non Database Pages',
                        label: 'Non Database Pages',
                    },
                    database_faults: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'database_faults',
                        longLabel: 'database_faults',
                        label: 'Database Faults',
                    },
                    non_database_faults: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'non_database_faults',
                        longLabel: 'Non Database Faults',
                        label: 'Non Database Faults',
                    },
                    size: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'size',
                        defaultPrecision: 1,
                        defaultPrefix: pageResponse_constants_1.UNIT_PREFIX.giga,
                        longLabel: 'Memory',
                        label: 'Memory Used',
                    },
                },
                configuration: {
                    paging_option: {
                        label: 'paging_option'
                    },
                },
            },
            lan: {
                label: 'lan',
                parent: `server`,
                metrics: {
                    total_bytes_recv: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'total_bytes_recv',
                        label: 'Read',
                    },
                    total_bytes_sent: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'total_bytes_sent',
                        label: 'Write',
                    },
                    total_packets_recv: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'total_packets_recv',
                        label: 'Read',
                    },
                    total_packets_sent: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'total_packets_sent',
                        label: 'Write',
                    },
                    total_inbound_packets_discarded: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'total_inbound_packets_discarded',
                        label: 'Read',
                    },
                    total_outbound_packets_discarded: {
                        ...Definitions_1.metricTypes.packets_per_second,
                        metric: 'total_outbound_packets_discarded',
                        label: 'Write',
                    },
                },
                configuration: {
                    line_type: {
                        label: 'line_type'
                    },
                    mac_addr: {
                        label: 'mac_addr'
                    },
                    status: {
                        label: 'status'
                    },
                },
            },
            asp: {
                label: 'ASP',
                parent: `server`,
                metrics: {
                    mem_alloc: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_alloc',
                        longLabel: 'ASP Used',
                        label: 'ASP Used',
                    },
                    mem_free: {
                        ...Definitions_1.metricTypes.memory,
                        metric: 'mem_free',
                        longLabel: 'ASP FREE',
                        label: 'ASP Free',
                    },
                    mem_alloc_perc: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'mem_alloc_perc',
                        longLabel: 'ASP MEM PERC',
                        label: 'ASP MEM PERC',
                    },
                    io_to_memory: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'io_to_memory',
                        label: 'Read',
                    },
                    io_from_memory: {
                        ...Definitions_1.metricTypes.iops,
                        metric: 'io_from_memory',
                        label: 'Write',
                    },
                    data_to_memory: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'data_to_memory',
                        label: 'Read',
                    },
                    data_from_memory: {
                        ...Definitions_1.metricTypes.throughput,
                        metric: 'data_from_memory',
                        label: 'Write',
                    },
                    disk_busy: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'disk_busy',
                        longLabel: 'Disk busy',
                        label: 'Disk busy',
                    },
                    disk_io_aggreg: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'disk_io_aggreg',
                        longLabel: 'Disk IO aggregated',
                        label: 'Disk IO aggregated',
                    },
                    svc_time: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'svc_time',
                        longLabel: 'Disk service',
                        label: 'SVC',
                    },
                    wait_time: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'wait_time',
                        longLabel: 'Disk wait',
                        label: 'Wait',
                    },
                    latency_time: {
                        ...Definitions_1.metricTypes.latency,
                        metric: 'latency_time',
                        longLabel: 'Disk latency',
                        label: 'Latency Total',
                    },
                },
                configuration: {
                    bubble: {
                        label: 'bubble'
                    },
                    disk_serial: {
                        label: 'disk_serial'
                    },
                },
            },
            job: {
                label: 'JOB',
                parent: `server`,
                metrics: {
                    percent_used: {
                        ...Definitions_1.metricTypes.percent,
                        metric: 'percent_used',
                        defaultPrecision: 1,
                        longLabel: 'CPU aggreg',
                        label: 'CPU aggreg',
                        minY: undefined
                    },
                    diskIO_total: {
                        unit: subsystemMetricsResponse_constants_1.UNIT.count,
                        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
                        defaultPrecision: 0,
                        metric: 'diskIO_total',
                        longLabel: 'IO TOP',
                        label: 'IO TOP',
                    },
                    proctime_total: {
                        ...Definitions_1.metricTypes.cpu_core,
                        defaultPrecision: 1,
                        metric: 'proctime_total',
                        longLabel: 'CPU TOP',
                        label: 'CPU TOP',
                    },
                },
                configuration: {
                    jobtype: {
                        label: 'jobtype'
                    },
                    job_user: {
                        label: 'job_user'
                    },
                    jobfunction: {
                        label: 'jobfunction'
                    },
                    jobstatus: {
                        label: 'jobstatus'
                    },
                    jobnumber: {
                        label: 'jobnumber'
                    },
                    jobname: {
                        label: 'jobname'
                    },
                },
            },
        },
    },
};
//# sourceMappingURL=Virtualization.js.map