"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MetricsService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsService = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const typeorm_1 = require("@nestjs/typeorm");
const hwTypeKey_constants_1 = require("../../common/constants/hwTypeKey.constants");
const dataSource_1 = require("../../config/dataSource");
const aggregation_entity_1 = require("../../entity/aggregation.entity");
const itemProperty_entity_1 = require("../../entity/itemProperty.entity");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_2 = require("typeorm");
let MetricsService = MetricsService_1 = class MetricsService {
    constructor(configService, itemPropertiesRepository, objectItemsRepository, aggregationRepository, dataSource) {
        this.configService = configService;
        this.itemPropertiesRepository = itemPropertiesRepository;
        this.objectItemsRepository = objectItemsRepository;
        this.aggregationRepository = aggregationRepository;
        this.dataSource = dataSource;
        this.logger = new common_1.Logger(MetricsService_1.name);
        this.compressedLimit = 2 * 24 * 60 * 60 * 1000;
        this._5minAggregationLimit = 60 * 60 * 1000;
    }
    async saveMetrics(metrics, hw_type, subsystem) {
        if (hw_type && subsystem) {
            const table = 'metrics.metrics_' + hw_type.toLowerCase() + '_' + subsystem.toLowerCase();
            const repository = this.dataSource.getRepository(table);
            const columns = repository.metadata.columns.map((column) => {
                return column.propertyName;
            });
            const now = new Date().getTime();
            let manualAggregationTime = undefined;
            const experimentMetrics = metrics.filter((obj) => {
                if (hw_type == hwTypeKey_constants_1.HwTypeKey.LINUX && manualAggregationTime == undefined && now - new Date(obj.time).getTime() >= this._5minAggregationLimit) {
                    manualAggregationTime = obj.time;
                }
                if (now - new Date(obj.time).getTime() >= this.compressedLimit) {
                    console.warn("CompressLimitSkip [" + hw_type + " " + subsystem + " " + obj.item_id + "]: " + (now - new Date(obj.time).getTime()) + " < " + this.compressedLimit);
                    return false;
                }
                return true;
            }).map((obj) => {
                let rObj = {
                    time: obj.time,
                    item_id: obj.item_id,
                    ...obj.data,
                };
                return rObj;
            });
            const chunk = 1000;
            let inserted = 0;
            for (let index = 0; index < experimentMetrics.length; index += chunk) {
                const toSaveChunk = experimentMetrics.slice(index, index + chunk);
                if (index === 0) {
                    let difference = Object.keys(toSaveChunk[0]).filter((firstItemEntry) => !columns.includes(firstItemEntry));
                    if (difference.length >= 1) {
                        this.logger.warn('Undefined metrics: ' + difference.toString() + ' in hw_type: ' + hw_type + ' and subsystem: ' + subsystem);
                    }
                }
                try {
                    const saved = await repository.createQueryBuilder().insert().values(toSaveChunk).orIgnore().execute();
                    inserted += saved.generatedMaps.length;
                }
                catch (e) {
                    console.error(e);
                    console.error(toSaveChunk);
                }
            }
            if (manualAggregationTime) {
                await this.aggregationRepository.createQueryBuilder().insert().into(aggregation_entity_1.Aggregation).values({
                    table: table,
                    time: new Date(manualAggregationTime)
                }).orIgnore().execute();
            }
            return inserted;
        }
        else {
            return 0;
        }
    }
    async getMetricsGapFill(uuid, metric, start, end, interval, func) {
        const entityManager = (0, dataSource_1.getDataSource)();
        const timeZone = this.configService.get('APP_TIMEZONE') ? this.configService.get('APP_TIMEZONE') : 'UTC';
        await entityManager.query("SET TIME zone '" + timeZone + "'");
        const selectQuery = "SELECT time_bucket_gapfill('" +
            interval +
            " minute', time) AS newTime, " +
            func +
            "((data->>'" +
            metric +
            "')::numeric) as value FROM metrics WHERE data ? '" +
            metric +
            "' AND item_id = '" +
            uuid +
            "' AND time >= '" +
            start +
            "' AND time < '" +
            end +
            "' GROUP BY newTime";
        return await entityManager.query(selectQuery);
    }
    async getMetricsGapFill2(uuids, metric, start, end, interval, func) {
        const entityManager = (0, dataSource_1.getDataSource)();
        const timeZone = this.configService.get('APP_TIMEZONE') ? this.configService.get('APP_TIMEZONE') : 'UTC';
        await entityManager.query("SET TIME zone '" + timeZone + "'");
        const selectQuery = "SELECT time_bucket_gapfill('" +
            interval +
            " minute', time) AS newTime, " +
            func +
            "((data->>'" +
            metric +
            "')::numeric) as value, item_id FROM metrics WHERE data ? '" +
            metric +
            "' AND item_id IN " +
            uuids +
            " AND time >= '" +
            start +
            "' AND time < '" +
            end +
            "' GROUP BY item_id, newTime";
        return await entityManager.query(selectQuery);
    }
    async getValue(id, name) {
        return this.itemPropertiesRepository.findOneBy({ item_id: id, property_name: name });
    }
    async getItem(id) {
        return this.objectItemsRepository.findOneBy({ item_id: id });
    }
    async getItems(itemIds, parentType) {
        return !parentType
            ? await this.objectItemsRepository.find({ where: { item_id: (0, typeorm_2.In)(itemIds) }, relations: { server_group_items: { _server_group: true } } })
            : await this.objectItemsRepository.find({
                where: {
                    item_id: (0, typeorm_2.In)(itemIds),
                    parents: {
                        _parent: {
                            subsystem: parentType === 'server'
                                ? (0, typeorm_2.In)(['server', 'node', 'esxi', 'host'])
                                : parentType
                        }
                    }
                }, relations: {
                    parents: {
                        _parent: true
                    },
                    server_group_items: {
                        _server_group: true
                    }
                }
            });
    }
    async getSubsystemMetrics(hw_type, subsystem) {
        return [];
    }
    async saveMetricsDefinitions(definitions) {
        return null;
    }
    calculate(statistics, value) {
        statistics.sum += value;
        statistics.count += 1;
        statistics.avg = statistics.sum / statistics.count;
        statistics.min = statistics.min === null || statistics.min > value ? value : statistics.min;
        statistics.max = statistics.max === null || statistics.max < value ? value : statistics.max;
        return statistics;
    }
    toIso(datetime) {
        let time = new Date(datetime);
        return time.toISOString();
    }
    toTimestamp(datetime, precision) {
        let time = new Date(datetime + ' UTC');
        return time.getTime() / 1000 - ((time.getTime() / 1000) % precision);
    }
    checkValue(value) {
        if (isNaN(value) == true || value.length < 1) {
            return null;
        }
        else {
            return this.noExponents(value);
        }
    }
    noExponents(number) {
        const data = String(number).split(/[eE]/);
        if (data.length == 1)
            return number;
        let z = '', sign = number < 0 ? '-' : '', str = data[0].replace('.', ''), mag = Number(data[1]) + 1;
        if (mag < 0) {
            z = sign + '0.';
            while (mag++)
                z += '0';
            return +(z + str.replace(/^\-/, ''));
        }
        if (mag < str.length) {
            let diff = str.length - mag, start = str.slice(0, str.length - diff), end = str.slice(-diff);
            return +(start + "." + end);
        }
        mag -= str.length;
        while (mag--)
            z += '0';
        return +(str + z);
    }
};
exports.MetricsService = MetricsService;
exports.MetricsService = MetricsService = MetricsService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(1, (0, typeorm_1.InjectRepository)(itemProperty_entity_1.ItemProperty)),
    __param(2, (0, typeorm_1.InjectRepository)(objectItem_entity_1.ObjectItem)),
    __param(3, (0, typeorm_1.InjectRepository)(aggregation_entity_1.Aggregation)),
    __param(4, (0, typeorm_1.InjectEntityManager)()),
    __metadata("design:paramtypes", [config_1.ConfigService,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.EntityManager])
], MetricsService);
//# sourceMappingURL=metrics.service.js.map