"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MetricsTableService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsTableService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const typeorm_2 = require("typeorm");
const PM2_1 = require("../configuration/PM2");
const WinstonLogger_1 = require("../log/WinstonLogger");
const MetricsWrapper_1 = require("./MetricsWrapper");
const TableGenerator_1 = require("./TableGenerator");
let MetricsTableService = MetricsTableService_1 = class MetricsTableService {
    constructor(dataSource, hostcfgRepository) {
        this.dataSource = dataSource;
        this.hostcfgRepository = hostcfgRepository;
        this.logger = new common_1.Logger(MetricsTableService_1.name);
        this.intervals = TableGenerator_1.TableGenerator.intervals;
        this.maxMetrics = TableGenerator_1.TableGenerator.getMaxMetrics();
        this.allowedHwTypes = ['linux', 'docker', 'windows', 'timescaledb', 'as400', 'solaris'];
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('metrics-table');
    }
    async onModuleInit() {
        if (PM2_1.PM2.isMasterBackend()) {
            await this.generate();
        }
    }
    async reGenerate(hwType, subsystem) {
        console.log("[reGenerate] start");
        const maxMetrics = await this.maxMetrics;
        const entityManager = this.dataSource;
        const entity = entityManager.connection.entityMetadatas.find(ent => ent.tableName === 'metrics_' + hwType + "_" + subsystem);
        const actualTableColumns = (await entityManager.query(`SELECT * FROM information_schema.columns WHERE table_schema = 'metrics' AND table_name = 'metrics_${hwType}_${subsystem}';`))
            .map(column => column.column_name);
        let newColumns = [];
        let oldColumns = [];
        entity.columns.forEach(col => {
            if (!actualTableColumns.includes(col.propertyName)) {
                newColumns.push(col);
            }
            else {
                oldColumns.push(col);
            }
        });
        if (newColumns.length >= 1) {
            let insertColumns = 'bucket, item_id';
            for (const actualTableColumn of actualTableColumns) {
                if (['time', 'item_id'].includes(actualTableColumn)) {
                    continue;
                }
                insertColumns += ', \"' + actualTableColumn + '\"';
            }
            insertColumns += "";
            try {
                const min5minutesInterval = await MetricsWrapper_1.MetricsWrapper.getMin5MinutesInterval();
                await entityManager.transaction(async (transactionalEntityManager) => {
                    let sql = `ALTER TABLE metrics.metrics_${hwType}_${subsystem}`;
                    for (const [index, col] of newColumns.entries()) {
                        console.log(`"Add column ${col.propertyName}"`);
                        if (index === 0) {
                            sql += ` ADD COLUMN "${col.propertyName}" ${col.type}`;
                        }
                        else {
                            sql += `, ADD COLUMN "${col.propertyName}" ${col.type}`;
                        }
                    }
                    await transactionalEntityManager.query(sql);
                    this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Updated source table');
                    for (const [index, interval] of this.intervals.entries()) {
                        if (index === 0 && min5minutesInterval.includes(hwType)) {
                            continue;
                        }
                        let idx = 0;
                        let id = 0;
                        while (1) {
                            const exists = await entityManager.query(`SELECT to_regclass('metrics.${entity.tableName}_${interval.label}_${id}')`);
                            if (exists[0]['to_regclass'] === null) {
                                idx = id;
                                break;
                            }
                            id++;
                        }
                        if (idx == 0) {
                            sql = `ALTER MATERIALIZED VIEW metrics.${entity.tableName}_${interval.label} RENAME TO ${entity.tableName}_${interval.label}_${idx}`;
                            await transactionalEntityManager.query(sql);
                            this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Rename origin table');
                            let sourceTableNameOld;
                            sql = `select materialization_hypertable_schema, materialization_hypertable_name from timescaledb_information.continuous_aggregates where view_schema = 'metrics' AND view_name = '${entity.tableName}_${interval.label}_${idx}'`;
                            const oldTableChunks = await transactionalEntityManager.query(sql);
                            if (oldTableChunks[0]['materialization_hypertable_schema'] && oldTableChunks[0]['materialization_hypertable_name']) {
                                sourceTableNameOld = `${oldTableChunks[0]['materialization_hypertable_name']}`;
                            }
                            const hasMax = index !== 0 && maxMetrics[hwType] && maxMetrics[hwType][subsystem] ? true : false;
                            sql = `
                CREATE MATERIALIZED VIEW metrics.${entity.tableName}_${interval.label}_${idx + 1}
                WITH (timescaledb.continuous) AS
                SELECT time_bucket(INTERVAL '${interval.interval}', "${index === 0 || (index === 1 && min5minutesInterval.includes(hwType)) ? 'time' : 'bucket'}") AS bucket, item_id
              `;
                            for (const column of newColumns) {
                                if ((column.propertyName !== 'time' && column.propertyName !== 'bucket') && column.propertyName !== 'item_id') {
                                    if (hasMax && maxMetrics[hwType][subsystem].includes(column.propertyName)) {
                                        sql += ', MAX("' + column.propertyName + '") as "' + column.propertyName + '"';
                                    }
                                    else {
                                        sql += ', AVG("' + column.propertyName + '") as "' + column.propertyName + '"';
                                    }
                                }
                            }
                            if (index === 0 || (index === 1 && min5minutesInterval.includes(hwType))) {
                                sql += ' FROM metrics.' + entity.tableName + ' GROUP BY 1, item_id WITH NO DATA; ';
                            }
                            else {
                                sql += ' FROM metrics.' + entity.tableName + '_' + this.intervals[index - 1].label + '_' + (+idx + 1) + ' GROUP BY 1, item_id WITH NO DATA; ';
                            }
                            await transactionalEntityManager.query(sql);
                            this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created continuous aggregate');
                            sql = `select materialization_hypertable_schema, materialization_hypertable_name from timescaledb_information.continuous_aggregates where view_schema = 'metrics' AND view_name = '${entity.tableName}_${interval.label}_${idx + 1}'`;
                            const chunks = await transactionalEntityManager.query(sql);
                            let sourceTableNameNew;
                            if (chunks[0]['materialization_hypertable_schema'] && chunks[0]['materialization_hypertable_name']) {
                                sourceTableNameNew = `${chunks[0]['materialization_hypertable_name']}`;
                                sql = `SELECT set_chunk_time_interval('${chunks[0]['materialization_hypertable_schema']}.${chunks[0]['materialization_hypertable_name']}', ${interval.chunk_interval});`;
                                await transactionalEntityManager.query(sql);
                                this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Edited chunk interval');
                            }
                            sql = `
                SELECT add_continuous_aggregate_policy('metrics.${entity.tableName}_${interval.label}_${idx + 1}',
                  start_offset => INTERVAL '${interval.start_offset}',
                  end_offset => ${interval.end_offset},
                  schedule_interval => INTERVAL '${interval.schedule_interval}');
              `;
                            await transactionalEntityManager.query(sql);
                            sql = `SELECT add_retention_policy('metrics.${entity.tableName}_${interval.label}_${idx + 1}', INTERVAL '${interval.retention_policy}');`;
                            await transactionalEntityManager.query(sql);
                            this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created continuous aggregate retention policy');
                            if (interval.compress_after) {
                                sql = `ALTER MATERIALIZED VIEW metrics.${entity.tableName}_${interval.label}_${idx + 1} set (timescaledb.compress = true);`;
                                await transactionalEntityManager.query(sql);
                                sql = `SELECT add_compression_policy('metrics.${entity.tableName}_${interval.label}_${idx + 1}', compress_after=>'${interval.compress_after}'::interval);`;
                                await transactionalEntityManager.query(sql);
                                this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created compression');
                            }
                            if (interval.materialized) {
                                sql = `ALTER MATERIALIZED VIEW metrics.${entity.tableName}_${interval.label}_${idx + 1} set (timescaledb.materialized_only = true);`;
                                await transactionalEntityManager.query(sql);
                                this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created materialized option');
                            }
                            if (sourceTableNameNew && sourceTableNameOld) {
                                const schema = "_timescaledb_internal";
                                sql = `CREATE OR REPLACE VIEW metrics.${entity.tableName}_${interval.label} 
                      AS SELECT ${sourceTableNameOld}.bucket ,
                      ${sourceTableNameOld}.item_id `;
                                for (const column of newColumns) {
                                    sql += `, ${sourceTableNameNew}."${column.propertyName}"`;
                                }
                                for (const column of oldColumns) {
                                    if ((column.propertyName !== 'time' && column.propertyName !== 'bucket') && column.propertyName !== 'item_id') {
                                        sql += `, ${sourceTableNameOld}."${column.propertyName}"`;
                                    }
                                }
                                sql += ` FROM ${schema}.${sourceTableNameNew}`;
                                sql += ` LEFT JOIN ${schema}.${sourceTableNameOld}
                        ON ${schema}.${sourceTableNameNew}.item_id = ${schema}.${sourceTableNameOld}.item_id 
                        AND ${schema}.${sourceTableNameNew}.bucket = ${schema}.${sourceTableNameOld}.bucket `;
                                await transactionalEntityManager.query(sql);
                                this.winstonLogger.log('info', '[' + entity.tableName + '] [split] Created splitted view');
                            }
                            else {
                            }
                        }
                        else {
                            while (1) {
                                const exists = await entityManager.query(`SELECT to_regclass('metrics.${entity.tableName}_${interval.label}_${id}')`);
                                if (exists[0]['to_regclass'] === null) {
                                    idx = id;
                                    break;
                                }
                                id++;
                            }
                            sql = `
                CREATE MATERIALIZED VIEW metrics.${entity.tableName}_${interval.label}_${idx}
                WITH (timescaledb.continuous) AS
                SELECT time_bucket(INTERVAL '${interval.interval}', "${index === 0 || (index === 1 && min5minutesInterval.includes(hwType)) ? 'time' : 'bucket'}") AS bucket, item_id
              `;
                            const hasMax = index !== 0 && maxMetrics[hwType] && maxMetrics[hwType][subsystem] ? true : false;
                            for (const column of newColumns) {
                                if ((column.propertyName !== 'time' && column.propertyName !== 'bucket') && column.propertyName !== 'item_id') {
                                    if (hasMax && maxMetrics[hwType][subsystem].includes(column.propertyName)) {
                                        sql += ', MAX("' + column.propertyName + '") as "' + column.propertyName + '"';
                                    }
                                    else {
                                        sql += ', AVG("' + column.propertyName + '") as "' + column.propertyName + '"';
                                    }
                                }
                            }
                            if (index === 0 || (index === 1 && min5minutesInterval.includes(hwType))) {
                                sql += ' FROM metrics.' + entity.tableName + ' GROUP BY 1, item_id WITH NO DATA; ';
                            }
                            else {
                                sql += ' FROM metrics.' + entity.tableName + '_' + this.intervals[index - 1].label + '_' + (+idx) + ' GROUP BY 1, item_id WITH NO DATA; ';
                            }
                            await transactionalEntityManager.query(sql);
                            this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created continuous aggregate');
                            sql = `select materialization_hypertable_schema, materialization_hypertable_name from timescaledb_information.continuous_aggregates where view_schema = 'metrics' AND view_name = '${entity.tableName}_${interval.label}_${idx}'`;
                            const chunks = await transactionalEntityManager.query(sql);
                            let sourceTableNameNew;
                            if (chunks[0]['materialization_hypertable_schema'] && chunks[0]['materialization_hypertable_name']) {
                                sourceTableNameNew = `${chunks[0]['materialization_hypertable_name']}`;
                                sql = `SELECT set_chunk_time_interval('${chunks[0]['materialization_hypertable_schema']}.${chunks[0]['materialization_hypertable_name']}', ${interval.chunk_interval});`;
                                await transactionalEntityManager.query(sql);
                                this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Edited chunk interval');
                            }
                            sql = `
                SELECT add_continuous_aggregate_policy('metrics.${entity.tableName}_${interval.label}_${idx}',
                  start_offset => INTERVAL '${interval.start_offset}',
                  end_offset => ${interval.end_offset},
                  schedule_interval => INTERVAL '${interval.schedule_interval}');
              `;
                            await transactionalEntityManager.query(sql);
                            sql = `SELECT add_retention_policy('metrics.${entity.tableName}_${interval.label}_${idx}', INTERVAL '${interval.retention_policy}');`;
                            await transactionalEntityManager.query(sql);
                            this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created continuous aggregate retention policy');
                            if (interval.compress_after) {
                                sql = `ALTER MATERIALIZED VIEW metrics.${entity.tableName}_${interval.label}_${idx} set (timescaledb.compress = true);`;
                                await transactionalEntityManager.query(sql);
                                sql = `SELECT add_compression_policy('metrics.${entity.tableName}_${interval.label}_${idx}', compress_after=>'${interval.compress_after}'::interval);`;
                                await transactionalEntityManager.query(sql);
                                this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created compression');
                            }
                            if (interval.materialized) {
                                sql = `ALTER MATERIALIZED VIEW metrics.${entity.tableName}_${interval.label}_${idx} set (timescaledb.materialized_only = true);`;
                                await transactionalEntityManager.query(sql);
                                this.winstonLogger.log('info', '[' + entity.tableName + '] [tmp] Created materialized option');
                            }
                            let sql_view = `create or replace view metrics.${entity.tableName}_${interval.label} as select `;
                            let first_mat_table;
                            for (let index_i = 0; index_i <= idx; index_i++) {
                                sql = `select materialization_hypertable_schema, materialization_hypertable_name from timescaledb_information.continuous_aggregates where view_schema = 'metrics' AND view_name = '${entity.tableName}_${interval.label}_${index_i}'`;
                                let test = await transactionalEntityManager.query(sql);
                                let materialization_hypertable_name = test[0].materialization_hypertable_name;
                                let materialization_hypertable_schema = test[0].materialization_hypertable_schema;
                                sql = `select table_name, table_schema, column_name from INFORMATION_SCHEMA.COLUMNS where table_name = '${materialization_hypertable_name}'`;
                                let metrics_in_materialized_hypertable = await transactionalEntityManager.query(sql);
                                if (index_i == 0) {
                                    first_mat_table = materialization_hypertable_name;
                                    sql_view += ` ${materialization_hypertable_name}.bucket,  
                                ${materialization_hypertable_name}.item_id `;
                                }
                                metrics_in_materialized_hypertable.forEach(element => {
                                    if (!(element.column_name == "bucket" || element.column_name == "item_id")) {
                                        sql_view += `, ${materialization_hypertable_name}."${element.column_name}" `;
                                    }
                                });
                            }
                            for (let index_i = 0; index_i <= idx; index_i++) {
                                sql = `select materialization_hypertable_schema, materialization_hypertable_name from timescaledb_information.continuous_aggregates where view_schema = 'metrics' AND view_name = '${entity.tableName}_${interval.label}_${index_i}'`;
                                let test = await transactionalEntityManager.query(sql);
                                let materialization_hypertable_name = test[0].materialization_hypertable_name;
                                let materialization_hypertable_schema = test[0].materialization_hypertable_schema;
                                if (index_i == 0) {
                                    sql_view += ` FROM ${materialization_hypertable_schema}.${materialization_hypertable_name} `;
                                }
                                else {
                                    sql_view += `LEFT JOIN ${materialization_hypertable_schema}.${materialization_hypertable_name} ON ${materialization_hypertable_name}.item_id = ${first_mat_table}.item_id AND ${materialization_hypertable_name}.bucket = ${first_mat_table}.bucket `;
                                }
                            }
                            await transactionalEntityManager.query(`drop view metrics.${entity.tableName}_${interval.label}`);
                            await transactionalEntityManager.query(sql_view);
                        }
                    }
                });
            }
            catch (e) {
                console.error(e);
            }
        }
    }
    async generate() {
        const entityManager = this.dataSource;
        const entities = entityManager.connection.entityMetadatas;
        const schema_check = await entityManager.query("SELECT schema_name FROM information_schema.schemata WHERE schema_name = 'metrics';");
        if (!schema_check) {
            await entityManager.query('CREATE SCHEMA IF NOT EXISTS metrics;');
        }
        const min5minutesInterval = await MetricsWrapper_1.MetricsWrapper.getMin5MinutesInterval();
        for (const entity of entities) {
            if (entity.schema === "metrics") {
                const tableNameSplit = entity.tableName.split("_");
                const hwType = tableNameSplit[1];
                const hasHostCfg = await this.hostcfgRepository.findOneBy({ hw_type: hwType });
                if (hasHostCfg || this.allowedHwTypes.includes(hwType)) {
                    try {
                        await TableGenerator_1.TableGenerator.createPerformanceTableFromEntity(entity);
                    }
                    catch (e) {
                        console.error("Metrics table error [" + entity.tableName + "]: " + e);
                    }
                }
            }
        }
    }
};
exports.MetricsTableService = MetricsTableService;
exports.MetricsTableService = MetricsTableService = MetricsTableService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectEntityManager)()),
    __param(1, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __metadata("design:paramtypes", [typeorm_2.EntityManager,
        typeorm_2.Repository])
], MetricsTableService);
//# sourceMappingURL=metricsTable.service.js.map