"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicroservicesGateway = void 0;
const websockets_1 = require("@nestjs/websockets");
const socket_io_1 = require("socket.io");
const App_1 = require("../configuration/App");
const Microservice_1 = require("./Microservice");
const microservices_service_1 = require("./microservices.service");
let MicroservicesGateway = class MicroservicesGateway {
    constructor(microservicesService) {
        this.microservicesService = microservicesService;
        this.DEFAULT_TIMEOUT = 30 * 1000;
        this.TIMEOUT_PER_HW_TYPE = {
            'vspg': 2 * 60 * 1000,
            'power': 2 * 60 * 1000
        };
    }
    async handleConnection(socket, data) {
        const hostcfg_id = data.url.split("hostcfg_id=")[1];
        if (hostcfg_id) {
            let cfg = await this.microservicesService.getCfg(hostcfg_id);
            if (!cfg) {
                socket.send(JSON.stringify({
                    label: "Host configuration",
                    success: false,
                    text: "host configuration was not found"
                }));
                socket.close();
                return;
            }
            const TIMEOUT = this.TIMEOUT_PER_HW_TYPE[cfg.hw_type] !== undefined
                ? this.TIMEOUT_PER_HW_TYPE[cfg.hw_type]
                : this.DEFAULT_TIMEOUT;
            const secret = App_1.App.getSecret();
            let ms = new Microservice_1.Microservice();
            await ms.init(cfg.hw_type);
            if (ms.isRegistered() && ms.isActive()) {
                socket.send(JSON.stringify({
                    label: "Microservice test",
                    success: true
                }));
                const testData = cfg.data;
                testData['hostcfg_id'] = cfg.hostcfg_id;
                testData['hostalias'] = cfg.label;
                const msVariables = await this.microservicesService.getMicroserviceVariables(cfg.hw_type, testData["hostcfg_id"]);
                const net = require('net');
                let client = new net.Socket();
                const jwt = require('jsonwebtoken');
                let token = await jwt.sign({ microservice: ms.microservice }, secret);
                let message = {
                    jwt: token,
                    task: 'test',
                    data: {
                        ...msVariables,
                        ...testData
                    },
                };
                client.connect(ms.port, ms.host, () => {
                    client.write(JSON.stringify(message));
                });
                let messages = 0;
                client.on('data', (received) => {
                    try {
                        messages++;
                        const result = JSON.parse(received.toString());
                        if (result && Array.isArray(result)) {
                            for (const resultIndex in result) {
                                if (result[resultIndex].success !== undefined && result[resultIndex].label) {
                                    socket.send(JSON.stringify({
                                        label: result[resultIndex].label,
                                        success: result[resultIndex].success,
                                        text: result[resultIndex].text ? result[resultIndex].text : undefined,
                                        acceptable: result[resultIndex].acceptable ? result[resultIndex].acceptable : undefined,
                                    }));
                                }
                            }
                        }
                        else if (result && result.label && result.success !== undefined) {
                            clearTimeout(timeoutHandler);
                            timeoutHandler = setTimeout(() => {
                                if (!client.destroyed) {
                                    socket.send(JSON.stringify({
                                        label: "Microservice data result",
                                        success: false,
                                        text: "Timeouted"
                                    }));
                                    socket.close();
                                    client.destroy();
                                }
                            }, TIMEOUT);
                            socket.send(JSON.stringify({
                                label: result.label,
                                success: result.success,
                                text: result.text ? result.text : undefined,
                                acceptable: result.acceptable ? result.acceptable : undefined,
                            }));
                        }
                        else if (result && result.connected !== undefined) {
                            socket.send(JSON.stringify({
                                label: "Data test",
                                success: result.connected,
                                text: result.message ? result.message.join('\n') : undefined
                            }));
                        }
                        else {
                            socket.send(JSON.stringify({
                                label: "Microservice data result",
                                success: false,
                                text: "microservice return empty data"
                            }));
                            socket.close();
                            client.destroy();
                            return;
                        }
                    }
                    catch (e) {
                        socket.send(JSON.stringify({
                            label: "Corrupted data",
                            success: false,
                            text: e.toString()
                        }));
                        console.debug(received.toString());
                    }
                });
                client.on('close', () => {
                    if (messages === 0) {
                        socket.send(JSON.stringify({
                            label: "Microservice data result",
                            success: false,
                            text: "microservice does not return any data"
                        }));
                    }
                    socket.close();
                    client.destroy();
                });
                let timeoutHandler = setTimeout(() => {
                    if (!client.destroyed) {
                        socket.send(JSON.stringify({
                            label: "Microservice data result",
                            success: false,
                            text: "Timeouted"
                        }));
                        socket.close();
                        client.destroy();
                    }
                }, TIMEOUT);
            }
            else {
                socket.send(JSON.stringify({
                    label: "Microservice test",
                    success: false,
                    text: "microservice is not active or registered"
                }));
                socket.close();
                return;
            }
        }
        else {
            socket.send(JSON.stringify({
                label: "Host configuration",
                success: false,
                text: "no hostcfg_id was sent"
            }));
            socket.close();
            return;
        }
    }
    async connTest(protocol, host, port) {
        var net = require('net');
        return new Promise((resolve, reject) => {
            const timeout = 5000;
            const sock = net.createConnection({
                host: host,
                port: port
            }, function () {
                sock.destroy();
                resolve('');
            });
            sock.on('error', function (err) {
                reject(err);
            });
            setTimeout(() => {
                sock.end();
                reject('Connection timeout');
            }, timeout);
        });
    }
};
exports.MicroservicesGateway = MicroservicesGateway;
__decorate([
    (0, websockets_1.WebSocketServer)(),
    __metadata("design:type", socket_io_1.Server)
], MicroservicesGateway.prototype, "server", void 0);
exports.MicroservicesGateway = MicroservicesGateway = __decorate([
    (0, websockets_1.WebSocketGateway)({
        cors: {
            origin: '*',
        },
        path: '/websocket/mstest'
    }),
    __metadata("design:paramtypes", [microservices_service_1.MicroservicesService])
], MicroservicesGateway);
//# sourceMappingURL=microservice.gateway.js.map