"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicroservicesController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const swagger_1 = require("@nestjs/swagger");
const microserviceRequest_dto_1 = require("../../common/microserviceRequest.dto");
const microserviceJobInterval_dto_1 = require("../../common/microserviceJobInterval.dto");
const xormon_exception_1 = require("../../exception/xormon.exception");
const Microservice_1 = require("./Microservice");
const microserviceJobInterval_service_1 = require("./microserviceJobInterval.service");
const microservices_service_1 = require("./microservices.service");
const microserviceData_dto_1 = require("../../common/microserviceData.dto");
const microserviceData_service_1 = require("./microserviceData.service");
const Audit_1 = require("../configuration/Audit");
const auditEvent_entity_1 = require("../../entity/auditEvent.entity");
let MicroservicesController = class MicroservicesController {
    constructor(microservicesService, configService, microserviceJobsIntervalService, microserviceData) {
        this.microservicesService = microservicesService;
        this.configService = configService;
        this.microserviceJobsIntervalService = microserviceJobsIntervalService;
        this.microserviceData = microserviceData;
    }
    async listMicroservices() {
        const services = await this.microservicesService.getServices();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: services
        };
    }
    async registerMicroservice(microservice) {
        const registered = await this.microservicesService.registerService(microservice);
        if (registered) {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: "Service successfully registered"
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error occurred while registering the microservice", common_1.HttpStatus.BAD_GATEWAY);
        }
    }
    async deleteMicroservice(res, microservice) {
        let deleted = await this.microservicesService.deleteMicroservice(microservice);
        if (deleted) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'Microservice successfully deleted',
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Microservice has not been deleted', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async describeMicroservice(microservice) {
        const ms = await this.microservicesService.describeService(microservice);
        if (ms) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: ms
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occured', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getMicroserviceData(microservice, hostcfg_id) {
        const data = await this.microserviceData.getMicroserviceData(microservice, hostcfg_id);
        if (!data) {
            throw new xormon_exception_1.XormonException('data not found', common_1.HttpStatus.NOT_FOUND);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: data.value
        };
    }
    async storeMicroserviceData(microservice, data) {
        const store = await this.microserviceData.saveMicroserviceData(data);
        if (store === false) {
            throw new xormon_exception_1.XormonException('an error occurred while saving data', common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.CREATED,
            message: 'microservice data stored'
        };
    }
    async microserviceJobs(microservice) {
        const jobs = await this.microservicesService.getMicroserviceJobs(microservice);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: jobs
        };
    }
    async microserviceJobsHistory(microservice) {
        const jobs = await this.microservicesService.getMicroserviceJobsHistory(microservice);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: jobs.map(job => {
                return {
                    job: job.job,
                    identification: job.identification,
                    status: job.status,
                    pid: job.pid,
                    start: job.start,
                    end: job.end
                };
            })
        };
    }
    async resetMicroserviceJobInterval(microservice, job) {
        if (!microservice || !job) {
            throw new xormon_exception_1.XormonException(`Missing parameters`, common_1.HttpStatus.BAD_REQUEST);
        }
        let reset = false;
        try {
            for (let microserviceJob of Microservice_1.Microservice.jobs()) {
                if (microserviceJob.microservice === microservice && microserviceJob.job === job) {
                    reset = await this.microserviceJobsIntervalService.resetInterval(microserviceJob);
                    break;
                }
            }
        }
        catch (error) {
            throw new xormon_exception_1.XormonException(`Failed at resetting job interval for microservice: ${microservice} and specific job: ${job}`, common_1.HttpStatus.BAD_REQUEST);
        }
        if (!reset) {
            throw new xormon_exception_1.XormonException(`Failed at resetting job interval for microservice: ${microservice} and specific job: ${job}`, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: `Successfuly resetted job interval for microservice: ${microservice} and job: ${job}`
        };
    }
    async updateMicroserviceJobInterval(microserviceJobIntervalDTO, microservice, job) {
        let updated = await this.microserviceJobsIntervalService.updateInterval(microservice, job, microserviceJobIntervalDTO.interval);
        if (!updated) {
            throw new xormon_exception_1.XormonException(`Failed at updating job interval for microservice: ${microservice} and job: ${job}`, common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            message: `Successfuly updated job interval for microservice: ${microservice} and job: ${job}`
        };
    }
    async startMicroservice(request, microservice) {
        const data = await this.microservicesService.startService(microservice, true);
        if (data) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.MICROSERVICE_START, data.label);
            return {
                statusCode: common_1.HttpStatus.OK,
                data: data
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occured', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async stopMicroservice(request, microservice) {
        let data = await this.microservicesService.stopService(microservice, true);
        if (data) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.MICROSERVICE_STOP, data.label);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: "Service stopped successfully",
                data: data
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occured', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async restartMicroservice(res, microservice) {
        const restarted = await this.microservicesService.restartService(microservice);
        if (restarted) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: "Service restarted successfully"
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occured', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async testkMicroservice(microservice, data, hostcfg_id) {
        if (hostcfg_id) {
            let cfg = await this.microservicesService.getCfg(hostcfg_id);
            if (!cfg) {
                throw new xormon_exception_1.XormonException("Hostcfg does not exist", common_1.HttpStatus.NOT_FOUND);
            }
            if (cfg.data) {
                data = cfg.data;
                data['hostcfg_id'] = cfg.hostcfg_id;
                data['hostalias'] = cfg.label;
            }
        }
        const returned = await this.microservicesService.sendTask(microservice.toLowerCase(), 'test', data);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: returned
        };
    }
    async taskMicroservice(microservice, task, data, hostcfg_id) {
        if (hostcfg_id) {
            let cfg = await this.microservicesService.getCfg(hostcfg_id);
            if (!cfg) {
                throw new xormon_exception_1.XormonException("Hostcfg does not exist", common_1.HttpStatus.NOT_FOUND);
            }
            if (cfg.data) {
                data = cfg.data;
                data['hostcfg_id'] = cfg.hostcfg_id;
                data['hostalias'] = cfg.label;
            }
            else {
                throw new xormon_exception_1.XormonException("Hostcfg does not exist", common_1.HttpStatus.NOT_FOUND);
            }
        }
        const returned = await this.microservicesService.sendTask(microservice.toLowerCase(), task, data);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: returned
        };
    }
};
exports.MicroservicesController = MicroservicesController;
__decorate([
    openapi.ApiOperation({ description: "list all microservices" }),
    (0, common_1.Get)('/'),
    openapi.ApiResponse({ status: 200, type: require("../../common/backendResponse.dto").BackendServicesResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "listMicroservices", null);
__decorate([
    openapi.ApiOperation({ description: "register microservice" }),
    (0, common_1.Post)('/'),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [microserviceRequest_dto_1.MicroserviceRequestDTO]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "registerMicroservice", null);
__decorate([
    openapi.ApiOperation({ description: "delete microservice" }),
    (0, common_1.Delete)('/:microservice'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Res)({ passthrough: true })),
    __param(1, (0, common_1.Param)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "deleteMicroservice", null);
__decorate([
    openapi.ApiOperation({ description: "desribe microservice" }),
    (0, common_1.Get)('/:microservice'),
    openapi.ApiResponse({ status: 200, type: require("../../common/backendResponse.dto").BackendServiceResponseDTO }),
    __param(0, (0, common_1.Param)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "describeMicroservice", null);
__decorate([
    openapi.ApiOperation({ description: "get microservice data" }),
    (0, common_1.Get)('/:microservice/data/:hostcfg_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('microservice')),
    __param(1, (0, common_1.Param)('hostcfg_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "getMicroserviceData", null);
__decorate([
    openapi.ApiOperation({ description: "store microservice data" }),
    (0, common_1.Put)('/:microservice/data'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('microservice')),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, microserviceData_dto_1.MicroserviceDataDTO]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "storeMicroserviceData", null);
__decorate([
    openapi.ApiOperation({ description: "microservice jobs" }),
    (0, common_1.Get)('/:microservice/jobs'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Param)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "microserviceJobs", null);
__decorate([
    openapi.ApiOperation({ description: "microservice jobs history" }),
    (0, common_1.Get)('/:microservice/jobs/history'),
    openapi.ApiResponse({ status: 200, type: require("../../common/microserviceJob.dto").MicroserviceJobsResponseDTO }),
    __param(0, (0, common_1.Param)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "microserviceJobsHistory", null);
__decorate([
    openapi.ApiOperation({ description: "reset microservice job interval" }),
    (0, common_1.Get)(':microservice/job/:job/reset'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Param)("microservice")),
    __param(1, (0, common_1.Param)("job")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "resetMicroserviceJobInterval", null);
__decorate([
    openapi.ApiOperation({ description: "update microservice job interval" }),
    (0, common_1.Put)(':microservice/job/:job'),
    openapi.ApiResponse({ status: 200 }),
    __param(0, (0, common_1.Body)()),
    __param(1, (0, common_1.Param)("microservice")),
    __param(2, (0, common_1.Param)("job")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [microserviceJobInterval_dto_1.MicroserviceJobIntervalDTO, String, String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "updateMicroserviceJobInterval", null);
__decorate([
    openapi.ApiOperation({ description: "start microservice" }),
    (0, common_1.Patch)('/:microservice/start'),
    openapi.ApiResponse({ status: 200, type: require("../../common/microserviceResponse.dto").MicroserviceChangeStateResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "startMicroservice", null);
__decorate([
    openapi.ApiOperation({ description: "stop microservice" }),
    (0, common_1.Patch)('/:microservice/stop'),
    openapi.ApiResponse({ status: 200, type: require("../../common/microserviceResponse.dto").MicroserviceChangeStateResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "stopMicroservice", null);
__decorate([
    openapi.ApiOperation({ description: "restart microservice" }),
    (0, common_1.Patch)('/:microservice/restart'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Res)({ passthrough: true })),
    __param(1, (0, common_1.Param)('microservice')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "restartMicroservice", null);
__decorate([
    openapi.ApiOperation({ description: "apitest /datatest to microservice" }),
    (0, common_1.Post)('/:microservice/test'),
    openapi.ApiResponse({ status: 201, type: require("../../common/microserviceResponse.dto").MicroserviceResponseDTO }),
    __param(0, (0, common_1.Param)('microservice')),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Query)('hostcfg_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Object, Number]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "testkMicroservice", null);
__decorate([
    openapi.ApiOperation({ description: "send task to microservice" }),
    (0, common_1.Post)('/:microservice/:task'),
    openapi.ApiResponse({ status: 201, type: require("../../common/microserviceResponse.dto").MicroserviceResponseDTO }),
    __param(0, (0, common_1.Param)('microservice')),
    __param(1, (0, common_1.Param)('task')),
    __param(2, (0, common_1.Body)()),
    __param(3, (0, common_1.Query)('hostcfg_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, Object, Number]),
    __metadata("design:returntype", Promise)
], MicroservicesController.prototype, "taskMicroservice", null);
exports.MicroservicesController = MicroservicesController = __decorate([
    (0, swagger_1.ApiTags)('Microservices'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/microservices'),
    __metadata("design:paramtypes", [microservices_service_1.MicroservicesService,
        config_1.ConfigService,
        microserviceJobInterval_service_1.MicroserviceJobIntervalService,
        microserviceData_service_1.MicroserviceDataService])
], MicroservicesController);
//# sourceMappingURL=microservices.controller.js.map