"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var MicroservicesService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicroservicesService = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const dataSource_1 = require("../../config/dataSource");
const hostcfg_entity_1 = require("../../entity/hostcfg.entity");
const microservice_entity_1 = require("../../entity/microservice.entity");
const microserviceJobs_entity_1 = require("../../entity/microserviceJobs.entity");
const microserviceJobsStatus_entity_1 = require("../../entity/microserviceJobsStatus.entity");
const xormon_exception_1 = require("../../exception/xormon.exception");
const typeorm_2 = require("typeorm");
const App_1 = require("../configuration/App");
const PM2_1 = require("../configuration/PM2");
const Microservice_1 = require("./Microservice");
const microserviceVariables_entity_1 = require("../../entity/microserviceVariables.entity");
const Crypto_1 = require("../configuration/Crypto");
const Variables_1 = require("../configuration/Variables");
let MicroservicesService = MicroservicesService_1 = class MicroservicesService {
    constructor(microservicesRepository, microservicesJobsRepository, microservicesJobsStatusRepository, hostcfgRepository, microserviceVariablesRepository) {
        this.microservicesRepository = microservicesRepository;
        this.microservicesJobsRepository = microservicesJobsRepository;
        this.microservicesJobsStatusRepository = microservicesJobsStatusRepository;
        this.hostcfgRepository = hostcfgRepository;
        this.microserviceVariablesRepository = microserviceVariablesRepository;
        this.logger = new common_1.Logger(MicroservicesService_1.name);
    }
    async onApplicationBootstrap() {
        if (!PM2_1.PM2.isRunning() || !PM2_1.PM2.isMasterBackend()) {
            return;
        }
        this.logger.log(`Initializing microservices`);
        await PM2_1.PM2.restartService("metrics-server");
        await PM2_1.PM2.restartService("tasks-server");
        const os = require('os');
        const cpus = os.cpus().length;
        this.logger.log(`[metrics-server scaling] ${cpus} cores detected`);
        if (cpus > 4) {
            const base = 2;
            const max = 4;
            let toSpawn = (cpus / 2) - base;
            let taskScale = 0;
            toSpawn = Math.floor(toSpawn);
            if (toSpawn > max) {
                taskScale = toSpawn - max > max ? max : toSpawn - max;
                toSpawn = max;
            }
            else if (toSpawn > 2) {
                taskScale = 1;
                toSpawn--;
            }
            if (toSpawn > 0) {
                this.logger.log(`[metrics-server scaling] spawning ${toSpawn} process${toSpawn > 1 ? "es" : ""}`);
                await PM2_1.PM2.scale('metrics-server', toSpawn + base);
            }
            if (taskScale > 0) {
                this.logger.log(`[tasks-server scaling] spawning ${taskScale} process${taskScale > 1 ? "es" : ""}`);
                await PM2_1.PM2.scale('tasks-server', taskScale + 1);
            }
        }
        this.startAnalyzerService();
        const jobsToSave = [];
        const jobsToSaveMsVar = [];
        const jobs = Microservice_1.Microservice.jobs();
        const savedJobs = await this.microserviceVariablesRepository.findBy({
            hw_type: (0, typeorm_2.In)(jobs.map(j => j.microservice)),
            key: (0, typeorm_2.In)(jobs.map(j => j.job))
        });
        for (const job of jobs) {
            const variableJob = savedJobs.find(j => j.key === job.job && j.hw_type === job.microservice);
            if (!variableJob) {
                jobsToSaveMsVar.push({
                    job: job.job,
                    interval: job.interval,
                    microservice: job.microservice
                });
                jobsToSave.push({
                    job: job.job,
                    interval: job.interval,
                    microservice: job.microservice
                });
            }
            else {
                jobsToSave.push({
                    job: variableJob.key,
                    interval: parseInt(variableJob.value),
                    microservice: variableJob.hw_type
                });
            }
        }
        const msList = Microservice_1.Microservice.list();
        const osAgent = await this.microservicesRepository.findOneBy({ hw_type: 'linux' });
        if (!osAgent) {
            const osAgentIndex = msList.findIndex(ms => ms.hw_type === 'linux');
            if (osAgentIndex !== -1) {
                msList[osAgentIndex].onStartup = true;
            }
        }
        await this.microserviceVariablesRepository.save(jobsToSaveMsVar.map(j => {
            return {
                key: j.job,
                value: j.interval.toString(),
                hw_type: j.microservice
            };
        }));
        await this.microservicesRepository.save(msList);
        await this.microservicesJobsRepository.save(jobsToSave);
        if (App_1.App.isProduction() || await App_1.App.isMaster()) {
            this.startServicesFromDB();
        }
        else {
            let microservices = await this.microservicesRepository.find();
            for (let i = 0; i < microservices.length; i++) {
                await this.stopService(microservices[i].microservice);
            }
        }
    }
    async startAnalyzerService() {
        const analyzerLabel = 'analyzer';
        const definition = {
            name: analyzerLabel,
            script: `./microservices/${analyzerLabel}/analyzer-service.py`,
            cwd: `./microservices/${analyzerLabel}`,
            error_file: `${App_1.App.LOGS_PATH}/${analyzerLabel}-error.log`,
            out_file: `${App_1.App.LOGS_PATH}/${analyzerLabel}-error.log`,
            interpreter: `${App_1.App.ROOT_PATH}/server-nest/venv/bin/python3`,
            env: {
                DB_HOST: process.env['DB_HOST'],
                DB_PORT: process.env['DB_PORT'],
                DB_USERNAME: process.env['DB_USERNAME'],
                DB_PASSWORD: await Crypto_1.Crypto.decrypt(process.env['DB_PASSWORD']),
                DB_DATABASE: process.env['DB_DATABASE']
            }
        };
        try {
            await PM2_1.PM2.registerService(definition);
            const analyzer = await PM2_1.PM2.describeService(analyzerLabel);
            if (!analyzer.active) {
                this.logger.warn("Failed to start analyzer service");
            }
        }
        catch (e) {
            this.logger.warn("Failed to register analyzer service");
            this.logger.warn(`[Analyzer]: ${e}`);
        }
    }
    async startServicesFromDB() {
        const path = require('path');
        const jwt = require('jsonwebtoken');
        let defaultRegistered = ['linux'];
        let hostcfgHwTypes = (await this.hostcfgRepository.find())
            .map(hostcfg => hostcfg.hw_type);
        hostcfgHwTypes = hostcfgHwTypes.filter(function (hwType, pos) {
            return hostcfgHwTypes.indexOf(hwType) == pos;
        });
        defaultRegistered = defaultRegistered.concat(hostcfgHwTypes);
        let services = [];
        const secret = App_1.App.getSecret();
        let otherMicroservices = await this.microservicesRepository.findBy({
            onStartup: false,
            hw_type: (0, typeorm_2.Not)((0, typeorm_2.In)(defaultRegistered))
        });
        for (let otherMs of otherMicroservices) {
            this.stopService(otherMs.microservice);
        }
        let microservices = await this.microservicesRepository.find({ where: [
                { onStartup: true },
                { microservice: (0, typeorm_2.In)(defaultRegistered) }
            ] });
        for (const microservice of microservices) {
            let data = JSON.parse('{ "name": "' +
                microservice.microservice +
                '", "script": "./microservices/' +
                microservice.microservice +
                '/' +
                microservice.script +
                '", "cwd": "./microservices/' +
                microservice.microservice +
                '/", "error_file": "' + App_1.App.LOGS_PATH + '/' +
                microservice.microservice +
                '-error.log", "out_file": "' + App_1.App.LOGS_PATH + '/' +
                microservice.microservice +
                '.log" }');
            const service = new Microservice_1.Microservice();
            await service.init(microservice.microservice);
            if (!service.isAvailable()) {
                await service.register(data);
                if (!microservice.onStartup) {
                    await service.stop();
                }
            }
            else if (microservice.onStartup) {
                await service.start();
            }
            if (microservice.needConf) {
                setTimeout(async () => {
                    let token = await jwt.sign({ microservice: microservice.microservice }, secret);
                    let message = {
                        jwt: token,
                    };
                    let received;
                    try {
                        received = await service.sendData(JSON.stringify(message));
                    }
                    catch (error) {
                        this.logger.log('[' + microservice.microservice + '] unable to send configuration');
                    }
                }, 3000);
            }
            services.push({
                status: service.isActive() ? 'online' : 'offline',
                name: service.microservice,
            });
        }
        for (let i = 0; i < services.length; i++) {
            this.logger.log('Service ' + services[i].name + ': ' + services[i].status);
        }
    }
    async getServices() {
        let data = [];
        const coreServices = Microservice_1.Microservice.getCoreServices();
        const servicesFromDB = await this.microservicesRepository.find();
        let list = await PM2_1.PM2.listServices();
        for (let i = 0; i < list.length; i++) {
            const service = list[i];
            if (coreServices.includes(service.name)) {
                continue;
            }
            let active = service.pm2_env.status === 'online' ? true : false;
            let serviceFromDB = servicesFromDB.find(ser => ser.microservice === service.name);
            data.push({
                label: serviceFromDB ? serviceFromDB.label : service.name,
                microservice: service.name,
                script: service.pm2_env.script,
                active: active,
                pid: service.pid,
                version: service.pm2_env.version,
                monitor: {
                    memory_byte: service.monit.memory,
                    cpu_percent: service.monit.cpu,
                },
            });
        }
        return data;
    }
    async registerService(microservice) {
        if (microservice) {
            let check = await this.microservicesRepository.findBy({ microservice: microservice.microservice });
            if (check.length !== 0) {
                return false;
            }
            else {
                let time = new Date().toUTCString();
                let hw_type = microservice.hw_type ? microservice.hw_type : null;
                let save = this.saveMicroservice({
                    microservice: microservice.microservice,
                    script: microservice.script,
                    host: microservice.host,
                    port: microservice.port,
                    hw_type: hw_type,
                    created: time,
                    updated: time,
                });
                if (save) {
                    return true;
                }
                else {
                    return false;
                }
            }
        }
        else {
            return false;
        }
    }
    async deleteMicroservice(microsevice) {
        const deleted = await this.microservicesRepository.delete({ microservice: microsevice });
        if (deleted.affected === 1) {
            await this.microservicesJobsRepository.delete({ microservice: microsevice });
            await this.microservicesJobsStatusRepository.delete({ microservice: microsevice });
            return true;
        }
        else {
            return false;
        }
    }
    async describeService(name) {
        let microservice = new Microservice_1.Microservice();
        await microservice.init(name);
        if (!microservice.isRegistered()) {
            throw new xormon_exception_1.XormonException("Service not found", common_1.HttpStatus.NOT_FOUND);
        }
        else if (!microservice.isAvailable()) {
            throw new xormon_exception_1.XormonException("Service is not available", common_1.HttpStatus.BAD_REQUEST);
        }
        return {
            label: microservice.label,
            microservice: microservice.microservice,
            active: microservice.isActive(),
            version: microservice.version,
            script: microservice.script,
            monitor: microservice.monitor,
        };
    }
    async startService(name, changeOnStartup = false) {
        let microservice = new Microservice_1.Microservice();
        await microservice.init(name);
        if (changeOnStartup) {
            microservice.changeOnStartup(true);
        }
        if (microservice.isRegistered()) {
            if (microservice.isActive()) {
                return microservice.getData();
            }
            else {
                await microservice.start();
                await this.sleep(1000);
                if (microservice.isActive()) {
                    return microservice.getData();
                }
                else {
                    throw new xormon_exception_1.XormonException("Service cannot be turned on", common_1.HttpStatus.BAD_REQUEST);
                }
            }
        }
        else {
            throw new xormon_exception_1.XormonException("Service not found", common_1.HttpStatus.NOT_FOUND);
        }
    }
    async stopService(name, changeOnStartup = false) {
        if (name === 'backend') {
            throw new xormon_exception_1.XormonException("Backend can not be turned off", common_1.HttpStatus.BAD_REQUEST);
        }
        let microservice = new Microservice_1.Microservice();
        await microservice.init(name);
        if (changeOnStartup) {
            microservice.changeOnStartup(false);
        }
        if (microservice.isRegistered()) {
            if (microservice.isActive()) {
                await microservice.stop();
                if (!microservice.isActive()) {
                    return microservice.getData();
                }
                else {
                    return null;
                }
            }
            else {
                return microservice.getData();
            }
        }
        else {
            throw new xormon_exception_1.XormonException("Service not found", common_1.HttpStatus.NOT_FOUND);
        }
    }
    async restartService(name) {
        let microservice = new Microservice_1.Microservice();
        await microservice.init(name);
        if (microservice.isRegistered()) {
            await microservice.start();
            if (microservice.isActive()) {
                return true;
            }
            else {
                return false;
            }
        }
        else {
            throw new xormon_exception_1.XormonException("Service not found", common_1.HttpStatus.NOT_FOUND);
        }
    }
    async sendTask(name, task, data) {
        let microservice = new Microservice_1.Microservice();
        await microservice.init(name);
        if (!microservice.isRegistered() || !microservice.isActive()) {
            await microservice.start();
            this.sleep(500);
        }
        if (microservice.isRegistered()) {
            if (microservice.isActive()) {
                const jwt = require('jsonwebtoken');
                let token = await jwt.sign({ microservice: microservice.microservice }, App_1.App.getSecret());
                let logger = new common_1.Logger(MicroservicesService_1.name);
                const msVariables = await this.getMicroserviceVariables(name, data["hostcfg_id"]);
                let message = {
                    jwt: token,
                    task: task,
                    data: {
                        ...msVariables,
                        ...data
                    },
                };
                logger.debug('[API] Task: ' + task + ' Service: ' + name);
                let received;
                try {
                    received = await microservice.sendData(JSON.stringify(message));
                }
                catch (error) {
                    logger.debug(error);
                    throw new xormon_exception_1.XormonException("Service is not available", common_1.HttpStatus.BAD_REQUEST);
                }
                if (this.isJson(received.toString())) {
                    return JSON.parse(received.toString());
                }
                return received.toString();
            }
            else {
                throw new xormon_exception_1.XormonException("Service is not active", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        else {
            throw new xormon_exception_1.XormonException("Service not found", common_1.HttpStatus.NOT_FOUND);
        }
    }
    async saveMicroservice(items) {
        const insert = await (0, dataSource_1.getDataSource)().createQueryBuilder().insert().into(microservice_entity_1.Microservice).values(items).execute();
        if (insert.generatedMaps.length >= 1) {
            return true;
        }
        else {
            return false;
        }
    }
    async getMicroservice(microservice) {
        return this.microservicesRepository.findOneBy({ microservice: microservice });
    }
    async getMicroserviceJobsHistory(microservice) {
        return this.microservicesJobsStatusRepository.find({ where: { microservice: microservice }, order: { start: 'DESC' }, take: 50 });
    }
    async getMicroserviceJobs(microservice) {
        return this.microservicesJobsRepository.find({ where: { microservice: microservice } });
    }
    async getCfg(hostcfg_id) {
        return this.hostcfgRepository.findOneBy({ hostcfg_id: hostcfg_id });
    }
    isJson(str) {
        try {
            JSON.parse(str);
        }
        catch (e) {
            return false;
        }
        return true;
    }
    async sleep(ms) {
        return new Promise((resolve) => setTimeout(resolve, ms));
    }
    async getMicroserviceVariables(hw_type, hostcfg_id) {
        const baseVariables = await Variables_1.Variables.getAllKeys();
        const hwTypeVariables = await Variables_1.Variables.getAllKeys(hw_type);
        const hostcfgVariables = await Variables_1.Variables.getAllKeys(null, hostcfg_id);
        return {
            ...baseVariables,
            ...hwTypeVariables,
            ...hostcfgVariables
        };
    }
};
exports.MicroservicesService = MicroservicesService;
exports.MicroservicesService = MicroservicesService = MicroservicesService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(microservice_entity_1.Microservice)),
    __param(1, (0, typeorm_1.InjectRepository)(microserviceJobs_entity_1.MicroserviceJob)),
    __param(2, (0, typeorm_1.InjectRepository)(microserviceJobsStatus_entity_1.MicroserviceJobStatus)),
    __param(3, (0, typeorm_1.InjectRepository)(hostcfg_entity_1.HostCfg)),
    __param(4, (0, typeorm_1.InjectRepository)(microserviceVariables_entity_1.MicroserviceVariables)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], MicroservicesService);
//# sourceMappingURL=microservices.service.js.map