"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArchitecturePublicController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const architecture_dto_1 = require("../../../common/architecture.dto");
const architecture_interceptor_1 = require("../../../interceptor/architecture.interceptor");
const architecture_service_1 = require("../../architecture/architecture.service");
;
;
const MetricsWrapper_1 = require("../../metrics/MetricsWrapper");
const ApiKey_1 = require("../ApiKey");
const ACLCheck_1 = require("../../acl/ACLCheck");
let ArchitecturePublicController = class ArchitecturePublicController {
    constructor(architectureService) {
        this.architectureService = architectureService;
    }
    async getDevices(request) {
        const user = await ApiKey_1.ApiKey.getUserByRequest(request, true);
        let devices = await this.architectureService.getRootItems(true);
        const ids = await ACLCheck_1.ACLCheck.filterItems(user.groups.map(g => g.group_id), devices.map(i => i.item_id));
        devices = devices.filter(item => ids.includes(item.item_id));
        return {
            statusCode: common_1.HttpStatus.OK,
            data: devices.map(device => {
                return {
                    item_id: device.item_id,
                    label: device.label,
                    hw_type: device.hw_type,
                    class: device.hw_type_entity.class,
                    subsystem: device.subsystem,
                    hostcfg_id: device.hostcfg_relations
                        ? device.hostcfg_relations[0]?._hostcfg.hostcfg_id
                        : undefined
                };
            }),
        };
    }
    async getClasses() {
        let classes = await this.architectureService.getClasses();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: classes,
        };
    }
    async getClassHwTypes(_class, list_models = false) {
        let hwTypes = await this.architectureService.getHwTypes(null, _class, list_models);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: hwTypes,
        };
    }
    async getHwTypeSubsystems(_class, hwType) {
        let subsystems = await this.architectureService.getSubsystems(null, undefined, hwType);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: subsystems.map(subsystem => {
                return {
                    label: subsystem.label,
                    subsystem: subsystem.subsystem
                };
            }),
        };
    }
    async getHwTypeSubsystemMetrics(_class, hwType, subsystem) {
        let metrics = await MetricsWrapper_1.MetricsWrapper.getMetrics(hwType, subsystem);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: metrics.map(metric => {
                return {
                    metric: metric.metric,
                    label: metric.label,
                    shortcut: metric.shortcut,
                    unit: metric.unit
                };
            }),
        };
    }
    getHwTypeSubsystemProperties(_class, hwType, subsystem) {
        let props = [];
        const properties = MetricsWrapper_1.MetricsWrapper.getConfiguration(_class, hwType, subsystem);
        for (const propertyKey in properties) {
            props.push({
                property_name: propertyKey,
                label: properties[propertyKey] && properties[propertyKey].label ? properties[propertyKey].label : propertyKey
            });
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: props
        };
    }
    async itemsByHwTypeAndSubsystem(_class, hw_type, subsystem) {
        let items = await this.architectureService.getByHwTypeAndSubsystem(hw_type, subsystem);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items.map(item => {
                return {
                    item_id: item.item_id,
                    label: item.label,
                    hw_type: item.hw_type,
                    subsystem: item.subsystem
                };
            }),
        };
    }
    async search(request, subsystem, hw_type, _class, parents, search, page, limit) {
        const user = await ApiKey_1.ApiKey.getUserByRequest(request, true);
        limit = limit ? limit : 50;
        if (parents !== undefined) {
            if (!Array.isArray(parents)) {
                parents = parents.split(",");
            }
        }
        let itemsWithName = [];
        let items = await this.architectureService.getItems({
            subsystem: subsystem,
            hw_type: hw_type,
            _class: _class,
            parents: parents,
            withParents: false,
            search: search
        });
        const ids = await ACLCheck_1.ACLCheck.filterItems(user.groups.map(g => g.group_id), items.map(i => i.item_id));
        items = items.filter(item => ids.includes(item.item_id));
        const metadata = {
            start: page ? page * limit : 0,
            end: page ? (page * limit + limit - 1 >= items.length ? items.length : page * limit + limit - 1) : limit - 1,
            max: items.length >= 1 ? items.length - 1 : 0,
            page: page ? +page : 0,
        };
        if (metadata.start > metadata.max) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    metadata: {
                        start: metadata.start,
                        end: metadata.end,
                        max: metadata.max,
                        page: metadata.page,
                        info: 'the page number has a higher start index than the max index',
                    },
                    items: [],
                },
            };
        }
        if (items.length >= 1) {
            items = items.slice(metadata.start, metadata.end + 1);
            for (const item of items) {
                const itemParents = [];
                if (item.parents && item.parents[0]._parent) {
                    for (const itemParent of item.parents) {
                        itemParents.push({
                            label: itemParent._parent.label,
                            item_id: itemParent._parent.item_id,
                            subsystem: itemParent._parent.subsystem,
                            hw_type: itemParent._parent.hw_type,
                        });
                    }
                }
                itemsWithName.push({
                    label: item.label,
                    item_id: item.item_id,
                    hw_type: item.hw_type,
                    class: item.hw_type_entity.class,
                    subsystem: item.subsystem,
                    parents: itemParents.length >= 1 ? itemParents : undefined
                });
            }
        }
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                metadata: metadata,
                items: itemsWithName,
            },
        };
    }
};
exports.ArchitecturePublicController = ArchitecturePublicController;
__decorate([
    openapi.ApiOperation({ description: "return devices (root items)" }),
    (0, common_1.Get)('devices'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ArchitecturePublicController.prototype, "getDevices", null);
__decorate([
    openapi.ApiOperation({ description: "return classes" }),
    (0, common_1.UseInterceptors)(architecture_interceptor_1.ArchitectureInterceptor),
    (0, common_1.Get)('classes'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").ClassesResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], ArchitecturePublicController.prototype, "getClasses", null);
__decorate([
    openapi.ApiOperation({ description: "return hw_types by class" }),
    (0, common_1.Get)('class/:class/hw_types'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").HwTypesResponseDTO }),
    __param(0, (0, common_1.Param)('class')),
    __param(1, (0, common_1.Query)("list_models", new common_1.ParseBoolPipe({ optional: true }))),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, Boolean]),
    __metadata("design:returntype", Promise)
], ArchitecturePublicController.prototype, "getClassHwTypes", null);
__decorate([
    openapi.ApiOperation({ description: "return subsystems by hw_type" }),
    (0, common_1.Get)('class/:class/hw_type/:hw_type/subsystems'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").SubsystemsResponseDTO }),
    __param(0, (0, common_1.Param)('class')),
    __param(1, (0, common_1.Param)('hw_type')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ArchitecturePublicController.prototype, "getHwTypeSubsystems", null);
__decorate([
    openapi.ApiOperation({ description: "return metrics by hw_type and subsystem" }),
    (0, common_1.Get)('class/:class/hw_type/:hw_type/subsystem/:subsystem/metrics'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").MetricsResponseDTO }),
    __param(0, (0, common_1.Param)('class')),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('subsystem')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], ArchitecturePublicController.prototype, "getHwTypeSubsystemMetrics", null);
__decorate([
    openapi.ApiOperation({ description: "return properties by hw_type and subsystem" }),
    (0, common_1.Get)('class/:class/hw_type/:hw_type/subsystem/:subsystem/properties'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").PropsResponseDTO }),
    __param(0, (0, common_1.Param)('class')),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('subsystem')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", architecture_dto_1.PropsResponseDTO)
], ArchitecturePublicController.prototype, "getHwTypeSubsystemProperties", null);
__decorate([
    openapi.ApiOperation({ description: "return items by hw_type and subsystem" }),
    (0, common_1.Get)('class/:class/hw_type/:hw_type/subsystem/:subsystem/items'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Param)('class')),
    __param(1, (0, common_1.Param)('hw_type')),
    __param(2, (0, common_1.Param)('subsystem')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String, String]),
    __metadata("design:returntype", Promise)
], ArchitecturePublicController.prototype, "itemsByHwTypeAndSubsystem", null);
__decorate([
    openapi.ApiOperation({ description: "search items" }),
    (0, common_1.Get)('search'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/architecture.dto").ItemsSearchResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('subsystem')),
    __param(2, (0, common_1.Query)('hw_type')),
    __param(3, (0, common_1.Query)('class')),
    __param(4, (0, common_1.Query)('parents')),
    __param(5, (0, common_1.Query)('search')),
    __param(6, (0, common_1.Query)('page')),
    __param(7, (0, common_1.Query)('limit')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String, String, Array, String, Number, Number]),
    __metadata("design:returntype", Promise)
], ArchitecturePublicController.prototype, "search", null);
exports.ArchitecturePublicController = ArchitecturePublicController = __decorate([
    (0, swagger_1.ApiTags)('PublicArchitecture'),
    (0, common_1.Controller)('api/public/v1/architecture'),
    __metadata("design:paramtypes", [architecture_service_1.ArchitectureService])
], ArchitecturePublicController);
//# sourceMappingURL=architecture.public.controller.js.map