"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HostcfgPublicController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const hostCfgResponse_dto_1 = require("../../../common/hostCfgResponse.dto");
const License_1 = require("../../../config/License");
const dataSource_1 = require("../../../config/dataSource");
const auditEvent_entity_1 = require("../../../entity/auditEvent.entity");
const hwType_entity_1 = require("../../../entity/hwType.entity");
const xormon_exception_1 = require("../../../exception/xormon.exception");
const adminRoles_guard_1 = require("../../../guard/adminRoles.guard");
const Audit_1 = require("../../configuration/Audit");
const Item_1 = require("../../configuration/Item");
const configuration_service_1 = require("../../configuration/configuration.service");
const hostCfg_service_1 = require("../../configuration/hostCfg.service");
let HostcfgPublicController = class HostcfgPublicController {
    constructor(hostCfgService, configurationService) {
        this.hostCfgService = hostCfgService;
        this.configurationService = configurationService;
    }
    async getHostCfg(_class, hw_type) {
        let hostcfg = await this.hostCfgService.getHostCfgData();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: (await License_1.License.checkHostcfgLimits(hostcfg))
                .sort((a, b) => a.label.localeCompare(b.label, undefined, { numeric: true, sensitivity: 'base' }))
        };
    }
    async getHostCfgDefinitions() {
        let HostcfgDefinitions = this.hostCfgService.getHostcfgDefinition();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: HostcfgDefinitions
        };
    }
    async getHostCfgDefinitionsHwType(hwType) {
        let HostcfgDefinitions = this.hostCfgService.getHostcfgDefinition(hwType);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: HostcfgDefinitions
        };
    }
    async saveHostCfg(request, hostcfg, source) {
        const hwType = await (0, dataSource_1.getDataSource)().getRepository(hwType_entity_1.HwType).findOneBy({ hw_type: hostcfg.hw_type });
        const technology = ['san', 'storage'].includes(hwType.class)
            ? hwType.class
            : hwType.hw_type;
        if (!(await License_1.License.checkTechnology(technology, true))) {
            throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
        }
        const saved = await this.hostCfgService.saveHostcfgData(hostcfg, source);
        if (saved.hostcfg_id) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.HOSTCFG_ADD, hostcfg.label + ' [' + hostcfg.hw_type + ']');
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'successfully saved',
                data: saved,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while saving hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateHostCfg(request, hostcfg_id, hostcfg) {
        const hwType = await (0, dataSource_1.getDataSource)().getRepository(hwType_entity_1.HwType).findOneBy({ hw_type: hostcfg.hw_type });
        const technology = ['san', 'storage'].includes(hwType.class)
            ? hwType.class
            : hwType.hw_type;
        if (!(await License_1.License.checkTechnology(technology, true))) {
            throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
        }
        const [updated, old] = await this.hostCfgService.updateHostcfgData(hostcfg_id, hostcfg);
        if (updated) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.HOSTCFG_EDIT, hostcfg.label === old.label ? hostcfg.label + ' [' + hostcfg.hw_type + ']' : "Old alias: " + old.label + ", new alias: " + hostcfg.label + " [" + hostcfg.hw_type + "]");
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'hostcfg successfully updated',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while updating hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async deleteHostCfg(request, hostcfg_id) {
        let hostcfg = await this.hostCfgService.getHostCfg(hostcfg_id);
        if (!hostcfg || !hostcfg_id) {
            throw new xormon_exception_1.XormonException("Hostcfg not found", common_1.HttpStatus.NOT_FOUND);
        }
        const hwType = await (0, dataSource_1.getDataSource)().getRepository(hwType_entity_1.HwType).findOneBy({ hw_type: hostcfg.hw_type });
        const technology = ['san', 'storage'].includes(hwType.class)
            ? hwType.class
            : hwType.hw_type;
        if (!(await License_1.License.checkTechnology(technology, true))) {
            throw new xormon_exception_1.XormonException("You do not have an active API license for the given technology", common_1.HttpStatus.PAYMENT_REQUIRED);
        }
        let deleted = undefined;
        try {
            await Item_1.Item.removeDeviceInfo(hostcfg_id);
            deleted = await this.hostCfgService.deleteHostCfg(hostcfg);
        }
        catch (e) {
            throw new xormon_exception_1.XormonException("An error occurred while deleting hostcfg", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
        if (deleted) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.HOSTCFG_DELETE, hostcfg.label + ' [' + hostcfg.hw_type + ']');
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'deleted ' + hostcfg.label,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("An error occurred while deleting hostcfg", common_1.HttpStatus.BAD_REQUEST);
        }
    }
};
exports.HostcfgPublicController = HostcfgPublicController;
__decorate([
    openapi.ApiOperation({ description: "returns hostcfg" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Get)('/'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/hostCfgResponse.dto").HostCfgsResponseDTO }),
    __param(0, (0, common_1.Query)('class')),
    __param(1, (0, common_1.Query)('hw_type')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], HostcfgPublicController.prototype, "getHostCfg", null);
__decorate([
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Get)('/definitions'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/hostCfgResponse.dto").HostCfgsDefinitionResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], HostcfgPublicController.prototype, "getHostCfgDefinitions", null);
__decorate([
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Get)('/definitions/:hw_type'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/hostCfgResponse.dto").HostCfgsDefinitionResponseDTO }),
    __param(0, (0, common_1.Param)("hw_type")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Promise)
], HostcfgPublicController.prototype, "getHostCfgDefinitionsHwType", null);
__decorate([
    openapi.ApiOperation({ description: "save hostcfg" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Post)('/'),
    openapi.ApiResponse({ status: 201, type: require("../../../common/hostCfgResponse.dto").HostCfgResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __param(2, (0, common_1.Query)('source')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, hostCfgResponse_dto_1.HostCfgDTO, String]),
    __metadata("design:returntype", Promise)
], HostcfgPublicController.prototype, "saveHostCfg", null);
__decorate([
    openapi.ApiOperation({ description: "update hostcfg" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Put)('/:hostcfg_id'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hostcfg_id')),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, hostCfgResponse_dto_1.HostCfgDTO]),
    __metadata("design:returntype", Promise)
], HostcfgPublicController.prototype, "updateHostCfg", null);
__decorate([
    openapi.ApiOperation({ description: "delete hostcfg" }),
    (0, common_1.UseGuards)(adminRoles_guard_1.AdminRolesGuard),
    (0, common_1.Delete)('/:hostcfg_id'),
    openapi.ApiResponse({ status: 200, type: require("../../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('hostcfg_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], HostcfgPublicController.prototype, "deleteHostCfg", null);
exports.HostcfgPublicController = HostcfgPublicController = __decorate([
    (0, swagger_1.ApiTags)('PublicHostcfg'),
    (0, common_1.Controller)('api/public/v1/hostcfg'),
    __metadata("design:paramtypes", [hostCfg_service_1.HostCfgService,
        configuration_service_1.ConfigurationService])
], HostcfgPublicController);
//# sourceMappingURL=hostcfg.public.controller.js.map