"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.describeMetaData = exports.getHwTypeByVendorDevice = exports.getClassMetadata = exports.getVendorDevice = exports.getVendorKeys = exports.createVendorMapFromObject = void 0;
require("reflect-metadata");
const data = require("./publicApiHostcfgData");
const vendors_constants_1 = require("../../../common/constants/vendors.constants");
const hwTypeKey_constants_1 = require("../../../common/constants/hwTypeKey.constants");
const hostcfg_metadata_1 = require("../../../resolve/hostcfg.metadata");
const deviceProps_1 = require("../../../common/hostcfgs/deviceProps");
function createVendorMapFromObject() {
    const map = {};
    const classes = [vendors_constants_1.vendorsLan, vendors_constants_1.vendorsSan, vendors_constants_1.vendorsStorage];
    for (const _class of classes) {
        for (const [vendorKey, vendor] of Object.entries(_class)) {
            for (const [deviceKey, deviceType] of (Object.entries(vendor.devices))) {
                if (!(deviceType.hwType in map)) {
                    map[deviceType.hwType] = {
                        vendors: {
                            [vendor.label]: { label: vendorKey }
                        },
                        labels: {
                            [vendor.label + " " + deviceType.label]: vendorKey + "__" + deviceKey
                        }
                    };
                }
                else {
                    map[deviceType.hwType].vendors[vendor.label] = { label: vendorKey };
                    map[deviceType.hwType].labels[deviceType.label] = vendorKey + "__" + deviceKey;
                }
            }
        }
    }
    return map;
}
exports.createVendorMapFromObject = createVendorMapFromObject;
function getVendorKeys() {
    const classes = [vendors_constants_1.vendorsLan, vendors_constants_1.vendorsSan, vendors_constants_1.vendorsStorage];
    const vendors = [];
    for (const _class of classes) {
        for (const [vendorKey, vendor] of Object.entries(_class)) {
            vendors.push(vendorKey);
        }
    }
    return vendors;
}
exports.getVendorKeys = getVendorKeys;
function getVendorDevice(_vendor) {
    const classes = [vendors_constants_1.vendorsLan, vendors_constants_1.vendorsSan, vendors_constants_1.vendorsStorage];
    const vendorDevices = [];
    let found = false;
    for (const _class of classes) {
        for (const [vendorKey, vendor] of Object.entries(_class)) {
            if (vendorKey === _vendor) {
                for (const [deviceKey, deviceType] of (Object.entries(vendor.devices))) {
                    vendorDevices.push(deviceKey);
                }
                found = true;
                break;
            }
        }
        if (found)
            break;
    }
    return vendorDevices;
}
exports.getVendorDevice = getVendorDevice;
const hwTypeDataMap = {
    [hwTypeKey_constants_1.HwTypeKey.VMWARE]: data.VmwareProps,
    [hwTypeKey_constants_1.HwTypeKey.PROXMOX]: data.ProxmoxProps,
    [hwTypeKey_constants_1.HwTypeKey.NUTANIX]: data.NutanixProps,
    [hwTypeKey_constants_1.HwTypeKey.POWER]: data.PowerProps,
    [hwTypeKey_constants_1.HwTypeKey.OVIRT]: data.OvirtProps,
    [hwTypeKey_constants_1.HwTypeKey.XENSERVER]: data.XenserverProps,
    [hwTypeKey_constants_1.HwTypeKey.ORACLEVM]: deviceProps_1.AuthBaseProps,
    [hwTypeKey_constants_1.HwTypeKey.CLOUDSTACK]: deviceProps_1.AuthBaseProps,
    [hwTypeKey_constants_1.HwTypeKey.FUSIONCOMPUTE]: data.FusioncomputeProps,
    [hwTypeKey_constants_1.HwTypeKey.POSTGRES]: deviceProps_1.AbstractDBProps,
    [hwTypeKey_constants_1.HwTypeKey.SQLSERVER]: deviceProps_1.AbstractDBProps,
    [hwTypeKey_constants_1.HwTypeKey.ORACLEDB]: data.OracledbProps,
    [hwTypeKey_constants_1.HwTypeKey.DB2]: deviceProps_1.AbstractDBProps,
    [hwTypeKey_constants_1.HwTypeKey._3PAR]: data.Par3Props,
    [hwTypeKey_constants_1.HwTypeKey.COMPELLENT]: data.CompellentDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.DATADOMAIN]: deviceProps_1.ExtensiveSshDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.DOTHILL]: data.DothillProps,
    [hwTypeKey_constants_1.HwTypeKey.ESERIES]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.IBMCOSS]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.ISILON]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.POWERFLEX]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.RUBRIK]: data.RubrikProps,
    [hwTypeKey_constants_1.HwTypeKey.STOREONCE]: deviceProps_1.ProtoRestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.SWIZ]: data.SwizProps,
    [hwTypeKey_constants_1.HwTypeKey.QNAP]: data.QnapProps,
    [hwTypeKey_constants_1.HwTypeKey.ORACLEZFS]: deviceProps_1.TwoNodeRestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.COHESITY]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.VMAX]: data.VmaxProps,
    [hwTypeKey_constants_1.HwTypeKey.GPFS]: data.GpfsProps,
    [hwTypeKey_constants_1.HwTypeKey.NETBACKUP]: deviceProps_1.TokenRestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.MACROSAN]: data.MacrosanProps,
    [hwTypeKey_constants_1.HwTypeKey.CEPH]: data.CephProps,
    [hwTypeKey_constants_1.HwTypeKey.DATACORE]: data.DatacoreProps,
    [hwTypeKey_constants_1.HwTypeKey.DS5K]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.DS8K]: data.Ds8kProps,
    [hwTypeKey_constants_1.HwTypeKey.ECS]: data.EcsProps,
    [hwTypeKey_constants_1.HwTypeKey.ETERNUS]: data.EternusProps,
    [hwTypeKey_constants_1.HwTypeKey.FALCONSTOR]: data.FalconstorProps,
    [hwTypeKey_constants_1.HwTypeKey.FREENAS]: deviceProps_1.ProtoRestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.HCP]: data.HcpProps,
    [hwTypeKey_constants_1.HwTypeKey.HNAS]: deviceProps_1.SnmpStorageDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.HUS]: deviceProps_1.BaseDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.INFINIBOX]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.INFORTREND]: deviceProps_1.SnmpStorageDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.NETAPP]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.NIMBLE]: data.NimbleProps,
    [hwTypeKey_constants_1.HwTypeKey.PURE]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.PUREBLADE]: deviceProps_1.TokenRestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.QUMULO]: deviceProps_1.TokenRestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.RAIDIX]: data.RaidixProps,
    [hwTypeKey_constants_1.HwTypeKey.SOLIDFIRE]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.STOREVIRTUAL]: deviceProps_1.ExtensiveSshDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.SYNOLOGY]: deviceProps_1.SnmpStorageDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.VNX]: data.VnxProps,
    [hwTypeKey_constants_1.HwTypeKey.VNXE]: deviceProps_1.UniCliDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.VNXF]: deviceProps_1.ExtensiveSshDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.VPLEX]: data.VplexProps,
    [hwTypeKey_constants_1.HwTypeKey.VSPG]: data.VspgProps,
    [hwTypeKey_constants_1.HwTypeKey.UNITY]: deviceProps_1.UniCliDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.XIV]: data.XivProps,
    [hwTypeKey_constants_1.HwTypeKey.XTREMIO]: data.XtremioProps,
    [hwTypeKey_constants_1.HwTypeKey.STORAGEGRID]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.POWERSTORE]: deviceProps_1.RestDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.DDNINTELLIFLASH]: data.DdnintelliflashProps,
    [hwTypeKey_constants_1.HwTypeKey.OCEANSTOR]: data.OceanstorProps,
    [hwTypeKey_constants_1.HwTypeKey.OCEANSTORPACIFIC]: data.OceanstorProps,
    [hwTypeKey_constants_1.HwTypeKey.OCEANSTORPACIFICNAS]: data.OceanstorProps,
    [hwTypeKey_constants_1.HwTypeKey.OCEANSTORNAS]: data.OceanstorProps,
    [hwTypeKey_constants_1.HwTypeKey.SANBRCD]: deviceProps_1.SnmpNetworkDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.SANCISCO]: deviceProps_1.SnmpDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.LANCISCO]: deviceProps_1.SnmpDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.LANOTHERS]: deviceProps_1.SnmpDeviceProps,
    [hwTypeKey_constants_1.HwTypeKey.KUBERNETES]: data.KubernetesProps,
    [hwTypeKey_constants_1.HwTypeKey.OPENSHIFT]: data.OpenshiftProps,
    [hwTypeKey_constants_1.HwTypeKey.AZURE]: data.AzureProps,
    [hwTypeKey_constants_1.HwTypeKey.AWS]: data.AwsProps,
    [hwTypeKey_constants_1.HwTypeKey.GCLOUD]: data.GcloudProps
};
function isPrimitive(type) {
    return [String, Number, Boolean, Symbol, BigInt].includes(type);
}
function isEnum(type, property) {
    return Reflect.hasMetadata(hostcfg_metadata_1.enumTypeMetadataKey, type, property);
}
function hasDescription(type, property) {
    return Reflect.hasMetadata(hostcfg_metadata_1.descriptionMetadataKey, type, property);
}
function getClassMetadata(target) {
    const metadata = {};
    const instance = new target();
    Object.keys(instance).forEach(property => {
        const type = (0, hostcfg_metadata_1.getType)(target.prototype, property);
        const isPropRequired = (0, hostcfg_metadata_1.isRequired)(target.prototype, property);
        const defaultValue = (0, hostcfg_metadata_1.getDefaultValue)(target.prototype, property);
        const description = hasDescription(target.prototype, property) ? (0, hostcfg_metadata_1.getDescription)(target.prototype, property) : null;
        let typeInfo = { type: type.name.toLowerCase(), required: isPropRequired };
        if (defaultValue) {
            typeInfo.defaultValue = defaultValue;
        }
        if (description) {
            typeInfo.description = description;
        }
        if (type === Array) {
            const itemType = (0, hostcfg_metadata_1.getArrayItemType)(target.prototype, property);
            if (itemType) {
                typeInfo.type = 'array';
                typeInfo.arrayItemType = itemType.name.toLowerCase();
                if (isPrimitive(itemType)) {
                    typeInfo.arrayItemType = itemType.name.toLowerCase();
                }
                else if (isEnum(itemType, property)) {
                    typeInfo.arrayItemType = 'enum';
                    typeInfo.arrayItemValues = (0, hostcfg_metadata_1.getEnumValues)(target.prototype, property);
                }
                else if (typeof itemType === 'function') {
                    typeInfo.arrayItemType = 'class';
                    typeInfo.arrayItemProperties = getClassMetadata(itemType);
                }
            }
        }
        else if (isEnum(target.prototype, property)) {
            typeInfo.type = 'enum';
            typeInfo.possibleValues = (0, hostcfg_metadata_1.getEnumValues)(target.prototype, property);
        }
        else if (isPrimitive(type)) {
            typeInfo.type = type.name.toLowerCase();
        }
        else if (typeof type === 'function') {
            typeInfo.type = 'class';
            typeInfo.properties = getClassMetadata(type);
        }
        metadata[property] = typeInfo;
    });
    return metadata;
}
exports.getClassMetadata = getClassMetadata;
function getHwTypeByVendorDevice(device) {
    const vendorMap = createVendorMapFromObject();
    for (const hwType in vendorMap) {
        if (vendorMap[hwType].labels) {
            for (const label in vendorMap[hwType].labels) {
                if (vendorMap[hwType].labels[label].includes(device)) {
                    return [describeMetaData(hwType), hwType];
                }
            }
        }
    }
}
exports.getHwTypeByVendorDevice = getHwTypeByVendorDevice;
function describeMetaData(hwType) {
    const vendorMap = createVendorMapFromObject();
    if (hwType) {
        if (hwTypeDataMap[hwType]) {
            let cfgDescription = getClassMetadata(hwTypeDataMap[hwType]);
            delete cfgDescription.vendor;
            delete cfgDescription.updated;
            delete cfgDescription.created;
            delete cfgDescription.disabled;
            delete cfgDescription.radio;
            delete cfgDescription.sourceAddress;
            delete cfgDescription.licType;
            delete cfgDescription.type;
            if (vendorMap[hwType]) {
                cfgDescription.device = {
                    ...cfgDescription.device,
                    possibleValues: Object.values(vendorMap[hwType].labels)
                };
            }
            else {
                delete cfgDescription.device;
            }
            return cfgDescription;
        }
        else {
            return null;
        }
    }
    else {
        let map = {};
        for (const [key, data] of Object.entries(hwTypeDataMap)) {
            let newData = getClassMetadata(data);
            map[key] = {
                ...newData,
                device: {
                    ...newData.device,
                    possibleValues: vendorMap[key] ? Object.values(vendorMap[key].labels) : null
                }
            };
            delete map[key].vendor;
            delete map[key].updated;
            delete map[key].created;
            delete map[key].disabled;
            delete map[key].radio;
            delete map[key].sourceAddress;
            delete map[key].licType;
            delete map[key].type;
            if (map[key].device && map[key].device.possibleValues === null) {
                delete map[key].device;
            }
        }
        return map;
    }
}
exports.describeMetaData = describeMetaData;
//# sourceMappingURL=publicApiMap.js.map