"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GraphDataFactory = void 0;
const dataSource_1 = require("../../config/dataSource");
const objectItem_entity_1 = require("../../entity/objectItem.entity");
const typeorm_1 = require("typeorm");
const Metric_1 = require("../metrics/Metric");
const MetricsWrapper_1 = require("../metrics/MetricsWrapper");
const Vega_1 = require("./Vega");
class GraphDataFactory {
    static calculateGraphDataInterval(start, end, metricClass) {
        let interval = Metric_1.Metric.getInterval(start, end);
        if (interval === Metric_1.Metric.INTERVAL_1_M && metricClass.isStorage()) {
            interval = Metric_1.Metric.INTERVAL_5_M;
        }
        return interval;
    }
    static async getMetricClass(metricName, itemIds) {
        const metricsClass = new Metric_1.Metric(metricName, itemIds);
        await metricsClass.init();
        return metricsClass;
    }
    static async fetchObjectItems(itemIds) {
        return (0, dataSource_1.getDataSource)().getRepository(objectItem_entity_1.ObjectItem).findBy({
            item_id: (0, typeorm_1.In)(itemIds)
        });
    }
    static async fillGraphData(graphData) {
        if (!graphData.haveObjectItemsLoaded()) {
            const objectItems = await GraphDataFactory.fetchObjectItems(graphData.itemIds);
            graphData.setObjectItems(objectItems);
        }
        if (!graphData.haveNonEmptyObjectItems()) {
            return graphData;
        }
        const metricsWrapper = new MetricsWrapper_1.MetricsWrapper([graphData.metricName], graphData.itemIds);
        await metricsWrapper.init();
        let metricsData = await metricsWrapper.getDataFromTable(graphData.start, graphData.end, true, true);
        const metricClass = await this.getMetricClass(graphData.metricName, graphData.itemIds);
        const rawData = metricsData;
        const interval = GraphDataFactory.calculateGraphDataInterval(graphData.start, graphData.end, metricClass);
        await graphData.fillAndProcessData(rawData, interval);
        return graphData;
    }
    static makeVegaData(graphData, singleItem = undefined) {
        return Vega_1.Vega.makeImgVegaData(graphData.getTimeDataPoints(singleItem), graphData.getMetricSpecification(), graphData.getSortedLabels(singleItem), graphData.stacked, graphData.getMax(), graphData.width, graphData.height, graphData.alertLimit, graphData.includeLegend, graphData.graphTitle, graphData.alertingOptions);
    }
    static async getGraphImageData(graphData, singleItem = undefined) {
        const vegaData = GraphDataFactory.makeVegaData(graphData, singleItem);
        const imageData = await Vega_1.Vega.createImageBuffer(vegaData, graphData.format);
        if (graphData.format === 'svg') {
            return imageData;
        }
        const PNGdata = Buffer.from(imageData).toString('base64');
        return PNGdata;
    }
}
exports.GraphDataFactory = GraphDataFactory;
//# sourceMappingURL=GraphData.factory.js.map