"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServerGroupController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const serverGroup_service_1 = require("./serverGroup.service");
const serverGroup_dto_1 = require("../../common/serverGroup.dto");
const User_1 = require("../users/User");
const xormon_exception_1 = require("../../exception/xormon.exception");
const PM2_1 = require("../configuration/PM2");
let ServerGroupController = class ServerGroupController {
    constructor(serverGroupService) {
        this.serverGroupService = serverGroupService;
    }
    async getServerGroups(request, hw_types, _class) {
        const user = await User_1.User.getUserByRequest(request);
        let hwTypes = hw_types.split(",");
        const serverGroups = await this.serverGroupService.getServerGroups(user.user_id, hwTypes, _class);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: serverGroups.map((sg) => {
                return {
                    server_group_id: sg.server_group_id,
                    description: sg.description,
                    label: sg.label,
                    items: sg.server_group_items.map((sgi) => sgi._item),
                    hw_type: sg.hw_type,
                    class: sg.class
                };
            }),
        };
    }
    async getValidItems(hw_type, _class) {
        const items = await this.serverGroupService.getValidItems(hw_type, _class);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: items
        };
    }
    async createServerGroup(request, server_group) {
        const user = await User_1.User.getUserByRequest(request);
        server_group.user_id = user.user_id;
        if (server_group.items.length < 1) {
            throw new xormon_exception_1.XormonException("Trying to create group with 0 items", common_1.HttpStatus.BAD_REQUEST);
        }
        const sgroup = await this.serverGroupService.saveServerGroup(server_group);
        if (sgroup) {
            const sgWithDevices = await this.serverGroupService.getServerGroupWithRelation(sgroup.server_group_id);
            if (sgWithDevices.server_group_items.length > 0) {
                PM2_1.PM2.sendMessage('backend', {
                    action: "refresh-menu"
                });
                return {
                    statusCode: common_1.HttpStatus.OK,
                    data: {
                        server_group_id: sgWithDevices.server_group_id,
                        description: sgWithDevices.description,
                        label: sgWithDevices.label,
                        items: sgWithDevices.server_group_items.map((sgi) => sgi._item)
                    }
                };
            }
            else {
                throw new xormon_exception_1.XormonException("Created group with 0 items", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        else {
            throw new xormon_exception_1.XormonException("Failed at creating  group", common_1.HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }
    async updateServerGroup(request, server_group_id, server_group) {
        const user = await User_1.User.getUserByRequest(request);
        if (await this.serverGroupService.checkOwnership(server_group_id, user.user_id)) {
            const updated = await this.serverGroupService.updateServerGroup(server_group_id, server_group);
            if (updated) {
                PM2_1.PM2.sendMessage('backend', {
                    action: "refresh-menu"
                });
                return {
                    statusCode: common_1.HttpStatus.OK,
                    data: {
                        server_group_id: updated.server_group_id,
                        label: updated.label,
                        description: updated.description,
                        items: updated.server_group_items.map(i => i._item)
                    }
                };
            }
            else {
                throw new xormon_exception_1.XormonException("Failed while updating group", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        else {
            throw new common_1.ForbiddenException();
        }
    }
    async deleteServerGroup(request, server_group_id) {
        const user = await User_1.User.getUserByRequest(request);
        if (await this.serverGroupService.checkOwnership(server_group_id, user.user_id)) {
            const deleted = await this.serverGroupService.deleteServerGroup(server_group_id);
            if (deleted && deleted.affected) {
                PM2_1.PM2.sendMessage('backend', {
                    action: "refresh-menu"
                });
                return {
                    statusCode: common_1.HttpStatus.OK,
                    message: "Successfuly deleted linux group"
                };
            }
            else {
                throw new xormon_exception_1.XormonException("Failed at deleting group", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        else {
            throw new common_1.ForbiddenException();
        }
    }
};
exports.ServerGroupController = ServerGroupController;
__decorate([
    (0, common_1.Get)('v1/'),
    openapi.ApiResponse({ status: 200, type: require("../../common/serverGroup.dto").ServerGroupsResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Query)('hw_types')),
    __param(2, (0, common_1.Query)('class')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, String]),
    __metadata("design:returntype", Promise)
], ServerGroupController.prototype, "getServerGroups", null);
__decorate([
    (0, common_1.Get)('v1/items'),
    openapi.ApiResponse({ status: 200, type: require("../../common/serverGroup.dto").ItemsResponseDTO }),
    __param(0, (0, common_1.Query)('hw_type')),
    __param(1, (0, common_1.Query)('class')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String, String]),
    __metadata("design:returntype", Promise)
], ServerGroupController.prototype, "getValidItems", null);
__decorate([
    (0, common_1.Post)('v1/'),
    openapi.ApiResponse({ status: 201, type: require("../../common/serverGroup.dto").ServerGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, serverGroup_dto_1.ServerGroupDTO]),
    __metadata("design:returntype", Promise)
], ServerGroupController.prototype, "createServerGroup", null);
__decorate([
    (0, common_1.Put)('v1/:server_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/serverGroup.dto").ServerGroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)("server_group_id")),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String, serverGroup_dto_1.ServerGroupDTO]),
    __metadata("design:returntype", Promise)
], ServerGroupController.prototype, "updateServerGroup", null);
__decorate([
    (0, common_1.Delete)('v1/:server_group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)("server_group_id")),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], ServerGroupController.prototype, "deleteServerGroup", null);
exports.ServerGroupController = ServerGroupController = __decorate([
    (0, swagger_1.ApiTags)('ServerGroups'),
    (0, swagger_1.ApiBearerAuth)(),
    (0, common_1.Controller)('api/server_groups'),
    __metadata("design:paramtypes", [serverGroup_service_1.ServerGroupService])
], ServerGroupController);
//# sourceMappingURL=serverGroup.controller.js.map