"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var UsersController_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.UsersController = void 0;
const openapi = require("@nestjs/swagger");
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const groupsResponse_dto_1 = require("../../common/groupsResponse.dto");
const usersResponse_dto_1 = require("../../common/usersResponse.dto");
const auditEvent_entity_1 = require("../../entity/auditEvent.entity");
const xormon_exception_1 = require("../../exception/xormon.exception");
const Audit_1 = require("../configuration/Audit");
const ldap_service_1 = require("../ldap/ldap.service");
const ApiKey_1 = require("../public/ApiKey");
const User_1 = require("./User");
const users_service_1 = require("./users.service");
let UsersController = UsersController_1 = class UsersController {
    constructor(usersService, ldapService) {
        this.usersService = usersService;
        this.ldapService = ldapService;
        this.logger = new common_1.Logger(UsersController_1.name);
    }
    async registerUser(request, headers, user) {
        if (await this.usersService.findUser(user.username)) {
            throw new xormon_exception_1.XormonException("Username is already in use", common_1.HttpStatus.BAD_REQUEST);
        }
        const registeredUser = await User_1.User.register(user);
        if (registeredUser) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.USER_ADD, user.username);
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'User ' + user.username + ' registered',
                data: registeredUser
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error occurred while registering the user", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getProfile(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        if (user) {
            const [isManager, allowedFeatures] = await this.usersService.getUserAdminFeatures(user);
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    ...user,
                    isAdmin: user.groups.find((group) => group.group_id === 1) ? true : false,
                    allowedFeatures: allowedFeatures,
                    isManager: isManager
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException("User not found", common_1.HttpStatus.NOT_FOUND);
        }
    }
    async getUserApiKeys(request) {
        const user = await User_1.User.getUserByRequest(request, true);
        const apiKeys = await this.usersService.getApiKeys(user.user_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: apiKeys.map(apiKey => {
                return {
                    label: apiKey.label,
                    api_key: apiKey.api_key,
                    expiration: apiKey.expiration
                };
            })
        };
    }
    async generateUserApiKeys(request, apiKeyRequest) {
        const user = await User_1.User.getUserByRequest(request, true);
        const apiKey = await ApiKey_1.ApiKey.generateApiKey(user.user_id, apiKeyRequest.expiration, apiKeyRequest.label, false);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: {
                label: apiKey.label,
                api_key: apiKey.api_key,
                expiration: apiKey.expiration
            }
        };
    }
    async deleteUserApiKey(request, key) {
        const user = await User_1.User.getUserByRequest(request, true);
        const apiKey = await this.usersService.getApiKey(key);
        if (!apiKey) {
            throw new common_1.NotFoundException();
        }
        if (apiKey.user_id !== user.user_id) {
            throw new common_1.ForbiddenException();
        }
        await this.usersService.deleteApiKey(key);
        return {
            statusCode: common_1.HttpStatus.OK,
            message: "API Key successfully deleted"
        };
    }
    async updateProfile(request, update) {
        const user = await User_1.User.getUserByRequest(request);
        const updatedUser = user.ldap_id
            ? await User_1.User.update(user.user_id, {
                timezone: update.timezone ? update.timezone : undefined,
                configuration: update.configuration ? update.configuration : undefined
            }, true)
            : await User_1.User.update(user.user_id, {
                oldPassword: update.oldPassword ? update.oldPassword : undefined,
                password: update.password ? update.password : undefined,
                timezone: update.timezone ? update.timezone : undefined,
                email: update.email ?? undefined,
                configuration: update.configuration ? update.configuration : undefined
            }, true);
        if (updatedUser) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.USER_EDIT, updatedUser.username);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'User updated',
                data: updatedUser,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error occurred while updating the user", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateUser(request, headers, user_id, user) {
        const loggedUser = await User_1.User.getUserByRequest(request);
        const toUpdateUser = await this.usersService.getUser(user_id);
        if (!toUpdateUser) {
            throw new xormon_exception_1.XormonException("User not found", common_1.HttpStatus.NOT_FOUND);
        }
        if (toUpdateUser.ldap_id) {
            throw new xormon_exception_1.XormonException("LDAP user cannot be edited", common_1.HttpStatus.BAD_REQUEST);
        }
        if (user.username) {
            const checkUser = await this.usersService.findUser(user.username);
            if (checkUser && checkUser.user_id !== toUpdateUser.user_id) {
                throw new xormon_exception_1.XormonException("Username is already in use", common_1.HttpStatus.BAD_REQUEST);
            }
        }
        const updatedUser = await User_1.User.update(user_id, user);
        if (updatedUser) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.USER_EDIT, updatedUser.username, loggedUser.user_id);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'User updated',
                data: updatedUser,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error occurred while updating the user", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getUsers() {
        let users = await this.usersService.getUsers();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: users,
        };
    }
    async getUser(user_id) {
        let user = await this.usersService.getUser(user_id);
        if (user) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: user,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("User not found", common_1.HttpStatus.NOT_FOUND);
        }
    }
    async deleteUser(request, headers, user_id) {
        const user = await this.usersService.getUser(user_id);
        if (!user) {
            throw new xormon_exception_1.XormonException("User not found", common_1.HttpStatus.NOT_FOUND);
        }
        const deleted = await this.usersService.deleteUser(user_id);
        if (deleted) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.USER_DELETE, user.username);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'user deleted',
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error occurred while deleting the user", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async registerGroup(request, headers, group) {
        if (await this.usersService.findGroup(group.name)) {
            throw new xormon_exception_1.XormonException("Group name is already in use", common_1.HttpStatus.BAD_REQUEST);
        }
        let created = await this.usersService.saveGroup(group);
        if (created) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.GROUP_ADD, created.name);
            return {
                statusCode: common_1.HttpStatus.CREATED,
                message: 'user group created',
                data: created,
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Error occurred while creating the group", common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getLdapGroups() {
        const groups = await this.ldapService.getLdapGroups();
        return {
            statusCode: common_1.HttpStatus.OK,
            data: groups.map(ldap => {
                return {
                    ldap_group: ldap.ldap_group,
                    ldap_id: ldap.ldap_id,
                    ldap_label: ldap._ldap.label
                };
            })
        };
    }
    async updateLdapGroups(group_id, ldap_groups) {
        const saved = await this.ldapService.updateGroupLdapGroups(group_id, ldap_groups.ldap_groups);
        if (saved) {
            return {
                statusCode: common_1.HttpStatus.OK,
                message: "Ldap groups updated successfully"
            };
        }
    }
    async getGroups() {
        let groups = (await this.usersService.getGroups()).map((group) => {
            return {
                group_id: group.group_id,
                name: group.name,
                description: group.description,
                users: group._users.map((user) => {
                    return {
                        user_id: user.user_id,
                        username: user.username,
                        email: user.email,
                        active: user.active,
                    };
                }),
                ldapGroups: group._groupLdapGroups ? group._groupLdapGroups.map(ldapGroup => {
                    return {
                        ldap_id: ldapGroup.ldap_id,
                        ldap_group: ldapGroup.ldap_group
                    };
                }) : []
            };
        });
        return {
            statusCode: common_1.HttpStatus.OK,
            data: groups,
        };
    }
    async getGroup(group_id) {
        let group = await this.usersService.getGroup(group_id);
        if (group) {
            return {
                statusCode: common_1.HttpStatus.OK,
                data: {
                    group_id: group.group_id,
                    name: group.name,
                    description: group.description,
                    users: group._users.map((user) => {
                        return {
                            user_id: user.user_id,
                            username: user.username,
                            email: user.email,
                            active: user.active,
                        };
                    }),
                    ldapGroups: group._groupLdapGroups ? group._groupLdapGroups.map(ldapGroup => {
                        return {
                            ldap_id: ldapGroup.ldap_id,
                            ldap_group: ldapGroup.ldap_group
                        };
                    }) : []
                },
            };
        }
        else {
            throw new xormon_exception_1.XormonException("Group not found", common_1.HttpStatus.NOT_FOUND);
        }
    }
    async deleteGroup(request, headers, group_id) {
        if (group_id == 1) {
            throw new xormon_exception_1.XormonException('Admin group cannot be deleted', common_1.HttpStatus.BAD_REQUEST);
        }
        const group = await this.usersService.getGroup(group_id);
        if (!group) {
            throw new xormon_exception_1.XormonException('Group not found', common_1.HttpStatus.NOT_FOUND);
        }
        const deleted = await this.usersService.deleteGroup(group_id);
        if (deleted) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.GROUP_DELETE, group.name);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'group deleted',
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occurred while deleting the group', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async updateGroup(request, headers, group_id, group) {
        if (!this.usersService.getGroup(group_id)) {
            throw new xormon_exception_1.XormonException('Group not found', common_1.HttpStatus.NOT_FOUND);
        }
        if (group_id == 1) {
            if (!group.user_ids || group.user_ids.length < 1) {
                throw new xormon_exception_1.XormonException('Cannot remove all users from the Admin group', common_1.HttpStatus.BAD_REQUEST);
            }
        }
        const updated = await this.usersService.updateGroup(group_id, group);
        if (updated) {
            Audit_1.Audit.log(request, auditEvent_entity_1.auditEventId.GROUP_EDIT, group.name);
            return {
                statusCode: common_1.HttpStatus.OK,
                message: 'group updated',
                data: await this.usersService.getGroup(group_id),
            };
        }
        else {
            throw new xormon_exception_1.XormonException('Error occurred while updating the group', common_1.HttpStatus.BAD_REQUEST);
        }
    }
    async getAdminGroupConfig(group_id) {
        const tree = await this.usersService.getAdminGroupTree(group_id);
        return {
            statusCode: common_1.HttpStatus.OK,
            data: tree,
        };
    }
    async exportUsers() {
        const users = await this.usersService.exportUsers();
        return users.map(user => {
            return {
                username: user.username,
                email: user.email,
                active: user.active,
                password: user.password,
                timezone: user.timezone,
                configuration: user.configuration
            };
        });
    }
    async importUsers(users) {
        try {
            return {
                statusCode: common_1.HttpStatus.CREATED,
                data: await this.usersService.importUsers(users),
            };
        }
        catch (e) {
            console.error("[ImportUsers] Error: " + e);
            throw new xormon_exception_1.XormonException("An error occurred while importing users", common_1.HttpStatus.BAD_REQUEST);
        }
    }
};
exports.UsersController = UsersController;
__decorate([
    openapi.ApiOperation({ description: "register new user" }),
    (0, common_1.Post)('v1/users'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 201, type: require("../../common/usersResponse.dto").UserResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, usersResponse_dto_1.RegisterUserDTO]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "registerUser", null);
__decorate([
    openapi.ApiOperation({ description: "get logged user" }),
    (0, common_1.Get)('v1/user'),
    openapi.ApiResponse({ status: 200, type: require("../../common/usersResponse.dto").UserResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getProfile", null);
__decorate([
    openapi.ApiOperation({ description: "get logged user api keys" }),
    (0, common_1.Get)('v1/user/keys'),
    openapi.ApiResponse({ status: 200, type: require("../../common/usersResponse.dto").UserApiKeysResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getUserApiKeys", null);
__decorate([
    openapi.ApiOperation({ description: "generate new user api key" }),
    (0, common_1.Post)('v1/user/key'),
    openapi.ApiResponse({ status: 201, type: require("../../common/usersResponse.dto").UserApiKeyResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, usersResponse_dto_1.CreateUserApiKey]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "generateUserApiKeys", null);
__decorate([
    openapi.ApiOperation({ description: "delete api key" }),
    (0, common_1.Delete)('v1/user/key/:key'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Param)('key')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, String]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "deleteUserApiKey", null);
__decorate([
    openapi.ApiOperation({ description: "update logged user" }),
    (0, common_1.Put)('v1/user'),
    openapi.ApiResponse({ status: 200, type: require("../../common/usersResponse.dto").UserResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, usersResponse_dto_1.UpdateLoggedUserDTO]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "updateProfile", null);
__decorate([
    openapi.ApiOperation({ description: "update user" }),
    (0, common_1.Put)('v1/users/:user_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/usersResponse.dto").UserResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('user_id')),
    __param(3, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, Object]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "updateUser", null);
__decorate([
    openapi.ApiOperation({ description: "get all users" }),
    (0, common_1.Get)('v1/users'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/usersResponse.dto").UsersResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getUsers", null);
__decorate([
    openapi.ApiOperation({ description: "get user" }),
    (0, common_1.Get)('v1/users/:user_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/usersResponse.dto").UserResponseDTO }),
    __param(0, (0, common_1.Param)('user_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getUser", null);
__decorate([
    openapi.ApiOperation({ description: "delete user" }),
    (0, common_1.Delete)('v1/users/:user_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('user_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "deleteUser", null);
__decorate([
    openapi.ApiOperation({ description: "register new group" }),
    (0, common_1.Post)('v1/groups'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 201, type: require("../../common/groupsResponse.dto").GroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, groupsResponse_dto_1.GroupDTO]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "registerGroup", null);
__decorate([
    openapi.ApiOperation({ description: "get all ldap groups" }),
    (0, common_1.Get)('v1/ldap_groups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/groupsResponse.dto").LdapGroupsResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getLdapGroups", null);
__decorate([
    openapi.ApiOperation({ description: "update group ldap groups" }),
    (0, common_1.Put)('v1/ldap_groups/:group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Param)('group_id')),
    __param(1, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number, groupsResponse_dto_1.UpdateLdapGroupDTO]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "updateLdapGroups", null);
__decorate([
    openapi.ApiOperation({ description: "get all groups" }),
    (0, common_1.Get)('v1/groups'),
    openapi.ApiResponse({ status: 200, type: require("../../common/groupsResponse.dto").GroupsResponseDTO }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getGroups", null);
__decorate([
    openapi.ApiOperation({ description: "get group" }),
    (0, common_1.Get)('v1/groups/:group_id'),
    openapi.ApiResponse({ status: 200, type: require("../../common/groupsResponse.dto").GroupResponseDTO }),
    __param(0, (0, common_1.Param)('group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getGroup", null);
__decorate([
    openapi.ApiOperation({ description: "delete group" }),
    (0, common_1.Delete)('v1/groups/:group_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/basicResponse.dto").BasicResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "deleteGroup", null);
__decorate([
    openapi.ApiOperation({ description: "update group" }),
    (0, common_1.Put)('v1/groups/:group_id'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/groupsResponse.dto").GroupResponseDTO }),
    __param(0, (0, common_1.Req)()),
    __param(1, (0, common_1.Headers)()),
    __param(2, (0, common_1.Param)('group_id')),
    __param(3, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Object, Number, groupsResponse_dto_1.GroupDTO]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "updateGroup", null);
__decorate([
    openapi.ApiOperation({ description: "admin-role configuration tree" }),
    (0, common_1.Get)('v1/group/roles'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: require("../../common/adminGroups.dto").AdminGroupsResponseDTO }),
    __param(0, (0, common_1.Query)('group_id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Number]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "getAdminGroupConfig", null);
__decorate([
    (0, common_1.Get)('v1/export'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 200, type: [require("../../common/usersResponse.dto").UserBase] }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "exportUsers", null);
__decorate([
    (0, common_1.Post)('v1/import'),
    (0, common_1.SetMetadata)('roles', ['admin']),
    openapi.ApiResponse({ status: 201, type: require("../../common/basicResponse.dto").ArrayResponseDTO }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", Promise)
], UsersController.prototype, "importUsers", null);
exports.UsersController = UsersController = UsersController_1 = __decorate([
    (0, swagger_1.ApiTags)('Users'),
    (0, common_1.Controller)('api/users'),
    __metadata("design:paramtypes", [users_service_1.UsersService,
        ldap_service_1.LdapService])
], UsersController);
//# sourceMappingURL=users.controller.js.map