import { ConfigService } from '@nestjs/config';
import { ClassConfigFolder } from '@xormon/common/adminGroups.dto';
import { AdminFeatures, AdminRoles, GroupDTO } from '@xormon/common/groupsResponse.dto';
import { AdminGroup } from '@xormon/entity/adminGroup.entity';
import { Group as GroupEntity } from '@xormon/entity/group.entity';
import { User as UserEntity } from '@xormon/entity/user.entity';
import { UserApiKey } from '@xormon/entity/userApiKey.entity';
import { ACLService } from '@xormon/service/acl/acl.service';
import { Repository } from 'typeorm';
import { LdapService } from '../ldap/ldap.service';
import { UserBase } from '@xormon/common/usersResponse.dto';
export declare class UsersService {
    private configService;
    private readonly aclService;
    private readonly ldapService;
    private userApiKeyRepository;
    private usersRepository;
    private groupsRepository;
    private adminGroupsRepository;
    private logger;
    constructor(configService: ConfigService, aclService: ACLService, ldapService: LdapService, userApiKeyRepository: Repository<UserApiKey>, usersRepository: Repository<UserEntity>, groupsRepository: Repository<GroupEntity>, adminGroupsRepository: Repository<AdminGroup>);
    getUsers(): Promise<UserEntity[]>;
    getApiKeys(user_id: number): Promise<UserApiKey[]>;
    getApiKey(api_key: string): Promise<UserApiKey>;
    deleteApiKey(api_key: string): Promise<import("typeorm").DeleteResult>;
    getUser(user_id: number): Promise<UserEntity>;
    saveLdapUser(ldap_id: string, username: string): Promise<UserEntity>;
    findUser(username: string): Promise<UserEntity>;
    deleteUser(user_id: number): Promise<boolean>;
    getGroups(): Promise<GroupEntity[]>;
    findGroup(name: string): Promise<GroupEntity>;
    getGroup(group_id: number): Promise<GroupEntity>;
    saveGroup(group: GroupDTO): Promise<GroupEntity>;
    deleteGroup(group_id: number): Promise<boolean>;
    updateGroup(group_id: number, group: GroupDTO): Promise<boolean>;
    updateAdminRoles(group_id: number, manages: AdminRoles): Promise<void>;
    deleteAdminRoles(group_id: number): Promise<void>;
    getUserAdminFeatures(user: UserEntity): Promise<[boolean, AdminFeatures]>;
    getAdminGroupTree(group_id?: number): Promise<ClassConfigFolder[]>;
    exportUsers(): Promise<UserEntity[]>;
    importUsers(users: UserBase[]): Promise<string[]>;
}
