"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var UsersService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.UsersService = void 0;
const common_1 = require("@nestjs/common");
const config_1 = require("@nestjs/config");
const typeorm_1 = require("@nestjs/typeorm");
const adminGroups_dto_1 = require("../../common/adminGroups.dto");
const adminGroups_constants_1 = require("../../common/constants/adminGroups.constants");
const groupsResponse_dto_1 = require("../../common/groupsResponse.dto");
const dataSource_1 = require("../../config/dataSource");
const adminGroup_entity_1 = require("../../entity/adminGroup.entity");
const dashboard_entity_1 = require("../../entity/dashboard.entity");
const group_entity_1 = require("../../entity/group.entity");
const user_entity_1 = require("../../entity/user.entity");
const userApiKey_entity_1 = require("../../entity/userApiKey.entity");
const acl_service_1 = require("../acl/acl.service");
const Architecture_1 = require("../architecture/Architecture");
const typeorm_2 = require("typeorm");
const ldap_service_1 = require("../ldap/ldap.service");
let UsersService = UsersService_1 = class UsersService {
    constructor(configService, aclService, ldapService, userApiKeyRepository, usersRepository, groupsRepository, adminGroupsRepository) {
        this.configService = configService;
        this.aclService = aclService;
        this.ldapService = ldapService;
        this.userApiKeyRepository = userApiKeyRepository;
        this.usersRepository = usersRepository;
        this.groupsRepository = groupsRepository;
        this.adminGroupsRepository = adminGroupsRepository;
        this.logger = new common_1.Logger(UsersService_1.name);
    }
    async getUsers() {
        return await this.usersRepository.find({ relations: { groups: true }, where: { ldap_id: (0, typeorm_2.IsNull)() } });
    }
    async getApiKeys(user_id) {
        return await this.userApiKeyRepository.findBy({ user_id: user_id, isTemporary: false });
    }
    async getApiKey(api_key) {
        return await this.userApiKeyRepository.findOneBy({ api_key: api_key });
    }
    async deleteApiKey(api_key) {
        return await this.userApiKeyRepository.delete({ api_key: api_key });
    }
    async getUser(user_id) {
        return await this.usersRepository.findOne({ where: { user_id: user_id }, relations: { groups: true } });
    }
    async saveLdapUser(ldap_id, username) {
        const check = await this.usersRepository.findOneBy({ ldap_id: ldap_id, username: username });
        if (!check) {
            const user = await this.usersRepository.save({
                username: username,
                email: username,
                password: '',
                ldap_id: ldap_id,
                locale: 'en-US',
                configuration: {},
                active: true
            });
            await (0, dataSource_1.getDataSource)().getRepository(dashboard_entity_1.Dashboard).save({
                label: 'DefaultDashboard',
                parent: null,
                type: 'personal',
                default: true,
                user_id: user.user_id
            });
            return user;
        }
        else {
            return check;
        }
    }
    async findUser(username) {
        return await this.usersRepository.findOne({ where: { username: username } });
    }
    async deleteUser(user_id) {
        const deleted = await this.usersRepository.delete({ user_id: user_id });
        return deleted.affected === 1 ? true : false;
    }
    async getGroups() {
        return await this.groupsRepository.find({ relations: { _users: true, _groupLdapGroups: true } });
    }
    async findGroup(name) {
        return await this.groupsRepository.findOne({ where: [{ name: name }] });
    }
    async getGroup(group_id) {
        return await this.groupsRepository.findOne({
            where: {
                group_id: group_id
            }, relations: {
                _groupLdapGroups: true,
                _users: true
            }
        });
    }
    async saveGroup(group) {
        let groupToSave = new groupsResponse_dto_1.Group();
        groupToSave.name = group.name;
        groupToSave.description = group.description;
        const insert = await this.groupsRepository.save(groupToSave);
        if (insert.group_id) {
            if (group.manages) {
                await this.updateAdminRoles(insert.group_id, group.manages);
            }
            if (group.ldapGroups) {
                await this.ldapService.updateGroupLdapGroups(insert.group_id, group.ldapGroups);
                insert.ldapGroups = group.ldapGroups;
            }
            return insert;
        }
        else {
            return null;
        }
    }
    async deleteGroup(group_id) {
        await this.deleteAdminRoles(group_id);
        const deleted = await this.groupsRepository.delete({ group_id: group_id });
        return deleted.affected === 1 ? true : false;
    }
    async updateGroup(group_id, group) {
        let updated = false;
        let groupToSave = new groupsResponse_dto_1.Group();
        groupToSave.group_id = group_id;
        groupToSave.name = group.name;
        groupToSave.description = group.description;
        if (group.user_ids) {
            updated = (await this.groupsRepository.save({
                name: group.name,
                description: group.description,
                _users: group.user_ids.map(user => {
                    return {
                        user_id: user,
                        group_id: group_id
                    };
                }),
                group_id: +group_id
            })).group_id !== undefined;
        }
        else {
            updated = (await this.groupsRepository.update(group_id, groupToSave)).affected !== 0;
        }
        if (group.manages) {
            await this.updateAdminRoles(group_id, group.manages);
        }
        return updated;
    }
    async updateAdminRoles(group_id, manages) {
        await this.deleteAdminRoles(group_id);
        let aclRules = [];
        for (let item of manages.class) {
            await this.adminGroupsRepository.save({ group: { group_id: group_id }, type: adminGroups_constants_1.RULE_TYPE.CLASS, value: item });
            aclRules.push({
                type: adminGroups_constants_1.RULE_TYPE.CLASS,
                value0: item
            });
        }
        for (let item of manages.hwType) {
            await this.adminGroupsRepository.save({ group: { group_id: group_id }, type: adminGroups_constants_1.RULE_TYPE.HWTYPE, value: item });
            aclRules.push({
                type: adminGroups_constants_1.RULE_TYPE.HWTYPE,
                value0: item
            });
        }
        await this.aclService.saveRules(group_id, { statusCode: common_1.HttpStatus.OK, data: aclRules });
    }
    async deleteAdminRoles(group_id) {
        const rules = await this.adminGroupsRepository.find({ relations: { group: true }, where: { group: { group_id: group_id } } });
        ;
        rules.forEach(async (rule) => await this.adminGroupsRepository.delete({ rule_id: rule.rule_id }));
    }
    async getUserAdminFeatures(user) {
        let deviceConfig = false;
        let heatmapTiers = false;
        let isManager = false;
        if (user) {
            const groupIds = user.groups.map((group) => group.group_id);
            if (groupIds.includes(1)) {
                return [
                    false,
                    {
                        deviceConfig: true,
                        heatmapTiers: true
                    }
                ];
            }
            else {
                const rulesRepository = (0, dataSource_1.getDataSource)().getRepository(adminGroup_entity_1.AdminGroup);
                for (let groupId of groupIds) {
                    const classRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: groupId }, type: adminGroups_constants_1.RULE_TYPE.CLASS } });
                    const hwTypeRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: groupId }, type: adminGroups_constants_1.RULE_TYPE.HWTYPE } });
                    if ((classRules && classRules.length > 0) || (hwTypeRules && hwTypeRules.length > 0)) {
                        deviceConfig = true;
                        isManager = true;
                        for (let rule of classRules) {
                            if (rule.value && (rule.value == 'Storage' || rule.value == 'SAN')) {
                                heatmapTiers = true;
                                break;
                            }
                        }
                        if (deviceConfig && heatmapTiers) {
                            break;
                        }
                    }
                }
            }
        }
        return [
            isManager,
            {
                deviceConfig: deviceConfig,
                heatmapTiers: heatmapTiers
            }
        ];
    }
    async getAdminGroupTree(group_id) {
        let folders = [];
        let classRules = [];
        let hwTypeRules = [];
        if (group_id) {
            const rulesRepository = (0, dataSource_1.getDataSource)().getRepository(adminGroup_entity_1.AdminGroup);
            classRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: adminGroups_constants_1.RULE_TYPE.CLASS } });
            hwTypeRules = await rulesRepository.find({ relations: { group: true }, where: { group: { group_id: group_id }, type: adminGroups_constants_1.RULE_TYPE.HWTYPE } });
        }
        const classes = await Architecture_1.Architecture.getClasses();
        for (let iClass of classes) {
            let isAllowed = false;
            for (let rule of classRules) {
                if (rule.value && rule.value == iClass.class) {
                    isAllowed = true;
                    break;
                }
            }
            let classFolder = new adminGroups_dto_1.ClassConfigFolder(iClass.label, iClass.class, isAllowed);
            if (iClass.class !== 'storage' && iClass.class !== 'san' && iClass.class !== 'lan') {
                classFolder.children = [];
                const hwTypes = await Architecture_1.Architecture.getHwTypes(false, iClass.class);
                for (let hwType of hwTypes) {
                    let isAllowed = false;
                    for (let rule of hwTypeRules) {
                        if (rule.value && rule.value == hwType.hw_type) {
                            isAllowed = true;
                            break;
                        }
                    }
                    let hwTypeItem = new adminGroups_dto_1.HwTypeConfigItem(hwType.label, hwType.hw_type, isAllowed);
                    classFolder.children.push(hwTypeItem);
                }
            }
            folders.push(classFolder);
        }
        return folders;
    }
    async exportUsers() {
        return await this.usersRepository.createQueryBuilder("users").addSelect('users.password').getMany();
    }
    async importUsers(users) {
        let result = [];
        let existingUsernames = [];
        const existingUsers = await this.usersRepository.findBy({
            username: (0, typeorm_2.In)(users.map(user => user.username))
        });
        if (existingUsers && existingUsers.length >= 1) {
            existingUsernames = existingUsers.map(user => user.username);
            for (const existingUser of existingUsers) {
                result.push(`${existingUser.username} already exists, skipped`);
            }
        }
        result.push("Users imported");
        await this.usersRepository.save(users.filter(user => !existingUsernames.includes(user.username)));
        return result;
    }
};
exports.UsersService = UsersService;
exports.UsersService = UsersService = UsersService_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(3, (0, typeorm_1.InjectRepository)(userApiKey_entity_1.UserApiKey)),
    __param(4, (0, typeorm_1.InjectRepository)(user_entity_1.User)),
    __param(5, (0, typeorm_1.InjectRepository)(group_entity_1.Group)),
    __param(6, (0, typeorm_1.InjectRepository)(adminGroup_entity_1.AdminGroup)),
    __metadata("design:paramtypes", [config_1.ConfigService,
        acl_service_1.ACLService,
        ldap_service_1.LdapService,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository])
], UsersService);
//# sourceMappingURL=users.service.js.map