"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HostCfgSubscriber = void 0;
const hostcfg_entity_1 = require("../entity/hostcfg.entity");
const Crypto_1 = require("../service/configuration/Crypto");
const Item_1 = require("../service/configuration/Item");
const Microservice_1 = require("../service/microservices/Microservice");
const typeorm_1 = require("typeorm");
let HostCfgSubscriber = class HostCfgSubscriber {
    listenTo() {
        return hostcfg_entity_1.HostCfg;
    }
    async afterLoad(entity) {
        for (const key of Object.keys(entity.data).filter(item => Crypto_1.Crypto.passKeys.includes(item) ||
            (entity.data[item] && typeof entity.data[item] === 'object' && Object.keys(entity.data[item]).length >= 1))) {
            if (typeof entity.data[key] === 'object' && entity.data[key] !== null) {
                entity.data[key] = await this.findInObject(entity.data[key], false);
            }
            else {
                entity.data[key] = await Crypto_1.Crypto.decrypt(entity.data[key]);
            }
        }
    }
    async beforeInsert(event) {
        for (const key of Object.keys(event.entity.data).filter(item => Crypto_1.Crypto.passKeys.includes(item) ||
            (event.entity.data[item] && typeof event.entity.data[item] === 'object' && Object.keys(event.entity.data[item]).length >= 1))) {
            if (typeof event.entity.data[key] === 'object' && event.entity.data[key] !== null) {
                event.entity.data[key] = await this.findInObject(event.entity.data[key], true);
            }
            else if (Crypto_1.Crypto.passKeys.includes(key)) {
                event.entity.data[key] = await Crypto_1.Crypto.encrypt(event.entity.data[key]);
            }
        }
    }
    async findInObject(obj, encrypt = true, old) {
        if (Object.keys(obj).length >= 1) {
            for (const key in obj) {
                if (typeof obj[key] === 'object' && obj[key] !== null) {
                    obj[key] = await this.findInObject(obj[key], encrypt, old && old[key] ? old[key] : {});
                }
                else if (Crypto_1.Crypto.passKeys.includes(key)) {
                    if (encrypt) {
                        if (obj[key] === null && old && old[key] !== undefined) {
                            obj[key] = await Crypto_1.Crypto.encrypt(old[key]);
                        }
                        else {
                            obj[key] = await Crypto_1.Crypto.encrypt(obj[key]);
                        }
                    }
                    else {
                        obj[key] = await Crypto_1.Crypto.decrypt(obj[key]);
                    }
                }
            }
        }
        return obj;
    }
    async beforeUpdate(event) {
        for (const key of Object.keys(event.entity.data).filter(item => Crypto_1.Crypto.passKeys.includes(item) ||
            (event.entity.data[item] && typeof event.entity.data[item] === 'object' && Object.keys(event.entity.data[item]).length >= 1))) {
            if (typeof event.entity.data[key] === 'object' && event.entity.data[key] !== null) {
                event.entity.data[key] = await this.findInObject(event.entity.data[key], true, event.databaseEntity.data[key] ? event.databaseEntity.data[key] : {});
            }
            else if (Crypto_1.Crypto.passKeys.includes(key)) {
                if (event.entity.data[key] === null && event.databaseEntity.data[key] !== undefined) {
                    event.entity.data[key] = await Crypto_1.Crypto.encrypt(event.databaseEntity.data[key]);
                }
                else {
                    event.entity.data[key] = await Crypto_1.Crypto.encrypt(event.entity.data[key]);
                }
            }
        }
        if (event.entity.hw_type === 'ecs') {
            let microservice = new Microservice_1.Microservice();
            await microservice.init('ecs');
            if (!microservice.isActive()) {
                return;
            }
            let message = {
                task: 'getId',
                data: event.databaseEntity['data'],
            };
            message.data['hostalias'] = event.databaseEntity.label;
            message.data['hostcfg_id'] = event.entity.hostcfg_id;
            let received;
            try {
                received = await microservice.sendData(JSON.stringify(message));
            }
            catch (error) {
                console.error("[ECS] [getId] Error: " + error);
            }
            const decoded = JSON.parse(received.toString());
            let beforeArray = event.databaseEntity.data['vdc'];
            let afterArray = event.entity.data['vdc'];
            let ipMapping = {};
            let mapping = {};
            for (const vdcAlias in decoded) {
                const vdcMs = decoded[vdcAlias];
                const beforeVdc = beforeArray.find(vdc => vdc.alias === vdcAlias);
                if (beforeVdc) {
                    for (const nodeIndex of [1, 2]) {
                        if (beforeVdc[`node${nodeIndex}`] && vdcMs[`node${nodeIndex}`]) {
                            const vdcId = vdcMs[`node${nodeIndex}`];
                            if (!mapping[vdcId]) {
                                mapping[vdcId] = [];
                            }
                            ipMapping[beforeVdc[`node${nodeIndex}`]] = vdcId;
                            mapping[vdcId].push(beforeVdc[`node${nodeIndex}`]);
                        }
                    }
                }
            }
            let afterVdc = afterArray.map(vdc => [vdc.node1, vdc.node2].sort());
            let removedVdc = beforeArray.filter(vdc => afterVdc.find(nodes => nodes.includes(vdc.node1)) === undefined && afterVdc.find(nodes => nodes.includes(vdc.node2)) === undefined);
            let ipToRemove = [];
            for (const removedIndex in removedVdc) {
                const vdc = removedVdc[removedIndex];
                if (vdc.node1) {
                    ipToRemove.push(vdc.node1);
                }
                if (vdc.node2) {
                    ipToRemove.push(vdc.node2);
                }
            }
            let vdcToDelete = ipToRemove.map(ip => ipMapping[ip]);
            vdcToDelete = vdcToDelete.filter(function (item, pos) {
                return vdcToDelete.indexOf(item) == pos;
            });
            for (const vdc of vdcToDelete) {
                if (!vdc) {
                    continue;
                }
                const mappedIps = mapping[vdc];
                const ipCheck = mappedIps.filter(mappedIp => ipToRemove.includes(mappedIp));
                if (ipCheck.length === mappedIps.length) {
                    try {
                        Item_1.Item.removeDeviceInfo(vdc);
                    }
                    catch (error) {
                        console.error(`[ECS] [removeDeviceInfo] Error: ${error}`);
                    }
                }
            }
        }
    }
};
exports.HostCfgSubscriber = HostCfgSubscriber;
exports.HostCfgSubscriber = HostCfgSubscriber = __decorate([
    (0, typeorm_1.EventSubscriber)()
], HostCfgSubscriber);
//# sourceMappingURL=hostcfg.subscriber.js.map