"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var AlertTask_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AlertTask = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const alertingHwEmail_entity_1 = require("../entity/alertingHwEmail.entity");
const healthStatus_entity_1 = require("../entity/healthStatus.entity");
const healthStatusHistory_entity_1 = require("../entity/healthStatusHistory.entity");
const hostcfgStatus_entity_1 = require("../entity/hostcfgStatus.entity");
const mail_service_1 = require("../mail/mail.service");
const App_1 = require("../service/configuration/App");
const Settings_1 = require("../service/configuration/Settings");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const HealthStatus_1 = require("../service/menu/menu/HealthStatus");
const alert_handler_1 = require("./alert.handler");
const typeorm_2 = require("typeorm");
const healthStatus_task_1 = require("./healthStatus.task");
const alertingHistory_entity_1 = require("../entity/alertingHistory.entity");
let AlertTask = AlertTask_1 = class AlertTask {
    constructor(alertingHwEmailsRepository, hostcfgstatusRepository, healthStatusHistoryRepository, healthStatusRepository, alertingHistoryRepository, mailService, alertHandler) {
        this.alertingHwEmailsRepository = alertingHwEmailsRepository;
        this.hostcfgstatusRepository = hostcfgstatusRepository;
        this.healthStatusHistoryRepository = healthStatusHistoryRepository;
        this.healthStatusRepository = healthStatusRepository;
        this.alertingHistoryRepository = alertingHistoryRepository;
        this.mailService = mailService;
        this.alertHandler = alertHandler;
        this.logger = new common_1.Logger(AlertTask_1.name);
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('alerting');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async checkAlerts() {
        if (!App_1.App.isProduction() && !(await App_1.App.isMaster())) {
            return;
        }
        await this.mailService.handleTransportStatus();
        return this.alertHandler.checkAlerts(new Date());
    }
    async checkHwAlerts() {
        if (!(await App_1.App.isMaster())) {
            return;
        }
        await this.mailService.handleTransportStatus();
        let settings = await Settings_1.Settings.get({
            keys: ['ALERTING_XORMON_URL']
        });
        if (settings['ALERTING_XORMON_URL'] && settings['ALERTING_XORMON_URL'][settings['ALERTING_XORMON_URL'].length - 1] !== '/') {
            settings['ALERTING_XORMON_URL'] = settings['ALERTING_XORMON_URL'] + '/';
        }
        const queue = [];
        const LAST_CHECK = new Date(Date.now() - 30 * 60 * 1000);
        const mailGroups = await this.alertingHwEmailsRepository.find({
            relations: {
                _mail_group: true
            }
        });
        let mails = [];
        mailGroups.forEach(mailGroup => {
            mails = mails.concat(mailGroup._mail_group.mails);
        });
        const checkedItems = await this.hostcfgstatusRepository.find({
            relations: {
                _hostcfg: {
                    hw_type_entity: true,
                    relations: {
                        _item: {
                            parents: true
                        }
                    }
                }
            },
            where: {
                perfTriggered: false,
                _hostcfg: {
                    disabled: false,
                    relations: {
                        _item: [
                            {
                                parents: {
                                    parent: (0, typeorm_2.IsNull)()
                                },
                                hw_type: (0, typeorm_2.Not)((0, typeorm_2.In)(['timescaledb', 'linux'])),
                                subsystem: (0, typeorm_2.In)(['device', 'server', 'cluster', 'db', 'vcenter', 'host'])
                            },
                            {
                                hw_type: "sanbrcd",
                            }
                        ]
                    }
                }
            }
        });
        for (const hostcfgStatus of checkedItems) {
            if (!hostcfgStatus.data && hostcfgStatus.lastData && new Date().getTime() - hostcfgStatus.lastData.getTime() >= HealthStatus_1.HealthStatus.LAST_DATA_UPDATE_LIMIT) {
                const link = (0, healthStatus_task_1.createItemLink)(hostcfgStatus._hostcfg.hw_type_entity.class, hostcfgStatus._hostcfg.hw_type, hostcfgStatus._hostcfg.label, hostcfgStatus._hostcfg.relations[0].item_id, settings);
                queue.push(this.mailService.send(mails, `Xormon Performance Alert [error] ${hostcfgStatus._hostcfg.relations[0]._item.label}`, `
            <b>Xormon Alert</b><br>
            Device doesn't provide performance statistics for longer than 3 hour.<br>
            Check connection to the device under Settings -> Monitoring -> Device
            ${link ? "Check it out in the Xormon NG UI: <a href='" + link + "'>" + link + "</a>" : ""}
            `));
                this.winstonLogger.info(`Xormon Performance Alert, Alert level:${healthStatus_entity_1.Status.ERROR}, Item: ${hostcfgStatus._hostcfg.relations[0]._item.label}, Reason: Device doesn't provide performance statistics for longer than 1 hour`);
                hostcfgStatus.perfTriggered = true;
                delete hostcfgStatus._hostcfg;
                await this.hostcfgstatusRepository.save(hostcfgStatus);
            }
        }
        await Promise.all(queue);
    }
    async closeDifferingHealthStatusHistory() {
        const runningHsHistory = await this.healthStatusHistoryRepository.findBy({ endedAt: (0, typeorm_2.IsNull)() });
        const currentHs = await this.healthStatusRepository.find({ where: {
                item_id: (0, typeorm_2.In)(runningHsHistory.map((v) => v.item_id))
            } });
        let hsHistoryToEnd = [];
        const hsStatusMap = {};
        for (const hs of currentHs) {
            hsStatusMap[hs.item_id + hs.type] = hs;
        }
        for (const history of runningHsHistory) {
            const id = history.item_id + history.type;
            if (hsStatusMap[id] && hsStatusMap[id].status !== history.status) {
                history.endedAt = new Date();
                hsHistoryToEnd.push(history);
            }
        }
        await this.healthStatusHistoryRepository.save(hsHistoryToEnd);
    }
};
exports.AlertTask = AlertTask;
exports.AlertTask = AlertTask = AlertTask_1 = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(alertingHwEmail_entity_1.AlertingHwEmail)),
    __param(1, (0, typeorm_1.InjectRepository)(hostcfgStatus_entity_1.HostcfgStatus)),
    __param(2, (0, typeorm_1.InjectRepository)(healthStatusHistory_entity_1.HealthStatusHistory)),
    __param(3, (0, typeorm_1.InjectRepository)(healthStatus_entity_1.HealthStatus)),
    __param(4, (0, typeorm_1.InjectRepository)(alertingHistory_entity_1.AlertingHistory)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        typeorm_2.Repository,
        mail_service_1.MailService,
        alert_handler_1.AlertHandler])
], AlertTask);
//# sourceMappingURL=alert.task.js.map