"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExporterTask = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const exporter_constants_1 = require("../common/constants/exporter.constants");
const export_entity_1 = require("../entity/export.entity");
const mailGroup_entity_1 = require("../entity/mailGroup.entity");
const mail_service_1 = require("../mail/mail.service");
const App_1 = require("../service/configuration/App");
const CustomGroupWrapper_1 = require("../service/custom_group/CustomGroupWrapper");
const ItemGroupWrapper_1 = require("../service/custom_group/ItemGroupWrapper");
const Exporter_1 = require("../service/exporter/Exporter");
const ExporterFormatter_1 = require("../service/exporter/ExporterFormatter");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const path_1 = require("path");
const rrule_1 = require("rrule");
const typeorm_2 = require("typeorm");
let ExporterTask = class ExporterTask {
    constructor(exporterRepository, mailGroupRepository, mailService) {
        this.exporterRepository = exporterRepository;
        this.mailGroupRepository = mailGroupRepository;
        this.mailService = mailService;
        this.EXPORT_PATH = (0, path_1.resolve)('./files/exporter');
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('exporter');
    }
    onModuleInit() {
        const fs = require('fs');
        if (!fs.existsSync(this.EXPORT_PATH)) {
            fs.mkdirSync(this.EXPORT_PATH);
        }
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async exporter() {
        if (!App_1.App.isProduction() && !(await App_1.App.isMaster())) {
            return;
        }
        const d = new Date();
        d.setDate(d.getDate() - 1);
        d.setHours(22);
        const now = new Date();
        const exports = await this.exporterRepository.find({ where: { recurrence: (0, typeorm_2.Not)((0, typeorm_2.IsNull)()) }, relations: { _item_group: true } });
        for (const exp of exports) {
            try {
                this.winstonLogger.log("info", "[EID " + exp.export_id + "] " + exp.label + " - CHECK");
                const rule = new rrule_1.RRule({
                    freq: rrule_1.RRule[exp.recurrence.freq.toUpperCase()],
                    byweekday: exp.recurrence.weekday ? exp.recurrence.weekday.map(wday => rrule_1.RRule[wday]) : undefined,
                    bymonth: exp.recurrence.month ? exp.recurrence.month : undefined,
                    bysetpos: exp.recurrence.setpos ? [exp.recurrence.setpos] : undefined,
                    bymonthday: exp.recurrence.month_day ? [exp.recurrence.month_day] : undefined,
                    byyearday: exp.recurrence.year_day ? [exp.recurrence.year_day] : undefined,
                    byweekno: exp.recurrence.week_number ? [exp.recurrence.week_number] : undefined,
                    dtstart: d,
                    count: 10
                });
                const run = (rule.all()).find(date => date.toDateString() === now.toDateString());
                if (run) {
                    this.winstonLogger.log("info", "[EID " + exp.export_id + "] " + exp.label + " - START");
                    if (exp.type = exporter_constants_1.EXPORT_TYPE.timeseries) {
                        let items = [];
                        if (exp.item_group_id) {
                            items = await ItemGroupWrapper_1.ItemGroupWrapper.getItems(exp.item_group_id, 'export');
                        }
                        else if (exp.custom_group_id) {
                            items = await CustomGroupWrapper_1.CustomGroupWrapper.getItems(exp.item_group_id);
                        }
                        const data = {
                            ...exp.data,
                            uuids: items.length >= 1 ? items.map(item => item.item_id) : undefined
                        };
                        const interval = Exporter_1.Exporter.getTimeseriesInterval(data, exp.recurrence.freq);
                        if (interval.start && interval.end) {
                            data.start = interval.start;
                            data.end = interval.end;
                        }
                        const exportData = await Exporter_1.Exporter.timeseries(data);
                        const formatter = new ExporterFormatter_1.ExporterFormatter(exportData);
                        formatter.setStartEnd({ start: interval.start, end: interval.end });
                        const filePath = this.EXPORT_PATH + '/export-' + exp.label + '-' + now.getTime();
                        await formatter.exportTimeseriesToFile(exp.format, filePath);
                        if (exp.recurrence.mail) {
                            let mails = Array.isArray(exp.recurrence.mail) ? exp.recurrence.mail : [exp.recurrence.mail];
                            if (Array.isArray(exp.recurrence.mail)) {
                                mails = await this.getMails(exp.recurrence.mail);
                            }
                            this.mailService.send(mails, 'Export: ' + exp.label, "<b>The output can be found in the attachments</b>", [filePath + "." + exp.format]).catch(e => {
                                console.error("[Exporter] Email sending was not successful: " + e);
                            });
                        }
                    }
                }
                this.winstonLogger.log("info", "[EID " + exp.export_id + "] " + exp.label + " - END");
            }
            catch (e) {
                this.winstonLogger.log("error", "[EID " + exp.export_id + "] " + exp.label + " - ERROR: " + e);
            }
        }
    }
    async getMails(mailGroupIds) {
        const mailGroups = await this.mailGroupRepository.find({ where: { mail_group_id: (0, typeorm_2.In)(mailGroupIds) } });
        let mails = [];
        mailGroups.forEach(mg => {
            mails.push(...mg.mails);
        });
        return mails;
    }
};
exports.ExporterTask = ExporterTask;
exports.ExporterTask = ExporterTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(export_entity_1.Export)),
    __param(1, (0, typeorm_1.InjectRepository)(mailGroup_entity_1.MailGroup)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        mail_service_1.MailService])
], ExporterTask);
//# sourceMappingURL=exporter.task.js.map