"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OSAgentTask = void 0;
const common_1 = require("@nestjs/common");
const schedule_1 = require("@nestjs/schedule");
const dataSource_1 = require("../config/dataSource");
const microserviceVariables_entity_1 = require("../entity/microserviceVariables.entity");
const App_1 = require("../service/configuration/App");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const masterProcess_1 = require("./task-queue/masterProcess");
let OSAgentTask = class OSAgentTask {
    constructor() {
        this.fs = require('fs');
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('os-agent');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async runOSAgent() {
        if (await (0, masterProcess_1.getMasterProcessId)() !== (0, masterProcess_1.getCurrentProcessId)()) {
            return;
        }
        try {
            if (await this.fs.existsSync(App_1.App.ROOT_PATH + "/server-nest/os-agent")) {
                const msVariable = await (0, dataSource_1.getDataSource)().getRepository(microserviceVariables_entity_1.MicroserviceVariables).findOneBy({
                    hw_type: 'linux',
                    key: 'PORT'
                });
                const port = msVariable ? msVariable.value : '8162';
                const process = require('child_process');
                const agent = process.spawn('perl', ['lpar2rrd-agent.pl', `localhost:${port}`], {
                    cwd: App_1.App.ROOT_PATH + "/server-nest/os-agent",
                    timeout: 1000 * 60 * 5
                });
                agent.stdout.on('data', (data) => {
                    this.winstonLogger.log("info", data.toString());
                });
                agent.stderr.on('data', (data) => {
                    this.winstonLogger.error(data.toString());
                });
            }
        }
        catch (e) {
            this.winstonLogger.error("[OS-AGENT] Error: " + e);
        }
    }
};
exports.OSAgentTask = OSAgentTask;
__decorate([
    (0, schedule_1.Cron)(schedule_1.CronExpression.EVERY_MINUTE),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], OSAgentTask.prototype, "runOSAgent", null);
exports.OSAgentTask = OSAgentTask = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [])
], OSAgentTask);
//# sourceMappingURL=os-agent.task.js.map