"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReporterTask = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const report_entity_1 = require("../entity/report.entity");
const reportFile_entity_1 = require("../entity/reportFile.entity");
const mail_service_1 = require("../mail/mail.service");
const App_1 = require("../service/configuration/App");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
const Reporter_1 = require("../service/reporter/Reporter");
const rrule_1 = require("rrule");
const typeorm_2 = require("typeorm");
let ReporterTask = class ReporterTask {
    constructor(reportRepository, reportFileRepository, mailService) {
        this.reportRepository = reportRepository;
        this.reportFileRepository = reportFileRepository;
        this.mailService = mailService;
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('reporter');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async onModuleInit() {
        return Reporter_1.Reporter.ensureExportPath();
    }
    extractEmails(report) {
        if (!Array.isArray(report.mail_groups)) {
            return null;
        }
        const emails = [];
        report.mail_groups.forEach(mailGroup => {
            if (Array.isArray(mailGroup.mails)) {
                emails.push(...mailGroup.mails);
            }
        });
        if (emails.length === 0) {
            return null;
        }
        return [...new Set(emails)];
    }
    async updateReportLastRunAt(report, lastRunAt) {
        return this.reportRepository.update(report.report_id, {
            last_run_at: lastRunAt,
            updated_at: () => '"updated_at"',
        });
    }
    getReports() {
        return this.reportRepository.find({
            where: {
                recurrence: (0, typeorm_2.Not)((0, typeorm_2.IsNull)()),
                disabled: false,
            },
            relations: {
                report_items: true,
                mail_groups: true,
            },
            order: {
                report_items: {
                    item_index: "ASC"
                }
            },
        });
    }
    makeRule(report, date) {
        if (typeof report.recurrence !== "object") {
            return undefined;
        }
        if (typeof report.recurrence.freq !== "string") {
            this.winstonLogger.warn(`Wrong type of recurrence.freq in ${report.report_id}`);
            return undefined;
        }
        const rule = new rrule_1.RRule({
            freq: rrule_1.RRule[report.recurrence.freq.toUpperCase()],
            byweekday: report.recurrence.weekday ? report.recurrence.weekday.map(wday => rrule_1.RRule[wday]) : undefined,
            bymonth: report.recurrence.month ? report.recurrence.month : undefined,
            bysetpos: report.recurrence.setpos ? [report.recurrence.setpos] : undefined,
            bymonthday: report.recurrence.month_day ? [report.recurrence.month_day] : undefined,
            byyearday: report.recurrence.year_day ? [report.recurrence.year_day] : undefined,
            byweekno: report.recurrence.week_number ? [report.recurrence.week_number] : undefined,
            dtstart: date,
            count: 10
        });
        return rule;
    }
    async reporter() {
        if (!App_1.App.isProduction() && !(await App_1.App.isMaster())) {
            return;
        }
        const d = new Date();
        d.setDate(d.getDate() - 1);
        d.setHours(22);
        const now = new Date();
        const reports = await this.getReports();
        for (const report of reports) {
            try {
                this.winstonLogger.log("info", "[RID " + report.report_id + "] " + report.label + " - CHECK");
                const rule = this.makeRule(report, d);
                const run = rule ? (rule.all()).find(date => date.toDateString() === now.toDateString()) : false;
                if (run) {
                    this.winstonLogger.log("info", "[EID " + report.report_id + "] " + report.label + " - START");
                    const reportFile = await Reporter_1.Reporter.generateReport(report);
                    if (report.on_generate_create_report_file) {
                        await this.reportFileRepository.save(reportFile);
                    }
                    await this.updateReportLastRunAt(report, reportFile.created_at);
                    const emails = this.extractEmails(report);
                    if (emails) {
                        const path = Reporter_1.Reporter.getReportFilePath(reportFile);
                        const text = "<b>The output can be found in the attachments</b>";
                        const subject = `Report: ${report.label}`;
                        await this.mailService.send(emails, subject, text, [path]);
                        this.winstonLogger.log("info", `[EID ${report.report_id} ] ${report.label} sent to ${emails.length} emails - END`);
                    }
                    else {
                        this.winstonLogger.log("info", `[EID ${report.report_id} ] ${report.label} not sent to any email - END`);
                    }
                }
            }
            catch (e) {
                this.winstonLogger.log("error", "[EID " + report.report_id + "] " + report.label + " - ERROR: " + e);
            }
        }
    }
};
exports.ReporterTask = ReporterTask;
exports.ReporterTask = ReporterTask = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(report_entity_1.Report)),
    __param(1, (0, typeorm_1.InjectRepository)(reportFile_entity_1.ReportFile)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository,
        mail_service_1.MailService])
], ReporterTask);
//# sourceMappingURL=reporter.task.js.map