"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceNowHandler = void 0;
const common_1 = require("@nestjs/common");
const hostcfgData_types_1 = require("../common/hostcfgs/hostcfgData.types");
const request_builder_1 = require("../request/request.builder");
const WinstonLogger_1 = require("../service/log/WinstonLogger");
let ServiceNowHandler = class ServiceNowHandler {
    constructor() {
        this.winstonLogger = (0, WinstonLogger_1.LoggerFactory)('serviceNow');
    }
    onApplicationShutdown(signal) {
        this.winstonLogger.close();
    }
    async sendserviceNow(instance, body, credentials) {
        const url = `${hostcfgData_types_1.PROTO.HTTPS}://${instance}.service-now.com/api/global/em/jsonv2`;
        try {
            this.winstonLogger.debug(`Sending POST request to ${url} with body: ${JSON.stringify(body)}`);
            const response = await request_builder_1.RequestBuilder.simpleRequest({
                url,
                method: request_builder_1.RequestMethod.POST,
                protocol: hostcfgData_types_1.PROTO.HTTPS,
                body: JSON.stringify(body),
                headers: {
                    'Content-Type': 'application/json',
                    'Accept': 'application/json',
                    'authorization': 'Basic ' + Buffer.from(`${credentials.user}:${credentials.password}`).toString('base64'),
                },
                timeout: 5000
            });
            if (response.statusCode && response.statusCode >= 200 && response.statusCode < 300) {
                this.winstonLogger.info(`POST request to ${url} succeeded with status code: ${response.statusCode}`);
            }
            else {
                this.winstonLogger.warn(`POST request to ${url} returned status code: ${response.statusCode}`);
            }
            if (response.error) {
                throw new Error(`Failed request with error: ${response.error}`);
            }
        }
        catch (error) {
            throw error;
        }
    }
};
exports.ServiceNowHandler = ServiceNowHandler;
exports.ServiceNowHandler = ServiceNowHandler = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [])
], ServiceNowHandler);
//# sourceMappingURL=serviceNow.handler.js.map