import { OnApplicationBootstrap, OnModuleInit } from "@nestjs/common";
import { SchedulerRegistry } from "@nestjs/schedule";
import { TaskTypes } from "@xormon/common/constants/taskQueue.constants";
import { TaskQueue } from "@xormon/entity/taskQueue.entity";
import { Repository } from "typeorm";
import { HeatmapTask } from "./heatmap.task";
import { TopTask } from "./top.task";
import { CronJobParams } from "cron";
import { CleanupTask } from "./cleanup.task";
import { AlertTask } from "./alert.task";
import { ExporterTask } from "./exporter.task";
import { ReporterTask } from "./reporter.task";
import { MonitoringTask } from "./monitoring.task";
import { AggregationTask } from "./aggregation.task";
import { LogTask } from "./log.task";
import { CustomTask } from "./custom.task";
import { PredictionTask } from "./prediction.task";
import { StuckJobTask } from "./stuckJob.task";
import { HealthStatusTask } from "./healthStatus.task";
export declare class TaskDefinition {
    taskType: TaskTypes;
    priority: number;
    taskName: string;
    service: any;
    typeExists?: boolean;
    cron?: CronJobParams["cronTime"];
    payload?: any;
    number_of_items?: number;
    monitorBlacklist?: boolean;
    taskLabel?: string;
}
export type JobMapType = {
    [key in `${TaskTypes}`]: TaskDefinition;
};
export declare class TaskQueueService implements OnApplicationBootstrap, OnModuleInit {
    private taskQueueRepository;
    private schedulerRegistry;
    private hetmapTasks;
    private topTasks;
    private cleanupTasks;
    private alertTasks;
    private exporterTasks;
    private reporterTasks;
    private monitoringTasks;
    private aggregationTasks;
    private logTasks;
    private customTasks;
    private predictionTasks;
    private stuckJobTasks;
    private healthStatusTask;
    private workers;
    private workerLimit;
    private isMaster;
    private jobMap;
    constructor(taskQueueRepository: Repository<TaskQueue>, schedulerRegistry: SchedulerRegistry, hetmapTasks: HeatmapTask, topTasks: TopTask, cleanupTasks: CleanupTask, alertTasks: AlertTask, exporterTasks: ExporterTask, reporterTasks: ReporterTask, monitoringTasks: MonitoringTask, aggregationTasks: AggregationTask, logTasks: LogTask, customTasks: CustomTask, predictionTasks: PredictionTask, stuckJobTasks: StuckJobTask, healthStatusTask: HealthStatusTask);
    private isMasterProcess;
    onApplicationBootstrap(): Promise<void>;
    onModuleInit(): Promise<void>;
    insertTask(taskType: TaskTypes, priority: number, payload: any, number_of_items?: number, typeExists?: boolean): Promise<void>;
    insertTaskObject(task: TaskDefinition): Promise<void>;
    private checkTask;
    private getIdleWorker;
    notifyIdleWorker(): void;
    getJob(type: TaskTypes): TaskDefinition;
}
