use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;                
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use HP3PARCOLLECT;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3203;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;
 # waiting for a new client connection
while ($client = $server->accept()) {

    my $pid;
    unless ( defined( $pid = fork() ) ) {
        Xormon::error("Wait for fork");
        sleep 5;
    }

    if ($pid == 0) {
        Xormon::log("2\n");
        # get information about a newly connected client
        my $client_address = $client->peerhost();
        my $client_port = $client->peerport();
        Xormon::log("Connection from $client_address:$client_port");

        # received data template 
        #
        # { 
        #   "jwt"  : "<backendToken>",
        #   "task" : "<taskName>"
        #   "data" : {
        #     <configuration_from_hostcfg>
        #   }
        # }

        # read up to 2048 characters from the connected client
        my $data = "";
        $client->recv($data, 2048);

        my $decoded = from_json($data);

        Xormon::log("Received task: ".$decoded->{task});
        my $response = '';
        if ($decoded->{task} eq "fetch") {
          
          my $cfg = $decoded->{data};
          $client->send("Task successfuly started");
          $client->shutdown(SHUT_WR);

          my $xormon = Xormon->new("$client_address", $decoded->{jwt}, '3par', $cfg->{hostcfg_id}, $cfg->{hostalias});

          #print "FETCH\n";
          #print Dumper $cfg;

          $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

          # vygenerovani tokenu
          #
          # curl -X 'POST'   'http://localhost:3000/api/token/v1/generate'   -H 'accept: application/json'   -H 'Content-Type: application/json'   -d '{"username": "xormon", "password": "xormon"}'
          #
          #   {"statusCode":200,"message":"successfully logged in","data":{"state":"success","token":"eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VybmFtZSI6Inhvcm1vbiIsImVtYWlsIjoiYWRtaW5AeG9ybW9uLmNvbSIsImxvY2FsZSI6ImVuLVVTIiwiY29uZmlndXJhdGlvbiI6e30sImlhdCI6MTYyMTg0OTU2Nn0.SuRBdPKau_kE1u-lob0iYIssLbRHpoIobuoyu6U8ChA"}}

          # rucni spusteni microservice
          #
          # curl -X 'POST' 'http://localhost:3000/api/microservices/eseries/fetch?hostcfg_id=b82f64bc-8785-4a12-94cb-ebee06533f90' -H 'accept: application/json' -H 'Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1c2VybmFtZSI6Inhvcm1vbiIsImVtYWlsIjoiYWRtaW5AeG9ybW9uLmNvbSIsImxvY2FsZSI6ImVuLVVTIiwiY29uZmlndXJhdGlvbiI6e30sImlhdCI6MTYyMTg0OTU2Nn0.SuRBdPKau_kE1u-lob0iYIssLbRHpoIobuoyu6U8ChA' -d ''
          #
          #{"statusCode":200,"message":"Task successfuly started"}

          # target destinations check
          if (defined $decoded->{'destinations'}) {
            $xormon->setDestinations($decoded->{'destinations'});
          }

          my $rc = HP3PARCOLLECT::collect($xormon,$cfg);
          if ($rc eq "1") {
            $rc = 201;
          }

          my $status;
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }
          #HP3PARCOLLECT::process_collected_data($xormon,$cfg);

          $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
          Xormon::log("Task for ".$cfg->{hostalias}." is done. Status: $status");

        } elsif ($decoded->{task} eq "test") {
            my $cfg      = $decoded->{data};
            my $response;
            #print Dumper $cfg;
            if ( (defined $cfg->{'cliApiSshRadio'} && $cfg->{'cliApiSshRadio'} eq "cli" ) || ( defined $cfg->{'cliApiSshRadio'} && $cfg->{'cliApiSshRadio'} eq "ssh" ) ){
              $response = HP3PARCOLLECT::connection_test_cli($cfg);
            }else{
              $response = HP3PARCOLLECT::connection_test($client, $cfg);
            }

            $client->send($response);
            $client->shutdown(SHUT_WR);

        } else {
            $response = "undefined task";

            $client->send($response);
            $client->shutdown(SHUT_WR);

        }

        exit();

    } else {
        $client->autoflush(1);    
    }
}

$server->close();