"""
Controller is supposed to compose modules on high level of abstraction.

Lower priority:
TODO: enhanced backup saving system: if times_to_fill includes non-existent request, then save request to database
TODO: proper request
TODO: refactor around requests
TODO: size measurements

"""
import numpy as np
import pandas as pd
import json
import datetime
import time

import analyzer

import importer.importer as importer
import importer.dbwork as dbwork

import utility.postprocess as postprocess
import utility.requestwork as rwk

import measure.timing as timing
from utility.logger import log


def run_all_processings(data):
    """
    Development only - no check.
    """
    data = data["request"]

    # ideally make cycle/find how to get list of processings/responses per model
    prediction = analyzer.predict_cpu(data, 0, multiply_key="5m")

    return prediction


def load_data(request):
    """
    Proper way to prediction data.
    """
    (command, schema, table, metric,
     time_name, item_id, frequency, maximal_value) = rwk.read_request(request, show_read=True)

    prediction = {}

    if command == "predict_cpu":
        # same as action=calculate_now
        prediction = process_one_request(request)
    elif command == "predict_vm":
        # same as action=calculate_now
        prediction = process_one_request(request)
    elif command == "predict_capacity":
        # data from database
        try:
            prediction = load_prepared_data(request)
        except:
            log("Problem during prediction retrieval.")
            prediction = process_one_request(request)


    return prediction


def load_prepared_data(request):

    log("Loading prepared data")

    response = dbwork.load_prediction_response(request)

    return response


def add_requests_and_calculate_all(requests):
    """
    Combination:
    1) add all requests to prediction table as insert/update
    2) process all requests
    3) save calculated response data to prediction table

    TODO: refactor around requests
    TODO: cleaner solution

    """
    log("ADD REQUESTS")
    try:
        if type(requests["request"]) == list:
            res = more_requests_to_database(requests["request"])
        else:
            log("Wrong post data structure.")
            res = {"message": "check post body structure"}
            return res

        act()
    except:
        """
        Table gets recreated on error in the previous part of code.
        That kind of error indicates wrong table structure (requests get checked)
        or outer violation of table.
        For now is recreation the simplest solution on error.

        """
        log("Problem occured during the process of adding values to prediction table or computation.")
        log("Received data:")
        log(requests)

        # workaround for midnight computation
        if datetime.datetime.today().hour == 0:
            log("MIDNIGHT RUN - problem occured")
            # sleep time is arbitrary
            time.sleep(64)

        log("Dropping prediction table")
        dbwork.delete_prediction_table()

        if type(requests["request"]) == list:
            res = more_requests_to_database(requests["request"])
        else:
            log("Wrong post data structure.")
            res = {"message": "check post body structure"}
            return res

        act()

    return res


def load_data_for_capacity_table(requests):
    """
    1) execute one select on prediction table
    2) filter output for desired requests and return it

    """
    log("start")

    id_metric = []
    for request in requests["request"]:
        if rwk.check_request_keys(request):
            id_metric.append((request["id"], request["column"]))
        else: 
            log("PROBLEM")

    res = dbwork.load_all_prediction_times_to_fill(id_metric)

    return res


#--------------------------------------------------------------------

def act():
    """

    """
    log("TASK: compute times and responses for all saved requests")

    requests = dbwork.load_all_prediction_table_requests()

    for request in requests:
        log("processing ", request)
        request = {"request": request}

        (analyzer_command, schema, table, metric,
         time_name, item_id, frequency, maximal_value) = rwk.read_request(request, show_read=True)

        if analyzer_command == "predict_capacity":
            prediction = process_one_request(request)

            dbwork.update_prediction_table(request, prediction)

    pass


def process_one_request(request, more_models=True):
    """

    """

    t = timing.Timer("Processing command")

    if not rwk.check_request_keys(request):
        log("WRONG request structure!")
        return {}

    (analyzer_command, schema, table, metric,
     time_name, item_id, frequency, maximal_value) = rwk.read_request(request, show_read=True)

    t.start("database")
    log("COLLECTING METRIC FROM DATABASE")

    time_series = importer.collect_db_data(schema, table, metric, item_id, time_name)

    t.stop()

    if ( len(time_series) == 0):
        return {}

    t.start("preprocessing")
    log("PREPROCESSING")

    processed_df = importer.processing_of_import(time_series, frequency, label=table)

    t.stop()
    t.start("computations")
    log("DATA TO ANALYZER")

    prediction = analyzer.execute_command(analyzer_command, processed_df,
                                          maximal_value, frequency)

    t.stop()
    t.summary()

    return prediction


#--------------------------------------------------------------------

def request_to_database(request):

    log("Saving request to prediction table.")

    if rwk.check_request_keys(request):
        dbwork.update_prediction_table(request, {})
    else:
        log("Request is NOK:")
        log(request)
        return {"message": "Wrong request structure!"}

    return {}


def more_requests_to_database(requests):

    log("Saving requests to prediction table.")

    for request in requests:

        if rwk.check_request_keys(request):
            single_response = dbwork.update_prediction_table(request, {})
        else:
            log("Wrong request body")
            return {"Message": "Check post body"}

    return {"Message": "Done"}

#--------------------------------------------------------------------

def process_one_and_save(command):

    log(f"Command: {command}")

    response = process_one_request(command)

    dbwork.update_prediction_table(command, response)


def process_more_requests(requests):

    for request in requests["request"]:
        request = {"request": request}
        process_one_and_save(request)


#--------------------------------------------------------------------

def drop_prediction_table():

    dbwork.delete_prediction_table()

    return {}


def data_used_for_request(request):
    """
    This is used for testing/checking purposes. Returns data corresponding to request
    after gapfilling.

    TODO: cut out to make sure the same process is actually followed during act()

    """
    (analyzer_command, schema, table, metric, 
     time_name, item_id, frequency, maximal_value) = rwk.read_request(request)

    time_series_data = importer.collect_db_data(schema, table, metric, item_id, time_name)

    df = importer.processing_of_import(time_series_data, frequency, label=table)

    data = df["Value"].to_numpy(dtype=float)

    return {"data": data.tolist()}
