"""
Measures runtime, prints in-log message and summary (Timer.summary())

Low priority:
TODO: make children objects -> opens possibility of complete summary
"""
import time

class Timer:

    def __init__(self, message=""):

        self._start_time = None
        self.message = message
        self.messages = []
        self.all_times = []
        self.started = False


    def start(self, message=""):
        self.message2 = message
        self.started = True

        if self._start_time is not None:
            raise TimerError(f"Timer is running. Use .stop() to stop it")

        self._start_time = time.perf_counter()


    def stop(self, show=False):
        if self.started:
            self.started = False

            elapsed_time = time.perf_counter() - self._start_time

            self._start_time = None

            message_total = f"Timer: {elapsed_time:0.4f} s - {self.message} - {self.message2}"

            if show:
                print(message_total)

            self.messages.append(message_total)
            self.all_times.append(elapsed_time)
        else:
            print("TIMER: timing.py not started!!")


    def restart(self, message=""):

        if self.started is True:
            self.stop()

        self.start(message=message)


    def summary(self):
        print("")
        print("====================================================================")
        print(f"TIMER - {self.message} - SUMMARY: ")
        print("====================================================================")

        for message, total_time in zip(self.messages, self.all_times):
            time_percentage = total_time/sum(self.all_times)
            percentage = f"{time_percentage:.2%}"
            spacing = " "*(7-len(percentage))
            print(f"{spacing}{time_percentage:.2%} - {message}")

        print("====================================================================")
        time_percentage = 1
        percentage = f"{time_percentage:.2%}"
        spacing = " "*(7-len(percentage))
        print(f"{spacing}{time_percentage:.2%} - TOTAL: {sum(self.all_times):0.4f} s")
        print("====================================================================")


class TimerError(Exception):

    """A custom exception used to report errors in use of Timer class"""

