"""
Now: set of useful classes and modules.

TODO: separation
"""
import numpy as np
import pandas as pd
import math

import model.tools as tools

import classification.outlier as outlier


class Comparator:
    def complete_model_evaluation(self, train_data, test_data, models):
        print('----------------')
        print('MODEL evaluation')
        print('----------------')
        evaluation = {}

        for model in models:

            model.optimize(train_data)
            model.predict(len(test_data))
            dif = model.prediction - test_data

            evaluation_metric = (dif**2).mean()

            evaluation[model.name] = evaluation_metric

        return evaluation

class Modeller:
    # Intendet as composition of Model instances
    def __init__(self):

        pass
    
    def load_data(self, data):
        self.data = np.array(data)

    def initialize_model(self, model_name, params=None): 
        setattr(self, model_name, Model(model_name))

        if params is not None:
            getattr(self, model_name).initialize_params(params)


class Series:
    """
    TODO: prepare_z_score = compute score, save on position in series
    TODO: method to use mask based on z-score
    
    """
    # For development purposes: keep all objects
    # Else use destructive methods
    def __init__(self, data):
        """Now: use array to initialize series.
        Class has subclasses:
            Outliers: find extreme values
            Stats: basic statistical operations

        """
        self.series = data

        # use for proper trends: large gaps create slight differences in trends
        self.valid_map_exists = False

        self.undefs     = Undefs(self.series)
        self.outliers   = Outliers(self.series)
        self.stats      = Stats(self.series)

    def extend_valid_map(self, valid_map):
        if len(valid_map) == len(self.series):
            self.valid_map = valid_map
            self.valid_map_exists = True
        else:
            print("Cannot extend valid map - class Series")

    def make_clean(self):
        self.clean = Series(outlier.dull_outlier_cleaner(self.series, self.outliers.ind))

    def __str__(self):
        return f"{self.series}"


    def indexes_to_value(self, indexes, val):
        modification = Series([])

        if not index < len(self.series):
            print("Index out of range!")
        else:
            for value in self.series:
                modification.series.append(value)

            for index in indexes:
                modification.series[index]=val

        return modification


    def outlier_indexes(self):
        # [100,0,0,0,0,0,0,...,0] => [0]
        # [100,0,100,0,0,0,...,0] => [0, 2]
        return outlier.stripe_outlier_detection(self.series)

    def make_differenced_series(self):
        # [1,2,3,4]  => [1,1,1,1]
        # [5,7,9,11] => [5,2,2,2]
        # ! [1000,1001,1002,1003] => [1000,1,1,1]
        self.differenced = Series(tools.differentiate_series(self.series))

    def make_integrated_series(self):
        self.integrated = Series(tools.integrate_series(self.series))

    def make_trendline(self, steps_ahead = 0):
        if self.valid_map_exists:
            y = np.array(self.series)
            x = np.arange(0,len(y))
            ok_map = np.array(self.valid_map)

            x_ok = x[ok_map]
            y_ok = y[ok_map]

            try:
                polynomial = np.polyfit(x_ok, y_ok, 1)
            except:
                # should not happen TODO: add check
                print("SERIES: make_trendline: ! EXCEPTION !")
                n = np.array(self.series).size
                x = np.arange(0, n)
                polynomial = np.polyfit(x, np.array(self.series), 1)

            n = np.array(self.series).size + steps_ahead
            x = np.arange(0, n)
            trendline_ok = polynomial[0] * x
            trendline_ok = np.add(trendline_ok, polynomial[1])

            self.trendline_slope = polynomial[0]
            self.trendline_start_y = polynomial[1]
            self.trendline = trendline_ok
        else:
            n = np.array(self.series).size

            x = np.arange(0, n)
            polynomial = np.polyfit(x, np.array(self.series), 1)

            #trendline = polynomial[0] * x
            # keep here for future use, values are same as polynomial[1] from definition
            #shift_mean_interval = np.mean(self.series) - np.mean(trendline)

            n = n + steps_ahead

            x = np.arange(0, n)
            trendline = polynomial[0] * x

            #trendline = np.add(trendline, shift_mean_interval)
            trendline = np.add(trendline, polynomial[1])

            self.trendline_slope = polynomial[0]
            self.trendline_start_y = polynomial[1]
            self.trendline = trendline


class Outliers:
    def __init__(self, data):
        self.series = data
        self.ind = outlier.stripe_outlier_detection(self.series)

    def detect(self):
        return outlier.stripe_outlier_detection(self.series)

class Prediction:
    def __init__(self, data):
        self.data = data

class Stats:
    # Basic statistical operations
    def __init__(self, data):
        self.data = data
    
    def make_mean(self):
        self.mean = np.mean(self.data)
        return np.mean(self.data)

    def make_variance(self):
        self.variance = tools.get_variance(self.data)
        return tools.get_variance(self.data)
    
    def make_std(self):
        self.std = math.sqrt(self.make_variance())
    
    def make_mode(self):
        self.mode = np.mode(self.data)

class Undefs:
    def __init__(self, data):
        self.data = data

    def make_list_of_indexes(self):
        self.indexes = []
        for i, datapoint in enumerate(self.data):
            if datapoint == None:
                self.indexes.append(i)

    def work_undefs(self):
        pass

#dataset = [1,2,1,1,1,1,1,1,1,1,1,1,100]
#
#my_series = Series(dataset)
#
#print(my_series)
##print(my_series.stats.variance())
#
#print(my_series.outliers.ind )
#
##print(my_series.stats.mean())


class Model:
    # Base class, models should be derivates of this class
    def __init__(self, model_name):
        """Now: use array to initialize series.
        Class has subclasses:
            Outliers: find extreme values
            Stats: basic statistical operations

        """
        pass

    def fit(self, data, return_series=False):
        # data here for now, later make access to loaded data  from modeller
        pass

    def init_params(self, params):
        pass

    def predict(self):
        series = []
        return series

    def make_data_package(self):
        pass

    def evaluate(self, train_data, test_data):
        pass
