"""
INCLUDES:
------------------------
simple splitting
seasonal interpolation methods
seasonal extrapolation methods

windowing

ACF function
DFT plot
period finding algorithms

#from scipy.signal import argrelextrema

Low priority:
TODO: prepare extract season function
    - main problem is the amount/frequency of data and noise handling
TODO: prepare common decomposition object

"""
import math
import numpy as np

import model.tools as tools
import model.models as models
from model.tools import correlogram

import measure.timing as timing
from utility.logger import log


def prepare_splits(input_data, period):
    log("")
    data_ready_to_split = np.array(input_data[(len(input_data) % period):])

    split_points = np.arange(1, int(data_ready_to_split.size / period +1 ))*int(period)

    splits = np.split(data_ready_to_split, np.array(split_points))[:-1] # -1 is empty, change split points
    splits = np.array(splits)
    return splits


def argrelextrema(res_correlogram, greater):
    return None

def center_seasons(splits):
    log("")

    splits_centered = []

    for split in splits:
        splits_centered.append(np.subtract(split, np.mean(split)))

    splits_centered = np.array(splits_centered, dtype = object)

    return splits_centered


def get_fast_robust_seasonal_interpolation(input_data, period, shifting="median", method="median", percentile=95, alpha = 0.7):
    log("")

    _position = len(input_data) % period

    data_ready_to_split = np.array(input_data[_position:])
    split_points = np.arange(1, int(data_ready_to_split.size / period+1))*int(period)
    splits = np.split(data_ready_to_split, np.array(split_points))[:-1]
    splits = np.array(splits)

    if shifting == "median":
        row_centers = np.median(splits, axis=1)
    elif shifting == "mean":
        row_centers = np.mean(splits, axis=1)
    elif shifting == "percentile":
        row_centers = np.percentile(splits, percentile, axis=1)
    else:
        row_centers = np.median(splits, axis=1)

    row_centers = np.tile(np.array([row_centers]).transpose(), (1, period))
    splits_centered = np.subtract(splits, row_centers)

    if method=="median":
        expect = np.median(splits_centered, axis=0)
    elif method=="mean":
        expect = np.mean(splits_centered, axis=0)
    elif method=="exponential":
        # TODO: some version of regression-based exponential smoothing

        coeffs = np.array([np.power((1-alpha),k) for k in range(splits_centered.shape[0])])
        coeffs = np.divide(coeffs, coeffs.sum())[::-1]

        data_multiplied = np.multiply(splits_centered, np.tile(np.array([coeffs]).transpose(), (1, splits_centered.shape[1])))
        expect = np.sum(data_multiplied, axis=0)
        expect = np.average(a=splits_centered, axis=0, weights=coeffs)
    else:
        log("Unspecified method")
        expect = np.median(splits, axis=0)

    return expect


def prepare_extrapolation(period_interpolation, splits, steps_ahead=60, method="simple", shifting="median", percentile=95, alpha=0.2):
    log("")

    if shifting == "median":
        data_centers = np.median(splits, axis=1)
        interpolation_shift = np.median(period_interpolation)
    elif shifting == "mean":
        data_centers = np.mean(splits, axis=1)
        interpolation_shift = np.mean(period_interpolation)
    elif shifting == "percentile":
        data_centers = np.percentile(splits, percentile, axis=1)
        interpolation_shift = np.percentile(period_interpolation, percentile)
    else:
        data_centers = np.median(splits, axis=1)
        interpolation_shift = np.median(period_interpolation)

    #plt.plot(data_centers)
    #plt.show()
    if method == "median":
        series = models.Series(data_centers)
        series.make_trendline(steps_ahead=steps_ahead)
        final_shift = series.trendline[-steps_ahead:]
    elif method == "exponential":
        n = np.array(data_centers).size
        x = np.arange(0, n)
        coeffs = np.array([np.power((1-alpha),k) for k in range(n)])
        coeffs = np.divide(coeffs, coeffs.sum())[::-1]
        polynomial = np.polyfit(x, np.array(data_centers), 1, w=coeffs)

        x = np.arange(0, (n+steps_ahead))
        trendline = polynomial[0] * x

        trendline = np.add(np.multiply(x, polynomial[0]), polynomial[1])
        final_shift = trendline[-steps_ahead:]
    else:
        log("Unspecified method - use simple")
        series = models.Series(data_centers)
        series.make_trendline(steps_ahead=steps_ahead)
        final_shift = series.trendline[-steps_ahead:]

    extrapolation = np.array([])

    num_of_concatenations = steps_ahead
    for con_num in range(num_of_concatenations):
        #local_split = np.add(period_interpolation, final_shift[con_num]-np.mean(period_interpolation))
        local_split = np.add(period_interpolation, final_shift[con_num]-interpolation_shift)
        extrapolation = np.concatenate((extrapolation, local_split))

    return extrapolation


def center_seasons_by_start(splits):
    log("")

    center_timer = timing.Timer("CENTER")
    center_timer.start()

    splits_centered = []

    for split in splits:
        split_len = int(len(split)/5)
        centering = np.mean(split[:split_len])
        splits_centered.append(np.subtract(split, centering))

    splits_centered = np.array(splits_centered, dtype = object)

    center_timer.stop()

    return splits_centered

def get_seasonal_means_and_sigmas(splits_centered, expected_period):
    """
    Compute mean value on seasoned position. Compute its std.
    TODO:
    change to numpy operations
    """
    mean_splits = []
    sigmas = []

    sigmas_upper = []
    sigmas_lower = []


    for position in range(int(expected_period)):
        mean_on_position = 0
        sum_on_position = 0
        season_counter = 0

        for season in splits_centered:
          try:
            sum_on_position += splits_centered[season_counter][position]
            season_counter += 1
          except:
            pass

        try:
            mean_on_position = sum_on_position/season_counter
        except:
            mean_on_position = sum_on_position

        mean_splits.append(mean_on_position)

        season_counter = 0
        upper_season_counter = 0
        lower_season_counter = 0

        variance_on_position = 0

        upper_variance_on_position = 0
        lower_variance_on_position = 0

        for season in splits_centered:
            try:
                loc_dif = mean_on_position - splits_centered[season_counter][position]
                variance_on_position += loc_dif ** 2

                if (loc_dif > 0):
                    upper_variance_on_position += loc_dif ** 2
                    upper_season_counter += 1
                elif (loc_dif < 0):
                    lower_variance_on_position += loc_dif ** 2
                    lower_season_counter += 1

                season_counter += 1
            except:
                pass

        try:
            variance_on_position /= season_counter
        except:
            pass

        try:
            upper_variance_on_position /= upper_season_counter
        except:
            upper_variance_on_position = 0


        try:
            lower_variance_on_position /= lower_season_counter
        except:
            lower_variance_on_position = 0

        sigma_on_position = np.power(variance_on_position, 0.5) 
        sigma_upper_on_position = np.power(upper_variance_on_position, 0.5) 
        sigma_lower_on_position = np.power(lower_variance_on_position, 0.5) 

        sigmas.append(sigma_on_position)
        sigmas_upper.append(sigma_upper_on_position)
        sigmas_lower.append(sigma_lower_on_position)

    return mean_splits, sigmas, sigmas_upper, sigmas_lower


def get_robust_seasonal_means_and_sigmas(splits_centered, expected_period):

    for i, split in enumerate(splits_centered):
        splits_centered[i] = np.array(split)

    #splits_centered = np.array(splits_centered)
    """
    Compute mean value on seasoned position. Compute its std.

    TODO:
    change to numpy operations
    """
    mean_splits = []
    sigmas = []

    sigmas_upper = []
    sigmas_lower = []


    for position in range(int(expected_period)):
        center_position = 0
        sum_on_position = 0
        season_counter = 0

        positioned_points = []
        for season in splits_centered:

            try:
                positioned_points.append(splits_centered[season_counter][position])

                season_counter += 1
            except:
                pass


        #center_position = np.mean(positioned_points)
        #center_position = np.max(positioned_points)
        center_position = np.median(positioned_points)

        mean_splits.append(center_position)

        season_counter = 0
        upper_season_counter = 0
        lower_season_counter = 0

        variance_on_position = 0

        upper_variance_on_position = 0
        lower_variance_on_position = 0

        for season in splits_centered:
            try:
                loc_dif = center_position - splits_centered[season_counter][position]
                variance_on_position += loc_dif ** 2

                if (loc_dif > 0):
                    upper_variance_on_position += loc_dif ** 2
                    upper_season_counter += 1
                elif (loc_dif < 0):
                    lower_variance_on_position += loc_dif ** 2
                    lower_season_counter += 1

                season_counter += 1
            except:
                pass

        try:
            variance_on_position /= season_counter
        except:
            pass

        try:
            upper_variance_on_position /= upper_season_counter
        except:
            upper_variance_on_position = 0

        try:
            lower_variance_on_position /= lower_season_counter
        except:
            lower_variance_on_position = 0

        sigma_on_position = np.power(variance_on_position, 0.5) 
        sigma_upper_on_position = np.power(upper_variance_on_position, 0.5) 
        sigma_lower_on_position = np.power(lower_variance_on_position, 0.5) 

        sigmas.append(sigma_on_position)
        sigmas_upper.append(sigma_upper_on_position)
        sigmas_lower.append(sigma_lower_on_position)

    return mean_splits, sigmas, sigmas_upper, sigmas_lower

def get_fast_robust_seasonal_means_and_sigmas(x, expected_period):
    a = x.reshape(expected_period,  int(x.shape[0]/expected_period) )
    row_centers = np.mean(a, axis=0)
    a = np.subtract(a, row_centers)
    
    #centers = np.percentile(a, 95, axis=1)
    
    centers = np.median(a, axis=1)

    return centers


def self_convolution_error(series):
    """
    Start of self-convolution with MSE computation
    IN: series, (arbi_range_min/max, sample size/sample location)
    OUT: series of positioned MSE

    Algorithm sustains periodicity and generates sin-like function.
    Provides easier evaluation.

    USECASE EXAMPLE: 
    Let TS be candidate on seasonal pattern

    (Not every pattern is simple to recognize, especially in small datasets,
     results on ACF/DFT are affected by error)

    Select random whole pattern and let convolution run
    After that evaluate periodicity
    """
    log("")
    #series = np.array(tools.differentiate_series(series)[1:])
    # range should be continuous, not including sample (mse=0)
    arbi_range_max = series.size

    arbi_range_min = int(arbi_range_max/3)+1

    sample_start = 0
    sample_end = int(arbi_range_min)
    
    sample = np.array(series[sample_start:sample_end])
    
    mses = []

    # ADD: automatic start/end
    # ADD: check sample_start/end is in range
    sample_len = sample.size

    for k in range(arbi_range_min, arbi_range_max-1-sample_len):
        try:
            selection = series[k:(k+sample_len)]

            # extension in case of extreme values
            n = np.array(selection).size
            t = np.arange(0, n)
            p = np.polyfit(t, np.array(selection), 1)
            selection = np.array(selection) - p[0] * t

            # sample and selection have common mean
            mean_dif = np.mean(sample) - np.mean(selection)
            selection = np.add(selection, mean_dif)

            subtractions = np.subtract(sample, selection)
            subtractions = np.absolute(subtractions)

            mse = math.sqrt(sum(np.multiply(subtractions, subtractions))/len(subtractions))

            mses.append(mse)
        except:
            mses.append(mses[-1])

    return mses

def get_expected_period(series):
    mses_series = models.Series(series)

    mses_series.make_differenced_series()

    differenced = np.array(mses_series.differenced.series[1:])

    mean = np.mean(differenced)

    # CORRELOGRAM
    # repetition for smoothing
    # local extrema founder might be 
    res_correlogram = correlogram(differenced)
    res_correlogram = correlogram(res_correlogram)
    res_correlogram = correlogram(res_correlogram/1000)
    res_correlogram = correlogram(res_correlogram)
    res_correlogram = correlogram(res_correlogram)


    for lag in range(1, differenced.size):

        cor_lag = 0

        for index_now in range(differenced.size - lag):
            cor_lag += ( differenced[ index_now ] - mean ) * ( differenced[ index_now + lag ] - mean )

        #correlogram[lag] = cor_lag #* (1 / differenced.size)


    # Collect extremes: candidates on periods

    # for local maxima
    local_maxima = argrelextrema(res_correlogram, np.greater)

    # for local minima
    local_minima = argrelextrema(res_correlogram, np.less)

    local_maxima = local_maxima[0]

    # ADD: condition result_ok
    result_ok = True

    expected_period = 1
    if ( result_ok ):
        differences = tools.differentiate_series(local_maxima)

        expected_period = tools.get_median(differences)

    log("Period candidate: ", expected_period)

    return expected_period