use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder

use Xormon;

use lib "../../../perl-modules";
use Test;
use COHESITY;
#use COHESITY_NEXT;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3219;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";

Xormon::log("Service listening on port $port");

my $client;

while ($client = $server->accept()) {
    my $pid;

    unless ( defined( $pid = fork() ) ) {
        Xormon::error("Wait for fork");
        sleep 5;
    }

    if ($pid == 0) {

        my $client_address  = $client->peerhost();
        my $client_port     = $client->peerport();

        my $response = '';

        Xormon::log("Connection from $client_address:$client_port");

        my $data = "";

        $client->recv($data, 2048);

        my $decoded = from_json($data);

        Xormon::log("Received task: ".$decoded->{task});

        #if ( $decoded->{task} eq "fetch" ) {
        if ( $decoded->{task} eq "fetch" ) {

            $client->send("Task successfuly started");

            $client->shutdown(SHUT_WR);

            my $cfg    = $decoded->{data};

            # token == $decoded->{jwt}
            my $xormon = Xormon->new($client_address, $decoded->{jwt}, 'cohesity', $cfg->{hostcfg_id}, $cfg->{hostalias});
            $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

            ############################### AGENT SECTION ###############################
            # target destinations check
            if ( defined $decoded->{'destinations'} ) {
                $xormon->setDestinations($decoded->{'destinations'});
            }


            COHESITY::collect($xormon,$cfg);

            ############################### END AGENT SECTION ###############################

            Xormon::log("Task [fetch] for ".$cfg->{hostalias}." is done");

        }
        elsif ($decoded->{task} eq "test") {
            my $cfg = $decoded->{data};

            COHESITY::connection_test($client, $cfg);

            $client->send($response);

            $client->shutdown(SHUT_WR);

        }
        else {

            $response = "Undefined task";

            $client->send($response);

            $client->shutdown(SHUT_WR);

        }

        exit();

    }
    else {
        $client->autoflush(1);
    }
}


$server->close();