package DATACORE;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Data::Dumper;
use Xormon;
use Time::Local;
#use Date::parse;
use JSON qw(encode_json decode_json);
use HTTP::Date;
use Test;
use XormonLogger;
use Cwd qw(cwd);


my %data;
my %volumes_response_time;
my $latest_volumes_file;
my $xormon;
my $cfg;
my %conf;
my $logger;

my $act_dir     = cwd;
my $wrkdir      = "";
my $act_time    = "";
my $status_map;
my %metrics_config;
my @ips_servers = ();
my $iteration   = 10;
my $sample      = 300;

## sometimes we get error 'Recv failure: Connection reset by peer' due to old ssl_ciphers, so we use variable in advanced settings/magic to change it to:
## SSL_cipher_list => 'DEFAULT:!DH:@SECLEVEL=0'
my $ssl_cipher_list = 'DEFAULT:!DH';

sub collect {
  $xormon = shift;
  $cfg    = shift;
  $act_time = time;
  %conf        = ();
  %data        = ();
  @ips_servers = ();

  %metrics_config = metrics_load();
  $status_map = set_status_map();

  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")        && return "error"; }
  $logger = XormonLogger->new( 'datacore', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiUsername'} )   { $logger->error("Cannot continue, 'apiUsername' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )   { $logger->error("Cannot continue, 'apiPassword' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'nodeHost'} )      { $logger->error("Cannot continue, 'nodeHost' is not configured!")        && return "error"; }
  unless ( exists $cfg->{'proto'} )         { $logger->error("Cannot continue, 'protocol' is not configured!")        && return "error"; }
  if ( exists $cfg->{'ssl_cipher_list'} && $cfg->{'ssl_cipher_list'} ne "" ) {
    $ssl_cipher_list = $cfg->{'ssl_cipher_list'};
  }
  $wrkdir       = "$act_dir/data/$cfg->{'hostcfg_id'}";

  $latest_volumes_file  = "$wrkdir/latest_volumes_file.json";
  if ( -f $latest_volumes_file ) {
    eval { %volumes_response_time = %{ Xormon::read_json($latest_volumes_file) }; };
    if ($@) {
      $logger->error("Read json file $latest_volumes_file failed");
    }
  }

  # if proxyHost not defined, use nodeHost for the proxyHost key
  if ( !exists $cfg->{'proxyHost'} || $cfg->{'proxyHost'} eq '' ) {
    $cfg->{'proxyHost'} = $cfg->{'nodeHost'};
  }

  @ips_servers = ();
  $cfg->{api_version} = 1;
  $cfg->{new_api} = 0;

  get_api_version ();
  get_data();
  # print Dumper \%data;

  my $perf_out = prepare_perf();
  # print Dumper $perf_out;
  my $rc       = $xormon->saveJSON2($perf_out);
  my $status   = "error";
  if ( defined $rc ) {
    $logger->info("save perf: $rc");

    if ( $rc eq "201" ) {
      $status = "successful";
    }
    elsif ( $rc eq "401" ) {
      $status = "unauthorized";
      last;
    }
    else {
      $status = "error";
      last;
    }
  }
  else {
    $logger->info("save perf: UN");
  }

  if ( $status eq "successful" ) {
    $xormon->hasData(1);
  }
  else {
    $xormon->hasData(0, "no available stats: $status");
  }

  send_conf();
  health_status();

  # print Dumper $perf_out;
  # print Dumper \%data;
  return $status;
}

sub prepare_perf {
  my %stats_last;
  my %stats_out;

  my $last_stats_file      = "$wrkdir/last_perf.json";
  my $convert_counters = 0;
  my $timediff         = "";
  if ( -f $last_stats_file ) {
    eval { %stats_last = %{ Xormon::read_json($last_stats_file) }; };
    if ( exists $stats_last{ms_timestamp} ) {
      $timediff = $act_time - $stats_last{ms_timestamp};
      if ( $timediff < ( 12 * $sample ) && $timediff >= ( $sample - 60 ) ) { # convert counter only when timediff is between 4 and 10 minutes
        $convert_counters = 1;
      }
      else {
        $logger->error( "Cannot convert counters to gauge. \$act_time=$act_time (".localtime($act_time)."), \$stats_last{ms_timestamp}=$stats_last{ms_timestamp} (".localtime($stats_last{ms_timestamp})."), \$timediff=$timediff " . __FILE__ . ":" . __LINE__ );
      }
    }
    else {
      $logger->error( "Something wrong \$stats_last{ms_timestamp} doesn't exist!" . __FILE__ . ":" . __LINE__ );
    }
  }

  $data{ms_timestamp} = $act_time;
  foreach my $subsystem (keys %metrics_config) {
    if ($subsystem eq "pool") {
      next; # skip pool for now, we have to sum data from nodes in sum_data_pool
    }
    if (exists $data{$subsystem}) {
      foreach my $key (keys %{ $data{$subsystem} }) {
        my $var = $data{$subsystem}{$key};

        if (exists $var->{id}) {
          my $id = $var->{id};
          if ($convert_counters && exists $stats_last{$subsystem}{$id}) {
            my $hash_key = lc(Xormon::create_hash_key($cfg->{hostcfg_id} . "$subsystem" . $id)); $hash_key = lc($hash_key);
            foreach my $metric (@{$metrics_config{$subsystem}}) {
              if (exists $data{$subsystem}{$id}{$metric} && exists $data{$subsystem}{$id}{timestamp} && exists $stats_last{$subsystem}{$id}{$metric} && exists $stats_last{$subsystem}{$id}{timestamp}) {               
                $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{$metric} = make_gauge(
                  $data{$subsystem}{$id}{timestamp},
                  $stats_last{$subsystem}{$id}{timestamp},
                  $data{$subsystem}{$id}{$metric},
                  $stats_last{$subsystem}{$id}{$metric}
                );
              }
            }
            ## CAPACITY
            if ( exists $data{$subsystem}{$id}{'total'} && Xormon::isdigit($data{$subsystem}{$id}{'total'})) {
              $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_total'} = $data{$subsystem}{$id}{'total'};
            }
            if ( exists $data{$subsystem}{$id}{'used'} && Xormon::isdigit($data{$subsystem}{$id}{'used'})) {
              $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_used'} = $data{$subsystem}{$id}{'used'};
            }
            if ( exists $data{$subsystem}{$id}{'free'} && Xormon::isdigit($data{$subsystem}{$id}{'free'})) {
              $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_free'} = $data{$subsystem}{$id}{'free'};
            }
            if ( exists $data{$subsystem}{$id}{'virtual_capacity_total'} && Xormon::isdigit($data{$subsystem}{$id}{'virtual_capacity_total'})) {
              $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'virtual_capacity_total'} = $data{$subsystem}{$id}{'virtual_capacity_total'};
            }
            if (Xormon::isdigit($stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_total'}) && $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_total'} !=0 && Xormon::isdigit($stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_used'}) ) {
              my $pom = Xormon::formatNumberByType("float4", $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_used'} / ($stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_total'} / 100) );
              $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{'capacity_used_percent'} = $pom;
            }
            ## RESPONSE TIME MAGIC
            if ($subsystem eq "volume") {
              my $numerator_r   = $data{$subsystem}{$id}{numerator_r};
              my $denominator_r = $data{$subsystem}{$id}{denominator_r};
              my $sample_rate1  = exists $stats_last{$subsystem}{$id}{sample_rate_resp_t} ? $stats_last{$subsystem}{$id}{sample_rate_resp_t} : "";
              my $sample_rate2  = $data{$subsystem}{$id}{sample_rate_resp_t};
              my $numerator_w   = $data{$subsystem}{$id}{numerator_w};
              my $denominator_w = $data{$subsystem}{$id}{denominator_w};
              if ( !defined $numerator_r && !defined $denominator_r && !defined $sample_rate1 && !defined $sample_rate2 && !defined $sample_rate2 && !defined $numerator_w && !defined $denominator_w ) {
                $logger->error("Something is wrong. Volume $id is not response time data");
              }
              if (defined $sample_rate1 && defined $sample_rate2 && $sample_rate1 < $sample_rate2) {
                if ( Xormon::isdigit($numerator_r) && Xormon::isdigit($denominator_r) ) {
                  if ( $numerator_r == 0 ) {
                    #if (isdigit($inventory_data{$section}{$volume_id}{resp_t_r_1}) && $inventory_data{$section}{$volume_id}{resp_t_r_1} == 0){
                    $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_r} = 0;
                    #}
                    #if (!isdigit($inventory_data{$section}{$volume_id}{resp_t_r_1})){
                    #  $inventory_data{$section}{$volume_id}{resp_t_r_1} = 0;
                    #}
                  }
                  else {
                    if ( $denominator_r == 0 ) { next; }
                    my $resp_t_r = $numerator_r / $denominator_r;
                    if ( Xormon::isdigit($resp_t_r) && $resp_t_r !~ m/^-/ ) {
                      $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_r} = $resp_t_r;
                    }
                  }
                }
                if ( Xormon::isdigit($numerator_w) && Xormon::isdigit($denominator_w) ) {
                  if ( $numerator_w == 0 ) {
                    #if (isdigit($inventory_data{$section}{$volume_id}{resp_t_w_1}) && $inventory_data{$section}{$volume_id}{resp_t_w_1} == 0){
                    $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_w} = 0;
                    #}
                    #if (!isdigit($inventory_data{$section}{$volume_id}{resp_t_w_1})){
                    #  $inventory_data{$section}{$volume_id}{resp_t_w_1} = 0;
                    #}
                  }
                  else {
                    if ( $denominator_w == 0 ) { next; }
                    my $resp_t_w = $numerator_w / $denominator_w;
                    if ( Xormon::isdigit($resp_t_w) && $resp_t_w !~ m/^-/ ) {
                      $stats_out{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_w} = $resp_t_w;
                    }
                  }
                }
                delete $data{$subsystem}{$id}{numerator_r};
                delete $data{$subsystem}{$id}{denominator_r};
                delete $data{$subsystem}{$id}{numerator_w};
                delete $data{$subsystem}{$id}{denominator_w};
              }
            } 
          }
        }
      }
    }
  }

  if ( $convert_counters && exists $data{pool}) { 
    my $pools_ref = sum_data_pool(\%stats_out, \%stats_last);
    # Merge pool data into stats_out
    foreach my $pool_id (keys %{$pools_ref}) {
      my $hash_key = lc(Xormon::create_hash_key($cfg->{hostcfg_id} . "pool" . $pool_id)); $hash_key = lc($hash_key);
      foreach my $time (keys %{$pools_ref->{$pool_id}}) {
        foreach my $metric (keys %{$pools_ref->{$pool_id}{$time}}) {
          $stats_out{datacore}{pool}{$hash_key}{$time}{$metric} = $pools_ref->{$pool_id}{$time}{$metric};
        }
      }
    }
  }

  ### BLOCK SIZE, CACHE, RESPONSE TIME, DEVICE STATS
  calculate_other_metrics(\%stats_out);

  ## save volumes response times, logicaldisks
  eval { Xormon::save_as_json( $latest_volumes_file, \%volumes_response_time ); };
  if ($@) {
    $logger->error("Save data to file $latest_volumes_file failed");
  }

  # save current counters for the further usage
  eval { Xormon::save_as_json($last_stats_file,\%data); };
  if ($@) {
    $logger->error("Save data to file $last_stats_file failed");
  }
  # print Dumper \%stats_out;
  return \%stats_out;
}

sub sum_data_pool {
  my %pools;
  my $subsystem = "pool";
  my ($stats_out_ref, $stats_last_ref) = @_;
  foreach my $key (keys %{ $data{$subsystem} }) {
    my $long_id = $key; # long id with node prefix
    my $var = $data{$subsystem}{$key};
    my ( undef, $pool_id ) = split( ":", $long_id );
    $pool_id =~ s/\{//g;
    $pool_id =~ s/\}//g;

    if (exists $stats_last_ref->{$subsystem}{$long_id} ) {
      foreach my $metric (@{$metrics_config{$subsystem}}) {
        if (exists $data{$subsystem}{$long_id}{$metric} && exists $data{$subsystem}{$long_id}{timestamp} && exists $stats_last_ref->{$subsystem}{$long_id}{$metric} && exists $stats_last_ref->{$subsystem}{$long_id}{timestamp}) {               
          my $num = make_gauge(
            $data{$subsystem}{$long_id}{timestamp},
            $stats_last_ref->{$subsystem}{$long_id}{timestamp},
            $data{$subsystem}{$long_id}{$metric},
            $stats_last_ref->{$subsystem}{$long_id}{$metric}
          );
          if ( Xormon::isdigit($pools{$pool_id}{$act_time}{$metric}) ) {
            $pools{$pool_id}{$act_time}{$metric} += $num;
          } else {
            $pools{$pool_id}{$act_time}{$metric} = $num;
          }
        }
      }
      ## CAPACITY
      if ( exists $data{$subsystem}{$long_id}{'total'} && Xormon::isdigit($data{$subsystem}{$long_id}{'total'})) {
        $pools{$pool_id}{$act_time}{'capacity_total'} = $data{$subsystem}{$long_id}{'total'};
      }
      if ( exists $data{$subsystem}{$long_id}{'used'} && Xormon::isdigit($data{$subsystem}{$long_id}{'used'})) {
        $pools{$pool_id}{$act_time}{'capacity_used'} = $data{$subsystem}{$long_id}{'used'};
      }
      if ( exists $data{$subsystem}{$long_id}{'free'} && Xormon::isdigit($data{$subsystem}{$long_id}{'free'})) {
        $pools{$pool_id}{$act_time}{'capacity_free'} = $data{$subsystem}{$long_id}{'free'};
      }
      if ( exists $data{$subsystem}{$long_id}{'virtual_capacity_total'} && Xormon::isdigit($data{$subsystem}{$long_id}{'virtual_capacity_total'})) {
        $pools{$pool_id}{$act_time}{'virtual_capacity_total'} = $data{$subsystem}{$long_id}{'virtual_capacity_total'};
      }
      if ($pools{$pool_id}{$act_time}{'capacity_total'} !=0 && Xormon::isdigit($pools{$pool_id}{$act_time}{'capacity_used'}) ) {
        my $pom = Xormon::formatNumberByType("float4", $pools{$pool_id}{$act_time}{'capacity_used'} / ($pools{$pool_id}{$act_time}{'capacity_total'} / 100) );
        $pools{$pool_id}{$act_time}{'capacity_used_percent'} = $pom;
      }
      if ( Xormon::isdigit($pools{$pool_id}{$act_time}{'capacity_total'}) && Xormon::isdigit($pools{$pool_id}{$act_time}{'virtual_capacity_total'}) ) {
        $pools{$pool_id}{$act_time}{'capacity_oversubscr'} = $pools{$pool_id}{$act_time}{'virtual_capacity_total'} - $pools{$pool_id}{$act_time}{'capacity_total'};
        if ($pools{$pool_id}{$act_time}{'capacity_oversubscr'} < 0){$pools{$pool_id}{$act_time}{'capacity_oversubscr'} = 0;}
        if ($pools{$pool_id}{$act_time}{'capacity_total'} != 0) {
          $pools{$pool_id}{$act_time}{'capacity_oversubscr_percent'} = Xormon::formatNumberByType("float4", ($pools{$pool_id}{$act_time}{'virtual_capacity_total'} / $pools{$pool_id}{$act_time}{'capacity_total'}) * 100);
        }
      }
    } 
  }
  return \%pools;
  # print Dumper \%pools;
}

sub get_data {
  my $section;
  my $out;

  ## VOLUME
  $section = "volume";
  $out = cmd("virtualdisks");
  # print Dumper $out;

  my $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("virtualdisks");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/VirtualDiskPerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  # if ( $out eq "" ) {
  #   return;
  # }
  ###
  # print Dumper @{$performance_data};
  # print Dumper $out;

  foreach my $volume ( @{$out} ) {
    my $id      = $volume->{Id};
    my $size    = $volume->{Size}->{Value};
    my $alias   = $volume->{Alias};
    my $subtype = $volume->{SubType};
    my $status  = $volume->{DiskStatus};
    if ( defined $subtype && $subtype eq "3" ) { next; }
    if ( !defined $alias ) {
      $alias = "";
    }
    $alias =~ s/\+//g;
    $alias =~ s/\@//g;
  
    if ( defined $id && $id ne "" ) {
      if ( !defined $alias || $alias eq "" ) {
        $alias = $id;
      }
      my $url_id = $id;
      $id =~ s/\}//g;
      $id =~ s/\{//g;
      if ( !defined $data{$section}{$id}{id} ) {
        $data{$section}{$id}{id} = $id;
      }
      if ( defined $status && $status ne "" ) {
        my ( $status_new, $hs_status ) = get_status( $status, "volume" );
        $data{$section}{$id}{status} = $status_new;
        $data{$section}{$id}{name}   = $alias;
        if ( defined $hs_status ) {
          $data{$section}{$id}{hs_status} = $hs_status;
        }
      }
      if ( Xormon::isdigit($size) ) {
        $data{$section}{$id}{"total"} = $size;
      }
      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            # $data{$section}{$id}{perf} = $element->{PerformanceData}; ### TEST
            push( @{$out}, $element->{PerformanceData} );

            #$out = $element->{PerformanceData};
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$url_id";
        $out = cmd($url);
      }

      if ( !defined $out || $out eq "" ) {
        next;
      }
      # print Dumper $out;
      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}               = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}              = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}          = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}            = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}           = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}            = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";
        $data{$section}{$id}{cache_r_hit}        = defined $perf_block->{"CacheReadHits"}           ? $perf_block->{"CacheReadHits"}           : "";
        $data{$section}{$id}{cache_r_hit_bytes}  = defined $perf_block->{"CacheReadHitBytes"}       ? $perf_block->{"CacheReadHitBytes"}       : "";
        $data{$section}{$id}{cache_w_hit}        = defined $perf_block->{"CacheWriteHits"}          ? $perf_block->{"CacheWriteHits"}          : "";
        $data{$section}{$id}{cache_w_hit_bytes}  = defined $perf_block->{"CacheWriteHitBytes"}      ? $perf_block->{"CacheWriteHitBytes"}      : "";
        $data{$section}{$id}{cache_r_miss}       = defined $perf_block->{"CacheReadMisses"}         ? $perf_block->{"CacheReadMisses"}         : "";
        $data{$section}{$id}{cache_r_miss_bytes} = defined $perf_block->{"CacheReadMissBytes"}      ? $perf_block->{"CacheReadMissBytes"}      : "";
        $data{$section}{$id}{cache_w_miss}       = defined $perf_block->{"CacheWriteMisses"}        ? $perf_block->{"CacheWriteMisses"}        : "";
        $data{$section}{$id}{cache_w_miss_bytes} = defined $perf_block->{"CacheWriteMissBytes"}     ? $perf_block->{"CacheWriteMissBytes"}     : "";
        $data{$section}{$id}{used}               = defined $perf_block->{"BytesAllocated"}          ? $perf_block->{"BytesAllocated"}          : "";

        ## calculate free from used and capacity
        if ( Xormon::isdigit($data{$section}{$id}{used}) ) {
          if ( Xormon::isdigit( $data{$section}{$id}{"total"} ) ) {
            $data{$section}{$id}{"free"} = $data{$section}{$id}{"total"} - $data{$section}{$id}{"used"};
          }
        }

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $alias : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }

  ### LOGICAL DISKS 
  my $mapping = cmd("logicaldisks");
  # if ( $out eq "" ) {
  #   return;
  # }
  my $virtual_volume = "";


  ### rest api 2.0 - output shoulds be empty
  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$mapping} ) { last; }
      # if ( !isdigit($index_data) ) {
      #   $out = cmd("logicaldisks");
      # }
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( $virtual_volume ne "" && @{$virtual_volume} ) { last; }
      $virtual_volume = cmd("virtuallogicalunits");
      # if ( !isdigit($index_data2) ) {
      #   $virtual_volume = cmd("virtuallogicalunits");
      # }
      # else {
      #   $virtual_volume = $config_data[$index_data2];
      # }

      # print Dumper \$virtual_volume;
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/VirtualLogicalUnitPerformance");
      if ( $performance_data eq "" ) { $cfg->{api_version} = 1; last; }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  ###

  # if ( $out eq "" || ( $virtual_volume eq "" && $cfg->{api_version} == 2 ) ) {
  #   return;
  # }

  if ( exists $cfg->{api_version} && $cfg->{api_version} == 2 ) {
    foreach my $element ( @{$performance_data} ) {
      my $object_id = $element->{ObjectId};
      foreach my $unit ( @{$virtual_volume} ) {
        my $id           = $unit->{Id};
        my $logical_unit = $unit->{LogicalDiskId};
        if ( defined $id && defined $logical_unit && defined $object_id && $object_id eq $id ) {
          push( @{ $element->{logical_disk} }, $logical_unit );
        }
      }
    }
  }

  ##### SPACE FOR VOLUME RESPONSE TIME
  foreach my $host ( @{$mapping} ) {
    my $id        = $host->{Id};
    my $volume_id = $host->{VirtualDiskId};
    my $alias     = $host->{Alias};
    if ( !defined $alias ) {
      $alias = "";
    }
    if ( defined $id && $id ne "" && defined $volume_id && $volume_id ne "" ) {
      if ( !defined $data{$section}{$volume_id}{id} ) { next; }
      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        foreach my $element ( @{$performance_data} ) {
          if ( !defined $element->{logical_disk} ) { next; }
          foreach my $logical_disk ( @{ $element->{logical_disk} } ) {
            if ( $id eq $logical_disk && defined $element->{PerformanceData} ) {
              #### dodelat object id do performance_data
              $element->{PerformanceData}->{ObjectId} = $element->{ObjectId};
              push( @{$out}, $element->{PerformanceData} );
            }
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$id";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }
      foreach my $perf ( @{$out} ) {
        my $read_io     = $perf->{TotalReads};
        my $write_io    = $perf->{TotalWrites};
        my $resp_t_r    = $perf->{TotalReadTime};
        my $resp_t_w    = $perf->{TotalWriteTime};
        my $object_id   = $perf->{ObjectId};
        if ( !defined $object_id && $cfg->{api_version} == 1 ) {
          $object_id = $id;
        }

        my $sample_rate = $perf->{CollectionTime};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }

        if ( !-f $latest_volumes_file && defined $object_id ) {
          if ( defined $read_io ) {
            $volumes_response_time{$object_id}{read_io} = $read_io;
          }
          if ( defined $write_io ) {
            $volumes_response_time{$object_id}{write_io} = $write_io;
          }
          if ( defined $resp_t_r ) {
            $volumes_response_time{$object_id}{resp_t_r} = $resp_t_r;
          }
          if ( defined $resp_t_w ) {
            $volumes_response_time{$object_id}{resp_t_w} = $resp_t_w;
          }
          if ( defined $sample_rate ) {
            $volumes_response_time{$object_id}{time} = $sample_rate;
          }
          next;
        }
        # if ($datacore_debug) { print "DEBUG: $$ set_data_resp_volume for volume $volume_id | object_id $object_id " . localtime() . " " . time() . "\n"; }
        # if ( $i == 0 ) {
        #   if ($datacore_debug) {
        #     print "DEBUG: $$ data for $object_id " . localtime() . " " . time() . "\n";
        #     print Dumper \$perf;
        #   }
        # }
        # $i++;
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $alias : $id : $sample_rate\n");
          next;
        }
        # if ($datacore_debug) { print "DEBUG: $$ START get_correct_response_time_volume_values for $object_id | $sample_rate " . localtime() . " " . time() . "\n"; }

        my ( $read_io_tmp, $write_io_tmp, $resp_t_r_tmp, $resp_t_w_tmp, $sample_rate_tmp ) = get_correct_response_time_volume_values( $object_id, $read_io, $write_io, $resp_t_r, $resp_t_w, $sample_rate );
        if ( !Xormon::isdigit($sample_rate_tmp) ) {
          $logger->error("Wrong data sample_rate: $read_io_tmp, $write_io_tmp, $resp_t_r_tmp, $resp_t_w_tmp, $sample_rate_tmp\n");
          next;
        }
        # print "IOR $read_io_tmp, IOW $write_io_tmp, RTR $resp_t_r_tmp, RTW $resp_t_w_tmp, RATE $sample_rate_tmp\n";
        $read_io     = $read_io_tmp;
        $write_io    = $write_io_tmp;
        $resp_t_r    = $resp_t_r_tmp;
        $resp_t_w    = $resp_t_w_tmp;
        $sample_rate = $sample_rate_tmp;

        if ( Xormon::isdigit($sample_rate) ) {
          if ( !Xormon::isdigit( $data{$section}{$volume_id}{sample_rate_resp_t} ) ) {
            $data{$section}{$volume_id}{sample_rate_resp_t} = $sample_rate;
          }
        }

        if ( Xormon::isdigit($read_io) && Xormon::isdigit($resp_t_r) ) {
          my $numerator = $read_io * $resp_t_r;
          if ( Xormon::isdigit( $data{$section}{$volume_id}{numerator_r} ) ) {
            my $num = $data{$section}{$volume_id}{numerator_r};
            $data{$section}{$volume_id}{numerator_r} = $num + $numerator;
          }
          else {
            $data{$section}{$volume_id}{numerator_r} = $numerator;
          }

          if ( Xormon::isdigit( $data{$section}{$volume_id}{denominator_r} ) ) {
            my $num = $data{$section}{$volume_id}{denominator_r};
            $data{$section}{$volume_id}{denominator_r} = $num + $read_io;
          }
          else {
            $data{$section}{$volume_id}{denominator_r} = $read_io;
          }
        }
        if ( Xormon::isdigit($write_io) && Xormon::isdigit($resp_t_w) ) {
          my $numerator = $write_io * $resp_t_w;
          if ( Xormon::isdigit( $data{$section}{$volume_id}{numerator_w} ) ) {
            my $num = $data{$section}{$volume_id}{numerator_w};
            $data{$section}{$volume_id}{numerator_w} = $num + $numerator;
          }
          else {
            $data{$section}{$volume_id}{numerator_w} = $numerator;
          }

          if ( Xormon::isdigit( $data{$section}{$volume_id}{denominator_w} ) ) {
            my $num = $data{$section}{$volume_id}{denominator_w};
            $data{$section}{$volume_id}{denominator_w} = $num + $write_io;
          }
          else {
            $data{$section}{$volume_id}{denominator_w} = $write_io;
          }
        }
      }
    }
  }

  ### MAPPING POOL/VOLUME
  my $section_pool = "pool";
  foreach my $host ( @{$mapping} ) {
    my $id_vol  = $host->{VirtualDiskId};
    my $id_pool = $host->{PoolId};
    if ( defined $id_vol && $id_vol ne "" && defined $id_pool && $id_pool ne "" ) {
      if ( defined $data{$section}{$id_vol} ) {
        my ( undef, $pool_id ) = split( ":", $id_pool );
        if ( defined $pool_id && $pool_id ne "" ) {
          $pool_id =~ s/\{//g;
          $pool_id =~ s/\}//g;

          if (!defined $data{$section}{$id_vol}{pool_ids}) {
            $data{$section}{$id_vol}{pool_ids} = {};
          }
          $data{$section}{$id_vol}{pool_ids}{$pool_id} = 1;
          ### pool virtual allocation
          if ( Xormon::isdigit( $data{$section}{$id_vol}{'total'} ) ) {
            if ( Xormon::isdigit( $data{$section_pool}{$id_pool}{'virtual_capacity_total'} ) ) {
              $data{$section_pool}{$id_pool}{'virtual_capacity_total'} += $data{$section}{$id_vol}{'total'};
            }
            else {
              $data{$section_pool}{$id_pool}{'virtual_capacity_total'} = $data{$section}{$id_vol}{'total'};
            }
          }
        }
      }
    }
  }

  ## HOST
  $section = "host";
  $out = cmd("hosts");
  # print Dumper $out;

  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("hosts");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/MachinePerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  ###
  # if ( !isdigit($index) ) {
  #   push( @config_data, $out );
  # }
  # if ( $out eq "" ) {
  #   return;
  # }

  foreach my $host ( @{$out} ) {
    my $id     = $host->{Id};
    my $name   = $host->{HostName};
    my $status = $host->{State};
    my $host_group = $host->{HostGroupId};

    # my $type       = $host->{Type};

    if ( defined $id && $id ne "" && defined $name && $name ne "" ) {
      my $url_id = $id;
      $id =~ s/\}//g;
      $id =~ s/\{//g;
      if ( !defined $data{$section}{$id}{id} || !defined $data{$section}{$id}{name} ) {
        $data{$section}{$id}{id} = $id;
        $name =~ s/,//g;
        $name =~ s/\)//g;
        $name =~ s/\(//g;
        $name =~ s/\*//g;
        $name = remove_whitespace($name);
        $data{$section}{$id}{name} = $name;
      }

      if ( defined $status && $status ne "" ) {
        my ( $status_new, $hs_status ) = get_status( $status, "host" );
        $data{$section}{$id}{status} = $status_new;
        if ( defined $hs_status ) {
          $data{$section}{$id}{hs_status} = $hs_status;
        }
      }

      if ( defined $host_group && $host_group ne "" ) {
        $host_group =~ s/\{//g;
        $host_group =~ s/\}//g;
        $data{$section}{$id}{host_group} = $host_group;
      }

      if ( $virtual_volume ne "" ) {
        foreach my $mapped ( @{$virtual_volume} ) {
          my $host_mapping = $mapped->{PhysicalDiskId};
          if ( !defined $host_mapping || $host_mapping eq "" ) { next; }
          my ( $host_id, $volume_id ) = split( "-", $host_mapping );
          if ( !defined $host_id || $host_id ne $id ) { next; }
          if ( defined $volume_id && $volume_id ne "" ) {
            $data{$section}{$id}{vol_ids}{$volume_id} = 1;
          }
        }
      }

      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id ) {
            $object_id =~ s/\{//g;
            $object_id =~ s/\}//g;
          }
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            push( @{$out}, $element->{PerformanceData} );

            #$out = $element->{PerformanceData};
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$url_id";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }
      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}               = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}              = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}          = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}            = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}           = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}            = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $name : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }

  ## NODE
  $section = "node";
  $out = cmd("servers");

  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("hosts");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/ServerMachinePerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  ###
  # if ( !isdigit($index) ) {
  #   push( @config_data, $out );
  # }
  # if ( $out eq "" ) {
  #   return;
  # }

  foreach my $node ( @{$out} ) {
    my $id     = $node->{Id};
    my $alias  = $node->{Caption};
    my $status = $node->{PowerState};
    if ( !defined $alias ) {
      $alias = "";
    }
    if ( defined $id && $id ne "" ) {
      if ( !defined $alias || $alias eq "" ) { $alias = $id; }
      if ( !defined $data{$section}{$id}{id} ) {
        $data{$section}{$id}{id} = $id;
      }
      if ( defined $status && $status ne "" ) {
        my ( $status_new, $hs_status ) = get_status( $status, "node" );
        $data{$section}{$id}{status} = $status_new;
        $data{$section}{$id}{name}   = $alias;
        if ( defined $hs_status ) {
          $data{$section}{$id}{hs_status} = $hs_status;
        }
      }
      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            push( @{$out}, $element->{PerformanceData} );
            #$out = $element->{PerformanceData};
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$id";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }
      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}               = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}              = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}          = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}            = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}           = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}            = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";
        $data{$section}{$id}{cache_r_hit}        = defined $perf_block->{"CacheReadHits"}           ? $perf_block->{"CacheReadHits"}           : "";
        $data{$section}{$id}{cache_r_hit_bytes}  = defined $perf_block->{"CacheReadHitBytes"}       ? $perf_block->{"CacheReadHitBytes"}       : "";
        $data{$section}{$id}{cache_w_hit}        = defined $perf_block->{"CacheWriteHits"}          ? $perf_block->{"CacheWriteHits"}          : "";
        $data{$section}{$id}{cache_w_hit_bytes}  = defined $perf_block->{"CacheWriteHitBytes"}      ? $perf_block->{"CacheWriteHitBytes"}      : "";
        $data{$section}{$id}{cache_r_miss}       = defined $perf_block->{"CacheReadMisses"}         ? $perf_block->{"CacheReadMisses"}         : "";
        $data{$section}{$id}{cache_r_miss_bytes} = defined $perf_block->{"CacheReadMissBytes"}      ? $perf_block->{"CacheReadMissBytes"}      : "";
        $data{$section}{$id}{cache_w_miss}       = defined $perf_block->{"CacheWriteMisses"}        ? $perf_block->{"CacheWriteMisses"}        : "";
        $data{$section}{$id}{cache_w_miss_bytes} = defined $perf_block->{"CacheWriteMissBytes"}     ? $perf_block->{"CacheWriteMissBytes"}     : "";
        $data{$section}{$id}{cache_size}         = defined $perf_block->{"CacheSize"}               ? $perf_block->{"CacheSize"}               : "";
        $data{$section}{$id}{cache_free}         = defined $perf_block->{"FreeCache"}               ? $perf_block->{"FreeCache"}               : "";

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $alias : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }

  ## HOST GROUP
  $section = "host_group";
  $out = cmd("hostgroups");

  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("hostgroups");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/HostGroupPerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  ###
  # if ( !isdigit($index) ) {
  #   push( @config_data, $out );
  # }
  # if ( $out eq "" ) {
  #   return;
  # }

  foreach my $host_group ( @{$out} ) {
    my $id   = $host_group->{Id};
    my $name = $host_group->{Alias};
    if ( defined $id && $id ne "" && defined $name && $name ne "" ) {
      my $id_url = $id;
      $id =~ s/\{//g;
      $id =~ s/\}//g;
      if ( !defined $data{$section}{$id}{id} ) {
        $data{$section}{$id}{id}   = $id;
        $data{$section}{$id}{name} = $name;
      }

      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id ) {
            $object_id =~ s/\{//g;
            $object_id =~ s/\}//g;
          }
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            push( @{$out}, $element->{PerformanceData} );

            #$out = $element->{PerformanceData};
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$id_url";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }
      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}               = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}              = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}          = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}            = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}           = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}            = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $name : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }

  ## POOL
  $section = "pool";
  $out = cmd("pools");

  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("pools");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/DiskPoolPerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  # print Dumper $out;
  ###
  # if ( !isdigit($index) ) {
  #   push( @config_data, $out );
  # }
  # if ( $out eq "" ) {
  #   return;
  # }

  foreach my $pool ( @{$out} ) {
    my $id     = $pool->{Id};
    my $alias  = $pool->{Alias};
    my $status = $pool->{PoolStatus};
    if ( !defined $alias ) {
      $alias = "";
    }
    if ( defined $id && $id ne "" ) {
      my $url_id = $id;
      if ( !defined $alias || $alias eq "" ) { $alias = $id; }
      my ( undef, $pool_id ) = split( ":", $id );
      $pool_id =~ s/\{//g;
      $pool_id =~ s/\}//g;
      if ( !defined $pool_id ) { next; }
      $id = $pool_id;

      if ( !defined $data{$section}{$url_id}{id} ) {
        $data{$section}{$url_id}{id} = $id;
      }
      # $pools{$id} = $id;
      $id = $url_id;
      if ( defined $status && $status ne "" ) {
        my ( $status_new, $hs_status ) = get_status( $status, "pool" );
        $data{$section}{$id}{status} = $status_new;
        $data{$section}{$id}{name}   = $alias;
        if ( defined $hs_status ) {
          $data{$section}{$id}{hs_status} = $hs_status;
        }
      }
      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            push( @{$out}, $element->{PerformanceData} );

            #$out = $element->{PerformanceData};
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$url_id";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }
      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}               = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}              = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}          = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}            = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}           = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}            = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";
        $data{$section}{$id}{resp_t_r}           = defined $perf_block->{"TotalReadTime"}           ? $perf_block->{"TotalReadTime"}           : "";
        $data{$section}{$id}{resp_t_w}           = defined $perf_block->{"TotalWriteTime"}          ? $perf_block->{"TotalWriteTime"}          : "";
        $data{$section}{$id}{total}              = defined $perf_block->{"BytesTotal"}              ? $perf_block->{"BytesTotal"}              : "";
        $data{$section}{$id}{used}               = defined $perf_block->{"BytesAllocated"}          ? $perf_block->{"BytesAllocated"}          : "";
        $data{$section}{$id}{free}               = defined $perf_block->{"BytesAvailable"}          ? $perf_block->{"BytesAvailable"}          : "";

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $alias : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }

  ## DRIVE
  $section = "drive";
  $out = cmd("physicaldisks");

  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("physicaldisks");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/PhysicalDiskPerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  # print Dumper $out;
  ###
  # if ( !isdigit($index) ) {
  #   push( @config_data, $out );
  # }
  # if ( $out eq "" ) {
  #   return;
  # }
  foreach my $drive ( @{$out} ) {
    if ( defined $drive->{Type}    && $drive->{Type} eq "3" )                { next; }    ### trunk
    if ( defined $drive->{Caption} && $drive->{Caption} =~ m/Mirror Trunk/ ) { next; }    #trunk
    ## skip if non windows disk
    my $test_disk = $drive->{InquiryData}->{Serial};
    my $test_disk2 = $drive->{IsBootDisk};
    next if !defined $test_disk || !defined $test_disk2 || $test_disk2 =~ /^(true|1)$/;
    my $id    = $drive->{Id};
    my $name  = $drive->{Alias};
    my $status = $drive->{DiskStatus};
    # my $cap           = $host->{Size}->{Value};
    # my $free_cap      = $host->{FreeSpace}->{Value};
    # my $serial_number = $host->{InquiryData}->{Serial};
    # my $vendor        = $host->{InquiryData}->{Vendor};
    # my $type          = $host->{Type};
    my $node   = $drive->{ExtendedCaption};

    if ( defined $id && $id ne "" ) {
      $id =~ s/\}//g;
      $id =~ s/\{//g;
      if ( !defined $name || $name eq "" ) {
        my $caption = $drive->{Caption};
        if ( defined $caption && $caption ne "" ) {
          $name = $caption;
        }
        else {
          $name = $id;
        }
      }

      if ($node) {
        my @caption_values = split(" ", $node);
        my $last_caption = $caption_values[-1];
        if ($last_caption && grep { $data{'node'}{$_}{name} eq $last_caption } keys %{ $data{'node'} }) {
          $name = "$last_caption-$name";
        }
      }

      my $url_id = $id;
      $id =~ s/\{//g;
      $id =~ s/\}//g;
      if ( !defined $data{$section}{$id}{id} ) {
        $data{$section}{$id}{id} = $id;
      }
      if ( defined $status && $status ne "" ) {
        my ( $status_new, $hs_status ) = get_status( $status, "drive" );
        $data{$section}{$id}{status} = $status_new;
        $data{$section}{$id}{name}   = $name;
        if ( defined $hs_status ) {
          $data{$section}{$id}{hs_status} = $hs_status;
        }
      }

      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id ) {
            $object_id =~ s/\{//g;
            $object_id =~ s/\}//g;
          }
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            push( @{$out}, $element->{PerformanceData} );

            #$out = $element->{PerformanceData};
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$url_id";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }
      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}               = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}              = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}          = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}            = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}           = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}            = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";
        $data{$section}{$id}{total}              = defined $perf_block->{"BytesTotal"}              ? $perf_block->{"BytesTotal"}              : "";
        $data{$section}{$id}{used}               = defined $perf_block->{"BytesAllocated"}          ? $perf_block->{"BytesAllocated"}          : "";
        $data{$section}{$id}{free}               = defined $perf_block->{"BytesAvailable"}          ? $perf_block->{"BytesAvailable"}          : "";
        $data{$section}{$id}{resp_t_r}           = defined $perf_block->{"TotalReadsTime"}          ? $perf_block->{"TotalReadsTime"}          : "";
        $data{$section}{$id}{resp_t_w}           = defined $perf_block->{"TotalWritesTime"}         ? $perf_block->{"TotalWritesTime"}         : "";

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $name : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }

  ## PORT
  # $section = "port";
  $out = cmd("ports");

  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("ports");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/ServerScsiPortPerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  ###
  # if ( !isdigit($index) ) {
  #   push( @config_data, $out );
  # }
  # if ( $out eq "" ) {
  #   return;
  # }
  # print Dumper $out;
  # print Dumper @{$performance_data};
  foreach my $port ( @{$out} ) {
    my $section  = "";
    my $protocol = "";
    my $id    = $port->{Id};
    my $type  = $port->{PortType};
    my $mac   = $port->{PhysicalName};
    my $name  = $port->{Alias};
    my $port_name = $port->{PortName};
    # my $connect   = $host->{Connected};
    # my $mode      = $host->{PortMode};
    # my $host_id   = $host->{HostId};
    my $node      = $port->{ExtendedCaption};

    if ($node) {
      my @caption_values = split(" ", $node);
      my $last_caption = $caption_values[-1];
      if ($last_caption && grep { $data{'node'}{$_}{name} eq $last_caption } keys %{ $data{'node'} }) {
          $name = "$last_caption-$name";
      }
    }
    $name = $port_name // $id unless $name;

    if ( defined $id && $id ne "" && defined $mac && $mac ne "" ) {
      if ( defined $type && $type ne "" && $type !=0 ) { # skip 0 for unknown ports
        ( $protocol, $section ) = get_status( $type, "port" );
        # print "$protocol, $section\n";
        if ( !defined $data{$section}{$id}{id} || !defined $data{$section}{$id}{Protocol} ) {
          $data{$section}{$id}{id} = $id;
          $data{$section}{$id}{Protocol} = $protocol;
          $data{$section}{$id}{name} = $name;
          $data{$section}{$id}{wwn} = $port_name;
        }
      }

      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            push( @{$out}, $element->{PerformanceData} );
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{prefix}" . "$id";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }
      # print "DEBUG: Type: $type, Protocol: $protocol, Section: $section\n";
      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}                          = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}                         = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}                     = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}                       = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}                      = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}                       = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";
        $data{$section}{$id}{resp_t_r_initiator}            = defined $perf_block->{"InitiatorReadTime"}       ? $perf_block->{"InitiatorReadTime"}       : "";
        $data{$section}{$id}{resp_t_w_initiator}            = defined $perf_block->{"InitiatorWriteTime"}      ? $perf_block->{"InitiatorWriteTime"}      : "";
        $data{$section}{$id}{resp_t_r_target}               = defined $perf_block->{"TargetReadTime"}          ? $perf_block->{"TargetReadTime"}          : "";
        $data{$section}{$id}{resp_t_w_target}               = defined $perf_block->{"TargetWriteTime"}         ? $perf_block->{"TargetWriteTime"}         : "";
        $data{$section}{$id}{target_read}                   = defined $perf_block->{"TargetReads"}             ? $perf_block->{"TargetReads"}             : "";
        $data{$section}{$id}{target_write}                  = defined $perf_block->{"TargetWrites"}            ? $perf_block->{"TargetWrites"}            : "";
        $data{$section}{$id}{initiator_read}                = defined $perf_block->{"InitiatorReads"}          ? $perf_block->{"InitiatorReads"}          : "";
        $data{$section}{$id}{initiator_write}               = defined $perf_block->{"InitiatorWrites"}         ? $perf_block->{"InitiatorWrites"}         : "";

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $name : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }
  # print Dumper \%data;
  ## VOLUME FOLDER
  $section = "volume_folder";
  $out = cmd("virtualdiskgroups");

  $performance_data = "";
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$out} ) { last; }
      $out = cmd("virtualdiskgroups");
    }
    for ( my $i = 1; $i < $iteration; $i++ ) {
      $performance_data = cmd("performancebytype/VirtualDiskGroupPerformance");
      if ( $performance_data eq "" ) { 
        $cfg->{api_version} = 1; 
        last; 
      }
      sleep 1;
      if ( @{$performance_data} ) { last; }
    }
  }
  ###
  # if ( !isdigit($index) ) {
  #   push( @config_data, $out );
  # }
  # if ( $out eq "" ) {
  #   return;
  # }
  foreach my $vol_folder ( @{$out} ) {
    my $id    = $vol_folder->{Id};
    my $size  = $vol_folder->{TotalCapacity}->{Value};
    my $alias = $vol_folder->{Alias};
    if ( !defined $alias ) {
      $alias = "";
    }
    if ( defined $id && $id ne "" ) {
      my $url_id = $id;
      $id =~ s/\}//g;
      $id =~ s/\{//g;
      if ( !defined $data{$section}{$id}{id} ) {
        $data{$section}{$id}{id} = $id;
      }
      if ( Xormon::isdigit($size) ) {
        $data{$section}{$id}{'total'} = $size;
      }
      $data{$section}{$id}{name} = $alias;

      my $out = undef;
      if ( $cfg->{api_version} == 2 && @{$performance_data} ) {
        my $index = -1;
        foreach my $element ( @{$performance_data} ) {
          $index++;
          my $object_id = $element->{ObjectId};
          if ( defined $object_id ) {
            $object_id =~ s/\}//g;
            $object_id =~ s/\{//g;
          }
          if ( defined $object_id && $id eq $object_id && defined $element->{PerformanceData} ) {
            push( @{$out}, $element->{PerformanceData} );

            #$out = $element->{PerformanceData};
            delete $performance_data->[$index];
            last;
          }
        }
      }
      else {
        my $url = "$cfg->{'prefix'}" . "$url_id";
        $out = cmd($url);
      }
      if ( !defined $out || $out eq "" ) {
        next;
      }

      foreach my $perf_block ( @{$out} ) {
        $data{$section}{$id}{read}                          = defined $perf_block->{"TotalBytesRead"}          ? $perf_block->{"TotalBytesRead"}          : "";
        $data{$section}{$id}{write}                         = defined $perf_block->{"TotalBytesWritten"}       ? $perf_block->{"TotalBytesWritten"}       : "";
        $data{$section}{$id}{data_rate}                     = defined $perf_block->{"TotalBytesTransferred"}   ? $perf_block->{"TotalBytesTransferred"}   : "";
        $data{$section}{$id}{read_io}                       = defined $perf_block->{"TotalReads"}              ? $perf_block->{"TotalReads"}              : "";
        $data{$section}{$id}{write_io}                      = defined $perf_block->{"TotalWrites"}             ? $perf_block->{"TotalWrites"}             : "";
        $data{$section}{$id}{io_rate}                       = defined $perf_block->{"TotalOperations"}         ? $perf_block->{"TotalOperations"}         : "";
        $data{$section}{$id}{cache_r_hit}                   = defined $perf_block->{"CacheReadHits"}           ? $perf_block->{"CacheReadHits"}           : "";
        $data{$section}{$id}{cache_w_hit}                   = defined $perf_block->{"CacheWriteHits"}          ? $perf_block->{"CacheWriteHits"}          : "";
        $data{$section}{$id}{cache_r_hit_bytes}             = defined $perf_block->{"CacheReadHitBytes"}       ? $perf_block->{"CacheReadHitBytes"}       : "";
        $data{$section}{$id}{cache_w_hit_bytes}             = defined $perf_block->{"CacheWriteHitBytes"}      ? $perf_block->{"CacheWriteHitBytes"}      : "";
        $data{$section}{$id}{cache_r_miss}                  = defined $perf_block->{"CacheReadMisses"}         ? $perf_block->{"CacheReadMisses"}         : "";
        $data{$section}{$id}{cache_w_miss}                  = defined $perf_block->{"CacheWriteMisses"}        ? $perf_block->{"CacheWriteMisses"}        : "";
        $data{$section}{$id}{cache_r_miss_bytes}            = defined $perf_block->{"CacheReadMissBytes"}      ? $perf_block->{"CacheReadMissBytes"}      : "";
        $data{$section}{$id}{cache_w_miss_bytes}            = defined $perf_block->{"CacheWriteMissBytes"}     ? $perf_block->{"CacheWriteMissBytes"}     : "";

        my $sample_rate = $perf_block->{"CollectionTime"};
        $sample_rate =~ s/\D//g;
        if ( Xormon::isdigit($sample_rate) ) {
          $sample_rate = int( $sample_rate / 1000 );
        }
        # print "sample_rate: $sample_rate\n";
        ### sometimes bad timestamp so it must be checked
        my $sample_rate_size = length($sample_rate);
        if ( $sample_rate_size != 10 ) {
          $logger->error("Wrong data sample: $section : $alias : $id : $sample_rate\n");
          next;
        }
        $data{$section}{$id}{timestamp} = $sample_rate;
      }
    }
  }

  # set_data_resp_volume
}



sub set_status_map {
  my %status_map = (
    "volume" => {
      "0" => [ "Online", "hs_good" ],
      "1" => [ "Offline", "hs_warning" ],
      "2" => [ "Failed Redundancy", "hs_error" ],
      "3" => [ "Failed", "hs_error" ],
      "4" => [ "Unknown", "hs_unknown" ],
    },
    "node" => {
      "0" => [ "Unknown", "hs_unknown" ],
      "1" => [ "ACOffline", "hs_warning" ],
      "2" => [ "ACOnline", "hs_good" ],
      "3" => [ "BatteryLow", "hs_warning" ],
    },
    "host" => {
      "0" => [ "Unknown", "hs_unknown" ],
      "1" => [ "Not connected", "hs_warning" ],
      "2" => [ "Connected", "hs_good" ],
    },
    "drive" => {
      "0" => [ "Unknown", "hs_unknown" ],
      "1" => [ "Online", "hs_good" ],
      "2" => [ "NotReady", "hs_warning" ],
      "3" => [ "NoMedia", "hs_good" ],
      "4" => [ "Offline", "hs_error" ],
      "5" => [ "Failed", "hs_error" ],
      "6" => [ "Missing", "hs_good" ],
    },
    "pool" => {
      "0" => [ "Running", "hs_good" ],
      "1" => [ "Initializing", "hs_good" ],
      "4" => [ "MissingDisks", "hs_error" ],
      "5" => [ "Foreign", "hs_warning" ],
      "6" => [ "Removing", "hs_warning" ],
      "7" => [ "Offline", "hs_error" ],
      "8" => [ "Unknown", "hs_unknown" ],
      "9" => [ "FailedRedundancy", "hs_error" ],
      "10" => [ "Suspended", "hs_error" ],
    },
    "port" => {
      "0" => [ "Unknown", "unknown" ],
      "1" => [ "SCSI", "scsiport" ],
      "2" => [ "FC", "fcport" ],
      "3" => [ "iSCSI", "iscsiport" ],
      "4" => [ "Loopback", "loopbackport" ],
    },
  );
  return \%status_map;
}

sub get_status {
  my ($status, $type, $only_first) = @_;
  if (exists $status_map->{$type} && exists $status_map->{$type}{$status}) {
      my ($state, $health_status) = @{$status_map->{$type}{$status}};
      return $only_first ? $state : ($state, $health_status);
  }
  return $only_first ? "Unknown type or status" : ("Unknown type or status", "unknown");
}

sub get_api_version {
  my $pom = cmd_test("servers");
  if ( $pom == 1 ) {
    $pom = cmd_test("1.0/servers");
    if ( $pom == 0 ) {
      $cfg->{new_api} = 1;
    }
  }
  else {
    $cfg->{new_api} = 0;
  }
  my $servers = cmd("servers");
  if ( defined $servers && ref($servers) eq "ARRAY" && @{$servers} ) {
    push( @ips_servers, "$cfg->{'nodeHost'} :stor2rrd: $cfg->{'proxyHost'}" );
    if ( $cfg->{'nodeHost'} ne $cfg->{'proxyHost'} ) {
      push( @ips_servers, "$cfg->{'nodeHost'} :stor2rrd: $cfg->{'nodeHost'}" );
      push( @ips_servers, "$cfg->{'proxyHost'} :stor2rrd: $cfg->{'proxyHost'}" );
    }
    foreach my $server ( @{$servers} ) {
      my $ips = $server->{IpAddresses};
      if ( defined $ips && ref($ips) eq "ARRAY" ) {
        foreach my $ip_address ( @{$ips} ) {
          if ( grep { "$ip_address :stor2rrd: $cfg->{'proxyHost'}" eq $_ } @ips_servers ) {
            next;
          }
          push( @ips_servers, "$ip_address :stor2rrd: $cfg->{'proxyHost'}" );
          if ( $ip_address ne $cfg->{'proxyHost'} ) {
            push( @ips_servers, "$ip_address :stor2rrd: $ip_address" );
          }
        }
      }
    }
  }
  ###
  $pom = cmd_test("servers");
  if ( defined $pom ) {
    $cfg->{new_api} = $pom;
  }

  if ( $cfg->{new_api} == 1 ) {
    $cfg->{prefix} = "performance/";
  }
  my $connected = 0;
  #### test rest api 2.0
  if ( $cfg->{new_api} == 1 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      my $nodes = cmd("servers");
      # print Dumper $nodes;
      # if ( @{$nodes} ) {
      if ( ref($nodes) eq 'ARRAY' ) {
        foreach my $node ( @{$nodes} ) {
          my $product_version = $node->{ProductVersion};
          if (defined $product_version && $product_version ne "") {
            $conf{$cfg->{hostcfg_id}}{version} = $product_version;
            $connected = 1;
          }
          if ( defined $node->{ProductName} && $node->{ProductName} ne "" ) {
            $conf{$cfg->{hostcfg_id}}{model} = $node->{ProductName};
          }
          if ( defined $product_version && $product_version =~ m/^10/ ) {
            my ( $version, undef ) = split( ",", $product_version );
            my @versions = split( " ", $version );
            if ( Xormon::isdigit( $versions[-1] ) && $versions[-1] >= 9 ) {
              $cfg->{api_version} = 2;
              last;
            }
          }
        }
        last;
      }
    }
  }
  $xormon->isConnected($connected);
  # print Dumper $cfg;
}

sub connection_test {
  my $client = shift;
  $cfg       = shift;

  if ( exists $cfg->{'ssl_cipher_list'} && $cfg->{'ssl_cipher_list'} ne "" ) {
    $ssl_cipher_list = $cfg->{'ssl_cipher_list'};
  }

  my $api_version        = 1;
  my @nodes              = ();
  my @ips_servers        = ();

  $logger    = XormonLogger->new( 'datacore', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  my $test   = Test->new($client);

  #
  # test main variables
  #
  my @variables = ("hostcfg_id", "hostalias", "nodeHost", "proto", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Device cfg", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Device cfg", 1 , "");

  # if proxyHost not defined, use nodeHost for the proxyHost key
  if ( !exists $cfg->{'proxyHost'} || $cfg->{'proxyHost'} eq '' ) {
    $cfg->{'proxyHost'} = $cfg->{'nodeHost'};
  }
  # save port to variable for use in API calls (80 or 443)
  if ( $cfg->{'proto'} eq "http" ) {
    $cfg->{'apiPort'} = 80;
  }
  elsif ( $cfg->{'proto'} eq "https" ) {
    $cfg->{'apiPort'} = 443;
  }
  # test IP and port
  my ( $code, $text ) = Xormon::conntest_tcp($cfg->{'proxyHost'},$cfg->{'apiPort'});
  $test->addSection("API host network connection", $code, $text);
  if ( $code == 0 ) { return $test->stop(); }

  # print Dumper $cfg;

  my $pom = cmd_test("servers");
  if ( $pom == 1 ) {
    $pom = cmd_test("1.0/servers");
    if ( $pom == 0 ) {
      $api_version = 1;
    }
  }
  else {
    $api_version = 0;
  }
  my $servers = cmd( "servers", 1 );
  if ( $servers eq "" ) {
    $servers = cmd( "1.0/servers", 2 );
  }
  # print Dumper $servers;


  # rework IPs check
  # if ( defined $servers && ref($servers) eq "ARRAY" && @{$servers} ) {
  #   push( @ips_servers, "$cfg->{'nodeHost'} :stor2rrd: $cfg->{'proxyHost'}" );
  #   foreach my $server ( @{$servers} ) {
  #     my $ips = $server->{IpAddresses};
  #     if ( defined $ips && ref($ips) eq "ARRAY" ) {
  #       foreach my $ip_address ( @{$ips} ) {
  #         if ( grep { "$cfg->{'nodeHost'} :stor2rrd: $cfg->{'proxyHost'}" eq $_ } @ips_servers ) {
  #           next;
  #         }
  #         push( @ips_servers, "$cfg->{'nodeHost'} :stor2rrd: $cfg->{'proxyHost'}" );
  #       }
  #     }
  #   }
  # }

  my $data    = cmd( "servers", 1 );
  my $version = 0;
  if ( $data eq "" ) {
    my $second_test = cmd( "1.0/servers", 2 );
    my $url         = "1.0";

    #print Dumper \$second_test;
    if ( ref($second_test) eq 'ARRAY' ) {
      foreach my $node ( @{$second_test} ) {
        my $product_version = $node->{ProductVersion};
        if ( defined $product_version && $product_version =~ m/^10/ ) {
          my ( $version, undef ) = split( ",", $product_version );
          my @versions = split( " ", $version );
          if ( Xormon::isdigit( $versions[-1] ) && $versions[-1] >= 9 && $url =~ m/1.0/ ) {
            $api_version = 2;
            last;
          }
        }
      }
    }
    if ( $second_test eq "" ) {
      if ( $cfg->{'nodeHost'} ne $cfg->{'proxyHost'} ) {
        $logger->error("connection to \"$cfg->{'proxyHost'}\" has failed!");
        $test->addSection("API authorization", 0 , "Connection to $cfg->{'proxyHost'} failed!");
      }
      else {
        $logger->error("connection to \"$cfg->{'nodeHost'}\" has failed!");
        $test->addSection("API authorization", 0 , "Connection to $cfg->{'nodeHost'} failed!");
      }
      return $test->stop();
    }
    else {
      $test->addSection("API authorization", 1);
      $version = 1;
    }
  }

  my $cmd             = "virtualdisks";
  my $prefix_perf_url = "performance?id=";
  if ( $version == 1 ) {
    $cmd             = "1.0/virtualdisks";
    $prefix_perf_url = "1.0/performance/";
  }
  $data = cmd( $cmd, 2 );

  #print Dumper \$data;
  my $vol_perf = "";

  if ( $api_version == 1 ) {

    foreach my $host ( @{$data} ) {
      my $id = $host->{Id};
      if ( !defined $id || $id eq "" ) { next; }
      my $url = "$prefix_perf_url" . "$id";
      my $out = cmd( $url, 2 );
      if ( $out eq "" ) { next; }
      $vol_perf = "1";

      print Dumper \$out;
    }
    if ( $vol_perf eq "" ) {
      $test->addSection("Performance data", 0);
    }
    else {
      $test->addSection("Performance data", 1);
    }
  }
  else {
    my $data = "";
    for ( my $i = 1; $i < 4; $i++ ) {
      $data = cmd( "1.0/performancebytype/VirtualDiskPerformance", 2 );
      sleep 1;
      if ( @{$data} ) { last; }
    }
    if ( $data ne "" ) {
      $test->addSection("Performance data", 1,);
    }
    else {
      $test->addSection("Performance data", 0);
    }
  }
  return $test->stop();
}

sub remove_whitespace {
  my $value = shift;
  if ( !defined $value || $value eq "" ) {
    return "";
  }
  $value =~ s/^\s+|\s+$//g;
  return $value;
}

sub cmd_test {
  ## sometimes we get error 'Recv failure: Connection reset by peer' due to old ssl_ciphers, so we use variable in advanced settings/magic to change it to:
  ## $ssl_cipher_list => 'DEFAULT:!DH:@SECLEVEL=0'

  my $element = shift;
  my $url     = "$cfg->{proto}://$cfg->{proxyHost}/RestService/rest.svc/$element";
  my $ua      = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => $ssl_cipher_list, verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req     = HTTP::Request->new( GET => $url );
  $req->header( 'Content-Type'  => 'application/json' );
  $req->header( 'Accept'        => 'application/json' );
  $req->header( 'Connection'    => 'keep-alive' );
  $req->header( 'ServerHost'    => $cfg->{'nodeHost'} );
  $req->header( 'Authorization' => "Basic $cfg->{apiUsername} $cfg->{apiPassword}" );
  my $res = $ua->request($req);

  if ( !$res->is_success ) {
    #error("ERROR: Command $url failed!");
    #error( $res->content );
    #return "";
    return 1;
  }
  else {
    ### for debug
    # if ( $element !~ m/performance/ ) {
      #my $file = "$debug_data/$element.txt";
      #store \$res->content, $file or error("Store data to file $file failed.");
    # }
    ###
    #my $data = decode_json( $res->content );
    #return $data;
    return 0;
  }
}

sub cmd {
  my $element       = shift;
  # my $element_save = $element;
  # $element_save =~ s/\//-/g;
  # my $file = "$configuration_data/$element.json";
  # my $data;
  # if ( -f $file ) {
  #   eval { $data = Xorux_lib::read_json($file); };
  #   if ($@) {
  #     error("Read json file $file failed");
  #   }
  #   else {
  #     if ( ref($data) eq "ARRAY" ) {
  #       if ( @{$data} ) {
  #         return $data;
  #       }
  #     }
  #     if ( ref($data) eq "HASH" ) {
  #       if ( keys %{$data} ) {
  #         return $data;
  #       }
  #     }
  #   }
  # }

  # $cmd_count++;

  ## sometimes we get error 'Recv failure: Connection reset by peer' due to old ssl_ciphers, so we use variable for debug to change it to:
  ## SSL_cipher_list => 'DEFAULT:!DH:@SECLEVEL=0'
  my $url = "$cfg->{proto}://$cfg->{proxyHost}/RestService/rest.svc/$element";
  if ( exists $cfg->{new_api} && $cfg->{new_api} eq "1" ) {
    $url = "$cfg->{proto}://$cfg->{proxyHost}/RestService/rest.svc/1.0/$element";
  }
  # if ($datacore_debug) { print "DEBUG: $$ START API cmd $url " . localtime() . " " . time() . "\n"; }
  $logger->info("url: $url");
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => $ssl_cipher_list, verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( GET => $url );
  $req->header( 'Content-Type'  => 'application/json' );
  $req->header( 'Accept'        => 'application/json' );
  $req->header( 'Connection'    => 'keep-alive' );
  $req->header( 'ServerHost'    => $cfg->{'nodeHost'} );
  $req->header( 'Authorization' => "Basic $cfg->{apiUsername} $cfg->{apiPassword}" );
  my $res = $ua->request($req);
  # if ($datacore_debug) { print "DEBUG: $$ END API cmd $url   " . localtime() . " " . time() . "\n"; }

  if ( !$res->is_success ) {
    $logger->error("ERROR: Command $url failed!");
    $logger->error( $res->content );
    return "";
  }
  else {
    my $data = decode_json( $res->content );
    return $data;
  }
}

sub calculate_other_metrics {
  my ($stats_out) = @_;

  my @block_size_metrics = (
    { io_metric => 'io_rate',  data_metric => 'data_rate', block_size_metric => 'block_size' },
    { io_metric => 'read_io',  data_metric => 'read',      block_size_metric => 'block_size_r' },
    { io_metric => 'write_io', data_metric => 'write',     block_size_metric => 'block_size_w' },
  );
  my @cache_metrics = (
    { io_metric => 'read_io',  hit_metric => 'cache_r_hit',   cache_metric => 'r_cache_hit' },
    { io_metric => 'write_io', hit_metric => 'cache_w_hit',   cache_metric => 'w_cache_hit' },
    { io_metric => 'read_io',  hit_metric => 'cache_r_miss',  cache_metric => 'r_cache_miss' },
    { io_metric => 'write_io', hit_metric => 'cache_w_miss',  cache_metric => 'w_cache_miss' },
  );
  my @port_resp_metrics = (
    { final => 'resp_t_r', target_resp => 'resp_t_r_target',  target_io => 'target_read',  init_resp => 'resp_t_r_initiator', init_io => 'initiator_read' },
    { final => 'resp_t_w', target_resp => 'resp_t_w_target',  target_io => 'target_write', init_resp => 'resp_t_w_initiator', init_io => 'initiator_write' },
  );
  my @resp_metrics = (
    { resp_metric => 'resp_t_r',  io_metric => 'read_io' },
    { resp_metric => 'resp_t_w',  io_metric => 'write_io' },
  );
  # print Dumper \%{ $stats_out->{datacore} };

  foreach my $subsystem (keys %{ $stats_out->{datacore} }) {
    next if $subsystem eq "device";
    foreach my $hash_key (keys %{ $stats_out->{datacore}{$subsystem} }) {
      foreach my $act_time (keys %{ $stats_out->{datacore}{$subsystem}{$hash_key} }) {
        ## block size
        foreach my $metric (@block_size_metrics) {
          my ($io_metric, $data_metric, $block_size_metric) = @{$metric}{qw(io_metric data_metric block_size_metric)};
          if (Xormon::isdigit($stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$io_metric}) && Xormon::isdigit($stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$data_metric})) {
            my $io = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$io_metric};
            my $data = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$data_metric};
            if ($io != 0) {
              $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$block_size_metric} = 
                Xormon::formatNumberByType("float4", ($data / $io));
            } else {
              $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$block_size_metric} = 0;
            }
          }
        }
        ## cache
        foreach my $metric (@cache_metrics) {
          my ($io_metric, $hit_metric, $cache_metric) = @{$metric}{qw(io_metric hit_metric cache_metric)};
          if (Xormon::isdigit($stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$io_metric}) && Xormon::isdigit($stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$hit_metric})) {
            my $io = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$io_metric};
            my $cache = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$hit_metric};
            if ($io != 0) {
              $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$cache_metric} = 
              Xormon::formatNumberByType("float4", ($cache / ( $io / 100 ) ) );
            } else {
              $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$cache_metric} = 0;
            }
          }
        }
        ## calculate resp_t_r and resp_t_w from target and initiator for ports subsystems
        foreach my $metric (@port_resp_metrics) {
          my ($final, $target_resp, $target_io, $init_resp, $init_io) = @{$metric}{qw(final target_resp target_io init_resp init_io)};
          my $target_read         = defined $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$target_io}   ? $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$target_io}   : "";
          my $initiator_read      = defined $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$init_io}     ? $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$init_io}     : "";
          my $initiator_time_read = defined $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$init_resp}   ? $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$init_resp}   : "";
          my $target_time_read    = defined $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$target_resp} ? $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$target_resp} : "";
          my $numerator           = "";
          my $denominator         = "";
          if ( Xormon::isdigit($target_read) ) {
            $denominator = $target_read;
            if ( Xormon::isdigit($target_time_read) ) {
              $numerator = $target_time_read * $target_read;
            }
          }
          if ( Xormon::isdigit($initiator_read) ) {
            if ( Xormon::isdigit($denominator) ) {
              $denominator = $denominator + $initiator_read;
            }
            else {
              $denominator = $initiator_read;
            }
            if ( Xormon::isdigit($initiator_time_read) ) {
              if ( Xormon::isdigit($numerator) ) {
                $numerator = $numerator + ( $initiator_read * $initiator_time_read );
              }
              else {
                $numerator = $initiator_read * $initiator_time_read;
              }
            }
          }
          if ( Xormon::isdigit($denominator) && $denominator != 0 ) {
            my $pom = $numerator / $denominator;
            $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$final} = Xormon::formatNumberByType( "float4", $numerator / $denominator );
          }
        }
        ## resp_t
        foreach my $metric (@resp_metrics) {
          my ($resp_metric, $io_metric) = @{$metric}{qw(resp_metric io_metric)};
          if (Xormon::isdigit($stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$io_metric}) && Xormon::isdigit($stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$resp_metric})) {
            my $io = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$io_metric};
            my $resp = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$resp_metric};
            if ($resp == 0 || $io == 0) {
              $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$resp_metric} = 0;
            } else {
              $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$resp_metric} = Xormon::formatNumberByType("float4", ( $resp / $io ) );
            }
          }
          else {
            $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{$resp_metric} = "";
          }
        }
        ## total resp_t
        if ( Xormon::isdigit( $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_r} ) && 
          Xormon::isdigit( $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{read_io} ) && 
          Xormon::isdigit( $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_w} ) && 
          Xormon::isdigit( $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{write_io} ) ) {

          my $resp_t_r = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_r};
          my $resp_t_w = $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t_w};

          if ($resp_t_r == 0 && $resp_t_w == 0) {
            $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t} = 0;
          } else {
            my $resp_t_help = Xormon::calculate_weighted_avg( [
              { count => $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{read_io}, value => $resp_t_r },
              { count => $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{write_io}, value => $resp_t_w }
            ] );
            if (defined $resp_t_help) {
              $stats_out->{datacore}{$subsystem}{$hash_key}{$act_time}{resp_t} = Xormon::formatNumberByType("float4", $resp_t_help);
            }
          }
        }
      }
    }
  }
  ### DEVICE STATS
  foreach my $pool_for_device (keys %{ $stats_out->{datacore}{pool} }) {
    foreach my $dev_time (keys %{ $stats_out->{datacore}{pool}{$pool_for_device} }) {
      foreach my $dev_metric (keys %{ $stats_out->{datacore}{pool}{$pool_for_device}{$dev_time} }) {
        if ( $dev_metric eq "capacity_used_percent" || $dev_metric eq "resp_t" ) { next; } ## skip sum of percent, it calculates later
          if (!exists $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$dev_time}{$dev_metric}) {
            $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$dev_time}{$dev_metric} = 0;
          }
          if ($dev_metric eq "resp_t_r" || $dev_metric eq "resp_t_w" ) {
            my $io_metric = $dev_metric eq "resp_t_r" ? "read_io" : "write_io";
            if (Xormon::isdigit($stats_out->{datacore}{pool}{$pool_for_device}{$dev_time}{$io_metric})) {
              $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$dev_time}{$dev_metric} += $stats_out->{datacore}{pool}{$pool_for_device}{$dev_time}{$io_metric} * $stats_out->{datacore}{pool}{$pool_for_device}{$act_time}{$dev_metric};
            }
            next;
          }
        $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$dev_time}{$dev_metric} += $stats_out->{datacore}{pool}{$pool_for_device}{$dev_time}{$dev_metric};
      }
    }
  }
  if (Xormon::isdigit($stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{'capacity_total'}) && $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{'capacity_total'} !=0 && Xormon::isdigit($stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{'capacity_used'}) ) {
    my $pom = Xormon::formatNumberByType("float4", $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{'capacity_used'} / ($stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{'capacity_total'} / 100) );
    $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{'capacity_used_percent'} = $pom;
  }

  ## resp_t
  foreach my $metric (@resp_metrics) {
    my ($resp_metric, $io_metric) = @{$metric}{qw(resp_metric io_metric)};
    if (Xormon::isdigit($stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{$io_metric}) && Xormon::isdigit($stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{$resp_metric})) {
      my $io = $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{$io_metric};
      my $resp = $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{$resp_metric};
      if ($resp == 0 || $io == 0) {
        $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{$resp_metric} = 0;
      } else {
        $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{$resp_metric} = Xormon::formatNumberByType("float4", ( $resp / $io ) );
      }
    }
    else {
      $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{$resp_metric} = "";
    }
  }
  # RESP_T TOTAL
  if ( Xormon::isdigit( $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{resp_t_r} ) && 
  Xormon::isdigit( $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{read_io} ) && 
  Xormon::isdigit( $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{resp_t_w} ) && 
  Xormon::isdigit( $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{write_io} ) ) {

    my $resp_t_r = $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{resp_t_r};
    my $resp_t_w = $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{resp_t_w};

    if ($resp_t_r == 0 && $resp_t_w == 0) {
      $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{resp_t} = 0;
    } else {
      my $resp_t_help = Xormon::calculate_weighted_avg( [
        { count => $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{read_io}, value => $resp_t_r },
        { count => $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{write_io}, value => $resp_t_w }
      ] );
      if (defined $resp_t_help) {
        $stats_out->{datacore}{device}{$cfg->{hostcfg_id}}{$act_time}{resp_t} = Xormon::formatNumberByType("float4", $resp_t_help);
      }
    }
  }
}

sub make_gauge {
  my ($current_timestamp, $last_timestamp, $counter_now, $counter_last) = @_;
  # Calculate timediff from the timestamps
  my $timediff = $current_timestamp - $last_timestamp;
  if (Xormon::isdigit($timediff) && Xormon::isdigit($counter_now) && Xormon::isdigit($counter_last) && $timediff > 0) {
    if ($counter_now > $counter_last) {
      return int(($counter_now - $counter_last) / $timediff);
    } elsif ($counter_now < $counter_last) {
      # probably counter overflow
      return "";
    } else {
      return 0;
    }
  }
  return "";
}

sub send_conf {
  my @subsystems = qw(node pool volume volume_folder drive host host_group fcport iscsiport scsiport loopbackport); # add all perf subsystems
  my @architecture;

  my %device_arch = (
    "item_id" => $cfg->{'hostcfg_id'},
    "hostcfg_id" => $cfg->{'hostcfg_id'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => "datacore",
    "subsystem" => "device",
    "parents" => [],
  );
  push(@architecture, \%device_arch);

  foreach my $subsystem (@subsystems) {
    if (defined $data{$subsystem} && $data{$subsystem} ne "") {
      foreach my $element (keys %{ $data{$subsystem} }) {
        if (defined $data{$subsystem}{$element}{'id'} && $data{$subsystem}{$element}{'id'} ne "") {
          my $id = $data{$subsystem}{$element}{'id'};
          # if ( $subsystem eq "pool" && exists $pool_check{$id}) {next;} # skip duplicate pools
          my $name = $element;
          my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . $subsystem . $id);
          $hash_key = lc($hash_key);
          if (defined $data{$subsystem}{$element}{'name'} && $data{$subsystem}{$element}{'name'} ne '') {
            $name = $data{$subsystem}{$element}{'name'};
          }
          my %arch = (
            "item_id"    => $hash_key,
            "label"      => $name,
            "hw_type"    => "datacore",
            "subsystem"  => $subsystem,
            "hostcfg_id" => "$cfg->{'hostcfg_id'}",
            "parents"    => [$cfg->{'hostcfg_id'}]
          );
          # add lun_id for volumes
          if ( $subsystem eq "volume" ) {
            $conf{$hash_key}{lun_id} = $id;
          }
          # add pool mapping for volumes
          if ( $subsystem eq "volume" && defined $data{$subsystem}{$element}{pool_ids} && ref( $data{$subsystem}{$element}{pool_ids} ) eq "HASH" ) {
            foreach my $pool_id( keys %{ $data{$subsystem}{$element}{pool_ids} } ) {
              my $pool_hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "pool" . $pool_id);
              push( @{ $arch{parents} }, $pool_hash_key );
            }
          }
          # add host mapping for volumes
          if ( $subsystem eq "host" && defined $data{$subsystem}{$element}{vol_ids} && ref( $data{$subsystem}{$element}{vol_ids} ) eq "HASH" ) {
            foreach my $vol_id( keys %{ $data{$subsystem}{$element}{vol_ids} } ) {
              my $vol_hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "volume" . $vol_id);
              push( @{ $arch{children} }, $vol_hash_key );
            }
          }
          # host_group for host
          if ( $subsystem eq "host" && defined $data{$subsystem}{$element}{host_group} ) {
            my $hg_id = $data{$subsystem}{$element}{host_group};
            my $hg_hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "host_group" . $hg_id);
            push( @{ $arch{parents} }, $hg_hash_key );
          }
          # wwn for ports
          if ( $subsystem =~ /^(scsiport|iscsiport|fcport|loopbackport)$/ && defined $data{$subsystem}{$element}{wwn} ) {
            my $wwn = Xormon::formatWWN($data{$subsystem}{$element}{wwn});
            $conf{$hash_key}{wwn} = $wwn;
            push( @{ $arch{wwn} }, $wwn );
          }
          push(@architecture, \%arch);
        }
      }
    }
  }
  
  #save configuration
  my $rc_conf = $xormon->saveConfJSON(\%conf);
  $logger->info("save conf: $rc_conf");
  #save architecture
  # print Dumper @architecture;
  my $rc_arch = $xormon->saveArchitecture(\@architecture);
  $logger->info("save architecture: $rc_arch");
}

sub health_status {
  my $report         = "";
  my $error_health   = 0;
  my $warning_health = 0;
  my @error_message  = ();
  my %hs_data;

  my $alerts = cmd("alerts");
  if ( $cfg->{api_version} == 2 ) {
    for ( my $i = 1; $i < $iteration; $i++ ) {
      if ( @{$alerts} || ( ref($alerts) eq "ARRAY" && $iteration > 2 ) ) { last; }
      $alerts = cmd("alerts");
    }
  }
  if ( ref($alerts) ne "ARRAY" ) {
    $alerts = ();
  }

  foreach my $element ( @{$alerts} ) {
    my $timestamp        = $element->{TimeStamp};
    my $id               = $element->{Id}->{SequenceNumber};
    my $machine_name     = defined $element->{MachineName}                  ? $element->{MachineName}                               : "";        
    my $level            = defined $element->{Level}                        ? get_level($element->{Level})                          : "";
    my $need_acknowledge = defined $element->{NeedsAcknowledge}             ? $element->{NeedsAcknowledge}                          : "";
    my $message_text     = defined $element->{MessageText}                  ? $element->{MessageText}                               : "";
    my $priority         = defined $element->{HighPriority}                 ? $element->{HighPriority}                              : "";
    my $source_type      = defined $element->{Sources}->[0]->{"SourceType"} ? get_level($element->{"Sources"}->[0]->{"SourceType"}, "SOURCE") : "";
    my $source           = defined $element->{"Sources"}->[0]->{"SourceCaption"} ? $element->{"Sources"}->[0]->{"SourceCaption"}              : "";
    if ( !defined $id || $id eq "" ) { next; }

    if ( defined $timestamp && $timestamp ne "" ) {
      $timestamp =~ s/\D//g;
      if ( Xormon::isdigit($timestamp) ) {
        $timestamp = int( $timestamp / 1000 );
      }
      else { $timestamp = ""; }
    }
    else {
      $timestamp = "";
    }
    
    if ( $level eq "Error" && ( $need_acknowledge eq "false" || $need_acknowledge eq "0" ) ) {
      $error_health++;
      my $line = "$timestamp  $id  $level  $machine_name  $source_type  $source  $message_text";
      push( @error_message, "$line\n" );
    }

    my %alert;
    $alert{id}           = $id;
    $alert{time}         = 0 + $timestamp;
    $alert{level}        = $level;
    $alert{machine_name} = $machine_name;
    $alert{type}         = $source_type;
    $alert{source}       = $source;
    $alert{message} =      $message_text;
    push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{alerts} }, \%alert );
  }

  foreach my $subsystem ( keys %data ) {
    if ($subsystem eq "ms_timestamp"){ next; } # skip ms_timestamp for service data, its not subsystem
    foreach my $element ( keys %{ $data{$subsystem} } ) {
      if ( defined $data{$subsystem}{$element}{hs_status} && $data{$subsystem}{$element}{hs_status} ne "" ) {
        my $hs     = $data{$subsystem}{$element}{hs_status};
        my $name   = defined $data{$subsystem}{$element}{name}   ? $data{$subsystem}{$element}{name}   : "";
        my $status = defined $data{$subsystem}{$element}{status} ? $data{$subsystem}{$element}{status} : "";
        if ( $hs eq "hs_error" ) {
          $error_health++;
          my $line = "$subsystem  $name  $status";
          push( @error_message, "$line\n" );
        }
        if ( $hs eq "hs_warning" ) {
          $warning_health++;
          my $line = "$subsystem  $name  $status";
          push( @error_message, "$line\n" );
        }
        my %component;
        $component{section} = $subsystem;
        $component{name}    = $name;
        $component{status}  = $status;
        # $component{hs}      = $hs;
        push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{components} }, \%component );
      }
    }
  }

  #'MonitoredObjectId' => 'EB19D2CE-6D18-49D2-A8BB-A8697842E93B',
  #'Internal' => bless( do{\(my $o = 0)}, 'JSON::backportPP::Boolean' ),
  #'TimeStamp' => '/Date(1520258740296+0100)/',
  #'Id' => 'ServerMachinePowerStateMonitor<EB19D2CE-6D18-49D2-A8BB-A8697842E93B>',
  #'State' => 2,
  #'TemplateId' => 'T(DataCore.Executive.Controller.ServerMachinePowerStateMonitor<DataCore.Executive.Controller.ServerMachine>)',
  #'Caption' => 'Battery Power State',
  #'SequenceNumber' => 4616289,
  #'MessageText' => 'AC line power available',
  #'Description' => 'Monitors the battery power state for DataCore Servers.',
  #'ExtendedCaption' => 'Battery Power State of WS2012R2-1 in Server Group'

  ## Monitors
  my $monitors = cmd("monitors");
  # print Dumper $monitors;
  if ( ref($monitors) ne "ARRAY" ) {
    $monitors = ();
  }

  foreach my $element ( @{$monitors} ) {
    my $timestamp    = $element->{TimeStamp};
    my $id           = $element->{SequenceNumber};
    my $caption      = defined $element->{Caption}         ? $element->{Caption}                       : "";
    my $level        = defined $element->{State}           ? get_level( $element->{State}, "MONITOR" ) : "";
    my $message      = defined $element->{MessageText}     ? $element->{MessageText}                   : "";
    my $description  = defined $element->{ExtendedCaption} ? $element->{ExtendedCaption}               : "";
    my $tz           = "";

    if ( !defined $id || $id eq "" ) { next; }
    if ( defined $timestamp && $timestamp ne "" ) {
      $timestamp =~ s/\/Date\(//g;
      $timestamp =~ s/\)\///g;
      if ( $timestamp =~ m/\+/ ) {
        ( my $t, $tz ) = split( /\+/, $timestamp );
        $timestamp = $t;
        $tz        = "+$tz";
      }
      elsif ( $timestamp =~ m/-/ ) {
        ( my $t, $tz ) = split( "-", $timestamp );
        $timestamp = $t;
        $tz        = "-$tz";
      }
      else {
        $tz = "$tz";
      }
      if ( Xormon::isdigit($timestamp) ) {
        $timestamp = int( $timestamp / 1000 );
      }
      else { $timestamp = ""; }
    }
    else {
      $timestamp = "";
    }

    if ( $level eq "Critical" ) {
      $error_health++;
      my $line = "$timestamp  $id  $level  $caption  $caption  $message  $description";
      push( @error_message, "$line\n" );
    }
    my %monitor;
    $monitor{time}        = 0 + $timestamp;
    $monitor{id}          = $id;
    $monitor{level}       = $level;
    $monitor{caption}     = $caption;
    $monitor{message}     = $message;
    $monitor{description} = $description;

    push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{monitors} }, \%monitor );
  }

  if ( $error_health == 0 ) {
    if ( $warning_health == 0 ) {
      $report = "ok";
    }
    else {
      $report = "warning";
    }
  }
  else {
    $report = "error";
  }
  
  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  # print Dumper \%hs_data;

  my @hs = (
    {
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $report,
      "data" => join("\n", @error_message),
    }
  );

  # print Dumper \@hs;

  my $rc = $xormon->saveStatus(\@hs);
  $logger->info("save health status: $rc");
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);
  $logger->info("save health status table: $rc_hs");
  return 1;
}

sub get_correct_response_time_volume_values {
  my $id       = shift;
  my $read_io  = shift;
  my $write_io = shift;
  my $resp_t_r = shift;
  my $resp_t_w = shift;
  my $time     = shift;

  my $read_io_return  = "";
  my $write_io_return = "";
  my $resp_t_r_return = "";
  my $resp_t_w_return = "";

  if ( !Xormon::isdigit($time) || !defined $id ) {
    $volumes_response_time{$id}{read_io}  = $read_io;
    $volumes_response_time{$id}{write_io} = $write_io;
    $volumes_response_time{$id}{resp_t_r} = $resp_t_r;
    $volumes_response_time{$id}{resp_t_w} = $resp_t_w;
    $volumes_response_time{$id}{time}     = "";
    return "";
  }

  if ( defined $volumes_response_time{$id} ) {
    # if ($datacore_debug) { print "DEBUG: $$ get_correct_response_time_volume_values for $id | $time element exists " . localtime() . " " . time() . "\n"; }
    my $read_io_tmp  = $volumes_response_time{$id}{read_io};
    my $write_io_tmp = $volumes_response_time{$id}{write_io};
    my $resp_t_r_tmp = $volumes_response_time{$id}{resp_t_r};
    my $resp_t_w_tmp = $volumes_response_time{$id}{resp_t_w};
    my $time_tmp     = $volumes_response_time{$id}{time};
    if ( Xormon::isdigit($time_tmp) ) {
      my $time_dif = ( $time - $time_tmp );
      if ( Xormon::isdigit($read_io) && Xormon::isdigit($read_io_tmp) && $time_dif > 0 ) {
        my $value = ( $read_io - $read_io_tmp ) / $time_dif;
        if ( Xormon::isdigit($value) && $value !~ m/^-/ ) {
          $read_io_return = $value;
        }
      }
      if ( Xormon::isdigit($write_io) && Xormon::isdigit($write_io_tmp) && $time_dif > 0 ) {
        my $value = ( $write_io - $write_io_tmp ) / $time_dif;
        if ( Xormon::isdigit($value) && $value !~ m/^-/ ) {
          $write_io_return = $value;
        }
      }
      if ( Xormon::isdigit($resp_t_r) && Xormon::isdigit($resp_t_r_tmp) && $time_dif > 0 ) {
        my $value = ( $resp_t_r - $resp_t_r_tmp ) / $time_dif;
        if ( Xormon::isdigit($value) && $value !~ m/^-/ ) {
          $resp_t_r_return = $value;
        }
      }
      if ( Xormon::isdigit($resp_t_w) && Xormon::isdigit($resp_t_w_tmp) && $time_dif > 0 ) {
        my $value = ( $resp_t_w - $resp_t_w_tmp ) / $time_dif;
        if ( Xormon::isdigit($value) && $value !~ m/^-/ ) {
          $resp_t_w_return = $value;
        }
      }
    }
  }
  $volumes_response_time{$id}{read_io}  = $read_io;
  $volumes_response_time{$id}{write_io} = $write_io;
  $volumes_response_time{$id}{resp_t_r} = $resp_t_r;
  $volumes_response_time{$id}{resp_t_w} = $resp_t_w;
  $volumes_response_time{$id}{time}     = $time;

  return ( $read_io_return, $write_io_return, $resp_t_r_return, $resp_t_w_return, $time );
}

sub get_level {
  my ($level, $source) = @_;
  my %level_map = (
    'DEFAULT' => {
      '0'  => 'Trace',
      '1'  => 'Info',
      '2'  => 'Warning',
      '3'  => 'Error',
      '4'  => 'Diagnostic',
    },
    'SOURCE' => {
      '0'  => 'General',
      '1'  => 'Disk pools',
      '2'  => 'Mirrors',
      '3'  => 'Monitors',
      '4'  => 'Paths',
      '5'  => 'Physical Disks',
      '6'  => 'Ports',
      '7'  => 'Replications',
      '8'  => 'Rollbacks',
      '9'  => 'Security',
      '10' => 'Snapshots',
      '11' => 'Hosts',
      '12' => 'DataCore Servers',
      '13' => 'Tasks',
      '14' => 'Virtual Disk Sources',
      '15' => 'Virtual Disks',
      '16' => 'Support',
      '17' => 'Performance recording',
    },
    'OTHER' => {
      '1'  => 'Undefined',
      '2'  => 'Healthy',
      '4'  => 'Attention',
      '8'  => 'Warning',
      '16' => 'Critical',
      '31' => 'All',
    },
  );

  if (!defined $source) {
    return $level_map{'DEFAULT'}{$level} // $level;
  }
  elsif (defined $source && $source eq 'SOURCE') {
    return $level_map{'SOURCE'}{$level} // $level;
  }
  else {
    return $level_map{'OTHER'}{$level} // $level;
  }
}

sub metrics_load {
  # hash of metrics to create gauges from counters 
  my %metrics_config = (
    'volume' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'cache_r_hit', 'cache_w_hit', 'cache_r_miss', 'cache_w_miss', 'cache_r_hit_bytes', 'cache_w_hit_bytes', 'cache_r_miss_bytes', 'cache_w_miss_bytes'],
    'host' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate' ],
    'host_group' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate',],
    'node' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'cache_r_hit', 'cache_w_hit', 'cache_r_miss', 'cache_w_miss'],
    'drive' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'resp_t_r', 'resp_t_w'],
    'scsiport' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'resp_t_r_initiator', 'resp_t_w_initiator', 'resp_t_r_target', 'resp_t_w_target', 'target_read', 'target_write', 'initiator_read', 'initiator_write' ],
    'fcport' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'resp_t_r_initiator', 'resp_t_w_initiator', 'resp_t_r_target', 'resp_t_w_target', 'target_read', 'target_write', 'initiator_read', 'initiator_write'],
    'iscsiport' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'resp_t_r_initiator', 'resp_t_w_initiator', 'resp_t_r_target', 'resp_t_w_target', 'target_read', 'target_write', 'initiator_read', 'initiator_write' ],
    'loopbackport' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'resp_t_r_initiator', 'resp_t_w_initiator', 'resp_t_r_target', 'resp_t_w_target', 'target_read', 'target_write', 'initiator_read', 'initiator_write' ],
    'volume_folder' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'cache_r_hit', 'cache_w_hit', 'cache_r_miss', 'cache_w_miss', 'cache_r_hit_bytes', 'cache_w_hit_bytes', 'cache_r_miss_bytes', 'cache_w_miss_bytes'],
    'pool' => ['read', 'write', 'data_rate', 'read_io', 'write_io', 'io_rate', 'resp_t_r', 'resp_t_w' ],
  );
  return %metrics_config;
}


1;
