use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib '../../../perl-modules';
use Db2;
use Xormon;
use File::Temp qw(tempfile);
use File::Copy;
use Test;
use XormonLogger;



$SIG{CHLD} = 'IGNORE';


use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

# microservice port
my $DEFAULT_PORT = 3304;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);
my $logger;

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {

  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }
  if ($pid == 0) {
    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    # received data template
    #
    # {
    #   "jwt"  : "<backendToken>",
    #   "task" : "<taskName>"
    #   "data" : {
    #     <configuration_from_hostcfg>
    #   }
    # }
    # read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);
    my $decoded = from_json($data);
    my $cfg = $decoded->{data};
    $logger = XormonLogger->new( 'db2', $cfg->{hostcfg_id}, $cfg->{hostalias} );
    $logger->info("Connection from $client_address:$client_port");
    $logger->info("Received task: ".$decoded->{task});
    my $response = '';
    if ($decoded->{task} eq "fetch") {
      $client->send("Task successfuly started");
      $client->shutdown(SHUT_WR);
      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'db2', $cfg->{hostcfg_id}, $cfg->{hostalias});
      my $cfg = $decoded->{data};
      $xormon->startJob('fetch', $cfg->{hostalias}, $$);
      ############################### AGENT SECTION ###############################

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }

      # get data from technology
      my $dataToBackend = api2json($xormon, $cfg);
      #print Dumper $dataToBackend;
      # send data to backend
      my $rc = $xormon->saveJSON2($dataToBackend);
      ############################### END AGENT SECTION ###############################
      # status code
      my $status;
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $xormon->endJob('fetch', $cfg->{hostalias}, $status, $$);
      $logger->info("Task [fetch] for ".$cfg->{hostalias}." is done");
    }
    elsif ($decoded->{task} eq "test") {
      apitest($client, $cfg);
      $client->shutdown(SHUT_WR);
      $logger->info("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
    } else {
      $response = "undefined task";
      $client->send($response);
      $client->shutdown(SHUT_WR);
    }
    exit();
  } else {
    $client->autoflush(1);
  }
}


my %whitelisted;
my $alias;
my @pre_conf;
my $use_whitelist;
my @health_status;
my @architecture;


sub api2json {
  my $xormon = shift;
  my $cdata  = shift;

  my $db_name     = $cdata->{instance};
  my $port        = $cdata->{port};
  my $username    = $cdata->{username};
  my $password    = $cdata->{password};
  my $ip          = $cdata->{host};
  $alias          = $cdata->{hostalias};
  $use_whitelist  = $cdata->{use_whitelist};
  my $uuid        = $cdata->{hostcfg_id};
  my $mvc_dir     = $FindBin::Bin;
  my %instances;
  my %perf;

  my $connected = Test::connTest($cdata->{host}, $cdata->{port});
  if ($connected == 1) {
    $xormon->isConnected(1);
  } else {
    $xormon->isConnected(0, "TCP connection to " . $cdata->{host} . ":" . $cdata->{port} . " has failed!");
    return ();
  }

  #print Dumper \%ENV;
  my $data = Db2::get_stats( $db_name, $port, $username, $password, $ip, $alias, $uuid );


  if (! defined $data->{error}){
    my %arc;

    #print Dumper \@architecture;
    #print Dumper $data;
    #print Dumper \%cnf;
    my $rc = $xormon->saveStatus($data->{health_status});
    #Xormon::log("Save Health Status for host  to DB. RC : $rc\n");
    my $arc_resp = $xormon->saveArchitecture($data->{arch});
    $logger->info("Save Architecture for host  to DB. RC : $arc_resp\n");
    my $configuration_rc = $xormon->saveConfJSON( $data->{conf}, 1  );
    $logger->info("Save Configuration for host  to DB. RC : $configuration_rc\n");
    $xormon->hasData(1);

    return $data->{perf};
  } else {
      $xormon->hasData(0, "Unable to get data")
  }
}

sub apitest {
  my $client  = shift;
  my $cdata   = shift;
  my $db_name     = $cdata->{instance};
  my $port        = $cdata->{port};
  my $username    = $cdata->{username};
  my $password    = $cdata->{password};
  my $ip          = $cdata->{host};
  $alias          = $cdata->{hostalias};
  $use_whitelist  = $cdata->{use_whitelist};
  my $uuid        = $cdata->{hostcfg_id};
  my @rows;
  my $DB2_CLI_DRIVER_INSTALL_PATH = $ENV{DB2_CLI_DRIVER_INSTALL_PATH};
  my $test = Test->new($client);
  my ($code, $text) = Xormon::conntest_tcp($ip, $port);
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $module = "DBD::DB2";
  my $module_def = 1;
  eval "use $module; 1" or $module_def = 0;
  if ( !$module_def ) {
    push(@rows, "ERROR: $module is not installed. Read more here: https://www.ibm.com/support/pages/db2-perl-database-interface-luw");
    $test->addSection("Driver test", 0 , \@rows);
    return $test->stop();
  }

  my $data = Db2::get_stats($db_name, $port, $username, $password, $ip, $alias, $uuid);
  if ( $data and $data ne "err" and $data->{perf} ) {
    push(@rows, "Data ok.");
    $test->addSection("Data test", 1 , \@rows);
    return $test->stop();
  }
  else {
    push(@rows, "Invalid data: DB2_CLI_DRIVER_INSTALL_PATH: $DB2_CLI_DRIVER_INSTALL_PATH" );
    $test->addSection("Driver test", 0 , \@rows);
    return $test->stop();
  }
    return $test->stop();
  #}
}

$server->close();
