package DDNINTELLIFLASH;

use strict;
use warnings;
use Data::Dumper;
use JSON qw(encode_json decode_json);
use Date::Parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
#use lib "/home/xormon/nextgen/perl-modules";
use POSIX ":sys_wait_h";
use LWP::UserAgent;
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
use Xormon;
no warnings 'utf8';
my $logger;

my %data;
my $xormon;
my $cfg;
my $stats_found = 0;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $active_minute = "";
my $debug_data = "";
my $debug_data_configuration = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my @architecture = ();
my $service_data_out;
my $st_type = "ddnintelliflash";
my @rows_api_test = ();
my @rows_api_test_data = ();
my $protocol = "";
my $wrkdir = "";
my $service_data_in;
my $connection_test = 0;
my $cookie = "";
my @health_status = ();
my %hosts;
my @hardware = ();
my $replication_capacity_used = "";
my @health_status_global = ();
my $token = "";
my $device_id = "";
my $api_host = "";
my $port = 443; ### default port 
#my $port = 8080;
my %performance_statistics;
my %metric_inventory;
my $timestamp_global;
my $enviroment = 1;
my @sfas = ();
my %volume_mapping;
my @clusters = ();
my @nodes = ();
my $last_minute = 10;






sub collect {
  $xormon = shift;
  $cfg    = shift;

  $timestamp_global = get_global_timestamp();
  unless ( exists $cfg->{'hostcfg_id'} )        { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'ddnintelliflash', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'ipNode'} )         { $logger->error("Cannot continue, 'ipNode' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )      { $logger->error("Cannot continue, 'apiUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )      { $logger->error("Cannot continue, 'apiPassword' is not configured!") && return "error"; }
  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  my $act_timestamp     = time();
  $service_data_in = $xormon->getData();
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }

  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $st_serial = $cfg->{'uuid'};
  $api_host = $cfg->{'ipNode'};
  if (defined $cfg->{"DDNINTELLIFLASH_ENVIROMENT"} && $cfg->{"DDNINTELLIFLASH_ENVIROMENT"} eq "development"){
    $enviroment = 0;
  }
  data_structure();
  $cookie = cmd("rest/session","POST", 1, "session");
  if (!defined $cookie || $cookie eq ""){
    $xormon->isConnected(0, "Connection failed!");
    $logger->error("Cookie is not defined") && return "error";
  }
    ### check connection
  if (!check_connection()){
    cmd("rest/session","DELETE", 1, "session_delete");
    $xormon->isConnected(0, "Connection failed!");
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
  ###
  $xormon->isConnected(1);
  set_data_node();
  #print Dumper \%performance_data;
  set_data_sfa();
  set_data_volume();
  set_data_drive();
  set_data_pool();
  set_data_cluster();
  set_data_filesystem();
  set_data_oss();
  set_data_ost();
  set_data_device();
  delete_empty_metrics();
  calculate_missing_metrics();
  #print Dumper \%performance_data;
  #print Dumper \%configuration_data;
  #return "successful";

  my $rc = $xormon->saveJSON2(\%performance_data);
  #print Dumper \%performance_data;
  my $status = get_status($rc, "saveJSON2 (performance)");

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }

  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");
  if (( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data_configuration/config-$st_serial.out" )){
    `touch "$debug_data_configuration/config-$st_serial.out"`;
    set_data_sfa_conf();
    set_data_health_status();
    my @types = ("pool", "volume", "sfa", "node", "fs", "drive", "exascaler", "ost", "oss", "device");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      foreach my $type (@types){
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
      }
      if ($active == 0){
        delete $configuration_data{$uid};
      }
    }
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    ($status, my $status_configuration, my $status_health) = set_architecture();
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "saveData");
    }
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;
  }
  cmd("rest/session","DELETE", 1, "session_delete"); ### logout
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}



sub set_data_health_status{
  my $status_global = "ok";
  my $message_global = "";
  ### sfa entity
  foreach my $sfa (@sfas){
    my $status_sfa = "ok";
    my $message_sfa = "";
    my @sfa_hs = ();
    my $data = cmd("rest/events_new?product=SFA&nodelabels=$sfa&starttime=1","GET",1,"health_status_sfa_$sfa");
    if (!defined $data  || $data eq "" || !defined $data->{data}  || ref($data->{data}) ne "ARRAY"){next;}
    foreach my $sfa (@{$data->{data}}){
      my %alert;
      my $acknowledged = $sfa->{"acknowledged"};
      if (defined $acknowledged && $acknowledged eq "true"){next;}
      my $id = $sfa->{id};
      if (!defined $id || $id eq ""){next;}
      my $event_type = $sfa->{eventType};
      my $eventUniqueId = $sfa->{eventUniqueId};
      my $firstOccuredTime = $sfa->{firstOccuredTime};
      my $lastOccuredTime = $sfa->{lastOccuredTime};
      my $severity = $sfa->{severity};
      my $descripton = $sfa->{impactedObjects}->{DESCRIPTION};
      if (!defined $event_type){$event_type = "";}
      if (!defined $eventUniqueId){$eventUniqueId = "";}
      if (!defined $firstOccuredTime){$$firstOccuredTime = "";}
      if (!defined $severity){$severity = "";}
      if (!defined $descripton){$descripton = "";}
      $alert{id} = $id;
      $alert{event_type} = $event_type;
      $alert{unique_id} = $eventUniqueId;
      $alert{first_occured_time} = $firstOccuredTime;
      $alert{last_occured_time} = $lastOccuredTime;
      $alert{severity} = $severity;
      $alert{description} = $descripton;
      $status_global = "error";
      $message_global = $descripton;
      $status_sfa = "error";
      $message_sfa = $descripton;
      push(@sfa_hs,\%alert);
    }
    my $uid = Xormon::create_hash_key($st_serial."sfa".$sfa);
    if (defined $configuration_data{$uid}){
      $configuration_data{$uid}{events} = encode_json(\@sfa_hs);
      $configuration_data{$uid}{status} = $status_global;
      $configuration_data{$uid}{status_data} = $message_global; 
    }
    if ($status_global eq "ok" && defined $configuration_data{$uid}{status} && $configuration_data{$uid}{status} eq "error"){
      $status_global = "error";
      $message_global = $configuration_data{$uid}{status_data};
    }
    if ($status_sfa eq "ok" && defined $configuration_data{$uid}{status} && $configuration_data{$uid}{status} eq "error"){
      $status_sfa = "error";
      $message_sfa = $configuration_data{$uid}{status_data};
    }
    my %sfa_hs;
    $sfa_hs{item_id} = $uid;
    $sfa_hs{status} = $status_sfa;
    $sfa_hs{data} = $message_sfa;
    push(@health_status_global, \%sfa_hs);
  }
  foreach my $cluster (@clusters){
    my $status_es = "ok";
    my $message_es = "";
    my @es_hs = ();
    my $data = cmd("rest/events_new?product=ES&nodelabels=$cluster&starttime=1","GET",1,"health_status_es_$cluster");
    if (!defined $data  || $data eq "" || !defined $data->{data}  || ref($data->{data}) ne "ARRAY"){next;}
    foreach my $es (@{$data->{data}}){
      my %alert;
      my $acknowledged = $es->{"acknowledged"};
      if (defined $acknowledged && $acknowledged eq "true"){next;}
      my $id = $es->{id};
      if (!defined $id || $id eq ""){next;}
      my $event_type = $es->{eventType};
      my $eventUniqueId = $es->{eventUniqueId};
      my $firstOccuredTime = $es->{firstOccuredTime};
      my $lastOccuredTime = $es->{lastOccuredTime};
      my $severity = $es->{severity};
      my $descripton = $es->{impactedObjects}->{DESCRIPTION};
      if (!defined $event_type){$event_type = "";}
      if (!defined $eventUniqueId){$eventUniqueId = "";}
      if (!defined $firstOccuredTime){$$firstOccuredTime = "";}
      if (!defined $severity){$severity = "";}
      if (!defined $descripton){$descripton = "";}
      $alert{id} = $id;
      $alert{event_type} = $event_type;
      $alert{unique_id} = $eventUniqueId;
      $alert{first_occured_time} = $firstOccuredTime;
      $alert{last_occured_time} = $lastOccuredTime;
      $alert{severity} = $severity;
      $alert{description} = $descripton;
      $status_global = "error";
      $message_global = $descripton;
      $status_es = "error";
      $message_es = $descripton;
      push(@es_hs,\%alert);
    }
    my $uid = Xormon::create_hash_key($st_serial."exascaler".$cluster);
    if (defined $configuration_data{$uid}){
      $configuration_data{$uid}{events} = encode_json(\@es_hs);
      $configuration_data{$uid}{status} = $status_global;
      $configuration_data{$uid}{status_data} = $message_global; 
    }
    if ($status_global eq "ok" && defined $configuration_data{$uid}{status} && $configuration_data{$uid}{status} eq "error"){
      $status_global = "error";
      $message_global = $configuration_data{$uid}{status_data};
    }
    if ($status_es eq "ok" && defined $configuration_data{$uid}{status} && $configuration_data{$uid}{status} eq "error"){
      $status_es = "error";
      $message_es = $configuration_data{$uid}{status_data};
    }
    my %es_hs;
    $es_hs{item_id} = $uid;
    $es_hs{status} = $status_es;
    $es_hs{data} = $message_es;
    push(@health_status_global, \%es_hs);
  }
  
  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status_global,
    "data" => $message_global
  );
  #print "$st_serial | $status_global | $message_global\n";
  push(@health_status_global, \%vm_status);
  #my $rc = $xormon->saveStatus(\@hs);
  #my $status = get_status($rc, "health status");
  #$logger->info("Task healh status ".$cfg->{hostalias}." is done. Status: $status");
  #print Dumper \@health_status_global;
  #print Dumper \@health_status;
  return;
}


sub set_data_sfa{
  my $section = "sfa";
  my %metrics;
  $metrics{"SFA_VDStat_TotalKBytesRead"} = "read";
  $metrics{"SFA_VDStat_TotalKBytesWritten"} = "write";
  $metrics{"SFA_VDStat_TotalReadIOs"} = "read_io";
  $metrics{"SFA_VDStat_TotalWriteIOs"} = "write_io";
  $metrics{"SFA_PDStat_KBytesRead"} = "read_b";
  $metrics{"SFA_PDStat_KBytesWritten"} = "write_b";
  $metrics{"SFA_PDStat_ReadIOs"} = "read_io_b";
  $metrics{"SFA_PDStat_WriteIOs"} = "write_io_b";
  $metrics{"SFA_PDStat_TotalReadIOLatency"} = "resp_t_r_b";
  $metrics{"SFA_PDStat_TotalWriteIOLatency"} = "resp_t_w_b";
  #print Dumper \%metrics;  
  my $data = cmd("rest/noderegistartion/sfa","GET",1,"sfas");
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){return;}
  foreach my $sfa (@{$data->{data}}){
    my %params;
    if (!defined $sfa->{nodelabel} || $sfa->{nodelabel} eq ""){next;}
    $params{"SFAHOST"} = $sfa->{nodelabel};
    my $uid = Xormon::create_hash_key($st_serial.$section.$sfa->{nodelabel});
    $configuration_data{$uid}{label} = $sfa->{nodelabel};
    $configuration_data{$uid}{TYPE} = $section;
    push (@sfas,$sfa->{nodelabel});
    my $json = get_json(\%params,\%metrics);
    my $data = cmd("datapoints/query/","POST",1,$sfa->{nodelabel},$json);
    #print "$json\n";
    #print Dumper \$data;
    my %data = get_performance_data($data,$section,-1,\%metrics);
    #print Dumper \%data;
    #$performance_data{$st_type}{$section}{$sfa->{nodelabel}} = \%data;
    foreach my $time (keys %data){
      foreach my $item (keys %{$data{$time}}){
        $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
      }
    }
    %data = get_performance_data($data,$section,-2,\%metrics);
    #print Dumper \%data;
    foreach my $time (keys %data){
      foreach my $item (keys %{$data{$time}}){
        $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
  calculate_performance_data($section,\%metrics);
  #print Dumper \$performance_data{$st_type}{$section};
  
  
}
sub set_data_sfa_conf{

  my $data = cmd("rest/sfas","GET",1,"sfas_detail");
  my $section = "sfa";
  my %items;
  $items{"ip1"} = "ctla";
  $items{"ip2"} = "ctlb";
  $items{"ePlatform"} = "e_platform";
  $items{"sfxCapable"} = "sfx_capable";
  $items{"sfxEnabled"} = "sfx_enabled";
  $items{"healthState"} = "status";
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){return;}
  foreach my $sfa (@{$data->{data}}){
    my $name = $sfa->{name};
    if (!defined $name || $name eq ""){next;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$name);
    if (defined $configuration_data{$uid}){
      foreach my $item (keys %items){
        if (defined $sfa->{$item}){
          if ($item eq "healthState" ){
            $configuration_data{$uid}{$items{$item}} = get_hs_status($sfa->{$item},$section);
          }
          else{
            $configuration_data{$uid}{$items{$item}} = $sfa->{$item};
          }
        }
      }
      my $data = cmd("rest/sfas/$name/controllers", "GET",1,"$name" . "_controllers");
      if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){next;}
      foreach my $ctl (@{$data->{data}}){
        if (defined $ctl->{vendorEquipmentType}){
          $configuration_data{$uid}{model} = $ctl->{vendorEquipmentType};
          last;
        }
      }
      $data = cmd("rest/sfas/$name/enclosures", "GET",1,"$name" . "enclosures");
      if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){next;}
      foreach my $ctl (@{$data->{data}}){
        if (defined $ctl->{biosVersion}){
          $configuration_data{$uid}{bios} = $ctl->{biosVersion};
        }
        if (defined $ctl->{bmcVersion}){
          $configuration_data{$uid}{bmc_version} = $ctl->{bmcVersion};
        }
        if (defined $ctl->{firmwareVersion}){
          $configuration_data{$uid}{version} = $ctl->{firmwareVersion};
        }
      }
    }
  } 
  
  
}



sub get_hs_status{
  my $status = shift;
  my $section = shift;
  #0 -> NA
  #1 -> OK
  #2 -> Non_Critical
  #3 -> Critical
  #255 -> Unknown
  if (!Xormon::isdigit($status) ){return "unknown";}
  if ($status == 1){return "ok";}
  if ($status == 0){return "unknown";}
  if ($status == 2){return "warning";}
  if ($status == 3){return "error";}
  if ($status == 255){return "unknown";}
  return "unknown";

}

sub get_json{
  my $param = shift;
  my $metric_tmp = shift;
  my $relative_start = shift;
  if (!defined $relative_start || !Xormon::isdigit($relative_start)){
    $relative_start = 15;
  }
  my %params = %{$param};
  my %metrics = %{$metric_tmp};
  my $json = "{\"metrics\":[";
  foreach my $metric (keys %metrics){
    #print "$metric $json\n";
    my $unit = "";
    if ($metrics{$metric} =~ m/resp_t/){
      $unit = ",\"Unit\": [\"us\"]";
    }
    $json = $json . "{\"tags\":{";
    foreach my $param (keys %params){
      $json = $json . "\"$param\":[\"" . $params{$param} . "\"],";
    }
    chop $json;
    $json = $json . "$unit},\"name\":\"$metric\",\"aggregators\":[{\"name\":\"avg\",\"sampling\":{\"value\":\"300\",\"unit\":\"seconds\"},\"align_sampling\":true}]},";
  }
  chop $json;
  $json = $json . "],\"plugins\":[],\"cache_time\":0,\"start_relative\":{\"value\":\"$relative_start\",\"unit\":\"minutes\"}}";
  return $json;
  #print "$json\n";
}

sub get_performance_data{
  my $data = shift;
  my $section = shift;
  my $sample_rate = shift;
  my $metrics = shift;
  my $type_data = shift;
  my %metrics = %{$metrics};
  my %datas;
  my $limit = 2;
  if (!defined $data || $data eq "" || !defined $data->{queries} || ref($data->{queries}) ne "ARRAY"){
    return %datas;
  }
  foreach my $element (@{$data->{queries}}){
    if (!defined $element->{results}){next;}
    foreach my $metric (@{$element->{results}}){
      my $met = $metric->{name};
      if (defined $type_data && $type_data eq "avg"){
        $met = $metrics{$met};
        $limit = 1;
      }
      else{
        $met = $metrics{$met} . $sample_rate;
      }
      my $values = $metric->{values};
      if (!defined $met || !defined $values || scalar(@{$values}) < $limit){next;}
      my $timestamp = $values->[$sample_rate]->[0];
      my $value = $values->[$sample_rate]->[1];
      if (Xormon::isdigit($value) && Xormon::isdigit($timestamp)){
        $timestamp = $timestamp / 1000;
        if (defined $type_data && $type_data eq "avg" && !$enviroment){
          $timestamp = time();
        }
        $datas{$timestamp}{$met} = round($value,0);
      }
    }
  }
  return %datas;
}

sub calculate_performance_data{
  my $section = shift;
  my $metric_tmp = shift;
  my %metrics = %{$metric_tmp};
  #print Dumper \$performance_data{$st_type}{$section};
  foreach my $element (keys %{$performance_data{$st_type}{$section}}){
    my @keys = keys( %{$performance_data{$st_type}{$section}{$element}});
    my $t1 = $keys[0];
    my $t2 = $keys[1];
    if ($t2 < $t1){
      my $pom = $t2;
      $t2 = $t1;
      $t1 = $pom;
    }
    if (!Xormon::isdigit($t1) || !Xormon::isdigit($t2) || $t2 - $t1 < 0 || $t2 - $t1 == 0){
      next;
    }
    foreach my $metric (keys %metrics){
      my $denominator = 1;
      my $rounded = 0;
      my $metric_value = $metrics{$metric};
      if ($metric_value =~ m/resp_t/){
        $denominator = 1000;
        $rounded = 3;
      }
      my $value_sec_to_last = $performance_data{$st_type}{$section}{$element}{$t1}{"$metric_value-2"};
      my $value_last = $performance_data{$st_type}{$section}{$element}{$t2}{"$metric_value-1"};
      if (Xormon::isdigit($value_sec_to_last) && Xormon::isdigit($value_last) && $value_last - $value_sec_to_last > 0){
        $performance_data{$st_type}{$section}{$element}{$t2}{$metric_value} = round(($value_last - $value_sec_to_last) / (($t2 - $t1) * $denominator),$rounded);
        delete $performance_data{$st_type}{$section}{$element}{$t2}{"$metric_value-1"};
      }
      delete $performance_data{$st_type}{$section}{$element}{$t2}{"$metric_value-1"};
    }
    delete $performance_data{$st_type}{$section}{$element}{$t1};
    $performance_data{$st_type}{$section}{$element}{$t2}{"io_rate"} = get_metrics_sum("read_io","write_io",$performance_data{$st_type}{$section}{$element}{$t2});
    $performance_data{$st_type}{$section}{$element}{$t2}{"data_rate"} = get_metrics_sum("read","write",$performance_data{$st_type}{$section}{$element}{$t2});
    if ($section eq "sfa"){
      $performance_data{$st_type}{$section}{$element}{$t2}{"io_rate_b"} = get_metrics_sum("read_io_b","write_io_b",$performance_data{$st_type}{$section}{$element}{$t2});
      $performance_data{$st_type}{$section}{$element}{$t2}{"data_rate_b"} = get_metrics_sum("read_b","write_b",$performance_data{$st_type}{$section}{$element}{$t2});
      $performance_data{$st_type}{$section}{$element}{$t2}{"resp_t_b"} = round(calculate_weighted_average($performance_data{$st_type}{$section}{$element}{$t2},"resp_t_r_b","read_io_b","resp_t_w_b","write_io_b"),3);
    }
    if ($section eq "drive"){
      $performance_data{$st_type}{$section}{$element}{$t2}{"resp_t"} = round(calculate_weighted_average($performance_data{$st_type}{$section}{$element}{$t2},"resp_t_r","read_io","resp_t_w","write_io"),3);
    }
    if (!$enviroment){
      my $t = time();
      foreach my $item (keys %{$performance_data{$st_type}{$section}{$element}{$t2}}){
        $performance_data{$st_type}{$section}{$element}{$t}{$item} = $performance_data{$st_type}{$section}{$element}{$t2}{$item};
      }
      delete $performance_data{$st_type}{$section}{$element}{$t2};
    }
  }

}

sub calculate_weighted_average{
  my $data_tmp = shift;
  my $numerator_metric1 = shift;
  my $denominator_metric1 = shift;
  my $numerator_metric2 = shift;
  my $denominator_metric2 = shift;


  my %data = %{$data_tmp};

  my $sum = "";
  my $denominator = "";
  if (Xormon::isdigit($data{$denominator_metric1}) && Xormon::isdigit($data{$numerator_metric1})){
    $sum = $data{$denominator_metric1} * $data{$numerator_metric1};
    $denominator = $data{$denominator_metric1};
  }
  if (Xormon::isdigit($data{$denominator_metric2}) && Xormon::isdigit($data{$numerator_metric2})){
    if (Xormon::isdigit($sum)){
      $sum = $sum + ($data{$denominator_metric2} * $data{$numerator_metric2});
      $denominator = $denominator + $data{$denominator_metric2};
    }
    else{
      $sum = $data{$denominator_metric2} * $data{$numerator_metric2};
      $denominator = $data{$denominator_metric2};
    }
  }
  if (Xormon::isdigit($sum) && Xormon::isdigit($denominator)){
    if ($denominator == 0){
      return 0;
    }
    return $sum / $denominator;
  }
  return "";
}

sub get_metrics_sum{
  my $item1 = shift;
  my $item2 = shift;
  my $data_tmp = shift;
  my %data = %{$data_tmp};
  my $value1 = $data{$item1};
  my $value2 = $data{$item2};

  if (Xormon::isdigit($value2)){
    if (Xormon::isdigit($value1)){
      return ($value2 + $value1);
    }
    else{
      return $value2;
    }
  }
  elsif(Xormon::isdigit($value1)){
    return $value1;
  }
  return "";

}
sub get_diference_value{
  my $item1 = shift;
  my $item2 = shift;
  my $data_tmp = shift;
  my %data = %{$data_tmp};
  my $value1 = $data{$item1};
  my $value2 = $data{$item2};

  if (Xormon::isdigit($value1) && Xormon::isdigit($value2) && $value1 >= $value2){
    return ($value1 - $value2);
  }
  return "";

}

sub set_performance_data{
  my $data = shift;
  my $metric_original = shift;
  my $tag = shift;
  my $section = shift;
  my $rounded = 0;
  my $denominator = 1;
  if ($metric_original =~ m/resp/){$rounded = 3; $denominator = 1000;}
  if (!defined $data || $data eq "" || ref($data->{queries}) ne "ARRAY"){return;}
  foreach my $element (@{$data->{queries}}){
    if (!defined $element->{results}){next;}
    foreach my $metric (@{$element->{results}}){
      if (!defined $metric->{tags}->{$tag} || ref($metric->{tags}->{$tag}) ne "ARRAY"){next;}
      my $label = $metric->{tags}->{$tag}->[0];
      if (defined $label){
        my $values = $metric->{values};
        if (!defined $values){next;}
        my $timestamp = $values->[-1]->[0];
        my $value = $values->[-1]->[1];
        if (Xormon::isdigit($value) && Xormon::isdigit($timestamp)){
          $timestamp = $timestamp / 1000;
          $performance_data{$st_type}{$section}{$label}{$timestamp}{$metric_original} = round($value / (300 * $denominator),$rounded);
        }
      }
    }
  }
}


sub set_data_device{
  my $type = "sfa";
  my $section = "device";
  my @items = ("read_io","write_io","read","write", "data_rate", "io_rate");
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$type}{$uid}}){
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$type}{$uid}{$timestamp}{$item};
        if (Xormon::isdigit($value)){
          if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$st_serial} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item}){
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} = $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} + $value;
          }
          else{
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp}{$item} = $value;
          }
        }
      }
    }
  }
  @items = ("capacity_total", "capacity_used", "capacity_free");
  $type = "exascaler";
  my $timestamp_device = (keys %{$performance_data{$st_type}{$section}{$st_serial}})[0];
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$type}{$uid}}){
      if (!Xormon::isdigit($timestamp_device)){$timestamp_device = $timestamp;}
      foreach my $item (@items){
        my $value = $performance_data{$st_type}{$type}{$uid}{$timestamp}{$item};
        if (Xormon::isdigit($value)){
          if (defined $performance_data{$st_type}{$section} && defined $performance_data{$st_type}{$section}{$st_serial} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp_device} && defined $performance_data{$st_type}{$section}{$st_serial}{$timestamp_device}{$item}){
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp_device}{$item} = $performance_data{$st_type}{$section}{$st_serial}{$timestamp_device}{$item} + $value;
          }
          else{
            $performance_data{$st_type}{$section}{$st_serial}{$timestamp_device}{$item} = $value;
          }
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
}



sub sum_metrics{
  my $data_tmp = shift;
  my $metric1 = shift;
  my $metric2 = shift;
  my %data = %{$data_tmp};
  my $sum = "";
  if (Xormon::isdigit($data{$metric1})){
    $sum = $data{$metric1};
  }
  if (Xormon::isdigit($data{$metric2})){
    if (Xormon::isdigit($sum)){
      $sum = $sum + $data{$metric2};
    }
    else{
      $sum = $data{$metric2};
    }
  }
  return $sum;
}



sub sum_all_metrics {

  my $data_tmp = shift;
  my %data     = %{$data_tmp};
  my $sum      = 0;
  foreach my $item ( keys %data ) {
    if ( Xormon::isdigit( $data{$item} ) ) {
      $sum = $sum + $data{$item};
    }
  }
  return $sum;
}


sub set_data_pool{
  my $section  = "pool";
  foreach my $sfa (@sfas){
    my $data = cmd("rest/sfas/$sfa/storagepools","GET",1,"pools-$sfa");
    if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){next;}
    foreach my $pool (@{$data->{data}}){
      if (!defined $pool->{uuid} || $pool->{uuid} eq "" || !defined $pool->{index} || $pool->{index} eq ""){next;}
      my $capacity_total = $pool->{rawCapacity};
      my $capacity_free = $pool->{freeCapacity};
      my $hs_status = $pool->{state};
      my $hs_description = $pool->{healthStateReason};
      my $block_size = $pool->{blockSize};
      if ($volume_mapping{$pool->{index}}){
        $configuration_data{$pool->{uuid}}{id} = $pool->{index};
        $configuration_data{$pool->{uuid}}{TYPE} = $section;
        if (defined $hs_status){
          my %hs;
          $hs{status} = get_hs_status($hs_status);
          $hs{data} = $hs_description;
          $hs{item_id} = $pool->{uuid};
          push(@health_status_global,\%hs);
        }
        my $uid_sfa = Xormon::create_hash_key($st_serial."sfa".$sfa);
        if (defined $configuration_data{$uid_sfa}){
          push(@{$configuration_data{$pool->{uuid}}{parents}},$uid_sfa);
          if (get_hs_status($hs_status) eq "error"){
            $configuration_data{$uid_sfa}{status} = "error";
            $configuration_data{$uid_sfa}{status_data} = $hs_description; 
          }
        }
        if (defined $pool->{name} && $pool->{name} ne ""){
          $configuration_data{$pool->{uuid}}{label} = $pool->{name};
        }
        else{
          $configuration_data{$pool->{uuid}}{label} = $pool->{index};
        }
        foreach my $uid (@{$volume_mapping{$pool->{index}}}){
          push(@{$configuration_data{$uid}{parents}},$pool->{uuid});
          foreach my $metric (("data_rate", "io_rate", "read", "write", "read_io", "write_io")){
            foreach my $timestamp ( keys %{$performance_data{$st_type}{volume}{$uid}} ){
              if (Xormon::isdigit($performance_data{$st_type}{$section}{$pool->{uuid}}{$timestamp}{$metric})){
                $performance_data{$st_type}{$section}{$pool->{uuid}}{$timestamp}{$metric} = $performance_data{$st_type}{$section}{$pool->{uuid}}{$timestamp}{$metric} + $performance_data{$st_type}{volume}{$uid}{$timestamp}{$metric}; 
              }
              else{
                $performance_data{$st_type}{$section}{$pool->{uuid}}{$timestamp}{$metric} = $performance_data{$st_type}{volume}{$uid}{$timestamp}{$metric};
              }
              if (Xormon::isdigit($capacity_total) && Xormon::isdigit($block_size)){
                $performance_data{$st_type}{$section}{$pool->{uuid}}{$timestamp}{"capacity_total"} = $capacity_total * $block_size;
                $configuration_data{$pool->{uuid}}{"capacity_total"} = $capacity_total * $block_size;
              }
              if (Xormon::isdigit($capacity_free) && Xormon::isdigit($block_size)){
                $performance_data{$st_type}{$section}{$pool->{uuid}}{$timestamp}{"capacity_free"} = $capacity_free * $block_size;
                $configuration_data{$pool->{uuid}}{"capacity_free"} = $capacity_free * $block_size;
              }
              if (Xormon::isdigit($capacity_free) && Xormon::isdigit($block_size) && Xormon::isdigit($capacity_total)){
                $performance_data{$st_type}{$section}{$pool->{uuid}}{$timestamp}{"capacity_used"} = ($capacity_total * $block_size) - ($capacity_free * $block_size);
                $configuration_data{$pool->{uuid}}{"capacity_used"} = ($capacity_total * $block_size) - ($capacity_free * $block_size);
              }
            }
          }
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_drive{
  my $section  = "drive";
  my %metrics;
  $metrics{"SFA_PDStat_KBytesRead"} = "read";
  $metrics{"SFA_PDStat_KBytesWritten"} = "write";
  $metrics{"SFA_PDStat_ReadIOs"} = "read_io";
  $metrics{"SFA_PDStat_WriteIOs"} = "write_io";
  $metrics{"SFA_PDStat_TotalReadIOLatency"} = "resp_t_r";
  $metrics{"SFA_PDStat_TotalWriteIOLatency"} = "resp_t_w";
  foreach my $sfa (@sfas){
    my $data = cmd("rest/sfas/$sfa/pds","GET",1,"drives-$sfa");
    if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){next;}
    foreach my $drive (@{$data->{data}}){
      my %params;
      if (!defined $drive->{index} || $drive->{index} eq ""){next;}
      my $hs_status = $drive->{healthState};
      my $hs_description = $drive->{healthStateReason};
      $params{"Index"} = $drive->{index};
      $params{"SFAHOST"} = $sfa;
      my $uid = Xormon::create_hash_key($st_serial.$sfa.$section.$drive->{index});
      $configuration_data{$uid}{TYPE} = $section;
      $configuration_data{$uid}{label} = $drive->{index};
      if (defined $hs_status){
        my %hs;
        $hs{status} = get_hs_status($hs_status);
        $hs{data} = $hs_description;
        $hs{item_id} = $uid;
        push(@health_status_global,\%hs);
      }
      my $uid_sfa = Xormon::create_hash_key($st_serial."sfa".$sfa);
      if (defined $configuration_data{$uid_sfa}){
        push(@{$configuration_data{$uid}{parents}},$uid_sfa);
        if (get_hs_status($hs_status) eq "error"){
          $configuration_data{$uid_sfa}{status} = "error";
          $configuration_data{$uid_sfa}{status_data} = $hs_description; 
        }
      }
      my $json = get_json(\%params,\%metrics);
      my $data = cmd("datapoints/query/","POST",1,"$sfa-" . $drive->{index},$json);
      my %data = get_performance_data($data,$section,-1,\%metrics);
      foreach my $time (sort { $a <=> $b } keys %data){
        foreach my $item (keys %{$data{$time}}){
          $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
        }
        if (Xormon::isdigit($drive->{formattedCapacity}) && Xormon::isdigit($drive->{blockSize})){
          $performance_data{$st_type}{$section}{$uid}{$time}{"capacity_total"} = $drive->{formattedCapacity} * $drive->{blockSize};
          $configuration_data{$uid}{"capacity_total"} = $drive->{formattedCapacity} * $drive->{blockSize};
        }
      }
      %data = get_performance_data($data,$section,-2,\%metrics);
      foreach my $time (sort { $a <=> $b } keys %data){
        foreach my $item (keys %{$data{$time}}){
          $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
        }
      }
    }
  }
  calculate_performance_data($section,\%metrics);
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_volume{
  my $section  = "volume";
  my %metrics;
  $metrics{"SFA_VDStat_KBytesRead"} = "read";
  $metrics{"SFA_VDStat_KBytesWritten"} = "write";
  $metrics{"SFA_VDStat_ReadIOs"} = "read_io";
  $metrics{"SFA_VDStat_WriteIOs"} = "write_io";
  foreach my $sfa (@sfas){
    my $data = cmd("rest/sfas/$sfa/vds","GET",1,"volumes-$sfa");
    if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){next;}
    foreach my $volume (@{$data->{data}}){
      my %params;
      if (!defined $volume->{index} || $volume->{index} eq "" || !defined $volume->{uuid} || $volume->{uuid} eq ""){next;}
      my $hs_status = $volume->{healthState};
      my $hs_description = $volume->{healthStateReason};
      if (!defined $hs_description){$hs_description = "";}
      $params{"Index"} = $volume->{index};
      $params{"SFAHOST"} = $sfa;
      $configuration_data{$volume->{uuid}}{id} = $volume->{index};
      $configuration_data{$volume->{uuid}}{TYPE} = $section;
      $configuration_data{$volume->{uuid}}{"lun_id"} = $volume->{uuid};
      if (defined $hs_status){
        my %hs;
        $hs{status} = get_hs_status($hs_status);
        $hs{data} = $hs_description;
        $hs{item_id} = $volume->{uuid};
        push(@health_status_global,\%hs);
      }
      my $uid_sfa = Xormon::create_hash_key($st_serial."sfa".$sfa);
      if (defined $configuration_data{$uid_sfa}){
        push(@{$configuration_data{$volume->{uuid}}{parents}},$uid_sfa);
        if (get_hs_status($hs_status) eq "error"){
          $configuration_data{$uid_sfa}{status} = "error";
          $configuration_data{$uid_sfa}{status_data} = $hs_description; 
        }
      }
      if (defined $volume->{name} && $volume->{name} ne ""){
        $configuration_data{$volume->{uuid}}{label} = $volume->{name};
      }
      else{
        $configuration_data{$volume->{uuid}}{label} = $volume->{index};
      }
      my $json = get_json(\%params,\%metrics);
      my $data = cmd("datapoints/query/","POST",1,$volume->{uuid},$json);
      my %data = get_performance_data($data,$section,-1,\%metrics);
      foreach my $time (sort { $a <=> $b } keys %data){
        foreach my $item (keys %{$data{$time}}){
            $performance_data{$st_type}{$section}{$volume->{uuid}}{$time}{$item} = $data{$time}{$item};
        }
        if (Xormon::isdigit($volume->{capacity}) && Xormon::isdigit($volume->{blockSize})){
          $performance_data{$st_type}{$section}{$volume->{uuid}}{$time}{"capacity_total"} = $volume->{capacity} * $volume->{blockSize};
          $configuration_data{$volume->{uuid}}{"capacity_total"} =  $volume->{capacity} * $volume->{blockSize};
        }
        if (defined $volume->{poolIndex}){
          push(@{$volume_mapping{$volume->{poolIndex}}},$volume->{uuid}); 
          $performance_data{$st_type}{$section}{$volume->{uuid}}{$time}{"pool"} = $volume->{poolIndex};
        }
      }
      %data = get_performance_data($data,$section,-2,\%metrics);
      foreach my $time (sort { $a <=> $b } keys %data){
        foreach my $item (keys %{$data{$time}}){
          $performance_data{$st_type}{$section}{$volume->{uuid}}{$time}{$item} = $data{$time}{$item};
        }
      }
    }
  }
  calculate_performance_data($section,\%metrics);
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_node{
  my $section = "node"; ### myslim že je to spise cluster
  my $json = '{"metrics":[{"tags":{"partition":["/data"],"host":["DDNInsight"]},"name":"disk_total","aggregators":[{"name":"avg","sampling":{"value":"300","unit":"seconds"},"align_sampling":true}]},{"tags":{"host":["DDNInsight"],"partition":["/data"]},"name":"disk_used","aggregators":[{"name":"avg","sampling":{"value":"300","unit":"seconds"},"align_sampling":true}]}],"plugins":[],"cache_time":0,"time_zone":"Europe/Budapest","start_relative":{"value":"10","unit":"minutes"}}';

  my $json2 = '{"metrics":[{"tags":{"host":["DDNInsight"]},"name":"mem_total","aggregators":[{"name":"avg","sampling":{"value":"300","unit":"seconds"},"align_sampling":true}]},{"tags":{"host":["DDNInsight"]},"name":"mem_used","aggregators":[{"name":"avg","sampling":{"value":"300","unit":"seconds"},"align_sampling":true}]},{"tags":{"host":["DDNInsight"]},"name":"system_load5","aggregators":[{"name":"avg","sampling":{"value":"300","unit":"seconds"},"align_sampling":true}]}],"plugins":[],"cache_time":0,"time_zone":"Europe/Budapest","start_relative":{"value":"10","unit":"minutes"}}';
  my $data = cmd("rest/nodes","GET",1,"nodes");
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){return;}
  foreach my $node (@{$data->{data}}){
    my $name = $node->{hostname};
    if (!defined $name || $name eq ""){next;}
    my %data;
    my $uid = Xormon::create_hash_key($st_serial.$section.$name);
    ### add configuration_data
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    foreach my $partion (("/data","/")){
      $json =~ s/DDNInsight/$name/g;
      $json =~ s/\/data/$partion/g;
      my $partion_new = "capacity_system";
      if ($partion =~ m/data/){
        $partion_new = "capacity_data";
      }
      my $data = cmd("datapoints/query/","POST",1,"$name-$partion_new",$json);
      $json =~ s/$name/DDNInsight/g;
      $json =~ s/$partion/\/data/g;
      if (!defined $data || $data eq "" || !defined $data->{queries} || ref($data->{queries}) ne "ARRAY"){next;}
      foreach my $element (@{$data->{queries}}){
        if (!defined $element->{results}){next;}
        foreach my $metric (@{$element->{results}}){
          my $met = $metric->{name};
          if ($met eq "disk_total" && $partion_new eq "capacity_data"){$met = "user_capacity_total";}
          if ($met eq "disk_used" && $partion_new eq "capacity_data"){$met = "user_capacity_used";}
          if ($met eq "disk_total" && $partion_new eq "capacity_system"){$met = "system_capacity_total";}
          if ($met eq "disk_used" && $partion_new eq "capacity_system"){$met = "system_capacity_used";}
          my $values = $metric->{values};
          if (!defined $met || !defined $values){next;}
          my $timestamp = $values->[-1]->[0];
          my $value = $values->[-1]->[1];
          if (Xormon::isdigit($value) && Xormon::isdigit($timestamp)){
            $timestamp = $timestamp / 1000;
            if (!$enviroment){
              $timestamp = time();
            }
            $data{$timestamp}{$met} = $value;
          }
        }
      }
    }

    my $data = cmd("datapoints/query/","POST",1,"$name-memory_cpu",$json2);
    if (!defined $data || $data eq "" || !defined $data->{queries} || ref($data->{queries}) ne "ARRAY"){next;}
    foreach my $element (@{$data->{queries}}){
      if (!defined $element->{results}){next;}
      foreach my $metric (@{$element->{results}}){
        my $met = $metric->{name};
        if ($met eq "mem_total"){$met = "memory_total";}
        if ($met eq "mem_used"){$met = "memory_used";}
        if ($met eq "system_load5"){$met = "cpu_system";}
        my $values = $metric->{values};
        if (!defined $met || !defined $values){next;}
        my $timestamp = $values->[-1]->[0];
        my $value = $values->[-1]->[1];
        if (Xormon::isdigit($value) && Xormon::isdigit($timestamp)){
          $timestamp = $timestamp / 1000;
          if (!$enviroment){
            $timestamp = time();
          }
          $data{$timestamp}{$met} = $value;
        }
      }
    }
    $performance_data{$st_type}{$section}{$uid} = \%data;
    ### configuration data
    foreach my $metric (keys %{$node}){
      if ($metric =~ m/events/){next;}
      $configuration_data{$uid}{$metric} = $node->{$metric};
      if ($metric eq "ip"){
        $configuration_data{$uid}{host} = $node->{$metric};
      }
    }
  }
  my %data;
  $data{user_capacity_free}[0] = "user_capacity_total";
  $data{user_capacity_free}[1] = "user_capacity_used";
  $data{system_capacity_free}[0] = "system_capacity_total";
  $data{system_capacity_free}[1] = "system_capacity_used";
  $data{memory_free}[0] = "memory_total";
  $data{memory_free}[1] = "memory_used";
  foreach my $node (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$node}}){
      foreach my $metric (keys %data){
        my $metric1 = $performance_data{$st_type}{$section}{$node}{$timestamp}{$data{$metric}[0]};
        my $metric2 = $performance_data{$st_type}{$section}{$node}{$timestamp}{$data{$metric}[1]};
        if (Xormon::isdigit($metric1) && Xormon::isdigit($metric2) && $metric1 >= $metric2 && $metric1 > 0){
          $performance_data{$st_type}{$section}{$node}{$timestamp}{$metric} = $metric1 - $metric2;
          if ($metric eq "memory_free"){
            $performance_data{$st_type}{$section}{$node}{$timestamp}{"memory_used_percent"} = round(($metric2 / $metric1) * 100,2);
          }
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}


sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    if (keys %{$performance_data{$st_type}{$type} } == 0){
      delete $performance_data{$st_type}{$type};
      next;
    }
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        if ($first_time eq "" && Xormon::isdigit($time)){
          $first_time = $time;
        }
        if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
          $first_time = $time;
        }
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
          if ($item =~ m/numerator|denominator/){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
      }
      if (!keys %{$performance_data{$st_type}{$type}{$uid}}){
        delete $performance_data{$st_type}{$type}{$uid};
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
}

sub calculate_missing_metrics{

  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";


  foreach my $type (keys %{$performance_data{$st_type}}){
    $item_total = "capacity_total";
    $item_used = "capacity_used";
    $item_used_percent = "capacity_used_percent";
    if ($type eq "node"){
      $item_total = "system_capacity_total";
      $item_used = "system_capacity_used";
      $item_used_percent = "system_capacity_used_percent";
    }
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if ($type eq "node"){
          $item_total = "user_capacity_total";
          $item_used = "user_capacity_used";
          $item_used_percent = "user_capacity_used_percent";
          if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
            if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
              $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
            }
            else{
              my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
              $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
            }
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io},0);
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }
      }
    }
  }
}




sub cmd{
  my $cmd = shift;
  my $type = shift;
  my $execute = shift;
  my $debug_file = shift;
  my $json_body = shift;
  if (!defined $type || $type eq ""){$type = "GET";}
  if (!defined $execute || $execute eq ""){
    $execute = 1;
  }
  ### debug
  if (!$enviroment){
    $execute = 0;
  }
  if ($cmd =~ m/session/ && $type eq "POST" && $execute == 0 && !$enviroment){
    return "123";
  }
  if (!defined $debug_file){$debug_file = "";}
  if ($execute == 0 && -f "$debug_data_configuration/$debug_file.json"){
    return Xormon::read_json("$debug_data_configuration/$debug_file.json");
  }
  elsif(-f "$debug_data/$debug_file.json" && $execute == 0){
    return Xormon::read_json("$debug_data/$debug_file.json");
  }
  my $url = "https://$api_host:$port/$cmd";
  my $ua  = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( $type => $url );
  $req->header( 'Content-Type' => 'application/json' );
  $req->header( 'Accept'       => 'application/json' );
  $req->header( 'Connection'   => 'keep-alive' );
  if ($cmd =~ m/session/ && $type eq "POST"){
    my %login;
    $login{userName} = $cfg->{'apiUsername'};
    $login{password} = $cfg->{'apiPassword'};
    $login{signInMethod} = "local";
    $json_body = encode_json(\%login);
  }
  else{
    $req->header( 'Cookie' => $cookie );
  }
  if ($cmd =~ m/datapoints|kairosdbapi/){
    $req->authorization_basic( $cfg->{'apiUsername'}, $cfg->{'apiPassword'} );
  }
  if (defined $json_body && $json_body ne ""){
    $req->content($json_body);
  }
  my $res = $ua->request($req);
  #print $res->content;
  if ( ! $res->is_success ) {
    if (!$connection_test){
      $logger->error("API command $cmd failed");
    }
    else{
      push(@rows_api_test,"API command $cmd failed\n");
      push(@rows_api_test,$res->content . "\n");
      push(@rows_api_test_data,"API command $cmd failed\n");
      push(@rows_api_test_data,$res->content . "\n");
    }
    my $out = $res->content;
    $logger->error($out);
    return "";
  }
  #print "$cmd is succesful\n";
  if ($cmd =~ m/session/ && $type eq "POST"){
    return $res->headers->{"set-cookie"};
  }
  my $data = decode_json( $res->content );
  #print Dumper \$data;
  if (defined $debug_file && $debug_file ne ""){
    #if ($debug_file =~ m/_/){
    eval {Xormon::save_as_json("$debug_data/$debug_file.json",$data);};
    #}
    #else{
    #  eval {Xormon::save_as_json("$debug_data_configuration/$debug_file.json",$data);};
    #}
    if ($@){
      $logger->error("Save data to file $debug_data/$debug_file.json failed");
    }
  }
  return $data;
}

sub set_architecture{


  my %device_data;
  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  #if (defined $cfg->{serial_number}){
  #  $device_arch{serial_number} = $cfg->{serial_number};
  #}
  #if (defined $cfg->{version}){
  #  $device_arch{version} = $cfg->{version};
  #}
  #if (defined $cfg->{model}){
  #  $device_arch{model} = $cfg->{model};
  #}
  push(@architecture, \%device_arch);
  #print Dumper \$configuration_data{$st_serial};
  #return;

  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if ($type eq "node"){
      foreach my $item (keys %{$configuration_data{$uid}}){
        if ($item =~ m/prediction|^ts$|label|TYPE/){next;}
        $device_data{$item} = $configuration_data{$uid}{$item};  
      }
    }
    if (!defined $name || !defined $type){next;}
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      ## delete children and parent from item_properties
      delete $configuration_data{$uid}{children};

    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      delete $configuration_data{$uid}{parents};

    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "parents" => [$cfg->{'uuid'}]
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    if ($type eq "port_type"){
      $arch{port_type} = $name;
    }
    if ($type eq "volume"){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($uid);
    }
    if (($type eq "port") && defined $configuration_data{$uid}{wwn}){
      # WWN for mapping
      $arch{wwn} = Xormon::formatWWN($configuration_data{$uid}{wwn});
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);

  }
  #print Dumper \@architecture;
  #foreach my $item (keys %{$data{conf}{device}}){
  #  $configuration_data{$st_serial}{$item} = $data{conf}{device}{$item};
  #}
  #print Dumper \$configuration_data{device}{$st_serial};

  #save configuration
  #$configuration_data{$cfg->{'uuid'}}{"label"} = $cfg->{'hostalias'};
  #event log
  $configuration_data{$cfg->{'uuid'}}{events} = encode_json(\@health_status);
  foreach my $item (keys %device_data){
    $configuration_data{$cfg->{'uuid'}}{$item} = $device_data{$item};
  }
  #$configuration_data{$cfg->{'uuid'}}{hardware} = encode_json(\@hardware);
  #print Dumper \$configuration_data{$cfg->{'uuid'}}{alerts};
  #print Dumper \@architecture;
  #print Dumper \%configuration_data;
  ### device information save


  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  #$logger->info("Task for configuration ".$cfg->{hostalias}." is done. Status: $status");

  #save architecture
  #print Dumper \@architecture;

  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");
  #print Dumper \@health_status_global;
  $rc = $xormon->saveStatus(\@health_status_global);
  my $status_health = get_status($rc, "health status");
  return ($status_architecture, $status_configuration, $status_health);
}


sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;
  my %uids_sfa;
  my %uids_cluster;

  my $test = Test->new($client);
  my @variables = ("hostcfg_id", "hostalias", "ipNode", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  $logger = XormonLogger->new( $st_type, $cfg->{hostcfg_id}, $cfg->{hostalias} );
  $storage_name = $cfg->{hostalias};
  $st_serial = $cfg->{hostcfg_id};
  data_structure();

  #### connection network test ####
  my $tcp_text = "";
  my ($code1, $text) = Xormon::conntest_tcp($cfg->{ipNode},$port);
  $tcp_text = $tcp_text . "$text\n";
  $test->addSection("API host network connection", $code1, $tcp_text);
  if ($code1 == 0){
    return $test->stop();
  }
  ### API authorization
  $api_host = $cfg->{'ipNode'};
  $cookie = cmd("rest/session","POST", 1, "session");

  if (!defined $cookie || $cookie eq ""){
    push(@rows_api_test, "Cookie, token or device_id is not defined\n");
    $test->addSection("API authorization", 0, \@rows_api_test);
    return $test->stop();
  }
  my $data = cmd("rest/nodes","GET",1,"nodes");
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){cmd("rest/session","DELETE", 1, "session_delete");$test->addSection("API authorization", 0, \@rows_api_test);return $test->stop();}
  foreach my $node (@{$data->{data}}){
    foreach my $metric (reverse sort keys %{$node}){
      if ($metric =~ m/cores|cpu|hostname|ip|os|status|timezone/){
        push(@rows_api_test,"$metric: " . $node->{$metric} . "\n");
      }
    }
  }
  $data = cmd("rest/noderegistartion/sfa","GET",1,"sfas");
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){$data->{data} = ();}
  my $line_sfa = "";
  my $line_cluster = "";
  foreach my $sfa (@{$data->{data}}){
    if (!defined $sfa->{nodelabel} || $sfa->{nodelabel} eq ""){next;}
    my $uid = Xormon::create_hash_key($st_serial."sfa".$sfa->{nodelabel});
    $uids_sfa{$uid} = $sfa->{nodelabel};
    if ($line_sfa eq ""){
      $line_sfa = $sfa->{nodelabel};
    }
    else{
      $line_sfa = $line_sfa . "," . $sfa->{nodelabel};
    }
  }
  $data = cmd("rest/es/configdata/clusters","GET",1,"cluster");
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){$data->{data} = ();}
  foreach my $cluster (@{$data->{data}}){
    my $name = $cluster->{clusterName};
    if (!defined $name || $name eq ""){next;}
    my $uid = Xormon::create_hash_key($st_serial."exascaler".$name);
    $uids_cluster{$uid} = $name;
    if ($line_cluster eq ""){
      $line_cluster = $name;
    }
    else{
      $line_cluster = $line_cluster . "," . $name;
    }
  }
  push(@rows_api_test,"SFA: $line_sfa\n");
  push(@rows_api_test,"Exascaler cluster: $line_cluster\n");
  $test->addSection("API authorization", 1, \@rows_api_test);

  ### API test data
  set_data_sfa();
  set_data_cluster();
  if (!(@sfas) && !(@clusters)){
    $test->addSection("API test data", 0, "Not found any SFA or Exascaler cluster. Performance data nok");
  }
  else{
    my $message_global = "Performance data nok";
    my $return_value = 0;
    push(@rows_api_test_data,"SFA:\n");
    foreach my $uid (keys %uids_sfa){
      if (defined $performance_data{$st_type}{sfa}{$uid}){
        push(@rows_api_test_data,$uids_sfa{$uid} . ": ok\n");
        $message_global = "Performance data ok";
        $return_value = 1;
      }
      else{
        push(@rows_api_test_data,$uids_sfa{$uid} . ": n/a\n");
      }
    }
    push(@rows_api_test_data,"\nExascaler cluster:\n");
    foreach my $uid (keys %uids_cluster){
      if (defined $performance_data{$st_type}{exascaler}{$uid}){
        push(@rows_api_test_data,$uids_cluster{$uid} . ": ok\n");
        $message_global = "Performance data ok";
        $return_value = 1;
      }
      else{
        push(@rows_api_test_data,$uids_cluster{$uid} . ": n/a\n");
      }
    }
    push(@rows_api_test_data,"\n$message_global\n");
    $test->addSection("API test data", $return_value, \@rows_api_test_data);
  }
  cmd("rest/session","DELETE", 1, "session_delete");
  return $test->stop();
}

sub data_structure {
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  #$debug_data_performance = "$directory/$active_minute";
  #$debug_data = "$directory/configuration";
  $debug_data = "$directory/$active_minute";
  $debug_data_configuration = "$directory/configuration";
  #print "$debug_data\n";
  $wrkdir = "$data_directory/data/$st_serial";

} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub check_connection{
  my $data = cmd("rest/metrics/metrics_description", "GET", 1, "metrics_description");
  if ( $data eq "" ) {
    if ($connection_test){
      push(@rows_api_test_data,"connection to $api_host on port $port has failed!\n");
    }
    else{
      $logger->error("connection to $api_host on port $port has failed!");
    }
    return 0;
  }
  return 1;
}

sub get_health_status_text {
  my $number = shift;

  if ( $number == 1 ) {
    return "normal";
  }
  if ( $number == 0 ) {
    return "unknown";
  }
  if ( $number == 2 ) {
    return "faulty";
  }
  if ( $number == 3 ) {
    return "Pre-Fail";
  }
  if ( $number == 9 ) {
    return "inconsistent";
  }
  if ( $number == 4 ) {
    return "partially damaged";
  }
  if ( $number == 12 ) {
    return "low battery";
  }
  if ( $number == 5 ) {
    return "degraded";
  }

  return $number;

}


sub get_global_timestamp{
  my $act_timestamp = time();
  (my $sec,my $min,my $hour,my $mday,my $mon, my $year,my $wday,my $yday,my $isdst) = localtime();
  return $act_timestamp - $sec;

}

sub set_data_filesystem{
  my $section = "fs";
  my %metrics;
  $metrics{"ost_kbytesinfo_total"} = "capacity_total";
  $metrics{"ost_kbytesinfo_used"} = "capacity_used";
  foreach my $cluster (@clusters){
    my $data = cmd("rest/es/configdata/$cluster/filesystems","GET",1,"$cluster-filesystems");
    #print Dumper \$data;
    if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){return;}
    foreach my $filesystem (@{$data->{data}}){
      my $name = $filesystem->{name};
      if (!defined $name || $name eq ""){next;}
      my %params;
      $params{"fs_name"} = $name;
      $params{"cluster"} = $cluster; ### change uid
      my $uid = Xormon::create_hash_key($st_serial.$cluster.$section.$name);
      $configuration_data{$uid}{label} = $name;
      $configuration_data{$uid}{TYPE} = $section;
      my $uid_cluster = Xormon::create_hash_key($st_serial."exascaler".$cluster);
      if (defined $configuration_data{$uid_cluster}){
        push(@{$configuration_data{$uid}{parents}},$uid_cluster);
      }
      my $json = get_json(\%params,\%metrics,$last_minute);
      my $filename = "fs_data1_$cluster" . "_$name";
      my $data = cmd("datapoints/query/","POST",,1,$filename,$json);
      my %data = get_performance_data($data,$section,-1,\%metrics,"avg");
      foreach my $time (sort { $a <=> $b } keys %data){
        my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
        foreach my $item (keys %{$data{$time}}){
          if (Xormon::isdigit($timestamp_item)){
            $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
            if ($item =~ m/capacity/){
              $configuration_data{$uid}{$item} = $data{$time}{$item};
            }
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
            if ($item =~ m/capacity/){
              $configuration_data{$uid}{$item} = $data{$time}{$item};
            }
          }
        }
      }
      my %metrics2;
      foreach my $metric (("read","write")){
        $metrics2{"ost_stats_samples"} = "$metric" . "_io";
        $metrics2{"ost_stats_bytes"} = $metric;
        $params{optype} = $metric;
        $json = get_json(\%params,\%metrics2,$last_minute);
        my $filename = "fs_data2_$cluster" . "_$name" . "_$metric";
        $data = cmd("datapoints/query/","POST",1,$filename,$json);
        %data = get_performance_data($data,$section,-1,\%metrics2,"avg");
        foreach my $time (sort { $a <=> $b } keys %data){
          my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
          foreach my $item (keys %{$data{$time}}){
            if (Xormon::isdigit($timestamp_item)){
              $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
            }
            else{
              $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
            }
          }
        }
      }
    }
  }
  foreach my $filesystem (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$filesystem}}){
      $performance_data{$st_type}{$section}{$filesystem}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io",$performance_data{$st_type}{$section}{$filesystem}{$timestamp});
      $performance_data{$st_type}{$section}{$filesystem}{$timestamp}{"data_rate"} = get_metrics_sum("read","write",$performance_data{$st_type}{$section}{$filesystem}{$timestamp});
      $performance_data{$st_type}{$section}{$filesystem}{$timestamp}{"memory_total"} = get_metrics_sum("memory_used","memory_free",$performance_data{$st_type}{$section}{$filesystem}{$timestamp});
      $performance_data{$st_type}{$section}{$filesystem}{$timestamp}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$filesystem}{$timestamp});
      $configuration_data{$filesystem}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$filesystem}{$timestamp});
     }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}
sub set_data_oss{
  my $section = "oss";
  my %metrics;
  $metrics{"aggregation.cpu-average.cpu.system"} = "cpu_system";
  $metrics{"aggregation.cpu-average.cpu.user"} = "cpu_user";
  $metrics{"memory.used.memory"} = "memory_used";
  $metrics{"memory.free.memory"} = "memory_free";
  #$metrics{"memory.buffered.memory"} = "memory_buffer";
  $metrics{"ost_kbytesinfo_total"} = "capacity_total";
  $metrics{"ost_kbytesinfo_used"} = "capacity_used";
  foreach my $cluster (@clusters){
    my $data = cmd("rest/es/configdata/$cluster/nodes","GET",1,"$cluster-nodes");
    if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){return;}
    #print Dumper \$data;
    foreach my $oss (@{$data->{data}}){
      my $name = $oss->{nodeName};
      if (!defined $name || $name eq ""){next;}
      push(@nodes,$name);
      my %params;
      $params{"fqdn"} = $name;
      $params{"cluster"} = $cluster; ### change uid
      my $uid = Xormon::create_hash_key($st_serial.$cluster.$section.$name);
      $configuration_data{$uid}{label} = $name;
      $configuration_data{$uid}{TYPE} = $section;
      my $uid_cluster = Xormon::create_hash_key($st_serial."exascaler".$cluster);
      if (defined $configuration_data{$uid_cluster}){
        push(@{$configuration_data{$uid}{parents}},$uid_cluster);
      }
      my $json = get_json(\%params,\%metrics,$last_minute);
      my $filename = "oss_data1_" . $cluster . "_$name";
      my $data = cmd("datapoints/query/","POST",1,$filename,$json);
      my %data = get_performance_data($data,$section,-1,\%metrics,"avg");
      foreach my $time (sort { $a <=> $b } keys %data){
        my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
        foreach my $item (keys %{$data{$time}}){
          if (Xormon::isdigit($timestamp_item)){
            $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
            if ($item =~ m/capacity/){
              $configuration_data{$uid}{$item} = $data{$time}{$item};
            }
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
            if ($item =~ m/capacity/){
              $configuration_data{$uid}{$item} = $data{$time}{$item};
            }
          }
        }
      }
      my %metrics2;
      foreach my $metric (("read","write")){
        $metrics2{"ost_stats_samples"} = "$metric" . "_io";
        $metrics2{"ost_stats_bytes"} = $metric;
        $params{optype} = $metric;
        $json = get_json(\%params,\%metrics2,$last_minute);
        my $filename = "oss_data2_" . $cluster . "_$name" . "$metric";
        $data = cmd("datapoints/query/","POST",1,$filename,$json);
        #print Dumper \$data;
        %data = get_performance_data($data,$section,-1,\%metrics2,"avg");
        foreach my $time (sort { $a <=> $b } keys %data){
          my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
          foreach my $item (keys %{$data{$time}}){
            if (Xormon::isdigit($timestamp_item)){
              $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
            }
            else{
              $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
            }
          }
        }
      }
    }
  }
  foreach my $oss (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$oss}}){
      $performance_data{$st_type}{$section}{$oss}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io",$performance_data{$st_type}{$section}{$oss}{$timestamp});
      $performance_data{$st_type}{$section}{$oss}{$timestamp}{"data_rate"} = get_metrics_sum("read","write",$performance_data{$st_type}{$section}{$oss}{$timestamp});
      $performance_data{$st_type}{$section}{$oss}{$timestamp}{"memory_total"} = get_metrics_sum("memory_used","memory_free",$performance_data{$st_type}{$section}{$oss}{$timestamp});
      $performance_data{$st_type}{$section}{$oss}{$timestamp}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$oss}{$timestamp});
      $configuration_data{$oss}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$oss}{$timestamp});
      if (Xormon::isdigit( $performance_data{$st_type}{$section}{$oss}{$timestamp}{"memory_total"}) && Xormon::isdigit( $performance_data{$st_type}{$section}{$oss}{$timestamp}{"memory_used"}) && Xormon::isdigit( $performance_data{$st_type}{$section}{$oss}{$timestamp}{"memory_total"}) > 0){
        $performance_data{$st_type}{$section}{$oss}{$timestamp}{"memory_used_percent"} = round(($performance_data{$st_type}{$section}{$oss}{$timestamp}{"memory_used"} / $performance_data{$st_type}{$section}{$oss}{$timestamp}{"memory_total"}) * 100,2);
      }

     }
  }
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_cluster{
  my $section = "exascaler";
  my %metrics;
  #$metrics{"ost_stats_samples"} = "read_io";
  #$metrics{"ost_stats_bytes"} = "read";
  $metrics{"aggregation.cpu-average.cpu.system"} = "cpu_system";
  $metrics{"aggregation.cpu-average.cpu.user"} = "cpu_user";
  $metrics{"memory.used.memory"} = "memory_used";
  $metrics{"memory.free.memory"} = "memory_free";
  $metrics{"memory.buffered.memory"} = "memory_buffer";
  $metrics{"ost_kbytesinfo_total"} = "capacity_total";
  $metrics{"ost_kbytesinfo_used"} = "capacity_used";
  my $data = cmd("rest/es/configdata/clusters","GET",1,"cluster");
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){return;}
  #print Dumper $data;
  foreach my $cluster (@{$data->{data}}){
    my $name = $cluster->{clusterName};
    my $version = $cluster->{version};
    if (!defined $name || $name eq ""){next;}
    if (!defined $version){$version = "";}
    push(@clusters,$name);
    my %params;
    $params{"cluster"} = $name;
    my $uid = Xormon::create_hash_key($st_serial.$section.$name);
    $configuration_data{$uid}{label} = $name;
    $configuration_data{$uid}{TYPE} = $section;
    $configuration_data{$uid}{version} = $version;
    #$params{optype} = "read";
    my $json = get_json(\%params,\%metrics,$last_minute);
    my $data = cmd("datapoints/query/","POST",1,"cluster_data_$name",$json);
    #print Dumper $data;
    #print "$json\n";
    my %data = get_performance_data($data,$section,-1,\%metrics,"avg");
    #print Dumper \%data;
    
    foreach my $time (sort { $a <=> $b } keys %data){
      my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
      foreach my $item (keys %{$data{$time}}){
        if (Xormon::isdigit($timestamp_item)){
          $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
        }
        else{
          $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
        }
      }
    } 
    my %metrics2;
    foreach my $metric (("read","write")){
      $metrics2{"ost_stats_samples"} = "$metric" . "_io";
      $metrics2{"ost_stats_bytes"} = $metric;
      $params{optype} = $metric;
      $json = get_json(\%params,\%metrics2,$last_minute);
      $data = cmd("datapoints/query/","POST",1,"cluster_data_$name" . "_$metric",$json);
      %data = get_performance_data($data,$section,-1,\%metrics2,"avg");
      foreach my $time (sort { $a <=> $b } keys %data){
        my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
        foreach my $item (keys %{$data{$time}}){
          if (Xormon::isdigit($timestamp_item)){
            $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
          }
          else{
            $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
          }
        }
      }
    }
  }
  $data = cmd("rest/noderegistartion/es","GET",1,"es_detail");
  if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){return;}
  foreach my $es (@{$data->{data}}){
    if (!defined $es->{nodelabel}){next;}
    my $uid = Xormon::create_hash_key($st_serial.$section.$es->{nodelabel});
    if (defined $configuration_data{$uid} && defined $es->{pmcip}){
      $configuration_data{$uid}{"ip"} = $es->{pmcip};
      #print Dumper \$configuration_data{$uid};
    }
  }
  foreach my $cluster (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$cluster}}){
      $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io",$performance_data{$st_type}{$section}{$cluster}{$timestamp});
      $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"data_rate"} = get_metrics_sum("read","write",$performance_data{$st_type}{$section}{$cluster}{$timestamp});
      $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"memory_total"} = get_metrics_sum("memory_used","memory_free",$performance_data{$st_type}{$section}{$cluster}{$timestamp});
      $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$cluster}{$timestamp});
      if (Xormon::isdigit( $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"memory_total"}) && Xormon::isdigit( $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"memory_used"}) && Xormon::isdigit( $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"memory_total"}) > 0){
        $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"memory_used_percent"} = round(($performance_data{$st_type}{$section}{$cluster}{$timestamp}{"memory_used"} / $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"memory_total"}) * 100,2);
      }
      if (Xormon::isdigit($performance_data{$st_type}{$section}{$cluster}{$timestamp}{"capacity_total"})){
        $configuration_data{$cluster}{"capacity_total"} = $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"capacity_total"};
      }
      if (Xormon::isdigit($performance_data{$st_type}{$section}{$cluster}{$timestamp}{"capacity_used"})){
        $configuration_data{$cluster}{"capacity_used"} = $performance_data{$st_type}{$section}{$cluster}{$timestamp}{"capacity_used"};
      }
      $configuration_data{$cluster}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$cluster}{$timestamp});

    }
  }
  #calculate_performance_data($section,\%metrics);
  #print Dumper \$performance_data{$st_type}{$section};
}

sub set_data_ost{
  my $section = "ost";
  my @uids = ();
  my %metrics;
  $metrics{"ost_kbytesinfo_total"} = "capacity_total";
  $metrics{"ost_kbytesinfo_used"} = "capacity_used";
  foreach my $cluster (@clusters){
    foreach my $node (@nodes){
      my $data = cmd("rest/es/configdata/$cluster/$node/node/disks","GET",1,"$cluster-$node-ost");
      if (!defined $data || $data eq "" || !defined $data->{data} || ref($data->{data}) ne "ARRAY"){next;}
      foreach my $filesystem (@{$data->{data}}){
        my $name = $filesystem->{name};
        if (!defined $name || $name eq ""){next;}
        my %params;
        $params{"ost_index"} = uc($name); ### I do not know why upercase
        $params{"cluster"} = $cluster; ### change uid
        my $uid = Xormon::create_hash_key($st_serial.$cluster.$section.$name);
        $configuration_data{$uid}{label} = $name;
        $configuration_data{$uid}{TYPE} = $section;
        push(@uids,$uid);
        my $uid_cluster = Xormon::create_hash_key($st_serial."exascaler".$cluster);
        if (defined $configuration_data{$uid_cluster}){
          push(@{$configuration_data{$uid}{parents}},$uid_cluster);
        }
        my $json = get_json(\%params,\%metrics,$last_minute);
        my $filename = "ost_data1_" . $cluster . "_$name";
        my $data = cmd("datapoints/query/","POST",1,$filename,$json);
        my %data = get_performance_data($data,$section,-1,\%metrics,"avg");
        foreach my $time (sort { $a <=> $b } keys %data){
          my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
          foreach my $item (keys %{$data{$time}}){
            if (Xormon::isdigit($timestamp_item)){
              $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
              if ($item =~ m/capacity/){
                $configuration_data{$uid}{$item} = $data{$time}{$item};
              }
            }
            else{
              $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
              if ($item =~ m/capacity/){
                $configuration_data{$uid}{$item} = $data{$time}{$item};
              }
            }
          }
        }
        my %metrics2;
        foreach my $metric (("read","write")){
          $metrics2{"ost_stats_samples"} = "$metric" . "_io";
          $metrics2{"ost_stats_bytes"} = $metric;
          $params{optype} = $metric;
          my $json = get_json(\%params,\%metrics2,$last_minute);
          my $filename = "ost_data_$cluster" . "_$name" . "_$metric";
          my $data = cmd("datapoints/query/","POST",1,$filename,$json);
          #print "$json\n";
          my %data = get_performance_data($data,$section,-1,\%metrics2,"avg");
          foreach my $time (sort { $a <=> $b } keys %data){
            my $timestamp_item = (keys %{$performance_data{$st_type}{$section}{$uid}})[0];
            foreach my $item (keys %{$data{$time}}){
              if (Xormon::isdigit($timestamp_item)){
                $performance_data{$st_type}{$section}{$uid}{$timestamp_item}{$item} = $data{$time}{$item};
              }
              else{
                $performance_data{$st_type}{$section}{$uid}{$time}{$item} = $data{$time}{$item};
              }
            }
          }
        }
      }
    }
  }
  foreach my $ost (keys %{$performance_data{$st_type}{$section}}){
    foreach my $timestamp (keys %{$performance_data{$st_type}{$section}{$ost}}){
      $performance_data{$st_type}{$section}{$ost}{$timestamp}{"io_rate"} = get_metrics_sum("read_io","write_io",$performance_data{$st_type}{$section}{$ost}{$timestamp});
      $performance_data{$st_type}{$section}{$ost}{$timestamp}{"data_rate"} = get_metrics_sum("read","write",$performance_data{$st_type}{$section}{$ost}{$timestamp});
      $performance_data{$st_type}{$section}{$ost}{$timestamp}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$ost}{$timestamp});
      $configuration_data{$ost}{"capacity_free"} = get_diference_value("capacity_total", "capacity_used", $performance_data{$st_type}{$section}{$ost}{$timestamp});
    }
  }
}






1;
