package GoogleCloud;

use strict;
use warnings;

use HTTP::Request::Common;
use LWP;
use Data::Dumper;
use JSON;
use POSIX qw(strftime ceil);
use Digest::MD5 qw(md5_hex);
use Date::Parse;
use Time::Local;



my @metrics             = ( { 'name' => 'cpu_util',    'url' => 'compute.googleapis.com/instance/cpu/utilization',          'type' => 'doubleValue' }, { 'name' => 'read_bytes',        'url' => 'compute.googleapis.com/instance/disk/read_bytes_count',         'type' => 'int64Value' }, { 'name' => 'write_bytes', 'url' => 'compute.googleapis.com/instance/disk/write_bytes_count', 'type' => 'int64Value' }, { 'name' => 'read_ops', 'url' => 'compute.googleapis.com/instance/disk/read_ops_count', 'type' => 'int64Value' }, { 'name' => 'write_ops', 'url' => 'compute.googleapis.com/instance/disk/write_ops_count', 'type' => 'int64Value' }, { 'name' => 'received_bytes', 'url' => 'compute.googleapis.com/instance/network/received_bytes_count', 'type' => 'int64Value' }, { 'name' => 'sent_bytes', 'url' => 'compute.googleapis.com/instance/network/sent_bytes_count', 'type' => 'int64Value' } );
my @metrics_agent       = ( { 'name' => 'mem_used',    'url' => 'agent.googleapis.com/memory/bytes_used',                   'type' => 'doubleValue' }, { 'name' => 'mem_usage',         'url' => 'agent.googleapis.com/memory/percent_used',                      'type' => 'doubleValue' } );
my @metrics_db          = ( { 'name' => 'cpu_util',    'url' => 'cloudsql.googleapis.com/database/cpu/utilization',         'type' => 'doubleValue' }, { 'name' => 'disk_used',         'url' => 'cloudsql.googleapis.com/database/disk/bytes_used',              'type' => 'int64Value' }, { 'name' => 'disk_quota', 'url' => 'cloudsql.googleapis.com/database/disk/quota', 'type' => 'int64Value' }, { 'name' => 'read_ops', 'url' => 'cloudsql.googleapis.com/database/disk/read_ops_count', 'type' => 'int64Value' }, { 'name' => 'write_ops', 'url' => 'cloudsql.googleapis.com/database/disk/write_ops_count', 'type' => 'int64Value' }, { 'name' => 'mem_total', 'url' => 'cloudsql.googleapis.com/database/memory/quota', 'type' => 'int64Value' }, { 'name' => 'mem_used', 'url' => 'cloudsql.googleapis.com/database/memory/usage', 'type' => 'int64Value' }, { 'name' => 'mem_total', 'url' => 'cloudsql.googleapis.com/database/memory/quota', 'type' => 'int64Value' }, { 'name' => 'connections', 'url' => 'cloudsql.googleapis.com/database/network/connections', 'type' => 'int64Value' }, { 'name' => 'received_bytes', 'url' => 'cloudsql.googleapis.com/database/network/received_bytes_count', 'type' => 'int64Value' }, { 'name' => 'sent_bytes', 'url' => 'cloudsql.googleapis.com/database/network/sent_bytes_count', 'type' => 'int64Value' } );
my @metrics_db_mysql    = ( { 'name' => 'innodb_read', 'url' => 'cloudsql.googleapis.com/database/mysql/innodb_pages_read', 'type' => 'int64Value' },  { 'name' => 'innodb_write',      'url' => 'cloudsql.googleapis.com/database/mysql/innodb_pages_written',   'type' => 'int64Value' }, { 'name' => 'queries', 'url' => 'cloudsql.googleapis.com/database/mysql/queries', 'type' => 'int64Value' }, { 'name' => 'questions', 'url' => 'cloudsql.googleapis.com/database/mysql/questions', 'type' => 'int64Value' }, { 'name' => 'innodb_buffer_free', 'url' => 'cloudsql.googleapis.com/database/mysql/innodb_buffer_pool_pages_free', 'type' => 'int64Value' }, { 'name' => 'innodb_buffer_total', 'url' => 'cloudsql.googleapis.com/database/mysql/innodb_buffer_pool_pages_total', 'type' => 'int64Value' }, { 'name' => 'innodb_data_fsyncs', 'url' => 'cloudsql.googleapis.com/database/mysql/innodb_data_fsyncs', 'type' => 'int64Value' }, { 'name' => 'innodb_os_fsyncs', 'url' => 'cloudsql.googleapis.com/database/mysql/innodb_os_log_fsyncs', 'type' => 'int64Value' } );
my @metrics_db_postgres = ( { 'name' => 'connections', 'url' => 'cloudsql.googleapis.com/database/postgresql/num_backends', 'type' => 'int64Value' },  { 'name' => 'transaction_count', 'url' => 'cloudsql.googleapis.com/database/postgresql/transaction_count', 'type' => 'int64Value' } );

my @metrics_to_divide   = ("sent_bytes", "recieved_bytes", "read_ops", "write_ops", "read_bytes", "write_bytes", "innodb_write", "innodb_read");

sub new {
  my ( $self, $name, $print, $uuid, @credentials ) = @_;

  my $o = {};

  my $cred_path = "./credfiles";
  my $cred_file = $cred_path . "/gcloud_" . $name . "_credentials.json";

  unless ( -d $cred_path ) {
    mkdir( $cred_path, 0755 ) || warn( localtime() . ": Cannot mkdir $cred_path: $!" . __FILE__ . ':' . __LINE__ );
  }

  open my $cf, ">", $cred_file;
  print $cf JSON->new->pretty->encode($credentials[0][0]);
  close $cf;

  $ENV{GOOGLE_APPLICATION_CREDENTIALS} = $cred_file;
  my $result  = `export GOOGLE_APPLICATION_CREDENTIALS="$cred_file"`;
  my $token   = `gcloud auth application-default print-access-token`;
  my $project = $credentials[0][0]{project_id};

  $o->{uuid}            = defined $uuid ? $uuid : $name;
  $o->{credentials}     = $credentials[0];
  $o->{token}           = $token;
  $o->{project}         = $project;
  $o->{zones}           = &getZones( $token, $project );
  $o->{name}            = $name;
  $o->{current_account} = 0;

  bless $o, $self;
  return $o;
}

sub testToken {
  my ($self) = @_;
  return $self->{token};
}

sub setNextServiceAccount {
  my ($self) = @_;

  my $is_last = 0;

  if( scalar( @{ $self->{credentials} } ) == $self->{current_account}){
    $is_last = 1;
    return $is_last;
  }

  my $cred_path = "./credfiles";
  my $cred_file = $cred_path . "/gcloud_" . $self->{name} . "_credentials.json";

  unless ( -d $cred_path ) {
    mkdir( $cred_path, 0755 ) || warn( localtime() . ": Cannot mkdir $cred_path: $!" . __FILE__ . ':' . __LINE__ );
  }

  open my $cf, ">", $cred_file;
  print $cf JSON->new->pretty->encode($self->{credentials}[$self->{current_account}]);
  close $cf;

  $ENV{GOOGLE_APPLICATION_CREDENTIALS} = $cred_file;
  my $result  = `export GOOGLE_APPLICATION_CREDENTIALS="$cred_file"`;
  my $token   = `gcloud auth application-default print-access-token`;
  my $project = $self->{credentials}[$self->{current_account}]->{project_id};

  $self->{token}   = $token;
  $self->{project} = $project;
  $self->{zones}   = &getZones( $token, $project );

  $self->{current_account}++;

  return $is_last;

}

sub resetServiceAccounts {
  my ($self) =  @_;
  $self->{current_account} = 0;
}

sub getZones {
  my ( $token, $project ) = @_;

  my @zones;

  my $response = gcloudRequest( "https://compute.googleapis.com/compute/v1/projects/$project/regions", $token );
  for ( @{ $response->{items} } ) {
    my $region = $_;
    my $add    = 0;

    for ( @{ $region->{quotas} } ) {
      my $quota = $_;

      if ( $quota->{metric} eq "INSTANCES" && $quota->{usage} >= 1 ) {
        $add = 1;
      }
    }

    if ( $add eq "1" ) {
      for ( @{ $region->{zones} } ) {
        my $zone     = $_;
        my @splitted = split( /\//, $zone );

        #print "\n$splitted[8]";
        push( @zones, $splitted[8] );
      }
    }

  }

  return \@zones;
}

sub listInstances {
  my ( $self, $print ) = @_;

  my %data;

  $data{specification}{hostcfg_uuid}{ $self->{uuid} } = $self->{uuid};
  
  while($self->setNextServiceAccount() eq 0){

    for ( @{ $self->{zones} } ) {
      my $region = $_;
      my $region_id = md5_hex($region.$self->{uuid});

      my $response = gcloudRequest( "https://compute.googleapis.com/compute/v1/projects/$self->{project}/zones/$region/instances", $self->{token} );

      for ( @{ $response->{items} } ) {
        my $item = $_;
        $item->{id} = md5_hex($item->{id}.$self->{uuid});
        $data{specification}{compute}{ $item->{id} }{status}            = $item->{status};
        $data{specification}{compute}{ $item->{id} }{name}              = $item->{name};
        $data{specification}{compute}{ $item->{id} }{cpuPlatform}       = $item->{cpuPlatform};
        $data{specification}{compute}{ $item->{id} }{machineType}       = $item->{machineType};
        $data{specification}{compute}{ $item->{id} }{creationTimestamp} = $item->{creationTimestamp};
        $data{specification}{compute}{ $item->{id} }{region}            = $region;
        $data{specification}{compute}{ $item->{id} }{ip}                = $item->{networkInterfaces}->[0]->{accessConfigs}->[0]->{natIP};
        $data{specification}{compute}{ $item->{id} }{hostcfg_uuid}      = $self->{uuid};

        if ( !defined $data{specification}{region}{$region_id} ) {
          $data{specification}{region}{$region_id}{running} = 0;
          $data{specification}{region}{$region_id}{stopped} = 0;
        }

        if ( $item->{status} eq "RUNNING" ) {
          if ( defined $data{specification}{region}{$region_id}{running} ) {
            $data{specification}{region}{$region_id}{running} = $data{specification}{region}{$region_id}{running} + 1;
          }
          else {
            $data{specification}{region}{$region_id}{running} = 1;
          }
        }
        else {
          if ( defined $data{specification}{region}{$region_id}{stopped} ) {
            $data{specification}{region}{$region_id}{stopped} = $data{specification}{region}{$region_id}{stopped} + 1;
          }
          else {
            $data{specification}{region}{$region_id}{stopped} = 1;
          }
        }

        my $diskSize = 0;
        for ( @{ $item->{disks} } ) {
          my $disk = $_;
          $diskSize = $diskSize + $disk->{diskSizeGb};
        }

        $data{specification}{compute}{ $item->{id} }{diskSize} = $diskSize;

        my @adresa = split /\//, $item->{machineType};

        $data{specification}{compute}{ $item->{id} }{size} = $adresa[10];

        $data{label}{compute}{ $item->{id} } = $item->{name};

        if ( exists $data{architecture}{region_compute}{$region_id}[0] ) {
          push( @{ $data{architecture}{region_compute}{$region_id} }, $item->{id} );
        }
        else {
          $data{architecture}{region_compute}{$region_id}[0] = $item->{id};
        }
        $data{label}{compute_region}{$region_id} = $region;
      }

    }

    #databases
    
    my $cred_file = "./credfiles/gcloud_" . $self->{name} . "_credentials.json";
    my $result  = `gcloud auth login --cred-file="$cred_file" -q`;
    
    my $result2  = `gcloud config set project $self->{project}`;
    $ENV{CLOUDSDK_CORE_PROJECT} = $self->{project};
    my $databases_plain = `gcloud sql instances list --format=json -q`;

    if($databases_plain ne ""){
      my $databases = decode_json($databases_plain);

      #print Dumper($databases);

      for ( @{$databases} ) {
        my $database = $_;

        my $type = "undef";
        if ( index( $database->{databaseVersion}, "MYSQL" ) != -1 ) {
          $type = "mysql";
        }
        elsif ( index( $database->{databaseVersion}, "POSTGRES" ) != -1 ) {
          $type = "postgres";
        }

        my $database_id = "$database->{project}:$database->{name}";
        $database_id =~ s/:/__/g;
        $database_id = md5_hex($database_id.$self->{uuid});

        $data{specification}{database}{$database_id}{name}            = $database->{name};
        $data{specification}{database}{$database_id}{region}          = $database->{region};
        $data{specification}{database}{$database_id}{databaseVersion} = $database->{databaseVersion};
        $data{specification}{database}{$database_id}{dataDiskSizeGb}  = $database->{settings}->{dataDiskSizeGb};
        $data{specification}{database}{$database_id}{dataDiskType}    = $database->{settings}->{dataDiskType};
        $data{specification}{database}{$database_id}{size}            = $database->{settings}->{tier};
        $data{specification}{database}{$database_id}{status}          = $database->{state};
        $data{specification}{database}{$database_id}{hostcfg_uuid}    = $self->{uuid};

        $data{engines}{$database_id} = $type;

        for ( @{ $database->{ipAddresses} } ) {
          my $ip = $_;
          if ( $ip->{type} eq "PRIMARY" ) {
            $data{specification}{database}{$database_id}{ip} = $ip->{ipAddress};
          }
        }

        my $database_region_id = md5_hex($database->{gceZone}.$self->{uuid});

        if ( exists $data{architecture}{region_database}{ $database_region_id }[0] ) {
          push( @{ $data{architecture}{region_database}{ $database_region_id } }, $database_id );
        }
        else {
          $data{architecture}{region_database}{ $database_region_id }[0] = $database_id;
        }

        $data{label}{database}{$database_id} = $database->{name};
        $data{label}{database_region}{$database_region_id} = $database->{gceZone};

      }
    }
  }

  $self->resetServiceAccounts();

  return \%data;

}

sub testInstances {
  my ($self) = @_;

  my %data;

  my $error = 0;
  for ( @{ $self->{zones} } ) {
    my $region = $_;

    my $response = gcloudRequestTest( "https://compute.googleapis.com/compute/v1/projects/$self->{project}/zones/$region/instances", $self->{token} );

    if ( $response eq "1" ) {
      $error = 1;
    }

  }
  return $error;

}

sub reformatTime {
  my ($time) = @_;
  my %data;
  foreach my $time_key ( keys %{$time} ) {
    $data{$time_key} = sprintf( "%02d", $time->{$time_key} );
  }
  return \%data;
}

sub getMetrics {
  my ($self, $start, $end) = @_;

  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = gmtime($start);
  $year += 1900;
  $mon  += 1;
  my $reformated = reformatTime( { 'month' => $mon, 'day' => $mday, 'hour' => $hour, 'min' => $min, 'sec' => $sec } );
  my $start_time = "$year-$reformated->{month}-$reformated->{day}T$reformated->{hour}:$reformated->{min}:$reformated->{sec}";

  ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = gmtime($end);
  $year += 1900;
  $mon  += 1;
  $reformated = reformatTime( { 'month' => $mon, 'day' => $mday, 'hour' => $hour, 'min' => $min, 'sec' => $sec } );
  my $end_time = "$year-$reformated->{month}-$reformated->{day}T$reformated->{hour}:$reformated->{min}:$reformated->{sec}";

  my %agent;
  my %data;

  while($self->setNextServiceAccount() eq 0){

    for (@metrics) {
      my $metric = $_;

      my $response = gcloudRequest( "https://monitoring.googleapis.com/v3/projects/$self->{project}/timeSeries?filter=metric.type=\"" . $metric->{url} . "\"&interval.endTime=" . $end_time . "Z&interval.startTime=" . $start_time . "Z", $self->{token} );

      for ( @{ $response->{timeSeries} } ) {
        my $timeserie = $_;

        for ( @{ $timeserie->{points} } ) {
          my $point = $_;

          my $item_id = md5_hex($timeserie->{resource}->{labels}->{instance_id}.$self->{uuid});
          my $pretty_time = str2time( $point->{interval}->{endTime} );
          $pretty_time                                                  = int($pretty_time);
          $pretty_time                                                  = $pretty_time - ( $pretty_time % 60 );
          $agent{ $timeserie->{resource}->{labels}->{instance_id} }     = 0;
          if($metric->{name} eq "cpu_util"){
            $data{gcloud}{compute}{ $item_id }{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } * 100;
          }
          elsif ( grep( /^$metric->{name}$/,@metrics_to_divide)){
            $data{gcloud}{compute}{ $item_id }{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } / 60;
          }
          else{
            $data{gcloud}{compute}{ $item_id }{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} };

          }
        }
      }
    }

    #all databases
    for (@metrics_db) {
      my $metric = $_;


      my $response = gcloudRequest( "https://monitoring.googleapis.com/v3/projects/$self->{project}/timeSeries?filter=metric.type=\"" . $metric->{url} . "\"&interval.endTime=" . $end_time . "Z&interval.startTime=" . $start_time . "Z", $self->{token} );

      #print Dumper($response);

      for ( @{ $response->{timeSeries} } ) {
        my $timeserie = $_;

        for ( @{ $timeserie->{points} } ) {
          my $point = $_;

          my $pretty_time = str2time( $point->{interval}->{endTime} );
          $pretty_time = int($pretty_time);
          $pretty_time = $pretty_time - ( $pretty_time % 60 );

          my $database_id = $timeserie->{resource}->{labels}->{database_id};
          $database_id =~ s/:/__/g;
          $database_id = md5_hex($database_id.$self->{uuid});

          if($metric->{name} eq "cpu_util"){
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } * 100;
          }
          elsif ( grep( /^$metric->{name}$/,@metrics_to_divide)){
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } / 60;
          }
          else{
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} };
          }
        }
      }
    }

    #mysql
    for (@metrics_db_mysql) {
      my $metric = $_;

      my $response = gcloudRequest( "https://monitoring.googleapis.com/v3/projects/$self->{project}/timeSeries?filter=metric.type=\"" . $metric->{url} . "\"&interval.endTime=" . $end_time . "Z&interval.startTime=" . $start_time . "Z", $self->{token} );

      #print Dumper($response);

      for ( @{ $response->{timeSeries} } ) {
        my $timeserie = $_;

        for ( @{ $timeserie->{points} } ) {
          my $point = $_;

          my $pretty_time = str2time( $point->{interval}->{endTime} );
          $pretty_time = int($pretty_time);
          $pretty_time = $pretty_time - ( $pretty_time % 60 );

          my $database_id = $timeserie->{resource}->{labels}->{database_id};
          $database_id =~ s/:/__/g;
          $database_id = md5_hex($database_id.$self->{uuid});

          if($metric->{name} eq "cpu_util"){
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } * 100;
          }
          elsif ( grep( /^$metric->{name}$/,@metrics_to_divide)){
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } / 60;
          }
          else{
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} };
          }
        }
      }
    }

    #postgres
    for (@metrics_db_postgres) {
      my $metric = $_;

      my $response = gcloudRequest( "https://monitoring.googleapis.com/v3/projects/$self->{project}/timeSeries?filter=metric.type=\"" . $metric->{url} . "\"&interval.endTime=" . $end_time . "Z&interval.startTime=" . $start_time . "Z", $self->{token} );

      #print Dumper($response);

      for ( @{ $response->{timeSeries} } ) {
        my $timeserie = $_;

        for ( @{ $timeserie->{points} } ) {
          my $point = $_;

          my $pretty_time = str2time( $point->{interval}->{endTime} );
          $pretty_time = int($pretty_time);
          $pretty_time = $pretty_time - ( $pretty_time % 60 );

          my $database_id = $timeserie->{resource}->{labels}->{database_id};
          $database_id =~ s/:/__/g;
          $database_id = md5_hex($database_id.$self->{uuid});


          if($metric->{name} eq "cpu_util"){
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } * 100;
          }
          elsif ( grep( /^$metric->{name}$/,@metrics_to_divide)){
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} } / 60;
          }
          else{
            $data{gcloud}{database}{$database_id}{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} };
          }
        }
      }
    }

    #agent process
    ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = gmtime( $start - 180 );
    $year += 1900;
    $mon  += 1;
    $reformated = reformatTime( { 'month' => $mon, 'day' => $mday, 'hour' => $hour, 'min' => $min, 'sec' => $sec } );
    $start_time = "$year-$reformated->{month}-$reformated->{day}T$reformated->{hour}:$reformated->{min}:$reformated->{sec}";

    ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = gmtime( $end + 60 );
    $year += 1900;
    $mon  += 1;
    $reformated = reformatTime( { 'month' => $mon, 'day' => $mday, 'hour' => $hour, 'min' => $min, 'sec' => $sec } );
    $end_time   = "$year-$reformated->{month}-$reformated->{day}T$reformated->{hour}:$reformated->{min}:$reformated->{sec}";

    my $response = gcloudRequest( "https://monitoring.googleapis.com/v3/projects/$self->{project}/timeSeries?filter=metric.type=\"agent.googleapis.com/processes/count_by_state\"&interval.endTime=" . $end_time . "Z&interval.startTime=" . $start_time . "Z", $self->{token} );

    for ( @{ $response->{timeSeries} } ) {
      my $timeserie = $_;

      for ( @{ $timeserie->{points} } ) {
        my $point = $_;

        my $pretty_time = str2time( $point->{interval}->{endTime} );
        $pretty_time                                              = int($pretty_time);
        $pretty_time                                              = $pretty_time - ( $pretty_time % 60 );
        $agent{ $timeserie->{resource}->{labels}->{instance_id} } = 1;
        if ( defined $data{gcloud}{compute}{ $timeserie->{resource}->{labels}->{instance_id} }{$pretty_time} ) {
          $data{glcoud}{compute}{ $timeserie->{resource}->{labels}->{instance_id} }{$pretty_time}{'process'}{ $timeserie->{metric}->{labels}->{state} } = $point->{value}->{doubleValue};
        }
      }
    }
    for (@metrics_agent) {
      my $metric = $_;
      $response = gcloudRequest( "https://monitoring.googleapis.com/v3/projects/$self->{project}/timeSeries?filter=metric.type=\"" . $metric->{url} . "\"&interval.endTime=" . $end_time . "Z&interval.startTime=" . $start_time . "Z", $self->{token} );

      for ( @{ $response->{timeSeries} } ) {
        my $timeserie = $_;

        for ( @{ $timeserie->{points} } ) {
          my $point = $_;

          my $pretty_time = str2time( $point->{interval}->{endTime} );
          $pretty_time = int($pretty_time);
          $pretty_time = $pretty_time - ( $pretty_time % 60 );
          if ( defined $data{gcloud}{compute}{ $timeserie->{resource}->{labels}->{instance_id} }{$pretty_time} ) {
            $data{gcloud}{compute}{ $timeserie->{resource}->{labels}->{instance_id} }{$pretty_time}{ $metric->{name} } = $point->{value}->{ $metric->{type} };
          }
        }
      }
    }
  }

  $self->resetServiceAccounts();

  return \%data;
}

sub gcloudRequest {
  my $url   = shift;
  my $token = shift;

  my $api_ua = LWP::UserAgent->new(
    timeout  => 30,
    ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 }
  );
  $api_ua->default_header( Authorization => 'Bearer ' . $token );

  my $api_response = $api_ua->get($url);

  if ( $api_response->is_success ) {
    my $api_data = decode_json( $api_response->content );
    use Data::Dumper;

    #print Dumper($api_data);
    return $api_data;
  }
  else {
    print "Error:\n";
    print "Code was ", $api_response->code,    "\n";
    print "Msg: ",     $api_response->message, "\n";
    error( "Error request ($url):\nCode was: " . Dumper( $api_response->code ) . "\nMsg: " . Dumper( $api_response->message ) );
    my $api_data = decode_json("{}");
    return $api_data;

    #die;
  }
}

sub gcloudRequestTest {
  my $url   = shift;
  my $token = shift;

  my $api_ua = LWP::UserAgent->new(
    timeout  => 30,
    ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 }
  );
  $api_ua->default_header( Authorization => 'Bearer ' . $token );

  my $api_response = $api_ua->get($url);

  if ( $api_response->is_success ) {
    return 1;
  }
  else {
    return 0;
  }
}

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print STDERR "$act_time: $text : $!\n";
  return 1;
}

1;
