package IBMCOSS;

use lib '../../../perl-modules';

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Date::Parse;
use POSIX qw( strftime );
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use Xormon;
use JSON qw(encode_json decode_json);
use Test;
use XormonLogger;

my %data;
my %perf;
my $xormon;
my $cfg;
my $logger;

my $service_data_in  = {};
my $service_data_out = {};

sub collect {
  $xormon = shift;
  $cfg    = shift;

  #print Dumper $cfg;

  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")    && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")     && return "error"; }
  $logger = XormonLogger->new( 'ibmcoss', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )       { $logger->error("Cannot continue, 'apiHost' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'apiPort'} )       { $logger->error("Cannot continue, 'apiPort' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )   { $logger->error("Cannot continue, 'apiUsername' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )   { $logger->error("Cannot continue, 'apiPassword' is not configured!")   && return "error"; }

  ## GET MICROSERVICE DATA
  $service_data_in  = $xormon->getData();
  $service_data_out = {};

  my $starttime;
  my $time      = time();
  my $endtime   = int( $time / 300 ) * 300 * 1000;        # 5mins in millisec
  $cfg->{'endtime'} = $endtime;
  if ( exists $service_data_in->{last_perf_timestamp_ms} && Xormon::isdigit($service_data_in->{last_perf_timestamp_ms}) ) {
    my $last_time = $service_data_in->{last_perf_timestamp_ms};
    my $dif_time = $endtime - $last_time;
    if ($dif_time < 3600000) {
      $starttime = $last_time;
    }
    else {
      $starttime = $endtime - ( 3600 * 1000 ) - 300000;    # 1hour back in millisec
    }
  }
  else {
    $starttime = $endtime - ( 3600 * 1000 ) - 300000;    # 1hour back in millisec
  }
  $cfg->{'starttime'} = $starttime;

  $xormon->startJob('fetch', $cfg->{hostalias}, $$);

  # collect perf data
  my $res_data = get_data();
  if ($res_data == 0) { return "error"; }
  health_status();

  # save data to DB
  print_data();


  return 1;
}

sub get_data {

  my $api_ip    = $cfg->{'apiHost'};
  my $starttime = $cfg->{'starttime'};
  my $endtime   = $cfg->{'endtime'};
  my $return   = "";
  my $n_return = "";
  my $metric   = "";

  #################
  ## SYSTEM + HS ##
  #################

  $return = new_get_request("https://$api_ip/manager/api/json/1.0/viewSystem.adm");
  if ( !$return ) {
    $xormon->isConnected(0, "cant get data from api");
    $logger->error("Can't get data from $api_ip!") && return 0;
  }
  else {
    $xormon->isConnected(1);
  }

  #print Dumper $return;
  if ( $return && defined $return->{'responseData'}->{'sites'} && ref( $return->{'responseData'}->{'sites'} ) eq "ARRAY" ) {
    foreach my $site ( @{ $return->{'responseData'}->{'sites'} } ) {
      if ( defined $site->{'id'} && $site->{'id'} ne "" ) {
        my $id = $site->{'id'};
        if ( defined $site->{'uuid'} && $site->{'uuid'} ne "" ) {
          $data{'site'}{$id}{'uuid'} = $site->{'uuid'};
        }
        if ( defined $site->{'name'} && $site->{'name'} ne "" ) {
          $data{'site'}{$id}{'name'} = $site->{'name'};
        }
        if ( defined $site->{'health'} && $site->{'health'} ne "" ) {
          $data{'site'}{$id}{'health'} = $site->{'health'};
        }
      }
    }
  }
  if ( $return && defined $return->{'responseData'}->{'devices'} && ref( $return->{'responseData'}->{'devices'} ) eq "ARRAY" ) {
    foreach my $device ( @{ $return->{'responseData'}->{'devices'} } ) {
      if ( defined $device->{'id'} && $device->{'id'} ne "" ) {
        my $id          = $device->{'id'};
        my $device_type = "";
        if ( defined $device->{'deviceType'} && $device->{'deviceType'} ne "" ) {
          $device_type = $device->{'deviceType'};
          if ( $device_type eq "slicestor" ) {
            if ( defined $device->{'capacity'} && $device->{'capacity'} ne "" ) {
              $data{$device_type}{$id}{'capacity'} = $device->{'capacity'};
            }
            if ( defined $device->{'utilization'} && $device->{'utilization'} ne "" ) {
              $data{$device_type}{$id}{'used'} = $device->{'utilization'};
            }
          }
        }
        if ( defined $device->{'version'} && $device->{'version'} ne "" ) {
          $data{$device_type}{$id}{'version'} = $device->{'version'};
        }
        if ( defined $device->{'health'} && $device->{'health'} ne "" ) {
          $data{$device_type}{$id}{'health'} = $device->{'health'};
        }
      }
    }
  }
  if ( $return && defined $return->{'responseData'}->{'vaults'} && ref( $return->{'responseData'}->{'vaults'} ) eq "ARRAY" ) {
    foreach my $vault ( @{ $return->{'responseData'}->{'vaults'} } ) {
      if ( defined $vault->{'id'} && $vault->{'id'} ne "" ) {
        my $id = $vault->{'id'};
        if ( defined $vault->{'storageHealth'} && $vault->{'storageHealth'} ne "" ) {
          $data{'vault'}{$id}{'storageHealth'} = $vault->{'storageHealth'};
        }
        if ( defined $vault->{'accessHealth'} && $vault->{'accessHealth'} ne "" ) {
          $data{'vault'}{$id}{'accessHealth'} = $vault->{'accessHealth'};
        }
        if ( defined $vault->{'softQuota'} && $vault->{'softQuota'} ne "" ) {
          $data{'vault'}{$id}{'soft_quota'} = $vault->{'softQuota'};
        }
        if ( defined $vault->{'hardQuota'} && $vault->{'hardQuota'} ne "" ) {
          $data{'vault'}{$id}{'hard_quota'} = $vault->{'hardQuota'};
        }
        if ( defined $vault->{'utilization'} && $vault->{'utilization'} ne "" ) {
          $data{'vault'}{$id}{'used_b'} = $vault->{'utilization'};
        }
        if ( defined $vault->{'storagePoolGroup'} && $vault->{'storagePoolGroup'} ne "" ) {
          $data{'vault'}{$id}{'pool_id'} = $vault->{'storagePoolGroup'};
        }
      }
    }
  }
  if ( $return && defined $return->{'responseData'}->{'storagePools'} && ref( $return->{'responseData'}->{'storagePools'} ) eq "ARRAY" ) {
    foreach my $pool ( @{ $return->{'responseData'}->{'storagePools'} } ) {
      if ( defined $pool->{'id'} && $pool->{'id'} ne "" ) {
        my $id = $pool->{'id'};
        if ( defined $pool->{'utilization'} && $pool->{'utilization'} ne "" ) {
          $data{'pool'}{$id}{'used_b'} = $pool->{'utilization'};
        }
        if ( defined $pool->{'capacity'} && $pool->{'capacity'} ne "" ) {
          $data{'pool'}{$id}{'capacity_b'} = $pool->{'capacity'};
        }
      }
    }
  }

  ############
  ## DRIVES ## has to run before devices to get drive configuration per device
  ############

  $return = new_get_request("https://$api_ip/manager/api/json/1.0/driveReport.adm");

  #print Dumper $return;
  if ( $return && defined $return->{'responseData'}->{'devices'} && ref( $return->{'responseData'}->{'devices'} ) eq "ARRAY" ) {
    foreach my $device ( @{ $return->{'responseData'}->{'devices'} } ) {
      if ( defined $device->{'id'} && $device->{'id'} ne "" ) {
        my $device_id   = $device->{'id'};
        my $device_name = "";
        if ( defined $device->{'drives'} && ref( $device->{'drives'} ) eq "ARRAY" ) {
          foreach my $drive ( @{ $device->{'drives'} } ) {
            if ( defined $drive->{'uuid'} && $drive->{'uuid'} ne "" ) {
              my $drive_uuid = $drive->{'uuid'};
              if ( defined $device->{'alias'} && $device->{'alias'} ne "" ) {
                $device_name = $device->{'alias'};
              }
              $data{'drive'}{$device_id}{$drive_uuid}{'device_name'} = $device_name;
              if ( defined $drive->{'deviceName'} && $drive->{'deviceName'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'name'} = $drive->{'deviceName'};
              }
              if ( defined $drive->{'status'} && $drive->{'status'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'status'} = $drive->{'status'};
              }
              if ( defined $drive->{'capacity'} && $drive->{'capacity'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'capacity'} = $drive->{'capacity'};
              }
              if ( defined $drive->{'firmware'} && $drive->{'firmware'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'firmware'} = $drive->{'firmware'};
              }
              if ( defined $drive->{'slotId'} && $drive->{'slotId'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'slotId'} = $drive->{'slotId'};
              }
              if ( defined $drive->{'enclosureId'} && $drive->{'enclosureId'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'enclosureId'} = $drive->{'enclosureId'};
              }
              if ( defined $drive->{'chassisId'} && $drive->{'chassisId'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'chassisId'} = $drive->{'chassisId'};
              }
              if ( defined $drive->{'bay'} && $drive->{'bay'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'bay'} = $drive->{'bay'};
              }
              if ( defined $drive->{'model'} && $drive->{'model'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'model'} = $drive->{'model'};
              }
              if ( defined $drive->{'serial'} && $drive->{'serial'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'serial'} = $drive->{'serial'};
              }
              if ( defined $drive->{'driveUsage'} && $drive->{'driveUsage'} ne "" ) {
                $data{'drive'}{$device_id}{$drive_uuid}{'driveUsage'} = $drive->{'driveUsage'};
              }
            }
          }
        }
      }
    }
  }

  ###########
  ## POOLS ##
  ###########
  my $subsystem = "pool";

  $return = new_get_request("https://$api_ip/manager/api/json/1.0/listStoragePools.adm");

  #print Dumper $return;
  if ( defined $return->{'responseData'}->{'storagePools'} && ref( $return->{'responseData'}->{'storagePools'} ) eq "ARRAY" ) {
    foreach my $pool ( @{ $return->{'responseData'}->{'storagePools'} } ) {
      if ( defined $pool->{'id'} && $pool->{'id'} ne "" ) {
        my $id = $pool->{'id'};
        if ( defined $pool->{'name'} && $pool->{'name'} ne "" ) {
          $data{'pool'}{$id}{'name'} = $pool->{'name'};

          # get slicestors for pool
          if ( defined $pool->{'storagePools'} && ref( $pool->{'storagePools'} ) eq "ARRAY" ) {
            foreach my $sec_pool ( @{ $pool->{'storagePools'} } ) {
              if ( defined $sec_pool->{'id'} && $sec_pool->{'id'} ne "" && $sec_pool->{'id'} eq $id ) {
                if ( defined $sec_pool->{'devices'} && ref( $sec_pool->{'devices'} ) eq "ARRAY" ) {
                  foreach my $dev ( @{ $sec_pool->{'devices'} } ) {
                    if ( defined $dev->{'id'} && $dev->{'id'} ne "" ) {
                      my $dev_id = $dev->{'id'};
                      $data{'slicestor'}{$dev_id}{'pool_id'}   = $id;
                      $data{'slicestor'}{$dev_id}{'pool_name'} = $pool->{'name'};
                    }
                  }
                }
              }
            }
          }
        }
        ## POOL CAP
        $metric   = "total_cap";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/storagePoolCapacity.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
          get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
        }
        ## POOL USED
        $metric   = "used_cap";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/storagePoolUsed.json?startDate=$starttime&endDate=$endtime");

        #print Dumper \$n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
          get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
        }
        ## SLICES
        $metric   = "slices";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeSlicesStorageQuantity/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper \$n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_data( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## SCANNED SOURCES
        $metric   = "scanned_srcs";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeScanningSourcesScanned/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## REBUILD QUEUE
        $metric   = "rebuild_q";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/rebuildQueueLengthEstimate.json?startDate=$starttime&endDate=$endtime");
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
          if ( defined @{$n_return}[0]->{'data'} && @{$n_return}[0]->{'data'} ne "" && ref( @{$n_return}[0]->{'data'} ) eq "ARRAY" && scalar( @{$n_return}[0]->{'data'} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
        }
        ## REBUILD writes_in_counter
        $metric   = "rebuild_writes_in";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeRebuildWritesReceived/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## REBUILD writes_out_counter
        $metric   = "rebuild_writes_out";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeRebuildWritesSent/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## REBUILD deletes_in_counter
        $metric   = "rebuild_deletes_in";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeRebuildDeletesReceived/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## REBUILD deletes_out_counter
        $metric   = "rebuild_deletes_out";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeRebuildDeletesSent/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## REBUILD bytes_in_counter
        $metric   = "rebuild_bytes_in";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeRebuildBytesReceived/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## REBUILD bytes_out_counter
        $metric   = "rebuild_bytes_out";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/storagePoolGroup/$id/statistic/stripeRebuildBytesSent/*.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
      }
    }
  }
  ############
  ## VAULTS ##
  ############
  $subsystem = "vault";

  $return = new_get_request("https://$api_ip/manager/api/json/1.0/listVaults.adm");
  if ( defined $return->{'responseData'}->{'vaults'} && ref( $return->{'responseData'}->{'vaults'} ) eq "ARRAY" ) {
    foreach my $vault ( @{ $return->{'responseData'}->{'vaults'} } ) {
      if ( defined $vault->{'id'} && $vault->{'id'} ne "" ) {
        my $id = $vault->{'id'};
        if ( defined $vault->{'uuid'} && $vault->{'uuid'} ne "" ) {
          $data{'vault'}{$id}{'uuid'} = $vault->{'uuid'};
        }
        if ( defined $vault->{'name'} && $vault->{'name'} ne "" ) {
          $data{'vault'}{$id}{'name'} = $vault->{'name'};
        }
        ## USED
        $metric = "used";
        my $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/used.json?startDate=$starttime&endDate=$endtime");
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
          get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
        }
        ## SCANNED SOURCES
        $metric   = "scanned_srcs";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/vaultRebuildScanningRate.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $dev ( @{$n_return} ) {
            if ( defined $dev->{'header'}->{'itemId'} && $dev->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $dev->{'header'}->{'itemId'};
              if ( defined $dev->{'data'} && $dev->{'data'} ne "" && ref( $dev->{'data'} ) eq "ARRAY" && scalar( @{ $dev->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $dev->{'data'} );
              }
            }
          }
        }
        ## REBUILD
        # writes in
        $metric   = "vault_writes_in";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/rebuildWritesReceived.json?startDate=$starttime&endDate=$endtime");
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $rbld ( @{$n_return} ) {
            if ( defined $rbld->{'header'}->{'itemId'} && $rbld->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $rbld->{'header'}->{'itemId'};
              if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $rbld->{'data'} );
              }
            }
          }
        }

        # writes out
        $metric   = "vault_writes_out";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/rebuildWritesSent.json?startDate=$starttime&endDate=$endtime");
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $rbld ( @{$n_return} ) {
            if ( defined $rbld->{'header'}->{'itemId'} && $rbld->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $rbld->{'header'}->{'itemId'};
              if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $rbld->{'data'} );
              }
            }
          }
        }

        # deletes in
        $metric   = "vault_deletes_in";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/rebuildDeletesReceived.json?startDate=$starttime&endDate=$endtime");
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $rbld ( @{$n_return} ) {
            if ( defined $rbld->{'header'}->{'itemId'} && $rbld->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $rbld->{'header'}->{'itemId'};
              if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $rbld->{'data'} );
              }
            }
          }
        }

        # deletes out
        $metric   = "vault_deletes_out";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/rebuildDeletesSent.json?startDate=$starttime&endDate=$endtime");
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $rbld ( @{$n_return} ) {
            if ( defined $rbld->{'header'}->{'itemId'} && $rbld->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $rbld->{'header'}->{'itemId'};
              if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $rbld->{'data'} );
              }
            }
          }
        }

        # bytes in
        $metric   = "vault_bytes_in";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/rebuildBytesReceived.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $rbld ( @{$n_return} ) {
            if ( defined $rbld->{'header'}->{'itemId'} && $rbld->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $rbld->{'header'}->{'itemId'};
              if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $rbld->{'data'} );
              }
            }
          }
        }

        # bytes out
        $metric   = "vault_bytes_out";
        $n_return = new_get_request("https://$api_ip/manager/api/sandra/vault/$id/statistic/rebuildBytesSent.json?startDate=$starttime&endDate=$endtime");

        #print Dumper $n_return;
        if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
          foreach my $rbld ( @{$n_return} ) {
            if ( defined $rbld->{'header'}->{'itemId'} && $rbld->{'header'}->{'itemId'} ne "" ) {
              my $item_id = $rbld->{'header'}->{'itemId'};
              if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                get_5min_counter( $subsystem, $id, $metric, $item_id, $rbld->{'data'} );
              }
            }
          }
        }
      }
    }
  }
  #############
  ## DEVICES ##
  #############

  $return = new_get_request("https://$api_ip/manager/api/json/1.0/listDevices.adm");

  #print Dumper $return;
  if ( defined $return->{'responseData'}->{'devices'} && ref( $return->{'responseData'}->{'devices'} ) eq "ARRAY" ) {
    foreach my $device ( @{ $return->{'responseData'}->{'devices'} } ) {

      #print Dumper $device;
      if ( defined $device->{'id'} && $device->{'id'} ne "" ) {
        my $id = $device->{'id'};
        if ( defined $device->{'deviceType'} && $device->{'deviceType'} ne "" ) {
          $subsystem = $device->{'deviceType'};
          ## CPU LOAD
          $metric   = "cpu_load";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/load.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
          ## CPU PERC
          $metric   = "cpu_perc";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/cpu.json?startDate=$starttime&endDate=$endtime");
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
            foreach my $cpu ( @{$n_return} ) {
              if ( defined $cpu->{'header'}->{'key'} && $cpu->{'header'}->{'key'} ne "" ) {
                my $cpu_type = $cpu->{'header'}->{'key'};
                if ( defined $cpu->{'data'} && $cpu->{'data'} ne "" && ref( $cpu->{'data'} ) eq "ARRAY" && scalar( @{ $cpu->{'data'} } ) ) {
                  get_5min_counter( $subsystem, $id, $metric, "$cpu_type", $cpu->{'data'} );
                }
              }
            }
          }
          ## MEMORY
          ## Resident Memory Set Size %
          # mem used
          $metric   = "sys_mem_used";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/systemUsedBytes.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
          # mem total
          $metric   = "sys_mem_total";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/systemTotalBytes.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
          ##
          ## On Heap Resource Permits %
          # onheap used
          $metric   = "onheap_mem_used";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/onHeapUsedBytes.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }

          # onheap total
          $metric   = "onheap_mem_total";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/onHeapTotalBytes.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
          ##
          ## Off Heap Resource Permits %
          # offheap used
          $metric   = "offheap_mem_used";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/offHeapUsedBytes.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }

          # offheap total
          $metric   = "offheap_mem_total";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/offHeapTotalBytes.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
          ## NETWORK
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/network/*.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
            foreach my $network ( @{$n_return} ) {
              if ( defined $network->{'header'}->{'rowName'} && $network->{'header'}->{'rowName'} ne "" ) {
                my $net_id = $network->{'header'}->{'rowName'};
                if ( $net_id eq "lo" ) {
                  next;
                }
                if ( defined $network->{'data'} && $network->{'data'} ne "" && ref( $network->{'data'} ) eq "ARRAY" && scalar( @{ $network->{'data'} } ) ) {
                  if ( defined $network->{'header'}->{'columnName'} && $network->{'header'}->{'columnName'} eq "in" ) {
                    $metric = "net_in";
                  }
                  elsif ( defined $network->{'header'}->{'columnName'} && $network->{'header'}->{'columnName'} eq "out" ) {
                    $metric = "net_out";
                  }
                  get_5min_counter( $subsystem, $id, $metric, $net_id, $network->{'data'} );
                }
              }
            }
          }
          # tcp timeouts
          $metric   = "tcp_timeouts";
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/netstat.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_counter( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
          ## DRIVES
          $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/disk/*.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
            foreach my $disk_io ( @{$n_return} ) {
              if ( defined $disk_io->{'header'}->{'columnName'} && $disk_io->{'header'}->{'columnName'} ne "" ) {
                if ( $disk_io->{'header'}->{'columnName'} eq "read" ) {
                  $metric = "data_read";
                }
                elsif ( $disk_io->{'header'}->{'columnName'} eq "write" ) {
                  $metric = "data_write";
                }
                else {
                  $metric = $disk_io->{'header'}->{'columnName'};
                }
                if ( defined $disk_io->{'header'}->{'rowName'} && $disk_io->{'header'}->{'rowName'} ne "" ) {
                  my $disk_uuid = $disk_io->{'header'}->{'rowName'};
                  if ( defined $disk_io->{'data'} && $disk_io->{'data'} ne "" && ref( $disk_io->{'data'} ) eq "ARRAY" && scalar( @{ $disk_io->{'data'} } ) ) {
                    get_5min_counter( $subsystem, $id, $metric, $disk_uuid, $disk_io->{'data'} );
                  }
                }
              }
            }
          }
          ###########
          # MANAGER #
          ###########
          if ( $subsystem eq "manager" ) {
            ## TODO
          }    # end of manager
          ############
          # ACCESSER #
          ############
          if ( $subsystem eq "accesser" ) {
            if ( defined $data{'pool'} && $data{'pool'} ne "" ) {
              foreach my $pool ( keys %{ $data{'pool'} } ) {
                $metric   = "ack_t";
                $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/accesserRtt.json?averageBy=group&storagePoolIds=$pool&percentile=&startDate=$starttime&endDate=$endtime");

                #print Dumper $n_return;
                if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
                  get_5min_data( $subsystem, $id, $metric, $pool, @{$n_return}[0]->{'data'} );
                }
              }
            }

            # get data for VAULTS - from client
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/clientAcc/*.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $ele ( @{$n_return} ) {
                if ( defined $ele->{'header'}->{'rowName'} && $ele->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $ele->{'header'}->{'rowName'};
                  if ( defined $ele->{'header'}->{'key'} && $ele->{'header'}->{'key'} ne "" ) {
                    my $ele_key = $ele->{'header'}->{'key'};
                    if ( $ele_key eq "accesserVault.all.externalOut" || $ele_key eq "accesserVault.all.externalIn" ) {
                      next;
                    }
                    if ( defined $ele->{'data'} && $ele->{'data'} ne "" && ref( $ele->{'data'} ) eq "ARRAY" && scalar( @{ $ele->{'data'} } ) ) {
                      if ( defined $ele->{'header'}->{'columnName'} && $ele->{'header'}->{'columnName'} eq "externalIn" ) {
                        $metric = "client_in";
                      }
                      elsif ( defined $ele->{'header'}->{'columnName'} && $ele->{'header'}->{'columnName'} eq "externalOut" ) {
                        $metric = "client_out";
                      }
                      my $vault_id = "";
                      foreach my $vlt ( keys %{ $data{'vault'} } ) {
                        if ( defined $data{'vault'}{$vlt}{'uuid'} && $data{'vault'}{$vlt}{'uuid'} eq $vault_uuid ) {
                          my $conf_uuid = $data{'vault'}{$vlt}{'uuid'};
                          $vault_id = $vlt;
                        }
                      }
                      if ( $vault_id ne "" ) {
                        get_5min_counter( "vault", $vault_id, $metric, $id, $ele->{'data'} );
                      }
                    }
                  }
                }
              }
            }
            # GET data for vaults from storage
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/accesserVault/*.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $ele ( @{$n_return} ) {
                if ( defined $ele->{'header'}->{'rowName'} && $ele->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $ele->{'header'}->{'rowName'};
                  if ( defined $ele->{'header'}->{'key'} && $ele->{'header'}->{'key'} ne "" ) {
                    my $ele_key = $ele->{'header'}->{'key'};
                    if ( $ele_key eq "accesserVault.all.externalOut" || $ele_key eq "accesserVault.all.externalIn" ) {
                      next;
                    }
                    if ( defined $ele->{'data'} && $ele->{'data'} ne "" && ref( $ele->{'data'} ) eq "ARRAY" && scalar( @{ $ele->{'data'} } ) ) {
                      if ( defined $ele->{'header'}->{'columnName'} && $ele->{'header'}->{'columnName'} eq "internalIn" ) {
                        $metric = "slicestor_in";
                      }
                      elsif ( defined $ele->{'header'}->{'columnName'} && $ele->{'header'}->{'columnName'} eq "internalOut" ) {
                        $metric = "slicestor_out";
                      }
                      my $vault_id = "";
                      foreach my $vlt ( keys %{ $data{'vault'} } ) {
                        if ( defined $data{'vault'}{$vlt}{'uuid'} && $data{'vault'}{$vlt}{'uuid'} eq $vault_uuid ) {
                          my $conf_uuid = $data{'vault'}{$vlt}{'uuid'};
                          $vault_id = $vlt;
                        }
                      }
                      if ( $vault_id ne "" ) {
                        get_5min_counter( "vault", $vault_id, $metric, $id, $ele->{'data'} );
                      }
                    }
                  }
                }
              }
            }
          }    # end of accesser

          #############
          # SLICESTOR #
          #############
          if ( $subsystem eq "slicestor" ) {
            ## SLICESTOR SCANNING RATE
            $metric   = "slicestor_scan_rate";
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/deviceScanningSourcesScanned.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
              get_5min_counter( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
            }
            ## REBUILD PER VAULT
            # bytes in
            $metric   = "slicestor_bytes_in";
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/rebuildBytesReceived.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $rbld ( @{$n_return} ) {
                if ( defined $rbld->{'header'}->{'rowName'} && $rbld->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $rbld->{'header'}->{'rowName'};
                  if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                    get_5min_counter( $subsystem, $id, $metric, $vault_uuid, $rbld->{'data'} );
                  }
                }
              }
            }

            # bytes out
            $metric   = "slicestor_bytes_out";
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/rebuildBytesSent.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $rbld ( @{$n_return} ) {
                if ( defined $rbld->{'header'}->{'rowName'} && $rbld->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $rbld->{'header'}->{'rowName'};
                  if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                    get_5min_counter( $subsystem, $id, $metric, $vault_uuid, $rbld->{'data'} );
                  }
                }
              }
            }
            # writes in
            $metric   = "slicestor_writes_in";
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/rebuildWritesReceived.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $rbld ( @{$n_return} ) {
                if ( defined $rbld->{'header'}->{'rowName'} && $rbld->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $rbld->{'header'}->{'rowName'};
                  if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                    get_5min_counter( $subsystem, $id, $metric, $vault_uuid, $rbld->{'data'} );
                  }
                }
              }
            }

            # writes out
            $metric   = "slicestor_writes_out";
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/rebuildWritesSent.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $rbld ( @{$n_return} ) {
                if ( defined $rbld->{'header'}->{'rowName'} && $rbld->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $rbld->{'header'}->{'rowName'};
                  if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                    get_5min_counter( $subsystem, $id, $metric, $vault_uuid, $rbld->{'data'} );
                  }
                }
              }
            }

            # deletes in
            $metric   = "slicestor_deletes_in";
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/rebuildDeletesReceived.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $rbld ( @{$n_return} ) {
                if ( defined $rbld->{'header'}->{'rowName'} && $rbld->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $rbld->{'header'}->{'rowName'};
                  if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                    get_5min_counter( $subsystem, $id, $metric, $vault_uuid, $rbld->{'data'} );
                  }
                }
              }
            }
            # deletes out
            $metric   = "slicestor_deletes_out";
            $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/rebuildDeletesSent.json?startDate=$starttime&endDate=$endtime");

            #print Dumper $n_return;
            if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" ) {
              foreach my $rbld ( @{$n_return} ) {
                if ( defined $rbld->{'header'}->{'rowName'} && $rbld->{'header'}->{'rowName'} ne "" ) {
                  my $vault_uuid = $rbld->{'header'}->{'rowName'};
                  if ( defined $rbld->{'data'} && $rbld->{'data'} ne "" && ref( $rbld->{'data'} ) eq "ARRAY" && scalar( @{ $rbld->{'data'} } ) ) {
                    get_5min_counter( $subsystem, $id, $metric, $vault_uuid, $rbld->{'data'} );
                  }
                }
              }
            }
          }    # end of slicestor

          if ( defined $device->{'uuid'} && $device->{'uuid'} ne "" ) {
            $data{$subsystem}{$id}{'uuid'} = $device->{'uuid'};
          }
          if ( defined $device->{'hostname'} && $device->{'hostname'} ne "" ) {
            $data{$subsystem}{$id}{'hostname'} = $device->{'hostname'};
          }
          if ( defined $device->{'alias'} && $device->{'alias'} ne "" ) {
            $data{$subsystem}{$id}{'alias'} = $device->{'alias'};
          }
          if ( defined $device->{'site'} && $device->{'site'} ne "" ) {
            my $dev_site_id = $device->{'site'};
            $data{$subsystem}{$id}{'site'} = $dev_site_id;
            if ( defined $data{'site'}{$dev_site_id}{'name'} && $data{'site'}{$dev_site_id}{'name'} ne "" ) {
              $data{$subsystem}{$id}{'site_name'} = $data{'site'}{$dev_site_id}{'name'};
            }
          }
          if ( defined $device->{'ip'} && $device->{'ip'} ne "" ) {
            $data{$subsystem}{$id}{'ip'} = $device->{'ip'};
            if ( $device->{'ip'} eq $api_ip ) {
              $data{'system'}{'ip'} = $device->{'ip'};
              if ( exists $data{$subsystem}{$id}{'version'} && $data{$subsystem}{$id}{'version'} ne "" ) {
                $data{'system'}{'version'} = $data{$subsystem}{$id}{'version'};
              }
              if ( exists $data{$subsystem}{$id}{'alias'} && $data{$subsystem}{$id}{'alias'} ne "" ) {
                $data{'system'}{'node_name'} = $data{$subsystem}{$id}{'alias'};
              }
              if ( exists $data{$subsystem}{$id}{'hostname'} && $data{$subsystem}{$id}{'hostname'} ne "" ) {
                $data{'system'}{'host_name'} = $data{$subsystem}{$id}{'hostname'};
              }
            }
          }
        }    # if (defined $device->{'deviceType'}
      }
    }
  }

  #print Dumper \%data;

  #print Dumper \%perf;
  return 1;
}

sub print_data {
  my @architecture;
  my $data_out = "";
  my %conf;
  my $st_uuid = $cfg->{'hostcfg_id'};

  ## DEVICE CONF
  my %device_arch = (
    "item_id" => $st_uuid,
    "label" => $cfg->{'hostalias'},
    "hw_type" => "ibmcoss",
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $st_uuid
  );
  push(@architecture, \%device_arch);

  if ( defined $data{'system'}{'ip'} && $data{'system'}{'ip'} ne "" ) {
    $conf{$st_uuid}{'ip'} = $data{'system'}{'ip'};
  }
  if ( defined $data{'system'}{'host_name'} && $data{'system'}{'host_name'} ne "" ) {
    $conf{$st_uuid}{'hostname'} = $data{'system'}{'host_name'};
  }
  if ( defined $data{'system'}{'version'} && $data{'system'}{'version'} ne "" ) {
    $conf{$st_uuid}{'version'} = $data{'system'}{'version'};
    $conf{$st_uuid}{'model'} = 'IBM Cloud Object Storage System';
  }
  if ( defined $data{'system'}{'node_name'} && $data{'system'}{'node_name'} ne "" ) {
    $conf{$st_uuid}{'node_name'} = $data{'system'}{'node_name'};
  }

  # ## SITE CONF
  # if ( exists $data{'site'} && $data{'site'} ne "" ) {
  #   foreach my $site ( keys %{ $data{'site'} } ) {
  #     if ( defined $data{'site'}{$site}{'uuid'} && $data{'site'}{$site}{'uuid'} ne "" ) {
  #       my $site_uuid = $data{'site'}{$site}{'uuid'};
  #       my $site_name = "$site_uuid";
  #       if ( defined $data{'site'}{$site}{'name'} && $data{'site'}{$site}{'name'} ne "" ) {
  #         $site_name = $data{'site'}{$site}{'name'};
  #         $conf{$site_uuid}{'name'} = $site_name;
  #       }
  #       if ( defined $data{'site'}{$site}{'health'} && $data{'site'}{$site}{'health'} ne "" ) {
  #         $conf{$site_uuid}{'health'} = $data{'site'}{$site}{'health'};
  #       }
  #       my %arch = (
  #         "item_id" => $site_uuid,
  #         "label" => $site_name,
  #         "hw_type" => "ibmcoss",
  #         "subsystem" => "site",
  #         "parents" => [$cfg->{'hostcfg_id'}],
  #       );
  #       push(@architecture, \%arch);
  #     }
  #   }
  # }

  ## MANAGER
  if ( exists $data{'manager'} && $data{'manager'} ne "" ) {
    foreach my $manager ( keys %{ $data{'manager'} } ) {
      if ( defined $data{'manager'}{$manager}{'uuid'} && $data{'manager'}{$manager}{'uuid'} ne "" ) {
        my $manager_uuid = $data{'manager'}{$manager}{'uuid'};
        my $manager_name = "$manager_uuid";

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."MANAGER".$manager_uuid); $hash_key = lc($hash_key);

        if ( defined $data{'manager'}{$manager}{'alias'} && $data{'manager'}{$manager}{'alias'} ne "" ) {
          $manager_name = $data{'manager'}{$manager}{'alias'};
          $conf{$hash_key}{'name'} = $manager_name;
        }
        # if ( defined $data{'manager'}{$manager}{'site'} && $data{'manager'}{$manager}{'site'} ne "" ) {
        #   $conf{$hash_key}{'site'} = "$data{'manager'}{$manager}{'site'}";
        # }
        # if ( defined $data{'manager'}{$manager}{'hostname'} && $data{'manager'}{$manager}{'hostname'} ne "" ) {
        #   $conf{$hash_key}{'hostname'} = "$data{'manager'}{$manager}{'hostname'}";
        # }
        # if ( defined $data{'manager'}{$manager}{'ip'} && $data{'manager'}{$manager}{'ip'} ne "" ) {
        #   $conf{$hash_key}{'ip'} = "$data{'manager'}{$manager}{'ip'}";
        # }
        # if ( defined $data{'manager'}{$manager}{'version'} && $data{'manager'}{$manager}{'version'} ne "" ) {
        #   $conf{$hash_key}{'version'} = "$data{'manager'}{$manager}{'version'}";
        # }
        # if ( defined $data{'manager'}{$manager}{'health'} && $data{'manager'}{$manager}{'health'} ne "" ) {
        #   $conf{$hash_key}{'health'} = "$data{'manager'}{$manager}{'health'}";
        # }
        my %arch = (
          "item_id" => $hash_key,
          "label" => $manager_name,
          "hw_type" => "ibmcoss",
          "subsystem" => "manager",
          "parents" => [$cfg->{'hostcfg_id'}],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
        push(@architecture, \%arch);
      }
    }
  }

  ## ACCESSER
  if ( exists $data{'accesser'} && $data{'accesser'} ne "" ) {
    foreach my $accesser ( keys %{ $data{'accesser'} } ) {
      if ( defined $data{'accesser'}{$accesser}{'uuid'} && $data{'accesser'}{$accesser}{'uuid'} ne "" ) {
        my $accesser_uuid = "$data{'accesser'}{$accesser}{'uuid'}";
        my $accesser_name = $accesser_uuid;

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."ACCESSER".$accesser_uuid); $hash_key = lc($hash_key);

        if ( defined $data{'accesser'}{$accesser}{'alias'} && $data{'accesser'}{$accesser}{'alias'} ne "" ) {
          $accesser_name = $data{'accesser'}{$accesser}{'alias'};
          $conf{$hash_key}{'name'} = $accesser_name;
        }
        # if ( defined $data{'accesser'}{$accesser}{'site'} && $data{'accesser'}{$accesser}{'site'} ne "" ) {
        #   $conf{$hash_key}{'site'} = "$data{'accesser'}{$accesser}{'site'}";
        # }
        # if ( defined $data{'accesser'}{$accesser}{'hostname'} && $data{'accesser'}{$accesser}{'hostname'} ne "" ) {
        #   $conf{$hash_key}{'hostname'} = "$data{'accesser'}{$accesser}{'hostname'}";
        # }
        # if ( defined $data{'accesser'}{$accesser}{'ip'} && $data{'accesser'}{$accesser}{'ip'} ne "" ) {
        #   $conf{$hash_key}{'ip'} = "$data{'accesser'}{$accesser}{'ip'}";
        # }
        # if ( defined $data{'accesser'}{$accesser}{'version'} && $data{'accesser'}{$accesser}{'version'} ne "" ) {
        #   $conf{$hash_key}{'version'} = "$data{'accesser'}{$accesser}{'version'}";
        # }
        # if ( defined $data{'accesser'}{$accesser}{'health'} && $data{'accesser'}{$accesser}{'health'} ne "" ) {
        #   $conf{$hash_key}{'health'} = "$data{'accesser'}{$accesser}{'health'}";
        # }
        my %arch = (
          "item_id" => $hash_key,
          "label" => $accesser_name,
          "hw_type" => "ibmcoss",
          "subsystem" => "accesser",
          "parents" => [$cfg->{'hostcfg_id'}],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
        push(@architecture, \%arch);
      }
    }
  }

  ## SLICESTOR
  if ( exists $data{'slicestor'} && $data{'slicestor'} ne "" ) {
    foreach my $slicestor ( keys %{ $data{'slicestor'} } ) {
      if ( defined $data{'slicestor'}{$slicestor}{'uuid'} && $data{'slicestor'}{$slicestor}{'uuid'} ne "" ) {
        my $slicestor_uuid = "$data{'slicestor'}{$slicestor}{'uuid'}";
        my $slicestor_name = $slicestor_uuid;

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."SLICESTOR".$slicestor_uuid); $hash_key = lc($hash_key);

        if ( defined $data{'slicestor'}{$slicestor}{'alias'} && $data{'slicestor'}{$slicestor}{'alias'} ne "" ) {
          $slicestor_name = $data{'slicestor'}{$slicestor}{'alias'};
          $conf{$hash_key}{'name'} = $slicestor_name;
        }
        # if ( defined $data{'slicestor'}{$slicestor}{'site'} && $data{'slicestor'}{$slicestor}{'site'} ne "" ) {
        #   $conf{$hash_key}{'site'} = "$data{'slicestor'}{$slicestor}{'site'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'site_name'} && $data{'slicestor'}{$slicestor}{'site_name'} ne "" ) {
        #   $conf{$hash_key}{'site_name'} = "$data{'slicestor'}{$slicestor}{'site_name'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'hostname'} && $data{'slicestor'}{$slicestor}{'hostname'} ne "" ) {
        #   $conf{$hash_key}{'hostname'} = "$data{'slicestor'}{$slicestor}{'hostname'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'ip'} && $data{'slicestor'}{$slicestor}{'ip'} ne "" ) {
        #   $conf{$hash_key}{'ip'} = "$data{'slicestor'}{$slicestor}{'ip'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'version'} && $data{'slicestor'}{$slicestor}{'version'} ne "" ) {
        #   $conf{$hash_key}{'version'} = "$data{'slicestor'}{$slicestor}{'version'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'health'} && $data{'slicestor'}{$slicestor}{'health'} ne "" ) {
        #   $conf{$hash_key}{'health'} = "$data{'slicestor'}{$slicestor}{'health'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'pool_id'} && $data{'slicestor'}{$slicestor}{'pool_id'} ne "" ) {
        #   $conf{$hash_key}{'pool_id'} = "$data{'slicestor'}{$slicestor}{'pool_id'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'pool_name'} && $data{'slicestor'}{$slicestor}{'pool_name'} ne "" ) {
        #   $conf{$hash_key}{'pool_name'} = "$data{'slicestor'}{$slicestor}{'pool_name'}";
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'capacity'} && $data{'slicestor'}{$slicestor}{'capacity'} ne "" ) {
        #   $conf{$hash_key}{'capacity'} = $data{'slicestor'}{$slicestor}{'capacity'};
        # }
        # if ( defined $data{'slicestor'}{$slicestor}{'used'} && $data{'slicestor'}{$slicestor}{'used'} ne "" ) {
        #   $conf{$hash_key}{'used'} = "$data{'slicestor'}{$slicestor}{'used'}";
        # }
        my %arch = (
          "item_id" => $hash_key,
          "label" => $slicestor_name,
          "hw_type" => "ibmcoss",
          "subsystem" => "slicestor",
          "parents" => [$cfg->{'hostcfg_id'}],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
        push(@architecture, \%arch);
      }
    }
  }

  ## VAULT
  if ( exists $data{'vault'} && $data{'vault'} ne "" ) {
    foreach my $vault ( keys %{ $data{'vault'} } ) {
      if ( defined $data{'vault'}{$vault}{'uuid'} && $data{'vault'}{$vault}{'uuid'} ne "" ) {
        my $vault_uuid = $data{'vault'}{$vault}{'uuid'};
        my $vault_name = $vault_uuid;

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."VAULT".$vault_uuid); $hash_key = lc($hash_key);

        if ( defined $data{'vault'}{$vault}{'name'} && $data{'vault'}{$vault}{'name'} ne "" ) {
          $vault_name = $data{'vault'}{$vault}{'name'};
          $conf{$hash_key}{'name'} = $vault_name;
        }
        # if ( defined $data{'vault'}{$vault}{'storageHealth'} && $data{'vault'}{$vault}{'storageHealth'} ne "" ) {
        #   $conf{$vault_uuid}{'stor_health'} = $data{'vault'}{$vault}{'storageHealth'};
        # }
        # if ( defined $data{'vault'}{$vault}{'accessHealth'} && $data{'vault'}{$vault}{'accessHealth'} ne "" ) {
        #   $conf{$vault_uuid}{'access_health'} = $data{'vault'}{$vault}{'accessHealth'};
        # }
        # if ( defined $data{'vault'}{$vault}{'soft_quota'} && Xormon::isdigit( $data{'vault'}{$vault}{'soft_quota'} ) ) {
        #   $conf{$vault_uuid}{'soft_q'} = $data{'vault'}{$vault}{'soft_quota'};
        # }
        # if ( defined $data{'vault'}{$vault}{'hard_quota'} && Xormon::isdigit( $data{'vault'}{$vault}{'hard_quota'} ) ) {
        #   $conf{$vault_uuid}{'hard_q'} = $data{'vault'}{$vault}{'hard_quota'};
        # }
        if ( defined $data{'vault'}{$vault}{'pool_id'} && $data{'vault'}{$vault}{'pool_id'} ne "" ) {
          my $pool_id = $data{'vault'}{$vault}{'pool_id'};
          $conf{$hash_key}{'pool_id'} = $pool_id;
          if ( defined $data{'pool'}{$pool_id}{'name'} && $data{'pool'}{$pool_id}{'name'} ne "" ) {
            $conf{$hash_key}{'pool_name'} = $data{'pool'}{$pool_id}{'name'};
          }
        }
        # if ( defined $data{'vault'}{$vault}{'used_b'} && $data{'vault'}{$vault}{'used_b'} ne "" ) {
        #   $conf{$vault_uuid}{'used_b'} = $data{'vault'}{$vault}{'used_b'};
        # }
        my %arch = (
          "item_id" => $hash_key,
          "label" => $vault_name,
          "hw_type" => "ibmcoss",
          "subsystem" => "vault",
          "parents" => [$cfg->{'hostcfg_id'}],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
        push(@architecture, \%arch);
      }
    }
  }

  ## POOL
  if ( exists $data{'pool'} && $data{'pool'} ne "" ) {
    foreach my $pool ( keys %{ $data{'pool'} } ) {
      my $hash_key = Xormon::create_hash_key($st_uuid."POOL".$pool); $hash_key = lc($hash_key);
      my $pool_name = $hash_key;
      if ( defined $data{'pool'}{$pool}{'name'} && $data{'pool'}{$pool}{'name'} ne "" ) {
        $pool_name = $data{'pool'}{$pool}{'name'};
        $conf{$hash_key}{'name'} = $pool_name;
      }
      if ( defined $data{'pool'}{$pool}{'capacity_b'} && $data{'pool'}{$pool}{'capacity_b'} ne "" ) {
        $conf{$hash_key}{'capacity_total'} = $data{'pool'}{$pool}{'capacity_b'};
      }
      if ( defined $data{'pool'}{$pool}{'used_b'} && $data{'pool'}{$pool}{'used_b'} ne "" ) {
        $conf{$hash_key}{'capacity_used'} = $data{'pool'}{$pool}{'used_b'};
      }
      if ( Xormon::isdigit($data{'pool'}{$pool}{'capacity_b'}) && Xormon::isdigit($data{'pool'}{$pool}{'used_b'}) ) {
        $conf{$hash_key}{'capacity_free'} = $data{'pool'}{$pool}{'capacity_b'} - $data{'pool'}{$pool}{'used_b'};
      }
      my %arch = (
        "item_id" => $hash_key,
        "label" => $pool_name,
        "hw_type" => "ibmcoss",
        "subsystem" => "pool",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
      push(@architecture, \%arch);
    }
  }

  ## DRIVE
  if ( exists $data{'drive'} && $data{'drive'} ne "" ) {
    foreach my $dev_id ( keys %{ $data{'drive'} } ) {
      foreach my $drive_uuid ( keys %{ $data{'drive'}{$dev_id} } ) {
        my $custom_uuid = "$dev_id-$drive_uuid";
        my $drive_name = $drive_uuid;
        my $custom_name = "";
        my $usage = "";

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."DRIVE".$custom_uuid); $hash_key = lc($hash_key);

        # check "data" usage, other disks do not need to be graphed
        if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'driveUsage'} && $data{'drive'}{$dev_id}{$drive_uuid}{'driveUsage'} ne "" ) {
          $usage = $data{'drive'}{$dev_id}{$drive_uuid}{'driveUsage'};
        }

        if ( $usage eq "data" ) {

          $conf{$hash_key}{'uuid'} = $drive_uuid;
          if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'name'} && $data{'drive'}{$dev_id}{$drive_uuid}{'name'} ne "" ) {
            $drive_name = $data{'drive'}{$dev_id}{$drive_uuid}{'name'};
            $conf{$hash_key}{'name'} = $drive_name;
          }
          if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'device_name'} && $data{'drive'}{$dev_id}{$drive_uuid}{'device_name'} ne "" ) {
            my $device_name = $data{'drive'}{$dev_id}{$drive_uuid}{'device_name'};
            $conf{$hash_key}{'device_name'} = $device_name;
            $custom_name = "$device_name" . "_" . "$drive_name";
            $conf{$hash_key}{'name'} = $custom_name;
          }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'status'} && $data{'drive'}{$dev_id}{$drive_uuid}{'status'} ne "" ) {
          #   $conf{$custom_uuid}{'status'} = $data{'drive'}{$dev_id}{$drive_uuid}{'status'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'capacity'} && $data{'drive'}{$dev_id}{$drive_uuid}{'capacity'} ne "" ) {
          #   $conf{$custom_uuid}{'capacity'} = $data{'drive'}{$dev_id}{$drive_uuid}{'capacity'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'firmware'} && $data{'drive'}{$dev_id}{$drive_uuid}{'firmware'} ne "" ) {
          #   $conf{$custom_uuid}{'firmware'} = $data{'drive'}{$dev_id}{$drive_uuid}{'firmware'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'slotId'} && $data{'drive'}{$dev_id}{$drive_uuid}{'slotId'} ne "" ) {
          #   $conf{$custom_uuid}{'slot_id'} = $data{'drive'}{$dev_id}{$drive_uuid}{'slotId'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'enclosureId'} && $data{'drive'}{$dev_id}{$drive_uuid}{'enclosureId'} ne "" ) {
          #   $conf{$custom_uuid}{'enclosure_id'} = $data{'drive'}{$dev_id}{$drive_uuid}{'enclosureId'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'chassisId'} && $data{'drive'}{$dev_id}{$drive_uuid}{'chassisId'} ne "" ) {
          #   $conf{$custom_uuid}{'chassis_id'} = $data{'drive'}{$dev_id}{$drive_uuid}{'chassisId'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'bay'} && $data{'drive'}{$dev_id}{$drive_uuid}{'bay'} ne "" ) {
          #   $conf{$custom_uuid}{'bay'} = $data{'drive'}{$dev_id}{$drive_uuid}{'bay'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'model'} && $data{'drive'}{$dev_id}{$drive_uuid}{'model'} ne "" ) {
          #   $conf{$custom_uuid}{'model'} = $data{'drive'}{$dev_id}{$drive_uuid}{'model'};
          # }
          # if ( defined $data{'drive'}{$dev_id}{$drive_uuid}{'serial'} && $data{'drive'}{$dev_id}{$drive_uuid}{'serial'} ne "" ) {
          #   $conf{$custom_uuid}{'serial'} = $data{'drive'}{$dev_id}{$drive_uuid}{'serial'};
          # }

          my %arch = (
            "item_id" => $hash_key,
            "label" => $custom_name,
            "hw_type" => "ibmcoss",
            "subsystem" => "drive",
            "parents" => [$cfg->{'hostcfg_id'}],
            "hostcfg_id" => $cfg->{'hostcfg_id'}
          );
          push(@architecture, \%arch);
        }
      }
    }
  }

  ##########
  ## PERF ##
  ##########

  #print Dumper \%perf;
  my $last_perf = "";

  foreach my $timestamp ( sort { $a <=> $b } keys %perf ) {

    my $ts_ref = $perf{$timestamp};


    ## POOL
    if ( defined $ts_ref->{'pool'} && $ts_ref->{'pool'} ne "" ) {

      my $dev_data_out              = "";
      my $device_capacity_total     = "";
      my $device_capacity_used      = "";
      my $device_capacity_used_pct  = "";
      my $device_capacity_free      = "";

      $data_out .= "uuid;time;capacity_total;capacity_used;capacity_free;capacity_used_percent;rebuild_queue;acknowledge_time;rebuild_bytes_in;rebuild_bytes_out;rebuild_bytes_total;rebuild_writes_in;rebuild_writes_out;rebuild_writes_total;rebuild_deletes_in;rebuild_deletes_out;rebuild_deletes_total;scanned_srcs;slices;network_rx;network_tx;network_rate\n";

      foreach my $pool ( keys %{ $ts_ref->{'pool'} } ) {

        my $hash_key = Xormon::create_hash_key($st_uuid."POOL".$pool); $hash_key = lc($hash_key);
        my $pool_name = $conf{$hash_key}{'name'};
        my $pool_ref     = $ts_ref->{'pool'}{$pool};

        my $total_cap    = "";
        my $used_cap     = "";
        my $used_pct     = "";
        my $free_cap     = "";
        my $rebuild_q    = "";
        my $ack_time     = "";
        my $rbl_b_in     = "";
        my $rbl_b_out    = "";
        my $rbl_b_tot    = "";
        my $rbl_w_in     = "";
        my $rbl_w_out    = "";
        my $rbl_w_tot    = "";
        my $rbl_del_in   = "";
        my $rbl_del_out  = "";
        my $rbl_del_tot  = "";
        my $scanned_srcs = "";
        my $slices       = "";
        my $net_in       = "";
        my $net_out      = "";
        my $net_tot      = "";

        if ( defined $pool_ref->{'total_cap'} && $pool_ref->{'total_cap'} ne "" ) {
          $total_cap = $pool_ref->{'total_cap'};
          if ($device_capacity_total) {
            $device_capacity_total += $total_cap;
          }
          else {
            $device_capacity_total = $total_cap;
          }
        }
        if ( defined $pool_ref->{'used_cap'} && $pool_ref->{'used_cap'} ne "" ) {
          $used_cap = $pool_ref->{'used_cap'};
          if ($device_capacity_used) {
            $device_capacity_used += $used_cap;
          }
          else {
            $device_capacity_used = $used_cap;
          }
        }
        if ( Xormon::isdigit($total_cap) && Xormon::isdigit($used_cap) ) {
          $free_cap = $total_cap - $used_cap;
          if ($device_capacity_free) {
            $device_capacity_free += $free_cap;
          }
          else {
            $device_capacity_free = $free_cap;
          }
        }
        if ( defined $pool_ref->{'rebuild_q'} && $pool_ref->{'rebuild_q'} ne "" ) {
          $rebuild_q = $pool_ref->{'rebuild_q'};
        }
        if ( defined $pool_ref->{'ack_t'} && $pool_ref->{'ack_t'} ne "" ) {
          $ack_time = Xormon::formatNumberByType( 'float4', (avg_hash_values( $pool_ref->{'ack_t'} ) ) );
        }
        if ( defined $pool_ref->{'rebuild_bytes_in'} && $pool_ref->{'rebuild_bytes_in'} ne "" ) {
          $rbl_b_in  = sum_hash_values( $pool_ref->{'rebuild_bytes_in'} );
          $rbl_b_tot = $rbl_b_in;
        }
        if ( defined $pool_ref->{'rebuild_bytes_out'} && $pool_ref->{'rebuild_bytes_out'} ne "" ) {
          $rbl_b_out = sum_hash_values( $pool_ref->{'rebuild_bytes_out'} );
          if ( Xormon::isdigit($rbl_b_in) ) {
            $rbl_b_tot = $rbl_b_tot + $rbl_b_out;
          }
          else {
            $rbl_b_tot = $rbl_b_out;
          }
        }
        if ( defined $pool_ref->{'rebuild_writes_in'} && $pool_ref->{'rebuild_writes_in'} ne "" ) {
          $rbl_w_in  = sum_hash_values( $pool_ref->{'rebuild_writes_in'} );
          $rbl_w_tot = $rbl_w_in;
        }
        if ( defined $pool_ref->{'rebuild_writes_out'} && $pool_ref->{'rebuild_writes_out'} ne "" ) {
          $rbl_w_out = sum_hash_values( $pool_ref->{'rebuild_writes_out'} );
          if ( Xormon::isdigit($rbl_w_in) ) {
            $rbl_w_tot = $rbl_w_tot + $rbl_w_out;
          }
          else {
            $rbl_w_tot = $rbl_w_out;
          }
        }
        if ( defined $pool_ref->{'rebuild_deletes_in'} && $pool_ref->{'rebuild_deletes_in'} ne "" ) {
          $rbl_del_in  = sum_hash_values( $pool_ref->{'rebuild_deletes_in'} );
          $rbl_del_tot = $rbl_del_in;
        }
        if ( defined $pool_ref->{'rebuild_deletes_out'} && $pool_ref->{'rebuild_deletes_out'} ne "" ) {
          $rbl_del_out = sum_hash_values( $pool_ref->{'rebuild_deletes_out'} );
          if ( Xormon::isdigit($rbl_del_in) ) {
            $rbl_del_tot = $rbl_del_tot + $rbl_del_out;
          }
          else {
            $rbl_del_tot = $rbl_del_out;
          }
        }
        if ( defined $pool_ref->{'scanned_srcs'} && $pool_ref->{'scanned_srcs'} ne "" ) {
          $scanned_srcs = sum_hash_values( $pool_ref->{'scanned_srcs'} );
        }
        if ( defined $pool_ref->{'slices'} && $pool_ref->{'slices'} ne "" ) {
          $slices = sum_hash_values( $pool_ref->{'slices'} );
        }
        if ( defined $pool_ref->{'net_in'} && $pool_ref->{'net_in'} ne "" ) {
          foreach my $slicestor ( keys %{ $pool_ref->{'net_in'} } ) {
            my $net = sum_hash_values( $pool_ref->{'net_in'}->{$slicestor} );
            if ( Xormon::isdigit($net_in) ) {
              $net_in = $net_in + $net;
            }
            else {
              $net_in = $net;
            }
          }
          $net_tot = $net_in;
        }
        if ( defined $pool_ref->{'net_out'} && $pool_ref->{'net_out'} ne "" ) {
          foreach my $slicestor ( keys %{ $pool_ref->{'net_out'} } ) {
            my $net = sum_hash_values( $pool_ref->{'net_out'}->{$slicestor} );
            if ( Xormon::isdigit($net_out) ) {
              $net_out = $net_out + $net;
            }
            else {
              $net_out = $net;
            }
          }
          if ( Xormon::isdigit($net_in) ) {
            $net_tot = $net_tot + $net_out;
          }
          else {
            $net_tot = $net_out;
          }
        }
        if ( Xormon::isdigit($total_cap) && Xormon::isdigit($used_cap) && ($total_cap != 0) ){
          $used_pct = Xormon::formatNumberByType("float4", $used_cap / ($total_cap / 100) );
        }
        $data_out .= "$hash_key;$timestamp;$total_cap;$used_cap;$free_cap;$used_pct;$rebuild_q;$ack_time;$rbl_b_in;$rbl_b_out;$rbl_b_tot;$rbl_w_in;$rbl_w_out;$rbl_w_tot;$rbl_del_in;$rbl_del_out;$rbl_del_tot;$scanned_srcs;$slices;$net_in;$net_out;$net_tot\n";
      }    # end foreach pool

      $dev_data_out .= "uuid;time;capacity_total;capacity_used;capacity_free;capacity_used_percent\n";
      if ( Xormon::isdigit($device_capacity_total) && Xormon::isdigit($device_capacity_used) && ($device_capacity_total != 0) ){
        $device_capacity_used_pct = Xormon::formatNumberByType("float4", $device_capacity_used / ($device_capacity_total / 100) );
      }
      $dev_data_out .= "$st_uuid;$timestamp;$device_capacity_total;$device_capacity_used;$device_capacity_free;$device_capacity_used_pct\n";

      #SAVE DEVICE
      #print $data_out;
      my $dev_rc = $xormon->saveCSV($dev_data_out,"ibmcoss","device");
      my $dev_status;

      # status code
      if ($dev_rc eq "201") {
        $dev_status = "successful";
      } elsif ($dev_rc eq "401") {
        $dev_status = "unauthorized";
      } else {
        $dev_status = "error";
      }
      $logger->info("TIME $timestamp : DEVICE status : $dev_status");

      #SAVE POOL
      #print $data_out;
      my $rc = $xormon->saveCSV($data_out,"ibmcoss","pool");
      my $status;

      # status code
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $logger->info("TIME $timestamp : POOL status : $status");
    }    # end if defined $ts_ref->{'pool'}
    $data_out = "";


    # VAULT
    if ( defined $ts_ref->{'vault'} && $ts_ref->{'vault'} ne "" ) {

      $data_out .= "uuid;time;capacity_used;rebuild_bytes_in;rebuild_bytes_out;rebuild_bytes_total;rebuild_writes_in;rebuild_writes_out;rebuild_writes_total;rebuild_deletes_in;rebuild_deletes_out;rebuild_deletes_total;scanned_srcs;client_in;client_out;client_total;slicestor_in;slicestor_out;slicestor_total\n";

      foreach my $vault ( keys %{ $ts_ref->{'vault'} } ) {
        my $vault_ref     = $ts_ref->{'vault'}{$vault};
        my $vault_uuid    = "";
        my $used          = "";
        my $rbl_b_in      = "";
        my $rbl_b_out     = "";
        my $rbl_b_tot     = "";
        my $rbl_w_in      = "";
        my $rbl_w_out     = "";
        my $rbl_w_tot     = "";
        my $rbl_del_in    = "";
        my $rbl_del_out   = "";
        my $rbl_del_tot   = "";
        my $scanned_srcs  = "";
        my $client_in     = "";
        my $client_out    = "";
        my $client_tot    = "";
        my $slicestor_in  = "";
        my $slicestor_out = "";
        my $slicestor_tot = "";

        if ( defined $data{'vault'}{$vault}{'uuid'} && $data{'vault'}{$vault}{'uuid'} ne "" ) {
          $vault_uuid = $data{'vault'}{$vault}{'uuid'};
        }

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."VAULT".$vault_uuid); $hash_key = lc($hash_key);

        if ( defined $vault_ref->{'used'} && $vault_ref->{'used'} ne "" ) {
          $used = $vault_ref->{'used'};
        }
        if ( defined $vault_ref->{'vault_bytes_in'} && $vault_ref->{'vault_bytes_in'} ne "" ) {
          $rbl_b_in  = sum_hash_values( $vault_ref->{'vault_bytes_in'} );
          $rbl_b_tot = $rbl_b_in;
        }
        if ( defined $vault_ref->{'vault_bytes_out'} && $vault_ref->{'vault_bytes_out'} ne "" ) {
          $rbl_b_out = sum_hash_values( $vault_ref->{'vault_bytes_out'} );
          if ( Xormon::isdigit($rbl_b_in) ) {
            $rbl_b_tot = $rbl_b_tot + $rbl_b_out;
          }
          else {
            $rbl_b_tot = $rbl_b_out;
          }
        }
        if ( defined $vault_ref->{'vault_writes_in'} && $vault_ref->{'vault_writes_in'} ne "" ) {
          $rbl_w_in  = sum_hash_values( $vault_ref->{'vault_writes_in'} );
          $rbl_w_tot = $rbl_w_in;
        }
        if ( defined $vault_ref->{'vault_writes_out'} && $vault_ref->{'vault_writes_out'} ne "" ) {
          $rbl_w_out = sum_hash_values( $vault_ref->{'vault_writes_out'} );
          if ( Xormon::isdigit($rbl_w_in) ) {
            $rbl_w_tot = $rbl_w_tot + $rbl_w_out;
          }
          else {
            $rbl_w_tot = $rbl_w_out;
          }
        }
        if ( defined $vault_ref->{'vault_deletes_in'} && $vault_ref->{'vault_deletes_in'} ne "" ) {
          $rbl_del_in  = sum_hash_values( $vault_ref->{'vault_deletes_in'} );
          $rbl_del_tot = $rbl_del_in;
        }
        if ( defined $vault_ref->{'vault_deletes_out'} && $vault_ref->{'vault_deletes_out'} ne "" ) {
          $rbl_del_out = sum_hash_values( $vault_ref->{'vault_deletes_out'} );
          if ( Xormon::isdigit($rbl_del_in) ) {
            $rbl_del_tot = $rbl_del_tot + $rbl_del_out;
          }
          else {
            $rbl_del_tot = $rbl_del_out;
          }
        }
        if ( defined $vault_ref->{'scanned_srcs'} && $vault_ref->{'scanned_srcs'} ne "" ) {
          $scanned_srcs = sum_hash_values( $vault_ref->{'scanned_srcs'} );
        }
        if ( defined $vault_ref->{'client_in'} && $vault_ref->{'client_in'} ne "" ) {
          $client_in  = sum_hash_values( $vault_ref->{'client_in'} );
          $client_tot = $client_in;
        }
        if ( defined $vault_ref->{'client_out'} && $vault_ref->{'client_out'} ne "" ) {
          $client_out = sum_hash_values( $vault_ref->{'client_out'} );
          if ( Xormon::isdigit($client_in) ) {
            $client_tot = $client_tot + $client_out;
          }
          else {
            $client_tot = $client_out;
          }
        }
        if ( defined $vault_ref->{'slicestor_in'} && $vault_ref->{'slicestor_in'} ne "" ) {
          $slicestor_in  = sum_hash_values( $vault_ref->{'slicestor_in'} );
          $slicestor_tot = $slicestor_in;
        }
        if ( defined $vault_ref->{'slicestor_out'} && $vault_ref->{'slicestor_out'} ne "" ) {
          $slicestor_out = sum_hash_values( $vault_ref->{'slicestor_out'} );
          if ( Xormon::isdigit($slicestor_in) ) {
            $slicestor_tot = $slicestor_tot + $slicestor_out;
          }
          else {
            $slicestor_tot = $slicestor_out;
          }
        }
        $data_out .= "$hash_key;$timestamp;$used;$rbl_b_in;$rbl_b_out;$rbl_b_tot;$rbl_w_in;$rbl_w_out;$rbl_w_tot;$rbl_del_in;$rbl_del_out;$rbl_del_tot;$scanned_srcs;$client_in;$client_out;$client_tot;$slicestor_in;$slicestor_out;$slicestor_tot\n";
      }    # end foreach vault

      #SAVE VAULT
      #print $data_out;
      my $rc = $xormon->saveCSV($data_out,"ibmcoss","vault");
      my $status;

      #status code
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $logger->info("TIME $timestamp : VAULT status : $status");
    }    # end if defined $ts_ref->{'VAULT'}
    $data_out = "";


    ## MANAGER
    if ( defined $ts_ref->{'manager'} && $ts_ref->{'manager'} ne "" ) {

      $data_out .= "uuid;time;cpu_busy;cpu_user;cpu_system;cpu_wait;system_memory_total;system_memory_used;onheap_memory_total;onheap_memory_used;offheap_memory_total;offheap_memory_used;network_rx;network_tx;network_rate;tcp_timeouts;os_r;os_w;os_total\n";

      foreach my $manager ( keys %{ $ts_ref->{'manager'} } ) {
        my $dev_ref      = $ts_ref->{'manager'}{$manager};
        my $uuid         = "";
        # my $cpu_load     = "";
        my $cpu_tot      = "";
        my $cpu_usr      = "";
        my $cpu_sys      = "";
        my $cpu_wait     = "";
        my $mem_sys_tot  = "";
        my $mem_sys_used = "";
        my $mem_on_tot   = "";
        my $mem_on_used  = "";
        my $mem_off_tot  = "";
        my $mem_off_used = "";
        my $net_in       = "";
        my $net_out      = "";
        my $net_tot      = "";
        my $tcp_timeouts = "";
        my $os_r         = "";
        my $os_w         = "";
        my $os_tot       = "";

        if ( defined $data{'manager'}{$manager}{'uuid'} && $data{'manager'}{$manager}{'uuid'} ne "" ) {
          $uuid = $data{'manager'}{$manager}{'uuid'};
        }

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."MANAGER".$uuid); $hash_key = lc($hash_key);

        # if ( defined $dev_ref->{'cpu_load'} && $dev_ref->{'cpu_load'} ne "" ) {
        #   $cpu_load = $dev_ref->{'cpu_load'};
        # }
        if ( defined $dev_ref->{'cpu_perc'} && $dev_ref->{'cpu_perc'} ne "" ) {
          $cpu_tot = sum_hash_values( $dev_ref->{'cpu_perc'} );
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuUser'} && $dev_ref->{'cpu_perc'}{'cpuUser'} ne "" ) {
          $cpu_usr = $dev_ref->{'cpu_perc'}{'cpuUser'};
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuSystem'} && $dev_ref->{'cpu_perc'}{'cpuSystem'} ne "" ) {
          $cpu_sys = $dev_ref->{'cpu_perc'}{'cpuSystem'};
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuWait'} && $dev_ref->{'cpu_perc'}{'cpuWait'} ne "" ) {
          $cpu_wait = $dev_ref->{'cpu_perc'}{'cpuWait'};
        }
        if ( defined $dev_ref->{'sys_mem_total'} && $dev_ref->{'sys_mem_total'} ne "" ) {
          $mem_sys_tot = $dev_ref->{'sys_mem_total'};
        }
        if ( defined $dev_ref->{'sys_mem_used'} && $dev_ref->{'sys_mem_used'} ne "" ) {
          $mem_sys_used = $dev_ref->{'sys_mem_used'};
        }
        if ( defined $dev_ref->{'onheap_mem_total'} && $dev_ref->{'onheap_mem_total'} ne "" ) {
          $mem_on_tot = $dev_ref->{'onheap_mem_total'};
        }
        if ( defined $dev_ref->{'onheap_mem_used'} && $dev_ref->{'onheap_mem_used'} ne "" ) {
          $mem_on_used = $dev_ref->{'onheap_mem_used'};
        }
        if ( defined $dev_ref->{'offheap_mem_total'} && $dev_ref->{'offheap_mem_total'} ne "" ) {
          $mem_off_tot = $dev_ref->{'offheap_mem_total'};
        }
        if ( defined $dev_ref->{'offheap_mem_used'} && $dev_ref->{'offheap_mem_used'} ne "" ) {
          $mem_off_used = $dev_ref->{'offheap_mem_used'};
        }
        if ( defined $dev_ref->{'net_in'} && $dev_ref->{'net_in'} ne "" ) {
          $net_in  = sum_hash_values( $dev_ref->{'net_in'} );
          $net_tot = $net_in;
        }
        if ( defined $dev_ref->{'net_out'} && $dev_ref->{'net_out'} ne "" ) {
          $net_out = sum_hash_values( $dev_ref->{'net_out'} );
          if ( Xormon::isdigit($net_in) ) {
            $net_tot = $net_tot + $net_out;
          }
          else {
            $net_tot = $net_out;
          }
        }
        if ( defined $dev_ref->{'tcp_timeouts'} && $dev_ref->{'tcp_timeouts'} ne "" ) {
          $tcp_timeouts = $dev_ref->{'tcp_timeouts'};
        }
        if ( defined $dev_ref->{'os_read'} && $dev_ref->{'os_read'} ne "" ) {
          $os_r   = sum_hash_values( $dev_ref->{'os_read'} );
          $os_tot = $os_r;
        }
        if ( defined $dev_ref->{'os_write'} && $dev_ref->{'os_write'} ne "" ) {
          $os_w = sum_hash_values( $dev_ref->{'os_write'} );
          if ( Xormon::isdigit($os_r) ) {
            $os_tot = $os_tot + $os_w;
          }
          else {
            $os_tot = $os_w;
          }
        }
        $data_out .= "$hash_key;$timestamp;$cpu_tot;$cpu_usr;$cpu_sys;$cpu_wait;$mem_sys_tot;$mem_sys_used;$mem_on_tot;$mem_on_used;$mem_off_tot;$mem_off_used;$net_in;$net_out;$net_tot;$tcp_timeouts;$os_r;$os_w;$os_tot\n";
      }    # end foreach manager

      #SAVE MANAGER
      #print $data_out;
      my $rc = $xormon->saveCSV($data_out,"ibmcoss","manager");
      my $status;

      #status code
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $logger->info("TIME $timestamp : MANAGER status : $status");
    }    # end if defined $ts_ref->{'manager'}
    $data_out = "";


    ## ACCESSER
    if ( defined $ts_ref->{'accesser'} && $ts_ref->{'accesser'} ne "" ) {

      $data_out .= "uuid;time;cpu_busy;cpu_user;cpu_system;cpu_wait;system_memory_total;system_memory_used;onheap_memory_total;onheap_memory_used;offheap_memory_total;offheap_memory_used;acknowledge_time;network_rx;network_tx;network_rate;tcp_timeouts;os_r;os_w;os_total\n";

      foreach my $accesser ( keys %{ $ts_ref->{'accesser'} } ) {
        my $dev_ref      = $ts_ref->{'accesser'}{$accesser};
        my $uuid         = "";
        # my $cpu_load     = "";
        my $cpu_tot      = "";
        my $cpu_usr      = "";
        my $cpu_sys      = "";
        my $cpu_wait     = "";
        my $mem_sys_tot  = "";
        my $mem_sys_used = "";
        my $mem_on_tot   = "";
        my $mem_on_used  = "";
        my $mem_off_tot  = "";
        my $mem_off_used = "";
        my $ack_time     = "";
        my $net_in       = "";
        my $net_out      = "";
        my $net_tot      = "";
        my $tcp_timeouts = "";
        my $os_r         = "";
        my $os_w         = "";
        my $os_tot       = "";

        if ( defined $data{'accesser'}{$accesser}{'uuid'} && $data{'accesser'}{$accesser}{'uuid'} ne "" ) {
          $uuid = $data{'accesser'}{$accesser}{'uuid'};
        }

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."ACCESSER".$uuid); $hash_key = lc($hash_key);

        # if ( defined $dev_ref->{'cpu_load'} && $dev_ref->{'cpu_load'} ne "" ) {
        #   $cpu_load = $dev_ref->{'cpu_load'};
        # }
        if ( defined $dev_ref->{'cpu_perc'} && $dev_ref->{'cpu_perc'} ne "" ) {
          $cpu_tot = sum_hash_values( $dev_ref->{'cpu_perc'} );
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuUser'} && $dev_ref->{'cpu_perc'}{'cpuUser'} ne "" ) {
          $cpu_usr = $dev_ref->{'cpu_perc'}{'cpuUser'};
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuSystem'} && $dev_ref->{'cpu_perc'}{'cpuSystem'} ne "" ) {
          $cpu_sys = $dev_ref->{'cpu_perc'}{'cpuSystem'};
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuWait'} && $dev_ref->{'cpu_perc'}{'cpuWait'} ne "" ) {
          $cpu_wait = $dev_ref->{'cpu_perc'}{'cpuWait'};
        }
        if ( defined $dev_ref->{'sys_mem_total'} && $dev_ref->{'sys_mem_total'} ne "" ) {
          $mem_sys_tot = $dev_ref->{'sys_mem_total'};
        }
        if ( defined $dev_ref->{'sys_mem_used'} && $dev_ref->{'sys_mem_used'} ne "" ) {
          $mem_sys_used = $dev_ref->{'sys_mem_used'};
        }
        if ( defined $dev_ref->{'onheap_mem_total'} && $dev_ref->{'onheap_mem_total'} ne "" ) {
          $mem_on_tot = $dev_ref->{'onheap_mem_total'};
        }
        if ( defined $dev_ref->{'onheap_mem_used'} && $dev_ref->{'onheap_mem_used'} ne "" ) {
          $mem_on_used = $dev_ref->{'onheap_mem_used'};
        }
        if ( defined $dev_ref->{'offheap_mem_total'} && $dev_ref->{'offheap_mem_total'} ne "" ) {
          $mem_off_tot = $dev_ref->{'offheap_mem_total'};
        }
        if ( defined $dev_ref->{'offheap_mem_used'} && $dev_ref->{'offheap_mem_used'} ne "" ) {
          $mem_off_used = $dev_ref->{'offheap_mem_used'};
        }
        if ( defined $dev_ref->{'ack_t'} && $dev_ref->{'ack_t'} ne "" ) {
          $ack_time = avg_hash_values( $dev_ref->{'ack_t'} );
        }
        if ( defined $dev_ref->{'net_in'} && $dev_ref->{'net_in'} ne "" ) {
          $net_in  = sum_hash_values( $dev_ref->{'net_in'} );
          $net_tot = $net_in;
        }
        if ( defined $dev_ref->{'net_out'} && $dev_ref->{'net_out'} ne "" ) {
          $net_out = sum_hash_values( $dev_ref->{'net_out'} );
          if ( Xormon::isdigit($net_in) ) {
            $net_tot = $net_tot + $net_out;
          }
          else {
            $net_tot = $net_out;
          }
        }
        if ( defined $dev_ref->{'tcp_timeouts'} && $dev_ref->{'tcp_timeouts'} ne "" ) {
          $tcp_timeouts = $dev_ref->{'tcp_timeouts'};
        }
        if ( defined $dev_ref->{'os_read'} && $dev_ref->{'os_read'} ne "" ) {
          $os_r   = sum_hash_values( $dev_ref->{'os_read'} );
          $os_tot = $os_r;
        }
        if ( defined $dev_ref->{'os_write'} && $dev_ref->{'os_write'} ne "" ) {
          $os_w = sum_hash_values( $dev_ref->{'os_write'} );
          if ( Xormon::isdigit($os_r) ) {
            $os_tot = $os_tot + $os_w;
          }
          else {
            $os_tot = $os_w;
          }
        }
        $data_out .= "$hash_key;$timestamp;$cpu_tot;$cpu_usr;$cpu_sys;$cpu_wait;$mem_sys_tot;$mem_sys_used;$mem_on_tot;$mem_on_used;$mem_off_tot;$mem_off_used;$ack_time;$net_in;$net_out;$net_tot;$tcp_timeouts;$os_r;$os_w;$os_tot\n";
      }    # end foreach accesser

      #SAVE ACCESSER
      #print $data_out;
      my $rc = $xormon->saveCSV($data_out,"ibmcoss","accesser");
      my $status;

      #status code
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $logger->info("TIME $timestamp : ACCESSER status : $status");
    }    # end if defined $ts_ref->{'accesser'}
    $data_out = "";


    ## SLICESTOR
    if ( defined $ts_ref->{'slicestor'} && $ts_ref->{'slicestor'} ne "" ) {

      $data_out.= "uuid;time;cpu_busy;cpu_user;cpu_system;cpu_wait;system_memory_total;system_memory_used;onheap_memory_total;onheap_memory_used;offheap_memory_total;offheap_memory_used;network_rx;network_tx;network_rate;tcp_timeouts;os_r;os_w;os_total;scanned_srcs;rebuild_writes_in;rebuild_writes_out;rebuild_writes_total;rebuild_deletes_in;rebuild_deletes_out;rebuild_deletes_total;rebuild_bytes_in;rebuild_bytes_out;rebuild_bytes_total;capacity_total;capacity_used;capacity_used_percent\n";

      foreach my $slicestor ( keys %{ $ts_ref->{'slicestor'} } ) {
        my $dev_ref      = $ts_ref->{'slicestor'}{$slicestor};
        my $uuid         = "";
        # my $cpu_load     = "";
        my $cpu_tot      = "";
        my $cpu_usr      = "";
        my $cpu_sys      = "";
        my $cpu_wait     = "";
        my $mem_sys_tot  = "";
        my $mem_sys_used = "";
        my $mem_on_tot   = "";
        my $mem_on_used  = "";
        my $mem_off_tot  = "";
        my $mem_off_used = "";
        my $net_in       = "";
        my $net_out      = "";
        my $net_tot      = "";
        my $tcp_timeouts = "";
        my $os_r         = "";
        my $os_w         = "";
        my $os_tot       = "";
        my $scanned_srcs = "";
        my $writes_in    = "";
        my $writes_out   = "";
        my $writes_tot   = "";
        my $deletes_in   = "";
        my $deletes_out  = "";
        my $deletes_tot  = "";
        my $bytes_in     = "";
        my $bytes_out    = "";
        my $bytes_tot    = "";
        my $total_cap    = "";
        my $used_cap     = "";
        my $used_pct     = "";

        if ( defined $data{'slicestor'}{$slicestor}{'uuid'} && $data{'slicestor'}{$slicestor}{'uuid'} ne "" ) {
          $uuid = $data{'slicestor'}{$slicestor}{'uuid'};
        }

        my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."SLICESTOR".$uuid); $hash_key = lc($hash_key);

        # if ( defined $dev_ref->{'cpu_load'} && $dev_ref->{'cpu_load'} ne "" ) {
        #   $cpu_load = $dev_ref->{'cpu_load'};
        # }
        if ( defined $dev_ref->{'cpu_perc'} && $dev_ref->{'cpu_perc'} ne "" ) {
          $cpu_tot = sum_hash_values( $dev_ref->{'cpu_perc'} );
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuUser'} && $dev_ref->{'cpu_perc'}{'cpuUser'} ne "" ) {
          $cpu_usr = $dev_ref->{'cpu_perc'}{'cpuUser'};
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuSystem'} && $dev_ref->{'cpu_perc'}{'cpuSystem'} ne "" ) {
          $cpu_sys = $dev_ref->{'cpu_perc'}{'cpuSystem'};
        }
        if ( defined $dev_ref->{'cpu_perc'}{'cpuWait'} && $dev_ref->{'cpu_perc'}{'cpuWait'} ne "" ) {
          $cpu_wait = $dev_ref->{'cpu_perc'}{'cpuWait'};
        }
        if ( defined $dev_ref->{'sys_mem_total'} && $dev_ref->{'sys_mem_total'} ne "" ) {
          $mem_sys_tot = $dev_ref->{'sys_mem_total'};
        }
        if ( defined $dev_ref->{'sys_mem_used'} && $dev_ref->{'sys_mem_used'} ne "" ) {
          $mem_sys_used = $dev_ref->{'sys_mem_used'};
        }
        if ( defined $dev_ref->{'onheap_mem_total'} && $dev_ref->{'onheap_mem_total'} ne "" ) {
          $mem_on_tot = $dev_ref->{'onheap_mem_total'};
        }
        if ( defined $dev_ref->{'onheap_mem_used'} && $dev_ref->{'onheap_mem_used'} ne "" ) {
          $mem_on_used = $dev_ref->{'onheap_mem_used'};
        }
        if ( defined $dev_ref->{'offheap_mem_total'} && $dev_ref->{'offheap_mem_total'} ne "" ) {
          $mem_off_tot = $dev_ref->{'offheap_mem_total'};
        }
        if ( defined $dev_ref->{'offheap_mem_used'} && $dev_ref->{'offheap_mem_used'} ne "" ) {
          $mem_off_used = $dev_ref->{'offheap_mem_used'};
        }
        if ( defined $dev_ref->{'net_in'} && $dev_ref->{'net_in'} ne "" ) {
          $net_in  = sum_hash_values( $dev_ref->{'net_in'} );
          $net_tot = $net_in;
        }
        if ( defined $dev_ref->{'net_out'} && $dev_ref->{'net_out'} ne "" ) {
          $net_out = sum_hash_values( $dev_ref->{'net_out'} );
          if ( Xormon::isdigit($net_in) ) {
            $net_tot = $net_tot + $net_out;
          }
          else {
            $net_tot = $net_out;
          }
        }
        if ( defined $dev_ref->{'tcp_timeouts'} && $dev_ref->{'tcp_timeouts'} ne "" ) {
          $tcp_timeouts = $dev_ref->{'tcp_timeouts'};
        }
        if ( defined $dev_ref->{'os_read'} && $dev_ref->{'os_read'} ne "" ) {
          $os_r   = sum_hash_values( $dev_ref->{'os_read'} );
          $os_tot = $os_r;
        }
        if ( defined $dev_ref->{'os_write'} && $dev_ref->{'os_write'} ne "" ) {
          $os_w = sum_hash_values( $dev_ref->{'os_write'} );
          if ( Xormon::isdigit($os_r) ) {
            $os_tot = $os_tot + $os_w;
          }
          else {
            $os_tot = $os_w;
          }
        }
        if ( defined $dev_ref->{'slicestor_scan_rate'} && $dev_ref->{'slicestor_scan_rate'} ne "" ) {
          $scanned_srcs = $dev_ref->{'slicestor_scan_rate'};
        }
        if ( defined $dev_ref->{'slicestor_writes_in'} && $dev_ref->{'slicestor_writes_in'} ne "" ) {
          $writes_in  = sum_hash_values( $dev_ref->{'slicestor_writes_in'} );
          $writes_tot = $writes_in;
        }
        if ( defined $dev_ref->{'slicestor_writes_out'} && $dev_ref->{'slicestor_writes_out'} ne "" ) {
          $writes_out = sum_hash_values( $dev_ref->{'slicestor_writes_out'} );
          if ( Xormon::isdigit($writes_in) ) {
            $writes_tot = $writes_in + $writes_out;
          }
          else {
            $writes_tot = $writes_out;
          }
        }
        if ( defined $dev_ref->{'slicestor_deletes_in'} && $dev_ref->{'slicestor_deletes_in'} ne "" ) {
          $deletes_in  = sum_hash_values( $dev_ref->{'slicestor_deletes_in'} );
          $deletes_tot = $deletes_in;
        }
        if ( defined $dev_ref->{'slicestor_deletes_out'} && $dev_ref->{'slicestor_deletes_out'} ne "" ) {
          $deletes_out = sum_hash_values( $dev_ref->{'slicestor_deletes_out'} );
          if ( Xormon::isdigit($deletes_in) ) {
            $deletes_tot = $deletes_in + $deletes_out;
          }
          else {
            $deletes_tot = $deletes_out;
          }
        }
        if ( defined $dev_ref->{'slicestor_bytes_in'} && $dev_ref->{'slicestor_bytes_in'} ne "" ) {
          $bytes_in  = sum_hash_values( $dev_ref->{'slicestor_bytes_in'} );
          $bytes_tot = $bytes_in;
        }
        if ( defined $dev_ref->{'slicestor_bytes_out'} && $dev_ref->{'slicestor_bytes_out'} ne "" ) {
          $bytes_out = sum_hash_values( $dev_ref->{'slicestor_bytes_out'} );
          if ( Xormon::isdigit($bytes_in) ) {
            $bytes_tot = $bytes_in + $bytes_out;
          }
          else {
            $bytes_tot = $bytes_out;
          }
        }
        if ( defined $data{'slicestor'}{$slicestor}{'capacity'} && $data{'slicestor'}{$slicestor}{'capacity'} ne "" ) {
          $total_cap = $data{'slicestor'}{$slicestor}{'capacity'};    # get value from conf hash
        }
        if ( defined $data{'slicestor'}{$slicestor}{'used'} && $data{'slicestor'}{$slicestor}{'used'} ne "" ) {
          $used_cap = $data{'slicestor'}{$slicestor}{'used'};         # get value from conf hash
        }
        if ( Xormon::isdigit($total_cap) && Xormon::isdigit($used_cap) && ($total_cap != 0) ){
          $used_pct = Xormon::formatNumberByType("float4", $used_cap / ($total_cap / 100) );
        }
        $data_out .= "$hash_key;$timestamp;$cpu_tot;$cpu_usr;$cpu_sys;$cpu_wait;$mem_sys_tot;$mem_sys_used;$mem_on_tot;$mem_on_used;$mem_off_tot;$mem_off_used;$net_in;$net_out;$net_tot;$tcp_timeouts;$os_r;$os_w;$os_tot;$scanned_srcs;$writes_in;$writes_out;$writes_tot;$deletes_in;$deletes_out;$deletes_tot;$bytes_in;$bytes_out;$bytes_tot;$total_cap;$used_cap;$used_pct\n";
      }    # end foreach slicestor

      #SAVE SLICESTOR
      #print $data_out;
      my $rc = $xormon->saveCSV($data_out,"ibmcoss","slicestor");
      my $status;

      #status code
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $logger->info("TIME $timestamp : SLICESTOR status : $status");

    }    # end if defined $ts_ref->{'slicestor'}
    $data_out = "";

    ## DRIVE
    if ( defined $ts_ref->{'DRIVE'} && $ts_ref->{'DRIVE'} ne "" ) {

      $data_out .= "uuid;time;read;write;data_rate\n";

      foreach my $dev_id ( keys %{ $ts_ref->{'DRIVE'} } ) {
        my $dev_ref = $ts_ref->{'DRIVE'}{$dev_id};
        foreach my $drive ( keys %{$dev_ref} ) {
          my $drive_ref = $dev_ref->{$drive};
          my $cust_uuid = "$dev_id-$drive";
          my $data_r    = "";
          my $data_w    = "";
          my $data      = "";

          my $hash_key = Xormon::create_hash_key($cfg->{hostcfg_id}."DRIVE".$cust_uuid); $hash_key = lc($hash_key);

          if ( defined $data{'drive'}{$dev_id}{$drive}{'driveUsage'} && $data{'drive'}{$dev_id}{$drive}{'driveUsage'} eq "data" ) {

            if ( defined $drive_ref->{'data_read'} && $drive_ref->{'data_read'} ne "" ) {
              $data_r = $drive_ref->{'data_read'};
              $data   = $data_r;
            }
            if ( defined $drive_ref->{'data_write'} && $drive_ref->{'data_write'} ne "" ) {
              $data_w = $drive_ref->{'data_write'};
              if ( Xormon::isdigit($data_r) ) {
                $data = $data + $data_w;
              }
              else {
                $data = $data_w;
              }
            }
            $data_out .= "$hash_key;$timestamp;$data_r;$data_w;$data\n";
          }
        }    # end foreach drive
      }    # end foreach dev_id

      #SAVE DRIVE
      #print $data_out;
      my $rc = $xormon->saveCSV($data_out,"ibmcoss","drive");
      my $status;

      #status code
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }
      $logger->info("TIME $timestamp : DRIVE status : $status");

    }    # end if defined $ts_ref->{'DRIVE'}
    $data_out = "";

    $last_perf = $timestamp;
  }   # end foreach timestamp

  if ( Xormon::isdigit($last_perf) ) {
    if ( ( time() - $last_perf ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: $last_perf");
    }
    else {
      $xormon->hasData(1);
    }
  }
  else {
    $xormon->hasData(0, "no available stats");
  }

  if ( Xormon::isdigit($last_perf) ) {
    $service_data_out->{last_perf_timestamp_ms} = $last_perf * 1000; # ts to ms
  }

  $xormon->saveData(\%{ $service_data_out });

  #print Dumper \%conf;
  #save configuration
  $xormon->saveConfJSON(\%conf);

  #save architecture
  #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);
}

sub connection_test {
  my $client = shift;
  $cfg = shift;

  my @rows;
  my %conn_data;

  my $test = Test->new($client);

  # # test main variables
  # #
  # unless ( exists $cfg->{'hostcfg_id'} )    { push(@rows, "Cannot continue, 'hostcfg_id' is not configured!");   return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'hostalias'} )     { push(@rows, "Cannot continue, 'hostalias' is not configured!");    return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiHost'} )       { push(@rows, "Cannot continue, 'apiHost' is not configured!");      return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiPort'} )       { push(@rows, "Cannot continue, 'apiPort' is not configured!");      return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiUsername'} )   { push(@rows, "Cannot continue, 'apiUsername' is not configured!");  return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiPassword'} )   { push(@rows, "Cannot continue, 'apiPassword' is not configured!");  return Xormon::conn_test(0, \@rows); }

  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiPort", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  my $api_ip   = $cfg->{'apiHost'};

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'apiHost'},$cfg->{'apiPort'});
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $time      = time();
  my $endtime   = int( $time / 300 ) * 300 * 1000;        # 5mins in millisec
  my $endtime_e = int( $time / 300 ) * 300 - 300;         # 5mins
  my $starttime = $endtime - ( 3600 * 1000 ) - 300000;    # 1hour back in millisec
  my $return;

  $return = new_get_request("https://$api_ip/manager/api/json/1.0/viewSystem.adm");
  if ( $return && defined $return->{'responseData'}->{'devices'} && ref( $return->{'responseData'}->{'devices'} ) eq "ARRAY" ) {
    foreach my $device ( @{ $return->{'responseData'}->{'devices'} } ) {
      if ( defined $device->{'id'} && $device->{'id'} ne "" ) {
        my $id          = $device->{'id'};
        my $device_type = "";
        if ( defined $device->{'deviceType'} && $device->{'deviceType'} ne "" ) {
          $device_type = $device->{'deviceType'};
          if ( $device_type eq "slicestor" ) {
            if ( defined $device->{'capacity'} && $device->{'capacity'} ne "" ) {
              $conn_data{$device_type}{$id}{'capacity'} = $device->{'capacity'};
            }
            if ( defined $device->{'utilization'} && $device->{'utilization'} ne "" ) {
              $conn_data{$device_type}{$id}{'used'} = $device->{'utilization'};
            }
          }
        }
        if ( defined $device->{'version'} && $device->{'version'} ne "" ) {
          $conn_data{$device_type}{$id}{'version'} = $device->{'version'};
        }
        if ( defined $device->{'health'} && $device->{'health'} ne "" ) {
          $conn_data{$device_type}{$id}{'health'} = $device->{'health'};
        }
        if ( defined $device->{'uuid'} && $device->{'uuid'} ne "" ) {
          $conn_data{$device_type}{$id}{'uuid'} = $device->{'uuid'};
        }
        if ( defined $device->{'hostname'} && $device->{'hostname'} ne "" ) {
          $conn_data{$device_type}{$id}{'hostname'} = $device->{'hostname'};
        }
        if ( defined $device->{'alias'} && $device->{'alias'} ne "" ) {
          $conn_data{$device_type}{$id}{'alias'} = $device->{'alias'};
        }
        if ( defined $device->{'site'} && $device->{'site'} ne "" ) {
          $conn_data{$device_type}{$id}{'site'} = $device->{'site'};
        }
        if ( defined $device->{'ip'} && $device->{'ip'} ne "" ) {
          $conn_data{$device_type}{$id}{'ip'} = $device->{'ip'};
        }
      }
    }
    $test->addSection("API authorization", 1, "Data returned from $api_ip");
  }
  else {
    $test->addSection("API authorization", 0, "No data returned from $api_ip!");
    return $test->stop();
  }

  if ( exists $conn_data{'manager'} && $conn_data{'manager'} ne "" ) {
    my $ip       = "";
    my $hostname = "";
    my $version  = "";
    my $name     = "";
    foreach my $manager ( keys %{ $conn_data{'manager'} } ) {
      if ( defined $conn_data{'manager'}{$manager}{'version'} && $conn_data{'manager'}{$manager}{'version'} ne "" ) {
        $version = $conn_data{'manager'}{$manager}{'version'};
      }
      if ( defined $conn_data{'manager'}{$manager}{'alias'} && $conn_data{'manager'}{$manager}{'alias'} ne "" ) {
        $name = $conn_data{'manager'}{$manager}{'alias'};
      }
      if ( defined $conn_data{'manager'}{$manager}{'hostname'} && $conn_data{'manager'}{$manager}{'hostname'} ne "" ) {
        $hostname = $conn_data{'manager'}{$manager}{'hostname'};
      }
      if ( defined $conn_data{'manager'}{$manager}{'ip'} && $conn_data{'manager'}{$manager}{'ip'} ne "" ) {
        $ip = $conn_data{'manager'}{$manager}{'ip'};
      }
      if ( ($ip eq $api_ip) || ($hostname eq $api_ip) ) {
        push(@rows, "IP : $ip\n");
        push(@rows, "hostname : $hostname\n");
        push(@rows, "name : $name\n");
        push(@rows, "version : $version\n");
        $test->addSection("Manager data", 1, \@rows);
        last;
      }
    }
    if ($ip eq "" || $hostname eq ""){
      $test->addSection("Manager data", 0, "No valid data from managers!");
      return $test->stop();
    }
  }
  else {
    $test->addSection("Manager data", 0, "Manager data not received from $api_ip!");
    return $test->stop();
  }

  $return = new_get_request("https://$api_ip/manager/api/json/1.0/listDevices.adm");
  if ( $return && defined $return->{'responseData'}->{'devices'} && ref( $return->{'responseData'}->{'devices'} ) eq "ARRAY" ) {
    foreach my $device ( @{ $return->{'responseData'}->{'devices'} } ) {

      #print Dumper $device;
      if ( defined $device->{'id'} && $device->{'id'} ne "" ) {
        my $id = $device->{'id'};
        if ( defined $device->{'deviceType'} && $device->{'deviceType'} ne "" ) {
          my $subsystem = $device->{'deviceType'};
          ## CPU LOAD
          my $metric   = "cpu_load";
          my $n_return = new_get_request("https://$api_ip/manager/api/sandra/device/$id/statistic/load.json?startDate=$starttime&endDate=$endtime");

          #print Dumper $n_return;
          if ( defined $n_return && $n_return ne "" && ref($n_return) eq "ARRAY" && scalar( @{$n_return} ) ) {
            get_5min_data( $subsystem, $id, $metric, "", @{$n_return}[0]->{'data'} );
          }
          else {
            $test->addSection("Performance data", 0, "Performance data not returned from $api_ip!");
            return $test->stop();
          }
        }
      }
    }
  }
  else {
    $test->addSection("Performance data", 0, "Performance data not returned from $api_ip!");
    return $test->stop();
  }

  if ( defined $perf{$endtime_e}{'manager'} && $perf{$endtime_e}{'manager'} ne "" ) {
    $test->addSection("Performance data", 1, "Performance data OK");
  }
  else {
    $test->addSection("Performance data", 0, "Performance data not returned from $api_ip!");

  }
  return $test->stop();

}

sub new_get_request {
  my $url    = shift;
  my $api_user = $cfg->{'apiUsername'};
  my $api_pass = $cfg->{'apiPassword'};
  my $result = "";

  my $agent   = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $api_user, $api_pass );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    $logger->error("GET url           : $url");
    $logger->error("Request error     : $st_line");

    return 0;
  }

  #print Dumper $results;

  if ( defined $results->content && $results->content ne "" ) {
    if ( $url =~ m/ajax\/stream.adm/ ) {
      my (@arr) = split( /\n/, $results->content );
      foreach my $line (@arr) {
        if ( $line =~ m/^jsonEventData =/ ) {
          $line =~ s/^jsonEventData.=.//g;
          $line =~ s/;$//g;
          my $result = decode_json($line);
          return $result;
        }
      }
    }
    else {
      my $result = decode_json( $results->content );
      return $result;
    }
  }
  return 0;
}

sub get_5min_data {
  my $subsystem = shift;
  my $id        = shift;
  my $metric    = shift;
  my $device_id = shift;
  my $hash_ref  = shift;
  my $first     = 0;

  foreach my $timestamp ( @{$hash_ref} ) {
    if ( exists $timestamp->{'timestamp'} && Xormon::isdigit( $timestamp->{'timestamp'} ) ) {
      my $sec_timestamp = int( $timestamp->{'timestamp'} / 1000 );
      my $perftimestamp = int( $sec_timestamp / 300 ) * 300;

      my $value = "";
      if ( exists $timestamp->{'value'} && Xormon::isdigit( $timestamp->{'value'} ) ) {
        $value = $timestamp->{'value'};
      }
      if ($first) {
        if ( $device_id ne "" ) {
          $perf{$perftimestamp}{$subsystem}{$id}{$metric}{$device_id} = $value;
          if ( $subsystem eq "accesser" && $metric eq "ack_t" ) {
            $perf{$perftimestamp}{'pool'}{$device_id}{$metric}{$id} = $value;
          }
        }
        else {
          $perf{$perftimestamp}{$subsystem}{$id}{$metric} = $value;
        }
      }
    }
    $first = 1;
  }
}

sub get_5min_counter {
  my $subsystem      = shift;
  my $id             = shift;
  my $metric         = shift;
  my $sub_metric     = shift;
  my $hash_ref       = shift;
  my $prev_timestamp = "";
  my $prev_counter   = "";

  foreach my $timestamp ( @{$hash_ref} ) {
    if ( exists $timestamp->{'timestamp'} && Xormon::isdigit( $timestamp->{'timestamp'} ) ) {
      my $sec_timestamp = int( $timestamp->{'timestamp'} / 1000 );
      my $perftimestamp = int( $sec_timestamp / 300 ) * 300;
      my $cur_counter   = "";
      my $value         = "";
      if ( exists $timestamp->{'value'} && Xormon::isdigit( $timestamp->{'value'} ) ) {
        $cur_counter = $timestamp->{'value'};
        if ( $prev_timestamp ne "" && $prev_counter ne "" ) {
          my $timestamp_diff = $sec_timestamp - $prev_timestamp;
          if ( $timestamp_diff != 0 && ($cur_counter >= $prev_counter)) {
            $value = Xormon::formatNumberByType( 'float4', ( $cur_counter - $prev_counter ) / $timestamp_diff );
          }
          if ( ( $subsystem eq "manager" || $subsystem eq "slicestor" || $subsystem eq "accesser" ) && ( $metric eq "data_read" || $metric eq "data_write" ) ) {
            if ( defined $data{'drive'}{$id}{$sub_metric}{'driveUsage'} && $data{'drive'}{$id}{$sub_metric}{'driveUsage'} ne "" ) {
              my $disk_usage = $data{'drive'}{$id}{$sub_metric}{'driveUsage'};
              if ( $disk_usage eq "os" ) {
                if ( $metric eq "data_read" ) {
                  $perf{$perftimestamp}{$subsystem}{$id}{'os_read'}{$sub_metric} = $value;
                }
                elsif ( $metric eq "data_write" ) {
                  $perf{$perftimestamp}{$subsystem}{$id}{'os_write'}{$sub_metric} = $value;
                }
              }
              elsif ( $disk_usage eq "data" ) {
                $perf{$perftimestamp}{'DRIVE'}{$id}{$sub_metric}{$metric} = $value;
              }
            }
          }
          elsif ( $subsystem eq "slicestor" && ( $metric eq "net_in" || $metric eq "net_out" ) ) {
            $perf{$perftimestamp}{$subsystem}{$id}{$metric}{$sub_metric} = $value;
            if ( defined $data{$subsystem}{$id}{'pool_id'} && $data{$subsystem}{$id}{'pool_id'} ne "" ) {
              my $pool_id = $data{$subsystem}{$id}{'pool_id'};
              $perf{$perftimestamp}{'pool'}{$pool_id}{$metric}{$id}{$sub_metric} = $value;
            }
          }
          else {
            if ( $sub_metric ne "" ) {
              $perf{$perftimestamp}{$subsystem}{$id}{$metric}{$sub_metric} = $value;
            }
            else {
              $perf{$perftimestamp}{$subsystem}{$id}{$metric} = $value;
            }
          }
        }
      }
      $prev_timestamp = $sec_timestamp;
      $prev_counter   = $cur_counter;
    }
  }
}

sub sum_hash_values {
  my $hash_ref = shift;
  my $tot      = "";

  foreach my $id_value ( values %{$hash_ref} ) {
    if ( Xormon::isdigit($id_value) ) {
      if ( $tot eq "" ) {
        $tot = $id_value;
      }
      else {
        $tot += $id_value;
      }
    }
  }
  if ( $tot !=0) {
    Xormon::formatNumberByType( 'float4', $tot );
  }
  return $tot;
}

sub avg_hash_values {
  my $hash_ref = shift;
  my $ret      = "";
  my $tot      = "";
  my $i        = 0;

  foreach my $id_value ( values %{$hash_ref} ) {
    if ( Xormon::isdigit($id_value) ) {
      $i++;
      if ( $tot eq "" ) {
        $tot = $id_value;
      }
      else {
        $tot += $id_value;
      }
    }
  }
  if ( $i > 0 ) {
    $ret = sprintf( '%.1f', $tot / $i );
  }
  return $ret;
}

sub health_status {
  my $act_timestamp       = time();
  my $loc_time            = localtime($act_timestamp);
  my $one_day_old_time_ms = ( $act_timestamp - 86400 ) * 1000;
  my $st_uuid   = $cfg->{'hostcfg_id'};
  my $api_ip    = $cfg->{'apiHost'};
  my $return    = "";
  my @hs        = ();

  my %hs_data;
  my $main_state      = "ok";
  my @error_message   = ();

  if ( defined $data{'slicestor'} && $data{'slicestor'} ne "" ) {
    my $dev_type = "SLICESTOR";
    foreach my $slicestor ( keys %{ $data{'slicestor'} } ) {
      my $conf_ref = $data{'slicestor'}{$slicestor};
      my $status   = "";
      my $dev_name = "";
      if ( defined $conf_ref->{'health'} && $conf_ref->{'health'} ne "" ) {
        $status = $conf_ref->{'health'};
        if ( defined $conf_ref->{'alias'} && $conf_ref->{'alias'} ne "" ) {
          $dev_name = $conf_ref->{'alias'};
        }
        if ( $status ne "ok" ) {
          $main_state = "error";
          my $error_line = "$dev_name accessHealth: $status";
          push( @error_message, "$error_line\n" );
        }
      }
      my %slice;
      $slice{dev_type}   = $dev_type;
      $slice{dev_name}   = $dev_name;
      $slice{status}     = $status;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{devices} }, \%slice );
    }
  }
  if ( defined $data{'manager'} && $data{'manager'} ne "" ) {
    my $dev_type = "MANAGER";
    foreach my $manager ( keys %{ $data{'manager'} } ) {
      my $conf_ref = $data{'manager'}{$manager};
      my $status   = "";
      my $dev_name = "";
      if ( defined $conf_ref->{'health'} && $conf_ref->{'health'} ne "" ) {
        $status = $conf_ref->{'health'};
        if ( defined $conf_ref->{'alias'} && $conf_ref->{'alias'} ne "" ) {
          $dev_name = $conf_ref->{'alias'};
        }
        if ( $status ne "ok" ) {
          $main_state = "error";
          my $error_line = "$dev_name accessHealth: $status";
          push( @error_message, "$error_line\n" );
        }
      }
      my %manager;
      $manager{dev_type}   = $dev_type;
      $manager{dev_name}   = $dev_name;
      $manager{status}     = $status;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{devices} }, \%manager );
    }
  }

  if ( defined $data{'accesser'} && $data{'accesser'} ne "" ) {
    my $dev_type = "ACCESSER";
    foreach my $accesser ( keys %{ $data{'accesser'} } ) {
      my $conf_ref = $data{'accesser'}{$accesser};
      my $status   = "";
      my $dev_name = "";
      if ( defined $conf_ref->{'health'} && $conf_ref->{'health'} ne "" ) {
        $status = $conf_ref->{'health'};
        if ( defined $conf_ref->{'alias'} && $conf_ref->{'alias'} ne "" ) {
          $dev_name = $conf_ref->{'alias'};
        }
        if ( $status ne "ok" ) {
          $main_state = "error";
          my $error_line = "$dev_name accessHealth: $status";
          push( @error_message, "$error_line\n" );
        }
      }
      my %accesser;
      $accesser{dev_type}   = $dev_type;
      $accesser{dev_name}   = $dev_name;
      $accesser{status}     = $status;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{devices} }, \%accesser );
    }
  }
  if ( defined $data{'vault'} && $data{'vault'} ne "" ) {
    foreach my $vault ( keys %{ $data{'vault'} } ) {
      my $vault_ref  = $data{'vault'}{$vault};
      my $st_health  = "";
      my $ac_health  = "";
      my $vault_name = "";
      if ( defined $vault_ref->{'uuid'} && $vault_ref->{'uuid'} ne "" ) {
        if ( defined $vault_ref->{'storageHealth'} && $vault_ref->{'storageHealth'} ne "" ) {
          if ( defined $vault_ref->{'name'} && $vault_ref->{'name'} ne "" ) {
            $vault_name = $vault_ref->{'name'};
          }
          $st_health = $vault_ref->{'storageHealth'};
          if ( $st_health ne "ok" ) {
            $main_state = "error";
            my $error_line = "$vault_name storageHealth: $st_health";
            push( @error_message, "$error_line\n" );
          }
        }
        if ( defined $vault_ref->{'accessHealth'} && $vault_ref->{'accessHealth'} ne "" ) {
          $ac_health = $vault_ref->{'accessHealth'};
          if ( $ac_health eq "unconfigured" ) {
            my $error_line = "$vault_name accessHealth: $ac_health";
            push( @error_message, "$error_line\n" );
          }
          elsif ( $ac_health ne "all" ) {
            $main_state = "error";
            my $error_line = "$vault_name accessHealth: $ac_health";
            push( @error_message, "$error_line\n" );
          }
        }
      }
      my %vault;
      $vault{name}        = $vault_name;
      $vault{st_health}   = $st_health;
      $vault{ac_health}   = $ac_health;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{vaults} }, \%vault );
    }
  }
  if ( defined $data{'drive'} && $data{'drive'} ne "" ) {
    foreach my $node ( keys %{ $data{'drive'} } ) {
      my $node_ref = $data{'drive'}{$node};
      foreach my $drive ( keys %{$node_ref} ) {
        my $drive_ref = $node_ref->{$drive};
        my $name      = "";
        my $dev_name  = "";
        my $status    = "";
        my $usage     = "";
        my $con_name  = "";
        if ( defined $drive_ref->{'name'} && $drive_ref->{'name'} ne "" ) {
          $name = $drive_ref->{'name'};
        }
        if ( defined $drive_ref->{'device_name'} && $drive_ref->{'device_name'} ne "" ) {
          $dev_name = $drive_ref->{'device_name'};
        }
        if ( $name && $dev_name ) {
          $con_name = "$dev_name" . "_" . "$name";
        }
        else {
          $con_name = $name;
        }
        if ( defined $drive_ref->{'driveUsage'} && $drive_ref->{'driveUsage'} ne "" ) {
          $usage = $drive_ref->{'driveUsage'};
        }
        if ( defined $drive_ref->{'status'} && $drive_ref->{'status'} ne "" ) {
          $status = $drive_ref->{'status'};
          if ( $status ne "ONLINE" ) {
            $main_state = "error";
            my $error_line = "$con_name status: $status";
            push( @error_message, "$error_line\n" );
          }
        }
        my %drive;
        $drive{name}       = $name;
        $drive{dev_name}   = $dev_name;
        $drive{status}     = $status;
        $drive{usage}      = $usage;
        push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{drives} }, \%drive );
      }
    }
  }


  ## EVENTS

  #test all audits
  #$return = new_get_request("https://$api_ip/manager/ajax/stream.adm?pageOrigin=home&eventLevels=warning%2Cerror%2Ccritical&streamTypes=allLogMessages%2CopenIncidentStates%2CallAudits&includeInternalActions=false&startTimestamp=1655887234000&streamSize=50");

  $return = new_get_request("https://$api_ip/manager/ajax/stream.adm?pageOrigin=home&eventLevels=warning%2Cerror%2Ccritical&streamTypes=allLogMessages%2CopenIncidentStates%2CnoAudits&includeInternalActions=false&startTimestamp=$one_day_old_time_ms&streamSize=50");
  #print Dumper $return;
  if ( defined $return && ref($return) eq "ARRAY" ) {

    foreach my $inc ( @{$return} ) {
      my $ev_id       = $inc->{'id'};
      if ( !defined $ev_id || $ev_id eq "" ) { next; }
      my $event_type  = $inc->{'streamableType'};
      my $ev_severity = $inc->{'eventLevel'};
      my $ev_type     = $inc->{'eventSource'}{'type'};
      my $ev_object   = $inc->{'eventSource'}{'id'};
      my $ev_msg      = $inc->{'message'};
      my $ev_time     = $inc->{'timestamp'};
      if ( !defined $event_type )   { $event_type   = ""; }
      if ( !defined $ev_severity )  { $ev_severity  = ""; }
      if ( !defined $ev_type )      { $ev_type      = ""; }
      if ( !defined $ev_object )    { $ev_object    = ""; }
      if ( !defined $ev_msg )       { $ev_msg       = ""; }
      if ( !defined $ev_time )      { $ev_time      = ""; }

      if ( $event_type eq "incidentState" ) {
        $event_type = "incident";
        $main_state = "error";
      }
      elsif ( $event_type eq "logMessage" ) {
        $event_type = "message";
      }
      else {
        $event_type = $event_type;
      }

      my %event;
      $event{id}       = $ev_id;
      $event{type}     = $event_type;
      $event{severity} = $ev_severity;
      $event{source}   = $ev_type;
      $event{object}   = $ev_object;
      $event{message}  = $ev_msg;
      $event{time}     = $ev_time;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{events} }, \%event );
    }
  }

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }
  # save events
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);

  my %main_hs = (
      "item_id" => $st_uuid,
      "status" => $main_state,
      "data" => join("\n", @error_message),
  );
  push(@hs, \%main_hs);

  $xormon->saveStatus(\@hs);


}



1;
