package Netapp;

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Xormon;
use JSON qw(encode_json decode_json);
use MIME::Base64;
use REST::Client;

my $data;


#######################
# use to initialize connection with arguments in hash
# my $netapp = NETAPP->new( { scheme => "https", host => "x.x.x.x", port => 443, username => "xormon", password => "unobscuredPassword" } );
#

sub new {
  my ( $classname, $args ) = @_;
  my $self = {};
  bless( $self, $classname );

  $args->{scheme} ||= 'https';
  $args->{port}   ||= 443;
  # $args->{password} = Xormon::unobscure_password($args->{password});

  my $ua = LWP::UserAgent->new;
  $ua->ssl_opts(SSL_verify_mode => 0, verify_hostname => 0);

  my $client = REST::Client->new({
    host    => "$args->{scheme}://$args->{host}:$args->{port}/api",
    timeout => 30,
    useragent => $ua
  });

  $client->addHeader( 'Cache-control', 'no-cache' );
  $client->addHeader( 'Authorization', 'Basic ' . encode_base64( $args->{username} . ':' . $args->{password} ) );

  $self->{client} = $client;

  return $self;
}

sub readRestApi {
  my ( $self, $params ) = @_;

  my $client = $self->{client};
=pod
  200 OK: Returned for success when not creating a new object
  201 Created: Returned for success after the creation of an object
  202 Accepted: Returned when a job has been successfully started, but the operation is not complete
  400 Bad Request: Returned if the input could not be parsed
  401 Unauthorized: Returned if user authentication failed
  403 Forbidden: Returned for authorization (RBAC) errors
  404 Not Found: Returned when the specified resource does not exist
  405 Method Not Allowed: Returned when the specified resource does not support the method (for example, POST or DELETE calls)
  409 Conflict: Returned when there is a conflict with a different object that must be created, modified, or deleted before this operation can succeed
  429 Too Many Requests: Returned when the client has sent more requests than the server can handle. The client should try again later (defined by the Retry-After header)
  500 Internal Error: Returned for most other internal error codes
  502 Bad Gateway: Returned if the application is temporarily unreachable. Try again later
  503 Service Unavailable: Returned if the server is temporarily overloaded. Try again later.
=cut

  my $cntrs  = $params->{counters} ? "&counters.name=" . urlencode( $params->{counters} ) : "";
  my $fields = $params->{fields} ? "&fields=" . $params->{fields} : "";
  $self->{client}->GET("$params->{path}?pretty=false${fields}${cntrs}&return_timeout=120&max_records=10000");
  my $status = $self->{client}->responseCode();
  if ( $status >= 400 ) {
    my $res = decode_json( $self->{client}->responseContent() );
    Xormon::error( "WARNING: get_perf: API call '$params->{path}?fields=$params->{fields}' does not provide data! $res->{error}{message} " . __FILE__ . ":" . __LINE__ );
    return { status => 0, content => $res->{error}{message} };
  }
  my $res = decode_json( $self->{client}->responseContent() );

  if ( $res->{num_records} ) {
    if ( $params->{counters} ) {
      foreach my $record ( @{ $res->{records} } ) {
        if ( $record->{counters} ) {
          my %counters = map { $_->{name} => $_->{value} } @{ $record->{counters} };
          $record->{counters} = \%counters;
        }
        if ( $record->{properties} ) {
          my %properties = map { $_->{name} => $_->{value} } @{ $record->{properties} };
          $record->{properties} = \%properties;
        }
      }
    }
    return { status => 1, records => $res->{num_records}, content => $res->{records} };
  }
  else {
    return { status => 1, records => 0, content => $res };
  }
}

sub file_write {
  my $file = shift;
  open my $IO, ">", $file or die "Cannot open $file for output: $!\n";
  print $IO @_;
  close $IO;
}

sub file_write_append {
  my $file = shift;
  my $IO;
  if ( -f $file ) {
    open $IO, ">>", $file or die "Cannot open $file for output: $!\n";
  }
  else {
    open $IO, ">", $file or die "Cannot open $file for output: $!\n";
  }
  print $IO @_;
  close $IO;
}

sub file_read {
  my $file = shift;
  open my $IO, "<", $file or die "Cannot open $file for input: $!\n";
  my @data = <$IO>;
  close $IO;
  wantarray ? @data : join( '' => @data );
}

sub urlencode {
  my $s = shift;
  $s =~ s/ /+/g;
  $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
}

1;
