package NIMBLE;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Date::Parse;
use Xormon;
use JSON qw(encode_json decode_json);
use Time::Local;
use Test;
use XormonLogger;

#use Date::Parse;

my %data;
my $xormon;
my $cfg;
my $logger;

my $SSH;
my $delimeter = "#########XORUX#########";
my $timeout_config = 120;
my $sample_rate = 300;

my $stats_start_time = "";
my $stats_end_time   = "";

my $conf_timestamp = "";

my $service_data_in  = {};
my $service_data_out = {};


sub collect {
  $xormon = shift;
  $cfg    = shift;

  # test main variables

  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")     && return "error"; }
  $logger = XormonLogger->new( 'nimble', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'arrayName'} )     { $logger->error("Cannot continue, 'arrayName' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'sshHost'} )       { $logger->error("Cannot continue, 'sshHost' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'sshPort'} )       { $logger->error("Cannot continue, 'sshPort' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'sshUsername'} )   { $logger->error("Cannot continue, 'sshUsername' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'isKeyFile'} )     { $logger->error("Cannot continue, 'isKeyFile' is not configured!")     && return "error"; }

  # my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
  #my $time_to_output_file = sprintf( "%4d%02d%02d_%02d%02d",         $year + 1900, $month + 1, $day, $hour, $min );
  # my $start_time          = sprintf( "%4d:%02d:%02dT%02d:%02d:%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );
  my $cur_timestamp = time;

  $SSH = "ssh";

  if ($cfg->{'isKeyFile'} == 0 ) {
    $logger->info("Using user/password");

    unless ( exists $cfg->{'sshPassword'} )   { $logger->error("Cannot continue, 'sshPassword' is not configured!")  && return "error"; }

    my $ssh = Xormon::openssh_new(  $cfg->{'sshHost'},  $cfg->{'sshPort'},  $cfg->{'sshUsername'}, $timeout_config, $cfg->{'sshPassword'} );
    if ( !defined $ssh || $ssh eq "" || $ssh =~ m/^Error|^ERROR/ ) {
      $xormon->isConnected(0, "ssh unable to connect to remote machine");
      $logger->error("Error: Unable to connect to remote machine ($cfg->{'hostalias'})") && return "error";
    }
    else {
      $xormon->isConnected(1);
      $cfg->{'openssh'} = $ssh;
    }
  }
  elsif ($cfg->{'isKeyFile'} == 1 && defined $cfg->{'sshKeyFile'} && $cfg->{'sshKeyFile'} ne "") {
    $logger->info("Using SSH key '$cfg->{'sshKeyFile'}'");
    $SSH = "ssh -i $cfg->{'sshKeyFile'}";
  }
  else {
    $logger->info("Using default SSH key");
  }


  ## GET MICROSERVICE DATA
  $service_data_in  = $xormon->getData();
  $service_data_out = {};

  array_time();
  array_info();
  array_stats();

  ctrlr_info();

  vol_info();
  vol_stats();

  disk_info();
  disk_stats();

  net_stats();
  fc_stats();

  #print Dumper %data;

  # #
  # Save per each timestamp
  # #
  if ( exists $data{'perf'} ) {
    my $timestamp_last = "";

    foreach my $timestamp_act ( sort { $a <=> $b } keys( %{ $data{'perf'} } ) ) {
      if ( !Xormon::isdigit($timestamp_act) ) { next; }
      $conf_timestamp = $timestamp_act;

      # leave out the first timestamp, it is there only for first counter calculation
      if ( defined $timestamp_last && $timestamp_last ne '' ) {

        # check if there is a right sample rate between last 2 statistic sets
        if ( $timestamp_act - $timestamp_last != $sample_rate ) {
          $logger->error( "Different sample rate between last 2 statistic sets! Stats skipped! \"$timestamp_act - $timestamp_last != $sample_rate\" $!" . __FILE__ . ":" . __LINE__ ) && next;
        }

        save_timestamp($timestamp_act,$timestamp_last);

      }
      $timestamp_last = $timestamp_act;
    } ## end foreach my $timestamp_act

    if (defined $timestamp_last && $timestamp_last ne "") {
      $xormon->hasData(1);
    }
    else {
      $xormon->hasData(0, "no last perf timetamp");
    }
  }

  #print Dumper %data;

  # #
  # # print CONF file
  # #
  if ( exists $service_data_in->{last_conf_timestamp} && Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) {
    my $last_conf_time = $service_data_in->{last_conf_timestamp};
    my $dif_time = $cur_timestamp - $last_conf_time;
    if ( $dif_time < 3600 ) {
      $service_data_out->{last_conf_timestamp} = $last_conf_time;
    }
    else {
      $service_data_out->{last_conf_timestamp} = $cur_timestamp;
      fc_info();
      print_conf();
    }
  }
  else {
    $service_data_out->{last_conf_timestamp} = $cur_timestamp;
    fc_info();
    print_conf();
  }

  # #
  # # health status
  # #
  # can run every five minutes
  health_status();

  # # #
  # # # events
  # # #
  if ( exists $service_data_in->{last_events_timestamp} && Xormon::isdigit($service_data_in->{last_events_timestamp}) ) {
    my $last_events_time = $service_data_in->{last_events_timestamp};
    my $dif_time = $cur_timestamp - $last_events_time;
    if ( $dif_time < 3600 ) {
      $service_data_out->{last_events_timestamp} = $last_events_time;
    }
    else {
      $service_data_out->{last_events_timestamp} = $cur_timestamp;
      events();
    }
  }
  else {
    $service_data_out->{last_events_timestamp} = $cur_timestamp;
    events();
  }

  if ( exists $cfg->{'debug'} && $cfg->{'debug'} eq "data" ) {
    print Dumper \%data
  }

  ## SAVE TO DB
  #print Dumper $service_data_out;
  $xormon->saveData(\%{ $service_data_out });

  ## close openssh session
  if ( ($cfg->{'isKeyFile'} == 0) && $cfg->{'openssh'} ) {
    if ( Xormon::openssh_disconnect( $cfg->{'openssh'}, $timeout_config ) == 0 ) {
      $logger->error("Unable to disconnect from remote machine");
      return "error";
    }
  }

  return 1;
}

sub save_timestamp {
  my $timestamp_act  = shift;
  my $timestamp_last = shift;

  my @architecture;
  my $status;
  my $data_out = "";
  my %volumes;

  # VOLUMES
  if ( defined $data{'perf'}{$timestamp_act}{'VOLUMES'} && $data{'perf'}{$timestamp_act}{'VOLUMES'} ne '' ) {

    #$data_out .= "uuid,time,read_io,read (KB/s),write_io,write (KB/s),data_rate (KB/s),io_rate,seqRead%,nsMemHit%,nsSsdHit%,resp_t_r,resp_t_w,Size (MB)\n";
    $data_out .= "uuid;time;read_io;read;write_io;write;data_rate;io_rate;r_cache_hit;r_ssd_hit;resp_t_r;resp_t_w;resp_t;block_size_r;block_size_w;block_size;capacity_total\n";

    # for architecture parent
    my $pool_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "array_total"); $pool_key = lc($pool_key);

    foreach my $volume ( sort keys( %{ $data{'perf'}{$timestamp_act}{'VOLUMES'} } ) ) {

      #print "$volume\n";

      # my $date      = "";
      my $read_io   = "";
      my $read      = "";
      my $write_io  = "";
      my $write     = "";
      my $data_rate = "";
      my $io_rate   = "";
      # my $seq_read  = "";
      my $mem_hit      = "";
      my $ssd_hit      = "";
      my $resp_t_r     = "";
      my $resp_t_w     = "";
      my $resp_t       = "";
      my $block_size_r = "";
      my $block_size_w = "";
      my $block_size   = "";

      my $size      = "";

      my $folder_name = "";
      my $pool        = "";
      my $serial      = "";

      if ( defined $data{'VOLUMES'}{$volume}{'Serial number'} && $data{'VOLUMES'}{$volume}{'Serial number'} ne '' && $data{'VOLUMES'}{$volume}{'Serial number'} ne "N/A" ) {
        $serial = $data{'VOLUMES'}{$volume}{'Serial number'};
      }
      if ( !defined $serial || $serial eq '' ) {
        $logger->error( "Save volume: Volume ID/Serial not found! Volume \"$volume\" skipped... $!" . __FILE__ . ":" . __LINE__ ) && next;
      }

      my $stats_act  = $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume};
      my $stats_last = $data{'perf'}{$timestamp_last}{'VOLUMES'}{$volume};

      if ( defined $stats_act->{'reads'} && Xormon::isdigit( $stats_act->{'reads'} ) && defined $stats_last->{'reads'} && Xormon::isdigit( $stats_last->{'reads'} ) ) {
        $read_io = ( $stats_act->{'reads'} - $stats_last->{'reads'} ) / $sample_rate;
        $read_io = sprintf( '%.0f', $read_io );
        if ( $read_io < 0 ) { $read_io = ""; }
        if ( defined $read_io && Xormon::isdigit($read_io) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'} + $read_io;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'} = $read_io;
          }
        }
      }
      if ( defined $stats_act->{'rdKB'} && Xormon::isdigit( $stats_act->{'rdKB'} ) && defined $stats_last->{'rdKB'} && Xormon::isdigit( $stats_last->{'rdKB'} ) ) {
        $read = ( $stats_act->{'rdKB'} - $stats_last->{'rdKB'} ) / $sample_rate;
        $read = Xormon::returnBytes( $read, "kB"); # to bytes
        if ( $read < 0 ) { $read = ""; }
        if ( defined $read && Xormon::isdigit($read) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} + $read;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} = $read;
          }
        }
      }
      if ( defined $stats_act->{'rdKiB'} && Xormon::isdigit( $stats_act->{'rdKiB'} ) && defined $stats_last->{'rdKiB'} && Xormon::isdigit( $stats_last->{'rdKiB'} ) ) {
        $read = ( $stats_act->{'rdKiB'} - $stats_last->{'rdKiB'} ) / $sample_rate;
        $read = Xormon::returnBytes( $read, "kiB"); # to bytes
        if ( $read < 0 ) { $read = ""; }
        if ( defined $read && Xormon::isdigit($read) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} + $read;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} = $read;
          }
        }
      }
      if ( defined $stats_act->{'writes'} && Xormon::isdigit( $stats_act->{'writes'} ) && defined $stats_last->{'writes'} && Xormon::isdigit( $stats_last->{'writes'} ) ) {
        $write_io = ( $stats_act->{'writes'} - $stats_last->{'writes'} ) / $sample_rate;
        $write_io = sprintf( '%.0f', $write_io );
        if ( $write_io < 0 ) { $write_io = ""; }
        if ( defined $write_io && Xormon::isdigit($write_io) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'} + $write_io;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'} = $write_io;
          }
        }
      }
      if ( defined $stats_act->{'wrKB'} && Xormon::isdigit( $stats_act->{'wrKB'} ) && defined $stats_last->{'wrKB'} && Xormon::isdigit( $stats_last->{'wrKB'} ) ) {
        $write = ( $stats_act->{'wrKB'} - $stats_last->{'wrKB'} ) / $sample_rate;
        $write = Xormon::returnBytes( $write, "kB"); # to bytes
        if ( $write < 0 ) { $write = ""; }
        if ( defined $write && Xormon::isdigit($write) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} + $write;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} = $write;
          }
        }
      }
      if ( defined $stats_act->{'wrKiB'} && Xormon::isdigit( $stats_act->{'wrKiB'} ) && defined $stats_last->{'wrKiB'} && Xormon::isdigit( $stats_last->{'wrKiB'} ) ) {
        $write = ( $stats_act->{'wrKiB'} - $stats_last->{'wrKiB'} ) / $sample_rate;
        $write = Xormon::returnBytes( $write, "kiB"); # to bytes
        if ( $write < 0 ) { $write = ""; }
        if ( defined $write && Xormon::isdigit($write) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} + $write;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} = $write;
          }
        }
      }
      if ( defined $read_io && Xormon::isdigit($read_io) && defined $write_io && Xormon::isdigit($write_io) ) {
        $io_rate = $read_io + $write_io;
        if ( defined $io_rate && Xormon::isdigit($io_rate) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'} + $io_rate;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'} = $io_rate;
          }
        }
      }
      if ( defined $read && Xormon::isdigit($read) && defined $write && Xormon::isdigit($write) ) {
        $data_rate = $read + $write;
        if ( defined $data_rate && Xormon::isdigit($data_rate) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'} ) ) {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'} = $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'} + $data_rate;
          }
          else {
            $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'} = $data_rate;
          }
        }
      }

      if ( defined $data{'VOLUMES'}{$volume}{'Folder'} && $data{'VOLUMES'}{$volume}{'Folder'} ne '' && $data{'VOLUMES'}{$volume}{'Folder'} ne "N/A" ) {
        $folder_name = $data{'VOLUMES'}{$volume}{'Folder'};
        $folder_name = remove_unauthorized_characters($folder_name);
      }
      # if ( defined $stats_act->{'date'} && $stats_act->{'date'} ne '' ) {
      #   $date = $stats_act->{'date'};
      # }
      # if ( defined $stats_act->{'seqRead%'} && $stats_act->{'seqRead%'} ne '' ) {
      #   $seq_read = $stats_act->{'seqRead%'};
      # }
      if ( defined $stats_act->{'nsMemHit%'} && $stats_act->{'nsMemHit%'} ne '' ) {
        $mem_hit = $stats_act->{'nsMemHit%'};
      }
      if ( defined $stats_act->{'nsSsdHit%'} && $stats_act->{'nsSsdHit%'} ne '' ) {
        $ssd_hit = $stats_act->{'nsSsdHit%'};
      }
      if ( defined $data{'VOLUMES'}{$volume}{'Size (MB)'} && $data{'VOLUMES'}{$volume}{'Size (MB)'} ne '' && $data{'VOLUMES'}{$volume}{'Size (MB)'} ne "N/A" ) {
        if ( Xormon::isdigit( $data{'VOLUMES'}{$volume}{'Size (MB)'} ) ) {
          $size = Xormon::returnBytes( $data{'VOLUMES'}{$volume}{'Size (MB)'}, "MB" );
        }
      }
      if ( defined $data{'VOLUMES'}{$volume}{'Size (MiB)'} && $data{'VOLUMES'}{$volume}{'Size (MiB)'} ne '' && $data{'VOLUMES'}{$volume}{'Size (MiB)'} ne "N/A" ) {
        if ( Xormon::isdigit( $data{'VOLUMES'}{$volume}{'Size (MiB)'} ) ) {
          $size = Xormon::returnBytes( $data{'VOLUMES'}{$volume}{'Size (MiB)'}, "MiB" );
        }
      }

      # calculate resp_t_r/resp_t_w

      # resp_t_w
      if ( defined $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'writes'} && $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'writes'} ne '' ) {
        my $io_total            = 0;
        my $multiplied_io_total = 0;
        foreach my $stat_name ( sort keys( %{ $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'writes'} } ) ) {
          if ( $stat_name =~ m/date/ || $stat_name =~ m/io_type/ || $stat_name =~ m/yyyy-mm-dd/ ) { next; }

          if ( defined $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'writes'}{$stat_name} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'writes'}{$stat_name} ) && defined $data{'perf'}{$timestamp_last}{'VOLUMES'}{$volume}{'latency'}{'writes'}{$stat_name} && Xormon::isdigit( $data{'perf'}{$timestamp_last}{'VOLUMES'}{$volume}{'latency'}{'writes'}{$stat_name} ) ) {
            my $counter_diff = $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'writes'}{$stat_name} - $data{'perf'}{$timestamp_last}{'VOLUMES'}{$volume}{'latency'}{'writes'}{$stat_name};

            if ( Xormon::isdigit($counter_diff) && $counter_diff > 0 ) {
              $io_total += $counter_diff;

              my $multiplier    = set_multiplier($stat_name);
              my $multiplied_io = $counter_diff * $multiplier;
              $multiplied_io_total += $multiplied_io;
            }
          }
        }
        if ( Xormon::isdigit($io_total) && Xormon::isdigit($multiplied_io_total) ) {
          if ( $io_total > 0 && $multiplied_io_total > 0 ) {
            $resp_t_w = $multiplied_io_total / $io_total;
            $resp_t_w = sprintf( '%.2f', $resp_t_w );
          }
          else {
            $resp_t_w = 0;
          }
        }
      }
      # resp_t_r
      if ( defined $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'reads'} && $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'reads'} ne '' ) {
        my $io_total            = 0;
        my $multiplied_io_total = 0;
        foreach my $stat_name ( sort keys( %{ $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'reads'} } ) ) {
          if ( $stat_name =~ m/date/ || $stat_name =~ m/io_type/ || $stat_name =~ m/yyyy-mm-dd/ ) { next; }

          if ( defined $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'reads'}{$stat_name} && Xormon::isdigit( $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'reads'}{$stat_name} ) && defined $data{'perf'}{$timestamp_last}{'VOLUMES'}{$volume}{'latency'}{'reads'}{$stat_name} && Xormon::isdigit( $data{'perf'}{$timestamp_last}{'VOLUMES'}{$volume}{'latency'}{'reads'}{$stat_name} ) ) {
            my $counter_diff = $data{'perf'}{$timestamp_act}{'VOLUMES'}{$volume}{'latency'}{'reads'}{$stat_name} - $data{'perf'}{$timestamp_last}{'VOLUMES'}{$volume}{'latency'}{'reads'}{$stat_name};

            if ( Xormon::isdigit($counter_diff) && $counter_diff > 0 ) {
              $io_total += $counter_diff;

              my $multiplier    = set_multiplier($stat_name);
              my $multiplied_io = $counter_diff * $multiplier;
              $multiplied_io_total += $multiplied_io;
            }
          }
        }
        if ( Xormon::isdigit($io_total) && Xormon::isdigit($multiplied_io_total) ) {
          if ( $io_total > 0 && $multiplied_io_total > 0 ) {
            $resp_t_r = $multiplied_io_total / $io_total;
            $resp_t_r = sprintf( '%.2f', $resp_t_r );
          }
          else {
            $resp_t_r = 0;
          }
        }
      }
      # calculate resp_t from r and w
      if ( Xormon::isdigit( $resp_t_r ) && Xormon::isdigit( $read_io ) && Xormon::isdigit( $resp_t_w ) && Xormon::isdigit( $write_io ) ) {
        my $resp_t_help = Xormon::calculate_weighted_avg( [ { count => $read_io, value => $resp_t_r }, { count => $write_io, value => $resp_t_w } ] );
        if (defined $resp_t_help) { $resp_t = Xormon::formatNumberByType("float4",$resp_t_help); }
      }

      # add metrics to hash for HOSTS
      if (defined $read_io && Xormon::isdigit($read_io)) {
        $volumes{$timestamp_act}{$volume}{'read_io'} = $read_io;
      }
      if (defined $write_io && Xormon::isdigit($write_io)) {
        $volumes{$timestamp_act}{$volume}{'write_io'} = $write_io;
      }
      if (defined $io_rate && Xormon::isdigit($io_rate)) {
        $volumes{$timestamp_act}{$volume}{'io_rate'} = $io_rate;
      }
      if (defined $read && Xormon::isdigit($read)) {
        $volumes{$timestamp_act}{$volume}{'read'} = $read;
      }
      if (defined $write && Xormon::isdigit($write)) {
        $volumes{$timestamp_act}{$volume}{'write'} = $write;
      }
      if (defined $data_rate && Xormon::isdigit($data_rate)) {
        $volumes{$timestamp_act}{$volume}{'data_rate'} = $data_rate;
      }
      if (defined $resp_t_r && Xormon::isdigit($resp_t_r)) {
        $volumes{$timestamp_act}{$volume}{'resp_t_r'} = $resp_t_r;
      }
      if (defined $resp_t_w && Xormon::isdigit($resp_t_w)) {
        $volumes{$timestamp_act}{$volume}{'resp_t_w'} = $resp_t_w;
      }
      if (defined $resp_t && Xormon::isdigit($resp_t)) {
        $volumes{$timestamp_act}{$volume}{'resp_t'} = $resp_t;
      }

      # BLOCK SIZE
      if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
        if ( $io_rate == 0 ) {
          $block_size = 0;
        }
        else{
          $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
        }
      }
      if ( Xormon::isdigit($read_io) && Xormon::isdigit($read) ) {
        if ( $read_io == 0 ) {
          $block_size_r = 0;
        }
        else{
          $block_size_r = Xormon::formatNumberByType("float4", ($read / $read_io));
        }
      }
      if ( Xormon::isdigit($write_io) && Xormon::isdigit($write) ) {
        if ( $write_io == 0 ) {
          $block_size_w = 0;
        }
        else{
          $block_size_w = Xormon::formatNumberByType("float4", ($write / $write_io));
        }
      }

      ### VOLUME-FOLDER
      if (defined $folder_name && $folder_name ne ""){
        # IO
        if ( defined $read_io && Xormon::isdigit($read_io) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read_io'} && Xormon::isdigit($data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read_io'}) ) {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read_io'} = $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read_io'} + $read_io;
          }
          else {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read_io'} = $read_io;
          }
        }
        if ( defined $write_io && Xormon::isdigit($write_io) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write_io'} && Xormon::isdigit($data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write_io'}) ) {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write_io'} = $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write_io'} + $write_io;
          }
          else {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write_io'} = $write_io;
          }
        }
        if ( defined $io_rate && Xormon::isdigit($io_rate) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'io_rate'} && Xormon::isdigit($data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'io_rate'}) ) {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'io_rate'} = $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'io_rate'} + $io_rate;
          }
          else {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'io_rate'} = $io_rate;
          }
        }
        # DATA
        if ( defined $read && Xormon::isdigit($read) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read'} && Xormon::isdigit($data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read'}) ) {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read'} = $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read'} + $read;
          }
          else {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'read'} = $read;
          }
        }
        if ( defined $write && Xormon::isdigit($write) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write'} && Xormon::isdigit($data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write'}) ) {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write'} = $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write'} + $write;
          }
          else {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'write'} = $write;
          }
        }
        if ( defined $data_rate && Xormon::isdigit($data_rate) ) {
          if ( defined $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'data_rate'} && Xormon::isdigit($data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'data_rate'}) ) {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'data_rate'} = $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'data_rate'} + $data_rate;
          }
          else {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'data_rate'} = $data_rate;
          }
        }
        # FOLDER resp_t_r
        if (defined $resp_t_r && $resp_t_r ne "" && $read_io && $read_io ne "" ){
          if ( Xormon::isdigit( $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t_r'} ) ){
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t_r'} += $read_io * $resp_t_r;
          }
          else
          {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t_r'} = $read_io * $resp_t_r;
          }
        }
        # FOLDER resp_t_w
        if (defined $resp_t_w && $resp_t_w ne "" && $write_io && $write_io ne "" ){
          if ( Xormon::isdigit( $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t_w'} ) ){
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t_w'} += $write_io * $resp_t_w;
          }
          else
          {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t_w'} = $write_io * $resp_t_w;
          }
        }
        # FOLDER resp_t
        if (defined $resp_t && $resp_t ne "" && $io_rate && $io_rate ne "" ){
          if ( Xormon::isdigit( $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t'} ) ){
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t'} += $io_rate * $resp_t;
          }
          else
          {
            $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$folder_name}{'resp_t'} = $io_rate * $resp_t;
          }
        }
      }

      ### DEVICE RESP_T
      # DEVICE resp_t_r
      if (defined $resp_t_r && $resp_t_r ne "" && $read_io && $read_io ne "" ){
        if ( Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_r'} ) ){
          $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_r'} += $read_io * $resp_t_r;
        }
        else
        {
          $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_r'} = $read_io * $resp_t_r;
        }
      }
      # DEVICE resp_t_w
      if (defined $resp_t_w && $resp_t_w ne "" && $write_io && $write_io ne "" ){
        if ( Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_w'} ) ){
          $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_w'} += $write_io * $resp_t_w;
        }
        else
        {
          $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_w'} = $write_io * $resp_t_w;
        }
      }
      # DEVICE resp_t
      if (defined $resp_t && $resp_t ne "" && $io_rate && $io_rate ne "" ){
        if ( Xormon::isdigit( $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t'} ) ){
          $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t'} += $io_rate * $resp_t;
        }
        else
        {
          $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t'} = $io_rate * $resp_t;
        }
      }

      if ( defined $data{'VOLUMES'}{$volume}{'Pool'} && $data{'VOLUMES'}{$volume}{'Pool'} ne '' && $data{'VOLUMES'}{$volume}{'Pool'} ne "N/A" ) {
        $pool = $data{'VOLUMES'}{$volume}{'Pool'};
        $pool = remove_unauthorized_characters($pool);
      }

      $volume = remove_unauthorized_characters($volume);

      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "VOLUME" . $volume); $hash_key = lc($hash_key);
      #print "$hash_key\n";

      my %arch = (
          "item_id" => $hash_key,
          "label" => $volume,
          "hw_type" => "nimble",
          "subsystem" => "volume",
          "parents" => [$cfg->{'hostcfg_id'},$pool_key],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
      push(@architecture, \%arch);

      $data_out .= "$hash_key;$timestamp_act;$read_io;$read;$write_io;$write;$data_rate;$io_rate;$mem_hit;$ssd_hit;$resp_t_r;$resp_t_w;$resp_t;$block_size_r;$block_size_w;$block_size;$size\n";
    }
    #print $data_out;
    #print Dumper $data{'perf'}{$timestamp_act}{'ARRAY'};

    #SAVE VOLUME
    my $rc = $xormon->saveCSV($data_out,"nimble","volume");

    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("VOLUME status : $status");
    $data_out = "";
  } # end if volumes


  # ARRAY
  if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'} && $data{'perf'}{$timestamp_act}{'ARRAY'} ne '' ) {
    my $device_data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;capacity_used;capacity_total;capacity_free;capacity_used_percent;ssd_hit;r_cache_hit\n";
    $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t;block_size_r;block_size_w;block_size;capacity_used;capacity_total;capacity_free;capacity_used_percent;ssd_hit;r_cache_hit\n";

    my $Array_name                    = "";
    # my $date                          = "";
    # my $Member_of_pool                = "";
    # my $Volume_space_saved_mb         = "";
    # my $Group_ID                      = "";
    my $Total_array_usage_mb          = "";
    # my $Member_GID                    = "";
    my $Total_array_capacity_mb       = "";
    my $Available_space_mb            = "";
    my $used_pct  = "";
    # my $Total_array_cache_capacity_mb = "";
    # my $Volume_usage_mb               = "";
    my $read_io = "";
    my $write_io = "";
    my $io_rate = "";
    my $read = "";
    my $write = "";
    my $data_rate = "";
    my $resp_t_r = "";
    my $resp_t_w = "";
    my $resp_t = "";
    my $block_size = "";
    my $block_size_r = "";
    my $block_size_w = "";

    if ( defined $data{'ARRAY'}{'Array name'} && $data{'ARRAY'}{'Array name'} ne '' ) {
      $Array_name = $data{'ARRAY'}{'Array name'};
    }
    # if ( defined $data{'ARRAY'}{'Member of pool'} && $data{'ARRAY'}{'Member of pool'} ne '' ) {
    #   $Member_of_pool = $data{'ARRAY'}{'Member of pool'};
    # }
    # if ( defined $data{'ARRAY'}{'Volume space saved (MB)'} && $data{'ARRAY'}{'Volume space saved (MB)'} ne '' ) {
    #   $Volume_space_saved_mb = $data{'ARRAY'}{'Volume space saved (MB)'};
    #   if ( Xormon::isdigit($Volume_space_saved_mb) ) {
    #     $Volume_space_saved_mb = Xormon::returnBytes( $Volume_space_saved_mb, "MB" ); # from MB to bytes
    #   }
    #   else {
    #     $Volume_space_saved_mb = "";
    #   }
    # }
    # if ( defined $data{'ARRAY'}{'Volume space saved (MiB)'} && $data{'ARRAY'}{'Volume space saved (MiB)'} ne '' ) {
    #   $Volume_space_saved_mb = $data{'ARRAY'}{'Volume space saved (MiB)'};
    #   if ( Xormon::isdigit($Volume_space_saved_mb) ) {
    #     $Volume_space_saved_mb = Xormon::returnBytes( $Volume_space_saved_mb, "MiB" ); # from MiB to bytes
    #   }
    #   else {
    #     $Volume_space_saved_mb = "";
    #   }
    # }
    # if ( defined $data{'ARRAY'}{'Group ID'} && $data{'ARRAY'}{'Group ID'} ne '' ) {
    #   $Group_ID = $data{'ARRAY'}{'Group ID'};
    # }
    if ( defined $data{'ARRAY'}{'Total array usage (MB)'} && $data{'ARRAY'}{'Total array usage (MB)'} ne '' ) {
      $Total_array_usage_mb = $data{'ARRAY'}{'Total array usage (MB)'};
      if ( Xormon::isdigit($Total_array_usage_mb) ) {
        $Total_array_usage_mb = Xormon::returnBytes( $Total_array_usage_mb, "MB" ); # from MB to bytes
      }
      else {
        $Total_array_usage_mb = "";
      }
    }
    if ( defined $data{'ARRAY'}{'Total array usage (MiB)'} && $data{'ARRAY'}{'Total array usage (MiB)'} ne '' ) {
      $Total_array_usage_mb = $data{'ARRAY'}{'Total array usage (MiB)'};
      if ( Xormon::isdigit($Total_array_usage_mb) ) {
        $Total_array_usage_mb = Xormon::returnBytes( $Total_array_usage_mb, "MiB" ); # from MiB to bytes
      }
      else {
        $Total_array_usage_mb = "";
      }
    }
    # if ( defined $data{'ARRAY'}{'Member GID'} && $data{'ARRAY'}{'Member GID'} ne '' ) {
    #   $Member_GID = $data{'ARRAY'}{'Member GID'};
    # }
    if ( defined $data{'ARRAY'}{'Total array capacity (MB)'} && $data{'ARRAY'}{'Total array capacity (MB)'} ne '' ) {
      $Total_array_capacity_mb = $data{'ARRAY'}{'Total array capacity (MB)'};
      if ( Xormon::isdigit($Total_array_capacity_mb) ) {
        $Total_array_capacity_mb = Xormon::returnBytes( $Total_array_capacity_mb, "MB" ); # from MB to bytes
      }
      else {
        $Total_array_capacity_mb = "";
      }
    }
    if ( defined $data{'ARRAY'}{'Total array capacity (MiB)'} && $data{'ARRAY'}{'Total array capacity (MiB)'} ne '' ) {
      $Total_array_capacity_mb = $data{'ARRAY'}{'Total array capacity (MiB)'};
      if ( Xormon::isdigit($Total_array_capacity_mb) ) {
        $Total_array_capacity_mb = Xormon::returnBytes( $Total_array_capacity_mb, "MiB" ); # from MiB to bytes
      }
      else {
        $Total_array_capacity_mb = "";
      }
    }
    if ( defined $data{'ARRAY'}{'Available space (MB)'} && $data{'ARRAY'}{'Available space (MB)'} ne '' ) {
      $Available_space_mb = $data{'ARRAY'}{'Available space (MB)'};
      if ( Xormon::isdigit($Available_space_mb) ) {
        $Available_space_mb = Xormon::returnBytes( $Available_space_mb, "MB" ); # from MB to bytes
      }
      else {
        $Available_space_mb = "";
      }
    }
    if ( defined $data{'ARRAY'}{'Available space (MiB)'} && $data{'ARRAY'}{'Available space (MiB)'} ne '' ) {
      $Available_space_mb = $data{'ARRAY'}{'Available space (MiB)'};
      if ( Xormon::isdigit($Available_space_mb) ) {
        $Available_space_mb = Xormon::returnBytes( $Available_space_mb, "MiB" ); # from MiB to bytes
      }
      else {
        $Available_space_mb = "";
      }
    }

    if ( Xormon::isdigit($Total_array_capacity_mb) && Xormon::isdigit($Total_array_usage_mb) && ($Total_array_capacity_mb != 0) ){
      $used_pct = Xormon::formatNumberByType("float4", $Total_array_usage_mb / ($Total_array_capacity_mb / 100) );
    }

    # if ( defined $data{'ARRAY'}{'Total array cache capacity (MB)'} && $data{'ARRAY'}{'Total array cache capacity (MB)'} ne '' ) {
    #   $Total_array_cache_capacity_mb = $data{'ARRAY'}{'Total array cache capacity (MB)'};
    #   if ( Xormon::isdigit($Total_array_cache_capacity_mb) ) {
    #     $Total_array_cache_capacity_mb = Xormon::returnBytes( $Total_array_cache_capacity_mb, "MB" ); # from MB to bytes
    #   }
    #   else {
    #     $Total_array_cache_capacity_mb = "";
    #   }
    # }
    # if ( defined $data{'ARRAY'}{'Total array cache capacity (MiB)'} && $data{'ARRAY'}{'Total array cache capacity (MiB)'} ne '' ) {
    #   $Total_array_cache_capacity_mb = $data{'ARRAY'}{'Total array cache capacity (MiB)'};
    #   if ( Xormon::isdigit($Total_array_cache_capacity_mb) ) {
    #     $Total_array_cache_capacity_mb = Xormon::returnBytes( $Total_array_cache_capacity_mb, "MiB" ); # from MiB to bytes
    #   }
    #   else {
    #     $Total_array_cache_capacity_mb = "";
    #   }
    # }
    # if ( defined $data{'ARRAY'}{'Volume usage (MB)'} && $data{'ARRAY'}{'Volume usage (MB)'} ne '' ) {
    #   $Volume_usage_mb = $data{'ARRAY'}{'Volume usage (MB)'};
    #   if ( Xormon::isdigit($Volume_usage_mb) ) {
    #     $Volume_usage_mb = Xormon::returnBytes( $Volume_usage_mb, "MB" ); # from MB to bytes
    #   }
    #   else {
    #     $Volume_usage_mb = "";
    #   }
    # }
    # if ( defined $data{'ARRAY'}{'Volume usage (MiB)'} && $data{'ARRAY'}{'Volume usage (MiB)'} ne '' ) {
    #   $Volume_usage_mb = $data{'ARRAY'}{'Volume usage (MiB)'};
    #   if ( Xormon::isdigit($Volume_usage_mb) ) {
    #     $Volume_usage_mb = Xormon::returnBytes( $Volume_usage_mb, "MiB" ); # from MiB to bytes
    #   }
    #   else {
    #     $Volume_usage_mb = "";
    #   }
    # }

    my $stats_act  = $data{'perf'}{$timestamp_act}{'ARRAY'};
    my $stats_last = $data{'perf'}{$timestamp_last}{'ARRAY'};

    # my $netRxKB       = "";
    # my $reads         = "";
    my $nsSsdHit_prct = "";
    # my $ssdWrKB       = "";
    my $nsMemHit_prct = "";
    # my $hddWrKB       = "";
    # my $ssdRdKB       = "";
    # my $netTxKB       = "";
    # my $hddRdKB       = "";
    # my $writes        = "";
    # my $seqRead_prct  = "";


    # if ( defined $stats_act->{'date'} && $stats_act->{'date'} ne '' ) {
    #   $date = $stats_act->{'date'};
    # }

    # if ( defined $stats_act->{'netRxKB'} && Xormon::isdigit( $stats_act->{'netRxKB'} ) && defined $stats_last->{'netRxKB'} && Xormon::isdigit( $stats_last->{'netRxKB'} ) ) {
    #   $netRxKB = ( $stats_act->{'netRxKB'} - $stats_last->{'netRxKB'} ) / $sample_rate;
    #   $netRxKB = Xormon::returnBytes( $netRxKB, "kB"); # to bytes
    #   if ( $netRxKB < 0 ) { $netRxKB = ""; }
    # }
    # if ( defined $stats_act->{'reads'} && Xormon::isdigit( $stats_act->{'reads'} ) && defined $stats_last->{'reads'} && Xormon::isdigit( $stats_last->{'reads'} ) ) {
    #   $reads = ( $stats_act->{'reads'} - $stats_last->{'reads'} ) / $sample_rate;
    #   $reads = sprintf( '%.0f', $reads );
    #   if ( $reads < 0 ) { $reads = ""; }
    # }
    # if ( defined $stats_act->{'ssdWrKB'} && Xormon::isdigit( $stats_act->{'ssdWrKB'} ) && defined $stats_last->{'ssdWrKB'} && Xormon::isdigit( $stats_last->{'ssdWrKB'} ) ) {
    #   $ssdWrKB = ( $stats_act->{'ssdWrKB'} - $stats_last->{'ssdWrKB'} ) / $sample_rate;
    #   $ssdWrKB = Xormon::returnBytes( $ssdWrKB, "kB"); # to bytes
    #   if ( $ssdWrKB < 0 ) { $ssdWrKB = ""; }
    # }
    # if ( defined $stats_act->{'hddWrKB'} && Xormon::isdigit( $stats_act->{'hddWrKB'} ) && defined $stats_last->{'hddWrKB'} && Xormon::isdigit( $stats_last->{'hddWrKB'} ) ) {
    #   $hddWrKB = ( $stats_act->{'hddWrKB'} - $stats_last->{'hddWrKB'} ) / $sample_rate;
    #   $hddWrKB = Xormon::returnBytes( $hddWrKB, "kB"); # to bytes
    #   if ( $hddWrKB < 0 ) { $hddWrKB = ""; }
    # }
    # if ( defined $stats_act->{'ssdRdKB'} && Xormon::isdigit( $stats_act->{'ssdRdKB'} ) && defined $stats_last->{'ssdRdKB'} && Xormon::isdigit( $stats_last->{'ssdRdKB'} ) ) {
    #   $ssdRdKB = ( $stats_act->{'ssdRdKB'} - $stats_last->{'ssdRdKB'} ) / $sample_rate;
    #   $ssdRdKB = Xormon::returnBytes( $ssdRdKB, "kB"); # to bytes
    #   if ( $ssdRdKB < 0 ) { $ssdRdKB = ""; }
    # }
    # if ( defined $stats_act->{'netTxKB'} && Xormon::isdigit( $stats_act->{'netTxKB'} ) && defined $stats_last->{'netTxKB'} && Xormon::isdigit( $stats_last->{'netTxKB'} ) ) {
    #   $netTxKB = ( $stats_act->{'netTxKB'} - $stats_last->{'netTxKB'} ) / $sample_rate;
    #   $netTxKB = Xormon::returnBytes( $netTxKB, "kB"); # to bytes
    #   if ( $netTxKB < 0 ) { $netTxKB = ""; }
    # }
    # if ( defined $stats_act->{'hddRdKB'} && Xormon::isdigit( $stats_act->{'hddRdKB'} ) && defined $stats_last->{'hddRdKB'} && Xormon::isdigit( $stats_last->{'hddRdKB'} ) ) {
    #   $hddRdKB = ( $stats_act->{'hddRdKB'} - $stats_last->{'hddRdKB'} ) / $sample_rate;
    #   $hddRdKB = Xormon::returnBytes( $hddRdKB, "kB"); # to bytes
    #   if ( $hddRdKB < 0 ) { $hddRdKB = ""; }
    # }
    # if ( defined $stats_act->{'writes'} && Xormon::isdigit( $stats_act->{'writes'} ) && defined $stats_last->{'writes'} && Xormon::isdigit( $stats_last->{'writes'} ) ) {
    #   $writes = ( $stats_act->{'writes'} - $stats_last->{'writes'} ) / $sample_rate;
    #   $writes = sprintf( '%.0f', $writes );
    #   if ( $writes < 0 ) { $writes = ""; }
    # }

    # if ( defined $stats_act->{'netRxKiB'} && Xormon::isdigit( $stats_act->{'netRxKiB'} ) && defined $stats_last->{'netRxKiB'} && Xormon::isdigit( $stats_last->{'netRxKiB'} ) ) {
    #   $netRxKB = ( $stats_act->{'netRxKiB'} - $stats_last->{'netRxKiB'} ) / $sample_rate;
    #   $netRxKB = Xormon::returnBytes( $netRxKB, "kiB"); # from kiB to bytes
    #   if ( $netRxKB < 0 ) { $netRxKB = ""; }
    # }
    # if ( defined $stats_act->{'ssdWrKiB'} && Xormon::isdigit( $stats_act->{'ssdWrKiB'} ) && defined $stats_last->{'ssdWrKiB'} && Xormon::isdigit( $stats_last->{'ssdWrKiB'} ) ) {
    #   $ssdWrKB = ( $stats_act->{'ssdWrKiB'} - $stats_last->{'ssdWrKiB'} ) / $sample_rate;
    #   $ssdWrKB = Xormon::returnBytes( $ssdWrKB, "kiB"); # from kiB to bytes
    #   if ( $ssdWrKB < 0 ) { $ssdWrKB = ""; }
    # }
    # if ( defined $stats_act->{'hddWrKiB'} && Xormon::isdigit( $stats_act->{'hddWrKiB'} ) && defined $stats_last->{'hddWrKiB'} && Xormon::isdigit( $stats_last->{'hddWrKiB'} ) ) {
    #   $hddWrKB = ( $stats_act->{'hddWrKiB'} - $stats_last->{'hddWrKiB'} ) / $sample_rate;
    #   $hddWrKB = Xormon::returnBytes( $hddWrKB, "kiB"); # from kiB to bytes
    #   if ( $hddWrKB < 0 ) { $hddWrKB = ""; }
    # }
    # if ( defined $stats_act->{'ssdRdKiB'} && Xormon::isdigit( $stats_act->{'ssdRdKiB'} ) && defined $stats_last->{'ssdRdKiB'} && Xormon::isdigit( $stats_last->{'ssdRdKiB'} ) ) {
    #   $ssdRdKB = ( $stats_act->{'ssdRdKiB'} - $stats_last->{'ssdRdKiB'} ) / $sample_rate;
    #   $ssdRdKB = Xormon::returnBytes( $ssdRdKB, "kiB"); # from kiB to bytes
    #   if ( $ssdRdKB < 0 ) { $ssdRdKB = ""; }
    # }
    # if ( defined $stats_act->{'netTxKiB'} && Xormon::isdigit( $stats_act->{'netTxKiB'} ) && defined $stats_last->{'netTxKiB'} && Xormon::isdigit( $stats_last->{'netTxKiB'} ) ) {
    #   $netTxKB = ( $stats_act->{'netTxKiB'} - $stats_last->{'netTxKiB'} ) / $sample_rate;
    #   $netTxKB = Xormon::returnBytes( $netTxKB, "kiB"); # from kiB to bytes
    #   if ( $netTxKB < 0 ) { $netTxKB = ""; }
    # }
    # if ( defined $stats_act->{'hddRdKiB'} && Xormon::isdigit( $stats_act->{'hddRdKiB'} ) && defined $stats_last->{'hddRdKiB'} && Xormon::isdigit( $stats_last->{'hddRdKiB'} ) ) {
    #   $hddRdKB = ( $stats_act->{'hddRdKiB'} - $stats_last->{'hddRdKiB'} ) / $sample_rate;
    #   $hddRdKB = Xormon::returnBytes( $hddRdKB, "kiB"); # from kiB to bytes
    #   if ( $hddRdKB < 0 ) { $hddRdKB = ""; }
    # }

    # if ( defined $stats_act->{'seqRead%'} && Xormon::isdigit( $stats_act->{'seqRead%'} ) ) {
    #   $seqRead_prct = $stats_act->{'seqRead%'};
    #   if ( $seqRead_prct < 0 ) { $seqRead_prct = ""; }
    # }
    if ( defined $stats_act->{'nsSsdHit%'} && Xormon::isdigit( $stats_act->{'nsSsdHit%'} ) ) {
      $nsSsdHit_prct = $stats_act->{'nsSsdHit%'};
      if ( $nsSsdHit_prct < 0 ) { $nsSsdHit_prct = ""; }
    }
    if ( defined $stats_act->{'nsMemHit%'} && Xormon::isdigit( $stats_act->{'nsMemHit%'} ) ) {
      $nsMemHit_prct = $stats_act->{'nsMemHit%'};
      if ( $nsMemHit_prct < 0 ) { $nsMemHit_prct = ""; }
    }

    #my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "ARRAY" . $Array_name); $hash_key = lc($hash_key);
    #print "$hash_key\n";

    # IO
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'} ne '' ) {
      $read_io = $data{'perf'}{$timestamp_act}{'ARRAY'}{'read_io'};
    }
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'} ne '' ) {
      $write_io = $data{'perf'}{$timestamp_act}{'ARRAY'}{'write_io'};
    }
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'} ne '' ) {
      $io_rate = $data{'perf'}{$timestamp_act}{'ARRAY'}{'io_rate'};
    }
    # DATA
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'} ne '' ) {
      $read = $data{'perf'}{$timestamp_act}{'ARRAY'}{'read'};
    }
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'} ne '' ) {
      $write = $data{'perf'}{$timestamp_act}{'ARRAY'}{'write'};
    }
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'} ne '' ) {
      $data_rate = $data{'perf'}{$timestamp_act}{'ARRAY'}{'data_rate'};
    }
    # RESP_T
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_r'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_r'} ne '' && defined $read_io && $read_io ne "" && $read_io != 0 ) {
      $resp_t_r = Xormon::formatNumberByType("float4", ( $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_r'} / $read_io ) );
    }
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_w'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_w'} ne '' && defined $write_io && $write_io ne "" && $write_io != 0 ) {
      $resp_t_w = Xormon::formatNumberByType("float4", ( $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t_w'} / $write_io ) );
    }
    if ( defined $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t'} && $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t'} ne '' && defined $io_rate && $io_rate ne "" && $io_rate != 0 ) {
      $resp_t = Xormon::formatNumberByType("float4", ( $data{'perf'}{$timestamp_act}{'ARRAY'}{'resp_t'} / $io_rate ) );
    }
    # BLOCK SIZE
    if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
      if ( $io_rate == 0 ) {
        $block_size = 0;
      }
      else{
        $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
      }
    }
    if ( Xormon::isdigit($read_io) && Xormon::isdigit($read) ) {
      if ( $read_io == 0 ) {
        $block_size_r = 0;
      }
      else{
        $block_size_r = Xormon::formatNumberByType("float4", ($read / $read_io));
      }
    }
    if ( Xormon::isdigit($write_io) && Xormon::isdigit($write) ) {
      if ( $write_io == 0 ) {
        $block_size_w = 0;
      }
      else{
        $block_size_w = Xormon::formatNumberByType("float4", ($write / $write_io));
      }
    }

    my $pool_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "array_total"); $pool_key = lc($pool_key);
    $device_data_out .= "$cfg->{'hostcfg_id'};$timestamp_act;$read_io;$write_io;$io_rate;$read;$write;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$Total_array_usage_mb;$Total_array_capacity_mb;$Available_space_mb;$used_pct;$nsSsdHit_prct;$nsMemHit_prct\n";
    $data_out .= "$pool_key;$timestamp_act;$read_io;$write_io;$io_rate;$read;$write;$data_rate;$resp_t_r;$resp_t_w;$resp_t;$block_size_r;$block_size_w;$block_size;$Total_array_usage_mb;$Total_array_capacity_mb;$Available_space_mb;$used_pct;$nsSsdHit_prct;$nsMemHit_prct\n";

    my %device_arch = (
      "item_id" => $cfg->{'hostcfg_id'},
      "label" => $cfg->{'hostalias'},
      "hw_type" => "nimble",
      "subsystem" => "device",
      "parents" => [],
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%device_arch);

    my %arch = (
      "item_id" => $pool_key,
      "label" => "array_total",
      "hw_type" => "nimble",
      "subsystem" => "pool",
      "parents" => [$cfg->{'hostcfg_id'}],
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%arch);

    # SAVE DEVICE
    my $rc = $xormon->saveCSV($device_data_out,"nimble","device");

    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("DEVICE status : $status");

    # SAVE POOL
    $rc = $xormon->saveCSV($data_out,"nimble","pool");

    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("POOL status : $status");
    $data_out = "";
  }

  # VOLUME-FOLDER
  if ( defined $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'} && $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'} ne '' ) {

    $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t\n";

    foreach my $vol_folder ( keys( %{ $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'} } ) ) {
      my $read_io   = "";
      my $write_io  = "";
      my $io_rate   = "";
      my $read      = "";
      my $write     = "";
      my $data_rate = "";
      my $resp_t_r  = "";
      my $resp_t_w  = "";
      my $resp_t    = "";

      my $stats_folder  = $data{'perf'}{$timestamp_act}{'VOLUME-FOLDER'}{$vol_folder};

      # IO
      if (defined $stats_folder->{'read_io'} && $stats_folder->{'read_io'} ne ""){
        $read_io = $stats_folder->{'read_io'};
      }
      if (defined $stats_folder->{'write_io'} && $stats_folder->{'write_io'} ne ""){
        $write_io = $stats_folder->{'write_io'};
      }
      if (defined $stats_folder->{'io_rate'} && $stats_folder->{'io_rate'} ne ""){
        $io_rate = $stats_folder->{'io_rate'};
      }
      # DATA
      if (defined $stats_folder->{'read'} && $stats_folder->{'read'} ne ""){
        $read = $stats_folder->{'read'};
      }
      if (defined $stats_folder->{'write'} && $stats_folder->{'write'} ne ""){
        $write = $stats_folder->{'write'};
      }
      if (defined $stats_folder->{'data_rate'} && $stats_folder->{'data_rate'} ne ""){
        $data_rate = $stats_folder->{'data_rate'};
      }
      # RESP_T
      if ( defined $stats_folder->{'resp_t_r'} && $stats_folder->{'resp_t_r'} ne '' && defined $read_io && $read_io ne "" && $read_io != 0 ) {
        $resp_t_r = Xormon::formatNumberByType("float4", ( $stats_folder->{'resp_t_r'} / $read_io ) );
      }
      if ( defined $stats_folder->{'resp_t_w'} && $stats_folder->{'resp_t_w'} ne '' && defined $write_io && $write_io ne "" && $write_io != 0 ) {
        $resp_t_w = Xormon::formatNumberByType("float4", ( $stats_folder->{'resp_t_w'} / $write_io ) );
      }
      if ( defined $stats_folder->{'resp_t'} && $stats_folder->{'resp_t'} ne '' && defined $io_rate && $io_rate ne "" && $io_rate != 0 ) {
        $resp_t = Xormon::formatNumberByType("float4", ( $stats_folder->{'resp_t'} / $io_rate ) );
      }

      $vol_folder = remove_unauthorized_characters($vol_folder);

      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "VOLUME_FOLDER" . $vol_folder); $hash_key = lc($hash_key);
      #print "$hash_key\n";

      my %arch = (
          "item_id" => $hash_key,
          "label" => $vol_folder,
          "hw_type" => "nimble",
          "subsystem" => "volume_folder",
          "parents" => [$cfg->{'hostcfg_id'}],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
      push(@architecture, \%arch);

      $data_out .= "$hash_key;$timestamp_act;$read_io;$write_io;$io_rate;$read;$write;$data_rate;$resp_t_r;$resp_t_w;$resp_t\n";

    }

    #print $data_out;
    # SAVE VOLUME FOLDER
    my $rc = $xormon->saveCSV($data_out,"nimble","volume_folder");

    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("VOLUME FOLDER status : $status");
    $data_out = "";
  }

  # DISK
  if ( defined $data{'DISKS'} && $data{'DISKS'} ne '' ) {

    $data_out .= "uuid;time;read_io;read;write_io;write;data_rate;io_rate;resp_t_r;resp_t_w\n";
    foreach my $disk_controller ( sort keys( %{ $data{'DISKS'} } ) ) {
      if ( defined $data{'DISKS'}{$disk_controller} && $data{'DISKS'}{$disk_controller} ne '' ) {
        foreach my $disk_id ( sort { $a <=> $b } keys( %{ $data{'DISKS'}{$disk_controller} } ) ) {

          my $disk_label = $disk_id . "." . $disk_controller;

          my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "DRIVE" . $disk_label); $hash_key = lc($hash_key);
          #print "$hash_key\n";

          my %arch = (
              "item_id" => $hash_key,
              "label" => $disk_label,
              "hw_type" => "nimble",
              "subsystem" => "drive",
              "parents" => [$cfg->{'hostcfg_id'}],
              "hostcfg_id" => $cfg->{'hostcfg_id'}
            );
          push(@architecture, \%arch);

          my $read_io   = "";
          my $read      = "";
          my $write_io  = "";
          my $write     = "";
          my $data_rate = "";
          my $io_rate   = "";
          my $resp_t_r  = "";
          my $resp_t_w  = "";

          my $stats_act  = $data{'perf'}{$timestamp_act}{'DISKS'}{$disk_controller}{$disk_id};
          my $stats_last = $data{'perf'}{$timestamp_last}{'DISKS'}{$disk_controller}{$disk_id};


          if ( defined $stats_act->{'reads'} && Xormon::isdigit( $stats_act->{'reads'} ) && defined $stats_last->{'reads'} && Xormon::isdigit( $stats_last->{'reads'} ) ) {
            $read_io = ( $stats_act->{'reads'} - $stats_last->{'reads'} ) / $sample_rate;
            $read_io = sprintf( '%.0f', $read_io );
            if ( $read_io < 0 ) { $read_io = ""; }
          }
          if ( defined $stats_act->{'rdKB'} && Xormon::isdigit( $stats_act->{'rdKB'} ) && defined $stats_last->{'rdKB'} && Xormon::isdigit( $stats_last->{'rdKB'} ) ) {
            $read = ( $stats_act->{'rdKB'} - $stats_last->{'rdKB'} ) / $sample_rate;
            $read = sprintf ( '%.0f', Xormon::returnBytes( $read, "kB") ); # to bytes
            if ( $read < 0 ) { $read = ""; }
          }
          if ( defined $stats_act->{'rdKiB'} && Xormon::isdigit( $stats_act->{'rdKiB'} ) && defined $stats_last->{'rdKiB'} && Xormon::isdigit( $stats_last->{'rdKiB'} ) ) {
            $read = ( $stats_act->{'rdKiB'} - $stats_last->{'rdKiB'} ) / $sample_rate;
            $read = sprintf( '%.0f', Xormon::returnBytes( $read, "kiB") ); # to bytes
            if ( $read < 0 ) { $read = ""; }
          }
          if ( defined $stats_act->{'writes'} && Xormon::isdigit( $stats_act->{'writes'} ) && defined $stats_last->{'writes'} && Xormon::isdigit( $stats_last->{'writes'} ) ) {
            $write_io = ( $stats_act->{'writes'} - $stats_last->{'writes'} ) / $sample_rate;
            $write_io = sprintf( '%.0f', $write_io );
            if ( $write_io < 0 ) { $write_io = ""; }
          }
          if ( defined $stats_act->{'wrKB'} && Xormon::isdigit( $stats_act->{'wrKB'} ) && defined $stats_last->{'wrKB'} && Xormon::isdigit( $stats_last->{'wrKB'} ) ) {
            $write = ( $stats_act->{'wrKB'} - $stats_last->{'wrKB'} ) / $sample_rate;
            $write = sprintf( '%.2f', Xormon::returnBytes( $write, "kB") ); # to bytes
            if ( $write < 0 ) { $write = ""; }
          }
          if ( defined $stats_act->{'wrKiB'} && Xormon::isdigit( $stats_act->{'wrKiB'} ) && defined $stats_last->{'wrKiB'} && Xormon::isdigit( $stats_last->{'wrKiB'} ) ) {
            $write = ( $stats_act->{'wrKiB'} - $stats_last->{'wrKiB'} ) / $sample_rate;
            $write = sprintf( '%.2f', Xormon::returnBytes( $write, "kiB") ); # to bytes
            if ( $write < 0 ) { $write = ""; }
          }
          if ( defined $read_io && Xormon::isdigit($read_io) && defined $write_io && Xormon::isdigit($write_io) ) {
            $io_rate = $read_io + $write_io;
          }
          if ( defined $read && Xormon::isdigit($read) && defined $write && Xormon::isdigit($write) ) {
            $data_rate = $read + $write;
          }
          if ( defined $stats_act->{'rdAvgMsec'} && Xormon::isdigit( $stats_act->{'rdAvgMsec'} ) ) {
            $resp_t_r = $stats_act->{'rdAvgMsec'};
          }
          if ( defined $stats_act->{'wrAvgMsec'} && Xormon::isdigit( $stats_act->{'wrAvgMsec'} ) ) {
            $resp_t_w = $stats_act->{'wrAvgMsec'};
          }
          $data_out .= "$hash_key;$timestamp_act;$read_io;$read;$write_io;$write;$data_rate;$io_rate;$resp_t_r;$resp_t_w\n";
        }
      }
    }
    #print $data_out;

    #SAVE DRIVE
    my $rc = $xormon->saveCSV($data_out,"nimble","drive");

    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("DRIVE status : $status");
    $data_out = "";
  } # end if DISK

  # Ethernet ports
  if ( defined $data{'perf'}{$timestamp_act}{'PORTS'}{'NET'} && $data{'perf'}{$timestamp_act}{'PORTS'}{'NET'} ne '' ) {
    $data_out .= "uuid;time;read_packet;write_packet;read;write;read_error;write_error;data_rate;packet_rate\n";
    foreach my $port_id ( sort keys( %{ $data{'perf'}{$timestamp_act}{'PORTS'}{'NET'} } ) ) {

      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "ETHPORT" . $port_id); $hash_key = lc($hash_key);

      #  'NET' => {
      #             'eth5' => {
      #                         'dev' => 'nimble71:eth5',
      #                         'rxPackets' => 64120657,
      #                         'txErrors' => 0,
      #                         'rxErrors' => 0,
      #                         'rxKiB' => 7535538,
      #                         'txPackets' => 4896692,
      #                         'txKiB' => 821085,
      #                         'date' => '2023-02-02_14:05:00',
      #                         'yyyy-mm-dd_hh:mm:ss' => ' '
      #                       },

      my $txPackets = "";
      my $rxPackets = "";
      my $txKB      = "";
      my $rxKB      = "";
      my $txErrors  = "";
      my $rxErrors  = "";

      my $pkts_rate = "";
      my $data_rate = "";

      my $stats_act  = $data{'perf'}{$timestamp_act}{'PORTS'}{'NET'}{$port_id};
      my $stats_last = $data{'perf'}{$timestamp_last}{'PORTS'}{'NET'}{$port_id};

      if ( defined $stats_act->{'txPackets'} && Xormon::isdigit( $stats_act->{'txPackets'} ) && defined $stats_last->{'txPackets'} && Xormon::isdigit( $stats_last->{'txPackets'} ) ) {
        $txPackets = ( $stats_act->{'txPackets'} - $stats_last->{'txPackets'} ) / $sample_rate;
        $txPackets = sprintf( '%.2f', $txPackets );
        if ( $txPackets < 0 ) { $txPackets = ""; }
      }
      if ( defined $stats_act->{'rxPackets'} && Xormon::isdigit( $stats_act->{'rxPackets'} ) && defined $stats_last->{'rxPackets'} && Xormon::isdigit( $stats_last->{'rxPackets'} ) ) {
        $rxPackets = ( $stats_act->{'rxPackets'} - $stats_last->{'rxPackets'} ) / $sample_rate;
        $rxPackets = sprintf( '%.2f', $rxPackets );
        if ( $rxPackets < 0 ) { $rxPackets = ""; }
      }
      if ( defined $stats_act->{'txKB'} && Xormon::isdigit( $stats_act->{'txKB'} ) && defined $stats_last->{'txKB'} && Xormon::isdigit( $stats_last->{'txKB'} ) ) {
        $txKB = ( $stats_act->{'txKB'} - $stats_last->{'txKB'} ) / $sample_rate;
        $txKB = Xormon::returnBytes( $txKB, "kB"); # to bytes
        if ( $txKB < 0 ) { $txKB = ""; }
      }
      if ( defined $stats_act->{'rxKB'} && Xormon::isdigit( $stats_act->{'rxKB'} ) && defined $stats_last->{'rxKB'} && Xormon::isdigit( $stats_last->{'rxKB'} ) ) {
        $rxKB = ( $stats_act->{'rxKB'} - $stats_last->{'rxKB'} ) / $sample_rate;
        $rxKB = Xormon::returnBytes( $rxKB, "kB"); # to bytes
        if ( $rxKB < 0 ) { $rxKB = ""; }
      }
      if ( defined $stats_act->{'txKiB'} && Xormon::isdigit( $stats_act->{'txKiB'} ) && defined $stats_last->{'txKiB'} && Xormon::isdigit( $stats_last->{'txKiB'} ) ) {
        $txKB = ( $stats_act->{'txKiB'} - $stats_last->{'txKiB'} ) / $sample_rate;
        $txKB = Xormon::returnBytes( $txKB, "kiB"); # to bytes
        if ( $txKB < 0 ) { $txKB = ""; }
      }
      if ( defined $stats_act->{'rxKiB'} && Xormon::isdigit( $stats_act->{'rxKiB'} ) && defined $stats_last->{'rxKiB'} && Xormon::isdigit( $stats_last->{'rxKiB'} ) ) {
        $rxKB = ( $stats_act->{'rxKiB'} - $stats_last->{'rxKiB'} ) / $sample_rate;
        $rxKB = Xormon::returnBytes( $rxKB, "kiB"); # to bytes
        if ( $rxKB < 0 ) { $rxKB = ""; }
      }
      if ( defined $stats_act->{'txErrors'} && Xormon::isdigit( $stats_act->{'txErrors'} ) && defined $stats_last->{'txErrors'} && Xormon::isdigit( $stats_last->{'txErrors'} ) ) {
        $txErrors = ( $stats_act->{'txErrors'} - $stats_last->{'txErrors'} ) / $sample_rate;
        $txErrors = sprintf( '%.2f', $txErrors );
        if ( $txErrors < 0 ) { $txErrors = ""; }
      }
      if ( defined $stats_act->{'rxErrors'} && Xormon::isdigit( $stats_act->{'rxErrors'} ) && defined $stats_last->{'rxErrors'} && Xormon::isdigit( $stats_last->{'rxErrors'} ) ) {
        # print "$port_id\n";
        # print "prvni $stats_act->{'rxErrors'}\n";
        # print "druhy $stats_last->{'rxErrors'}\n";

        $rxErrors = ( $stats_act->{'rxErrors'} - $stats_last->{'rxErrors'} ) / $sample_rate;
        $rxErrors = sprintf( '%.2f', $rxErrors );
        if ( $rxErrors < 0 ) { $rxErrors = ""; }
      }
      if ( defined $txKB && Xormon::isdigit($txKB) && defined $rxKB && Xormon::isdigit($rxKB) ) {
        $data_rate = $txKB + $rxKB;
      }
      if ( defined $txPackets && Xormon::isdigit($txPackets) && defined $rxPackets && Xormon::isdigit($rxPackets) ) {
        $pkts_rate = $txPackets + $rxPackets;
      }

      my %arch = (
          "item_id" => $hash_key,
          "label" => $port_id,
          "hw_type" => "nimble",
          "subsystem" => "ethport",
          "parents" => [$cfg->{'hostcfg_id'}],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
      push(@architecture, \%arch);


      $data_out .= "$hash_key;$timestamp_act;$rxPackets;$txPackets;$rxKB;$txKB;$rxErrors;$txErrors;$data_rate;$pkts_rate\n";
    }
    #print "$data_out";

    #SAVE ETHPORT
    my $rc = $xormon->saveCSV($data_out,"nimble","ethport");


    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("ETHPORT status : $status");
    $data_out = "";
  }

  if ( defined $data{'perf'}{$timestamp_act}{'PORTS'}{'FC'} && $data{'perf'}{$timestamp_act}{'PORTS'}{'FC'} ne '' ) {

    $data_out .= "uuid;time;read;write;data_rate\n";

    # find active Controller
    my $active_controller_name = "";
    if ( defined $data{'CONTROLLER'} && $data{'CONTROLLER'} ne '' ) {
      foreach my $controller ( sort keys( %{ $data{'CONTROLLER'} } ) ) {
        if ( defined $data{'CONTROLLER'}{$controller}{'State'} && $data{'CONTROLLER'}{$controller}{'State'} eq "active" ) {
          $active_controller_name = $controller;
          last;
        }
      }
    }

    foreach my $port_id ( sort keys( %{ $data{'perf'}{$timestamp_act}{'PORTS'}{'FC'} } ) ) {

      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "FCPORT" . $port_id); $hash_key = lc($hash_key);

      my $date        = "";
      my $dev         = "";
      my $txKB        = "";
      my $rxKB        = "";
      # my $rxBBcreds   = "";
      # my $txBBcreds   = "";
      # my $Inv_tx_Word = "";
      # my $CRC_Errors  = "";
      # my $loss_sync   = "";
      # my $loss_sig    = "";
      # my $link_fail   = "";

      my $data_rate = "";

      my $stats_act  = $data{'perf'}{$timestamp_act}{'PORTS'}{'FC'}{$port_id};
      my $stats_last = $data{'perf'}{$timestamp_last}{'PORTS'}{'FC'}{$port_id};

      # if ( defined $stats_act->{'date'} && $stats_act->{'date'} ne '' ) {
      #   $date = $stats_act->{'date'};
      # }
      # if ( defined $stats_act->{'dev'} && $stats_act->{'dev'} ne '' ) {
      #   $dev = $stats_act->{'dev'};
      # }
      if ( defined $stats_act->{'txKB'} && Xormon::isdigit( $stats_act->{'txKB'} ) && defined $stats_last->{'txKB'} && Xormon::isdigit( $stats_last->{'txKB'} ) ) {
        $txKB = ( $stats_act->{'txKB'} - $stats_last->{'txKB'} ) / $sample_rate;
        $txKB = Xormon::returnBytes( $txKB, "kB"); # to bytes
        if ( $txKB < 0 ) { $txKB = ""; }
      }
      if ( defined $stats_act->{'rxKB'} && Xormon::isdigit( $stats_act->{'rxKB'} ) && defined $stats_last->{'rxKB'} && Xormon::isdigit( $stats_last->{'rxKB'} ) ) {
        $rxKB = ( $stats_act->{'rxKB'} - $stats_last->{'rxKB'} ) / $sample_rate;
        $rxKB = Xormon::returnBytes( $rxKB, "kB"); # to bytes
        if ( $rxKB < 0 ) { $rxKB = ""; }
      }
      if ( defined $stats_act->{'txKiB'} && Xormon::isdigit( $stats_act->{'txKiB'} ) && defined $stats_last->{'txKiB'} && Xormon::isdigit( $stats_last->{'txKiB'} ) ) {
        $txKB = ( $stats_act->{'txKiB'} - $stats_last->{'txKiB'} ) / $sample_rate;
        $txKB = Xormon::returnBytes( $txKB, "kiB"); # to bytes
        if ( $txKB < 0 ) { $txKB = ""; }
      }
      if ( defined $stats_act->{'rxKiB'} && Xormon::isdigit( $stats_act->{'rxKiB'} ) && defined $stats_last->{'rxKiB'} && Xormon::isdigit( $stats_last->{'rxKiB'} ) ) {
        $rxKB = ( $stats_act->{'rxKiB'} - $stats_last->{'rxKiB'} ) / $sample_rate;
        $rxKB = Xormon::returnBytes( $rxKB, "kiB"); # to bytes
        if ( $rxKB < 0 ) { $rxKB = ""; }
      }
      # if ( defined $stats_act->{'rxBBcreds'} && Xormon::isdigit( $stats_act->{'rxBBcreds'} ) && defined $stats_last->{'rxBBcreds'} && Xormon::isdigit( $stats_last->{'rxBBcreds'} ) ) {
      #   $rxBBcreds = ( $stats_act->{'rxBBcreds'} - $stats_last->{'rxBBcreds'} ) / $sample_rate;
      #   $rxBBcreds = sprintf( '%.2f', $rxBBcreds );
      #   if ( $rxBBcreds < 0 ) { $rxBBcreds = ""; }
      # }
      # if ( defined $stats_act->{'txBBcreds'} && Xormon::isdigit( $stats_act->{'txBBcreds'} ) && defined $stats_last->{'txBBcreds'} && Xormon::isdigit( $stats_last->{'txBBcreds'} ) ) {
      #   $txBBcreds = ( $stats_act->{'txBBcreds'} - $stats_last->{'txBBcreds'} ) / $sample_rate;
      #   $txBBcreds = sprintf( '%.2f', $txBBcreds );
      #   if ( $txBBcreds < 0 ) { $txBBcreds = ""; }
      # }
      # if ( defined $stats_act->{'Inv tx Word'} && Xormon::isdigit( $stats_act->{'Inv tx Word'} ) && defined $stats_last->{'Inv tx Word'} && Xormon::isdigit( $stats_last->{'Inv tx Word'} ) ) {
      #   $Inv_tx_Word = ( $stats_act->{'Inv tx Word'} - $stats_last->{'Inv tx Word'} ) / $sample_rate;
      #   $Inv_tx_Word = sprintf( '%.2f', $Inv_tx_Word );
      #   if ( $Inv_tx_Word < 0 ) { $Inv_tx_Word = ""; }
      # }
      # if ( defined $stats_act->{'CRC Errors'} && Xormon::isdigit( $stats_act->{'CRC Errors'} ) && defined $stats_last->{'CRC Errors'} && Xormon::isdigit( $stats_last->{'CRC Errors'} ) ) {
      #   $CRC_Errors = ( $stats_act->{'CRC Errors'} - $stats_last->{'CRC Errors'} ) / $sample_rate;
      #   $CRC_Errors = sprintf( '%.2f', $CRC_Errors );
      #   if ( $CRC_Errors < 0 ) { $CRC_Errors = ""; }
      # }
      # if ( defined $stats_act->{'loss sync'} && Xormon::isdigit( $stats_act->{'loss sync'} ) && defined $stats_last->{'loss sync'} && Xormon::isdigit( $stats_last->{'loss sync'} ) ) {
      #   $loss_sync = ( $stats_act->{'loss sync'} - $stats_last->{'loss sync'} ) / $sample_rate;
      #   $loss_sync = sprintf( '%.2f', $loss_sync );
      #   if ( $loss_sync < 0 ) { $loss_sync = ""; }
      # }
      # if ( defined $stats_act->{'loss sig'} && Xormon::isdigit( $stats_act->{'loss sig'} ) && defined $stats_last->{'loss sig'} && Xormon::isdigit( $stats_last->{'loss sig'} ) ) {
      #   $loss_sig = ( $stats_act->{'loss sig'} - $stats_last->{'loss sig'} ) / $sample_rate;
      #   $loss_sig = sprintf( '%.2f', $loss_sig );
      #   if ( $loss_sig < 0 ) { $loss_sig = ""; }
      # }
      # if ( defined $stats_act->{'link fail'} && Xormon::isdigit( $stats_act->{'link fail'} ) && defined $stats_last->{'link fail'} && Xormon::isdigit( $stats_last->{'link fail'} ) ) {
      #   $link_fail = ( $stats_act->{'link fail'} - $stats_last->{'link fail'} ) / $sample_rate;
      #   $link_fail = sprintf( '%.2f', $link_fail );
      #   if ( $link_fail < 0 ) { $link_fail = ""; }
      # }

      if ( defined $txKB && Xormon::isdigit($txKB) && defined $rxKB && Xormon::isdigit($rxKB) ) {
        $data_rate = $txKB + $rxKB;
      }

      my %arch = (
        "item_id" => $hash_key,
        "label" => $port_id,
        "hw_type" => "nimble",
        "subsystem" => "fcport",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
      # WWPN for mapping
      if ( defined $active_controller_name && $active_controller_name ne '' ) {
        if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWPN'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWPN'} ne '' ) {
          $arch{wwn} = Xormon::formatWWN($data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWPN'});
        }
      }
      push(@architecture, \%arch);
      $data_out .= "$hash_key;$timestamp_act;$rxKB;$txKB;$data_rate\n";
    }

    #SAVE FCPORT
    my $rc = $xormon->saveCSV($data_out,"nimble","fcport");


    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("FCPORT status : $status");
    $data_out = "";
  }

  if ( $data{'HOSTS'}{'Initiator Group'} && $data{'HOSTS'}{'Initiator Group'} ne '' ) {

    $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t_r;resp_t_w;resp_t\n";

    foreach my $host ( sort keys( %{ $data{'HOSTS'}{'Initiator Group'} } ) ) {

      my $host_read_io   = "";
      my $host_write_io  = "";
      my $host_io_rate   = "";
      my $host_read      = "";
      my $host_write     = "";
      my $host_data_rate = "";
      my $host_resp_t_r  = "";
      my $host_resp_t_w  = "";
      my $host_resp_t    = "";

      my @vols = ();

      if ( defined $data{'HOSTS'}{'Initiator Group'}{$host}{'vols'} && $data{'HOSTS'}{'Initiator Group'}{$host}{'vols'} ne '' ) {

        foreach my $volume ( sort keys( %{ $data{'HOSTS'}{'Initiator Group'}{$host}{'vols'} } ) ) {

          my $read_io   = "";
          my $write_io  = "";
          my $io_rate   = "";
          my $read      = "";
          my $write     = "";
          my $data_rate = "";
          my $resp_t_r  = "";
          my $resp_t_w  = "";
          my $resp_t    = "";

          if ( defined $volumes{$timestamp_act}{$volume}{'read_io'} && $volumes{$timestamp_act}{$volume}{'read_io'} ne "" ) {
            $read_io = $volumes{$timestamp_act}{$volume}{'read_io'};
            $read_io = sprintf( '%.0f', $read_io );
            if ( $read_io < 0 ) { $read_io = ""; }
            if ( defined $read_io && Xormon::isdigit($read_io) ) {
              if ( defined $host_read_io && Xormon::isdigit($host_read_io) ) {
                $host_read_io = $host_read_io + $read_io;
              }
              else {
                $host_read_io = $read_io;
              }
            }
          }
          if ( defined $volumes{$timestamp_act}{$volume}{'write_io'} && $volumes{$timestamp_act}{$volume}{'write_io'} ne "" ) {
            $write_io = $volumes{$timestamp_act}{$volume}{'write_io'};
            $write_io = sprintf( '%.0f', $write_io );
            if ( $write_io < 0 ) { $write_io = ""; }
            if ( defined $write_io && Xormon::isdigit($write_io) ) {
              if ( defined $host_write_io && Xormon::isdigit($host_write_io) ) {
                $host_write_io = $host_write_io + $write_io;
              }
              else {
                $host_write_io = $write_io;
              }
            }
          }
          if ( defined $volumes{$timestamp_act}{$volume}{'io_rate'} && $volumes{$timestamp_act}{$volume}{'io_rate'} ne "" ) {
            $io_rate = $volumes{$timestamp_act}{$volume}{'io_rate'};
            if ( $io_rate < 0 ) { $io_rate = ""; }
            if ( defined $io_rate && Xormon::isdigit($io_rate) ) {
              if ( defined $host_io_rate && Xormon::isdigit($host_io_rate) ) {
                $host_io_rate = $host_io_rate + $io_rate;
              }
              else {
                $host_io_rate = $io_rate;
              }
            }
          }
          if ( defined $volumes{$timestamp_act}{$volume}{'read'} && $volumes{$timestamp_act}{$volume}{'read'} ne "" ) {
            $read = $volumes{$timestamp_act}{$volume}{'read'};
            $read = sprintf( '%.0f', $read );
            if ( $read < 0 ) { $read = ""; }
            if ( defined $read && Xormon::isdigit($read) ) {
              if ( defined $host_read && Xormon::isdigit($host_read) ) {
                $host_read = $host_read + $read;
              }
              else {
                $host_read = $read;
              }
            }
          }
          if ( defined $volumes{$timestamp_act}{$volume}{'write'} && $volumes{$timestamp_act}{$volume}{'write'} ne "" ) {
            $write = $volumes{$timestamp_act}{$volume}{'write'};
            $write = sprintf( '%.0f', $write );
            if ( $write < 0 ) { $write = ""; }
            if ( defined $write && Xormon::isdigit($write) ) {
              if ( defined $host_write && Xormon::isdigit($host_write) ) {
                $host_write = $host_write + $write;
              }
              else {
                $host_write = $write;
              }
            }
          }
          if ( defined $volumes{$timestamp_act}{$volume}{'data_rate'} && $volumes{$timestamp_act}{$volume}{'data_rate'} ne "" ) {
            $data_rate = $volumes{$timestamp_act}{$volume}{'data_rate'};
            if ( $data_rate < 0 ) { $data_rate = ""; }
            if ( defined $data_rate && Xormon::isdigit($data_rate) ) {
              if ( defined $host_data_rate && Xormon::isdigit($host_data_rate) ) {
                $host_data_rate = $host_data_rate + $data_rate;
              }
              else {
                $host_data_rate = $data_rate;
              }
            }
          }
          # resp_t
          if ( defined $volumes{$timestamp_act}{$volume}{'resp_t'} && $volumes{$timestamp_act}{$volume}{'resp_t'} ne "" && $io_rate && $io_rate ne "") {
            $resp_t = $volumes{$timestamp_act}{$volume}{'resp_t'};
            if ( $resp_t < 0 ) { $resp_t = ""; }
            if ( defined $resp_t && Xormon::isdigit($resp_t) ) {
              if ( defined $host_resp_t && Xormon::isdigit($host_resp_t) ) {
                $host_resp_t += $io_rate * $resp_t;
              }
              else {
                $host_resp_t = $io_rate * $resp_t;
              }
            }
          }
          if ( defined $volumes{$timestamp_act}{$volume}{'resp_t_r'} && $volumes{$timestamp_act}{$volume}{'resp_t_r'} ne "" && $read_io && $read_io ne "") {
            $resp_t_r = $volumes{$timestamp_act}{$volume}{'resp_t_r'};
            if ( $resp_t_r < 0 ) { $resp_t_r = ""; }
            if ( defined $resp_t_r && Xormon::isdigit($resp_t_r) ) {
              if ( defined $host_resp_t_r && Xormon::isdigit($host_resp_t_r) ) {
                $host_resp_t_r += $read_io * $resp_t_r;
              }
              else {
                $host_resp_t_r = $read_io * $resp_t_r;
              }
            }
          }
          if ( defined $volumes{$timestamp_act}{$volume}{'resp_t_w'} && $volumes{$timestamp_act}{$volume}{'resp_t_w'} ne "" && $write_io && $write_io ne "") {
            $resp_t_w = $volumes{$timestamp_act}{$volume}{'resp_t_w'};
            if ( $resp_t_w < 0 ) { $resp_t_w = ""; }
            if ( defined $resp_t_w && Xormon::isdigit($resp_t_w) ) {
              if ( defined $host_resp_t_w && Xormon::isdigit($host_resp_t_w) ) {
                $host_resp_t_w += $write_io * $resp_t_w;
              }
              else {
                $host_resp_t_w = $write_io * $resp_t_w;
              }
            }
          }

          $volume = remove_unauthorized_characters($volume);
          my $vol_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "VOLUME" . $volume); $vol_key = lc($vol_key);
          push(@vols, $vol_key);

        } # end foreach vol
      } # end if defined vols

      # RESP_T calculation
      if ( defined $host_resp_t_r && $host_resp_t_r ne '' && defined $host_read_io && $host_read_io ne "" && $host_read_io != 0 ) {
        $host_resp_t_r = Xormon::formatNumberByType("float4", ( $host_resp_t_r / $host_read_io ) );
      }
      if ( defined $host_resp_t_w && $host_resp_t_w ne '' && defined $host_write_io && $host_write_io ne "" && $host_write_io != 0 ) {
        $host_resp_t_w = Xormon::formatNumberByType("float4", ( $host_resp_t_w / $host_write_io ) );
      }
      if ( defined $host_resp_t && $host_resp_t ne '' && defined $host_io_rate && $host_io_rate ne "" && $host_io_rate != 0 ) {
        $host_resp_t = Xormon::formatNumberByType("float4", ( $host_resp_t / $host_io_rate ) );
      }

      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "HOST" . $host); $hash_key = lc($hash_key);
      #print "$hash_key\n";

      my %arch = (
          "item_id" => $hash_key,
          "label" => $host,
          "hw_type" => "nimble",
          "subsystem" => "host",
          "parents" => [$cfg->{'hostcfg_id'}],
          "hostcfg_id" => $cfg->{'hostcfg_id'}
        );
      if (@vols) {$arch{'children'} = \@vols;}
      push(@architecture, \%arch);

      $data_out .= "$hash_key;$timestamp_act;$host_read_io;$host_write_io;$host_io_rate;$host_read;$host_write;$host_data_rate;$host_resp_t_r;$host_resp_t_w;$host_resp_t\n";
    } # end for each host

    #SAVE HOST
    my $rc = $xormon->saveCSV($data_out,"nimble","host");


    # status code
    if ($rc eq "201") {
      $status = "successful";
    } elsif ($rc eq "401") {
      $status = "unauthorized";
    } else {
      $status = "error";
    }
    $logger->info("HOST status : $status");
    $data_out = "";

  } # end if if $data{'HOSTS'}{'Initiator Group'}



  #save architecture
  #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

  return 1;
}

sub array_time {
  my ($date) = run_ssh_cmd("date");

  # sometimes "date" cmd returns blank output, so try it again 3 more times
  if ( !defined $date || $date eq '' ) {
    for ( my $i = 0; $i < 3; $i++ ) {
      sleep(3);
      ($date) = run_ssh_cmd("date");
      if ( defined $date && $date ne '' ) { last; }
    }
  }

  if ( defined $date && $date ne '' ) {
    if ($cfg->{'isKeyFile'} == 1) {
      $xormon->isConnected(1);
    }
    chomp $date;
    my $act_timestamp = str2time($date);
    my $act_date      = localtime($act_timestamp);

    my $end_timestamp   = int( ($act_timestamp - 300) / $sample_rate ) * $sample_rate;
    my $start_timestamp = $end_timestamp - ( 2 * $sample_rate );

    # if ( -f "$wrkdir/$storage_name/last_stats" ) {
    #   open( LAST, "<$wrkdir/$storage_name/last_stats" ) || error( "Couldn't open file $wrkdir/$storage_name/last_stats $!" . __FILE__ . ":" . __LINE__ ) && exit;
    #   my ($ts) = <LAST>;
    #   close(LAST);

    #   if ( isdigit($ts) && $ts <= $end_timestamp ) {
    #     if ( ( $end_timestamp - $ts ) > 86400 ) {
    #       error( "Data were not be collected longer than 1 day. Create only one perf file at this time. $!" . __FILE__ . ":" . __LINE__ );
    #     }
    #     elsif ( ( $end_timestamp - $ts ) < $sample_rate ) {
    #       error( "Last perf file was created less than a sample_rate=$sample_rate ago. Do not run agent at this time! Exitting... $!" . __FILE__ . ":" . __LINE__ ) && exit;
    #     }
    #     else {
    #       $start_timestamp = $ts - $sample_rate;
    #     }
    #   }
    # }
    my $start_date = localtime($start_timestamp);
    my $end_date   = localtime($end_timestamp);

    # time to stats cmd
    # --from 2017-11-7,14:25 --to 2017-11-7,14:30
    my ( $sec_s, $min_s, $hour_s, $day_s, $month_s, $year_s, $wday_s, $yday_s, $isdst_s ) = localtime($start_timestamp);
    my ( $sec_e, $min_e, $hour_e, $day_e, $month_e, $year_e, $wday_e, $yday_e, $isdst_e ) = localtime($end_timestamp);
    $stats_start_time = sprintf( "%4d-%02d-%02d,%02d:%02d", $year_s + 1900, $month_s + 1, $day_s, $hour_s, $min_s );
    $stats_end_time   = sprintf( "%4d-%02d-%02d,%02d:%02d", $year_e + 1900, $month_e + 1, $day_e, $hour_e, $min_e );

    if ( exists $cfg->{'debug'} && $cfg->{'debug'} eq "time" ) {
      $logger->info("ARRAY date       : $date");
      $logger->info("ARRAY timestamp  : $act_timestamp : $act_date");
      $logger->info("stats start time : $start_timestamp : $start_date");
      $logger->info("stats end time   : $end_timestamp : $end_date");
      $logger->info("stats time range : $stats_start_time - $stats_end_time");
    }

    return 1;
  }
  else {
    if ($cfg->{'isKeyFile'} == 1) {
      $xormon->isConnected(0, "Nimble OS time/date not found");
    }
    $logger->error( "Nimble OS time/date not found! Exiting... $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
}

sub array_info {
  my $array_name = $cfg->{'arrayName'};
  my @array_list = run_ssh_cmd("array --list");

  if (@array_list) {
    foreach my $line (@array_list) {
      chomp $line;
      if ( $line =~ m/^Name\s+/ ) { next; }
      if ( $line =~ m/^---+\+/ )  { next; }

      if ( $line =~ m/^$array_name\s+/ ) {
        my @array_info = run_ssh_cmd("array --info \"$array_name\"");
        foreach my $line_l (@array_info) {
          chomp $line_l;
          my ( $key, $value ) = split( /:\s+/, $line_l );
          $data{'ARRAY'}{$key} = $value;
        }
        #print Dumper %data;

        return 1;
      }
    }
  }

  $logger->error( "Array name \"$array_name\" not found! $!" . __FILE__ . ":" . __LINE__ );
  foreach (@array_list) {
    chomp $_;
    if ( $_ eq '' ) { next; }
    $logger->error("$_");
  }
  return 0;
}

sub array_stats {
  my $array_name = $cfg->{'arrayName'};
  my @array_stats = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --array $array_name");
  # print "STATS @array_stats\n";
  if (@array_stats) {
    my @header;
    my $stat_date  = "";
    my $stat_time  = "";
    my $perf_lines = 0;
    foreach my $line (@array_stats) {
      chomp $line;

      if ( $line =~ m/try increasing interval to/ ) { next; }

      if ( $line =~ m/^yyyy-mm-dd,hh:mm:ss,/ ) {
        $perf_lines = 1;
        $line =~ s/^yyyy-mm-dd,hh:mm:ss,/yyyy-mm-dd_hh:mm:ss,/;
        @header = split( ",", $line );
        next;
      }
      if ( $perf_lines == 1 ) {
        my ( $time, undef ) = split( /,/, $line );

        if ( defined $time && $time =~ m/^[0-9][0-9][0-9][0-9]-[0-9]+-[0-9]+_[0-9]+:[0-9]+:[0-9]+$/ ) {
          my ( $date_part, $time_part ) = split( /_/, $time );
          my ( $year, $month, $day ) = split( /-/, $date_part );
          my ( $hour, $min,   $sec ) = split( /:/, $time_part );

          my $timestamp = timelocal( $sec, $min, $hour, $day, $month - 1, $year );
          $stat_date = $time;
          $stat_time = $timestamp;
        }

        if ( defined $stat_time && Xormon::isdigit($stat_time) ) {
          $data{'perf'}{$stat_time}{'ARRAY'}{'date'} = $stat_date;
          my @values    = split( ",", $line );
          my $value_idx = 0;
          foreach my $value (@values) {
            chomp $value;
            $data{'perf'}{$stat_time}{'ARRAY'}{ $header[$value_idx] } = $value;
            $value_idx++;
          }
        }
      }
    }
  }

  return 1;
}

sub ctrlr_info {
  my $array_name = $cfg->{'arrayName'};
  my @controller_list = run_ssh_cmd("ctrlr --list --array $array_name");

  my $controllers_cmd_line = "";
  my $controller_idx       = 0;
  if (@controller_list) {
    foreach my $line (@controller_list) {
      chomp $line;
      if ( $line =~ m/^[A-Z]\s+/ ) {
        $controller_idx++;

        my ( $controller_id, undef ) = split( /\s+/, $line );

        $controllers_cmd_line .= "echo \'$delimeter $controller_id $delimeter\'\; ctrlr --info $controller_id --array $array_name\;";
      }
    }
  }

  if ( $controller_idx > 0 ) {
    my @controllers_info = run_ssh_cmd("$controllers_cmd_line");
    my $controller_id    = "";
    foreach my $line_l (@controllers_info) {
      chomp $line_l;
      $line_l =~ s/^\s+//;

      if ( $line_l =~ m/^$delimeter/ && $line_l =~ m/$delimeter$/ ) {
        $controller_id = $line_l;
        $controller_id =~ s/$delimeter//g;
        $controller_id =~ s/^\s+//g;
        $controller_id =~ s/\s+$//g;
        next;
      }
      if ( $line_l =~ m/^ERROR:/ ) { next; }

      if ( defined $controller_id && $controller_id ne '' ) {
        my ( $key, $value ) = split( /:\s+/, $line_l );
        $data{'CONTROLLER'}{$controller_id}{$key} = $value;
      }
    }
  }

  return 1;
}

sub vol_info {
  my @vol_list = run_ssh_cmd("vol --list");

  my @vols_cmds;
  my $vols_cmd_line = "";
  my $vol_idx       = 0;
  if (@vol_list) {
    foreach my $line (@vol_list) {
      chomp $line;
      if ( $line =~ m/^Name\s+/ ) { next; }
      if ( $line =~ m/^---+\+/ )  { next; }
      if ( $line =~ m/^    \s+/ ) { next; }

      $vol_idx++;
      my ( $vol_name, undef ) = split( /\s\s+/, $line );

      my $act_cmd = "echo \'$delimeter $vol_name $delimeter\'\; vol --info \'$vol_name\'\;";

      if ( exists $data{'ARRAY'}{'Member of pool'} && $data{'ARRAY'}{'Member of pool'} ne '' && $data{'ARRAY'}{'Member of pool'} ne 'default' ) {

        # ERROR: Failed to get volume info. Found 2 volumes named XY in pools: XY1-pool, XY2-pool. Use --pool option to specify pool.
        $act_cmd = "echo \'$delimeter $vol_name $delimeter\'\; vol --info \'$vol_name\' --pool \'$data{'ARRAY'}{'Member of pool'}\'\;";
      }

      if ( length($vols_cmd_line) + length($act_cmd) > 130000 ) {
        push( @vols_cmds, $vols_cmd_line );
        $vols_cmd_line = "";
      }

      $vols_cmd_line .= $act_cmd;
    }
    if ( defined $vols_cmd_line && $vols_cmd_line ne '' ) {
      push( @vols_cmds, $vols_cmd_line );
    }
  }

  if ( $vol_idx > 0 ) {
    foreach my $cmd_act (@vols_cmds) {
      my @vols_info = run_ssh_cmd("$cmd_act");
      my $vol_name  = "";
      foreach my $line_l (@vols_info) {
        chomp $line_l;
        $line_l =~ s/^\s+//;

        if ( $line_l =~ m/^$delimeter/ && $line_l =~ m/$delimeter$/ ) {
          $vol_name = $line_l;
          $vol_name =~ s/$delimeter//g;
          $vol_name =~ s/^\s+//g;
          $vol_name =~ s/\s+$//g;
          next;
        }
        if ( $line_l =~ m/^ERROR:/ ) { next; }

        if ( defined $vol_name && $vol_name ne '' ) {
          my ( $key, $value ) = split( /:\s+/, $line_l );
          $data{'VOLUMES'}{$vol_name}{$key} = $value;

          # HOSTS
          if ( $key eq "Initiator Group" && defined $value && $value ne '' && $value ne '*' ) {
            $data{'HOSTS'}{'Initiator Group'}{$value}{'vols'}{$vol_name} = $vol_name;
          }
        }
      }
    }
  }

  return 1;
}

sub vol_stats {
  my $vol_count = keys %{ $data{'VOLUMES'} };

  my $pool_info = "";
  if ( exists $data{'ARRAY'}{'Member of pool'} && $data{'ARRAY'}{'Member of pool'} ne '' && $data{'ARRAY'}{'Member of pool'} ne 'default' ) {

    # ERROR: Failed to get volume info. Found 2 volumes named XY in pools: XY1-pool, XY2-pool. Use --pool option to specify pool.
    $pool_info = "--pool \'$data{'ARRAY'}{'Member of pool'}\'";
  }

  if ( $vol_count < 255 ) {
    my @vol_stats = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --vol all $pool_info");

    if ( ( grep {/volumes is invalid when there are more than 255 volumes/} @vol_stats ) > 0 ) {
      my @list = set_vol_list2stats();
      foreach my $line (@list) {
        my @vol_stats = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --vol $line $pool_info");
        parse_vol_stats( \@vol_stats );
      }
    }
    else {
      parse_vol_stats( \@vol_stats );
    }
  }
  else {
    my @list = set_vol_list2stats();
    foreach my $line (@list) {
      my @vol_stats = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --vol $line $pool_info");
      parse_vol_stats( \@vol_stats );
    }
  }
  #
  # volume latency
  #
  my @vol_lat_cmds;
  my $vol_lat_cmd_line = "";
  my @vols_lat;
  if ( defined $data{'VOLUMES'} && $data{'VOLUMES'} ne '' ) {
    foreach my $vol ( sort keys( %{ $data{'VOLUMES'} } ) ) {
      chomp $vol;

      my $act_cmd = "echo \'$delimeter VOLUME_SECTION_START $vol $delimeter\'\; stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --perf $vol --latency\; echo \'$delimeter VOLUME_SECTION_END $vol $delimeter\'\;";

      if ( exists $data{'ARRAY'}{'Member of pool'} && $data{'ARRAY'}{'Member of pool'} ne '' && $data{'ARRAY'}{'Member of pool'} ne 'default' ) {

        # ERROR: Failed to get volume info. Found 2 volumes named XY in pools: XY1-pool, XY2-pool. Use --pool option to specify pool.
        $act_cmd = "echo \'$delimeter VOLUME_SECTION_START $vol $delimeter\'\; stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --perf $vol --pool \'$data{'ARRAY'}{'Member of pool'}\' --latency\; echo \'$delimeter VOLUME_SECTION_END $vol $delimeter\'\;";
      }

      # test if cmd length is not over limit
      my $tot_cmd_length = length($vol_lat_cmd_line);
      my $act_cmd_length = length($act_cmd);

      if ( $tot_cmd_length + $act_cmd_length > 130000 ) {
        push( @vol_lat_cmds, $vol_lat_cmd_line );
        $vol_lat_cmd_line = "";
      }

      $vol_lat_cmd_line .= $act_cmd;
    }
    if ( defined $vol_lat_cmd_line && $vol_lat_cmd_line ne '' ) {
      push( @vol_lat_cmds, $vol_lat_cmd_line );
    }

    # run all latency commands and push their outputs to single array
    foreach my $cmd (@vol_lat_cmds) {
      my @out = run_ssh_cmd("$cmd");
      push( @vols_lat, @out );
    }

    my @vol_lat;
    my $vol_section = 0;
    my $vol_act     = "";
    foreach my $line (@vols_lat) {
      chomp $line;
      $line =~ s/^\s+//;

      if ( $line =~ m/^ERROR:/ ) { next; }

      if ( $line =~ m/^$delimeter VOLUME_SECTION_START/ ) {
        $vol_section = 1;
        $vol_act     = $line;
        $vol_act =~ s/^$delimeter VOLUME_SECTION_START //;
        $vol_act =~ s/ $delimeter$//;
        next;
      }
      if ( $line =~ m/^$delimeter VOLUME_SECTION_END $vol_act $delimeter$/ ) {
        $vol_section = 0;

        # parse csv lines for actual volume
        if ( exists $data{'VOLUMES'}{$vol_act} ) {
          vol_latency( "$vol_act", \@vol_lat );
        }
        @vol_lat = ();
        next;
      }
      if ( $vol_section == 1 ) {
        push( @vol_lat, "$line\n" );
      }

    }
  }

  return 1;
}

sub parse_vol_stats {
  my $stats     = shift;
  my $storage_name = $cfg->{'hostalias'};
  my @vol_stats = @{$stats};

  # if ( $debug == 11 ) {
  #   if ( !-d "$wrkdir/$storage_name/IOSTATS" ) {
  #     mkdir( "$wrkdir/$storage_name/IOSTATS", 0755 ) || Xormon::error( "Cannot mkdir $wrkdir/$storage_name/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  #   }
  #   my $file_out = "$wrkdir/$storage_name/IOSTATS/" . time() . "-vol_stats.txt";
  #   open( my $file, ">$file_out" ) || error( "Couldn't open file $file_out $!" . __FILE__ . ":" . __LINE__ ) && exit;
  #   print $file localtime() . " : $stats_start_time - $stats_end_time : " . localtime($stats_start_time) . " - " . localtime($stats_end_time) . "\n";
  #   print $file "######################################################################\n";
  #   print $file @vol_stats;
  #   close($file);
  # }

  if (@vol_stats) {
    my @header;
    my $stat_date  = "";
    my $stat_time  = "";
    my $perf_lines = 0;
    foreach my $line (@vol_stats) {
      chomp $line;

      if ( $line =~ m/try increasing interval to/ )                  { next; }
      if ( $line =~ m/ERROR: Failed to find statistics for volume/ ) { next; }

      if ( $line =~ m/^yyyy-mm-dd,hh:mm:ss,/ ) {
        $perf_lines = 1;
        $line =~ s/^yyyy-mm-dd,hh:mm:ss,/yyyy-mm-dd_hh:mm:ss,/;
        @header = split( ",", $line );
        next;
      }
      if ( $perf_lines == 1 ) {
        my ( $time, $vol, undef ) = split( /,/, $line );

        if ( defined $time && $time =~ m/^[0-9][0-9][0-9][0-9]-[0-9]+-[0-9]+_[0-9]+:[0-9]+:[0-9]+$/ ) {
          my ( $date_part, $time_part ) = split( /_/, $time );
          my ( $year, $month, $day ) = split( /-/, $date_part );
          my ( $hour, $min,   $sec ) = split( /:/, $time_part );

          my $timestamp = timelocal( $sec, $min, $hour, $day, $month - 1, $year );
          $stat_date = $time;
          $stat_time = $timestamp;
        }

        if ( defined $stat_time && Xormon::isdigit($stat_time) ) {
          $data{'perf'}{$stat_time}{'VOLUMES'}{$vol}{'date'} = $stat_date;
          my @values    = split( ",", $line );
          my $value_idx = 0;
          foreach my $value (@values) {
            chomp $value;
            $data{'perf'}{$stat_time}{'VOLUMES'}{$vol}{ $header[$value_idx] } = $value;
            $value_idx++;
          }
        }
      }
    }
  }

  return 1;
}

sub vol_latency {
  my $vol     = shift;
  my $array   = shift;
  my @vol_lat = @{$array};

  #my @vol_lat = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --perf $vol --latency");
  if (@vol_lat) {
    my @header;
    my $stat_date  = "";
    my $stat_time  = "";
    my $perf_lines = 0;
    foreach my $line (@vol_lat) {
      chomp $line;

      if ( $line =~ m/^yyyy-mm-dd,hh:mm:ss,/ ) {
        $perf_lines = 1;
        $line =~ s/^yyyy-mm-dd,hh:mm:ss,/yyyy-mm-dd_hh:mm:ss,/;
        @header = split( ",", $line );
        next;
      }
      if ( $perf_lines == 1 ) {
        my ( $time, $stat_name, undef ) = split( /,/, $line );

        if ( defined $time && $time =~ m/^[0-9][0-9][0-9][0-9]-[0-9]+-[0-9]+_[0-9]+:[0-9]+:[0-9]+$/ ) {
          my ( $date_part, $time_part ) = split( /_/, $time );
          my ( $year, $month, $day ) = split( /-/, $date_part );
          my ( $hour, $min,   $sec ) = split( /:/, $time_part );

          my $timestamp = timelocal( $sec, $min, $hour, $day, $month - 1, $year );
          $stat_date = $time;
          $stat_time = $timestamp;
        }

        if ( defined $stat_time && Xormon::isdigit($stat_time) ) {
          $data{'perf'}{$stat_time}{'VOLUMES'}{$vol}{'latency'}{$stat_name}{'date'} = $stat_date;
          my @values    = split( ",", $line );
          my $value_idx = 0;
          foreach my $value (@values) {
            chomp $value;
            $data{'perf'}{$stat_time}{'VOLUMES'}{$vol}{'latency'}{$stat_name}{ $header[$value_idx] } = $value;
            $value_idx++;
          }
        }

      }
    }
  }

  return 1;
}

sub disk_info {
  my $array_name = $cfg->{'arrayName'};
  my @disk_list = run_ssh_cmd("disk --list --array $array_name");

  my $disk_idx = 0;
  if (@disk_list) {
    foreach my $line (@disk_list) {
      chomp $line;
      if ( $line =~ m/^\s*[0-9]+\s+/ || $line =~ m/^\s*[0-9]+\.[A-B]\s+/ ) {
        $disk_idx++;

        $line =~ s/^\s+//;

        my ( $disk_id, $serial, $type, $size, undef ) = split( /\s+/, $line );

        my $controller   = "";
        my $disk_id_orig = $disk_id;

        if ( $disk_id =~ m/\.[A-B]$/ ) {
          $controller = $disk_id;
          $controller =~ s/^[0-9]+\.//;
          $disk_id    =~ s/\.[A-B]$//;
        }
        else {
          # find active Controller
          if ( defined $data{'CONTROLLER'} && $data{'CONTROLLER'} ne '' ) {
            foreach my $ctrl ( sort keys( %{ $data{'CONTROLLER'} } ) ) {
              if ( defined $data{'CONTROLLER'}{$ctrl}{'State'} && $data{'CONTROLLER'}{$ctrl}{'State'} eq "active" ) {
                $controller = $ctrl;
                last;
              }
            }
          }
        }
        if ( !defined $controller || $controller eq '' ) {
          error( "Controller not found!" . __FILE__ . ":" . __LINE__ );
          $controller = "NA";
        }

        if ( defined $type && $type ne '' ) {
          $data{'DISKS'}{$controller}{$disk_id}{'type'} = $type;
        }
        if ( defined $serial && $serial ne '' ) {
          $data{'DISKS'}{$controller}{$disk_id}{'serial'} = $serial;
        }
        if ( defined $size && Xormon::isdigit($size) ) {
          $data{'DISKS'}{$controller}{$disk_id}{'size'} = $size;
        }

        # Health status
        #
        # 1 /dev/sdf             HDD      39.73 in use  okay            VM-004531 A.0
        # Disk hardware state. Disk state. Possible values: 'valid', 'in use', 'failed', absent', 'removed', 'void', 't_fail', 'foreign'.
        # RAID status for the disk (N/A, okay, resynchronizing, spare, faulty). Disk RAID state. Possible values: 'N/A', 'okay', 'resynchronizing', 'spare', 'faulty'.
        my $hw_state   = "";
        my $raid_state = "";
        my $state_line = $line;

        $state_line =~ s/^$disk_id_orig\s+//;
        $state_line =~ s/^$serial\s+//;
        $state_line =~ s/^$type\s+//;
        $state_line =~ s/^$size\s+//;

        if ( $state_line =~ m/^in use\s+/ ) {
          $hw_state = "in use";

          $state_line =~ s/^in use\s+//;
          ( $raid_state, undef ) = split( /\s+/, $state_line );
        }
        else {
          ( $hw_state, $raid_state, undef ) = split( /\s+/, $state_line );
        }

        #print "$disk_id : $hw_state : $raid_state\n";

        $data{'DISKS'}{$controller}{$disk_id}{'Disk hardware state'} = $hw_state;
        $data{'DISKS'}{$controller}{$disk_id}{'RAID status'}         = $raid_state;
      }
    }
  }

  return 1;
}

sub disk_stats {
  my $array_name = $cfg->{'arrayName'};
  my @disk_stats = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --disk all");
  if (@disk_stats) {
    my @header;
    my $stat_date  = "";
    my $stat_time  = "";
    my $perf_lines = 0;
    foreach my $line (@disk_stats) {
      chomp $line;

      if ( $line =~ m/try increasing interval to/ ) { next; }

      if ( $line =~ m/^yyyy-mm-dd,hh:mm:ss,/ ) {
        $perf_lines = 1;
        $line =~ s/^yyyy-mm-dd,hh:mm:ss,/yyyy-mm-dd_hh:mm:ss,/;
        @header = split( ",", $line );
        next;
      }
      if ( $perf_lines == 1 ) {
        my ( $time, $disk_id, undef ) = split( /,/, $line );

        if ( defined $time && $time =~ m/^[0-9][0-9][0-9][0-9]-[0-9]+-[0-9]+_[0-9]+:[0-9]+:[0-9]+$/ ) {
          my ( $date_part, $time_part ) = split( /_/, $time );
          my ( $year, $month, $day ) = split( /-/, $date_part );
          my ( $hour, $min,   $sec ) = split( /:/, $time_part );

          my $timestamp = timelocal( $sec, $min, $hour, $day, $month - 1, $year );
          $stat_date = $time;
          $stat_time = $timestamp;
        }

        if ( defined $stat_time && Xormon::isdigit($stat_time) ) {
          if ( $disk_id !~ m/^$array_name:/ ) { next; }    # only this array disks
          $disk_id =~ s/^$array_name://;

          my $controller   = "";
          my $disk_id_orig = $disk_id;

          if ( $disk_id =~ m/\.[A-B]$/ ) {
            $controller = $disk_id;
            $controller =~ s/^[0-9]+\.//;
            $disk_id    =~ s/\.[A-B]$//;
          }
          else {
            # find active Controller
            if ( defined $data{'CONTROLLER'} && $data{'CONTROLLER'} ne '' ) {
              foreach my $ctrl ( sort keys( %{ $data{'CONTROLLER'} } ) ) {
                if ( defined $data{'CONTROLLER'}{$ctrl}{'State'} && $data{'CONTROLLER'}{$ctrl}{'State'} eq "active" ) {
                  $controller = $ctrl;
                  last;
                }
              }
            }
          }
          if ( !defined $controller || $controller eq '' ) {
            $logger->error( "Controller not found!" . __FILE__ . ":" . __LINE__ );
            $controller = "NA";
          }

          $data{'perf'}{$stat_time}{'DISKS'}{$controller}{$disk_id}{'date'} = $stat_date;
          my @values    = split( ",", $line );
          my $value_idx = 0;
          foreach my $value (@values) {
            chomp $value;
            $data{'perf'}{$stat_time}{'DISKS'}{$controller}{$disk_id}{ $header[$value_idx] } = $value;
            $value_idx++;
          }
        }
      }
    }
  }

  return 1;
}

sub net_stats {
  my $array_name = $cfg->{'arrayName'};
  my @net_stats = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --net all");
  # print "stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --net all\n";
  # print "@net_stats\n";
    if ( grep( /--/, @net_stats ) ) {
      print "SPATNY @net_stats\n";
    }
  if (@net_stats) {
    my @header;
    my $stat_date  = "";
    my $stat_time  = "";
    my $perf_lines = 0;
    foreach my $line (@net_stats) {
      chomp $line;

      if ( $line =~ m/try increasing interval to/ ) { next; }

      if ( $line =~ m/^yyyy-mm-dd,hh:mm:ss,/ ) {
        $perf_lines = 1;
        $line =~ s/^yyyy-mm-dd,hh:mm:ss,/yyyy-mm-dd_hh:mm:ss,/;
        @header = split( ",", $line );
        next;
      }
      if ( $perf_lines == 1 ) {
        my ( $time, $net, undef ) = split( /,/, $line );

        if ( defined $time && $time =~ m/^[0-9][0-9][0-9][0-9]-[0-9]+-[0-9]+_[0-9]+:[0-9]+:[0-9]+$/ ) {
          my ( $date_part, $time_part ) = split( /_/, $time );
          my ( $year, $month, $day ) = split( /-/, $date_part );
          my ( $hour, $min,   $sec ) = split( /:/, $time_part );

          my $timestamp = timelocal( $sec, $min, $hour, $day, $month - 1, $year );
          $stat_date = $time;
          $stat_time = $timestamp;
        }

        if ( defined $stat_time && Xormon::isdigit($stat_time) ) {
          if ( $net !~ m/^$array_name:/ ) { next; }    # only this array nets
          $net =~ s/^$array_name://;

          $data{'perf'}{$stat_time}{'PORTS'}{'NET'}{$net}{'date'} = $stat_date;
          $data{'PORTS'}{'NET'}{$net}{'date'} = $stat_date;
          my @values    = split( ",", $line );
          my $value_idx = 0;
          foreach my $value (@values) {
            chomp $value;
            $data{'perf'}{$stat_time}{'PORTS'}{'NET'}{$net}{ $header[$value_idx] } = $value;
            $value_idx++;
          }
        }
      }
    }
  }

  return 1;
}

sub fc_stats {
  my $array_name = $cfg->{'arrayName'};
  my $now_time = time;
  my $disable_fc_collection = 0;

  # if FC adapters are not used, then try to collect data for them only once per day
  # this solution reduces error messages logging

  if ( exists $service_data_in->{fc_port_disabled} && Xormon::isdigit($service_data_in->{fc_port_disabled}) ) {
    my $last_fc_time = $service_data_in->{fc_port_disabled};
    my $dif_time = $now_time - $last_fc_time;
    if ( $dif_time < 86399 ) {
      $service_data_out->{fc_port_disabled} = $last_fc_time;
      return 1;
    }
  }

  my @fc_stats = run_ssh_cmd("stats --from $stats_start_time --to $stats_end_time --interval $sample_rate --show_time --csv --fc all");
  if (@fc_stats) {
    my @header;
    my $stat_date  = "";
    my $stat_time  = "";
    my $perf_lines = 0;
    foreach my $line (@fc_stats) {
      chomp $line;

      if ( $line =~ m/ERROR: no available fcs matching criteria/ ) { $disable_fc_collection = 1; }

      if ( $line =~ m/try increasing interval to/ ) { next; }

      if ( $line =~ m/^yyyy-mm-dd,hh:mm:ss,/ ) {
        $perf_lines = 1;
        $line =~ s/^yyyy-mm-dd,hh:mm:ss,/yyyy-mm-dd_hh:mm:ss,/;
        @header = split( ",", $line );
        next;
      }
      if ( $perf_lines == 1 ) {
        my ( $time, $fc, undef ) = split( /,/, $line );

        if ( defined $time && $time =~ m/^[0-9][0-9][0-9][0-9]-[0-9]+-[0-9]+_[0-9]+:[0-9]+:[0-9]+$/ ) {
          my ( $date_part, $time_part ) = split( /_/, $time );
          my ( $year, $month, $day ) = split( /-/, $date_part );
          my ( $hour, $min,   $sec ) = split( /:/, $time_part );

          my $timestamp = timelocal( $sec, $min, $hour, $day, $month - 1, $year );
          $stat_date = $time;
          $stat_time = $timestamp;
        }

        if ( defined $stat_time && Xormon::isdigit($stat_time) ) {
          if ( $fc !~ m/^$array_name:/ ) { next; }    # only this array nets
          $fc =~ s/^$array_name://;

          $data{'perf'}{$stat_time}{'PORTS'}{'FC'}{$fc}{'date'} = $stat_date;
          $data{'PORTS'}{'FC'}{$fc}{'date'} = $stat_date;
          my @values    = split( ",", $line );
          my $value_idx = 0;
          foreach my $value (@values) {
            chomp $value;
            $data{'perf'}{$stat_time}{'PORTS'}{'FC'}{$fc}{ $header[$value_idx] } = $value;
            $value_idx++;
          }
        }
      }
    }
  }

  if ( $disable_fc_collection == 1 ) {
    $service_data_out->{fc_port_disabled} = $now_time;
    $logger->error("FC adapters are not used, data collection for them will be disabled for the next 24 hours!");
  }

  return 1;
}


sub events {
  my $array_name = $cfg->{'arrayName'};

  # get critical events (last 90 days)
  my %events;
  my @events;
  my $act_timestamp  = time();
  my $from_timestamp = $act_timestamp - ( 90 * 3600 * 24 );

  # --from 2017-11-7,14:25
  my ( $sec_s, $min_s, $hour_s, $day_s, $month_s, $year_s, $wday_s, $yday_s, $isdst_s ) = localtime($from_timestamp);
  my $from_date = sprintf( "%4d-%02d-%02d,%02d:%02d", $year_s + 1900, $month_s + 1, $day_s, $hour_s, $min_s );

  my @events_list = run_ssh_cmd("alert --list --array $array_name --severity critical --from $from_date");

  #----------+--------+--------------------+------+-----------+----------+---------
  #ID         Severity Time                 Type   Category    Array      Detail
  #----------+--------+--------------------+------+-----------+----------+---------
  #      2953 WARNING  Oct 29 2017 16:35:41   2028 hardware    VM-XXX  Controller A rebooted unexpectedly. Contact Nimble Storage Support.
  foreach my $line (@events_list) {
    chomp $line;

    if ( $line =~ m/^-----/ ) { next; }
    if ( $line =~ m/^ID\s+/ ) { next; }
    if ( $line =~ m/^INFO/ )  { next; }

    $line =~ s/^\s+//g;
    $line =~ s/\s+$//g;

    my $id       = "";
    my $severity = "";
    my $month    = "";
    my $day      = "";
    my $year     = "";
    my $time     = "";
    my $type     = "";
    my $category = "";
    my $array    = "";
    my $detail   = "";

    ( $id, $severity, $month, $day, $year, $time, $type, $category, $array, undef ) = split( /\s+/, $line );
    $detail = $line;
    $detail =~ s/^$id\s+//;
    $detail =~ s/^$severity\s+//;
    $detail =~ s/^$month\s+//;
    $detail =~ s/^$day\s+//;
    $detail =~ s/^$year\s+//;
    $detail =~ s/^$time\s+//;
    $detail =~ s/^$type\s+//;
    $detail =~ s/^$category\s+//;
    $detail =~ s/^$array\s+//;

    my %event;
    $event{id}       = $id;
    $event{severity} = $severity;
    $event{date}     = "$year-$month-$day $time";
    $event{type}     = $type;
    $event{category} = $category;
    $event{array}    = $array;
    $event{detail}   = $detail;
    push(@events,\%event);
  }

  $events{ $cfg->{'hostcfg_id'} }{events} = encode_json(\@events);


  # save events
  my $rc = $xormon->saveConfJSON(\%events, 1);
  my $ret_status;
  # status code
  if ($rc eq "201") {
    $ret_status = "successful";
  } elsif ($rc eq "401") {
    $ret_status = "unauthorized";
  } else {
    $ret_status = "error";
  }
  $logger->info("Save events table: $ret_status");

  # print "EVENTS\n";
  # print Dumper \%events;
  return 1;
}

sub health_status {

  my $array_name = $cfg->{'arrayName'};
  my $act_timestamp  = time();
  my $main_state     = "ok";
  my @error_message  = ();

  my $array_status = "";

  my %hs_data;
  my $ret_status;


  # Array status
  #
  if ( defined $data{'ARRAY'}{'Status'} && $data{'ARRAY'}{'Status'} ne '' ) {
    $array_status = $data{'ARRAY'}{'Status'};
    my %array = ('array_name' => $array_name, 'array_status' => $array_status);
    push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{hs_array} }, \%array );
    if ( $array_status ne "reachable" ) {
      $main_state   = "error";
      my $error_line = "STATUS ARRAY  :  $array_status";
      push( @error_message, "$error_line\n" );
    }
  }

  # Controller status
  #
  if ( defined $data{'CONTROLLER'} && $data{'CONTROLLER'} ne '' ) {
    foreach my $controller ( sort keys( %{ $data{'CONTROLLER'} } ) ) {
      my $power_supply_st     = "";
      my $state               = "";
      my $cooling_fans        = "";
      my $system_partition_st = "";
      my $temperature_sensors = "";
      my $serial              = "";

      if ( defined $data{'CONTROLLER'}{$controller}{'Serial number'} && $data{'CONTROLLER'}{$controller}{'Serial number'} ne '' ) {
        $serial = $data{'CONTROLLER'}{$controller}{'Serial number'};
      }

      if ( defined $data{'CONTROLLER'}{$controller}{'State'} && $data{'CONTROLLER'}{$controller}{'State'} ne '' ) {
        $state = $data{'CONTROLLER'}{$controller}{'State'};
        if ( $state ne "active" && $state ne "standby" ) {
          $main_state   = "error";
        }
      }
      if ( defined $data{'CONTROLLER'}{$controller}{'Power supply'} && $data{'CONTROLLER'}{$controller}{'Power supply'} ne '' ) {
        $power_supply_st = $data{'CONTROLLER'}{$controller}{'Power supply'};
        if ( $power_supply_st ne "OK" ) {
          $main_state   = "error";
        }
      }
      if ( defined $data{'CONTROLLER'}{$controller}{'Cooling fans'} && $data{'CONTROLLER'}{$controller}{'Cooling fans'} ne '' ) {
        $cooling_fans = $data{'CONTROLLER'}{$controller}{'Cooling fans'};
        if ( $cooling_fans ne "OK" ) {
          $main_state   = "error";
        }
      }

      if ( defined $data{'CONTROLLER'}{$controller}{'System partition status'} && $data{'CONTROLLER'}{$controller}{'System partition status'} ne '' ) {
        $system_partition_st = $data{'CONTROLLER'}{$controller}{'System partition status'};
        if ( $system_partition_st ne "OK" ) {
          $main_state   = "error";
        }
      }
      if ( defined $data{'CONTROLLER'}{$controller}{'Temperature sensors'} && $data{'CONTROLLER'}{$controller}{'Temperature sensors'} ne '' ) {
        $temperature_sensors = $data{'CONTROLLER'}{$controller}{'Temperature sensors'};
        if ( $system_partition_st ne "OK" ) {
          $main_state   = "error";
        }
      }
      if ( $system_partition_st ne "OK" || $system_partition_st ne "OK" || $cooling_fans ne "OK" || $power_supply_st ne "OK" || ( $state ne "active" && $state ne "standby" ) ) {
        my $error_line = "$controller  $serial  $state  $power_supply_st  $cooling_fans  $system_partition_st  $temperature_sensors";
        push( @error_message, "$error_line\n" );
      }
      my %ctrlr;
      $ctrlr{id}                  = $controller;
      $ctrlr{serial}              = $serial;
      $ctrlr{state}               = $state;
      $ctrlr{power_supply}        = $power_supply_st;
      $ctrlr{cooling_fans}        = $cooling_fans;
      $ctrlr{system_partition}    = $system_partition_st;
      $ctrlr{temperature_sensors} = $temperature_sensors;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{hs_ctrlrs} }, \%ctrlr );
    }
  }

  if ( defined $data{'DISKS'} && $data{'DISKS'} ne '' ) {
    foreach my $disk_controller ( sort keys( %{ $data{'DISKS'} } ) ) {
      if ( defined $data{'DISKS'}{$disk_controller} && $data{'DISKS'}{$disk_controller} ne '' ) {
        foreach my $disk_id ( sort { $a <=> $b } keys( %{ $data{'DISKS'}{$disk_controller} } ) ) {

          my $disk_label = $disk_id . "." . $disk_controller;

          my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "DRIVE" . $disk_label); $hash_key = lc($hash_key);

          my $hw_state   = "";
          my $raid_state = "";
          my $serial     = "";

          if ( defined $data{'DISKS'}{$disk_controller}{$disk_id}{'serial'} && $data{'DISKS'}{$disk_controller}{$disk_id}{'serial'} ne '' ) {
            $serial = $data{'DISKS'}{$disk_controller}{$disk_id}{'serial'};
          }
          if ( defined $data{'DISKS'}{$disk_controller}{$disk_id}{'Disk hardware state'} && $data{'DISKS'}{$disk_controller}{$disk_id}{'Disk hardware state'} ne '' ) {
            $hw_state = $data{'DISKS'}{$disk_controller}{$disk_id}{'Disk hardware state'};
            if ( $hw_state eq "failed" || $hw_state eq "t_fail" ) {
              $main_state   = "error";
            }
          }
          if ( defined $data{'DISKS'}{$disk_controller}{$disk_id}{'RAID status'} && $data{'DISKS'}{$disk_controller}{$disk_id}{'RAID status'} ne '' ) {
            $raid_state = $data{'DISKS'}{$disk_controller}{$disk_id}{'RAID status'};
            if ( $raid_state eq "faulty" ) {
              $main_state   = "error";
            }
          }
          if ( ( $hw_state eq "failed" || $hw_state eq "t_fail" ) || $raid_state eq "faulty" ) {
            my $error_line = "$disk_id  $disk_controller  $serial  $hw_state  $raid_state";
            push( @error_message, "$error_line\n" );
          }

          my %drive;
          $drive{id}         = $disk_id;
          $drive{ctrlr}      = $disk_controller;
          $drive{serial}     = $serial;
          $drive{hw_state}   = $hw_state;
          $drive{raid_state} = $raid_state;
          $drive{hash_id}    = $hash_key;
          push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{hs_drives} }, \%drive );
        }
      }
    }
  }

  # Global Health status part
  #

  $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{last_update_time} = localtime();

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  my @hs = (
    {
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $main_state,
      "data" => join("\n", @error_message),
    }
  );

  #print Dumper \%hs_data;


  # # send statuses
  my $rc = $xormon->saveStatus(\@hs);
  # status code
  if ($rc eq "201") {
    $ret_status = "successful";
  } elsif ($rc eq "401") {
    $ret_status = "unauthorized";
  } else {
    $ret_status = "error";
  }
  $logger->info("Save health status: $ret_status");

  # # save HS data
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);
  # status code
  if ($rc_hs eq "201") {
    $ret_status = "successful";
  } elsif ($rc_hs eq "401") {
    $ret_status = "unauthorized";
  } else {
    $ret_status = "error";
  }
  $logger->info("Save health status table: $ret_status");

  return 1;
}



sub print_conf {
  my %conf;

  if ( defined $data{'ARRAY'} && $data{'ARRAY'} ne "" ) {
    ## Device
    my $st_uuid = $cfg->{'hostcfg_id'};
    #my $hash_key = Xormon::create_hash_key($st_uuid . "array_total"); $hash_key = lc($hash_key);

    if ( defined $data{'ARRAY'}{'Group Management IP'} && $data{'ARRAY'}{'Group Management IP'} ne '' ) {
      $conf{$st_uuid}{'ip'} = $data{'ARRAY'}{'Group Management IP'};
    }
    if ( defined $data{'ARRAY'}{'Serial'} && $data{'ARRAY'}{'Serial'} ne '' ) {
      $conf{$st_uuid}{'serial'} = $data{'ARRAY'}{'Serial'};
    }
    if ( defined $data{'ARRAY'}{'Model'} && $data{'ARRAY'}{'Model'} ne '' ) {
      $conf{$st_uuid}{'model'} = $data{'ARRAY'}{'Model'};
    }
    if ( defined $data{'ARRAY'}{'Version'} && $data{'ARRAY'}{'Version'} ne '' ) {
      $conf{$st_uuid}{'version'} = $data{'ARRAY'}{'Version'};
    }
    if ( defined $data{'ARRAY'}{'Extended Model'} && $data{'ARRAY'}{'Extended Model'} ne '' ) {
      $conf{$st_uuid}{'extended_model'} = $data{'ARRAY'}{'Extended Model'};
    }
      if ( defined $data{'ARRAY'}{'Supported configuration'} && $data{'ARRAY'}{'Supported configuration'} ne '' ) {
      $conf{$st_uuid}{'supported_configuration'} = $data{'ARRAY'}{'Supported configuration'};
    }
    if ( defined $data{'ARRAY'}{'Snapshot space reduction'} && $data{'ARRAY'}{'Snapshot space reduction'} ne '' ) {
      $conf{$st_uuid}{'snapshot_space_reduction'} = $data{'ARRAY'}{'Snapshot space reduction'};
    }
    if ( defined $data{'ARRAY'}{'All-Flash'} && $data{'ARRAY'}{'All-Flash'} ne '' ) {
      $conf{$st_uuid}{'all_flash'} = $data{'ARRAY'}{'All-Flash'};
    }
    if ( defined $data{'ARRAY'}{'Volume compression'} && $data{'ARRAY'}{'Volume compression'} ne '' ) {
      $conf{$st_uuid}{'volume_compression'} = $data{'ARRAY'}{'Volume compression'};
    }
    if ( defined $data{'ARRAY'}{'Array name'} && $data{'ARRAY'}{'Array name'} ne '' ) {
      $conf{$st_uuid}{'node_name'} = $data{'ARRAY'}{'Array name'};
    }
    if ( defined $data{'ARRAY'}{'Member of pool'} && $data{'ARRAY'}{'Member of pool'} ne '' ) {
      $conf{$st_uuid}{'member_of_pool'} = $data{'ARRAY'}{'Member of pool'};
    }
    if ( defined $data{'ARRAY'}{'Group ID'} && $data{'ARRAY'}{'Group ID'} ne '' ) {
      $conf{$st_uuid}{'group_id'} = $data{'ARRAY'}{'Group ID'};
    }
    if ( defined $data{'ARRAY'}{'Status'} && $data{'ARRAY'}{'Status'} ne '' ) {
      $conf{$st_uuid}{'status'} = $data{'ARRAY'}{'Status'};
    }
    if ( defined $data{'ARRAY'}{'Snapshot compression'} && $data{'ARRAY'}{'Snapshot compression'} ne '' ) {
      $conf{$st_uuid}{'snapshot_compression'} = $data{'ARRAY'}{'Snapshot compression'};
    }
    if ( defined $data{'ARRAY'}{'Link-local IP address'} && $data{'ARRAY'}{'Link-local IP address'} ne '' ) {
      $conf{$st_uuid}{'link_local_address'} = $data{'ARRAY'}{'Link-local IP address'};
    }

    # Fake Pool
    my $cap_tot   = "";
    my $cap_free  = "";
    my $cap_used  = "";

    my $pool_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "array_total"); $pool_key = lc($pool_key);

    if ( defined $data{'ARRAY'}{'Total array usage (MB)'} && $data{'ARRAY'}{'Total array usage (MB)'} ne '' && Xormon::isdigit( $data{'ARRAY'}{'Total array usage (MB)'} ) ) {
      $cap_used = Xormon::returnBytes( $data{'ARRAY'}{'Total array usage (MB)'}, "MB" );
    }
    if ( defined $data{'ARRAY'}{'Total array usage (MiB)'} && $data{'ARRAY'}{'Total array usage (MiB)'} ne '' && Xormon::isdigit( $data{'ARRAY'}{'Total array usage (MiB)'} ) ) {
      $cap_used = Xormon::returnBytes( $data{'ARRAY'}{'Total array usage (MiB)'}, "MiB" );
    }
    if ( defined $data{'ARRAY'}{'Total array capacity (MB)'} && $data{'ARRAY'}{'Total array capacity (MB)'} ne '' && Xormon::isdigit( $data{'ARRAY'}{'Total array capacity (MB)'} ) ) {
      $cap_tot = Xormon::returnBytes( $data{'ARRAY'}{'Total array capacity (MB)'}, "MB" );
    }
    if ( defined $data{'ARRAY'}{'Total array capacity (MiB)'} && $data{'ARRAY'}{'Total array capacity (MiB)'} ne '' && Xormon::isdigit( $data{'ARRAY'}{'Total array capacity (MiB)'} ) ) {
      $cap_tot = Xormon::returnBytes( $data{'ARRAY'}{'Total array capacity (MiB)'}, "MiB" );
    }
    if ( defined $data{'ARRAY'}{'Available space (MB)'} && $data{'ARRAY'}{'Available space (MB)'} ne '' && Xormon::isdigit( $data{'ARRAY'}{'Available space (MB)'} ) ) {
      $cap_free = Xormon::returnBytes( $data{'ARRAY'}{'Available space (MB)'}, "MB" );
    }
    if ( defined $data{'ARRAY'}{'Available space (MiB)'} && $data{'ARRAY'}{'Available space (MiB)'} ne '' && Xormon::isdigit( $data{'ARRAY'}{'Available space (MiB)'} ) ) {
      $cap_free = Xormon::returnBytes( $data{'ARRAY'}{'Available space (MiB)'}, "MiB" );
    }
    #print Dumper $data{'ARRAY'};
    $conf{$pool_key}{'capacity_total'}  = $cap_tot;
    $conf{$pool_key}{'capacity_used'} = $cap_used;
    $conf{$pool_key}{'capacity_free'} = $cap_free;
  }

  # Volumes
  if ( defined $data{'VOLUMES'} && $data{'VOLUMES'} ne '' ) {
    foreach my $volume ( sort keys( %{ $data{'VOLUMES'} } ) ) {

      $volume = remove_unauthorized_characters($volume);
      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "VOLUME" . $volume); $hash_key = lc($hash_key);

      #print "$volume\n";
      my $pool        = "";
      my $serial      = "";
      my $pool_name   = "Array Total";
      my $pool_id     = "array_total";

      if ( defined $data{'VOLUMES'}{$volume}{'Folder'} && $data{'VOLUMES'}{$volume}{'Folder'} ne '' && $data{'VOLUMES'}{$volume}{'Folder'} ne "N/A" ) {
        $conf{$hash_key}{'folder'} = $data{'VOLUMES'}{$volume}{'Folder'};
      }
      if ( defined $data{'VOLUMES'}{$volume}{'Size (MB)'} && $data{'VOLUMES'}{$volume}{'Size (MB)'} ne '' && $data{'VOLUMES'}{$volume}{'Size (MB)'} ne "N/A" ) {
        if ( Xormon::isdigit( $data{'VOLUMES'}{$volume}{'Size (MB)'} ) ) {
          $conf{$hash_key}{'size'} = Xormon::returnBytes( $data{'VOLUMES'}{$volume}{'Size (MB)'}, "MB" );
        }
      }
      if ( defined $data{'VOLUMES'}{$volume}{'Size (MiB)'} && $data{'VOLUMES'}{$volume}{'Size (MiB)'} ne '' && $data{'VOLUMES'}{$volume}{'Size (MiB)'} ne "N/A" ) {
        if ( Xormon::isdigit( $data{'VOLUMES'}{$volume}{'Size (MiB)'} ) ) {
          $conf{$hash_key}{'size'} = Xormon::returnBytes( $data{'VOLUMES'}{$volume}{'Size (MiB)'}, "MiB" );
        }
      }
      if ( defined $data{'VOLUMES'}{$volume}{'Pool'} && $data{'VOLUMES'}{$volume}{'Pool'} ne '' && $data{'VOLUMES'}{$volume}{'Pool'} ne "N/A" ) {
        $pool = $data{'VOLUMES'}{$volume}{'Pool'};
        $pool = remove_unauthorized_characters($pool);
        $conf{$hash_key}{'pool'}      = $data{'VOLUMES'}{$volume}{$pool};
        $conf{$hash_key}{'pool_name'} = $pool_name;
        $conf{$hash_key}{'pool_id'}   = $pool_id;
      }

      ## add lun_id for volumes
      if ( defined $data{'VOLUMES'}{$volume}{'Serial number'} && $data{'VOLUMES'}{$volume}{'Serial number'} ne '' && $data{'VOLUMES'}{$volume}{'Serial number'} ne "N/A" ) {
        $serial = $data{'VOLUMES'}{$volume}{'Serial number'};
        $conf{$hash_key}{'lun_id'} = $serial;
      }
      # if ( !defined $serial || $serial eq '' ) {
      #   Xormon::error( "Volume ID/Serial not found! Volume \"$volume\" skipped... $!" . __FILE__ . ":" . __LINE__ ) && next;
      # }
    }
  }

  # # Hosts
  # print CONF "Host name,Host id,volumes,volumes ids\n";

  # if ( defined $data{'HOSTS'}{'Initiator Group'} && $data{'HOSTS'}{'Initiator Group'} ne '' ) {
  #   foreach my $host_group ( sort keys( %{ $data{'HOSTS'}{'Initiator Group'} } ) ) {
  #     my $vols    = "";
  #     my $ids     = "";
  #     my $vol_idx = 0;
  #     if ( defined $data{'HOSTS'}{'Initiator Group'}{$host_group}{'vols'} && $data{'HOSTS'}{'Initiator Group'}{$host_group}{'vols'} ne '' ) {
  #       foreach my $vol ( sort keys( %{ $data{'HOSTS'}{'Initiator Group'}{$host_group}{'vols'} } ) ) {
  #         my $serial = "";
  #         if ( defined $data{'VOLUMES'}{$vol}{'Serial number'} && $data{'VOLUMES'}{$vol}{'Serial number'} ne '' && $data{'VOLUMES'}{$vol}{'Serial number'} ne "N/A" ) {
  #           $serial = $data{'VOLUMES'}{$vol}{'Serial number'};
  #         }
  #         if ( !defined $serial || $serial eq '' ) {
  #           Xormon::error( "Create conf file - host section: Volume ID/Serial not found! Volume \"$vol\" skipped... $!" . __FILE__ . ":" . __LINE__ ) && next;
  #         }

  #         $vol_idx++;

  #         $vol = remove_unauthorized_characters($vol);

  #         if ( $vol_idx == 1 ) {
  #           $vols = $vol;
  #           $ids  = $serial;
  #         }
  #         if ( $vol_idx > 1 ) {
  #           $vols .= " $vol";
  #           $ids  .= " $serial";
  #         }
  #       }
  #     }

  #     $host_group = remove_unauthorized_characters($host_group);

  #     print CONF "$host_group,$host_group,$vols,$ids\n";
  #   }
  # }


  # FC ports

  if ( defined $data{'PORTS'}{'FC'} && $data{'PORTS'}{'FC'} ne '' ) {

    # find active Controller
    my $active_controller_name = "";
    if ( defined $data{'CONTROLLER'} && $data{'CONTROLLER'} ne '' ) {
      foreach my $controller ( sort keys( %{ $data{'CONTROLLER'} } ) ) {
        if ( defined $data{'CONTROLLER'}{$controller}{'State'} && $data{'CONTROLLER'}{$controller}{'State'} eq "active" ) {
          $active_controller_name = $controller;
          last;
        }
      }
    }
    if ( defined $active_controller_name && $active_controller_name ne '' ) {
      foreach my $port_id ( sort keys( %{ $data{'PORTS'}{'FC'} } ) ) {

        my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "FCPORT" . $port_id); $hash_key = lc($hash_key);

        # if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWNN'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWNN'} ne '' ) {
        #   $conf{$hash_key}{'WWNN'} = $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWNN'};
        # }
        if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWPN'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWPN'} ne '' ) {
          $conf{$hash_key}{'wwn'} = Xormon::formatWWN($data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'WWPN'});
        }
        # if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'FC-ID'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'FC-ID'} ne '' ) {
        #   $fc_id = $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'FC-ID'};
        # }
        # if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch name'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch name'} ne '' ) {
        #   $fabric_switch_name = $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch name'};
        # }
        # if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch port'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch port'} ne '' ) {
        #   $fabric_switch_port = $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch port'};
        # }
        # if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch WWNN'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch WWNN'} ne '' ) {
        #   $fabric_switch_wwnn = $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch WWNN'};
        # }
        # if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch WWPN'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch WWPN'} ne '' ) {
        #   $fabric_switch_wwpn = $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric switch WWPN'};
        # }
        # if ( defined $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric WWN'} && $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric WWN'} ne '' ) {
        #   $fabric_wwn = $data{'PORTS'}{'FC'}{$port_id}{$active_controller_name}{'Fabric WWN'};
        # }
      }
    }
    else {
      $logger->error( "Can't identificate active controller!" . __FILE__ . ":" . __LINE__ );
    }
  }

  # # Disk
  # if ( defined $data{'DISKS'} && $data{'DISKS'} ne '' ) {
  #   foreach my $disk_controller ( sort keys( %{ $data{'DISKS'} } ) ) {
  #     if ( defined $data{'DISKS'}{$disk_controller} && $data{'DISKS'}{$disk_controller} ne '' ) {
  #       foreach my $disk_id ( sort { $a <=> $b } keys( %{ $data{'DISKS'}{$disk_controller} } ) ) {

  #         my $disk_label = $disk_id . "." . $disk_controller;
  #         my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "DRIVE" . $disk_label); $hash_key = lc($hash_key);

  #         my $size = "";
  #         my $type = "";
  #         my $dev  = "";

  #         if ( defined $data{'DISKS'}{$disk_controller}{$disk_id}{'size'} && $data{'DISKS'}{$disk_controller}{$disk_id}{'size'} ne '' ) {
  #           $size = $data{'DISKS'}{$disk_controller}{$disk_id}{'size'};
  #           $conf{$hash_key}{'size'} = $size;
  #         }
  #         if ( defined $data{'DISKS'}{$disk_controller}{$disk_id}{'type'} && $data{'DISKS'}{$disk_controller}{$disk_id}{'type'} ne '' ) {
  #           $type = $data{'DISKS'}{$disk_controller}{$disk_id}{'type'};
  #           $conf{$hash_key}{'type'} = $type;

  #         }
  #         if ( defined $data{'perf'}{$conf_timestamp}{'DISKS'}{$disk_controller}{$disk_id}{'dev'} && $data{'perf'}{$conf_timestamp}{'DISKS'}{$disk_controller}{$disk_id}{'dev'} ne '' ) {
  #           $dev = $data{'perf'}{$conf_timestamp}{'DISKS'}{$disk_controller}{$disk_id}{'dev'};
  #           $conf{$hash_key}{'dev'} = $dev;
  #         }
  #       }
  #     }
  #   }
  # }

  # # Ethernet ports
  # if ( defined $data{'PORTS'}{'NET'} && $data{'PORTS'}{'NET'} ne '' ) {
  #   foreach my $port_id ( sort keys( %{ $data{'PORTS'}{'NET'} } ) ) {
  #     my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "ETHPORT" . $port_id); $hash_key = lc($hash_key);
  #     if ( defined $data{'perf'}{$conf_timestamp}{'PORTS'}{'NET'}{$port_id}{'dev'} && $data{'perf'}{$conf_timestamp}{'PORTS'}{'NET'}{$port_id}{'dev'} ne '' ) {
  #       $conf{$hash_key}{'dev'} = $data{'perf'}{$conf_timestamp}{'PORTS'}{'NET'}{$port_id}{'dev'};
  #     }
  #   }
  # }

  #print Dumper \%conf;

  #save configuration
  $xormon->saveConfJSON(\%conf);
  return 1;
}


sub fc_info {
  my $array_name = $cfg->{'arrayName'};
  my @fc_list = run_ssh_cmd("fc --list");

  my $fc_idx       = 0;
  my $fcs_cmd_line = "";
  if (@fc_list) {
    foreach my $line (@fc_list) {
      chomp $line;
      if ( $line =~ m/^fc[0-9]+/ ) {
        my ( $fc_name, $controller, undef ) = split( /\s+/, $line );
        my ( $fc_name_simple, undef ) = split( /\./, $fc_name );

        if ( exists $data{'PORTS'}{'FC'}{$fc_name_simple} ) {
          $fc_idx++;
          $fcs_cmd_line = "echo \'$delimeter $fc_name_simple : $controller $delimeter\'\; fc --info \'$fc_name\' --array $array_name --ctrlr $controller\;";
        }
        else {
          error( "Perf stats not found for FC port \"$fc_name ($fc_name_simple)\"! Configuration data collection skipped for this port... : " . __FILE__ . ":" . __LINE__ ) && next;
        }
      }
    }
  }

  if ( $fc_idx > 0 ) {
    my @fcs_info   = run_ssh_cmd("$fcs_cmd_line");
    my $fc_id      = "";
    my $controller = "";
    foreach my $line_l (@fcs_info) {
      chomp $line_l;
      $line_l =~ s/^\s+//;

      if ( $line_l =~ m/^$delimeter/ && $line_l =~ m/$delimeter$/ ) {
        $line_l =~ s/$delimeter//g;
        $line_l =~ s/^\s+//g;
        $line_l =~ s/\s+$//g;
        ( $fc_id, $controller ) = split( / : /, $line_l );
        next;
      }
      if ( $line_l =~ m/^ERROR:/ ) { next; }

      if ( defined $fc_id && $fc_id ne '' && defined $controller && $controller ne '' && $line_l =~ m/:\s+/ ) {
        my ( $key, $value ) = split( /:\s+/, $line_l );
        $data{'PORTS'}{'FC'}{$fc_id}{$controller}{$key} = $value;
      }
    }
  }
 return 1;
}

sub set_vol_list2stats {
  my @lines;
  my $vols = "";
  if ( defined $data{'VOLUMES'} && $data{'VOLUMES'} ne '' ) {
    foreach my $vol ( sort keys( %{ $data{'VOLUMES'} } ) ) {
      chomp $vol;

      my $vol_length = length($vol);
      if ( ( length($vols) + 1 + $vol_length ) > 250 ) {

        # use $vols
        $vols =~ s/^,//;
        push( @lines, $vols );
        $vols = $vol;
      }
      else {
        $vols .= ",$vol";
      }
    }
  }
  if ( defined $vols && $vols ne '' ) {
    $vols =~ s/^,//;
    push( @lines, $vols );
  }

  return @lines;
}

sub set_multiplier {
  my $range_name = shift;

  my $multiplier = "";
  if ( $range_name eq "0u-100u" ) {
    $multiplier = 0.05;    # 50us
  }
  if ( $range_name eq "100u-200u" ) {
    $multiplier = 0.15;    # 150us
  }
  if ( $range_name eq "200u-500u" ) {
    $multiplier = 0.35;    # 350us
  }
  if ( $range_name eq "500u-1m" ) {
    $multiplier = 0.75;    # 750us
  }
  if ( $range_name eq "1m-2m" ) {
    $multiplier = 1.5;     # 1.5ms
  }
  if ( $range_name eq "2m-5m" ) {
    $multiplier = 3.5;     # 3.5ms
  }
  if ( $range_name eq "5m-10m" ) {
    $multiplier = 7.5;     # 7.5ms
  }
  if ( $range_name eq "10m-20m" ) {
    $multiplier = 15;      # 15ms
  }
  if ( $range_name eq "20m-50m" ) {
    $multiplier = 35;      # 35ms
  }
  if ( $range_name eq "50m-100m" ) {
    $multiplier = 75;      # 75ms
  }
  if ( $range_name eq "100m-200m" ) {
    $multiplier = 150;     # 150ms
  }
  if ( $range_name eq "200m-500m" ) {
    $multiplier = 350;     # 350ms
  }
  if ( $range_name eq "500m-1s" ) {
    $multiplier = 750;     # 750ms
  }
  if ( $range_name eq "1s-2s" ) {
    $multiplier = 1500;    # 1500ms
  }
  if ( $range_name eq "2s-5s" ) {
    $multiplier = 3500;    # 3500ms
  }
  if ( $range_name eq "5s-10s" ) {
    $multiplier = 7500;    # 7500ms
  }
  if ( $range_name eq "10s-20s" ) {
    $multiplier = 15000;    # 15000ms
  }
  if ( $range_name eq "20s-30s" ) {
    $multiplier = 25000;    # 25000ms
  }
  if ( $range_name eq "30s-max" ) {
    $multiplier = 35000;    # 25000ms
  }

  if ( !defined $multiplier || $multiplier eq '' || !Xormon::isdigit($multiplier) ) {
    $multiplier = 0;
  }

  return $multiplier;
}


sub remove_unauthorized_characters {
  my $name = shift;

  $name =~ s/://g;
  $name =~ s/\///g;
  $name =~ s/\\//g;
  $name =~ s/"//g;
  $name =~ s/'//g;

  if ( defined $ENV{UNAUTHORIZED_CHARACTERS} && $ENV{UNAUTHORIZED_CHARACTERS} ne '' ) {
    my $string = $ENV{UNAUTHORIZED_CHARACTERS};
    my @chars  = split( "", $string );

    foreach my $char (@chars) {
      $name =~ s/$char//g;
    }
  }

  return $name;

}

sub connection_test {
  my $client = shift;
  $cfg       = shift;
  $SSH       = "ssh";
  my @rows_test = ();
  my $test = Test->new($client);

  if ($cfg->{'isKeyFile'} == 1 && defined $cfg->{'sshKeyFile'} && $cfg->{'sshKeyFile'} ne "") {
    $SSH = "ssh -i $cfg->{'sshKeyFile'}";
  }

  my @variables = ("hostcfg_id", "hostalias", "sshHost", "sshPort", "sshUsername", "isKeyFile", "arrayName");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
    if (!$cfg->{'isKeyFile'}){
      unless ( exists $cfg->{'sshPassword'} ) {
        $test->addSection("Validation of variables", 0 , "Cannot continue, sshPassword is not configured!");
        return $test->stop();
      }
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'sshHost'},$cfg->{'sshPort'});
  $test->addSection("Network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $array_name = $cfg->{'arrayName'};

  if ($cfg->{'isKeyFile'} == 0 ) {
    unless ( exists $cfg->{'sshPassword'} ) {
      $test->addSection("SSH authorization", 0, "Cannot continue, 'sshPassword' is not configured!");
      return $test->stop();
    }
    my $ssh = Xormon::openssh_new(  $cfg->{'sshHost'},  $cfg->{'sshPort'},  $cfg->{'sshUsername'}, $timeout_config, $cfg->{'sshPassword'} );
    if ( !defined $ssh || $ssh eq "" || $ssh =~ m/^Error|^ERROR/ ) {
      push(@rows_test, "Unable to connect to remote machine ($cfg->{'hostalias'})!\n");
      $test->addSection("SSH authorization", 0, @rows_test);
      return $test->stop();
    }
    else {
      push(@rows_test, "Connected to $cfg->{'hostalias'} with password\n");
      $cfg->{'openssh'} = $ssh;
    }
  }
  elsif ($cfg->{'isKeyFile'} == 1 && defined $cfg->{'sshKeyFile'} && $cfg->{'sshKeyFile'} ne "") {
    push(@rows_test, "Trying to connect to $cfg->{'hostalias'}, '$cfg->{'sshUsername'}', sshkey: $cfg->{'sshKeyFile'} \n");
  }
  else {
    push(@rows_test, "Trying to connect to $cfg->{'hostalias'} with username '$cfg->{'sshUsername'}' and default sshkey\n");
  }

  my @array_list = run_ssh_cmd("array --list");
  my $arr_ok = 1;

  if (@array_list) {
    my $name_found = 0;
    foreach my $line (@array_list) {
      chomp $line;
      if ( $line =~ m/^Name\s+/ ) { next; }
      if ( $line =~ m/^---+\+/ )  { next; }

      if ( $line =~ m/^$array_name\s+/ ) {
        push(@rows_test,"Array $array_name found\n");
        $name_found = 1;
        my @array_info = run_ssh_cmd("array --info \"$array_name\"");
        foreach my $line_l (@array_info) {
          chomp $line_l;
          my ( $key, $value ) = split( /:\s+/, $line_l );
          $data{'ARRAY'}{$key} = $value;
        }
        #print Dumper %data;
      }
    }
    if ($name_found == 0) {
      push(@rows_test,"Array $array_name not found!\n");
      $arr_ok = 0;
    }
  }
  else {
    push(@rows_test,"CMD \"array --list\" failed!\n");
    $arr_ok = 0;
  }
  $test->addSection("SSH authorization", $arr_ok, "@rows_test");
  if ($arr_ok == 0) {
    if ($cfg->{'isKeyFile'} == 0 ) {
      Xormon::openssh_disconnect( $cfg->{'openssh'}, $timeout_config );
    }
    return $test->stop();
  }

  if ($cfg->{'isKeyFile'} == 0 ) {
    Xormon::openssh_disconnect( $cfg->{'openssh'}, $timeout_config );
  }
  return $test->stop();
}

sub run_ssh_cmd {
  my $cmd = shift;
  my @output;

  if ( ($cfg->{'isKeyFile'} == 0) && $cfg->{'openssh'} ) {
    @output = Xormon::openssh_runcmd( $cfg->{'openssh'}, "$cmd", 60, "ARRAY" );
  }
  else {

    my $storage_ip = $cfg->{'sshHost'};
    my $ssh_port   = $cfg->{'sshPort'};
    my $user       = $cfg->{'sshUsername'};

    my $timeout = 300;

    eval {
      # Set alarm
      my $act_time = localtime();
      local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
      alarm($timeout);

      @output = `$SSH $user\@$storage_ip -p $ssh_port "$cmd" 2>&1`;
      #print "out @output\n";
      if ( exists $cfg->{'debug'} && $cfg->{'debug'} eq "cmd" ) { 
        $logger->info("$SSH $user\@$storage_ip -p $ssh_port \"$cmd\" 2>&1");
      }

      # end of alarm
      alarm(0);
    };
    if ($@) {
      if ( $@ =~ /died in SIG ALRM/ ) {
        my $act_time = localtime();
        $logger->error( "command timed out in eval after : $timeout seconds : " . __FILE__ . ":" . __LINE__ );
        Xoxmon::error( "SSH command failed: \"$SSH $user\@$storage_ip -p $ssh_port \"$cmd\" 2>&1\" : " . __FILE__ . ":" . __LINE__ );
        @output = ();
      }
    }
  }

  my $error_found = 0;
  my @errors;
  if (@output) {
    foreach my $line (@output) {
      chomp $line;
      if ( $line =~ m/^ERROR:/ ) {
        if ( $line =~ m/^ERROR: no available hdds matching criteria/ )  { next; }    # there is an error, but perf data there are presented
        if ( $line =~ m/^ERROR: Failed to find statistics for volume/ ) { next; }

        $error_found = 1;
        push( @errors, "$line" );
      }
    }
  }

  my $output_items_count = @output;
  if ( defined $output_items_count && Xormon::isdigit($output_items_count) && $output_items_count == 0 ) {
    $logger->error( "Empty output from CMD: \"$cmd\"  $!" . __FILE__ . ":" . __LINE__ );
  }

  if ( $error_found == 1 ) {
    $logger->error( "ERROR: CMD: \"$cmd\"  $!" . __FILE__ . ":" . __LINE__ );
    foreach my $error (@errors) {
      $logger->error("$error");
    }

    #@output = ();
  }

  return @output;
}



1;
