use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib '../../../perl-modules';
use OracleDB;
use Xormon;
use File::Temp qw(tempfile);
use File::Copy;
use Test;
use XormonLogger;


$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3303;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);
my $logger;

my $server = IO::Socket->new(
  Domain => AF_INET,
  Type => SOCK_STREAM,
  Proto => 'tcp',
  LocalHost => '0.0.0.0',
  LocalPort => $port,
  ReusePort => 1,
  Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {
  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }

  if ($pid == 0) {
    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    #received data template
    #
    #{
    #  "jwt"  : "<backendToken>",
    #  "task" : "<taskName>"
    #  "data" : {
    #    <configuration_from_hostcfg>
    #  }
    #}

    #read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);

    my $decoded = from_json($data);
    my $cfg = $decoded->{data};
    $logger = XormonLogger->new( 'oracledb', $cfg->{hostcfg_id}, $cfg->{hostalias} );
    $logger->info("Connection from $client_address:$client_port\n");
    $logger->info("Received task: ".$decoded->{task});
    my $response = '';
    if ($decoded->{task} eq "fetch") {

      $client->send("Task successfuly started");
      $client->shutdown(SHUT_WR);

      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'oracledb', $cfg->{hostcfg_id}, $cfg->{hostalias});

      my $cfg = $decoded->{data};

      $xormon->startJob('fetch', $cfg->{hostalias}, $$);

      ############################### AGENT SECTION ###############################

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }

      #print Dumper $cfg;
      # get data from technology
      my $dataToBackend = api2json($xormon, $cfg);
      #print Dumper $dataToBackend;
      # send data to backend
      my $rc = $xormon->saveJSON2($dataToBackend);

      ############################### END AGENT SECTION ###############################

      # status code
      my $status;
      if ($rc eq "201") {
        $status = "successful";
      } elsif ($rc eq "401") {
        $status = "unauthorized";
      } else {
        $status = "error";
      }

      $xormon->endJob('fetch', $cfg->{hostalias}, $status, $$);

      $logger->info("Task [fetch] for ".$cfg->{hostalias}." is done");

      } elsif ($decoded->{task} eq "test") {
        my $cfg = $decoded->{data};
        apitest($client, $cfg);
        $client->shutdown(SHUT_WR);
        $logger->info("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
      } else {
        $response = "undefined task";

        $client->send($response);
        $client->shutdown(SHUT_WR);
      }

    exit();

    } else {
      $client->autoflush(1);
    }
}

my $time = time();
my $ft   = filetime();
my %instance_names;
my %host_names;
my %pdb_names;

sub api2json {
  my $xormon = shift;
  my $data   = shift;

  my $port        = $data->{port};
  my $username    = $data->{username};
  my $password    = $data->{password};
  my $alias       = $data->{hostalias};
  my $type        = $data->{type};
  my $hostcfg_id  = $data->{hostcfg_id};
  my $driver      = $data->{driver};
  my $mvc_dir     = $FindBin::Bin;
  my $iostats_dir = "$mvc_dir/iostats";

  if ( -f "$driver/sqlplus" ) {
    $driver = $driver;
  }
  elsif ( -f "$driver/bin/sqlplus" ) {
    $driver = "$driver/bin";
  }

  unless ( -d $iostats_dir ) {
    mkdir( "$iostats_dir", 0755 ) || Xormon::error(  "Cannot mkdir $iostats_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }
  my %perf;
  my %conf;
  my @architecture;
  my @hs;

  my @hosts = @{$data->{hosts}};
  my $folder = $type;
  my $folder_id = "$hostcfg_id";# OracleDB::md5_string("OracleDB-$alias");
  my $menu_group = defined $data->{main_group} && $data->{main_group} ne "" ? $data->{main_group} : $data->{type};
  my $sub_group = defined $data->{sub_group} && $data->{sub_group} ne "" ? $data->{sub_group} : "";
  my $menu_group_id = defined $data->{main_group_id} ? $data->{main_group_id} : OracleDB::md5_string($menu_group);  
  my $sub_group_id  = defined $data->{sub_group_id} ? $data->{sub_group_id} : OracleDB::md5_string($sub_group);
  my @groups = ();

  push(@architecture, create_arc_hash($menu_group_id, $menu_group, "menugroup", \@groups, $hostcfg_id));
  @groups = ($menu_group_id);
  if( defined $sub_group && $sub_group ne "" && $sub_group_id && $sub_group_id ne "") {
    push(@architecture, create_arc_hash($sub_group_id, $sub_group, "submenugroup", \@groups, $hostcfg_id));
    @groups = ($sub_group_id);
  }
  push(@architecture, create_arc_hash("$folder_id","$alias","ODBFOLDER", \@groups, $hostcfg_id));
  #my $clp = get_conflikeperf_data($alias, $username, $password, $hosts[0], $port, $db_name, $type, $mvc_dir);

  #my %waitclass = work_clp($clp, "waitclass");
  #my %services  = work_clp($clp, "services");
  #warn Dumper $clp;
  my $run_conf     = 0;
  my $checkup_file = "$mvc_dir/iostats/$alias-conf_hourly";
  if ( -e $checkup_file ) {
    my $modtime  = ( stat($checkup_file) )[9];
    my $timediff = time - $modtime;
    if ( $timediff >= 3600 ) {
      $run_conf = 1;
      open my $fh, '>', $checkup_file;
      print $fh "1\n";
      close $fh;
    }
  }
  elsif ( !-e $checkup_file ) {
    $run_conf = 1;
    open my $fh, '>', $checkup_file;
    print $fh "1\n";
    close $fh;
  }
  my $connected = 0;

  my %host_ids;
  for my $i (0 .. $#hosts) {
    my $host = $hosts[$i]{host};
    my $db_name = $hosts[$i]{service};

    $connected = Test::connTest($host, $data->{port});

    my $instance_id = OracleDB::md5_string("$data->{hostcfg_id}-$i-$host");
    my $temp_hash = get_instance_data($instance_id, $alias, $username, $password, $host, $port, $db_name, $type, $mvc_dir, $driver);
    my $hash = $temp_hash->{data};
    my $arc = prepare_data($hash, $host, $type, $i, $alias);
    push(@hs, $temp_hash->{status});
    push(@hs, { "item_id" => $temp_hash->{status}{item_id}, "type" => "connection", "status" => $temp_hash->{status}{status}, "data" => $temp_hash->{status}{data} });


    my $time = time;
    $perf{$instance_id}{$time} = $arc->{data};
    if ($arc->{data}) {
      push(@hs, { "item_id" => $instance_id, "type" => "data", "status" => "ok", "data" => "" });
    } else {
      push(@hs, { "item_id" => $instance_id, "type" => "data", "status" => "error", "data" => "Couldn't collect data" });
    }
    if ($run_conf) {
      my $conf_hash = get_conf_data($alias, $username, $password, $host, $port, $db_name, $type, $mvc_dir, $driver);
      for my $k_type (keys %{$conf_hash}){
        if($type eq "Standalone"){
          $conf{$instance_id}{$k_type} = encode_json($conf_hash->{$k_type});
        }else{
          $conf{$folder_id}{$k_type} = encode_json($conf_hash->{$k_type});
        }
      }
    }
    my $host_id = OracleDB::md5_string($arc->{"host name"});
    my @parents = ();
    #host mapping checker
    #hopefully this is only for internal purposes and no production looks like this, better safe than sorry
    if(!$host_ids{$host_id}) {
      $host_ids{$host_id} = 1;
      @parents = ($folder_id);
      push(@architecture, create_arc_hash($host_id, $arc->{"host name"},"HOST", \@parents, $hostcfg_id));
      push(@parents, $host_id);  
    }
    push(@parents, (@groups));
    push(@architecture, create_arc_hash("$instance_id",$arc->{"instance name"},"INSTANCE", \@parents, $hostcfg_id));

    push(@parents, $instance_id);
    my @pdbs;
    if ($data->{type} =~ /ultitenant/){

      @pdbs = @{$hosts[$i]{pdbs}};

      foreach my $pdb (@pdbs){

        my $pdb_instance_id = OracleDB::md5_string("$data->{hostcfg_id}-$i-$host"."_$pdb");
        my $temp_hash = get_instance_data($pdb_instance_id, $alias, $username, $password, $host, $port, $pdb, "PDB", $mvc_dir, $driver, $pdb);
        push(@hs, $temp_hash->{status});
        my $hash = $temp_hash->{data};
        #print Dumper $hash;

        $arc = prepare_data($hash, $host."_".$pdb, "PDB", $i,$alias);

        #print "THIS $pdb_instance_id";
        if ($arc->{data}) {
          push(@hs, { "item_id" => $pdb_instance_id, "type" => "data", "status" => "ok", "data" => "" });
        } else {
          push(@hs, { "item_id" => $pdb_instance_id, "type" => "data", "status" => "error", "data" => "Couldn't collect data" });
        }
        $perf{$pdb_instance_id}{$time} = $arc->{data};
        if ($run_conf) {
          my $conf_hash_pdb = get_conf_data($alias, $username, $password, $host, $port, $pdb, "PDB", $mvc_dir, $driver);
          for my $k_type (keys %{$conf_hash_pdb}){
            $conf{$instance_id}{$k_type} = encode_json($conf_hash_pdb->{$k_type});
          }
        }
        @parents = ();
        push(@parents, \@groups,$folder_id,$instance_id);
        push(@architecture, create_arc_hash("$pdb_instance_id",$arc->{"instance name"},"INSTANCE",\@parents,$hostcfg_id));
      }
    }
  }

  if ($connected == 1) {
    $xormon->isConnected(1);
  } else {
    $xormon->isConnected(0, "TCP connection to " . $data->{host} . ":" . $data->{port} . " has failed!");
    return ();
  }
  #  print "hs";
  #print Dumper \@hs;
  #  print "perf";
  #print Dumper \%perf;
  #  print "architecture";
  #print Dumper \@architecture;
  #warn Dumper \%conf;
  #warn Dumper \%perf;
  if (scalar keys %perf > 0) {
    $xormon->hasData(1);
  } else {
    $xormon->hasData(0, "Unable to get data")
  }
  my $rc = $xormon->saveStatus(\@hs);
  $logger->info("Save Health Status for host  to DB. RC : $rc\n");
  my $arc_resp = $xormon->saveArchitecture(\@architecture);
  $logger->info("Save Architecture for host  to DB. RC : $arc_resp\n");
  my $configuration_rc = $xormon->saveConfJSON( \%conf, 1  );
  $logger->info("Save Configuration for host  to DB. RC : $configuration_rc\n");

  my %weird_perf;

  $weird_perf{oracledb}{instance} = \%perf;

  #print Dumper \%weird_perf;
  return \%weird_perf;
}

sub create_arc_hash {
  my $item_id    = shift;
  my $label      = shift;
  my $subsystem  = shift;
  my $parents    = shift;
  my $hostcfg_id = shift;
  my $hw_type = "ORACLEDB";
  my @parents = @{$parents};

  my %arch = (
    "item_id" => $item_id,
    "label" => $label,
    "hw_type" => $hw_type,
    "subsystem" => $subsystem,
    "parents" => \@parents,
    "hostcfg_id" => $hostcfg_id
  );

  return \%arch;
}

sub apitest {
  my $client = shift;
  my $data   = shift;

  my $port       = $data->{port};
  my $username   = $data->{username};
  my $password   = $data->{password};
  my $alias      = $data->{hostalias};
  my $type       = $data->{type};
  my $hostcfg_id = $data->{hostcfg_id};
  my $driver     = $data->{driver};
  my $mvc_dir    = $FindBin::Bin;

  my %perf;
  my %conf;
  my @architecture;
  my $ok_check = 1;

  my $test = Test->new($client);
  my @rows;

  #DRIVER CHECK
  if ($driver and $driver ne "") {
    if ( -f "$driver/sqlplus" ) {
      $driver = $driver;
    }
    elsif ( -f "$driver/bin/sqlplus" ) {
      $driver = "$driver/bin";
    }
    unless ( -f "$driver/sqlplus" ) {
      my @rows;
      push(@rows, "ORACLE_HOME is not set correctly or sqlplus is not installed in the following path. \n$driver");
      $test->addSection("Driver test", 0 , \@rows);
      return $test->stop();
    }
    my @rows;
    push(@rows, "ORACLE_HOME is set correctly");
    $test->addSection("Driver test", 1 , \@rows);
  } else {
    my @rows;
    push(@rows, "ORACLE_HOME is empty or not set");
    $test->addSection("Driver test", 0 , \@rows);
    return $test->stop();
  }

  my @hosts = @{$data->{hosts}};

  for my $i (0 .. $#hosts){
    my $host = $hosts[$i]{host};
    my $db_name = $hosts[$i]{service};
    my $temp_hash = get_instance_data("filler", $alias, $username, $password, $host, $port, $db_name, $type, $mvc_dir, $driver);
    my $hash = $temp_hash->{data};
    my $status = $temp_hash->{status}{status};
    my $err = "";
    if($temp_hash->{status}{status} eq "error"){
      $err = $temp_hash->{status}{data};
      if($ok_check){
        $ok_check = 0;
      }
    }
    push(@rows, "Connection test status for host $host with name $db_name is: $status \n$err \n");

    my @pdbs;
    if ($data->{type} =~ /ultitenant/){
      @pdbs = @{$hosts[$i]{pdbs}};
      foreach my $pdb (@pdbs){
        $temp_hash = get_instance_data("filler", $alias, $username, $password, $host, $port, $pdb, "PDB", $mvc_dir, $driver, $pdb);
        $hash = $temp_hash->{data};
        my $status = $temp_hash->{status}->{status};
        my $err = "";
        if($temp_hash->{status}{status} eq "error"){
          $err = $temp_hash->{status}{data};
          if($ok_check){
            $ok_check = 0;
          }
        }

        push(@rows, "Connection test status for host $host with name $pdb is: $status \n$err\n");
      }
    }
  }
  $test->addSection("Data test", $ok_check , \@rows);
  return $test->stop();
}

sub clean_data {
  my $data     = shift;
  my $instance = shift;
  my $type     = shift;
  my $index    = shift;

  delete $data->{$instance}->{'Instance name'};
  delete $data->{$instance}->{'Host name'};
  delete $data->{$instance}->{'PDB name'};


  if ($index > 0){
    delete $data->{$instance}->{Capacity};
  }

  my $metrics = OracleDB::get_metrics_dictionary();
  my %new_data;

  for my $metric (keys %{$data->{$instance}}){
    if (defined $metrics->{$metric}){
      $new_data{$instance}{$metrics->{$metric}} = $data->{$instance}->{$metric};
    }
  }

  return \%new_data;
}

sub prepare_data {
  my $data       = shift;
  my $instance   = shift;
  my $type       = shift;
  my $host_index = shift;
  my $alias      = shift;

  my %arc;

  $arc{'instance name'} = defined $data->{$instance}->{'Instance name'} ? $data->{$instance}->{'Instance name'} : $instance;
  $arc{'host name'} = defined $data->{$instance}->{'Host name'} ? $data->{$instance}->{'Host name'} : "not-found";
  $arc{'host'} = $instance;
  $arc{'type'} = $type;

  $data->{$instance}->{'CPU Usage Per Sec'} = $data->{$instance}->{'CPU Usage Per Sec'} / 10;
  $data->{$instance}->{'CPU Usage Per Txn'} = $data->{$instance}->{'CPU Usage Per Txn'} / 100;
  $data->{$instance}->{'IO Megabytes per Second'} = $data->{$instance}->{'I/O Megabytes per Second'} * (1024**2);
  
  if($type eq "RAC"){
    my $bo;
    my $bt;
    my $bth;
    my $bf;
    $bo  = defined $data->{$instance}->{'gc cr block receive time'}      ? $data->{$instance}->{'gc cr block receive time'} : 0;
    $bt  = defined $data->{$instance}->{'gc cr blocks received'}         ? $data->{$instance}->{'gc cr blocks received'} : 1;
    $bth = defined $data->{$instance}->{'gc current block receive time'} ? $data->{$instance}->{'gc current block receive time'} : 0;
    $bf  = defined $data->{$instance}->{'gc current blocks received'}    ? $data->{$instance}->{'gc current blocks received'} : 1;
    $bt  = $bt == 0 ? 1 : $bt;
    $bf  = $bf == 0 ? 1 : $bf;

    $data->{$instance}->{'GC Avg CR Block receive ms'}  = sprintf($bo / $bt);
    $data->{$instance}->{'GC Avg CUR Block receive ms'} = sprintf($bth / $bf);
  }

  if ($type eq "PDB"){  

    $data->{$instance}->{"IO Megabytes per Second"} = (sprintf("%e",$data->{$instance}->{"Physical Read Bytes Per Sec"}) +
                                                       sprintf("%e",$data->{$instance}->{"Physical Write Bytes Per Sec"}));
    $data->{$instance}->{"IO Requests per Second"}  = (sprintf("%e",$data->{$instance}->{"Physical Reads Per Sec"}) +
                                                       sprintf("%e",$data->{$instance}->{"Physical Writes Per Sec"}));
    #$data->{$instance}->{"IO Requests per Second"}  = "$data->{$instance}->{'IO Requests per Second'}";
    #$data->{$instance}->{"IO Megabytes per Second"} = "$data->{$instance}->{'IO Megabytes per Second'}";
    $arc{'instance name'} = defined $data->{$instance}->{'PDB name'} ? $data->{$instance}->{'PDB name'} : $instance;
  }

  $data = clean_data($data, $instance, $type, $host_index);

  if ($type eq "PDB"){
    my $data_c = work_counters($type, $instance, $data->{$instance}, $instance, $type, $instance,$alias);
    $data->{$instance} = $data_c;
  }

  $arc{data} = $data->{$instance};

   #print Dumper \%arc;
  return \%arc;
}

sub get_instance_data {
  my $id       = shift;
  my $alias    = shift;
  my $username = shift;
  my $password = shift;
  my $ip       = shift;
  my $port     = shift;
  my $db_name  = shift;
  my $type     = shift;
  my $mvc_dir  = shift;
  my $driver   = shift;
  my $pdb      = shift;

  $pdb = defined $pdb ? $pdb : "";
  my $iostats_dir = "$mvc_dir/iostats";

  my $connect_s = "$username/$password"."@".'"(DESCRIPTION=(ADDRESS=(PROTOCOL=TCP)(HOST='.$ip.')(PORT='.$port.'))(CONNECT_DATA=(SERVICE_NAME='.$db_name.')))"';

  my $file_type = $type;
  if ($type eq "RACMultitenant"){
    $file_type = "Multitenant";
  }
  my $hash_ref = sysmetric_history($id, $db_name,"DB", $connect_s, "$file_type"."_L.sql", $type, $ip, $driver, $pdb);

  return $hash_ref;
}

sub get_conf_data {
  my $alias    = shift;
  my $username = shift;
  my $password = shift;
  my $ip       = shift;
  my $port     = shift;
  my $db_name  = shift;
  my $type     = shift;
  my $mvc_dir  = shift;
  my $driver   = shift;
  my $pdb      = shift;

  $pdb = defined $pdb ? $pdb : "";

  my $iostats_dir = "$mvc_dir/iostats";

  my $connect_s = "$username/$password"."@".'"(DESCRIPTION=(ADDRESS=(PROTOCOL=TCP)(HOST='.$ip.')(PORT='.$port.'))(CONNECT_DATA=(SERVICE_NAME='.$db_name.')))"';

  my $file_type = $type;
  if ($type =~ /ultitenant/){
    $file_type = "CDB";
  }

  my @metrics = ("$file_type"."\.sql");
  my $hash_ref = tablespaces($connect_s, \@metrics, $driver, "Standalone");

  return $hash_ref;
}

sub get_conflikeperf_data {
  my $alias    = shift;
  my $username = shift;
  my $password = shift;
  my $ip       = shift;
  my $port     = shift;
  my $db_name  = shift;
  my $type     = shift;
  my $mvc_dir  = shift;
  my $driver   = shift;
  my $pdb      = shift;

  $pdb = defined $pdb ? $pdb : "";

  my $iostats_dir = "$mvc_dir/iostats";

  my $connect_s = "$username/$password"."@".'"(DESCRIPTION=(ADDRESS=(PROTOCOL=TCP)(HOST='.$ip.')(PORT='.$port.'))(CONNECT_DATA=(SERVICE_NAME='.$db_name.')))"';

  my $file_type = "perflikeconf";
  if ($type =~ /Multitenant/){
    $file_type .= "_pdb";
  }elsif($type =~ /RAC/){
    $file_type .= "_rac";
  }
  my @metrics = ("$file_type"."\.sql");

  my $hash_ref = tablespaces($connect_s, \@metrics, $driver, $type);

  return $hash_ref;
}

sub work_clp {
  my $data      = shift;
  my $file      = shift;
  my $stat_type = shift;
  my $sh_type   = shift;
  my $mvc_dir = $FindBin::Bin;
  my ( $can_read, $ref );
  my $save_type = $sh_type;
  if($save_type eq "RACMultitenant"){
    $save_type = "RAC";
  }
  # read perf file
  if ( -f "$mvc_dir/$file" ) {
    ( $can_read, $ref ) = read_json( "$mvc_dir/$file" );
    #backup_file( $file );
  }
  unless ( $can_read ) {
    $logger->error( "$file cannot be loaded" );
    return 0;
  }

  $data = $ref;
  my %dict;
#  print "oracleDB-json2rrd.pl : processing file $file\n";
  my %perf_data;


  for my $instance (keys %{$data->{$save_type}->{$stat_type}}){
    if($instance =~/CDB\$ROOT/){
      next;
    }
    my @act = @{$data->{$save_type}->{$stat_type}->{$instance}};
 #   print Dumper \@act;
    foreach my $stat (@act){
      $perf_data{"$instance".",".$stat->{"Wait class"}}->{"TIME_WAITED"} = $stat->{"TIME_WAITED"};
      $perf_data{"$instance".",".$stat->{"Wait class"}}->{"TIME_WAITED_FG"} = $stat->{"TIME_WAITED_FG"};
      $perf_data{"$instance".",".$stat->{"Wait class"}}->{"TOTAL_WAITS"} = $stat->{"TOTAL_WAITS"};
      $perf_data{"$instance".",".$stat->{"Wait class"}}->{"TOTAL_WAITS_FG"} = $stat->{"TOTAL_WAITS_FG"};
    }
  }
  my ($p_can_read, $p_ref);
  my $file_serv = "$mvc_dir/wait_classes_perf.json";
  if ( -f  $file_serv ) {
    ( $p_can_read, $p_ref ) = read_json(  $file_serv );
    #backup_file( $file );
  }else{
    write_json( $file_serv, \%perf_data);
  }
  if ($p_ref and ref $p_ref ne "HASH"){
    unlink($file_serv);
  }
  if ($p_can_read and ref $p_ref eq "HASH"){
    my %p_data = %{$p_ref};
    my %actual_data;
    #print Dumper \%p_data;
    #print Dumper \%perf_data;
    for my $type (keys %perf_data){
      my $rrd_path = "";
      my $type_ns = "$stat_type";
      $type_ns =~ s/ /_/g;
      #warn "\n\n$timediff\n\n";
      $type =~ s/-/,/g;
      my $val_one_fg = defined $perf_data{$type}{"TIME_WAITED_FG"} ? $perf_data{$type}{"TIME_WAITED_FG"} : 0;
      my $val_two_fg = (defined $p_data{$type}{"TOTAL_WAITS_FG"} and $p_data{$type}{"TOTAL_WAITS_FG"} != 0) ? $p_data{$type}{"TOTAL_WAITS_FG"} : 1;
      $val_one_fg =~ s/,/./g;
      $val_two_fg =~ s/,/./g;
      $actual_data{$type}{"Average wait FG ms"} = (sprintf("%e",$val_one_fg) * 10) / sprintf("%e",$val_two_fg);
      my $val_one = defined $perf_data{$type}{"TIME_WAITED"} ? $perf_data{$type}{"TIME_WAITED"} : 0;
      my $val_two = (defined $p_data{$type}{"TOTAL_WAITS"} and $p_data{$type}{"TOTAL_WAITS"} != 0) ? $p_data{$type}{"TOTAL_WAITS"} : 1;
      $val_one =~ s/,/./g;
      $val_two =~ s/,/./g;
      $actual_data{$type}{"Average wait ms"} = (sprintf("%e",$val_one) * 10) / sprintf("%e",$val_two);
      write_json( $file_serv, \%perf_data);

      my $name = $type;
      $name =~ s/\///g;
      #print "\n$name\n";
      my $uuid = md5_string($name);
      #print "\n$uuid\n";
      $dict{$uuid} = $type;
      #$rrd_path = get_filepath_rrd({type =>$type_ns, uuid => $alias, skip_acl => 1});
      #$rrd_path .= "$uuid.rrd";
#      print $rrd_path;
      #data_to_rrd($rrd_path, $act_time, $type_ns, \%{$actual_data{$type}}, "none");
    }
  }
  #print Dumper \%dict;
  if($p_can_read){
    write_json("$mvc_dir/dict.json", \%dict);
  }

  return 1;
}

sub tablespaces {
  my $connect_s = shift;
  my $met       = shift;
  my $driver    = shift;
  my $type      = shift;
  my $inst      = shift;
  my %hash;
  my @metrics = @{$met};
  my $odb_ic  = $driver;
  my $mvc_dir = $FindBin::Bin;
  my $sql_dir = "$mvc_dir/oracledb-sql";
  my %fcNames = (
    "PSU, patches info"          => "Date",
    "Tablespace info"            => "Tablespace",
    "PDB info"                   => "PDB",
    "Online Redo Logs"           => "Group",
    "Upgrade, Downgrade info"    => "ID",
    "SGA info"                   => "Instance",
    "IO Read Write per datafile" => "Datafile",
    "Installed DB components"    => "Component",
  );

  for my $i (0 .. $#metrics){
    my $result;

    $result = qx {$odb_ic/sqlplus -S -L $connect_s \@$sql_dir\/$metrics[$i]; echo \"ret:\$?\"};
    if ($result =~ /ret:1/){
      $result =~ s/ret:1//g;

      return 0;
    }else{
      $result =~ s/ret:0//g;
    }

    my @ssa = split( "##", $result );

    #    print Dumper \@ssa;
    for my $ts_number ( 0 .. $#ssa ) {
      my $name;
      if ( !$ssa[$ts_number] ) {
        next;
      }
      my @ada = split( /"\n|\n"|\n\n/, $ssa[$ts_number] );

      if ( index( $ada[$#ada], "rows selected." ) != -1 ) {
        pop(@ada);
        pop(@ada);
      }
      my @header;
      my $start_index = 2;
      if ( $ada[0] eq "" ) {
        $name        = $ada[1];
        $start_index = 3;
        @header      = split( /\|/, $ada[2] );
      }
      else {
        $name   = $ada[0];
        @header = split( /\|/, $ada[1] );
      }
      $name =~ s/\n//g;
      #print Dumper \@header;

      for my $i ( $start_index .. $#ada ) {
        next if (!defined $ada[$i] or $ada[$i] eq "\n" or $ada[$i] eq "");
        my %helper;
        my @rowdata = split( /\|/, $ada[$i] );
        my $id = defined $rowdata[0] ? $rowdata[0] : "";
        $id =~ s/\"//g;

        if ($name eq "PSU, patches info"){
          if (! defined $id or $id =~ /</){
            next;
          }
        }
        for my $j ( 1 .. $#header ) {
          my $current_metric = $header[$j];
          my $current_value  = defined $rowdata[$j] ? $rowdata[$j] : "";
          $current_metric =~ s/\"//g;
          $current_value  =~ s/\"//g;
          $helper{ $current_metric } = $current_value;
        }
        my $new_name = $fcNames{$name} ? $fcNames{$name} : "m_name";
        $helper{$new_name} = $id;
        push(@{$hash{$name}}, \%helper);
      }
    }
  }

  return \%hash;
}

sub sysmetric_history {
  my $id        = shift;
  my $db_name   = shift;
  my $save_type = shift;
  my $connect_s = shift;
  my $file      = shift;
  my $type      = shift;
  my $instname  = shift;
  my $driver    = shift;
  my $pdb       = shift;

  my $result = "";
  my %whole_info;
  my $mvc_dir = $FindBin::Bin;
  my $sql_dir = "$mvc_dir/oracledb-sql";
  my $odb_ic  = $driver;
  $result = qx {$odb_ic/sqlplus -S -L $connect_s \@$sql_dir\/$file; echo \"ret:\$?\"};
  #warn $result;
  if ($result =~ /ret:1/){
    $result =~ s/ret:1//g;
    my @arr = split(/\n/,$result);
    #print Dumper \@arr;
    my $err = "";
    foreach my $part (@arr){
      if($part eq ""){
        last;
      }
      if($part ne "ERROR:"){
        $err .= " $part";
      }
    }

    $whole_info{status}{"item_id"} = $id;
    $whole_info{status}{"status"} = "error";
    $whole_info{status}{"data"} = "$err";


    #warn $result;
    return \%whole_info;
  }else{
    $result =~ s/ret:0//g;

    $whole_info{status}{"item_id"} = $id;
    $whole_info{status}{"status"} = "ok";

  }
  #print $?;
  #warn $result;


  $result =~ s/[\n]+//g;
  if ($type eq "RAC" or $type eq "RACMultitenant"){
    $whole_info{data} = work_result($save_type, $result, $type, $instname);
  }elsif ($type eq "PDB"){
    $whole_info{data} = work_result($save_type, $result, $type, $instname, $db_name, $pdb);
  }elsif ($type eq "PDBrac"){
    $whole_info{data} = work_result($save_type, $result, $type, $instname, $db_name, $pdb);
  }else {
    $whole_info{data} = work_result($save_type, $result, $type, $instname, $db_name);
  }

  return \%whole_info;
}

sub work_result {
  my $save_type = shift;
  my $result    = shift;
  my $type      = shift;
  my $instname  = shift;
  my $db_name   = shift;
  my $pdb       = shift;

  my %hash;
  my @res_arr =  split(/\|/, $result);
    for my $i (0 .. $#res_arr){
      my @arr =  split(";;;", $res_arr[$i]);
      my $name = pop(@arr);
      pop(@arr);
      for my $j (0 .. $#arr){
        my @line = split(";",$arr[$j]);
        if(defined $line[3]){
          $line[3] =~ s/,/./g;
          my $value;
          if ($line[3] =~ /^\./){
            $value = "0"."$line[3]";
          }else{
            $value = $line[3];
          }
          my $perf_name = $instname;
          if ($type eq "PDB"){
            $perf_name = $instname."_".$pdb;
          }
          $hash{$perf_name}{$line[1]} = $value;
        }else{
          if($line[0] =~ /ORA-/){
            $logger->error( "error $line[0] DB: $save_type, $type, $instname");
          }
        }
      }
    }

  return \%hash;
}

sub round {
  my $value    = shift;
  my $decimals = shift;

  return $value = sprintf("%."."$decimals"."f", $value);
}

# transforms given gmtime to UTC #
sub gmtime2utc {
  my ($sec,$min,$hour,$mday,$mon,$year,$secbool) = @_;
  $year  += 1900;
  $mon   += 1;
  $mon    = sprintf("%02d", $mon);
  $mday   = sprintf("%02d", $mday);
  $hour   = sprintf("%02d", $hour);
  $min    = sprintf("%02d", $min);
  if(defined $secbool and $secbool == 1){
    return "$year-$mon-$mday"."T"."$hour:$min:$sec"."Z";
  }else{
    return "$year-$mon-$mday"."T"."$hour:$min:00Z";
  }
}

sub filetime {
  my ($sec,$min,$hour,$mday,$mon,$year) = (localtime($time-120));
  my $utctime = gmtime2utc($sec,$min,$hour,$mday,$mon,$year);
  $utctime = substr($utctime,0,-4);
  $utctime =~ s/-//g;
  $utctime =~ s/://g;
  $utctime =~ s/T/_/g;

  return $utctime;
}


sub write_json {
  my $path        = shift;
  my $hash_ref    = shift;
  my $path_suffix = "$path-tmp" . $$;
  my $json        = JSON->new->utf8;

  my ($fh, $file_name) = tempfile( UNLINK => 1 ); #use tempfile instead of the $path_suffix (HD)

  if ( ref($hash_ref) ne "HASH" && ref($hash_ref) ne "ARRAY"){
    $logger->error( "Hash ref or array ref expected (path:$path), got: \"$hash_ref\" (Ref:" . ref($hash_ref) . ") in \n");
    return 0;
  }
#  if ($ENV{JSON_PRETTY}){
    $json->pretty([1]);
#  }

  #open( my $FILE, '>', $path_suffix ) || error( "Couldn't open file $path_suffix $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  #print $FILE $json->encode( $hash_ref );
  #close $FILE;
  #rename($path_suffix, $path);

  print $fh $json->encode( $hash_ref );
  close($fh);

  copy($file_name, $path);

  return 1;
}

sub work_counters {
  my $type      = shift;
  my $instance  = shift;
  my $perf_data = shift;
  my $pdb       = shift;
  my %perf_data = defined $perf_data ? %{$perf_data} : undef;
  my $sh_type = shift;
  my $sh_host = shift;
  my $alias   = shift;
  my $mvc_dir = $FindBin::Bin;
  my $file_serv;
  my $iostats_dir = "$mvc_dir/iostats";
  my $denom = 1;
  my %not_a_counter = (
    'GC CR Block Received Per Second'       => "",
    'GC Current Block Received Per Second'  => "",
    'Global Cache Average CR Get Time'      => "",
    'Cell Physical IO Interconnect Bytes'   => "",
    'Global Cache Average Current Get Time' => "",
    'LOG files write latency'               => "",
    'DB files write latency'                => "",
    'GC Avg CR Block receive ms'            => "",
    'GC Avg CUR Block receive ms'           => "",
    'gc cr blocks received'                 => "",
    'gc current block receive time'         => "",
    'gc current blocks received'            => "",
    'gc cr block receive time'              => "",
    'Current Open Cursors Count'            => "",
    'Current Logons Count'                  => "",
    'dbfilesinglewrite'                     => "",
    'free'                                  => "",
    'dbfilescatteredread'                   => "",
    'SoftParseRatio'                        => "",
    'BufferCacheRatio'                      => "",
    'used'                                  => "",
    'CurrentOpenCursorsCount'               => "",
    'dbfilesequentialread'                  => "",
    'PhysicalWritesPerSec'                  => "",
    'NetworkTrafficVolumePerSec'            => "",
    'CurrentLogonsCount'                    => "",
    'MemorySortsRatio'                      => "",
    'logfilesync'                           => ""
  );



  $file_serv = "$iostats_dir/$instance-$type-$alias-lastData.json";

  my $type_ns = $type;
  $type_ns =~ s/ /_/g;

  my ( $p_can_read, $p_ref );
  if ( -f $file_serv ) {
    ( $p_can_read, $p_ref ) = read_json($file_serv);

    #backup_file( $file );
  }
  else {
    write_json( $file_serv, \%perf_data );
    return {};
  }
    if ($p_can_read) {
    my %p_data = %{$p_ref};

    #print "\n$alias - $instance\n";
    #print Dumper \%p_data;
    #print "$alias - $instance\n";
    #print Dumper \%perf_data;
    for my $metric ( keys %p_data ) {
      if ( $perf_data{$metric} ) {
        if ( exists $not_a_counter{$metric} ) {
          $p_data{$metric} = $perf_data{$metric};
        }
        else {
          $p_data{$metric} = ( sprintf( "%e", $perf_data{$metric} ) - sprintf( "%e", $p_data{$metric} ) ) / $denom;    #$perf_data{$metric};
        }
      }
    }
    unlink "$file_serv";
    write_json( $file_serv, \%perf_data );

    return \%p_data;
  }
}


sub read_json {
  my $file = shift;
  my $json = JSON->new->utf8;
  my $string;
  my $output;

  {
    local $/=undef;
    open ( my $FILE, '<', $file ) or $logger->error( "Couldn't open file $file $!") && return ( 0, undef );
    $string = <$FILE>;
    close $FILE;
  }

  if ($string) {
    eval {
      $output = $json->decode($string);
      1;
    } or do {
      $logger->error( "Couldn't parse json file $file $@ ") && return ( 0, undef );
    };
  }

  return ( 1, $output );
}

$server->close();
