package OracleVM;
use strict;
use warnings;
use HTTP::Request::Common;
use LWP;
use Data::Dumper;
use JSON;
use POSIX qw(strftime ceil);
use Date::Parse;
use Time::Local;
use XormonLogger;

my $xormon;
my $cfg;
my $logger;
my %data                   = ();
my %data_server_net        = ();
my %data_server_disk       = ();
my %server_interface_hash  = ();
my %server_disk_hash       = ();
my %vm_interface_hash      = ();
my %vm_disk_hash           = ();
my %data_vm_net            = ();
my %data_vm_disk           = ();
my %conf                   = ();
my $service_data_out       = {};
my @architecture;
my $act_time               = time();
my $act_date               = localtime($act_time);

my @server_metrics       = ( "CPU_UTILIZATION", "MEMORY_USED", "MEMORY_UTILIZATION", "FREE_MEMORY", "FREE_SWAP" );
my @vm_metrics           = ( "CPU_UTILIZATION","CPU_COUNT", "MEMORY_USED", "DISK_READ", "DISK_WRITE","NETWORK_SENT","NETWORK_RECEIVED" );
my @fs_metrics           = ( "FILE_SYSTEM_SPACE_AVAILABLE", "FILE_SYSTEM_SPACE_UTILIZATION", "FILE_SYSTEM_SPACE_TOTAL", "FILE_SYSTEM_TOTAL_FILES_SIZE" );
my @counter_metrics      = ( "DISK_READ", "DISK_WRITE", "NETWORK_SENT", "NETWORK_RECEIVED" );

my @disk_metrics         = ( "DISK_READ", "DISK_WRITE" );
my @net_metrics          = ( "NETWORK_SENT","NETWORK_RECEIVED" );

sub collect_data {
  $xormon = shift;
  $cfg                       = shift;
  my $start_time             = "";
  my $end_time               = "";
  my $start_time_to_manager  = time;
  $start_time_to_manager     = $start_time_to_manager * 1000;

  my $port       = $cfg->{port};
  my $host       = $cfg->{host};
  my $username   = $cfg->{username};
  $logger        = XormonLogger->new( 'oraclevm', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  my $password   = $cfg->{password};
  my $token      = $xormon->{token};
  $cfg->{token}  = $token;

  #print "================= COLLECT DATA ORVM =================\n";
  #print Dumper \$cfg;
  #print Dumper \$xormon;
  #print "HOST:$host,PORT:$port,PAS:$password,user:$username,token:$token\n";

  # REST API INFO
  my $url_stat_rest_api  = "https://$host:$port/ovm/core/wsapi/rest/Manager";

  my $info_about_api        = new_get_request($url_stat_rest_api, $username, $password);
  #print Dumper $info_about_api;

  if ($info_about_api ne "0"){
    $xormon->isConnected(1);
    foreach my $info_api ( @{ $info_about_api}  ) {
      my ($name,$timestamp_manager,$manager_status,$manager_version,$manager_read) = "";
      if ( defined $info_api->{'name'} && $info_api->{'name'} ne "" ) {
        $name              = $info_api->{'name'};
      }
      if ( defined $info_api->{'managerTime'} && $info_api->{'managerTime'} ne "" ) {
        $timestamp_manager = $info_api->{'managerTime'};
      }
      if ( defined $info_api->{'managerRunState'} && $info_api->{'managerRunState'} ne "" ) {
        $manager_status    = $info_api->{'managerRunState'};
      }
      if ( defined $info_api->{'managerVersion'} && $info_api->{'managerVersion'} ne "" ) {
        $manager_version   = $info_api->{'managerVersion'};
      }
      if ( defined $info_api->{'readOnly'} && $info_api->{'readOnly'} ne "" ) {
        $manager_read      = $info_api->{'readOnly'};
      }
      # manager time to timestamp
      $timestamp_manager = $timestamp_manager / 1000;
      $timestamp_manager = sprintf "%.0f", $timestamp_manager;
      #$start_time        = $timestamp_manager - 20 * 60;         # last 20min ( you can change 1-60min)
      #$start_time        = $start_time * 1000;                   # convert to ms
      #$end_time          = $timestamp_manager * 1000;            # convert to ms
      my $service_data = $xormon->getData();
      $start_time = (defined $service_data->{timestamp}) ? $service_data->{timestamp} : time()-1260;
      $end_time   = time()-180;

      $start_time = $start_time - ($start_time % 60) - 180;
      $end_time   = $end_time   - ($end_time % 60);
      my $start_time_collect = strftime( "%Y-%m-%d %H:%M:%S", localtime($start_time ) );
      my $end_time_collect   = strftime( "%Y-%m-%d %H:%M:%S", localtime($end_time) );
      $start_time = $start_time * 1000;  # convert to ms
      $end_time   = $end_time * 1000;    # convert to ms

      #print "$name,$timestamp_manager,$manager_status,$manager_version,$manager_read\n";

      my $date_human = strftime( "%Y-%m-%d %H:%M:%S", localtime($timestamp_manager) );
      #print "Manager name: $name\n";
      #print "Timestamp manager: $timestamp_manager($date_human)\n";
      my $my_time       = time;
      my $my_time_human = strftime( "%Y-%m-%d %H:%M:%S", localtime($my_time) );
      #print "Timestamp on LPAR2RRD server: $my_time($my_time_human)\n";
      #print "START COLLECTING TIME: $start_time_collect\n";
      #print "END COLLECTING TIME: $end_time_collect\n";
      #print "Version: $manager_version\n";
      # STARTING	The manager is starting. Jobs are not yet enabled
      # RUNNING	  The manager is running. Jobs are enabled
      # STOPPING	The manager is stopping. Jobs are disabled
      # STOPPED	  The manager is stopped
      save_arch_conf($host,$port,$username,$password,$cfg,$start_time,$end_time);
      create_perf($host,$port,$username,$password,$cfg,$start_time,$end_time);
      save_data_to_csv();
    }
    return 1;
  }
  # REST API no running
  else {
    Xormon::error("Does not work OracleVM rest_api           : $act_time : $act_date");
    $xormon->isConnected(0);
    return 0;
  }
}

sub save_arch_conf {
  my $host        = shift;
  my $port        = shift;
  my $username    = shift;
  my $password    = shift;
  my $cfg         = shift;
  my $start_time  = shift;
  my $end_time    = shift;

  my %conf_to_hash;
  my @architecture;

  # Values for collecting values from Server / VM
  my $db_url_server     = "https://$host:$port/ovm/core/wsapi/rest/Server";
  my $db_url_vm         = "https://$host:$port/ovm/core/wsapi/rest/Vm";
  my $db_url_repos      = "https://$host:$port/ovm/core/wsapi/rest/Repository";

  # REST API INFO
  my $url_stat_rest_api  = "https://$host:$port/ovm/core/wsapi/rest/Manager";
  # Request on collecting config DATA
  my $info_about_api     = new_get_request($url_stat_rest_api, $username, $password);
  my $info_about_server  = new_get_request($db_url_server,$username,$password );
  my $info_about_vm      = new_get_request($db_url_vm,$username,$password );
  my $info_about_repos   = new_get_request($db_url_repos,$username,$password );

  # Server conf
  #print Dumper $info_about_server;
  # VM conf
  #print Dumper $info_about_vm;
  #
  # Repository conf
  #print Dumper $info_about_repos;
  #
  # Save configuration
  ############
  ## Server ##
  ############
  if ( $info_about_server ne "0" ) {
    foreach my $server_info ( @{ $info_about_server } ) {
      my $server_uuid         = "";
      my $manager_uuid        = "";
      my $server_uuid_for_url = "";
      if ( defined $server_info->{'id'} && $server_info->{'id'} ne "" ) {
        if ( defined $server_info->{'id'}{'value'} && $server_info->{'id'}{'value'} ne "" ) {
          my $uuid = $server_info->{'id'}{'value'};
          $server_uuid_for_url = $server_info->{'id'}{'value'}; # uuid with colon for url
          $uuid =~ s/://g;
          $server_uuid = $uuid;
        }
      }
      if (defined $server_info->{'name'} && $server_info->{'name'} ne ""){
        $conf{$server_uuid}{'server_name'}   = $server_info->{'name'};
      }
      if (defined $server_info->{'hostname'} && $server_info->{'hostname'} ne ""){
        $conf{$server_uuid}{'hostname'}   = $server_info->{'hostname'};
      }
      if (defined $server_info->{'managerUuid'} && $server_info->{'managerUuid'} ne ""){
        $conf{$server_uuid}{'managerUuid'}   = $server_info->{'managerUuid'};
        $manager_uuid                        = "$server_info->{'managerUuid'}";
      }
      # the amount of physical memory on the Server in megabytes
      if (defined $server_info->{'memory'} && $server_info->{'memory'} ne ""){
        my $server_total_memory = $server_info->{'memory'} * 1000000;
        $conf{$server_uuid}{'total_memory'} = $server_total_memory ;
      }
      # the number of processor cores. 
      if (defined $server_info->{'totalProcessorCores'} && $server_info->{'totalProcessorCores'} ne ""){
        $conf{$server_uuid}{'total_processor_cores'} = $server_info->{'totalProcessorCores'};
      }
      # the amount of memory (in megabytes) that is usable by Virtual Machines 
      if (defined $server_info->{'usableMemory'} && $server_info->{'usableMemory'} ne ""){
        $conf{$server_uuid}{'usable_mem'} = $server_info->{'usableMemory'};
      }
      # the number of threads per core. 
      if (defined $server_info->{'threadsPerCore'} && $server_info->{'threadsPerCore'} ne ""){
        $conf{$server_uuid}{'threads_per_core'} = $server_info->{'threadsPerCore'};
      }
      # the number of enabled processor cores. 
      if (defined $server_info->{'enabledProcessorCores'} && $server_info->{'enabledProcessorCores'} ne ""){
        $conf{$server_uuid}{'enabled_cpu_cores'} = $server_info->{'enabledProcessorCores'};
      }
      # the number of populated processor sockets
      if (defined $server_info->{'populatedProcessorSockets'} && $server_info->{'populatedProcessorSockets'} ne ""){
        $conf{$server_uuid}{'cpu_sockets'} = $server_info->{'populatedProcessorSockets'};
      }
      if (defined $server_info->{'processorType'} && $server_info->{'processorType'} ne ""){
        $conf{$server_uuid}{'cpu_type'} = $server_info->{'processorType'};
      }
      if (defined $server_info->{'ipAddress'} && $server_info->{'ipAddress'} ne ""){
        $conf{$server_uuid}{'ip_address'} = $server_info->{'ipAddress'};
      }
      if (defined $server_info->{'manufacturer'} && $server_info->{'manufacturer'} ne ""){
        $conf{$server_uuid}{'manufac'} = $server_info->{'manufacturer'};
      }
      if (defined $server_info->{'productName'} && $server_info->{'productName'} ne ""){
        $conf{$server_uuid}{'product_name'} = $server_info->{'productName'};
      }
      if (defined $server_info->{'serialNumber'} && $server_info->{'serialNumber'} ne ""){
        $conf{$server_uuid}{'serial_num'} = $server_info->{'serialNumber'};
      }
      if (defined $server_info->{'biosVersion'} && $server_info->{'biosVersion'} ne ""){
        $conf{$server_uuid}{'bios_version'} = $server_info->{'biosVersion'};
      }
      if ( defined $server_info->{'hypervisor'} && $server_info->{'hypervisor'} ne "" ) {
        if ( defined $server_info->{'hypervisor'}{'type'} && $server_info->{'hypervisor'}{'type'} ne "" ) {
          $conf{$server_uuid}{'hypervisor_type'} = $server_info->{'hypervisor'}{'type'};
        }
        if ( defined $server_info->{'hypervisor'}{'version'} && $server_info->{'hypervisor'}{'version'} ne "" ) {
          $conf{$server_uuid}{'hypervisor_version'} = $server_info->{'hypervisor'}{'version'};
        }
      }
      # ServerPool info
      my $server_pool_uuid = "";
      if ( defined $server_info->{'serverPoolId'} && $server_info->{'serverPoolId'} ne "" ) {
        if ( defined $server_info->{'serverPoolId'}{'value'} && $server_info->{'serverPoolId'}{'value'} ne "" ) {
          $server_pool_uuid = $server_info->{'serverPoolId'}{'value'};
          $server_pool_uuid    =~ s/://g;
          $conf{$server_uuid}{'serverpool_uuid'} = $server_pool_uuid;
        }
        if ( defined $server_info->{'serverPoolId'}{'name'} && $server_info->{'serverPoolId'}{'name'} ne "" ) {
          $conf{$server_uuid}{'serverpool_name'} = $server_info->{'serverPoolId'}{'name'};
          # Hash for architecture
          $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'serverpool_name'}                           = "$server_info->{'serverPoolId'}{'name'}";
          $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_name'}       = "$server_info->{'name'}";
        }
      }
      # VM info
      if ( defined $server_info->{'vmIds'} && $server_info->{'vmIds'} ne "" ) {
        foreach ( @{ $server_info->{'vmIds'} } ) {
          #print Dumper $_;
          my $vm_uuid = $_->{'value'};
          $vm_uuid    =~ s/://g;
          my $vm_name = $_->{'name'};
          # Hash for architecture
          $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_name'}   = "$vm_name";
          # VM - DISK interface
          if ( $info_about_repos ne "0" ) {
            foreach my $repos_info ( @{ $info_about_repos } ) {
              if ( defined $repos_info->{'vmIds'} && $repos_info->{'vmIds'} ne "" ) {
                foreach ( @{ $repos_info->{'vmIds'} } ) {
                  my $vm_name_repo = $_->{'name'};
                  if ($vm_name_repo ne "$vm_name") {next}
                  if ( defined $repos_info->{'virtualDiskIds'} && $repos_info->{'virtualDiskIds'} ne "" ) {
                    foreach ( @{ $repos_info->{'virtualDiskIds'} } ) {
                      my $disk_name = $_->{'name'};
                      $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_disk'}{$disk_name} = $disk_name;
                    }
                  }
                }
              }
            }
          }
          foreach my $metric (@net_metrics){
            my $start_time_net = $end_time - 20000;
            my $db_url_stat       = "https://$host:$port/ovm/core/wsapi/rest/Statistic";
            my $info_about_vm_net = "$db_url_stat/Vm/$vm_uuid/$metric/range?startTime=$start_time_net&endTime=$end_time";
            my $vm_net_data = new_get_request($info_about_vm_net, $username, $password );
            if ( $vm_net_data ne "0" ) {
              foreach my $stat ( @{ $vm_net_data } ){
                my $net_name = "";
                if (defined $stat->{'component'} && $stat->{'component'} ne ""){
                  $net_name  = $stat->{'component'};
                }
                $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_interface'}{$net_name} = $net_name;
              }
            }
          }
        }
      }
      # SERVER - NET interface
      if ( defined $server_info->{'ethernetPortIds'} && $server_info->{'ethernetPortIds'} ne "" ) {
        foreach ( @{ $server_info->{'ethernetPortIds'} } ) {
          my $net_name = $_->{'name'};
          $net_name    =~ s/\s.*//;
          if ( $net_name !~ /^net|^eth|^em/ ) { next; }
          $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_interface'}{$net_name} = $net_name;
        }
      }
      # SERVER - DISK interface (info from PERF statistic, cause the CONF statistics do not contain it )
      foreach my $metric (@disk_metrics){
        my $start_time_disk = $end_time - 20000;
        my $db_url_stat       = "https://$host:$port/ovm/core/wsapi/rest/Statistic";
        my $info_about_server_disk = "$db_url_stat/Server/$server_uuid_for_url/$metric/range?startTime=$start_time_disk&endTime=$end_time";
        my $server_disk_data = new_get_request($info_about_server_disk, $username, $password );
        if ( $server_disk_data ne "0" ) {
          foreach my $stat ( @{ $server_disk_data } ){
            my $disk_name = "";
            if (defined $stat->{'component'} && $stat->{'component'} ne ""){
              $disk_name  = $stat->{'component'};
            }
            $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'disk_interface'}{$disk_name} = $disk_name;
          }
        }
      }
    }
  }
  ############
  ##   VM   ##
  ############
  if ( $info_about_vm ne "0" ) {
    foreach my $vm_info ( @{ $info_about_vm } ) {
      #print Dumper $vm_info;
      my $vm_uuid     = "";
      my $server_uuid = "";
      if ( defined $vm_info->{'id'} && $vm_info->{'id'} ne "" ) {
        if ( defined $vm_info->{'id'}{'value'} && $vm_info->{'id'}{'value'} ne "" ) {
          my $uuid = $vm_info->{'id'}{'value'};
          $uuid =~ s/://g;
          $vm_uuid = $uuid;
        }
      }
      if (defined $vm_info->{'name'} && $vm_info->{'name'} ne ""){
        $conf{$vm_uuid}{'vm_name'}   = $vm_info->{'name'};
      }
      if (defined $vm_info->{'hostname'} && $vm_info->{'hostname'} ne ""){
        $conf{$vm_uuid}{'hostname'}   = $vm_info->{'hostname'};
      }
      my $manager_uuid = "";
      if (defined $vm_info->{'managerUuid'} && $vm_info->{'managerUuid'} ne ""){
        $conf{$vm_uuid}{'managerUuid'}   = $vm_info->{'managerUuid'};
        $manager_uuid = $vm_info->{'managerUuid'};
      }
      if (defined $vm_info->{'memory'} && $vm_info->{'memory'} ne ""){
        my $vm_total_memory = $vm_info->{'memory'} * 1000000;
        $conf{$vm_uuid}{'total_memory'} = $vm_total_memory;
      }
      if (defined $vm_info->{'currentMemory'} && $vm_info->{'currentMemory'} ne ""){
        $conf{$vm_uuid}{'current_mem'} = $vm_info->{'currentMemory'};
      }
      if (defined $vm_info->{'osType'} && $vm_info->{'osType'} ne ""){
        $conf{$vm_uuid}{'os_type'} = $vm_info->{'osType'};
      }
      if (defined $vm_info->{'cpuCount'} && $vm_info->{'cpuCount'} ne ""){
        $conf{$vm_uuid}{'cpu_count'} = $vm_info->{'cpuCount'};
      }
      if (defined $vm_info->{'vmDomainType'} && $vm_info->{'vmDomainType'} ne ""){
        $conf{$vm_uuid}{'domain_type'} = $vm_info->{'vmDomainType'};
      }
      # Server info
      if ( defined $vm_info->{'serverId'} && $vm_info->{'serverId'} ne "" ) {
        if ( defined $vm_info->{'serverId'}{'value'} && $vm_info->{'serverId'}{'value'} ne "" ) {
          $server_uuid = $vm_info->{'serverPoolId'}{'value'};
          $server_uuid    =~ s/://g;
          $conf{$vm_uuid}{'server_uuid'} = $server_uuid;

        }
        if ( defined $vm_info->{'serverId'}{'name'} && $vm_info->{'serverId'}{'name'} ne "" ) {
          $conf{$vm_uuid}{'server_name'} = $vm_info->{'serverId'}{'name'};
        }
      }
      # ServerPool info
      my $server_pool_uuid = "";
      if ( defined $vm_info->{'serverPoolId'} && $vm_info->{'serverPoolId'} ne "" ) {
        if ( defined $vm_info->{'serverPoolId'}{'value'} && $vm_info->{'serverPoolId'}{'value'} ne "" ) {
          $server_pool_uuid = $vm_info->{'serverPoolId'}{'value'};
          $server_pool_uuid    =~ s/://g;
          $conf{$vm_uuid}{'serverpool_uuid'} = $server_pool_uuid;
        }
        if ( defined $vm_info->{'serverPoolId'}{'name'} && $vm_info->{'serverPoolId'}{'name'} ne "" ) {
          $conf{$vm_uuid}{'serverpool_name'} = $vm_info->{'serverPoolId'}{'name'};
        }
      }
    }
  }

  # Push Manager->ServerPool->Server/VM relations to architecture
  #
  #############
  ## Manager ##
  #############
  if ($info_about_api ne "0"){
    foreach my $info_api ( @{ $info_about_api}  ) {
      my %manager_arch = (
        "item_id" => $info_api->{'managerUuid'},
        "label" => $cfg->{hostalias},
        "hw_type" => "oraclevm",
        "subsystem" => "manager",
        "parents" => [],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
      push(@architecture, \%manager_arch);
    }
  }
  ###########################
  ## Server pool/Server/VM ##
  ###########################
  if ( defined $conf_to_hash{'manager'} && $conf_to_hash{'manager'} ne "" ){
    foreach my $manager_uuid ( keys( %{ $conf_to_hash{'manager'}} ) ) {
      if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'} ne "" ){
        foreach my $server_pool_uuid ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}} ) ) {
          my $server_pool_label = "";
          if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'serverpool_name'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'serverpool_name'} ne "" ){
            $server_pool_label = "$conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'serverpool_name'}";
          }
          my %server_pool_arch = (
            "item_id" => $server_pool_uuid,
            "label" => $server_pool_label,
            "hw_type" => "oraclevm",
            "subsystem" => "server_pool",
            "parents" => [$manager_uuid],
            "hostcfg_id" => $cfg->{'hostcfg_id'}
          );
          #print Dumper \%server_pool_arch;
          push(@architecture, \%server_pool_arch);
          if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'} ne "" ){
            foreach my $server_uuid ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}} ) ) {
              my $server_label = "";
              if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_name'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_name'} ne "" ){
                $server_label = "$conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_name'}";
              }
              my %server_arch = (
                "item_id" => $server_uuid,
                "label" => $server_label,
                "hw_type" => "oraclevm",
                "subsystem" => "server",
                "parents" => [$server_pool_uuid,$manager_uuid],
                "hostcfg_id" => $cfg->{'hostcfg_id'}
              );
              #print Dumper \%server_arch;
              push(@architecture, \%server_arch);
              if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'} ne "" ){
                foreach my $vm_uuid ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}} ) ) {
                  my $vm_label = "";
                  if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid} ne "" ){
                    $vm_label = "$conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_name'}";
                  }
                  my %vm_arch = (
                    "item_id" => $vm_uuid,
                    "label" => $vm_label,
                    "hw_type" => "oraclevm",
                    "subsystem" => "vm",
                    "parents" => [$server_uuid,$server_pool_uuid,$manager_uuid],
                    "hostcfg_id" => $cfg->{'hostcfg_id'}
                  );
                  #print Dumper \%vm_arch;
                  push(@architecture, \%vm_arch);
                }
              }
              if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'} ne "" ){
                foreach my $vm_uuid ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}} ) ) {
                  if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_disk'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_disk'} ne "" ){
                    foreach my $disk_name ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_disk'}} ) ) {
                      my $disk_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$vm_uuid" . $disk_name);
                      my %vm_arch_disk = (
                        "item_id" => $disk_uuid,
                        "label" => $disk_name,
                        "hw_type" => "oraclevm",
                        "subsystem" => "vm_disk",
                        "parents" => [$vm_uuid,$server_uuid,$server_pool_uuid,$manager_uuid],
                        "hostcfg_id" => $cfg->{'hostcfg_id'}
                      );
                      #print Dumper \%vm_arch_disk;
                      push(@architecture, \%vm_arch_disk);
                    }
                  }
                }
                foreach my $vm_uuid ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}} ) ) {
                  if ( defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_interface'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_interface'} ne "" ){
                    foreach my $net_name ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'vm'}{$vm_uuid}{'vm_interface'}} ) ) {
                      my $net_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$vm_uuid" . $net_name);
                      my %vm_arch_interface = (
                        "item_id" => $net_uuid,
                        "label" => $net_name,
                        "hw_type" => "oraclevm",
                        "subsystem" => "vm_interface",
                        "parents" => [$vm_uuid,$server_uuid,$server_pool_uuid,$manager_uuid],
                        "hostcfg_id" => $cfg->{'hostcfg_id'}
                      );
                      #print Dumper \%vm_arch_interface;
                      push(@architecture, \%vm_arch_interface);
                    }
                  }
                }
              }
              if (defined $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_interface'} && $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_interface'} ne "" ){
                foreach my $interface_name ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'server_interface'}} ) ) {
                  my $interface_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$server_uuid" . $interface_name);
                  my %server_arch_interface = (
                    "item_id" => $interface_uuid,
                    "label" => $interface_name,
                    "hw_type" => "oraclevm",
                    "subsystem" => "server_interface",
                    "parents" => [$server_uuid,$server_pool_uuid,$manager_uuid],
                    "hostcfg_id" => $cfg->{'hostcfg_id'}
                  );
                  #print Dumper \%server_arch_interface;
                  push(@architecture, \%server_arch_interface);
                }
                foreach my $disk_name ( keys( %{ $conf_to_hash{'manager'}{$manager_uuid}{'server_pool'}{$server_pool_uuid}{'server'}{$server_uuid}{'disk_interface'}} ) ) {
                  my $disk_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$server_uuid" . $disk_name);
                  my %server_arch_disk = (
                    "item_id" => $disk_uuid,
                    "label" => $disk_name,
                    "hw_type" => "oraclevm",
                    "subsystem" => "server_disk",
                    "parents" => [$server_uuid,$server_pool_uuid,$manager_uuid],
                    "hostcfg_id" => $cfg->{'hostcfg_id'}
                  );
                  #print Dumper \%server_arch_disk;
                  push(@architecture, \%server_arch_disk);
                }
              }
            }
          }
        }
      }
    }
  }
  #print "\n==CONF_TO_HASH==\n";
  #print Dumper \%conf_to_hash;

  #save architecture
  #print "================= ARCHITECTURE =================\n";
  #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

  #save configurations
  $xormon->saveConfJSON(\%conf);
  #print "================= CONF =================\n";
  #print Dumper \%conf;

}

sub create_perf {
  my $host        = shift;
  my $port        = shift;
  my $username    = shift;
  my $password    = shift;
  my $cfg         = shift;
  my $start_time  = shift;
  my $end_time    = shift;

  my $service_data = $xormon->getData();
  my $timestamp_start = (defined $service_data->{last_perf_timestamp}) ? $service_data->{last_perf_timestamp} : time()-1200;
  my $timestamp_end = time()-180;

  my @architecture;

  #print "\n===============SERVER fetching config data=================\n";
  #print "HOST:$host,PORT:$port,PAS:$password,user:$username\n";

  # Values for collecting values from Server,VM,ServerPool etc
  my $db_url_server     = "https://$host:$port/ovm/core/wsapi/rest/Server";
  my $db_url_vm         = "https://$host:$port/ovm/core/wsapi/rest/Vm";
  my $db_url_repos      = "https://$host:$port/ovm/core/wsapi/rest/Repository";

  # List of Server/VM
  my $db_url_server_id  = "https://$host:$port/ovm/core/wsapi/rest/Server/id";
  my $db_url_vm_id      = "https://$host:$port/ovm/core/wsapi/rest/Vm/id";

  my $db_url_stat       = "https://$host:$port/ovm/core/wsapi/rest/Statistic";
  # REST API INFO
  my $url_stat_rest_api  = "https://$host:$port/ovm/core/wsapi/rest/Manager";
  my $info_about_api     = new_get_request($url_stat_rest_api, $username, $password);

  # Request on collecting config DATA
  my $info_about_server  = new_get_request($db_url_server,$username,$password );
  my $info_about_vm      = new_get_request($db_url_vm,$username,$password );
  my $info_about_repos   = new_get_request($db_url_repos,$username,$password );

  # Request on collecting Server/VM/ServerPool names & ids list
  my $server_list        = new_get_request($db_url_server_id,$username,$password );
  my $vm_list            = new_get_request($db_url_vm_id,$username,$password );

  # Server list
  #print Dumper $server_list;
  # Server conf
  #print Dumper $info_about_server;
  # VM list
  #print Dumper $vm_list;
  # VM conf
  #print Dumper $info_about_vm;

  # SERVER PERF DATA
  if ( $server_list ne "0" ) {
    foreach my $metric (@server_metrics) {
      #print "Server request for $metric metric\n";
      foreach my $server ( @{ $server_list } ) {
        #print Dumper $server;
        my ($server_name,$server_uuid,$uuid) = "";
        if (defined $server->{'name'} && $server->{'name'} ne ""){
          $server_name = "$server->{'name'}";
        }
        if (defined $server->{'value'} && $server->{'value'} ne ""){
          $server_uuid = "$server->{'value'}";
          $uuid        = $server_uuid;
          $server_uuid =~ s/://g;
        }
        # Server - info from all metrics
        my $new_db_url = "$db_url_stat/Server/$uuid/$metric/range?startTime=$start_time&endTime=$end_time";
        my $time1 = $start_time / 1000;
        my $time2 = $end_time / 1000;
        my $start_human_date = strftime("%Y-%m-%d %H:%M:%S", localtime($time1 ));
        my $end_human_date   = strftime("%Y-%m-%d %H:%M:%S", localtime($time2 ));
        #print "$new_db_url - from:$start_human_date to:$end_human_date\n";
        my $server_data = new_get_request($new_db_url, $username, $password );
        if ( $server_data ne "0" ) {
          # ARRAY of HASH about Server info
          #print Dumper $server_data;
          my $i_test = 0;
          foreach my $stat ( @{ $server_data } ) {
            #print Dumper $stat;
            my $value    = "";
            my $start_t  = "";
            if (defined $stat->{'value'} && $stat->{'value'} ne ""){
              $value   = $stat->{'value'};
            }
            if (defined $stat->{'startTime'} && $stat->{'startTime'} ne ""){
              $start_t = $stat->{'startTime'};
              $start_t = $start_t / 1000;
              $start_t = sprintf '%.0f', $start_t;
            }
            if ( $metric =~ /CPU_UTILIZATION|MEMORY_UTILIZATION/ ) {
              $value = $value*100;
              $value = sprintf '%.2f', $value;
            }
            elsif ( $metric =~ /MEMORY_USED|FREE_MEMORY|FREE_SWAP/ ) {
              if ($metric ne "FREE_SWAP"){
                $value = Xormon::returnBytes( $value, "MB" );
              }
              else{
                $value = Xormon::returnBytes( $value, "kB" );
              }
            }
            else {
              $value = sprintf '%.2f', $value;
            }
            # save data
            if ( $metric eq "CPU_UTILIZATION" ) {
              foreach my $server_info ( @{ $info_about_server } ) {
                my $count_cpu           = scalar( @{ $server_info->{cpuIds} } );
                my ($host_name, $cpu_cores_per_socket, $cpu_proc_socket, $total_proc_cores, $enab_proc_cores, $thread_per_core) = "";
                if (defined $server_info->{'name'} && $server_info->{'name'} ne ""){
                  $host_name            = $server_info->{'name'};
                }
                # the number of populated processor sockets
                if (defined $server_info->{'populatedProcessorSockets'} && $server_info->{'populatedProcessorSockets'} ne ""){
                  $cpu_proc_socket      = $server_info->{'populatedProcessorSockets'};
                }
                # the number of cores per processor socket
                if (defined $server_info->{'coresPerProcessorSocket'} && $server_info->{'coresPerProcessorSocket'} ne ""){
                  $cpu_cores_per_socket = $server_info->{'coresPerProcessorSocket'};
                }
                # the number of processor cores
                if (defined $server_info->{'totalProcessorCores'} && $server_info->{'totalProcessorCores'} ne ""){
                  $total_proc_cores     = $server_info->{'totalProcessorCores'};
                }
                # the number of enabled processor cores
                if (defined $server_info->{'enabledProcessorCores'} && $server_info->{'enabledProcessorCores'} ne ""){
                  $enab_proc_cores      = $server_info->{'enabledProcessorCores'};
                }
                # the number of threads per core
                if (defined $server_info->{'threadsPerCore'} && $server_info->{'threadsPerCore'} ne ""){
                  $thread_per_core      = $server_info->{'threadsPerCore'};
                }
                if ( $host_name eq $server_name ) {
                  #my $cpu_test           = $enab_proc_cores * $thread_per_core;
                  #my $cpu_one_perc       = $count_cpu/100; # one percent per socket
                  #my $cpu_util_in_cores  = $cpu_one_perc*$value;
                  my $cpu1 = $enab_proc_cores/100;
                  my $cpu_util_in_cores = $cpu1 * $value;
                  $data{'server'}{'perf'}{$start_t}{$server_uuid}{$server_name}{COUNT_CORES}    = $enab_proc_cores;
                  $data{'server'}{'perf'}{$start_t}{$server_uuid}{$server_name}{COUNT_CPU}      = $count_cpu;
                  $data{'server'}{'perf'}{$start_t}{$server_uuid}{$server_name}{$metric}        = $cpu_util_in_cores;
                  $data{'server'}{'perf'}{$start_t}{$server_uuid}{$server_name}{CPU_PERC}       = $value;
                  # Total memory from conf
                  if (defined $server_info->{'memory'} && $server_info->{'memory'} ne ""){
                    my $total_memory = $server_info->{'memory'} * 1000000;
                    $data{'server'}{'perf'}{$start_t}{$server_uuid}{$server_name}{MEMORY_TOTAL}   = $total_memory;
                  }
                }
              }
            }
            else{
              my $timestamp_test = $start_t/1000;
              my $human_date = strftime("%Y-%m-%d %H:%M:%S", localtime($timestamp_test ));
              #print "$server_name,$human_date,$value($metric)\n";
              $data{'server'}{'perf'}{$start_t}{$server_uuid}{$server_name}{$metric} = $value;
            }
          }
        }
        else {
          Xormon::log("ERROR Server statistic");
        }
      }
    }

    # SERVER - NETWORK and DISK (counter values)
    foreach my $metric (@counter_metrics) {
      foreach my $server ( @{ $server_list } ) {
        #print Dumper $server;
        my ($server_name,$server_uuid,$uuid) = "";
        if (defined $server->{'name'} && $server->{'name'} ne ""){
          $server_name = "$server->{'name'}";
        }
        if (defined $server->{'value'} && $server->{'value'} ne ""){
          $server_uuid = "$server->{'value'}";
          $uuid        = $server_uuid;
          $server_uuid =~ s/://g;
        }
        # Server - info from all metrics
        my $new_db_url = "$db_url_stat/Server/$uuid/$metric/range?startTime=$start_time&endTime=$end_time";
        my $time1 = $start_time / 1000;
        my $time2 = $end_time / 1000;
        my $start_human_date = strftime("%Y-%m-%d %H:%M:%S", localtime($time1 ));
        my $end_human_date   = strftime("%Y-%m-%d %H:%M:%S", localtime($time2 ));
        #print "$new_db_url - from:$start_human_date to:$end_human_date\n";
        my $interface_data = new_get_request($new_db_url, $username, $password );
        if ( $interface_data ne "0" ) {
          #print Dumper $interface_data;
          foreach my $stat ( @{ $interface_data } ) {
            #print Dumper $stat;
            my $value    = "";
            my $start_t  = "";
            my $interface_name = "";
            if (defined $stat->{'value'} && $stat->{'value'} ne ""){
              $value   = $stat->{'value'};
            }
            if (defined $stat->{'startTime'} && $stat->{'startTime'} ne ""){
              $start_t = $stat->{'startTime'};
              $start_t = $start_t / 1000;
              $start_t = sprintf '%.0f', $start_t;
            }
            if (defined $stat->{'component'} && $stat->{'component'} ne ""){
              $interface_name = $stat->{'component'};
            }
            my $start_date = strftime("%Y-%m-%d %H:%M:%S", localtime($start_t ));
            $start_date =~ s/:\d+$//;
            if ($metric =~ /NETWORK/){
              if ( $interface_name !~ /^net|^eth|^em/ ) { next; }
              my $interface_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$server_uuid" . $interface_name);
              # Prepare hash for counting counters
              $server_interface_hash{'server'}{$interface_uuid}{$metric}{$start_date} = $value;
            }
            else{ # DISK
              my $disk_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$server_uuid" . $interface_name);
              # Prepare hash for counting counters
              $server_disk_hash{'server'}{$disk_uuid}{$metric}{$start_date} = $value;
            }
          }
        }
      }
    }
  }
  else {
    Xormon::log("Not found Server IDs");
  }
  #print Dumper \%server_interface_hash ;
  #print Dumper \%server_disk_hash ;

  foreach my $interface_uuid ( keys %{$server_interface_hash {'server'}} ) {
    foreach my $metric ( keys %{$server_interface_hash {'server'}{$interface_uuid}} ) {
      my $last_value  = 0;
      my $first_run   = 0;
      my $first_value = 0;
      foreach my $timestamp ( sort keys %{$server_interface_hash {'server'}{$interface_uuid}{$metric}} ) {
        my $value      = $server_interface_hash {'server'}{$interface_uuid}{$metric}{$timestamp};
        if ($first_run == 0){
          $last_value  = $value;
        }
        else{
          my $diff_value = $value - $last_value;
          $diff_value    = $diff_value / 60;
          $diff_value    = sprintf '%.0f', $diff_value;
          my ($year, $month, $day, $hour, $minute) = $timestamp =~ /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2})$/;
          my $time = timelocal(0, $minute, $hour, $day, $month - 1, $year - 1900);
          $data_server_net{'server'}{'perf'}{$time}{$interface_uuid}{$metric} = $diff_value;
          $last_value = $value;
        }
        $first_run = 1;
      }
    }
  }
  #print Dumper \%data_server_net;

  foreach my $disk_uuid ( keys %{$server_disk_hash  {'server'}} ) {
    foreach my $metric ( keys %{$server_disk_hash  {'server'}{$disk_uuid}} ) {
      my $last_value  = 0;
      my $first_run   = 0;
      my $first_value = 0;
      foreach my $timestamp ( sort keys %{$server_disk_hash  {'server'}{$disk_uuid}{$metric}} ) {
        my $value      = $server_disk_hash  {'server'}{$disk_uuid}{$metric}{$timestamp};
        if ($first_run == 0){
          $last_value  = $value;
        }
        else{
          my $diff_value = $value - $last_value;
          $diff_value    = $diff_value / 60;
          $diff_value    = sprintf '%.0f', $diff_value;
          my ($year, $month, $day, $hour, $minute) = $timestamp =~ /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2})$/;
          my $time = timelocal(0, $minute, $hour, $day, $month - 1, $year - 1900);
          $data_server_disk{'server'}{'perf'}{$time}{$disk_uuid}{$metric} = $diff_value;
          $last_value = $value;
        }
        $first_run = 1;
      }
    }
  }
  #print Dumper \%data_server_disk;

  # VM PERF DATA
  if ( $vm_list ne "0" ) {
    foreach my $metric (@vm_metrics) {
      #print "VM request for $metric metric\n";
      foreach my $vm ( @{ $vm_list } ) {
        #print Dumper $vm;
        my $vm_name = "";
        my $vm_uuid = "";
        my $uuid    = "";
        if (defined $vm->{'name'} && $vm->{'name'} ne ""){
          $vm_name = "$vm->{'name'}";
        }
        if (defined $vm->{'value'} && $vm->{'value'} ne ""){
          $vm_uuid     = "$vm->{'value'}";
          $uuid        = $vm_uuid;
          $vm_uuid     =~ s/://g;
          if ($metric eq "CPU_COUNT"){
            $uuid     =~ s/://g;
          }
        }
        # Server - info from all metrics
        my $new_db_url  = "$db_url_stat/Vm/$uuid/$metric/range?startTime=$start_time&endTime=$end_time";
        my $time1 = $start_time / 1000;
        my $time2 = $end_time / 1000;
        my $start_human_date = strftime("%Y-%m-%d %H:%M:%S", localtime($time1 ));
        my $end_human_date   = strftime("%Y-%m-%d %H:%M:%S", localtime($time2 ));
        #print "$new_db_url from:$start_human_date to:$end_human_date\n";
        my $vm_data = new_get_request($new_db_url, $username, $password );
        if ( $vm_data ne "0" ) {
          foreach my $stat ( @{ $vm_data } ){
            #print Dumper $stat;
            my ($value,$start_t) = "";
            if (defined $stat->{'value'} && $stat->{'value'} ne ""){
              $value   = $stat->{'value'};
            }
            if (defined $stat->{'startTime'} && $stat->{'startTime'} ne ""){
              $start_t = $stat->{'startTime'};
              $start_t = $start_t / 1000;
              $start_t = sprintf '%.0f', $start_t;
            }
            # save data
            if ( $metric =~ /CPU_UTILIZATION/ ) {
              foreach my $vm_info ( @{ $info_about_vm } ) {
                my $cpu_count = "";
                my $host_name = "";
                if (defined $vm_info->{'name'} && $vm_info->{'name'} ne ""){
                  $host_name            = $vm_info->{'name'};
                }
                if (defined $vm_info->{'cpuCount'} && $vm_info->{'cpuCount'} ne ""){
                  $cpu_count     = $vm_info->{'cpuCount'};
                }
                if ($host_name eq $vm_name){
                  $value = $value*100;
                  $value = sprintf '%.2f', $value;
                  my $cpu_one_perc       = $cpu_count/100; # one percent per socket
                  my $cpu_util_in_cores  = $cpu_one_perc*$value;
                  my $cpu1               = $value/100;
                  my $cpu_cores          = $cpu_util_in_cores/$cpu1;
                  $data{'vm'}{'perf'}{$start_t}{$vm_uuid}{$vm_name}{CPU_CORES} = $cpu_cores;
                  $data{'vm'}{'perf'}{$start_t}{$vm_uuid}{$vm_name}{CPU_PERC}  = $value;
                  $data{'vm'}{'perf'}{$start_t}{$vm_uuid}{$vm_name}{$metric}   = $cpu_util_in_cores;
                }
              }
            }
            elsif ($metric =~ /NETWORK_SENT|NETWORK_RECEIVED|DISK_READ|DISK_WRITE/ ) {
              my $net_name = "";
              my $start_date = strftime("%Y-%m-%d %H:%M:%S", localtime($start_t ));
              $start_date =~ s/:\d+$//;
              if (defined $stat->{'component'} && $stat->{'component'} ne ""){
                $net_name = $stat->{'component'};
              }
              if ($metric =~ /NETWORK/){
                my $interface_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$vm_uuid" . $net_name);
                # Prepare hash for counting counters
                $vm_interface_hash{'vm'}{$interface_uuid}{$metric}{$start_date} = $value;
              }
              elsif ($metric =~ /DISK/){
                if ( $info_about_repos ne "0" ) {
                  foreach my $repos_info ( @{ $info_about_repos } ) {
                    if ( defined $repos_info->{'virtualDiskIds'} && $repos_info->{'virtualDiskIds'} ne "" ) {
                      foreach ( @{ $repos_info->{'virtualDiskIds'} } ) {
                        my $disk_name_uuid = $_->{'value'};
                        if ($disk_name_uuid ne "$net_name") { next }
                        my $disk_name = $_->{'name'};
                        my $disk_uuid = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "$vm_uuid" . $disk_name);
                        # Prepare hash for counting counters
                        $vm_disk_hash{'vm'}{$disk_uuid}{$metric}{$start_date} = $value;
                      }
                    }
                  }
                }
              }
            }
            else{
              if ( $metric =~ /MEMORY_USED/ ) {
                $value = Xormon::returnBytes( $value, "MB" );
              }
              $data{'vm'}{'perf'}{$start_t}{$vm_uuid}{$vm_name}{$metric} = $value;
            }
          }
        }
      }
    }
  }
  else {
    Xormon::log("Not found VM IDs");
  }
  #print Dumper \%vm_interface_hash;
  #print Dumper \%vm_disk_hash;

  # VM - NETWORK and DISK (counter values)
  foreach my $interface_uuid ( keys %{$vm_interface_hash {'vm'}} ) {
    foreach my $metric ( keys %{$vm_interface_hash {'vm'}{$interface_uuid}} ) {
      my $last_value  = 0;
      my $first_run   = 0;
      my $first_value = 0;
      foreach my $timestamp ( sort keys %{$vm_interface_hash {'vm'}{$interface_uuid}{$metric}} ) {
        my $value      = $vm_interface_hash {'vm'}{$interface_uuid}{$metric}{$timestamp};
        if ($first_run == 0){
          $last_value  = $value;
        }
        else{
          my $diff_value = $value - $last_value;
          $diff_value    = $diff_value / 60;
          $diff_value    = sprintf '%.0f', $diff_value;
          my ($year, $month, $day, $hour, $minute) = $timestamp =~ /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2})$/;
          my $time = timelocal(0, $minute, $hour, $day, $month - 1, $year - 1900);
          $data_vm_net{'vm'}{'perf'}{$time}{$interface_uuid}{$metric} = $diff_value;
          $last_value = $value;
        }
        $first_run = 1;
      }
    }
  }
  #print Dumper \%data_server_net;

  foreach my $disk_uuid ( keys %{$vm_disk_hash {'vm'}} ) {
    foreach my $metric ( keys %{$vm_disk_hash {'vm'}{$disk_uuid}} ) {
      my $last_value  = 0;
      my $first_run   = 0;
      my $first_value = 0;
      foreach my $timestamp ( sort keys %{$vm_disk_hash {'vm'}{$disk_uuid}{$metric}} ) {
        my $value      = $vm_disk_hash {'vm'}{$disk_uuid}{$metric}{$timestamp};
        if ($first_run == 0){
          $last_value  = $value;
        }
        else{
          my $diff_value = $value - $last_value;
          $diff_value    = $diff_value / 60;
          $diff_value    = sprintf '%.0f', $diff_value;
          my ($year, $month, $day, $hour, $minute) = $timestamp =~ /^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2})$/;
          my $time = timelocal(0, $minute, $hour, $day, $month - 1, $year - 1900);
          $data_vm_disk{'vm'}{'perf'}{$time}{$disk_uuid}{$metric} = $diff_value;
          $last_value = $value;
        }
        $first_run = 1;
      }
    }
  }
  #print Dumper \%data_disk_net;

}

sub save_data_to_csv{
  my $rc;
  my $status;
  my $data_out            = "";
  my $last_perf_timestamp = "";

  ############
  ## Server ##
  ############
  my $htype = "server";
  $data_out .= "uuid;time;cpu_utilization_cores;cpu_utilization_percent;cpu_count;cpu_cores;free_mem;used_mem;free_swap;mem_perc;total_mem\n";
  foreach my $time ( sort keys %{ $data{$htype}{'perf'} } ) {
    foreach my $server_uuid ( sort keys %{ $data{$htype}{'perf'}{$time} } ) {
      foreach my $server_name ( sort keys %{ $data{$htype}{'perf'}{$time}{$server_uuid} } ) {
        my $perf_ref         = $data{$htype}{'perf'}{$time}{$server_uuid}{$server_name};
        my $cpu_util    = "";
        my $cpu_perc    = "";
        my $cpu_cores   = "";
        my $cpu_count   = "";
        my $free_mem    = "";
        my $used_mem    = "";
        my $mem_perc    = "";
        my $free_swap   = "";
        my $total_mem   = "";
        if ( defined $perf_ref->{'CPU_UTILIZATION'} && $perf_ref->{'CPU_UTILIZATION'} ne ""){
          $cpu_util    = $perf_ref->{'CPU_UTILIZATION'};
        }
        if ( defined $perf_ref->{'CPU_PERC'} && $perf_ref->{'CPU_PERC'} ne ""){
          $cpu_perc    = $perf_ref->{'CPU_PERC'};
        }
        if ( defined $perf_ref->{'COUNT_CORES'} && $perf_ref->{'COUNT_CORES'} ne ""){
          $cpu_cores = $perf_ref->{'COUNT_CORES'};
        }
        if ( defined $perf_ref->{'COUNT_CPU'} && $perf_ref->{'COUNT_CPU'} ne ""){
          $cpu_count  = $perf_ref->{'COUNT_CPU'};
        }
        if ( defined $perf_ref->{'FREE_MEMORY'} && $perf_ref->{'FREE_MEMORY'} ne ""){
          $free_mem    = $perf_ref->{'FREE_MEMORY'};
        }
        if ( defined $perf_ref->{'MEMORY_USED'} && $perf_ref->{'MEMORY_USED'} ne ""){
          $used_mem    = $perf_ref->{'MEMORY_USED'};
        }
        if ( defined $perf_ref->{'FREE_SWAP'} && $perf_ref->{'FREE_SWAP'} ne ""){
          $free_swap   = $perf_ref->{'FREE_SWAP'};
        }
        if ( defined $perf_ref->{'MEMORY_TOTAL'} && $perf_ref->{'MEMORY_TOTAL'} ne ""){
          $total_mem   = $perf_ref->{'MEMORY_TOTAL'};
        }
        if ( defined $perf_ref->{'MEMORY_UTILIZATION'} && $perf_ref->{'MEMORY_UTILIZATION'} ne ""){
          $mem_perc    = $perf_ref->{'MEMORY_UTILIZATION'};
        }
        $data_out  .= "$server_uuid;$time;$cpu_util;$cpu_perc;$cpu_count;$cpu_cores;$free_mem;$used_mem;$free_swap;$mem_perc;$total_mem\n";
        #print "$server_uuid;$time;cpu_util:$cpu_util;cpu_perc:$cpu_perc;count_cores:$count_cores;count_cpu:$count_cpu;free_mem:$free_mem;used_mem:$used_mem;free_swap:$free_swap;mem_perc:$mem_perc\n";
      }
    }
    $last_perf_timestamp = $time;
  }
  # Save to DB
  $rc = $xormon->saveCSV($data_out,"oraclevm","server");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("SERVER status : $status");

  $data_out = "";

  ############
  ##   VM   ##
  ############
  $htype = "vm";
  $data_out .= "uuid;time;cpu_utilization_cores;cpu_utilization_percent;cpu_count;used_mem;cpu_cores;\n";
  foreach my $time ( sort keys %{ $data{$htype}{'perf'} } ) {
    foreach my $vm_uuid ( sort keys %{ $data{$htype}{'perf'}{$time} } ) {
      foreach my $vm_name ( sort keys %{ $data{$htype}{'perf'}{$time}{$vm_uuid} } ) {
        my $perf_ref         = $data{$htype}{'perf'}{$time}{$vm_uuid}{$vm_name};
        my $cpu_util    = "";
        my $cpu_perc    = "";
        my $cpu_cores   = "";
        my $cpu_count   = "";
        my $used_mem    = "";
        if ( defined $perf_ref->{'CPU_UTILIZATION'} && $perf_ref->{'CPU_UTILIZATION'} ne ""){
          $cpu_util    = $perf_ref->{'CPU_UTILIZATION'};
        }
        if ( defined $perf_ref->{'CPU_PERC'} && $perf_ref->{'CPU_PERC'} ne ""){
          $cpu_perc  = $perf_ref->{'CPU_PERC'};
        }
        if ( defined $perf_ref->{'CPU_CORES'} && $perf_ref->{'CPU_CORES'} ne ""){
          $cpu_cores  = $perf_ref->{'CPU_CORES'};
        }
        if ( defined $perf_ref->{'CPU_COUNT'} && $perf_ref->{'CPU_COUNT'} ne ""){
          $cpu_count  = $perf_ref->{'CPU_COUNT'};
        }
        if ( defined $perf_ref->{'MEMORY_USED'} && $perf_ref->{'MEMORY_USED'} ne ""){
          $used_mem    = $perf_ref->{'MEMORY_USED'};
        }
        #print "$vm_uuid;$time;cpu_util:$cpu_util;cpu_perc:$cpu_perc;cpu_count:$cpu_count;used_mem:$used_mem\n";
        $data_out  .= "$vm_uuid;$time;$cpu_util;$cpu_perc;$cpu_count;$used_mem;$cpu_cores\n";
      }
    }
    $last_perf_timestamp = $time;
  }
  # Save to DB
  $rc = $xormon->saveCSV($data_out,"oraclevm","vm");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("VM status : $status");

  $data_out = "";

  ######################
  ## Server interface ##
  ######################
  #print Dumper \%data_server_net;
  $htype = "server";
  $data_out .= "uuid;time;network_received;network_transmitted\n";
  foreach my $time ( sort keys %{ $data_server_net{$htype}{'perf'} } ) {
    foreach my $interface_uuid ( sort keys %{ $data_server_net{$htype}{'perf'}{$time} } ) {
      my $perf_ref         = $data_server_net{$htype}{'perf'}{$time}{$interface_uuid};
      #print Dumper $perf_ref;
      my $network_received      = "";
      my $network_transmitted   = "";
      if ( defined $perf_ref->{'NETWORK_SENT'} && $perf_ref->{'NETWORK_SENT'} ne ""){
        $network_received      = $perf_ref->{'NETWORK_SENT'};
      }
      if ( defined $perf_ref->{'NETWORK_RECEIVED'} && $perf_ref->{'NETWORK_RECEIVED'} ne ""){
        $network_transmitted   = $perf_ref->{'NETWORK_RECEIVED'};
      }
      $data_out  .= "$interface_uuid;$time;$network_received;$network_transmitted\n";
      #print "$interface_uuid,$time,$network_received,$network_transmitted\n";
    }
    $last_perf_timestamp = $time;
  }
  # Save to DB
  $rc = $xormon->saveCSV($data_out,"oraclevm","server_interface");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("SERVER interface status : $status");

  $data_out = "";

  ######################
  ## Server disk      ##
  ######################
  #print Dumper \%data_server_disk;
  $data_out .= "uuid;time;disk_read;disk_write\n";
  foreach my $time ( sort keys %{ $data_server_disk{$htype}{'perf'} } ) {
    foreach my $disk_uuid ( sort keys %{ $data_server_disk{$htype}{'perf'}{$time} } ) {
      my $perf_ref         = $data_server_disk{$htype}{'perf'}{$time}{$disk_uuid};
      #print Dumper $perf_ref;
      my $disk_read    = "";
      my $disk_write   = "";
      if ( defined $perf_ref->{'DISK_READ'} && $perf_ref->{'DISK_READ'} ne ""){
        $disk_read    = $perf_ref->{'DISK_READ'};
      }
      if ( defined $perf_ref->{'DISK_WRITE'} && $perf_ref->{'DISK_WRITE'} ne ""){
        $disk_write   = $perf_ref->{'DISK_WRITE'};
      }
      $data_out  .= "$disk_uuid;$time;$disk_read;$disk_write\n";
      #print "$disk_uuid;$time;$disk_read;$disk_write\n";
    }
    $last_perf_timestamp = $time;
  }
  # Save to DB
  $rc = $xormon->saveCSV($data_out,"oraclevm","server_disk");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("Server disk status : $status");

  $data_out = "";

  ######################
  ## VM  interface    ##
  ######################
  #print Dumper \%data_vm_net;
  $htype = "vm";
  $data_out .= "uuid;time;network_received;network_transmitted\n";
  foreach my $time ( sort keys %{ $data_vm_net{$htype}{'perf'} } ) {
    foreach my $net_uuid ( sort keys %{ $data_vm_net{$htype}{'perf'}{$time} } ) {
      my $perf_ref         = $data_vm_net{$htype}{'perf'}{$time}{$net_uuid};
      #print Dumper $perf_ref;
      my $network_received    = "";
      my $network_transmitted   = "";
      if ( defined $perf_ref->{'NETWORK_SENT'} && $perf_ref->{'NETWORK_SENT'} ne ""){
        $network_received     = $perf_ref->{'NETWORK_SENT'};
      }
      if ( defined $perf_ref->{'NETWORK_RECEIVED'} && $perf_ref->{'NETWORK_RECEIVED'} ne ""){
        $network_transmitted  = $perf_ref->{'NETWORK_RECEIVED'};
      }
      $data_out  .= "$net_uuid;$time;$network_received;$network_transmitted\n";
      #print "$vm_name,$vm_uuid,$net_name,$network_received,$network_transmitted\n";
    }
    $last_perf_timestamp = $time;
  }
  # Save to DB
  $rc = $xormon->saveCSV($data_out,"oraclevm","vm_interface");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("VM interface : $status");

  $data_out = "";

  ######################
  ##    VM  disk      ##
  ######################
  #print Dumper \%data_vm_disk;
  $htype = "vm";
  $data_out .= "uuid;time;disk_read;disk_write\n";
  foreach my $time ( sort keys %{ $data_vm_disk{$htype}{'perf'} } ) {
    foreach my $disk_uuid ( sort keys %{ $data_vm_disk{$htype}{'perf'}{$time} } ) {
      my $perf_ref         = $data_vm_disk{$htype}{'perf'}{$time}{$disk_uuid};
      #print Dumper $perf_ref;
      my $disk_read    = "";
      my $disk_write   = "";
      if ( defined $perf_ref->{'DISK_READ'} && $perf_ref->{'DISK_READ'} ne ""){
        $disk_read    = $perf_ref->{'DISK_READ'};
      }
      if ( defined $perf_ref->{'DISK_WRITE'} && $perf_ref->{'DISK_WRITE'} ne ""){
        $disk_write   = $perf_ref->{'DISK_WRITE'};
      }
      $data_out  .= "$disk_uuid;$time;$disk_read;$disk_write\n";
      #print "$vm_name,$vm_uuid,$disk_name,$disk_read,$disk_write\n";
    }
    $last_perf_timestamp = $time;
  }
  # Save to DB
  $rc = $xormon->saveCSV($data_out,"oraclevm","vm_disk");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("VM disk status : $status");


  if ( defined $last_perf_timestamp && $last_perf_timestamp ne '' ) {
    $service_data_out->{last_perf_timestamp} = $last_perf_timestamp; # add last perf timestamp to db
    $xormon->saveData(\%{ $service_data_out });
  }

  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }

  $data_out = "";

}

sub decode_json_eval {
  my ($json) = @_;
  my $hash = ();
  eval {
    $hash = decode_json($json);
  };
  if ($@) {
    my $error = $@;
    error($error);
    return ();
  } else {
    return $hash;
  }
}

sub new_get_request {
  my $url      = shift;
  my $username = shift;
  my $password = shift;
  my $result   = "";

  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( GET => $url );
  $request->authorization_basic( "$username", "$password" );
  $request->header(
    'Accept'        => 'application/json',
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    Xormon::error("GET url           : $url");
    Xormon::error("Request error     : $st_line");

    return "0";
  }

  if ( defined $results->content && $results->content ne "" ) {
    my $result = decode_json( $results->content );

    #print "###############URL################\n";
    #print "$url\n";
    #print Dumper $result;
    return $result;
  }

  return "0";
}

sub log {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print "[$act_time]: $text \n";
  return 1;
} ## end sub error

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print STDERR "$act_time: $text : $!\n";
  return 1;
} ## end sub error

1;
