use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;                
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Test;
use OracleVM;
use Xormon;
use Digest::MD5 qw(md5_hex);
use POSIX;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

# microservice port
my $port = 3108;

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

 # waiting for a new client connection
while ($client = $server->accept()) {

    my $pid;
    unless ( defined( $pid = fork() ) ) {
        Xormon::error("Wait for fork");
        sleep 5;
    }

    if ($pid == 0) {
   
        # get information about a newly connected client
        my $client_address = $client->peerhost();
        my $client_port = $client->peerport();
        Xormon::log("Connection from $client_address:$client_port");

        # received data template 
        #
        # { 
        #   "jwt"  : "<backendToken>",
        #   "task" : "<taskName>"
        #   "data" : {
        #     <configuration_from_hostcfg>
        #   }
        # }

        # read up to 2048 characters from the connected client
        my $data = "";
        $client->recv($data, 2048);

        my $decoded = from_json($data);
        Xormon::log("Received task: ".$decoded->{task});
        my $cfg = $decoded->{data};

        my $response = '';
        if ($decoded->{task} eq "fetch") {

            $client->send("Task successfuly started");
            $client->shutdown(SHUT_WR);

            my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'oraclevm', $cfg->{hostcfg_id}, $cfg->{hostalias});

            # target destinations check
            if (defined $decoded->{'destinations'}) {
                $xormon->setDestinations($decoded->{'destinations'});
            }

            $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

            ############################### AGENT SECTION ###############################
            my $rc = OracleVM::collect_data($xormon,$cfg);

            ############################### END AGENT SECTION ###############################

            if ($rc eq "1"){
              $rc = 201;
            }

            # status code
            my $status;
            if ($rc eq "201") {
                $status = "successful";
            } elsif ($rc eq "401") {
                $status = "unauthorized";
            } else {
                $status = "error";
            }

            $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);

            Xormon::log("Task [fetch] for ".$cfg->{hostalias}." is done");

        } elsif ($decoded->{task} eq "test") {

            apitest($client, $cfg);

        } else {
            $response = "undefined task";

            $client->send($response);
            $client->shutdown(SHUT_WR);

        }

        exit();

    } else {
        $client->autoflush(1);    
    }
}

sub apitest {
    my $client = shift;
    my $data   = shift;

    my $test = Test->new($client);

    my $connected = Test::connTest($data->{host}, $data->{port});
    if ($connected == 1) {
      my $url_stat_rest_api  = "https://$data->{host}:$data->{port}/ovm/core/wsapi/rest/Manager";
      my $test_orvm     = test_request($url_stat_rest_api, $data->{username}, $data->{password});
      my $manager_status     = "";
      my $manager_time       = "";
      my $manager_name       = "";
      my $manager_version    = "";

      if ($test_orvm =~ /200 OK/){
        my $info_about_api     = OracleVM::new_get_request($url_stat_rest_api, $data->{username}, $data->{password});
        foreach my $info_api ( @{ $info_about_api}  ) {
          if ( defined $info_api->{'managerTime'} && $info_api->{'managerTime'} ne "" ) {
            my $manager_timestamp = $info_api->{'managerTime'};
            $manager_timestamp = $manager_timestamp / 1000;
            $manager_timestamp = sprintf "%.0f", $manager_timestamp;
            $manager_time = strftime( "%Y-%m-%d %H:%M:%S", localtime($manager_timestamp) );
          }
          if ( defined $info_api->{'managerRunState'} && $info_api->{'managerRunState'} ne "" ) {
            $manager_status    = $info_api->{'managerRunState'};
          }
          if ( defined $info_api->{'managerVersion'} && $info_api->{'managerVersion'} ne "" ) {
            $manager_version   = $info_api->{'managerVersion'};
          }
          if ( defined $info_api->{'name'} && $info_api->{'name'} ne "" ) {
            $manager_name   = $info_api->{'name'};
          }
        }
      }
      elsif ($test_orvm =~ /401 Unauthorized/){
        $test->addSection("Connection test", 0, "Probably you entered the wrong password.");
        $test->stop();
      }
      
      if ($manager_status =~ /RUNNING/){
        $test->addSection("Connection test", 1);
        $test->addSection("Info about OracleVM:", 1, "Manager name: $manager_name\nManager version: $manager_version\nManager time: $manager_time");
      }
      elsif ($manager_status =~ /STARTING/){
        $test->addSection("Connection test", 0, "OracleVM manager is still being set up. Wait about a few minutes on status RUNNING");
        $test->stop();
      }
      else{
        $test->addSection("Connection test", 0, "OracleVM manager is not RUNNING");
        $test->stop();
      }
      
      my $db_url_server     = "https://$data->{host}:$data->{port}/ovm/core/wsapi/rest/Server";
      my $info_about_server  = OracleVM::new_get_request($db_url_server, $data->{username}, $data->{password});
      if ( $info_about_server ne "0" ) {
        my $number_of_servers = 0;
        foreach my $server_info ( @{ $info_about_server } ) {
          if ( defined $server_info->{'id'} && $server_info->{'id'} ne "" ) {
            $number_of_servers++;
          }
        }
        $test->addSection("Data test", 1, "Number of Servers: $number_of_servers");
      }
      my $db_url_vm         = "https://$data->{host}:$data->{port}/ovm/core/wsapi/rest/Vm";
      my $info_about_vm  = OracleVM::new_get_request($db_url_vm, $data->{username}, $data->{password});
      if ( $info_about_vm ne "0" ) {
        my $number_of_vms = 0;
        foreach my $vm_info ( @{ $info_about_vm } ) {
          if ( defined $vm_info->{'id'} && $vm_info->{'id'} ne "" ) {
            $number_of_vms++;
          }
        }
        $test->addSection("Data test", 1, "Number of VMs: $number_of_vms");
      }
      $test->stop();
    } else {
      $test->addSection("Connection test", 0, "TCP connection to " . $data->{host} . ":" . $data->{port} . " has failed! Open it on the firewall.");
      $test->stop();
    }
}

sub test_request {
  my $url      = shift;
  my $username = shift;
  my $password = shift;
  my $result   = "";

  my $agent   = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( GET => $url );
  $request->authorization_basic( "$username", "$password" );
  $request->header(
    'Accept'        => 'application/json',
  );

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    Xormon::error("GET url           : $url");
    Xormon::error("Request error     : $st_line");
    return $st_line;
  }

  if ( defined $results->content && $results->content ne "" ) {
    my $result = decode_json( $results->content );
    my $st_line = $results->status_line;
    return $st_line;
  }

  return "0";
}

$server->close();
