use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib '../../../perl-modules';
use OVirt;
use Xormon;
use File::Temp qw(tempfile);
use File::Copy;
use XormonLogger;
use Test;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

# microservice port
my $DEFAULT_PORT = 3110;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);
my $logger;
my $DEBUG;

my $server = IO::Socket->new(
  Domain    => AF_INET,
  Type      => SOCK_STREAM,
  Proto     => 'tcp',
  LocalHost => '0.0.0.0',
  LocalPort => $port,
  ReusePort => 1,
  Listen    => 5,
)            || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

# waiting for a new client connection
while( $client = $server->accept() ) {

  my $pid;
  unless( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }

  if( $pid == 0 ) {

    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    Xormon::log("Connection from $client_address:$client_port");

    # received data template
    #
    # {
    #   "jwt"  : "<backendToken>",
    #   "task" : "<taskName>"
    #   "data" : {
    #     <configuration_from_hostcfg>
    #   }
    # }

    # read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);

    my $decoded = from_json($data);
    my $cfg = $decoded->{data};

    Xormon::log("Received task: " . $decoded->{task});
    my $response = '';
    if( $decoded->{task} eq "fetch" ) {

      $client->send("Task successfuly started");
      $client->shutdown(SHUT_WR);
      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'ovirt', $cfg->{hostcfg_id}, $cfg->{hostalias});

      $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

      ############################### AGENT SECTION ###############################
      $DEBUG = $cfg->{'debug'} ? $cfg->{'debug'} : 0;
      # get data from technology
      #my $dataToBackend = api2csv($cfg);
      my $rc = db2csv($xormon, $cfg);

      # send data to backend
      #my $rc = $xormon->saveCSV($dataToBackend);

      ############################### END AGENT SECTION ###############################

      # status code
      my $status;
      if( $rc eq "201" ) {
        $status = "successful";
      }
      elsif( $rc eq "401" ) {
        $status = "unauthorized";
      }
      else {
        $status = "error";
      }

      $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);

      Xormon::log("Task [fetch] for " . $cfg->{hostalias} . " is done");

    }
    elsif( $decoded->{task} eq "test" ) {

      $response = apitest($client, $cfg);

      $client->send($response);
      $client->shutdown(SHUT_WR);

    }
    else {
      $response = "undefined task";

      $client->send($response);
      $client->shutdown(SHUT_WR);

    }

    exit();

  }
  else {
    $client->autoflush(1);
  }
}

###############################################################################

my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
my $datetime = sprintf("%4d_%02d%02d_%02d%02d", $year + 1900, $month + 1, $day, $hour, $min);
my $act_timestamp = time;
my $database = "ovirt_engine_history";
my %conf = ( timestamp => $act_timestamp );
my %config = ( timestamp => $act_timestamp );
my %perf;
my %data;

my $service_data_out = {};

my $host_port;
my $host_ip;
my $password;
my $userid;
my $dbh;
my $dsn;

sub db2csv {
  my $xormon = shift;
  my $hostcfg = shift;

  my $test = Test->new($client);
  my @rows;

  my $module = "DBI";
  my $module_def = 1;
  eval "use $module; 1" or $module_def = 0;
  if( !$module_def ) {
    push(@rows, "ERROR: $module is not installed. https://xormon.com/install-linux.php");
    $test->addSection("Module test", 0, \@rows);
    return $test->stop();
  }
  require DBI;

  my $database = "ovirt_engine_history";
  my $host_port = $hostcfg->{port};
  my $host_ip = $hostcfg->{host};
  my $password = $hostcfg->{password};
  my $userid = $hostcfg->{username};

  my $connected = Test::connTest($host_ip, $host_port);
  if( $connected == 1 ) {
    $xormon->isConnected(1);
  }
  else {
    $xormon->isConnected(0, "TCP connection to " . $host_ip . ":" . $host_port . " has failed!");
    return 0;
  }

  $dsn = "DBI:Pg:dbname = $database; host = $host_ip; port = $host_port";
  $dbh = DBI->connect($dsn, $userid, $password, { RaiseError => 1 }) or die "DBI: " . $DBI::errstr;
  Xormon::log("database : connected to $host_ip successfully");

  my $db_time = get_db_time();

  my $service_data = $xormon->getData();
  my $last_update = ( defined $service_data->{last_perf_timestamp} ) ? $service_data->{last_perf_timestamp} : time() - 1200;

  $conf{db_hostname} = $host_ip;
  $conf{db_time} = $db_time;
  $conf{last_update} = $last_update;

  # save HostCfg UUID, to be mapped to respective items (datacenter)
  #my $hostcfg_uuid = $hosts{$host}{uuid};
  #$conf{hostcfg_uuid} = $hostcfg_uuid;

  unless( $db_time ) {
    die "cannot read DB time";
  }

  my $schema_version = get_schema_version();
  if( $schema_version == 0 ) {
    $xormon->hasData(0, "Unable to get data");
  }
  else {
    $xormon->hasData(1);
  }

  get_conf('datacenter', $OVirt::dc_current_stmt);
  get_conf('cluster', $OVirt::cl_current_stmt);
  get_conf('host', $OVirt::host_current_stmt);
  get_conf('host_nic', $OVirt::host_nic_current_stmt);
  get_conf('vm', $OVirt::vm_current_stmt);
  get_conf('vm_running_on_host', $OVirt::vm_running_on_host_stmt);
  get_conf('vm_nic', $OVirt::vm_nic_current_stmt);
  get_conf('vm_devices', $OVirt::vm_devices_current_stmt);
  get_conf('storage_domain', $OVirt::storage_domain_current_stmt);
  get_conf('storage_domain_size', $OVirt::storage_domain_size_stmt);
  get_conf('disk', $OVirt::vm_disk_current_stmt);

  Xormon::log(Dumper \%conf) if( $DEBUG >= 1 );

  get_perf('host', OVirt::get_host_perf_stmt($last_update, $db_time));
  get_perf('host_nic', OVirt::get_host_nic_perf_stmt($last_update, $db_time));
  get_perf('storage_domain', OVirt::get_storage_domain_perf_stmt($last_update, $db_time));
  get_perf('disk', OVirt::get_vm_disk_perf_stmt($last_update, $db_time, $schema_version));
  get_perf('vm', OVirt::get_vm_perf_stmt($last_update, $db_time));
  get_perf('vm_nic', OVirt::get_vm_nic_perf_stmt($last_update, $db_time));

  Xormon::log(Dumper \%data) if( $DEBUG >= 1 );

  $dbh->disconnect();

  ##############################################

  my @architecture = @{prepare_arch($hostcfg)};
  $xormon->saveArchitecture(\@architecture);

  prepare_config();
  $xormon->saveConfJSON(\%config);

  my $rc;
  prepare_perf();
  $rc = perf2csv($xormon, 'host');
  $rc = perf2csv($xormon, 'host_nic');
  $rc = perf2csv($xormon, 'storage_domain');
  $rc = perf2csv($xormon, 'disk');
  $rc = perf2csv($xormon, 'vm');
  $rc = perf2csv($xormon, 'vm_nic');

  return $rc;
}

sub get_conf {
  my $type = shift;
  my $stmt = shift;
  my $sth = $dbh->prepare($stmt);
  my $rv = $sth->execute() or die $DBI::errstr;

  if( $rv < 0 ) {
    Xormon::log($DBI::errstr);
  }

  my $rows_fetch = 0;
  my $rows_add = 0;
  while( my $ref = $sth->fetchrow_hashref() ) {
    $rows_fetch++;
    if( $ref->{uuid} ) {
      $rows_add++;
      $conf{$type}{ $ref->{uuid} } = $ref;
    }
  }
  Xormon::log("get configuration type $type. Rows fetch: $rows_fetch add: $rows_add ") if( $DEBUG >= 1 );
  return 1;
}

sub get_perf {
  my $type = shift;
  my $stmt = ${shift @_};
  my $sth = $dbh->prepare($stmt);
  my $rv = $sth->execute() or die $DBI::errstr;

  if( $rv < 0 ) {
    Xormon::log($DBI::errstr);
  }

  my $last_uuid;
  my $save_flag = 0;

  my $rows_fetch = 0;
  while( my @row = $sth->fetchrow_array() ) {
    my $uuid = $row[0];
    my $timestamp = $row[1];
    $rows_fetch++;
    push @{$data{$type}{$uuid}}, \@row;
  }
  Xormon::log("get perf type $type. Rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  return 1;
}

sub prepare_arch {
  my $cfg = shift;

  my @architecture;

  # datacenter
  my $rows_fetch = 0;
  foreach my $datacenter_uuid ( keys(%{$conf{'datacenter'}}) ) {
    $rows_fetch++;
    my %datacenter_arch = (
      "item_id"    => $datacenter_uuid,
      "label"      => $conf{datacenter}{$datacenter_uuid}{datacenter_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "datacenter",
      "parents"    => [],
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%datacenter_arch);
  }
  Xormon::log("create arch: datacenter rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  # cluster
  $rows_fetch = 0;
  foreach my $cluster_uuid ( keys(%{$conf{'cluster'}}) ) {
    $rows_fetch++;
    my $parent_datacenter = defined($cluster_uuid) ? $conf{cluster}{$cluster_uuid}{datacenter_id} : undef;
    my @parents;
    push(@parents, $parent_datacenter) if( defined($parent_datacenter) );
    my %cluster_arch = (
      "item_id"    => $cluster_uuid,
      "label"      => $conf{cluster}{$cluster_uuid}{cluster_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "cluster",
      "parents"    => \@parents,
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%cluster_arch);
  }
  Xormon::log("create arch: cluster rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  # host
  $rows_fetch = 0;
  foreach my $host_uuid ( keys(%{$conf{'host'}}) ) {
    $rows_fetch++;
    my $parent_cluster = defined($host_uuid) ? $conf{host}{$host_uuid}{cluster_id} : undef;
    my $parent_datacenter = defined($parent_cluster) ? $conf{cluster}{$parent_cluster}{datacenter_id} : undef;
    my @parents;
    push(@parents, $parent_datacenter) if( defined($parent_datacenter) );
    push(@parents, $parent_cluster) if( defined($parent_cluster) );
    my %host_arch = (
      "item_id"    => $host_uuid,
      "label"      => $conf{host}{$host_uuid}{host_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "host",
      "parents"    => \@parents,
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%host_arch);
  }
  Xormon::log("create arch: host rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  # host nic
  $rows_fetch = 0;
  foreach my $host_nic_uuid ( keys(%{$conf{'host_nic'}}) ) {
    $rows_fetch++;
    my $parent_host = defined($host_nic_uuid) ? $conf{host_nic}{$host_nic_uuid}{host_id} : undef;
    my $parent_cluster = defined($parent_host) ? $conf{host}{$parent_host}{cluster_id} : undef;
    my $parent_datacenter = defined($parent_cluster) ? $conf{cluster}{$parent_cluster}{datacenter_id} : undef;
    my @parents;
    push(@parents, $parent_datacenter) if( defined($parent_datacenter) );
    push(@parents, $parent_cluster) if( defined($parent_cluster) );
    push(@parents, $parent_host) if( defined($parent_host) );
    my %host_nic_arch = (
      "item_id"    => $host_nic_uuid,
      "label"      => $conf{host_nic}{$host_nic_uuid}{host_interface_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "host_nic",
      "parents"    => \@parents,
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%host_nic_arch);
  }
  Xormon::log("create arch: host nic rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  # vm
  $rows_fetch = 0;
  foreach my $vm_uuid ( keys(%{$conf{'vm'}}) ) {
    $rows_fetch++;
    my $parent_host = defined($vm_uuid) ? $conf{vm_running_on_host}{$vm_uuid}{currently_running_on_host} : undef;
    my $parent_cluster = defined($vm_uuid) ? $conf{vm}{$vm_uuid}{cluster_id} : undef;
    my $parent_datacenter = defined($parent_cluster) ? $conf{cluster}{$parent_cluster}{datacenter_id} : undef;
    my @parents;
    push(@parents, $parent_datacenter) if( defined($parent_datacenter) );
    push(@parents, $parent_cluster) if( defined($parent_cluster) );
    push(@parents, $parent_host) if( defined($parent_host) );
    my %vm_arch = (
      "item_id"    => $vm_uuid,
      "label"      => $conf{vm}{$vm_uuid}{vm_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "vm",
      "parents"    => \@parents,
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%vm_arch);
  }
  Xormon::log("create arch: vm rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  # vm nic
  $rows_fetch = 0;
  foreach my $vm_nic_uuid ( keys(%{$conf{'vm_nic'}}) ) {
    $rows_fetch++;
    my $parent_vm = defined($vm_nic_uuid) ? $conf{vm_devices}{$vm_nic_uuid}{vm_id} : undef;
    my $parent_host = defined($parent_vm) ? $conf{vm_running_on_host}{$parent_vm}{currently_running_on_host} : undef;
    my $parent_cluster = defined($parent_host) ? $conf{host}{$parent_host}{cluster_id} : undef;
    my $parent_datacenter = defined($parent_cluster) ? $conf{cluster}{$parent_cluster}{datacenter_id} : undef;
    my @parents;
    push(@parents, $parent_datacenter) if( defined($parent_datacenter) );
    push(@parents, $parent_cluster) if( defined($parent_cluster) );
    push(@parents, $parent_host) if( defined($parent_host) );
    push(@parents, $parent_vm) if( defined($parent_vm) );
    my %vm_nic_arch = (
      "item_id"    => $vm_nic_uuid,
      "label"      => $conf{vm_nic}{$vm_nic_uuid}{vm_interface_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "vm_nic",
      "parents"    => \@parents,
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%vm_nic_arch);
  }
  Xormon::log("create arch: vm nic rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  # storage domain
  $rows_fetch = 0;
  foreach my $storage_domain_uuid ( keys(%{$conf{'storage_domain'}}) ) {
    $rows_fetch++;
    my $parent_datacenter = defined($storage_domain_uuid) ? $conf{storage_domain}{$storage_domain_uuid}{datacenter_id} : undef;
    my @parents;
    push(@parents, $parent_datacenter) if( defined($parent_datacenter) );
    my %storage_domain_arch = (
      "item_id"    => $storage_domain_uuid,
      "label"      => $conf{storage_domain}{$storage_domain_uuid}{storage_domain_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "storage_domain",
      "parents"    => \@parents,
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%storage_domain_arch);
  }
  Xormon::log("create arch: storage domain rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  # disk
  $rows_fetch = 0;
  foreach my $disk_uuid ( keys(%{$conf{'disk'}}) ) {
    $rows_fetch++;
    my $parent_vm = defined($disk_uuid) ? $conf{vm_devices}{$disk_uuid}{vm_id} : undef;
    my $parent_host = defined($parent_vm) ? $conf{vm_running_on_host}{$parent_vm}{currently_running_on_host} : undef;
    my $parent_cluster = defined($parent_host) ? $conf{host}{$parent_host}{cluster_id} : undef;
    my $parent_storage_domain = $conf{disk}{$disk_uuid}{storage_domain_id};
    my $parent_datacenter = defined($parent_storage_domain) ? $conf{storage_domain}{$parent_storage_domain}{datacenter_id} : undef;
    my @parents;
    push(@parents, $parent_datacenter) if( defined($parent_datacenter) );
    push(@parents, $parent_storage_domain) if( defined($parent_storage_domain) );
    push(@parents, $parent_cluster) if( defined($parent_cluster) );
    push(@parents, $parent_host) if( defined($parent_host) );
    push(@parents, $parent_vm) if( defined($parent_vm) );
    my %disk_arch = (
      "item_id"    => $disk_uuid,
      "label"      => $conf{disk}{$disk_uuid}{disk_name},
      "hw_type"    => "ovirt",
      "subsystem"  => "disk",
      "parents"    => \@parents,
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    );
    push(@architecture, \%disk_arch);
  }
  Xormon::log("create arch: disk rows fetch: $rows_fetch") if( $DEBUG >= 1 );

  return \@architecture;
}

sub prepare_config {
  # data format:
  # $config{$uuid}{'property'} = $value;

  {
    my $subsystem = 'cluster';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$conf{$subsystem}}) ) {
      if( $uuid ne "" ) {
        $rows_fetch++;
        $config{$uuid}{compatibility_version} = $conf{$subsystem}{$uuid}{compatibility_version};
      }
    }
    Xormon::log("prepare config: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  {
    my $subsystem = 'host';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$conf{$subsystem}}) ) {
      $rows_fetch++;
      if( $uuid ne "" ) {
        $config{$uuid}{fqdn_or_ip} = $conf{$subsystem}{$uuid}{fqdn_or_ip};
        $config{$uuid}{memory_size} = $conf{$subsystem}{$uuid}{memory_size_mb} * ( 1024 ** 2 );
        $config{$uuid}{swap_size} = $conf{$subsystem}{$uuid}{swap_size_mb} * ( 1024 ** 2 );
        $config{$uuid}{cpu_model} = $conf{$subsystem}{$uuid}{cpu_model};
        $config{$uuid}{cpu_speed} = $conf{$subsystem}{$uuid}{cpu_speed_mh} * ( 10 ** 6 );
        $config{$uuid}{number_of_cores} = $conf{$subsystem}{$uuid}{number_of_cores};
        $config{$uuid}{number_of_sockets} = $conf{$subsystem}{$uuid}{number_of_sockets};
        $config{$uuid}{host_os} = $conf{$subsystem}{$uuid}{host_os};
        $config{$uuid}{kernel_version} = $conf{$subsystem}{$uuid}{kernel_version};
        $config{$uuid}{kvm_version} = $conf{$subsystem}{$uuid}{kvm_version};
        $config{$uuid}{threads_per_core} = $conf{$subsystem}{$uuid}{threads_per_core};
        $config{$uuid}{hardware_product_name} = $conf{$subsystem}{$uuid}{hardware_product_name};
        $config{$uuid}{hardware_serial_number} = $conf{$subsystem}{$uuid}{hardware_serial_number};
      }
    }
    Xormon::log("prepare config: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  {
    my $subsystem = 'vm';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$conf{$subsystem}}) ) {
      $rows_fetch++;
      $config{$uuid}{memory_size} = $conf{$subsystem}{$uuid}{memory_size_mb} * ( 1024 ** 2 );
      $config{$uuid}{cpu_per_socket} = $conf{$subsystem}{$uuid}{cpu_per_socket};
      $config{$uuid}{number_of_sockets} = $conf{$subsystem}{$uuid}{number_of_sockets};
      $config{$uuid}{operating_system} = $conf{$subsystem}{$uuid}{operating_system};
      $config{$uuid}{vm_type} = $conf{$subsystem}{$uuid}{vm_type};
    }
    Xormon::log("prepare config: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  {
    my $subsystem = 'storage_domain';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$conf{$subsystem}}) ) {
      $rows_fetch++;
      $config{$uuid}{free_disk_size} = $conf{$subsystem . '_size'}{$uuid}{available_disk_size_gb} * ( 1024 ** 3 );
      $config{$uuid}{used_disk_size} = $conf{$subsystem . '_size'}{$uuid}{used_disk_size_gb} * ( 1024 ** 3 );
      $config{$uuid}{total_disk_size} = $config{$uuid}{free_disk_size} + $config{$uuid}{used_disk_size};
    }
    Xormon::log("prepare config: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  {
    my $subsystem = 'disk';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$conf{$subsystem}}) ) {
      $rows_fetch++;
      $config{$uuid}{vm_disk_size} = $conf{$subsystem}{$uuid}{vm_disk_size_mb} * ( 10 ** 6 );
      # $config{$uuid}{vm_disk_used} from perf data
    }
    Xormon::log("prepare config: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }
}

sub prepare_perf {
  # data format:
  # subsys->uuid->[uuid,timestamp,m1,m2,m3]

  # host
  # metrics from db query
  # perf.cpu_usage_percent,
  # perf.user_cpu_usage_percent,
  # perf.system_cpu_usage_percent,
  # perf.ksm_cpu_percent,
  # perf.cpu_load,
  # perf.total_vms_vcpus,
  # perf.memory_usage_percent,
  # perf.ksm_shared_memory_mb,
  # perf.swap_used_mb,
  # perf.host_status
  # metrics saved to lpar2rrd rrd
  # 'cpu_usage_p', 'cpu_usage_c', 'user_cpu_usage_p', 'system_cpu_usage_p',
  # 'ksm_cpu_p',   'cpu_load',   'total_vms_vcpus', 'number_of_cores', 'memory_used',
  # 'memory_free', 'memory_ksm', 'swap_used',       'reserve1',        'reserve2'
  {
    my $subsystem = 'host';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$data{$subsystem}}) ) {
      my @perf_data = @{$data{$subsystem}{$uuid}};
      foreach my $data_row ( @perf_data ) {
        $rows_fetch++;
        my @row = @{$data_row};
        my $uuid = $row[0];
        my $timestamp = $row[1];

        my $memory_size = $config{$uuid}{memory_size};
        my $number_of_cores = $config{$uuid}{number_of_cores} if defined $config{$uuid}{number_of_cores};

        my $cpu_utilization_percent = $row[2];
        my $cpu_utilization_cores = $number_of_cores * ( $cpu_utilization_percent / 100 )
          if defined $number_of_cores && defined $cpu_utilization_percent;
        # user and system cpu utilization values may be negative in the source database
        my $user_cpu_utilization_percent = defined $row[3] && $row[3] < 0 ? 0 : $row[3];
        my $system_cpu_utilization_percent = defined $row[4] && $row[4] < 0 ? 0 : $row[4];
        my $ksm_cpu_utilization_percent = $row[5];
        my $cpu_load = $row[6];
        my $total_vms_vcpus = $row[7];
        my $mem_percent = $row[8];
        my $used_mem = ( $memory_size / 100 ) * $mem_percent
          if defined $memory_size && defined $mem_percent;
        my $free_mem = $memory_size - $used_mem
          if defined $memory_size && defined $used_mem;
        my $ksm_mem = $row[9] * ( 10 ** 6 );
        my $used_swap = $row[10] * ( 10 ** 6 );

        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_percent} = $cpu_utilization_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_cores} = $cpu_utilization_cores;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{user_cpu_utilization_percent} = $user_cpu_utilization_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{system_cpu_utilization_percent} = $system_cpu_utilization_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{ksm_cpu_utilization_percent} = $ksm_cpu_utilization_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_load} = $cpu_load;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{total_vms_vcpus} = $total_vms_vcpus;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_cores} = $number_of_cores;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{mem_percent} = $mem_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_mem} = $used_mem;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{free_mem} = $free_mem;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{ksm_mem} = $ksm_mem;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_swap} = $used_swap;
      }
    }
    Xormon::log("prepare perf: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  # host_nic
  # metrics from db query
  # received_total_byte,
  # transmitted_total_byte
  # metrics saved to lpar2rrd rrd
  # 'received_byte',      'transmitted_byte'
  {
    my $subsystem = 'host_nic';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$data{$subsystem}}) ) {
      $rows_fetch++;
      my @perf_data = @{$data{$subsystem}{$uuid}};
      my $received_diff;
      my $transmitted_diff;

      foreach my $data_row ( @perf_data ) {
        my @row = @{$data_row};
        if( defined $received_diff && defined $transmitted_diff ) {
          my $uuid = $row[0];
          my $timestamp = $row[1];

          my $network_received = $row[2] / 60 - $received_diff if defined $row[2];
          my $network_transmitted = $row[3] / 60 - $transmitted_diff if defined $row[3];
          $received_diff = $row[2] / 60;
          $transmitted_diff = $row[3] / 60;

          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_received} = $network_received;
          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_transmitted} = $network_transmitted;
        }
        else {
          $received_diff = $row[2] / 60;
          $transmitted_diff = $row[3] / 60;
        }
      }
    }
    Xormon::log("prepare perf: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  # vm
  # metrics from db query
  # perf.cpu_usage_percent,
  # perf.user_cpu_usage_percent,
  # perf.system_cpu_usage_percent,
  # perf.memory_usage_percent,
  # perf.memory_buffered_kb,
  # perf.memory_cached_kb,
  # perf.vm_status
  # metrics saved to lpar2rrd rrd
  # 'cpu_usage_p', 'cpu_usage_c', 'user_cpu_usage_p', 'system_cpu_usage_p',
  # 'number_of_cores', 'memory_used', 'memory_free', 'memory_buffered', 'memory_cached'
  {
    my $subsystem = 'vm';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$data{$subsystem}}) ) {
      $rows_fetch++;
      my @perf_data = @{$data{$subsystem}{$uuid}};
      foreach my $data_row ( @perf_data ) {
        my @row = @{$data_row};
        my $uuid = $row[0];
        my $timestamp = $row[1];

        my $memory_size = $config{$uuid}{memory_size};
        my $number_of_cores = $config{$uuid}{number_of_cores} if defined $config{$uuid}{number_of_cores};
        if( defined $config{$uuid}{number_of_sockets} && defined $config{$uuid}{cpu_per_socket} ) {
          $number_of_cores = $config{$uuid}{number_of_sockets} * $config{$uuid}{cpu_per_socket};
        }

        my $cpu_utilization_percent = $row[2];
        my $cpu_utilization_cores = $number_of_cores * ( $cpu_utilization_percent / 100 )
          if defined $number_of_cores && defined $cpu_utilization_percent;
        # user and system cpu utilization values may be negative in the source database
        my $user_cpu_utilization_percent = defined $row[3] && $row[3] < 0 ? 0 : $row[3];
        my $system_cpu_utilization_percent = defined $row[4] && $row[4] < 0 ? 0 : $row[4];
        my $mem_percent = $row[5];
        my $buffered_mem = defined $row[6] ? $row[6] * ( 10 ** 3 ) : 0
          if defined $memory_size && defined $mem_percent;
        my $cached_mem = defined $row[7] ? $row[7] * ( 10 ** 3 ) : 0
          if defined $memory_size && defined $mem_percent;
        my $used_mem = ( $memory_size / 100 ) * $mem_percent
          if defined $memory_size && defined $mem_percent;
        my $free_mem = $memory_size - ( $used_mem + $buffered_mem + $cached_mem )
          if defined $memory_size && defined $used_mem;

        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_percent} = $cpu_utilization_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_cores} = $cpu_utilization_cores;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{user_cpu_utilization_percent} = $user_cpu_utilization_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{system_cpu_utilization_percent} = $system_cpu_utilization_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_cores} = $number_of_cores;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{mem_percent} = $mem_percent;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_mem} = $used_mem;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{free_mem} = $free_mem;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{buffered_mem} = $buffered_mem;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cached_mem} = $cached_mem;
      }
    }
    Xormon::log("prepare perf: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  # vm_nic
  # metrics from db query
  # nic.received_total_byte,
  # nic.transmitted_total_byte
  # metrics saved to lpar2rrd rrd
  # 'received_byte',      'transmitted_byte'
  {
    my $subsystem = 'vm_nic';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$data{$subsystem}}) ) {
      $rows_fetch++;
      my @perf_data = @{$data{$subsystem}{$uuid}};
      my $received_diff;
      my $transmitted_diff;

      foreach my $data_row ( @perf_data ) {
        my @row = @{$data_row};
        if( defined $received_diff && defined $transmitted_diff ) {
          my $uuid = $row[0];
          my $timestamp = $row[1];

          my $network_received = $row[2] / 60 - $received_diff if defined $row[2];
          my $network_transmitted = $row[3] / 60 - $transmitted_diff if defined $row[3];
          $received_diff = $row[2] / 60;
          $transmitted_diff = $row[3] / 60;

          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_received} = $network_received;
          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_transmitted} = $network_transmitted;
        }
        else {
          $received_diff = $row[2] / 60;
          $transmitted_diff = $row[3] / 60;
        }
      }
    }
    Xormon::log("prepare perf: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  # storage_domain
  # metrics from db query
  # perf.available_disk_size_gb,
  # perf.used_disk_size_gb
  # metrics saved to lpar2rrd rrd
  # 'total_disk_size_gb', 'used_disk_size_gb'
  {
    my $subsystem = 'storage_domain';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$data{$subsystem}}) ) {
      $rows_fetch++;
      my @perf_data = @{$data{$subsystem}{$uuid}};
      foreach my $data_row ( @perf_data ) {
        my @row = @{$data_row};
        my $uuid = $row[0];
        my $timestamp = $row[1];

        my $free_disk_size = defined $row[2] ? $row[2] * ( 1024 ** 3 ) : undef;
        my $used_disk_size = defined $row[3] ? $row[3] * ( 1024 ** 3 ) : undef;
        my $total_disk_size = $free_disk_size + $used_disk_size
          if defined $free_disk_size && defined $used_disk_size;

        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used} = $used_disk_size;
        $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{total} = $total_disk_size;
      }
    }
    Xormon::log("prepare perf: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  # disk
  # metrics from db query (v405)
  # disk.write_rate_bytes_per_second,
  # disk.read_rate_bytes_per_second,
  # disk.write_latency_seconds,
  # disk.read_latency_seconds,
  # disk.vm_disk_actual_size_mb,
  # disk.vm_disk_status,
  # disk.write_ops_total_count,
  # disk.read_ops_total_count
  # metrics saved to lpar2rrd rrd
  # 'data_current_write', 'data_current_read', 'disk_write_latency',
  # 'disk_read_latency',  'vm_disk_size_mb',
  # 'disk_write_iops', 'disk_read_iops'
  {
    my $subsystem = 'disk';
    my $rows_fetch = 0;
    foreach my $uuid ( keys(%{$data{$subsystem}}) ) {
      $rows_fetch++;
      my @perf_data = @{$data{$subsystem}{$uuid}};
      my $disk_write_diff;
      my $disk_read_diff;

      my $parent_vm = $conf{vm_devices}{$uuid}{vm_id};
      my $vm_status = $conf{vm_running_on_host}{$parent_vm}{vm_status};

      if( defined $vm_status && $vm_status ) {
        foreach my $data_row ( @perf_data ) {
          my @row = @{$data_row};
          my $uuid = $row[0];
          my $timestamp = $row[1];

          my $disk_write = $row[2];
          my $disk_read = $row[3];
          my $disk_write_latency = $row[4] * 1000;
          my $disk_read_latency = $row[5] * 1000;
          my $size = $row[6] * ( 1024 ** 2 );

          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write} = $disk_write;
          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read} = $disk_read;
          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write_latency} = $disk_write_latency;
          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read_latency} = $disk_read_latency;
          $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{size} = $size;

          if( defined $disk_write_diff && defined $disk_read_diff ) {
            my $disk_write_iops = $row[8] / 60 - $disk_write_diff;
            my $disk_read_iops = $row[9] / 60 - $disk_read_diff;
            $disk_write_diff = $row[8] / 60;
            $disk_read_diff = $row[9] / 60;

            $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write_iops} = $disk_write_iops;
            $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read_iops} = $disk_read_iops;
          }
          else {
            $disk_write_diff = $row[8] / 60;
            $disk_read_diff = $row[9] / 60;
          }
        }
      }
    }
    Xormon::log("prepare perf: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }
}

###############################################################################

sub perf2csv {
  my $xormon = shift;
  my $subsys = shift;
  my $rc;
  my $status;
  my $data_out = '';
  my $last_update;

  if( $subsys eq 'host' ) {
    my $subsystem = 'host';
    my $rows_fetch = 0;
    $data_out .= "uuid;time;cpu_utilization_percent;cpu_utilization_cores;user_cpu_utilization_percent;system_cpu_utilization_percent;ksm_cpu_utilization_percent;cpu_load;total_vms_vcpus;cpu_cores;mem_percent;used_mem;free_mem;ksm_mem;used_swap\n";
    foreach my $timestamp ( sort keys %{$perf{$subsystem}{'perf'}} ) {
      foreach my $uuid ( sort keys %{$perf{$subsystem}{'perf'}{$timestamp}} ) {
        $rows_fetch++;
        my $cpu_utilization_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_percent} : "";
        my $cpu_utilization_cores = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_cores} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_cores} : "";
        my $user_cpu_utilization_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{user_cpu_utilization_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{user_cpu_utilization_percent} : "";
        my $system_cpu_utilization_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{system_cpu_utilization_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{system_cpu_utilization_percent} : "";
        my $ksm_cpu_utilization_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{ksm_cpu_utilization_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{ksm_cpu_utilization_percent} : "";
        my $cpu_load = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_load} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_load} : "";
        my $total_vms_vcpus = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{total_vms_vcpus} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{total_vms_vcpus} : "";
        my $cpu_cores = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_cores} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_cores} : "";
        my $mem_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{mem_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{mem_percent} : "";
        my $used_mem = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_mem} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_mem} : "";
        my $free_mem = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{free_mem} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{free_mem} : "";
        my $ksm_mem = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{ksm_mem} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{ksm_mem} : "";
        my $used_swap = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_swap} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_swap} : "";

        $data_out .= "$uuid;$timestamp;$cpu_utilization_percent;$cpu_utilization_cores;$user_cpu_utilization_percent;$system_cpu_utilization_percent;$ksm_cpu_utilization_percent;$cpu_load;$total_vms_vcpus;$cpu_cores;$mem_percent;$used_mem;$free_mem;$ksm_mem;$used_swap\n";
      }
      $last_update = $timestamp;
    }
    Xormon::log("generate csv: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }
  elsif( $subsys eq 'host_nic' ) {
    my $subsystem = 'host_nic';
    my $rows_fetch = 0;
    $data_out .= "uuid;time;network_received;network_transmitted\n";
    foreach my $timestamp ( sort keys %{$perf{$subsystem}{'perf'}} ) {
      foreach my $uuid ( sort keys %{$perf{$subsystem}{'perf'}{$timestamp}} ) {
        $rows_fetch++;
        my $network_received = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_received} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_received} : "";
        my $network_transmitted = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_transmitted} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_transmitted} : "";

        $data_out .= "$uuid;$timestamp;$network_received;$network_transmitted\n";
      }
      $last_update = $timestamp;
    }
    Xormon::log("generate csv: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }
  elsif( $subsys eq 'vm' ) {
    my $subsystem = 'vm';
    my $rows_fetch = 0;
    $data_out .= "uuid;time;cpu_utilization_percent;cpu_utilization_cores;user_cpu_utilization_percent;system_cpu_utilization_percent;cpu_cores;mem_percent;used_mem;free_mem;buffered_mem;cached_mem\n";
    foreach my $timestamp ( sort keys %{$perf{$subsystem}{'perf'}} ) {
      foreach my $uuid ( sort keys %{$perf{$subsystem}{'perf'}{$timestamp}} ) {
        $rows_fetch++;
        my $cpu_utilization_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_percent} : "";
        my $cpu_utilization_cores = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_cores} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_utilization_cores} : "";
        my $user_cpu_utilization_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{user_cpu_utilization_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{user_cpu_utilization_percent} : "";
        my $system_cpu_utilization_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{system_cpu_utilization_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{system_cpu_utilization_percent} : "";
        my $cpu_cores = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_cores} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cpu_cores} : "";
        my $mem_percent = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{mem_percent} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{mem_percent} : "";
        my $used_mem = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_mem} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used_mem} : "";
        my $free_mem = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{free_mem} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{free_mem} : "";
        my $buffered_mem = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{buffered_mem} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{buffered_mem} : "";
        my $cached_mem = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cached_mem} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{cached_mem} : "";

        $data_out .= "$uuid;$timestamp;$cpu_utilization_percent;$cpu_utilization_cores;$user_cpu_utilization_percent;$system_cpu_utilization_percent;$cpu_cores;$mem_percent;$used_mem;$free_mem;$buffered_mem;$cached_mem\n";
      }
      $last_update = $timestamp;
    }
    Xormon::log("generate csv: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }
  elsif( $subsys eq 'vm_nic' ) {
    my $subsystem = 'vm_nic';
    my $rows_fetch = 0;
    $data_out .= "uuid;time;network_received;network_transmitted\n";
    foreach my $timestamp ( sort keys %{$perf{$subsystem}{'perf'}} ) {
      foreach my $uuid ( sort keys %{$perf{$subsystem}{'perf'}{$timestamp}} ) {
        $rows_fetch++;
        my $network_received = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_received} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_received} : "";
        my $network_transmitted = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_transmitted} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{network_transmitted} : "";

        $data_out .= "$uuid;$timestamp;$network_received;$network_transmitted\n";
      }
      $last_update = $timestamp;
    }
    Xormon::log("generate csv: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }
  elsif( $subsys eq 'storage_domain' ) {
    my $subsystem = 'storage_domain';
    my $rows_fetch = 0;
    $data_out .= "uuid;time;used;total\n";
    foreach my $timestamp ( sort keys %{$perf{$subsystem}{'perf'}} ) {
      foreach my $uuid ( sort keys %{$perf{$subsystem}{'perf'}{$timestamp}} ) {
        $rows_fetch++;
        my $used = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{used} : "";
        my $total = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{total} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{total} : "";

        $data_out .= "$uuid;$timestamp;$used;$total\n";
      }
      $last_update = $timestamp;
    }
    Xormon::log("generate csv: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }
  elsif( $subsys eq 'disk' ) {
    my $subsystem = 'disk';
    my $rows_fetch = 0;
    $data_out .= "uuid;time;disk_write;disk_read;disk_write_iops;disk_read_iops;disk_write_latency;disk_read_latency;size\n";
    foreach my $timestamp ( sort keys %{$perf{$subsystem}{'perf'}} ) {
      foreach my $uuid ( sort keys %{$perf{$subsystem}{'perf'}{$timestamp}} ) {
        $rows_fetch++;
        my $disk_write = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write} : "";
        my $disk_read = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read} : "";
        my $disk_write_iops = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write_iops} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write_iops} : "";
        my $disk_read_iops = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read_iops} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read_iops} : "";
        my $disk_write_latency = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write_latency} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_write_latency} : "";
        my $disk_read_latency = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read_latency} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{disk_read_latency} : "";
        my $size = defined $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{size} ? $perf{$subsystem}{'perf'}{$timestamp}{$uuid}{size} : "";

        $data_out .= "$uuid;$timestamp;$disk_write;$disk_read;$disk_write_iops;$disk_read_iops;$disk_write_latency;$disk_read_latency;$size\n";
      }
      $last_update = $timestamp;
    }
    Xormon::log("generate csv: $subsystem rows fetch: $rows_fetch") if( $DEBUG >= 1 );
  }

  #Xormon::log($data_out);

  # Save to DB
  $rc = $xormon->saveCSV($data_out, 'ovirt', $subsys);
  if( $rc eq "201" ) {
    $status = "successful";
  }
  elsif( $rc eq "401" ) {
    $status = "unauthorized";
  }
  else {
    $status = "error";
  }
  Xormon::log("$subsys status : $status");

  return $rc;
}

###############################################################################

sub get_schema_version {
  my $sth = $dbh->prepare("SELECT version FROM schema_version WHERE current = TRUE;");
  my $rv = $sth->execute() or die $DBI::errstr;

  if( $rv < 0 ) {
    Xormon::log($DBI::errstr);
  }

  my @row = $sth->fetchrow_array();

  if( scalar @row > 0 ) {
    my $current = substr($row[0], 1, 3);
    Xormon::log("current oVirt schema $current ($row[0])");
    return $current;
  }

  return 0;
}

sub get_db_time {
  my $sth = $dbh->prepare("SELECT CAST(EXTRACT(EPOCH FROM NOW()) AS BIGINT) AS timestamp;");
  my $rv = $sth->execute() or die $DBI::errstr;

  if( $rv < 0 ) {
    Xormon::log($DBI::errstr);
  }

  my @row = $sth->fetchrow_array();

  if( scalar @row > 0 && $row[0] =~ /^\d+$/ ) {
    return $row[0];
  }

  return 0;
}

###############################################################################

sub apitest {
  my $client = shift;
  my $data = shift;

  my $database = "ovirt_engine_history";
  my $host_port = $data->{port};
  my $host_ip = $data->{host};
  my $password = $data->{password};
  my $userid = $data->{username};

  my $test = Test->new($client);
  my @rows;

  my ( $code, $text ) = Xormon::conntest_tcp($host_ip, $host_port);
  $test->addSection("DB host network connection", $code, $text);
  if( $code == 0 ) {
    return $test->stop();
  }

  my $module = "DBI";
  my $module_def = 1;
  eval "use $module; 1" or $module_def = 0;
  if( !$module_def ) {
    push(@rows, "ERROR: $module is not installed. https://xormon.com/install-linux.php");
    $test->addSection("Module test", 0, \@rows);
    return $test->stop();
  }
  require DBI;

  $dsn = "DBI:Pg:dbname = $database; host = $host_ip; port = $host_port";
  $dbh = DBI->connect($dsn, $userid, $password, { PrintError => 0 });
  #$dbh = DBI->connect( $dsn, $userid, $password, { RaiseError => 1 } ) or die "DBI: " . $DBI::errstr;
  my $err = DBI->errstr;

  if( defined $err ) {
    push(@rows, "ERROR: failed to read from database\n$err");
    $test->addSection("DB data test", 0, \@rows);
    return $test->stop();
  }

  my $schema_version = get_schema_version();
  if( $schema_version ) {
    push(@rows, "oVirt DB schema $schema_version.");
    $test->addSection("DB data test", 1, \@rows);
  }
  else {
    push(@rows, "ERROR: failed to read from database ($err).");
    $test->addSection("DB data test", 0, \@rows);
  }

  return $test->stop();
}

###############################################################################

$server->close();
