package CMC_CSV;

use strict;
use warnings;

use Data::Dumper;

use CMC_COMMON;


my $REAL_DATA = 1;

my %ng_section_rename = (
  "Pools"       => "pool",
  "Systems"     => "server",
  "Systems_OS"  => "serveros",
);

sub get_govern_hash_timed {
  my %govern_timestamped_data = (
    'Pools' => [ 
      'cm_aix', 'cm_otherlinux', # cm = core minutes
      'cm_sles', 'cm_vios', 'cm_ibmi',
      'cm_rhel', 'cm_rhelcoreos', 'cm_total',

      'cmc_aix', 'cmc_linuxvios', 'cmc_anyos', # cmc = core metered credits
      'cmc_sles', 'cmc_vios', 'cmc_ibmi',
      'cmc_rhel', 'cmc_rhelcoreos', 'cmc_total',

      'cmm_aix', 'cmm_linuxvios', 'cmm_anyos', # cmm = core metered minutes
      'cmm_sles', 'cmm_vios', 'cmm_ibmi',
      'cmm_rhel', 'cmm_rhelcoreos', 'cmm_total',

      'mm_aix', 'mm_otherlinux', # mm = MemoryMinutes, now showing only total
      'mm_sles', 'mm_vios', 'mm_ibmi',
      'mm_rhel', 'mm_rhelcoreos', 'mm_total',

      'mm_systemother',

      'mm_credits', 'mm_minutes', # MemoryMeteredCredits, MemoryMeteredMinutes

      "base_memory",

      "base_core_any_os", "base_core_linuxvios",

      "base_core_rhel", "base_core_rhcos",
      "base_core_sles", "base_core_aix",
      "base_core_ibmi",
      #'reserve_1', 'reserve2',
      #'reserve_3', 'reserve4',
      #'reserve_5', 'reserve6',
      #'reserve_7', 'reserve8',
      #'reserve_9', 'reserve10',

    ],
    'Systems' => [
      'proc_available', 'proc_installed', 
      'mem_available', 'mem_installed',

      'base_anyoscores',

      'utilizedProcUnits', 'totalProcUnits',

      # SYSTEM OS
      "base_memory",

      "base_core_any_os", "base_core_linuxvios",

      "base_core_rhel", "base_core_rhcos",
      "base_core_sles", "base_core_aix",
      "base_core_ibmi",

      "core_aix", "core_ibmi",

      "core_rhelcoreos", "core_rhel",
      "core_sles", "core_other_linux",
      "core_vios",

      "core_total",

      "mem_aix", "mem_ibmi",

      "mem_rhelcoreos", "mem_rhel",
      "mem_sles", "mem_other_linux",
      "mem_vios",

      "mem_system_other",

      "mem_total",
    ]
  );

  return \%govern_timestamped_data;
}

sub get_govern_hash_os {
  my %govern_server_os = (
    'Systems_OS' => [
      "base_memory",

      "base_core_any_os", "base_core_linuxvios",

      "base_core_rhel", "base_core_rhcos",
      "base_core_sles", "base_core_aix",
      "base_core_ibmi",

      "core_aix", "core_ibmi",

      "core_rhelcoreos", "core_rhel",
      "core_sles", "core_other_linux",
      "core_vios",

      "core_total",

      "mem_aix", "mem_ibmi",

      "mem_rhelcoreos", "mem_rhel",
      "mem_sles", "mem_other_linux",
      "mem_vios",

      "mem_system_other",

      "mem_total",
      #'res_1', 'res_2',
      #'res_3', 'res_4',
      #'res_5', 'res_6',
      #'res_7', 'res_8',
      #'res_9', 'res_10',
    ]
  );

  return \%govern_server_os;
}

my %section_renames = (
  "Pools"       => "pool",
  "Systems"     => "server",
  "Systems_OS"  => "serveros",
);

my $GIBI_BYTE = 1024*1024*1024;

my %multiplier = (
  "Systems" => {
    'mem_installed' => $GIBI_BYTE,
    'mem_available' => $GIBI_BYTE,
    'mem_total' => $GIBI_BYTE,
    'base_memory' => $GIBI_BYTE,
  },
  "Systems_OS" => {
    'mem_installed' => $GIBI_BYTE,
    'mem_available' => $GIBI_BYTE,
    'mem_total' => $GIBI_BYTE,
    'base_memory' => $GIBI_BYTE,
  },
  "Pools" => {
    'mem_installed' => $GIBI_BYTE,
    'mem_available' => $GIBI_BYTE,
    'mem_total' => $GIBI_BYTE,
    'base_memory' => $GIBI_BYTE,
    
    # mm = MemoryMinutes
    'mm_aix'          => $GIBI_BYTE,
    'mm_otherlinux'   => $GIBI_BYTE,
    'mm_sles'         => $GIBI_BYTE,
    'mm_vios'         => $GIBI_BYTE,
    'mm_ibmi'         => $GIBI_BYTE,
    'mm_rhel'         => $GIBI_BYTE,
    'mm_rhelcoreos'   => $GIBI_BYTE,
    'mm_total'        => $GIBI_BYTE,
    'mm_systemother'  => $GIBI_BYTE,

    # mm = MemoryMeteredMinutes
    'mm_minutes' => $GIBI_BYTE,

    "mem_aix" => $GIBI_BYTE,
    "mem_ibmi" => $GIBI_BYTE,
    "mem_rhelcoreos" => $GIBI_BYTE,
    "mem_rhel" => $GIBI_BYTE,
    "mem_sles" => $GIBI_BYTE,
    "mem_other_linux" => $GIBI_BYTE,
    "mem_vios" => $GIBI_BYTE,
    "mem_system_other" => $GIBI_BYTE,
    #"mem_total" => $GIBI_BYTE,
  }
);

# "mem_aix",
# "mem_ibmi",
# "mem_rhelcoreos",
# "mem_rhel",
# "mem_sles",
# "mem_other_linux",
# "mem_vios",
# "mem_system_other",
# "mem_total",

sub make_csv {
  my %data_hash   = %{$_[0]};
  my %govern_hash = %{$_[1]};
  my $console     = $_[2];
  my $console_url = $_[3];

  $REAL_DATA = CMC_COMMON::use_api_data($console_url);

  my %section_csv = ();

  for my $section (keys %govern_hash) {

    my @metrics = @{$govern_hash{$section}};

    my $csv = "Id;time;";
    $csv .= join(";", @metrics);
    $csv .= ";\n";

    my $counter = 0;
    for my $UUID (keys %{$data_hash{$section}}){

      my $UUID_save = CMC_COMMON::unique_id($console, $section_renames{$section}, $UUID);

      my @timestamps = sort keys %{$data_hash{$section}{$UUID}{Metrics}};

      my $time_now = time();
      my $time_shift = 0;

      if ( ! $REAL_DATA ) {
        $time_shift = $time_now - $timestamps[-1];
      }

      for my $timestamp (sort keys %{$data_hash{$section}{$UUID}{Metrics}}){

        my $save_timestamp = $timestamp + $time_shift;

        my @line = ($UUID_save, $save_timestamp);

        for my $metric (@metrics) {

          my $value = "";

          if (defined $data_hash{$section}{$UUID}{Metrics}{$timestamp}{$metric}) {
            $value = $data_hash{$section}{$UUID}{Metrics}{$timestamp}{$metric};
          }
          if (defined $multiplier{$section}{$metric} && $value) {
            $value *= $multiplier{$section}{$metric};
          }

          push(@line, $value);

        }

        $csv .= join(";", @line);
        $csv .= ";\n";

      }

    }

    my %ng_section_rename = (
        "Pools"       => "pool",
        "Systems"     => "server",
        #"Systems_OS"  => "server",
    );

    if ( defined $ng_section_rename{$section} ) {
      my $renamed_section = $ng_section_rename{$section};
      print "RENAME $section TO $renamed_section\n";
      $section_csv{$renamed_section} = $csv;
    }
    else {
      print "WARNING: Trying to rename unknown section: $section \n";
    }

  }

  return \%section_csv;
}




1;