package CMC_TEST;

use strict;
use warnings;

use Data::Dumper;

use Date::Parse;
use Time::Local;
use JSON;

use CMC_COLLECT;

my $docs_install_link = "https://xormon.com/IBM-Power-monitoring-installation.php#CMC";

# Decision tree (LPAR2RRD, needs update):
# ? real data ?
#---------------------------
# YES: ? collecting from endpoint /inventory/tags is OK ?
#     ---------------------------
#     YES: continue
#     NO: print RETURN message
#         EXIT
#     ---------------------------
# NO: load data from JSON in bin: inventory_tags.json
#---------------------------

# if here, then exists: %inventory_tags

# SETUP data checking variable: data_check = 1

# ? proper JSON structure ?
#---------------------------
# YES: collect data
# NO: data_check = 0
#     fill error data message 
#---------------------------

# ? Collected server data ? and data_check = 1
#---------------------------
# YES: Prepare informative data message
# NO:  data_check = 0
#      fill error data message
#---------------------------

#---------------------------
# FINISH:
#---------------------------
# if connection ok -> print OK message
# if data OK -> print informative data message
# if data NOK-> print error data message
my %data;

#------------------------------
# Xorux_lib::
#------------------------------
# PARAMETERS
my $portal_url;
my $CMC_client_id;
my $CMC_client_secret;
my $proxy;

# NOTE: Do not leave any possibility of fake start

my $EndTS;
my $Frequency;
my $StartTS;

sub test {
    my $test = shift;
    $portal_url        = shift;
    $CMC_client_id     = shift;
    $CMC_client_secret = shift;
    $proxy             = shift;

    my $data_message = '';

    print "RUNNING TEST \n";

    my $connected = 1;

    #if ($connected) {
    #    $test->addSection("TCP Connection", 1);
    #}
    #else {
    #    my $message = "TCP connection to $portal_url has failed! Open it on the firewall.";
    #    Xormon::log($message);
    #    $test->addSection("TCP Connection", 0, $message);
    #    $test->stop();
    #    return 0;
    #}

    #$test->stop();

    #return 0;
    #-----------------------------------------------------------------------------------
    # TIME HANDLING
    # OUT: $StartTS, $EndTS
    #-----------------------------------------------------------------------------------
    my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst );
    my $use_time = time();
    my $time = $use_time;
    my $secs_delay = 600;
    my $start_time = $use_time - $secs_delay;
    ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = gmtime($start_time);

    $StartTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, $min );

    my $end_time = $use_time;
    ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = gmtime($end_time);

    $EndTS = sprintf( "%4d-%02d-%02dT%02d:%02d:00Z", $year + 1900, $month + 1, $day, $hour, $min );

    $Frequency = "Minute";

    #---------------------------------------------------------------------------------

    my $data_test_result = 1;


    my %inventory_tags;
    %inventory_tags = CMC_COLLECT::general_hash_request("GET",  "https://${portal_url}/api/public/v1/ep/inventory/tags", $portal_url, $CMC_client_id, $CMC_client_secret, $proxy );           

    if (defined $inventory_tags{Tags} && ref($inventory_tags{Tags}) eq 'ARRAY' ){
    
      for (my $i=0; $i<scalar(@{$inventory_tags{Tags}}); $i++){
        for (my $j=0; $j<scalar(@{$inventory_tags{Tags}[$i]{Systems}}); $j++){
          my %subhash = %{$inventory_tags{Tags}[$i]{Systems}[$j]};
          # possibly could be ""
          if (defined $subhash{PoolID} && $subhash{PoolID}){ 
            my $pool_id = $subhash{PoolID};
            my $UUID    = $subhash{UUID};

            $data{Pools}{$pool_id}{Name} = $subhash{PoolName};

            $data{Pools}{$pool_id}{Systems}{$UUID}{Name} = $subhash{Name};
            $data{Systems}{$UUID}{Pools}{$pool_id}{Name} = $subhash{PoolName};

            $data{Systems}{$UUID}{Name}                = $subhash{Name};
            $data{Systems}{$UUID}{State}               = $subhash{State};
          }
        }
      }

    }
    else {
        # BAD JSON FORMAT, NO DATA
        $data_test_result = 0;

        $data_message .= "Endpoint: https://${portal_url}/api/public/v1/ep/inventory/tags (Proxy: $proxy) \n";
        $data_message .= "No data. \n";
        $data_message .= "Note that all servers in PEP 2.0  must be tagged in order for the CMC API to return performance metrics. \n\n";
        $data_message .= "<a href=\"${docs_install_link}\">Installation documentation</a>";

        $test->addSection("Data Test", 0, $data_message);
        $test->stop();
        return 0;
    }


    # NO LOADED DATA
    if ( scalar(keys %{$data{Systems}}) eq 0 && $data_test_result) {
        $data_test_result = 0;

        $data_message .= "Endpoint: https://${portal_url}/api/public/v1/ep/inventory/tags (Proxy: $proxy) \n";
        $data_message .= "No systems data. \n";
        $data_message .= "Note that all servers in PEP 2.0  must be tagged in order for the CMC API to return performance metrics. \n\n";
        $data_message .= "<a href=\"${docs_install_link}\">Installation documentation</a>";

        $test->addSection("Data Test", 0, $data_message);
        $test->stop();
        return 0;
    }

    # USAGE TAGS
    my $url_usage_tags = "https://${portal_url}/api/public/v1/ep/usage/tags";


    # CHECK ARRAY OF METRICS
    my $metrics_not_ok = 0;
    if ($data_test_result){
      my %usage_tags;

      %usage_tags = CMC_COLLECT::general_hash_request("GET", create_url_usage_tags($url_usage_tags), $portal_url, $CMC_client_id, $CMC_client_secret, $proxy);

      if (defined $usage_tags{Tags}){
        for (my $i=0; $i<scalar(@{$usage_tags{Tags}}); $i++){

          # TAGGED SYSTEMS
          if ($usage_tags{Tags}[$i]{SystemsUsage}){
            for (my $j=0; $j<scalar(@{$usage_tags{Tags}[$i]{SystemsUsage}{Systems}}); $j++){
              my %subhash = %{$usage_tags{Tags}[$i]{SystemsUsage}{Systems}[$j]};

              my $UUID                                         = $subhash{UUID};
              if (defined $data{Systems}{$UUID}){
                if (! defined $data{Systems}{$UUID} && defined $subhash{Usage}{Usage}){
                  $metrics_not_ok = 1;
                  $data{Systems}{$UUID}{Message} = 'Usage data not approachable.';
                }
                else{
                  $data{Systems}{$UUID}{Message} = ''; 
                }
              }
            }
          }
        }
      }  
    }

    my @tagged_pools = keys %{$data{Pools}};

    my $url_usage_pools = "https://${portal_url}/api/public/v1/ep/usage/pools";

    my %usage_pools;
    %usage_pools = CMC_COLLECT::general_hash_request("GET", create_url_usage_pools($url_usage_pools), $portal_url, $CMC_client_id, $CMC_client_secret, $proxy);

    if (defined $usage_pools{Pools}){
      for (my $i=0; $i<scalar(@{$usage_pools{Pools}}); $i++){
        my $ID = $usage_pools{Pools}[$i]{PoolID};
        # possibly could be ""
        if ($ID){ 
          $data{Pools}{$ID}{Name} = $usage_pools{Pools}[$i]{PoolName};
        }
      }
    }
    else{
        # BAD JSON FORMAT, NO DATA
        $data_test_result = 0; 

        $data_message .= "Endpoint: https://${portal_url}/api/public/v1/ep/usage/pools (Proxy: $proxy) \n";
        $data_message .= "No pool data. \n";

        $test->addSection("Data Test", 0, $data_message);
        $test->stop();
        return 0;
    }

    # for test:
    #$data{Pools}{UNTAG}{Name} = 'UNTAGGED_Pool_NAME_x';

    my @all_pools = keys %{$data{Pools}};

    my %pool_check;
    for my $some_pool (@all_pools){
      $pool_check{$some_pool} = 0;
    }
    for my $tagged_pool (@tagged_pools){
      $pool_check{$tagged_pool} = 1;
    }

    my @untagged_pools;
    my $tagging_problem = 0;
    for my $some_pool (keys %pool_check){
      if ($some_pool){
        if (! $pool_check{$some_pool}){
          $tagging_problem = 1;
          push (@untagged_pools, $data{Pools}{$some_pool}{Name});

        }
      }
    }

    #
    # CREATE MAIN DATA MESSAGE
    #
    if ( $data_test_result ){
      if (defined  $data{Pools} && scalar(keys %{$data{Pools}} gt 0)){
        for my $pool_id (sort keys  %{$data{Pools}}){
          my $pool_name = $data{Pools}{$pool_id}{Name};

          #$data_message .= "<hr>Pool: $pool_name \n";
          $data_message .= "Pool: $pool_name \n";
          $data_message .= "Servers: ";
          if (! keys %{ $data{Pools}{$pool_id}{Systems}}){
            $data_message .= "N/A";
          }
          $data_message .= "\n";
          my $longest_group_spacing = 25;

          for my $UUID (keys %{ $data{Pools}{$pool_id}{Systems}}){
            my $server_name  =  $data{Pools}{$pool_id}{Systems}{$UUID}{Name};
            if (length($server_name) > $longest_group_spacing ){
              $longest_group_spacing = length($server_name);   
            }
          }

          $longest_group_spacing += 5;

          for my $UUID (keys %{ $data{Pools}{$pool_id}{Systems}}){
            my $server_name  =  $data{Pools}{$pool_id}{Systems}{$UUID}{Name}; 
            $server_name =~ s/\s*$//;
            my $name_len = $longest_group_spacing - length($server_name);
            my $len_name = length($server_name);
            my $spaces = ' 'x$name_len;
            my $name_print = "$server_name".$spaces;
            my $server_state =  "$data{Systems}{$UUID}{State} $data{Systems}{$UUID}{Message}"; 
            $data_message .= "  $name_print $server_state \n";
            #$data_message .= "  $server_name  \n";# $server_state $name_len $len_name \n";
          }

          $data_message .= "\n";


          if ($metrics_not_ok){
            #$data_message .= '\n Some server data are not approachable.';
          }
        }
      }
      else{
        # NO POOLS FOUND
        $data_test_result = 0;
        $data_message .= "No PEP 2.0 found \n";
        $data_message .= "Note that all servers in PEP 2.0  must be tagged in order for the CMC API to return performance metrics. \n\n";
        $data_message .= "<a href=\"${docs_install_link}\">Installation documentation</a>";

        $test->addSection("Data Test", 0, $data_message);
        $test->stop();
        return 0;
      }
    }

    if ($tagging_problem){
      $data_message .= "\nYou have some pools with no tagged servers: @untagged_pools";
      $data_message .= "\nNote that all servers in PEP 2.0  must be tagged in order for the CMC API to return performance metrics. \n\n";
      $data_message .= "<a href=\"${docs_install_link}\">Installation documentation</a>";

      $test->addSection("Data Test", 0, $data_message);
      $test->stop();
      return 0;
    }

    my $message;
    $test->addSection("Data Test", 1, "<pre>$data_message</pre>");
    $test->stop();
    return 1;
}


#-----------------------------------------------------------------------------------

sub create_url_usage_tags {
  my $base_url = shift;
  $base_url .= "?EndTS=${EndTS}&Frequency=${Frequency}&StartTS=${StartTS}";
  return $base_url;
}

sub create_url_usage_pools {
  my $base_url = shift;

  $base_url .= "?EndTS=${EndTS}&Frequency=Minute&StartTS=${StartTS}";

  return $base_url;
}


1;