use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Test;
use Power;
use Xormon;
use HTTP::Date;
use POSIX('strftime');
require "xml.pl";

use CMC;
use CMC_TEST;
use CMC_COMMON;

$SIG{CHLD} = 'IGNORE';

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3106;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
    Domain => AF_INET,
    Type => SOCK_STREAM,
    Proto => 'tcp',
    LocalHost => '0.0.0.0',
    LocalPort => $port,
    ReusePort => 1,
    Listen => 5,
) || die "Can't open socket: $@";

Xormon::log("Service listening on port $port");

my $client;

my $hw_type = 'power';

 # waiting for a new client connection
while ($client = $server->accept()) {

    my $pid;
    unless (defined( $pid = fork() ) ) {
        Xormon::error("Wait for fork");
        sleep 5;
    }

    if ($pid == 0) {
        # get information about a newly connected client
        my $client_address = $client->peerhost();
        my $client_port = $client->peerport();

        Xormon::log("Connection from $client_address:$client_port");

        # received data template
        #
        # {
        #   "jwt"  : "<backendToken>",
        #   "task" : "<taskName>"
        #   'data' : {
        #     <configuration_from_hostcfg>
        #   }
        # }

        # read up to 2048 characters from the connected client
        my $data = "";
        $client->recv($data, 2048);

        my $decoded = from_json($data);
        my $cfg = $decoded->{'data'};

        #
        # CMC
        #
        # TODO: move out, change call - CMC::service_task(...)
        if ( $cfg->{type} eq "CMC") {
            my $xormonx = Xormon->new("$client_address", $decoded->{'jwt'}, $hw_type, $cfg->{'hostcfg_id'}, $cfg->{'hostalias'});

            #print Dumper $cfg;

            $xormonx->startJob($decoded->{'task'}, $cfg->{'hostalias'}, $$);

            my $status = "successful";
            my $rc;

            if ( $decoded->{'task'} eq 'test' ) {
                Xormon::log("TASK TEST CMC");

                my ($host, $username, $password, $proxy_url) = CMC_COMMON::load_config($cfg);

                my $test = Test->new($client); # closed in CMC_TEST

                my $cmc_test_return = CMC_TEST::test($test, $host, $username, $password, $proxy_url);

                if ( defined $cmc_test_return && $cmc_test_return ) {
                    $rc = 201;
                }
                else {
                    $rc = 401;
                }

            }
            elsif ( $decoded->{'task'} eq 'cmc' ) {
                my $cmc_collect_return = CMC::collect($xormonx, $cfg);

                if ( defined $cmc_collect_return && $cmc_collect_return ) {
                    $rc = 201;
                }
            }
            else {
                $rc = 0;
            }

            if ($rc eq "201") {
                $status = "successful";
            }
            elsif ($rc eq "401") {
                $status = "unauthorized";
            }
            else {
                $status = "error";
            }

            $xormonx->endJob($decoded->{'task'}, $cfg->{'hostalias'}, $status, $$);

            exit();
        }
        #
        # CMC END
        #


        my $power = Power->new($cfg);

        $power->{'logger'}->info("Received task: ".$decoded->{'task'});

        my $xormon = Xormon->new("$client_address", $decoded->{'jwt'}, $hw_type, $cfg->{'hostcfg_id'}, $cfg->{'hostalias'});
        $power->{'xormon'} = $xormon;

        $power->{'microservice_data'} = $power->{'xormon'}->getData();
        $power->{'microservice_data_into_db'} = $power->{'microservice_data'} if (defined $power->{'microservice_data'});

        # sends 0 connection state for all servers related to hostcfg
        my ($combined_connection, $connection_message) = hosts_connection_check($power);
        if ( !$combined_connection ) {
            save_hmc_connection_status($power, 0, $cfg->{'hostcfg_id'}, $connection_message);
            $power->{'HMC_CONNECTION_SAVED'} = 1;
        }

        # login and get session, also check the default host (if it is really available), if not use backup host
        if ($power->login_to_hmc()){
            # this is more of an authorization check
            $power->{'logger'}->error("The login was not successful. End job.");

            $power->{'xormon'}->isConnected(0, "The login was not successful. End job.");
            
            $power->{'xormon'}->endJob($decoded->{'task'}, $cfg->{'hostalias'}, "error", $$);
            # TODO - stop it here (?)
        }
        else {
            $power->{'xormon'}->isConnected(1);
        }

        if ($decoded->{'task'} eq 'perf' || $decoded->{'task'} eq 'conf') {
            ############################# INIT SECTION ####################################
            $client->send("Task [$decoded->{'task'}] successfuly started");
            $client->shutdown(SHUT_WR);
            $power->{'xormon'}->startJob($decoded->{'task'}, $cfg->{'hostalias'}, $$);

            #check if there is a valid session token in database, log out if neccesary and login again
            $power->check_db_token_and_logout_if_necessary();
            $power->{'microservice_data_into_db'}{'SessionToken'} = $power->{'APISession'};
            $power->{'microservice_data_into_db'}{'SessionToken_created'} = time;
            $power->{'task'} = $decoded->{'task'};

            my $resp = $power->{'xormon'}->saveData($power->{'microservice_data_into_db'});

            $power->{'logger'}->info("Save API Session for $power->{'hostcfg_id'}. RC : $resp");

            if ( $power->{'APISession'} eq 'invalid_session_token' ) {
                my $message = "Login to $power->{'host'}:$power->{'port'} was not successful. Wrong username ($power->{'username'}) or password. Also HMC can be unavailable/restarting.";
                $power->{'logger'}->error($message);

                $power->{'xormon'}->endJob($decoded->{'task'}, $cfg->{'hostalias'}, "error", $$);
                $power->{'logger'}->info("Task [$decoded->{'task'}] for ". $cfg->{'hostalias'}. " is done");
                exit;
            }

            ############################### AGENT SECTION ###############################
            # target destinations check
            if (defined $decoded->{'destinations'}) {
                $power->{'xormon'}->setDestinations($decoded->{'destinations'});
            }

            # get data from technology and save to DB
            my $rc = api2json($cfg, $power);

            if ($rc != 201){
                $power->{'logger'}->info("Problem with api2json function");
            }

            ############################### END AGENT SECTION ###############################
            # status code
            my $status;
            if ($rc eq "201") {
                $status = "successful";
            } elsif ($rc eq "401") {
                $status = "unauthorized";
            } else {
                $status = "error";
            }

            $power->{'xormon'}->endJob($decoded->{'task'}, $cfg->{'hostalias'}, $status, $$);

        }
        elsif ($decoded->{'task'} eq "test") {
            ############################# INIT SECTION ####################################
            $power->{'xormon'}->startJob($decoded->{'task'}, $cfg->{'hostalias'}, $$);

            ############################### TEST SECTION ###############################
            #API Test
            my $response = apitest($client, $power);
            my $rc = 201;

            ############################### END TEST SECTION ###############################
            # status code
            my $status;
            if ($rc eq "201") {
                $status = "successful";
            } elsif ($rc eq "401") {
                $status = "unauthorized";
            } else {
                $status = "error";
            }
            $power->{'xormon'}->endJob($decoded->{'task'}, $cfg->{'hostalias'}, $status, $$);

        }
        else {
            my $response = "undefined task";
            $client->send($response);
            $client->shutdown(SHUT_WR);
            $power->{'logger'}->info("Task [$decoded->{'task'}] not defined.");
        }

        my $rc_logout = $power->logout($power->{'APISession'});
        my $login_url = $power->{'protocol'}."://".$power->{'host'}.":".$power->{'port'}."/rest/api/web/Logon";
        my $message = "Logout $power->{'username'} ($rc_logout) at $login_url";

        if ($rc_logout == 204){
            $power->{'logger'}->info("". $message);
            $power->{'microservice_data_into_db'}{'SessionToken'} = "session_logged_out";
        }
        else {
            $power->{'logger'}->error($message);
        }

        #save power->{'microservice_data_into_db'}
        my $resp = $power->{'xormon'}->saveData($power->{'microservice_data_into_db'});
        $power->{'logger'}->info("Save Microservice Data ($resp)");

        $power->{'logger'}->info("Task [$decoded->{'task'}] is done");
        $power->{'logger'}->debug("Total API requests in [$decoded->{'task'}] job: $power->{'numOfRequests'}");

        exit();
    }
    else {
        $client->autoflush(1);
    }
}

# Try connection to HMC (or both if dual HMC mode)
# OK = both HMC approachable
# NOK = at least one HMC cannot be approached
#   OK  -> 1, ""
#   NOK -> 0, "Message describing problem"
sub hosts_connection_check {
    my ($power) = @_;

    my $connected_1 = Test::connTest($power->{'host'}, $power->{'port'});
    
    my $connected_2 = 1;
    if ($power->{'dual_hmc'}) {
        $connected_2 = Test::connTest($power->{'second_host'}, $power->{'port'});
    }
    
    my $message = "";

    my $hmc_address = $power->{'protocol'}."://".$power->{'host'}.":".$power->{'port'};
    my $hmc_user = $power->{'username'} || "undefined user";

    if ( ! $connected_1 ) {
        $message .= "Cannot connect to HMC at  $power->{'protocol'}://$power->{'host'}:$power->{'port'} as ${hmc_user}. ";
    }
    if ( ! $connected_2 ) {
        $message .= "Cannot connect to HMC at $power->{'protocol'}://$power->{'second_host'}:$power->{'port'} as ${hmc_user}.";
    }

    # NOTE: think of sending warning if 1 host is not ok and error if both are not ok
    my $connection_combined = $connected_1 && $connected_2;

    return ($connection_combined, $message);
}

sub save_hmc_connection_status {
    my $power               = shift;
    my $connection_state    = shift; # 1|0
    my $hostcfg_id_send     = shift;
    my $message             = shift || "";

    if ( ! $connection_state ) {
        $power->{'logger'}->info("$message");
        $power->{'logger'}->error("$message");
    }

    $power->{'logger'}->info("HMC connection: $connection_state ");

    my %server_data_status = (
        'hwType'        => 'power',
        'connection'    => int($connection_state), # keep int: otherwise it might get passed as string and not evaluated properly
        'hostcfg_id'    => $hostcfg_id_send,
        'message'       => $message,
    );
    
    my $hs_rc = $power->{'xormon'}->isConnectedForHostcfgId(\%server_data_status);

    $power->{'logger'}->info("HMC status saved ($hs_rc)");

};

sub api2json {
    my ($data, $power) = @_;

    #getServers
    $power->{'logger'}->info("Fetch server from $data->{'host'}");
    my $servers = $power->getServers();

    my $LastFetchTimeServer = $power->{'microservice_data'}->{'LastFetchTimeServer'} ? $power->{'microservice_data'}->{'LastFetchTimeServer'} : 0;
    my $LastFetchTimeLpar = $power->{'microservice_data'}->{'LastFetchTimeLpar'} ? $power->{'microservice_data'}->{'LastFetchTimeLpar'} : 0;

    my $LastFetch = $LastFetchTimeServer ? $LastFetchTimeServer : ($LastFetchTimeLpar ? $LastFetchTimeLpar : 0);
    $power->{'logger'}->debug("LastFetch : $LastFetch");

    #$power->{'logger'}->error("Last Fetch: ($LastFetch) lpar $LastFetchTimeLpar ? server $LastFetchTimeLpar");

    #
    # Architecture: servers
    #
    my $subsystem = 'server';
    for (@{$servers}){
        my $server = $_;

        my $server_uid = $server->{'Metadata'}{'Atom'}{'AtomID'} ? $server->{'Metadata'}{'Atom'}{'AtomID'} : undef;
        my $server_name = $server->{'SystemName'}{'content'} ? $server->{'SystemName'}{'content'} : undef;

        $power->{'logger'}->debug("Working (init) for server $server_name ($server_uid)");

        if (!defined $server_uid || !defined $server_name){
            $power->{'logger'}->error("Not defined server_uid for initial architecture, skip");
            next;
        }
        #architecture server
        my %server_arch = (
            'item_id'       => $server_uid,
            "label"         => $server->{'SystemName'}{'content'},
            "hw_type"       => $hw_type,
            "subsystem"     => $subsystem,
            "parents"       => [],
            "hostcfg_id"    => $data->{'hostcfg_id'}
        );
        push(@{$power->{'architecture_server'}}, \%server_arch);

    }
    my $architecture_server_rc = $power->{'xormon'}->saveArchitecture( $power->{'architecture_server'} );
    $architecture_server_rc = defined $architecture_server_rc ? $architecture_server_rc : "unknown" ;
    $power->{'logger'}->info("Save Architecture ($architecture_server_rc) Server");

    #
    # Architecture: LPARs
    #
    my $lpars = $power->getLpars();
    $subsystem = 'vm';
    for (my $i=0; $i<$power->lpar_multiplicator(); $i++){
        for (@{$lpars}){
            my $lpar = $_;
            my $lpar_uid = $lpar->{'PartitionUUID'}{'content'};
            my $lpar_label = $lpar->{'PartitionName'}{'content'} ? $lpar->{'PartitionName'}{'content'} : $lpar_uid;

            $power->{'logger'}->debug("Working (init) for lpar $lpar_label ($lpar_uid)");

            if ($i>0){
                $lpar_uid = "$lpar_uid$i";
                $lpar_label = "$lpar_label-copy-$i";
            }

            my $server_href = $lpar->{'AssociatedManagedSystem'}{'href'};
            my @server_href_split = split ('/', $server_href);
            my $server_uid = $server_href_split[-1];
            #architecture lpar
            my %lpar_arch = (
                'item_id'       => $lpar_uid,
                "label"         => $lpar_label,
                "hw_type"       => $hw_type,
                "subsystem"     => $subsystem,
                "parents"       => [ $server_uid ],
                "hostcfg_id"    => $data->{'hostcfg_id'}
            );
            #print "Lpar : $lpar_uid label: $lpar_label\n";
            push(@{$power->{'architecture_lpar'}}, \%lpar_arch);
        }
    }

    my $architecture_lpars_rc = $power->{'xormon'}->saveArchitecture( $power->{'architecture_lpar'} );
    $architecture_lpars_rc = defined $architecture_lpars_rc ? $architecture_lpars_rc : "unknown" ;
    $power->{'logger'}->info("Save Architecture ($architecture_lpars_rc) LPAR");

    #
    # Architecture: VIOSs
    #
    my $vioses = $power->getVioses();
    for (@{$vioses}){
        my $vios = $_;
        my $vios_uid = $vios->{'PartitionUUID'}{'content'};
        my $vios_label = $vios->{'PartitionName'}{'content'} ? $vios->{'PartitionName'}{'content'} : $vios_uid;

        $power->{'logger'}->debug("Working (init) for vios $vios_label ($vios_uid)");

        my $server_href = $vios->{'AssociatedManagedSystem'}{'href'};
        my @server_href_split = split ("/", $server_href);
        my $server_uid = $server_href_split[-1];
        #architecture vios
        my %vios_arch = (
            'item_id'       => $vios_uid,
            "label"         => $vios_label,
            "hw_type"       => $hw_type,
            "subsystem"     => $subsystem,
            "parents"       => [ $server_uid ],
            "hostcfg_id"    => $data->{'hostcfg_id'}
        );
        #print "Vios : $vios_uid label: $vios_label\n";
        push(@{$power->{'architecture_vios'}}, \%vios_arch);
    }

    my $architecture_vioses_rc = $power->{'xormon'}->saveArchitecture( $power->{'architecture_vios'} );
    $architecture_vioses_rc = defined $architecture_vioses_rc ? $architecture_vioses_rc : "unknown" ;
    $power->{'logger'}->info("Save Architecture ($architecture_vioses_rc) VIOS");

    ####### PERFORMANCE #######
    if ($power->{'task'} eq 'perf'){

        my $cpu_utilization_coresTotalLparsForThisServer = {};
        my %sharedProcessorPools;
        my $i = 0;

        my $hasPerfData = 0;
        for (@{$servers}){
            my $server_uid = $_->{'Metadata'}{'Atom'}{'AtomID'};
            my $server_name = $_->{'SystemName'}{'content'};

            $power->{'logger'}->debug("Working (performance) for server $server_name ($server_uid)");

            if (!defined $server_uid || !defined $server_name){
                $power->{'logger'}->error("Not defined server_uid for performance, skip");
                next;
            }

            #SharedProcessorPools
            my $serverSharedProcessorPool = $power->getServerSharedProcessorPool($server_uid);

            if ( scalar @{ $serverSharedProcessorPool } > 0 ){
                for (@{ $serverSharedProcessorPool }){
                    my $shp = $_;
                    my $PoolID = $shp->{'PoolID'}{'content'};
                    my $PoolUID= $shp->{'Metadata'}{'Atom'}{'AtomID'};
                    my $PoolName= $shp->{'PoolName'}{'content'};
                    $power->{'logger'}->debug("Working (performance) for shared pool $PoolName ($PoolID)");
                    $sharedProcessorPools{$server_uid}{'SharedProcessorPool'}{$PoolID}{'uid'} = $PoolUID;
                    $sharedProcessorPools{$server_uid}{'SharedProcessorPool'}{$PoolUID}{'id'} = $PoolID;
                    $sharedProcessorPools{$server_uid}{'SharedProcessorPool'}{$PoolID}{'label'} = $PoolName;
                    $sharedProcessorPools{$server_uid}{'SharedProcessorPool'}{$PoolUID}{'label'} = $PoolName;
                }
            }
            #Aggregated Metrics
            #my $AggregatedMetrics = $power->getServerAggregated($server_uid, $LastFetch);
            #print Dumper $AggregatedMetrics;

            #Processed Metrics
            my $ProcessedMetrics = $power->getServerProcessed($server_uid, $LastFetch);
            my $EnergyMetrics = $power->getServerProcessed($server_uid, $LastFetch, undef, "Energy");

            my $data_out = {};
            for (keys %{ $ProcessedMetrics }){
                my $sample_uid = $_;
                my $sample = $ProcessedMetrics->{$sample_uid};

                #LogicalPartition Performance Data
                if ($sample->{'category'}{'term'} eq "LogicalPartition") {
                    (my $label, my $uid) = split ("_", $sample->{'title'}{'content'});
                    $power->{'logger'}->info("Fetch Performance data ($server_name LPAR) $uid\_$label");
                    $power->LparPerformanceToData($data_out, $sample, $server_uid, $cpu_utilization_coresTotalLparsForThisServer);

                    # TODO: get his deeper, add more info
                    $power->{'STATUS'}{'SERVER'}{$server_uid}{'perf-status'} = 'ok';
                    $hasPerfData = 1;
                }
            }
            
            for (keys %{ $ProcessedMetrics }){
                my $sample_uid = $_;

                my $sample = $ProcessedMetrics->{$sample_uid};

                #ManagedSystem Performance Data
                if ($sample->{'category'}{'term'} eq "ManagedSystem") {
                    my ($label, undef, undef) = split ('\+', $sample->{'title'}{'content'});
                    ($label, my $uid) = split ("_", $label);
                    $power->{'logger'}->info("Fetch Performance data ($server_name Server) $uid\_$label");
                    $power->ManagedSystemPerformanceToData($data_out, $sample, $server_uid, \%sharedProcessorPools, $cpu_utilization_coresTotalLparsForThisServer);
                    
                    # TODO: get his deeper, add more info
                    $power->{'STATUS'}{'SERVER'}{$server_uid}{'perf-status'} = 'ok';
                    $hasPerfData = 1;
                }
            }
            


            for (keys %{ $EnergyMetrics }){
                my $sample_uid = $_;
                my $sample = $EnergyMetrics->{$sample_uid};

                # ManagedSystem Energy Data
                if ($sample->{'category'}{'term'} eq "ManagedSystem") {
                    my ($label, undef, undef) = split ('\+', $sample->{'title'}{'content'});
                    ($label, my $uid) = split ("_", $label);
                    $power->{'logger'}->info("Fetch Energy data ($server_name) $uid\_$label");
                    $power->ManagedSystemEnergyToData($data_out, $sample, $server_uid,);

                    # TODO: get his deeper, add more info
                    $power->{'STATUS'}{'SERVER'}{$server_uid}{'perf-status'} = 'ok';
                    $hasPerfData = 1;

                }
                else {
                    $power->{'logger'}->error("Not implemented Energy data for category $sample->{'category'}{'term'} $server_name");
                }
            }

            $power->{'xormon'}->saveJSON2($data_out) if scalar keys %{$data_out} > 0;
        }

        # TODO: REMOVE THIS
        if ($hasPerfData) {
            $power->{'xormon'}->hasData(1);
        } else {
            $power->{'xormon'}->hasData(0, "Unable to get Power performance data (ProcessedMetrics) for lpars nor servers");
        }
    }

    ####### CONFIGURATION #######
    my %conf_out;
    if ($power->{'task'} eq 'conf'){
        $subsystem = 'server';
        for (@{$servers}){
            my $server_uid = $_->{'Metadata'}{'Atom'}{'AtomID'};
            my $server_name = $_->{'SystemName'}{'content'};

            $power->{'logger'}->debug("Working (configuration) for server $server_name ($server_uid)");

            if (!defined $server_uid || !defined $server_name) {
                $power->{'logger'}->error("Not defined server_uid for configuration, skip");
                next;
            }

            #SharedProcessorPools
            my $serverSharedProcessorPool = $power->getServerSharedProcessorPool($server_uid);
            #$serverSharedProcessorPool;
            if ( scalar @{ $serverSharedProcessorPool } > 0 ){
                for (@{ $serverSharedProcessorPool }){
                    my $shp = $_;
                    my $PoolID = $shp->{'PoolID'}{'content'};
                    my $PoolUID= $shp->{'Metadata'}{'Atom'}{'AtomID'};
                    my $PoolName= $shp->{'PoolName'}{'content'};
                    my $shp_detail = $power->getServerSharedProcessorPoolDetail($server_uid, $PoolUID);
                    my $t = [];

                    if (ref($shp_detail->{'AssignedPartitions'}{'link'}) eq 'HASH'){
                        push @{$t}, $shp_detail->{'AssignedPartitions'}{'link'};
                    } elsif (ref($shp_detail->{'AssignedPartitions'}{'link'}) eq 'ARRAY'){
                        $t = $shp_detail->{'AssignedPartitions'}{'link'};
                    }

                    foreach (@{$t}){
                        my $link = $_->{href};
                        my @parts = split ('/', $link);
                        my $lpar_uid = $parts[-1];
                        $conf_out{'vm'}{$lpar_uid}{'SharedPoolLabel'} = $PoolName;
                        $conf_out{'vm'}{$lpar_uid}{'SharedPoolUUID'} = $PoolUID;
                    }

                    #foreach (keys %{$shp}){
                    #    eval{
                    #        $conf_out{'pool'}{$PoolUID}{$_} = $shp->{$_}{'content'} if (defined $shp->{$_}{'content'});
                    #    };
                    #}

                    my $numOfPartitions = scalar @{$t};
                    $power->{'logger'}->info("Fetch Shared Processor Pool ($server_name) $shp->{'PoolName'}{'content'} ($numOfPartitions LPARs)");

                    #if pool has some available partitions, is active
                    if ($numOfPartitions){
                        for (keys %{$shp_detail}){
                            eval {
                                $conf_out{'pool'}{$PoolUID}{$_} = $shp_detail->{$_}{'content'} if (defined $shp_detail->{$_}{'content'});
                            };
                        }
                    }
                }
            }

            #health status to db item properties
            $conf_out{$subsystem}{$server_uid}{'AggregationEnabled'} = $_->{'AggregationEnabled'}{'content'};
            $conf_out{$subsystem}{$server_uid}{'LongTermMonitorEnabled'} = $_->{'LongTermMonitorEnabled'}{'content'};
            $conf_out{$subsystem}{$server_uid}{'ShortTermMonitorEnabled'} = $_->{'ShortTermMonitorEnabled'}{'content'};
            $conf_out{$subsystem}{$server_uid}{'EnergyMonitorEnabled'} = $_->{'EnergyMonitorEnabled'}{'content'};
            $conf_out{$subsystem}{$server_uid}{'EnergyMonitoringCapable'} = $_->{'EnergyMonitoringCapable'}{'content'};
            $conf_out{$subsystem}{$server_uid}{'ComputeLTMEnabled'} = $_->{'ComputeLTMEnabled'}{'content'};

            # Configuration Metrics Server
            $power->{'logger'}->info("Fetch Configuration data Server ($server_name)");
            my $server = $power->getManagedSystemConfiguration($server_uid);
            my $confirurationDetailRc = $power->getManagedSystemConfigurationDetail(\%conf_out, $server);

            #
            # health status server
            # TODO: remove this
            my $server_state = defined $server->{'State'}{'content'} ? $server->{'State'}{'content'} : 'unknown';
            my $health_status;

            if ($server_state eq 'operating'){
                $health_status = 'ok'
            }
            else {
                $health_status = 'warning';
            }

            my %server_status = (
                'item_id'   => $server_uid,
                'status'    => $health_status,
                'updated'   => "".localtime()."",
                'data'      => $server_state
            );

            push(@{$power->{'status'}}, \%server_status);
        }

        #save health status
        #print Dumper $power->{'status'};
        my $hs_rc = $power->{'xormon'}->saveStatus($power->{'status'});
        $power->{'logger'}->info("Save HealthStatus ($hs_rc)");
    }

    server_hs($servers, $power);

    #
    # save architecture
    #
    foreach (@{$power->available_subsystems}){
        if (defined $power->{'architecture'}->{$_->{'subsystem'}}){
            #print Dumper $power->{'architecture'}->{$_->{'subsystem'}};
            my $architecture_rc = $power->{'xormon'}->saveArchitecture( $power->{'architecture'}->{$_->{'subsystem'}} );
            $architecture_rc = defined $architecture_rc ? $architecture_rc : "unknown" ;
            $power->{'logger'}->info("Save Architecture ($architecture_rc) $_->{'label'}");
        }
    }

    #
    # save configuration
    #
    #print Dumper $conf_out{'virtualfibrenpiv'};
    foreach (@{$power->available_subsystems}){
        if (defined $conf_out{$_->{'subsystem'}} && $conf_out{$_->{'subsystem'}} ne ""){
            my $configuration_rc = $power->{'xormon'}->saveConfJSON( $conf_out{$_->{'subsystem'}} );
            $power->{'logger'}->info("Save Configuration ($configuration_rc) $_->{'label'}");
        } else {
            #$power->{'logger'}->info("No Configuration for host $power->{'hostcfg_id'} ($_->{'label'})");
        }
    }

    return 201;
}

# SERVER HEALTH, DATA, CONNECTION
#
# TODO: change to power method
# TODO: use $power to save specific info about performance data
# NOTE: make status array in apitest? => more frequent update
# NOTE: perf run vs conf run - they could overwrite each other
sub server_hs {
    my $servers_info = shift;
    my $power = shift;

    $power->{'logger'}->info("HEALTH: servers");

    my @status_servers = ();

    for my $server_info (@{$servers_info}){
        # NOTE: This is not ideal - health status is based on servers in response
        my $server_uid =  $server_info->{'Metadata'}{'Atom'}{'AtomID'};
        my $server_name =  $server_info->{'SystemName'}{'content'};

        if ( !defined $server_uid || !defined $server_name ) {
            $power->{'logger'}->error("Not defined server_uid for configuration, skip");
            next;
        }

        #$server_info->{'AggregationEnabled'}{'content'};
        #$server_info->{'LongTermMonitorEnabled'}{'content'};
        #$server_info->{'ShortTermMonitorEnabled'}{'content'};
        #$server_info->{'EnergyMonitorEnabled'}{'content'};
        #$server_info->{'EnergyMonitoringCapable'}{'content'};
        #$server_info->{'ComputeLTMEnabled'}{'content'};

        #
        # DATA
        #
        # Check server state: operating, powered off, unknown
        my $data_status = 'ok';
        my $data_message = '';


        if ( $server_info->{'LongTermMonitorEnabled'}{'content'} eq 'false' ) {
            $data_status = 'warning';
            # IMPORTANT: this message is used in HS table regex: if there is this message, then there will be documentation link
            $data_message .= "Long term monitor is not enabled";
            #$data_message .= qq(Follow <a href="https://xormon.com/IBM-Power-monitoring-installation.php" target="_blank">this</a> to enable it.);

        }
        elsif ( $server_info->{'AggregationEnabled'}{'content'} eq 'false' ) {
            $data_status = 'warning';
            $data_message .= "Aggregation is not enabled.";
        }

        # TODO: add time dependence - how many wrong perf collections is backward-collectable? 
        #       => either warning (does not collect, but still could if you fix it) OR error (missing data)
        if ( $power->{'task'} eq 'perf' ) {

            if ( defined $power->{'STATUS'}{'SERVER'}{$server_uid}{'perf-status'} ) {

                if ( $power->{'STATUS'}{'SERVER'}{$server_uid}{'perf-status'} eq 'ok' ) {
                    # Do nothing... pass status
                    #$data_status = 'ok';
                }
                else {
                    $data_status = 'error';

                    if ( $data_message ) {
                        $data_message .= ", no performance data";
                    }
                    else {
                        $data_message .= "No performance data.";
                    }
                }

            }
            else {
                # server was not in performance part of code (does it mean no data?)
                if ( $data_status eq 'ok' ) {
                    $data_status = 'unknown';
                } 

                if ( $data_message ) {
                    $data_message .= ", no performance data";
                }
                else {
                    $data_message .= "No performance data.";
                }

            }
        }


        #
        # HEALTH
        #
        # Check server state: operating, powered off, unknown
        # data = server state
        # TODO: severity for no data = unknown?
        my $server = $power->getManagedSystemConfiguration($server_uid);
        my $server_state = defined $server->{'State'}{'content'} ? $server->{'State'}{'content'} : 'unknown';
        my $health_status;

        if ($server_state eq 'operating'){
            $health_status = 'ok';
        }
        else {
            $health_status = 'warning';
        }

        #
        # CONNECTION
        #
        # Connection state is ok, as this code happens in HMC response.
        my $server_connection_status = 'ok';

        my %server_status = (
            'item_id'   => $server_uid,
            'status'    => $health_status,
            'updated'   => "".localtime()."",
            'data'      => $server_state
        );

        my %server_conn_status = (
            'item_id'   => $server_uid,
            'status'    => $server_connection_status,
            'type'      => 'connection',
            'data'      => ''
        );

        my %server_data_status = (
            'item_id'   => $server_uid,
            'status'    => $data_status,
            'type'      => 'data',
            'data'      => $data_message
        );

        push(@status_servers, \%server_status);

        # Dual HMC might already save connection, one HMC cannot be reached, second provides data
        if ( defined  $power->{'HMC_CONNECTION_SAVED'} && ! $power->{'HMC_CONNECTION_SAVED'} ) {
            push(@status_servers, \%server_conn_status);
        }

        if ( $power->{'task'} eq 'perf' ) {
            push(@status_servers, \%server_data_status);
        }

    }

    my $hs_rc = $power->{'xormon'}->saveStatus(\@status_servers);
    $power->{'logger'}->info("Save Health Status ($hs_rc)");
}

sub apitest {
    my $client = shift;
    my $power  = shift;

    my $test = Test->new($client);

    my $connected = Test::connTest($power->{'host'}, $power->{'port'});
    if ($connected == 1) {
        my $address_message = $power->{'host'} . ":" . $power->{'port'};
        # show address
        $test->addSection("TCP Connection - $address_message", 1);
    }
    else {
        my $message = "TCP connection to " . $power->{'host'} . ":" . $power->{'port'} . " has failed! Open it on the firewall.";
        $power->{'logger'}->error($message);
        $test->addSection("TCP Connection", 0, $message);
    }

    if ( $power->{'APISession'} eq 'invalid_session_token') {
        my $message = "Login to $power->{'host'}:$power->{'port'} was not successful. Wrong username ($power->{'username'}) or password. Also HMC can be unavailable/restarting.";
        $power->{'logger'}->error($message);
        $test->addSection("Authorization test" , 0, $message);
        $test->stop();
    }
    else {
        $test->addSection("Authorization test", 1);
    }

    my $getServers = $power->getServers();
    my $numOfServers = scalar (@{$getServers});

    #my $SSPs = $power->getSSPs();

    if ( $numOfServers <= 0){
        my $login_url = $power->{'protocol'}."://".$power->{'host'}.":".$power->{'port'}."/rest/api/web/Logon";
        my $message = "HMC didn't return any ManagedSystems.\nEnter this URL into a browser and login as $power->{'username'}.\n $login_url";
        $test->addSection("Managed Systems", 0, $message);
        $power->{'logger'}->error($message);
        $test->stop();
    }
    else {
        $test->addSection("Managed Systems ($numOfServers)", 1);
    }

    for my $managedsystem (@{ $getServers }){

        my $ms_status = 'OK';
        my $server_uid = $managedsystem->{'Metadata'}{'Atom'}{'AtomID'};
        my $server_name = $managedsystem->{'SystemName'}{'content'};

        #print "Managed System $server_name ($managedsystem->{'MachineTypeModelSerialNumber'}{'Model'}{'content'}-$managedsystem->{'MachineTypeModelSerialNumber'}{'MachineType'}{'content'} $managedsystem->{'MachineTypeModelSerialNumber'}{'SerialNumber'}{'content'})\n";
        #print "  Aggregation:\t\t$managedsystem->{'AggregationEnabled'}{'content'}\n";
        #print "  LongTermMonitor:\t\t$managedsystem->{'LongTermMonitorEnabled'}{'content'}\n";
        #print "  ShortTermMonitorEnabled:\t$managedsystem->{'ShortTermMonitorEnabled'}{'content'}\n";
        #print "  ComputeLTMEnabled:\t$managedsystem->{'ComputeLTMEnabled'}{'content'}\n";
        #print "  EnergyMonitorEnabled:\t$managedsystem->{'EnergyMonitorEnabled'}{'content'}\n";
        #print "  AssociatedManagedSystem:\t$managedsystem->{'AssociatedManagedSystem'}{'href'}\n";

        if ($managedsystem->{'LongTermMonitorEnabled'}{'content'} eq 'false'){
            my $message = "$server_name : LongTermMonitor is disabled for $server_name \n";
            $message .= qq(Follow <a href="https://xormon.com/IBM-Power-monitoring-installation.php" target="_blank">this</a> to enable it.);

            $power->{'logger'}->error($message );
            $test->addSection("$server_name Performance: LongTermMonitor" , 0, $message, 1);

        }
        elsif ($managedsystem->{'AggregationEnabled'}{'content'} eq 'false') {
            # Aggregation should be enabled by enabling long term monitor
            my $message =  "$server_name : Aggregation is disabled for $server_name";

            $power->{'logger'}->error($message);
            $test->addSection("$server_name Performance: Aggregation" , 0, $message, 1);

        }
        else {
            #$test->addSection("$server_name Performance" , 1);
        }

        #
        # Server Configuration Check
        #
        my $server_configuration = $power->getManagedSystemConfiguration($server_uid);
        my $server_state = $server_configuration->{'State'}{'content'} || 'undefined';

        # Send messages about some specific states of HW, e.g. HW state or firmware version that has some knowns bugs

        #
        # Server status check
        # If server is not in operating state, send a message but do not set status to 0, it can happen when some servers are powered off
        if ($server_state ne 'operating'){
            my $message = "$server_name : Server is in $server_state state";
            $power->{'logger'}->error($message);
            $test->addSection("$server_name Configuration" , 0, "$server_name : Server is in $server_state state", 1);

        }
        else {
            #$test->addSection("$server_name Configuration" , 1);
        }

        #
        # Firmware version check
        # If there is a specific problem with specific firmware, send a short description of the problem and link how to resolve it.
        if ( $server_configuration->{'SystemFirmware'}{'content'} eq "some problematic version of MS firmware" ){
            my $message = '';
            $power->{'logger'}->error("$server_name : Problem with $server_configuration->{'SystemFirmware'}{'content'} is described here: https://www.xormon.com/help/");
            $test->addSection("$server_name Firmware" , 0, "$server_name : Problem with $server_configuration->{'SystemFirmware'}{'content'} is described here: https://www.xormon.com/help/");

        }
        else {
            #$test->addSection("$server_name Firmware" , 1);
        }

        #
        # Time difference check
        my $xormon_time = time;
        my $server_time = int($server_configuration->{'SystemTime'}{'content'} / 1000);
        my $time_diff_xormon_server = $xormon_time - $server_time;

        if ( $time_diff_xormon_server > (2 * 86400) ) {
            my $message = "There is a different time on Xormon server ($xormon_time) and Managed System ($server_time). \n";
            $message .= "Difference of $time_diff_xormon_server seconds can result in data collection problems.";

            $power->{'logger'}->error($message );
            $test->addSection("$server_name Time" , 0, $message, 1);

        }
        #elsif ( $time_diff_xormon_server > 3600 ) {
        #    # 1 hour difference can result in performance collection failures (confirmed from support)
        #    #my $message = "$server_name : There is a different time on Xormon server and Managed System:" . time . " SystemTime: " . $server_configuration->{'SystemTime'}{'content'} / 1000;
        #    my $message = "There is a different time on Xormon server ($xormon_time) and Managed System ($server_time). \n";
        #    $message .= "Difference of $time_diff_xormon_server seconds can result in performance collection problems.";
        #
        #    $power->{'logger'}->error($message );
        #    $test->addSection("$server_name Time" , 0, $message, 1);
        #
        #}
        else {
            #$test->addSection("$server_name Time" , 1);
        }

        #
        # Server Performance Data Check
        #
        #my $processedMetrics = $power->getServerProcessed($server_uid, time-1200);
        my $url = "/rest/api/pcm/ManagedSystem/$server_uid/ProcessedMetrics";
        my $resp = $power->apiRequest('GET', 'application/xml', $url);

        my $processedMetrics = {};
        if (defined $resp->{'entry'}{'id'}){
            # NOTE
            $processedMetrics->{'only_one_uuid'} = $resp->{'entry'};
        }
        else {
            $processedMetrics = $resp->{'entry'};
        }

        if (scalar keys %{ $processedMetrics } < 1){
            my $message = 'ProcessedMetrics endpoint returned empty response.';
            $power->{'logger'}->error($message );
            $test->addSection("$server_name ProcessedMetrics" , 0, $message, 1);

        }
        else {
            #$test->addSection("$server_name ProcessedMetrics" , 1);
        }

        if ($managedsystem->{'AggregationEnabled'}{'content'} ne 'false' && $managedsystem->{'LongTermMonitorEnabled'}{'content'} ne 'false' && $server_state eq 'operating' && scalar keys %{ $processedMetrics } > 1){
            $test->addSection("$server_name" , 1);
        }

    }

    return $test->stop();
}
$server->close();
