package POWERFLEX;

use lib '../../../perl-modules';

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use Xormon;
use JSON qw(encode_json decode_json);
use Date::Parse;
use Test;
use XormonLogger;

my %data;
my $xormon;
my $cfg;
my $logger;

sub collect {
  $xormon = shift;
  $cfg    = shift;
  #print Dumper $cfg;


  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")    && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")     && return "error"; }
  $logger = XormonLogger->new( 'powerflex', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )       { $logger->error("Cannot continue, 'host' is not configured!")          && return "error"; }
  unless ( exists $cfg->{'apiPort'} )       { $logger->error("Cannot continue, 'api-port' is not configured!")      && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )   { $logger->error("Cannot continue, 'apiUsername' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )   { $logger->error("Cannot continue, 'apiPassword' is not configured!")   && return "error"; }

  my $api_ip   = $cfg->{'apiHost'};
  my $api_port = $cfg->{'apiPort'};
  $cfg->{'sample_rate'} = 300;

  # if ( defined $cfg->{'api-password'} && $cfg->{'api-password'} ne "" ) {
  #   my $encode = Xormon::unobscure_password($cfg->{'api-password'});
  #   if ( defined $encode ) {
  #     $cfg->{'api-pass'} = $encode;
  #   }
  # }

  my $timestamp = time();
  #$timestamp = int( $timestamp / 300 ) * 300;
  $cfg->{'timestamp'} = $timestamp;

  my $type;
  my $return;

  my $url = "https://$api_ip:$api_port";
  $cfg->{'url'} = $url;

  ## login
  my $token = login( "$url/api/login" );
  if ( !$token ) {
    $xormon->isConnected(0, "Unsuccessful login! Token not granted");
    $logger->error( "Unsuccessful login! Token not granted, exiting..." ) && return "error";
  }
  else {
    $cfg->{'gen_token'} = $token;
    $xormon->isConnected(1);
    $logger->info("Login successful!");
  }

  #
  # collect perf data
  #

  ##
  ## system
  $type   = "System";
  $return = new_get_request("$url/api/types/$type/instances");
  #print Dumper $return;

  if ( defined $return && ref($return) eq "ARRAY" ) {
    ## array for json to post
    my @post_props = ( "numOfProtectionDomains", "numOfSdc", "numOfSds", "numOfVolumes", "numOfDevices", "numOfVtrees", "numOfStoragePools", "numOfUnmappedVolumes", "numOfMappedToAllVolumes", "numOfThickBaseVolumes", "numOfThinBaseVolumes", "numOfSnapshots", "numOfVolumesInDeletion", "primaryReadBwc", "primaryWriteBwc", "userDataSdcReadLatency", "userDataSdcWriteLatency", "primaryReadFromRmcacheBwc", "rplReceiveBwc", "rplTransmitBwc", "rplApplyLatency", "rplReceiveLatency", "rplTransmitLatency", "rplTotalJournalCap", "rplUsedJournalCap", "normRebuildReadBwc", "normRebuildWriteBwc", "fwdRebuildReadBwc", "fwdRebuildWriteBwc", "bckRebuildReadBwc", "bckRebuildWriteBwc", "pendingNormRebuildCapacityInKb", "pendingFwdRebuildCapacityInKb", "pendingBckRebuildCapacityInKb", "rebalanceReadBwc", "rebalanceWriteBwc", "pendingRebalanceCapacityInKb", "netMaxUserDataCapacityInKb", "netCapacityInUseInKb", "netUnusedCapacityInKb", "maxUserDataCapacityInKb", "capacityInUseInKb", "unusedCapacityInKb" );
    my $json_body->{'properties'} = \@post_props;
    foreach my $system ( @{$return} ) {
      if ( defined $system->{'id'} && $system->{'id'} ne "" ) {
        my $id = $system->{'id'};
        $data{$type}{'conf'}{'id'} = $id;
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $system->{'name'} && $system->{'name'} ne "" ) {
          $data{$type}{'conf'}{'node_name'} = $system->{'name'};
        }
        if ( defined $system->{'systemVersionName'} && $system->{'systemVersionName'} ne "" ) {
          $data{$type}{'conf'}{'systemVersionName'} = $system->{'systemVersionName'};
        }
        if ( defined $system->{'installId'} && $system->{'installId'} ne "" ) {
          $data{$type}{'conf'}{'serial'} = $system->{'installId'};
        }
        if ( defined $system->{'swid'} && $system->{'swid'} ne "" ) {
          $data{$type}{'conf'}{'swid'} = $system->{'swid'};
        }
        if ( defined $system->{'mdmCluster'}->{'clusterMode'} && $system->{'mdmCluster'}->{'clusterMode'} ne "" ) {
          $data{$type}{'conf'}{'clusterMode'} = $system->{'mdmCluster'}->{'clusterMode'};
        }
        ## HEALTH status
        if ( defined $system->{'mdmCluster'} && $system->{'mdmCluster'} ne "" ) {
         $data{$type}{'health'}{'mdmCluster'} = $system->{'mdmCluster'};
        }

        my $post_request = new_post_request( "$url/api/types/$type/instances/action/querySelectedStatistics", $json_body );
        if ( defined $return && ref($return) eq "ARRAY" ) {
          $xormon->hasData(1);
          #print Dumper \$post_request;
          foreach my $metric (@post_props) {
            if ( defined $post_request->{$metric} && $post_request->{$metric} ne "" ) {
              if ( $metric =~ /^numOf/ ) {    ## save subsystem numbers to conf key
                $data{$type}{'conf'}{$metric} = $post_request->{$metric};
              }
              elsif ( $metric =~ /InKb$/ ) {
                my $metric_b = $metric =~ s/InKb/InB/r;
                my $value_kb  = $post_request->{$metric};
                my $value_B  = Xormon::returnBytes( $value_kb, "kB");
                $data{$type}{'perf'}{$metric_b} = $value_B;
              }
              elsif ( $metric eq "rplTotalJournalCap" || $metric eq "rplUsedJournalCap" ) {
                my $value_kb = $post_request->{$metric};
                my $value_B  = Xormon::returnBytes( $value_kb, "kB");
                $data{$type}{'perf'}{$metric} = $value_B;
              }
              elsif ( $metric =~ /^userDataSdcReadLatency$/ || $metric =~ /^userDataSdcWriteLatency$/ || $metric =~ /^rplApplyLatency$/ || $metric =~ /^rplReceiveLatency$/ || $metric =~ /^rplTransmitLatency$/ ) {
                $data{$type}{'perf'}{$metric} = calc_lat( $post_request->{$metric} );
              }
              elsif ( $metric =~ /Bwc$/ ) {    ## if bwc metric, calculate IO and DATA
                my $metric_io = "$metric" . "Io";
                my $metric_bw = "$metric" . "Bw";
                $data{$type}{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$metric} );
                $data{$type}{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$metric} );
              }
              else {
                $data{$type}{'perf'}{$metric} = $post_request->{$metric};
              }
            }    ## else API not return
          }
        }
        else {
          $xormon->hasData(0, "no return from system query");
        }
      }
    }
  }

  ## pool
  $type   = "StoragePool";
  $return = new_get_request("$url/api/types/$type/instances");

  if ( defined $return && ref($return) eq "ARRAY" ) {
    my @post_props = ( "numOfDevices", "numOfVtrees", "numOfVolumes", "numOfUnmappedVolumes", "numOfMappedToAllVolumes", "numOfThickBaseVolumes", "numOfThinBaseVolumes", "numOfSnapshots", "numOfVolumesInDeletion", "primaryReadBwc", "primaryWriteBwc", "userDataSdcReadLatency", "userDataSdcWriteLatency", "primaryReadFromRmcacheBwc", "normRebuildReadBwc", "normRebuildWriteBwc", "pendingNormRebuildCapacityInKb", "fwdRebuildReadBwc", "fwdRebuildWriteBwc", "pendingFwdRebuildCapacityInKb", "bckRebuildReadBwc", "bckRebuildWriteBwc", "pendingBckRebuildCapacityInKb", "rebalanceReadBwc", "rebalanceWriteBwc", "pendingRebalanceCapacityInKb", "netMaxUserDataCapacityInKb", "netCapacityInUseInKb", "netUnusedCapacityInKb", "maxUserDataCapacityInKb", "capacityInUseInKb", "unusedCapacityInKb" );
    my $json_body->{'properties'} = \@post_props;

    foreach my $pool ( @{$return} ) {
      if ( defined $pool->{'id'} && $pool->{'id'} ne "" ) {
        my $id = $pool->{'id'};
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $pool->{'name'} && $pool->{'name'} ne "" ) {
          $data{$type}{$id}{'conf'}{'name'} = $pool->{'name'};
        }
        if ( defined $pool->{'mediaType'} && $pool->{'mediaType'} ne "" ) {
          $data{$type}{$id}{'conf'}{'mediaType'} = $pool->{'mediaType'};
        }
        if ( defined $pool->{'protectionDomainId'} && $pool->{'protectionDomainId'} ne "" ) {
          $data{$type}{$id}{'conf'}{'protectionDomainId'} = $pool->{'protectionDomainId'};
        }
      }
    }

    my $post_request = new_post_request( "$url/api/types/$type/instances/action/querySelectedStatistics", $json_body );
    if ($post_request) {
      foreach my $key ( keys %{$post_request} ) {
        foreach my $metric (@post_props) {
          if ( defined $post_request->{$key}{$metric} && $post_request->{$key}{$metric} ne "" ) {
            if ( $metric =~ /^numOf/ ) {    ## save subsystem numbers to conf key
              $data{$type}{$key}{'conf'}{$metric} = $post_request->{$key}{$metric};
            }
            elsif ( $metric =~ /InKb$/ ) {
              my $metric_b = $metric =~ s/InKb/InB/r;
              my $value_kb  = $post_request->{$key}{$metric};
              my $value_B  = Xormon::returnBytes( $value_kb, "kB");
              $data{$type}{$key}{'perf'}{$metric_b} = $value_B;
            }
            elsif ( $metric =~ /^userDataSdcReadLatency$/ || $metric =~ /^userDataSdcWriteLatency$/ ) {
              $data{$type}{$key}{'perf'}{$metric} = calc_lat( $post_request->{$key}{$metric} );
            }
            elsif ( $metric =~ /Bwc$/ ) {
              my $metric_io = "$metric" . "Io";
              my $metric_bw = "$metric" . "Bw";
              $data{$type}{$key}{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$key}{$metric} );
              $data{$type}{$key}{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$key}{$metric} );
            }
            else {
              $data{$type}{$key}{'perf'}{$metric} = $post_request->{$key}{$metric};
            }
          }
        }
      }
    }
  }

  ## sds
  $type   = "Sds";
  $return = new_get_request("$url/api/types/$type/instances");

  if ( defined $return && ref($return) eq "ARRAY" ) {
    my @post_props = ( "numOfDevices", "primaryReadBwc", "primaryReadFromRmcacheBwc", "totalReadBwc", "totalWriteBwc", "normRebuildReadBwc", "normRebuildWriteBwc", "fwdRebuildReadBwc", "fwdRebuildWriteBwc", "bckRebuildReadBwc", "bckRebuildWriteBwc", "netMaxUserDataCapacityInKb", "netCapacityInUseInKb", "netUnusedCapacityInKb", "maxUserDataCapacityInKb", "capacityInUseInKb", "unusedCapacityInKb", "rebalanceReadBwc", "rebalanceWriteBwc" );
    my $json_body->{'properties'} = \@post_props;

    foreach my $sds ( @{$return} ) {
      if ( defined $sds->{'id'} && $sds->{'id'} ne "" ) {
        my $id = $sds->{'id'};
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $sds->{'name'} && $sds->{'name'} ne "" ) {
          $data{$type}{$id}{'conf'}{'name'} = $sds->{'name'};
        }
        if ( defined $sds->{'port'} && $sds->{'port'} ne "" ) {
          $data{$type}{$id}{'conf'}{'port'} = $sds->{'port'};
        }
        if ( defined $sds->{'protectionDomainId'} && $sds->{'protectionDomainId'} ne "" ) {
          $data{$type}{$id}{'conf'}{'protectionDomainId'} = $sds->{'protectionDomainId'};
        }
        if ( defined $sds->{'mdmConnectionState'} && $sds->{'mdmConnectionState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'mdmConnectionState'} = $sds->{'mdmConnectionState'};
        }
        if ( defined $sds->{'perfProfile'} && $sds->{'perfProfile'} ne "" ) {
          $data{$type}{$id}{'conf'}{'perfProfile'} = $sds->{'perfProfile'};
        }
        if ( defined $sds->{'membershipState'} && $sds->{'membershipState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'membershipState'} = $sds->{'membershipState'};
        }
        if ( defined $sds->{'sdsState'} && $sds->{'sdsState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'sdsState'} = $sds->{'sdsState'};
        }
      }
    }
    my $post_request = new_post_request( "$url/api/types/$type/instances/action/querySelectedStatistics", $json_body );

    if ($post_request) {
      foreach my $key ( keys %{$post_request} ) {
        foreach my $metric (@post_props) {
          if ( defined $post_request->{$key}{$metric} && $post_request->{$key}{$metric} ne "" ) {
            if ( $metric =~ /^numOf/ ) {    ## save subsystem numbers to conf key
              $data{$type}{$key}{'conf'}{$metric} = $post_request->{$key}{$metric};
            }
            elsif ( $metric =~ /InKb$/ ) {
              # my $metric_tb = $metric =~ s/InKb/InTb/r;
              # my $value_kb  = $post_request->{$key}{$metric};
              # my $value_tb  = sprintf( '%.2f', $value_kb / $divider );
              # $data{$type}{$key}{'perf'}{$metric_tb} = $value_tb;

              my $metric_b = $metric =~ s/InKb/InB/r;
              my $value_kb  = $post_request->{$key}{$metric};
              my $value_B  = Xormon::returnBytes( $value_kb, "kB");
              $data{$type}{$key}{'perf'}{$metric_b} = $value_B;
            }
            elsif ( $metric =~ /Bwc$/ ) {
              my $metric_io = "$metric" . "Io";
              my $metric_bw = "$metric" . "Bw";
              $data{$type}{$key}{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$key}{$metric} );
              $data{$type}{$key}{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$key}{$metric} );
            }
            else {
              $data{$type}{$key}{'perf'}{$metric} = $post_request->{$key}{$metric};
            }
          }
        }
      }
    }
  }

  ## sdc
  $type   = "Sdc";
  $return = new_get_request("$url/api/types/$type/instances");

  #print Dumper $return;

  if ( defined $return && ref($return) eq "ARRAY" ) {
    my @post_props = ( "userDataReadBwc", "userDataWriteBwc", "numOfMappedVolumes", "volumeIds", "userDataSdcReadLatency", "userDataSdcWriteLatency" );
    my $json_body->{'properties'} = \@post_props;
    foreach my $sdc ( @{$return} ) {
      if ( defined $sdc->{'id'} && $sdc->{'id'} ne "" ) {
        my $id = $sdc->{'id'};
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $sdc->{'name'} && $sdc->{'name'} ne "" ) {
          $data{$type}{$id}{'conf'}{'name'} = $sdc->{'name'};
        }
        if ( defined $sdc->{'sdcIp'} && $sdc->{'sdcIp'} ne "" ) {
          $data{$type}{$id}{'conf'}{'sdcIp'} = $sdc->{'sdcIp'};
        }
        if ( defined $sdc->{'osType'} && $sdc->{'osType'} ne "" ) {
          $data{$type}{$id}{'conf'}{'osType'} = $sdc->{'osType'};
        }
        if ( defined $sdc->{'mdmConnectionState'} && $sdc->{'mdmConnectionState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'mdmConnectionState'} = $sdc->{'mdmConnectionState'};
        }
        if ( defined $sdc->{'perfProfile'} && $sdc->{'perfProfile'} ne "" ) {
          $data{$type}{$id}{'conf'}{'perfProfile'} = $sdc->{'perfProfile'};
        }
        if ( defined $sdc->{'sdcGuid'} && $sdc->{'sdcGuid'} ne "" ) {
          $data{$type}{$id}{'conf'}{'sdcGuid'} = $sdc->{'sdcGuid'};
        }
        if ( defined $sdc->{'systemId'} && $sdc->{'systemId'} ne "" ) {
          $data{$type}{$id}{'conf'}{'systemId'} = $sdc->{'systemId'};
        }
        if ( defined $sdc->{'versionInfo'} && $sdc->{'versionInfo'} ne "" ) {
          $data{$type}{$id}{'conf'}{'versionInfo'} = $sdc->{'versionInfo'};
        }
      }
    }

    my $post_request = new_post_request( "$url/api/types/$type/instances/action/querySelectedStatistics", $json_body );

    if ($post_request) {
      foreach my $key ( keys %{$post_request} ) {
        foreach my $metric (@post_props) {
          if ( defined $post_request->{$key}{$metric} && $post_request->{$key}{$metric} ne "" ) {
            if ( $metric =~ /^volumeIds/ ) {
              $data{$type}{$key}{'conf'}{$metric} = $post_request->{$key}{$metric};
            }
            elsif ( $metric =~ /^userDataSdcReadLatency$/ || $metric =~ /^userDataSdcWriteLatency$/ ) {
              $data{$type}{$key}{'perf'}{$metric} = calc_lat( $post_request->{$key}{$metric} );
            }
            elsif ( $metric =~ /^numOf/ ) {    ## save subsystem numbers to conf key
              $data{$type}{$key}{'conf'}{$metric} = $post_request->{$key}{$metric};
            }
            elsif ( $metric =~ /Bwc$/ ) {
              my $metric_io = "$metric" . "Io";
              my $metric_bw = "$metric" . "Bw";
              $data{$type}{$key}{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$key}{$metric} );
              $data{$type}{$key}{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$key}{$metric} );
            }
            else {
              $data{$type}{$key}{'perf'}{$metric} = $post_request->{$key}{$metric};
            }
          }
        }
      }
    }
  }

  ## volume
  $type   = "Volume";
  $return = new_get_request("$url/api/types/$type/instances");

  #print Dumper $return;
  if ( defined $return && ref($return) eq "ARRAY" ) {
    my @json_props = ( "numOfMappedSdcs", "numOfChildVolumes", "numOfDescendantVolumes", "userDataReadBwc", "userDataWriteBwc", "userDataSdcReadLatency", "userDataSdcWriteLatency" );
    my $json_body->{'properties'} = \@json_props;
    foreach my $volume ( @{$return} ) {
      if ( defined $volume->{'id'} && $volume->{'id'} ne "" ) {
        my $id = $volume->{'id'};
        $data{$type}{$id}{'conf'}{'id'} = $id;
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $volume->{'name'} && $volume->{'name'} ne "" ) {
          $data{$type}{$id}{'conf'}{'name'} = $volume->{'name'};
        }
        if ( defined $volume->{'storagePoolId'} && $volume->{'storagePoolId'} ne "" ) {
          $data{$type}{$id}{'conf'}{'storagePoolId'} = $volume->{'storagePoolId'};
        }
        if ( defined $volume->{'volumeType'} && $volume->{'volumeType'} ne "" ) {
          $data{$type}{$id}{'conf'}{'volumeType'} = $volume->{'volumeType'};
        }
        if ( defined $volume->{'sizeInKb'} && $volume->{'sizeInKb'} ne "" ) {
          my $cap = Xormon::returnBytes( $volume->{'sizeInKb'}, "kB" );
          $data{$type}{$id}{'conf'}{'capacity_total'} = $cap;
          $data{$type}{$id}{'perf'}{'sizeInB'} = $cap;
        }
      }
    }
    my $post_request = new_post_request( "$url/api/types/$type/instances/action/querySelectedStatistics", $json_body );

    if ($post_request) {
      foreach my $key ( keys %{$post_request} ) {
        foreach my $metric (@json_props) {
          if ( defined $post_request->{$key}{$metric} && $post_request->{$key}{$metric} ne "" ) {
            if ( $metric =~ /^numOf/ ) {    ## save subsystem numbers to conf key
              $data{$type}{$key}{'conf'}{$metric} = $post_request->{$key}{$metric};
            }
            elsif ( $metric =~ /Bwc$/ ) {
              my $metric_io = "$metric" . "Io";
              my $metric_bw = "$metric" . "Bw";
              $data{$type}{$key}{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$key}{$metric} );
              $data{$type}{$key}{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$key}{$metric} );
            }
            elsif ( $metric =~ /^userDataSdcReadLatency$/ || $metric =~ /^userDataSdcWriteLatency$/ ) {
              $data{$type}{$key}{'perf'}{$metric} = calc_lat( $post_request->{$key}{$metric} );
            }
            else {
              $data{$type}{$key}{'perf'}{$metric} = $post_request->{$key}{$metric};
            }
          }
        }
      }
    }
  }

  ## protection domain
  $type   = "ProtectionDomain";
  $return = new_get_request("$url/api/types/$type/instances");

  #print Dumper $return;

  if ( defined $return && ref($return) eq "ARRAY" ) {
    my @post_props = ( "numOfSds", "numOfStoragePools", "numOfFaultSets", "numOfUnmappedVolumes", "numOfMappedToAllVolumes", "numOfThickBaseVolumes", "numOfThinBaseVolumes", "numOfSnapshots", "numOfVolumesInDeletion", "primaryReadBwc", "primaryWriteBwc", "userDataSdcReadLatency", "userDataSdcWriteLatency", "primaryReadFromRmcacheBwc", "normRebuildReadBwc", "normRebuildWriteBwc", "fwdRebuildReadBwc", "fwdRebuildWriteBwc", "bckRebuildReadBwc", "bckRebuildWriteBwc", "pendingNormRebuildCapacityInKb", "pendingFwdRebuildCapacityInKb", "pendingBckRebuildCapacityInKb", "rebalanceReadBwc", "rebalanceWriteBwc", "pendingRebalanceCapacityInKb", "netMaxUserDataCapacityInKb", "netCapacityInUseInKb", "netUnusedCapacityInKb", "maxUserDataCapacityInKb", "capacityInUseInKb", "unusedCapacityInKb" );
    my $json_body->{'properties'} = \@post_props;

    foreach my $prot_dom ( @{$return} ) {
      if ( defined $prot_dom->{'id'} && $prot_dom->{'id'} ne "" ) {
        my $id = $prot_dom->{'id'};
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $prot_dom->{'name'} && $prot_dom->{'name'} ne "" ) {
          $data{$type}{$id}{'conf'}{'name'} = $prot_dom->{'name'};
        }
        if ( defined $prot_dom->{'protectionDomainState'} && $prot_dom->{'protectionDomainState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'protectionDomainState'} = $prot_dom->{'protectionDomainState'};
        }
        if ( defined $prot_dom->{'systemId'} && $prot_dom->{'systemId'} ne "" ) {
          $data{$type}{$id}{'conf'}{'systemId'} = $prot_dom->{'systemId'};
        }
      }
    }
    my $post_request = new_post_request( "$url/api/types/$type/instances/action/querySelectedStatistics", $json_body );

    if ($post_request) {
      foreach my $key ( keys %{$post_request} ) {
        foreach my $metric (@post_props) {
          if ( defined $post_request->{$key}{$metric} && $post_request->{$key}{$metric} ne "" ) {
            if ( $metric =~ /^numOf/ ) {    ## save subsystem numbers to conf key
              $data{$type}{$key}{'conf'}{$metric} = $post_request->{$key}{$metric};
            }
            elsif ( $metric =~ /InKb$/ ) {
              # if ( $metric eq "pendingNormRebuildCapacityInKb" || $metric eq "pendingFwdRebuildCapacityInKb" || $metric eq "pendingBckRebuildCapacityInKb" || $metric eq "pendingRebalanceCapacityInKb" ) {
              #   my $metric_gb = $metric =~ s/InKb/InGb/r;
              #   my $value_kb  = $post_request->{$key}{$metric};
              #   my $value_gb  = sprintf( '%.2f', $value_kb / ( 1024 * 1024 ) );
              #   $data{$type}{$key}{'perf'}{$metric_gb} = $value_gb;
              # }
              # else {
              #   my $metric_tb = $metric =~ s/InKb/InTb/r;
              #   my $value_kb  = $post_request->{$key}{$metric};
              #   my $value_tb  = sprintf( '%.2f', $value_kb / $divider );
              #   $data{$type}{$key}{'perf'}{$metric_tb} = $value_tb;
              #   if ( $metric =~ /^netMaxUserDataCapacityInKb$/ || $metric =~ /^netCapacityInUseInKb$/ || $metric =~ /^netUnusedCapacityInKb$/ ) {
              #     $data{$type}{$key}{'conf'}{$metric_tb} = $value_tb;
              #   }
              # }

              my $metric_b = $metric =~ s/InKb/InB/r;
              my $value_kb  = $post_request->{$key}{$metric};
              my $value_B  = Xormon::returnBytes( $value_kb, "kB");
              $data{$type}{$key}{'perf'}{$metric_b} = $value_B;
            }
            elsif ( $metric =~ /Bwc$/ ) {
              my $metric_io = "$metric" . "Io";
              my $metric_bw = "$metric" . "Bw";
              $data{$type}{$key}{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$key}{$metric} );
              $data{$type}{$key}{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$key}{$metric} );
            }
            elsif ( $metric =~ /^userDataSdcReadLatency$/ || $metric =~ /^userDataSdcWriteLatency$/ ) {
              $data{$type}{$key}{'perf'}{$metric} = calc_lat( $post_request->{$key}{$metric} );
            }
            else {
              $data{$type}{$key}{'perf'}{$metric} = $post_request->{$key}{$metric};
            }
          }
        }
      }
    }
  }

  ## devices
  $type   = "Device";
  $return = new_get_request("$url/api/types/$type/instances");

  #print Dumper $return;
  if ( defined $return && ref($return) eq "ARRAY" ) {
    my @post_props = ( "primaryReadBwc", "primaryReadFromRmcacheBwc", "avgReadLatencyInMicrosec", "avgWriteLatencyInMicrosec", "totalReadBwc", "totalWriteBwc", "normRebuildReadBwc", "normRebuildWriteBwc", "fwdRebuildReadBwc", "fwdRebuildWriteBwc", "bckRebuildReadBwc", "bckRebuildWriteBwc", "rebalanceReadBwc", "rebalanceWriteBwc", "netMaxUserDataCapacityInKb", "netCapacityInUseInKb", "netUnusedCapacityInKb", "maxUserDataCapacityInKb", "capacityInUseInKb", "unusedCapacityInKb" );
    my $json_body->{'properties'} = \@post_props;
    foreach my $device ( @{$return} ) {
      if ( defined $device->{'id'} && $device->{'id'} ne "" ) {
        my $id = $device->{'id'};
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $device->{'name'} && $device->{'name'} ne "" ) {
          $data{$type}{$id}{'conf'}{'name'} = $device->{'name'};
        }
        if ( defined $device->{'vendorName'} && $device->{'vendorName'} ne "" ) {
          $data{$type}{$id}{'conf'}{'vendorName'} = $device->{'vendorName'};
        }
        if ( defined $device->{'modelName'} && $device->{'modelName'} ne "" ) {
          $data{$type}{$id}{'conf'}{'modelName'} = $device->{'modelName'};
        }
        if ( defined $device->{'slotNumber'} && $device->{'slotNumber'} ne "" ) {
          $data{$type}{$id}{'conf'}{'slotNumber'} = $device->{'slotNumber'};
        }
        if ( defined $device->{'serialNumber'} && $device->{'serialNumber'} ne "" ) {
          $data{$type}{$id}{'conf'}{'serialNumber'} = $device->{'serialNumber'};
        }
        if ( defined $device->{'mediaType'} && $device->{'mediaType'} ne "" ) {
          $data{$type}{$id}{'conf'}{'mediaType'} = $device->{'mediaType'};
        }
        if ( defined $device->{'storagePoolId'} && $device->{'storagePoolId'} ne "" ) {
          $data{$type}{$id}{'conf'}{'storagePoolId'} = $device->{'storagePoolId'};
        }
        if ( defined $device->{'maxCapacityInKb'} && $device->{'maxCapacityInKb'} ne "" ) {
          $data{$type}{$id}{'conf'}{'maxCapacityInKb'} = $device->{'maxCapacityInKb'};
        }
        if ( defined $device->{'deviceOriginalPathName'} && $device->{'deviceOriginalPathName'} ne "" ) {
          $data{$type}{$id}{'conf'}{'deviceOriginalPathName'} = $device->{'deviceOriginalPathName'};
        }
        if ( defined $device->{'deviceCurrentPathName'} && $device->{'deviceCurrentPathName'} ne "" ) {
          $data{$type}{$id}{'conf'}{'deviceCurrentPathName'} = $device->{'deviceCurrentPathName'};
        }
        if ( defined $device->{'deviceType'} && $device->{'deviceType'} ne "" ) {
          $data{$type}{$id}{'conf'}{'deviceType'} = $device->{'deviceType'};
        }
        if ( defined $device->{'deviceState'} && $device->{'deviceState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'deviceState'} = $device->{'deviceState'};
        }
        if ( defined $device->{'errorState'} && $device->{'errorState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'errorState'} = $device->{'errorState'};
        }
        if ( defined $device->{'aggregatedState'} && $device->{'aggregatedState'} ne "" ) {
          $data{$type}{$id}{'conf'}{'aggregatedState'} = $device->{'aggregatedState'};
        }
        if ( defined $device->{'sdsId'} && $device->{'sdsId'} ne "" ) {
          $data{$type}{$id}{'conf'}{'sdsId'} = $device->{'sdsId'};
        }
      }
    }

    my $post_request = new_post_request( "$url/api/types/$type/instances/action/querySelectedStatistics", $json_body );
    #print Dumper $post_request;
    if ($post_request) {
      foreach my $key ( keys %{$post_request} ) {
        foreach my $metric (@post_props) {
          if ( defined $post_request->{$key}{$metric} && $post_request->{$key}{$metric} ne "" ) {
            if ( $metric =~ /Bwc$/ ) {
              my $metric_io = "$metric" . "Io";
              my $metric_bw = "$metric" . "Bw";
              $data{$type}{$key}{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$key}{$metric} );
              $data{$type}{$key}{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$key}{$metric} );
            }
            elsif ( $metric =~ /InKb$/ ) {
              # my $metric_tb = $metric =~ s/InKb/InTb/r;
              # my $value_kb  = $post_request->{$key}{$metric};
              # my $value_tb  = sprintf( '%.2f', $value_kb / $divider );
              # $data{$type}{$key}{'perf'}{$metric_tb} = $value_tb;
              my $metric_b = $metric =~ s/InKb/InB/r;
              my $value_kb  = $post_request->{$key}{$metric};
              my $value_B  = Xormon::returnBytes( $value_kb, "kB");
              $data{$type}{$key}{'perf'}{$metric_b} = $value_B;

            }
            elsif ($metric eq "avgReadLatencyInMicrosec" || $metric eq "avgWriteLatencyInMicrosec") {
              my $metric_sec = $metric =~ s/Microsec/Millisec/r; #change key in print data also
              my $value_micro = $post_request->{$key}{$metric};
              my $value_sec = Xormon::formatNumberByType( "float4", ($value_micro / 1000) ); #microsec to millisec
              $data{$type}{$key}{'perf'}{$metric_sec} = $value_sec;
            }
            else {
              $data{$type}{$key}{'perf'}{$metric} = $post_request->{$key}{$metric};
            }
          }
        }
      }
    }
  }
  #print Dumper \%data;

  health_status();
  my $rc_print = print_data();

  ## logout
  my $logout = logout("$url/api/logout");

  if ($logout) {
    $logger->info("Logout successful!");
  }
  else {
    $logger->info("Logout failed!");
  }

  return $rc_print;
}

sub print_data {
  my @architecture;
  my $data_out = "";
  my %conf;
  my $st_uuid = $cfg->{'hostcfg_id'};
  my $rc;
  my $status;
  my $time = $cfg->{'timestamp'};
  my $main_rc = "successful";

  ############
  ## SYSTEM ##
  ############
  # my $hash_key = Xormon::create_hash_key($st_uuid."SYSTEM"); $hash_key = lc($hash_key);
  my %device_arch = (
    "item_id" => $st_uuid,
    "label" => $cfg->{'hostalias'},
    "hw_type" => "powerflex",
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $st_uuid
  );
  push(@architecture, \%device_arch);

  # my %arch = (
  #     "item_id" => $hash_key,
  #     "label" => $cfg->{'hostalias'},
  #     "hw_type" => "powerflex",
  #     "subsystem" => "system",
  #     "parents" => [$cfg->{'hostcfg_id'}],
  #     "hostcfg_id" => $cfg->{'hostcfg_id'}
  #   );
  # push(@architecture, \%arch);

  my $htype    = "System";

  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;block_size;block_size_r;block_size_w;resp_t;resp_t_r;resp_t_w;r_cache_hit;rebalance_rate;rebalance_pending;rebuild_rate;rebuild_pending;capacity_total;capacity_used;capacity_free;capacity_used_percent;raw_capacity_total;raw_capacity_used;raw_capacity_free;replication_in;replication_out;replication_app_resp_t;replication_in_resp_t;replication_out_resp_t;replication_journal_total;replication_journal_used\n";

  my $conf_ref = $data{$htype}{'conf'};
  my $perf_ref = $data{$htype}{'perf'};

  ## system conf print
  foreach my $key ( keys %{$conf_ref} ) {
    # if ( $key eq "id" ) {
    #   $conf{$st_uuid}{'id'} = $conf_ref->{'id'};
    # }
    # if ( $key eq "name" ) {
    #   $conf{$st_uuid}{'name'} = $conf_ref->{'name'};
    # }
    if ( $key eq "systemVersionName" ) {
      my $tmp_ver = $conf_ref->{systemVersionName};
      if ( $tmp_ver =~ /Version:/ ) {
        my ( $sys_model, $sys_version ) = split( / Version: /, $tmp_ver );
        $conf{$st_uuid}{'model'} = $sys_model;
        $conf{$st_uuid}{'version'} = $sys_version;
        next;
      }
    }
    my $value = $conf_ref->{$key};
    $conf{$st_uuid}{$key} = $value;
  }
  $conf{$st_uuid}{'ip'} = $cfg->{'apiHost'};

  my $io_r          = "";
  my $io_w          = "";
  my $io_rate       = "";
  my $data_r        = "";
  my $data_w        = "";
  my $data_rate     = "";
  my $block_size    = "";
  my $block_size_r  = "";
  my $block_size_w  = "";
  my $resp_t_r      = "";
  my $resp_t_w      = "";
  my $resp_t        = "";
  my $cache_hit_r   = "";
  # replication
  my $rpl_rx        = "";
  my $rpl_tx        = "";
  my $rpl_ap_lat    = "";
  my $rpl_rx_lat    = "";
  my $rpl_tx_lat    = "";
  my $rpl_total     = "";
  my $rpl_used      = "";
  #reba,rbld
  my $rebalance     = "";
  my $pend_reba_cap = "";
  my $rebuild       = "";
  my $pend_rbld_cap = "";
  #cap
  my $total         = "";
  my $used          = "";
  my $free          = "";
  my $used_pct      = "";
  my $r_total       = "";
  my $r_used        = "";
  my $r_free        = "";

  #io
  if ( defined $perf_ref->{'primaryReadBwcIo'} && $perf_ref->{'primaryReadBwcIo'} ne ""){
    $io_r = $perf_ref->{'primaryReadBwcIo'};
    $io_rate = $io_r;
  }
  if ( defined $perf_ref->{'primaryWriteBwcIo'} && $perf_ref->{'primaryWriteBwcIo'} ne ""){
    $io_w = $perf_ref->{'primaryWriteBwcIo'};
    if (Xormon::isdigit($io_r)){
      $io_rate += $io_w;
    }
    else{
      $io_rate = $io_w;
    }
  }
  #data
  if ( defined $perf_ref->{'primaryReadBwcBw'} && $perf_ref->{'primaryReadBwcBw'} ne ""){
    $data_r = $perf_ref->{'primaryReadBwcBw'};
    $data_rate = $data_r;
  }
  if ( defined $perf_ref->{'primaryWriteBwcBw'} && $perf_ref->{'primaryWriteBwcBw'} ne ""){
    $data_w = $perf_ref->{'primaryWriteBwcBw'};
    if (Xormon::isdigit($data_r)){
      $data_rate += $data_w;
    }
    else{
      $data_rate = $data_w;
    }
  }
  # calculate block size
  if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
    if ( $io_rate == 0 ) {
      $block_size = 0;
    }
    else{
      $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
    }
  }
  if ( Xormon::isdigit($io_r) && Xormon::isdigit($data_r) ) {
    if ( $io_r == 0 ) {
      $block_size_r = 0;
    }
    else{
      $block_size_r = Xormon::formatNumberByType("float4", ($data_r / $io_r));
    }
  }
  if ( Xormon::isdigit($io_w) && Xormon::isdigit($data_w) ) {
    if ( $io_w == 0 ) {
      $block_size_w = 0;
    }
    else{
      $block_size_w = Xormon::formatNumberByType("float4", ($data_w / $io_w));
    }
  }
  #resp_t
  if ( defined $perf_ref->{'userDataSdcReadLatency'} && $perf_ref->{'userDataSdcReadLatency'} ne ""){
    $resp_t_r = Xormon::formatNumberByType("float4",$perf_ref->{'userDataSdcReadLatency'});
  }
  if ( defined $perf_ref->{'userDataSdcWriteLatency'} && $perf_ref->{'userDataSdcWriteLatency'} ne ""){
    $resp_t_w = Xormon::formatNumberByType("float4",$perf_ref->{'userDataSdcWriteLatency'});
  }
  # calculate resp_t
  if ( Xormon::isdigit( $resp_t_r ) && Xormon::isdigit( $io_r ) && Xormon::isdigit( $resp_t_w ) && Xormon::isdigit( $io_w ) ) {
    my $resp_t_help = Xormon::calculate_weighted_avg( [ { count => $io_r, value => $resp_t_r }, { count => $io_w, value => $resp_t_w } ] );
    if (defined $resp_t_help) { $resp_t = Xormon::formatNumberByType("float4",$resp_t_help); }
  }
  #cache_hit
  if ( Xormon::isdigit($io_r) && defined $perf_ref->{'primaryReadFromRmcacheBwcIo'} && $perf_ref->{'primaryReadFromRmcacheBwcIo'} ne ""){
    if ( $io_r == 0 ) {
      $cache_hit_r = 0;
    }
    else{
      $cache_hit_r = Xormon::formatNumberByType("float4", ($perf_ref->{'primaryReadFromRmcacheBwcIo'} / ( $io_r / 100 ) ) );
    }
  }
  #replication
  if ( defined $perf_ref->{'rplReceiveBwcBw'} && $perf_ref->{'rplReceiveBwcBw'} ne ""){
    $rpl_rx = $perf_ref->{'rplReceiveBwcBw'};
  }
  if ( defined $perf_ref->{'rplTransmitBwcBw'} && $perf_ref->{'rplTransmitBwcBw'} ne ""){
    $rpl_tx = $perf_ref->{'rplTransmitBwcBw'};
  }
  if ( defined $perf_ref->{'rplApplyLatency'} && $perf_ref->{'rplApplyLatency'} ne ""){
    $rpl_ap_lat = $perf_ref->{'rplApplyLatency'};
  }
  if ( defined $perf_ref->{'rplReceiveLatency'} && $perf_ref->{'rplReceiveLatency'} ne ""){
    $rpl_rx_lat = $perf_ref->{'rplReceiveLatency'};
  }
  if ( defined $perf_ref->{'rplTransmitLatency'} && $perf_ref->{'rplTransmitLatency'} ne ""){
    $rpl_tx_lat = $perf_ref->{'rplTransmitLatency'};
  }
  if ( defined $perf_ref->{'rplTotalJournalCap'} && $perf_ref->{'rplTotalJournalCap'} ne ""){
    $rpl_total = $perf_ref->{'rplTotalJournalCap'};
  }
  if ( defined $perf_ref->{'rplUsedJournalCap'} && $perf_ref->{'rplUsedJournalCap'} ne ""){
    $rpl_used = $perf_ref->{'rplUsedJournalCap'};
  }
  #rebalance
  if ( defined $perf_ref->{'rebalanceReadBwcBw'} && $perf_ref->{'rebalanceReadBwcBw'} ne ""){
    $rebalance = $perf_ref->{'rebalanceReadBwcBw'};
  }
  if ( defined $perf_ref->{'rebalanceWriteBwcBw'} && $perf_ref->{'rebalanceWriteBwcBw'} ne ""){
    if (Xormon::isdigit($rebalance)){
      $rebalance += $perf_ref->{'rebalanceWriteBwcBw'};
    }
    else{
      $rebalance = $perf_ref->{'rebalanceWriteBwcBw'};
    }
  }
  if ( defined $perf_ref->{'pendingRebalanceCapacityInB'} && $perf_ref->{'pendingRebalanceCapacityInB'} ne ""){
    $pend_reba_cap = $perf_ref->{'pendingRebalanceCapacityInB'};
  }
  #rbld
  if ( defined $perf_ref->{'normRebuildReadBwcBw'} && $perf_ref->{'normRebuildReadBwcBw'} ne ""){
      $rebuild = $perf_ref->{'normRebuildReadBwcBw'};
  }
  if ( defined $perf_ref->{'normRebuildWriteBwcBw'} && $perf_ref->{'normRebuildWriteBwcBw'} ne ""){
    if (Xormon::isdigit($rebuild)){
      $rebuild += $perf_ref->{'normRebuildWriteBwcBw'};
    }
    else{
      $rebuild = $perf_ref->{'normRebuildWriteBwcBw'};
    }
  }
  if ( defined $perf_ref->{'fwdRebuildReadBwcBw'} && $perf_ref->{'fwdRebuildReadBwcBw'} ne ""){
    if (Xormon::isdigit($rebuild)){
      $rebuild += $perf_ref->{'fwdRebuildReadBwcBw'};
    }
    else{
      $rebuild = $perf_ref->{'fwdRebuildReadBwcBw'};
    }
  }
  if ( defined $perf_ref->{'fwdRebuildWriteBwcBw'} && $perf_ref->{'fwdRebuildWriteBwcBw'} ne ""){
    if (Xormon::isdigit($rebuild)){
      $rebuild += $perf_ref->{'fwdRebuildWriteBwcBw'};
    }
    else{
      $rebuild = $perf_ref->{'fwdRebuildWriteBwcBw'};
    }
  }
  if ( defined $perf_ref->{'bckRebuildReadBwcBw'} && $perf_ref->{'bckRebuildReadBwcBw'} ne ""){
    if (Xormon::isdigit($rebuild)){
      $rebuild += $perf_ref->{'bckRebuildReadBwcBw'};
    }
    else{
      $rebuild = $perf_ref->{'bckRebuildReadBwcBw'};
    }
  }
  if ( defined $perf_ref->{'bckRebuildWriteBwcBw'} && $perf_ref->{'bckRebuildWriteBwcBw'} ne ""){
    if (Xormon::isdigit($rebuild)){
      $rebuild += $perf_ref->{'bckRebuildWriteBwcBw'};
    }
    else{
      $rebuild = $perf_ref->{'bckRebuildWriteBwcBw'};
    }
  }
  #rbld_cap
  if ( defined $perf_ref->{'pendingNormRebuildCapacityInB'} && $perf_ref->{'pendingNormRebuildCapacityInB'} ne ""){
      $pend_rbld_cap = $perf_ref->{'pendingNormRebuildCapacityInB'};
  }
  if ( defined $perf_ref->{'pendingFwdRebuildCapacityInB'} && $perf_ref->{'pendingFwdRebuildCapacityInB'} ne ""){
    if (Xormon::isdigit($pend_rbld_cap)){
      $pend_rbld_cap += $perf_ref->{'pendingFwdRebuildCapacityInB'};
    }
    else{
      $pend_rbld_cap = $perf_ref->{'pendingFwdRebuildCapacityInB'};
    }
  }
  if ( defined $perf_ref->{'pendingBckRebuildCapacityInB'} && $perf_ref->{'pendingBckRebuildCapacityInB'} ne ""){
    if (Xormon::isdigit($pend_rbld_cap)){
      $pend_rbld_cap += $perf_ref->{'pendingBckRebuildCapacityInB'};
    }
    else{
      $pend_rbld_cap = $perf_ref->{'pendingBckRebuildCapacityInB'};
    }
  }
  #capacity
  if ( defined $perf_ref->{'netMaxUserDataCapacityInB'} && $perf_ref->{'netMaxUserDataCapacityInB'} ne ""){
    $total = $perf_ref->{'netMaxUserDataCapacityInB'};
  }
  if ( defined $perf_ref->{'netCapacityInUseInB'} && $perf_ref->{'netCapacityInUseInB'} ne ""){
    $used = $perf_ref->{'netCapacityInUseInB'};
  }
  if ( defined $perf_ref->{'netUnusedCapacityInB'} && $perf_ref->{'netUnusedCapacityInB'} ne ""){
    $free = $perf_ref->{'netUnusedCapacityInB'};
  }
  if ( defined $perf_ref->{'maxUserDataCapacityInB'} && $perf_ref->{'maxUserDataCapacityInB'} ne ""){
    $r_total = $perf_ref->{'maxUserDataCapacityInB'};
  }
  if ( defined $perf_ref->{'capacityInUseInB'} && $perf_ref->{'capacityInUseInB'} ne ""){
    $r_used = $perf_ref->{'capacityInUseInB'};
  }
  if ( defined $perf_ref->{'unusedCapacityInB'} && $perf_ref->{'unusedCapacityInB'} ne ""){
    $r_free = $perf_ref->{'unusedCapacityInB'};
  }
  if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
    $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
  }
  $data_out .= "$st_uuid;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$block_size;$block_size_r;$block_size_w;$resp_t;$resp_t_r;$resp_t_w;$cache_hit_r;$rebalance;$pend_reba_cap;$rebuild;$pend_rbld_cap;$total;$used;$free;$used_pct;$r_total;$r_used;$r_free;$rpl_rx;$rpl_tx;$rpl_ap_lat;$rpl_rx_lat;$rpl_tx_lat;$rpl_total;$rpl_used\n";
  #print "RESP $resp_t\n"; TODO undef sub

  #SAVE SYSTEM
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"powerflex","device");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
    $main_rc = $status;
  }
  $logger->info("SYSTEM status : $status");

  $data_out = "";

  ##########
  ## POOL ##
  ##########
  $htype = "StoragePool";
  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;block_size;block_size_r;block_size_w;resp_t;resp_t_r;resp_t_w;r_cache_hit;rebalance_rate;rebalance_pending;rebuild_rate;rebuild_pending;capacity_total;capacity_used;capacity_free;capacity_used_percent;raw_capacity_total;raw_capacity_used;raw_capacity_free\n";

  foreach my $p_key ( keys %{ $data{$htype} } ) {
    my $p_id             = $p_key;
    my $conf_ref         = $data{$htype}{$p_id}{'conf'};
    my $perf_ref         = $data{$htype}{$p_id}{'perf'};

    my $p_name = "";

    if ( defined $conf_ref->{'name'} && $conf_ref->{'name'} ne "" ) {
      $p_name = $conf_ref->{'name'};
    }

    my $hash_key = Xormon::create_hash_key($st_uuid."POOL".$p_name); $hash_key = lc($hash_key);
    #print "$hash_key\n";

    my %arch = (
        "item_id" => $hash_key,
        "label" => $p_name,
        "hw_type" => "powerflex",
        "subsystem" => "pool",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
    push(@architecture, \%arch);

    #conf
    if ( defined $conf_ref->{'protectionDomainId'} && $conf_ref->{'protectionDomainId'} ne "" ) {
      my $pd_id = $conf_ref->{'protectionDomainId'};
      if (defined $data{'ProtectionDomain'}{$pd_id}{'conf'}{'name'} && $data{'ProtectionDomain'}{$pd_id}{'conf'}{'name'} ne "") {
        my $pd_name = $data{'ProtectionDomain'}{$pd_id}{'conf'}{'name'};
        $conf{$hash_key}{'pd_name'} = $pd_name;
      }
    }
    if ( defined $conf_ref->{'name'} && $conf_ref->{'name'} ne "" ) {
      $conf{$hash_key}{'name'} = $conf_ref->{'name'};
    }
    if ( defined $conf_ref->{'mediaType'} && $conf_ref->{'mediaType'} ne "" ) {
      $conf{$hash_key}{'mediatype'} = $conf_ref->{'mediaType'};
    }

    # foreach my $property ( keys %{$conf_ref} ) {
    #   my $value = $conf_ref->{$property};
    #   $conf{$hash_key}{$property} = $value;
    # }

    #io,data,block_size,resp_t,cache_hit
    my $io_r          = "";
    my $io_w          = "";
    my $io_rate       = "";
    my $data_r        = "";
    my $data_w        = "";
    my $data_rate     = "";
    my $block_size    = "";
    my $block_size_r  = "";
    my $block_size_w  = "";
    my $resp_t        = "";
    my $resp_t_r      = "";
    my $resp_t_w      = "";
    my $cache_hit_r   = "";
    #cap
    my $total         = "";
    my $used          = "";
    my $free          = "";
    my $used_pct      = "";
    my $r_total       = "";
    my $r_used        = "";
    my $r_free        = "";
    #reba,rbld
    my $rebalance     = "";
    my $pend_reba_cap = "";
    my $rebuild       = "";
    my $pend_rbld_cap = "";

    #io
    if ( defined $perf_ref->{'primaryReadBwcIo'} && $perf_ref->{'primaryReadBwcIo'} ne ""){
      $io_r = $perf_ref->{'primaryReadBwcIo'};
      $io_rate = $io_r;
    }
    if ( defined $perf_ref->{'primaryWriteBwcIo'} && $perf_ref->{'primaryWriteBwcIo'} ne ""){
      $io_w = $perf_ref->{'primaryWriteBwcIo'};
      if (Xormon::isdigit($io_r)){
        $io_rate += $io_w;
      }
      else{
        $io_rate = $io_w;
      }
    }
    #data
    if ( defined $perf_ref->{'primaryReadBwcBw'} && $perf_ref->{'primaryReadBwcBw'} ne ""){
      $data_r = $perf_ref->{'primaryReadBwcBw'};
      $data_rate = $data_r;
    }
    if ( defined $perf_ref->{'primaryWriteBwcBw'} && $perf_ref->{'primaryWriteBwcBw'} ne ""){
      $data_w = $perf_ref->{'primaryWriteBwcBw'};
      if (Xormon::isdigit($data_r)){
        $data_rate += $data_w;
      }
      else{
        $data_rate = $data_w;
      }
    }
    # calculate block size
    if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
      if ( $io_rate == 0 ) {
        $block_size = 0;
      }
      else{
        $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
      }
    }
    if ( Xormon::isdigit($io_r) && Xormon::isdigit($data_r) ) {
      if ( $io_r == 0 ) {
        $block_size_r = 0;
      }
      else{
        $block_size_r = Xormon::formatNumberByType("float4", ($data_r / $io_r));
      }
    }
    if ( Xormon::isdigit($io_w) && Xormon::isdigit($data_w) ) {
      if ( $io_w == 0 ) {
        $block_size_w = 0;
      }
      else{
        $block_size_w = Xormon::formatNumberByType("float4", ($data_w / $io_w));
      }
    }
    #resp_t
    if ( defined $perf_ref->{'userDataSdcReadLatency'} && $perf_ref->{'userDataSdcReadLatency'} ne ""){
      $resp_t_r = $perf_ref->{'userDataSdcReadLatency'};
    }
    if ( defined $perf_ref->{'userDataSdcWriteLatency'} && $perf_ref->{'userDataSdcWriteLatency'} ne ""){
      $resp_t_w = $perf_ref->{'userDataSdcWriteLatency'};
    }
    # calculate resp_t
    if ( Xormon::isdigit( $resp_t_r ) && Xormon::isdigit( $io_r ) && Xormon::isdigit( $resp_t_w ) && Xormon::isdigit( $io_w ) ) {
      my $resp_t_help = Xormon::calculate_weighted_avg( [ { count => $io_r, value => $resp_t_r }, { count => $io_w, value => $resp_t_w } ] );
      if (defined $resp_t_help) { $resp_t = Xormon::formatNumberByType("float4",$resp_t_help); }
    }
    #cache_hit
    if ( Xormon::isdigit($io_r) && defined $perf_ref->{'primaryReadFromRmcacheBwcIo'} && $perf_ref->{'primaryReadFromRmcacheBwcIo'} ne ""){
      if ( $io_r == 0 ) {
        $cache_hit_r = 0;
      }
      else{
        $cache_hit_r = Xormon::formatNumberByType("float4", ($perf_ref->{'primaryReadFromRmcacheBwcIo'} / ( $io_r / 100 ) ) );
      }
    }
    #rebalance
    if ( defined $perf_ref->{'rebalanceReadBwcBw'} && $perf_ref->{'rebalanceReadBwcBw'} ne ""){
      $rebalance = $perf_ref->{'rebalanceReadBwcBw'};
    }
    if ( defined $perf_ref->{'rebalanceWriteBwcBw'} && $perf_ref->{'rebalanceWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebalance)){
        $rebalance += $perf_ref->{'rebalanceWriteBwcBw'};
      }
      else{
        $rebalance = $perf_ref->{'rebalanceWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'pendingRebalanceCapacityInB'} && $perf_ref->{'pendingRebalanceCapacityInB'} ne ""){
      $pend_reba_cap = $perf_ref->{'pendingRebalanceCapacityInB'};
    }
    #rbld
    if ( defined $perf_ref->{'normRebuildReadBwcBw'} && $perf_ref->{'normRebuildReadBwcBw'} ne ""){
        $rebuild = $perf_ref->{'normRebuildReadBwcBw'};
    }
    if ( defined $perf_ref->{'normRebuildWriteBwcBw'} && $perf_ref->{'normRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'normRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'normRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildReadBwcBw'} && $perf_ref->{'fwdRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildWriteBwcBw'} && $perf_ref->{'fwdRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildReadBwcBw'} && $perf_ref->{'bckRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildWriteBwcBw'} && $perf_ref->{'bckRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildWriteBwcBw'};
      }
    }
    #rbld_cap
    if ( defined $perf_ref->{'pendingNormRebuildCapacityInB'} && $perf_ref->{'pendingNormRebuildCapacityInB'} ne ""){
        $pend_rbld_cap = $perf_ref->{'pendingNormRebuildCapacityInB'};
    }
    if ( defined $perf_ref->{'pendingFwdRebuildCapacityInB'} && $perf_ref->{'pendingFwdRebuildCapacityInB'} ne ""){
      if (Xormon::isdigit($pend_rbld_cap)){
        $pend_rbld_cap += $perf_ref->{'pendingFwdRebuildCapacityInB'};
      }
      else{
        $pend_rbld_cap = $perf_ref->{'pendingFwdRebuildCapacityInB'};
      }
    }
    if ( defined $perf_ref->{'pendingBckRebuildCapacityInB'} && $perf_ref->{'pendingBckRebuildCapacityInB'} ne ""){
      if (Xormon::isdigit($pend_rbld_cap)){
        $pend_rbld_cap += $perf_ref->{'pendingBckRebuildCapacityInB'};
      }
      else{
        $pend_rbld_cap = $perf_ref->{'pendingBckRebuildCapacityInB'};
      }
    }
    #capacity
    if ( defined $perf_ref->{'netMaxUserDataCapacityInB'} && $perf_ref->{'netMaxUserDataCapacityInB'} ne ""){
      $total = $perf_ref->{'netMaxUserDataCapacityInB'};
      $conf{$hash_key}{'capacity_total'} = $total;
    }
    if ( defined $perf_ref->{'netCapacityInUseInB'} && $perf_ref->{'netCapacityInUseInB'} ne ""){
      $used = $perf_ref->{'netCapacityInUseInB'};
      $conf{$hash_key}{'capacity_used'} = $used;
    }
    if ( defined $perf_ref->{'netUnusedCapacityInB'} && $perf_ref->{'netUnusedCapacityInB'} ne ""){
      $free = $perf_ref->{'netUnusedCapacityInB'};
      $conf{$hash_key}{'capacity_free'} = $free;
    }
    if ( defined $perf_ref->{'maxUserDataCapacityInB'} && $perf_ref->{'maxUserDataCapacityInB'} ne ""){
      $r_total = $perf_ref->{'maxUserDataCapacityInB'};
    }
    if ( defined $perf_ref->{'capacityInUseInB'} && $perf_ref->{'capacityInUseInB'} ne ""){
      $r_used = $perf_ref->{'capacityInUseInB'};
    }
    if ( defined $perf_ref->{'unusedCapacityInB'} && $perf_ref->{'unusedCapacityInB'} ne ""){
      $r_free = $perf_ref->{'unusedCapacityInB'};
    }

    if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
      $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
    }
    $data_out .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$block_size;$block_size_r;$block_size_w;$resp_t;$resp_t_r;$resp_t_w;$cache_hit_r;$rebalance;$pend_reba_cap;$rebuild;$pend_rbld_cap;$total;$used;$free;$used_pct;$r_total;$r_used;$r_free\n";
  }

  #SAVE POOL
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"powerflex","pool");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
    $main_rc = $status;
  }
  $logger->info("POOL status : $status");
  $data_out = "";

  ############
  ## VOLUME ##
  ############
  $htype = "Volume";
  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;block_size;block_size_r;block_size_w;resp_t;resp_t_r;resp_t_w;capacity_total\n";

  foreach my $vol_key ( keys %{ $data{$htype} } ) {
    my $vol_id           = $vol_key;
    my $conf_ref         = $data{$htype}{$vol_id}{'conf'};
    my $perf_ref         = $data{$htype}{$vol_id}{'perf'};

    my $vol_name = "";

    if ( defined $conf_ref->{'name'} && $conf_ref->{'name'} ne "" ) {
      $vol_name = $conf_ref->{'name'};
    }
    else {
      $vol_name = "vol:$vol_id";    ## if volume name not defined
    }

    my $hash_key = Xormon::create_hash_key($st_uuid."VOLUME".$vol_name); $hash_key = lc($hash_key);
    my $pool_key = "";

    #conf
    if ( defined $conf_ref->{'storagePoolId'} && $conf_ref->{'storagePoolId'} ne "" ) {
      my $pool_id = $conf_ref->{'storagePoolId'};
      if (defined $data{'StoragePool'}{$pool_id}{'conf'}{'name'} && $data{'StoragePool'}{$pool_id}{'conf'}{'name'} ne "") {
        my $pool_name = $data{'StoragePool'}{$pool_id}{'conf'}{'name'};
        $conf{$hash_key}{'pool_name'} = $pool_name;
        $pool_key = Xormon::create_hash_key($st_uuid."POOL".$pool_name); $hash_key = lc($hash_key);
      }
    }
    # if ( defined $conf_ref->{'volumeType'} && $conf_ref->{'volumeType'} ne "" ) {
    #   $conf{$hash_key}{'volume_type'} = $conf_ref->{'volumeType'};
    # }
    ## add lun_id for volume
    if ( defined $conf_ref->{'id'} && $conf_ref->{'id'} ne "" ) {
      $conf{$hash_key}{'lun_id'} = $conf_ref->{'id'};
    }
    # if ( defined $conf_ref->{'capacity_total'} && $conf_ref->{'capacity_total'} ne "" ) {
    #   $conf{$hash_key}{'capacity_total'} = $conf_ref->{'capacity_total'};
    # }


    # foreach my $property ( keys %{$conf_ref} ) {
    #   my $value = $conf_ref->{$property};
    #   $conf{$hash_key}{$property} = $value;
    # }

    my %arch = (
        "item_id" => $hash_key,
        "label" => $vol_name,
        "hw_type" => "powerflex",
        "subsystem" => "volume",
        "parents" => [$cfg->{'hostcfg_id'}, $pool_key],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
    push(@architecture, \%arch);

    #io,data,block_size,resp_t
    my $io_r          = "";
    my $io_w          = "";
    my $io_rate       = "";
    my $data_r        = "";
    my $data_w        = "";
    my $data_rate     = "";
    my $block_size    = "";
    my $block_size_r  = "";
    my $block_size_w  = "";
    my $resp_t        = "";
    my $resp_t_r      = "";
    my $resp_t_w      = "";
    my $capacity      = "";

    #io
    if ( defined $perf_ref->{'userDataReadBwcIo'} && $perf_ref->{'userDataReadBwcIo'} ne ""){
      $io_r = $perf_ref->{'userDataReadBwcIo'};
      $io_rate = $io_r;
    }
    if ( defined $perf_ref->{'userDataWriteBwcIo'} && $perf_ref->{'userDataWriteBwcIo'} ne ""){
      $io_w = $perf_ref->{'userDataWriteBwcIo'};
      if (Xormon::isdigit($io_r)){
        $io_rate += $io_w;
      }
      else{
        $io_rate = $io_w;
      }
    }
    #data
    if ( defined $perf_ref->{'userDataReadBwcBw'} && $perf_ref->{'userDataReadBwcBw'} ne ""){
      $data_r = $perf_ref->{'userDataReadBwcBw'};
      $data_rate = $data_r;
    }
    if ( defined $perf_ref->{'userDataWriteBwcBw'} && $perf_ref->{'userDataWriteBwcBw'} ne ""){
      $data_w = $perf_ref->{'userDataWriteBwcBw'};
      if (Xormon::isdigit($data_r)){
        $data_rate += $data_w;
      }
      else{
        $data_rate = $data_w;
      }
    }
    # calculate block size
    if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
      if ( $io_rate == 0 ) {
        $block_size = 0;
      }
      else{
        $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
      }
    }
    if ( Xormon::isdigit($io_r) && Xormon::isdigit($data_r) ) {
      if ( $io_r == 0 ) {
        $block_size_r = 0;
      }
      else{
        $block_size_r = Xormon::formatNumberByType("float4", ($data_r / $io_r));
      }
    }
    if ( Xormon::isdigit($io_w) && Xormon::isdigit($data_w) ) {
      if ( $io_w == 0 ) {
        $block_size_w = 0;
      }
      else{
        $block_size_w = Xormon::formatNumberByType("float4", ($data_w / $io_w));
      }
    }
    #resp_t
    if ( defined $perf_ref->{'userDataSdcReadLatency'} && $perf_ref->{'userDataSdcReadLatency'} ne ""){
      $resp_t_r = $perf_ref->{'userDataSdcReadLatency'};
    }
    if ( defined $perf_ref->{'userDataSdcWriteLatency'} && $perf_ref->{'userDataSdcWriteLatency'} ne ""){
      $resp_t_w = $perf_ref->{'userDataSdcWriteLatency'};
    }
    # calculate resp_t
    if ( Xormon::isdigit( $resp_t_r ) && Xormon::isdigit( $io_r ) && Xormon::isdigit( $resp_t_w ) && Xormon::isdigit( $io_w ) ) {
      my $resp_t_help = Xormon::calculate_weighted_avg( [ { count => $io_r, value => $resp_t_r }, { count => $io_w, value => $resp_t_w } ] );
      if (defined $resp_t_help) { $resp_t = Xormon::formatNumberByType("float4",$resp_t_help); }
    }
    if ( defined $perf_ref->{'sizeInB'} && $perf_ref->{'sizeInB'} ne ""){
      $capacity = $perf_ref->{'sizeInB'};
    }
    $data_out .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$block_size;$block_size_r;$block_size_w;$resp_t;$resp_t_r;$resp_t_w;$capacity\n";
  }

  #SAVE VOLUME
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"powerflex","volume");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
    $main_rc = $status;
  }
  $logger->info("VOLUME status : $status");
  $data_out = "";



  #########
  ## SDS ##
  #########
  $htype = "Sds";
  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;r_cache_hit;rebalance_rate;rebuild_rate\n";

  foreach my $sds_key ( keys %{ $data{$htype} } ) {
    my $sds_id           = $sds_key;
    my $conf_ref         = $data{$htype}{$sds_id}{'conf'};
    my $perf_ref         = $data{$htype}{$sds_id}{'perf'};

    my $sds_name = "";

    if ( defined $conf_ref->{'name'} && $conf_ref->{'name'} ne "" ) {
      $sds_name = $conf_ref->{'name'};
    }
    else {
      $sds_name = "sds:$sds_id";    ## if sds name not defined
    }

    my $hash_key = Xormon::create_hash_key($st_uuid."SDS".$sds_name); $hash_key = lc($hash_key);
    #print "$hash_key\n";

    my %arch = (
        "item_id" => $hash_key,
        "label" => $sds_name,
        "hw_type" => "powerflex",
        "subsystem" => "sds",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
    push(@architecture, \%arch);

    #conf
    if ( defined $conf_ref->{'protectionDomainId'} && $conf_ref->{'protectionDomainId'} ne "" ) {
      my $pd_id = $conf_ref->{'protectionDomainId'};
      if (defined $data{'ProtectionDomain'}{$pd_id}{'conf'}{'name'} && $data{'ProtectionDomain'}{$pd_id}{'conf'}{'name'} ne "") {
        my $pd_name = $data{'ProtectionDomain'}{$pd_id}{'conf'}{'name'};
        $conf{$hash_key}{'pd_name'} = $pd_name;
      }
    }
    # foreach my $property ( keys %{$conf_ref} ) {
    #   my $value = $conf_ref->{$property};
    #   $conf{$hash_key}{$property} = $value;
    # }


    my $io_r          = "";
    my $io_w          = "";
    my $io_rate       = "";
    my $data_r        = "";
    my $data_w        = "";
    my $data_rate     = "";
    my $r_cache_hit   = "";
    my $rebalance     = "";
    my $rebuild       = "";

    #io
    if ( defined $perf_ref->{'totalReadBwcIo'} && $perf_ref->{'totalReadBwcIo'} ne ""){
      $io_r = $perf_ref->{'totalReadBwcIo'};
      $io_rate = $io_r;
    }
    if ( defined $perf_ref->{'totalWriteBwcIo'} && $perf_ref->{'totalWriteBwcIo'} ne ""){
      $io_w = $perf_ref->{'totalWriteBwcIo'};
      if (Xormon::isdigit($io_r)){
        $io_rate += $io_w;
      }
      else{
        $io_rate = $io_w;
      }
    }
    #data
    if ( defined $perf_ref->{'totalReadBwcBw'} && $perf_ref->{'totalReadBwcBw'} ne ""){
      $data_r = $perf_ref->{'totalReadBwcBw'};
      $data_rate = $data_r;
    }
    if ( defined $perf_ref->{'totalWriteBwcBw'} && $perf_ref->{'totalWriteBwcBw'} ne ""){
      $data_w = $perf_ref->{'totalWriteBwcBw'};
      if (Xormon::isdigit($data_r)){
        $data_rate += $data_w;
      }
      else{
        $data_rate = $data_w;
      }
    }
    #cache_hit
    if ( defined $perf_ref->{'primaryReadFromRmcacheBwcIo'} && $perf_ref->{'primaryReadFromRmcacheBwcIo'} ne ""){
      my $cache_io = $perf_ref->{'primaryReadFromRmcacheBwcIo'};
      if ( defined $perf_ref->{'primaryReadBwcIo'} && Xormon::isdigit( $perf_ref->{'primaryReadBwcIo'} ) ) {
        my $io_r_primary = $perf_ref->{'primaryReadBwcIo'};
        if ( $io_r_primary == 0 ) {
          $r_cache_hit = 0;
        }
        else {
          $r_cache_hit = Xormon::formatNumberByType("float4", ($cache_io / ( $io_r_primary / 100 ) ) );
        }
      }
    }
    #rebalance
    if ( defined $perf_ref->{'rebalanceReadBwcBw'} && $perf_ref->{'rebalanceReadBwcBw'} ne ""){
      $rebalance = $perf_ref->{'rebalanceReadBwcBw'};
    }
    if ( defined $perf_ref->{'rebalanceWriteBwcBw'} && $perf_ref->{'rebalanceWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebalance)){
        $rebalance += $perf_ref->{'rebalanceWriteBwcBw'};
      }
      else{
        $rebalance = $perf_ref->{'rebalanceWriteBwcBw'};
      }
    }
    #rbld
    if ( defined $perf_ref->{'normRebuildReadBwcBw'} && $perf_ref->{'normRebuildReadBwcBw'} ne ""){
        $rebuild = $perf_ref->{'normRebuildReadBwcBw'};
    }
    if ( defined $perf_ref->{'normRebuildWriteBwcBw'} && $perf_ref->{'normRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'normRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'normRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildReadBwcBw'} && $perf_ref->{'fwdRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildWriteBwcBw'} && $perf_ref->{'fwdRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildReadBwcBw'} && $perf_ref->{'bckRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildWriteBwcBw'} && $perf_ref->{'bckRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildWriteBwcBw'};
      }
    }
    $data_out .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$r_cache_hit;$rebalance;$rebuild\n";
  }

  # #SAVE SDS
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"powerflex","sds");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
    $main_rc = $status;
  }
  $logger->info("SDS status : $status");
  $data_out = "";


  #########
  ## SDC ##
  #########
  $htype = "Sdc";
  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t;resp_t_r;resp_t_w\n";

  foreach my $sdc_key ( keys %{ $data{$htype} } ) {
    my $sdc_id           = $sdc_key;
    my $conf_ref         = $data{$htype}{$sdc_id}{'conf'};
    my $perf_ref         = $data{$htype}{$sdc_id}{'perf'};

    my $sdc_name = "";

    if ( defined $conf_ref->{'name'} && $conf_ref->{'name'} ne "" ) {
      $sdc_name = $conf_ref->{'name'};
    }
    else {
      $sdc_name = "sds:$sdc_id";    ## if sds name not defined
    }

    my $hash_key = Xormon::create_hash_key($st_uuid."SDC".$sdc_name); $hash_key = lc($hash_key);
    #print "$hash_key\n";

    my %arch = (
        "item_id" => $hash_key,
        "label" => $sdc_name,
        "hw_type" => "powerflex",
        "subsystem" => "sdc",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
    push(@architecture, \%arch);

    #conf
    # foreach my $property ( keys %{$conf_ref} ) {
    #   if ($property eq "volumeIds" && ref($conf_ref->{$property}) eq "ARRAY" && scalar( @{ $conf_ref->{$property} } )) {
    #     my $vol_string = join(',', @{ $conf_ref->{$property} });         #string from array
    #     $conf{$hash_key}{'volume_ids'} = $vol_string;
    #     next;
    #   }
    #   my $value = $conf_ref->{$property};
    #   $conf{$hash_key}{$property} = $value;
    # }


    if (defined $conf_ref->{'volumeIds'} && ref($conf_ref->{'volumeIds'}) eq "ARRAY" && scalar( @{ $conf_ref->{'volumeIds'} } )) {
      my $vol_string = join(',', @{ $conf_ref->{'volumeIds'} });         #string from array
      $conf{$hash_key}{'volume_ids'} = $vol_string;
    }

    my $io_r          = "";
    my $io_w          = "";
    my $io_rate       = "";
    my $data_r        = "";
    my $data_w        = "";
    my $data_rate     = "";
    my $resp_t        = "";
    my $resp_t_r      = "";
    my $resp_t_w      = "";

    #io
    if ( defined $perf_ref->{'userDataReadBwcIo'} && $perf_ref->{'userDataReadBwcIo'} ne ""){
      $io_r = $perf_ref->{'userDataReadBwcIo'};
      $io_rate = $io_r;
    }
    if ( defined $perf_ref->{'userDataWriteBwcIo'} && $perf_ref->{'userDataWriteBwcIo'} ne ""){
      $io_w = $perf_ref->{'userDataWriteBwcIo'};
      if (Xormon::isdigit($io_r)){
        $io_rate += $io_w;
      }
      else{
        $io_rate = $io_w;
      }
    }
    #data
    if ( defined $perf_ref->{'userDataReadBwcBw'} && $perf_ref->{'userDataReadBwcBw'} ne ""){
      $data_r = $perf_ref->{'userDataReadBwcBw'};
      $data_rate = $data_r;
    }
    if ( defined $perf_ref->{'userDataWriteBwcBw'} && $perf_ref->{'userDataWriteBwcBw'} ne ""){
      $data_w = $perf_ref->{'userDataWriteBwcBw'};
      if (Xormon::isdigit($data_r)){
        $data_rate += $data_w;
      }
      else{
        $data_rate = $data_w;
      }
    }
    #resp_t
    if ( defined $perf_ref->{'userDataSdcReadLatency'} && $perf_ref->{'userDataSdcReadLatency'} ne ""){
      $resp_t_r = $perf_ref->{'userDataSdcReadLatency'};
    }
    if ( defined $perf_ref->{'userDataSdcWriteLatency'} && $perf_ref->{'userDataSdcWriteLatency'} ne ""){
      $resp_t_w = $perf_ref->{'userDataSdcWriteLatency'};
    }
    # calculate resp_t
    if ( Xormon::isdigit( $resp_t_r ) && Xormon::isdigit( $io_r ) && Xormon::isdigit( $resp_t_w ) && Xormon::isdigit( $io_w ) ) {
      my $resp_t_help = Xormon::calculate_weighted_avg( [ { count => $io_r, value => $resp_t_r }, { count => $io_w, value => $resp_t_w } ] );
      if (defined $resp_t_help) { $resp_t = Xormon::formatNumberByType("float4",$resp_t_help); }
    }
    $data_out .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t;$resp_t_r;$resp_t_w\n";
  }

  #SAVE SDC
  #print $data_out;

  $rc = $xormon->saveCSV($data_out,"powerflex","sdc");
  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
    $main_rc = $status;
  }
  $logger->info("SDC status : $status");
  $data_out = "";


  #######################
  ## PROTECTION DOMAIN ##
  #######################
  $htype = "ProtectionDomain";
  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t;resp_t_r;resp_t_w;r_cache_hit;rebalance_rate;rebalance_pending;rebuild_rate;rebuild_pending\n";

  foreach my $pd_key ( keys %{ $data{$htype} } ) {
    my $pd_id            = $pd_key;
    my $conf_ref         = $data{$htype}{$pd_id}{'conf'};
    my $perf_ref         = $data{$htype}{$pd_id}{'perf'};

    my $pd_name = "";

    if ( defined $conf_ref->{'name'} && $conf_ref->{'name'} ne "" ) {
      $pd_name = $conf_ref->{'name'};
    }
    else {
      $pd_name = "pd:$pd_id";    ## if name not defined
    }

    my $hash_key = Xormon::create_hash_key($st_uuid."PD".$pd_name); $hash_key = lc($hash_key);
    #print "$hash_key\n";

    my %arch = (
        "item_id" => $hash_key,
        "label" => $pd_name,
        "hw_type" => "powerflex",
        "subsystem" => "protection_domain",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
    push(@architecture, \%arch);

    #conf
    # foreach my $property ( keys %{$conf_ref} ) {
    #   my $value = $conf_ref->{$property};
    #   $conf{$hash_key}{$property} = $value;
    # }

    my $io_r          = "";
    my $io_w          = "";
    my $io_rate       = "";
    my $data_r        = "";
    my $data_w        = "";
    my $data_rate     = "";
    my $resp_t        = "";
    my $resp_t_r      = "";
    my $resp_t_w      = "";
    my $cache_hit_r   = "";
    #reba,rbld
    my $rebalance     = "";
    my $pend_reba_cap = "";
    my $rebuild       = "";
    my $pend_rbld_cap = "";

    #io
    if ( defined $perf_ref->{'primaryReadBwcIo'} && $perf_ref->{'primaryReadBwcIo'} ne ""){
      $io_r = $perf_ref->{'primaryReadBwcIo'};
      $io_rate = $io_r;
    }
    if ( defined $perf_ref->{'primaryWriteBwcIo'} && $perf_ref->{'primaryWriteBwcIo'} ne ""){
      $io_w = $perf_ref->{'primaryWriteBwcIo'};
      if (Xormon::isdigit($io_r)){
        $io_rate += $io_w;
      }
      else{
        $io_rate = $io_w;
      }
    }
    #data
    if ( defined $perf_ref->{'primaryReadBwcBw'} && $perf_ref->{'primaryReadBwcBw'} ne ""){
      $data_r = $perf_ref->{'primaryReadBwcBw'};
      $data_rate = $data_r;
    }
    if ( defined $perf_ref->{'primaryWriteBwcBw'} && $perf_ref->{'primaryWriteBwcBw'} ne ""){
      $data_w = $perf_ref->{'primaryWriteBwcBw'};
      if (Xormon::isdigit($data_r)){
        $data_rate += $data_w;
      }
      else{
        $data_rate = $data_w;
      }
    }
    #resp_t
    if ( defined $perf_ref->{'userDataSdcReadLatency'} && $perf_ref->{'userDataSdcReadLatency'} ne ""){
      $resp_t_r = $perf_ref->{'userDataSdcReadLatency'};
    }
    if ( defined $perf_ref->{'userDataSdcWriteLatency'} && $perf_ref->{'userDataSdcWriteLatency'} ne ""){
      $resp_t_w = $perf_ref->{'userDataSdcWriteLatency'};
    }
    # calculate resp_t
    if ( Xormon::isdigit( $resp_t_r ) && Xormon::isdigit( $io_r ) && Xormon::isdigit( $resp_t_w ) && Xormon::isdigit( $io_w ) ) {
      my $resp_t_help = Xormon::calculate_weighted_avg( [ { count => $io_r, value => $resp_t_r }, { count => $io_w, value => $resp_t_w } ] );
      if (defined $resp_t_help) { $resp_t = Xormon::formatNumberByType("float4",$resp_t_help); }
    }
    #cache_hit
    if ( Xormon::isdigit($io_r) && defined $perf_ref->{'primaryReadFromRmcacheBwcIo'} && $perf_ref->{'primaryReadFromRmcacheBwcIo'} ne ""){
      if ( $io_r == 0 ) {
        $cache_hit_r = 0;
      }
      else{
        $cache_hit_r = Xormon::formatNumberByType("float4", ($perf_ref->{'primaryReadFromRmcacheBwcIo'} / ( $io_r / 100 ) ) );
      }
    }
    #rebalance
    if ( defined $perf_ref->{'rebalanceReadBwcBw'} && $perf_ref->{'rebalanceReadBwcBw'} ne ""){
      $rebalance = $perf_ref->{'rebalanceReadBwcBw'};
    }
    if ( defined $perf_ref->{'rebalanceWriteBwcBw'} && $perf_ref->{'rebalanceWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebalance)){
        $rebalance += $perf_ref->{'rebalanceWriteBwcBw'};
      }
      else{
        $rebalance = $perf_ref->{'rebalanceWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'pendingRebalanceCapacityInB'} && $perf_ref->{'pendingRebalanceCapacityInB'} ne ""){
      $pend_reba_cap = $perf_ref->{'pendingRebalanceCapacityInB'};
    }
    #rbld
    if ( defined $perf_ref->{'normRebuildReadBwcBw'} && $perf_ref->{'normRebuildReadBwcBw'} ne ""){
        $rebuild = $perf_ref->{'normRebuildReadBwcBw'};
    }
    if ( defined $perf_ref->{'normRebuildWriteBwcBw'} && $perf_ref->{'normRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'normRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'normRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildReadBwcBw'} && $perf_ref->{'fwdRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildWriteBwcBw'} && $perf_ref->{'fwdRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildReadBwcBw'} && $perf_ref->{'bckRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildWriteBwcBw'} && $perf_ref->{'bckRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildWriteBwcBw'};
      }
    }
    #rbld_cap
    if ( defined $perf_ref->{'pendingNormRebuildCapacityInB'} && $perf_ref->{'pendingNormRebuildCapacityInB'} ne ""){
        $pend_rbld_cap = $perf_ref->{'pendingNormRebuildCapacityInB'};
    }
    if ( defined $perf_ref->{'pendingFwdRebuildCapacityInB'} && $perf_ref->{'pendingFwdRebuildCapacityInB'} ne ""){
      if (Xormon::isdigit($pend_rbld_cap)){
        $pend_rbld_cap += $perf_ref->{'pendingFwdRebuildCapacityInB'};
      }
      else{
        $pend_rbld_cap = $perf_ref->{'pendingFwdRebuildCapacityInB'};
      }
    }
    if ( defined $perf_ref->{'pendingBckRebuildCapacityInB'} && $perf_ref->{'pendingBckRebuildCapacityInB'} ne ""){
      if (Xormon::isdigit($pend_rbld_cap)){
        $pend_rbld_cap += $perf_ref->{'pendingBckRebuildCapacityInB'};
      }
      else{
        $pend_rbld_cap = $perf_ref->{'pendingBckRebuildCapacityInB'};
      }
    }
    $data_out .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t;$resp_t_r;$resp_t_w;$cache_hit_r;$rebalance;$pend_reba_cap;$rebuild;$pend_rbld_cap\n";
  }

  #SAVE PROTECTION DOMAIN
  #print $data_out;

  $rc = $xormon->saveCSV($data_out,"powerflex","protection_domain");
  #status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
    $main_rc = $status;
  }
  $logger->info("PROTECTION_DOMAIN status : $status");
  $data_out = "";


  ###########
  ## DRIVE ##
  ###########
  $htype = "Device";
  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;resp_t;resp_t_r;resp_t_w;r_cache_hit;rebalance_rate;rebuild_rate\n";

  foreach my $dr_key ( keys %{ $data{$htype} } ) {
    my $dr_id            = $dr_key;
    my $conf_ref         = $data{$htype}{$dr_id}{'conf'};
    my $perf_ref         = $data{$htype}{$dr_id}{'perf'};

    my $dr_name  = "";
    my $r_name   = "";
    my $sds_name = "";

    if ( defined $conf_ref->{'name'} && $conf_ref->{'name'} ne "" ) {
      $dr_name = $conf_ref->{'name'};
    }
    else {
      $dr_name = $dr_id;    ## if name not defined
    }

    if ( defined $conf_ref->{'sdsId'} && $conf_ref->{'sdsId'} ne "" ) {
      my $sds_id = $conf_ref->{'sdsId'};
      if ( defined $data{'Sds'}{$sds_id}{'conf'}{'name'} && $data{'Sds'}{$sds_id}{'conf'}{'name'} ne "" ) {
        $sds_name = $data{'Sds'}{$sds_id}{'conf'}{'name'};
      }
      else {
        $sds_name = $sds_id;
      }
      $r_name   = "$sds_name" . "_" . "$dr_name";
    }
    else {
      $r_name = "$dr_id" . "_" . "$dr_name";
    }

    my $hash_key = Xormon::create_hash_key($st_uuid."DRIVE".$r_name); $hash_key = lc($hash_key);
    #print "$hash_key\n";

    my %arch = (
        "item_id" => $hash_key,
        "label" => $r_name,
        "hw_type" => "powerflex",
        "subsystem" => "drive",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      );
    push(@architecture, \%arch);

    #conf
    if ( defined $conf_ref->{'sdsId'} && $conf_ref->{'sdsId'} ne "" ) {
        $conf{$hash_key}{'sds_name'} = $sds_name;
    }
    if ( defined $conf_ref->{'storagePoolId'} && $conf_ref->{'storagePoolId'} ne "" ) {
      my $pool_id = $conf_ref->{'storagePoolId'};
      if (defined $data{'StoragePool'}{$pool_id}{'conf'}{'name'} && $data{'StoragePool'}{$pool_id}{'conf'}{'name'} ne "") {
        my $pool_name = $data{'StoragePool'}{$pool_id}{'conf'}{'name'};
        $conf{$hash_key}{'pool_name'} = $pool_name;
      }
    }
    # if ( $r_name ne "" ) {
    #   $conf{$hash_key}{'gen_name'} = $r_name;
    # }

    # foreach my $property ( keys %{$conf_ref} ) {
    #   my $value = $conf_ref->{$property};
    #   $conf{$hash_key}{$property} = $value;
    # }

    # aggregatedState
    # maxCapacityInKb
    # gen_name
    # pool_name
    # mediaType
    # errorState
    # deviceCurrentPathName
    # storagePoolId
    # deviceOriginalPathName
    # deviceState
    # sds_name
    # sdsId
    # name
    # deviceType


    my $io_r          = "";
    my $io_w          = "";
    my $io_rate       = "";
    my $data_r        = "";
    my $data_w        = "";
    my $data_rate     = "";
    my $resp_t        = "";
    my $resp_t_r      = "";
    my $resp_t_w      = "";
    my $cache_hit_r   = "";
    #reba,rbld
    my $rebalance     = "";
    my $rebuild       = "";

    #io
    if ( defined $perf_ref->{'totalReadBwcIo'} && $perf_ref->{'totalReadBwcIo'} ne ""){
      $io_r = $perf_ref->{'totalReadBwcIo'};
      $io_rate = $io_r;
    }
    if ( defined $perf_ref->{'totalWriteBwcIo'} && $perf_ref->{'totalWriteBwcIo'} ne ""){
      $io_w = $perf_ref->{'totalWriteBwcIo'};
      if (Xormon::isdigit($io_r)){
        $io_rate += $io_w;
      }
      else{
        $io_rate = $io_w;
      }
    }
    #data
    if ( defined $perf_ref->{'totalReadBwcBw'} && $perf_ref->{'totalReadBwcBw'} ne ""){
      $data_r = $perf_ref->{'totalReadBwcBw'};
      $data_rate = $data_r;
    }
    if ( defined $perf_ref->{'totalWriteBwcBw'} && $perf_ref->{'totalWriteBwcBw'} ne ""){
      $data_w = $perf_ref->{'totalWriteBwcBw'};
      if (Xormon::isdigit($data_r)){
        $data_rate += $data_w;
      }
      else{
        $data_rate = $data_w;
      }
    }
    #resp_t
    if ( defined $perf_ref->{'avgReadLatencyInMillisec'} && $perf_ref->{'avgReadLatencyInMillisec'} ne ""){
      $resp_t_r = $perf_ref->{'avgReadLatencyInMillisec'};
    }
    if ( defined $perf_ref->{'avgWriteLatencyInMillisec'} && $perf_ref->{'avgWriteLatencyInMillisec'} ne ""){
      $resp_t_w = $perf_ref->{'avgWriteLatencyInMillisec'};
    }
    # calculate resp_t
    if ( Xormon::isdigit( $resp_t_r ) && Xormon::isdigit( $io_r ) && Xormon::isdigit( $resp_t_w ) && Xormon::isdigit( $io_w ) ) {
      my $resp_t_help = Xormon::calculate_weighted_avg( [ { count => $io_r, value => $resp_t_r }, { count => $io_w, value => $resp_t_w } ] );
      if (defined $resp_t_help) { $resp_t = Xormon::formatNumberByType("float4",$resp_t_help); }
    }
    #cache_hit
    if ( defined $perf_ref->{'primaryReadFromRmcacheBwcIo'} && $perf_ref->{'primaryReadFromRmcacheBwcIo'} ne ""){
      my $cache_io = $perf_ref->{'primaryReadFromRmcacheBwcIo'};
      if ( defined $perf_ref->{'primaryReadBwcIo'} && Xormon::isdigit( $perf_ref->{'primaryReadBwcIo'} ) ) {
        my $io_r_primary = $perf_ref->{'primaryReadBwcIo'};
        if ( $io_r_primary == 0 ) {
          $cache_hit_r = 0;
        }
        else {
          $cache_hit_r = Xormon::formatNumberByType("float4", ($cache_io / ( $io_r_primary / 100 ) ) );
        }
      }
    }
    #rebalance
    if ( defined $perf_ref->{'rebalanceReadBwcBw'} && $perf_ref->{'rebalanceReadBwcBw'} ne ""){
      $rebalance = $perf_ref->{'rebalanceReadBwcBw'};
    }
    if ( defined $perf_ref->{'rebalanceWriteBwcBw'} && $perf_ref->{'rebalanceWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebalance)){
        $rebalance += $perf_ref->{'rebalanceWriteBwcBw'};
      }
      else{
        $rebalance = $perf_ref->{'rebalanceWriteBwcBw'};
      }
    }
    #rbld
    if ( defined $perf_ref->{'normRebuildReadBwcBw'} && $perf_ref->{'normRebuildReadBwcBw'} ne ""){
        $rebuild = $perf_ref->{'normRebuildReadBwcBw'};
    }
    if ( defined $perf_ref->{'normRebuildWriteBwcBw'} && $perf_ref->{'normRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'normRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'normRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildReadBwcBw'} && $perf_ref->{'fwdRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'fwdRebuildWriteBwcBw'} && $perf_ref->{'fwdRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'fwdRebuildWriteBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildReadBwcBw'} && $perf_ref->{'bckRebuildReadBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildReadBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildReadBwcBw'};
      }
    }
    if ( defined $perf_ref->{'bckRebuildWriteBwcBw'} && $perf_ref->{'bckRebuildWriteBwcBw'} ne ""){
      if (Xormon::isdigit($rebuild)){
        $rebuild += $perf_ref->{'bckRebuildWriteBwcBw'};
      }
      else{
        $rebuild = $perf_ref->{'bckRebuildWriteBwcBw'};
      }
    }
    $data_out .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$resp_t;$resp_t_r;$resp_t_w;$cache_hit_r;$rebalance;$rebuild\n";
  }

  #SAVE DRIVE
  #print $data_out;

  $rc = $xormon->saveCSV($data_out,"powerflex","drive");
  #status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
    $main_rc = $status;
  }
  $logger->info("DRIVE status : $status");
  $data_out = "";

  #print Dumper \%conf;
  #save configuration
  my $rc_conf = $xormon->saveConfJSON(\%conf);
  $logger->info("save conf: $rc_conf");

  #save architecture
  my $rc_arch = $xormon->saveArchitecture(\@architecture);
  $logger->info("save architecture: $rc_arch");

  return $main_rc;
}

sub health_status {
  my %hs_data;
  my $return         = "";
  my @error_message  = ();
  my $act_timestamp  = time();
  my $main_state     = "ok";
  my $url            = $cfg->{'url'};
  my $st_uuid        = $cfg->{'hostcfg_id'};

  if ( defined $data{'System'}{'health'}{'mdmCluster'} && $data{'System'}{'health'}{'mdmCluster'} ne "" ) {
    my $hash_ref      = $data{'System'}{'health'}{'mdmCluster'};
    # cluster
    my $cluster_state = "";
    my $cluster_mode  = "";
    if ( exists $hash_ref->{'clusterState'} ) {
      $cluster_state = $hash_ref->{'clusterState'};
      if ( $cluster_state ne "ClusteredNormal" ) {
        $main_state = "error";
        my $error_line = "ClusterState: $cluster_state";
        push( @error_message, "$error_line\n" );
      }
    }
    if ( exists $hash_ref->{'clusterMode'} ) {
      $cluster_mode = $hash_ref->{'clusterMode'};
    }
    # master
    if ( exists $hash_ref->{'master'} ) {
      my $name   = "";
      my $role   = "";
      my $status = "";
      my @ips    = ();
      my $mode   = "Primary";
      if ( exists $hash_ref->{'master'}{'name'} ) {
        $name = $hash_ref->{'master'}{'name'};
      }
      else {
        $name = $hash_ref->{'master'}{'id'};
      }
      if ( exists $hash_ref->{'master'}{'role'} ) {
        $role = $hash_ref->{'master'}{'role'};
      }
      if ( exists $hash_ref->{'master'}{'status'} ) {
        $status = $hash_ref->{'master'}{'status'};
        if ( $status ne "Normal" ) {
          $main_state = "error";
          my $error_line = "$name status: $status";
          push( @error_message, "$error_line\n" );
        }
      }
      if ( exists $hash_ref->{'master'}{'ips'} ) {
        foreach my $ip ( @{ $hash_ref->{'master'}{'ips'} } ) {
          push( @ips, $ip );
        }
      }
      my %mdm;
      $mdm{name}       = $name;
      $mdm{role}       = $role;
      $mdm{status}     = $status;
      $mdm{mode}       = $mode;
      $mdm{ips}        = \@ips;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{mdm} }, \%mdm );
    }
    # slave
    if ( exists $hash_ref->{'slaves'} && $hash_ref->{'slaves'} ne "" ) {
      foreach my $slv ( @{ $hash_ref->{'slaves'} } ) {
        my $name   = "";
        my $role   = "";
        my $status = "";
        my @ips    = ();
        my $mode   = "Secondary";
        if ( exists $slv->{'name'} ) {
          $name = $slv->{'name'};
        }
        else {
          $name = $slv->{'id'};
        }
        if ( exists $slv->{'role'} ) {
          $role = $slv->{'role'};
        }
        if ( exists $slv->{'ips'} ) {
          foreach my $ip ( @{ $slv->{'ips'} } ) {
            push( @ips, $ip );
          }
        }
        if ( exists $slv->{'status'} ) {
          $status = $slv->{'status'};
          if ( $status ne "Normal" ) {
            $main_state = "error";
            my $error_line = "$name status: $status";
            push( @error_message, "$error_line\n" );
          }
        }
        my %slave;
        $slave{name}       = $name;
        $slave{role}       = $role;
        $slave{status}     = $status;
        $slave{mode}       = $mode;
        $slave{ips}        = \@ips;
        push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{mdm} }, \%slave );
      }
    }
    # tie breaker
    if ( exists $hash_ref->{'tieBreakers'} && $hash_ref->{'tieBreakers'} ne "" ) {
      foreach my $tb ( @{ $hash_ref->{'tieBreakers'} } ) {
        my $name   = "";
        my $role   = "";
        my $status = "";
        my @ips    = ();
        my $mode   = "TieBreaker";
        if ( exists $tb->{'name'} ) {
          $name = $tb->{'name'};
        }
        else {
          $name = $tb->{'id'};
        }
        if ( exists $tb->{'role'} ) {
          $role = $tb->{'role'};
        }
        if ( exists $tb->{'ips'} ) {
          foreach my $ip ( @{ $tb->{'ips'} } ) {
            push( @ips, $ip );
          }
        }
        if ( exists $tb->{'status'} ) {
          $status = $tb->{'status'};
          if ( $status ne "Normal" ) {
            $main_state = "error";
            my $error_line = "$name status: $status";
            push( @error_message, "$error_line\n" );
          }
        }
        my %tie;
        $tie{name}       = $name;
        $tie{role}       = $role;
        $tie{status}     = $status;
        $tie{mode}       = $mode;
        $tie{ips}        = \@ips;
        push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{mdm} }, \%tie );
      }
    }
  }

  if ( defined $data{'ProtectionDomain'} && $data{'ProtectionDomain'} ne "" ) {
    foreach my $id ( keys %{ $data{'ProtectionDomain'} } ) {
      my $protdom_name  = "";
      my $protdom_state = "";

      if ( exists $data{'ProtectionDomain'}{$id}{'conf'}{'name'} ) {
        $protdom_name = $data{'ProtectionDomain'}{$id}{'conf'}{'name'};
      }
      if ( exists $data{'ProtectionDomain'}{$id}{'conf'}{'protectionDomainState'} ) {
        $protdom_state = $data{'ProtectionDomain'}{$id}{'conf'}{'protectionDomainState'};
        if ( $protdom_state ne "Active" ) {
          $main_state = "error";
          my $error_line = "$protdom_name status: $protdom_state";
          push( @error_message, "$error_line\n" );
        }
      }
      my $protdom_hash = Xormon::create_hash_key($st_uuid."PD".$protdom_name); $protdom_hash = lc($protdom_hash);
      my %pd;
      $pd{name}       = $protdom_name;
      $pd{state}      = $protdom_state;
      $pd{hash_id}    = $protdom_hash;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{protection_domain} }, \%pd );
    }
  }

  if ( defined $data{'Sds'} && $data{'Sds'} ne "" ) {
    foreach my $id ( keys %{ $data{'Sds'} } ) {
      my $sds_name       = "";
      my $sds_state      = "";
      my $sds_membership = "";
      my $sds_connection = "";
      if ( exists $data{'Sds'}{$id}{'conf'}{'name'} ) {
        $sds_name = $data{'Sds'}{$id}{'conf'}{'name'};
      }
      if ( exists $data{'Sds'}{$id}{'conf'}{'sdsState'} ) {
        $sds_state = $data{'Sds'}{$id}{'conf'}{'sdsState'};
      }
      if ( exists $data{'Sds'}{$id}{'conf'}{'membershipState'} ) {
        $sds_membership = $data{'Sds'}{$id}{'conf'}{'membershipState'};
      }
      if ( exists $data{'Sds'}{$id}{'conf'}{'mdmConnectionState'} ) {
        $sds_connection = $data{'Sds'}{$id}{'conf'}{'mdmConnectionState'};
      }
      if ( $sds_state ne "Normal" || $sds_membership ne "Joined" || $sds_connection ne "Connected" ) {
        $main_state = "error";
        my $error_line = "$sds_name state: $sds_state membership: $sds_membership connection: $sds_connection";
        push( @error_message, "$error_line\n" );
      }
      my $sds_hash = Xormon::create_hash_key($st_uuid."SDS".$sds_name); $sds_hash = lc($sds_hash);
      my %sds;
      $sds{name}       = $sds_name;
      $sds{state}      = $sds_state;
      $sds{membership} = $sds_membership;
      $sds{connection} = $sds_connection;
      $sds{hash_id}    = $sds_hash;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{sds} }, \%sds );
    }
  }


   if ( defined $data{'Device'} && $data{'Device'} ne "" ) {

    foreach my $id ( keys %{ $data{'Device'} } ) {
      my $dev_name  = "";
      my $dev_path  = "";
      my $dev_sds   = "";
      my $dev_state = "";
      my $dev_error = "";
      my $dev_agr   = "";
      my $r_name    = "";
      if ( exists $data{'Device'}{$id}{'conf'}{'name'} ) {
        $dev_name = $data{'Device'}{$id}{'conf'}{'name'};
      }
      else {
        $dev_name = $id;
      }
      if ( exists $data{'Device'}{$id}{'conf'}{'deviceOriginalPathName'} ) {
        $dev_path = $data{'Device'}{$id}{'conf'}{'deviceOriginalPathName'};
      }
      if ( exists $data{'Device'}{$id}{'conf'}{'sdsId'} ) {
        my $sds_id = $data{'Device'}{$id}{'conf'}{'sdsId'};
        if ( exists $data{'Sds'}{$sds_id}{'conf'}{'name'} ) {
          $dev_sds = $data{'Sds'}{$sds_id}{'conf'}{'name'};
        }
        else {
          $dev_sds = $sds_id;
        }
        $r_name   = "$dev_sds" . "_" . "$dev_name";
      }
      else {
        $r_name = "$id" . "_" . "$dev_name";
      }
      if ( exists $data{'Device'}{$id}{'conf'}{'deviceState'} ) {
        $dev_state = $data{'Device'}{$id}{'conf'}{'deviceState'};
      }
      if ( exists $data{'Device'}{$id}{'conf'}{'errorState'} ) {
        $dev_error = $data{'Device'}{$id}{'conf'}{'errorState'};
        if ( $dev_error ne "None" && $dev_error ne "Notice" && $dev_error ne "Info" ) {
          $main_state = "error";
          my $error_line = "$dev_name sds: $dev_sds error: $dev_error";
          push( @error_message, "$error_line\n" );
        }
      }
      if ( exists $data{'Device'}{$id}{'conf'}{'aggregatedState'} ) {
        $dev_agr = $data{'Device'}{$id}{'conf'}{'aggregatedState'};
      }

      my $dev_hash = Xormon::create_hash_key($st_uuid."DRIVE".$r_name); $dev_hash = lc($dev_hash);

      my %drive;
      $drive{name}      = $r_name;
      $drive{path}      = $dev_path;
      $drive{sds}       = $dev_sds;
      $drive{state}     = $dev_state;
      $drive{error}     = $dev_error;
      $drive{aggregate} = $dev_agr;
      $drive{hash_id}   = $dev_hash;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{drives} }, \%drive );
    }
  }

  ## ALERT TABLE
  $return = new_get_request("$url/api/types/Alert/instances");

  #print Dumper \$return;

  if ( defined $return && ref($return) eq "ARRAY" ) {
    foreach my $alert ( @{$return} ) {
      my $time_diff    = "";

      my $al_severity  = $alert->{'severity'};
      my $al_objtype   = $alert->{'affectedObject'}{'type'};
      my $al_name      = $alert->{'alertValues'}{'Name'};
      my $al_type      = $alert->{'alertType'};
      my $al_msg       = $alert->{'alertValues'}{'Message'};
      my $al_starttime = $alert->{'startTime'};
      my $al_lastobs   = $alert->{'lastObserved'};
      if ( !defined $al_severity )  { $al_severity  = ""; }
      if ( !defined $al_objtype )   { $al_objtype   = ""; }
      if ( !defined $al_name )      { $al_name      = ""; }
      if ( !defined $al_type )      { $al_type      = ""; }
      if ( !defined $al_msg )       { $al_msg       = ""; }
      if ( !defined $al_starttime ) { $al_starttime = ""; }
      if ( !defined $al_lastobs )   { $al_lastobs   = ""; }

      if ( defined $al_starttime ) {
        # 2022-02-11T08:16:28.017Z
        my $al_start_utime = str2time( $alert->{'startTime'} );
        ## skip 7 days and older alerts
        $time_diff = $act_timestamp - $al_start_utime;
        if ( $time_diff > 604800 ) {
          next;
        }
      }
      # if ( defined $alert->{'lastObserved'} ) {
      #   # 2022-02-11T08:16:28.017Z
      #   my $al_lastobs_utime = str2time( $alert->{'lastObserved'} );
      #   $al_lastobs = strftime( '%Y-%m-%d %H:%M:%S', localtime($al_lastobs_utime) );
      # }
      # my $alert_line = "$al_severity : $al_objtype : $al_name : $al_type : $al_starttime\n";

      if ( !defined $al_severity )  { $al_severity  = ""; }
      if ( !defined $al_objtype )   { $al_objtype   = ""; }
      if ( !defined $al_name )      { $al_name      = ""; }
      if ( !defined $al_type )      { $al_type      = ""; }
      if ( !defined $al_msg )       { $al_msg       = ""; }
      if ( !defined $al_starttime ) { $al_starttime = ""; }
      if ( !defined $al_lastobs )   { $al_lastobs   = ""; }

      my %alert;
      $alert{severity}   = $al_severity;
      $alert{object}     = $al_objtype;
      $alert{name}       = $al_name;
      $alert{type}       = $al_type;
      $alert{message}    = $al_msg;
      $alert{starttime}  = $al_starttime;
      $alert{lasttime}   = $al_lastobs;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{alerts} }, \%alert );
    }
  }

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  my @hs = (
    {
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $main_state,
      "data" => join("\n", @error_message),
    }
  );

  my $rc = $xormon->saveStatus(\@hs);
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);
}


sub connection_test {
  my $client = shift;
  $cfg = shift;

  $logger = XormonLogger->new('powerflex', $cfg->{hostcfg_id}, $cfg->{hostalias} );

  my @rows;
  my %conn_data;

  my $test = Test->new($client);

  # test main variables
  #
  # unless ( exists $cfg->{'hostcfg_id'} )    { push(@rows, "Cannot continue, 'hostcfg_idd' is not configured!");   return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'hostalias'} )     { push(@rows, "Cannot continue, 'hostalias' is not configured!");     return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiHost'} )       { push(@rows, "Cannot continue, 'apiHost' is not configured!");       return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiPort'} )       { push(@rows, "Cannot continue, 'apiPort' is not configured!");       return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiUsername'} )   { push(@rows, "Cannot continue, 'apiUsername' is not configured!");   return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiPassword'} )   { push(@rows, "Cannot continue, 'apiPassword' is not configured!");   return Xormon::conn_test(0, \@rows); }

  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiPort", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  my $api_ip   = $cfg->{'apiHost'};
  my $api_port = $cfg->{'apiPort'};
  $cfg->{'sample_rate'} = 300;

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($api_ip,$api_port);
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  my $timestamp = time();
  $cfg->{'timestamp'} = $timestamp;

  my $url = "https://$api_ip:$api_port";

  ## login
  my $token = login( "$url/api/login" );
  if ( !$token ) {
    $test->addSection("API authorization", 0, "Token NOT granted from $api_ip!");
    return $test->stop();
  }
  else {
    $cfg->{'gen_token'} = $token;
    $test->addSection("API authorization", 1, "Token granted from $api_ip");
  }

  #
  # collect system data
  #

  my $return = new_get_request("$url/api/types/System/instances");
  #print Dumper $return;

  if ( defined $return && ref($return) eq "ARRAY" ) {
    ## array for json to post
    my @post_props = ( "numOfProtectionDomains", "numOfSdc", "numOfSds", "numOfVolumes", "numOfDevices", "numOfVtrees", "numOfStoragePools", "numOfUnmappedVolumes", "numOfMappedToAllVolumes", "numOfThickBaseVolumes", "numOfThinBaseVolumes", "numOfSnapshots", "numOfVolumesInDeletion", "primaryReadBwc", "primaryWriteBwc", "userDataSdcReadLatency", "userDataSdcWriteLatency", "primaryReadFromRmcacheBwc", "rplReceiveBwc", "rplTransmitBwc", "rplApplyLatency", "rplReceiveLatency", "rplTransmitLatency", "rplTotalJournalCap", "rplUsedJournalCap", "normRebuildReadBwc", "normRebuildWriteBwc", "fwdRebuildReadBwc", "fwdRebuildWriteBwc", "bckRebuildReadBwc", "bckRebuildWriteBwc", "pendingNormRebuildCapacityInKb", "pendingFwdRebuildCapacityInKb", "pendingBckRebuildCapacityInKb", "rebalanceReadBwc", "rebalanceWriteBwc", "pendingRebalanceCapacityInKb", "netMaxUserDataCapacityInKb", "netCapacityInUseInKb", "netUnusedCapacityInKb", "maxUserDataCapacityInKb", "capacityInUseInKb", "unusedCapacityInKb" );
    my $json_body->{'properties'} = \@post_props;
    foreach my $system ( @{$return} ) {
      if ( defined $system->{'id'} && $system->{'id'} ne "" ) {
        my $id = $system->{'id'};
        $conn_data{'conf'}{'id'} = $id;
        push @{ $json_body->{'ids'} }, $id;

        if ( defined $system->{'name'} && $system->{'name'} ne "" ) {
          $conn_data{'conf'}{'name'} = $system->{'name'};
        }
        if ( defined $system->{'systemVersionName'} && $system->{'systemVersionName'} ne "" ) {
          $conn_data{'conf'}{'systemVersionName'} = $system->{'systemVersionName'};
        }
        if ( defined $system->{'installId'} && $system->{'installId'} ne "" ) {
          $conn_data{'conf'}{'installId'} = $system->{'installId'};
        }
        if ( defined $system->{'swid'} && $system->{'swid'} ne "" ) {
          $conn_data{'conf'}{'swid'} = $system->{'swid'};
        }
        if ( defined $system->{'mdmCluster'}->{'clusterMode'} && $system->{'mdmCluster'}->{'clusterMode'} ne "" ) {
          $conn_data{'conf'}{'clusterMode'} = $system->{'mdmCluster'}->{'clusterMode'};
        }

        my $post_request = new_post_request( "$url/api/types/System/instances/action/querySelectedStatistics", $json_body );
        if ( defined $return && ref($return) eq "ARRAY" ) {

          #print Dumper \$post_request;
          foreach my $metric (@post_props) {
            if ( defined $post_request->{$metric} && $post_request->{$metric} ne "" ) {
              if ( $metric =~ /^numOf/ ) {    ## save subsystem numbers to conf key
                $conn_data{'conf'}{$metric} = $post_request->{$metric};
              }
              elsif ( $metric =~ /InKb$/ ) {
                my $metric_b = $metric =~ s/InKb/InB/r;
                my $value_kb  = $post_request->{$metric};
                my $value_B  = Xormon::returnBytes( $value_kb, "kB");
                $conn_data{'perf'}{$metric_b} = $value_B;
              }
              elsif ( $metric eq "rplTotalJournalCap" || $metric eq "rplUsedJournalCap" ) {
                my $value_kb = $post_request->{$metric};
                my $value_B  = Xormon::returnBytes( $value_kb, "kB");
                $conn_data{'perf'}{$metric} = $value_B;
              }
              elsif ( $metric =~ /^userDataSdcReadLatency$/ || $metric =~ /^userDataSdcWriteLatency$/ || $metric =~ /^rplApplyLatency$/ || $metric =~ /^rplReceiveLatency$/ || $metric =~ /^rplTransmitLatency$/ ) {
                $conn_data{'perf'}{$metric} = calc_lat( $post_request->{$metric} );
              }
              elsif ( $metric =~ /Bwc$/ ) {    ## if bwc metric, calculate IO and DATA
                my $metric_io = "$metric" . "Io";
                my $metric_bw = "$metric" . "Bw";
                $conn_data{'perf'}{$metric_io} = calc_bwc_io( $post_request->{$metric} );
                $conn_data{'perf'}{$metric_bw} = calc_bwc_bw( $post_request->{$metric} );
              }
              else {
                $conn_data{'perf'}{$metric} = $post_request->{$metric};
              }
            }    ## else API not return
          }
        }
        else {
          $test->addSection("API Data", 0 , "Data NOT returned!");
          return $test->stop();
        }
      }
    }
  }
  else {
    $test->addSection("API Data", 0 , "Instance NOT returned!");
    return $test->stop();
  }

  if (exists $conn_data{conf} && $conn_data{conf} ne '') {
    if ( exists $conn_data{conf}{id} )                { push(@rows, "id      : $conn_data{conf}{id}\n"); }
    if ( exists $conn_data{conf}{name} )              { push(@rows, "name    : $conn_data{conf}{name}\n"); }
    if ( exists $conn_data{conf}{clusterMode} )       { push(@rows, "mode    : $conn_data{conf}{clusterMode}\n"); }
    if ( exists $conn_data{conf}{systemVersionName} ) {
      if ( $conn_data{conf}{systemVersionName} =~ /Version:/ ) {
        my ( $sys_model, $sys_version ) = split( / Version: /, $conn_data{conf}{systemVersionName} );
        push(@rows, "version : $sys_version\n");
      }
    }
  }
  else {
    $test->addSection("API Data", 0 , "System information not found!");
    return $test->stop();
  }

  unless ( exists $conn_data{perf}{primaryReadBwcIo} && Xormon::isdigit( $conn_data{perf}{primaryReadBwcIo} ) ) {
    $test->addSection("API Data", 0 , "Performance data missing!");
    return $test->stop();
  }
  else {
    push (@rows, "Performance data OK\n" );
  }

  $test->addSection("API Data", 1 , \@rows);
  return $test->stop();
}


sub new_get_request {
  my $url      = shift;
  my $api_user = $cfg->{'apiUsername'};
  my $token    = $cfg->{'gen_token'};

  my $agent = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( GET => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $api_user, $token );

  my $results = $agent->request($request);
  my $st_line = $results->status_line;

  #print "HTTP code: $st_line\n";

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    my $res_con = $results->content;
    $logger->error("GET url      : $url");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    return 0;
  }

  my $result = decode_json( $results->content );

  return $result;
}

sub new_post_request {
  my $url       = shift;
  my $json_body = shift;

  my $api_user = $cfg->{'apiUsername'};
  my $token    = $cfg->{'gen_token'};

  #print Dumper $json_body;
  my $body = encode_json($json_body);

  #print Dumper $body;

  my $agent = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( POST => $url );
  $request->header( 'Content-Type' => 'application/json', 'Accept' => 'application/json' );
  $request->authorization_basic( $api_user, $token );
  $request->content($body);

  my $results = $agent->request($request);
  my $st_line = $results->status_line;

  #print "HTTP code: $st_line\n";

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    my $res_con = $results->content;
    $logger->error("POST url     : $url");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    return 0;
  }

  my $result = decode_json( $results->content );

  #print Dumper \$result;

  return $result;
}

sub calc_bwc_io {
  my $metric_ref = shift;
  if ( defined $metric_ref->{'numOccured'} && defined $metric_ref->{'numSeconds'} ) {
    my $num_occured = $metric_ref->{'numOccured'};
    my $num_seconds = $metric_ref->{'numSeconds'};
    if ( $num_seconds != 0 ) {
      #my $calc_io = sprintf( "%.0f", $num_occured / $num_seconds );
      my $calc_io =  Xormon::formatNumberByType("float4", ($num_occured / $num_seconds));
      return $calc_io;
    }
    else {
      return 0;
    }
  }
}

sub calc_bwc_bw {
  my $metric_ref = shift;
  if ( defined $metric_ref->{'totalWeightInKb'} && defined $metric_ref->{'numSeconds'} ) {
    my $total_weight = $metric_ref->{'totalWeightInKb'};
    my $num_seconds  = $metric_ref->{'numSeconds'};
    if ( $num_seconds != 0 ) {
      my $calc_bw = Xormon::returnBytes(($total_weight / $num_seconds), "kB");
      return $calc_bw;
    }
    else {
      return 0;
    }
  }
}

sub calc_lat {
  my $metric_ref = shift;
  if ( defined $metric_ref->{'totalWeightInKb'} && defined $metric_ref->{'numOccured'} ) {
    my $total_weight = $metric_ref->{'totalWeightInKb'};
    my $num_occured  = $metric_ref->{'numOccured'};
    if ( $num_occured != 0 ) {
      #my $calc_lat = (( $total_weight / $num_occured ) / 1000);
      my $calc_lat =  Xormon::formatNumberByType("float4", (($total_weight / $num_occured ) / 1000));
      #print "$calc_lat\n";
      return $calc_lat;
    }
    else {
      return 0;
    }
  }
}

sub login {
  my $url      = shift;
  my $api_user = $cfg->{'apiUsername'};
  my $api_pass = $cfg->{'apiPassword'};

  my $agent = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( GET => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $api_user, $api_pass );

  #print Dumper $agent;
  #print Dumper $request;

  my $response = $agent->request($request);
  my $st_line  = $response->status_line;
  if ( !$response->is_success ) {
    my $res_con = $response->content;
    $logger->error("GET url      : $url");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    return 0;
  }

  #print Dumper $response;

  my $result = $response->content;
  $result =~ s/"//g;
  return $result;
}

sub logout {
  my $url   = shift;
  my $agent = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );

  my $api_user = $cfg->{'apiUsername'};
  my $token    = $cfg->{'gen_token'};

  my $request = HTTP::Request->new( GET => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $api_user, $token );

  #print Dumper $agent;
  #print Dumper $request;

  my $response = $agent->request($request);
  my $st_line  = $response->status_line;

  #print "URL GET : $url\n";
  #print "HTTP code: $st_line\n";

  if ( !$response->is_success ) {
    my $st_line = $response->status_line;
    my $res_con = $response->content;
    $logger->error("GET url      : $url");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    return 0;
  }
  return 1;
}

1;
