package POWERSTORE;

use lib '../../../perl-modules';

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Date::Parse;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use Xormon;
use File::Path qw(rmtree);
use JSON qw(encode_json decode_json);
use Test;
use XormonLogger;
use Cwd qw(cwd);

# microservice's directory
my $act_dir = cwd;
my $wrkdir  = "";


my %data;
my $xormon;
my $cfg;
my $logger;
my $debug = 0;
my $stat_found = 0;

my $url_paths;
my %last_stats; # last timestamps of the current stats
my %capacity;
my %host_mapping;

my $service_data_in;
my $service_data_out;

sub collect {
  $xormon = shift;
  $cfg    = shift;

  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")    && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")     && return "error"; }
  $logger = XormonLogger->new( 'powerstore', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost'} )       { $logger->error("Cannot continue, 'host' is not configured!")          && return "error"; }
  unless ( exists $cfg->{'apiPort'} )       { $logger->error("Cannot continue, 'api-port' is not configured!")      && return "error"; }
  unless ( exists $cfg->{'apiUsername'} )   { $logger->error("Cannot continue, 'apiUsername' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'apiPassword'} )   { $logger->error("Cannot continue, 'apiPassword' is not configured!")   && return "error"; }
  $cfg->{'sample_rate'} = 300;

  if ( $debug == 2 ) {
    $wrkdir = "$act_dir/data/$cfg->{'hostcfg_id'}";
    unless ( -d "$act_dir/data" )   { mkdir( "$act_dir/data", 0755 )   || $logger->error( "Cannot mkdir '$act_dir/data': $!" . __FILE__ . ":" . __LINE__ ); }
    unless ( -d "$wrkdir" )         { mkdir( "$wrkdir", 0755 )         || $logger->error( "Cannot mkdir '$wrkdir': $!" . __FILE__ . ":" . __LINE__ ); }
  }

  %last_stats   = ();
  %host_mapping = ();
  %capacity     = ();

  #
  # get microservice data
  #
  $service_data_in  = $xormon->getData();
  $service_data_out = {};

  $logger->info("powerstore ms: START") if $debug;

  $cfg->{'url'}  = "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}/api/rest";
  $url_paths  = set_url_paths();
  $cfg->{'sunix'}   = set_time_range();
  $cfg->{'start_time'} = time();

  collect_data("conf");
  set_host_mapping();
  collect_data("perf");
  if ( exists $data{conf}{appliance} ) {
    $xormon->isConnected(1);
  }
  else {
    $xormon->isConnected(0, "Appliance information not found");
    $logger->error("Appliance information not found") && return "error";
  }
  print_conf();
  print_perf();

  # health status
  collect_data("health_status");
  health_status();

  # print Dumper $cfg;

  $xormon->saveData(\%{ $service_data_out });

  $logger->info("powerstore ms: END") if $debug;

  return 1;
}

sub set_time_range {

  my $act_time = time();
  my $sunix    = "";

  if ( exists $service_data_in->{last_perf_timestamp} && Xormon::isdigit($service_data_in->{last_perf_timestamp}) ) {
    my $ts = $service_data_in->{last_perf_timestamp};
    if ( Xormon::isdigit($ts) && $ts <= $act_time ) {
      if ( ( $act_time - $ts ) > 86399 ) {
        $logger->error( "Data were not be collected longer than 24 hours. Do not use sunix parameter, collect data for the whole day back. $!" . __FILE__ . ":" . __LINE__ );
      }
      elsif ( ( $act_time - $ts ) < $cfg->{'sample_rate'} ) {
        $logger->error( "Last perf file was created less than $cfg->{'sample_rate'} seconds ago. Do not run agent at this time! Exitting... $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
      else {
        #$sunix = $ts - $sample_rate;
        $sunix = $ts;
        $logger->info( "sunix: $sunix : " . localtime($sunix) ) if $debug;
      }
    }
  }

  return $sunix;
}

sub connection_test {

  my $client = shift;
  $cfg = shift;
  $logger = XormonLogger->new( 'powerstore', $cfg->{hostcfg_id}, $cfg->{hostalias} );

  my @rows;
  my $test = Test->new($client);

  my @variables = ("hostcfg_id", "hostalias", "apiHost", "apiPort", "apiUsername", "apiPassword");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  $cfg->{'url'}  = "https://$cfg->{'apiHost'}:$cfg->{'apiPort'}/api/rest";

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{apiHost},$cfg->{apiPort});
  $test->addSection("API host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  $url_paths = {
    conf => [
      { path   => 'appliance',
        select => [ 'eth_ports', 'express_service_code', 'fc_ports', 'hardware', 'id', 'ip_pool_addresses', 'maintenance_windows', 'model', 'name', 'nodes', 'sas_ports', 'service_tag', 'software_installed', 'veth_ports', 'virtual_volumes', 'volumes' ],
      }
    ],
    perf => [
      { path   => 'appliance',
        select => [ 'eth_ports', 'express_service_code', 'fc_ports', 'hardware', 'id', 'ip_pool_addresses', 'maintenance_windows', 'model', 'name', 'nodes', 'sas_ports', 'service_tag', 'software_installed', 'veth_ports', 'virtual_volumes', 'volumes' ],
        entity => [
          { 'performance_metrics_by_appliance' => [
              { "appliance_id"                    => "appliance_id" },
              { "avg_io_size"                     => "io_size" },
              { "avg_io_workload_cpu_utilization" => "io_workload_cpu_utilization" },
              { "avg_latency"                     => "resp_t" },
              { "avg_read_bandwidth"              => "read" },
              { "avg_read_iops"                   => "read_io" },
              { "avg_read_latency"                => "resp_t_r" },
              { "avg_read_size"                   => "read_size" },
              { "avg_total_bandwidth"             => "data_rate" },
              { "avg_total_iops"                  => "io_rate" },
              { "avg_write_bandwidth"             => "write" },
              { "avg_write_iops"                  => "write_io" },
              { "avg_write_latency"               => "resp_t_w" },
              { "avg_write_size"                  => "write_size" },
            ]
          }
        ]
      }
    ]
  };

  collect_data("conf");
  if ( exists $data{conf}{appliance} ) {
    foreach my $id ( sort keys %{ $data{conf}{appliance} } ) {
      my $hash_ref = $data{conf}{appliance}{$id};

      if ( defined $hash_ref->{model} )       { push(@rows, "Model: $hash_ref->{model}\n"); }
      if ( defined $hash_ref->{service_tag} ) { push(@rows, "Service tag: $hash_ref->{service_tag}\n"); }
      if ( defined $hash_ref->{name} )        { push(@rows, "name: $hash_ref->{name}\n"); }
      if ( defined $hash_ref->{id} )          { push(@rows, "id: $hash_ref->{id}\n"); }
    }
  }
  else {
    $test->addSection("API Data", 0 , "Appliance information not found!");
    return $test->stop();
  }

  collect_data("perf");
  if ( $stat_found > 0 ) {
    push (@rows, "Performance data OK\n" );
  }
  else {
    $test->addSection("API Data", 0 , "Performance data missing!");
    return $test->stop();
  }
  $test->addSection("API Data", 1 , \@rows);
  return $test->stop();
}


sub set_host_mapping {

  if ( exists $data{conf}{host_volume_mapping} ) {
    foreach my $id ( sort keys %{ $data{conf}{host_volume_mapping} } ) {
      my $hash_ref = $data{conf}{host_volume_mapping}{$id};

      if ( !defined $hash_ref->{volume_id} || $hash_ref->{volume_id} eq '' ) { next; }
      if ( defined $hash_ref->{host_id} && $hash_ref->{host_id} ne '' ) {
        $host_mapping{HOST}{ $hash_ref->{host_id} }{ $hash_ref->{volume_id} } = $hash_ref->{volume_id};
      }
      if ( defined $hash_ref->{host_group_id} && $hash_ref->{host_group_id} ne '' ) {
        $host_mapping{HOST_GROUP}{ $hash_ref->{host_group_id} }{ $hash_ref->{volume_id} } = $hash_ref->{volume_id};
      }
    }
  }

  return 1;
}

sub print_conf {
  my @architecture;
  my %conf;
  $logger->info("print conf") if $debug;

  #
  # DEVICE
  #
  # Architecture
  my %device_arch = (
    "item_id"    => $cfg->{'hostcfg_id'},
    "hostcfg_id" => $cfg->{'hostcfg_id'},
    "label"      => $cfg->{'hostalias'},
    "hw_type"    => "powerstore",
    "subsystem"  => "device",
    "parents"    => []
  );
  push(@architecture, \%device_arch);

  my $app_name = "";
  my $app_id   = "";

  if ( exists $data{conf}{appliance} ) {
    foreach my $id ( sort keys %{ $data{conf}{appliance} } ) {
      my $hash_ref = $data{conf}{appliance}{$id};
      if ( defined $hash_ref->{model} )       { $conf{ $cfg->{'hostcfg_id'} }{model}    = $hash_ref->{model}; }
      if ( defined $hash_ref->{service_tag} ) { $conf{ $cfg->{'hostcfg_id'} }{serial}   = $hash_ref->{service_tag}; }
      if ( defined $hash_ref->{name} )        { $app_name = $hash_ref->{name}; }
      if ( defined $hash_ref->{id} )          { $app_id   = $hash_ref->{id}; }
      $conf{ $cfg->{'hostcfg_id'} }{'ip'} = $cfg->{'apiHost'};
    }
  }

  #
  # appliance
  #
  if (defined $app_id && $app_id ne "") {
    my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "POOL" . $app_id);
    if ($app_name eq "") { $app_name = $app_id; }
    if ( exists $capacity{appliance}{$app_id} ) {
      my $values = $capacity{appliance}{$app_id};
      if ( exists $values->{physical_total}      && Xormon::isdigit( $values->{physical_total} ) )      { $conf{$hash_key}{capacity_total}         = $values->{physical_total}; }
      if ( exists $values->{physical_used}       && Xormon::isdigit( $values->{physical_used} ) )       { $conf{$hash_key}{capacity_used}          = $values->{physical_used}; }
      if ( exists $values->{logical_provisioned} && Xormon::isdigit( $values->{logical_provisioned} ) ) { $conf{$hash_key}{virtual_capacity_total} = $values->{logical_provisioned}; }
      if ( exists $values->{logical_used}        && Xormon::isdigit( $values->{logical_used} ) )        { $conf{$hash_key}{virtual_capacity_used}  = $values->{logical_used}; }
      if ( Xormon::isdigit($conf{$hash_key}{capacity_total}) && Xormon::isdigit($conf{$hash_key}{capacity_used}) ) {
        $conf{$hash_key}{capacity_free} = $conf{$hash_key}{capacity_total} - $conf{$hash_key}{capacity_used};
      }
      if ( Xormon::isdigit($conf{$hash_key}{virtual_capacity_total}) && Xormon::isdigit($conf{$hash_key}{virtual_capacity_used}) ) {
        $conf{$hash_key}{virtual_capacity_free} = $conf{$hash_key}{virtual_capacity_total} - $conf{$hash_key}{virtual_capacity_used};
      }
    }
    my %arch = (
      "item_id"   => $hash_key,
      "hostcfg_id" => $cfg->{'hostcfg_id'},
      "label"     => $app_name,
      "hw_type"   => "powerstore",
      "subsystem" => "pool",
      "parents"   => [$cfg->{'hostcfg_id'}]
    );
    push(@architecture, \%arch);
  }
  #
  # cluster
  #
  if ( exists $data{conf}{cluster} ) {
    foreach my $id ( sort keys %{ $data{conf}{cluster} } ) {
      my $hash_ref = $data{conf}{cluster}{$id};
      my $name = $id;
      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "CLUSTER" . $id);

      if (defined $hash_ref->{name}) {
        # $conf{$hash_key}{name} = $hash_ref->{name};
        $name = $hash_ref->{name};
      }
      # $conf{$hash_key}{name}                = ( defined $hash_ref->{name} )                ? $hash_ref->{name}                : "";
      # $conf{$hash_key}{global_id}           = ( defined $hash_ref->{global_id} )           ? $hash_ref->{global_id}           : "";
      # $conf{$hash_key}{appliance_count}     = ( defined $hash_ref->{appliance_count} )     ? $hash_ref->{appliance_count}     : "";
      # $conf{$hash_key}{management_address}  = ( defined $hash_ref->{management_address} )  ? $hash_ref->{management_address}  : "";
      # $conf{$hash_key}{master_appliance_id} = ( defined $hash_ref->{master_appliance_id} ) ? $hash_ref->{master_appliance_id} : "";
      # $conf{$hash_key}{physical_mtu}        = ( defined $hash_ref->{physical_mtu} )        ? $hash_ref->{physical_mtu}        : "";
      # $conf{$hash_key}{state}               = ( defined $hash_ref->{state} )               ? $hash_ref->{state}               : "";
      my %arch = (
        "item_id"   => $hash_key,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $name,
        "hw_type"   => "powerstore",
        "subsystem" => "cluster",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
    }
  }

  #
  # eth_port
  #
  if ( exists $data{conf}{eth_port} ) {

    foreach my $id ( sort keys %{ $data{conf}{eth_port} } ) {
      my $hash_ref = $data{conf}{eth_port}{$id};

      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "ETHPORT" . $id);
      my $name = $id;
      # my $name                  = ( defined $hash_ref->{name} )                  ? $hash_ref->{name}                  : "";
      if (defined $hash_ref->{name}) {
        # $conf{$hash_key}{name} = $hash_ref->{name};
        $name = $hash_ref->{name};
      }

      # $conf{$hash_key}{node_id}             = ( defined $hash_ref->{node_id} )             ? $hash_ref->{node_id}             : "";
      # $conf{$hash_key}{appliance_id}        = ( defined $hash_ref->{appliance_id} )        ? $hash_ref->{appliance_id}        : "";
      # $conf{$hash_key}{bond_id}             = ( defined $hash_ref->{bond_id} )             ? $hash_ref->{bond_id}             : "";
      # $conf{$hash_key}{io_module_id}        = ( defined $hash_ref->{io_module_id} )        ? $hash_ref->{io_module_id}        : "";
      # $conf{$hash_key}{partner_id}          = ( defined $hash_ref->{partner_id} )          ? $hash_ref->{partner_id}          : "";
      # $conf{$hash_key}{sfp_id}              = ( defined $hash_ref->{sfp_id} )              ? $hash_ref->{sfp_id}              : "";
      # $conf{$hash_key}{port_index}          = ( defined $hash_ref->{port_index} )          ? $hash_ref->{port_index}          : "";
      # $conf{$hash_key}{current_mtu}         = ( defined $hash_ref->{current_mtu} )         ? $hash_ref->{current_mtu}         : "";
      # $conf{$hash_key}{current_speed}       = ( defined $hash_ref->{current_speed} )       ? $hash_ref->{current_speed}       : "";
      # $conf{$hash_key}{mac_address}         = ( defined $hash_ref->{mac_address} )         ? $hash_ref->{mac_address}         : "";
      # $conf{$hash_key}{port_connector_type} = ( defined $hash_ref->{port_connector_type} ) ? $hash_ref->{port_connector_type} : "";

      my %arch = (
        "item_id"   => $hash_key,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $name,
        "hw_type"   => "powerstore",
        "subsystem" => "ethport",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );
      push(@architecture, \%arch);
    }
  }

  #
  # fc_port
  #
  if ( exists $data{conf}{fc_port} ) {
    foreach my $id ( sort keys %{ $data{conf}{fc_port} } ) {
      my $hash_ref = $data{conf}{fc_port}{$id};
      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "FCPORT" . $id);
      my $name = $id;
      # my $id                    = ( defined $hash_ref->{id} )                    ? $hash_ref->{id}                    : "";
      # my $name                  = ( defined $hash_ref->{name} )                  ? $hash_ref->{name}                  : "";
      if (defined $hash_ref->{name}) {
        # $conf{$hash_key}{name} = $hash_ref->{name};
        $name = $hash_ref->{name};
      }

      # $conf{$hash_key}{appliance_id}        = ( defined $hash_ref->{appliance_id} )        ? $hash_ref->{appliance_id}        : "";
      # $conf{$hash_key}{node_id}             = ( defined $hash_ref->{node_id} )             ? $hash_ref->{node_id}             : "";
      # $conf{$hash_key}{io_module_id}        = ( defined $hash_ref->{io_module_id} )        ? $hash_ref->{io_module_id}        : "";
      # $conf{$hash_key}{sfp_id}              = ( defined $hash_ref->{sfp_id} )              ? $hash_ref->{sfp_id}              : "";
      # $conf{$hash_key}{port_index}          = ( defined $hash_ref->{port_index} )          ? $hash_ref->{port_index}          : "";
      # $conf{$hash_key}{wwn}                 = ( defined $hash_ref->{wwn} )                 ? $hash_ref->{wwn}                 : "";
      # $conf{$hash_key}{port_connector_type} = ( defined $hash_ref->{port_connector_type} ) ? $hash_ref->{port_connector_type} : "";
      # $conf{$hash_key}{current_speed}       = ( defined $hash_ref->{current_speed} )       ? $hash_ref->{current_speed}       : "";

      my %arch = (
        "item_id"   => $hash_key,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $name,
        "hw_type"   => "powerstore",
        "subsystem" => "fcport",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );
      ## WWN
      if (defined $hash_ref->{wwn} && $hash_ref->{wwn} ne "") {
        my $wwn = Xormon::formatWWN($hash_ref->{wwn});
        $arch{wwn} = $wwn;
        $conf{$hash_key}{wwn} = $wwn;
      }
      push(@architecture, \%arch);
    }
  }

  #
  # node
  #
  if ( exists $data{conf}{node} ) {
    foreach my $id ( sort keys %{ $data{conf}{node} } ) {
      my $hash_ref = $data{conf}{node}{$id};
      my $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "NODE" . $id);

      $conf{$hash_key}{appliance_id} = ( defined $hash_ref->{appliance_id} ) ? $hash_ref->{appliance_id} : "";
      # $conf{$hash_key}{slot}         = ( defined $hash_ref->{slot} )         ? $hash_ref->{slot}         : "";
      my %arch = (
        "item_id"   => $hash_key,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $id,
        "hw_type"   => "powerstore",
        "subsystem" => "node",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );
      if (defined $conf{$hash_key}{appliance_id} && $conf{$hash_key}{appliance_id} ne "") {
        my $app_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "POOL" . $conf{$hash_key}{appliance_id});
        push( @{ $arch{parents} }, $app_key );
      }
      push(@architecture, \%arch);
    }
    $conf{ $cfg->{'hostcfg_id'} }{'node_name'} = join(',', sort keys %{ $data{conf}{node} });
  }

  #
  # volume
  #
  if ( exists $data{conf}{volume} ) {

    foreach my $id ( sort keys %{ $data{conf}{volume} } ) {
      my $hash_ref = $data{conf}{volume}{$id};
      my $name = $id;
      # my $id                    = ( defined $hash_ref->{id} )                    ? $hash_ref->{id}                    : "";
      # my $name                  = ( defined $hash_ref->{name} )                  ? $hash_ref->{name}                  : "";
      if (defined $hash_ref->{name}) {
        # $conf{$id}{name} = $hash_ref->{name};
        $name = $hash_ref->{name};
      }
      ## add lun_id for volumes
      $conf{$id}{lun_id} = $id;
      
      $conf{$id}{appliance_id}          = ( defined $hash_ref->{appliance_id} )          ? $hash_ref->{appliance_id}          : "";
      # $conf{$id}{performance_policy_id} = ( defined $hash_ref->{performance_policy_id} ) ? $hash_ref->{performance_policy_id} : "";
      $conf{$id}{size}                  = ( defined $hash_ref->{size} )                  ? $hash_ref->{size}                  : "";
      # $conf{$id}{state}                 = ( defined $hash_ref->{state} )                 ? $hash_ref->{state}                 : "";
      # $conf{$id}{type}                  = ( defined $hash_ref->{type} )                  ? $hash_ref->{type}                  : "";
      $conf{$id}{wwn}                   = ( defined $hash_ref->{wwn} )                   ? $hash_ref->{wwn}                   : "";

      if ( exists $capacity{volume}{$id}{'thin_savings'}        && Xormon::isdigit( $capacity{volume}{$id}{'thin_savings'} ) )        { $conf{$id}{'thin_savings'} = $capacity{volume}{$id}{'thin_savings'}; }
      if ( exists $capacity{volume}{$id}{'logical_used'}        && Xormon::isdigit( $capacity{volume}{$id}{'logical_used'} ) )        { $conf{$id}{'logical_used'} = $capacity{volume}{$id}{'logical_used'}; }
      if ( exists $capacity{volume}{$id}{'logical_provisioned'} && Xormon::isdigit( $capacity{volume}{$id}{'logical_provisioned'} ) ) { $conf{$id}{'logical_provisioned'} = $capacity{volume}{$id}{'logical_provisioned'}; }

      my %arch = (
        "item_id"   => $id,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $name,
        "hw_type"   => "powerstore",
        "subsystem" => "volume",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );
      if (defined $conf{$id}{appliance_id} && $conf{$id}{appliance_id} ne "") {
        my $app_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . "POOL" . $conf{$id}{appliance_id});
        push( @{ $arch{parents} }, $app_key );
      }
      if (defined $conf{$id}{wwn} && $conf{$id}{wwn} ne "") {
        $arch{wwn} = Xormon::formatWWN($conf{$id}{wwn});
      }
      push(@architecture, \%arch);
    }
  }

  #
  # volume_group
  #
  if ( exists $data{conf}{volume_group} ) {

    foreach my $id ( sort keys %{ $data{conf}{volume_group} } ) {
      my $hash_ref = $data{conf}{volume_group}{$id};

      my $name = $id;
      # my $id                    = ( defined $hash_ref->{id} )                    ? $hash_ref->{id}                    : "";
      # my $name                  = ( defined $hash_ref->{name} )                  ? $hash_ref->{name}                  : "";
      if (defined $hash_ref->{name}) {
        $conf{$id}{name} = $hash_ref->{name};
        $name = $hash_ref->{name};
      }
      # $conf{$id}{type} = ( defined $hash_ref->{type} ) ? $hash_ref->{type} : "";

      my %arch = (
        "item_id"   => $id,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $name,
        "hw_type"   => "powerstore",
        "subsystem" => "volume_group",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );

      if ( defined $hash_ref->{volumes} && ref( $hash_ref->{volumes} ) eq "ARRAY" ) {
        foreach ( @{ $hash_ref->{volumes} } ) {
          if ( ref($_) eq "HASH" ) {
            while ( my ( $key, $val ) = each( %{$_} ) ) {
              if ( defined $key && $key eq "id" && defined $val && $val ne '' ) {
                push( @{ $arch{children} }, $val ); # children VOLUMES
              }
            }
          }
        }
      }
      push(@architecture, \%arch);
    }
  }

  #
  # host
  #
  if ( exists $data{conf}{host} ) {

    foreach my $id ( sort keys %{ $data{conf}{host} } ) {
      my $hash_ref = $data{conf}{host}{$id};

      my $name = $id;
      # my $id                    = ( defined $hash_ref->{id} )                    ? $hash_ref->{id}                    : "";
      # my $name                  = ( defined $hash_ref->{name} )                  ? $hash_ref->{name}                  : "";
      if (defined $hash_ref->{name}) {
        $conf{$id}{name} = $hash_ref->{name};
        $name = $hash_ref->{name};
      }
      # $conf{$id}{os_type} = ( defined $hash_ref->{os_type} ) ? $hash_ref->{os_type} : "";

      my %arch = (
        "item_id"   => $id,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $name,
        "hw_type"   => "powerstore",
        "subsystem" => "host",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );

      my $host_group_id = ( defined $hash_ref->{host_group_id} ) ? $hash_ref->{host_group_id} : "";

      #"host_virtual_volume_mappings" : [],
      #"mapped_hosts" : [],

      my %vols_act;
      my $vol_idx = 0;
      if ( exists $host_mapping{HOST}{$id} ) {
        foreach ( keys %{ $host_mapping{HOST}{$id} } ) {
          $vols_act{$_} = $_;
          $vol_idx++;
        }
      }
      if ( defined $host_group_id && $host_group_id ne '' ) {
        $conf{$id}{host_group_id} = $host_group_id;
        if ( exists $host_mapping{HOST_GROUP}{$host_group_id} ) {
          foreach ( keys %{ $host_mapping{HOST_GROUP}{$host_group_id} } ) {
            $vols_act{$_} = $_;
            $vol_idx++;
          }
        }
      }

      if ( $vol_idx > 0 ) {
        $arch{children} = [ keys %vols_act ]; # Add 'children' key with an array of volumes
      }
      push(@architecture, \%arch);
    }
  }

  #
  # host_group
  #
  if ( exists $data{conf}{host_group} ) {

    foreach my $id ( sort keys %{ $data{conf}{host_group} } ) {
      my $hash_ref = $data{conf}{host_group}{$id};

      my $name = $id;
      # my $id                    = ( defined $hash_ref->{id} )                    ? $hash_ref->{id}                    : "";
      # my $name                  = ( defined $hash_ref->{name} )                  ? $hash_ref->{name}                  : "";
      if (defined $hash_ref->{name}) {
        $conf{$id}{name} = $hash_ref->{name};
        $name = $hash_ref->{name};
      }

      my %arch = (
        "item_id"   => $id,
        "hostcfg_id" => $cfg->{'hostcfg_id'},
        "label"     => $name,
        "hw_type"   => "powerstore",
        "subsystem" => "host_group",
        "parents"   => [$cfg->{'hostcfg_id'}]
      );

      if ( exists $host_mapping{HOST_GROUP}{$id} ) {
        $arch{children} = [ keys %{ $host_mapping{HOST_GROUP}{$id} } ]
      }
      push(@architecture, \%arch);
    }
  }


  if ( $debug == 2 ) {
    my $debug_conf = \%conf;
    eval { Xormon::save_as_json("$wrkdir/last_conf.json", $debug_conf); };

    my $debug_arch = \@architecture;
    eval { Xormon::save_as_json("$wrkdir/arch.json", $debug_arch); };
  }

  # #save architecture
  # #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

  # #print Dumper \%conf;
  # #save configuration
  $xormon->saveConfJSON(\%conf);


  return 1;
}

sub prepare_perf {
  my $date_time = shift;
  my %data_out;

  my $timestamp = str2time_ntz($date_time);

  # go through each section and print data to perf file if exists
  foreach my $type ( @{ $url_paths->{perf} } ) {
    if ( exists $data{perf}{$date_time}{ $type->{path} } ) {
      my $subsystem;
      if ($type->{path} eq "eth_port") {
        $subsystem = "ethport";
      }
      elsif ($type->{path} eq "fc_port") {
        $subsystem = "fcport";
      }
      ## save appliance as pool
      elsif ($type->{path} eq "appliance") {
        $subsystem = "pool";
      }
      else {
        $subsystem = $type->{path};
      }

      foreach my $id ( sort keys %{ $data{perf}{$date_time}{ $type->{path} } } ) {

        # device hostcfg_id
        # appliance hash
        # cluster hash
        # node hash
        # FCPORT hash
        # ETHPORT hash
        # volume UUID
        # volume-group UUID

        # host uuid
        # host group uuid

        my $hash_key;

        if ($type->{path} eq "volume" || $type->{path} eq "volume_group" ) {
          $hash_key = $id;
        }
        else {
          $hash_key = Xormon::create_hash_key($cfg->{'hostcfg_id'} . uc($subsystem) . $id);
        }

        if ( exists $type->{entity} ) {
          foreach my $entity_hash ( @{ $type->{entity} } ) {
            if ( ref($entity_hash) eq "HASH" ) {
              while ( my ( $entity_key, $entity_val ) = each( %{$entity_hash} ) ) {
                if ( ref($entity_val) eq "ARRAY" ) {
                  foreach my $metric_hash ( @{$entity_val} ) {
                    while ( my ( $metric_key, $metric_label ) = each( %{$metric_hash} ) ) {
                      if ( exists $data{perf}{$date_time}{ $type->{path} }{$id}{$entity_key}{$metric_key} ) {
                        my $value = $data{perf}{$date_time}{ $type->{path} }{$id}{$entity_key}{$metric_key};
                        # Convert from us to ms for resp_t
                        if ($metric_label =~ /^(resp_t|resp_t_r|resp_t_w)$/ && Xormon::isdigit($value)) {
                          $value = $value / 1000;
                        }
                        $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{$metric_label} = $value;
                      }
                    }
                  }
                }
              }
            }
          }
        }
        ## VOLUME CAPACITY FROM CONF
        if ($subsystem eq "volume") {
          if ( exists $data{conf}{volume}{$id}{'size'} && Xormon::isdigit($data{conf}{volume}{$id}{'size'})) {
            $data_out{powerstore}{volume}{$hash_key}{$timestamp}{'capacity_total'} = $data{conf}{volume}{$id}{'size'};
          }
        }
        ## BLOCK SIZE FOR VOLUME AND APPLIANCE(POOL)
        if ($subsystem eq "volume" || $subsystem eq "pool") {
          if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'io_rate'}) && Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'data_rate'}) ) {
            my $io   = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'io_rate'};
            my $data = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'data_rate'};
            if ($io != 0) {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'block_size'} = Xormon::formatNumberByType("float4", ($data / $io));
            } else {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'block_size'} = 0;
            }
          }
          if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read_io'}) && Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read'}) ) {
            my $io   = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read_io'};
            my $data = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read'};
            if ($io != 0) {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'block_size_r'} = Xormon::formatNumberByType("float4", ($data / $io));
            } else {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'block_size_r'} = 0;
            }
          }
          if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write_io'}) && Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write'}) ) {
            my $io   = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write_io'};
            my $data = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write'};
            if ($io != 0) {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'block_size_w'} = Xormon::formatNumberByType("float4", ($data / $io));
            } else {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'block_size_w'} = 0;
            }
          }
        }

        ## ETHPORT TOTALS for packet and data
        if ($subsystem eq "ethport") {
          if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read'}) ) {
            $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'data_rate'} = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read'};
          }
          if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write'}) ) {
            if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'data_rate'}) ) {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'data_rate'} += $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write'};
            }
            else {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'data_rate'} = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write'};
            }
          }
          if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read_packet'}) ) {
            $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'packet_rate'} = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'read_packet'};
          }
          if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write_packet'}) ) {
            if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'packet_rate'}) ) {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'packet_rate'} += $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write_packet'};
            }
            else {
              $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'packet_rate'} = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'write_packet'};
            }
          }
        }

        ## CAPACITY FREE  AND PERCENT CALCULATE
        if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_total'}) && Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_used'}) ) {
          my $total = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_total'};
          my $used  = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_used'};
          $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_free'} = $total - $used;
          if ($total != 0) {
            $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_used_percent'} = Xormon::formatNumberByType("float4", $used / ($total / 100) );
          }
        }
        if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_total'}) && Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_used'}) ) {
          my $total = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_total'};
          my $used  = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_used'};
          $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_free'} = $total - $used;
        }

        ## oversubscr calculation
        if ( Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_total'}) && Xormon::isdigit($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_total'}) ) {
          $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_oversubscr'} = $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_total'} - $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_total'};
          if ($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_oversubscr'} < 0){$data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_oversubscr'} = 0;}
          if ($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_total'} != 0) {
            $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_oversubscr_percent'} = Xormon::formatNumberByType("float4", ($data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'virtual_capacity_total'} / $data_out{powerstore}{$subsystem}{$hash_key}{$timestamp}{'capacity_total'}) * 100);
          }
        }
      } # foreach id
    }
  }

  ## HOST and HOST_GROUP SECTION
  my @host_metrics = ('read_io', 'write_io', 'io_rate', 'read', 'write', 'data_rate', 'resp_t', 'resp_t_r', 'resp_t_w');
  # host
  if ( exists $data{conf}{host} ) {
    foreach my $id ( sort keys %{ $data{conf}{host} } ) {
      my $hash_ref = $data{conf}{host}{$id};
      my $host_group_id = ( defined $hash_ref->{host_group_id} ) ? $hash_ref->{host_group_id} : "";
      my %vols_act;
      my $vol_idx = 0;
      if ( exists $host_mapping{HOST}{$id} ) {
        foreach ( keys %{ $host_mapping{HOST}{$id} } ) {
          $vols_act{$_} = $_;
          $vol_idx++;
        }
      }
      if ( defined $host_group_id && $host_group_id ne '' ) {
        if ( exists $host_mapping{HOST_GROUP}{$host_group_id} ) {
          foreach ( keys %{ $host_mapping{HOST_GROUP}{$host_group_id} } ) {
            $vols_act{$_} = $_;
            $vol_idx++;
          }
        }
      }
      if ( $vol_idx > 0 ) {
        # $arch{children} = [ keys %vols_act ]; # Add 'children' key with an array of volumes
        foreach my $volume_id (keys %vols_act) {
          foreach my $host_metric (@host_metrics) {
            if ( exists $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$host_metric} && Xormon::isdigit($data_out{powerstore}{volume}{$volume_id}{$timestamp}{$host_metric}) ) {
              my $host_value = $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$host_metric};
              if ($host_metric eq 'resp_t' || $host_metric eq 'resp_t_r' || $host_metric eq 'resp_t_w') {
                my $io_metric;
                if ($host_metric eq 'resp_t') {
                  $io_metric = 'io_rate';
                }
                elsif ($host_metric eq 'resp_t_r') {
                  $io_metric = 'read_io';
                }
                elsif ($host_metric eq 'resp_t_w') {
                  $io_metric = 'write_io';
                }
                if (exists $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$io_metric} && Xormon::isdigit($data_out{powerstore}{volume}{$volume_id}{$timestamp}{$io_metric})) {
                  my $io = $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$io_metric};
                  $data_out{powerstore}{host}{$id}{$timestamp}{$host_metric} += Xormon::isdigit($host_value) ? ($host_value * $io) : 0;
                }
              } else {
                $data_out{powerstore}{host}{$id}{$timestamp}{$host_metric} += Xormon::isdigit($host_value) ? $host_value : 0;
              }
            }
          }
        }
        if (Xormon::isdigit($data_out{powerstore}{host}{$id}{$timestamp}{'resp_t'}) && Xormon::isdigit($data_out{powerstore}{host}{$id}{$timestamp}{'io_rate'}) && $data_out{powerstore}{host}{$id}{$timestamp}{'io_rate'} !=0){
          $data_out{powerstore}{host}{$id}{$timestamp}{'resp_t'} = Xormon::formatNumberByType("float4", ( $data_out{powerstore}{host}{$id}{$timestamp}{'resp_t'} / $data_out{powerstore}{host}{$id}{$timestamp}{'io_rate'} ) );
        }
        if (Xormon::isdigit($data_out{powerstore}{host}{$id}{$timestamp}{'resp_t_r'}) && Xormon::isdigit($data_out{powerstore}{host}{$id}{$timestamp}{'read_io'}) && $data_out{powerstore}{host}{$id}{$timestamp}{'read_io'} !=0){
          $data_out{powerstore}{host}{$id}{$timestamp}{'resp_t_r'} = Xormon::formatNumberByType("float4", ( $data_out{powerstore}{host}{$id}{$timestamp}{'resp_t_r'} / $data_out{powerstore}{host}{$id}{$timestamp}{'read_io'} ) );
        }
        if (Xormon::isdigit($data_out{powerstore}{host}{$id}{$timestamp}{'resp_t_w'}) && Xormon::isdigit($data_out{powerstore}{host}{$id}{$timestamp}{'write_io'}) && $data_out{powerstore}{host}{$id}{$timestamp}{'write_io'} !=0){
          $data_out{powerstore}{host}{$id}{$timestamp}{'resp_t_w'} = Xormon::formatNumberByType("float4", ( $data_out{powerstore}{host}{$id}{$timestamp}{'resp_t_w'} / $data_out{powerstore}{host}{$id}{$timestamp}{'write_io'} ) );
        }
      }
    }
  }
  # host_group
  if ( exists $data{conf}{host_group} ) {
    foreach my $id ( sort keys %{ $data{conf}{host_group} } ) {
      if ( exists $host_mapping{HOST_GROUP}{$id} ) {
        foreach my $volume_id (keys %{ $host_mapping{HOST_GROUP}{$id} }) {
          foreach my $host_metric (@host_metrics) {
            if ( exists $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$host_metric} && Xormon::isdigit($data_out{powerstore}{volume}{$volume_id}{$timestamp}{$host_metric}) ) {
              my $host_value = $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$host_metric};
              if ($host_metric eq 'resp_t' || $host_metric eq 'resp_t_r' || $host_metric eq 'resp_t_w') {
                my $io_metric;
                if ($host_metric eq 'resp_t') {
                  $io_metric = 'io_rate';
                }
                elsif ($host_metric eq 'resp_t_r') {
                  $io_metric = 'read_io';
                }
                elsif ($host_metric eq 'resp_t_w') {
                  $io_metric = 'write_io';
                }
                if (exists $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$io_metric} && Xormon::isdigit($data_out{powerstore}{volume}{$volume_id}{$timestamp}{$io_metric})) {
                  my $io = $data_out{powerstore}{volume}{$volume_id}{$timestamp}{$io_metric};
                  $data_out{powerstore}{host_group}{$id}{$timestamp}{$host_metric} += Xormon::isdigit($host_value) ? ($host_value * $io) : 0;
                }
              } else {
                $data_out{powerstore}{host_group}{$id}{$timestamp}{$host_metric} += Xormon::isdigit($host_value) ? $host_value : 0;
              }
            }
          }
        }
        if (Xormon::isdigit($data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t'}) && Xormon::isdigit($data_out{powerstore}{host_group}{$id}{$timestamp}{'io_rate'}) && $data_out{powerstore}{host_group}{$id}{$timestamp}{'io_rate'} !=0){
          $data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t'} = Xormon::formatNumberByType("float4", ( $data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t'} / $data_out{powerstore}{host_group}{$id}{$timestamp}{'io_rate'} ) );
        }
        if (Xormon::isdigit($data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t_r'}) && Xormon::isdigit($data_out{powerstore}{host_group}{$id}{$timestamp}{'read_io'}) && $data_out{powerstore}{host_group}{$id}{$timestamp}{'read_io'} !=0){
          $data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t_r'} = Xormon::formatNumberByType("float4", ( $data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t_r'} / $data_out{powerstore}{host_group}{$id}{$timestamp}{'read_io'} ) );
        }
        if (Xormon::isdigit($data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t_w'}) && Xormon::isdigit($data_out{powerstore}{host_group}{$id}{$timestamp}{'write_io'}) && $data_out{powerstore}{host_group}{$id}{$timestamp}{'write_io'} !=0){
          $data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t_w'} = Xormon::formatNumberByType("float4", ( $data_out{powerstore}{host_group}{$id}{$timestamp}{'resp_t_w'} / $data_out{powerstore}{host_group}{$id}{$timestamp}{'write_io'} ) );
        }
      }
    } # foreach id host_group
  }

  ## save appliance stats also for device stats
  if (exists $data_out{powerstore}{pool}) {
    foreach my $app (keys %{ $data_out{powerstore}{pool}} ) {
      foreach my $statistic (keys %{$data_out{powerstore}{pool}{$app}{$timestamp}}) {
        $data_out{powerstore}{device}{$cfg->{'hostcfg_id'}}{$timestamp}{$statistic} = $data_out{powerstore}{pool}{$app}{$timestamp}{$statistic};
      }
    }
  }

  return \%data_out;
}


sub print_perf {
  # find last timestamp which has the most stats available
  # e.g.:
  # 1664866200 = 15 IDs
  # 1664866500 = 30 IDs
  # 1664866800 =  2 IDs
  # --> create last perf file for timestamp 1664866500
  my $status;
  $cfg->{'eunix'} = set_eunix();
  my $sunix = $cfg->{'sunix'};
  my $last_perf = "";

  if ( exists $data{perf} ) {
    foreach my $date_time ( sort keys %{ $data{perf} } ) {
      my $timestamp = str2time_ntz($date_time);
      if ( !defined $timestamp ) { next; }

      #print "$date_time : $timestamp : ".localtime($timestamp)." : ".gmtime($timestamp)."\n";

      # test if timestamp is newer than last created perf file, if not then skip this interval
      if ( defined $sunix && Xormon::isdigit($sunix) && $sunix >= $timestamp ) { next; }

      # exclude last stats, they have not to be complete
      #if ( $timestamp > ( $start_time - $sample_rate ) ) { next; }
      if ( $timestamp > ( $cfg->{'start_time'} - ( 2 * $cfg->{'sample_rate'} ) ) ) { next; }

      # exclude stats where are not all metrics available, create perf file only for timestamp where are the stats available for the most of the items
      if ( defined $cfg->{'eunix'} && $timestamp > $cfg->{'eunix'} ) { next; }

      my $perf_out = prepare_perf($date_time);
      my $rc       = $xormon->saveJSON2($perf_out);
      $logger->info("save perf: $rc : ".localtime($timestamp)." : $timestamp");
      if ( $rc eq "201" ) {
        $status = "successful";
      }
      elsif ( $rc eq "401" ) {
        $status = "unauthorized";
        last;
      }
      else {
        $status = "error";
        last;
      }

      if ( defined $timestamp && $timestamp ne '' ) {
        $service_data_out->{last_perf_timestamp} = $timestamp;
        $last_perf = $timestamp;
        if ($debug == 2) {
          eval { Xormon::save_as_json("$wrkdir/perf_$timestamp.json", $perf_out); };
        }
      }
    }
  }
  if ( Xormon::isdigit($last_perf) ) {
    # check if there is current data
    # if ( ( time() - $last_perf ) > 10800 ) {
    #   $xormon->hasData(0, "last available stats are older than 3 hours: $last_perf");
    # }
    # else {
      $xormon->hasData(1);
    # }
  }
  else {
    $xormon->hasData(0, "no available stats");
  }

  return 1;
}

sub set_eunix {
  # find last timestamp which has the most stats available
  # e.g.:
  # 1664866200 = 15 IDs
  # 1664866500 = 30 IDs
  # 1664866800 =  2 IDs
  # --> create last perf file for timestamp 1664866500
  my %timestamps;
  my $highest_num      = "";
  my $timestamps_count = scalar( keys %last_stats );
  my $timestamp_idx    = 0;
  my $current_time     = time();

  $logger->info("set_eunix: count of timestamps=$timestamps_count") if $debug;
  foreach my $timestamp_act ( sort { $a <=> $b } keys %last_stats ) {
    $timestamp_idx++;
    my $count = scalar( keys %{ $last_stats{$timestamp_act} } );
    # KZ: update Jan 16 2024 --> the following didn't work as it was expected
    # if ( $timestamps_count > 36 && $timestamp_idx < 18 ) {
    #   # update March 14
    #   # do not use eunix when there is more timestamps than 3 hours back
    #   # in some cases there was used timestamp which was one day back, because this timestamp had the most IDs...
    #   # e.g.:
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678626000 has 281 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678626300 has 760 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678626600 has 210 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678626900 has 1 IDs
    #   # ...
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678711500 has 3 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678711800 has 3 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678712100 has 113 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678712400 has 666 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: timestamp=1678712700 has 480 IDs
    #   # Mon Mar 13 06:09:15 2023: set_eunix: eunix=1678626300 : Sun Mar 12 06:05:00 2023

    #   $logger->info("set_eunix: timestamp=$timestamp_act has $count IDs : timestamp skipped, there are more than 36 timestamps \($timestamps_count\)") if $debug;
    # }
    if ( ( $current_time - $timestamp_act ) > (60 * 60 * 3) ) { # max 3 hours back
      $logger->info("set_eunix: timestamp=$timestamp_act sample skipped, older than 3 hours \($count IDs\)") if $debug;
      next;
    }
    else {
      $logger->info("set_eunix: timestamp=$timestamp_act has $count IDs") if $debug;
      $timestamps{$count} = $timestamp_act;
      if ( defined $highest_num && $highest_num ne '' ) {
        if ( $highest_num < $count ) {
          $highest_num = $count;
        }
      }
      else {
        $highest_num = $count;
      }
    }
  }

  if ( defined $highest_num && $highest_num ne '' ) {
    $logger->info("set_eunix: eunix=$timestamps{$highest_num} : ".localtime($timestamps{$highest_num})) if $debug;
    return $timestamps{$highest_num};
  }
  else {
    return undef;
  }
}

sub collect_data {
  my $type = shift;               # perf/conf

  foreach my $hash_ref ( @{ $url_paths->{$type} } ) {
    my $select = "";
    if ( exists $hash_ref->{select} && ref( $hash_ref->{select} ) eq "ARRAY" ) {
      $select = "?select=" . join( ",", @{ $hash_ref->{select} } );

      if ( exists $hash_ref->{params} && ref( $hash_ref->{params} ) eq "ARRAY" ) {
        $select .= "&" . join( "&", @{ $hash_ref->{params} } );
      }
    }
    else {
      if ( exists $hash_ref->{params} && ref( $hash_ref->{params} ) eq "ARRAY" ) {
        $select = "?" . join( "&", @{ $hash_ref->{params} } );
      }
    }

    my $conf_res = new_get_request("$cfg->{'url'}/$hash_ref->{path}$select");

    if ( defined $conf_res && ref($conf_res) eq "ARRAY" ) {
      foreach my $conf_val ( @{$conf_res} ) {
        if ( exists $conf_val->{id} ) {

          # save conf
          $data{conf}{ $hash_ref->{path} }{ $conf_val->{id} } = $conf_val;

          # collect perf
          if ( exists $hash_ref->{entity} ) {
            foreach my $entity_hash ( @{ $hash_ref->{entity} } ) {
              if ( ref($entity_hash) eq "HASH" ) {
                while ( my ( $entity_key, undef ) = each( %{$entity_hash} ) ) {
                  my $perf_act = new_post_request( "$cfg->{'url'}/metrics/generate", { entity => $entity_key, "entity_id" => $conf_val->{id}, interval => "Five_Mins" } );
                  parse_perf( $conf_val->{id}, $hash_ref->{path}, $entity_key, $perf_act );
                }
              }
            }
          }
        }
      }
    }
  }

  return 1;
}

sub new_post_request {
  my $url_act = shift;
  my $body    = shift;
  my $api_url = $cfg->{'url'};

  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( POST => $url_act );
  $request->header(
    'Content-Type' => 'application/json',
    'Accept'       => 'application/json'
  );
  $request->authorization_basic( $cfg->{apiUsername}, $cfg->{apiPassword} );
  if ( defined $cfg->{token} && $cfg->{token} ne '' ) {
    $request->header( 'DELL-EMC-TOKEN' => $cfg->{token} );
  }

  #if ( defined $cookie && $cookie ne '' ) {
  #  $request->header( 'Authorization' => $cookie );
  #}
  #else {
  #  $request->authorization_basic( $user, $pass );
  #}

  # message("POST url : $url_act") if $debug;
  my $json_body;
  if ( defined $body && $body ne '' ) {
    $json_body = encode_json($body);
    $request->content($json_body);
    # message("JSON body: $json_body") if $debug;
  }

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    my $res_con = $results->content;
    $logger->error("POST url     : $url_act");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");
    $logger->error("JSON body    : $json_body");

    return 0;
  }

  if ( exists $results->headers->{'dell-emc-token'} ) {
    $cfg->{token} = $results->headers->{'dell-emc-token'};
  }
  if ( exists $results->headers->{'set-cookie'} ) {
    $cfg->{cookie} = $results->headers->{'set-cookie'};

    #$cookie =~ s/^auth_cookie=//;
    #$cookie =~ s/;.*//g;
  }

  my $result;
  eval { $result = decode_json( $results->content ); };

  if ( $debug == 2 ) {
    my $output_file = $url_act;
    $output_file =~ s/^$api_url\///;
    $output_file =~ s/\//_/g;
    $output_file =~ s/\?select=.*//g;

    if ( exists $body->{'entity'} )    { $output_file .= "_$body->{'entity'}"; }
    if ( exists $body->{'entity_id'} ) { $output_file .= "_$body->{'entity_id'}"; }

    eval { Xormon::save_as_json( "$wrkdir/$output_file.json", $result ); };
  }

  return $result;
}


sub new_get_request {
  my $url_act = shift;
  my $api_url = $cfg->{'url'};

  my $agent = LWP::UserAgent->new( ssl_opts => { SSL_cipher_list => 'DEFAULT:!DH', verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( GET => $url_act );
  $request->header(
    'Content-Type' => 'application/json',
    'Accept'       => 'application/json'
  );
  $request->authorization_basic( $cfg->{apiUsername}, $cfg->{apiPassword} );

  #if ( defined $cookie && $cookie ne '' ) {
  #  $request->header( 'Authorization' => $cookie );
  #}
  #else {
  #  $request->authorization_basic( $user, $pass );
  #}

  $logger->info("GET url  : $url_act") if $debug;

  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    my $res_con = $results->content;
    $logger->error("GET url      : $url_act");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    return 0;
  }

  if ( exists $results->headers->{'dell-emc-token'} ) {
    $cfg->{token} = $results->headers->{'dell-emc-token'};
  }
  if ( exists $results->headers->{'set-cookie'} ) {
    $cfg->{cookie} = $results->headers->{'set-cookie'};

    #$cookie =~ s/^auth_cookie=//;
    #$cookie =~ s/;.*//g;
  }
  my $result;
  eval { $result = decode_json( $results->content ); };

  if ( $debug == 2 ) {
    my $output_file = $url_act;
    $output_file =~ s/^$api_url\///;
    $output_file =~ s/\//_/g;
    $output_file =~ s/\?select=.*//g;

    eval { Xormon::save_as_json( "$wrkdir/$output_file.json", $result ); };
  }

  return $result;
}

sub parse_perf {
  my $id     = shift;
  my $type   = shift;
  my $entity = shift;
  my $perf   = shift;

  #print Dumper $ref;

  my $last_date_time = "";
  if ( defined $perf && ref($perf) eq "ARRAY" ) {
    foreach my $hash_ref ( @{$perf} ) {
      if ( exists $hash_ref->{timestamp} ) {
        $data{perf}{ $hash_ref->{timestamp} }{$type}{$id}{$entity} = $hash_ref;
        $last_date_time = $hash_ref->{timestamp};
        $stat_found = 1;
      }
    }
  }
  if ( defined $last_date_time && $last_date_time ne '' ) {
    my $timestamp = str2time_ntz($last_date_time);

    if ( defined $timestamp ) {
      $last_stats{$timestamp}{$id} = $id;
      $logger->info("parse_perf: id=$id, type=$type, entity=$entity, timestamp=$timestamp") if $debug eq "3";
    }
  }

  return 1;
}

sub str2time_ntz {
  # Convert date and time without TZ (in Zulu/UTC/GMT)
  my $date_time = shift;

  # must be in this format 2021-08-29T07:20:00Z, print error if not for further debugging
  if ( $date_time !~ m/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]T[0-9][0-9]:[0-9][0-9]:[0-9][0-9]Z$/ ) {
    $logger->error("date-time \"$date_time\" is not in proper format!") && return undef;
  }

  return str2time($date_time);    # NOTE: when date_time is not in proper format, then str2time returns undef value
}

sub health_status {
  my $main_state     = "";
  my @error_message = ();
  my %hs_data;

  #
  # check hardware statuses
  #
  my $hw_status = "ok";
  if ( exists $data{conf}{hardware} ) {
    $main_state     = "ok";
    foreach my $id ( sort keys %{ $data{conf}{hardware} } ) {
      my $hash_ref = $data{conf}{hardware}{$id};

      my $name            = ( defined $hash_ref->{name} )            ? $hash_ref->{name}            : "";
      my $type            = ( defined $hash_ref->{type} )            ? $hash_ref->{type}            : "";
      my $slot            = ( defined $hash_ref->{slot} )            ? $hash_ref->{slot}            : "";
      my $serial_number   = ( defined $hash_ref->{serial_number} )   ? $hash_ref->{serial_number}   : "";
      my $lifecycle_state = ( defined $hash_ref->{lifecycle_state} ) ? $hash_ref->{lifecycle_state} : "";

      if ( $lifecycle_state eq "Failed" ) {
        $main_state   = "error";
        $hw_status    = "NOT OK";

        my $error_line = "TYPE $type slot: $slot state: $lifecycle_state";
        push( @error_message, "$error_line\n" );
      }

      my %hw;
      $hw{id}     = $id;
      $hw{name}   = $name;
      $hw{type}   = $type;
      $hw{slot}   = $slot;
      $hw{serial} = $serial_number;
      $hw{state}  = $lifecycle_state;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{hw} }, \%hw );

    }
  }

  #
  # check device alerts
  #
  my $alerts_status = "ok";
  if ( exists $data{conf}{alert} ) {
    foreach my $id ( sort keys %{ $data{conf}{alert} } ) {
      my $hash_ref = $data{conf}{alert}{$id};

      my $severity               = ( defined $hash_ref->{severity} )               ? $hash_ref->{severity}               : "";
      my $state                  = ( defined $hash_ref->{state} )                  ? $hash_ref->{state}                  : "";
      my $acknowledged_timestamp = ( defined $hash_ref->{acknowledged_timestamp} ) ? $hash_ref->{acknowledged_timestamp} : "";
      my $description_l10n       = ( defined $hash_ref->{description_l10n} )       ? $hash_ref->{description_l10n}       : "";
      my $event_code             = ( defined $hash_ref->{event_code} )             ? $hash_ref->{event_code}             : "";
      my $resource_name          = ( defined $hash_ref->{resource_name} )          ? $hash_ref->{resource_name}          : "";
      my $resource_type          = ( defined $hash_ref->{resource_type} )          ? $hash_ref->{resource_type}          : "";
      my $events                 = ( defined $hash_ref->{events} )                 ? $hash_ref->{events}                 : "";

      if ( $state eq "ACTIVE" && ( $severity eq "Critical" || $severity eq "Major" ) ) {
        $main_state    = "error";
        $alerts_status = "NOT OK";
        my $error_line = "ALERT $severity description: $description_l10n";
        push( @error_message, "$error_line\n" );
      }

      my %alert;
      $alert{id}                     = $id;
      $alert{state}                  = $state;
      $alert{severity}               = $severity;
      $alert{acknowledged_timestamp} = $acknowledged_timestamp;
      $alert{description_l10n}       = $description_l10n;
      $alert{event_code}             = $event_code;
      $alert{resource_name}          = $resource_name;
      $alert{resource_type}          = $resource_type;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{alerts} }, \%alert );
    }
  }

  if ($debug == 2) {
    eval { Xormon::save_as_json( "$wrkdir/hs.json", \%hs_data ); };
  }


  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }

  my @hs = (
    {
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $main_state,
      "data" => join("\n", @error_message),
    }
  );

  if ($debug == 2) {
    eval { Xormon::save_as_json( "$wrkdir/hs_state.json", \@hs ); };
  }

  my $rc = $xormon->saveStatus(\@hs);
  $logger->info("save health status: $rc");
  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);
  $logger->info("save health status table: $rc_hs");

  return 1;
}

sub set_url_paths {
  return {
    conf => [
      { path   => 'host',
        params => [ 'limit=2000' ],
        select => [ 'host_group_id', 'host_virtual_volume_mappings', 'id', 'mapped_hosts', 'name', 'os_type' ]
      },
      # { path   => 'host_virtual_volume_mapping',
      #   params => [ 'limit=2000' ],
      #   select => [ 'host', 'host_group', 'host_group_id', 'host_id', 'id', 'virtual_volume', 'virtual_volume_id' ]
      # },
      { path   => 'host_volume_mapping',
        params => [ 'limit=2000' ],
        select => [ 'host', 'host_group', 'host_group_id', 'host_id', 'id', 'logical_unit_number', 'volume', 'volume_id' ]
      },
      { path   => 'host_group',
        params => [ 'limit=2000' ],
        select => [ 'description', 'host_virtual_volume_mappings', 'hosts', 'id', 'mapped_host_groups', 'name' ]
      },
    ],
    health_status => [
      { path   => 'alert',
        select => [ 'acknowledged_timestamp', 'called_home_timestamp', 'cleared_timestamp', 'description_l10n', 'email_sent_timestamp', 'event_code', 'events', 'generated_timestamp', 'id', 'is_acknowledged', 'raised_timestamp', 'resource_id', 'resource_name', 'resource_type', 'resource_type_l10n', 'severity', 'severity_l10n', 'state', 'state_l10n' ]
      },
      { path   => 'hardware',
        select => [ 'appliance', 'appliance_id', 'children', 'extra_details', 'id', 'io_module_eth_ports', 'io_module_fc_ports', 'io_module_sas_ports', 'is_marked', 'lifecycle_state', 'lifecycle_state_l10n', 'name', 'node_eth_ports', 'node_fc_ports', 'node_sas_ports', 'parent', 'parent_id', 'part_number', 'serial_number', 'sfp_eth_ports', 'sfp_fc_ports', 'sfp_sas_ports', 'slot', 'status_led_state', 'status_led_state_l10n', 'type', 'type_l10n' ]
      },
    ],
    perf => [
      { path   => 'appliance',
        select => [ 'eth_ports', 'express_service_code', 'fc_ports', 'hardware', 'id', 'ip_pool_addresses', 'maintenance_windows', 'model', 'name', 'nodes', 'sas_ports', 'service_tag', 'software_installed', 'veth_ports', 'virtual_volumes', 'volumes' ],
        entity => [
          { 'performance_metrics_by_appliance' => [
              { "appliance_id"                    => "appliance_id" },
              { "avg_read_iops"                   => "read_io" },
              { "avg_write_iops"                  => "write_io" },
              { "avg_total_iops"                  => "io_rate" },
              { "avg_read_bandwidth"              => "read" },
              { "avg_write_bandwidth"             => "write" },
              { "avg_total_bandwidth"             => "data_rate" },
              { "avg_latency"                     => "resp_t" },
              { "avg_read_latency"                => "resp_t_r" },
              { "avg_write_latency"               => "resp_t_w" },
              # { "avg_io_workload_cpu_utilization" => "io_workload_cpu_utilization" },
              # { "avg_io_size"                     => "io_size" },
              # { "avg_read_size"                   => "read_size" },
              # { "avg_write_size"                  => "write_size" },
            ]
          },
          { 'space_metrics_by_appliance' => [
              { "physical_total"                  => "capacity_total" },
              { "physical_used"                   => "capacity_used" },
              { "logical_provisioned"             => "virtual_capacity_total" },
              { "logical_used"                    => "virtual_capacity_used" },
              { "shared_logical_used"             => "shared_logical_used" },
              { "data_reduction"                  => "data_reduction" },
              { "data_physical_used"              => "data_physical_used" },
              { "efficiency_ratio"                => "efficiency_ratio" },
              { "snapshot_savings"                => "snapshot_savings" },
              { "thin_savings"                    => "thin_savings" },
              # { "logical_used_file_system"        => "logical_used_file_system" },
              # { "logical_used_volume"             => "logical_used_volume" },
              # { "logical_used_vvol"               => "logical_used_vvol" },
              # { "shared_logical_used_file_system" => "shared_logical_used_file_system" },
              # { "shared_logical_used_volume"      => "shared_logical_used_volume" },
              # { "shared_logical_used_vvol"        => "shared_logical_used_vvol" },
            ]
          },
        ]
      },
      { path   => 'cluster',
        select => [ 'appliance_count', 'global_id', 'id', 'is_encryption_enabled', 'management_address', 'master_appliance_id', 'name', 'physical_mtu', 'state', 'state_l10n' ],
        params => [ 'limit=2000' ],
        entity => [
          { 'performance_metrics_by_cluster' => [
              { "cluster_id"          => "cluster_id" },
              { "avg_read_iops"       => "read_io" },
              { "avg_write_iops"      => "write_io" },
              { "avg_total_iops"      => "io_rate" },
              { "avg_read_bandwidth"  => "read" },
              { "avg_write_bandwidth" => "write" },
              { "avg_total_bandwidth" => "data_rate" },
              { "avg_read_latency"    => "resp_t_r" },
              { "avg_write_latency"   => "resp_t_w" },
              { "avg_latency"         => "resp_t" },
              # { "avg_io_size"         => "io_size" },
              # { "avg_read_size"       => "read_size" },
              # { "avg_write_size"      => "write_size" },
            ]
          },
          { 'space_metrics_by_cluster' => [
              { "physical_total"      => "capacity_total" },
              { "physical_used"       => "capacity_used" },
              { "logical_provisioned" => "virtual_capacity_total" },
              { "logical_used"        => "virtual_capacity_used" },
              { "shared_logical_used" => "shared_logical_used" },
              { "data_reduction"      => "data_reduction" },
              { "data_physical_used"  => "data_physical_used" },
              { "efficiency_ratio"    => "efficiency_ratio" },
              { "snapshot_savings"    => "snapshot_savings" },
              { "thin_savings"        => "thin_savings" },
            ]
          },
        ]
      },
      { path   => 'eth_port',
        select => [ 'appliance_id', 'bond_id', 'current_mtu', 'current_speed', 'id', 'io_module_id', 'mac_address', 'name', 'node_id', 'partner_id', 'port_connector_type', 'port_index', 'sfp_id' ],
        params => [ 'limit=2000' ],
        entity => [
          { 'performance_metrics_by_fe_eth_port' => [
              { "fe_port_id"                    => "fe_port_id" },
              { "node_id"                       => "node_id" },
              { "appliance_id"                  => "appliance_id" },
              { "avg_bytes_tx_ps"               => "read" },
              { "avg_bytes_rx_ps"               => "write" },
              { "avg_pkt_tx_ps"                 => "read_packet" },
              { "avg_pkt_rx_ps"                 => "write_packet" },
              { "avg_pkt_rx_crc_error_ps"       => "crc" },
              { "avg_pkt_rx_no_buffer_error_ps" => "buffer_error" },
              { "avg_pkt_tx_error_ps"           => "error_rate" },
            ]
          },
        ]
      },
      { path   => 'fc_port',
        select => [ 'appliance_id', 'current_speed', 'id', 'io_module_id', 'is_link_up', 'name', 'node_id', 'partner_id', 'port_connector_type', 'port_index', 'sfp_id', 'wwn' ],
        params => [ 'limit=2000' ],
        entity => [
          { 'performance_metrics_by_fe_fc_port' => [
              { "fe_port_id"                     => "fe_port_id" },
              { "node_id"                        => "node_id" },
              { "appliance_id"                   => "appliance_id" },
              { "avg_read_iops"                  => "read_io" },
              { "avg_write_iops"                 => "write_io" },
              { "avg_total_iops"                 => "io_rate" },
              { "avg_read_bandwidth"             => "read" },
              { "avg_write_bandwidth"            => "write" },
              { "avg_total_bandwidth"            => "data_rate" },
              { "avg_read_latency"               => "resp_t_r" },
              { "avg_write_latency"              => "resp_t_w" },
              { "avg_latency"                    => "resp_t" },
              { "avg_unaligned_read_iops"        => "un_read_io" },
              { "avg_unaligned_write_iops"       => "un_write_io" },
              { "avg_loss_of_signal_count_ps"    => "loss_sig" },
              { "avg_loss_of_sync_count_ps"      => "loss_sync" },
              { "avg_link_failure_count_ps"      => "link_fail" },
              { "avg_invalid_crc_count_ps"       => "crc" },
              # { "avg_current_logins"             => "avg_current_logins" },
              # { "avg_dumped_frames_ps"           => "avg_dumped_frames_ps" },
              # { "avg_invalid_tx_word_count_ps"   => "avg_invalid_tx_word_count_ps" },
              # { "avg_prim_seq_prot_err_count_ps" => "avg_prim_seq_prot_err_count_ps" },
              # { "avg_unaligned_bandwidth"        => "avg_unaligned_bandwidth" },
              # { "avg_unaligned_iops"             => "avg_unaligned_iops" },
              # { "avg_unaligned_read_bandwidth"   => "avg_unaligned_read_bandwidth" },
              # { "avg_unaligned_write_bandwidth"  => "avg_unaligned_write_bandwidth" },
              # { "avg_io_size"                    => "io_size" },
              # { "avg_read_size"                  => "read_size" },
              # { "avg_write_size"                 => "write_size" },
            ]
          },
        ]
      },
      { path   => 'node',
        select => [ 'appliance_id', 'id', 'slot' ],
        entity => [
          { 'performance_metrics_by_node' => [
              { "node_id"                         => "node_id" },
              { "appliance_id"                    => "appliance_id" },
              { "avg_read_iops"                   => "read_io" },
              { "avg_write_iops"                  => "write_io" },
              { "avg_total_iops"                  => "io_rate" },
              { "avg_read_bandwidth"              => "read" },
              { "avg_write_bandwidth"             => "write" },
              { "avg_total_bandwidth"             => "data_rate" },
              { "avg_read_latency"                => "resp_t_r" },
              { "avg_write_latency"               => "resp_t_w" },
              { "avg_latency"                     => "resp_t" },
              { "avg_unaligned_read_iops"         => "un_read_io" },
              { "avg_unaligned_write_iops"        => "un_write_io" },
              # { "avg_io_workload_cpu_utilization" => "avg_io_workload_cpu_utilization" },
              # { "avg_unaligned_bandwidth"         => "avg_unaligned_bandwidth" },
              # { "avg_unaligned_iops"              => "avg_unaligned_iops" },
              # { "avg_unaligned_read_bandwidth"    => "avg_unaligned_read_bandwidth" },
              # { "avg_unaligned_write_bandwidth"   => "avg_unaligned_write_bandwidth" },
              # { "avg_current_logins"              => "avg_current_logins" },
              # { "avg_read_size"                   => "read_size" },
              # { "avg_io_size"                     => "io_size" },
              # { "avg_write_size"                  => "write_size" },
            ]
          },
          # { 'performance_metrics_by_fe_eth_node' => [
          #     { "avg_bytes_rx_ps"               => "fe_eth_bytes_rx_ps" },
          #     { "avg_bytes_tx_ps"               => "fe_eth_bytes_tx_ps" },
          #     { "avg_pkt_rx_crc_error_ps"       => "fe_eth_pkt_rx_crc_error_ps" },
          #     { "avg_pkt_rx_no_buffer_error_ps" => "fe_eth_pkt_rx_no_buffer_error_ps" },
          #     { "avg_pkt_rx_ps"                 => "fe_eth_pkt_rx_ps" },
          #     { "avg_pkt_tx_error_ps"           => "fe_eth_pkt_tx_error_ps" },
          #     { "avg_pkt_tx_ps"                 => "fe_eth_pkt_tx_ps" },
          #   ]
          # },
          # { 'performance_metrics_by_fe_fc_node' => [
          #     { "avg_current_logins"             => "fe_fc_current_logins" },
          #     { "avg_dumped_frames_ps"           => "fe_fc_dumped_frames_ps" },
          #     { "avg_invalid_crc_count_ps"       => "fe_fc_invalid_crc_count_ps" },
          #     { "avg_invalid_tx_word_count_ps"   => "fe_fc_invalid_tx_word_count_ps" },
          #     { "avg_io_size"                    => "fe_fc_io_size" },
          #     { "avg_latency"                    => "fe_fc_latency" },
          #     { "avg_link_failure_count_ps"      => "fe_fc_link_failure_count_ps" },
          #     { "avg_loss_of_signal_count_ps"    => "fe_fc_loss_of_signal_count_ps" },
          #     { "avg_loss_of_sync_count_ps"      => "fe_fc_loss_of_sync_count_ps" },
          #     { "avg_prim_seq_prot_err_count_ps" => "fe_fc_prim_seq_prot_err_count_ps" },
          #     { "avg_read_bandwidth"             => "fe_fc_read_bandwidth" },
          #     { "avg_read_iops"                  => "fe_fc_read_iops" },
          #     { "avg_read_latency"               => "fe_fc_read_latency" },
          #     { "avg_read_size"                  => "fe_fc_read_size" },
          #     { "avg_total_bandwidth"            => "fe_fc_total_bandwidth" },
          #     { "avg_total_iops"                 => "fe_fc_total_iops" },
          #     { "avg_unaligned_bandwidth"        => "fe_fc_unaligned_bandwidth" },
          #     { "avg_unaligned_iops"             => "fe_fc_unaligned_iops" },
          #     { "avg_unaligned_read_bandwidth"   => "fe_fc_unaligned_read_bandwidth" },
          #     { "avg_unaligned_read_iops"        => "fe_fc_unaligned_read_iops" },
          #     { "avg_unaligned_write_bandwidth"  => "fe_fc_unaligned_write_bandwidth" },
          #     { "avg_unaligned_write_iops"       => "fe_fc_unaligned_write_iops" },
          #     { "avg_write_bandwidth"            => "fe_fc_write_bandwidth" },
          #     { "avg_write_iops"                 => "fe_fc_write_iops" },
          #     { "avg_write_latency"              => "fe_fc_write_latency" },
          #     { "avg_write_size"                 => "fe_fc_write_size" },
          #   ]
          # },
          { 'performance_metrics_smb_by_node' => [
              { "avg_read_iops"       => "smb_read_io" },
              { "avg_write_iops"      => "smb_write_io" },
              { "avg_read_bandwidth"  => "smb_read" },
              { "avg_write_bandwidth" => "smb_write" },
              { "avg_read_latency"    => "smb_resp_t_r" },
              { "avg_write_latency"   => "smb_resp_t_w" },
              # { "avg_total_bandwidth" => "smb_total_bandwidth" },
              # { "avg_iops"            => "smb_iops" },
              # { "avg_latency"         => "smb_latency" },
              # { "avg_io_size"         => "smb_io_size" },
              # { "avg_read_size"       => "smb_read_size" },
              # { "avg_write_size"      => "smb_write_size" },
              # { "avg_calls"           => "smb_calls" },
            ]
          },
          # { 'performance_metrics_smb_branch_cache_by_node' => [
          #     { "hash_avg_latency"     => "smb_hash_avg_latency" },
          #     { "hash_avg_size"        => "smb_hash_avg_size" },
          #     { "total_rejected_tasks" => "smb_total_rejected_tasks" },
          #     { "total_tasks"          => "smb_total_tasks" },
          #   ]
          # },
          # { 'performance_metrics_smb1_by_node' => [
          #     { "avg_calls"         => "smb1_calls" },
          #     { "avg_io_size"       => "smb1_io_size" },
          #     { "avg_iops"          => "smb1_iops" },
          #     { "avg_latency"       => "smb1_latency" },
          #     { "avg_read_iops"     => "smb1_read_iops" },
          #     { "avg_read_latency"  => "smb1_read_latency" },
          #     { "avg_read_size"     => "smb1_read_size" },
          #     { "avg_write_iops"    => "smb1_write_iops" },
          #     { "avg_write_latency" => "smb1_write_latency" },
          #     { "avg_write_size"    => "smb1_write_size" },
          #   ]
          # },
          # { 'performance_metrics_smb2_by_node' => [
          #     { "avg_calls"         => "smb2_calls" },
          #     { "avg_io_size"       => "smb2_io_size" },
          #     { "avg_iops"          => "smb2_iops" },
          #     { "avg_latency"       => "smb2_latency" },
          #     { "avg_read_iops"     => "smb2_read_iops" },
          #     { "avg_read_latency"  => "smb2_read_latency" },
          #     { "avg_read_size"     => "smb2_read_size" },
          #     { "avg_write_iops"    => "smb2_write_iops" },
          #     { "avg_write_latency" => "smb2_write_latency" },
          #     { "avg_write_size"    => "smb2_write_size" },
          #   ]
          # },
          { 'performance_metrics_nfs_by_node' => [
              { "avg_read_iops"       => "nfs_read_io" },
              { "avg_write_iops"      => "nfs_write_io" },
              { "avg_read_bandwidth"  => "nfs_read" },
              { "avg_write_bandwidth" => "nfs_write" },
              { "avg_read_latency"    => "nfs_resp_t_r" },
              { "avg_write_latency"   => "nfs_resp_t_w" },
              # { "avg_total_iops"      => "nfs_total_iops" },
              # { "avg_latency"         => "nfs_latency" },
              # { "avg_total_bandwidth" => "nfs_total_bandwidth" },
              # { "avg_io_size"         => "nfs_io_size" },
              # { "avg_read_size"       => "nfs_read_size" },
              # { "avg_write_size"      => "nfs_write_size" },
            ]
          },
          # { 'performance_metrics_nfsv3_by_node' => [
          #     { "avg_failed_md_ops" => "nfsv3_failed_md_ops" },
          #     { "avg_md_latency"    => "nfsv3_md_latency" },
          #     { "avg_md_ops"        => "nfsv3_md_ops" },
          #     { "avg_read_iops"     => "nfsv3_read_iops" },
          #     { "avg_total_iops"    => "nfsv3_total_iops" },
          #     { "avg_write_iops"    => "nfsv3_write_iops" },
          #   ]
          # },
          # { 'performance_metrics_nfsv4_by_node' => [
          #     { "avg_failed_md_ops" => "nfsv4_failed_md_ops" },
          #     { "avg_md_latency"    => "nfsv4_md_latency" },
          #     { "avg_md_ops"        => "nfsv4_md_ops" },
          #     { "avg_read_iops"     => "nfsv4_read_iops" },
          #     { "avg_total_iops"    => "nfsv4_total_iops" },
          #     { "avg_write_iops"    => "nfsv4_write_iops" },
          #   ]
          # },
        ]
      },
      { path   => 'volume',
        select => [ 'appliance_id', 'id', 'name', 'performance_policy_id', 'size', 'state', 'type', 'volume_groups', 'wwn' ],
        params => [ 'limit=2000' ],
        entity => [
          { 'performance_metrics_by_volume' => [
              { "appliance_id"        => "appliance_id" },
              { "volume_id"           => "volume_id" },
              { "avg_read_iops"       => "read_io" },
              { "avg_write_iops"      => "write_io" },
              { "avg_total_iops"      => "io_rate" },
              { "avg_read_bandwidth"  => "read" },
              { "avg_write_bandwidth" => "write" },
              { "avg_total_bandwidth" => "data_rate" },
              { "avg_read_latency"    => "resp_t_r" },
              { "avg_write_latency"   => "resp_t_w" },
              { "avg_latency"         => "resp_t" },
              # { "avg_read_size"       => "read_size" },
              # { "avg_io_size"         => "io_size" },
              # { "avg_write_size"      => "write_size" },
            ]
          },
          { 'space_metrics_by_volume' => [
              { "logical_provisioned" => "virtual_capacity_total" },
              { "logical_used"        => "virtual_capacity_used" },
              # { "thin_savings"        => "thin_savings" },
            ]
          },
        ]
      },
      { path   => 'volume_group',
        select => [ 'id', 'name', 'type', 'volumes' ],
        params => [ 'limit=2000' ],
        entity => [
          { 'performance_metrics_by_vg' => [
              { "vg_id"               => "vg_id" },
              { "avg_read_iops"       => "read_io" },
              { "avg_write_iops"      => "write_io" },
              { "avg_total_iops"      => "io_rate" },
              { "avg_read_bandwidth"  => "read" },
              { "avg_write_bandwidth" => "write" },
              { "avg_total_bandwidth" => "data_rate" },
              { "avg_read_latency"    => "resp_t_r" },
              { "avg_write_latency"   => "resp_t_w" },
              { "avg_latency"         => "resp_t" },
              # { "avg_io_size"         => "io_size" },
              # { "avg_read_size"       => "read_size" },
              # { "avg_write_size"      => "write_size" },
            ]
          },
          # { 'space_metrics_by_vg' => [
          #     { "logical_provisioned"     => "logical_provisioned" },
          #     { "logical_used"            => "logical_used" },
          #     { "snap_clone_logical_used" => "snap_clone_logical_used" },
          #     { "snapshot_savings"        => "snapshot_savings" },
          #     { "thin_savings"            => "thin_savings" },
          #   ]
          # },
        ]
      },
    ]
  };

}

1;
