package QNAP;

use strict;
use warnings;

use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use Data::Dumper;
use Xormon;
use Time::Local;
#use Date::parse;
use JSON qw(encode_json decode_json);
use HTTP::Date;
use Test;
use XormonLogger;


my %data;
my %cfg_in;
my $xormon;
my $cfg;
my $hash;
my $logger;
my $ms_status;

my $service_data_in  = {};
my $service_data_out = {};

sub collect_data {
  $xormon = shift;
  $cfg    = shift;
  $ms_status = "successful";

  #print Dumper $cfg;
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")     && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")      && return "error"; }
  $logger = XormonLogger->new( 'qnap', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'sshUsername'} )   { $logger->error("Cannot continue, 'sshUsername' is not configured!")   && return "error"; }
  unless ( exists $cfg->{'sshPort'} )       { $logger->error("Cannot continue, 'sshPort' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'sshHost'} )       { $logger->error("Cannot continue, 'sshHost' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'qnapName'} )      { $logger->error("Cannot continue, 'qnapName' is not configured!")      && return "error"; }

  my $storage_name = $cfg->{'hostalias'};
  my $user         = $cfg->{'sshUsername'};
  my $host         = $cfg->{'sshHost'};
  my $array_name   = $cfg->{'qnapName'};

  my %cfg_in;
  my %data;
  my $hash;

  $cfg->{'SSH'}       = "ssh";
  my $password        = $cfg->{'sshPassword'};
  $cfg->{'password_enable'} = (!$cfg->{'isKeyFile'});
  $cfg->{'ssh_connect'}    = "";

  if ( defined $cfg->{'sshPort'} && $cfg->{'sshPort'} ne "" ) {
      $cfg->{'SSH'} = $cfg->{'SSH'} . " -p $cfg->{'sshPort'}";
  }

  if ( defined $cfg->{'sshKeyFile'} && $cfg->{'sshKeyFile'} ne "" ) {
    $cfg->{'SSH'} = $cfg->{'SSH'} . " -i $cfg->{'sshKeyFile'} ";
  }

  if ( !Xormon::isdigit($cfg->{'password_enable'}) ) {
    $cfg->{'password_enable'} = 0;
  }

  ## GET MICROSERVICE DATA
  $service_data_in  = $xormon->getData();
  $service_data_out = {};

  #######################################
  # collect configuration               #
  #######################################

  if ( $cfg->{'password_enable'} == 1 ) {
    $cfg->{'ssh_connect'} = Xormon::openssh_new( $cfg->{'sshHost'}, $cfg->{'sshPort'}, $cfg->{'sshUsername'}, 30, $cfg->{'sshPassword'} );
    if ( $cfg->{'ssh_connect'} =~ m/^Error|^ERROR/ ) {
      $logger->error($cfg->{'ssh_connect'});
      $xormon->isConnected(0, "openssh error");
      $xormon->hasData(0, "openssh error");
      close_session();
      return "error";
    }
  }

  my $print_conf_file = 0;
  my $last_conf_time = "";
  my $cur_time = time();

  # print "LAST CONF\n";
  # print Dumper $service_data_in->{data}{value}{last_conf_timestamp};

  if ( exists $service_data_in->{last_conf_timestamp} && Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) {
    $last_conf_time = $service_data_in->{last_conf_timestamp};
    my $dif_time = $cur_time - $last_conf_time;
    if ( $dif_time < 3600 ) {
      $service_data_out->{last_conf_timestamp} = $last_conf_time;
      load_cfg();
    }
    else {
      $print_conf_file = 1;
      get_cfg();
    }
  }
  else {
    $print_conf_file = 1;
    get_cfg();
  }


  get_data();

  if ( $print_conf_file == 1 ) {
    print_conf_data();
  }

  print_perf_data();

  health_status();

  ## SAVE TO DB
  #print Dumper $service_data_out;
  $xormon->saveData(\%{ $service_data_out });

  #close_session();
  return $ms_status;
}

sub print_conf_data {
  #my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
  #my $time_output_file = sprintf( "%4d%02d%02d_%02d%02d", $year + 1900, $month + 1, $day, $hour, $min );
  my @architecture;
  my $st_serial = $cfg->{'hostcfg_id'};
  my %conf;

  #print "Configuration Data\n";
  #print "------------------\n";
  my $hostname = $cfg->{'hostalias'};

  if ( defined $data{'conf'}{'firmware_version'} && $data{'conf'}{'firmware_version'} ne '' ) {
    $conf{ $cfg->{'hostcfg_id'} }{'version'} = $data{'conf'}{'firmware_version'};
  }
  if ( defined $data{'conf'}{'nas_uuid'} && $data{'conf'}{'nas_uuid'} ne '' ) {
    $conf{ $cfg->{'hostcfg_id'} }{'nas_uuid'} = $data{'conf'}{'nas_uuid'};
  }
  if ( defined $data{'conf'}{'cluster_uuid'} && $data{'conf'}{'cluster_uuid'} ne '' ) {
    $conf{ $cfg->{'hostcfg_id'} }{'cluster_uuid'} = $data{'conf'}{'cluster_uuid'};
  }
  if ( defined $data{'conf'}{'added_ip'} && $data{'conf'}{'added_ip'} ne '' ) {
    $conf{ $cfg->{'hostcfg_id'} }{'ip'} = $data{'conf'}{'added_ip'};
  }
  if ( defined $data{'conf'}{'agent_version'} && $data{'conf'}{'agent_version'} ne '' ) {
    $conf{ $cfg->{'hostcfg_id'} }{'agent_version'} = $data{'conf'}{'agent_version'};
  }
  if ( defined $data{'conf'}{'model_name'} && $data{'conf'}{'model_name'} ne '' ) {
    $conf{ $cfg->{'hostcfg_id'} }{'model'} = $data{'conf'}{'model_name'};
  }
  $conf{ $cfg->{'hostcfg_id'} }{'node_name'} = $cfg->{'qnapName'};
  #print "IP address : $added_ip\n";
  #print "Agent version : $agent_version\n";
  #print "Hostname : $hostname\n";
  #print "nas uuid : $nas_uuid\n";
  #print "cluster uuid : $cluster_uuid\n";

  #SYSTEM STORAGE CONFIGURATION";
  my %device_arch = (
    "item_id" => $cfg->{'hostcfg_id'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => "qnap",
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'hostcfg_id'}
  );

  push(@architecture, \%device_arch);

  ### VOLUME ###

  if ( defined $data{'conf'}{'vol'} && $data{'conf'}{'vol'} ne "" ) {
    foreach my $vol_uuid ( keys( %{ $data{'conf'}{'vol'} } ) ) {
      my $hash_key = Xormon::create_hash_key($st_serial."VOLUME".$vol_uuid); $hash_key = lc($hash_key);

      my $name      = "";
      my $uuid      = $vol_uuid;
      my $capacity  = "";
      my $pool_id   = "";
      my $pool_uuid = "";

      if ( defined $data{'conf'}{'vol'}{$uuid}{'vol_name'} && $data{'conf'}{'vol'}{$uuid}{'vol_name'} ne '' ) {
        $name = $data{'conf'}{'vol'}{$uuid}{'vol_name'};
      }
      if ( defined $data{'conf'}{'vol'}{$uuid}{'capacity'} && $data{'conf'}{'vol'}{$uuid}{'capacity'} ne '' ) {
        $capacity = $data{'conf'}{'vol'}{$uuid}{'capacity'};
      }
      if ( defined $data{'conf'}{'vol'}{$uuid}{'pool_id'} && $data{'conf'}{'vol'}{$uuid}{'pool_id'} ne '' ) {
        $pool_id = $data{'conf'}{'vol'}{$uuid}{'pool_id'};
      }
      if ( defined $data{'conf'}{'pool'} && $data{'conf'}{'pool'} ne "" ) {
        foreach my $each_pool_uuid ( keys( %{ $data{'conf'}{'pool'} } ) ) {
          if ( $data{'conf'}{'pool'}{$each_pool_uuid}{'id'} == $data{'conf'}{'vol'}{$uuid}{'pool_id'} ) {
            $pool_uuid = $each_pool_uuid;
          }
        }
      }

      $conf{$hash_key}{'lun_id'} = $uuid;

      my $pool_key = Xormon::create_hash_key($st_serial."POOL".$pool_uuid ); $hash_key = lc($hash_key);

      my %arch = (
        "item_id" => $hash_key,
        "label" => $name,
        "hw_type" => "qnap",
        "subsystem" => "volume",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'},$pool_key]
      );

      push(@architecture, \%arch);
      #print "$name,$uuid,$capacity,$pool_id,$pool_uuid\n";
    }
  }

  ### POOL ###
  if ( defined $data{'conf'}{'pool'} && $data{'conf'}{'pool'} ne "" ) {
    foreach my $uuid ( keys( %{ $data{'conf'}{'pool'} } ) ) {
      my $hash_key = Xormon::create_hash_key($st_serial."POOL".$uuid); $hash_key = lc($hash_key);
      my $pool_id   = "";
      my $pool_uuid = $uuid;
      my $total_cap = "";
      my $used_cap  = "";
      my $free_cap  = "";

      if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'id'} && $data{'conf'}{'pool'}{$pool_uuid}{'id'} ne '' ) {
        $pool_id = $data{'conf'}{'pool'}{$pool_uuid}{'id'};
      }
      if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'capacity'} && $data{'conf'}{'pool'}{$pool_uuid}{'capacity'} ne '' ) {
        $total_cap = $data{'conf'}{'pool'}{$pool_uuid}{'capacity'};
        $conf{ $cfg->{'hostcfg_id'} }{'capacity_total'} = $data{'conf'}{'pool'}{$pool_uuid}{'capacity'};
      }
      if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'used_capacity'} && $data{'conf'}{'pool'}{$pool_uuid}{'used_capacity'} ne '' ) {
        $used_cap = "$data{'conf'}{'pool'}{$pool_uuid}{'used_capacity'}";
        $conf{ $cfg->{'hostcfg_id'} }{'capacity_used'} = "$data{'conf'}{'pool'}{$pool_uuid}{'used_capacity'}";
      }
      if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'} && $data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'} ne '' ) {
        $free_cap = $data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'};
        $conf{ $cfg->{'hostcfg_id'} }{'capacity_free'} = $data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'};
      }
      my %arch = (
        "item_id" => $hash_key,
        "label" => $pool_id,
        "hw_type" => "qnap",
        "subsystem" => "pool",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

      push(@architecture, \%arch);
      #print "$pool_id,$pool_uuid,$total_cap,$used_cap,$free_cap\n";
    }
  }

  ### DRIVE ###
  if ( defined $data{'conf'}{'disk'} && $data{'conf'}{'disk'} ne "" ) {
    foreach my $uuid ( keys( %{ $data{'conf'}{'disk'} } ) ) {
      my $hash_key = Xormon::create_hash_key($st_serial."DISK".$uuid); $hash_key = lc($hash_key);
      my $disk_uuid  = $uuid;
      my $serial_num = "";
      my $type       = "";
      my $slot       = "";
      my $bus_type   = "";
      my $curr_speed = "";
      my $capacity   = "";
      if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'serial_number'} && $data{'conf'}{'disk'}{$disk_uuid}{'serial_number'} ne '' ) {
        $serial_num = $data{'conf'}{'disk'}{$disk_uuid}{'serial_number'};
      }
      if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'type'} && $data{'conf'}{'disk'}{$disk_uuid}{'type'} ne '' ) {
        $type = $data{'conf'}{'disk'}{$disk_uuid}{'type'};
      }
      if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'capacity'} && $data{'conf'}{'disk'}{$disk_uuid}{'type'} ne '' ) {
        $capacity = $data{'conf'}{'disk'}{$disk_uuid}{'capacity'};
      }
      if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'slot'} && $data{'conf'}{'disk'}{$disk_uuid}{'slot'} ne '' ) {
        $slot = $data{'conf'}{'disk'}{$disk_uuid}{'slot'};
      }
      if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'bus_type'} && $data{'conf'}{'disk'}{$disk_uuid}{'bus_type'} ne '' ) {
        $bus_type = $data{'conf'}{'disk'}{$disk_uuid}{'bus_type'};
      }
      if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'curr_speed'} && $data{'conf'}{'disk'}{$disk_uuid}{'curr_speed'} ne '' ) {
        $curr_speed = $data{'conf'}{'disk'}{$disk_uuid}{'curr_speed'};
      }

      my %arch = (
        "item_id" => $hash_key,
        "label" => $uuid,
        "hw_type" => "qnap",
        "subsystem" => "drive",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

      push(@architecture, \%arch);
      #print "$disk_uuid,$serial_num,$type,$capacity,$slot,$bus_type,$curr_speed\n";
    }
  }


  ### PORT ###
  if ( defined $data{'conf'}{'network'} && $data{'conf'}{'network'} ne "" ) {
    foreach my $net_uuid ( keys( %{ $data{'conf'}{'network'} } ) ) {
      my $hash_key = Xormon::create_hash_key($st_serial."PORT".$net_uuid); $hash_key = lc($hash_key);
      if ( $net_uuid =~ m/^bond/ ) { next; }    # KZ: exclude bond ports because we do not colloect perf data for them, so do not add them to SQLite DB (XorMon)
      my $uuid         = $net_uuid;
      my $display_name = "";
      if ( defined $data{'conf'}{'network'}{$uuid}{'display_name'} && $data{'conf'}{'network'}{$uuid}{'display_name'} ne '' ) {
        $display_name = $data{'conf'}{'network'}{$uuid}{'display_name'};
      }

      my %arch = (
        "item_id" => $hash_key,
        "label" => $display_name,
        "hw_type" => "qnap",
        "subsystem" => "port",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

      push(@architecture, \%arch);
      #print "$uuid,$display_name\n";
    }
  }

  ### RAID ###
  if ( defined $data{'conf'}{'raid'} && $data{'conf'}{'raid'} ne "" ) {
    foreach my $raid_uuid ( keys( %{ $data{'conf'}{'raid'} } ) ) {
      my $hash_key = Xormon::create_hash_key($st_serial."RAID".$raid_uuid); $hash_key = lc($hash_key);
      my $uuid    = $raid_uuid;
      my $raid_id = "";
      if ( defined $data{'conf'}{'raid'}{$uuid}{'raid_id'} && $data{'conf'}{'raid'}{$uuid}{'raid_id'} ne '' ) {
        $raid_id = $data{'conf'}{'raid'}{$uuid}{'raid_id'};
      }
      my %arch = (
        "item_id" => $hash_key,
        "label" => $raid_id,
        "hw_type" => "qnap",
        "subsystem" => "raid_group",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

      push(@architecture, \%arch);
      #print "$uuid,$raid_id\n";
    }
  }

  ### CPU ###
  if ( defined $data{'conf'}{'cpu'} && $data{'conf'}{'cpu'} ne "" ) {
    foreach my $cpu_uuid ( keys( %{ $data{'conf'}{'cpu'} } ) ) {
      my $hash_key = Xormon::create_hash_key($st_serial."CPU".$cpu_uuid); $hash_key = lc($hash_key);
      my $uuid = $cpu_uuid;
      my $id   = "";
      if ( defined $data{'conf'}{'cpu'}{$uuid}{'cpu_id'} && $data{'conf'}{'cpu'}{$uuid}{'cpu_id'} ne '' ) {
        $id = $data{'conf'}{'cpu'}{$uuid}{'cpu_id'};
      }
      my %arch = (
        "item_id" => $hash_key,
        "label" => $id,
        "hw_type" => "qnap",
        "subsystem" => "cpu",
        "hostcfg_id" => "$cfg->{'hostcfg_id'}",
        "parents" => [$cfg->{'hostcfg_id'}]
      );

      push(@architecture, \%arch);
      #print "$uuid,$id\n";
    }
  }
  #save configuration
  my $rc_conf = $xormon->saveConfJSON(\%conf);
  $logger->info("save conf: $rc_conf");
  #save architecture
  my $rc_arch = $xormon->saveArchitecture(\@architecture);
  $logger->info("save architecture: $rc_arch");

  return 1;
}    #end sub print_conf_data

sub print_perf_data {
  if ( defined $data{'perf'} && $data{'perf'} ne '' ) {
    # print Dumper $data{'perf'};
    my $time_stamp = "";
    my $st_serial = $cfg->{'hostcfg_id'};
    my $rc;
    my $status;
    my $data_out;
    foreach my $time ( sort { $a <=> $b } keys( %{ $data{'perf'} } ) ) {
      my $now = time();
      $now = $now - 600;
      if ( $now > $time ) {
        $time_stamp = $time;

        ### Volume STATISTICS ###
        if ( defined $data{'perf'}{$time}{'vol'} && $data{'perf'}{$time}{'vol'} ne '' ) {
          $data_out = "id;timestamp;resp_t;data_rate;io_rate;capacity_total;capacity_used;capacity_free;capacity_used_percent\n";
          foreach my $uuid ( sort keys %{ $data{'perf'}{$time}{'vol'} } ) {
            my $hash_key = Xormon::create_hash_key($st_serial."VOLUME".$uuid); $hash_key = lc($hash_key);
            my $volume      = "";
            my $volume_uuid = "$uuid";
            my $timestamp   = $time;
            my $pool_id     = "";
            my $allocated   = "";
            my $latency     = "";
            my $free        = "";
            my $total       = "";
            my $used        = "";
            my $usedpct     = "";
            my $throughput  = "";
            my $iops        = "";


            if ( defined $data{'conf'}{'vol'}{$volume_uuid}{'vol_name'} && $data{'conf'}{'vol'}{$volume_uuid}{'vol_name'} ne '' ) {
              $volume = $data{'conf'}{'vol'}{$volume_uuid}{'vol_name'};
            }
            if ( defined $data{'conf'}{'vol'}{$volume_uuid}{'pool_id'} && $data{'conf'}{'vol'}{$volume_uuid}{'pool_id'} ) {
              $pool_id = $data{'conf'}{'vol'}{$volume_uuid}{'pool_id'};
            }
            if ( defined $data{'conf'}{'vol'}{$volume_uuid}{'allocated'} && $data{'conf'}{'vol'}{$volume_uuid}{'allocated'} ne '' ) {
              $allocated = Xormon::formatNumberByType( "float4",$data{'conf'}{'vol'}{$volume_uuid}{'allocated'} );
            }
            if ( defined $data{'perf'}{$time}{'vol'}{$volume_uuid}{'latency'} && $data{'perf'}{$time}{'vol'}{$volume_uuid}{'latency'} ne '' ) {
              $latency = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'vol'}{$volume_uuid}{'latency'} );
            }
            if ( defined $data{'conf'}{'vol'}{$volume_uuid}{'free'} && $data{'conf'}{'vol'}{$volume_uuid}{'free'} ne '' ) {
              $free = Xormon::formatNumberByType( "float4",$data{'conf'}{'vol'}{$volume_uuid}{'free'} );
            }
            if ( defined $data{'conf'}{'vol'}{$volume_uuid}{'total'} && $data{'conf'}{'vol'}{$volume_uuid}{'total'} ne '' ) {
              $total = Xormon::formatNumberByType( "float4",$data{'conf'}{'vol'}{$volume_uuid}{'total'} );
            }
            if ( defined $data{'conf'}{'vol'}{$volume_uuid}{'used'} && $data{'conf'}{'vol'}{$volume_uuid}{'used'} ne '' ) {
              $used = Xormon::formatNumberByType( "float4",$data{'conf'}{'vol'}{$volume_uuid}{'used'} );
            }
            if ( defined $data{'perf'}{$time}{'vol'}{$volume_uuid}{'throughput'} && $data{'perf'}{$time}{'vol'}{$volume_uuid}{'throughput'} ne '' ) {
              $throughput = Xormon::formatNumberByType( "float4", $data{'perf'}{$time}{'vol'}{$volume_uuid}{'throughput'} );
            }
            if ( defined $data{'perf'}{$time}{'vol'}{$volume_uuid}{'iops'} && $data{'perf'}{$time}{'vol'}{$volume_uuid}{'iops'} ne '' ) {
              $iops = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'vol'}{$volume_uuid}{'iops'} );
            }
            # add pct capacity
            if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
              $usedpct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
            }

            $data_out .= "$hash_key;$timestamp;$latency;$throughput;$iops;$total;$used;$free;$usedpct\n";
          }

          #Save Volume
          $rc = $xormon->saveCSV($data_out,"qnap","volume");

          # status code
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }
          $logger->info("Volume status : $status");
          if ( $status ne 'successful' ) { $ms_status = $status; }
          #Xormon::log("Volume status : $status");
          #Xormon::log("DATA OUT: \n");
          #Xormon::log("$data_out\n");
          $data_out = "";
        }

        ### DRIVE STATISTICS ###
        if ( defined $data{'perf'}{$time}{'disk'} && $data{'perf'}{$time}{'disk'} ne '' ) {
          $data_out .= "id;timestamp;io_rate;data_rate;resp_t\n";
          foreach my $uuid ( sort keys %{ $data{'perf'}{$time}{'disk'} } ) {
            my $hash_key = Xormon::create_hash_key($st_serial."DISK".$uuid); $hash_key = lc($hash_key);
            my $disk_uuid     = $uuid;
            my $timestamp     = $time;
            my $serial_number = "";
            my $capacity      = "";
            my $type          = "";
            my $iops          = "";
            my $throughput    = "";
            my $latency       = "";
            my $temp          = "";

            if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'capacity'} && $data{'conf'}{'disk'}{$disk_uuid}{'capacity'} ne '' ) {
              $capacity = Xormon::formatNumberByType( "float4",$data{'conf'}{'disk'}{$disk_uuid}{'capacity'} );
            }
            if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'type'} && $data{'conf'}{'disk'}{$disk_uuid}{'type'} ne '' ) {
              $type = $data{'conf'}{'disk'}{$disk_uuid}{'type'};
            }
            if ( defined $data{'conf'}{'disk'}{$disk_uuid}{'serial_number'} && $data{'conf'}{'disk'}{$disk_uuid}{'serial_number'} ne '' ) {
              $serial_number = $data{'conf'}{'disk'}{$disk_uuid}{'serial_number'};
            }
            if ( defined $data{'perf'}{$time}{'disk'}{$disk_uuid}{'iops'} && $data{'perf'}{$time}{'disk'}{$disk_uuid}{'iops'} ne '' ) {
              $iops = Xormon::formatNumberByType( "float4", $data{'perf'}{$time}{'disk'}{$disk_uuid}{'iops'} );
            }
            if ( defined $data{'perf'}{$time}{'disk'}{$disk_uuid}{'throughput'} && $data{'perf'}{$time}{'disk'}{$disk_uuid}{'throughput'} ne '' ) {
              $throughput = Xormon::formatNumberByType( "float4", $data{'perf'}{$time}{'disk'}{$disk_uuid}{'throughput'} );
            }
            if ( defined $data{'perf'}{$time}{'disk'}{$disk_uuid}{'latency'} && $data{'perf'}{$time}{'disk'}{$disk_uuid}{'latency'} ne '' ) {
              $latency = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'disk'}{$disk_uuid}{'latency'} );
            }
            if ( defined $data{'perf'}{$time}{'disk'}{$disk_uuid}{'temp'} && $data{'perf'}{$time}{'disk'}{$disk_uuid}{'temp'} ne '' ) {
              $temp = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'disk'}{$disk_uuid}{'temp'} );
            }
            $data_out .= "$hash_key;$timestamp;$iops;$throughput;$latency\n";
          }
          #Save Drive
          $rc = $xormon->saveCSV($data_out,"qnap","drive");

          # status code
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }
          $logger->info("Drive status : $status");
          if ( $status ne 'successful' ) { $ms_status = $status; }
          #Xormon::log("DATA OUT: \n");
          #Xormon::log("$data_out\n");
          $data_out = "";
        }

        ### PORT STATISTICS ###
        if ( defined $data{'perf'}{$time}{'network'} && $data{'perf'}{$time}{'network'} ne '' ) {
          $data_out .= "id;timestamp;read;write\n";
          foreach my $uuid ( sort keys %{ $data{'perf'}{$time}{'network'} } ) {
            my $hash_key = Xormon::create_hash_key($st_serial."PORT".$uuid); $hash_key = lc($hash_key);
            my $port_uuid = $uuid;
            my $timestamp = $time;
            my $bytesin   = "";
            my $bytesout  = "";
            if ( defined $data{'perf'}{$time}{'network'}{$port_uuid}{'bytesout'} && $data{'perf'}{$time}{'network'}{$port_uuid}{'bytesout'} ne '' ) {
              $bytesout = Xormon::formatNumberByType( "float4", $data{'perf'}{$time}{'network'}{$port_uuid}{'bytesout'} );
            }
            if ( defined $data{'perf'}{$time}{'network'}{$port_uuid}{'bytesin'} && $data{'perf'}{$time}{'network'}{$port_uuid}{'bytesin'} ne '' ) {
              $bytesin = Xormon::formatNumberByType( "float4", $data{'perf'}{$time}{'network'}{$port_uuid}{'bytesin'} );
            }
            $data_out .= "$hash_key;$timestamp;$bytesin;$bytesout\n";
          }
          #Save PORT
          $rc = $xormon->saveCSV($data_out,"qnap","port");

          # status code
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }

          $logger->info("Port status : $status");
          if ( $status ne 'successful' ) { $ms_status = $status; }
          #Xormon::log("DATA OUT: \n");
          #Xormon::log("$data_out\n");
          $data_out = "";
        }

        ### RAID STATISTICS ###
        if ( defined $data{'perf'}{$time}{'raid'} && $data{'perf'}{$time}{'raid'} ne '' ) {
          $data_out = "id;timestamp;data_rate;io_rate\n";
          foreach my $uuid ( sort keys %{ $data{'perf'}{$time}{'raid'} } ) {
            my $hash_key = Xormon::create_hash_key($st_serial."RAID".$uuid); $hash_key = lc($hash_key);
            my $raid_uuid  = $uuid;
            my $timestamp  = $time;
            my $throughput = "";
            my $iops       = "";
            if ( defined $data{'perf'}{$time}{'raid'}{$raid_uuid}{'throughput'} && $data{'perf'}{$time}{'raid'}{$raid_uuid}{'throughput'} ne '' ) {
              $throughput = Xormon::formatNumberByType( "float4", $data{'perf'}{$time}{'raid'}{$raid_uuid}{'throughput'} );
            }
            if ( defined $data{'perf'}{$time}{'raid'}{$raid_uuid}{'iops'} && $data{'perf'}{$time}{'raid'}{$raid_uuid}{'iops'} ne '' ) {
              $iops = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'raid'}{$raid_uuid}{'iops'} );
            }
            $data_out .= "$hash_key;$timestamp;$throughput;$iops\n";
          }

          #Save RAID
          $rc = $xormon->saveCSV($data_out,"qnap","raid_group");

          # status code
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }

          $logger->info("Raid group status : $status");
          if ( $status ne 'successful' ) { $ms_status = $status; }
          #Xormon::log("DATA OUT: \n");
          #Xormon::log("$data_out\n");
          $data_out = "";
        }

        ### POOL STATISTICS and DEVICE statistics ###
        if ( defined $data{'perf'}{$time}{'pool'} && $data{'perf'}{$time}{'pool'} ne '' ) {
          my $device_out = "id;timestamp;data_rate;resp_t;io_rate;capacity_used;capacity_free;capacity_total;capacity_used_percent\n";

          my $device_io       = "";
          my $device_data     = "";
          my $device_resp_t   = "";
          my $device_total    = "";
          my $device_free     = "";
          my $device_used     = "";
          my $device_used_pct = "";

          my @lat_array = ();

          $data_out = "id;timestamp;data_rate;resp_t;io_rate;capacity_used;capacity_free;capacity_total;capacity_used_percent\n";
          foreach my $uuid ( sort keys %{ $data{'perf'}{$time}{'pool'} } ) {
            my %pool_latency_hash = ();
            my $hash_key = Xormon::create_hash_key($st_serial."POOL".$uuid); $hash_key = lc($hash_key);
            my $pool_id    = "";
            my $timestamp  = $time;
            my $pool_uuid  = $uuid;
            my $throughput = "";
            my $latency    = "";
            my $iops       = "";
            my $allocated  = "";
            my $free       = "";
            my $total      = "";
            my $used_pct   = "";

            if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'id'} && $data{'conf'}{'pool'}{$pool_uuid}{'id'} ne '' ) {
              $pool_id = $data{'conf'}{'pool'}{$pool_uuid}{'id'};
            }
            if ( defined $data{'perf'}{$time}{'pool'}{$pool_uuid}{'throughput'} && $data{'perf'}{$time}{'pool'}{$pool_uuid}{'throughput'} ne '' ) {
              $throughput = Xormon::formatNumberByType( "float4", $data{'perf'}{$time}{'pool'}{$pool_uuid}{'throughput'} );
              if (Xormon::isdigit($device_data)){
                $device_data += $throughput;
              }
              else{
                $device_data = $throughput;
              }
            }
            if ( defined $data{'perf'}{$time}{'pool'}{$pool_uuid}{'latency'} && $data{'perf'}{$time}{'pool'}{$pool_uuid}{'latency'} ne '' ) {
              $latency = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'pool'}{$pool_uuid}{'latency'} );
              $pool_latency_hash{'value'} = $latency;
            }
            if ( defined $data{'perf'}{$time}{'pool'}{$pool_uuid}{'iops'} && $data{'perf'}{$time}{'pool'}{$pool_uuid}{'iops'} ne '' ) {
              $iops = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'pool'}{$pool_uuid}{'iops'} );
              $pool_latency_hash{'count'} = $iops;
              if (Xormon::isdigit($device_io)){
                $device_io += $iops;
              }
              else{
                $device_io = $iops;
              }
            }
            if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'allocated'} && $data{'conf'}{'pool'}{$pool_uuid}{'allocated'} ne '' ) {
              $allocated = Xormon::formatNumberByType( "float4",$data{'conf'}{'pool'}{$pool_uuid}{'allocated'} );
              if (Xormon::isdigit($device_used)){
                $device_used += $allocated;
              }
              else{
                $device_used = $allocated;
              }
            }
            if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'} && $data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'} ne '' ) {
              $free = Xormon::formatNumberByType( "float4",$data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'} );
              if (Xormon::isdigit($device_free)){
                $device_free += $free;
              }
              else{
                $device_free = $free;
              }
            }
            if ( defined $data{'conf'}{'pool'}{$pool_uuid}{'total'} && $data{'conf'}{'pool'}{$pool_uuid}{'total'} ne '' ) {
              $total = Xormon::formatNumberByType( "float4",$data{'conf'}{'pool'}{$pool_uuid}{'total'} );
              if (Xormon::isdigit($device_total)){
                $device_total += $total;
              }
              else{
                $device_total = $total;
              }
            }
            push @lat_array, \%pool_latency_hash;

            if ( Xormon::isdigit($total) && Xormon::isdigit($allocated) && ($total != 0) ){
              $used_pct = Xormon::formatNumberByType("float4", $allocated / ($total / 100) );
            }
            $data_out .= "$hash_key;$timestamp;$throughput;$latency;$iops;$allocated;$free;$total;$used_pct\n";

          }

          #Save POOL
          $rc = $xormon->saveCSV($data_out,"qnap","pool");

          # status code
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }

          $device_resp_t = Xormon::formatNumberByType( "float4", Xormon::calculate_weighted_avg(\@lat_array));
          if ( Xormon::isdigit($device_total) && Xormon::isdigit($device_used) && ($device_total != 0) ){
            $device_used_pct = Xormon::formatNumberByType("float4", $device_used / ($device_total / 100) );
          }

          $device_out .= "$cfg->{'hostcfg_id'};$time;$device_data;$device_resp_t;$device_io;$device_used;$device_free;$device_total;$device_used_pct\n";

          # #Save Device
          $rc = $xormon->saveCSV($device_out,"qnap","device");

          # status code
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }

          $logger->info("Pool status : $status");
          if ( $status ne 'successful' ) { $ms_status = $status; }
          #Xormon::log("DATA OUT: \n");
          #Xormon::log("$data_out\n");
          $data_out = "";
        }
        $logger->info("Device status : $status");
        if ( $status ne 'successful' ) { $ms_status = $status; }
        #Xormon::log("DATA OUT: \n");
        #Xormon::log("$data_out\n");

        ### CPU STATISTICS ###
        if ( defined $data{'conf'}{'cpu'} && $data{'conf'}{'cpu'} ne '' ) {
          $data_out = "id;timestamp;cpu_system;cpu_other;cpu_user;cpu_idle;cpu_wait;memory_free;memory_total;memory_used\n";
          foreach my $uuid ( sort keys %{ $data{'conf'}{'cpu'} } ) {
            my $hash_key = Xormon::create_hash_key($st_serial."CPU".$uuid); $hash_key = lc($hash_key);
            my $timestamp = $time;
            my $sys       = "";
            my $other     = "";
            my $user      = "";
            my $idle      = "";
            my $iowait    = "";
            my $mem_free  = "";
            my $mem_total = "";
            my $mem_used  = "";

            if ( defined $data{'perf'}{$time}{'cpu'}{'sys'}{'usage'} && $data{'perf'}{$time}{'cpu'}{'sys'}{'usage'} ne '' ) {
              $sys = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'cpu'}{'sys'}{'usage'} );
            }
            if ( defined $data{'perf'}{$time}{'cpu'}{'other'}{'usage'} && $data{'perf'}{$time}{'cpu'}{'other'}{'usage'} ne '' ) {
              $other = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'cpu'}{'other'}{'usage'} );
            }
            if ( defined $data{'perf'}{$time}{'cpu'}{'user'}{'usage'} && $data{'perf'}{$time}{'cpu'}{'user'}{'usage'} ne '' ) {
              $user = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'cpu'}{'user'}{'usage'} );
            }
            if ( defined $data{'perf'}{$time}{'cpu'}{'idle'}{'idle'} && $data{'perf'}{$time}{'cpu'}{'idle'}{'idle'} ne '' ) {
              $idle = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'cpu'}{'idle'}{'idle'} );
            }
            if ( defined $data{'perf'}{$time}{'cpu'}{'iowait'}{'usage'} && $data{'perf'}{$time}{'cpu'}{'iowait'}{'usage'} ne '' ) {
              $iowait = Xormon::formatNumberByType( "float4",$data{'perf'}{$time}{'cpu'}{'iowait'}{'usage'} );
            }
            if ( defined $data{'perf'}{$time}{'mem'}{'physical'}{'free'} && $data{'perf'}{$time}{'mem'}{'physical'}{'free'} ne '' ) {
              $mem_free = Xormon::formatNumberByType( "float4", Xormon::returnBytes( $data{'perf'}{$time}{'mem'}{'physical'}{'free'}, "kB") );
            }
            if ( defined $data{'perf'}{$time}{'mem'}{'physical'}{'total'} && $data{'perf'}{$time}{'mem'}{'physical'}{'total'} ne '' ) {
              $mem_total = Xormon::formatNumberByType( "float4", Xormon::returnBytes( $data{'perf'}{$time}{'mem'}{'physical'}{'total'}, "kB") );
            }
            if ( defined $data{'perf'}{$time}{'mem'}{'physical'}{'used'} && $data{'perf'}{$time}{'mem'}{'physical'}{'used'} ne '' ) {
              $mem_used = Xormon::formatNumberByType( "float4", Xormon::returnBytes( $data{'perf'}{$time}{'mem'}{'physical'}{'used'}, "kB") );
            }

            $data_out .= "$hash_key;$timestamp;$sys;$other;$user;$idle;$iowait;$mem_free;$mem_total;$mem_used\n";
          }
          #Save CPU
          $rc = $xormon->saveCSV($data_out,"qnap","cpu");

          # status code
          if ($rc eq "201") {
            $status = "successful";
          } elsif ($rc eq "401") {
            $status = "unauthorized";
          } else {
            $status = "error";
          }

          $logger->info("CPU status : $status");
          if ( $status ne 'successful' ) { $ms_status = $status; }
          #Xormon::log("DATA OUT: \n");
          #Xormon::log("$data_out\n");
          $data_out = "";
        }
      }

      if ( $ms_status ne 'successful' ) { last; }

    } ## foreach time
    # $logger->info("time: $time_stamp");
    #
    # save microservice data
    #
    if ( defined $time_stamp && $time_stamp ne '' ) {
      $service_data_out->{last_perf_timestamp} = $time_stamp; # add last perf timestamp to db
    }
  }
  return 1;
}

sub get_data {

  my $cluster_uuid = $cfg_in{"cluster_uuid"};
  my $nas_uuid     = $cfg_in{"nas_uuid"};

  if ( !defined $cluster_uuid || $cluster_uuid eq '' ) {
    $logger->error("cluster_uuid must be set") && return "error";
  }
  if ( !defined $nas_uuid || $nas_uuid eq '' ) {
    $logger->error("nas_uuid must be set") && return "error";
  }
  my $path = "/opt/qnap-cms/qnap-cms/rrds/$cluster_uuid/$nas_uuid/";


  ## SET TIME
  my $last_time = "";
  my $cur_time = time();
  if ( exists $service_data_in->{last_perf_timestamp} && Xormon::isdigit($service_data_in->{last_perf_timestamp}) ) {
    $last_time = $service_data_in->{last_perf_timestamp};
    $service_data_out->{last_perf_timestamp} = $last_time;
    if ( ($cur_time - $last_time) > 3600 ) {
      $last_time = $cur_time - 3600;
    }
  }
  else {
    $last_time = $cur_time - 3600;
  }


  ## GET ALL RRDTOOL DATA
  my $cmd_fetch          = 'cd ' . $path . '; for rrd in `ls | egrep -v "_smartattr-|_temp.rrd$|^network_all_|^network_bond|^enc_|^fan_|^mem_swap_|^multicpu_|_fan.rrd$|^shared_|ssd_all_count.rrd|^vol_all_|^pool_all_|^vol_.*_allocated\.rrd|^vol_.*_free\.rrd|^vol_.*_total\.rrd|^vol_.*_used.*rrd|^pool_.*_allocated\.rrd|^pool_.*_free\.rrd|^pool_.*_total\.rrd|^pool_.*_used.*rrd"`; do echo "RRD:$rrd"; rrdtool fetch $rrd AVERAGE -s ' . $last_time . '; done';
  my @rrdtool_fetch_data = run_ssh($cmd_fetch);
  if ( !@rrdtool_fetch_data ) {
    $xormon->isConnected(0, "rrdtool_fetch data missing");
    $xormon->hasData(0, "rrdtool_fetch data missing");
    $logger->error("array rrdtool_fetch data must be set") && return "error";
  }
  $xormon->isConnected(1);

  ### PERF DATA
  my $name = "";
  my $uuid = "";
  my $stat = "";
  my $last_perf = "";
  foreach (@rrdtool_fetch_data) {
    chomp($_);
    $_ =~ s/^\s+//;
    $_ =~ s/\s+$//;
    my $time   = "";
    my $values = "";
    if ( $_ =~ m/^RRD:/ ) {
      $_ =~ s/RRD://;
      if ( $_ =~ m/cpu_idle/ ) {
        $uuid = "idle";
        ( $name, $stat ) = split /_/, $_;
        ( $stat, undef ) = split /\./, $stat;
      }
      else {
        ( $name, $uuid, $stat ) = split /_/, $_;
        ( $stat, undef ) = split /\./, $stat;
      }
    }
    if ( $_ =~ m/^[0-9]+:/ && defined $name && $name ne "" && defined $uuid && $uuid ne "" && defined $stat && $stat ne "" ) {
      ( $time, $values ) = split /:/, $_;
      if ( defined $time && $time ne "" && defined $values && $time ne "" ) {
        $last_perf = $time;
        $values =~ s/ //;
        $values = sprintf( "%.15g", $values );
        if ( isdigit($values) ) {
          $data{'perf'}{$time}{$name}{$uuid}{$stat} = $values;
        }
        else {
          $data{'perf'}{$time}{$name}{$uuid}{$stat} = "";    ## -nan undef value $values
        }
      }
    }
  }

  if ( Xormon::isdigit($last_perf) ) {
    if ( ( time() - $last_perf ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: $last_perf");
    }
    else {
      $xormon->hasData(1);
    }
  }
  else {
    $xormon->hasData(0, "no available stats");
  }

  ### CONFIGURATION DATA
  if ( defined $cfg_in{'added_ip'} && $cfg_in{'added_ip'} ne "" ) {
    $data{'conf'}{'added_ip'} = $cfg_in{'added_ip'};
  }
  if ( defined $cfg_in{'agent_version'} && $cfg_in{'agent_version'} ne "" ) {
    $data{'conf'}{'agent_version'} = $cfg_in{'agent_version'};
  }
  if ( defined $cfg_in{'model_name'} && $cfg_in{'model_name'} ne "" ) {
    $data{'conf'}{'model_name'} = $cfg_in{'model_name'};
  }
  if ( defined $cfg_in{'firmware_ver'}{'version'} && $cfg_in{'firmware_ver'}{'version'} ne "" ) {
    $data{'conf'}{'firmware_version'} = $cfg_in{'firmware_ver'}{'version'};
  }
  if ( defined $cfg_in{'nas_uuid'} && $cfg_in{'nas_uuid'} ne "" ) {
    $data{'conf'}{'nas_uuid'} = $cfg_in{'nas_uuid'};
  }
  if ( defined $cfg_in{'cluster_uuid'} && $cfg_in{'cluster_uuid'} ne "" ) {
    $data{'conf'}{'cluster_uuid'} = $cfg_in{'cluster_uuid'};
  }
  if ( defined $cfg_in{'sys_health'} && $cfg_in{'sys_health'} ne "" ) {
    $data{'conf'}{'sys_health'} = $cfg_in{'sys_health'};
  }

  ### VOLUME CONF
  if ( defined $cfg_in{'volume'} && ref( $cfg_in{'volume'} ) eq "ARRAY" ) {
    foreach my $hash ( @{ $cfg_in{'volume'} } ) {
      my $vol_uuid = "";
      if ( defined $hash->{'uuid'} && $hash->{'uuid'} ne "" ) {
        my $vol_uuid = $hash->{'uuid'};
        if ( defined $hash->{'vol_name'}->{'volume_name'} && $hash->{'vol_name'}->{'volume_name'} ne "" ) {
          $data{'conf'}{'vol'}{$vol_uuid}{'vol_name'} = $hash->{'vol_name'}->{'volume_name'};
        }
        if ( defined $hash->{'capacity'} && $hash->{'capacity'} ne "" ) {
          $data{'conf'}{'vol'}{$vol_uuid}{'capacity'} = $hash->{'capacity'};
        }
        if ( defined $hash->{'pool_id'} && $hash->{'pool_id'} ne "" ) {
          $data{'conf'}{'vol'}{$vol_uuid}{'pool_id'} = $hash->{'pool_id'};
        }
        ### add allocated && free && total && used
        if ( defined $hash->{'metric'}->{'allocated'} && $hash->{'metric'}->{'allocated'} ne "" ) {
          $data{'conf'}{'vol'}{$vol_uuid}{'allocated'} = $hash->{'metric'}->{'allocated'};
        }
        if ( defined $hash->{'metric'}->{'free'} && $hash->{'metric'}->{'free'} ne "" ) {
          $data{'conf'}{'vol'}{$vol_uuid}{'free'} = $hash->{'metric'}->{'free'};
        }
        if ( defined $hash->{'metric'}->{'total'} && $hash->{'metric'}->{'total'} ne "" ) {
          $data{'conf'}{'vol'}{$vol_uuid}{'total'} = $hash->{'metric'}->{'total'};
        }
        if ( defined $hash->{'metric'}->{'used'} && $hash->{'metric'}->{'used'} ne "" ) {
          $data{'conf'}{'vol'}{$vol_uuid}{'used'} = $hash->{'metric'}->{'used'};
        }
      }
    }
  }

  ### POOL CONF
  if ( defined $cfg_in{'storage_pool'} && ref( $cfg_in{'storage_pool'} ) eq "ARRAY" ) {
    foreach my $hash ( @{ $cfg_in{'storage_pool'} } ) {
      my $pool_uuid = "";
      if ( defined $hash->{'uuid'} && $hash->{'uuid'} ne "" ) {
        $pool_uuid = $hash->{'uuid'};
        if ( defined $hash->{'pool_id'} && $hash->{'pool_id'} ne "" ) {
          $data{'conf'}{'pool'}{$pool_uuid}{'id'} = $hash->{'pool_id'};
        }
        if ( defined $hash->{'capacity'} && $hash->{'capacity'} ne "" ) {
          $data{'conf'}{'pool'}{$pool_uuid}{'capacity'} = $hash->{'capacity'};
        }
        if ( defined $hash->{'metric'}->{'usedpct'} && $hash->{'metric'}->{'usedpct'} ne "" ) {
          $data{'conf'}{'pool'}{$pool_uuid}{'used_capacity'} = $hash->{'metric'}->{'usedpct'};
        }
        if ( defined $hash->{'metric'}->{'free'} && $hash->{'metric'}->{'free'} ne "" ) {
          $data{'conf'}{'pool'}{$pool_uuid}{'free_capacity'} = $hash->{'metric'}->{'free'};
        }

        #add total, allocated
        if ( defined $hash->{'metric'}->{'total'} && $hash->{'metric'}->{'total'} ne "" ) {
          $data{'conf'}{'pool'}{$pool_uuid}{'total'} = $hash->{'metric'}->{'total'};
        }
        if ( defined $hash->{'metric'}->{'allocated'} && $hash->{'metric'}->{'allocated'} ne "" ) {
          $data{'conf'}{'pool'}{$pool_uuid}{'allocated'} = $hash->{'metric'}->{'allocated'};
        }
      }
    }
  }

  ### DRIVE CONF
  if ( defined $cfg_in{'disk'} && ref( $cfg_in{'disk'} ) eq "ARRAY" ) {
    foreach my $hash ( @{ $cfg_in{'disk'} } ) {
      my $disk_uuid = "";
      if ( defined $hash->{'uuid'} && $hash->{'uuid'} ne "" ) {
        $disk_uuid = $hash->{'uuid'};
        if ( defined $hash->{'serial_number'} && $hash->{'serial_number'} ne "" ) {
          $data{'conf'}{'disk'}{$disk_uuid}{'serial_number'} = $hash->{'serial_number'};
        }
        if ( defined $hash->{'type'} && $hash->{'type'} ne "" ) {
          $data{'conf'}{'disk'}{$disk_uuid}{'type'} = $hash->{'type'};
        }
        if ( defined $hash->{'capacity'} && $hash->{'capacity'} ne "" ) {
          $data{'conf'}{'disk'}{$disk_uuid}{'capacity'} = $hash->{'capacity'};
        }
        if ( defined $hash->{'slot'} && $hash->{'slot'} ne "" ) {
          $data{'conf'}{'disk'}{$disk_uuid}{'slot'} = $hash->{'slot'};
        }
        if ( defined $hash->{'bus_type'} && $hash->{'bus_type'} ne "" ) {
          $data{'conf'}{'disk'}{$disk_uuid}{'bus_type'} = $hash->{'bus_type'};
        }
        if ( defined $hash->{'curr_speed'} && $hash->{'curr_speed'} ne "" ) {
          $data{'conf'}{'disk'}{$disk_uuid}{'curr_speed'} = $hash->{'curr_speed'};
        }
      }
    }
  }

  ### Network Port CONF
  if ( defined $cfg_in{'network'} && ref( $cfg_in{'network'} ) eq "ARRAY" ) {
    foreach my $hash ( @{ $cfg_in{'network'} } ) {
      my $port_uuid = "";
      if ( defined $hash->{'uuid'} && $hash->{'uuid'} ne "" ) {
        $port_uuid = $hash->{'uuid'};
        $data{'conf'}{'network'}{$port_uuid}{'uuid'} = $port_uuid;
        if ( defined $hash->{'display_name'} && $hash->{'display_name'} ne "" ) {
          $data{'conf'}{'network'}{$port_uuid}{'display_name'} = $hash->{'display_name'};
        }
      }
    }
  }

  ### CPU CONF
  if ( defined $cfg_in{'cpu'} && ref( $cfg_in{'cpu'} ) eq "ARRAY" ) {
    foreach my $hash ( @{ $cfg_in{'cpu'} } ) {
      my $cpu_uuid = "";
      if ( defined $hash->{'uuid'} && $hash->{'uuid'} ne "" ) {
        $cpu_uuid = $hash->{'uuid'};
        if ( defined $hash->{'cpu_id'} && $hash->{'cpu_id'} ne "" ) {
          $data{'conf'}{'cpu'}{$cpu_uuid}{'cpu_id'} = $hash->{'cpu_id'};
        }
      }
    }
  }

  ### RAID CONF
  if ( defined $cfg_in{'raid'} && ref( $cfg_in{'raid'} ) eq "ARRAY" ) {
    foreach my $hash ( @{ $cfg_in{'raid'} } ) {
      my $raid_uuid = "";
      if ( defined $hash->{'uuid'} && $hash->{'uuid'} ne "" ) {
        $raid_uuid = $hash->{'uuid'};
        if ( defined $hash->{'raid_id'} && $hash->{'raid_id'} ne "" ) {
          $data{'conf'}{'raid'}{$raid_uuid}{'raid_id'} = $hash->{'raid_id'};
        }
      }
    }
  }
  return 1;
}

sub run_ssh {
  my $cmd    = shift;
  my @output = ();

  if ( $cfg->{'password_enable'} == 1 ) {
    @output = Xormon::openssh_runcmd( $cfg->{'ssh_connect'}, "$cmd", 60, "ARRAY" );
  }
  else {
    @output = `$cfg->{'SSH'} $cfg->{'sshUsername'}\@$cfg->{'sshHost'} '$cmd' 2>&1`;
  }
  if ( $? != 0 ) {
    $logger->error( "ssh command failed: $cmd\n @output\n return code ($?) " . __FILE__ . ":" . __LINE__ );
    return ();
  }

  #if ( !$cfg->{'isKeyFile'} ) {
  #  @output = Xorux_lib::openssh_runcmd( $cfg->{'ssh_connect'}, "$cmd", 60, "ARRAY" );
  #}
  #else {
  #Xormon::log("$cfg->{'SSH'} $cfg->{'sshUsername'}\@$cfg->{'sshHost'}");
  #@output = `$cfg->{'SSH'} $cfg->{'sshUsername'}\@$cfg->{'sshHost'} '$cmd' 2>&1`;
  #}
  #if ( $? != 0 ) {
  #  error( "ssh command failed: $cmd\n @output\n return code ($?) " . __FILE__ . ":" . __LINE__ );
  #  return ();
  #}

  if ( scalar(@output) == 0 ){
    $logger->error( "ssh command failed: $cmd\n @output\n return code (X) " . __FILE__ . ":" . __LINE__ );
    return ();
  }

  my @errors       = ();
  my $errors_found = 0;
  if (@output) {
    foreach my $line (@output) {
      chomp($line);
      if ( $line =~ m/^ERROR:|^Error command:/ ) {
        push( @errors, "$line" );
        $errors_found = 1;
      }
    }
  }
  else {
    $logger->error("ERROR: The ssh command did not return any data: $cmd");
  }

  if ( $errors_found == 1 ) {
    $logger->error("ERROR: CMD: '$cmd'");
    foreach my $error (@errors) {
      $logger->error("$error");
    }
    @output = ();
  }

  return @output;

}

sub isdigit {
  my $digit = shift;
  if ( !defined($digit) ) {
    return 0;
  }
  if ( $digit eq '' ) {
    return 0;
  }
  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    #ís number
    return 1;
  }
  return 0;
}

sub load_cfg {
  if ( exists $service_data_in->{'last_config'} && $service_data_in->{'last_config'} ne "" ) {
    %cfg_in = %{$service_data_in->{'last_config'}};
    $service_data_out->{last_config} = \%cfg_in;
    #print Dumper \%cfg_in;
  }
  else {
    get_cfg();
  }
}    ### end sub load_cfg

sub get_cfg {
  my $cmd  = "mongoexport --collection='nas_server' --port 6604 --db cms --jsonArray --out=/tmp/stor2rrd-$cfg->{'qnapName'}.json; echo 'START_JSON'; cat /tmp/stor2rrd-$cfg->{'qnapName'}.json 2>&1; echo 'JSON_JSON'; rm -rf /tmp/stor2rrd-$cfg->{'qnapName'}.json;";
  my @data = run_ssh($cmd);
  my $json   = "";
  my $isjson = 0;
  foreach (@data) {
    chomp($_);

    # skip some trash
    if ( $_ =~ m/^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]T[0-9].*objects found$/ ) { next; }

    if ( $_ eq "START_JSON" ) {
      $isjson = 1;
      next;
    }
    if ( $_ eq "JSON_JSON" ) {
      $isjson = 0;
      next;
    }
    if ( $isjson == 1 ) {
      $json .= $_;
    }
  }

  my $not_found = 0;
  if ( defined $json && $json ne '' ) {
    $hash = decode_json($json);
    if ( defined $hash && ref($hash) eq "ARRAY" ) {
      foreach my $configuration ( @{$hash} ) {
        if ( defined $configuration && $configuration ne "" && ref($configuration) eq "HASH" ) {
          if ( $configuration->{'hostname'} eq $cfg->{'qnapName'} ) {
            %cfg_in    = %{$configuration};
            $not_found = 1;
          }
          else {
            $logger->error("QNAP NAME: $cfg->{'qnapName'} not same as hostname: $configuration->{'hostname'} in config json!");
          }
        }
      }
    }
  }
  else {
    $logger->error("json data must be set!") && return "error";
  }

  $service_data_out->{last_config} = \%cfg_in;
  $service_data_out->{last_conf_timestamp} = time();

  if ( $not_found == 0 ) {
    $logger->error("configuration for QNAP NAME: $cfg->{'qnapName'} not found") && return "error";
  }

  return 1;
}

sub health_status(){
  my @hs = ();
  my $sent_status = "";
  # print Dumper $data{'conf'}{'sys_health'};
  my $status = ( exists $data{'conf'}{'sys_health'} ) ? $data{'conf'}{'sys_health'} : "unknown";
  if ( !defined $status || $status eq '' ) {
    $logger->error("Health status was not found!");
  }

  if ( $status eq "good" ) {
    $sent_status = "ok";
  }
  elsif ( $status eq "warning" || $status eq "unknown" ) {
    $sent_status = "warning";
  }
  else{
    $sent_status = "error";
  }

  @hs = (
    {
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $sent_status,
      "data" => "QNAP status: $status",
    }
  );

  $xormon->saveStatus(\@hs);

}

sub close_session {
  if ( $cfg->{'password_enable'} == 1 && defined $cfg->{'ssh_connect'} && $cfg->{'ssh_connect'} ne "" && $cfg->{'ssh_connect'} !~ m/^Error|^ERROR/ ) {
    my $out = Xormon::openssh_disconnect( $cfg->{'ssh_connect'}, 30 );
    if ( $out == 0 ) {
      $logger->error("Unable to disconnect from remote machine");
    }
  }
}

sub connection_test {
  my $client = shift;
  $cfg = shift;

  $logger = XormonLogger->new('qnap', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  my $test = Test->new($client);

  $cfg->{'password_enable'} = (!$cfg->{'isKeyFile'});
  $cfg->{'SSH'}       = "ssh";
  my $error = 1;
  my @rows;
  ###################
  #main
  my @variables = ("hostcfg_id", "hostalias", "sshHost", "sshPort", "sshUsername", "isKeyFile", "qnapName");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
    if (!$cfg->{'isKeyFile'}){
      unless ( exists $cfg->{'sshPassword'} ) {
        $test->addSection("Validation of variables", 0 , "Cannot continue, sshPassword is not configured!");
        return $test->stop();
      }
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'sshHost'},$cfg->{'sshPort'});
  $test->addSection("Network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }

  if ( $cfg->{'password_enable'} == 1 ) {
      $cfg->{'ssh_connect'} = Xormon::openssh_new( $cfg->{'sshHost'}, $cfg->{'sshPort'}, $cfg->{'sshUsername'}, 30, $cfg->{'sshPassword'} );
      if ( $cfg->{'ssh_connect'} =~ m/^Error|^ERROR/ ) {
        $test->addSection("SSH connection", 0 , "$cfg->{'ssh_connect'}");
        return $test->stop();
      }
      else {
        $test->addSection("SSH connection", 1 , "Connection ok");
      }
  }
  get_cfg();

  if ( defined $cfg_in{'firmware_ver'}{'version'} && $cfg_in{'firmware_ver'}{'version'} ne "" ) {
    push(@rows, "FirmWare Version: $cfg_in{'firmware_ver'}{'version'}\n");
  }
  if ( defined $cfg_in{'nas_uuid'} && $cfg_in{'nas_uuid'} ne "" ) {
    push(@rows, "NAS UUID: $cfg_in{'nas_uuid'}\n");
  }
  if ( defined $cfg_in{'cluster_uuid'} && $cfg_in{'cluster_uuid'} ne "" ) {
    push(@rows, "Cluster UUID: $cfg_in{'cluster_uuid'}\n");
  }
  if ( defined $cfg_in{'added_ip'} && $cfg_in{'added_ip'} ne "" ) {
    push(@rows, "Added IP: $cfg_in{'added_ip'}\n");
  }
  if ( defined $cfg_in{'agent_version'} && $cfg_in{'agent_version'} ne "" ) {
    push(@rows, "Agent Version: $cfg_in{'agent_version'}\n");
  }
  if ( defined $cfg_in{'model_name'} && $cfg_in{'model_name'} ne "" ) {
    push(@rows, "FirmWare model name: $cfg_in{'model_name'}\n");
  }
  if ( defined $cfg_in{'hostname'} && $cfg_in{'hostname'} ne "" ) {
    push(@rows, "Hostname: $cfg_in{'hostname'}\n");
  }

  my $path = "/opt/qnap-cms/qnap-cms/rrds/$cfg_in{'cluster_uuid'}/$cfg_in{'nas_uuid'}/";

  ## SET TIME
  my $dif_time = time() - 3600;

  my $cmd_fetch          = 'cd ' . $path . '; for rrd in `ls | egrep -v "_smartattr-|_temp.rrd$|^network_all_|^network_bond|^enc_|^fan_|^mem_swap_|^multicpu_|_fan.rrd$|^shared_|ssd_all_count.rrd|^vol_all_|^pool_all_|^vol_.*_allocated\.rrd|^vol_.*_free\.rrd|^vol_.*_total\.rrd|^vol_.*_used.*rrd|^pool_.*_allocated\.rrd|^pool_.*_free\.rrd|^pool_.*_total\.rrd|^pool_.*_used.*rrd"`; do echo "RRD:$rrd"; rrdtool fetch $rrd AVERAGE -s ' . $dif_time . ';break; done';
  my @rrdtool_fetch_data = run_ssh($cmd_fetch);

  if ( !@rrdtool_fetch_data ) {
    push(@rows, "array rrdtool_fetch data must be set\n");
    $error = 0;
    #close_session();
    #exit 1;
  }
  else {
    push(@rows, "\nPerformance data: OK");
    close_session();
    #exit;
  }

  $test->addSection("Data", $error , \@rows);
  return $test->stop();

}
1;
