package RUBRIK;

use lib '../../../perl-modules';

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;
use Data::Dumper;
use Date::Parse;
use POSIX qw( strftime );
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use Xormon;
use JSON qw(encode_json decode_json);
use Test;
use XormonLogger;

my %data;
my $xormon;
my $cfg;
my $logger;
my $ms_status;

sub collect {
  $xormon = shift;
  $cfg    = shift;
  #print Dumper $cfg;
  $ms_status = "successful";
  #
  # test main variables
  #
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")    && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!")     && return "error"; }
  $logger = XormonLogger->new( 'rubrik', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'apiHost1'} )      { $logger->error("Cannot continue, 'apiHost1' is not configured!")      && return "error"; }
  unless ( exists $cfg->{'apiPort'} )       { $logger->error("Cannot continue, 'apiPort' is not configured!")       && return "error"; }
  unless ( exists $cfg->{'isToken'} )       { $logger->error("Cannot continue, 'isToken' is not configured!")       && return "error"; }

  my $api_ip       = $cfg->{'apiHost1'};
  my $ip2          = $cfg->{'apiHost2'}      || "";
  my $ip3          = $cfg->{'apiHost3'}      || "";
  my $user_token   = $cfg->{'apiToken'}      || "";

  my $time         = time();
  my $return;
  my %data;

  my @ips = ();

  push( @ips, $api_ip );
  if ( defined $ip2 && $ip2 ne "" ) {
    push( @ips, $ip2 );
  }
  if ( defined $ip3 && $ip3 ne "" ) {
    push( @ips, $ip3 );
  }

  $xormon->startJob('fetch', $cfg->{hostalias}, $$);

  # TOKEN MODE
  if ( $cfg->{'isToken'} ) {
    if ( !defined $user_token || $user_token eq "" ) {
      $logger->error("USER TOKEN is not defined!") && return "error";
    }

    foreach my $ip (@ips) {
      my ( $ret, $date_ret ) = login_with_token("https://$ip/api/v1/cluster/me");
      if ( $ret && $date_ret ) {
        $cfg->{'api_ip'}    = $ip;
        $cfg->{'st_time'}   = $date_ret;
        $cfg->{'token'}     = $user_token;
        $logger->info ("Login successful for IP $ip with user token!");
        last;
      }
      else {
        $logger->error("Login failed to $ip with user token!") && next;
      }
    }
  }
  else {
  # USER MODE
    if ( !defined $cfg->{'apiUsername'} || $cfg->{'apiUsername'} eq "" || !defined $cfg->{'apiPassword'} || $cfg->{'apiPassword'} eq "" ) {
      $logger->error("USER or PASSWORD is not defined!") && return "error";
    }
    foreach my $ip (@ips) {
      ## login and get token
      my ( $token, $api_date ) = get_api_token("https://$ip/api/v1/session");

      if ( $token && $api_date ) {
        $logger->info ("Login successful for IP $ip! Token granted!");
        $cfg->{'token'} = $token;
        $cfg->{'st_time'} = $api_date;
        $cfg->{'api_ip'} = $ip;
        last;
      }
      else {
        $logger->error("Token not granted!") && next;
      }
    }
  }

  if ( !$cfg->{'token'} ) {
    $xormon->isConnected(0, "Could not login to API");
    $logger->error("Could not login to API! Exiting...") && return "error";
  }
  elsif ( !$cfg->{'st_time'} ) {
    $xormon->isConnected(0, "No time value from API");
    $logger->error("No time value from API! Exiting...") && return "error";
  }
  else {
    # convert apitime to unixtime
    my $api_date = $cfg->{'st_time'};
    $cfg->{'timestamp'} = str2time("$api_date");
    $xormon->isConnected(1);
  }

  # collect perf data
  get_data();
  health_status();

  #remove token if user mode was used
  if ( !$cfg->{'isToken'} ) {
    my $logout = logout("https://$api_ip/api/v1/session/me");
    if ( $logout == 1) {
      $logger->info("Logout successful");
    }
    else {
      $logger->error("Logout not successful");
    }
  }

  # save data to DB
  print_data();

  return $ms_status;
}

sub get_data {

  my $clu_id = "";
  my $api_ip = $cfg->{'api_ip'};
  my $return = new_get_request("https://$api_ip/api/v1/cluster/me");

  # CLUSTER
  if ($return) {
    if ( defined $return->{'id'} && $return->{'id'} ne "" ) {
      $data{'cluster'}{'conf'}{'id'} = $return->{'id'};
      $clu_id = $return->{'id'};
    }
    if ( defined $return->{'name'} && $return->{'name'} ne "" ) {
      $data{'cluster'}{'conf'}{'name'} = $return->{'name'};
    }
    if ( defined $return->{'version'} && $return->{'version'} ne "" ) {
      $data{'cluster'}{'conf'}{'version'} = $return->{'version'};
    }
    if ( defined $return->{'apiVersion'} && $return->{'apiVersion'} ne "" ) {
      $data{'cluster'}{'conf'}{'apiVersion'} = $return->{'apiVersion'};
    }
  }

  ###############
  ### STORAGE ###
  ###############
  $return = new_get_request("https://$api_ip/api/internal/cluster/$clu_id/io_stats?range=-5min");
  ## IO
  if ( defined $return->{'iops'} && $return->{'iops'} ne "" ) {
    $xormon->hasData(1);
    my $io_ref = $return->{'iops'};
    if ( defined $io_ref->{'readsPerSecond'} && $io_ref->{'readsPerSecond'} ne "" && ref( $io_ref->{'readsPerSecond'} ) eq "ARRAY" ) {
      $data{'cluster'}{'perf'}{'io_read'} = get_value( $io_ref->{'readsPerSecond'} );
    }
    if ( defined $io_ref->{'writesPerSecond'} && $io_ref->{'writesPerSecond'} ne "" && ref( $io_ref->{'writesPerSecond'} ) eq "ARRAY" ) {
      $data{'cluster'}{'perf'}{'io_write'} = get_value( $io_ref->{'writesPerSecond'} );
    }
  }
  else {
    $xormon->hasData(0, "no return from cluster query");
  }
  ## DATA
  if ( defined $return->{'ioThroughput'} && $return->{'ioThroughput'} ne "" ) {
    my $data_ref = $return->{'ioThroughput'};
    if ( defined $data_ref->{'readBytePerSecond'} && $data_ref->{'readBytePerSecond'} ne "" && ref( $data_ref->{'readBytePerSecond'} ) eq "ARRAY" ) {
      $data{'cluster'}{'perf'}{'data_read'} = get_value( $data_ref->{'readBytePerSecond'} );
    }
    if ( defined $data_ref->{'writeBytePerSecond'} && $data_ref->{'writeBytePerSecond'} ne "" && ref( $data_ref->{'writeBytePerSecond'} ) eq "ARRAY" ) {
      $data{'cluster'}{'perf'}{'data_write'} = get_value( $data_ref->{'writeBytePerSecond'} );
    }
  }
  ## REPLICATION RX
  $return = new_get_request("https://$api_ip/api/internal/stats/replication/incoming/time_series?range=-5min");
  if ( $return && $return ne "" && ref($return) eq "ARRAY" ) {
    $data{'cluster'}{'perf'}{'repl_rx'} = get_value($return);
  }
  ## REPLICATION TX
  $return = new_get_request("https://$api_ip/api/internal/stats/replication/outgoing/time_series?range=-5min");
  if ( $return && $return ne "" && ref($return) eq "ARRAY" ) {
    $data{'cluster'}{'perf'}{'repl_tx'} = get_value($return);
  }
  ## ARCHIVE THROUGHPUT
  $return = new_get_request("https://$api_ip/api/internal/stats/archival/bandwidth/time_series?range=-5min");
  if ( $return && $return ne "" && ref($return) eq "ARRAY" ) {
    $data{'cluster'}{'perf'}{'archive'} = get_value($return);
  }
  ## INGEST
  $return = new_get_request("https://$api_ip/api/internal/stats/physical_ingest/time_series?range=-5min");
  if ( $return && $return ne "" && ref($return) eq "ARRAY" ) {
    $data{'cluster'}{'perf'}{'phys_ingest'} = get_value($return);
  }
  # MEMORY
  $return = new_get_request("https://$api_ip/api/internal/cluster/$clu_id/memory_capacity");
  if ( defined $return->{'bytes'} && $return->{'bytes'} ne "" ) {
    $data{'cluster'}{'conf'}{'memory'} = $return->{'bytes'};
  }
  # FLASH CAPACITY
  $return = new_get_request("https://$api_ip/api/internal/cluster/$clu_id/flash_capacity");
  if ( defined $return->{'bytes'} && $return->{'bytes'} ne "" ) {
    $data{'cluster'}{'conf'}{'flash_cap'} = $return->{'bytes'};
  }

  # DISK CAPACITY
  $return = new_get_request("https://$api_ip/api/internal/cluster/$clu_id/disk_capacity");
  if ( defined $return->{'bytes'} && $return->{'bytes'} ne "" ) {
    $data{'cluster'}{'conf'}{'disk_cap'} = $return->{'bytes'};
  }

  # CAPACITY
  $return = new_get_request("https://$api_ip/api/internal/stats/system_storage");
  if ( defined $return->{'total'} && $return->{'total'} ne "" ) {
    $data{'cluster'}{'perf'}{'total_cap'} = $return->{'total'};
  }
  if ( defined $return->{'used'} && $return->{'used'} ne "" ) {
    $data{'cluster'}{'perf'}{'used_cap'} = $return->{'used'};
  }
  if ( defined $return->{'available'} && $return->{'available'} ne "" ) {
    $data{'cluster'}{'perf'}{'free_cap'} = $return->{'available'};
  }
  if ( defined $return->{'snapshot'} && $return->{'snapshot'} ne "" ) {
    $data{'cluster'}{'perf'}{'snapshot_cap'} = $return->{'snapshot'};
  }
  if ( defined $return->{'liveMount'} && $return->{'liveMount'} ne "" ) {
    $data{'cluster'}{'perf'}{'livemount_cap'} = $return->{'liveMount'};
  }
  if ( defined $return->{'pendingSnapshot'} && $return->{'pendingSnapshot'} ne "" ) {
    $data{'cluster'}{'perf'}{'pendsnap_cap'} = $return->{'pendingSnapshot'};
  }
  if ( defined $return->{'cdp'} && $return->{'cdp'} ne "" ) {
    $data{'cluster'}{'perf'}{'cdp_cap'} = $return->{'cdp'};
  }
  if ( defined $return->{'miscellaneous'} && $return->{'miscellaneous'} ne "" ) {
    $data{'cluster'}{'perf'}{'misc_cap'} = $return->{'miscellaneous'};
  }

  # SNAPSHOT STORAGE PHYSICAL
  $return = new_get_request("https://$api_ip/api/internal/stats/snapshot_storage/physical");
  if ( defined $return->{'value'} && $return->{'value'} ne "" ) {
    $data{'cluster'}{'perf'}{'snapshot_physical'} = $return->{'value'};
  }

  # SNAPSHOT STORAGE INGESTED
  $return = new_get_request("https://$api_ip/api/internal/stats/snapshot_storage/ingested");
  if ( defined $return->{'value'} && $return->{'value'} ne "" ) {
    $data{'cluster'}{'perf'}{'snapshot_ingested'} = $return->{'value'};
  }

  # CLOUD STORAGE PHYSICAL
  $return = new_get_request("https://$api_ip/api/internal/stats/cloud_storage/physical");
  if ( defined $return->{'value'} && $return->{'value'} ne "" ) {
    $data{'cluster'}{'perf'}{'cloud_physical'} = $return->{'value'};
  }

  # CLOUD STORAGE INGESTED
  $return = new_get_request("https://$api_ip/api/internal/stats/cloud_storage/ingested");
  if ( defined $return->{'value'} && $return->{'value'} ne "" ) {
    $data{'cluster'}{'perf'}{'cloud_ingested'} = $return->{'value'};
  }

  # SLA DOMAINS
  $return = new_get_request("https://$api_ip/api/v2/sla_domain");
  if ( $return->{'data'} && $return->{'data'} ne "" && ref( $return->{'data'} ) eq "ARRAY" ) {
    foreach my $domain ( @{ $return->{'data'} } ) {
      if ( defined $domain->{'id'} && $domain->{'id'} ne "" ) {
        my $dom_id = $domain->{'id'};
        if ( defined $domain->{'name'} && $domain->{'name'} ne "" ) {
          $data{'sla_domain'}{$dom_id}{'name'} = $domain->{'name'};
        }
        my $n_return = new_get_request("https://$api_ip/api/internal/stats/sla_domain_storage/$dom_id");
        if ( defined $n_return->{'value'} && $n_return->{'value'} ne "" ) {
          $data{'sla_domain'}{$dom_id}{'cap'} = $n_return->{'value'};
        }
      }
    }
  }

  # AVG INCREMENT PER DAY
  $return = new_get_request("https://$api_ip/api/internal/stats/average_storage_growth_per_day");
  if ( defined $return->{'bytes'} && $return->{'bytes'} ne "" ) {
    $data{'cluster'}{'perf'}{'avg_daily_inc'} = $return->{'bytes'};
  }

  # TIME TO FILL UP
  $return = new_get_request("https://$api_ip/api/internal/stats/runway_remaining");
  if ( defined $return->{'days'} && $return->{'days'} ne "" ) {
    $data{'cluster'}{'perf'}{'days_to_fill'} = $return->{'days'};
  }


  #############
  ### NODES ###
  #############
  ## STATS
  $return = new_get_request("https://$api_ip/api/internal/node/stats?range=-5min");

  #print Dumper $return;
  if ( defined $return->{'data'} && $return->{'data'} ne "" && ref( $return->{'data'} ) eq "ARRAY" ) {
    foreach my $node ( @{ $return->{'data'} } ) {

      #print Dumper $node;
      if ( defined $node->{'id'} && $node->{'id'} ne "" ) {
        my $node_id = $node->{'id'};
        if ( $node_id =~ m/cluster:::/ ) {
          $node_id =~ s/cluster::://g;
        }
        ## PERF
        # cpu
        if ( defined $node->{'cpuStat'} && $node->{'cpuStat'} ne "" && ref( $node->{'cpuStat'} ) eq "ARRAY" ) {
          $data{'node'}{$node_id}{'perf'}{'cpu'} = get_value( $node->{'cpuStat'} );
        }

        # io
        if ( defined $node->{'iops'} && $node->{'iops'} ne "" ) {
          my $io_ref = $node->{'iops'};
          if ( defined $io_ref->{'readsPerSecond'} && $io_ref->{'readsPerSecond'} ne "" && ref( $io_ref->{'readsPerSecond'} ) eq "ARRAY" ) {
            $data{'node'}{$node_id}{'perf'}{'io_read'} = get_value( $io_ref->{'readsPerSecond'} );
          }
          if ( defined $io_ref->{'writesPerSecond'} && $io_ref->{'writesPerSecond'} ne "" && ref( $io_ref->{'writesPerSecond'} ) eq "ARRAY" ) {
            $data{'node'}{$node_id}{'perf'}{'io_write'} = get_value( $io_ref->{'writesPerSecond'} );
          }
        }

        # data
        if ( defined $node->{'ioThroughput'} && $node->{'ioThroughput'} ne "" ) {
          my $data_ref = $node->{'ioThroughput'};
          if ( defined $data_ref->{'readBytePerSecond'} && $data_ref->{'readBytePerSecond'} ne "" && ref( $data_ref->{'readBytePerSecond'} ) eq "ARRAY" ) {
            $data{'node'}{$node_id}{'perf'}{'data_read'} = get_value( $data_ref->{'readBytePerSecond'} );
          }
          if ( defined $data_ref->{'writeBytePerSecond'} && $data_ref->{'writeBytePerSecond'} ne "" && ref( $data_ref->{'writeBytePerSecond'} ) eq "ARRAY" ) {
            $data{'node'}{$node_id}{'perf'}{'data_write'} = get_value( $data_ref->{'writeBytePerSecond'} );
          }
        }


        # network
        if ( defined $node->{'networkStat'} && $node->{'networkStat'} ne "" ) {
          my $net_ref = $node->{'networkStat'};
          if ( defined $net_ref->{'bytesReceived'} && $net_ref->{'bytesReceived'} ne "" && ref( $net_ref->{'bytesReceived'} ) eq "ARRAY" ) {
            $data{'node'}{$node_id}{'perf'}{'net_rx'} = get_value( $net_ref->{'bytesReceived'} );
          }
          if ( defined $net_ref->{'bytesTransmitted'} && $net_ref->{'bytesTransmitted'} ne "" && ref( $net_ref->{'bytesTransmitted'} ) eq "ARRAY" ) {
            $data{'node'}{$node_id}{'perf'}{'net_tx'} = get_value( $net_ref->{'bytesTransmitted'} );
          }
        }

        # CONF
        $return = new_get_request("https://$api_ip/api/internal/node/$node_id");

        #print Dumper $return;
        if ( defined $return->{'hostname'} && $return->{'hostname'} ne "" ) {
          $data{'node'}{$node_id}{'conf'}{'hostname'} = $return->{'hostname'};
        }
        if ( defined $return->{'brikId'} && $return->{'brikId'} ne "" ) {
          $data{'node'}{$node_id}{'conf'}{'brikid'} = $return->{'brikId'};
        }
        if ( defined $return->{'ipAddress'} && $return->{'ipAddress'} ne "" ) {
          $data{'node'}{$node_id}{'conf'}{'ip'} = $return->{'ipAddress'};
        }
        if ( defined $return->{'status'} && $return->{'status'} ne "" ) {
          $data{'node'}{$node_id}{'conf'}{'status'} = $return->{'status'};
        }
        if ( defined $return->{'cpuCores'} && $return->{'cpuCores'} ne "" ) {
          $data{'node'}{$node_id}{'conf'}{'cores'} = $return->{'cpuCores'};
        }
        if ( defined $return->{'ram'} && $return->{'ram'} ne "" ) {
          $data{'node'}{$node_id}{'conf'}{'ram'} = $return->{'ram'};
        }

        ## NODE DRIVES
        if ( defined $return->{'hdd'} && $return->{'hdd'} ne "" && ref( $return->{'hdd'} ) eq "ARRAY" ) {
          foreach my $hdd ( @{ $return->{'hdd'} } ) {

            #print Dumper $hdd;
            if ( defined $hdd->{'id'} && $hdd->{'id'} ne "" ) {
              my $hdd_id = $hdd->{'id'};
              if ( defined $hdd->{'nodeId'} && $hdd->{'nodeId'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'node_id'} = $hdd->{'nodeId'};
              }
              if ( defined $hdd->{'diskType'} && $hdd->{'diskType'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'type'} = $hdd->{'diskType'};
              }
              if ( defined $hdd->{'status'} && $hdd->{'status'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'status'} = $hdd->{'status'};
              }
              if ( defined $hdd->{'isDegraded'} && $hdd->{'isDegraded'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'degraded'} = $hdd->{'isDegraded'};
              }
              if ( defined $hdd->{'path'} && $hdd->{'path'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'path'} = $hdd->{'path'};
              }
              if ( defined $hdd->{'capacityBytes'} && $hdd->{'capacityBytes'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'total_cap'} = $hdd->{'capacityBytes'};
              }
              if ( defined $hdd->{'unallocatedBytes'} && $hdd->{'unallocatedBytes'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'unalloc_cap'} = $hdd->{'unallocatedBytes'};
              }
              if ( defined $hdd->{'usableBytes'} && $hdd->{'usableBytes'} ne "" ) {
                $data{'disk'}{$node_id}{$hdd_id}{'usable_cap'} = $hdd->{'usableBytes'};
              }
            }
          }
        }
        if ( defined $return->{'ssd'} && $return->{'ssd'} ne "" && ref( $return->{'ssd'} ) eq "ARRAY" ) {
          foreach my $ssd ( @{ $return->{'ssd'} } ) {

            #print Dumper $ssd;
            if ( defined $ssd->{'id'} && $ssd->{'id'} ne "" ) {
              my $ssd_id = $ssd->{'id'};
              if ( defined $ssd->{'nodeId'} && $ssd->{'nodeId'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'node_id'} = $ssd->{'nodeId'};
              }
              if ( defined $ssd->{'diskType'} && $ssd->{'diskType'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'type'} = $ssd->{'diskType'};
              }
              if ( defined $ssd->{'status'} && $ssd->{'status'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'status'} = $ssd->{'status'};
              }
              if ( defined $ssd->{'isDegraded'} && $ssd->{'isDegraded'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'degraded'} = $ssd->{'isDegraded'};
              }
              if ( defined $ssd->{'path'} && $ssd->{'path'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'path'} = $ssd->{'path'};
              }
              if ( defined $ssd->{'capacityBytes'} && $ssd->{'capacityBytes'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'total_cap'} = $ssd->{'capacityBytes'};
              }
              if ( defined $ssd->{'unallocatedBytes'} && $ssd->{'unallocatedBytes'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'unalloc_cap'} = $ssd->{'unallocatedBytes'};
              }
              if ( defined $ssd->{'usableBytes'} && $ssd->{'usableBytes'} ne "" ) {
                $data{'disk'}{$node_id}{$ssd_id}{'usable_cap'} = $ssd->{'usableBytes'};
              }
            }
          }    # end foreach $ssd
        }    # end if defined $return->{'ssd'}
      }    # end if defined $node{id}
    }    # end foreach $node
  }    # end if defined $return->{'data'}



  #print Dumper %data;
}

sub print_data {
  my @architecture;
  my $data_out = "";
  my %conf;
  my $st_uuid = $cfg->{'hostcfg_id'};
  my $rc;
  my $status;
  my $time = $cfg->{'timestamp'};

  ##################
  ## CLUSTER arch ##
  ##################
  # my $hash_key = Xormon::create_hash_key($st_uuid."SYSTEM"); $hash_key = lc($hash_key);
  my %device_arch = (
    "item_id" => $st_uuid,
    "label" => $cfg->{'hostalias'},
    "hw_type" => "rubrik",
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $st_uuid
  );
  push(@architecture, \%device_arch);

  # my $arch = {
  #     "item_id" => $hash_key,
  #     "label" => $cfg->{'hostalias'},
  #     "hw_type" => "rubrik",
  #     "subsystem" => "system",
  #     "parents" => [$cfg->{'hostcfg_id'}],
  #   };
  # push(@architecture, $arch);

  $data_out .= "uuid;time;read_io;write_io;io_rate;read;write;data_rate;block_size;block_size_r;block_size_w;archive;replication_in;replication_out;physical_ingest;capacity_total;capacity_used;capacity_free;capacity_used_percent;snapshot_capacity_used;snapshot_capacity_pending;livemount_capacity_used;cdp_capacity_used;system_capacity_used;snapshot_capacity_physical;snapshot_capacity_ingested;snapshot_capacity_reduction_percent;cloud_physical;cloud_ingested;avg_daily_inc;days_to_fill\n";

  ## system conf print
  if ( defined $data{'cluster'}{'conf'}{'id'} && $data{'cluster'}{'conf'}{'id'} ne "" ) {
    $conf{$st_uuid}{'id'} = $data{'cluster'}{'conf'}{'id'};
    $conf{$st_uuid}{'model'} = 'Rubrik CDM';
  }
  if ( defined $data{'cluster'}{'conf'}{'name'} && $data{'cluster'}{'conf'}{'name'} ne "" ) {
    $conf{$st_uuid}{'node_name'} = $data{'cluster'}{'conf'}{'name'};
  }
  if ( defined $data{'cluster'}{'conf'}{'version'} && $data{'cluster'}{'conf'}{'version'} ne "" ) {
    $conf{$st_uuid}{'version'} = $data{'cluster'}{'conf'}{'version'};
  }
  if ( defined $data{'cluster'}{'conf'}{'apiVersion'} && $data{'cluster'}{'conf'}{'apiVersion'} ne "" ) {
    $conf{$st_uuid}{'apiVersion'} = $data{'cluster'}{'conf'}{'apiVersion'};
  }
  if ( defined $data{'cluster'}{'conf'}{'memory'} && $data{'cluster'}{'conf'}{'memory'} ne "" ) {
    $conf{$st_uuid}{'memory'} = $data{'cluster'}{'conf'}{'memory'};
  }
  if ( defined $data{'cluster'}{'conf'}{'disk_cap'} && $data{'cluster'}{'conf'}{'disk_cap'} ne "" ) {
    $conf{$st_uuid}{'disk_capacity'} = $data{'cluster'}{'conf'}{'disk_cap'};
  }
  if ( defined $data{'cluster'}{'conf'}{'flash_cap'} && $data{'cluster'}{'conf'}{'flash_cap'} ne "" ) {
    $conf{$st_uuid}{'flash_capacity'} = $data{'cluster'}{'conf'}{'flash_cap'};
  }

  ## add ip to conf db
  $conf{$st_uuid}{'ip'} = $cfg->{'api_ip'};

  my $io_r           = "";
  my $io_w           = "";
  my $io_rate        = "";
  my $data_r         = "";
  my $data_w         = "";
  my $data_rate      = "";
  my $block_size     = "";
  my $block_size_r   = "";
  my $block_size_w   = "";
  my $data_arch      = "";
  my $repl_rx        = "";
  my $repl_tx        = "";
  my $phys_ingest    = "";
  my $total          = "";
  my $used           = "";
  my $free           = "";
  my $used_pct       = "";
  my $snapshot_cap   = "";
  my $pendsnap_cap   = "";
  my $livemount_cap  = "";
  my $cdp_cap        = "";
  my $misc_cap       = "";
  my $snap_physical  = "";
  my $snap_ingested  = "";
  my $snap_reduction = 0;
  my $cloud_physical = "";
  my $cloud_ingested = "";
  my $avg_daily_inc  = "";
  my $days_to_fill   = "";

  if ( defined $data{'cluster'}{'perf'}{'io_read'} && $data{'cluster'}{'perf'}{'io_read'} ne "" ) {
    $io_r    = $data{'cluster'}{'perf'}{'io_read'};
    $io_rate = $io_r;
  }
  if ( defined $data{'cluster'}{'perf'}{'io_write'} && $data{'cluster'}{'perf'}{'io_write'} ne "" ) {
    $io_w = $data{'cluster'}{'perf'}{'io_write'};
    if ( Xormon::isdigit($io_r) ) {
      $io_rate = $io_rate + $io_w;
    }
    else {
      $io_rate = $io_w;
    }
  }
  if ( defined $data{'cluster'}{'perf'}{'data_read'} && $data{'cluster'}{'perf'}{'data_read'} ne "" ) {
    $data_r    = $data{'cluster'}{'perf'}{'data_read'};
    $data_rate = $data_r;
  }
  if ( defined $data{'cluster'}{'perf'}{'data_write'} && $data{'cluster'}{'perf'}{'data_write'} ne "" ) {
    $data_w = $data{'cluster'}{'perf'}{'data_write'};
    if ( Xormon::isdigit($data_r) ) {
      $data_rate = $data_rate + $data_w;
    }
    else {
      $data_rate = $data_w;
    }
  }
  # calculate block size
  if ( Xormon::isdigit($io_rate) && Xormon::isdigit($data_rate) ) {
    if ( $io_rate == 0 ) {
      $block_size = 0;
    }
    else{
      $block_size = Xormon::formatNumberByType("float4", ($data_rate / $io_rate));
    }
  }
  if ( Xormon::isdigit($io_r) && Xormon::isdigit($data_r) ) {
    if ( $io_r == 0 ) {
      $block_size_r = 0;
    }
    else{
      $block_size_r = Xormon::formatNumberByType("float4", ($data_r / $io_r));
    }
  }
  if ( Xormon::isdigit($io_w) && Xormon::isdigit($data_w) ) {
    if ( $io_w == 0 ) {
      $block_size_w = 0;
    }
    else{
      $block_size_w = Xormon::formatNumberByType("float4", ($data_w / $io_w));
    }
  }

  if ( defined $data{'cluster'}{'perf'}{'archive'} && $data{'cluster'}{'perf'}{'archive'} ne "" ) {
    $data_arch = $data{'cluster'}{'perf'}{'archive'};
  }
  if ( defined $data{'cluster'}{'perf'}{'repl_rx'} && $data{'cluster'}{'perf'}{'repl_rx'} ne "" ) {
    $repl_rx = $data{'cluster'}{'perf'}{'repl_rx'};
  }
  if ( defined $data{'cluster'}{'perf'}{'repl_tx'} && $data{'cluster'}{'perf'}{'repl_tx'} ne "" ) {
    $repl_tx = $data{'cluster'}{'perf'}{'repl_tx'};
  }
  if ( defined $data{'cluster'}{'perf'}{'phys_ingest'} && $data{'cluster'}{'perf'}{'phys_ingest'} ne "" ) {
    $phys_ingest = $data{'cluster'}{'perf'}{'phys_ingest'};
  }
  if ( defined $data{'cluster'}{'perf'}{'total_cap'} && $data{'cluster'}{'perf'}{'total_cap'} ne "" ) {
    $total      = $data{'cluster'}{'perf'}{'total_cap'};
  }
  if ( defined $data{'cluster'}{'perf'}{'used_cap'} && $data{'cluster'}{'perf'}{'used_cap'} ne "" ) {
    $used      = $data{'cluster'}{'perf'}{'used_cap'};
  }
  if ( defined $data{'cluster'}{'perf'}{'free_cap'} && $data{'cluster'}{'perf'}{'free_cap'} ne "" ) {
    $free      = $data{'cluster'}{'perf'}{'free_cap'};
  }
  # calculate used percent
  if ( Xormon::isdigit($total) && Xormon::isdigit($used) && ($total != 0) ){
    $used_pct = Xormon::formatNumberByType("float4", $used / ($total / 100) );
  }
  if ( defined $data{'cluster'}{'perf'}{'snapshot_cap'} && $data{'cluster'}{'perf'}{'snapshot_cap'} ne "" ) {
    $snapshot_cap = $data{'cluster'}{'perf'}{'snapshot_cap'};
  }
  if ( defined $data{'cluster'}{'perf'}{'pendsnap_cap'} && $data{'cluster'}{'perf'}{'pendsnap_cap'} ne "" ) {
    $pendsnap_cap = $data{'cluster'}{'perf'}{'pendsnap_cap'};
  }
  if ( defined $data{'cluster'}{'perf'}{'livemount_cap'} && $data{'cluster'}{'perf'}{'livemount_cap'} ne "" ) {
    $livemount_cap = $data{'cluster'}{'perf'}{'livemount_cap'};
  }
  if ( defined $data{'cluster'}{'perf'}{'cdp_cap'} && $data{'cluster'}{'perf'}{'cdp_cap'} ne "" ) {
    $cdp_cap = $data{'cluster'}{'perf'}{'cdp_cap'};
  }
  if ( defined $data{'cluster'}{'perf'}{'misc_cap'} && $data{'cluster'}{'perf'}{'misc_cap'} ne "" ) {
    $misc_cap = $data{'cluster'}{'perf'}{'misc_cap'};
  }
  if ( defined $data{'cluster'}{'perf'}{'snapshot_physical'} && $data{'cluster'}{'perf'}{'snapshot_physical'} ne "" ) {
    $snap_physical = $data{'cluster'}{'perf'}{'snapshot_physical'};
  }
  if ( defined $data{'cluster'}{'perf'}{'snapshot_ingested'} && $data{'cluster'}{'perf'}{'snapshot_ingested'} ne "" ) {
    $snap_ingested = $data{'cluster'}{'perf'}{'snapshot_ingested'};
  }
  # calculate percent reduction
  if ( Xormon::isdigit($snap_physical) && Xormon::isdigit($snap_ingested) ) {
    if ( $snap_ingested != 0 ) {
      $snap_reduction = sprintf( '%.2f', ( $snap_ingested - $snap_physical ) / ( $snap_ingested / 100 ) );
    }
  }
  if ( defined $data{'cluster'}{'perf'}{'cloud_physical'} && $data{'cluster'}{'perf'}{'cloud_physical'} ne "" ) {
    $cloud_physical = $data{'cluster'}{'perf'}{'cloud_physical'};
  }
  if ( defined $data{'cluster'}{'perf'}{'cloud_ingested'} && $data{'cluster'}{'perf'}{'cloud_ingested'} ne "" ) {
    $cloud_ingested = $data{'cluster'}{'perf'}{'cloud_ingested'};
  }
  if ( defined $data{'cluster'}{'perf'}{'avg_daily_inc'} && $data{'cluster'}{'perf'}{'avg_daily_inc'} ne "" ) {
    $avg_daily_inc = $data{'cluster'}{'perf'}{'avg_daily_inc'};
  }
  if ( defined $data{'cluster'}{'perf'}{'days_to_fill'} && $data{'cluster'}{'perf'}{'days_to_fill'} ne "" ) {
    $days_to_fill = $data{'cluster'}{'perf'}{'days_to_fill'};
  }

  $data_out .= "$st_uuid;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$block_size;$block_size_r;$block_size_w;$data_arch;$repl_rx;$repl_tx;$phys_ingest;$total;$used;$free;$used_pct;$snapshot_cap;$pendsnap_cap;$livemount_cap;$cdp_cap;$misc_cap;$snap_physical;$snap_ingested;$snap_reduction;$cloud_physical;$cloud_ingested;$avg_daily_inc;$days_to_fill\n";

  #SAVE CLUSTER
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"rubrik","device");

  #status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("CLUSTER status : $status");
  if ( $status ne 'successful' ) { $ms_status = $status; }

  ###############
  ## POOL arch ##
  ###############
  my $hash_key = Xormon::create_hash_key($st_uuid."POOL"); $hash_key = lc($hash_key);
  my $rub_name = $conf{$st_uuid}{'name'} || "POOL";
  my $arch = {
      "item_id" => $hash_key,
      "label" => $rub_name,
      "hw_type" => "rubrik",
      "subsystem" => "pool",
      "parents" => [$cfg->{'hostcfg_id'}],
      "hostcfg_id" => $cfg->{'hostcfg_id'}
    };
  push(@architecture, $arch);

  $conf{$hash_key}{'capacity_total'} = $total;
  $conf{$hash_key}{'capacity_used'} = $used;
  $conf{$hash_key}{'capacity_free'} = $free;

  $data_out = "uuid;time;read_io;write_io;io_rate;read;write;data_rate;block_size;block_size_r;block_size_w;archive;replication_in;replication_out;physical_ingest;capacity_total;capacity_used;capacity_free;capacity_used_percent;snapshot_capacity_used;snapshot_capacity_pending;livemount_capacity_used;cdp_capacity_used;system_capacity_used;snapshot_capacity_physical;snapshot_capacity_ingested;snapshot_capacity_reduction_percent;cloud_physical;cloud_ingested;avg_daily_inc;days_to_fill\n";
  $data_out .= "$hash_key;$time;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$block_size;$block_size_r;$block_size_w;$data_arch;$repl_rx;$repl_tx;$phys_ingest;$total;$used;$free;$used_pct;$snapshot_cap;$pendsnap_cap;$livemount_cap;$cdp_cap;$misc_cap;$snap_physical;$snap_ingested;$snap_reduction;$cloud_physical;$cloud_ingested;$avg_daily_inc;$days_to_fill\n";

  #SAVE POOL
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"rubrik","pool");

  #status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("POOL status : $status");
  if ( $status ne 'successful' ) { $ms_status = $status; }

  $data_out = "";

  ##########
  ## NODE ##
  ##########

  $data_out .= "uuid;time;cpu_system;read_io;write_io;io_rate;read;write;data_rate;network_rx;network_tx;network_rate\n";

  foreach my $node ( keys %{ $data{'node'} } ) {

    my $hash_key = Xormon::create_hash_key($st_uuid."NODE".$node); $hash_key = lc($hash_key);
    #print "$hash_key\n";

    my $arch = {
        "item_id" => $hash_key,
        "label" => $node,
        "hw_type" => "rubrik",
        "subsystem" => "node",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      };
    push(@architecture, $arch);

    # CONF
    if ( defined $data{'node'}{$node}{'conf'}{'brikid'} && $data{'node'}{$node}{'conf'}{'brikid'} ne "" ) {
      $conf{$hash_key}{'brik_id'} = $data{'node'}{$node}{'conf'}{'brikid'};
    }
    if ( defined $data{'node'}{$node}{'conf'}{'hostname'} && $data{'node'}{$node}{'conf'}{'hostname'} ne "" ) {
      $conf{$hash_key}{'hostname'} = $data{'node'}{$node}{'conf'}{'hostname'};
    }
    if ( defined $data{'node'}{$node}{'conf'}{'ip'} && $data{'node'}{$node}{'conf'}{'ip'} ne "" ) {
      $conf{$hash_key}{'ip'} = $data{'node'}{$node}{'conf'}{'ip'};
    }
    if ( defined $data{'node'}{$node}{'conf'}{'cores'} && $data{'node'}{$node}{'conf'}{'cores'} ne "" ) {
      $conf{$hash_key}{'cores'} = $data{'node'}{$node}{'conf'}{'cores'};
    }
    if ( defined $data{'node'}{$node}{'conf'}{'ram'} && $data{'node'}{$node}{'conf'}{'ram'} ne "" ) {
      $conf{$hash_key}{'ram'} = $data{'node'}{$node}{'conf'}{'ram'};
    }
    if ( defined $data{'node'}{$node}{'conf'}{'status'} && $data{'node'}{$node}{'conf'}{'status'} ne "" ) {
      $conf{$hash_key}{'status'} = $data{'node'}{$node}{'conf'}{'status'};
    }

    # PERF
    my $cpu       = "";
    my $io_r      = "";
    my $io_w      = "";
    my $io_rate   = "";
    my $data_r    = "";
    my $data_w    = "";
    my $data_rate = "";
    my $net_rx    = "";
    my $net_tx    = "";
    my $net_rate  = "";

    if ( defined $data{'node'}{$node}{'perf'}{'cpu'} && $data{'node'}{$node}{'perf'}{'cpu'} ne "" ) {
      $cpu = "$data{'node'}{$node}{'perf'}{'cpu'}";
    }
    if ( defined $data{'node'}{$node}{'perf'}{'io_read'} && $data{'node'}{$node}{'perf'}{'io_read'} ne "" ) {
      $io_r    = "$data{'node'}{$node}{'perf'}{'io_read'}";
      $io_rate = $io_r;
    }
    if ( defined $data{'node'}{$node}{'perf'}{'io_write'} && $data{'node'}{$node}{'perf'}{'io_write'} ne "" ) {
      $io_w = "$data{'node'}{$node}{'perf'}{'io_write'}";
      if ( Xormon::isdigit($io_r) ) {
        $io_rate = $io_rate + $io_w;
      }
      else {
        $io_rate = $io_w;
      }
    }
    if ( defined $data{'node'}{$node}{'perf'}{'data_read'} && $data{'node'}{$node}{'perf'}{'data_read'} ne "" ) {
      $data_r    = $data{'node'}{$node}{'perf'}{'data_read'};
      $data_rate = $data_r;
    }
    if ( defined $data{'node'}{$node}{'perf'}{'data_write'} && $data{'node'}{$node}{'perf'}{'data_write'} ne "" ) {
      $data_w = $data{'node'}{$node}{'perf'}{'data_write'};
      if ( Xormon::isdigit($data_r) ) {
        $data_rate = $data_rate + $data_w;
      }
      else {
        $data_rate = $data_w;
      }
    }
    if ( defined $data{'node'}{$node}{'perf'}{'net_rx'} && $data{'node'}{$node}{'perf'}{'net_rx'} ne "" ) {
      $net_rx   = $data{'node'}{$node}{'perf'}{'net_rx'};
      $net_rate = $net_rx;
    }
    if ( defined $data{'node'}{$node}{'perf'}{'net_tx'} && $data{'node'}{$node}{'perf'}{'net_tx'} ne "" ) {
      $net_tx = $data{'node'}{$node}{'perf'}{'net_tx'};
      if ( Xormon::isdigit($net_rx) ) {
        $net_rate = $net_rate + $net_tx;
      }
      else {
        $net_rate = $net_tx;
      }
    }
    $data_out .= "$hash_key;$time;$cpu;$io_r;$io_w;$io_rate;$data_r;$data_w;$data_rate;$net_rx;$net_tx;$net_rate\n";

  }

  #SAVE NODE
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"rubrik","node");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("NODE status : $status");
  if ( $status ne 'successful' ) { $ms_status = $status; }

  $data_out = "";

  ################
  ## SLA DOMAIN ##
  ################

  $data_out .= "uuid;time;capacity_used\n";

  foreach my $domain ( keys %{ $data{'sla_domain'} } ) {
    my $name = "$domain";

    if ( defined $data{'sla_domain'}{$domain}{'name'} && $data{'sla_domain'}{$domain}{'name'} ne "" ) {
      $name = $data{'sla_domain'}{$domain}{'name'};
    }

    my $hash_key = Xormon::create_hash_key($st_uuid."SLA".$domain); $hash_key = lc($hash_key);
    $conf{$hash_key}{'name'} = $name;

    my $arch = {
        "item_id" => $hash_key,
        "label" => $name,
        "hw_type" => "rubrik",
        "subsystem" => "sla_domain",
        "parents" => [$cfg->{'hostcfg_id'}],
        "hostcfg_id" => $cfg->{'hostcfg_id'}
      };
    push(@architecture, $arch);

    # PERF
    my $cap = "";

    if ( defined $data{'sla_domain'}{$domain}{'cap'} && Xormon::isdigit( $data{'sla_domain'}{$domain}{'cap'} ) ) {
      $cap = $data{'sla_domain'}{$domain}{'cap'};
    }
    $data_out .= "$hash_key;$time;$cap\n";
  }

  #SAVE SLA DOMAIN
  #print $data_out;
  $rc = $xormon->saveCSV($data_out,"rubrik","sla_domain");

  # status code
  if ($rc eq "201") {
    $status = "successful";
  } elsif ($rc eq "401") {
    $status = "unauthorized";
  } else {
    $status = "error";
  }
  $logger->info("SLA DOMAIN status : $status");
  if ( $status ne 'successful' ) { $ms_status = $status; }

  $data_out = "";

  #print Dumper \%conf;
  #save configuration
  $xormon->saveConfJSON(\%conf);

  #save architecture
  #print Dumper \@architecture;
  $xormon->saveArchitecture(\@architecture);

}

sub get_value {
  my $hash_ref = shift;
  my $i        = 0;
  my $sum      = 0;

  foreach my $minute ( @{$hash_ref} ) {

    #print Dumper $minute;
    if ( exists $minute->{'stat'} && $minute->{'stat'} ne "" && Xormon::isdigit( $minute->{'stat'} ) ) {
      my $value = $minute->{'stat'};
      $sum = $sum + $value;
      $i++;
    }
  }
  if ( $i > 0 ) {

    #my $avg = $sum / $i;
    my $avg = sprintf( '%.0f', $sum / $i );
    return $avg;
  }
  else {
    $logger->error("No time data found!");
    return "";
  }
}

sub connection_test {
  my $client = shift;
  $cfg = shift;

  my %conn_data;

  $logger = XormonLogger->new( 'rubrik', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  my $test = Test->new($client);

  # test main variables
  #
  # unless ( exists $cfg->{'hostcfg_id'} )    { push(@rows, "Cannot continue, 'hostcfg_idd' is not configured!");   return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'hostalias'} )     { push(@rows, "Cannot continue, 'hostalias' is not configured!");     return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiHost1'} )      { push(@rows, "Cannot continue, 'apiHost1' is not configured!");      return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'apiPort'} )       { push(@rows, "Cannot continue, 'apiPort' is not configured!");       return Xormon::conn_test(0, \@rows); }
  # unless ( exists $cfg->{'isToken'} )       { push(@rows, "Cannot continue, 'isToken' is not configured!");       return Xormon::conn_test(0, \@rows); }

  my @variables = ("hostcfg_id", "hostalias", "apiHost1", "apiPort", "isToken" );
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
  }
  $test->addSection("Validation of variables", 1 , "");

  my $api_user     = $cfg->{'apiUsername'}   || "";
  my $api_pass     = $cfg->{'apiPassword'}   || "";

  my $api_ip       = $cfg->{'apiHost1'}      || "";
  my $ip2          = $cfg->{'apiHost2'}      || "";
  my $ip3          = $cfg->{'apiHost3'}      || "";
  my $user_token   = $cfg->{'apiToken'}      || "";

  my $token    = "";
  my $api_date = "";
  my @host_arr = ();
  my @conn_arr = ();

  push( @host_arr, $api_ip );
  if ( defined $ip2 && $ip2 ne "" ) {
    push( @host_arr, $ip2 );
  }
  if ( defined $ip3 && $ip3 ne "" ) {
    push( @host_arr, $ip3 );
  }

  #### connection network test for each host first ####
  my @ip_rows;
  my $connected = 0;

  foreach my $conn_ip (@host_arr) {
    my ($code, $text) = Xormon::conntest_tcp($conn_ip,$cfg->{'apiPort'});
    push(@ip_rows, "$text\n");
    if ($code == 1){
      $connected = 1;
      push (@conn_arr, $conn_ip); # new array for conntest success
    }
  }
  $test->addSection("API host network connection", $connected, \@ip_rows);
  if ($connected == 0) {
    return $test->stop();
  }

  #### try to get data from network conntest hosts only ####
  @ip_rows = ();
  my $auth_ok = 0;

  ### user token
  if ( $cfg->{'isToken'} ) {
    if ( defined $user_token && $user_token ne "" ) {
      foreach my $host_ip (@conn_arr) {
        my ( $ret, $date_ret ) = login_with_token("https://$host_ip/api/v1/cluster/me");
        if ( $ret && $date_ret ) {
          $api_ip = $host_ip;
          $api_date = $date_ret;
          $cfg->{'token'} = $user_token;
          $token = $user_token;
          push (@ip_rows, "Connected with token to $host_ip\n");
          $auth_ok = 1;
          last;
        }
        else {
          push(@ip_rows, "Login failed to $host_ip with user token!\n");
          next;
        }
      }
    }
    else {
      $test->addSection("API authorization", 0, "Token NOT defined!");
      return $test->stop();
    }
    $test->addSection("API authorization", $auth_ok, \@ip_rows);
    if ($auth_ok == 0) {
      return $test->stop();
    }
  }
  ### user/pass
  elsif ( !$cfg->{'isToken'} ) {
    if ( !defined $api_user || $api_user eq "" || !defined $api_pass || $api_pass eq "" ) {
      $test->addSection("API authorization", 0, "Login failed! Username or password missing!");
      return $test->stop();
    }
    foreach my $host_ip (@conn_arr) {
      ( $token, $api_date ) = get_api_token("https://$host_ip/api/v1/session");
      if ( !$token ) {
        push(@ip_rows, "Login failed with username and password to $host_ip!\n");
        next;
      }
      else {
        $api_ip = $host_ip;
        $cfg->{'token'} = $token;
        $auth_ok = 1;
        push(@ip_rows, "Login successful with username and password to $host_ip\n");
        last;
      }
    }
    $test->addSection("API authorization", $auth_ok, \@ip_rows);
    if ($auth_ok == 0) {
      return $test->stop();
    }
  }
  else {
    $test->addSection("API authorization", 0, "Unknown authorization!");
    return $test->stop();
  }

  ## COLLECT TEST DATA
  my $return = "";
  my $clu_id = "";
  $return = new_get_request("https://$api_ip/api/v1/cluster/me");
  if ($return) {
    if ( defined $return->{'id'} && $return->{'id'} ne "" ) {
      $conn_data{'cluster'}{'conf'}{'id'} = $return->{'id'};
      $clu_id = $return->{'id'};
    }
    if ( defined $return->{'name'} && $return->{'name'} ne "" ) {
      $conn_data{'cluster'}{'conf'}{'name'} = $return->{'name'};
    }
    if ( defined $return->{'version'} && $return->{'version'} ne "" ) {
      $conn_data{'cluster'}{'conf'}{'version'} = $return->{'version'};
    }
    if ( defined $return->{'apiVersion'} && $return->{'apiVersion'} ne "" ) {
      $conn_data{'cluster'}{'conf'}{'apiVersion'} = $return->{'apiVersion'};
    }
  }
  if ( $clu_id ne "" ) {
    $return = new_get_request("https://$api_ip/api/internal/cluster/$clu_id/io_stats?range=-5min");
    ## IO
    if ( defined $return->{'iops'} && $return->{'iops'} ne "" ) {
      my $io_ref = $return->{'iops'};
      if ( defined $io_ref->{'readsPerSecond'} && $io_ref->{'readsPerSecond'} ne "" && ref( $io_ref->{'readsPerSecond'} ) eq "ARRAY" ) {
        $conn_data{'cluster'}{'perf'}{'io_read'} = $io_ref->{'readsPerSecond'};
      }
      if ( defined $io_ref->{'writesPerSecond'} && $io_ref->{'writesPerSecond'} ne "" && ref( $io_ref->{'writesPerSecond'} ) eq "ARRAY" ) {
        $conn_data{'cluster'}{'perf'}{'io_write'} = $io_ref->{'writesPerSecond'};
      }
    }
    ## DATA
    if ( defined $return->{'ioThroughput'} && $return->{'ioThroughput'} ne "" ) {
      my $data_ref = $return->{'ioThroughput'};
      if ( defined $data_ref->{'readBytePerSecond'} && $data_ref->{'readBytePerSecond'} ne "" && ref( $data_ref->{'readBytePerSecond'} ) eq "ARRAY" ) {
        $conn_data{'cluster'}{'perf'}{'data_read'} = $data_ref->{'readBytePerSecond'};
      }
      if ( defined $data_ref->{'writeBytePerSecond'} && $data_ref->{'writeBytePerSecond'} ne "" && ref( $data_ref->{'writeBytePerSecond'} ) eq "ARRAY" ) {
        $conn_data{'cluster'}{'perf'}{'data_write'} = $data_ref->{'writeBytePerSecond'};
      }
    }
  }

  if ( defined $conn_data{'cluster'}{'conf'} && $conn_data{'cluster'}{'conf'} ne "" ) {
    my @conf_rows;
    if ( exists $conn_data{'cluster'}{'conf'}{'name'} )        { push(@conf_rows, "name        : $conn_data{'cluster'}{'conf'}{name}\n"); }
    if ( exists $conn_data{'cluster'}{'conf'}{'id'} )          { push(@conf_rows, "id          : $conn_data{'cluster'}{'conf'}{id}\n"); }
    if ( exists $conn_data{'cluster'}{'conf'}{'version'} )     { push(@conf_rows, "version     : $conn_data{'cluster'}{'conf'}{version}\n"); }
    if ( exists $conn_data{'cluster'}{'conf'}{'apiVersion'} )  { push(@conf_rows, "API version : $conn_data{'cluster'}{'conf'}{apiVersion}\n"); }
    $test->addSection("API data", 1, \@conf_rows);
  }
  else {
    $test->addSection("API data", 0, "Could not get any system information!");
    return $test->stop();
  }

  if ( (exists $conn_data{'cluster'}{'perf'}{'data_read'} && ref( $conn_data{'cluster'}{'perf'}{'data_read'} ) eq "ARRAY") && (exists $conn_data{'cluster'}{'perf'}{'data_write'} && ref( $conn_data{'cluster'}{'perf'}{'data_write'} ) eq "ARRAY") && (exists $conn_data{'cluster'}{'perf'}{'io_read'} && ref( $conn_data{'cluster'}{'perf'}{'io_read'} ) eq "ARRAY") && (exists $conn_data{'cluster'}{'perf'}{'io_write'} && ref( $conn_data{'cluster'}{'perf'}{'io_write'} ) eq "ARRAY") ) {
    $test->addSection("API performance stats", 1, "Performance data OK" );
  }
  else {
    $test->addSection("API performance stats", 0, "Performance data not found!");
    return $test->stop();
  }

  if (!$cfg->{'isToken'}) {
    my $logout = logout("https://$api_ip/api/v1/session/me");
  }

  return $test->stop();
}

sub new_get_request {
  my $url    = shift;
  my $token  = $cfg->{'token'};
  my $result = "";

  my $agent   = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url );
  $request->header(
    'Accept'        => 'application/json',
    'Authorization' => 'Bearer ' . $token . ''
  );
  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    $logger->error("GET url           : $url");
    $logger->error("Request error     : $st_line");

    return 0;
  }

  #print Dumper $results;
  if ( defined $results->content && $results->content ne "" ) {
    my $result = decode_json( $results->content );
    return $result;
  }
  return 0;
}

sub login_with_token {
  my $url    = shift;
  my $token  = $cfg->{'apiToken'};
  my $result = "";

  my $agent   = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $request = HTTP::Request->new( GET => $url );
  $request->header(
    'Accept'        => 'application/json',
    'Authorization' => 'Bearer ' . $token . ''
  );
  my $results = $agent->request($request);

  if ( !$results->is_success ) {
    my $st_line = $results->status_line;
    $logger->error("GET url           : $url");
    $logger->error("Request error     : $st_line");

    return ( '0', '0' );
  }
  else {
    if ( defined $results->content && $results->content ne "" ) {
      my $result   = decode_json( $results->content );
      my $api_time = "";
      if ( defined $results->headers->{'date'} && $results->headers->{'date'} ne "" ) {
        $api_time = $results->headers->{'date'};
        return ( "$result", "$api_time" );
      }
      else {
        $logger->error("Storage time not found!");
      }
    }
    else {
      $logger->error("Content not found!");
    }
  }
  return ( '0', '0' );
}

sub get_api_token {

  my $api_user = $cfg->{'apiUsername'};
  my $api_pass = $cfg->{'apiPassword'};
  my $url      = shift;
  my $agent    = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );

  my $request = HTTP::Request->new( POST => $url );
  $request->header( 'Content-Type' => 'application/json' );
  $request->authorization_basic( $api_user, $api_pass );

  my $response = $agent->request($request);
  my $st_line  = $response->status_line;

  if ( !$response->is_success ) {
    my $st_line = $response->status_line;
    my $res_con = $response->content;
    $logger->error("POST url      : $url");
    $logger->error("Request error: $st_line");
    $logger->error("Request error: $res_con");

    return 0;
  }
  else {
    if ( defined $response->content && $response->content ne "" ) {
      my $result = decode_json( $response->content );
      if ( defined $result->{'token'} && $result->{'token'} ne "" ) {
        my $tok      = $result->{'token'};
        my $api_time = "";
        if ( defined $response->headers->{'date'} && $response->headers->{'date'} ne "" ) {
          $api_time = $response->headers->{'date'};
          return ( "$tok", "$api_time" );
        }
        else {
          $logger->error("Storage time not found!");
        }
      }
      else {
        $logger->error("Token not found in response! Login failed!");
      }
    }
    else {
      $logger->error("Token not found! Login failed!");
    }
  }
}

sub logout {
  my $url = shift;
  my $token = $cfg->{'token'};
  my $ua  = LWP::UserAgent->new( ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0 } );
  my $req = HTTP::Request->new( DELETE => $url );
  $req->header(
    'Accept'        => 'application/json',
    'Authorization' => 'Bearer ' . $token . ''
  );
  my $res = $ua->request($req);
  if ( !$res->is_success ) {
    return 0;
  }
  else {
    return 1;
  }
}

sub health_status {
  my @error_message  = ();
  my $act_timestamp = time();
  my $loc_time      = localtime($act_timestamp);
  my $week_old_time = $act_timestamp - 604800;
  $week_old_time = strftime( '%FT%TZ', localtime($week_old_time) );

  my %hs_data;
  my $api_ip         = $cfg->{'api_ip'};
  my $main_state     = "ok";
  # node
  if ( defined $data{'node'} && $data{'node'} ne "" ) {
    foreach my $node ( keys %{ $data{'node'} } ) {
      my $conf_ref = $data{'node'}{$node}{'conf'};
      my $status   = "";
      if ( defined $conf_ref->{'status'} && $conf_ref->{'status'} ne "" ) {
        $status = $conf_ref->{'status'};
        if ( $status ne "OK" ) {
          $main_state = "error";
          my $error_line = "NODE $node status: $status";
          push( @error_message, "$error_line\n" );
        }
      }
      my %hs_node;
      $hs_node{name}   = $node;
      $hs_node{status} = $status;
      push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{nodes} }, \%hs_node );
    }
  }
  # disk
  if ( defined $data{'disk'} && $data{'disk'} ne "" ) {
    foreach my $node ( keys %{ $data{'disk'} } ) {
      my $node_ref = $data{'disk'}{$node};
      foreach my $disk ( keys %{$node_ref} ) {
        my $disk_ref = $node_ref->{$disk};
        my $type     = "";
        my $status   = "";
        my $degraded = "FALSE";
        if ( defined $disk_ref->{'type'} && $disk_ref->{'type'} ne "" ) {
          $type = $disk_ref->{'type'};
        }
        if ( defined $disk_ref->{'status'} && $disk_ref->{'status'} ne "" ) {
          $status = $disk_ref->{'status'};
          if ( $status ne "ACTIVE" ) {
            $main_state = "error";
            my $error_line = "DISK $disk of $node status: $status";
            push( @error_message, "$error_line\n" );
          }
        }
        if ( defined $disk_ref->{'degraded'} && $disk_ref->{'degraded'} ne "" ) {
          my $deg_stat = $disk_ref->{'degraded'};
          if ($deg_stat) {    # boolean
            $degraded   = "TRUE";
            $main_state = "error";
            my $error_line = "DISK $disk of $node DEGRADED";
            push( @error_message, "$error_line\n" );
          }
        }
        my %hs_disk;
        $hs_disk{node}     = $node;
        $hs_disk{type}     = $type;
        $hs_disk{id}       = $disk;
        $hs_disk{status}   = $status;
        $hs_disk{degraded} = $degraded;
        push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{disks} }, \%hs_disk );
      }
    }
  }

  # events
  my $return = new_get_request("https://$api_ip/api/v1/event/latest?limit=50&before_date=$week_old_time&event_severity=Critical");
  #print Dumper $return;

  if ( defined $return->{'data'} && ref( $return->{'data'} ) eq "ARRAY" ) {
    foreach my $arr_event ( @{ $return->{'data'} } ) {
      if ( defined $arr_event->{'latestEvent'} && $arr_event->{'latestEvent'} ne "" ) {
        my $event       = $arr_event->{'latestEvent'};
        my $ev_id       = $event->{'id'};
        my $ev_severity = $event->{'eventSeverity'};
        my $ev_type     = $event->{'eventType'};
        my $ev_name     = $event->{'eventName'};
        my $ev_object_t = $event->{'objectType'};
        my $ev_object_n = $event->{'objectName'};
        my $ev_time     = $event->{'time'};
        my $ev_msg      = "";
        if ( !defined $ev_id )       { $ev_id         = ""; }
        if ( !defined $ev_severity ) { $ev_severity   = ""; }
        if ( !defined $ev_type )     { $ev_type       = ""; }
        if ( !defined $ev_name )     { $ev_name       = ""; }
        if ( !defined $ev_object_t ) { $ev_object_t   = ""; }
        if ( !defined $ev_object_n ) { $ev_object_n   = ""; }
        if ( !defined $ev_time )     { $ev_time       = ""; }

        if ( defined $event->{'eventInfo'} && $event->{'eventInfo'} ne "" ) {
          my $ev_info = decode_json( $event->{'eventInfo'} );
          if ( defined $ev_info->{'message'} && $ev_info->{'message'} ne "" ) {
            $ev_msg = $ev_info->{'message'};
          }
        }
        my %event;
        $event{id}         = $ev_id;
        $event{severity}   = $ev_severity;
        $event{type}       = $ev_type;
        $event{name}       = $ev_name;
        $event{objecttype} = $ev_object_t;
        $event{objectname} = $ev_object_n;
        $event{time}       = $ev_time;
        $event{message}    = $ev_msg;
        push(@{ $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data}{events} }, \%event );
      }
    }
  }

  if ( exists $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} ) {
    $hs_data{ $cfg->{'hostcfg_id'} }{health_status_data} = encode_json($hs_data{ $cfg->{'hostcfg_id'} }{health_status_data});
  }


  my @hs = (
    {
      "item_id" => $cfg->{'hostcfg_id'},
      "status" => $main_state,
      "data" => join("\n", @error_message),
    }
  );

  my $rc = $xormon->saveStatus(\@hs);

  my $rc_hs = $xormon->saveConfJSON(\%hs_data, 1);

}

1;
