# prepare vmware perf data to csv for NG

package VmwareX;

use strict;
use Date::Parse;
use File::Copy;
use Math::BigInt;
use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder

use Xormon;
use POSIX;
use Digest::MD5 qw(md5_hex);

$SIG{CHLD} = 'IGNORE';

my $et_VirtualMachine         = "VirtualMachine";
my $et_HostSystem             = "HostSystem";
my $et_Datastore              = "Datastore";
my $et_ResourcePool           = "ResourcePool";
my $et_Datacenter             = "Datacenter";
my $et_ClusterComputeResource = "ClusterComputeResource";

my $all_vmware_VMs = "vmware_VMs";
my $no_inserted    = 66;

#my $pwd = $ENV{'PWD'};
my $pwd = `pwd`;
chomp $pwd;
# e.g. /home/xormon/nextgen/server-nest/microservices/vmware

my $basedir = "$pwd/lpar2rrd";
my $tmpdir  = "$basedir/tmp";
if ( defined $ENV{TMPDIR_LPAR} ) {
  $tmpdir = $ENV{TMPDIR_LPAR};
}
my $DEBUG  = $ENV{DEBUG};
my $wrkdir = "$basedir/data";
# $|=1; # immediate print

my $vmware_data_dir = "$tmpdir/VMWARE/";
my $data_file       = "$tmpdir/VMWARE/perf.txt";
my $data_file_tmp   = "$tmpdir/VMWARE/perf.tmp";
my $data_file_snd   = "$tmpdir/VMWARE/perf.snd";

# arrays for entities with 1st signal line
my @esxis = ();
push @esxis,"uuid;time;cpu_alloc;cpu_usage;host_hz;memory_active;memory_granted;memory_baloon;disk_usage;disk_read;disk_write;network_usage;network_received;network_transmitted;memory_swapin;memory_swapout;memory_compress;memory_decompress;cpu_utilization_percent;memory_host_size;cpu_ready;power_usage;\n";

my @vms   = ();
push @vms,"uuid;time;cpu_alloc;cpu_usage;host_hz;memory_active;memory_granted;memory_baloon;disk_usage;disk_read;disk_write;network_usage;network_received;network_transmitted;memory_swapin;memory_swapout;memory_compres;memory_decompres;cpu_utilization_percent;vcpu;cpu_ready;vm_cpu_ready;cpu_utilization_cores\n";

my @clusters = ();
push @clusters,"uuid;time;cpu_usage;cpu_usage_proc;cpu_reserved;cpu_total;cluster_eff_cpu;cluster_eff_mem;memory_total;memory_shared;memory_zero;memory_baloon;memory_consumed;memory_overhead;memory_active;memory_granted;memory_compress;memory_reserved;memory_swap;memory_cmprssion;memory_dcompress;memory_usage_proc;cpu_utilization_cores;total_cluster_cores\n";

my @respools = ();
push @respools,"uuid;time;cpu_usage;memory_shared;memory_zero;memory_baloon;memory_consumed;memory_overhead;memory_active;memory_granted;memory_compress;memory_swap;memory_cmprssion;memory_dcompress;cpu_limit;cpu_reservation;memory_limit;memory_reservation;cpu_usage_proc;\n";

my @datastores = ();
push @datastores,"uuid;time;free_space;disk_used;disk_provisioned;disk_capacity;read;write;read_avg;write_avg;read_latency;write_latency;max_depth;\n";

my @datastore_vms = ();
push @datastore_vms,"uuid;time;iops_read;iops_write;provisioned_space;used_space;\n";

my @esxi_vm = ();
push @esxi_vm,"uuid;time;cpu_usage_proc;start_or_vmotion;\n";

# # export VMWARE_PROXY_SEND=1 or 2
 my $store_data = 1;
# if ( defined $ENV{VMWARE_PROXY_SEND} && $ENV{VMWARE_PROXY_SEND} == 1 ) {
#   $store_data = 0;
# }

my @architecture; # used for pushing datastore_vm & esxi_vm objects
my %vm_datastore; # used for pushing datastore_vm objects provisioned & used space
my %vm_hash = (); # for parents VMs uniting
my %conf_vm_datastore;

sub push_data_from_timed_files {    # from VMWARE
  my $xormon = shift;
  my $host_name   = shift;
  my $result_hash_ref = shift;
  my @arr       = ();

  my $dh;
  if ( !opendir( $dh, $vmware_data_dir ) ) {
    print " Can't open $vmware_data_dir : $! " . __FILE__ . ":" . __LINE__ . "\n";
    return 0;
  }

  # read whole dir
  my @all_items = readdir $dh;
  closedir($dh);

  # take files like 1599813464_10.22.11.10.txt.NG # host_name is 10.22.11.10
  my @files = grep /^\d\d\d\d\d\d\d\d\d\d_$host_name\.txt\.NG$/, @all_items;

  if ( scalar @files < 1 ) {
    print "push_data_from_timed_files: no files.txt.NG for host name $host_name detected\n";
    return 0;
  }

  # 1672111217_10.22.111.4.txt.NG.saved
  my @saved_files    = grep /^\d\d\d\d\d\d\d\d\d\d_$host_name\.txt\.NG\.saved$/,  @all_items;
  #  my @tar_files    = grep /^\d\d\d\d\d\d\d\d\d\d_$host_name\.txt\.tar$/,    @all_items;
  #  my @tar_gz_files = grep /^\d\d\d\d\d\d\d\d\d\d_$host_name\.txt\.tar.gz$/, @all_items;

  # delete older saved files
  # do it before the rrd operation because it can stuck and then the files can fill disk space
  foreach my $file_n (@saved_files) {
    my $file  = "$vmware_data_dir$file_n";
    my $mtime = ( stat($file) )[9];
    if ( time - $mtime > 150 * 60 ) {
      if ( !unlink "$file" ) {
        Xormon::error( " Can't unlink $file : $!" . __FILE__ . ":" . __LINE__ );
      }
    }
  }

  %vm_hash = ();
  my $file = "$tmpdir/arch_$host_name"."_vm.json";
  if (-f $file) {
    # print "nasli $file\n";
    my $output = Xormon::read_json($file);
    #print Dumper $output;
    print "--------------------\n";
    foreach (@$output) {
      my $vm = $_;
      #print Dumper $_;
      # print Dumper $vm;
      #$vm_hash{$_->item_id} = $_;
      $vm_hash{$vm->{item_id}} = $vm;
    }
  }
  # print Dumper ("135",\%vm_hash);

  print "start VMW csv  : " . localtime() . " host name $host_name \$store_data= $store_data\n" if $DEBUG;
  print "start VMW csv  : " . localtime() . " \$vmware_data_dir is $vmware_data_dir\n";

  foreach my $file_n ( sort @files ) {
    my $file = "$vmware_data_dir$file_n";
	@architecture = (); # for every file
	%vm_datastore = ();
	%conf_vm_datastore = ();

    print "reading file   : start $file_n " . localtime() . "\n";
#    print "\$store_data $store_data\n";
    my $skipped_lines = 0;
    my $line_count = 0;
	my $char_result = "";
    if ($store_data) {
      open( my $XFHLT, "< $file" ) || Xormon::error( " Can't open $file : $!" . __FILE__ . ":" . __LINE__ ) && next;

      my $update_word = "update_";
      while ( my $line = <$XFHLT> ) {
        $line_count++;
        my ( $frase, $managedname, $host, $wrkdir_upd, $input, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, $trans_tmp, $last_file, $no_time, $vm_uuid, $hmc_user, $lpar_uuid, $entity_type, $frk, $data_line ) = split( ",", $line, 21 );

        #$wrkdir_upd is not used
        # next if $frase ne "update_line";
        my $count       = () = $line =~ /$update_word/g;
        if ( $count != 2 ) {    # the string must be there 2x
          Xormon::error("push_data_from_timed_files \$count of update_ is $count, line $line_count is skipped \$data_line $data_line");
          $skipped_lines++;
          next;
        }
        chomp $data_line;
        if ( $data_line =~ /,xorux_sentinel$/ ) {
          $data_line =~ s/,xorux_sentinel$//;
        }
        else {
          Xormon::error("no ,xorux_sentinel at the end of update_line $line_count $data_line");
          $skipped_lines++;
          next;
        }

        # next if data_line contains not allowed chars
        my $test_line = $data_line;
        $test_line =~ s/[\d\s\,U\.\'\-e]//g;

        # next if $test_line ne "";
        if ( $test_line ne "" ) {
          Xormon::error("\$test_line $line_count has notallowed chars $test_line\n $data_line");
          $skipped_lines++;
          next;
        }

        # print "$managedname, $host, $wrkdir, $input, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, $trans_tmp, $last_file, $no_time, $vm_uuid, $hmc_user, $lpar_uuid, $entity_type, $frk, $data_line\n";
        $char_result .= push_data_to_csv( $managedname, $host, $wrkdir, \$data_line, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, \@arr, $last_file, $no_time, $vm_uuid, $hmc_user, $lpar_uuid, $entity_type, $frk, $xormon );
      }
      close($XFHLT);
      print "$char_result\nreading file   : end $file_n " . localtime() . " \$skipped_lines=$skipped_lines from total $line_count\n";
	  # print Dumper(@architecture);

      #save architecture to DB
      my $resp_arch =  $xormon->saveArchitecture(\@architecture); # not when testing
      Xormon::log("result:,$resp_arch, :InsertArch datastore_vm $file\n");
      move ($file, "$file".".saved") or Xormon::error("cannot move $file to $file".".saved $!");
      #save configuration to DB
       my $configuration_rc = $xormon->saveConfJSON( \%conf_vm_datastore );
       Xormon::log("result:,$configuration_rc, Save Conf conf_vm_datastore");
      if ($configuration_rc ne "201") {
  	    Xormon::error("Save Conf conf_vm_datastore result:,$configuration_rc,");
 	    print Dumper \%conf_vm_datastore;
	  }
    }    ## if ($store_data)
    else {
      print "do not store  : data cus VMWARE_PROXY_SEND=1\n";
    }

    #    my $tar_result = `cd $vmware_data_dir; tar -cvhf "$file.tar" "$file_n" 2>&1`;    #
    #    print "tar_result $tar_result\n";
    #    if ( -f "$file.tar" ) {
    #      if ( !unlink $file ) {
    #        Xormon::error("cannot unlink $file");
    #      }
    #      else {
    #        utime undef, undef, "$file.tar";
    #      }
    #    }
    #    else {
    #      Xormon::error("cannot tar $file");
    #    }
  }

  # delete older snd files
  #  foreach my $file_n (@snd_files) {
  #    my $file  = "$vmware_data_dir$file_n";
  #    my $mtime = ( stat($file) )[9];
  #    if ( time - $mtime > 150 * 60 ) {
  #      if ( !unlink "$file" ) {
  #        Xormon::error( " Can't unlink $file : $!" . __FILE__ . ":" . __LINE__ );
  #      }
  #    }
  #  }

  # delete older tar files
  #  foreach my $file_n (@tar_files) {
  #    my $file  = "$vmware_data_dir$file_n";
  #    my $mtime = ( stat($file) )[9];
  #    if ( time - $mtime > 150 * 60 ) {
  #      if ( !unlink "$file" ) {
  #        Xormon::error( " Can't unlink $file : $!" . __FILE__ . ":" . __LINE__ );
  #      }
  #    }
  #  }

  # delete older tar.gz files
  #  foreach my $file_n (@tar_gz_files) {
  #    my $file  = "$vmware_data_dir$file_n";
  #    my $mtime = ( stat($file) )[9];
  #    if ( time - $mtime > 150 * 60 ) {
  #      if ( !unlink "$file" ) {
  #        Xormon::error( " Can't unlink $file : $!" . __FILE__ . ":" . __LINE__ );
  #      }
  #    }
  #  }

  print "end VMWARE csv : " . localtime() . " host name $host_name\n" if $DEBUG;

  # prepare result data for  microservice
  if (scalar @esxis >1) {
    # print "@esxis\n\n";
    $result_hash_ref->{ESXI} = "@esxis";
  }
  if (scalar @vms >1) {
    # print "@vms\n\n";
    $result_hash_ref->{VM} = "@vms";
  }
  if (scalar @respools >1) {
    # print "@respools\n\n";
    $result_hash_ref->{RESGROUP} = "@respools";
  }
  if (scalar @clusters >1) {
    # print "@clusters\n\n";
    $result_hash_ref->{CLUSTER} = "@clusters";
  }
  if (scalar @datastores >1) {
    # print "@datastores\n\n";
    $result_hash_ref->{DATASTORE} = "@datastores";
  }
  if (scalar @datastore_vms >1) {
    # print "@datastore_vms\n\n";
    $result_hash_ref->{DATASTORE_VM} = "@datastore_vms";
  }
  if (scalar @esxi_vm >1) {
    # print "267\n@esxi_vm\n\n";
    $result_hash_ref->{ESXI_VM} = "@esxi_vm";
  }
  print "lines prepared host name $host_name:  VM/".scalar @vms." ESXI/".scalar @esxis." RESPOOL/".scalar @respools." CLUSTER/".scalar @clusters." DATASTORE/".scalar @datastores." DATASTORE_VM/".scalar @datastore_vms." ESXI_VM/".scalar @esxi_vm."\n";
  return ("99");    # symbol of success
} ## end of sub push_data_from_timed_files

sub push_data_to_csv {    # read data from $data_file
  my ( $managedname, $host, $wrkdir, $input, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, $trans_tmp, $last_file, $no_time, $vm_uuid, $hmc_user, $lpar_uuid, $entity_type, $frk, $xormon ) = @_;
  $managedname =~ s/^\s+//; #leading spaces
  $lpar_uuid   =~ s/^\s+//; #leading spaces
  #update_line,10.22.66.27,10.22.66.35,/home/lpar2rrd/data,$update_string,m,Tue May 10 14:17:26 2022,$ds_name,0,0,60,1,@lpar_trans,last.txt,1080,,lpar2rrd,pool,HostSystem,(88307F0)
  #with vcenter uuid & esxi_moref
  #update_line,10.22.11.14,10.22.11.10,/home/lpar2rrd/data,$update_string,m,Mon Aug  8 09:16:18 2022,1,0,eb6102a7-1fa0-4376-acbb-f67e34a2212c_28,60,1,@lpar_trans,last.txt,1080,host-272,lpar2rrd,pool,HostSystem,(117285F0)

  # print "updating RRD   : $host:$managedname:data:$wrkdir/$managedname/$host/$lpar_uuid.rr$type_sam $frk\n" if $DEBUG;
  # ds_name is in $HMC

  if (! %vm_datastore) {
    get_datastore_info_from_html(\%vm_datastore,$SDMC);
	# print Dumper \%vm_datastore;
  }

  my @lines;
  my $update_line = $$input;
  chomp $update_line;
  my $control_line = "";  

  $update_line =~ s/,/;/g;
  $update_line =~ s/ $/;/g;
  $update_line =~ s/ /\|/g;
  $update_line =~ s/U//g;
  {
    $et_HostSystem eq $entity_type && do {
      $update_line = "vmware_$SDMC" . "_esxi_" . "$vm_uuid;$update_line\n";
	  # print "276 ,$managedname"."_esxi_"."$vm_uuid,$update_line\n";
      push @esxis,$update_line;
      last;
    };
    $et_VirtualMachine eq $entity_type && do {
      my $update_line_vm = "$lpar_uuid;$update_line\n";
      push @vms,$update_line_vm;
      # prepare update line and structure esxi_vm for vMotion graph with one metric CPU usage %
	  # update_line,vmware_VMs,,/home/xormon/nextgen/server-nest/microservices/vmware/lpar2rrd/data,$update_string,m,Tue Nov 21 13:40:24 2023,1,0,32a32979-81a3-4f89-a276-f7a502fab7c5_9,60,1,@lpar_trans,500c1ba0-c9ef-2d39-2306-1a1364f0ff4d.last,1080,host-1009,lpar2rrd,500c1ba0-c9ef-2d39-2306-1a1364f0ff4d,VirtualMachine,(699368F0),
	  # '1700569240,0,3000000,2693509324,85897216,4223664128,0,0,0,0,0,0,0,0,0,0,0,0.05,2,0.0175 '
	  # '1700569260,0,3000000,2693509324,85897216,4223664128,0,0,0,0,0,0,0,0,0,0,0,0.05,2,0.01 '
	  # ...
	  # '1700570420,0,84000000,2693509324,128847872,4158652416,0,1024,0,1024,1024,1024,0,0,0,0,0,1.56,2,0.185 ,xorux_sentinel'
	  # the last three values are CPU%, vCpu, ready
	  # esxi_moref is 'host-1009' is in $vm_uuid variable
	  # print "303 ,$update_line,\n";
      # 1700574040;0;53000000;2297338894;42946560;388300800;0;;;;0;0;0;0;0;0;0;1.16;2;0.1|1700574060;0;49000000;2297338894;42946560;388300800;0;;;;0;0;0;0;0;0;0;1.06;2;0.105|1700574080;...
	  my @atoms = split('\|',$update_line);
	  my $esxi_vm_upd_line = "";
	  $IVM = 1 if $IVM > 0; #it keeps VM unix time of start of vmotion or 0, it must 1 or 0 to be summed then for vmotion table day, week...
	  foreach (@atoms) {
	    (my $utime,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,undef,my $cpu_proc, undef) = split (";",$_);
		# print "308, $utime,$cpu_proc,\n";
		$esxi_vm_upd_line .= "$utime;$cpu_proc;$IVM|"; #$IVM is 1 or 0
		$IVM = 0;
	  }
	  $esxi_vm_upd_line =~ s/\|$/;/g;
	  $esxi_vm_upd_line = "vmware_$SDMC" . "_esxi_" . "$vm_uuid". "_vm_" . "$lpar_uuid;$esxi_vm_upd_line\n";
	  push @esxi_vm,$esxi_vm_upd_line;
	  # print "315 ,$esxi_vm_upd_line,\n";

	  my @parents = ();
      push @parents, "vmware_$SDMC" . "_esxi_" . "$vm_uuid"; # esxi as parent
      push @parents, $lpar_uuid; # vm as parent

      #architecture
      my %esxi_vm_arch = (
          "item_id" => "vmware_$SDMC" . "_esxi_" . "$vm_uuid". "_vm_" . "$lpar_uuid",
          "label" => "$HMC",
          "hw_type" => "vmware",
          "subsystem" => "esxi_vm",
          "hostcfg_id" => $xormon->{hostcfg_id},
          "parents" => [@parents]
      );
	  # print Dumper ("329",\%esxi_vm_arch);
       push(@architecture, \%esxi_vm_arch);
      last;
    };
    $et_ResourcePool eq $entity_type && do {
      $update_line = "$managedname" . "_" . "$lpar_uuid;$update_line\n";
      push @respools,$update_line;
      last;
    };
    $et_ClusterComputeResource eq $entity_type && do {
      $update_line = "$managedname" . "_" . "$host;$update_line\n";
      push @clusters,$update_line;
      last;
    };
    # type NG
	# update_line,vmware_32a32979-81a3-4f89-a276-f7a502fab7c5_9,datastore_datacenter-1001,/home/xormon/nextgen/server-nest/microservices/vmware/lpar2rrd/data,$update_string,NG,Fri Jun 16 15:00:40 2023,NFS-qnap101-ISO,0,0,60,1,@lpar_trans,4b6ba915-196518ee.last,5400,,lpar2rrd,4b6ba915-196518ee,Datastore,(303318F3),1686919240,1177779539968,1271953555456,1271953555456,2449733095424,0,0,0,0,0,0,U 1686919260,1177779539968,1271953555456,1271953555456,2449733095424,0,0,0,0,0,0,U 1686919280,1177779539968,1271953555456,1271953555456,2449733095424,0,0,0,0,0,0,U ... ,xorux_sentinel
    ( $et_Datastore eq $entity_type && $type_sam eq "NG" ) && do {
      $update_line = "$managedname" . "_" . "$host" . "_ds_" . "$lpar_uuid;$update_line\n";
      push @datastores,$update_line;
      last;
    };

    # type v
    #update_line,vmware_32a32979-81a3-4f89-a276-f7a502fab7c5_9,datastore_datacenter-1001,/home/xormon/nextgen/server-nest/microservices/vmware/lpar2rrd/data,$update_string,v,Fri Jun 16 15:00:40 2023,NFS-qnap101-ISO,0,0,60,1,@lpar_trans,,5400,500c968a-3329-4f0a-436e-e043744e8fc0,lpar2rrd,4b6ba915-196518ee,Datastore,(303318F3),1686919240,0,0 1686919260,0,0 1686919280,0,0 1686919300,0,0 1686919320,0,0 1686919340,0,0 1686919360,0,0 1686919380,0,0 ... 1686920420,0,0 ,xorux_sentinel
    ( $et_Datastore eq $entity_type && $type_sam eq "v" ) && do {
      $update_line = "$managedname" . "_" . "$host" . "_ds_" . "$lpar_uuid" . "_" . "$vm_uuid;$update_line\n";

	  # print "386 \$update_line $update_line\n";
      # 386 $update_line vmware_eb6102a7-1fa0-4376-acbb-f67e34a2212c_28_datastore_datacenter-2_ds_6630d7ad-6497690f-7ad2-e4115bd41b18_52c9c79b-3e81-9efd-6fc4-0fad12383212;1723810820;0;0|1723810840;0;0|1723810860;0;0|1723810880;0;0|1723810900;0;0|1723810920;0;0|1723810940;0;0|1723810960;0;0|1723810980;0;0|1723811000;0;0|1723811020;0;0|1723811040;0;0|1723811060;0;0|1723811080;0;0|1723811100;0;0|1723811120;0;0|1723811140;0;0|1723811160;0;0|1723811180;0;0|1723811200;0;0|1723811220;0;0;

      if (exists $vm_datastore{$vm_uuid}{$lpar_uuid}) {
		my $provisioned = $vm_datastore{$vm_uuid}{$lpar_uuid}{"provisioned"} *1024 *1024 *1024 ;
        my $used = $vm_datastore{$vm_uuid}{$lpar_uuid}{"used"} *1024 *1024 *1024;
        $update_line =~ s/\|/;$provisioned;$used\|/g;
        # and the last atom
        $update_line =~ s/;$/;$provisioned;$used;/;
        # print "395 ,$update_line,\n";
		# configuration
		$conf_vm_datastore{"$managedname" . "_" . "$host" . "_ds_" . "$lpar_uuid" . "_" . "$vm_uuid"}{"provisioned"} = $provisioned;
		$conf_vm_datastore{"$managedname" . "_" . "$host" . "_ds_" . "$lpar_uuid" . "_" . "$vm_uuid"}{"used"} = $used;
      }
      push @datastore_vms,$update_line;

	  my @parents = ();
    #  push @parents, "$managedname" . "_" . "$host" . "_ds_" . "$lpar_uuid";
    #  push @parents, $vm_uuid;
      if (exists $vm_hash{$vm_uuid}) {
        # $my_vm{$vm_uuid}{parents} = $vm_hash{$vm_uuid}{parents};
     #   my @one_arr = @{$my_vm{$vm_uuid}{parents}};
     #   my @two_arr = @{$vm_hash{$vm_uuid}{parents}};
     #   my @three_arr = (@one_arr, @two_arr);
		my @three_arr = @{$vm_hash{$vm_uuid}{parents}};
		push @three_arr, "$managedname" . "_" . "$host" . "_ds_" . "$lpar_uuid";
        push @three_arr, $vm_uuid;
        # print Dumper ("392",@three_arr);
        my %hash_ar   = map { $_, 1 } @three_arr;
        @three_arr = keys %hash_ar;
        # print Dumper ("395",@three_arr);
		@parents = @three_arr;

      #  $my_vm{$vm_uuid}{parents} = $vm_hash{$vm_uuid}{parents};
      }

      #architecture
      my %datastore_vm_arch = (
          "item_id" => "$managedname" . "_" . "$host" . "_ds_" . "$lpar_uuid" . "_" . "$vm_uuid",
          "label" => "$HMC",
          "hw_type" => "vmware",
          "subsystem" => "datastore_vm",
          "hostcfg_id" => $xormon->{hostcfg_id},
          "parents" => [@parents]
      );
      push(@architecture, \%datastore_vm_arch);
    };
  }
  if ( $control_line eq "" ) {
    #print "Unknown line $entity_type $type_sam $update_line\n";
    # print "$type_sam";
	return $type_sam;
  }
  else {
    #    print "\n\$control_line $control_line prepared line $update_line\n";
    # print ".";
	return ".";
  }
  # return;
} ## end of sub push_data_to_csv

################################################################################
# store VMWARE data from tmp/arch_10.22.11.10...json files to NG database

sub vmware_json2ng {
  my $xormon = shift;
  my $host_ip = shift;
  # json dir = tmpdir
  my @architecture;

  my $dh;
  if ( !opendir( $dh, $tmpdir ) ) {

    # Xormon::error( " Can't open $tmpdir : $! " . __FILE__ . ":" . __LINE__);
	Xormon::log("trying to create: \$tmpdir is $tmpdir\n");
	if (! mkdir($tmpdir)) {
      Xormon::error( " Can't create $tmpdir : $! " . __FILE__ . ":" . __LINE__);
      return 1;
	}
	my $tmpdir_vmware = "$tmpdir/VMWARE";
    if (! mkdir($tmpdir_vmware)) {
      Xormon::error( " Can't create $tmpdir_vmware : $! " . __FILE__ . ":" . __LINE__);
      return 1;
    }
	my $logs = "$basedir/logs";
	if (!-f $logs) {
      if (! mkdir($logs)) {
        Xormon::error( " Can't create $logs : $! " . __FILE__ . ":" . __LINE__);
        return 1;
      }
	}
    return 0;
  }
  my $result_del_arch = "no run";
  # test all files .json X .sent, write only if there is a file length difference, no, it is not good idea

  # $result_del_arch = $xormon->deleteArchitecture($ENV{HOSTCFG_ID});
  # read arch vmware json files
  my @files = sort( grep /^arch_$host_ip.*\.json$/, readdir $dh );
  rewinddir($dh);
  my @conf_files = sort( grep /^conf_$host_ip.*\.json$/, readdir $dh );
  closedir($dh);
  # print "354 \@files @files\n";
  Xormon::log("vmware_json2ng.pl : \$tmpdir is $tmpdir\n"); # \$result_del_arch $result_del_arch\n");

  open (my $FH,">","$tmpdir/vmware_arch_$host_ip".".json"); # start with empty file
  close $FH;

  foreach my $file_name (@files) {
	my $file = "$tmpdir/$file_name";
	# print "\$file $file\n";
	my $FH;
    if ( open ($FH, "$file")) {
	  close $FH;
	  my $file_sent = "$file"."_sent";
	  unlink $file_sent;
	  move $file,$file_sent or Xormon::error( "can't rename $file to $file_sent: $! " . __FILE__ . ":" . __LINE__ ) && next;
      my $json = Xormon::read_json($file_sent,"encode");
      # print Dumper("407 json",$json);

	  open ($FH,">>","$tmpdir/vmware_arch_$host_ip".".json"); # if you want to see it as a whole
	  print $FH Dumper($json);
	  close $FH;
    
      # print Dumper $json;
	  my $resp_arch = $xormon->saveArchitecture($json);
      Xormon::log("result:,$resp_arch, Insert tmp/$file_name\n");
	  Xormon::error("vmware_json2ng.pl :Insert tmp/$file_name result:,$resp_arch,\n") if $resp_arch ne "201";
	}
	else {
      Xormon::error( "can't open $file: $! " . __FILE__ . ":" . __LINE__ );
	}
  }
  # save conf files
  foreach my $file_name (@conf_files) {
    my $file_sent = "$tmpdir/$file_name"."_sent";
    unlink $file_sent;
    move "$tmpdir/$file_name",$file_sent or Xormon::error( "can't rename tmp/$file_name to $file_name.sent: $! " . __FILE__ . ":" . __LINE__ ) && next;

    my $result = Xormon::read_json($file_sent);
    # print Dumper $result;
    my $configuration_rc = $xormon->saveConfJSON( $result );
    Xormon::log("result:,$configuration_rc, Save Conf tmp/$file_name"); # .sent of course
	Xormon::error("Save Conf tmp/$file_name result:,$configuration_rc,") if $configuration_rc ne "201";
  }	

  # non vcenters: not supported yet

} ## end of sub vmware_json2ng

sub basename {
    my $full = shift;
    my $out  = "";

# basename without direct function
    my @base = split( /\//, $full );
    foreach my $m (@base) {
        $out = $m;
    }
    return $out;
}

sub get_datastore_info_from_html {
  my $datastore_storage = shift;
  my $vcenter_uuid = shift; # eb6102a7-1fa0-4376-acbb-f67e34a2212c_28

  my $pth = "$wrkdir/vmware_$vcenter_uuid/datastore_*/*.html";
  $pth =~ s/ /\\ /g;
  my $no_name = "";
  my @files   = (<$pth$no_name>);    # unsorted, workaround for space in names
  # print "541 @files\n";
  for my $file_path ( @files ) {

    if ( -f $file_path ) {

      # print "546 exists $file_path\n";
      open( my $myFF, "<:encoding(UTF-8)", "$file_path" ) || Xormon::error( "can't open $file_path: $! :" . __FILE__ . ":" . __LINE__ ) && next;
       my @dstr_info = (<$myFF>);
      close($myFF);
      next if scalar @dstr_info < 2;    # no VMs

      while ( my $Element = shift(@dstr_info) ) {
        last if index( $Element, "Provisioned space" ) > 0;    # previous lines
      }
      # <tbody><TR> <TD><B>kr01</B></A></TD> <TD align="right" nowrap>poweredOff</TD> <TD align="right" nowrap>108.6</TD> <TD align="right" nowrap>14.3</TD></TR>
      # <TR> <TD><A HREF="/lpar2rrd-cgi/detail.sh?host=10.22.111.3&server=&lpar=xormon-demo&item=lpar&entitle=0&none=none&d_platform=VMware&vm_dbi_uuid=32a32979-81a3-4f89-a276-f7a502fab7c5_9_vm_5009cfd9-9508-cc72-fc13-613259ee2108"><B>xormon-demo</B></A></TD> <TD align="right" nowrap>poweredOn</TD> <TD align="right" nowrap>100.0</TD> <TD align="right" nowrap>97.8</TD></TR>
      # </tbody></TABLE></CENTER><BR>
      while ( my $Element = shift(@dstr_info) ) {
        next if index( $Element, "nowrap>" ) < 0;    # probably last line or ??
        next if index( $Element, "vm_dbi_uuid=" ) < 0;    # probably last line or ??
        ( my $vm_name, undef, my $provisioned_space_gb, my $used_space_gb ) = split "nowrap>", $Element;
        # print "562,$Element,\n";
        (my $vm_uuid_ahref, $vm_name) = split "<B>", $Element;
        $vm_name =~ s/<.*//;
        chomp $vm_name;
        (undef,$vm_uuid_ahref) = split "_vm_",$vm_uuid_ahref;
        $vm_uuid_ahref =~ s/\".*//g;
        $provisioned_space_gb =~ s/<.*//;
        $used_space_gb =~ s/<.*//;
        # print "570 $vm_name $vm_uuid_ahref $provisioned_space_gb $used_space_gb\n";
        my $dstr_name = basename $file_path;
		$dstr_name =~ s/\.html//;
        $datastore_storage->{$vm_uuid_ahref}{$dstr_name}{"provisioned"}     = $provisioned_space_gb;
        $datastore_storage->{$vm_uuid_ahref}{$dstr_name}{"used"}            = $used_space_gb;
      }
      #exit;
    }
    else {
      print "583 file does NOT exist $file_path\n";
    }
  }
  # print Dumper ("586",$datastore_storage);
}

1;
