use FindBin;
use lib "$FindBin::Bin/"; # use lib in this folder
use lib "../../../perl-modules";
use LWP;
use JSON;
use strict;
use warnings;
use Data::Dumper;
use Xormon;
use Cwd qw(cwd);
use XIV;
use Test;
use XormonLogger;

my $logger;

# microservice's directory
my $act_dir = cwd;
my $wrkdir  = "";

#$SIG{CHLD} = 'IGNORE';
# cannot be used in this MS, bcs it produces warning "Can't ignore signal CHLD, forcing to default."
# it's probably caused by that there are running 2 another new child processes for each device (svcconfig.pl and svcperf.pl)

use IO::Socket qw(AF_INET AF_UNIX SOCK_STREAM SHUT_WR);

my $DEFAULT_PORT = 3222;
my $port = Xormon::checkPrefixPort($DEFAULT_PORT);

my $server = IO::Socket->new(
  Domain => AF_INET,
  Type => SOCK_STREAM,
  Proto => 'tcp',
  LocalHost => '0.0.0.0',
  LocalPort => $port,
  ReusePort => 1,
  Listen => 5,
) || die "Can't open socket: $@";
Xormon::log("Service listening on port $port");

my $client;

# waiting for a new client connection
while ($client = $server->accept()) {

  my $pid;
  unless ( defined( $pid = fork() ) ) {
    Xormon::error("Wait for fork");
    sleep 5;
  }

  if ($pid == 0) {

    # get information about a newly connected client
    my $client_address = $client->peerhost();
    my $client_port = $client->peerport();
    Xormon::log("Connection from $client_address:$client_port");

    # read up to 2048 characters from the connected client
    my $data = "";
    $client->recv($data, 2048);

    my $decoded = from_json($data);
    my $cfg     = $decoded->{data};

    #
    # Device configuration
    #
    export_device_cfg($cfg);

    Xormon::log("Received task: $decoded->{task} for Device: $cfg->{hostalias}");
    my $response = '';
    if ($decoded->{task} eq "fetch") {
      #
      # data collection
      #
      $client->send("Task $decoded->{task} successfuly started");
      $client->shutdown(SHUT_WR);

      my $xormon = Xormon->new("$client_address", $decoded->{jwt}, 'xiv', $cfg->{hostcfg_id}, $cfg->{hostalias});

      # start task
      $xormon->startJob($decoded->{task}, $cfg->{hostalias}, $$);

      # target destinations check
      if (defined $decoded->{'destinations'}) {
        $xormon->setDestinations($decoded->{'destinations'});
      }

      #
      # create conf/perf
      #
      collect_data();

      #
      # parse conf/perf files and send data
      #
      my $status = XIV::process_collected_data($xormon,$cfg,$wrkdir);

      # end task
      $xormon->endJob($decoded->{task}, $cfg->{hostalias}, $status, $$);
      Xormon::log("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done. Status: $status");
    }
    elsif ( $decoded->{task} eq "test" ) {
      #
      # connection test
      #
      connection_test();
      $client->shutdown(SHUT_WR);
      Xormon::log("Task '$decoded->{task}' for ".$cfg->{hostalias}." is done.");
    }
    else {
      $response = "undefined task";

      $client->send($response);
      $client->shutdown(SHUT_WR);

    }

    exit();

  } else {
    $client->autoflush(1);
  }
}

$server->close();

exit();

sub collect_data {
  # system()    you want to execute a command and don't want to capture its output
  # exec        you don't want to return to the calling perl script
  # backticks   you want to capture the output of the command
  # open        you want to pipe the command (as input or output) to your script

  #
  # create perf and conf
  #
  if ( is_process_running("xivperf.pl $ENV{'STORAGE_NAME'} XormonNG") ) {
    $logger->error( "WARNING: Process 'xivperf.pl $ENV{'STORAGE_NAME'} XormonNG' is still running, cannot run the new one... $!" . __FILE__ . ":" . __LINE__ );
  }
  else {
    $logger->info( "starting: $act_dir/xivperf.pl $ENV{'STORAGE_NAME'} XormonNG" );
    system ("perl", "-w", "$act_dir/xivperf.pl", "$ENV{'STORAGE_NAME'}", "XormonNG");
  }

  return 1;
}

sub is_process_running {
  my $process_string = shift;

  if ( ( grep {/$process_string$/} @{ get_ps_list() } ) > 0 ) {
    return 1;
  }
  else {
    return 0;
  }
}

sub get_ps_list {
  open(my $ps,"ps -ef |") || $logger->error("'ps -ef' command failed! $!" . __FILE__ . ":" . __LINE__ ) && exit 0;
  my @ps_list = <$ps>;
  chomp @ps_list;
  close($ps);

  return \@ps_list;
}

sub export_device_cfg {
  my $cfg = shift;

  #
  # mandatory cfg
  #
  unless ( exists $cfg->{'hostcfg_id'}      && $cfg->{'hostcfg_id'} ne '' )      { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!")       && exit 0; }
  unless ( exists $cfg->{'hostalias'}       && $cfg->{'hostalias'} ne '' )       { Xormon::error("Cannot continue, 'hostalias' is not configured!")        && exit 0; }
  $logger = XormonLogger->new( 'xiv', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'host'}            && $cfg->{'host'} ne '' )            { $logger->error("Cannot continue, 'host' is not configured!")             && exit 0; }
  unless ( exists $cfg->{'storageUsername'} && $cfg->{'storageUsername'} ne '' ) { $logger->error("Cannot continue, 'storageUsername' is not configured!")  && exit 0; }
  unless ( exists $cfg->{'password'}        && $cfg->{'password'} ne '' )        { $logger->error("Cannot continue, 'password' is not configured!")         && exit 0; }

  $ENV{'XormonNG'}     = 1;
  $ENV{'STORAGE_NAME'} = $cfg->{'hostalias'};
  $ENV{'XIV_IP'}       = $cfg->{'host'};
  $ENV{'XIV_USER'}     = $cfg->{'storageUsername'};
  $ENV{'XIV_PASSWD'}   = $cfg->{'password'};
  $ENV{'BINDIR'}       = $act_dir;
  #$ENV{'XIV_INTERVAL'} = 300;
  #$ENV{'SAMPLE_RATE'}  = 300;
  #$ENV{'XIV_CFG_INTERVAL'} = 300;
  $ENV{'DEBUG'}        = 1;
  $ENV{'DEBUG_FULL'}   = 1;
  $ENV{'DEBUG_SCRIPT'} = 1;

  #
  # create directories
  #
  $wrkdir = "$act_dir/data/$cfg->{'hostcfg_id'}";
  $ENV{'XIV_DIR'}  = $wrkdir;

  unless ( -d "$act_dir/data" )   { mkdir( "$act_dir/data", 0755 )   || $logger->error( "Cannot mkdir '$act_dir/data': $!" . __FILE__ . ":" . __LINE__ )   && exit; }
  unless ( -d "$wrkdir" )         { mkdir( "$wrkdir", 0755 )         || $logger->error( "Cannot mkdir '$wrkdir': $!" . __FILE__ . ":" . __LINE__ )         && exit; }
  unless ( -d "$wrkdir/iostats" ) { mkdir( "$wrkdir/iostats", 0755 ) || $logger->error( "Cannot mkdir '$wrkdir/iostats': $!" . __FILE__ . ":" . __LINE__ ) && exit; }

  #
  # test agent scripts
  #
  unless ( -f "$act_dir/xivperf.pl" ) { $logger->error( "Script '$act_dir/xivperf.pl doesn't exist!': $!" . __FILE__ . ":" . __LINE__ ) && exit; }

  #
  # remove old files
  #
  opendir( my $dir, "$wrkdir" ) || $logger->error( "Cannot open directory '$wrkdir' " . __FILE__ . ":" . __LINE__ ) && exit 1;
  my @files = grep !/^\.\.?$/, readdir($dir);
  closedir($dir);
  chomp @files;

  foreach my $filename (@files) {
    my $filepath = "$wrkdir/$filename";
    if ( -f $filepath ) {
      # just remove old files, there can be some potential issue with space, so rather check it
      #if ( $filename =~ m/_xivconf_/ && Xormon::file_time_diff($filepath) > 604800 ) { unlink($filepath); } # 7 days
      #if ( $filename =~ m/_xivperf_/ && Xormon::file_time_diff($filepath) > 604800 ) { unlink($filepath); } # 7 days
    }
  }

  return 1;
}

sub connection_test {
  my $test = Test->new($client);

  #
  # test IP and port
  #
  my ( $code, $text ) = Xormon::conntest_tcp($ENV{'XIV_IP'},5989); # always port 5989, there is no possibility to change it
  $test->addSection("XIV IP network connection", $code, $text);
  if ( $code == 0 ) { return $test->stop(); }

  #
  # test wbemcli package
  #
  my $wbemcli = "/usr/bin/wbemcli";
  unless ( -f $wbemcli ) {
    if ( -f "/opt/freeware/bin/wbemcli" ) {
      $wbemcli = "/opt/freeware/bin/wbemcli";
    }
    elsif ( -f "/usr/local/bin/wbemcli" ) {
      $wbemcli = "/usr/local/bin/wbemcli";
    }
    # not supported yet
    #elsif ( exists $ENV{WBEMCLI_CMD} && $ENV{WBEMCLI_CMD} ne '' && -f $ENV{WBEMCLI_CMD} ) {
    #  $wbemcli = $ENV{WBEMCLI_CMD};
    #}
    else {
      $test->addSection("wbemcli package", 0, "wbemcli binnary has not been found!");
      return $test->stop();
    }
  }
  $test->addSection("wbemcli package", 1, "");

  #
  # test wbemcli cmd
  #
  my $cmd  = "$wbemcli -noverify -nl ei https://$ENV{'XIV_USER'}:$ENV{'XIV_PASSWD'}\@$ENV{'XIV_IP'}:5989/root/ibm:IBMTSDS_StorageSystem";
  my $file = "$ENV{'XIV_DIR'}/connection_test.txt";
  if ( -f $file ) { unlink($file); }

  my $ret = `$cmd 1>$file 2>&1`;
  my @out;
  if ( -f $file ) {
    open( IN, "<$file" ) || $logger->error( "Couldn't open file $file $!" . __FILE__ . ":" . __LINE__ ) && return $test->stop();
    @out = <IN>;
    close(IN);

    unlink($file);
  }

  # add cmd string at the beginning of the array
  $cmd =~ s/:$ENV{'XIV_PASSWD'}\@/:XXXXXXXXXXX\@/;
  unshift @out, "\n";
  unshift @out, "$cmd\n";

  if ( (grep {/CodeLevel/} @out) > 0 ) {
    $test->addSection("wbemcli cmd", 1 , \@out);
  }
  else {
    $test->addSection("wbemcli cmd", 0 , \@out);
  }

  return $test->stop();
}
