"use strict";
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storage = void 0;
const pageResponse_constants_1 = require("../../../common/constants/pageResponse.constants");
const subsystemMetricsResponse_constants_1 = require("../../../common/constants/subsystemMetricsResponse.constants");
const Stor_1 = require("../../menu/menu/Stor");
const Color_1 = require("../Color");
const Definitions_1 = require("../Definitions");
const defaultStorageTops = {
    'IO': ['read_io', 'write_io', 'io_rate'],
    'Data': ['read', 'write', 'data_rate'],
    'Latency': ['resp_t_r', 'resp_t_w', 'resp_t']
};
class Storage {
    constructor(hwType) {
        this.hwType = hwType;
    }
    hasHeatmap() {
        return _a.technology[this.hwType]['pages'] && _a.technology[this.hwType]['pages'].includes(Stor_1.StoragePages.heatmap) ? true : false;
    }
}
exports.Storage = Storage;
_a = Storage;
Storage.order = 1;
Storage.metrics = {
    io_rate: {
        ...Definitions_1.metricTypes.iops,
        metric: 'io_rate',
        label: 'IO',
        color: Color_1.Color.pallete.redDark
    },
    read_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'read_io',
        label: 'IO Read',
        color: Color_1.Color.metric.read
    },
    write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'write_io',
        label: 'IO Write',
        color: Color_1.Color.metric.write
    },
    io_rate_b: {
        ...Definitions_1.metricTypes.iops,
        metric: 'io_rate_b',
        label: 'IO Rate',
        longLabel: 'IO Rate BE',
        color: Color_1.Color.pallete.redDark
    },
    read_io_b: {
        ...Definitions_1.metricTypes.iops,
        metric: 'read_io_b',
        label: 'Read IO',
        longLabel: 'Read IO BE',
        color: Color_1.Color.metric.read
    },
    write_io_b: {
        ...Definitions_1.metricTypes.iops,
        metric: 'write_io_b',
        label: 'Write IO',
        longLabel: 'Write IO BE',
        color: Color_1.Color.metric.write
    },
    un_read_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'un_read_io',
        label: 'Unaligned IO Read',
        color: Color_1.Color.metric.read
    },
    un_write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'un_write_io',
        label: 'Unaligned IO Write',
        color: Color_1.Color.metric.write
    },
    io_pending: {
        ...Definitions_1.metricTypes.iops,
        metric: 'io_pending',
        label: 'IO Pending',
        color: Color_1.Color.metric.read
    },
    data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'data_rate',
        label: 'Data',
        color: Color_1.Color.metric.read
    },
    read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'read',
        label: 'Read',
        longLabel: 'Data Read',
        color: Color_1.Color.metric.read
    },
    write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'write',
        label: 'Write',
        longLabel: 'Data Write',
        color: Color_1.Color.metric.write
    },
    data_rate_b: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'data_rate_b',
        label: 'Data Rate',
        longLabel: 'Data rate BE',
        color: Color_1.Color.pallete.redDark
    },
    read_b: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'read_b',
        label: 'Read',
        longLabel: "Read BE",
        color: Color_1.Color.metric.read
    },
    write_b: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'write_b',
        label: 'Write',
        longLabel: 'Write BE',
        color: Color_1.Color.metric.write
    },
    resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'resp_t',
        label: 'Latency',
        color: Color_1.Color.pallete.redDark
    },
    resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'resp_t_r',
        label: 'Latency Read',
        color: Color_1.Color.metric.read
    },
    resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'resp_t_w',
        label: 'Latency Write',
        color: Color_1.Color.metric.write
    },
    resp_t_b: {
        ...Definitions_1.metricTypes.latency,
        metric: 'resp_t_b',
        label: 'Latency BE',
        color: Color_1.Color.pallete.redDark
    },
    resp_t_r_b: {
        ...Definitions_1.metricTypes.latency,
        metric: 'resp_t_r_b',
        label: 'Latency Read BE',
        color: Color_1.Color.metric.read
    },
    resp_t_w_b: {
        ...Definitions_1.metricTypes.latency,
        metric: 'resp_t_w_b',
        label: 'Latency Write BE',
        color: Color_1.Color.metric.write
    },
    r_cache_hit: {
        metric: 'r_cache_hit',
        label: 'Read Cache Hit',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.metric.read
    },
    w_cache_hit: {
        metric: 'w_cache_hit',
        label: 'Write Cache Hit',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.metric.write
    },
    cache_usage: {
        metric: 'cache_usage',
        label: 'Cache Usage',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.redDark
    },
    r_cache_usage: {
        metric: 'r_cache_usage',
        label: 'Read Cache Usage',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.metric.write
    },
    w_cache_usage: {
        metric: 'w_cache_usage',
        label: 'Write Cache Usage',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.metric.read
    },
    cache_usage_percent: {
        metric: 'cache_usage_percent',
        label: 'Cache usage',
        longLabel: "Cache Usage %",
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.pallete.redDark
    },
    r_cache_usage_percent: {
        metric: 'r_cache_usage_percent',
        label: 'Cache Read Usage',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.metric.read
    },
    w_cache_usage_percent: {
        metric: 'w_cache_usage_percent',
        label: 'Cache Write Usage',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.metric.write
    },
    r_cache_miss_sec: {
        metric: 'r_cache_miss_sec',
        label: 'Block Cache Read Misses Misses/s',
        unit: subsystemMetricsResponse_constants_1.UNIT.misses_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.misses_per_second,
        color: Color_1.Color.metric.read
    },
    w_cache_miss_sec: {
        metric: 'w_cache_miss_sec',
        label: 'Block Cache Write Misses Misses/s',
        unit: subsystemMetricsResponse_constants_1.UNIT.misses_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.misses_per_second,
        color: Color_1.Color.metric.write
    },
    r_cache_hit_sec: {
        metric: 'r_cache_hit_sec',
        label: 'Block Cache Read Hits Hits/s',
        unit: subsystemMetricsResponse_constants_1.UNIT.hits_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.hits_per_second,
        color: Color_1.Color.pallete.red
    },
    w_cache_hit_sec: {
        metric: 'w_cache_hit_sec',
        label: 'Block Cache Write Hits Hits/s',
        unit: subsystemMetricsResponse_constants_1.UNIT.hits_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.hits_per_second,
        color: Color_1.Color.pallete.blue
    },
    block_size: {
        ...Definitions_1.metricTypes.block_size,
        metric: 'block_size',
        label: 'Block Size',
        color: Color_1.Color.metric.read,
    },
    block_size_r: {
        ...Definitions_1.metricTypes.block_size,
        metric: 'block_size_r',
        label: 'Block Size Read',
        color: Color_1.Color.metric.read
    },
    block_size_w: {
        ...Definitions_1.metricTypes.block_size,
        metric: 'block_size_w',
        label: 'Block Size Write',
        color: Color_1.Color.metric.write
    },
    capacity_total: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'capacity_total',
        label: 'Total',
        longLabel: 'Capacity Total',
        color: Color_1.Color.pallete.black
    },
    capacity_used: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'capacity_used',
        label: 'Used',
        longLabel: 'Capacity Used',
        color: Color_1.Color.pallete.red
    },
    capacity_free: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'capacity_free',
        label: 'Free',
        longLabel: 'Capacity Free',
        color: Color_1.Color.pallete.green
    },
    capacity_used_percent: {
        ...Definitions_1.metricTypes.percent,
        metric: 'capacity_used_percent',
        label: 'Capacity Used %',
        color: Color_1.Color.pallete.redDark,
    },
    sum_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'sum_io',
        label: 'IO Sum',
        color: Color_1.Color.pallete.redDark
    },
    cpu_system: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_system',
        label: 'System',
        longLabel: 'CPU System',
        color: Color_1.Color.metric.read
    },
    cpu_busy: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_busy',
        label: 'Busy',
        color: Color_1.Color.pallete.redDark
    },
    cpu_user: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_user',
        label: 'User',
        longLabel: 'CPU User',
        color: Color_1.Color.metric.read
    },
    cpu_wait: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_wait',
        longLabel: 'CPU Wait',
        label: 'Wait',
        color: Color_1.Color.metric.read
    },
    cpu_idle: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_idle',
        label: 'Idle',
        longLabel: 'cpu Idle',
        color: Color_1.Color.metric.read,
    },
    cpu_other: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_other',
        label: 'Other',
        longLabel: 'CPU Other',
        color: Color_1.Color.metric.read,
    },
    cpu_compress: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_compress',
        label: 'Compress',
        longLabel: 'CPU Compress',
        color: Color_1.Color.metric.read,
    },
    cpu_steal: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_steal',
        longLabel: 'CPU Steal',
        label: 'Steal',
        color: Color_1.Color.metric.read,
    },
    cpu_softirq: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_softirq',
        longLabel: 'CPU Softirq',
        label: 'Softirq',
        color: Color_1.Color.metric.read,
    },
    cpu_nice: {
        ...Definitions_1.metricTypes.percent,
        metric: 'cpu_nice',
        longLabel: 'CPU Nice',
        label: 'Nice',
        color: Color_1.Color.metric.read,
    },
    cpu_gpu_util_percent: {
        ...Definitions_1.metricTypes.percent,
        metric: 'gpu_util_percent',
        longLabel: 'Total',
        label: 'Total',
        color: Color_1.Color.metric.total,
    },
    compression_saving: {
        metric: 'compression_saving',
        label: 'Compression',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.redDark
    },
    dedupe_saving: {
        metric: 'dedupe_saving',
        label: 'Deduplication',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.redDark
    },
    ops_rate: {
        metric: 'ops_rate',
        label: 'OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.pallete.redDark
    },
    disk_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'disk_resp_t_w',
        label: 'Disk Write Latency',
        color: Color_1.Color.metric.write,
    },
    disk_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'disk_resp_t_r',
        label: 'Disk Read Latency',
        color: Color_1.Color.metric.read,
    },
    disk_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'disk_resp_t',
        label: 'Disk Latency',
        color: Color_1.Color.metric.read,
    },
    port_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'port_resp_t_w',
        label: 'Port Write Latency',
        color: Color_1.Color.metric.write,
    },
    port_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'port_resp_t_r',
        label: 'Port Read Latency',
        color: Color_1.Color.metric.read,
    },
    port_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'port_resp_t',
        label: 'Port Latency',
        color: Color_1.Color.metric.read,
    },
    read_ops: {
        metric: 'read_ops',
        label: 'OPS Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.read
    },
    write_ops: {
        metric: 'write_ops',
        label: 'OPS Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.write
    },
    nfs_ops_rate: {
        metric: 'nfs_ops_rate',
        label: 'NFS OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.pallete.redDark
    },
    nfs_other_ops_rate: {
        metric: 'nfs_other_ops_rate',
        label: 'NFS Other OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.pallete.redDark
    },
    nfs_read_ops: {
        metric: 'nfs_read_ops',
        label: 'NFS OPS Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.read,
    },
    nfs_write_ops: {
        metric: 'nfs_ops_rate',
        label: 'NFS OPS Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.write,
    },
    cifs_ops_rate: {
        metric: 'cifs_ops_rate',
        label: 'CIFS OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.read,
    },
    cifs_other_ops_rate: {
        metric: 'cifs_other_ops_rate',
        label: 'CIFS Other OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.read,
    },
    cifs_read_ops: {
        metric: 'cifs_read_ops',
        label: 'CIFS OPS Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.read,
    },
    cifs_write_ops: {
        metric: 'cifs_ops_rate',
        label: 'CIFS OPS Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        defaultPrecision: 0,
        color: Color_1.Color.metric.write
    },
    nfs_read_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'nfs_read_io',
        label: 'NFS IO Read',
        color: Color_1.Color.metric.read,
    },
    nfs_write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'nfs_write_io',
        label: 'NFS IO Write',
        color: Color_1.Color.metric.write,
    },
    nfs_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs_read',
        label: 'NFS Data Read',
        color: Color_1.Color.metric.read,
    },
    nfs_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs_write',
        label: 'NFS Data Write',
        color: Color_1.Color.metric.write
    },
    nfs_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs_data_rate',
        label: 'NFS Data Rate',
        color: Color_1.Color.metric.read,
    },
    nfs_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs_resp_t',
        label: 'NFS Latency',
        color: Color_1.Color.metric.read,
    },
    nfs_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs_resp_t_r',
        label: 'NFS Latency Read',
        color: Color_1.Color.metric.read,
    },
    nfs_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs_resp_t_w',
        label: 'NFS Latency Write',
        color: Color_1.Color.metric.write,
    },
    cifs_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'cifs_resp_t',
        label: 'CIFS Latency',
        color: Color_1.Color.metric.read,
    },
    cifs_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'cifs_resp_t_r',
        label: 'CIFS Latency Read',
        color: Color_1.Color.metric.read,
    },
    cifs_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'cifs_resp_t_w',
        label: 'CIFS Latency write',
        color: Color_1.Color.metric.write,
    },
    cifs_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'cifs_read',
        label: 'CIFS Data Read',
        color: Color_1.Color.metric.read,
    },
    cifs_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'cifs_write',
        label: 'CIFS Data Write',
        color: Color_1.Color.metric.write,
    },
    cifs_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'cifs_data_rate',
        label: 'CIFS Data Rate',
        color: Color_1.Color.metric.read,
    },
    nfs4_ops_rate: {
        metric: 'nfs4_ops_rate',
        label: 'NFS4 OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        color: Color_1.Color.metric.read,
    },
    smb_read_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'smb_read_io',
        label: 'SMB IO Read',
        color: Color_1.Color.metric.read,
    },
    smb_write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'smb_write_io',
        label: 'SMB IO Write',
        color: Color_1.Color.metric.write,
    },
    smb_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'smb_read',
        label: 'SMB Data Read',
        color: Color_1.Color.metric.read,
    },
    smb_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'smb_write',
        label: 'SMB Data Write',
        color: Color_1.Color.metric.write,
    },
    smb_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'smb_resp_t_r',
        label: 'SMB Latency Read',
        color: Color_1.Color.metric.read,
    },
    smb_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'smb_resp_t_w',
        label: 'SMB Latency Write',
        color: Color_1.Color.metric.write,
    },
    smb1_ops_rate: {
        metric: 'smb1_ops_rate',
        label: 'SMB1 OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        color: Color_1.Color.metric.read,
    },
    smb2_ops_rate: {
        metric: 'smb2_ops_rate',
        label: 'SMB2 OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
        color: Color_1.Color.metric.read,
    },
    loss_sig: {
        metric: 'loss_sig',
        label: 'Loss Signal',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
        color: Color_1.Color.metric.read,
    },
    loss_sync: {
        metric: 'loss_sync',
        label: 'Loss Sync',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
        color: Color_1.Color.pallete.redDark
    },
    link_fail: {
        metric: 'link_fail',
        label: 'Link Failure',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
        color: Color_1.Color.pallete.redDark
    },
    buffer_error: {
        metric: 'buffer_error',
        label: 'Buffer Error',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
        color: Color_1.Color.pallete.redDark
    },
    crc: {
        metric: 'crc',
        label: 'CRC Errors',
        unit: subsystemMetricsResponse_constants_1.UNIT.crc_errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.crc_errors_per_second,
        color: Color_1.Color.pallete.redDark
    },
    b2b: {
        metric: 'b2b',
        label: 'B2B',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.metric.read,
    },
    write_pending_percent: {
        metric: 'write_pending_percent',
        label: 'Write Pending',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
    },
    temperature: {
        metric: 'temperature',
        label: 'Temperature',
        unit: subsystemMetricsResponse_constants_1.UNIT.temperature,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.temperature,
        color: Color_1.Color.pallete.redDark
    },
    replication_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'replication_rate',
        label: 'Replication',
        longLabel: 'Replication Rate',
        color: Color_1.Color.metric.read
    },
    read_replication: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'read_replication',
        label: 'Read Replicaton',
        color: Color_1.Color.metric.read,
    },
    write_replication: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'write_replication',
        label: 'Write Replicaton',
        color: Color_1.Color.metric.write
    },
    copy_job_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'copy_job_read',
        label: 'Read Copy Job',
        color: Color_1.Color.metric.read
    },
    copy_job_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'copy_job_write',
        label: 'Write Copy Job',
        color: Color_1.Color.metric.write
    },
    data_job_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'data_job_read',
        label: 'Read Data Job',
        color: Color_1.Color.metric.read
    },
    data_job_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'data_job_write',
        label: 'Write Data Job',
        color: Color_1.Color.metric.write
    },
    stream_rate: {
        metric: 'stream_rate',
        label: 'Stream Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
        color: Color_1.Color.metric.read
    },
    stream_write_replication: {
        metric: 'stream_write_replication',
        label: 'Stream Write Repl Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
        color: Color_1.Color.metric.write
    },
    stream_read_replication: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'stream_read_replication',
        label: 'Stream Read Repl Rate',
        color: Color_1.Color.metric.read
    },
    stream_replication_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'stream_replication_rate',
        label: 'Stream Repl Rate',
        color: Color_1.Color.metric.read
    },
    stream_job_rate: {
        metric: 'stream_job_rate',
        label: 'Stream Job Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
    },
    logical_data_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'logical_data_read',
        label: 'Logical Read',
    },
    logical_data_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'logical_data_write',
        label: 'Logical Write',
    },
    logical_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'logical_data_rate',
        label: 'Logical Data Rate',
    },
    logical_copy_job_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'logical_copy_job_read',
        label: 'Logical Copy Read',
        color: Color_1.Color.metric.read
    },
    logical_copy_job_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'logical_copy_job_write',
        label: 'Logical Copy Write',
        color: Color_1.Color.metric.write
    },
    logical_data_job_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'logical_data_job_read',
        label: 'Logical Data Read',
        color: Color_1.Color.metric.read
    },
    logical_data_job_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'logical_data_job_write',
        label: 'Logical Data Write',
        color: Color_1.Color.metric.write
    },
    system_memory_total: {
        metric: 'system_memory_total',
        label: 'Total',
        longLabel: 'Memory System Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.metric.total
    },
    system_memory_used: {
        metric: 'system_memory_used',
        label: 'Used',
        longLabel: 'Memory System Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    onheap_memory_total: {
        metric: 'onheap_memory_total',
        label: 'Total',
        longLabel: "Memory Onheap Total",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.metric.total
    },
    onheap_memory_used: {
        metric: 'onheap_onheap_used',
        label: 'Used',
        longLabel: 'Memory On-heap Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    offheap_memory_total: {
        metric: 'offheap_memory_total',
        label: 'Total',
        longLabel: 'Memory Off-heap total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.black
    },
    offheap_memory_used: {
        metric: 'offheap_memory_used',
        label: 'Used',
        longLabel: 'Memory Off-heap Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    memory_used: {
        metric: 'memory_used',
        label: 'Used',
        longLabel: 'Memory Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    memory_free: {
        metric: 'memory_free',
        label: 'Free',
        longLabel: 'Memory Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.green
    },
    memory_total: {
        metric: 'memory_total',
        label: 'Total',
        longLabel: 'Memory Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    memory_cached: {
        metric: 'memory_cached',
        label: 'Cached',
        longLabel: 'Memory Cached',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.cyan
    },
    memory_buffer: {
        metric: 'memory_buffer',
        label: 'Buffer',
        longLabel: 'Memory Buffer',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.blue
    },
    memory_reserved: {
        metric: 'memory_reserved',
        label: 'Reserved',
        longLabel: 'Memory Reserverd',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.black
    },
    memory_gpu_util_percent: {
        metric: 'memory_gpu_util_percent',
        label: 'Memory Util GPU %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.pallete.totalBlue
    },
    memory_gpu_free: {
        metric: 'memory_gpu_free',
        label: 'Memory Free GPU %',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.green
    },
    memory_gpu_used: {
        metric: 'memory_gpu_used',
        label: 'Memory Used GPU %',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.totalRed
    },
    memory_gpu_total: {
        metric: 'memory_gpu_total',
        label: 'Memory Total GPU %',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.black
    },
    memory_util_percent: {
        metric: 'memory_util_percent',
        label: 'Memory Util %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.pallete.totalBlue
    },
    memory_used_percent: {
        metric: 'memory_used_percent',
        label: 'Memory Used %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.pallete.red
    },
    swap_out: {
        metric: 'swap_out',
        label: 'Swap Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.pages_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.pages_per_second,
        color: Color_1.Color.pallete.red
    },
    utilization: {
        metric: 'utilization',
        label: 'Utilization',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        color: Color_1.Color.metric.read
    },
    read_error: {
        metric: 'read_error',
        label: 'Read Error',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
        color: Color_1.Color.metric.read
    },
    write_error: {
        metric: 'write_error',
        label: 'Write Error',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
        color: Color_1.Color.metric.write
    },
    error_rate: {
        metric: 'error_rate',
        label: 'Error Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
        color: Color_1.Color.metric.read
    },
    read_drop: {
        metric: 'read_drop',
        label: 'Read Drop',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
        color: Color_1.Color.metric.read
    },
    write_drop: {
        metric: 'write_drop',
        label: 'Write Drop',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
        color: Color_1.Color.metric.write
    },
    drop_rate: {
        metric: 'drop_rate',
        label: 'Drop Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
        color: Color_1.Color.metric.read
    },
    read_packet: {
        metric: 'read_packet',
        label: 'Read Packet',
        unit: subsystemMetricsResponse_constants_1.UNIT.packets_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.packets_per_second,
    },
    write_packet: {
        metric: 'write_packet',
        label: 'Write Packet',
        unit: subsystemMetricsResponse_constants_1.UNIT.packets_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.packets_per_second,
    },
    packet_rate: {
        metric: 'packet_rate',
        label: 'Packet Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.packets_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.packets_per_second,
    },
    local_write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'local_write_io',
        label: 'Local Write IO',
    },
    mirrored_write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'mirrored_write_io',
        label: 'Mirrored Write IO',
    },
    local_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'local_write',
        label: 'Local Write',
    },
    mirrored_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'mirrored_write',
        label: 'Mirrored Write',
    },
    local_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'local_resp_t_w',
        label: 'Local Latency Write',
    },
    mirrored_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'mirrored_resp_t_w',
        label: 'Mirrored Latency Write',
    },
    w_cache_hit_io: {
        metric: 'w_cache_hit_io',
        label: 'Write Cache Hit IO',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    r_cache_hit_io: {
        metric: 'r_cache_hit_io',
        label: 'Read Cache Hit IO',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    io_rate_initiator: {
        ...Definitions_1.metricTypes.iops,
        metric: 'io_rate_initiator',
        label: 'IO Rate Initiator',
    },
    data_rate_initiator: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'data_rate_initiator',
        label: 'Data rate Initiator',
    },
    resp_t_initiator: {
        ...Definitions_1.metricTypes.latency,
        metric: 'resp_t_initiator',
        label: 'Latency Initiator',
    },
    physical_capacity_total: {
        metric: 'physical_capacity_total',
        label: 'Capacity Physical Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.metric.total
    },
    physical_capacity_used: {
        metric: 'physical_capacity_used',
        label: 'Capacity Physical Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    physical_capacity_free: {
        metric: 'physical_capacity_free',
        label: 'Capacity Physical Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    format_reserved_capacity_used: {
        metric: 'format_reserved_capacity_used',
        label: 'Reserved',
        longLabel: 'Format Reserved Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    fmc_compression_saving: {
        metric: 'fmc_compression_saving',
        label: 'FMC Compression',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    fmc_compression_percent: {
        metric: 'fmc_compression_percent',
        label: 'FMC Compression %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
    },
    dkc_compression_saving: {
        metric: 'dkc_compression_saving',
        label: 'DKC Compression',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    dkc_compression_percent: {
        metric: 'dkc_compression_percent',
        label: 'DKC Compression %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
    },
    dkc_dedupe_saving: {
        metric: 'dkc_dedupe_saving',
        label: 'DKC Deduplication',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    dkc_dedupe_percent: {
        metric: 'dkc_dedupe_percent',
        label: 'DKC Deduplication %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
    },
    dkc_zpr_saving: {
        metric: 'dkc_zpr_saving',
        label: 'DKC ZPR',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    dkc_zpr_percent: {
        metric: 'dkc_zpr_percent',
        label: 'DKC ZPR %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
    },
    snapshot_capacity_used: {
        metric: 'snapshot_capacity_used',
        label: 'Snapshot',
        longLabel: "Snapshot Capacity Used",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    snapshot_capacity_free: {
        metric: 'snapshot_capacity_free',
        label: 'Snapshot free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    capacity_data_in: {
        metric: 'capacity_data_in',
        label: 'Data In',
        longLabel: 'Capacity Data In',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    capacity_logical_usage: {
        metric: 'capacity_logical_usage',
        label: 'Logical Usage',
        longLabel: 'Capacity Logical Usage',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    capacity_data_written: {
        metric: 'capacity_data_written',
        label: 'Data Written',
        longLabel: 'Capacity Data Written',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    capacity_data_reduction: {
        metric: 'capacity_data_reduction',
        label: 'Data Reduction',
        longLabel: 'Capacity Data Reduction',
        unit: subsystemMetricsResponse_constants_1.UNIT.multiples,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.multiples,
    },
    capacity_storage_consumed: {
        metric: 'capacity_storage_consumed',
        label: 'Storage Consumed',
        longLabel: 'Capacity Storage Consumed',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    spare_capacity_used_percent: {
        ...Definitions_1.metricTypes.percent,
        metric: 'spare_capacity_used_percent',
        label: 'Spare Used %',
    },
    capacity_allocated: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'capacity_allocated',
        label: 'Allocated',
        longLabel: 'Capacity Allocated'
    },
    capacity_failed: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'capacity_failed',
        label: 'Failed',
        longLabel: 'Capacity Failed',
    },
    user_capacity_allocated: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'user_capacity_allocated',
        label: 'Allocated User',
        longLabel: 'Capacity Allocated User'
    },
    snapshot_capacity_allocated: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'snapshot_capacity_allocated',
        label: 'Allocated Snapshot',
        longLabel: 'Capacity Allocated Snapshot'
    },
    user_capacity_total: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'user_capacity_total',
        label: 'Total',
        longLabel: 'Capacity User Total',
    },
    user_capacity_used: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'user_capacity_used',
        label: 'Used',
        longLabel: 'Capacity User Used'
    },
    user_capacity_free: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'user_capacity_free',
        label: 'Free',
        longLabel: 'Capacity User Free',
    },
    user_capacity_used_percent: {
        ...Definitions_1.metricTypes.percent,
        metric: 'user_capacity_used_percent',
        label: 'Used User %',
        longLabel: 'Capacity User Used',
    },
    sum_capacity_used: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'sum_capacity_used',
        label: 'Used Total',
        longLabel: 'Capacity Total Used',
    },
    sum_capacity_used_percent: {
        ...Definitions_1.metricTypes.percent,
        metric: 'sum_capacity_used_percent',
        label: 'Used Total %',
        longLabel: 'Capacity Total Used %',
    },
    capacity_used_replication: {
        metric: 'capacity_used_replication',
        label: 'Used Replication',
        longLabel: 'Capacity Replication Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    capacity_used_data: {
        metric: 'capacity_used_data',
        label: 'Used Data',
        longLabel: 'Capacity Data Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    capacity_oversubscr: {
        metric: 'capacity_oversubscr',
        label: 'Oversubscription',
        longLabel: 'Capacity Oversubscription ',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    unused_capacity_total: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'unused_capacity_total',
        label: 'Unused',
        longLabel: 'Capacity Unused',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    spare_capacity_total: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'spare_capacity_total',
        label: 'Spare',
        longLabel: 'Capacity Spare',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    spare_capacity_used: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'spare_capacity_used',
        label: 'Spare Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    spare_capacity_free: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'spare_capacity_free',
        label: 'Spare Capacity Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.green
    },
    raw_capacity_used: {
        metric: 'raw_capacity_used',
        label: 'Used',
        longLabel: 'Raw Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    raw_capacity_free: {
        metric: 'raw_capacity_free',
        label: 'Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        longLabel: 'Raw Capacity Free',
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    virtual_capacity_total: {
        metric: 'virtual_capacity_total',
        label: 'Virtual',
        longLabel: 'Capacity Virtual',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.brownDark
    },
    virtual_capacity_used: {
        metric: 'virtual_capacity_used',
        label: 'Used',
        longLabel: 'Capacity Virtual Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    virtual_capacity_free: {
        metric: 'virtual_capacity_free',
        label: 'Free',
        longLabel: 'Capacity Virtual Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    raw_capacity_total: {
        metric: 'raw_capacity_total',
        label: 'Total',
        longLabel: 'Raw Capacity Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.black
    },
    raw_raid_capacity_total: {
        metric: 'raw_raid_capacity_total',
        label: 'Total',
        longLabel: 'Raw Raid Capacity Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    raw_raid_capacity_used: {
        metric: 'raw_raid_capacity_used',
        label: 'Used',
        longLabel: 'Raw Raid Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.red
    },
    raw_raid_capacity_free: {
        metric: 'raw_raid_capacity_free',
        label: 'Free',
        longLabel: 'Raw Raid Capacity Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.green
    },
    raw_replication_capacity_used: {
        metric: 'raw_replication_capacity_used',
        label: 'Replication',
        longLabel: 'Raw Replication Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    raw_snapshot_capacity_used: {
        metric: 'raw_snapshot_capacity_used',
        label: 'Snapshot',
        longLabel: 'Raw Snapshot Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    raw_data_capacity_used: {
        metric: 'raw_data_capacity_used',
        label: 'Data',
        longLabel: 'Raw Data Capacity used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    metadata_capacity_used: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'metadata_capacity_used',
        label: 'Used',
        longLabel: 'Metadata Capacity Used',
    },
    metadata_capacity_free: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'metadata_capacity_free',
        label: 'Metadata Capacity Free',
        longLabel: 'Metadata Capacity Free',
    },
    metadata_capacity_total: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'metadata_capacity_total',
        label: 'Metadata Capacity Total',
        longLabel: 'Metadata Capacity Total',
    },
    format_raid_capacity_total: {
        metric: 'format_raid_capacity_total',
        label: 'Total',
        longLabel: 'Format Raid Capacity Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    format_raid_capacity_used: {
        metric: 'format_raid_capacity_used',
        label: 'Used',
        longLabel: 'Format Raid Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    format_raid_capacity_free: {
        metric: 'format_raid_capacity_free',
        label: 'Free',
        longLabel: 'Format Raid Capacity Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    format_replication_capacity_used: {
        metric: 'format_replication_capacity_used',
        label: 'Replication',
        longLabel: 'Format Replication Capacity Free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    format_snapshot_capacity_used: {
        metric: 'format_snapshot_capacity_used',
        label: 'Snapshot',
        longLabel: 'Format Snapshot Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    format_data_capacity_used: {
        metric: 'format_data_capacity_used',
        label: 'Data',
        longLabel: 'Format Data Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    user_data_used: {
        metric: 'user_data_used',
        label: 'User Data Used',
        longLabel: 'Capacity Data User Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    tier1_capacity_total: {
        metric: 'tier1_capacity_total',
        label: 'Capacity Tier 1',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    tier0_capacity_total: {
        metric: 'tier0_capacity_total',
        label: 'Capacity Tier 0',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    tier2_capacity_total: {
        metric: 'tier2_capacity_total',
        label: 'Capacity Tier 2',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    reduction_involved_capacity_total: {
        metric: 'reduction_involved_capacity_total',
        label: 'Compression & Deduplication',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    replication_capacity_total: {
        metric: 'replication_capacity_total',
        label: 'Replication',
        longLabel: 'Replication Capacity Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    iscsi_resp_t: {
        metric: 'iscsi_resp_t',
        label: 'Latency',
        longLabel: 'iSCSI Latency',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    iscsi_resp_t_r: {
        metric: 'iscsi_resp_t_r',
        label: 'Latency read',
        longLabel: 'iSCSI Latency Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    iscsi_resp_t_w: {
        metric: 'iscsi_resp_t_w',
        label: 'Latency write',
        longLabel: 'iSCSI Latency Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    fc_resp_t: {
        metric: 'fc_resp_t',
        label: 'Latency',
        longLabel: 'FC Latency',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    fc_resp_t_r: {
        metric: 'fc_resp_t_r',
        label: 'Latency read',
        longLabel: 'FC Latency Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    fc_resp_t_w: {
        metric: 'fc_resp_t_w',
        label: 'Latency write',
        longLabel: 'FC Latency Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    rebalance_rate: {
        metric: 'rebalance_rate',
        label: 'Rebalance Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
        color: Color_1.Color.pallete.redDark,
    },
    rebuild_rate: {
        metric: 'rebuild_rate',
        label: 'Rebuild Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
        color: Color_1.Color.pallete.redDark,
    },
    rebalance_pending: {
        metric: 'rebalance_pending',
        label: 'Rebalance Pending Capacity',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.redDark,
    },
    rebuild_pending: {
        metric: 'rebuild_pending',
        label: 'Rebuild Pending Capacity',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
        color: Color_1.Color.pallete.redDark,
    },
    replication_in: {
        metric: 'replication_in',
        label: 'Replication In',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    replication_out: {
        metric: 'replication_out',
        label: 'Replication Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    replication_app_resp_t: {
        metric: 'replication_app_resp_t',
        label: 'Apply Latency',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    replication_in_resp_t: {
        metric: 'replication_in_resp_t',
        label: 'Replication In Latency',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    replication_out_resp_t: {
        metric: 'replication_out_resp_t',
        label: 'Replication Out Latency',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    replication_journal_total: {
        metric: 'replication_journal_total',
        label: 'Replication Journal Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    replication_journal_used: {
        metric: 'repl_journal_used',
        label: 'Replication Journal Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    housekeep_pend_data: {
        metric: 'housekeep_pend_data',
        label: 'Pending Data',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    dedupe_ratio: {
        metric: 'dedupe_ratio',
        label: 'Dedupe Ratio',
        unit: subsystemMetricsResponse_constants_1.UNIT.ratio,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.ratio,
    },
    disk_size: {
        metric: 'disk_size',
        label: 'Size On Disk',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    dedupe_ratio_percent: {
        metric: 'dedupe_ratio_percent',
        label: 'Ratio',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
    },
    network_rx: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'network_rx',
        label: 'In',
    },
    network_tx: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'network_tx',
        label: 'Out',
    },
    network_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'network_rate',
        label: 'Network',
    },
    network_rx_b: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'network_rx_b',
        label: 'In BE',
    },
    network_tx_b: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'network_tx_b',
        label: 'Out BE',
    },
    network_rate_b: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'network_rate_b',
        label: 'Network BE',
    },
    network_util: {
        metric: 'network_util',
        label: 'Net Util',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        defaultPrecision: 1
    },
    network_util_rx: {
        metric: 'network_util_rx',
        label: 'Util In',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        defaultPrecision: 1
    },
    network_util_tx: {
        metric: 'network_util_tx',
        label: 'Util Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        defaultPrecision: 1
    },
    phase: {
        metric: 'phase',
        label: 'Cleaning Phase',
        unit: subsystemMetricsResponse_constants_1.UNIT.phase,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.phase,
    },
    read_net: {
        metric: 'read_net',
        label: 'Read Net',
        unit: subsystemMetricsResponse_constants_1.UNIT.net_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.net_per_second,
    },
    write_net: {
        metric: 'write_net',
        label: 'Write Net',
        unit: subsystemMetricsResponse_constants_1.UNIT.net_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.net_per_second,
    },
    net_rate: {
        metric: 'net_rate',
        label: 'Net Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.net_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.net_per_second,
    },
    outbound_pre_comp: {
        metric: 'outbound_pre_comp',
        label: 'Outbound Pre Comp',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    inbound_pre_comp: {
        metric: 'inbound_pre_comp',
        label: 'Inbound Pre Comp',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    outbound_net: {
        metric: 'outbound_net',
        label: 'Outbound Network',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    inbound_net: {
        metric: 'inbound_net',
        label: 'Inbound Network',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    s3_total: {
        metric: 's3_total',
        label: 'S3 Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    s3_ingested: {
        metric: 's3_ingested',
        label: 'S3 Ingest',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    s3_retrieved: {
        metric: 's3_retrieved',
        label: 'S3 Retrieve',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    s3_ops_successful: {
        metric: 's3_ops_successful',
        label: 'S3 Successful',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    s3_ops_failed: {
        metric: 's3_ops_failed',
        label: 'S3 Failed',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    s3_ops_unauthorized: {
        metric: 's3_ops_unauthorized',
        label: 'S3 Unauthorized',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    swift_total: {
        metric: 'swift_total',
        label: 'Swift Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    swift_ingested: {
        metric: 'swift_ingested',
        label: 'Swift Ingest',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    swift_retrieved: {
        metric: 'swift_retrieved',
        label: 'Swift Retrieve',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second
    },
    swift_ops_successful: {
        metric: 'swift_ops_successful',
        label: 'Swift Successful',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    swift_ops_failed: {
        metric: 'swift_ops_failed',
        label: 'Swift Failed',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    swift_ops_unauthorized: {
        metric: 'swift_ops_unauthorized',
        label: 'Swift Unauthorized',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    ilm_queue_all: {
        metric: 'ilm_queue_all',
        label: 'ILM Queue All',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    ilm_queue_client: {
        metric: 'ilm_queue_client',
        label: 'ILM Queue Client',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    ilm_queue_background: {
        metric: 'ilm_queue_background',
        label: 'ILM Queue Background',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    ilm_queue_eval_rate: {
        metric: 'ilm_queue_eval_rate',
        label: 'ILM queue Eval Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    ilm_scan_rate: {
        metric: 'ilm_scan_rate',
        label: 'ILM Queue Scan Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    lb_total: {
        metric: 'lb_total',
        label: 'Load Balancer Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    lb_rx: {
        metric: 'lb_rx',
        label: 'Load Balancer In',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    lb_tx: {
        metric: 'lb_tx',
        label: 'Load Balancer Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    lb_request_rate: {
        metric: 'lb_request_rate',
        label: 'Load Balancer Request Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.IOPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.IOPS,
    },
    lb_error_rate: {
        metric: 'lb_error_rate',
        label: 'Load Balancer Error Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.errors_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.errors_per_second,
    },
    total_comp_factor: {
        metric: 'total_comp_factor',
        label: 'Total',
        longLabel: 'Total Comp Factor',
        unit: subsystemMetricsResponse_constants_1.UNIT.compression_factor,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.compression_factor
    },
    global_comp_factor: {
        metric: 'global_comp_factor',
        label: 'Global',
        longLabel: 'Global Comp Factor',
        unit: subsystemMetricsResponse_constants_1.UNIT.compression_factor,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.compression_factor
    },
    local_comp_factor: {
        metric: 'local_comp_factor',
        label: 'Local',
        longLabel: 'Local Comp Factor',
        unit: subsystemMetricsResponse_constants_1.UNIT.compression_factor,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.compression_factor
    },
    cfc_dirty: {
        metric: 'cfc_dirty',
        label: 'Cfc Dirty',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count
    },
    del_ack: {
        metric: 'del_ack',
        label: 'Del Ack',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count
    },
    livemount_capacity_used: {
        metric: 'livemount_capacity_used',
        label: 'Livemount',
        longLabel: 'Livemount Capacity Used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    cdp_capacity_used: {
        metric: 'cdp_capacity_used',
        label: 'CDP',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    system_capacity_used: {
        metric: 'system_capacity_used',
        label: 'Used',
        longLabel: 'Capacity System used',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    system_capacity_free: {
        metric: 'system_capacity_free',
        label: 'Free',
        longLabel: 'Capacity System free',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    system_capacity_total: {
        metric: 'system_capacity_total',
        label: 'Total',
        longLabel: 'Capacity System total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    system_capacity_used_percent: {
        ...Definitions_1.metricTypes.percent,
        metric: 'system_capacity_used_percent',
        label: 'System Capacity Used %',
        color: Color_1.Color.pallete.redDark,
    },
    shared_space_capacity_used: {
        metric: 'shared_space_capacity_used',
        longLabel: 'Capacity Shared Space',
        label: 'Share Space',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    snapshot_capacity_pending: {
        metric: 'snapshot_capacity_pending',
        label: 'Pending Snapshot',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    volumes_capacity_used: {
        metric: 'volumes_capacity_used',
        label: 'capacity Volumes',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    slices: {
        metric: 'slices',
        label: 'Slices',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
    },
    rebuild_queue: {
        metric: 'rebuild_queue',
        label: 'Rebuild Queue',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
    },
    rebuild_writes_total: {
        metric: 'rebuild_writes_total',
        label: 'Rebuild Writes Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.slices_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.slices_per_second,
    },
    rebuild_writes_in: {
        metric: 'rebuild_writes_in',
        label: 'Rebuild Writes In',
        unit: subsystemMetricsResponse_constants_1.UNIT.slices_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.slices_per_second,
    },
    rebuild_writes_out: {
        metric: 'rebuild_writes_out',
        label: 'Rebuild Writes Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.slices_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.slices_per_second,
    },
    rebuild_deletes_total: {
        metric: 'rebuild_deletes_total',
        label: 'Rebuild Deletes Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.slices_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.slices_per_second,
    },
    rebuild_deletes_in: {
        metric: 'rebuild_deletes_in',
        label: 'Rebuild Deletes In',
        unit: subsystemMetricsResponse_constants_1.UNIT.slices_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.slices_per_second,
    },
    rebuild_deletes_out: {
        metric: 'rebuild_deletes_out',
        label: 'Rebuild Deletes Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.slices_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.slices_per_second,
    },
    rebuild_bytes_total: {
        metric: 'rebuild_bytes_total',
        label: 'Rebuild bytes total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    rebuild_bytes_in: {
        metric: 'rebuild_bytes_in',
        label: 'Rebuild Bytes In',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    rebuild_bytes_out: {
        metric: 'rebuild_bytes_out',
        label: 'Rebuild Bytes Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    acknowledge_time: {
        metric: 'acknowledge_time',
        label: 'Acknowledge Time',
        unit: subsystemMetricsResponse_constants_1.UNIT.millisecond,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.millisecond,
    },
    scanned_srcs: {
        metric: 'scanned_srcs',
        label: 'Scanned Sources',
        unit: subsystemMetricsResponse_constants_1.UNIT.sources_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.sources_per_second,
    },
    tcp_timeouts: {
        metric: 'tcp_timeouts',
        label: 'TCP Timeouts',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
    },
    os_total: {
        metric: 'os_total',
        label: 'OS Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    os_r: {
        metric: 'os_r',
        label: 'OS Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    os_w: {
        metric: 'os_w',
        label: 'OS Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    client_total: {
        metric: 'client_total',
        label: 'Client Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    client_in: {
        metric: 'client_in',
        label: 'Client In',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    client_out: {
        metric: 'client_out',
        label: 'Client Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    slicestor_total: {
        metric: 'slicestor_total',
        label: 'Slicestor Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    slicestor_in: {
        metric: 'slicestor_in',
        label: 'Slicestor In',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    slicestor_out: {
        metric: 'slicestor_out',
        label: 'Slicestor Out',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    snapshot_capacity_physical: {
        metric: 'snapshot_capacity_physical',
        label: 'Physical',
        longLabel: 'Snapshot Capacity Physical',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    snapshot_capacity_ingested: {
        metric: 'snapshot_capacity_ingested',
        label: 'Ingested',
        longLabel: 'Snapshot Capacity Ingested',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    snapshot_capacity_reduction_percent: {
        metric: 'snapshot_capacity_reduction_percent',
        label: 'Snapshot Reduction %',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
    },
    cloud_physical: {
        metric: 'cloud_physical',
        label: 'Cloud Physical',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    cloud_ingested: {
        metric: 'cloud_ingested',
        label: 'Cloud Ingested',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    archive: {
        metric: 'archive',
        label: 'Archive',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    physical_ingest: {
        metric: 'physical_ingest',
        label: 'Physical Ingest',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes_per_second,
    },
    queue_length: {
        metric: 'queue_length',
        label: 'Queue Length',
        unit: subsystemMetricsResponse_constants_1.UNIT.unit_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.unit_per_second,
    },
    data_reduction_ratio: {
        metric: 'data_reduction_ratio',
        label: 'Data Reduction Ratio',
        unit: subsystemMetricsResponse_constants_1.UNIT.ratio,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.ratio,
    },
    read_tps: {
        metric: 'read_tps',
        label: 'Read Transactions',
        unit: subsystemMetricsResponse_constants_1.UNIT.transactions_per_sec,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.transactions_per_sec,
    },
    write_tps: {
        metric: 'write_tps',
        label: 'Write Transactions',
        unit: subsystemMetricsResponse_constants_1.UNIT.transactions_per_sec,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.transactions_per_sec,
    },
    tps_rate: {
        metric: 'tps_rate',
        label: 'Transactions Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.transactions_per_sec,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.transactions_per_sec,
    },
    error_rate_tps: {
        metric: 'error_tps',
        label: 'Error Transactions Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.transactions_per_sec,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.transactions_per_sec,
    },
    disk_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_read',
        label: 'Read',
        longLabel: 'Disk Read Rate'
    },
    disk_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_write',
        label: 'Write',
        longLabel: 'Disk Write Rate'
    },
    disk_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_data_rate',
        label: 'Data Rate',
        longLabel: 'Disk Data Rate'
    },
    disk_geo_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_geo_read',
        label: 'Geo Read',
    },
    disk_geo_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_geo_write',
        label: 'Geo Write',
    },
    disk_geo_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_geo_data_rate',
        label: 'Geo Data Rate',
    },
    disk_user_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_user_read',
        label: 'User Read',
    },
    disk_user_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_user_write',
        label: 'User Write',
    },
    disk_user_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'disk_user_data_rate',
        label: 'User Data Rate',
    },
    object_created: {
        metric: 'object_created',
        label: 'Object Created',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
        defaultPrecision: 0
    },
    object_deleted: {
        metric: 'object_deleted',
        label: 'Object Deleted',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
        defaultPrecision: 0
    },
    object_total: {
        metric: 'object_total',
        label: 'Object Total',
        unit: subsystemMetricsResponse_constants_1.UNIT.count,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.count,
        defaultPrecision: 0
    },
    soft_quota: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'soft_quota',
        label: 'Soft Quota',
    },
    hard_quota: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'hard_quota',
        label: 'Hard Quota'
    },
    delta_size: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'delta_size',
        label: 'Delta Size'
    },
    size: {
        ...Definitions_1.metricTypes.capacity,
        metric: 'size',
        label: 'Size',
    },
    file_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'file_data_rate',
        longLabel: 'File Data Rate',
        label: 'File Total',
    },
    file_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'file_read',
        label: 'File Read',
        longLabel: 'File Data Read'
    },
    file_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'file_write',
        label: 'File Write',
        longLabel: 'File Data Write'
    },
    ops_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'ops_resp_t',
        label: 'OPS Latency',
    },
    ops_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'ops_resp_t_r',
        label: 'OPS Latency Read',
    },
    ops_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'ops_resp_t_w',
        label: 'OPS Latency Write',
    },
    nfs41_ops_rate: {
        metric: 'nfs41_ops_rate',
        label: 'NFSv4.1 OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    nfs41_read_ops: {
        metric: 'nfs41_read_ops',
        label: 'NFSv4.1 OPS Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    nfs41_write_ops: {
        metric: 'nfs41_write_ops',
        label: 'NFSv4.1 OPS Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    nfs41_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs41_data_rate',
        label: 'NFSv4.1 Data Rate',
    },
    nfs41_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs41_read',
        label: 'NFSv4.1 Data Read',
    },
    nfs41_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs41_write',
        label: 'NFSv4.1 Data Write',
    },
    nfs41_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs41_resp_t',
        label: 'NFSv4.1 Latency',
    },
    nfs41_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs41_resp_t_r',
        label: 'NFSv4.1 Latency Read',
    },
    nfs41_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs41_resp_t_w',
        label: 'NFSv4.1 Latency Write',
    },
    nfs3_ops_rate: {
        metric: 'nfs3_ops_rate',
        label: 'NFSv3 OPS Rate',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    nfs3_read_ops: {
        metric: 'nfs3_read_ops',
        label: 'NFSv3 OPS Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    nfs3_write_ops: {
        metric: 'nfs3_write_ops',
        label: 'NFSv3 OPS Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    nfs3_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs3_data_rate',
        label: 'NFSv3 Data Rate',
    },
    nfs3_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs3_read',
        label: 'NFSv3 Data Read',
    },
    nfs3_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'nfs3_write',
        label: 'NFSv3 Data Write',
    },
    nfs3_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs3_resp_t',
        label: 'NFSv3 Latency',
    },
    nfs3_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs3_resp_t_r',
        label: 'NFSv3 Latency Read',
    },
    nfs3_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'nfs3_resp_t_w',
        label: 'NFSv3 Latency Write',
    },
    smb2_read_ops: {
        metric: 'smb2_read_ops',
        label: 'SMB2 OPS Read',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    smb2_write_ops: {
        metric: 'smb2_write_ops',
        label: 'SMB2 OPS Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.OPS,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.OPS,
    },
    smb2_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'smb2_data_rate',
        label: 'SMB2 Data Rate',
    },
    smb2_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'smb2_read',
        label: 'SMB2 Data Read',
    },
    smb2_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'smb2_write',
        label: 'SMB2 Data Write',
    },
    smb2_resp_t: {
        ...Definitions_1.metricTypes.latency,
        metric: 'smb2_resp_t',
        label: 'SMB2 Latency',
    },
    smb2_resp_t_r: {
        ...Definitions_1.metricTypes.latency,
        metric: 'smb2_resp_t_r',
        label: 'SMB2 Latency Read',
    },
    smb2_resp_t_w: {
        ...Definitions_1.metricTypes.latency,
        metric: 'smb2_resp_t_w',
        label: 'SMB2 Latency Write',
    },
    lun_read: {
        metric: 'lun_read',
        label: 'Read',
        longLabel: 'LUN Read ',
        unit: subsystemMetricsResponse_constants_1.UNIT.blocks_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.blocks_per_second,
        defaultPrecision: 1,
        color: Color_1.Color.metric.read
    },
    lun_write: {
        metric: 'lun_write',
        label: 'Write',
        longLabel: 'LUN Write',
        unit: subsystemMetricsResponse_constants_1.UNIT.blocks_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.blocks_per_second,
        defaultPrecision: 1,
        color: Color_1.Color.metric.write
    },
    lun_data_rate: {
        metric: 'lun_data_rate',
        label: 'Data',
        longLabel: 'LUN Data',
        unit: subsystemMetricsResponse_constants_1.UNIT.blocks_per_second,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.blocks_per_second,
        defaultPrecision: 1
    },
    lun_utilization: {
        metric: 'lun_utilization',
        label: 'Utilization',
        longLabel: 'LUN Util',
        unit: subsystemMetricsResponse_constants_1.UNIT.percent,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.percent,
        defaultPrecision: 1
    },
    capacity_ingested: {
        metric: 'capacity_ingested',
        label: 'Ingested',
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
        shortcut: subsystemMetricsResponse_constants_1.SHORTCUT.bytes,
    },
    hdfs_io_rate: {
        ...Definitions_1.metricTypes.iops,
        metric: 'hdfs_io_rate',
        label: 'HDFS total',
        longLabel: 'HDFS IO Rate',
    },
    hdfs_read_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'hdfs_read_io',
        label: 'HDFS Read',
        longLabel: 'HDFS IO Read',
    },
    hdfs_write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'hdfs_write_io',
        label: 'HDFS Write',
        longLabel: 'HDFS IO Write',
    },
    hdfs_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'hdfs_data_rate',
        label: 'HDFS Total',
        longLabel: 'HDFS Data Rate',
    },
    hdfs_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'hdfs_read',
        label: 'HDFS Read',
        longLabel: 'HDFS Data Read',
    },
    hdfs_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'hdfs_write',
        label: 'HDFS Write',
        longLabel: 'HDFS Data Write',
    },
    object_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'object_data_rate',
        label: 'Object Total',
        longLabel: 'Object Data Rate',
    },
    object_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'object_read',
        label: 'Object Read',
        longLabel: 'Object Data Read',
    },
    object_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'object_write',
        label: 'Object Write',
        longLabel: 'Object Data Write',
    },
    object_io_rate: {
        ...Definitions_1.metricTypes.iops,
        metric: 'object_io_rate',
        label: 'Object Total',
        longLabel: 'Object IO Rate',
    },
    object_read_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'object_read',
        label: 'Object Read',
        longLabel: 'Object IO Read',
    },
    object_write_io: {
        ...Definitions_1.metricTypes.iops,
        metric: 'object_write',
        label: 'Object Write',
        longLabel: 'Object IO Write',
    },
    fs_read: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'fs_read',
        label: 'FS Data Read',
        color: Color_1.Color.metric.read,
    },
    fs_write: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'fs_write',
        label: 'FS Data Write',
        color: Color_1.Color.metric.write
    },
    fs_data_rate: {
        ...Definitions_1.metricTypes.throughput,
        metric: 'fs_data_rate',
        label: 'FS Data Rate',
        color: Color_1.Color.metric.read,
    },
};
Storage.tabs = {
    totals: {
        iscsi_resp_t: {
            total: {
                label: 'ISCSI Total',
                metrics: [
                    {
                        ..._a.metrics.iscsi_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.redDark
                    },
                ],
            },
            read: {
                label: 'ISCSI Read',
                metrics: [
                    {
                        ..._a.metrics.iscsi_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.redDark
                    },
                ],
            },
            write: {
                label: 'ISCSI Write',
                metrics: [
                    {
                        ..._a.metrics.iscsi_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.blueDark
                    },
                ],
            },
        },
        fc_resp_t: {
            total: {
                label: 'FC Total',
                metrics: [
                    {
                        ..._a.metrics.fc_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.redDark
                    },
                ],
            },
            read: {
                label: 'FC Read',
                metrics: [
                    {
                        ..._a.metrics.fc_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.redDark
                    },
                ],
            },
            write: {
                label: 'FC Write',
                metrics: [
                    {
                        ..._a.metrics.fc_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.blueDark
                    },
                ],
            },
        },
        io: {
            total: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: -1,
                    },
                ],
            },
            total_b: {
                label: 'Total Back-end',
                metrics: [
                    {
                        ..._a.metrics.read_io_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_io_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: -1,
                    },
                ],
            },
            sum_io: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_io_rate: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_io_b: {
                label: 'Total Back-end',
                metrics: [
                    {
                        ..._a.metrics.read_io_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_io_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            aggregated: {
                label: 'Sum',
                metrics: [
                    {
                        ..._a.metrics.io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            aggregated_b: {
                label: 'Aggregated Back-end',
                metrics: [
                    {
                        ..._a.metrics.io_rate_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Read',
                metrics: [
                    {
                        ..._a.metrics.read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read_b: {
                label: 'Read Back-end',
                metrics: [
                    {
                        ..._a.metrics.read_io_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Write',
                metrics: [
                    {
                        ..._a.metrics.write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write_b: {
                label: 'Write Back-end',
                metrics: [
                    {
                        ..._a.metrics.write_io_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            pending: {
                label: 'Pending',
                metrics: [
                    {
                        ..._a.metrics.io_pending,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            local_write: {
                label: 'Local Write',
                metrics: [
                    {
                        ..._a.metrics.local_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            mirrored_write: {
                label: 'Mirrored Write',
                metrics: [
                    {
                        ..._a.metrics.mirrored_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            initiator: {
                label: 'IO Initiator',
                metrics: [
                    {
                        ..._a.metrics.io_rate_initiator,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            un_read_io: {
                label: 'Unaligned IO read',
                metrics: [
                    {
                        ..._a.metrics.un_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            un_write_io: {
                label: 'Unaligned IO write',
                metrics: [
                    {
                        ..._a.metrics.un_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_total_io_type: {
                label: 'IO Total Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        label: "File Total"
                    },
                ],
            },
            sum_total_io_object_type: {
                label: 'IO Total Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        label: "File Total"
                    },
                    {
                        ..._a.metrics.object_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_read_io_type: {
                label: 'IO Read Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        label: "File Read"
                    },
                ],
            },
            sum_read_io_object_type: {
                label: 'IO Read Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        label: "File Read"
                    },
                    {
                        ..._a.metrics.object_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_write_io_type: {
                label: 'IO Write Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        label: "File Write"
                    },
                ],
            },
            sum_write_io_object_type: {
                label: 'IO Write Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        label: "File Write"
                    },
                    {
                        ..._a.metrics.object_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            hdfs_io_rate: {
                label: 'HDFS Total',
                metrics: [
                    {
                        ..._a.metrics.hdfs_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            hdfs_read_io: {
                label: 'HDFS Read',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            hdfs_write_io: {
                label: 'HDFS Write',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            object_io_rate: {
                label: 'Object Total',
                metrics: [
                    {
                        ..._a.metrics.object_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            object_read_io: {
                label: 'Object Read',
                metrics: [
                    {
                        ..._a.metrics.object_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            object_write_io: {
                label: 'Object Write',
                metrics: [
                    {
                        ..._a.metrics.object_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        block_size: {
            total: {
                label: 'Average',
                metrics: [
                    {
                        ..._a.metrics.block_size,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Read',
                metrics: [
                    {
                        ..._a.metrics.block_size_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Write',
                metrics: [
                    {
                        ..._a.metrics.block_size_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        ops: {
            sum_ops: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            total: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Read',
                metrics: [
                    {
                        ..._a.metrics.read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Write',
                metrics: [
                    {
                        ..._a.metrics.write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            aggregated: {
                label: 'Sum',
                metrics: [
                    {
                        ..._a.metrics.ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs41_total: {
                label: 'NFSv4.1 Total',
                metrics: [
                    {
                        ..._a.metrics.nfs41_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs41_read: {
                label: 'NFSv4.1 Read',
                metrics: [
                    {
                        ..._a.metrics.nfs41_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs41_write: {
                label: 'NFSv4.1 Write',
                metrics: [
                    {
                        ..._a.metrics.nfs41_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs3_total: {
                label: 'NFSv3 Total',
                metrics: [
                    {
                        ..._a.metrics.nfs3_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs3_read: {
                label: 'NFSv3 Read',
                metrics: [
                    {
                        ..._a.metrics.nfs3_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs3_write: {
                label: 'NFSv3 Write',
                metrics: [
                    {
                        ..._a.metrics.nfs3_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb2_read: {
                label: 'SMB2 Read',
                metrics: [
                    {
                        ..._a.metrics.smb2_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb2_write: {
                label: 'SMB2 Write',
                metrics: [
                    {
                        ..._a.metrics.smb2_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        data: {
            sum_data: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            sum_data_b: {
                label: 'Total Back-end',
                metrics: [
                    {
                        ..._a.metrics.read_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            total: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_data_rate: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            total_b: {
                label: 'Total Back-end',
                metrics: [
                    {
                        ..._a.metrics.data_rate_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            aggregated: {
                label: 'Sum',
                metrics: [
                    {
                        ..._a.metrics.data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            aggregated_b: {
                label: 'Aggregated Back-end',
                metrics: [
                    {
                        ..._a.metrics.data_rate_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Read',
                metrics: [
                    {
                        ..._a.metrics.read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Write',
                metrics: [
                    {
                        ..._a.metrics.write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read_b: {
                label: 'Read Back-end',
                metrics: [
                    {
                        ..._a.metrics.read_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write_b: {
                label: 'Write Back-end',
                metrics: [
                    {
                        ..._a.metrics.write_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            dedupe_ratio: {
                label: 'Dedupe Ratio',
                metrics: [
                    {
                        ..._a.metrics.dedupe_ratio,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            stream_rate: {
                label: 'Stream',
                metrics: [
                    {
                        ..._a.metrics.stream_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            logical_data_read: {
                label: 'Logical Read',
                metrics: [
                    {
                        ..._a.metrics.logical_data_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            logical_data_write: {
                label: 'Logical Write',
                metrics: [
                    {
                        ..._a.metrics.logical_data_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            logical_data_rate: {
                label: 'Logical Data Rate',
                metrics: [
                    {
                        ..._a.metrics.logical_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            local_write: {
                label: 'Local Write',
                metrics: [
                    {
                        ..._a.metrics.local_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            mirrored_write: {
                label: 'Mirrored Write',
                metrics: [
                    {
                        ..._a.metrics.mirrored_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            scanned_srcs: {
                label: 'Scanned Sources',
                metrics: [
                    {
                        ..._a.metrics.scanned_srcs,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            client_total: {
                label: 'Client Total',
                metrics: [
                    {
                        ..._a.metrics.client_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            client_in: {
                label: 'Client In',
                metrics: [
                    {
                        ..._a.metrics.client_in,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            client_out: {
                label: 'Client Out',
                metrics: [
                    {
                        ..._a.metrics.client_out,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            slicestor_total: {
                label: 'Slicestor Total',
                metrics: [
                    {
                        ..._a.metrics.slicestor_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            slicestor_in: {
                label: 'Slicestor In',
                metrics: [
                    {
                        ..._a.metrics.slicestor_in,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            slicestor_out: {
                label: 'Slicestor Out',
                metrics: [
                    {
                        ..._a.metrics.slicestor_out,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            initiator: {
                label: 'Data Initiator',
                metrics: [
                    {
                        ..._a.metrics.data_rate_initiator,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            file_data_rate: {
                label: 'File Data',
                metrics: [
                    {
                        ..._a.metrics.file_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            file_read: {
                label: 'File Read',
                metrics: [
                    {
                        ..._a.metrics.file_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            file_write: {
                label: 'File Write',
                metrics: [
                    {
                        ..._a.metrics.file_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs41_data_rate: {
                label: 'NFSv4.1 Data',
                metrics: [
                    {
                        ..._a.metrics.nfs41_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs41_read: {
                label: 'NFSv4.1 Read',
                metrics: [
                    {
                        ..._a.metrics.nfs41_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs41_write: {
                label: 'NFSv4.1 Write',
                metrics: [
                    {
                        ..._a.metrics.nfs41_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs3_data_rate: {
                label: 'NFSv3 Data',
                metrics: [
                    {
                        ..._a.metrics.nfs3_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs3_read: {
                label: 'NFSv3 Read',
                metrics: [
                    {
                        ..._a.metrics.nfs3_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs3_write: {
                label: 'NFSv3 Write',
                metrics: [
                    {
                        ..._a.metrics.nfs3_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb2_data_rate: {
                label: 'SMB2 Data',
                metrics: [
                    {
                        ..._a.metrics.smb2_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb2_read: {
                label: 'SMB2 Read',
                metrics: [
                    {
                        ..._a.metrics.smb2_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb2_write: {
                label: 'SMB2 Write',
                metrics: [
                    {
                        ..._a.metrics.smb2_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_total_data_type: {
                label: 'Data Total Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.file_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.object_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_read_data_type: {
                label: 'Data Read Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.file_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.object_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_write_data_type: {
                label: 'Data Write Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.file_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.object_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            hdfs_data_rate: {
                label: 'HDFS Total',
                metrics: [
                    {
                        ..._a.metrics.hdfs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            hdfs_read: {
                label: 'HDFS Read',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            hdfs_write: {
                label: 'HDFS Write',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            object_data_rate: {
                label: 'Object Total',
                metrics: [
                    {
                        ..._a.metrics.object_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            object_read: {
                label: 'Object Read',
                metrics: [
                    {
                        ..._a.metrics.object_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            object_write: {
                label: 'Object Write',
                metrics: [
                    {
                        ..._a.metrics.object_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        protocol: {
            nfs: {
                label: 'NFS',
                metrics: [
                    {
                        ..._a.metrics.nfs_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_nfs: {
                label: 'NFS Total Ops',
                metrics: [
                    {
                        ..._a.metrics.nfs_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.nfs_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.nfs_other_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_data: {
                label: 'NFS Sum Data',
                metrics: [
                    {
                        ..._a.metrics.nfs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_nfs_data: {
                label: 'NFS Total Data',
                metrics: [
                    {
                        ..._a.metrics.nfs_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.nfs_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_read_ops: {
                label: 'NFS Read Ops',
                metrics: [
                    {
                        ..._a.metrics.nfs_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs_write_ops: {
                label: 'NFS Write Ops',
                metrics: [
                    {
                        ..._a.metrics.nfs_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs_other_ops_rate: {
                label: 'NFS Other Ops',
                metrics: [
                    {
                        ..._a.metrics.nfs_other_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            cifs: {
                label: 'CIFS',
                metrics: [
                    {
                        ..._a.metrics.cifs_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_cifs: {
                label: 'CIFS Total Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.cifs_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.cifs_other_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_data: {
                label: 'CIFS Sum Data',
                metrics: [
                    {
                        ..._a.metrics.cifs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_cifs_data: {
                label: 'CIFS Total Data',
                metrics: [
                    {
                        ..._a.metrics.cifs_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.cifs_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_read: {
                label: 'CIFS Read Data',
                metrics: [
                    {
                        ..._a.metrics.cifs_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            cifs_write: {
                label: 'CIFS Data Write',
                metrics: [
                    {
                        ..._a.metrics.cifs_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            cifs_read_ops: {
                label: 'CIFS Read Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            cifs_write_ops: {
                label: 'CIFS Write Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            cifs_other_ops_rate: {
                label: 'CIFS Other Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_other_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs4: {
                label: 'NFS4',
                metrics: [
                    {
                        ..._a.metrics.nfs4_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb_read_io: {
                label: 'SMB IO Read',
                metrics: [
                    {
                        ..._a.metrics.smb_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb_write_io: {
                label: 'SMB IO Write',
                metrics: [
                    {
                        ..._a.metrics.smb_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb_read: {
                label: 'SMB Data Read',
                metrics: [
                    {
                        ..._a.metrics.smb_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb_write: {
                label: 'SMB Data Write',
                metrics: [
                    {
                        ..._a.metrics.smb_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb_resp_t_r: {
                label: 'SMB Latency Read',
                metrics: [
                    {
                        ..._a.metrics.smb_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb_resp_t_w: {
                label: 'SMB Latency Read',
                metrics: [
                    {
                        ..._a.metrics.smb_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb1: {
                label: 'SMB1',
                metrics: [
                    {
                        ..._a.metrics.smb1_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            smb2: {
                label: 'SMB2',
                metrics: [
                    {
                        ..._a.metrics.smb2_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs_read_io: {
                label: 'NFS IO Read',
                metrics: [
                    {
                        ..._a.metrics.nfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs_write_io: {
                label: 'NFS IO Write',
                metrics: [
                    {
                        ..._a.metrics.nfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs_read: {
                label: 'NFS Read Data',
                metrics: [
                    {
                        ..._a.metrics.nfs_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs_write: {
                label: 'NFS Write Data',
                metrics: [
                    {
                        ..._a.metrics.nfs_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            nfs_resp_t: {
                label: 'NFS Total Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_resp_t_r: {
                label: 'NFS Read Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_resp_t_w: {
                label: 'NFS Write Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_resp_t: {
                label: 'CIFS Total Latency',
                metrics: [
                    {
                        ..._a.metrics.cifs_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_resp_t_r: {
                label: 'CIFS Read Latency',
                metrics: [
                    {
                        ..._a.metrics.cifs_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_resp_t_w: {
                label: 'CIFS Write Latency',
                metrics: [
                    {
                        ..._a.metrics.cifs_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            fs_data_rate: {
                label: 'FS Sum Data',
                metrics: [
                    {
                        ..._a.metrics.fs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_fs_data: {
                label: 'FS Total Data',
                metrics: [
                    {
                        ..._a.metrics.fs_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.fs_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            fs_read: {
                label: 'FS Read Data',
                metrics: [
                    {
                        ..._a.metrics.fs_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            fs_write: {
                label: 'FS Write Data',
                metrics: [
                    {
                        ..._a.metrics.fs_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        resp_t: {
            total: {
                label: 'Average',
                metrics: [
                    {
                        ..._a.metrics.resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Read',
                metrics: [
                    {
                        ..._a.metrics.resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Write',
                metrics: [
                    {
                        ..._a.metrics.resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            total_b: {
                label: 'Average Back-end',
                metrics: [
                    {
                        ..._a.metrics.resp_t_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read_b: {
                label: 'Read Back-end',
                metrics: [
                    {
                        ..._a.metrics.resp_t_r_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_b: {
                label: 'Write Back-end',
                metrics: [
                    {
                        ..._a.metrics.resp_t_w_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            port_resp_t_w: {
                label: 'Port Write Latency',
                metrics: [
                    {
                        ..._a.metrics.port_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            port_resp_t_r: {
                label: 'Port Read Latency',
                metrics: [
                    {
                        ..._a.metrics.port_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            port_resp_t: {
                label: 'Port Latency',
                metrics: [
                    {
                        ..._a.metrics.port_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            disk_resp_t_r: {
                label: 'Disk Read Latency',
                metrics: [
                    {
                        ..._a.metrics.disk_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            disk_resp_t_w: {
                label: 'Disk Write Latency',
                metrics: [
                    {
                        ..._a.metrics.disk_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            disk_resp_t: {
                label: 'Disk Latency',
                metrics: [
                    {
                        ..._a.metrics.disk_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            local_write: {
                label: 'Local Write',
                metrics: [
                    {
                        ..._a.metrics.local_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            mirrored_write: {
                label: 'Mirrored Write',
                metrics: [
                    {
                        ..._a.metrics.mirrored_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            acknowledge_time: {
                label: 'Acknowledge Time',
                metrics: [
                    {
                        ..._a.metrics.acknowledge_time,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            initiator: {
                label: 'Average Initiator',
                metrics: [
                    {
                        ..._a.metrics.resp_t_initiator,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            ops_total: {
                label: 'OPS Average',
                metrics: [
                    {
                        ..._a.metrics.ops_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            ops_read: {
                label: 'OPS Read',
                metrics: [
                    {
                        ..._a.metrics.ops_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            ops_write: {
                label: 'OPS Write',
                metrics: [
                    {
                        ..._a.metrics.ops_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_total: {
                label: 'NFSv4.1 Average',
                metrics: [
                    {
                        ..._a.metrics.nfs41_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_read: {
                label: 'NFSv4.1 Read',
                metrics: [
                    {
                        ..._a.metrics.nfs41_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_write: {
                label: 'NFSv4.1 Write',
                metrics: [
                    {
                        ..._a.metrics.nfs41_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_total: {
                label: 'NFSv3 Average',
                metrics: [
                    {
                        ..._a.metrics.nfs3_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_read: {
                label: 'NFSv3 Read',
                metrics: [
                    {
                        ..._a.metrics.nfs3_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_write: {
                label: 'NFSv3 Write',
                metrics: [
                    {
                        ..._a.metrics.nfs3_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_total: {
                label: 'SMB2 Average',
                metrics: [
                    {
                        ..._a.metrics.smb2_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_read: {
                label: 'SMB2 Read',
                metrics: [
                    {
                        ..._a.metrics.smb2_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_write: {
                label: 'SMB2 Write',
                metrics: [
                    {
                        ..._a.metrics.smb2_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        capacity: {
            free: {
                label: 'Free',
                metrics: [
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            used: {
                label: 'Used',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            used_percent: {
                label: 'Used %',
                metrics: [
                    {
                        ..._a.metrics.capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            user_used_percent: {
                label: 'User Used %',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            spare_used_percent: {
                label: 'Used Spare %',
                metrics: [
                    {
                        ..._a.metrics.spare_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_capacity_used: {
                label: 'Used Total',
                metrics: [
                    {
                        ..._a.metrics.sum_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_capacity_used_percent: {
                label: 'Used Total %',
                metrics: [
                    {
                        ..._a.metrics.sum_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            user_capacity_used: {
                label: 'Used User',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            snapshot_capacity_used: {
                label: 'Used Snapshot',
                metrics: [
                    {
                        ..._a.metrics.snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            raw_capacity_used: {
                label: 'Used Raw',
                metrics: [
                    {
                        ..._a.metrics.raw_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            oversubscr: {
                label: 'Oversubscription',
                metrics: [
                    {
                        ..._a.metrics.capacity_oversubscr,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            used_virtual: {
                label: 'Used Virtual',
                metrics: [
                    {
                        ..._a.metrics.virtual_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            virtual: {
                label: 'Virtual',
                metrics: [
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.brownDark
                    },
                ],
            },
            allocated: {
                label: 'Allocated',
                metrics: [
                    {
                        ..._a.metrics.capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            user_capacity_allocated: {
                label: 'Allocated User',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            snapshot_capacity_allocated: {
                label: 'Allocated Snapshot',
                metrics: [
                    {
                        ..._a.metrics.snapshot_capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_capacity_used_free: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    }
                ],
            },
            sum_capacity: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_capacity_virtual: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 3,
                        color: Color_1.Color.pallete.brownDark,
                    },
                ],
            },
            formatted_capacity_total: {
                label: 'Formatted',
                metrics: [
                    {
                        ..._a.metrics.format_replication_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.format_snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.orange,
                    },
                    {
                        ..._a.metrics.format_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.redDark,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 5,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            formatted_capacity_virtual: {
                label: 'Formatted',
                metrics: [
                    {
                        ..._a.metrics.format_replication_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.format_snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.orange,
                    },
                    {
                        ..._a.metrics.format_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.brownDark,
                        label: 'Virtual',
                    },
                    {
                        ..._a.metrics.format_raid_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 5,
                        color: Color_1.Color.pallete.redDark,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 6,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            raw_capacity_total: {
                label: 'Raw',
                metrics: [
                    {
                        ..._a.metrics.raw_replication_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.raw_snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.orange,
                    },
                    {
                        ..._a.metrics.raw_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.raw_raid_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.raw_raid_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.redDark,
                    },
                    {
                        ..._a.metrics.raw_raid_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 5,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            raw_capacity_total_vspg: {
                label: 'Raw Total',
                metrics: [
                    {
                        ..._a.metrics.raw_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.spare_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.raw_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.green,
                    },
                ],
            },
            OZFScapacity: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used_data,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.replication_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 2,
                    },
                    {
                        ..._a.metrics.snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 3,
                    },
                ],
            },
            disk_size: {
                label: 'Size On Disk',
                metrics: [
                    {
                        ..._a.metrics.disk_size,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            dedupe_ratio_percent: {
                label: 'Ratio',
                metrics: [
                    {
                        ..._a.metrics.dedupe_ratio_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            total: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            total_compression_factor: {
                label: 'Compression',
                metrics: [
                    {
                        ..._a.metrics.total_comp_factor,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            compression_saving: {
                label: 'Compression',
                metrics: [
                    {
                        ..._a.metrics.compression_saving,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dedupe_saving: {
                label: 'Deduplication',
                metrics: [
                    {
                        ..._a.metrics.dedupe_saving,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            data_in: {
                label: 'Data In',
                metrics: [
                    {
                        ..._a.metrics.data_in,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            capacity_data_in: {
                label: 'Data In',
                metrics: [
                    {
                        ..._a.metrics.capacity_data_in,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            capacity_data_written: {
                label: 'Data Written',
                metrics: [
                    {
                        ..._a.metrics.capacity_data_written,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            capacity_data_reduction: {
                label: 'Data Reduction',
                metrics: [
                    {
                        ..._a.metrics.capacity_data_reduction,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            capacity_logical_usage: {
                label: 'Logical Usage',
                metrics: [
                    {
                        ..._a.metrics.capacity_logical_usage,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            capacity_storage_consumed: {
                label: 'Storage Consumed',
                metrics: [
                    {
                        ..._a.metrics.capacity_storage_consumed,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            tier0_capacity_total: {
                label: 'Tier 0',
                metrics: [
                    {
                        ..._a.metrics.tier0_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            tier1_capacity_total: {
                label: 'Tier 1',
                metrics: [
                    {
                        ..._a.metrics.tier1_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            tier2_capacity_total: {
                label: 'Tier 2',
                metrics: [
                    {
                        ..._a.metrics.tier2_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_compression_capacity: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.red,
                        label: 'Total',
                    },
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                        color: Color_1.Color.pallete.blue,
                        label: 'Used',
                    },
                    {
                        ..._a.metrics.reduction_involved_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.lightblue,
                        label: 'Pre-saving',
                    },
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 3,
                        color: Color_1.Color.pallete.greenDark,
                        label: 'Post-saving',
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.brownDark,
                        label: 'Virtual',
                    }
                ],
            },
            data_reduction_ratio: {
                label: 'Data Reduction',
                metrics: [
                    {
                        ..._a.metrics.data_reduction_ratio,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            replication_capacity_total: {
                label: 'Replication',
                metrics: [
                    {
                        ..._a.metrics.replication_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            system_capacity_free: {
                label: 'System Free',
                metrics: [
                    {
                        ..._a.metrics.system_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            system_capacity_used: {
                label: 'System Used',
                metrics: [
                    {
                        ..._a.metrics.system_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            system_used_percent: {
                label: 'System Used %',
                metrics: [
                    {
                        ..._a.metrics.system_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            system_capacity_total: {
                label: 'System total',
                metrics: [
                    {
                        ..._a.metrics.system_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_system_capacity: {
                label: 'System capacity',
                metrics: [
                    {
                        ..._a.metrics.system_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.system_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.system_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_user_capacity: {
                label: 'User capacity',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.user_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.user_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            user_capacity_total: {
                label: 'User total',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            user_capacity_free: {
                label: 'User Free',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            capacity_ingested: {
                label: 'Ingested',
                metrics: [
                    {
                        ..._a.metrics.capacity_ingested,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    }
                ]
            },
        },
        busy: {
            sum_busy: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.cpu_busy,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.avg,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            total: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.cpu_busy,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        error: {
            total: {
                label: 'Error',
                metrics: [
                    {
                        ..._a.metrics.error_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        cpu: {
            sys: {
                label: 'System',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            user: {
                label: 'User',
                metrics: [
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            idle: {
                label: 'Idle',
                metrics: [
                    {
                        ..._a.metrics.cpu_idle,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cpu_wait: {
                label: 'Wait',
                metrics: [
                    {
                        ..._a.metrics.cpu_wait,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            gpu_util_percent: {
                label: 'GPU Utilization',
                metrics: [
                    {
                        ..._a.metrics.cpu_gpu_util_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            utilization: {
                label: 'Utilization',
                metrics: [
                    {
                        ..._a.metrics.utilization,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_sys: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.avg,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.avg,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.cpu_idle,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.avg,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_cpu_system_user: {
                label: 'CPU Total',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.avg,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.avg,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.blueDark,
                    }
                ],
            },
            sum_cpu: {
                label: 'CPU',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.gray,
                    },
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.yellow,
                    },
                    {
                        ..._a.metrics.cpu_wait,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.cpu_other,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                ],
            },
            compress: {
                label: 'Compress',
                metrics: [
                    {
                        ..._a.metrics.cpu_compress,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        other: {
            crc: {
                label: 'CRC',
                metrics: [
                    {
                        ..._a.metrics.crc,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            b2b: {
                label: 'B2B',
                metrics: [
                    {
                        ..._a.metrics.b2b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            loss_sig: {
                label: 'Loss Signal',
                metrics: [
                    {
                        ..._a.metrics.loss_sig,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            loss_sync: {
                label: 'Loss Sync',
                metrics: [
                    {
                        ..._a.metrics.loss_sync,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            link_fail: {
                label: 'Link Failure',
                metrics: [
                    {
                        ..._a.metrics.link_fail,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            buffer_error: {
                label: 'Buffer Error',
                metrics: [
                    {
                        ..._a.metrics.buffer_error,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        cache: {
            read_hit: {
                label: 'Read Hit',
                metrics: [
                    {
                        ..._a.metrics.r_cache_hit,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_hit: {
                label: 'Write Hit',
                metrics: [
                    {
                        ..._a.metrics.w_cache_hit,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cache_usage_total: {
                label: 'Cache Usage',
                metrics: [
                    {
                        ..._a.metrics.cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read_usage: {
                label: 'Read Usage',
                metrics: [
                    {
                        ..._a.metrics.r_cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write_usage: {
                label: 'Write Usage',
                metrics: [
                    {
                        ..._a.metrics.w_cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            cfc_dirty: {
                label: 'CFC Dirty',
                metrics: [
                    {
                        ..._a.metrics.cfc_dirty,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            del_ack: {
                label: 'Del Ack',
                metrics: [
                    {
                        ..._a.metrics.del_ack,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read_hit_io: {
                label: 'Read Hit IO',
                metrics: [
                    {
                        ..._a.metrics.r_cache_hit_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write_hit_io: {
                label: 'Write Hit IO',
                metrics: [
                    {
                        ..._a.metrics.w_cache_hit_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write_pending: {
                label: 'Write Pending',
                metrics: [
                    {
                        ..._a.metrics.write_pending_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cache_usage: {
                label: 'Cache Usage',
                metrics: [
                    {
                        ..._a.metrics.cache_usage_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            r_cache_usage_percent: {
                label: 'Cache Read Usage',
                metrics: [
                    {
                        ..._a.metrics.r_cache_usage_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            w_cache_usage_percent: {
                label: 'Cache Write Usage',
                metrics: [
                    {
                        ..._a.metrics.w_cache_usage_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            r_cache_miss_sec: {
                label: ' Block Cache Read Misses',
                metrics: [
                    {
                        ..._a.metrics.r_cache_miss_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            w_cache_miss_sec: {
                label: ' Block Write Read Misses',
                metrics: [
                    {
                        ..._a.metrics.w_cache_miss_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            r_cache_hit_sec: {
                label: 'Block Cache Read Hits',
                metrics: [
                    {
                        ..._a.metrics.r_cache_hit_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            w_cache_hit_sec: {
                label: 'Block Cache Write Hits',
                metrics: [
                    {
                        ..._a.metrics.w_cache_hit_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        rebalance: {
            rebalance_rate: {
                label: 'Rebalance Rate',
                metrics: [
                    {
                        ..._a.metrics.rebalance_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebalance_pending: {
                label: 'Rebalance Pending Capacity',
                metrics: [
                    {
                        ..._a.metrics.rebalance_pending,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        rebuild: {
            rebuild_rate: {
                label: 'Rebuild Rate',
                metrics: [
                    {
                        ..._a.metrics.rebuild_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_pending: {
                label: 'Rebuild Pending Capacity',
                metrics: [
                    {
                        ..._a.metrics.rebuild_pending,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_queue: {
                label: 'Rebuild Queue',
                metrics: [
                    {
                        ..._a.metrics.rebuild_queue,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_writes_total: {
                label: 'Rebuild Writes Total',
                metrics: [
                    {
                        ..._a.metrics.rebuild_writes_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_writes_in: {
                label: 'Rebuild Writes In',
                metrics: [
                    {
                        ..._a.metrics.rebuild_writes_in,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_writes_out: {
                label: 'Rebuild Writes Out',
                metrics: [
                    {
                        ..._a.metrics.rebuild_writes_out,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_deletes_total: {
                label: 'Rebuild Deletes Total',
                metrics: [
                    {
                        ..._a.metrics.rebuild_deletes_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_deletes_in: {
                label: 'Rebuild Deletes In',
                metrics: [
                    {
                        ..._a.metrics.rebuild_deletes_in,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_deletes_out: {
                label: 'Rebuild Deletes Out',
                metrics: [
                    {
                        ..._a.metrics.rebuild_deletes_out,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_bytes_total: {
                label: 'Rebuild Bytes Total',
                metrics: [
                    {
                        ..._a.metrics.rebuild_bytes_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_bytes_in: {
                label: 'Rebuild Bytes In',
                metrics: [
                    {
                        ..._a.metrics.rebuild_bytes_in,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            rebuild_bytes_out: {
                label: 'Rebuild Bytes Out',
                metrics: [
                    {
                        ..._a.metrics.rebuild_bytes_out,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        replication: {
            sum_repl: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.read_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            aggregated: {
                label: 'Aggregated',
                metrics: [
                    {
                        ..._a.metrics.read_replication,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_replication,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read_repl: {
                label: 'Read Replication',
                metrics: [
                    {
                        ..._a.metrics.read_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_repl: {
                label: 'Write Replication',
                metrics: [
                    {
                        ..._a.metrics.write_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            repl_in: {
                label: 'Replication In',
                metrics: [
                    {
                        ..._a.metrics.replication_in,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            repl_out: {
                label: 'Replication Out',
                metrics: [
                    {
                        ..._a.metrics.replication_out,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            total: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.replication_in,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.replication_out,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            stream_repl_rate: {
                label: 'Stream',
                metrics: [
                    {
                        ..._a.metrics.stream_read_replication,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.stream_write_replication,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        housekeep: {
            housekeep_pend_data: {
                label: 'Pending Data',
                metrics: [
                    {
                        ..._a.metrics.housekeep_pend_data,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        job: {
            copy_job_read: {
                label: 'Read Copy Job',
                metrics: [
                    {
                        ..._a.metrics.copy_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            copy_job_write: {
                label: 'Write Copy Job',
                metrics: [
                    {
                        ..._a.metrics.copy_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            data_job_read: {
                label: 'Read Data Job',
                metrics: [
                    {
                        ..._a.metrics.data_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            data_job_write: {
                label: 'Write Data Job',
                metrics: [
                    {
                        ..._a.metrics.data_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            stream_job_rate: {
                label: 'Stream Job Rate',
                metrics: [
                    {
                        ..._a.metrics.stream_job_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            logical_copy_job_read: {
                label: 'Logical Copy Read',
                metrics: [
                    {
                        ..._a.metrics.logical_copy_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            logical_copy_job_write: {
                label: 'Logical Copy Write',
                metrics: [
                    {
                        ..._a.metrics.logical_copy_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            logical_data_job_read: {
                label: 'Logical Data Read',
                metrics: [
                    {
                        ..._a.metrics.logical_data_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            logical_data_job_write: {
                label: 'Logical Data Write',
                metrics: [
                    {
                        ..._a.metrics.logical_data_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        network: {
            sum_network: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.network_tx,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.network_rx,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            network_rate: {
                label: 'Sum',
                metrics: [
                    {
                        ..._a.metrics.network_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            network_rx: {
                label: 'In',
                metrics: [
                    {
                        ..._a.metrics.network_rx,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            network_tx: {
                label: 'Out',
                metrics: [
                    {
                        ..._a.metrics.network_tx,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            sum_network_b: {
                label: 'Total BE',
                metrics: [
                    {
                        ..._a.metrics.network_tx_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.network_rx_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            network_rate_b: {
                label: 'Sum BE',
                metrics: [
                    {
                        ..._a.metrics.network_rate_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            network_rx_b: {
                label: 'In BE',
                metrics: [
                    {
                        ..._a.metrics.network_rx_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            network_tx_b: {
                label: 'Out BE',
                metrics: [
                    {
                        ..._a.metrics.network_tx_b,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            packet_rate: {
                label: 'Packet Rate',
                metrics: [
                    {
                        ..._a.metrics.packet_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read_packet: {
                label: 'Packet Read',
                metrics: [
                    {
                        ..._a.metrics.read_packet,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write_packet: {
                label: 'Packet Write',
                metrics: [
                    {
                        ..._a.metrics.write_packet,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read_error: {
                label: 'Error Read',
                metrics: [
                    {
                        ..._a.metrics.read_error,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write_error: {
                label: 'Error Write',
                metrics: [
                    {
                        ..._a.metrics.write_error,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            tcp_timeouts: {
                label: 'TCP Timeouts',
                metrics: [
                    {
                        ..._a.metrics.tcp_timeouts,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            }
        },
        net: {
            sum_net: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.read_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            total: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.net_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Read',
                metrics: [
                    {
                        ..._a.metrics.read_net,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Write',
                metrics: [
                    {
                        ..._a.metrics.write_net,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
        },
        memory: {
            overview: {
                label: "Memory Aggregated",
                metrics: [
                    {
                        ..._a.metrics.memory_reserved,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.memory_buffer,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                    {
                        ..._a.metrics.memory_cached,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 3,
                    },
                    {
                        ..._a.metrics.memory_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 4,
                    }
                ]
            },
            gpu_aggregated: {
                label: "Memory GPU Aggregated",
                metrics: [
                    {
                        ..._a.metrics.memory_gpu_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.memory_gpu_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.memory_gpu_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 2,
                    }
                ]
            },
            gpu_util_percent: {
                label: 'Memory GPU Percent',
                metrics: [
                    {
                        ..._a.metrics.memory_gpu_util_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            util_percent: {
                label: "Memory Percent",
                metrics: [
                    {
                        ..._a.metrics.memory_util_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    }
                ]
            },
            sum_memory: {
                label: 'Memory',
                metrics: [
                    {
                        ..._a.metrics.memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.memory_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                ],
            },
            aggregated: {
                label: 'Aggregated',
                metrics: [
                    {
                        ..._a.metrics.memory_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            free: {
                label: 'Free',
                metrics: [
                    {
                        ..._a.metrics.memory_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            used: {
                label: 'Used',
                metrics: [
                    {
                        ..._a.metrics.memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            memory: {
                label: 'Used percent',
                metrics: [
                    {
                        ..._a.metrics.memory_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            system_memory: {
                label: 'System Memory',
                metrics: [
                    {
                        ..._a.metrics.system_memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            onheap_memory: {
                label: 'On-heap Memory',
                metrics: [
                    {
                        ..._a.metrics.onheap_memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            offheap_memory: {
                label: 'Off-heap Memory',
                metrics: [
                    {
                        ..._a.metrics.offheap_memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        bound: {
            sum_bound_pre_comp: {
                label: 'Pre-comp',
                metrics: [
                    {
                        ..._a.metrics.outbound_pre_comp,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.inbound_pre_comp,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            sum_bound_net: {
                label: 'Network',
                metrics: [
                    {
                        ..._a.metrics.outbound_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.inbound_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
        },
        os: {
            os_total: {
                label: 'OS Total',
                metrics: [
                    {
                        ..._a.metrics.os_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            os_r: {
                label: 'OS Read',
                metrics: [
                    {
                        ..._a.metrics.os_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            os_w: {
                label: 'OS Write',
                metrics: [
                    {
                        ..._a.metrics.os_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        temperature: {
            temperature: {
                label: 'Temperature',
                metrics: [
                    {
                        ..._a.metrics.temperature,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        queue: {
            queue_length: {
                label: 'Queue Length',
                metrics: [
                    {
                        ..._a.metrics.queue_length,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        object: {
            total: {
                label: 'Count',
                metrics: [
                    {
                        ..._a.metrics.object_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    }
                ]
            },
            created: {
                label: 'Created',
                metrics: [
                    {
                        ..._a.metrics.object_created,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    }
                ]
            },
            deleted: {
                label: 'Deleted',
                metrics: [
                    {
                        ..._a.metrics.object_deleted,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    }
                ]
            },
        },
        quota: {
            soft_quota: {
                label: 'Soft Quota',
                metrics: [
                    {
                        ..._a.metrics.soft_quota,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            hard_quota: {
                label: 'Hard Quota',
                metrics: [
                    {
                        ..._a.metrics.hard_quota,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        delta: {
            delta_size: {
                label: 'Delta Size',
                metrics: [
                    {
                        ..._a.metrics.delta_size,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    }
                ]
            },
        },
        size: {
            size: {
                label: 'Size',
                metrics: [
                    {
                        ..._a.metrics.size,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    }
                ]
            },
        },
    },
    single: {
        io: {
            io_rate: {
                label: 'IO',
                metrics: [
                    {
                        ..._a.metrics.io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0
                    }
                ],
            },
            io_rate_b: {
                label: 'IO Back-end',
                metrics: [
                    {
                        ..._a.metrics.io_rate_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0
                    },
                ],
            },
            read_io: {
                label: 'IO Read',
                metrics: [
                    {
                        ..._a.metrics.read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_io: {
                label: 'IO Write',
                metrics: [
                    {
                        ..._a.metrics.write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            io_pending: {
                label: 'IO Pending',
                metrics: [
                    {
                        ..._a.metrics.io_pending,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            local_write: {
                label: 'Local IO Write',
                metrics: [
                    {
                        ..._a.metrics.local_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            mirrored_write_io: {
                label: 'Mirrored IO Write',
                metrics: [
                    {
                        ..._a.metrics.mirrored_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            un_read_io: {
                label: 'Unaligned IO Read',
                metrics: [
                    {
                        ..._a.metrics.un_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            un_write_io: {
                label: 'Unaligned IO Write',
                metrics: [
                    {
                        ..._a.metrics.un_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_total_io_type: {
                label: 'IO Total Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        label: "File Total"
                    },
                ],
            },
            sum_read_io_type: {
                label: 'IO Read Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        label: "File Read"
                    },
                ],
            },
            sum_write_io_type: {
                label: 'IO Write Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        label: "File Write"
                    },
                ],
            },
            sum_total_io_object_type: {
                label: 'IO Total Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        label: "File Total"
                    },
                    {
                        ..._a.metrics.object_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_read_io_object_type: {
                label: 'IO Read Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        label: "File Read"
                    },
                    {
                        ..._a.metrics.object_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_write_io_object_type: {
                label: 'IO Write Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        label: "File Write"
                    },
                    {
                        ..._a.metrics.object_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            hdfs_io_rate: {
                label: 'HDFS Total',
                metrics: [
                    {
                        ..._a.metrics.hdfs_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            hdfs_read_io: {
                label: 'HDFS Read',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            hdfs_write_io: {
                label: 'HDFS Write',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            object_io_rate: {
                label: 'Object Total',
                metrics: [
                    {
                        ..._a.metrics.object_io_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            object_read_io: {
                label: 'Object Read',
                metrics: [
                    {
                        ..._a.metrics.object_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            object_write_io: {
                label: 'Object Write',
                metrics: [
                    {
                        ..._a.metrics.object_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        ops: {
            ops_rate: {
                label: 'OPS',
                metrics: [
                    {
                        ..._a.metrics.ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read_ops: {
                label: 'OPS Read',
                metrics: [
                    {
                        ..._a.metrics.read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_ops: {
                label: 'OPS Write',
                metrics: [
                    {
                        ..._a.metrics.write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_total: {
                label: 'NFSv4.1 OPS Total',
                metrics: [
                    {
                        ..._a.metrics.nfs41_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_read: {
                label: 'NFSv4.1 OPS Read',
                metrics: [
                    {
                        ..._a.metrics.nfs41_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_write: {
                label: 'NFSv4.1 OPS Write',
                metrics: [
                    {
                        ..._a.metrics.nfs41_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_total: {
                label: 'NFSv3 OPS Total',
                metrics: [
                    {
                        ..._a.metrics.nfs3_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_read: {
                label: 'NFSv3 OPS Read',
                metrics: [
                    {
                        ..._a.metrics.nfs3_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_write: {
                label: 'NFSv3 OPS Write',
                metrics: [
                    {
                        ..._a.metrics.nfs3_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_read: {
                label: 'SMB2 OPS Read',
                metrics: [
                    {
                        ..._a.metrics.smb2_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_write: {
                label: 'SMB2 OPS Write',
                metrics: [
                    {
                        ..._a.metrics.nfs41_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        data: {
            data_rate: {
                label: 'Data',
                metrics: [
                    {
                        ..._a.metrics.data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            data_rate_b: {
                label: 'Data Back-end',
                metrics: [
                    {
                        ..._a.metrics.data_rate_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Read',
                metrics: [
                    {
                        ..._a.metrics.read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Write',
                metrics: [
                    {
                        ..._a.metrics.write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read_b: {
                label: 'Read Back-end',
                metrics: [
                    {
                        ..._a.metrics.read_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_b: {
                label: 'Write Back-end',
                metrics: [
                    {
                        ..._a.metrics.write_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            housekeep_pend_data: {
                label: 'Pending Data',
                metrics: [
                    {
                        ..._a.metrics.housekeep_pend_data,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dedupe_ratio: {
                label: 'Dedupe Ratio',
                metrics: [
                    {
                        ..._a.metrics.dedupe_ratio,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            stream_rate: {
                label: 'Stream',
                metrics: [
                    {
                        ..._a.metrics.stream_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            logical_data_read: {
                label: 'Logical Read',
                metrics: [
                    {
                        ..._a.metrics.logical_data_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            logical_data_write: {
                label: 'Logical Write',
                metrics: [
                    {
                        ..._a.metrics.logical_data_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            local_write: {
                label: 'Local Write',
                metrics: [
                    {
                        ..._a.metrics.local_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            mirrored_write: {
                label: 'Mirrored Write',
                metrics: [
                    {
                        ..._a.metrics.mirrored_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            scanned_srcs: {
                label: 'Scanned Sources',
                metrics: [
                    {
                        ..._a.metrics.scanned_srcs,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            file_data_rate: {
                label: 'File Data',
                metrics: [
                    {
                        ..._a.metrics.file_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            file_read: {
                label: 'File Read',
                metrics: [
                    {
                        ..._a.metrics.file_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            file_write: {
                label: 'File Write',
                metrics: [
                    {
                        ..._a.metrics.file_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_total: {
                label: 'NFSv4.1 Data Total',
                metrics: [
                    {
                        ..._a.metrics.nfs41_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_read: {
                label: 'NFSv4.1 Data Read',
                metrics: [
                    {
                        ..._a.metrics.nfs41_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_write: {
                label: 'NFSv4.1 Data Write',
                metrics: [
                    {
                        ..._a.metrics.nfs41_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_total: {
                label: 'NFSv3 Data Total',
                metrics: [
                    {
                        ..._a.metrics.nfs3_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_read: {
                label: 'NFSv3 Data Read',
                metrics: [
                    {
                        ..._a.metrics.nfs3_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_write: {
                label: 'NFSv3 Data Write',
                metrics: [
                    {
                        ..._a.metrics.nfs3_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_total: {
                label: 'SMB2 Data Total',
                metrics: [
                    {
                        ..._a.metrics.smb2_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_read: {
                label: 'SMB2 Data Read',
                metrics: [
                    {
                        ..._a.metrics.smb2_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_write: {
                label: 'SMB2 Data Write',
                metrics: [
                    {
                        ..._a.metrics.smb2_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_total_data_type: {
                label: 'Data Total Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.file_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.object_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_read_data_type: {
                label: 'Data Read Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.file_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.object_read,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_write_data_type: {
                label: 'Data Write Type',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.file_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.object_write,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            hdfs_data_rate: {
                label: 'HDFS Total',
                metrics: [
                    {
                        ..._a.metrics.hdfs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            hdfs_read: {
                label: 'HDFS Read',
                metrics: [
                    {
                        ..._a.metrics.hdfs_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            hdfs_write: {
                label: 'HDFS Write',
                metrics: [
                    {
                        ..._a.metrics.hdfs_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            object_data_rate: {
                label: 'Object Total',
                metrics: [
                    {
                        ..._a.metrics.object_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            object_read: {
                label: 'Object Read',
                metrics: [
                    {
                        ..._a.metrics.object_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            object_write: {
                label: 'Object Write',
                metrics: [
                    {
                        ..._a.metrics.object_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        resp_t: {
            resp_t: {
                label: 'Latency',
                metrics: [
                    {
                        ..._a.metrics.resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            resp_t_r: {
                label: 'Latency Read',
                metrics: [
                    {
                        ..._a.metrics.resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            resp_t_w: {
                label: 'Latency Write',
                metrics: [
                    {
                        ..._a.metrics.resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            resp_t_b: {
                label: 'Latency Back-end',
                metrics: [
                    {
                        ..._a.metrics.resp_t_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            resp_t_r_b: {
                label: 'Latency Read Back-end',
                metrics: [
                    {
                        ..._a.metrics.resp_t_r_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            resp_t_w_b: {
                label: 'Latency Write Back-end',
                metrics: [
                    {
                        ..._a.metrics.resp_t_w_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            port_resp_t: {
                label: 'Port Latency',
                metrics: [
                    {
                        ..._a.metrics.port_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            port_resp_t_r: {
                label: 'Port Read Latency',
                metrics: [
                    {
                        ..._a.metrics.port_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            port_resp_t_w: {
                label: 'Port Write Latency',
                metrics: [
                    {
                        ..._a.metrics.port_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            disk_resp_t: {
                label: 'Disk Latency',
                metrics: [
                    {
                        ..._a.metrics.disk_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            disk_resp_t_r: {
                label: 'Disk Read Latency',
                metrics: [
                    {
                        ..._a.metrics.disk_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            disk_resp_t_w: {
                label: 'Disk Write Latency',
                metrics: [
                    {
                        ..._a.metrics.disk_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            local_write: {
                label: 'Local Latency Write',
                metrics: [
                    {
                        ..._a.metrics.local_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            mirrored_resp_t_w: {
                label: 'Mirrored Latency Write',
                metrics: [
                    {
                        ..._a.metrics.mirrored_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            acknowledge_time: {
                label: 'Acknowledge Time',
                metrics: [
                    {
                        ..._a.metrics.acknowledge_time,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            ops_total: {
                label: 'OPS Latency',
                metrics: [
                    {
                        ..._a.metrics.ops_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            ops_read: {
                label: 'OPS Latency Read',
                metrics: [
                    {
                        ..._a.metrics.ops_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            ops_write: {
                label: 'OPS Latency Write',
                metrics: [
                    {
                        ..._a.metrics.ops_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_total: {
                label: 'NFSv4.1 Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs41_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_read: {
                label: 'NFSv4.1 Latency Read',
                metrics: [
                    {
                        ..._a.metrics.nfs41_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs41_write: {
                label: 'NFSv4.1 Latency Write',
                metrics: [
                    {
                        ..._a.metrics.nfs41_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_total: {
                label: 'NFSv3 Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs3_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_read: {
                label: 'NFSv3 Latency Read',
                metrics: [
                    {
                        ..._a.metrics.nfs3_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs3_write: {
                label: 'NFSv3 Latency Write',
                metrics: [
                    {
                        ..._a.metrics.nfs3_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_total: {
                label: 'SMB2 Latency',
                metrics: [
                    {
                        ..._a.metrics.smb2_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_read: {
                label: 'SMB2 Latency Read',
                metrics: [
                    {
                        ..._a.metrics.smb2_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2_write: {
                label: 'SMB2 Latency Write',
                metrics: [
                    {
                        ..._a.metrics.smb2_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        capacity: {
            sum_capacity: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_system_capacity: {
                label: 'System Capacity',
                metrics: [
                    {
                        ..._a.metrics.system_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.system_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.system_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_user_capacity: {
                label: 'User Capacity',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.user_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.user_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_capacity_virtual: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 3,
                        color: Color_1.Color.pallete.brownDark,
                    },
                ],
            },
            sum_capacity_spare: {
                label: 'Spare Capacity',
                metrics: [
                    {
                        ..._a.metrics.spare_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.spare_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.spare_capacity_total,
                        label: "Spare Capacity Total",
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_capacity_user: {
                label: 'User Capacity',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.user_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.user_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_capacity_user_allocated: {
                label: 'User Capacity',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.user_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.user_capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            sum_capacity_snapshot: {
                label: 'Snapshot Capacity',
                metrics: [
                    {
                        ..._a.metrics.snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.red
                    },
                    {
                        ..._a.metrics.snapshot_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.snapshot_capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            capacity_data_in: {
                label: 'Data In',
                metrics: [
                    {
                        ..._a.metrics.capacity_data_in,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.metric.read,
                    },
                ],
            },
            capacity_logical_usage: {
                label: 'Logical Usage',
                metrics: [
                    {
                        ..._a.metrics.capacity_logical_usage,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.metric.read
                    },
                ],
            },
            capacity_data_written: {
                label: 'Data Written',
                metrics: [
                    {
                        ..._a.metrics.capacity_data_written,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.metric.write,
                    },
                ],
            },
            capacity_data_reduction: {
                label: 'Data Reduction',
                metrics: [
                    {
                        ..._a.metrics.capacity_data_reduction,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.metric.read
                    },
                ],
            },
            data_reduction: {
                label: 'Data Reduction',
                metrics: [
                    {
                        ..._a.metrics.data_reduction,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.metric.read
                    },
                ],
            },
            capacity_storage_consumed: {
                label: 'Storage Consumed',
                metrics: [
                    {
                        ..._a.metrics.capacity_storage_consumed,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.metric.read
                    },
                ],
            },
            formatted_capacity_total: {
                label: 'Formatted',
                metrics: [
                    {
                        ..._a.metrics.format_replication_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.format_snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.orange,
                    },
                    {
                        ..._a.metrics.format_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.redDark,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 5,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            formatted_capacity_virtual: {
                label: 'Formatted',
                metrics: [
                    {
                        ..._a.metrics.format_replication_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.format_snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.orange,
                    },
                    {
                        ..._a.metrics.format_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.brownDark,
                        label: 'Virtual',
                    },
                    {
                        ..._a.metrics.format_raid_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 5,
                        color: Color_1.Color.pallete.redDark,
                    },
                    {
                        ..._a.metrics.format_raid_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 6,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            formatted_capacity_total_vspg: {
                label: 'Formatted',
                metrics: [
                    {
                        ..._a.metrics.format_reserved_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.format_snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.orange,
                    },
                    {
                        ..._a.metrics.format_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.brownDark,
                        label: 'Virtual',
                    },
                ],
            },
            oversubscr: {
                label: 'Oversubscription',
                metrics: [
                    {
                        ..._a.metrics.capacity_oversubscr,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            raw_capacity_total: {
                label: 'Raw',
                metrics: [
                    {
                        ..._a.metrics.raw_replication_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.raw_snapshop_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.orange,
                    },
                    {
                        ..._a.metrics.raw_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.raw_raid_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                    {
                        ..._a.metrics.raw_raid_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.redDark,
                    },
                    {
                        ..._a.metrics.raw_raid_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 5,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            raw_capacity_total_vspg: {
                label: 'Raw Total',
                metrics: [
                    {
                        ..._a.metrics.raw_data_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.red,
                    },
                    {
                        ..._a.metrics.spare_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.raw_capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.green,
                    },
                ],
            },
            capacity_used: {
                label: 'Used',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_capacity_compellent: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.unused_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                    {
                        ..._a.metrics.spare_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 3,
                    },
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 4,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            used_compellent: {
                label: 'Used',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.unused_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.spare_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_capacity_used: {
                label: 'Used Total',
                metrics: [
                    {
                        ..._a.metrics.sum_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            free: {
                label: 'Free',
                metrics: [
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            capacity_used_percent: {
                label: 'Used %',
                metrics: [
                    {
                        ..._a.metrics.capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            system_capacity_used_percent: {
                label: 'System Used %',
                metrics: [
                    {
                        ..._a.metrics.system_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            user_used_percent: {
                label: 'User Used %',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            spare_used_percent: {
                label: 'Spare Used %',
                metrics: [
                    {
                        ..._a.metrics.spare_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_capacity_used_percent: {
                label: 'Used Total %',
                metrics: [
                    {
                        ..._a.metrics.sum_capacity_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            fmc_compression_saving: {
                label: 'FMC Compression',
                metrics: [
                    {
                        ..._a.metrics.fmc_compression_saving,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            used_virtual: {
                label: 'Used Virtual',
                metrics: [
                    {
                        ..._a.metrics.virtual_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            fmc_compression_percent: {
                label: 'FMC Compression %',
                metrics: [
                    {
                        ..._a.metrics.fmc_compression_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dkc_compression_saving: {
                label: 'DKC Compression',
                metrics: [
                    {
                        ..._a.metrics.dkc_compression_saving,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dkc_compression_percent: {
                label: 'DKC Compression %',
                metrics: [
                    {
                        ..._a.metrics.dkc_compression_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dkc_dedupe_saving: {
                label: 'DKC Deduplication',
                metrics: [
                    {
                        ..._a.metrics.dkc_dedupe_saving,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dkc_dedupe_percent: {
                label: 'DKC Deduplicattion %',
                metrics: [
                    {
                        ..._a.metrics.dkc_dedupe_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dkc_zpr_saving: {
                label: 'DKC ZPR',
                metrics: [
                    {
                        ..._a.metrics.dkc_zpr_saving,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dkc_zpr_percent: {
                label: 'DKC ZPR %',
                metrics: [
                    {
                        ..._a.metrics.dkc_zpr_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            user_capacity_used: {
                label: 'Used User',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            snapshot_capacity_used: {
                label: 'Used Snapshot',
                metrics: [
                    {
                        ..._a.metrics.snapshot_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            virtual_capacity_total: {
                label: 'Virtual',
                metrics: [
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.brownDark
                    },
                ],
            },
            capacity_allocated: {
                label: 'Allocated',
                metrics: [
                    {
                        ..._a.metrics.capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            user_capacity_allocated: {
                label: 'Allocated User',
                metrics: [
                    {
                        ..._a.metrics.user_capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            snapshot_capacity_allocated: {
                label: 'Allocated Snapshot',
                metrics: [
                    {
                        ..._a.metrics.snapshot_capacity_allocated,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            raw_capacity_used: {
                label: 'Used Raw',
                metrics: [
                    {
                        ..._a.metrics.raw_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.pallete.redDark,
                    },
                ],
            },
            disk_size: {
                label: 'Size On Disk',
                metrics: [
                    {
                        ..._a.metrics.disk_size,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dedupe_ratio_percent: {
                label: 'Ratio',
                metrics: [
                    {
                        ..._a.metrics.dedupe_ratio_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            capacity_total: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            total_aggregated: {
                label: 'Capacity',
                metrics: [
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.capacity_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        color: Color_1.Color.pallete.brownDark,
                    },
                ],
            },
            total_compression_factor: {
                label: 'Compression',
                metrics: [
                    {
                        ..._a.metrics.total_comp_factor,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_compression_factor: {
                label: 'Compression',
                metrics: [
                    {
                        ..._a.metrics.total_comp_factor,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.global_comp_factor,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.local_comp_factor,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            compression_saving: {
                label: 'Compression',
                metrics: [
                    {
                        ..._a.metrics.compression_saving,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dedupe_saving: {
                label: 'Deduplication',
                metrics: [
                    {
                        ..._a.metrics.dedupe_saving,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            dedupe_compression: {
                label: 'Dedupe & Compression',
                metrics: [
                    {
                        ..._a.metrics.physical_capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.red
                    },
                    {
                        ..._a.metrics.fmc_compression_saving,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1
                    },
                    {
                        ..._a.metrics.dkc_compression_saving,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2
                    },
                    {
                        ..._a.metrics.dkc_dedupe_saving,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 3,
                    },
                    {
                        ..._a.metrics.dkc_zpr_saving,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 4,
                    },
                    {
                        ..._a.metrics.physical_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 5,
                        color: Color_1.Color.pallete.black,
                    },
                ],
            },
            tier0_capacity_total: {
                label: 'Tier 0',
                metrics: [
                    {
                        ..._a.metrics.tier0_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            tier1_capacity_total: {
                label: 'Tier 1',
                metrics: [
                    {
                        ..._a.metrics.tier1_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            tier2_capacity_total: {
                label: 'Tier 2',
                metrics: [
                    {
                        ..._a.metrics.tier2_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_compression_capacity: {
                label: 'Compression Total',
                metrics: [
                    {
                        ..._a.metrics.capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.black
                    },
                    {
                        ..._a.metrics.capacity_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: false,
                        order: 1,
                        color: Color_1.Color.pallete.red
                    },
                    {
                        ..._a.metrics.reduction_involved_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                        label: 'Pre-saving',
                    },
                    {
                        ..._a.metrics.virtual_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 3,
                    },
                ],
            },
            data_reduction_ratio: {
                label: 'Data Reduction',
                metrics: [
                    {
                        ..._a.metrics.data_reduction_ratio,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            replication_capacity_total: {
                label: 'Replication',
                metrics: [
                    {
                        ..._a.metrics.replication_capacity_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            capacity_ingested: {
                label: 'Ingested',
                metrics: [
                    {
                        ..._a.metrics.capacity_ingested,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: true,
                        order: 0,
                    }
                ]
            },
        },
        cache: {
            cache_hit: {
                label: 'Cache Hit',
                metrics: [
                    {
                        ..._a.metrics.r_cache_hit,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.w_cache_hit,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            cache_hits_misses_sec: {
                label: 'Cache',
                metrics: [
                    {
                        ..._a.metrics.r_cache_hit_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.w_cache_hit_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 1,
                    },
                    {
                        ..._a.metrics.r_cache_miss_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 2,
                    },
                    {
                        ..._a.metrics.w_cache_miss_sec,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 3,
                    },
                ],
            },
            r_cache_hit: {
                label: 'Read Cache Hit',
                metrics: [
                    {
                        ..._a.metrics.r_cache_hit,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            w_cache_hit: {
                label: 'Write Cache Hit',
                metrics: [
                    {
                        ..._a.metrics.w_cache_hit,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            r_cache_hit_io: {
                label: 'Read Cache Hit IO',
                metrics: [
                    {
                        ..._a.metrics.r_cache_hit_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            w_cache_hit_io: {
                label: 'Write Cache Hit IO',
                metrics: [
                    {
                        ..._a.metrics.w_cache_hit_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: true,
                        order: 0,
                    },
                ],
            },
            cache_usage_total: {
                label: 'Cache Usage',
                metrics: [
                    {
                        ..._a.metrics.cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cache_usage: {
                label: 'Cache Usage',
                metrics: [
                    {
                        ..._a.metrics.r_cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.w_cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            read_usage: {
                label: 'Read Cache Usage',
                metrics: [
                    {
                        ..._a.metrics.r_cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_usage: {
                label: 'Write Cache Usage',
                metrics: [
                    {
                        ..._a.metrics.w_cache_usage,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cfc_dirty: {
                label: 'CFC Dirty',
                metrics: [
                    {
                        ..._a.metrics.cfc_dirty,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            del_ack: {
                label: 'Del Ack',
                metrics: [
                    {
                        ..._a.metrics.del_ack,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_pending: {
                label: 'Write Pending',
                metrics: [
                    {
                        ..._a.metrics.write_pending_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cache_usage_percent: {
                label: 'Cache Usage',
                metrics: [
                    {
                        ..._a.metrics.cache_usage_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            r_cache_usage_percent: {
                label: 'Cache Read Usage',
                metrics: [
                    {
                        ..._a.metrics.r_cache_usage_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            w_cache_usage_percent: {
                label: 'Cache Write Usage',
                metrics: [
                    {
                        ..._a.metrics.w_cache_usage_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        cpu: {
            cpu_system: {
                label: 'System',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cpu_idle: {
                label: 'Idle',
                metrics: [
                    {
                        ..._a.metrics.cpu_idle,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cpu_user: {
                label: 'User',
                metrics: [
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cpu_busy: {
                label: 'Busy',
                metrics: [
                    {
                        ..._a.metrics.cpu_busy,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cpu_compress: {
                label: 'Compress',
                metrics: [
                    {
                        ..._a.metrics.cpu_compress,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            utilization: {
                label: 'Utilization',
                metrics: [
                    {
                        ..._a.metrics.utilization,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_sys: {
                label: 'Total',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                    {
                        ..._a.metrics.cpu_idle,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                    },
                ],
            },
            sum_cpu_system_user: {
                label: 'CPU Total',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    }
                ],
            },
            sum_cpu: {
                label: 'CPU',
                metrics: [
                    {
                        ..._a.metrics.cpu_system,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                        color: Color_1.Color.pallete.gray,
                    },
                    {
                        ..._a.metrics.cpu_user,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                        color: Color_1.Color.pallete.yellow,
                    },
                    {
                        ..._a.metrics.cpu_wait,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 2,
                        color: Color_1.Color.pallete.blue,
                    },
                    {
                        ..._a.metrics.cpu_other,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 3,
                        color: Color_1.Color.pallete.green,
                    },
                ],
            },
        },
        memory: {
            sum_memory_system: {
                label: 'System Memory',
                metrics: [
                    {
                        ..._a.metrics.system_memory_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.system_memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            sum_memory_onheap: {
                label: 'On-heap Memory',
                metrics: [
                    {
                        ..._a.metrics.onheap_memory_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.onheap_memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            sum_memory_offheap: {
                label: 'Off-heap Memory',
                metrics: [
                    {
                        ..._a.metrics.offheap_memory_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                    {
                        ..._a.metrics.offheap_memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: false,
                        order: 1,
                    },
                ],
            },
            memory: {
                label: 'Memory Used %',
                metrics: [
                    {
                        ..._a.metrics.memory_used_percent,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            sum_memory: {
                label: 'Memory',
                metrics: [
                    {
                        ..._a.metrics.memory_used,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.memory_free,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: true,
                        order: 1,
                    },
                ],
            },
        },
        other: {
            crc: {
                label: 'CRC',
                metrics: [
                    {
                        ..._a.metrics.crc,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            b2b: {
                label: 'B2B',
                metrics: [
                    {
                        ..._a.metrics.b2b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                    },
                ],
            },
            loss_sig: {
                label: 'Loss Signal',
                metrics: [
                    {
                        ..._a.metrics.loss_sig,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                    },
                ],
            },
            loss_sync: {
                label: 'Loss Sync',
                metrics: [
                    {
                        ..._a.metrics.loss_sync,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                    },
                ],
            },
            link_fail: {
                label: 'Link Failure',
                metrics: [
                    {
                        ..._a.metrics.link_fail,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                    },
                ],
            },
            buffer_error: {
                label: 'Buffer Error',
                metrics: [
                    {
                        ..._a.metrics.buffer_error,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                        stacked: false,
                    },
                ],
            },
        },
        error: {
            total: {
                label: 'Error',
                metrics: [
                    {
                        ..._a.metrics.error_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        protocol: {
            nfs: {
                label: 'NFS',
                metrics: [
                    {
                        ..._a.metrics.nfs_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_data_rate: {
                label: 'NFS Data Total',
                metrics: [
                    {
                        ..._a.metrics.nfs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs4: {
                label: 'NFS4',
                metrics: [
                    {
                        ..._a.metrics.nfs4_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb_read_io: {
                label: 'SMB IO Read',
                metrics: [
                    {
                        ..._a.metrics.smb_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb_write_io: {
                label: 'SMB IO Write',
                metrics: [
                    {
                        ..._a.metrics.smb_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb_read: {
                label: 'SMB Data Read',
                metrics: [
                    {
                        ..._a.metrics.smb_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb_write: {
                label: 'SMB Data Write',
                metrics: [
                    {
                        ..._a.metrics.smb_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb_resp_t_r: {
                label: 'SMB Latency Read',
                metrics: [
                    {
                        ..._a.metrics.smb_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb_resp_t_w: {
                label: 'SMB Latency Read',
                metrics: [
                    {
                        ..._a.metrics.smb_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb1: {
                label: 'SMB1',
                metrics: [
                    {
                        ..._a.metrics.smb1_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            smb2: {
                label: 'SMB2',
                metrics: [
                    {
                        ..._a.metrics.smb2_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_read_ops: {
                label: 'NFS Read Ops',
                metrics: [
                    {
                        ..._a.metrics.nfs_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_write_ops: {
                label: 'NFS Write Ops',
                metrics: [
                    {
                        ..._a.metrics.nfs_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_other_ops_rate: {
                label: 'NFS Other Ops',
                metrics: [
                    {
                        ..._a.metrics.nfs_other_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_read_io: {
                label: 'NFS IO Read',
                metrics: [
                    {
                        ..._a.metrics.nfs_read_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_write_io: {
                label: 'NFS IO Write',
                metrics: [
                    {
                        ..._a.metrics.nfs_write_io,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_read: {
                label: 'NFS Read Data',
                metrics: [
                    {
                        ..._a.metrics.nfs_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_write: {
                label: 'NFS Write Data',
                metrics: [
                    {
                        ..._a.metrics.nfs_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_resp_t: {
                label: 'NFS Total Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_resp_t_r: {
                label: 'NFS Read Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            nfs_resp_t_w: {
                label: 'NFS Write Latency',
                metrics: [
                    {
                        ..._a.metrics.nfs_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_ops_rate: {
                label: 'CIFS Total Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_read_ops: {
                label: 'CIFS Read Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_read_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_write_ops: {
                label: 'CIFS Write Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_write_ops,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_other_ops_rate: {
                label: 'CIFS Other Ops',
                metrics: [
                    {
                        ..._a.metrics.cifs_other_ops_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_data_rate: {
                label: 'CIFS Data Total',
                metrics: [
                    {
                        ..._a.metrics.cifs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_read: {
                label: 'CIFS Read Data',
                metrics: [
                    {
                        ..._a.metrics.cifs_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_write: {
                label: 'CIFS Write Data',
                metrics: [
                    {
                        ..._a.metrics.cifs_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_resp_t: {
                label: 'CIFS Total Latency',
                metrics: [
                    {
                        ..._a.metrics.cifs_resp_t,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_resp_t_r: {
                label: 'CIFS Read Latency',
                metrics: [
                    {
                        ..._a.metrics.cifs_resp_t_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            cifs_resp_t_w: {
                label: 'CIFS Write Latency',
                metrics: [
                    {
                        ..._a.metrics.cifs_resp_t_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            fs_data_rate: {
                label: 'FS Data Total',
                metrics: [
                    {
                        ..._a.metrics.fs_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            fs_read: {
                label: 'FS Read Data',
                metrics: [
                    {
                        ..._a.metrics.fs_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            fs_write: {
                label: 'FS Write Data',
                metrics: [
                    {
                        ..._a.metrics.fs_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            }
        },
        block_size: {
            block_size: {
                label: 'Block Size',
                metrics: [
                    {
                        ..._a.metrics.block_size,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                        color: Color_1.Color.metric.read
                    },
                ],
            },
            block_size_r: {
                label: 'Block Size Read',
                metrics: [
                    {
                        ..._a.metrics.block_size_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            block_size_w: {
                label: 'Block Size Write',
                metrics: [
                    {
                        ..._a.metrics.block_size_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        rebalance: {
            rebalance_rate: {
                label: 'Rebalance Rate',
                metrics: [
                    {
                        ..._a.metrics.rebalance_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebalance_pending: {
                label: 'Rebalance Pending Capacity',
                metrics: [
                    {
                        ..._a.metrics.rebalance_pending,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        rebuild: {
            rebuild_rate: {
                label: 'Rebuild Rate',
                metrics: [
                    {
                        ..._a.metrics.rebuild_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_pending: {
                label: 'Rebuild Pending Capacity',
                metrics: [
                    {
                        ..._a.metrics.rebuild_pending,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_queue: {
                label: 'Rebuild Queue',
                metrics: [
                    {
                        ..._a.metrics.rebuild_queue,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_writes_total: {
                label: 'Rebuild Writes Total',
                metrics: [
                    {
                        ..._a.metrics.rebuild_writes_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_writes_in: {
                label: 'Rebuild Writes In',
                metrics: [
                    {
                        ..._a.metrics.rebuild_writes_in,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_writes_out: {
                label: 'Rebuild Writes Out',
                metrics: [
                    {
                        ..._a.metrics.rebuild_writes_out,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_deletes_total: {
                label: 'Rebuild Deletes Total',
                metrics: [
                    {
                        ..._a.metrics.rebuild_deletes_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_deletes_in: {
                label: 'Rebuild Deletes In',
                metrics: [
                    {
                        ..._a.metrics.rebuild_deletes_in,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_deletes_out: {
                label: 'Rebuild Deletes Out',
                metrics: [
                    {
                        ..._a.metrics.rebuild_deletes_out,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_bytes_total: {
                label: 'Rebuild Bytes Total',
                metrics: [
                    {
                        ..._a.metrics.rebuild_bytes_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_bytes_in: {
                label: 'Rebuild Bytes In',
                metrics: [
                    {
                        ..._a.metrics.rebuild_bytes_in,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            rebuild_bytes_out: {
                label: 'Rebuild Bytes Out',
                metrics: [
                    {
                        ..._a.metrics.rebuild_bytes_out,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        housekeep: {
            housekeep_pend_data: {
                label: 'Pending Data',
                metrics: [
                    {
                        ..._a.metrics.housekeep_pend_data,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        job: {
            copy_job_read: {
                label: 'Read Copy Job',
                metrics: [
                    {
                        ..._a.metrics.copy_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            copy_job_write: {
                label: 'Write Copy Job',
                metrics: [
                    {
                        ..._a.metrics.copy_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            data_job_read: {
                label: 'Read Data Job',
                metrics: [
                    {
                        ..._a.metrics.data_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            data_job_write: {
                label: 'Write Data Job',
                metrics: [
                    {
                        ..._a.metrics.data_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            stream_job_rate: {
                label: 'Stream Job Rate',
                metrics: [
                    {
                        ..._a.metrics.stream_job_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            logical_copy_job_read: {
                label: 'Logical Copy Read',
                metrics: [
                    {
                        ..._a.metrics.logical_copy_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            logical_copy_job_write: {
                label: 'Logical Copy Write',
                metrics: [
                    {
                        ..._a.metrics.logical_copy_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            logical_data_job_read: {
                label: 'Logical Data Read',
                metrics: [
                    {
                        ..._a.metrics.logical_data_job_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            logical_data_job_write: {
                label: 'Logical Data Write',
                metrics: [
                    {
                        ..._a.metrics.logical_data_job_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        replication: {
            total: {
                label: 'Replication Rate',
                metrics: [
                    {
                        ..._a.metrics.replication_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read_replication: {
                label: 'Read Reaplication',
                metrics: [
                    {
                        ..._a.metrics.read_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_replication: {
                label: 'Write Replication',
                metrics: [
                    {
                        ..._a.metrics.write_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            stream_write_replication: {
                label: 'Stream Write Replication',
                metrics: [
                    {
                        ..._a.metrics.stream_write_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            stream_read_replication: {
                label: 'Stream Read Replication',
                metrics: [
                    {
                        ..._a.metrics.stream_read_replication,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        network: {
            network_rate: {
                label: 'Net',
                metrics: [
                    {
                        ..._a.metrics.network_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            network_rx: {
                label: 'Net In',
                metrics: [
                    {
                        ..._a.metrics.network_rx,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            network_tx: {
                label: 'Net Out',
                metrics: [
                    {
                        ..._a.metrics.network_tx,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            network_rate_b: {
                label: 'Net BE',
                metrics: [
                    {
                        ..._a.metrics.network_rate_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            network_rx_b: {
                label: 'Net In BE',
                metrics: [
                    {
                        ..._a.metrics.network_rx_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            network_tx_b: {
                label: 'Net Out BE',
                metrics: [
                    {
                        ..._a.metrics.network_tx_b,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            packet_rate: {
                label: 'Packet Rate',
                metrics: [
                    {
                        ..._a.metrics.packet_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read_packet: {
                label: 'Packet Read',
                metrics: [
                    {
                        ..._a.metrics.read_packet,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_packet: {
                label: 'Packet Write',
                metrics: [
                    {
                        ..._a.metrics.write_packet,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read_error: {
                label: 'Error Read',
                metrics: [
                    {
                        ..._a.metrics.read_error,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write_error: {
                label: 'Error Write',
                metrics: [
                    {
                        ..._a.metrics.write_error,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            tcp_timeouts: {
                label: 'TCP Timeouts',
                metrics: [
                    {
                        ..._a.metrics.tcp_timeouts,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        clean: {
            total: {
                label: 'Phase',
                metrics: [
                    {
                        ..._a.metrics.phase,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        net: {
            total: {
                label: 'Net',
                metrics: [
                    {
                        ..._a.metrics.net_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            read: {
                label: 'Net Read',
                metrics: [
                    {
                        ..._a.metrics.read_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            write: {
                label: 'Net Write',
                metrics: [
                    {
                        ..._a.metrics.write_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        bound: {
            outbound_pre_comp: {
                label: 'Outbound Pre Comp',
                metrics: [
                    {
                        ..._a.metrics.outbound_pre_comp,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            inbound_pre_comp: {
                label: 'Inbound Pre Comp',
                metrics: [
                    {
                        ..._a.metrics.inbound_pre_comp,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            outbound_net: {
                label: 'Outbound Net',
                metrics: [
                    {
                        ..._a.metrics.outbound_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            inbound_net: {
                label: 'Inbound Net',
                metrics: [
                    {
                        ..._a.metrics.inbound_net,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            free: {
                label: 'Free',
                metrics: [
                    {
                        ..._a.metrics.memory_free,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        os: {
            os_total: {
                label: 'OS Total',
                metrics: [
                    {
                        ..._a.metrics.os_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            os_r: {
                label: 'OS Read',
                metrics: [
                    {
                        ..._a.metrics.os_r,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            os_w: {
                label: 'OS Write',
                metrics: [
                    {
                        ..._a.metrics.os_w,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        temperature: {
            temperature: {
                label: 'Temperature',
                metrics: [
                    {
                        ..._a.metrics.temperature,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        queue: {
            queue_length: {
                label: 'Queue Length',
                metrics: [
                    {
                        ..._a.metrics.queue_length,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        object: {
            total: {
                label: 'Object',
                metrics: [
                    {
                        ..._a.metrics.object_total,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    }
                ]
            },
            created: {
                label: 'Created',
                metrics: [
                    {
                        ..._a.metrics.object_created,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    }
                ]
            },
            deleted: {
                label: 'Deleted',
                metrics: [
                    {
                        ..._a.metrics.object_deleted,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    }
                ]
            },
        },
        quota: {
            soft_quota: {
                label: 'Soft Quota',
                metrics: [
                    {
                        ..._a.metrics.soft_quota,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            hard_quota: {
                label: 'Hard Quota',
                metrics: [
                    {
                        ..._a.metrics.hard_quota,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
        delta: {
            delta_size: {
                label: 'Delta Size',
                metrics: [
                    {
                        ..._a.metrics.delta_size,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    }
                ]
            },
        },
        size: {
            size: {
                label: 'Size',
                metrics: [
                    {
                        ..._a.metrics.size,
                        type: pageResponse_constants_1.CHART_TYPE.area,
                        color: Color_1.Color.pallete.totalRed,
                        stacked: true,
                        order: 0,
                    },
                    {
                        ..._a.metrics.hard_quota,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        color: Color_1.Color.pallete.black,
                        stacked: false,
                        order: 2,
                    },
                    {
                        ..._a.metrics.soft_quota,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        color: Color_1.Color.pallete.gray,
                        stacked: false,
                        order: 1,
                    },
                ]
            },
        },
        swap: {
            swap_out: {
                label: 'Swap Out',
                metrics: [
                    {
                        ..._a.metrics.swap_out,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    }
                ]
            }
        },
        lun: {
            lun_read: {
                label: 'LUN Read',
                metrics: [
                    {
                        ..._a.metrics.lun_read,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            lun_write: {
                label: 'LUN Write',
                metrics: [
                    {
                        ..._a.metrics.lun_write,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            lun_data_rate: {
                label: 'LUN Data',
                metrics: [
                    {
                        ..._a.metrics.lun_data_rate,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
            lun_utilization: {
                label: 'LUN Util',
                metrics: [
                    {
                        ..._a.metrics.lun_utilization,
                        type: pageResponse_constants_1.CHART_TYPE.line,
                        stacked: false,
                        order: 0,
                    },
                ],
            },
        },
    },
};
Storage.configurations = {
    model: {
        label: "Model",
    },
    version: {
        label: "Version"
    },
    apiVersion: {
        label: "API version"
    },
    cluster_name: {
        label: "Cluster name"
    },
    cluster_guid: {
        label: "Cluster GUID"
    },
    serial: {
        label: "Serial"
    },
    name: {
        label: "Name"
    },
    node_name: {
        label: "Name"
    },
    node: {
        label: "Node"
    },
    ip: {
        label: "IP address"
    },
    hostname: {
        label: "Hostname"
    },
    clusterMode: {
        label: "Cluster mode"
    },
    id: {
        label: "ID"
    },
    node_count: {
        label: "Node Count"
    },
    metadata_available: {
        label: "Metadata Available [TiB]"
    },
    metadata_used: {
        label: "Metadata Used [%]"
    },
    numOfDevices: {
        label: "Devices"
    },
    numOfMappedToAllVolumes: {
        label: "Mapped to all volumes"
    },
    numOfProtectionDomains: {
        label: "Protection domains"
    },
    numOfSdc: {
        label: "SDCs"
    },
    numOfSds: {
        label: "SDSs"
    },
    numOfSnapshots: {
        label: "Snapshots"
    },
    numOfStoragePools: {
        label: "Pools"
    },
    numOfThickBaseVolumes: {
        label: "Thick volumes"
    },
    numOfThinBaseVolumes: {
        label: "Thin volumes"
    },
    numOfUnmappedVolumes: {
        label: "Unmapped volumes"
    },
    numOfVolumes: {
        label: "Volumes"
    },
    numOfVolumesInDeletion: {
        label: "In deletion volumes"
    },
    numOfVtrees: {
        label: "Vtrees"
    },
    patches: {
        label: "Patches",
    },
    agent_version: {
        label: "Agent version",
    },
    cluster_uuid: {
        label: "Cluster UUID",
    },
    nas_uuid: {
        label: "NAS UUID",
    },
    machine_name: {
        label: "Machine name",
    },
    model_name: {
        label: "Model name",
    },
    product_name: {
        label: "Product name",
    },
    performance_statistic_status: {
        label: "Performance statistic status",
    },
    performance_statistic_frequency: {
        label: "Performance statistic frequency",
    },
    last_conf_date: {
        label: "Date",
    },
    capacity_pools: {
        label: "Capacity pools",
    },
    volumes_allocated_capacity: {
        label: "Capacity allocated to volumes",
    },
    total_overallocation: {
        label: "Total overallocation",
    },
    total_mdisk_capacity: {
        label: "Total Mdisk capacity",
    },
    total_volumecopy_capacity: {
        label: "Total Volumecopy capacity",
    },
    total_free_capacity: {
        label: "Total free capacity",
    },
    total_allocated_extent_capacity: {
        label: "Total allocated extent capacity",
    },
    total_used_capacity: {
        label: "Total used capacity",
    },
    total_volume_capacity: {
        label: "Total volumes capacity",
    },
    symmetrix_id: {
        label: "Symmetrix ID",
    },
    ucode: {
        label: "ucode",
    },
    microcode: {
        label: "microcode",
    },
    wwn: {
        label: "WWN",
    },
    wwnn: {
        label: "WWNN",
    },
    mediatype: {
        label: "Mediatype",
    },
    pd_name: {
        label: "Protection domain name",
    },
    array_name: {
        label: "Array name",
    },
    sw_version: {
        label: "SW Version",
    },
    node_type: {
        label: "Node Type",
    },
    disk_count: {
        label: "Disk Count",
    },
    removal_state: {
        label: "Removal State",
    },
    extended_model: {
        label: "Extended model",
    },
    grid_name: {
        label: "Grid Name",
    },
    grid_id: {
        label: "Grid ID",
    },
    ctla: {
        label: "Controller A",
    },
    ctlb: {
        label: "Controller B",
    },
    health_status: {
        label: "Health Status",
    },
    running_status: {
        label: "Running status",
    },
    sys_sw_version: {
        label: "sys-sw-version",
    },
    sys_health_state: {
        label: "sys-health-state",
    },
    sys_state: {
        label: "sys-state",
    },
    license_id: {
        label: "license-id",
    },
    memory: {
        ...Definitions_1.metricTypes.memory,
        label: "Memory",
    },
    disk_capacity: {
        label: "HDD capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    flash_capacity: {
        label: "FLASH capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_allocated: {
        label: "Allocated capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_failed: {
        label: "Failed capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_total: {
        label: "Total capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_free: {
        label: "Free capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_used: {
        label: "Used capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_user: {
        label: "User capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_snap: {
        label: "Snap capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_admin: {
        label: "Admin capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_storage_consumed: {
        label: "Storage Consumed",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_data_in: {
        label: "Data In",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_data_written: {
        label: "Data Written",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_data_reduction: {
        label: "Data Reduction",
        unit: subsystemMetricsResponse_constants_1.UNIT.multiples,
    },
    capacity_logical_usage: {
        label: "Logical Usage",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_physical_usage: {
        label: "Physical Usage",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    capacity_node_total: {
        label: "Capacity",
        unit: subsystemMetricsResponse_constants_1.UNIT.bytes,
    },
    os: {
        label: "OS"
    },
    downServices: {
        label: "Services Down"
    },
    upServices: {
        label: "Services UP"
    },
    e_platform: {
        label: "E-platform"
    },
    sfx_capable: {
        label: "SFX Capable"
    },
    sfx_enabled: {
        label: "SFX Enable"
    },
    bios: {
        label: "BIOS"
    },
    bmc_version: {
        label: "BMC Version"
    },
    system_status: {
        label: "System Status",
    },
    temperature: {
        label: "Temperature",
    },
    power_status: {
        label: "Power Status"
    },
    system_fan_status: {
        label: "System FAN Status",
    },
    cpu_fan_status: {
        label: "CPU FAN Status",
    },
    system_name: {
        label: "System Name"
    },
    platform_type: {
        label: "Platform Type"
    },
    system_uuid: {
        label: "System UUID"
    },
    health_state: {
        label: "Health State"
    },
    health_details: {
        label: "Health Details"
    },
    license_activation_key: {
        label: "License Activation Key"
    },
};
Storage.pages = {
    totals: {
        data: {
            label: 'Data',
            tabs: {
                total: _a.tabs.totals.data.sum_data,
                aggregated: _a.tabs.totals.data.aggregated,
                read: _a.tabs.totals.data.read,
                write: _a.tabs.totals.data.write,
            }
        },
        io: {
            label: 'IO',
            tabs: {
                total: _a.tabs.totals.io.sum_io,
                aggregated: _a.tabs.totals.io.aggregated,
                read: _a.tabs.totals.io.read,
                write: _a.tabs.totals.io.write,
            },
        },
        resp_t: {
            label: 'Latency',
            tabs: {
                total: _a.tabs.totals.resp_t.total,
                read: _a.tabs.totals.resp_t.read,
                write: _a.tabs.totals.resp_t.write,
            },
        },
        block_size: {
            label: 'Block Size',
            tabs: {
                total: _a.tabs.totals.block_size.total,
                read: _a.tabs.totals.block_size.read,
                write: _a.tabs.totals.block_size.write,
            },
        },
        ops: {
            label: 'OPS',
            tabs: {
                total: _a.tabs.totals.ops.sum_ops,
                aggregated: _a.tabs.totals.ops.aggregated,
                read: _a.tabs.totals.ops.read,
                write: _a.tabs.totals.ops.write,
            },
        },
    }
};
Storage.defaultConfiguration = {
    model: _a.configurations.model,
    version: _a.configurations.version,
    serial: _a.configurations.serial,
};
Storage.defaultPoolConfiguration = {
    capacity_total: _a.configurations.capacity_total,
    capacity_used: _a.configurations.capacity_used,
    capacity_free: _a.configurations.capacity_free,
};
Storage.technology = {
    isilon: {
        label: 'Isilon',
        heatmap: false,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    cluster_name: _a.configurations.cluster_name,
                    apiVersion: _a.configurations.apiVersion,
                    cluster_guid: _a.configurations.cluster_guid,
                    version: _a.configurations.version
                },
                metrics: {
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            cpu: {
                label: 'CPU',
                totals: {
                    cpu: {
                        label: 'Total',
                        tabs: {
                            total: _a.tabs.totals.cpu.sum_sys,
                            busy: {
                                ..._a.tabs.totals.busy.sum_busy,
                                label: 'Busy Total',
                            },
                            busy_aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Busy Aggregated',
                            },
                            sys: _a.tabs.totals.cpu.sys,
                            user: _a.tabs.totals.cpu.user,
                            idle: _a.tabs.totals.cpu.idle,
                        },
                    },
                },
                single: {
                    cpu: _a.tabs.single.cpu.sum_sys,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    cpu_user: _a.tabs.single.cpu.cpu_user,
                    cpu_idle: _a.tabs.single.cpu.cpu_idle,
                },
            },
            node: {
                label: 'Node',
                totals: {
                    data: _a.pages.totals.data,
                    nfs: {
                        label: 'Protocol',
                        tabs: {
                            nfs: _a.tabs.totals.protocol.nfs,
                            nfs4: _a.tabs.totals.protocol.nfs4,
                            smb1: _a.tabs.totals.protocol.smb1,
                            smb2: _a.tabs.totals.protocol.smb2,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    nfs: _a.tabs.single.protocol.nfs,
                    nfs4: _a.tabs.single.protocol.nfs4,
                    smb1: _a.tabs.single.protocol.smb1,
                    smb2: _a.tabs.single.protocol.smb2,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    data: _a.pages.totals.data,
                    nfs: {
                        label: 'Protocol',
                        tabs: {
                            nfs: _a.tabs.totals.protocol.nfs,
                            nfs4: _a.tabs.totals.protocol.nfs4,
                            smb1: _a.tabs.totals.protocol.smb1,
                            smb2: _a.tabs.totals.protocol.smb2,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    nfs: _a.tabs.single.protocol.nfs,
                    nfs4: _a.tabs.single.protocol.nfs4,
                    smb1: _a.tabs.single.protocol.smb1,
                    smb2: _a.tabs.single.protocol.smb2,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    data: _a.pages.totals.data,
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        },
                    },
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.used
                        }
                    }
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                    capacity_used: _a.tabs.single.capacity.capacity_used
                },
            },
            port: {
                label: 'Port',
                totals: {
                    data: _a.pages.totals.data,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write
                },
            },
        },
    },
    eseries: {
        label: 'E-series',
        heatmap: true,
        status: {
            explicit: "device",
        },
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: 'Device',
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    wwn: _a.configurations.wwn,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            node: {
                label: 'Node',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            sys: _a.tabs.totals.cpu.sys,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                            r_cache_usage: _a.tabs.totals.cache.read_usage,
                            w_cache_usage: _a.tabs.totals.cache.write_usage,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.total,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    cache_hit: _a.tabs.single.cache.cache_hit,
                    cache_usage: _a.tabs.single.cache.cache_usage,
                    total: {
                        ..._a.tabs.single.capacity.capacity_total,
                        label: 'Capacity',
                    },
                },
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    total: {
                        ..._a.tabs.single.capacity.capacity_total,
                        label: 'Capacity',
                    },
                },
            },
            host_group: {
                label: 'Host Group',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.total,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    total: {
                        ..._a.tabs.single.capacity.capacity_total,
                        label: 'Capacity',
                    },
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            port: {
                label: 'Port',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    error: {
                        label: 'Error',
                        tabs: {
                            total: _a.tabs.totals.error.total,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    error: _a.tabs.single.error.total,
                },
            },
        },
    },
    vmax: {
        label: 'EMC VMAX/PowerMAX',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    symmetrix_id: _a.configurations.symmetrix_id,
                    ucode: _a.configurations.ucode,
                    microcode: _a.configurations.microcode,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                    virtual_capacity_total: _a.metrics.virtual_capacity_total,
                },
            },
            fedirector: {
                label: 'FE Director',
                totals: {
                    io: _a.pages.totals.io,
                    data: {
                        label: 'Data',
                        tabs: {
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cache_hit: _a.tabs.single.cache.cache_hit,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                },
            },
            bedirector: {
                label: 'BE Director',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                },
            },
            rdfdirector: {
                label: 'RDF Director',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                },
            },
            tier: {
                label: 'Tier',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            total_b: _a.tabs.totals.io.sum_io_b,
                            aggregated_b: _a.tabs.totals.io.aggregated_b,
                            read_b: _a.tabs.totals.io.read_b,
                            write_b: _a.tabs.totals.io.write_b,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            total_b: _a.tabs.totals.data.sum_data_b,
                            aggregated_b: {
                                ..._a.tabs.totals.data.total_b,
                                label: 'Aggregated Back-end',
                            },
                            read_b: _a.tabs.totals.data.read_b,
                            write_b: _a.tabs.totals.data.write_b,
                        },
                    },
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cache_hit: _a.tabs.single.cache.cache_hit,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            slo: {
                label: 'SLO',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cache_hit: _a.tabs.single.cache.cache_hit,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            virtual: _a.tabs.totals.capacity.virtual,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    virtual_capacity_total: _a.tabs.single.capacity.virtual_capacity_total,
                },
            },
            volume: {
                label: 'Volume',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    cache_hit: _a.tabs.single.cache.cache_hit,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            feport: {
                label: 'FEPort',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                },
            },
            beport: {
                label: 'BEPort',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                },
            },
            rdfport: {
                label: 'RDFPort',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                },
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
        },
    },
    ecs: {
        label: 'EMC ECS',
        heatmap: false,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {},
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            namespace: {
                label: 'Namespace',
                totals: {
                    data: _a.pages.totals.data,
                    object: {
                        label: 'Object',
                        tabs: {
                            ..._a.tabs.totals.object
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            ..._a.tabs.totals.size,
                            ..._a.tabs.totals.delta,
                        }
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    ..._a.tabs.single.object,
                    ..._a.tabs.single.size,
                    ..._a.tabs.single.delta,
                },
                metrics: {
                    hard_quota: _a.metrics.hard_quota,
                    size: _a.metrics.size,
                    capacity_free: _a.metrics.capacity_free
                }
            },
            bucket: {
                label: 'Bucket',
                totals: {
                    data: _a.pages.totals.data,
                    object: {
                        label: 'Object',
                        tabs: {
                            ..._a.tabs.totals.object
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            ..._a.tabs.totals.size,
                            ..._a.tabs.totals.delta,
                        }
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    ..._a.tabs.single.object,
                    ..._a.tabs.single.size,
                    ..._a.tabs.single.delta,
                },
                metrics: {
                    hard_quota: _a.metrics.hard_quota,
                    size: _a.metrics.size,
                    capacity_free: _a.metrics.capacity_free
                }
            },
            vdc: {
                label: "VDC",
                totals: {},
                single: {},
                metrics: {
                    write: _a.metrics.write,
                    read: _a.metrics.read,
                    data_rate: _a.metrics.data_rate,
                    cpu_system: _a.metrics.cpu_system,
                    memory_used: _a.metrics.memory_used,
                    memory_free: _a.metrics.memory_free,
                    memory: _a.metrics.memory_used_percent,
                    error_rate_tps: _a.metrics.error_rate_tps,
                    write_tps: _a.metrics.write_tps,
                    read_tps: _a.metrics.read_tps,
                    tps_rate: _a.metrics.tps_rate,
                    resp_t_w: _a.metrics.resp_t_w,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t: _a.metrics.resp_t,
                    network_tx: _a.metrics.network_tx,
                    network_rx: _a.metrics.network_rx,
                    network_rate: _a.metrics.network_rate,
                    network_util_rx: _a.metrics.network_util_rx,
                    network_util_tx: _a.metrics.network_util_tx,
                    network_util: _a.metrics.network_util,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    disk_write: _a.metrics.disk_write,
                    disk_read: _a.metrics.disk_read,
                    disk_data_rate: _a.metrics.disk_data_rate,
                    disk_geo_read: _a.metrics.disk_geo_read,
                    disk_geo_write: _a.metrics.disk_geo_write,
                    disk_geo_data_rate: _a.metrics.disk_geo_data_rate,
                    disk_user_read: _a.metrics.disk_user_read,
                    disk_user_write: _a.metrics.disk_user_write,
                    disk_user_data_rate: _a.metrics.disk_user_data_rate,
                }
            },
            node: {
                label: "Node",
                totals: {},
                single: {},
                metrics: {
                    write: _a.metrics.write,
                    read: _a.metrics.read,
                    data_rate: _a.metrics.data_rate,
                    cpu_system: _a.metrics.cpu_system,
                    memory_used: _a.metrics.memory_used,
                    memory_free: _a.metrics.memory_free,
                    memory: _a.metrics.memory_used_percent,
                    error_rate_tps: _a.metrics.error_rate_tps,
                    write_tps: _a.metrics.write_tps,
                    read_tps: _a.metrics.read_tps,
                    tps_rate: _a.metrics.tps_rate,
                    resp_t_w: _a.metrics.resp_t_w,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t: _a.metrics.resp_t,
                    network_tx: _a.metrics.network_tx,
                    network_rx: _a.metrics.network_rx,
                    network_rate: _a.metrics.network_rate,
                    network_util_rx: _a.metrics.network_util_rx,
                    network_util_tx: _a.metrics.network_util_tx,
                    network_util: _a.metrics.network_util,
                    disk_write: _a.metrics.disk_write,
                    disk_read: _a.metrics.disk_read,
                    disk_data_rate: _a.metrics.disk_data_rate,
                    disk_geo_read: _a.metrics.disk_geo_read,
                    disk_geo_write: _a.metrics.disk_geo_write,
                    disk_geo_data_rate: _a.metrics.disk_geo_data_rate,
                    disk_user_read: _a.metrics.disk_user_read,
                    disk_user_write: _a.metrics.disk_user_write,
                    disk_user_data_rate: _a.metrics.disk_user_data_rate,
                }
            },
            replication_group: {
                label: "RG",
                totals: {
                    data: _a.pages.totals.data,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                },
                metrics: {
                    write: _a.metrics.write,
                    read: _a.metrics.read,
                    data_rate: _a.metrics.data_rate,
                }
            },
            replication_group_link: {
                label: "RGL",
                totals: {},
                single: {},
                metrics: {
                    write: _a.metrics.write,
                    read: _a.metrics.read,
                    data_rate: _a.metrics.data_rate,
                }
            },
        },
    },
    xtremio: {
        label: 'EMC XtremIO',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    sys_health_state: _a.configurations.sys_health_state,
                    sys_state: _a.configurations.sys_state,
                    license_id: _a.configurations.license_id,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                }
            },
            node: {
                label: 'Node',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            sys: _a.tabs.totals.cpu.sys,
                        },
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            dedupe_ratio: _a.tabs.totals.data.dedupe_ratio,
                        },
                    },
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            virtual: _a.tabs.totals.capacity.virtual,
                        },
                    },
                    compression: {
                        label: 'Compression',
                        tabs: {
                            compression: _a.tabs.totals.capacity.total_compression_factor,
                            data_reduction: _a.tabs.totals.capacity.data_reduction_ratio,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    dedupe_ratio: _a.tabs.single.data.dedupe_ratio,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    virtual_capacity_total: _a.tabs.single.capacity.virtual_capacity_total,
                    compression: _a.tabs.single.capacity.total_compression_factor,
                    data_reduction_ratio: _a.tabs.single.capacity.data_reduction_ratio,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                },
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            host_group: {
                label: 'Host Group',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                },
            },
        },
    },
    swiz: {
        label: 'Storwize',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    machine_name: _a.configurations.machine_name,
                    product_name: _a.configurations.product_name,
                    performance_statistic_status: _a.configurations.performance_statistic_status,
                    performance_statistic_frequency: _a.configurations.performance_statistic_frequency,
                    last_conf_date: _a.configurations.last_conf_date,
                    capacity_pools: _a.configurations.capacity_pools,
                    volumes_allocated_capacity: _a.configurations.volumes_allocated_capacity,
                    total_overallocation: _a.configurations.total_overallocation,
                    total_mdisk_capacity: _a.configurations.total_mdisk_capacity,
                    total_volumecopy_capacity: _a.configurations.total_volumecopy_capacity,
                    total_free_capacity: _a.configurations.total_free_capacity,
                    total_allocated_extent_capacity: _a.configurations.total_allocated_extent_capacity,
                    total_used_capacity: _a.configurations.total_used_capacity,
                    total_volume_capacity: _a.configurations.total_volume_capacity,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    cache_usage: _a.metrics.cache_usage,
                    r_cache_hit: _a.metrics.r_cache_hit,
                    w_cache_hit: _a.metrics.w_cache_hit,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                }
            },
            cpu: {
                label: 'CPU',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            sys: _a.tabs.totals.cpu.sys,
                            compress: _a.tabs.totals.cpu.compress,
                        },
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    cpu_compress: _a.tabs.single.cpu.cpu_compress,
                },
            },
            node: {
                label: 'Node',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            sys: _a.tabs.totals.cpu.sys,
                            compress: _a.tabs.totals.cpu.compress,
                        },
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    cpu_compress: _a.tabs.single.cpu.cpu_compress,
                },
            },
            tier: {
                label: 'Tier',
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            formatted_capacity_total: _a.tabs.totals.capacity.formatted_capacity_total,
                            raw_capacity_total: _a.tabs.totals.capacity.raw_capacity_total,
                            compression_saving: _a.tabs.totals.capacity.compression_saving,
                            dedupe_saving: _a.tabs.totals.capacity.dedupe_saving,
                        },
                    },
                },
                single: {
                    formatted_capacity_total: _a.tabs.single.capacity.formatted_capacity_total,
                    raw_capacity_total: _a.tabs.single.capacity.raw_capacity_total,
                    compression_saving: _a.tabs.single.capacity.compression_saving,
                    dedupe_saving: _a.tabs.single.capacity.dedupe_saving,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            virtual: _a.tabs.totals.capacity.virtual,
                            formatted_capacity_virtual: _a.tabs.totals.capacity.formatted_capacity_virtual,
                            raw_capacity_total: _a.tabs.totals.capacity.raw_capacity_total,
                            compression_saving: _a.tabs.totals.capacity.compression_saving,
                            dedupe_saving: _a.tabs.totals.capacity.dedupe_saving,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    virtual_capacity_total: _a.tabs.single.capacity.virtual_capacity_total,
                    formatted_capacity_virtual: _a.tabs.single.capacity.formatted_capacity_virtual,
                    raw_capacity_total: _a.tabs.single.capacity.raw_capacity_total,
                    compression_saving: _a.tabs.single.capacity.compression_saving,
                    dedupe_saving: _a.tabs.single.capacity.dedupe_saving,
                },
            },
            io_group: {
                label: 'IO group',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            mdisk: {
                label: 'Managed disk',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                            cache_usage: _a.tabs.totals.cache.cache_usage_total,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    cache_hit: _a.tabs.single.cache.cache_hit,
                    cache_usage: _a.tabs.single.cache.cache_usage_total,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            port: {
                label: 'Port',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    others: {
                        label: 'Other',
                        tabs: {
                            crc: _a.tabs.totals.other.crc,
                            b2b: _a.tabs.totals.other.b2b,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    crc: _a.tabs.single.other.crc,
                    b2b: _a.tabs.single.other.b2b,
                },
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
        },
    },
    xiv: {
        label: 'IBM XIV/A9k',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    machine_name: _a.configurations.machine_name,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                },
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
        },
    },
    '3par': {
        label: '3PAR',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    id: _a.configurations.id,
                    node_name: _a.configurations.node_name,
                    patches: _a.configurations.patches,
                    capacity_total: _a.configurations.capacity_total,
                    capacity_free: _a.configurations.capacity_free,
                    capacity_allocated: _a.configurations.capacity_allocated,
                    capacity_failed: _a.configurations.capacity_failed,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            cpu: {
                label: 'CPU',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            sum_cpu_system_user: _a.tabs.totals.cpu.sum_cpu_system_user,
                        },
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    cpu_user: _a.tabs.single.cpu.cpu_user,
                },
            },
            tier: {
                label: 'Tier',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            sum_capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            pool: {
                label: 'Pool',
                configuration: {
                    ..._a.defaultPoolConfiguration,
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            allocated: _a.tabs.totals.capacity.allocated,
                            sum_capacity_used: _a.tabs.totals.capacity.sum_capacity_used,
                            sum_capacity_used_percent: _a.tabs.totals.capacity.sum_capacity_used_percent,
                            user_capacity_allocated: _a.tabs.totals.capacity.user_capacity_allocated,
                            snapshot_capacity_allocated: _a.tabs.totals.capacity.snapshot_capacity_allocated,
                            user_capacity_used: _a.tabs.totals.capacity.user_capacity_used,
                            snapshot_capacity_used: _a.tabs.totals.capacity.snapshot_capacity_used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    capacity_allocated: _a.tabs.single.capacity.capacity_allocated,
                    sum_capacity_used: _a.tabs.single.capacity.sum_capacity_used,
                    sum_capacity_used_percent: _a.tabs.single.capacity.sum_capacity_used_percent,
                    user_capacity_allocated: _a.tabs.single.capacity.user_capacity_allocated,
                    snapshot_capacity_allocated: _a.tabs.single.capacity.snapshot_capacity_allocated,
                    user_capacity_used: _a.tabs.single.capacity.user_capacity_used,
                    snapshot_capacity_used: _a.tabs.single.capacity.snapshot_capacity_used,
                    sum_capacity_user_allocated: {
                        ..._a.tabs.single.capacity.sum_capacity_user_allocated,
                        label: 'User Capacity',
                    },
                    sum_capacity_snapshot: {
                        ..._a.tabs.single.capacity.sum_capacity_snapshot,
                        label: 'Snapshot Capacity',
                    },
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: {
                                ..._a.tabs.totals.capacity.used,
                                label: 'Used User',
                            },
                            used_percent: {
                                ..._a.tabs.totals.capacity.used_percent,
                                label: 'Used User %',
                            },
                            snapshot_capacity_used: _a.tabs.totals.capacity.snapshot_capacity_used,
                            sum_capacity_used: _a.tabs.totals.capacity.sum_capacity_used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity: {
                        ..._a.tabs.single.capacity.sum_capacity,
                        label: 'Capacity User',
                    },
                    used_percent: {
                        ..._a.tabs.single.capacity.capacity_used_percent,
                        label: 'Used User %',
                    },
                    snapshot_capacity_used: _a.tabs.single.capacity.snapshot_capacity_used,
                    sum_capacity_used: _a.tabs.single.capacity.sum_capacity_used,
                },
            },
            vvol: {
                label: 'Virtual volume',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity_used_percent: _a.tabs.totals.capacity.used_percent,
                            user_capacity_used_percent: _a.tabs.totals.capacity.user_used_percent,
                            spare_capacity_used_percent: _a.tabs.totals.capacity.spare_used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    sum_capacity_user: _a.tabs.single.capacity.sum_capacity_user,
                    user_capacity_used_percent: _a.tabs.single.capacity.user_used_percent,
                    sum_capacity_spare: _a.tabs.single.capacity.sum_capacity_spare,
                    spare_capacity_used_percent: _a.tabs.single.capacity.spare_used_percent,
                },
            },
            port: {
                label: 'Port',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            cache: {
                label: 'Cache',
                totals: {
                    cache: {
                        label: 'Cache',
                        tabs: {
                            read_hit: _a.tabs.totals.cache.read_hit,
                            write_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                    tier: {
                        label: 'Tier',
                        tabs: {
                            cfc_dirty: _a.tabs.totals.cache.cfc_dirty,
                            del_ack: _a.tabs.totals.cache.del_ack,
                        },
                    },
                },
                single: {
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    cfc_dirty: _a.tabs.single.cache.cfc_dirty,
                    del_ack: _a.tabs.single.cache.del_ack,
                },
            },
        }
    },
    storeonce: {
        label: "HPE StoreOnce",
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            cpu: {
                label: 'CPU',
                totals: {
                    cpu: {
                        label: 'Total',
                        tabs: {
                            sys: _a.tabs.totals.cpu.sys,
                        }
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                }
            },
            pool: {
                label: 'Pool',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            aggregated: {
                                ..._a.tabs.totals.data.aggregated,
                                label: 'Aggregated',
                            },
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            dedupe_ratio: _a.tabs.totals.data.dedupe_ratio
                        },
                    },
                    replication: {
                        label: 'Replication',
                        tabs: {
                            total: _a.tabs.totals.replication.sum_repl,
                            read: _a.tabs.totals.replication.read_repl,
                            write: _a.tabs.totals.replication.write_repl,
                        },
                    },
                    job: {
                        label: 'Job',
                        tabs: {
                            data_job_read: _a.tabs.totals.job.data_job_read,
                            data_job_write: _a.tabs.totals.job.data_job_write,
                            copy_job_read: _a.tabs.totals.job.copy_job_read,
                            copy_job_write: _a.tabs.totals.job.copy_job_write,
                        },
                    },
                    housekeep: {
                        label: 'Housekeeping',
                        tabs: {
                            housekeep_pend_data: _a.tabs.totals.housekeep.housekeep_pend_data,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_replication: _a.tabs.single.replication.read_replication,
                    write_replication: _a.tabs.single.replication.write_replication,
                    data_job_read: _a.tabs.single.job.data_job_read,
                    data_job_write: _a.tabs.single.job.data_job_write,
                    copy_job_read: _a.tabs.single.job.copy_job_read,
                    copy_job_write: _a.tabs.single.job.copy_job_write,
                    housekeep_pend_data: _a.tabs.single.housekeep.housekeep_pend_data,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    dedupe_ratio: _a.tabs.single.data.dedupe_ratio
                },
            },
            drive: {
                label: "Drive",
                totals: {
                    data: _a.pages.totals.data,
                },
                single: {
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                },
            },
            ethport: {
                label: 'Port (ETH)',
                metrics: {
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    data_rate: _a.metrics.data_rate,
                },
                totals: {
                    data: _a.pages.totals.data,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write
                },
            },
            fcport: {
                label: 'Port (FC)',
                metrics: {
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    data_rate: _a.metrics.data_rate,
                },
                totals: {
                    data: _a.pages.totals.data,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write
                },
            },
            nas: {
                label: 'NAS',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            dedupe_ratio: _a.tabs.totals.data.dedupe_ratio,
                            stream: _a.tabs.totals.data.stream_rate
                        },
                    },
                    replication: {
                        label: 'Replication',
                        tabs: {
                            total: _a.tabs.totals.replication.sum_repl,
                            read: _a.tabs.totals.replication.read_repl,
                            write: _a.tabs.totals.replication.write_repl,
                            stream: _a.tabs.totals.replication.stream_repl_rate
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            user_capacity_used: _a.tabs.totals.capacity.user_capacity_used,
                            disk: _a.tabs.totals.capacity.disk_size,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    dedupe_ratio: _a.tabs.single.data.dedupe_ratio,
                    stream: _a.tabs.single.data.stream_rate,
                    read_replication: _a.tabs.single.replication.read_replication,
                    write_replication: _a.tabs.single.replication.write_replication,
                    stream_read_replication: _a.tabs.single.replication.stream_read_replication,
                    stream_write_replication: _a.tabs.single.replication.stream_write_replication,
                    user_capacity_used: _a.tabs.single.capacity.user_capacity_used,
                    disk: _a.tabs.single.capacity.disk_size,
                },
            },
            vtl: {
                label: 'Virtual Tape Library',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            dedupe_ratio: _a.tabs.totals.data.dedupe_ratio,
                            stream: _a.tabs.totals.data.stream_rate
                        },
                    },
                    replication: {
                        label: 'Replication',
                        tabs: {
                            total: _a.tabs.totals.replication.sum_repl,
                            read: _a.tabs.totals.replication.read_repl,
                            write: _a.tabs.totals.replication.write_repl,
                            stream: _a.tabs.totals.replication.stream_repl_rate
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            user_capacity_used: _a.tabs.totals.capacity.user_capacity_used,
                            disk: _a.tabs.totals.capacity.disk_size,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    dedupe_ratio: _a.tabs.single.data.dedupe_ratio,
                    stream: _a.tabs.single.data.stream_rate,
                    read_replication: _a.tabs.single.replication.read_replication,
                    write_replication: _a.tabs.single.replication.write_replication,
                    stream_read_replication: _a.tabs.single.replication.stream_read_replication,
                    stream_write_replication: _a.tabs.single.replication.stream_write_replication,
                    user_capacity_used: _a.tabs.single.capacity.user_capacity_used,
                    disk: _a.tabs.single.capacity.disk_size,
                },
            },
            cat: {
                label: 'Catalyst',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            dedupe_ratio: _a.tabs.totals.data.dedupe_ratio,
                            stream: _a.tabs.totals.data.stream_rate,
                            logical: _a.tabs.totals.data.logical_data_rate,
                            logical_read: _a.tabs.totals.data.logical_data_read,
                            logical_write: _a.tabs.totals.data.logical_data_write
                        },
                    },
                    job: {
                        label: 'Job',
                        tabs: {
                            read_job: _a.tabs.totals.job.data_job_read,
                            write_job: _a.tabs.totals.job.data_job_write,
                            read_copy_job: _a.tabs.totals.job.copy_job_read,
                            write_copy_job: _a.tabs.totals.job.copy_job_write,
                            stream: _a.tabs.totals.job.stream_job_rate,
                            logical_read_job: _a.tabs.totals.job.logical_data_job_read,
                            logical_write_job: _a.tabs.totals.job.logical_data_job_write,
                            logical_read_copy_job: _a.tabs.totals.job.logical_copy_job_read,
                            logical_write_copy_job: _a.tabs.totals.job.logical_copy_job_write,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            user_capacity_used: _a.tabs.totals.capacity.user_capacity_used,
                            disk: _a.tabs.totals.capacity.disk_size,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    dedupe_ratio: _a.tabs.single.data.dedupe_ratio,
                    stream: _a.tabs.single.data.stream_rate,
                    user_capacity_used: _a.tabs.single.capacity.user_capacity_used,
                    disk: _a.tabs.single.capacity.disk_size,
                    logical_read: _a.tabs.single.data.logical_data_read,
                    logical_write: _a.tabs.single.data.logical_data_write
                },
            },
        },
    },
    gpfs: {
        label: "IBM Spectrum Scale"
    },
    netbackup: {
        label: "NetBackup"
    },
    macrosan: {
        label: "MacroSAN"
    },
    oraclezfs: {
        label: 'Oracle ZFS',
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    resp_t: _a.metrics.resp_t,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            cpu: {
                label: 'CPU',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            cpu: _a.tabs.totals.cpu.sys,
                        },
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                },
            },
            node: {
                label: 'Node',
                metrics: {
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            iscsitotal: _a.tabs.totals.iscsi_resp_t.total,
                            iscsiread: _a.tabs.totals.iscsi_resp_t.read,
                            iscsiwrite: _a.tabs.totals.iscsi_resp_t.write,
                            total: _a.tabs.totals.fc_resp_t.total,
                            read: _a.tabs.totals.fc_resp_t.read,
                            write: _a.tabs.totals.fc_resp_t.write,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                },
            },
            pool: {
                label: 'Pool',
                metrics: {
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                },
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.aggregated,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            used: _a.tabs.totals.capacity.OZFScapacity,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.aggregated,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.total,
                            aggregated: _a.tabs.totals.io.aggregated,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                },
            },
            port: {
                label: 'Port',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                },
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                },
            },
            fs: {
                label: 'FS',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                },
            },
        },
    },
    ceph: {
        label: 'Ceph',
    },
    compellent: {
        label: 'Dell Compellent',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                    unused_capacity_total: _a.metrics.unused_capacity_total,
                    spare_capacity_total: _a.metrics.spare_capacity_total
                },
            },
            node: {
                label: 'Node',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys
                        },
                    },
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                }
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            pending: {
                                ..._a.tabs.totals.io.pending,
                                metrics: [
                                    {
                                        ..._a.metrics.io_pending,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                        }
                    },
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                metrics: [
                                    {
                                        ..._a.metrics.capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.capacity_free,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                    {
                                        ..._a.metrics.unused_capacity_total,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 2,
                                    },
                                    {
                                        ..._a.metrics.spare_capacity_total,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 3,
                                    },
                                ],
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free
                        },
                    },
                    block_size: {
                        label: 'Block size',
                        tabs: {
                            total: _a.tabs.totals.block_size.total
                        },
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    io_pending: _a.tabs.single.io.io_pending,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity_compellent,
                    used: _a.tabs.single.capacity.used_compellent,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: defaultStorageTops,
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            pending: _a.tabs.totals.io.pending
                        },
                    },
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    io_pending: _a.tabs.single.io.io_pending,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            volume_folder: {
                label: 'Volume Folder',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            pending: _a.tabs.totals.io.pending
                        },
                    },
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    io_pending: _a.tabs.single.io.io_pending,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            pending: _a.tabs.totals.io.pending
                        },
                    },
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    io_pending: _a.tabs.single.io.io_pending,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            fcport: {
                label: 'Port (FC)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            sasport: {
                label: 'Port (SAS)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            fcoeport: {
                label: 'Port (FCoE)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            iscsiport: {
                label: 'Port (iSCSI)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                }
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                }
            },
        }
    },
    datacore: {
        label: 'Datacore',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            node: {
                label: 'Node',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    r_cache_hit: _a.tabs.single.cache.read_hit,
                    w_cache_hit: _a.tabs.single.cache.write_hit,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            sum_capacity_virtual: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: {
                    'IO': ['read_io', 'write_io', 'io_rate'],
                    'Data': ['read', 'write', 'data_rate']
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity_total: _a.tabs.totals.capacity.total,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.read_hit,
                    w_cache_hit: _a.tabs.single.cache.write_hit,
                    capacity_total: _a.tabs.single.capacity.capacity_total,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                },
            },
            volume_folder: {
                label: 'Volume Folder',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    r_cache_hit: _a.tabs.single.cache.read_hit,
                    w_cache_hit: _a.tabs.single.cache.write_hit,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            fcport: {
                label: 'Port (FC)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                },
            },
            iscsiport: {
                label: 'Port (iSCSI)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            scsiport: {
                label: 'Port (SCSI)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            loopbackport: {
                label: 'Port (Loopback)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                },
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                },
            },
            host_group: {
                label: 'Host Group',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                },
            },
        },
    },
    datadomain: {
        label: 'EMC Datadomain',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: { phase: _a.tabs.single.clean.total },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            cpu: {
                label: 'CPU',
                totals: {
                    busy: {
                        label: 'Total',
                        tabs: {
                            busy: _a.tabs.totals.busy.total,
                            cpu_system: _a.tabs.totals.cpu.sys
                        }
                    },
                },
                single: {},
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                        }
                    },
                    resp_t: _a.pages.totals.resp_t,
                    replication: {
                        label: 'Replication',
                        tabs: {
                            total: _a.tabs.totals.replication.sum_repl,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    block_size: {
                        label: 'Block size',
                        tabs: {
                            total: _a.tabs.totals.block_size.total
                        },
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    repl_rate: _a.tabs.single.replication.total,
                    read_replication: _a.tabs.single.replication.read_replication,
                    write_replication: _a.tabs.single.replication.write_replication,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    busy: {
                        label: 'Busy',
                        tabs: {
                            total: _a.tabs.totals.busy.sum_busy,
                            aggregated: {
                                ..._a.tabs.totals.busy.total,
                                label: 'Aggregated',
                            },
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cpu_busy: _a.tabs.single.cpu.cpu_busy,
                },
            },
            fcport: {
                label: 'Port (FC)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                }
            },
            ethport: {
                label: 'Port (ETH)',
                totals: {
                    data: _a.pages.totals.data
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                }
            },
            sasport: {
                label: 'Port (SAS)',
                totals: {
                    data: _a.pages.totals.data
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                }
            },
            fs: {
                label: 'Filesystem',
                totals: {
                    ops: {
                        label: 'OPS',
                        tabs: {
                            total: _a.tabs.totals.ops.sum_ops,
                            aggregated: _a.tabs.totals.ops.aggregated,
                            read: _a.tabs.totals.ops.read,
                            write: _a.tabs.totals.ops.write,
                        },
                    },
                    data: _a.pages.totals.data,
                },
                single: {
                    ops_rate: _a.tabs.single.ops.ops_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_ops: _a.tabs.single.ops.read_ops,
                    write_ops: _a.tabs.single.ops.write_ops,
                }
            },
            ddboost: {
                label: 'Ddboost',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data
                        },
                    },
                    net: {
                        label: 'Net',
                        tabs: {
                            total: _a.tabs.totals.net.sum_net
                        },
                    },
                    bound: {
                        label: 'Bound',
                        tabs: {
                            pre_comp: _a.tabs.totals.bound.sum_bound_pre_comp,
                            network: _a.tabs.totals.bound.sum_bound_net,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    net: _a.tabs.single.net.total,
                    net_read: _a.tabs.single.net.read,
                    net_write: _a.tabs.single.net.write,
                    outbound_pre_comp: _a.tabs.single.bound.outbound_pre_comp,
                    inbound_pre_comp: _a.tabs.single.bound.inbound_pre_comp,
                    outbound_net: _a.tabs.single.bound.outbound_net,
                    inbound_net: _a.tabs.single.bound.inbound_net,
                }
            },
            mtree: {
                label: 'Mtree',
                totals: {
                    data: _a.pages.totals.data,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Capacity'
                            },
                            compression: _a.tabs.totals.capacity.total_compression_factor
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    capacity: {
                        ..._a.tabs.single.capacity.capacity_total,
                        label: 'Capacity'
                    },
                    compression: _a.tabs.single.capacity.sum_compression_factor
                },
            },
        },
    },
    ddnintelliflash: {
        label: "DDN Intelliflash",
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
        ],
        subsystems: {
            device: {
                totals: {},
                single: {},
                configuration: {
                    hostname: _a.configurations.hostname,
                    ip: _a.configurations.ip,
                    status: _a.configurations.status,
                    version: _a.configurations.version,
                    os: _a.configurations.os,
                    upServices: _a.configurations.upServices,
                    downServices: _a.configurations.downServices
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free
                },
            },
            node: {
                label: 'Node',
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            sum_capacity_system: _a.tabs.totals.capacity.sum_system_capacity,
                            system_capacity_total: _a.tabs.totals.capacity.system_capacity_total,
                            system_capacity_used: _a.tabs.totals.capacity.system_capacity_used,
                            system_capacity_used_percent: _a.tabs.totals.capacity.system_used_percent,
                            system_capacity_free: _a.tabs.totals.capacity.system_capacity_free,
                            sum_capacity_user: _a.tabs.totals.capacity.sum_user_capacity,
                            user_capacity_total: _a.tabs.totals.capacity.user_capacity_total,
                            user_capacity_used: _a.tabs.totals.capacity.user_capacity_used,
                            user_capacity_used_percent: _a.tabs.totals.capacity.user_used_percent,
                            user_capacity_free: _a.tabs.totals.capacity.user_capacity_free,
                        },
                    },
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            sum_memory: _a.tabs.totals.memory.sum_memory,
                            memory_total: _a.tabs.totals.memory.aggregated,
                            memory_used: _a.tabs.totals.memory.used,
                            memory_free: _a.tabs.totals.memory.free,
                            memory_percent: _a.tabs.totals.memory.memory
                        },
                    },
                },
                single: {
                    sum_memory: _a.tabs.single.memory.sum_memory,
                    memory_used_percent: _a.tabs.single.memory.memory,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    sum_system_capacity: _a.tabs.single.capacity.sum_system_capacity,
                    system_capacity_used_percent: _a.tabs.single.capacity.system_capacity_used_percent,
                    sum_user_capacity: _a.tabs.single.capacity.sum_user_capacity,
                    user_capacity_used_percent: _a.tabs.single.capacity.user_used_percent,
                },
            },
            sfa: {
                label: 'SFA',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            total_b: _a.tabs.totals.io.sum_io_b,
                            aggregated_b: _a.tabs.totals.io.aggregated_b,
                            read_b: _a.tabs.totals.io.read_b,
                            write_b: _a.tabs.totals.io.write_b,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            total_b: _a.tabs.totals.data.sum_data_b,
                            aggregated_b: {
                                ..._a.tabs.totals.data.total_b,
                                label: 'Aggregated Back-end',
                            },
                            read_b: _a.tabs.totals.data.read_b,
                            write_b: _a.tabs.totals.data.write_b,
                        },
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total_b,
                            read: _a.tabs.totals.resp_t.read_b,
                            write: _a.tabs.totals.resp_t.write_b,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    io_rate_b: _a.tabs.single.io.io_rate_b,
                    read_io_b: _a.tabs.single.io.read_io_b,
                    write_io_b: _a.tabs.single.io.write_io_b,
                    data_rate_b: _a.tabs.single.data.data_rate_b,
                    read_b: _a.tabs.single.data.read_b,
                    write_b: _a.tabs.single.data.write_b,
                    resp_t_b: _a.tabs.single.resp_t.resp_t_b,
                    resp_t_r_b: _a.tabs.single.resp_t.resp_t_r_b,
                    resp_t_w_b: _a.tabs.single.resp_t.resp_t_w_b,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
                configuration: {
                    model: _a.configurations.model,
                    bios: _a.configurations.bios,
                    bmc_version: _a.configurations.bmc_version,
                    version: _a.configurations.version,
                    ctla: _a.configurations.ctla,
                    ctlb: _a.configurations.ctlb,
                    e_platform: _a.configurations.e_platform,
                    sfx_capable: _a.configurations.sfx_capable,
                    sfx_enabled: _a.configurations.sfx_enabled
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: {
                    'IO': ['read_io', 'write_io', 'io_rate'],
                    'Data': ['read', 'write', 'data_rate']
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.total,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    capacity_total: _a.tabs.single.capacity.capacity_total,
                },
            },
            drive: {
                label: "Drive",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            exascaler: {
                label: 'EXAScaler',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sum_cpu_system_user,
                            sys: _a.tabs.totals.cpu.sys,
                            user: _a.tabs.totals.cpu.user
                        }
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            sum_memory: _a.tabs.totals.memory.sum_memory,
                            memory_total: _a.tabs.totals.memory.aggregated,
                            memory_used: _a.tabs.totals.memory.used,
                            memory_free: _a.tabs.totals.memory.free,
                            memory_percent: _a.tabs.totals.memory.memory
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    sum_memory: _a.tabs.single.memory.sum_memory,
                    memory_used_percent: _a.tabs.single.memory.memory,
                    cpu: _a.tabs.single.cpu.sum_cpu_system_user,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    cpu_user: _a.tabs.single.cpu.cpu_user,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
                configuration: {
                    ip: _a.configurations.ip,
                    version: _a.configurations.version
                },
            },
            fs: {
                label: 'FS',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            oss: {
                label: 'OSS',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sum_cpu_system_user,
                            sys: _a.tabs.totals.cpu.sys,
                            user: _a.tabs.totals.cpu.user
                        }
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            sum_memory: _a.tabs.totals.memory.sum_memory,
                            memory_total: _a.tabs.totals.memory.aggregated,
                            memory_used: _a.tabs.totals.memory.used,
                            memory_free: _a.tabs.totals.memory.free,
                            memory_percent: _a.tabs.totals.memory.memory
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    sum_memory: _a.tabs.single.memory.sum_memory,
                    memory_used_percent: _a.tabs.single.memory.memory,
                    cpu: _a.tabs.single.cpu.sum_cpu_system_user,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    cpu_user: _a.tabs.single.cpu.cpu_user,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent
                }
            },
            ost: {
                label: 'OST',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent
                }
            },
        },
    },
    dothill: {
        label: 'HPE MSA',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    machine_name: _a.configurations.machine_name,
                    PrimaryOwnerName: _a.configurations.PrimaryOwnerName,
                    PrimaryOwnerContact: _a.configurations.PrimaryOwnerContact,
                    OtherIdentifyingInfo: _a.configurations.OtherIdentifyingInfo,
                    NumberOfDrives: _a.configurations.NumberOfDrives,
                    NumberOfStoragePools: _a.configurations.NumberOfStoragePools,
                    serial: _a.configurations.serial
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            node: {
                label: 'Node',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            read_hit: _a.tabs.totals.cache.read_hit,
                            write_hit: _a.tabs.totals.cache.write_hit,
                            read_hit_io: _a.tabs.totals.cache.read_hit_io,
                            write_hit_io: _a.tabs.totals.cache.write_hit_io
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    r_cache_hit_io: _a.tabs.single.cache.r_cache_hit_io,
                    w_cache_hit_io: _a.tabs.single.cache.w_cache_hit_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            read_hit: _a.tabs.totals.cache.read_hit,
                            write_hit: _a.tabs.totals.cache.write_hit,
                            read_hit_io: _a.tabs.totals.cache.read_hit_io,
                            write_hit_io: _a.tabs.totals.cache.write_hit_io
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    r_cache_hit_io: _a.tabs.single.cache.r_cache_hit_io,
                    w_cache_hit_io: _a.tabs.single.cache.w_cache_hit_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            raid_group: {
                label: 'RAID Group',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: {
                    'IO': ['read_io', 'write_io', 'io_rate'],
                    'Data': ['read', 'write', 'data_rate'],
                    'Cache': ['r_cache_hit', 'w_cache_hit']
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            read_hit: _a.tabs.totals.cache.read_hit,
                            write_hit: _a.tabs.totals.cache.write_hit,
                            read_hit_io: _a.tabs.totals.cache.read_hit_io,
                            write_hit_io: _a.tabs.totals.cache.write_hit_io
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    r_cache_hit_io: _a.tabs.single.cache.r_cache_hit_io,
                    w_cache_hit_io: _a.tabs.single.cache.w_cache_hit_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.total
                        }
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    capacity: {
                        ..._a.tabs.single.capacity.capacity_total,
                        label: 'Capacity Total',
                    }
                },
            },
            fcport: {
                label: 'Port (FC)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            sasport: {
                label: 'Port (SAS)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            iscsiport: {
                label: 'Port (iSCSI)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
        },
    },
    ds5k: {
        label: 'NetApp E-series',
    },
    ds8k: {
        label: 'IBM DS8k',
    },
    eternus: {
        label: 'Fujitsu Eternus',
    },
    falconstor: {
        label: 'FalconStor',
    },
    freenas: {
        label: 'FreeNAS',
    },
    hcp: {
        label: 'HDS HCP',
        heatmap: false,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {
                    sum_cpu: {
                        label: 'CPU',
                        metrics: [
                            {
                                ..._a.metrics.cpu_system,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 0,
                                color: Color_1.Color.pallete.gray,
                            },
                            {
                                ..._a.metrics.cpu_user,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 1,
                                color: Color_1.Color.pallete.yellow,
                            },
                            {
                                ..._a.metrics.cpu_wait,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 2,
                                color: Color_1.Color.pallete.blue,
                            },
                        ],
                    },
                    ..._a.tabs.single.lun,
                    network_rate: _a.tabs.single.network.network_rate,
                    network_rx: _a.tabs.single.network.network_rx,
                    network_tx: _a.tabs.single.network.network_tx,
                    network_rate_b: _a.tabs.single.network.network_rate_b,
                    network_rx_b: _a.tabs.single.network.network_rx_b,
                    network_tx_b: _a.tabs.single.network.network_tx_b,
                    swap_out: _a.tabs.single.swap.swap_out,
                    object_total: _a.tabs.single.object.total,
                    capacity_ingested: _a.tabs.single.capacity.capacity_ingested,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            node: {
                label: "Node",
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            sys: _a.tabs.totals.cpu.sys,
                            user: _a.tabs.totals.cpu.user,
                            wait: _a.tabs.totals.cpu.cpu_wait
                        },
                    },
                    lun: {
                        label: 'LUN',
                        tabs: {
                            sum_data: {
                                label: 'Total',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_read,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.lun_write,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 1,
                                    },
                                ],
                            },
                            aggregated: {
                                label: 'Sum',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_data_rate,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: true,
                                        order: 0,
                                    }
                                ]
                            },
                            read: {
                                label: 'Read',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_read,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: true,
                                        order: 0,
                                    },
                                ],
                            },
                            write: {
                                label: 'Write',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_write,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: true,
                                        order: 0,
                                    },
                                ],
                            },
                            util: {
                                label: 'Utilization',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_utilization,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                    network: {
                        label: 'Network',
                        tabs: {
                            total: _a.tabs.totals.network.sum_network,
                            network_rate: _a.tabs.totals.network.network_rate,
                            rx: _a.tabs.totals.network.network_rx,
                            tx: _a.tabs.totals.network.network_tx,
                            total_b: _a.tabs.totals.network.sum_network_b,
                            network_rate_b: _a.tabs.totals.network.network_rate_b,
                            rx_b: _a.tabs.totals.network.network_rx_b,
                            tx_b: _a.tabs.totals.network.network_tx_b,
                        },
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            swap_out: {
                                label: 'Swap Out',
                                metrics: [
                                    {
                                        ..._a.metrics.swap_out,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    sum_cpu: {
                        label: 'CPU',
                        metrics: [
                            {
                                ..._a.metrics.cpu_system,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 0,
                                color: Color_1.Color.pallete.gray,
                            },
                            {
                                ..._a.metrics.cpu_user,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 1,
                                color: Color_1.Color.pallete.yellow,
                            },
                            {
                                ..._a.metrics.cpu_wait,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 2,
                                color: Color_1.Color.pallete.blue,
                            },
                        ],
                    },
                    ..._a.tabs.single.lun,
                    network_rate: _a.tabs.single.network.network_rate,
                    network_rx: _a.tabs.single.network.network_rx,
                    network_tx: _a.tabs.single.network.network_tx,
                    network_rate_b: _a.tabs.single.network.network_rate_b,
                    network_rx_b: _a.tabs.single.network.network_rx_b,
                    network_tx_b: _a.tabs.single.network.network_tx_b,
                    swap_out: _a.tabs.single.swap.swap_out,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            volume: {
                label: "Volume",
                totals: {
                    lun: {
                        label: 'LUN',
                        tabs: {
                            sum_data: {
                                label: 'Total',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_read,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.lun_write,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 1,
                                    },
                                ],
                            },
                            aggregated: {
                                label: 'Sum',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_data_rate,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: true,
                                        order: 0,
                                    }
                                ]
                            },
                            read: {
                                label: 'Read',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_read,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: true,
                                        order: 0,
                                    },
                                ],
                            },
                            write: {
                                label: 'Write',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_write,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: true,
                                        order: 0,
                                    },
                                ],
                            },
                            util: {
                                label: 'Utilization',
                                metrics: [
                                    {
                                        ..._a.metrics.lun_utilization,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    ..._a.tabs.single.lun,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
                metrics: {}
            },
            tenant: {
                label: 'Tenant',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    object: {
                        label: 'Object',
                        tabs: {
                            total: _a.tabs.totals.object.total
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            ..._a.tabs.totals.size,
                            ingested: _a.tabs.totals.capacity.capacity_ingested,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    object_total: _a.tabs.single.object.total,
                    ..._a.tabs.single.size,
                    capacity_ingested: _a.tabs.single.capacity.capacity_ingested,
                },
            },
            namespace: {
                label: 'Namespace',
                parent: 'tenant',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    object: {
                        label: 'Object',
                        tabs: {
                            total: _a.tabs.totals.object.total
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            ..._a.tabs.totals.size,
                            ingested: _a.tabs.totals.capacity.capacity_ingested,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    object_total: _a.tabs.single.object.total,
                    ..._a.tabs.single.size,
                    capacity_ingested: _a.tabs.single.capacity.capacity_ingested,
                },
            },
        },
    },
    hnas: {
        label: 'HDS HNAS',
    },
    hus: {
        label: 'HDS HUS',
    },
    infinibox: {
        label: 'Infinidat',
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    wwnn: _a.configurations.wwnn,
                    name: _a.configurations.name,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free
                },
            },
            system: {
                label: 'System',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: {
                                label: 'Total',
                                metrics: [
                                    {
                                        ..._a.metrics.io_rate,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                            read: {
                                label: 'Read',
                                metrics: [
                                    {
                                        ..._a.metrics.read_io,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                            write: {
                                label: 'Write',
                                metrics: [
                                    {
                                        ..._a.metrics.write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.items,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                }
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            capacity_virtual: {
                                label: 'Virtual Capacity',
                                metrics: [
                                    {
                                        ..._a.metrics.virtual_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.virtual_capacity_free,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            used_virtual: _a.tabs.totals.capacity.used_virtual,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    used_virtual: _a.tabs.single.capacity.used_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            volume: {
                label: 'Volume',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            port: {
                label: 'Port',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data: _a.tabs.totals.data.total,
                    read: _a.tabs.totals.data.read,
                    write: _a.tabs.totals.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            iscsiport: {
                label: 'Port (iSCSI)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data: _a.tabs.totals.data.total,
                    read: _a.tabs.totals.data.read,
                    write: _a.tabs.totals.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                }
            },
            naspool: {
                label: 'NASpool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                }
            },
            fs: {
                label: 'FS',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
        }
    },
    infortrend: {
        label: "Infortrend",
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.capacity_total_bytes
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    total: _a.tabs.single.capacity.capacity_total_bytes
                }
            },
            volume: {
                label: 'Volume',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.capacity_total_bytes
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    total: _a.tabs.single.capacity.capacity_total_bytes
                }
            },
            port: {
                label: 'Port',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    total: _a.tabs.single.data.data_rate,
                    write: _a.tabs.single.data.write,
                    read: _a.tabs.single.data.read,
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                }
            },
            drive: {
                label: 'Drive',
                totals: {
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w
                }
            },
            logicalDrive: {
                label: 'Logical Drive',
                totals: {
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w
                }
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io
                }
            },
        },
    },
    netapp: {
        label: 'NetApp',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                },
            },
            cluster: {
                label: "Cluster",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {},
            },
            node: {
                label: "Node",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.busy.total
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    cpu: {
                        ..._a.tabs.single.cpu.cpu_busy,
                        label: "CPU Load",
                    },
                }
            },
            pool: {
                label: "Pool",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            svm: {
                label: 'SVM',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            volume: {
                label: 'Volume',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            lun: {
                label: "LUN",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                },
            },
            fcport: {
                label: 'Port (FC)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_read: _a.tabs.single.data.read,
                    data_write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            ethport: {
                label: 'Port (ETH)',
                totals: {
                    data: _a.pages.totals.data,
                    packet: {
                        label: 'Packet',
                        tabs: {
                            packet_rate: _a.tabs.totals.network.packet_rate,
                            read_packet: _a.tabs.totals.network.read_packet,
                            write_packet: _a.tabs.totals.network.write_packet,
                        }
                    },
                    error: {
                        label: 'Error',
                        tabs: {
                            read_error: _a.tabs.totals.network.read_error,
                            write_error: _a.tabs.totals.network.write_error,
                        }
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    packet_rate: _a.tabs.single.network.packet_rate,
                    read_packet: _a.tabs.single.network.read_packet,
                    write_packet: _a.tabs.single.network.write_packet,
                    read_error: _a.tabs.single.network.read_error,
                    write_error: _a.tabs.single.network.write_error,
                }
            },
        }
    },
    nimble: {
        label: 'HPE Nimble',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    array_name: _a.configurations.array_name,
                    extended_model: _a.configurations.extended_model,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            sum_capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        }
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                }
            },
            volume: {
                label: 'Volume',
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        }
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    capacity_total: _a.tabs.single.capacity.total
                }
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                }
            },
            ethport: {
                label: 'Port (ETH)',
                totals: {
                    data: _a.pages.totals.data,
                    packet: {
                        label: 'Packet',
                        tabs: {
                            packet_rate: _a.tabs.totals.network.packet_rate,
                            read_packet: _a.tabs.totals.network.read_packet,
                            write_packet: _a.tabs.totals.network.write_packet,
                        }
                    },
                    error: {
                        label: 'Error',
                        tabs: {
                            read_error: _a.tabs.totals.network.read_error,
                            write_error: _a.tabs.totals.network.write_error,
                        }
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    packet_rate: _a.tabs.single.network.packet_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_packet: _a.tabs.single.network.read_packet,
                    write_packet: _a.tabs.single.network.write_packet,
                    read_error: _a.tabs.single.network.read_error,
                    write_error: _a.tabs.single.network.write_error,
                }
            },
            fcport: {
                label: 'Port (FC)',
                totals: {
                    data: _a.pages.totals.data,
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                }
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                }
            },
        }
    },
    oceanstor: {
        label: 'Huawei OceanStor',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    id: _a.configurations.id,
                    node_name: _a.configurations.node_name,
                    ..._a.defaultConfiguration,
                    health_status: _a.configurations.health_status,
                    running_status: _a.configurations.running_status,
                    capacity_total: _a.configurations.capacity_total,
                    capacity_used: _a.configurations.capacity_used,
                    capacity_free: _a.configurations.capacity_free
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                    virtual_capacity_total: _a.metrics.virtual_capacity_total
                },
            },
            node: {
                label: 'Node',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    io: _a.pages.totals.io,
                    ops: {
                        label: 'OPS',
                        tabs: {
                            nfs3_total: _a.tabs.totals.ops.nfs3_total,
                            nfs3_read: _a.tabs.totals.ops.nfs3_read,
                            nfs3_write: _a.tabs.totals.ops.nfs3_write,
                            nfs41_total: _a.tabs.totals.ops.nfs41_total,
                            nfs41_read: _a.tabs.totals.ops.nfs41_read,
                            nfs41_write: _a.tabs.totals.ops.nfs41_write,
                            smb2_total: {
                                ..._a.tabs.totals.protocol.smb2,
                                label: 'SMB2 total'
                            },
                            smb2_read: _a.tabs.totals.ops.smb2_read,
                            smb2_write: _a.tabs.totals.ops.smb2_write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            nfs3_total: _a.tabs.totals.data.nfs3_data_rate,
                            nfs3_read: _a.tabs.totals.data.nfs3_read,
                            nfs3_write: _a.tabs.totals.data.nfs3_write,
                            nfs41_total: _a.tabs.totals.data.nfs41_data_rate,
                            nfs41_read: _a.tabs.totals.data.nfs41_read,
                            nfs41_write: _a.tabs.totals.data.nfs41_write,
                            smb2_total: _a.tabs.totals.data.smb2_data_rate,
                            smb2_read: _a.tabs.totals.data.smb2_read,
                            smb2_write: _a.tabs.totals.data.smb2_write
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            nfs3_total: _a.tabs.totals.resp_t.nfs3_total,
                            nfs3_read: _a.tabs.totals.resp_t.nfs3_read,
                            nfs3_write: _a.tabs.totals.resp_t.nfs3_write,
                            nfs41_total: _a.tabs.totals.resp_t.nfs41_total,
                            nfs41_read: _a.tabs.totals.resp_t.nfs41_read,
                            nfs41_write: _a.tabs.totals.resp_t.nfs41_write,
                            smb2_total: _a.tabs.totals.resp_t.smb2_total,
                            smb2_read: _a.tabs.totals.resp_t.smb2_read,
                            smb2_write: _a.tabs.totals.resp_t.smb2_write,
                        }
                    },
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                            cache_usage_percent: _a.tabs.totals.cache.cache_usage,
                            r_cache_usage_percent: _a.tabs.totals.cache.r_cache_usage_percent,
                            w_cache_usage_percent: _a.tabs.totals.cache.w_cache_usage_percent,
                        },
                    },
                    queue: {
                        label: 'Queue',
                        tabs: {
                            queue: _a.tabs.totals.queue.queue_length
                        },
                    },
                    temperature: {
                        label: 'Temperature',
                        tabs: {
                            temperature: _a.tabs.totals.temperature.temperature
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    nfs41_ops_rate: _a.tabs.single.ops.nfs41_total,
                    nfs41_read_ops: _a.tabs.single.ops.nfs41_read,
                    nfs41_write_ops: _a.tabs.single.ops.nfs41_write,
                    nfs3_ops_rate: _a.tabs.single.ops.nfs3_total,
                    nfs3_read_ops: _a.tabs.single.ops.nfs3_read,
                    nfs3_write_ops: _a.tabs.single.ops.nfs3_write,
                    smb2_ops_rate: {
                        ..._a.tabs.single.protocol.smb2,
                        label: 'SMB2 OPS total'
                    },
                    smb2_read_ops: _a.tabs.single.ops.smb2_read,
                    smb2_write_ops: _a.tabs.single.ops.smb2_write,
                    nfs41_data: _a.tabs.single.data.nfs41_total,
                    nfs41_read: _a.tabs.single.data.nfs41_read,
                    nfs41_write: _a.tabs.single.data.nfs41_write,
                    nfs3_data: _a.tabs.single.data.nfs3_total,
                    nfs3_read: _a.tabs.single.data.nfs3_read,
                    nfs3_write: _a.tabs.single.data.nfs3_write,
                    smb2_data: _a.tabs.single.data.smb2_total,
                    smb2_read: _a.tabs.single.data.smb2_read,
                    smb2_write: _a.tabs.single.data.smb2_write,
                    nfs41_resp_t: _a.tabs.single.resp_t.nfs41_total,
                    nfs41_resp_t_r: _a.tabs.single.resp_t.nfs41_read,
                    nfs41_resp_t_w: _a.tabs.single.resp_t.nfs41_write,
                    nfs3_resp_t: _a.tabs.single.resp_t.nfs3_total,
                    nfs3_resp_t_r: _a.tabs.single.resp_t.nfs3_read,
                    nfs3_resp_t_w: _a.tabs.single.resp_t.nfs3_write,
                    smb2_resp_t: _a.tabs.single.resp_t.smb2_total,
                    smb2_resp_t_r: _a.tabs.single.resp_t.smb2_read,
                    smb2_resp_t_w: _a.tabs.single.resp_t.smb2_write,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    r_cache_hit_io: _a.tabs.single.cache.r_cache_hit_io,
                    w_cache_hit_io: _a.tabs.single.cache.w_cache_hit_io,
                    cache_usage_percent: _a.tabs.single.cache.cache_usage_percent,
                    r_cache_usage_percent: _a.tabs.single.cache.r_cache_usage_percent,
                    w_cache_usage_percent: _a.tabs.single.cache.w_cache_usage_percent,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    queue: _a.tabs.single.queue.queue_length,
                    temperature: _a.tabs.single.temperature.temperature,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    ops: _a.pages.totals.ops,
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            file_total: _a.tabs.totals.data.file_data_rate,
                            file_read: _a.tabs.totals.data.file_read,
                            file_write: _a.tabs.totals.data.file_write
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            ops_total: _a.tabs.totals.resp_t.ops_total,
                            ops_read: _a.tabs.totals.resp_t.ops_read,
                            ops_write: _a.tabs.totals.resp_t.ops_write
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                            tier0: _a.tabs.totals.capacity.tier0_capacity_total,
                            tier1: _a.tabs.totals.capacity.tier1_capacity_total,
                            tier2: _a.tabs.totals.capacity.tier2_capacity_total,
                            virtual: _a.tabs.totals.capacity.virtual
                        },
                    },
                    compression: {
                        label: 'Compression',
                        tabs: {
                            deduplication: _a.tabs.totals.capacity.dedupe_saving,
                            total: _a.tabs.totals.capacity.sum_compression_capacity,
                            compression: _a.tabs.totals.capacity.compression_saving,
                            data_reduction: _a.tabs.totals.capacity.data_reduction_ratio,
                            replication: _a.tabs.totals.capacity.replication_capacity_total
                        }
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    ops_rate: _a.tabs.single.ops.ops_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    data_file: _a.tabs.single.data.file_total,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    read_ops: _a.tabs.single.ops.read_ops,
                    write_ops: _a.tabs.single.ops.write_ops,
                    file_read: _a.tabs.single.data.file_read,
                    file_write: _a.tabs.single.data.file_write,
                    ops_resp_t: _a.tabs.single.resp_t.ops_total,
                    ops_resp_t_r: _a.tabs.single.resp_t.ops_read,
                    ops_resp_t_w: _a.tabs.single.resp_t.ops_write,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    r_cache_hit_io: _a.tabs.single.cache.r_cache_hit_io,
                    w_cache_hit_io: _a.tabs.single.cache.w_cache_hit_io,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    tier0: _a.tabs.single.capacity.tier0_capacity_total,
                    tier1: _a.tabs.single.capacity.tier1_capacity_total,
                    tier2: _a.tabs.single.capacity.tier2_capacity_total,
                    deduplication: _a.tabs.single.capacity.dedupe_saving,
                    sum_compression: _a.tabs.single.capacity.sum_compression_capacity,
                    compression: _a.tabs.single.capacity.compression_saving,
                    data_reduction_ratio: _a.tabs.single.capacity.data_reduction_ratio,
                    replication: _a.tabs.single.capacity.replication_capacity_total,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: {
                    'IO': ['read_io', 'write_io', 'io_rate'],
                    'Data': ['read', 'write', 'data_rate'],
                    'Latency': ['resp_t', 'resp_t_r', 'resp_t_w'],
                    'Cache': ['r_cache_hit', 'w_cache_hit']
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            read_hit: _a.tabs.totals.cache.read_hit,
                            write_hit: _a.tabs.totals.cache.write_hit,
                            read_hit_io: _a.tabs.totals.cache.read_hit_io,
                            write_hit_io: _a.tabs.totals.cache.write_hit_io
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                    queue: {
                        label: 'Queue',
                        tabs: {
                            queue: _a.tabs.totals.queue.queue_length
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    r_cache_hit_io: _a.tabs.single.cache.r_cache_hit_io,
                    w_cache_hit_io: _a.tabs.single.cache.w_cache_hit_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    queue: _a.tabs.single.queue.queue_length,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    queue: {
                        label: 'Queue',
                        tabs: {
                            queue: _a.tabs.totals.queue.queue_length
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    queue: _a.tabs.single.queue.queue_length,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            fcport: {
                label: 'Port (FC)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            fcoeport: {
                label: 'Port (FCoE)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            ethport: {
                label: 'Port (Ethernet)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    protocol: {
                        label: 'Protocol',
                        tabs: {
                            nfs_total_ops: _a.tabs.totals.protocol.sum_nfs,
                            nfs_rate_ops: {
                                ..._a.tabs.totals.protocol.nfs,
                                label: 'NFS sum ops',
                            },
                            nfs_read_ops: _a.tabs.totals.protocol.nfs_read_ops,
                            nfs_write_ops: _a.tabs.totals.protocol.nfs_write_ops,
                            nfs_other_ops_rate: _a.tabs.totals.protocol.nfs_other_ops_rate,
                            nfs_total_data: _a.tabs.totals.protocol.sum_nfs_data,
                            nfs_rate_data: _a.tabs.totals.protocol.nfs_data,
                            nfs_read: _a.tabs.totals.protocol.nfs_read,
                            nfs_write: _a.tabs.totals.protocol.nfs_write,
                            nfs_resp_t: _a.tabs.totals.protocol.nfs_resp_t,
                            nfs_resp_t_r: _a.tabs.totals.protocol.nfs_resp_t_r,
                            nfs_resp_t_w: _a.tabs.totals.protocol.nfs_resp_t_w,
                            cifs_total_ops: _a.tabs.totals.protocol.sum_cifs,
                            cifs_rate_ops: {
                                ..._a.tabs.totals.protocol.cifs,
                                label: 'CIFS sum ops',
                            },
                            cifs_read_ops: _a.tabs.totals.protocol.cifs_read_ops,
                            cifs_write_ops: _a.tabs.totals.protocol.cifs_write_ops,
                            cifs_other_ops_rate: _a.tabs.totals.protocol.cifs_other_ops_rate,
                            cifs_total_data: _a.tabs.totals.protocol.sum_cifs_data,
                            cifs_rate_data: _a.tabs.totals.protocol.cifs_data,
                            cifs_read: _a.tabs.totals.protocol.cifs_read,
                            cifs_write: _a.tabs.totals.protocol.cifs_write,
                            cifs_resp_t: _a.tabs.totals.protocol.cifs_resp_t,
                            cifs_resp_t_r: _a.tabs.totals.protocol.cifs_resp_t_r,
                            cifs_resp_t_w: _a.tabs.totals.protocol.cifs_resp_t_w,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    nfs_ops: {
                        ..._a.tabs.single.protocol.nfs,
                        label: 'NFS total ops',
                    },
                    nfs_read_ops: _a.tabs.single.protocol.nfs_read_ops,
                    nfs_write_ops: _a.tabs.single.protocol.nfs_write_ops,
                    nfs_other_ops_rate: _a.tabs.single.protocol.nfs_other_ops_rate,
                    nfs_data_rate: _a.tabs.single.protocol.nfs_data_rate,
                    nfs_read: _a.tabs.single.protocol.nfs_read,
                    nfs_write: _a.tabs.single.protocol.nfs_write,
                    nfs_resp_t: _a.tabs.single.protocol.nfs_resp_t,
                    nfs_resp_t_r: _a.tabs.single.protocol.nfs_resp_t_r,
                    nfs_resp_t_w: _a.tabs.single.protocol.nfs_resp_t_w,
                    cifs_ops: _a.tabs.single.protocol.cifs_ops_rate,
                    cifs_read_ops: _a.tabs.single.protocol.cifs_read_ops,
                    cifs_write_ops: _a.tabs.single.protocol.cifs_write_ops,
                    cifs_other_ops_rate: _a.tabs.single.protocol.cifs_other_ops_rate,
                    cifs_data_rate: _a.tabs.single.protocol.cifs_data_rate,
                    cifs_read: _a.tabs.single.protocol.cifs_read,
                    cifs_write: _a.tabs.single.protocol.cifs_write,
                    cifs_resp_t: _a.tabs.single.protocol.cifs_resp_t,
                    cifs_resp_t_r: _a.tabs.single.protocol.cifs_resp_t_r,
                    cifs_resp_t_w: _a.tabs.single.protocol.cifs_resp_t_w,
                },
            },
            sasport: {
                label: 'Port (SAS)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            logport: {
                label: 'Port (LOG)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    protocol: {
                        label: 'Protocol',
                        tabs: {
                            nfs_total_ops: _a.tabs.totals.protocol.sum_nfs,
                            nfs_rate_ops: {
                                ..._a.tabs.totals.protocol.nfs,
                                label: 'NFS sum ops',
                            },
                            nfs_read_ops: _a.tabs.totals.protocol.nfs_read_ops,
                            nfs_write_ops: _a.tabs.totals.protocol.nfs_write_ops,
                            nfs_other_ops_rate: _a.tabs.totals.protocol.nfs_other_ops_rate,
                            nfs_total_data: _a.tabs.totals.protocol.sum_nfs_data,
                            nfs_rate_data: _a.tabs.totals.protocol.nfs_data,
                            nfs_read: _a.tabs.totals.protocol.nfs_read,
                            nfs_write: _a.tabs.totals.protocol.nfs_write,
                            nfs_resp_t: _a.tabs.totals.protocol.nfs_resp_t,
                            nfs_resp_t_r: _a.tabs.totals.protocol.nfs_resp_t_r,
                            nfs_resp_t_w: _a.tabs.totals.protocol.nfs_resp_t_w,
                            cifs_total_ops: _a.tabs.totals.protocol.sum_cifs,
                            cifs_rate_ops: {
                                ..._a.tabs.totals.protocol.cifs,
                                label: 'CIFS sum ops',
                            },
                            cifs_read_ops: _a.tabs.totals.protocol.cifs_read_ops,
                            cifs_write_ops: _a.tabs.totals.protocol.cifs_write_ops,
                            cifs_other_ops_rate: _a.tabs.totals.protocol.cifs_other_ops_rate,
                            cifs_total_data: _a.tabs.totals.protocol.sum_cifs_data,
                            cifs_rate_data: _a.tabs.totals.protocol.cifs_data,
                            cifs_read: _a.tabs.totals.protocol.cifs_read,
                            cifs_write: _a.tabs.totals.protocol.cifs_write,
                            cifs_resp_t: _a.tabs.totals.protocol.cifs_resp_t,
                            cifs_resp_t_r: _a.tabs.totals.protocol.cifs_resp_t_r,
                            cifs_resp_t_w: _a.tabs.totals.protocol.cifs_resp_t_w,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    nfs_ops: {
                        ..._a.tabs.single.protocol.nfs,
                        label: 'NFS total ops',
                    },
                    nfs_read_ops: _a.tabs.single.protocol.nfs_read_ops,
                    nfs_write_ops: _a.tabs.single.protocol.nfs_write_ops,
                    nfs_other_ops_rate: _a.tabs.single.protocol.nfs_other_ops_rate,
                    nfs_data_rate: _a.tabs.single.protocol.nfs_data_rate,
                    nfs_read: _a.tabs.single.protocol.nfs_read,
                    nfs_write: _a.tabs.single.protocol.nfs_write,
                    nfs_resp_t: _a.tabs.single.protocol.nfs_resp_t,
                    nfs_resp_t_r: _a.tabs.single.protocol.nfs_resp_t_r,
                    nfs_resp_t_w: _a.tabs.single.protocol.nfs_resp_t_w,
                    cifs_ops: _a.tabs.single.protocol.cifs_ops_rate,
                    cifs_read_ops: _a.tabs.single.protocol.cifs_read_ops,
                    cifs_write_ops: _a.tabs.single.protocol.cifs_write_ops,
                    cifs_other_ops_rate: _a.tabs.single.protocol.cifs_other_ops_rate,
                    cifs_data_rate: _a.tabs.single.protocol.cifs_data_rate,
                    cifs_read: _a.tabs.single.protocol.cifs_read,
                    cifs_write: _a.tabs.single.protocol.cifs_write,
                    cifs_resp_t: _a.tabs.single.protocol.cifs_resp_t,
                    cifs_resp_t_r: _a.tabs.single.protocol.cifs_resp_t_r,
                    cifs_resp_t_w: _a.tabs.single.protocol.cifs_resp_t_w,
                },
            },
            host: {
                label: 'Host',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                }
            },
            fs: {
                label: 'FS',
                totals: {
                    ops: _a.pages.totals.ops,
                    data: _a.pages.totals.data,
                    resp: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    ops_rate: _a.tabs.single.ops.ops_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_ops: _a.tabs.single.ops.read_ops,
                    write_ops: _a.tabs.single.ops.write_ops,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
        },
    },
    oceanstorpacificnas: {
        label: 'Huawei OceanstorPacificNAS',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    node_name: _a.configurations.node_name
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            cluster: {
                label: 'Cluster',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            io_total_type: _a.tabs.totals.io.sum_total_io_type,
                            read_io: _a.tabs.totals.io.read,
                            io_read_type: _a.tabs.totals.io.sum_read_io_type,
                            write_io: _a.tabs.totals.io.write,
                            io_write_type: _a.tabs.totals.io.sum_write_io_type,
                            hdfs_io_rate: _a.tabs.totals.io.hdfs_io_rate,
                            hdfs_read_io: _a.tabs.totals.io.hdfs_read_io,
                            hdfs_write_io: _a.tabs.totals.io.hdfs_write_io,
                            ops_rate: {
                                ..._a.tabs.totals.ops.total,
                                label: 'OPS total'
                            },
                            ops_read: {
                                ..._a.tabs.totals.ops.read,
                                label: 'OPS read'
                            },
                            ops_write: {
                                ..._a.tabs.totals.ops.write,
                                label: 'OPS write'
                            },
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            data_total_type: _a.tabs.totals.data.sum_total_data_type,
                            read: _a.tabs.totals.data.read,
                            data_read_type: _a.tabs.totals.data.sum_read_data_type,
                            write: _a.tabs.totals.data.write,
                            data_write_type: _a.tabs.totals.data.sum_write_data_type,
                            hdfs_data_rate: _a.tabs.totals.data.hdfs_data_rate,
                            hdfs_read: _a.tabs.totals.data.hdfs_read,
                            hdfs_write: _a.tabs.totals.data.hdfs_write,
                            file_data_rate: _a.tabs.totals.data.file_data_rate,
                            file_read: _a.tabs.totals.data.file_read,
                            file_write: _a.tabs.totals.data.file_write,
                            object_data_rate: _a.tabs.totals.data.object_data_rate,
                            object_read: _a.tabs.totals.data.object_read,
                            object_write: _a.tabs.totals.data.object_write,
                        }
                    },
                    resp: {
                        label: 'Latency',
                        tabs: {
                            nfs_resp_t: _a.tabs.totals.protocol.nfs_resp_t,
                            nfs_resp_t_r: _a.tabs.totals.protocol.nfs_resp_t_r,
                            nfs_resp_t_w: _a.tabs.totals.protocol.nfs_resp_t_w
                        }
                    },
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    cache: {
                        label: 'Cache',
                        tabs: {
                            cache_usage_percent: _a.tabs.totals.cache.cache_usage,
                        },
                    },
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    io_total_type: _a.tabs.single.io.sum_total_io_type,
                    hdfs_io_rate: {
                        ..._a.tabs.single.io.hdfs_io_rate,
                        label: "HDFS IO Total"
                    },
                    ops_rate: {
                        ..._a.tabs.single.ops.ops_rate,
                        label: 'OPS total'
                    },
                    data_rate: _a.tabs.single.data.data_rate,
                    data_total_type: _a.tabs.single.data.sum_total_data_type,
                    hdfs_data_rate: {
                        ..._a.tabs.single.data.hdfs_data_rate,
                        label: "HDFS Data"
                    },
                    file_data_rate: _a.tabs.single.data.file_data_rate,
                    object_data_rate: {
                        ..._a.tabs.single.data.object_data_rate,
                        label: "Object Data"
                    },
                    read: _a.tabs.single.data.read,
                    data_read_type: _a.tabs.single.data.sum_read_data_type,
                    hdfs_read: _a.tabs.single.data.hdfs_read,
                    file_read: _a.tabs.single.data.file_read,
                    object_read: _a.tabs.single.data.object_read,
                    write: _a.tabs.single.data.write,
                    data_write_type: _a.tabs.single.data.sum_write_data_type,
                    hdfs_write: _a.tabs.single.data.hdfs_write,
                    file_write: _a.tabs.single.data.file_write,
                    object_write: _a.tabs.single.data.object_write,
                    read_io: _a.tabs.single.io.read_io,
                    io_read_type: _a.tabs.single.io.sum_read_io_type,
                    hdfs_read_io: {
                        ..._a.tabs.single.io.hdfs_read_io,
                        label: "HDFS Read IO"
                    },
                    ops_read: _a.tabs.single.ops.read_ops,
                    write_io: _a.tabs.single.io.write_io,
                    io_write_type: _a.tabs.single.io.sum_write_io_type,
                    hdfs_write_io: {
                        ..._a.tabs.single.io.hdfs_write_io,
                        label: "HDFS Write IO"
                    },
                    ops_write: _a.tabs.single.ops.write_ops,
                    nfs_resp_t: _a.tabs.single.protocol.nfs_resp_t,
                    nfs_resp_t_r: _a.tabs.single.protocol.nfs_resp_t_r,
                    nfs_resp_t_w: _a.tabs.single.protocol.nfs_resp_t_w,
                    cache_usage_percent: _a.tabs.single.cache.cache_usage_percent,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                },
            },
            node: {
                label: 'Node',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            io_total_object_type: _a.tabs.totals.io.sum_total_io_object_type,
                            read_io: _a.tabs.totals.io.read,
                            io_read_object_type: _a.tabs.totals.io.sum_read_io_object_type,
                            write_io: _a.tabs.totals.io.write,
                            io_write_object_type: _a.tabs.totals.io.sum_write_io_object_type,
                            hdfs_io_rate: _a.tabs.totals.io.hdfs_io_rate,
                            hdfs_read_io: _a.tabs.totals.io.hdfs_read_io,
                            hdfs_write_io: _a.tabs.totals.io.hdfs_write_io,
                            ops_rate: {
                                ..._a.tabs.totals.ops.total,
                                label: 'OPS total'
                            },
                            ops_read: {
                                ..._a.tabs.totals.ops.read,
                                label: 'OPS read'
                            },
                            ops_write: {
                                ..._a.tabs.totals.ops.write,
                                label: 'OPS write'
                            },
                            object_io_rate: _a.tabs.totals.io.object_io_rate,
                            object_read_io: _a.tabs.totals.io.object_read_io,
                            object_write_io: _a.tabs.totals.io.object_write_io,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            data_total_type: _a.tabs.totals.data.sum_total_data_type,
                            read: _a.tabs.totals.data.read,
                            data_read_type: _a.tabs.totals.data.sum_read_data_type,
                            write: _a.tabs.totals.data.write,
                            data_write_type: _a.tabs.totals.data.sum_write_data_type,
                            hdfs_data_rate: _a.tabs.totals.data.hdfs_data_rate,
                            hdfs_read: _a.tabs.totals.data.hdfs_read,
                            hdfs_write: _a.tabs.totals.data.hdfs_write,
                            file_data_rate: _a.tabs.totals.data.file_data_rate,
                            file_read: _a.tabs.totals.data.file_read,
                            file_write: _a.tabs.totals.data.file_write,
                            object_data_rate: _a.tabs.totals.data.object_data_rate,
                            object_read: _a.tabs.totals.data.object_read,
                            object_write: _a.tabs.totals.data.object_write,
                        }
                    },
                    resp: {
                        label: 'Latency',
                        tabs: {
                            nfs_resp_t: _a.tabs.totals.protocol.nfs_resp_t,
                            nfs_resp_t_r: _a.tabs.totals.protocol.nfs_resp_t_r,
                            nfs_resp_t_w: _a.tabs.totals.protocol.nfs_resp_t_w
                        }
                    },
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    cache: {
                        label: 'Cache',
                        tabs: {
                            cache_usage_percent: _a.tabs.totals.cache.cache_usage,
                        },
                    },
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    io_total_object_type: _a.tabs.single.io.sum_total_io_object_type,
                    hdfs_io_rate: {
                        ..._a.tabs.single.io.hdfs_io_rate,
                        label: "HDFS IO Total"
                    },
                    ops_rate: {
                        ..._a.tabs.single.ops.ops_rate,
                        label: 'OPS total'
                    },
                    object_io_rate: {
                        ..._a.tabs.single.io.object_io_rate,
                        label: "Object IO Total"
                    },
                    data_rate: _a.tabs.single.data.data_rate,
                    data_total_type: _a.tabs.single.data.sum_total_data_type,
                    hdfs_data_rate: {
                        ..._a.tabs.single.data.hdfs_data_rate,
                        label: "HDFS Data"
                    },
                    file_data_rate: _a.tabs.single.data.file_data_rate,
                    object_data_rate: {
                        ..._a.tabs.single.data.object_data_rate,
                        label: "Object Data"
                    },
                    read: _a.tabs.single.data.read,
                    data_read_type: _a.tabs.single.data.sum_read_data_type,
                    hdfs_read: _a.tabs.single.data.hdfs_read,
                    file_read: _a.tabs.single.data.file_read,
                    object_read: _a.tabs.single.data.object_read,
                    write: _a.tabs.single.data.write,
                    data_write_type: _a.tabs.single.data.sum_write_data_type,
                    hdfs_write: _a.tabs.single.data.hdfs_write,
                    file_write: _a.tabs.single.data.file_write,
                    object_write: _a.tabs.single.data.object_write,
                    read_io: _a.tabs.single.io.read_io,
                    io_read_object_type: _a.tabs.single.io.sum_read_io_object_type,
                    hdfs_read_io: {
                        ..._a.tabs.single.io.hdfs_read_io,
                        label: "HDFS Read IO"
                    },
                    ops_read: _a.tabs.single.ops.read_ops,
                    object_read_io: {
                        ..._a.tabs.single.io.object_read_io,
                        label: "Object Read IO"
                    },
                    write_io: _a.tabs.single.io.write_io,
                    io_write_object_type: _a.tabs.single.io.sum_write_io_object_type,
                    hdfs_write_io: {
                        ..._a.tabs.single.io.hdfs_write_io,
                        label: "HDFS Write IO"
                    },
                    ops_write: _a.tabs.single.ops.write_ops,
                    object_write_io: {
                        ..._a.tabs.single.io.object_write_io,
                        label: "Object Write IO"
                    },
                    nfs_resp_t: _a.tabs.single.protocol.nfs_resp_t,
                    nfs_resp_t_r: _a.tabs.single.protocol.nfs_resp_t_r,
                    nfs_resp_t_w: _a.tabs.single.protocol.nfs_resp_t_w,
                    cache_usage_percent: _a.tabs.single.cache.cache_usage_percent,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    compression: {
                        label: 'Compression',
                        tabs: {
                            deduplication: _a.tabs.totals.capacity.dedupe_saving,
                            total: _a.tabs.totals.capacity.sum_compression_capacity,
                            compression: _a.tabs.totals.capacity.compression_saving,
                            data_reduction: _a.tabs.totals.capacity.data_reduction_ratio,
                        }
                    },
                },
                single: {
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    deduplication: _a.tabs.single.capacity.dedupe_saving,
                    sum_compression: _a.tabs.single.capacity.sum_compression_capacity,
                    compression: _a.tabs.single.capacity.compression_saving,
                    data_reduction_ratio: _a.tabs.single.capacity.data_reduction_ratio,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            ethport: {
                label: 'Port (Ethernet)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
        },
    },
    oceanstorpacific: {
        label: 'Huawei OceanStorPacific',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    node_name: _a.configurations.node_name
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            node: {
                label: 'Node',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    cache: {
                        label: 'Cache',
                        tabs: {
                            cache_usage_percent: _a.tabs.totals.cache.cache_usage,
                        },
                    },
                },
                single: {
                    cache_usage_percent: _a.tabs.single.cache.cache_usage_percent,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    compression: {
                        label: 'Compression',
                        tabs: {
                            deduplication: _a.tabs.totals.capacity.dedupe_saving,
                            total: _a.tabs.totals.capacity.sum_compression_capacity,
                            compression: _a.tabs.totals.capacity.compression_saving,
                            data_reduction: _a.tabs.totals.capacity.data_reduction_ratio,
                        }
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    deduplication: _a.tabs.single.capacity.dedupe_saving,
                    sum_compression: _a.tabs.single.capacity.sum_compression_capacity,
                    compression: _a.tabs.single.capacity.compression_saving,
                    data_reduction_ratio: _a.tabs.single.capacity.data_reduction_ratio,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            ethport: {
                label: 'Port (Ethernet)',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    block_size: _a.pages.totals.block_size
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w
                }
            },
        },
    },
    oceanstornas: {
        label: 'Huawei OceanStorNAS',
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    id: _a.configurations.id,
                    node_name: _a.configurations.node_name,
                    ..._a.defaultConfiguration,
                    health_status: _a.configurations.health_status,
                    running_status: _a.configurations.running_status,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free
                },
            },
            cluster: {
                label: 'Cluster',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    io: _a.pages.totals.io,
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            fs_total: _a.tabs.totals.protocol.fs_data_rate,
                            fs_read: _a.tabs.totals.protocol.fs_read,
                            fs_write: _a.tabs.totals.protocol.fs_write
                        }
                    },
                    cache: {
                        label: 'Cache',
                        tabs: {
                            cache_usage_percent: _a.tabs.totals.cache.cache_usage,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    fs_data: _a.tabs.single.data.fs_total,
                    fs_read: _a.tabs.single.data.fs_read,
                    fs_write: _a.tabs.single.data.fs_write,
                    nfs_resp_t: _a.tabs.single.protocol.nfs_resp_t,
                    nfs_resp_t_r: _a.tabs.single.protocol.nfs_resp_t_r,
                    nfs_resp_t_w: _a.tabs.single.protocol.nfs_resp_t_w,
                    cifs_resp_t: _a.tabs.single.protocol.cifs_resp_t,
                    cifs_resp_t_r: _a.tabs.single.protocol.cifs_resp_t_r,
                    cifs_resp_t_w: _a.tabs.single.protocol.cifs_resp_t_w,
                    cache_usage_percent: _a.tabs.single.cache.cache_usage_percent,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free
                        },
                    },
                },
                single: {
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent
                },
            },
            node: {
                label: 'Node',
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    io: _a.pages.totals.io,
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            fs_total: _a.tabs.totals.protocol.fs_data_rate,
                            fs_read: _a.tabs.totals.protocol.fs_read,
                            fs_write: _a.tabs.totals.protocol.fs_write
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            nfs_resp_t: _a.tabs.totals.protocol.nfs_resp_t,
                            nfs_resp_t_r: _a.tabs.totals.protocol.nfs_resp_t_r,
                            nfs_resp_t_w: _a.tabs.totals.protocol.nfs_resp_t_w,
                            cifs_resp_t: _a.tabs.totals.protocol.cifs_resp_t,
                            cifs_resp_t_r: _a.tabs.totals.protocol.cifs_resp_t_r,
                            cifs_resp_t_w: _a.tabs.totals.protocol.cifs_resp_t_w,
                        }
                    },
                    cache: {
                        label: 'Cache',
                        tabs: {
                            cache_usage_percent: _a.tabs.totals.cache.cache_usage,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    fs_data: _a.tabs.single.data.fs_total,
                    fs_read: _a.tabs.single.data.fs_read,
                    fs_write: _a.tabs.single.data.fs_write,
                    nfs_resp_t: _a.tabs.single.protocol.nfs_resp_t,
                    nfs_resp_t_r: _a.tabs.single.protocol.nfs_resp_t_r,
                    nfs_resp_t_w: _a.tabs.single.protocol.nfs_resp_t_w,
                    cifs_resp_t: _a.tabs.single.protocol.cifs_resp_t,
                    cifs_resp_t_r: _a.tabs.single.protocol.cifs_resp_t_r,
                    cifs_resp_t_w: _a.tabs.single.protocol.cifs_resp_t_w,
                    cache_usage_percent: _a.tabs.single.cache.cache_usage_percent,
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                },
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                    queue: {
                        label: 'Queue',
                        tabs: {
                            queue: _a.tabs.totals.queue.queue_length
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    queue: _a.tabs.single.queue.queue_length,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                },
            },
            ethport: {
                label: 'Port (Ethernet)',
                totals: {
                    data: _a.pages.totals.data
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write
                },
            },
        },
    },
    pure: {
        label: 'PureStorage',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    name: _a.configurations.node_name,
                    ..._a.defaultConfiguration,
                    apiVersion: _a.configurations.apiVersion
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                    local_write_io: _a.metrics.local_write_io,
                    mirrored_write_io: _a.metrics.mirrored_write_io,
                    local_write: _a.metrics.local_write,
                    mirrored_write: _a.metrics.mirrored_write,
                    local_resp_t_w: _a.metrics.local_resp_t_w,
                    mirrored_resp_t_w: _a.metrics.mirrored_resp_t_w,
                    shared_space_capacity_used: _a.metrics.shared_space_capacity_used,
                    volumes_capacity_used: _a.metrics.volumes_capacity_used,
                    system_capacity_used: _a.metrics.system_capacity_used,
                    virtual_capacity_total: _a.metrics.virtual_capacity_total,
                    snapshot_capacity_used: _a.metrics.snapshot_capacity_used,
                },
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            read: _a.tabs.totals.io.read,
                            write: {
                                ..._a.tabs.totals.io.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.io.local_write,
                            mirrored_write: _a.tabs.totals.io.mirrored_write,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            read: _a.tabs.totals.data.read,
                            write: {
                                ..._a.tabs.totals.data.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.data.local_write,
                            mirrored_write: _a.tabs.totals.data.mirrored_write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            local_write: _a.tabs.totals.resp_t.local_write,
                            mirrored_write: _a.tabs.totals.resp_t.mirrored_write
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            aggregated: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                metrics: [
                                    {
                                        ..._a.metrics.volumes_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.snapshot_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                    {
                                        ..._a.metrics.shared_space_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 2,
                                    },
                                    {
                                        ..._a.metrics.system_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 3,
                                    },
                                ],
                                label: 'Aggregated'
                            },
                            free: _a.tabs.totals.capacity.free,
                            virtual: _a.tabs.totals.capacity.virtual
                        },
                    },
                    block_size: {
                        label: 'Block size',
                        tabs: {
                            total: _a.tabs.totals.block_size.total
                        },
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    local_write: _a.tabs.single.data.local_write,
                    mirrored_write: _a.tabs.single.data.mirrored_write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    local_write_io: _a.tabs.single.io.local_write,
                    mirrored_write_io: _a.tabs.single.io.mirrored_write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    local_resp_t_w: _a.tabs.single.resp_t.local_write,
                    mirrored_resp_t_w: _a.tabs.single.resp_t.mirrored_resp_t_w,
                    capacity: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: defaultStorageTops,
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: {
                                ..._a.tabs.totals.io.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.io.local_write,
                            mirrored_write: _a.tabs.totals.io.mirrored_write,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                            read: _a.tabs.totals.data.read,
                            write: {
                                ..._a.tabs.totals.data.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.data.local_write,
                            mirrored_write: _a.tabs.totals.data.mirrored_write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            local_write: _a.tabs.totals.resp_t.local_write,
                            mirrored_write: _a.tabs.totals.resp_t.mirrored_write
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                metrics: [
                                    {
                                        ..._a.metrics.capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.capacity_free,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                    {
                                        ..._a.metrics.virtual_capacity_total,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 1,
                                    }
                                ],
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                            virtual: _a.tabs.totals.capacity.virtual
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    local_write: _a.tabs.single.data.local_write,
                    mirrored_write: _a.tabs.single.data.mirrored_write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    local_write_io: _a.tabs.single.io.local_write,
                    mirrored_write_io: _a.tabs.single.io.mirrored_write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    local_resp_t_w: _a.tabs.single.resp_t.local_write,
                    mirrored_resp_t_w: _a.tabs.single.resp_t.mirrored_resp_t_w,
                    capacity: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            host: {
                label: 'Host',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: {
                                ..._a.tabs.totals.io.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.io.local_write,
                            mirrored_write: _a.tabs.totals.io.mirrored_write,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                            read: _a.tabs.totals.data.read,
                            write: {
                                ..._a.tabs.totals.data.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.data.local_write,
                            mirrored_write: _a.tabs.totals.data.mirrored_write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            local_write: _a.tabs.totals.resp_t.local_write,
                            mirrored_write: _a.tabs.totals.resp_t.mirrored_write
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    local_write: _a.tabs.single.data.local_write,
                    mirrored_write: _a.tabs.single.data.mirrored_write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    local_write_io: _a.tabs.single.io.local_write,
                    mirrored_write_io: _a.tabs.single.io.mirrored_write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    local_resp_t_w: _a.tabs.single.resp_t.local_write,
                    mirrored_resp_t_w: _a.tabs.single.resp_t.mirrored_resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            host_group: {
                label: 'Host Group',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: {
                                ..._a.tabs.totals.io.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.io.local_write,
                            mirrored_write: _a.tabs.totals.io.mirrored_write,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                            read: _a.tabs.totals.data.read,
                            write: {
                                ..._a.tabs.totals.data.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.data.local_write,
                            mirrored_write: _a.tabs.totals.data.mirrored_write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            local_write: _a.tabs.totals.resp_t.local_write,
                            mirrored_write: _a.tabs.totals.resp_t.mirrored_write
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    local_write: _a.tabs.single.data.local_write,
                    mirrored_write: _a.tabs.single.data.mirrored_write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    local_write_io: _a.tabs.single.io.local_write,
                    mirrored_write_io: _a.tabs.single.io.mirrored_write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    local_resp_t_w: _a.tabs.single.resp_t.local_write,
                    mirrored_resp_t_w: _a.tabs.single.resp_t.mirrored_resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            pod: {
                label: 'Pod',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: {
                                ..._a.tabs.totals.io.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write_io,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.io.local_write,
                            mirrored_write: _a.tabs.totals.io.mirrored_write,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: {
                                ..._a.tabs.totals.data.total,
                                label: 'Aggregated',
                            },
                            read: _a.tabs.totals.data.read,
                            write: {
                                ..._a.tabs.totals.data.write,
                                metrics: [
                                    {
                                        ..._a.metrics.local_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.mirrored_write,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            local_write: _a.tabs.totals.data.local_write,
                            mirrored_write: _a.tabs.totals.data.mirrored_write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            local_write: _a.tabs.totals.resp_t.local_write,
                            mirrored_write: _a.tabs.totals.resp_t.mirrored_write
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                metrics: [
                                    {
                                        ..._a.metrics.capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.capacity_free,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                    {
                                        ..._a.metrics.virtual_capacity_total,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 1,
                                    }
                                ],
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            virtual: _a.tabs.totals.capacity.virtual
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    local_write: _a.tabs.single.data.local_write,
                    mirrored_write: _a.tabs.single.data.mirrored_write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    local_write_io: _a.tabs.single.io.local_write,
                    mirrored_write_io: _a.tabs.single.io.mirrored_write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    local_resp_t_w: _a.tabs.single.resp_t.local_write,
                    mirrored_resp_t_w: _a.tabs.single.resp_t.mirrored_resp_t_w,
                    capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
        }
    },
    pureblade: {
        label: 'PureStorage Blade',
    },
    qnap: {
        label: "QNAP",
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    agent_version: _a.configurations.agent_version,
                    cluster_uuid: _a.configurations.cluster_uuid,
                    nas_uuid: _a.configurations.nas_uuid,
                    ip: _a.configurations.ip,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    resp_t: _a.metrics.resp_t,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            cpu: {
                label: 'System',
                totals: {
                    io: {
                        label: 'Totals',
                        tabs: {
                            cpu: _a.tabs.totals.cpu.sum_cpu,
                            mem: _a.tabs.totals.memory.sum_memory,
                        }
                    },
                },
                single: {
                    cpu: _a.tabs.single.cpu.sum_cpu,
                }
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io_rate,
                            agg: _a.tabs.totals.io.aggregated,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data_rate,
                            agg: _a.tabs.totals.data.aggregated,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            raid_group: {
                label: 'Raid Group',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.aggregated,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                }
            },
            port: {
                label: 'Port',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                },
                single: {
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                }
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.aggregated,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                }
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.aggregated,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
        },
    },
    qumulo: {
        label: 'Qumulo',
    },
    raidix: {
        label: 'RAIDIX',
    },
    solidfire: {
        label: 'NetApp SolidFire',
    },
    storevirtual: {
        label: 'HPE StoreVirtual',
    },
    synology: {
        label: 'Synology',
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    machine_name: _a.configurations.machine_name,
                    system_status: _a.configurations.system_status,
                    temperature: _a.configurations.temperature,
                    power_status: _a.configurations.power_status,
                    system_fan_status: _a.configurations.system_fan_status,
                    cpu_fan_status: _a.configurations.cpu_fan_status
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            system: {
                label: 'System',
                totals: {
                    io: {
                        label: 'Totals',
                        tabs: {
                            cpu: _a.tabs.totals.cpu.sum_cpu_system_user,
                            cpu_gpu_utilization: _a.tabs.totals.cpu.gpu_util_percent,
                            memory_overview: _a.tabs.totals.memory.overview,
                            memory_util_percent: _a.tabs.totals.memory.util_percent,
                            memory_gpu_aggregated: _a.tabs.totals.memory.gpu_aggregated,
                            memory_gpu_uitilization: _a.tabs.totals.memory.gpu_util_percent,
                        }
                    },
                },
                single: {}
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent
                }
            },
            volume: {
                label: 'Volume',
                top: {
                    'IO': ['read_io', 'write_io', 'io_rate'],
                    'Data': ['read', 'write'],
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent
                }
            },
            lun: {
                label: 'LUN',
                totals: {
                    io: _a.pages.totals.io,
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write
                        }
                    },
                    latency: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            read: _a.tabs.totals.resp_t.read,
                            write: _a.tabs.totals.resp_t.write,
                            port_resp_t: _a.tabs.totals.resp_t.port_resp_t,
                            disk_resp_t: _a.tabs.totals.resp_t.disk_resp_t,
                            port_resp_t_r: _a.tabs.totals.resp_t.port_resp_t_r,
                            disk_resp_t_r: _a.tabs.totals.resp_t.disk_resp_t_r,
                            port_resp_t_w: _a.tabs.totals.resp_t.port_resp_t_w,
                            disk_resp_t_w: _a.tabs.totals.resp_t.disk_resp_t_w,
                        }
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    port_resp_t: _a.tabs.single.resp_t.port_resp_t,
                    disk_resp_t: _a.tabs.single.resp_t.disk_resp_t,
                    port_resp_t_r: _a.tabs.single.resp_t.port_resp_t_r,
                    disk_resp_t_r: _a.tabs.single.resp_t.disk_resp_t_r,
                    port_resp_t_w: _a.tabs.single.resp_t.port_resp_t_w,
                    disk_resp_t_w: _a.tabs.single.resp_t.disk_resp_t_w,
                }
            },
            drive: {
                label: 'Drive',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write
                        }
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                }
            },
            port: {
                label: 'Port',
                totals: {
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.total,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                }
            },
        },
    },
    vnx: {
        label: 'EMC VNX',
    },
    vnxe: {
        label: 'EMC VNXe',
    },
    vnxf: {
        label: 'EMC VNXF',
    },
    vplex: {
        label: 'EMX VPLEX',
    },
    vspg: {
        label: 'HDS VSP-G',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    machine_name: _a.configurations.node_name,
                    model: _a.configurations.model,
                    serial: _a.configurations.serial,
                    ipa: _a.configurations.ctla,
                    ipb: _a.configurations.ctlb,
                    version: _a.configurations.version
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                    virtual_capacity_total: _a.metrics.virtual_capacity_total,
                },
            },
            cpu: {
                label: 'CPU',
                totals: {
                    cpu: {
                        label: 'Total',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.cpu.sum_sys,
                                metrics: [
                                    {
                                        ..._a.metrics.cpu_system,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.avg,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                            sys: _a.tabs.totals.cpu.sys,
                        },
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    write_pending: _a.tabs.single.cache.write_pending,
                    cache_usage: _a.tabs.single.cache.cache_usage_percent
                },
            },
            node: {
                label: 'Node',
                totals: {
                    cpu: {
                        label: 'Temperature',
                        tabs: {
                            temperature: _a.tabs.totals.temperature.temperature
                        },
                    },
                },
                single: {
                    temperature: _a.tabs.single.temperature.temperature
                },
            },
            tier: {
                label: 'Tier',
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                            raw_capacity_sum: _a.tabs.totals.capacity.raw_capacity_total_vspg,
                            raw_capacity_used: _a.tabs.totals.capacity.raw_capacity_used
                        }
                    },
                },
                single: {
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    raw_capacity_total: _a.tabs.single.capacity.raw_capacity_total_vspg,
                    raw_capacity_used: _a.tabs.single.capacity.raw_capacity_used
                }
            },
            raid_group: {
                label: 'RAID Group',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            aggregated_b: _a.tabs.totals.io.aggregated_b,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            aggregated_b: _a.tabs.totals.data.aggregated_b,
                        },
                    },
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    io_rate_b: _a.tabs.single.io.io_rate_b,
                    data_rate_b: _a.tabs.single.data.data_rate_b,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            aggregated_b: _a.tabs.totals.io.aggregated_b,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            aggregated_b: _a.tabs.totals.data.aggregated_b,
                        },
                    },
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    io_rate_b: _a.tabs.single.io.io_rate_b,
                    data_rate_b: _a.tabs.single.data.data_rate_b,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    formatted_capacity_total: _a.tabs.single.capacity.formatted_capacity_total_vspg,
                    formatted_capacity_used: {
                        ..._a.tabs.single.capacity.capacity_used,
                        label: 'Formatted used',
                    },
                    fmc_compression_saving: _a.tabs.single.capacity.fmc_compression_saving,
                    fmc_compression_percent: _a.tabs.single.capacity.fmc_compression_percent,
                    dkc_compression_saving: _a.tabs.single.capacity.dkc_compression_saving,
                    dkc_compression_percent: _a.tabs.single.capacity.dkc_compression_percent,
                    dkc_dedupe_saving: _a.tabs.single.capacity.dkc_dedupe_saving,
                    dkc_dedupe_percent: _a.tabs.single.capacity.dkc_dedupe_percent,
                    dkc_zpr_saving: _a.tabs.single.capacity.dkc_zpr_saving,
                    dkc_zpr_percent: _a.tabs.single.capacity.dkc_zpr_percent,
                    dedupe_compression: _a.tabs.single.capacity.dedupe_compression
                }
            },
            volume: {
                label: 'Volume',
                parent: 'pool',
                top: {
                    'IO': ['read_io', 'write_io', 'io_rate'],
                    'Data': ['read', 'write', 'data_rate'],
                    'Latency': ['resp_t', 'resp_t_r', 'resp_t_w'],
                    'Cache': ['r_cache_hit', 'w_cache_hit']
                },
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            aggregated_b: _a.tabs.totals.io.aggregated_b,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                            aggregated_b: _a.tabs.totals.data.aggregated_b,
                        },
                    },
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                            w_cache_hit: _a.tabs.totals.cache.write_hit,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    w_cache_hit: _a.tabs.single.cache.w_cache_hit,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    io_rate_b: _a.tabs.single.io.io_rate_b,
                    data_rate_b: _a.tabs.single.data.data_rate_b,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            port: {
                label: 'Port',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.io.aggregated,
                                label: 'IO',
                            },
                            total_initiator: _a.tabs.totals.io.initiator
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.data.aggregated,
                                label: 'Data',
                            },
                            total_initiator: _a.tabs.totals.data.initiator
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total,
                            total_initiator: _a.tabs.totals.resp_t.initiator
                        }
                    },
                    block_size: {
                        label: 'Block Size',
                        tabs: {
                            total: _a.tabs.totals.block_size.total
                        },
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    block_size: _a.tabs.single.block_size.block_size,
                }
            },
            cache: {
                label: 'Cache',
                totals: {
                    total: {
                        label: 'Total',
                        tabs: {
                            write_pending: _a.tabs.totals.cache.write_pending,
                            cache_usage: _a.tabs.totals.cache.cache_usage
                        },
                    },
                },
                single: {
                    write_pending: _a.tabs.single.cache.write_pending,
                    cache_usage: _a.tabs.single.cache.cache_usage_percent
                },
            },
            host: {
                label: 'Host',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.aggregated
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.aggregated
                        },
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            resp_t: _a.tabs.totals.resp_t.total
                        },
                    },
                    block_size: {
                        label: 'Block Size',
                        tabs: {
                            total: _a.tabs.totals.block_size.total
                        },
                    }
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    block_size: _a.tabs.single.block_size.block_size,
                }
            },
        }
    },
    unity: {
        label: 'EMC Unity',
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    system_name: _a.configurations.system_name,
                    platform_type: _a.configurations.platform_type,
                    system_uuid: _a.configurations.system_uuid,
                    health_state: _a.configurations.health_state,
                    health_details: _a.configurations.health_details,
                    license_activation_key: _a.configurations.license_activation_key,
                },
                metrics: {},
            },
            node: {
                label: "Node",
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            sys: _a.tabs.totals.cpu.sys,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                        },
                    },
                    protocol: {
                        label: 'Protocol',
                        tabs: {
                            nfs_read_ops: _a.tabs.totals.protocol.nfs_read_ops,
                            nfs_write_ops: _a.tabs.totals.protocol.nfs_write_ops,
                            nfs_read: _a.tabs.totals.protocol.nfs_read,
                            nfs_write: _a.tabs.totals.protocol.nfs_write,
                            cifs_read_ops: _a.tabs.totals.protocol.cifs_read_ops,
                            cifs_write_ops: _a.tabs.totals.protocol.cifs_write_ops,
                            cifs_read: _a.tabs.totals.protocol.cifs_read,
                            cifs_write: _a.tabs.totals.protocol.cifs_write,
                        }
                    },
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit_sec: _a.tabs.totals.cache.r_cache_hit_sec,
                            w_cache_hit_sec: _a.tabs.totals.cache.w_cache_hit_sec,
                            r_cache_miss_sec: _a.tabs.totals.cache.r_cache_miss_sec,
                            w_cache_miss_sec: _a.tabs.totals.cache.w_cache_miss_sec,
                        },
                    },
                },
                single: {
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    cpu_system: {
                        ..._a.tabs.single.cpu.cpu_system,
                        label: 'CPU',
                    },
                    nfs_read: _a.tabs.single.protocol.nfs_read,
                    nfs_write: _a.tabs.single.protocol.nfs_write,
                    nfs_read_ops: _a.tabs.totals.protocol.nfs_read_ops,
                    nfs_write_ops: _a.tabs.totals.protocol.nfs_write_ops,
                    cifs_read_ops: _a.tabs.single.protocol.cifs_read_ops,
                    cifs_write_ops: _a.tabs.single.protocol.cifs_write_ops,
                    cifs_read: _a.tabs.single.protocol.cifs_read,
                    cifs_write: _a.tabs.single.protocol.cifs_write,
                    cache_hits_misses_sec: _a.tabs.single.cache.cache_hits_misses_sec,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    cpu_system: _a.metrics.cpu_system,
                    nfs_read: _a.metrics.nfs_read,
                    nfs_write: _a.metrics.nfs_write,
                    nfs_read_ops: _a.metrics.nfs_read_ops,
                    nfs_write_ops: _a.metrics.nfs_write_ops,
                    cifs_read: _a.metrics.cifs_read,
                    cifs_write: _a.metrics.cifs_write,
                    cifs_write_ops: _a.metrics.cifs_write_ops,
                    cifs_read_ops: _a.metrics.cifs_read_ops,
                    w_cache_hit_io: _a.metrics.w_cache_hit_io,
                    r_cache_hit_io: _a.metrics.r_cache_hit_io,
                },
            },
            pool: {
                label: "Pool",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        }
                    },
                    block_size: {
                        label: 'Block Size',
                        tabs: {
                            total: _a.tabs.totals.block_size.total,
                            read: _a.tabs.totals.block_size.read,
                            write: _a.tabs.totals.block_size.write,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            virtual: _a.tabs.totals.capacity.virtual,
                            capacity_oversubscr: _a.tabs.totals.capacity.oversubscr
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    virtual_capacity_total: _a.tabs.single.capacity.virtual_capacity_total,
                    capacity_oversubscr: _a.tabs.single.capacity.oversubscr,
                },
                metrics: {
                    data_rate: _a.metrics.data_rate,
                    io_rate: _a.metrics.io_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    resp_t: _a.metrics.resp_t,
                    block_size: _a.metrics.block_size,
                    block_size_r: _a.metrics.block_size_r,
                    block_size_w: _a.metrics.block_size_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_free: _a.metrics.capacity_free,
                    virtual_capacity_total: _a.metrics.virtual_capacity_total,
                    capacity_oversubscr: _a.metrics.capacity_oversubscr,
                },
            },
            volume: {
                label: "Volume",
                top: {
                    'IO': ['read_io', 'write_io', 'io_rate'],
                    'Data': ['read', 'write', 'data_rate'],
                },
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        },
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        },
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                label: 'Total',
                            },
                            total: {
                                ..._a.tabs.totals.capacity.total,
                                label: 'Aggregated',
                            },
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            free: _a.tabs.totals.capacity.free,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    capacity_total: _a.tabs.single.capacity.capacity_total,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    capacity_free: _a.tabs.single.capacity.free,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    resp_t: _a.metrics.resp_t,
                    block_size: _a.metrics.block_size,
                    block_size_r: _a.metrics.block_size_r,
                    block_size_w: _a.metrics.block_size_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            drive: {
                label: "Drive",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    capacity_total: _a.tabs.single.capacity.capacity_total,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    resp_t: _a.metrics.resp_t,
                    capacity_total: _a.metrics.capacity_total,
                },
            },
            fcport: {
                label: "FC port",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    read_io: _a.metrics.read_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    write_io: _a.metrics.write_io,
                    io_rate: _a.metrics.io_rate,
                },
            },
            ethport: {
                label: "ETH port ",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                    packet: {
                        label: 'Packet',
                        tabs: {
                            packet_rate: _a.tabs.totals.network.packet_rate,
                            read_packet: _a.tabs.totals.network.read_packet,
                            write_packet: _a.tabs.totals.network.write_packet,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    packet_rate: _a.tabs.single.network.packet_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read_packet: _a.tabs.single.network.read_packet,
                    write_packet: _a.tabs.single.network.write_packet,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                    name: _a.configurations.name,
                    id: _a.configurations.id,
                    sp: _a.configurations.sp,
                    mac_address: _a.configurations.mac_address,
                    health_state: _a.configurations.health_state,
                    protocols: _a.configurations.protocols,
                    speed: _a.configurations.speed,
                    mtu_size: _a.configurations.mtu_size,
                },
                metrics: {
                    read_io: _a.metrics.read_io,
                    data_rate: _a.metrics.data_rate,
                    packet_rate: _a.metrics.packet_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    write_io: _a.metrics.write_io,
                    io_rate: _a.metrics.io_rate,
                    read_packet: _a.metrics.read_packet,
                    write_packet: _a.metrics.write_packet,
                },
            },
            iscsifeport: {
                label: "ISCSI port",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    read_io: _a.metrics.read_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    write_io: _a.metrics.write_io,
                    io_rate: _a.metrics.io_rate,
                },
            },
            host: {
                label: "Host",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                    resp_t: {
                        label: 'Latency',
                        tabs: {
                            total: _a.tabs.totals.resp_t.total
                        }
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    resp_t: _a.metrics.resp_t,
                },
            }, naspool: {
                label: "NAS pool",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    capacity_total: _a.tabs.single.capacity.capacity_total,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            nas: {
                label: "NAS",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    capacity_total: _a.tabs.single.capacity.capacity_total,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
            fs: {
                label: "File system",
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            agg: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            aggregated: _a.tabs.totals.data.aggregated,
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        }
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    capacity_total: _a.tabs.single.capacity.capacity_total,
                },
                configuration: {
                    ..._a.defaultConfiguration,
                    id: _a.configurations.id,
                    name: _a.configurations.name,
                    format: _a.configurations.format,
                    protocol: _a.configurations.protocol,
                    health_state: _a.configurations.health_state,
                    access_policy: _a.configurations.access_policy,
                    storage_pool: _a.configurations.storage_pool,
                    storage_pool_id: _a.configurations.storage_pool_id,
                    file_system: _a.configurations.file_system,
                    maximum_size: _a.configurations.maximum_size,
                    protection_size_used: _a.configurations.protection_size_used,
                    capacity_total: _a.configurations.capacity_total,
                    capacity_used: _a.configurations.capacity_use
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                },
            },
        },
    },
    storagegrid: {
        label: 'StorageGrid',
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    apiVersion: _a.configurations.apiVersion,
                    grid_id: _a.configurations.grid_id,
                    grid_name: _a.configurations.grid_name,
                },
                metrics: {
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            site: {
                label: "Site",
                totals: {},
                single: {},
                metrics: {
                    cpu_busy: _a.metrics.cpu_busy,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    metadata_capacity_total: _a.metrics.metadata_capacity_total,
                    metadata_capacity_free: _a.metrics.metadata_capacity_free,
                    metadata_capacity_used: _a.metrics.metadata_capacity_used,
                    user_data_used: _a.metrics.user_data_used,
                    memory: _a.metrics.memory_used_percent,
                    network_rx: _a.metrics.network_rx,
                    network_tx: _a.metrics.network_tx,
                    network_rate: _a.metrics.network_rate,
                    s3_total: _a.metrics.s3_total,
                    s3_ingested: _a.metrics.s3_ingested,
                    s3_retrieved: _a.metrics.s3_retrieved,
                    s3_ops_successful: _a.metrics.s3_ops_successful,
                    s3_ops_failed: _a.metrics.s3_ops_failed,
                    s3_ops_unauthorized: _a.metrics.s3_ops_unauthorized,
                    swift_total: _a.metrics.swift_total,
                    swift_ingested: _a.metrics.swift_ingested,
                    swift_retrieved: _a.metrics.swift_retrieved,
                    swift_ops_successful: _a.metrics.swift_ops_successful,
                    swift_ops_failed: _a.metrics.swift_ops_failed,
                    swift_ops_unauthorized: _a.metrics.swift_ops_unauthorized,
                    ilm_queue_all: _a.metrics.ilm_queue_all,
                    ilm_queue_client: _a.metrics.ilm_queue_client,
                    ilm_queue_background: _a.metrics.ilm_queue_background,
                    ilm_queue_eval_rate: _a.metrics.ilm_queue_eval_rate,
                    ilm_scan_rate: _a.metrics.ilm_scan_rate,
                    lb_total: _a.metrics.lb_total,
                    lb_rx: _a.metrics.lb_rx,
                    lb_tx: _a.metrics.lb_tx,
                    lb_request_rate: _a.metrics.lb_request_rate,
                    lb_error_rate: _a.metrics.lb_error_rate,
                }
            },
            node: {
                label: "Node",
                totals: {},
                single: {},
                metrics: {
                    cpu_system: _a.metrics.cpu_system,
                    cpu_user: _a.metrics.cpu_user,
                    cpu_wait: _a.metrics.cpu_wait,
                    cpu_steal: _a.metrics.cpu_steal,
                    cpu_softirq: _a.metrics.cpu_softirq,
                    cpu_nice: _a.metrics.cpu_nice,
                    cpu_load: _a.metrics.cpu_load,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    metadata_capacity_total: _a.metrics.metadata_capacity_total,
                    metadata_capacity_free: _a.metrics.metadata_capacity_free,
                    metadata_capacity_used: _a.metrics.metadata_capacity_used,
                    user_data_used: _a.metrics.user_data_used,
                    memory_free: _a.metrics.memory_free,
                    memory_used: _a.metrics.memory_used,
                    memory: _a.metrics.memory_used_percent,
                    network_rx: _a.metrics.network_rx,
                    network_tx: _a.metrics.network_tx,
                    network_rate: _a.metrics.network_rate,
                    read_error: _a.metrics.read_error,
                    write_error: _a.metrics.write_error,
                    error_rate: _a.metrics.error_rate,
                    s3_total: _a.metrics.s3_total,
                    s3_ingested: _a.metrics.s3_ingested,
                    s3_retrieved: _a.metrics.s3_retrieved,
                    s3_ops_successful: _a.metrics.s3_ops_successful,
                    s3_ops_failed: _a.metrics.s3_ops_failed,
                    s3_ops_unauthorized: _a.metrics.s3_ops_unauthorized,
                    swift_total: _a.metrics.swift_total,
                    swift_ingested: _a.metrics.swift_ingested,
                    swift_retrieved: _a.metrics.swift_retrieved,
                    swift_ops_successful: _a.metrics.swift_ops_successful,
                    swift_ops_failed: _a.metrics.swift_ops_failed,
                    swift_ops_unauthorized: _a.metrics.swift_ops_unauthorized,
                    ilm_queue_all: _a.metrics.ilm_queue_all,
                    ilm_queue_client: _a.metrics.ilm_queue_client,
                    ilm_queue_background: _a.metrics.ilm_queue_background,
                    ilm_queue_eval_rate: _a.metrics.ilm_queue_eval_rate,
                    ilm_scan_rate: _a.metrics.ilm_scan_rate,
                    lb_total: _a.metrics.lb_total,
                    lb_rx: _a.metrics.lb_rx,
                    lb_tx: _a.metrics.lb_tx,
                    lb_request_rate: _a.metrics.lb_request_rate,
                    lb_error_rate: _a.metrics.lb_error_rate,
                }
            },
            drive: {
                label: "Drive",
                totals: {},
                single: {},
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    data_rate: _a.metrics.data_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                }
            },
            port: {
                label: "Port",
                totals: {},
                single: {},
                metrics: {
                    data_rate: _a.metrics.data_rate,
                    error_rate: _a.metrics.error_rate,
                    drop_rate: _a.metrics.drop_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    read_error: _a.metrics.read_error,
                    write_error: _a.metrics.write_error,
                    read_drop: _a.metrics.read_drop,
                    write_drop: _a.metrics.write_drop,
                }
            },
        }
    },
    powerstore: {
        label: 'EMC PowerStore',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.mapping,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            cluster: {
                label: 'Cluster',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            pool: {
                label: 'Pool',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            sum_capacity_virtual: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            virtual_capacity: {
                                ..._a.tabs.totals.capacity.virtual_capacity,
                                metrics: [
                                    {
                                        ..._a.metrics.virtual_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.virtual_capacity_free,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                                label: 'Virtual'
                            },
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                }
            },
            node: {
                label: 'Node',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            un_read_io: _a.tabs.totals.io.un_read_io,
                            un_write_io: _a.tabs.totals.io.un_write_io,
                        },
                    },
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    protocol: {
                        label: 'Protocol',
                        tabs: {
                            smb_read_io: _a.tabs.totals.protocol.smb_read_io,
                            smb_write_io: _a.tabs.totals.protocol.smb_write_io,
                            smb_read: _a.tabs.totals.protocol.smb_read,
                            smb_write: _a.tabs.totals.protocol.smb_write,
                            smb_resp_t_r: _a.tabs.totals.protocol.smb_resp_t_r,
                            smb_resp_t_w: _a.tabs.totals.protocol.smb_resp_t_w,
                            nfs_read_io: _a.tabs.totals.protocol.nfs_read_io,
                            nfs_write_io: _a.tabs.totals.protocol.nfs_write_io,
                            nfs_read: _a.tabs.totals.protocol.nfs_read,
                            nfs_write: _a.tabs.totals.protocol.nfs_write,
                            nfs_resp_t_r: _a.tabs.totals.protocol.nfs_resp_t_r,
                            nfs_resp_t_w: _a.tabs.totals.protocol.nfs_resp_t_w,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    un_read_io: _a.tabs.single.io.un_read_io,
                    un_write_io: _a.tabs.single.io.un_write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    smb_read_io: _a.tabs.single.protocol.smb_read_io,
                    smb_write_io: _a.tabs.single.protocol.smb_write_io,
                    smb_read: _a.tabs.single.protocol.smb_read,
                    smb_write: _a.tabs.single.protocol.smb_write,
                    smb_resp_t_r: _a.tabs.single.protocol.smb_resp_t_r,
                    smb_resp_t_w: _a.tabs.single.protocol.smb_resp_t_w,
                    nfs_read_io: _a.tabs.single.protocol.nfs_read_io,
                    nfs_write_io: _a.tabs.single.protocol.nfs_write_io,
                    nfs_read: _a.tabs.single.protocol.nfs_read,
                    nfs_write: _a.tabs.single.protocol.nfs_write,
                    nfs_resp_t_r: _a.tabs.single.protocol.nfs_resp_t_r,
                    nfs_resp_t_w: _a.tabs.single.protocol.nfs_resp_t_w,
                }
            },
            volume: {
                label: 'Volume',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity: _a.tabs.totals.capacity.sum_capacity_virtual,
                            used: _a.tabs.totals.capacity.used,
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity_virtual: _a.tabs.single.capacity.sum_capacity_virtual,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                }
            },
            volume_group: {
                label: 'Volume group',
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            fcport: {
                label: 'FC Port',
                totals: {
                    io: {
                        label: 'IO',
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            aggregated: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                            un_read_io: _a.tabs.totals.io.un_read_io,
                            un_write_io: _a.tabs.totals.io.un_write_io,
                        },
                    },
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    error: {
                        label: 'Error',
                        tabs: {
                            loss_sig: _a.tabs.totals.other.loss_sig,
                            loss_sync: _a.tabs.totals.other.loss_sync,
                            link_fail: _a.tabs.totals.other.link_fail,
                            crc: _a.tabs.totals.other.crc,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    loss_sig: _a.tabs.single.other.loss_sig,
                    loss_sync: _a.tabs.single.other.loss_sync,
                    link_fail: _a.tabs.single.other.link_fail,
                    crc: _a.tabs.single.other.crc,
                },
            },
            ethport: {
                label: 'ETH Port',
                totals: {
                    data: _a.pages.totals.data,
                    packet: {
                        label: 'Packet',
                        tabs: {
                            packet_rate: _a.tabs.totals.network.packet_rate,
                            read_packet: _a.tabs.totals.network.read_packet,
                            write_packet: _a.tabs.totals.network.write_packet,
                        },
                    },
                    error: {
                        label: 'Error',
                        tabs: {
                            total: _a.tabs.totals.error.total,
                            buffer_error: _a.tabs.totals.other.buffer_error,
                            crc: _a.tabs.totals.other.crc,
                        },
                    },
                },
                single: {
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    packet_rate: _a.tabs.single.network.packet_rate,
                    read_packet: _a.tabs.single.network.read_packet,
                    write_packet: _a.tabs.single.network.write_packet,
                    error: _a.tabs.single.error.total,
                    buffer_error: _a.tabs.single.error.buffer_error,
                    crc: _a.tabs.single.other.crc,
                }
            },
            host: {
                label: 'Host',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                }
            },
            host_group: {
                label: 'Host Group',
                totals: {},
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                }
            },
        },
    },
    powerflex: {
        label: 'EMC PowerFlex',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.heatmap,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    id: _a.configurations.id,
                    name: _a.configurations.name,
                    clusterMode: _a.configurations.clusterMode,
                    numOfDevices: _a.configurations.numOfDevices,
                    numOfMappedToAllVolumes: _a.configurations.numOfMappedToAllVolumes,
                    numOfProtectionDomains: _a.configurations.numOfProtectionDomains,
                    numOfSdc: _a.configurations.numOfSdc,
                    numOfSds: _a.configurations.numOfSds,
                    numOfSnapshots: _a.configurations.numOfSnapshots,
                    numOfStoragePools: _a.configurations.numOfStoragePools,
                    numOfThickBaseVolumes: _a.configurations.numOfThickBaseVolumes,
                    numOfThinBaseVolumes: _a.configurations.numOfThinBaseVolumes,
                    numOfUnmappedVolumes: _a.configurations.numOfUnmappedVolumes,
                    numOfVolumes: _a.configurations.numOfVolumes,
                    numOfVolumesInDeletion: _a.configurations.numOfVolumesInDeletion,
                    numOfVtrees: _a.configurations.numOfVtrees,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    rebalance_rate: _a.metrics.rebalance_rate,
                    rebuild_rate: _a.metrics.rebuild_rate,
                },
            },
            system: {
                label: "System",
                metrics: {
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    r_cache_hit: _a.metrics.r_cache_hit,
                    rebuild_rate: _a.metrics.rebuild_rate,
                    rebuild_pending: _a.metrics.rebuild_pending,
                    rebalance_rate: _a.metrics.rebalance_rate,
                    rebalance_pending: _a.metrics.rebalance_pending,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    raw_capacity_used: _a.metrics.raw_capacity_used,
                    raw_capacity_free: _a.metrics.raw_capacity_free,
                    replication_in: _a.metrics.replication_in,
                    replication_out: _a.metrics.replication_out,
                    replication_app_resp_t: _a.metrics.replication_app_resp_t,
                    replication_in_resp_t: _a.metrics.replication_in_resp_t,
                    replication_out_resp_t: _a.metrics.replication_out_resp_t,
                    replication_journal_total: _a.metrics.replication_journal_total,
                    replication_journal_used: _a.metrics.replication_journal_used,
                },
                single: {},
                totals: {}
            },
            pool: {
                label: "Pool",
                configuration: {
                    ..._a.defaultPoolConfiguration,
                    mediatype: _a.configurations.mediatype,
                    pd_name: _a.configurations.pd_name,
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            net_used: {
                                ..._a.tabs.totals.capacity.used,
                                label: 'Net used',
                            },
                            raw_capacity_used: {
                                ..._a.tabs.totals.capacity.raw_capacity_used,
                                label: 'Raw used',
                            },
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    block_size: _a.pages.totals.block_size,
                    rebalance: {
                        label: 'Rebalance',
                        tabs: {
                            rebalance_rate: _a.tabs.totals.rebalance.rebalance_rate,
                            rebalance_pending: _a.tabs.totals.rebalance.rebalance_pending
                        },
                    },
                    rebuild: {
                        label: 'Rebuild',
                        tabs: {
                            rebuild_rate: _a.tabs.totals.rebuild.rebuild_rate,
                            rebuild_pending: _a.tabs.totals.rebuild.rebuild_pending,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    rebalance_rate: _a.tabs.single.rebalance.rebalance_rate,
                    rebalance_pending: _a.tabs.single.rebalance.rebalance_pending,
                    rebuild_rate: _a.tabs.single.rebuild.rebuild_rate,
                    rebuild_pending: _a.tabs.single.rebuild.rebuild_pending,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    raw_capacity_used: {
                        ..._a.tabs.single.capacity.raw_capacity_used,
                        label: 'Raw Used',
                    },
                },
            },
            volume: {
                parent: 'pool',
                label: "Volume",
                top: defaultStorageTops,
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    block_size: _a.pages.totals.block_size,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    capacity_total: _a.tabs.single.capacity.total
                },
            },
            drive: {
                label: "Drive",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        },
                    },
                    rebalance: {
                        label: 'Rebalance',
                        tabs: {
                            rebalance_rate: _a.tabs.totals.rebalance.rebalance_rate,
                        },
                    },
                    rebuild: {
                        label: 'Rebuild',
                        tabs: {
                            rebuild_rate: _a.tabs.totals.rebuild.rebuild_rate,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    rebalance_rate: _a.tabs.single.rebalance.rebalance_rate,
                    rebuild_rate: _a.tabs.single.rebuild.rebuild_rate,
                },
            },
            sdc: {
                label: "SDC",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                },
            },
            sds: {
                label: "SDS",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        },
                    },
                    rebalance: {
                        label: 'Rebalance',
                        tabs: {
                            rebalance_rate: _a.tabs.totals.rebalance.rebalance_rate,
                        },
                    },
                    rebuild: {
                        label: 'Rebuild',
                        tabs: {
                            rebuild_rate: _a.tabs.totals.rebuild.rebuild_rate,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    rebalance_rate: _a.tabs.single.rebalance.rebalance_rate,
                    rebuild_rate: _a.tabs.single.rebuild.rebuild_rate,
                },
            },
            protection_domain: {
                label: "Protection Domain",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cache: {
                        label: 'Cache',
                        tabs: {
                            r_cache_hit: _a.tabs.totals.cache.read_hit,
                        },
                    },
                    rebalance: {
                        label: 'Rebalance',
                        tabs: {
                            rebalance_rate: _a.tabs.totals.rebalance.rebalance_rate,
                            rebalance_pending: _a.tabs.totals.rebalance.rebalance_pending
                        },
                    },
                    rebuild: {
                        label: 'Rebuild',
                        tabs: {
                            rebuild_rate: _a.tabs.totals.rebuild.rebuild_rate,
                            rebuild_pending: _a.tabs.totals.rebuild.rebuild_pending,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    r_cache_hit: _a.tabs.single.cache.r_cache_hit,
                    rebalance_rate: _a.tabs.single.rebalance.rebalance_rate,
                    rebalance_pending: _a.tabs.single.rebalance.rebalance_pending,
                    rebuild_rate: _a.tabs.single.rebuild.rebuild_rate,
                    rebuild_pending: _a.tabs.single.rebuild.rebuild_pending,
                },
            },
        }
    },
    rubrik: {
        label: 'Rubrik CDM',
        heatmap: false,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    apiVersion: _a.configurations.apiVersion,
                    memory: _a.configurations.memory,
                    disk_capacity: _a.configurations.disk_capacity,
                    flash_capacity: _a.configurations.flash_capacity,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            pool: {
                label: "Pool",
                configuration: {
                    ..._a.defaultPoolConfiguration,
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    replication: {
                        label: 'Replication',
                        tabs: {
                            total: _a.tabs.totals.replication.total,
                            read: _a.tabs.totals.replication.replication_in,
                            write: _a.tabs.totals.replication.replication_out,
                        },
                    },
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            total: _a.tabs.totals.capacity.sum_capacity,
                            used: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                metrics: [
                                    {
                                        ..._a.metrics.system_capacity_used,
                                        label: 'System',
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.cdp_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                    {
                                        ..._a.metrics.livemount_capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 2,
                                    },
                                    {
                                        ..._a.metrics.snapshot_capacity_used,
                                        label: 'Snapshot',
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 3,
                                    },
                                    {
                                        ..._a.metrics.snapshot_capacity_pending,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 4,
                                    },
                                ],
                                label: 'Used'
                            },
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            snap_ingest: {
                                label: 'Snapshot Ingest',
                                metrics: [
                                    {
                                        ..._a.metrics.snapshot_capacity_physical,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        stacked: false,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.snapshot_capacity_ingested,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        stacked: false,
                                        order: 1,
                                    },
                                ],
                            },
                            data_reduction: {
                                label: 'Data Reduction',
                                metrics: [
                                    {
                                        ..._a.metrics.snapshot_capacity_reduction_percent,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                            cloud: {
                                label: 'Cloud',
                                metrics: [
                                    {
                                        ..._a.metrics.cloud_physical,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        stacked: false,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.cloud_ingested,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        stacked: false,
                                        order: 1,
                                    },
                                ],
                            },
                        }
                    },
                    block_size: _a.pages.totals.block_size,
                    archive: {
                        label: 'Archive',
                        tabs: {
                            archive: {
                                label: 'Archive',
                                metrics: [
                                    {
                                        ..._a.metrics.archive,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                    ingest: {
                        label: 'Ingest',
                        tabs: {
                            phys_ingest: {
                                label: 'Physical Ingest',
                                metrics: [
                                    {
                                        ..._a.metrics.physical_ingest,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                },
                single: {
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    io_rate: _a.tabs.single.io.io_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    data_rate: _a.tabs.single.data.data_rate,
                    block_size: _a.tabs.single.block_size.block_size,
                    block_size_r: _a.tabs.single.block_size.block_size_r,
                    block_size_w: _a.tabs.single.block_size.block_size_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                }
            },
            node: {
                label: "Node",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    cpu: {
                        label: 'Cpu',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    network: {
                        label: 'Network',
                        tabs: {
                            network_rate: _a.tabs.totals.network.network_rate,
                            rx: _a.tabs.totals.network.network_rx,
                            tx: _a.tabs.totals.network.network_tx,
                        },
                    },
                },
                single: {
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    sys: {
                        ..._a.tabs.single.cpu.cpu_system,
                        label: 'CPU',
                    },
                    network_rx: _a.tabs.single.network.network_rx,
                    network_tx: _a.tabs.single.network.network_tx,
                },
            }
        }
    },
    cohesity: {
        label: 'Cohesity',
        heatmap: true,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.healthStatus,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
        ],
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    name: _a.configurations.name,
                    model: _a.configurations.model,
                    version: _a.configurations.version,
                    node_count: _a.configurations.node_count,
                    metadata_available: _a.configurations.metadata_available,
                    metadata_used: _a.configurations.metadata_used,
                },
                metrics: {
                    io_rate: _a.metrics.io_rate,
                    read_io: _a.metrics.read_io,
                    write_io: _a.metrics.write_io,
                    data_rate: _a.metrics.data_rate,
                    read: _a.metrics.read,
                    write: _a.metrics.write,
                    resp_t: _a.metrics.resp_t,
                    resp_t_r: _a.metrics.resp_t_r,
                    resp_t_w: _a.metrics.resp_t_w,
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                    capacity_data_in: _a.metrics.capacity_data_in,
                    capacity_data_written: _a.metrics.capacity_data_written,
                    capacity_data_reduction: _a.metrics.capacity_data_reduction,
                    capacity_logical_usage: _a.metrics.capacity_logical_usage,
                    capacity_storage_consumed: _a.metrics.capacity_storage_consumed,
                },
            },
            cluster: {
                label: "Cluster",
                configuration: {
                    name: _a.configurations.name,
                    model: _a.configurations.model,
                    version: _a.configurations.version,
                    node_count: _a.configurations.node_count,
                    metadata_available: _a.configurations.metadata_available,
                    metadata_used: _a.configurations.metadata_used,
                },
                single: {},
                totals: {
                    capacity: {
                        label: 'Capacity',
                        single: true,
                        tabs: {
                            capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                                metrics: [
                                    {
                                        ..._a.metrics.capacity_used,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.capacity_free,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: true,
                                        order: 1,
                                    },
                                ],
                            },
                            capacity_used_percent: _a.tabs.totals.capacity.used_percent,
                            capacity_data_in: _a.tabs.totals.capacity.capacity_data_in,
                            capacity_data_written: _a.tabs.totals.capacity.capacity_data_written,
                            capacity_data_reduction: _a.tabs.totals.capacity.capacity_data_reduction,
                            capacity_logical_usage: _a.tabs.totals.capacity.capacity_logical_usage,
                        },
                    },
                    io: {
                        label: 'IO',
                        single: true,
                        tabs: {
                            total: _a.tabs.totals.io.sum_io,
                            io_rate: _a.tabs.totals.io.aggregated,
                            read: _a.tabs.totals.io.read,
                            write: _a.tabs.totals.io.write,
                        }
                    },
                    data: {
                        label: 'Data',
                        single: true,
                        tabs: {
                            total: _a.tabs.totals.data.sum_data,
                            data_rate: {
                                ..._a.tabs.totals.data.aggregated,
                                label: 'Data Rate',
                            },
                            read: _a.tabs.totals.data.read,
                            write: _a.tabs.totals.data.write,
                        },
                    },
                    resp_t: {
                        label: 'Latency',
                        single: true,
                        tabs: {
                            resp_t: _a.tabs.totals.resp_t.total,
                            resp_t_r: _a.tabs.totals.resp_t.read,
                            resp_t_w: _a.tabs.totals.resp_t.write,
                        },
                    },
                }
            },
            node: {
                label: "Node",
                configuration: {
                    sw_version: _a.configurations.sw_version,
                    node_type: _a.configurations.node_type,
                    disk_count: _a.configurations.disk_count,
                    removal_state: _a.configurations.removal_state,
                    capacity_node_total: _a.configurations.capacity_node_total,
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: _a.tabs.totals.cpu.sys,
                        },
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            total: _a.tabs.totals.memory.memory,
                        },
                    },
                },
                single: {
                    cpu_system: _a.tabs.single.cpu.cpu_system,
                    memory: _a.tabs.single.memory.memory,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    io_rate: _a.tabs.single.io.io_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    data_rate: _a.tabs.single.data.data_rate,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                }
            },
            pool: {
                label: "Pool",
                configuration: {
                    capacity_used: _a.configurations.capacity_used,
                    capacity_storage_consumed: _a.configurations.capacity_storage_consumed,
                    capacity_data_in: _a.configurations.capacity_data_in,
                    capacity_data_written: _a.configurations.capacity_data_written,
                    capacity_physical_usage: _a.configurations.capacity_physical_usage,
                    capacity_logical_usage: _a.configurations.capacity_logical_usage,
                },
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            sum_capacity: _a.tabs.totals.capacity.sum_capacity,
                            capacity_storage_consumed: _a.tabs.totals.capacity.capacity_storage_consumed,
                            capacity_data_in: _a.tabs.totals.capacity.capacity_data_in,
                            capacity_data_written: _a.tabs.totals.capacity.capacity_data_written,
                            capacity_data_reduction: _a.tabs.totals.capacity.capacity_data_reduction,
                            capacity_logical_usage: _a.tabs.totals.capacity.capacity_logical_usage,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_storage_consumed: _a.tabs.single.capacity.capacity_storage_consumed,
                    capacity_data_in: _a.tabs.single.capacity.capacity_data_in,
                    capacity_data_written: _a.tabs.single.capacity.capacity_data_written,
                    capacity_data_reduction: _a.tabs.single.capacity.capacity_data_reduction,
                    capacity_logical_usage: _a.tabs.single.capacity.capacity_logical_usage,
                },
            },
            drive: {
                label: "Drive",
                totals: {
                    io: _a.pages.totals.io,
                    data: _a.pages.totals.data,
                    resp_t: _a.pages.totals.resp_t,
                    cpu: {
                        label: 'Utilization',
                        tabs: {
                            total: _a.tabs.totals.cpu.utilization,
                        },
                    },
                },
                single: {
                    io_rate: _a.tabs.single.io.io_rate,
                    read_io: _a.tabs.single.io.read_io,
                    write_io: _a.tabs.single.io.write_io,
                    data_rate: _a.tabs.single.data.data_rate,
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                    resp_t: _a.tabs.single.resp_t.resp_t,
                    resp_t_r: _a.tabs.single.resp_t.resp_t_r,
                    resp_t_w: _a.tabs.single.resp_t.resp_t_w,
                    utilization: _a.tabs.single.cpu.utilization,
                },
            },
            port: {
                label: "Port",
                totals: {
                    data: _a.pages.totals.data,
                    error: {
                        label: 'Error',
                        tabs: {
                            read_error: _a.tabs.totals.network.read_error,
                            write_error: _a.tabs.totals.network.write_error,
                        },
                    },
                    packet: {
                        label: 'Packet',
                        tabs: {
                            packet_rate: _a.tabs.totals.network.packet_rate,
                            read_packet: _a.tabs.totals.network.read_packet,
                            write_packet: _a.tabs.totals.network.write_packet,
                        },
                    },
                },
                single: {
                    read_io: _a.tabs.single.data.read,
                    write_io: _a.tabs.single.data.write,
                    read_packet: _a.tabs.single.network.read_packet,
                    write_packet: _a.tabs.single.network.write_packet,
                    read_error: _a.tabs.single.network.read_error,
                    write_error: _a.tabs.single.network.write_error,
                },
            }
        }
    },
    ibmcoss: {
        heatmap: false,
        pages: [
            Stor_1.StoragePages.totals,
            Stor_1.StoragePages.configuration,
            Stor_1.StoragePages.capacity,
            Stor_1.StoragePages.healthStatus
        ],
        label: 'IBM Cloud Object Storage System',
        subsystems: {
            device: {
                label: "Device",
                totals: {},
                single: {},
                configuration: {
                    ..._a.defaultConfiguration,
                    hostname: _a.configurations.hostname,
                    ip: _a.configurations.ip,
                    node: _a.configurations.node,
                },
                metrics: {
                    capacity_total: _a.metrics.capacity_total,
                    capacity_used: _a.metrics.capacity_used,
                    capacity_free: _a.metrics.capacity_free,
                    capacity_used_percent: _a.metrics.capacity_used_percent,
                },
            },
            pool: {
                label: "Pool",
                configuration: {
                    ..._a.defaultPoolConfiguration,
                },
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            sum_capacity: {
                                ..._a.tabs.totals.capacity.sum_capacity,
                            },
                            used: {
                                ..._a.tabs.totals.capacity.used,
                            },
                            used_percent: _a.tabs.totals.capacity.used_percent,
                            slices: {
                                label: 'Slices',
                                metrics: [
                                    {
                                        ..._a.metrics.slices,
                                        type: pageResponse_constants_1.CHART_TYPE.area,
                                        stacked: true,
                                        order: 0,
                                    },
                                ],
                            },
                        },
                    },
                    network: {
                        label: 'Network',
                        tabs: {
                            total: {
                                label: 'Total',
                                metrics: [
                                    {
                                        ..._a.metrics.network_rx,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 0,
                                    },
                                    {
                                        ..._a.metrics.network_tx,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        aggregate: pageResponse_constants_1.METRIC_AGGREGATE.sum,
                                        stacked: false,
                                        order: 1,
                                    },
                                ],
                            },
                            network_rate: _a.tabs.totals.network.network_rate,
                            rx: _a.tabs.totals.network.network_rx,
                            tx: _a.tabs.totals.network.network_tx,
                        },
                    },
                    rebuild: {
                        label: 'Rebuild',
                        tabs: {
                            rebuild_queue: _a.tabs.totals.rebuild.rebuild_queue,
                            rebuild_writes_total: _a.tabs.totals.rebuild.rebuild_writes_total,
                            rebuild_writes_in: _a.tabs.totals.rebuild.rebuild_writes_in,
                            rebuild_writes_out: _a.tabs.totals.rebuild.rebuild_writes_out,
                            rebuild_deletes_total: _a.tabs.totals.rebuild.rebuild_deletes_total,
                            rebuild_deletes_in: _a.tabs.totals.rebuild.rebuild_deletes_in,
                            rebuild_deletes_out: _a.tabs.totals.rebuild.rebuild_deletes_out,
                            rebuild_bytes_total: _a.tabs.totals.rebuild.rebuild_bytes_total,
                            rebuild_bytes_in: _a.tabs.totals.rebuild.rebuild_bytes_in,
                            rebuild_bytes_out: _a.tabs.totals.rebuild.rebuild_bytes_out,
                        },
                    },
                    acknowledge_time: {
                        label: 'Acknowledge Time',
                        tabs: {
                            acknowledge_time: _a.tabs.totals.resp_t.acknowledge_time,
                        },
                    },
                    scanned_srcs: {
                        label: 'Scanned Sources',
                        tabs: {
                            scanned_srcs: _a.tabs.totals.data.scanned_srcs,
                        },
                    },
                },
                single: {
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    capacity_used_percent: _a.tabs.single.capacity.capacity_used_percent,
                    slices: {
                        label: 'Slices',
                        metrics: [
                            {
                                ..._a.metrics.slices,
                                type: pageResponse_constants_1.CHART_TYPE.line,
                                stacked: false,
                                order: 0,
                            },
                        ],
                    },
                    network_rate: _a.tabs.single.network.network_rate,
                    network_rx: _a.tabs.single.network.network_rx,
                    network_tx: _a.tabs.single.network.network_tx,
                    rebuild_queue: _a.tabs.single.rebuild.rebuild_queue,
                    rebuild_writes_total: _a.tabs.single.rebuild.rebuild_writes_total,
                    rebuild_writes_in: _a.tabs.single.rebuild.rebuild_writes_in,
                    rebuild_writes_out: _a.tabs.single.rebuild.rebuild_writes_out,
                    rebuild_deletes_total: _a.tabs.single.rebuild.rebuild_deletes_total,
                    rebuild_deletes_in: _a.tabs.single.rebuild.rebuild_deletes_in,
                    rebuild_deletes_out: _a.tabs.single.rebuild.rebuild_deletes_out,
                    rebuild_bytes_total: _a.tabs.single.rebuild.rebuild_bytes_total,
                    rebuild_bytes_in: _a.tabs.single.rebuild.rebuild_bytes_in,
                    rebuild_bytes_out: _a.tabs.single.rebuild.rebuild_bytes_out,
                    acknowledge_time: _a.tabs.single.resp_t.acknowledge_time,
                    scanned_srcs: _a.tabs.single.data.scanned_srcs,
                },
            },
            drive: {
                label: "Drive",
                totals: {
                    data: _a.pages.totals.data,
                },
                single: {
                    read: _a.tabs.single.data.read,
                    write: _a.tabs.single.data.write,
                },
            },
            manager: {
                label: "Manager",
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.busy.total,
                                label: 'CPU',
                            },
                        },
                    },
                    network: {
                        label: 'Network',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.network.network_rate,
                                label: 'Total',
                            },
                            rx: _a.tabs.totals.network.network_rx,
                            tx: _a.tabs.totals.network.network_tx,
                            tcp_timeouts: _a.tabs.totals.network.tcp_timeouts,
                        },
                    },
                    os_disk: {
                        label: 'OS disk',
                        tabs: {
                            os_total: _a.tabs.totals.os.os_total,
                            os_r: _a.tabs.totals.os.os_r,
                            os_w: _a.tabs.totals.os.os_w,
                        },
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            system_memory: _a.tabs.totals.memory.system_memory,
                            onheap_memory: _a.tabs.totals.memory.onheap_memory,
                            offheap_memory: _a.tabs.totals.memory.offheap_memory,
                        },
                    },
                },
                single: {
                    cpu: {
                        ..._a.tabs.single.cpu.cpu_busy,
                        label: 'CPU Total',
                    },
                    cpu_coss: {
                        label: 'CPU',
                        metrics: [
                            {
                                ..._a.metrics.cpu_system,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 0,
                            },
                            {
                                ..._a.metrics.cpu_user,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 1,
                            },
                            {
                                ..._a.metrics.cpu_wait,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 2,
                            },
                        ],
                    },
                    network_rate: _a.tabs.single.network.network_rate,
                    rx: _a.tabs.single.network.network_rx,
                    tx: _a.tabs.single.network.network_tx,
                    tcp_timeouts: _a.tabs.single.network.tcp_timeouts,
                    os_total: _a.tabs.single.os.os_total,
                    os_r: _a.tabs.single.os.os_r,
                    os_w: _a.tabs.single.os.os_w,
                    sum_memory_system: _a.tabs.single.memory.sum_memory_system,
                    sum_memory_onheap: _a.tabs.single.memory.sum_memory_onheap,
                    sum_memory_offheap: _a.tabs.single.memory.sum_memory_offheap,
                },
            },
            accesser: {
                label: "Accesser",
                totals: {
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.busy.total,
                                label: 'CPU',
                            },
                        },
                    },
                    network: {
                        label: 'Network',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.network.network_rate,
                                label: 'Total',
                            },
                            rx: _a.tabs.totals.network.network_rx,
                            tx: _a.tabs.totals.network.network_tx,
                            tcp_timeouts: _a.tabs.totals.network.tcp_timeouts,
                        },
                    },
                    os_disk: {
                        label: 'OS Disk',
                        tabs: {
                            os_total: _a.tabs.totals.os.os_total,
                            os_r: _a.tabs.totals.os.os_r,
                            os_w: _a.tabs.totals.os.os_w,
                        },
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            system_memory: _a.tabs.totals.memory.system_memory,
                            onheap_memory: _a.tabs.totals.memory.onheap_memory,
                            offheap_memory: _a.tabs.totals.memory.offheap_memory,
                        },
                    },
                    acknowledge_time: {
                        label: 'Acknowledge Time',
                        tabs: {
                            acknowledge_time: _a.tabs.totals.resp_t.acknowledge_time,
                        },
                    },
                },
                single: {
                    cpu: {
                        ..._a.tabs.single.cpu.cpu_busy,
                        label: 'CPU Total',
                    },
                    cpu_coss: {
                        label: 'CPU',
                        metrics: [
                            {
                                ..._a.metrics.cpu_system,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 0,
                            },
                            {
                                ..._a.metrics.cpu_user,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 1,
                            },
                            {
                                ..._a.metrics.cpu_wait,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 2,
                            },
                        ],
                    },
                    network_rate: _a.tabs.single.network.network_rate,
                    rx: _a.tabs.single.network.network_rx,
                    tx: _a.tabs.single.network.network_tx,
                    tcp_timeouts: _a.tabs.single.network.tcp_timeouts,
                    os_total: _a.tabs.single.os.os_total,
                    os_r: _a.tabs.single.os.os_r,
                    os_w: _a.tabs.single.os.os_w,
                    sum_memory_system: _a.tabs.single.memory.sum_memory_system,
                    sum_memory_onheap: _a.tabs.single.memory.sum_memory_onheap,
                    sum_memory_offheap: _a.tabs.single.memory.sum_memory_offheap,
                    acknowledge_time: _a.tabs.single.resp_t.acknowledge_time,
                },
            },
            slicestor: {
                label: "Slicestor",
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            used: {
                                label: 'Allocated',
                                ..._a.tabs.totals.capacity.used,
                            },
                            used_percent: _a.tabs.totals.capacity.used_percent,
                        },
                    },
                    cpu: {
                        label: 'CPU',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.busy.total,
                                label: 'CPU',
                            },
                        },
                    },
                    network: {
                        label: 'Network',
                        tabs: {
                            total: {
                                ..._a.tabs.totals.network.network_rate,
                                label: 'Total',
                            },
                            rx: _a.tabs.totals.network.network_rx,
                            tx: _a.tabs.totals.network.network_tx,
                            tcp_timeouts: _a.tabs.totals.network.tcp_timeouts,
                        },
                    },
                    os_disk: {
                        label: 'OS disk',
                        tabs: {
                            os_total: _a.tabs.totals.os.os_total,
                            os_r: _a.tabs.totals.os.os_r,
                            os_w: _a.tabs.totals.os.os_w,
                        },
                    },
                    memory: {
                        label: 'Memory',
                        tabs: {
                            system_memory: _a.tabs.totals.memory.system_memory,
                            onheap_memory: _a.tabs.totals.memory.onheap_memory,
                            offheap_memory: _a.tabs.totals.memory.offheap_memory,
                        },
                    },
                    rebuild: {
                        label: 'Rebuild',
                        tabs: {
                            rebuild_writes_total: _a.tabs.totals.rebuild.rebuild_writes_total,
                            rebuild_writes_in: _a.tabs.totals.rebuild.rebuild_writes_in,
                            rebuild_writes_out: _a.tabs.totals.rebuild.rebuild_writes_out,
                            rebuild_deletes_total: _a.tabs.totals.rebuild.rebuild_deletes_total,
                            rebuild_deletes_in: _a.tabs.totals.rebuild.rebuild_deletes_in,
                            rebuild_deletes_out: _a.tabs.totals.rebuild.rebuild_deletes_out,
                            rebuild_bytes_total: _a.tabs.totals.rebuild.rebuild_bytes_total,
                            rebuild_bytes_in: _a.tabs.totals.rebuild.rebuild_bytes_in,
                            rebuild_bytes_out: _a.tabs.totals.rebuild.rebuild_bytes_out,
                        },
                    },
                    scanned_srcs: {
                        label: 'Scanned Sources',
                        tabs: {
                            scanned_srcs: {
                                label: 'Scanned Sources',
                                metrics: [
                                    {
                                        ..._a.metrics.scanned_srcs,
                                        type: pageResponse_constants_1.CHART_TYPE.line,
                                        stacked: false,
                                        order: 0,
                                    },
                                ],
                            }
                        },
                    },
                },
                single: {
                    sum_capacity: _a.tabs.single.capacity.sum_capacity,
                    used_percent: _a.tabs.totals.capacity.used_percent,
                    scanned_srcs: _a.tabs.single.data.scanned_srcs,
                    cpu: {
                        ..._a.tabs.single.cpu.cpu_busy,
                        label: 'CPU Total',
                    },
                    cpu_coss: {
                        label: 'CPU',
                        metrics: [
                            {
                                ..._a.metrics.cpu_system,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 0,
                            },
                            {
                                ..._a.metrics.cpu_user,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 1,
                            },
                            {
                                ..._a.metrics.cpu_wait,
                                type: pageResponse_constants_1.CHART_TYPE.area,
                                stacked: true,
                                order: 2,
                            },
                        ],
                    },
                    network_rate: _a.tabs.single.network.network_rate,
                    rx: _a.tabs.single.network.network_rx,
                    tx: _a.tabs.single.network.network_tx,
                    tcp_timeouts: _a.tabs.single.network.tcp_timeouts,
                    os_total: _a.tabs.single.os.os_total,
                    os_r: _a.tabs.single.os.os_r,
                    os_w: _a.tabs.single.os.os_w,
                    sum_memory_system: _a.tabs.single.memory.sum_memory_system,
                    sum_memory_onheap: _a.tabs.single.memory.sum_memory_onheap,
                    sum_memory_offheap: _a.tabs.single.memory.sum_memory_offheap,
                    rebuild_writes_total: _a.tabs.single.rebuild.rebuild_writes_total,
                    rebuild_writes_in: _a.tabs.single.rebuild.rebuild_writes_in,
                    rebuild_writes_out: _a.tabs.single.rebuild.rebuild_writes_out,
                    rebuild_deletes_total: _a.tabs.single.rebuild.rebuild_deletes_total,
                    rebuild_deletes_in: _a.tabs.single.rebuild.rebuild_deletes_in,
                    rebuild_deletes_out: _a.tabs.single.rebuild.rebuild_deletes_out,
                    rebuild_bytes_total: _a.tabs.single.rebuild.rebuild_bytes_total,
                    rebuild_bytes_in: _a.tabs.single.rebuild.rebuild_bytes_in,
                    rebuild_bytes_out: _a.tabs.single.rebuild.rebuild_bytes_out,
                },
            },
            vault: {
                label: "Vault",
                totals: {
                    capacity: {
                        label: 'Capacity',
                        tabs: {
                            capacity_used: _a.tabs.totals.capacity.used,
                        },
                    },
                    rebuild: {
                        label: 'Rebuild',
                        tabs: {
                            rebuild_writes_total: _a.tabs.totals.rebuild.rebuild_writes_total,
                            rebuild_writes_in: _a.tabs.totals.rebuild.rebuild_writes_in,
                            rebuild_writes_out: _a.tabs.totals.rebuild.rebuild_writes_out,
                            rebuild_deletes_total: _a.tabs.totals.rebuild.rebuild_deletes_total,
                            rebuild_deletes_in: _a.tabs.totals.rebuild.rebuild_deletes_in,
                            rebuild_deletes_out: _a.tabs.totals.rebuild.rebuild_deletes_out,
                            rebuild_bytes_total: _a.tabs.totals.rebuild.rebuild_bytes_total,
                            rebuild_bytes_in: _a.tabs.totals.rebuild.rebuild_bytes_in,
                            rebuild_bytes_out: _a.tabs.totals.rebuild.rebuild_bytes_out,
                        },
                    },
                    scanned_srcs: {
                        label: 'Scanned Sources',
                        tabs: {
                            scanned_srcs: _a.tabs.totals.data.scanned_srcs,
                        },
                    },
                    accesser_throughput: {
                        label: 'Accesser Throughput',
                        tabs: {
                            client_total: _a.tabs.totals.data.client_total,
                            client_in: _a.tabs.totals.data.client_in,
                            client_out: _a.tabs.totals.data.client_out,
                            slicestor_total: _a.tabs.totals.data.slicestor_total,
                            slicestor_in: _a.tabs.totals.data.slicestor_in,
                            slicestor_out: _a.tabs.totals.data.slicestor_out,
                        },
                    },
                },
                single: {
                    capacity_used: _a.tabs.single.capacity.capacity_used,
                    scanned_srcs: _a.tabs.single.data.scanned_srcs,
                    rebuild_writes_total: _a.tabs.single.rebuild.rebuild_writes_total,
                    rebuild_writes_in: _a.tabs.single.rebuild.rebuild_writes_in,
                    rebuild_writes_out: _a.tabs.single.rebuild.rebuild_writes_out,
                    rebuild_deletes_total: _a.tabs.single.rebuild.rebuild_deletes_total,
                    rebuild_deletes_in: _a.tabs.single.rebuild.rebuild_deletes_in,
                    rebuild_deletes_out: _a.tabs.single.rebuild.rebuild_deletes_out,
                    rebuild_bytes_total: _a.tabs.single.rebuild.rebuild_bytes_total,
                    rebuild_bytes_in: _a.tabs.single.rebuild.rebuild_bytes_in,
                    rebuild_bytes_out: _a.tabs.single.rebuild.rebuild_bytes_out,
                    client_total: {
                        label: 'Client Total',
                        metrics: [
                            {
                                ..._a.metrics.client_total,
                                type: pageResponse_constants_1.CHART_TYPE.line,
                                stacked: false,
                                order: 0,
                            },
                        ],
                    },
                    client_in: {
                        label: 'Client In',
                        metrics: [
                            {
                                ..._a.metrics.client_in,
                                type: pageResponse_constants_1.CHART_TYPE.line,
                                stacked: false,
                                order: 0,
                            },
                        ],
                    },
                    client_out: {
                        label: 'Client Out',
                        metrics: [
                            {
                                ..._a.metrics.client_out,
                                type: pageResponse_constants_1.CHART_TYPE.line,
                                stacked: false,
                                order: 0,
                            },
                        ],
                    },
                    slicestor_total: {
                        label: 'Slicestor Total',
                        metrics: [
                            {
                                ..._a.metrics.slicestor_total,
                                type: pageResponse_constants_1.CHART_TYPE.line,
                                stacked: false,
                                order: 0,
                            },
                        ],
                    },
                    slicestor_in: {
                        label: 'Slicestor In',
                        metrics: [
                            {
                                ..._a.metrics.slicestor_in,
                                type: pageResponse_constants_1.CHART_TYPE.line,
                                stacked: false,
                                order: 0,
                            },
                        ],
                    },
                    slicestor_out: {
                        label: 'Slicestor Out',
                        metrics: [
                            {
                                ..._a.metrics.slicestor_out,
                                type: pageResponse_constants_1.CHART_TYPE.line,
                                stacked: false,
                                order: 0,
                            },
                        ],
                    }
                },
            },
        }
    },
};
//# sourceMappingURL=Storage.js.map