package DATADOMAIN;

use strict;
use warnings;
use lib "../../../perl-modules";
require Xormon;
use Data::Dumper;
use JSON qw(encode_json decode_json);
use Date::Parse;
use FindBin;
use Date::Parse;
use POSIX ":sys_wait_h";
use File::Copy;
use File::Temp qw(tempfile);
use Test;
use XormonLogger;
my $logger;

my $module     = "IO::Tty";
my $module_def = 1;
eval "use $module; 1" or $module_def = 0;

my %mon2num = qw(Jan 01 Feb 02 Mar 03 Apr 04 May 05 Jun 06 Jul 07 Aug 08 Sep 09 Oct 10 Nov 11 Dec 12);
# Time Zone
my %tzoffset = (
  "A"     => "-0100", "ACDT"    => "+1030", "ACST"    => "+0930", "ACT"    => "-0500", "ACWDT"  => "+0945", "ACWST"   => "+0845", "ADDT"   => "-0200", "ADT"    => "-0300", "AEDT"   => "+1100",
  "AEST"  => "+1000", "AFT"     => "+0430", "AHDT"    => "-0900", "AHST"   => "-1000", "AKDT"   => "-0800", "AKST"    => "-0900", "AKTST"  => "+0600", "AKTT"   => "+0500", "ALMST"  => "+0700",
  "ALMT"  => "+0600", "AMST"    => "-0300", "AMT"     => "-0400", "ANAST"  => "+1200", "ANAT"   => "+1200", "ANT"     => "-0430", "APT"    => "-0300", "AQTST"  => "+0500", "AQTT"   => "+0500",
  "ARST"  => "-0200", "ART"     => "-0300", "ASHST"   => "+0500", "ASHT"   => "+0400", "AST"    => "-0400", "AT-0200" => "",      "AWDT"   => "+0900", "AWST"   => "+0800", "AWT"    => "-0300",
  "AZOMT" => "+0000", "AZOST"   => "+0000", "AZOT"    => "-0100", "AZST"   => "+0500", "AZT"    => "+0400", "B"       => "-0200", "BAKST"  => "+0400", "BAKT"   => "+0400", "BDST"   => "+0700",
  "BDT"   => "+0600", "BEAT"    => "+0230", "BEAUT"   => "+0245", "BNT"    => "+0800", "BORT"   => "+0800", "BORTST"  => "+0820", "BOT"    => "-0400", "BRST"   => "-0200", "BRT"    => "-0300",
  "BST"   => "+0100", "BT+0300" => "",      "BTT"     => "+0600", "BURT"   => "+0630", "C"      => "-0300", "CADT"    => "+1030", "CANT"   => "-0100", "CAPT"   => "-0900", "CAST"   => "+1100", "CAT" => "+0200",
  "CAWT"  => "-0900", "CCT"     => "+0630", "CDDT"    => "-0400", "CDT"    => "-0500", "CEMT"   => "+0300", "CEST"    => "+0200", "CET"    => "+0100", "CGST"   => "-0100", "CGT"    => "-0200", "CHADT" => "+1345",
  "CHAST" => "+1245", "CHDT"    => "-0530", "CHOST"   => "+1000", "CHOT"   => "+0800", "CHST"   => "+1000", "CHUT"    => "+1000", "CKHST"  => "-0930", "CKT"    => "-1000", "CLDT"   => "-0300", "CLST" => "-0300",
  "CLT"   => "-0300", "CMT"     => "+0155", "COST"    => "-0400", "COT"    => "-0500", "CPT"    => "-0500", "CST"     => "-0600", "CUT"    => "+0220", "CVST"   => "-0100", "CVT"    => "-0100", "CWT" => "-0500",
  "CXT"   => "+0700", "D"       => "-0400", "DACT"    => "+0600", "DAVT"   => "+0700", "DDUT"   => "+1000", "DUSST"   => "+0600", "DUST"   => "+0600", "E"      => "-0500", "EADT"   => "+1100", "EASST" => "-0500",
  "EAST"  => "-0500", "EAT"     => "+0300", "ECT"     => "-0500", "EDDT"   => "-0300", "EDT"    => "-0400", "EEST"    => "+0300", "EET"    => "+0200", "EETDST" => "+0300", "EETEDT" => "+0300",
  "EGST"  => "+0000", "EGT"     => "-0100", "EHDT"    => "-0430", "EPT"    => "-0400", "EST"    => "-0500", "EWT"     => "-0400", "F"      => "-0600", "FET"    => "+0300", "FJST"   => "+1300", "FJT" => "+1200",
  "FKST"  => "-0300", "FKT"     => "-0400", "FNST"    => "-0100", "FNT"    => "-0200", "FORT"   => "+0500", "FRUST"   => "+0600", "FRUT"   => "+0600", "FST"    => "+0200", "FWT"    => "+0100", "G" => "-0700",
  "GALT"  => "-0600", "GAMT"    => "-0900", "GB+0100" => "",      "GBGT"   => "-0345", "GEST"   => "+0400", "GET"     => "+0400", "GFT"    => "-0300", "GHST"   => "+0020", "GILT"   => "+1200", "GMT" => "+0000",
  "GMT+1" => "+0100", "GMT+10"  => "+1000", "GMT+11"  => "+1100", "GMT+12" => "+1200", "GMT+2"  => "+0200", "GMT+3"   => "+0300", "GMT+4"  => "+0400", "GMT+5"  => "+0500", "GMT+6"  => "+0600",
  "GMT+7" => "+0700", "GMT+8"   => "+0800", "GMT+9"   => "+0900", "GMT-1"  => "-0100", "GMT-10" => "-1000", "GMT-11"  => "-1100", "GMT-12" => "-1200", "GMT-13" => "-1300", "GMT-14" => "-1400",
  "GMT-2" => "-0200", "GMT-3"   => "-0300", "GMT-4"   => "-0400", "GMT-5"  => "-0500", "GMT-6"  => "-0600", "GMT-7"   => "-0700", "GMT-8"  => "-0800", "GMT-9"  => "-0900", "GST"    => "-0200",
  "GYT"   => "-0400", "H"     => "-0800", "HADT"  => "-0900", "HAST"  => "-1000", "HDT"     => "-0930", "HKST"  => "+0900", "HKT"   => "+0800", "HOVST"   => "+0800", "HOVT"  => "+0700", "HST"   => "-1000",
  "I"     => "-0900", "ICT"   => "+0700", "IDDT"  => "+0400", "IDLE"  => "+1200", "IDLW"    => "-1200", "IDT"   => "+0300", "IHST"  => "+0600", "IOT"     => "+0600", "IRDT"  => "+0430", "IRKST" => "+0900",
  "IRKT"  => "+0800", "IRST"  => "+0330", "ISST"  => "+0000", "IST"   => "+0530", "IT+0330" => "",      "JAVT"  => "+0720", "JCST"  => "+0900", "JDT"     => "+1000", "JST"   => "+0900", "JWST"  => "+0800",
  "K"     => "-1000", "KART"  => "+0500", "KDT"   => "+1000", "KGST"  => "+0600", "KGT"     => "+0600", "KIZST" => "+0600", "KIZT"  => "+0500", "KOST"    => "+1100", "KRAST" => "+0800", "KRAT"  => "+0700",
  "KST"   => "+0900", "KUYST" => "+0500", "KUYT"  => "+0300", "KWAT"  => "-1200", "L"       => "-1100", "LHDT"  => "+1100", "LHST"  => "+1030", "LINT"    => "+1400", "LKT"   => "+0600", "M"     => "-1200",
  "MADMT" => "+0100", "MADST" => "+0000", "MADT"  => "-0100", "MAGST" => "+1200", "MAGT"    => "+1000", "MALST" => "+0720", "MALT"  => "+0730", "MART"    => "-0930", "MAWT"  => "+0500", "MDDT"  => "-0500",
  "MDT"   => "-0600", "MEST"  => "+0200", "MESZ"  => "+0200", "MET"   => "+0100", "METDST"  => "+0200", "MEWT"  => "+0100", "MEZ"   => "+0100", "MHT"     => "+1200", "MIST"  => "+1100", "MMT"   => "+0630",
  "MOST"  => "+0900", "MOT"   => "+0800", "MPT"   => "+1000", "MSD"   => "+0400", "MSK"     => "+0300", "MSM"   => "+0500", "MST"   => "-0700", "MUST"    => "+0500", "MUT"   => "+0400", "MVT"   => "+0500",
  "MWT"   => "-0600", "MYT"   => "+0800", "N"     => "+0100", "NCST"  => "+1200", "NCT"     => "+1100", "NDDT"  => "-0130", "NDT"   => "-0230", "NEGT"    => "-0330", "NEST"  => "+0120", "NET"   => "+0020",
  "NFT"   => "+1130", "NMT"   => "+1112", "NOVST" => "+0700", "NOVT"  => "+0600", "NPT"     => "+0545", "NRT"   => "+1200", "NST"   => "-0330", "NT-1100" => "",      "NUT"   => "-1100", "NWT"   => "-1000",
  "NZDT"  => "+1300", "NZMT"  => "+1130", "NZST"  => "+1200", "NZT"   => "+1200", "O"       => "+0200", "OMSST" => "+0700", "OMST"  => "+0600", "ORAST"   => "+0500", "ORAT"  => "+0500", "P"     => "+0300",
  "PDDT"  => "-0600", "PDT"   => "-0700", "PEST"  => "-0400", "PET"   => "-0500", "PETST"   => "+1200", "PETT"  => "+1200", "PGT"   => "+1000", "PHOT"    => "+1300", "PHST"  => "+0900", "PHT"   => "+0800",
  "PKST"  => "+0600", "PKT"   => "+0500", "PMDT"  => "-0200", "PMST"  => "-0300", "PMT"     => "+1000", "PNT"   => "-0830", "PONT"  => "+1100", "PPMT"    => "-0449", "PPT"   => "-0700", "PST"   => "-0800",
  "PWT"   => "+0900", "PYST"  => "-0300", "PYT"   => "-0400", "Q"     => "+0400", "QMT"     => "-0514", "QYZST" => "+0700", "QYZT"  => "+0600", "R"       => "+0500", "RET"   => "+0400", "ROK"   => "+0900",
  "ROTT"  => "-0300", "S"     => "+0600", "SAKST" => "+1100", "SAKT"  => "+1000", "SAMST"   => "+0400", "SAMT"  => "+0400", "SAST"  => "+0200", "SAT"     => "-0400", "SBT"   => "+1100", "SCT"   => "+0400",
  "SDMT"  => "-0440", "SDT"   => "-1000", "SGT"   => "+0800", "SHEST" => "+0600", "SHET"    => "+0500", "SMT"   => "+0216", "SRET"  => "+1100", "SRT"     => "-0300", "SST"   => "-1100", "STAT"  => "+0400",
  "SVEST" => "+0500", "SVET"  => "+0400", "SWAT"  => "+0130", "SWT"   => "+0100", "SYOT"    => "+0300", "T"     => "+0700", "TAHT"  => "-1000", "TASST"   => "+0600", "TAST"  => "+0600", "TBIST" => "+0400",
  "TBIT"  => "+0400", "TFT"   => "+0500", "TJT"   => "+0500", "TKT"   => "+1300", "TLT"     => "+0900", "TMT"   => "+0500", "TOST"  => "+1400", "TOT"     => "+1300", "TRST"  => "+0400", "TRT"   => "+0300",
  "TSAT"  => "+0300", "TVT"   => "+1200", "U"     => "+0800", "ULAST" => "+0900", "ULAT"    => "+0800", "URAST" => "+0500", "URAT"  => "+0400", "UT+0000" => "",      "UTC"   => "+0000", "UYHST" => "-0230",
  "UYST"  => "-0200", "UYT"   => "-0300", "UZST"  => "+0600", "UZT"   => "+0500", "V"       => "+0900", "VET"   => "-0430", "VLAST" => "+1100", "VLAT"    => "+1000", "VOLST" => "+0400", "VOLT"  => "+0400",
  "VOST"  => "+0600", "VUST"  => "+1200", "VUT"   => "+1100", "W"     => "+1000", "WAKT"    => "+1200", "WARST" => "-0300", "WART"  => "-0400", "WAST"    => "+0200", "WAT"   => "+0100", "WEMT"  => "+0200",
  "WEST"  => "+0100", "WET"   => "+0000", "WFT"   => "+1200", "WGST"  => "-0200", "WGT"     => "-0300", "WIB"   => "+0700", "WIT"   => "+0900", "WITA"    => "+0800", "WMT"   => "+0124", "WSDT"  => "+1400",
  "WSST"  => "+1300", "X"     => "+1100", "XJT"   => "+0600", "Y"     => "+1200", "YAKST"   => "+1000", "YAKT"  => "+0900", "YDDT"  => "-0700", "YDT"     => "-0800", "YEKST" => "+0600", "YEKT"  => "+0500",
  "YERST" => "+0400", "YERT"  => "+0400", "YPT"   => "-0800", "YST"   => "-0900", "YWT"     => "-0800", "Z"     => "+0000", "ZP4"   => "+0400", "ZP5"     => "+0500", "ZP6"   => "+0600",
);

### mtree kapacity jsou hodinové co s tím?
### ddbost chybí metriky jsou prázdné asi fake data
### rozlišovat verze


my %data;
my $xormon;
my $cfg;
my $stats_found = 0;
my $api_version = "";
my $urls;
my $data_directory = "$FindBin::Bin";
my $debug = 0;

my $storage_name = "";
my $active_minute = "";
my $debug_data = "";
my $debug_data_performance = "";
my %performance_data;
my %configuration_data;
my $st_serial;
my $request_count;
my @architecture = ();
my $service_data_out;
my $st_type = "datadomain";
my @rows_api_test = ();
my @rows_api_test_data = ();
my $protocol = "";
my $host_api = "";
my $wrkdir = "";
my %inventory_performance;
my %inventory_configuration;
my %header_section;
my $service_data_in;
my $connection_test = 0;
my @rows_api_test_tmp = ();
my %volume_inventory;
my $instance_storage = "";
my $cookie = "";
my $username = "";
my $password = "";
my $ssh = "ssh ";
my $key_file = "";
my $count          = 1;       ## count outputs statistics for section
my $interval       = 300;     ## time collecting statistics
my $timeout_config = 120;
my $timeout_perf   = 180;
my $timeout_perf2  = 600;
my $version    = 5;           ### default
my $act_time_global      = time();
my $mtree_index          = 4;
my $run_alert            = 0;
my %inventory_data;
my %data_structure;
my $time_start = "";
my $time_end   = "";
my $date_start = "";
my $date2_start = "";
my $TZ = "";
my @health_status = ();
my @health_status_global = ();
my $enviroment = 1;
my $version_original = "";




sub collect {
  $xormon = shift;
  $cfg    = shift;
  unless ( exists $cfg->{'hostcfg_id'} )    { Xormon::error("Cannot continue, 'hostcfg_id' is not configured!") && return "error"; }
  unless ( exists $cfg->{'hostalias'} )     { Xormon::error("Cannot continue, 'hostalias' is not configured!") && return "error"; }
  $logger = XormonLogger->new( 'datadomain', $cfg->{hostcfg_id}, $cfg->{hostalias} );
  unless ( exists $cfg->{'sshHost'} )      { $logger->error("Cannot continue, 'sshHost' is not configured!") && return "error"; }
  unless ( exists $cfg->{'sshPort'} )      { $logger->error("Cannot continue, 'sshPort' is not configured!") && return "error"; }
  unless ( exists $cfg->{'sshUsername'} )      { $logger->error("Cannot continue, 'sshUsername' is not configured!") && return "error"; }
  unless ( exists $cfg->{'isKeyFile'} )      { $logger->error("Cannot continue, 'isKeyFile' is not configured!") && return "error"; }

  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }
  my $act_timestamp     = time();
  #$xormon->startJob('fetch', $cfg->{hostalias});
  $service_data_in = $xormon->getData();
  if ( exists $service_data_in->{last_conf_timestamp} ) {
    $service_data_out->{last_conf_timestamp} = $service_data_in->{last_conf_timestamp};
  }
  if (defined $cfg->{DATADOMAIN_ENVIROMENT} && $cfg->{DATADOMAIN_ENVIROMENT} eq "development"){
    $enviroment = 0;
  }
  if (!$enviroment){
    $cfg->{'isKeyFile'} = 1;
  }
  #print "enviroment=$enviroment\n";
  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $username = $cfg->{'sshUsername'};
  $st_serial = $cfg->{'uuid'};
  if (defined $ENV{SSH}){
    $ssh = $ENV{SSH};
  }
  else{
    $ssh = "ssh -q -o ConnectTimeout=80 -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -o SendEnv=no ";
  }
  if ($cfg->{'isKeyFile'}){
    if (defined $cfg->{"sshKeyFile"}){
      $key_file = $cfg->{"sshKeyFile"};
      if (-f "$key_file"){
        $ssh = $ssh . "-i $key_file -p $cfg->{'sshPort'} $cfg->{'sshUsername'}" . "@" . $cfg->{'sshHost'};
      }
      else{
        $ssh = $ssh . "-p $cfg->{'sshPort'} $cfg->{'sshUsername'}" . "@" . $cfg->{'sshHost'};
      }
    }
    else{
      $ssh = $ssh . "-p $cfg->{'sshPort'} $cfg->{'sshUsername'}" . "@" . $cfg->{'sshHost'};
    }
  }
  else{
    unless ( exists $cfg->{'sshPassword'} )      { $logger->error("Cannot continue, 'sshPassword' is not configured!") && return "error"; }
    $password = $cfg->{'sshPassword'};
    $ssh = Xormon::openssh_new(  $cfg->{'sshHost'},  $cfg->{'sshPort'},  $cfg->{'sshUsername'}, $timeout_config, $password );
    if ( !defined $ssh || $ssh eq "" || $ssh =~ m/^Error|^ERROR/ ) {
      $logger->error("Error: Unable to connect to remote machine ($storage_name)");
      $xormon->isConnected(0, "Connection failed!");
      return "error";
    }
  }
  data_structure();

  ### check version

  my $cmd_version  = "system show version";
  my $file_version = "$debug_data_performance/version.txt";
  cmd_to_file( $cmd_version, $file_version, $timeout_perf );
  $version = get_version($file_version);
  if (!Xormon::isdigit($version) || $version < 6){
    $logger->error("Version $version is not supported");
    $xormon->isConnected(0, "Connection failed!");
    if (!$cfg->{'isKeyFile'}){
      Xormon::openssh_disconnect( $ssh, $timeout_config );
    }
    return "error";
  }
  ### check version

  ### hostcfg_status
  $xormon->isConnected(1);
  ### hostcfg_status

  get_performance_statistics();
  #print "end data performance " . localtime() . "\n";

  if (!$cfg->{'isKeyFile'}){
    if (Xormon::openssh_disconnect( $ssh, $timeout_config ) == 0){
      $logger->error("Unable to disconnect from remote machine");
      return "error";
    }
  }
  my @data_date = get_array_data("$debug_data_performance/date.txt");
  foreach my $date (@data_date) {
    chomp $date;
    if ( $date eq "" ) { next; }
    $date_start = $date;
    $TZ = get_timezone($date_start);
  }
  my @data_date2 = get_array_data("$debug_data_performance/date2.txt");
  foreach my $date (@data_date) {
    chomp $date;
    if ( $date eq "" ) { next; }
    $date2_start = $date;
  }


  set_data_perf();
  #print Dumper \%data_structure;
  set_data_ddboost();
  set_data_cifs();
  set_data_nfs();
  set_data_net();
  set_data_port_type();
  set_data_cpu();
  set_data_disk();
  set_data_mtree();
  set_data_mtree_cap("Mtree Level Statistics");
  set_data_mtree_conf("Mtree Level Statistics"); ### compression metrics
  #set_data_replication();  ### not
  set_data_pool();
  set_data_tier();
  set_data_clear();
  set_data_convert();
  set_data_device();

  #print Dumper \%performance_data;
  #print Dumper \%configuration_data;
  #return 1;
  #print Dumper \%inventory_data;
  #return 1;
  #print Dumper \$service_data_in;
  #print "instance_storage is $instance_storage\n";
  delete_empty_metrics();
  calculate_missing_metrics();
  #print Dumper \%configuration_data;
  #print Dumper \$performance_data{$st_type}{"pool"};


  #logout_api();
  #return 0;
  #print Dumper \%performance_data;
  #my $rc = 201;

  my $rc = $xormon->saveJSON2(\%performance_data);
  #print "rc is $rc\n";
  my $status = get_status($rc, "saveJSON2 (performance)");


  ### host_cfg_data
  if (Xormon::isdigit($service_data_out->{last_perf_timestamp})){
    if ( ( time() -  $service_data_out->{last_perf_timestamp} ) > 10800 ) {
      $xormon->hasData(0, "last available stats are older than 3 hours: " . $service_data_out->{last_perf_timestamp});
    }
    else{
      $xormon->hasData(1);
    }
  }
  else{
    $xormon->hasData(0, "no available stats");
  }
  ###

  #### architecture

  #configuration
  if ( ( ! exists $service_data_in->{last_conf_timestamp} ) || ( ! Xormon::isdigit($service_data_in->{last_conf_timestamp}) ) || ( $act_timestamp - $service_data_in->{last_conf_timestamp} > 3600 ) || ( !-f "$debug_data/config-$st_serial.out" )){
    `touch "$debug_data/config-$st_serial.out"`;
    #print Dumper \%configuration_data;
    if (!$cfg->{'isKeyFile'}){
      $ssh = Xormon::openssh_new(  $cfg->{'sshHost'},  $cfg->{'sshPort'},  $cfg->{'sshUsername'}, $timeout_config, $password );
    }
    get_configurace_statistics();
    if (!$cfg->{'isKeyFile'}){
      #Xormon::openssh_disconnect( $ssh, $timeout_config );
      if (Xormon::openssh_disconnect( $ssh, $timeout_config ) == 0){
        $logger->error("Unable to disconnect from remote machine");
        return "error";
      }
    }
    set_data_pool_conf();
    set_data_drive_conf();
    set_data_port_conf();
    set_data_port_type_conf();
    set_data_mtree_conf();
    set_data_fs_conf();
    set_data_tier_conf();
    if ( check_mtree_exclude() ) {
      set_data_mtree_cap("Mtree Level Configuration");
    }
    set_data_ddboost_conf();
    set_data_cpu_conf();
    set_data_convert();
    set_data_device_conf();
    #print Dumper \%configuration_data;
    my @types = ("cpu", "pool", "drive", "fcport", "mtree", "fs", "ddboost", "sasport", "ethport", "device", "tier");
    foreach my $uid (keys %configuration_data){
      my $active = 0;
      my $type = $configuration_data{$uid}{TYPE};
      foreach my $type (@types){
        #print "$type\n";
        if ($type eq "tier"){
          $active = 0;
          last;
        }
        if (defined $performance_data{$st_type}{$type}{$uid}){
          $active++;
          last;
        }
      }
      if ($active == 0){
        #print "mazeme uid $uid\n";
        #print Dumper \$performance_data{$st_type}{$type}{$uid};
        #print Dumper \$configuration_data{$uid};
        delete $configuration_data{$uid};
        #print "mazeme prvek $uid\n";
      }
    }
    #print Dumper \%configuration_data;
    #print Dumper $performance_data{$st_type}{pool};
    #return 1;
    $service_data_out->{last_conf_timestamp} = $act_timestamp;
    set_data_alerting();
    ($status, my $status_configuration, my $status_health) = set_architecture();
    if ($status eq "successful" && $status_configuration eq "successful" && $status_health eq "successful" ){
      my $save_data = $xormon->saveData(\%{ $service_data_out });
      my $status_save_data = get_status($save_data, "saveData");
    }
    $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
    return $status;

  }
  my $save_data = $xormon->saveData(\%{ $service_data_out });
  my $status_save_data = get_status($save_data, "SaveData");
  $logger->info("Task for ".$cfg->{hostalias}." is done. Status: $status");
  return $status;
}

sub get_performance_statistics{

  my $cmd  = "system show version";
  my $file = "$debug_data_performance/version.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "system show date";
  $file = "$debug_data_performance/date_first.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $version = get_version($file);

  if ( !test_file_exist("$debug_data/port_fc.txt") ) {
    $cmd  = "scsitarget endpoint show list";
    $file = "$debug_data_performance/port_fc.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }
  else{
    $file = "$debug_data/port_fc.txt";
  }
  my @get_fc_port = get_fc_port($file);

  ### use fork for system show stats and ddboost file-replication stats
  my @cmds  = ();
  my @files = ();

  if ( $version == 6 || $version > 6 ) {
    $cmd = "system show stats view protocol-aggregate cifs repl ddboost net interval 150 count 3";
  }
  if ( $version == 5 ) {
    $cmd = "system show stats view nfs cifs repl ddboost net interval 150 count 3";
  }
  $file = "$debug_data_performance/perf_stat.txt";
  push( @cmds,  $cmd );
  push( @files, $file );

  #cmd_to_file($cmd,$file, $timeout_perf2);

  foreach my $fc_port (@get_fc_port) {
    my $cmd  = "scsitarget endpoint show stats $fc_port interval 300 count 1";
    my $file = "$debug_data_performance/port_fc_$fc_port.txt";
    push( @cmds,  $cmd );
    push( @files, $file );
  }

  $cmd  = "ddboost file-replication show performance interval 300 count 1";
  $file = "$debug_data_performance/ddboost_file_replication.txt";
  push( @cmds,  $cmd );
  push( @files, $file );

  my @parent;
  my $size = scalar @cmds;
  for my $i ( 0 .. $size - 1 ) {
    my $proc = fork;
    if ( not defined $proc ) {
      $logger->error('Could not fork');
      next;
    }
    if ($proc) {
      push( @parent, "$proc" );
    }
    else {
      setpgrp( 0, 0 );
      my $out = cmd( $cmds[$i] );
      write_data_from_cmd( $out, $files[$i] );
      exit(0);
    }
  }
  foreach my $pid (@parent) {
    eval {
      local $SIG{ALRM} = sub { die "die in SIGALRM"; };
      alarm($timeout_perf2);
      waitpid( $pid, 0 );
      alarm(0);
    };
    if ( $@ =~ /die in SIGALRM/ ) {
      my $index    = -1;
      my $act_time = localtime();
      foreach my $proc (@parent) {
        $index++;
        $logger->error( "$act_time command: $cmds[$index] timed out after: $timeout_perf2 seconds " . __FILE__ . ":" . __LINE__ );
        kill -15, $proc;
        waitpid( $proc, 0 );
      }
    }
  }

  $cmd  = "system show date";
  $file = "$debug_data_performance/date.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  if ( $version == 6 || $version > 6 ) {

    $cmd  = "disk show performance interval 5min count 1";
    $file = "$debug_data_performance/disk_stat.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }
  if ( $version == 5 ) {
    $cmd  = "disk show performance";
    $file = "$debug_data_performance/disk_stat.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }

  ### we have to list mtree because performance only per mtree commnad
  ### command run only 1 x 30 minutes
  #2020/12/16 12:30    0.00   0.00        0/0/0/0/0/0--------
  #2020/12/16 12:40    0.00   0.00        0/0/0/0/0/0--------
  #2020/12/16 12:50    0.00   0.00        0/0/0/0/0/0--------
  #2020/12/16 13:00    0.00   0.00        0/0/0/0/0/0--------
  if ( -l "$debug_data_performance/mtree_stat.txt" ) {
    my $file_link = readlink("$debug_data_performance/mtree_stat.txt");
    unless ( -e $file_link ) {
      unlink "$file_link"                     or $logger->error("Unable to unlink $file_link: $!");
      unlink "$debug_data_performance/mtree_stat.txt" or $logger->error("Unable to unlink $debug_data_performance/mtree_stat.txt: $!");
    }
  }
  if ( -l "$debug_data_performance/mtree.txt" ) {
    my $file_link = readlink("$debug_data_performance/mtree.txt");
    unless ( -e $file_link ) {
      unlink "$file_link"                or $logger->error("Unable to unlink $file_link: $!");
      unlink "$debug_data_performance/mtree.txt" or $logger->error("Unable to unlink $debug_data_performance/mtree.txt: $!");
    }
  }

  if ( !-l "$debug_data_performance/mtree_stat.txt" ) {

    $cmd  = "mtree list";
    $file = "$debug_data_performance/mtree.txt";

    if ( -e "$file" ) {
      unlink "$file" or $logger->error("Unable to unlink $file: $!");
    }

    cmd_to_file( $cmd, $file, $timeout_perf );

    my @mtree = get_mtree($file);
    if ( -e "$debug_data_performance/mtree_stat.txt" ) {
      unlink "$debug_data_performance/mtree_stat.txt" or $logger->error("Unable to unlink $debug_data_performance/mtree_stat.txt: $!");
    }
    #my $active_mtree_cap = 0;
    #if ( -e "$data_directory/mtree_cap.txt" ) {
    #  unlink "$data_directory/mtree_cap.txt" or  error ("Unable to unlink $data_directory/mtree_cap.txt: $!");
    #}
    foreach my $mtree_element (@mtree) {
      $mtree_element = remove_whitespace($mtree_element);
      my $cmd  = "mtree show performance '$mtree_element' last 1hours";
      my $file = "$debug_data_performance/mtree_stat.txt";
      cmd_to_file( $cmd, $file, $timeout_perf, "save" );
    }
    set_symlink_mtree_stats();
    $mtree_index = 2;
  }
  else {
    $mtree_index = get_mtree_index("$debug_data_performance/mtree_index.txt");
  }

  if ( $version == 6 || $version > 6 ) {
    $cmd  = "system show performance custom-view throughput iops protocol-latency utilization duration 10 min interval 5 min";
    $file = "$debug_data_performance/system_stat.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }
  if ( $version == 5 ) {
    $cmd  = "system show performance custom-view throughput protocol cache-miss latency compression utilization duration 10 min interval 5 min";
    $file = "$debug_data_performance/system_stat.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }

  if ( !test_file_exist("$debug_data/disk_capacity.txt") ) {
    $cmd  = "disk show hardware";
    $file = "$debug_data_performance/disk_capacity.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }

  if ( !test_file_exist("$debug_data/storage_capacity.txt") ) {
    $cmd  = "storage show all";
    $file = "$debug_data_performance/storage_capacity.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }

  if ( !test_file_exist("$debug_data/tier_space.txt") ) {
    $cmd  = "filesys show space";
    $file = "$debug_data_performance/tier_space.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }

  if ( !test_file_exist("$debug_data/tier_compression.txt") ) {
    $cmd  = "filesys show compression last 1 hours";
    $file = "$debug_data_performance/tier_compression.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }
  if ( $run_alert || !test_file_exist("$debug_data/alert.txt") ) {
    $cmd  = "alerts show current";
    $file = "$debug_data/alert.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }

  if ( !test_file_exist("$debug_data/port.txt") ) {
    $cmd  = "system show ports";
    $file = "$debug_data_performance/port.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );
  }

  $cmd  = "system show date";
  $file = "$debug_data_performance/date2.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "filesys clean status";
  $file = "$debug_data_performance/filesystem.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

}

sub set_symlink_mtree_stats {

  for ( my $i = 1; $i < 6; $i++ ) {
    my $act_time_g    = $act_time_global + ( 300 * $i );
    my $act_directory = localtime($act_time_g);
    my @entry_time    = split( " ", $act_directory );
    ( undef, my $active_minute, undef ) = split( ":", $entry_time[3] );
    my $dir = "$wrkdir/IOSTATS/$active_minute";
    if ( !-d "$dir" ) {
      mkdir "$dir" || $logger->error( "Cannot read $dir: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    }
    if ( $i == 1 ) {
      write_data_from_cmd( "2", "$dir/mtree_index.txt" );
    }
    if ( $i > 1 && $i < 4 ) {
      write_data_from_cmd( "3", "$dir/mtree_index.txt" );
    }
    if ( $i > 3 && $i < 6 ) {
      write_data_from_cmd( "4", "$dir/mtree_index.txt" );
    }
    if ( test_file_exist("$debug_data_performance/mtree_stat.txt") ) {
      my $act_time = localtime();
      if ( test_file_exist("$dir/mtree_stat.txt") ) {
        unlink "$dir/mtree_stat.txt" or $logger->error("Unable to unlink $dir/mtree_stat.txt: $!");
      }
      if ( !-l "$dir/mtree_stat.txt" ) {
        symlink( "$debug_data_performance/mtree_stat.txt", "$dir/mtree_stat.txt" ) || $logger->error( "$act_time: Cannot ln -s $debug_data_performance/mtree_stat.txt $dir/mtree_stat.txt : $!" . __FILE__ . ":" . __LINE__ );
      }
    }
    if ( test_file_exist("$debug_data_performance/mtree.txt") ) {
      my $act_time = localtime();
      if ( test_file_exist("$dir/mtree.txt") ) {
        unlink "$dir/mtree.txt" or $logger->error("Unable to unlink $dir/mtree.txt: $!");
      }
      if ( !-l "$dir/mtree.txt" ) {
        symlink( "$debug_data_performance/mtree.txt", "$dir/mtree.txt" ) || $logger->error( "$act_time: Cannot ln -s $debug_data_performance/mtree.txt $dir/mtree.txt : $!" . __FILE__ . ":" . __LINE__ );
        }
    }
  }
}
sub get_serial_number {

  my $file = "$debug_data/serial_number.txt";
  my @data = get_array_data($file);
  my @out  = ();

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line = remove_whitespace($line);
    my ( $property, $value ) = split( ":", $line );
    if ( !defined $property || $property eq "" ) { next; }
    $property = remove_whitespace($property);
    $value    = remove_whitespace($value);
    if ( !defined $value ) {
      $value = "";
    }
    if ( $property eq "Serial number" ) {
      $configuration_data{$st_serial}{serial} = $value;
      next;
    }
    if ( $property eq "System software-id" ) {
      $configuration_data{$st_serial}{"system_software_id"} = $value;
      next;
    }
    if ( $property eq "Instance software-id" ) {
      $configuration_data{$st_serial}{"instance_software_id"} = $value;
      next;
    }
    $configuration_data{$st_serial}{$property} = $value;
    #push( @out, "$property : $value\n" );
  }

  $file = "$debug_data/model.txt";
  @data = get_array_data($file);

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line = remove_whitespace($line);
    my ( $property, $value ) = split( ":", $line );
    if ( !defined $property || $property eq "" ) { next; }
    $property = remove_whitespace($property);
    $value    = remove_whitespace($value);
    if ( !defined $value ) {
      $value = "";
    }
    if ( $property eq "Model number" ) {
      $configuration_data{$st_serial}{model} = $value;
      next;
    }
  }
}

sub set_data_device_conf{
  $configuration_data{$st_serial}{version} = $version_original;
  get_serial_number();

}

sub set_data_device{
  my $type = "pool";
  #my @metrics = ("io_rate","read_io", "write_io", "data_rate", "read", "write", "resp_t","resp_t_r","resp_t_w","capacity_total", "capacity_used", "capacity_free", "repl_rate", "repl_write", "repl_read", "cleanable_capacity_total", "reduction", "postcomp_capacity_total");
  my %data;
  my $time = "";
  #print Dumper \$performance_data{$st_type}{$type};
  foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
    $time = (keys %{$performance_data{$st_type}{$type}{$uid}})[0];
    if (!Xormon::isdigit($time) ){
      #print Dumper \$performance_data{$st_type}{$type}{$uid};
      delete $performance_data{$st_type}{$type}{$uid};
      next;
    }
    foreach my $metric (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
      if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$metric})){
        if (Xormon::isdigit($data{$metric})){
          $data{$metric} = $data{$metric} + $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
        else{
          $data{$metric} = $performance_data{$st_type}{$type}{$uid}{$time}{$metric};
        }
      }
    }
  }
  if (Xormon::isdigit($time)){
    #$performance_data{$st_type}{device}{$st_serial}{$time} = \%data;
    foreach my $metric (keys %data){
      $performance_data{$st_type}{device}{$st_serial}{$time}{$metric} = $data{$metric};
    }
    foreach my $fs (keys %{$performance_data{$st_type}{fs}}){
      if (!%{$performance_data{$st_type}{fs}{$fs}}){
        
      }
      foreach my $t (keys %{$performance_data{$st_type}{fs}{$fs}}){
        my $phase = $performance_data{$st_type}{fs}{$fs}{$t}{phase};
        if (Xormon::isdigit($phase)){
          $performance_data{$st_type}{device}{$st_serial}{$time}{phase} = $phase;
          delete $performance_data{$st_type}{fs}{$fs};
          last;
        }
      }
    }
  }
  #print Dumper \$performance_data{$st_type}{fs};
  
  #print Dumper \%data;
}

sub remove_whitespace {
  my $value = shift;
  if ( !defined $value || $value eq "" ) {
    return "";
  }
  $value =~ s/^\s+|\s+$//g;
  return $value;

}


sub delete_empty_metrics{
  my $first_time = "";
  foreach my $type (keys %{$performance_data{$st_type}}){
    if ($type eq "tier"){
      delete $performance_data{$st_type}{$type};
      next;
    }
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      if (!%{$performance_data{$st_type}{$type}{$uid}}){
        delete $performance_data{$st_type}{$type}{$uid};
        next;
      }
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        if (!%{$performance_data{$st_type}{$type}{$uid}{$time}}){
          delete $performance_data{$st_type}{$type}{$uid}{$time};
          next;
        }
        if ($first_time eq "" && Xormon::isdigit($time)){
          $first_time = $time;
        }
        if (Xormon::isdigit($first_time) && Xormon::isdigit($time) && $time < $first_time){
          $first_time = $time;
        }
        foreach my $item (keys %{$performance_data{$st_type}{$type}{$uid}{$time}}){
          if (!Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item}) || $performance_data{$st_type}{$type}{$uid}{$time}{$item} < 0){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
          if ($item =~ m/numerator|denominator/){
            delete $performance_data{$st_type}{$type}{$uid}{$time}{$item};
          }
        }
        if (!$enviroment){
          my $act_time = time();
          $performance_data{$st_type}{$type}{$uid}{$act_time} = delete $performance_data{$st_type}{$type}{$uid}{$time};
        }
      }
    }
  }
  if (Xormon::isdigit($first_time)){
    $service_data_out->{last_perf_timestamp} = $first_time;
  }
  else{
    if (Xormon::isdigit($service_data_in->{last_perf_timestamp})){
      $service_data_out->{last_perf_timestamp} = $service_data_in->{last_perf_timestamp};
    }
  }
  #print Dumper \%performance_data;
}

sub calculate_missing_metrics{
  my $item_total = "capacity_total";
  my $item_used = "capacity_used";
  my $item_used_percent = "capacity_used_percent";

  foreach my $type (keys %{$performance_data{$st_type}}){
    foreach my $uid (keys %{$performance_data{$st_type}{$type}}){
      foreach my $time (keys %{$performance_data{$st_type}{$type}{$uid}}){
        #capacity_used_percent
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{$item_used})){
          if ($performance_data{$st_type}{$type}{$uid}{$time}{$item_total} == 0){
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = 0;
          }
          else{
            my $pom = ($performance_data{$st_type}{$type}{$uid}{$time}{$item_used} / $performance_data{$st_type}{$type}{$uid}{$time}{$item_total}) * 100;
            $performance_data{$st_type}{$type}{$uid}{$time}{$item_used_percent} = round($pom,2);
          }
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{data_rate} / $performance_data{$st_type}{$type}{$uid}{$time}{io_rate});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{data_rate}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{io_rate}) && $performance_data{$st_type}{$type}{$uid}{$time}{io_rate} == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size} = 0;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{read} / $performance_data{$st_type}{$type}{$uid}{$time}{read_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{read_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{read_io} == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_r} = 0
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} > 0){
          my $block_size = round($performance_data{$st_type}{$type}{$uid}{$time}{write} / $performance_data{$st_type}{$type}{$uid}{$time}{write_io});
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = $block_size;
        }
        if (Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write}) && Xormon::isdigit($performance_data{$st_type}{$type}{$uid}{$time}{write_io}) && $performance_data{$st_type}{$type}{$uid}{$time}{write_io} == 0){
          $performance_data{$st_type}{$type}{$uid}{$time}{block_size_w} = 0;
        }
      }
    }
  }
}

sub set_architecture{

  my %device_arch = (
    "item_id" => $cfg->{'uuid'},
    "label" => $cfg->{'hostalias'},
    "hw_type" => $st_type,
    "subsystem" => "device",
    "parents" => [],
    "hostcfg_id" => $cfg->{'uuid'}
  );
  push(@architecture, \%device_arch);
  #print Dumper \%configuration_data;
  #return;
  #print Dumper \$configuration_data{"53c8952a19"};
  foreach my $uid (keys %configuration_data){
    my $name = $configuration_data{$uid}{label};
    my $type = $configuration_data{$uid}{TYPE};
    if (!defined $name || !defined $type){next;}
    if ($uid eq $st_serial){next;}
    my @children = ();
    my @parents = ();
    if (defined $configuration_data{$uid}{children}){
      @children = @{$configuration_data{$uid}{children}};
      delete $configuration_data{$uid}{children};
    }
    if (defined $configuration_data{$uid}{parents}){
      @parents = @{$configuration_data{$uid}{parents}};
      delete $configuration_data{$uid}{parents};
    }
    #print Dumper \@children;
    #print Dumper \@parents;
    my %arch = (
      "item_id" => $uid,
      "label" => $name,
      "hw_type" => $st_type,
      "subsystem" => $type,
      "hostcfg_id" => $cfg->{'uuid'},
      "parents" => [$cfg->{'uuid'}]
      );
    if (@parents){
      push(@{$arch{parents}},@parents);
    }
    if (@children){
      push(@{$arch{children}},@children);
    }
    #push(@{$arch{parents}},$cfg->{'uuid'});
    push(@architecture, \%arch);
  }
  #print Dumper \@architecture;
  #foreach my $item (keys %{$data{conf}{device}}){
  #  $configuration_data{$st_serial}{$item} = $data{conf}{device}{$item};
  #}
  #print Dumper \$configuration_data{device}{$st_serial};

  #save configuration
  #print Dumper \@architecture;

  $configuration_data{$cfg->{'uuid'}}{alerts} = encode_json(\@health_status);
  #print Dumper \%configuration_data;
  #print Dumper \@architecture;
  my $rc = $xormon->saveConfJSON(\%configuration_data);
  my $status_configuration = get_status($rc, "saveConfJSON (configuration)");
  #$logger->info("Task for configuration ".$cfg->{hostalias}." is done. Status: $status");

  #save architecture
  #print Dumper \@architecture;

  $rc = $xormon->saveArchitecture(\@architecture);
  my $status_architecture = get_status($rc, "saveArchitecture");
  #$logger->info("Task for saveArchitecture ".$cfg->{hostalias}." is done. Status: $status");

  $rc = $xormon->saveStatus(\@health_status_global);
  my $status_health = get_status($rc, "health status");

  return ($status_architecture, $status_configuration, $status_health);
}

sub connection_test {
  my $client = shift;
  $cfg = shift;
  $connection_test = 1;


    # test main variables

  #unless ( exists $cfg->{'hostcfg_id'} )    { push(@rows_api_test, "Cannot continue, 'hostcfg_id' is not configured!");    return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'hostalias'} )   { push(@rows_api_test, "Cannot continue, 'hostalias' is not configured!"); return Xormon::conn_test(0, \@rows_api_test);  }
  #unless ( exists $cfg->{'sshHost'} )          { push(@rows_api_test, "Cannot continue, 'sshHost' is not configured!"); return Xormon::conn_test(0, \@rows_api_test); }
  #unless ( exists $cfg->{'sshPort'} )      { push(@rows_api_test, "Cannot continue, 'sshPort' is not configured!"); return Xormon::conn_test(0, \@rows_api_test);}
  #unless ( exists $cfg->{'sshUsername'} )      { push(@rows_api_test, "Cannot continue, 'sshUsername' is not configured!"); return Xormon::conn_test(0, \@rows_api_test);}
  #unless ( exists $cfg->{'isKeyFile'} )      { push(@rows_api_test, "Cannot continue, 'isKeyFile' is not configured!"); return Xormon::conn_test(0, \@rows_api_test); }

  my $test = Test->new($client);

  ### Validation of variables

  my @variables = ("hostcfg_id", "hostalias", "sshHost", "sshPort", "sshUsername", "isKeyFile");
  foreach my $variable (@variables){
    unless ( exists $cfg->{$variable} ){
      $test->addSection("Validation of variables", 0 , "Cannot continue, $variable is not configured!");
      return $test->stop();
    }
    if (!$cfg->{'isKeyFile'}){
      unless ( exists $cfg->{'sshPassword'} ) {
        $test->addSection("Validation of variables", 0 , "Cannot continue, " .  $cfg->{'sshPassword'} ." is not configured!");
        return $test->stop();
      }
    }
  }
  $test->addSection("Validation of variables", 1 , "");
  $logger = XormonLogger->new( 'datadomain', $cfg->{hostcfg_id}, $cfg->{hostalias} );

  if (!defined $cfg->{sampleRate}){
    $cfg->{sampleRate} = 5;
  }

  $storage_name = $cfg->{hostalias};
  $cfg->{'uuid'} = $cfg->{'hostcfg_id'};
  $username = $cfg->{'sshUsername'};
  $st_serial = $cfg->{'uuid'};
  if (defined $ENV{SSH}){
    $ssh = $ENV{SSH};
  }
  else{
    $ssh = "ssh -q -o ConnectTimeout=80 -o StrictHostKeyChecking=no -o PreferredAuthentications=publickey -o SendEnv=no ";
  }
  if ($cfg->{'isKeyFile'}){
    if (defined $cfg->{"sshKeyFile"}){
      $key_file = $cfg->{"sshKeyFile"};
      if (-f "$key_file"){
        $ssh = $ssh . "-i $key_file -p $cfg->{'sshPort'} $cfg->{'sshUsername'}" . "@" . $cfg->{'sshHost'};
      }
      else{
        $ssh = $ssh . "-p $cfg->{'sshPort'} $cfg->{'sshUsername'}" . "@" . $cfg->{'sshHost'};
      }
    }
    else{
      $ssh = $ssh . "-p $cfg->{'sshPort'} $cfg->{'sshUsername'}" . "@" . $cfg->{'sshHost'};
    }
  }
  else{
    ### Check module perl-IO-Tty
    $password = $cfg->{'sshPassword'};
    my $module     = "IO::Tty";
    my $module_def = 1;
    eval "use $module; 1" or $module_def = 0;
    if (!$module_def ) {
      $test->addSection("Check module perl-IO-Tty", 0 , "ERROR: perl-IO-Tty is not installed, install it by: yum install perl-IO-Tty\nRead more here: http://www.stor2rrd.com/IO-Tty.htm");
      return $test->stop();
    }
    $test->addSection("Check module perl-IO-Tty", 1 , "");
  }


  #### connection network test ####
  my ($code, $text) = Xormon::conntest_tcp($cfg->{'sshHost'},$cfg->{'sshPort'});
  $test->addSection("SSH host network connection", $code, $text);
  if ($code == 0){
    return $test->stop();
  }
  #return $test->stop();
  data_structure();

  if ($cfg->{'isKeyFile'}){
    ### SSH authorization

    my $cmd  = "system show version";
    my $version = cmd($cmd,$timeout_config,1);
    if ($version eq 0){
      #return Xormon::conn_test(0, \@rows_api_test);
      $test->addSection("SSH authorization", 0, \@rows_api_test);
      return $test->stop();
    }
    my $version_line = $version;
    $version_line =~ s/Data Domain OS//g;
    $version_line = remove_whitespace($version_line);
    my @chars = split( "", $version_line );
    if ( Xormon::isdigit( $chars[0] ) ) {
      $version_line = $chars[0];
    }
    if (!Xormon::isdigit($version_line) || $version_line < 6){
      push(@rows_api_test,"Error: Version $version_line is not supported. Only version 6 and higher is supported.\n");
      $test->addSection("SSH authorization", 0, \@rows_api_test);
      return $test->stop();
    }
    chop $version;
    push(@rows_api_test,"Version: $version\n");
    $test->addSection("SSH authorization", 1, \@rows_api_test);
    #return $test->stop();

    ### SSH data test

    $cmd = "system show stats view protocol-aggregate cifs repl ddboost net interval 5 count 1";
    my $data = cmd($cmd,$timeout_config,1);
    if ($data eq 0){
      $test->addSection("SSH data test", 0, \@rows_api_test);
      return $test->stop();
      #return Xormon::conn_test(0, \@rows_api_test);
    }
    #chop $version;
    #push(@rows_api_test,"Version: $version");
    #push(@rows_api_test,"Configuration data: ok");
    #push(@rows_api_test,"Performance data ok");
    #return Xormon::conn_test($connection_test, \@rows_api_test);
    $test->addSection("SSH data test", 1, "");
    return $test->stop();
  }
  else{
    ### SSH authorization

    $ssh = Xormon::openssh_new(  $cfg->{'sshHost'},  $cfg->{'sshPort'},  $cfg->{'sshUsername'}, $timeout_config, $password );
    if ( !defined $ssh || $ssh eq "" || $ssh =~ m/^Error|^ERROR/ ) {
      push(@rows_api_test,"Error: Unable to connect to remote machine ($storage_name)\n");
      push(@rows_api_test,"Error: $ssh\n");
      $test->addSection("SSH authorization", 0, \@rows_api_test);
      return $test->stop();
    }

    my $cmd  = "system show version";
    my $version = Xormon::openssh_runcmd( $ssh, $cmd, $timeout_config );
    if ( !defined $version || $version eq "" || $version =~ m/^Error|^ERROR/ || $version eq "0") {
      push(@rows_api_test,"Error: ssh command $cmd failed\n");
      push(@rows_api_test,"Error: $version\n");
      Xormon::openssh_disconnect( $ssh, $timeout_config );
      #return Xormon::conn_test(0, \@rows_api_test);
      $test->addSection("SSH authorization", 0, \@rows_api_test);
      return $test->stop();
    }
    my $version_line = $version;
    $version_line =~ s/Data Domain OS//g;
    $version_line = remove_whitespace($version_line);
    my @chars = split( "", $version_line );
    if ( Xormon::isdigit( $chars[0] ) ) {
      $version_line = $chars[0];
    }
    if (!Xormon::isdigit($version_line) || $version_line < 6){
      push(@rows_api_test,"Error: Version $version_line is not supported. Only version 6 and higher is supported.\n");
      Xormon::openssh_disconnect( $ssh, $timeout_config );
      $test->addSection("SSH authorization", 0, \@rows_api_test);
      return $test->stop();
    }
    chop $version;
    push(@rows_api_test,"Version: $version\n");
    $test->addSection("SSH authorization", 1, \@rows_api_test);

    ### SSH data test

    $cmd = "system show stats view protocol-aggregate cifs repl ddboost net interval 5 count 1";
    my $data = Xormon::openssh_runcmd($ssh,$cmd,$timeout_config);
    if ( !defined $data || $data eq "" || $data =~ m/^Error|^ERROR/ || $data eq "0") {
      push(@rows_api_test_data,"Error: ssh command $cmd failed\n");
      push(@rows_api_test_data,"Error: $data\n");
      Xormon::openssh_disconnect( $ssh, $timeout_config );
      $test->addSection("SSH data test", 0, \@rows_api_test_data);
      return $test->stop();
      #return Xormon::conn_test(0, \@rows_api_test);
    }
    #chop $version;
    #push(@rows_api_test,"Version: $version");
    #push(@rows_api_test,"Configuration data: ok");
    #push(@rows_api_test,"Performance data ok");
    if (Xormon::openssh_disconnect( $ssh, $timeout_config ) == 0){
      push(@rows_api_test_data,"Error: Unable to disconnect from remote machine\n");
      $test->addSection("SSH data test", 0, "");
      return $test->stop();
    }
    $test->addSection("SSH data test", 1, "Performance data ok\n");
    return $test->stop();
    #return Xormon::conn_test($connection_test, \@rows_api_test);
  }
  #return $test->stop();
}

sub data_structure {
  if (!-d "$data_directory/data"){
    mkdir "$data_directory/data" || $logger->error( "Cannot mkdir $data_directory/data: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial"){
    mkdir "$data_directory/data/$st_serial" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if (!-d "$data_directory/data/$st_serial/IOSTATS"){
    mkdir "$data_directory/data/$st_serial/IOSTATS" || $logger->error( "Cannot mkdir $data_directory/data/$st_serial/IOSTATS: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }


  my $directory = "$data_directory/data/$st_serial/IOSTATS";
  if ( !-d $directory ) {
    mkdir $directory || $logger->error( "Cannot mkdir $directory: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  if ( !-d "$directory/configuration" ) {
    mkdir "$directory/configuration" || $logger->error( "Cannot mkdir $directory/configuration: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  for ( my $i = 0; $i < 60; $i = $i + 5 ) {
    if ( $i < 10 ) {
      if ( !-d "$directory/0$i" ) {
        mkdir "$directory/0$i" || $logger->error( "Cannot mkdir $directory/0$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
    else {
      if ( !-d "$directory/$i" ) {
        mkdir "$directory/$i" || $logger->error( "Cannot mkdir $directory/$i: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    }
  } ## end for ( my $i = 0; $i < 60...)
  my @entry_time = split( " ", localtime() );
  ( undef, $active_minute, undef ) = split( ":", $entry_time[3] );
  if ( !-d "$directory/$active_minute" ) {
    mkdir "$directory/$active_minute" || $logger->error( "Cannot mkdir $directory/$active_minute: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  }
  $debug_data = "$directory/configuration";
  $debug_data_performance = "$directory/$active_minute";
  $wrkdir = "$data_directory/data/$st_serial";
  $act_time_global = time();

} ## end sub data_structure

sub round {
  my $number         = shift;
  my $decimal_plates = shift;
  if ( !defined $decimal_plates ) {
    $decimal_plates = 3;
  }
  my $num = $number;
  if ( $number =~ /\./ ) {
    $number =~ s/.*\.//g;
    my $length = length($number);
    if ( $length > $decimal_plates ) {
      my $rounded = sprintf( "%.$decimal_plates" . "f", $num );
      return $rounded;
    }
    else {
      return $num;
    }
  } ## end if ( $number =~ /\./ )
  else {
    return $num;
  }
} ## end sub round

sub cmd_to_file{
  my $cmd      = shift;
  my $file     = shift;
  my $timeout  = shift;
  my $save     = shift;
  my $file_tmp = "$file-tmp";
  my ( $fh, $filename ) = tempfile( UNLINK => 1 );
  if (!$enviroment){
    return;
  }

  #print "ssh command start $cmd " . localtime() . "\n";

  if ($cfg->{'isKeyFile'}){
    my $pid = fork();
    if ( !defined $pid ) {
      $logger->error( "no fork $!" . __FILE__ . ":" . __LINE__ );
      return;
    }
    elsif ( $pid == 0 ) {
      setpgrp( 0, 0 );
      if ( defined $save && $save eq "save" ) {
        `$ssh '$cmd' >> $file;echo \$? > "$filename"`;
      }
      else {
        `$ssh '$cmd' > $file_tmp;echo \$? > "$filename"`;
      }
      if (test_file_exist($filename)){
        my $code = `cat $filename`;
        chomp $code;
        if ($code ne "0"){
          $logger->error("command failed $cmd (error code: $code) " . __FILE__ . ":" . __LINE__);
          my $output = `cat $file_tmp`;
          $logger->error($output);
        }
        else {
          if ( test_file_exist($file_tmp) && !defined $save ) {
          rename( "$file_tmp", "$file" );
          }
        }
      }
      else {
        if ( test_file_exist($file_tmp) && !defined $save ) {
          rename( "$file_tmp", "$file" );
        }
      }
      exit(0);
    }
    eval {
      local $SIG{ALRM} = sub { die "die in SIGALRM"; };
      alarm($timeout);
      waitpid( $pid, 0 );
      alarm(0);
    };
    if ( $@ =~ /die in SIGALRM/ ) {
      my $act_time = localtime();
      $logger->error( "$act_time command: $cmd timed out after: $timeout seconds " . __FILE__ . ":" . __LINE__ );
      kill -15, $pid;
      waitpid( $pid, 0 );
    }
    return;
  }
  else{
    my $out = "";
    my $pid = fork();
    if ( !defined $pid ) {
      $logger->error( "no fork $!" . __FILE__ . ":" . __LINE__ );
      return;
    }
    elsif ( $pid == 0 ) {
      setpgrp( 0, 0 );
      if ( defined $save && $save eq "save" ) {

        #`$connection '$cmd' >> "$file" 2>>\$ERRLOG_DATADOMAIN`;
        #$out = ssh_pass_cmd($cmd, $ip_address, $user, $password, $port, $timeout);
        $out = Xormon::openssh_runcmd( $ssh, $cmd, $timeout );
        if (defined $out){
          `echo "$out" >> "$file"`;
        }
      }
      else {
        #`$connection '$cmd' > "$file" 2>>\$ERRLOG_DATADOMAIN`;
        #$out = ssh_pass_cmd($cmd, $ip_address, $user, $password, $port, $timeout);
        $out = Xormon::openssh_runcmd( $ssh, $cmd, $timeout );
        if ( $out ne "" && $out !~ m/^Error|^ERROR/ ) {
          `echo "$out" > "$file"`;
        }
      }
      if ( $out =~ m/^Error|^ERROR/ ) {
        $logger->error( "command failed: $cmd " . __FILE__ . ":" . __LINE__ );
      }
      exit(0);
    }
    eval {
      local $SIG{ALRM} = sub { die "die in SIGALRM"; };
      alarm($timeout);
      waitpid( $pid, 0 );
      alarm(0);
    };
    if ( $@ =~ /die in SIGALRM/ ) {
      my $act_time = localtime();
      $logger->error( "$act_time command: $cmd timed out after: $timeout seconds " . __FILE__ . ":" . __LINE__ );
      kill -15, $pid;
      waitpid( $pid, 0 );
    }
    #print "ssh command end $cmd " . localtime() . "\n";
    return;
  }
}

sub test_file_exist {
  my $file = shift;
  if ( !-e $file || -z $file ) {
    return 0;
  }
  else {
    return 1;
  }
}


sub cmd {
  my $cmd = shift;
  my $timeout = shift;
  my $connection_test = shift;
  my $out = "0";
  #print "ssh command start $cmd " . localtime() . "\n";
  if (!$enviroment){
    return $out;
  }

  if (defined $timeout && defined $connection_test){
     eval {
    # Set alarm
      my $act_time = localtime();
      local $SIG{ALRM} = sub { die "$act_time: died in SIG ALRM"; };
      alarm($timeout);
      $out = `$ssh $cmd 2>&1;echo \$? 2>&1`;
      my @parse_output = split("\n",$out);
      my $last = pop(@parse_output);
      if ( $last ne "0" ) {
        $out =~ s/$last/Code: $last/g;
        push(@rows_api_test, "command failed: $cmd (code: $last) " . __FILE__ . ":" . __LINE__ . "\n");
        push(@rows_api_test, "$out\n");
        push(@rows_api_test_data, "command failed: $cmd (code: $last) " . __FILE__ . ":" . __LINE__ . "\n");
        push(@rows_api_test_data, "$out\n");
        $out = 0;
      }
      else{
        chop $out;
        chop $out;
      }
      #print "ssh command end $cmd " . localtime() . "\n";
      alarm(0);
    };
    if ($@) {
      if ( $@ =~ /died in SIG ALRM/ ) {
        my $act_time = localtime();

        #error( "command timed out in eval after : $timeout seconds : " . __FILE__ . ":" . __LINE__ );
        push( @rows_api_test, "command timed out in eval after : $timeout seconds\n" );
        push( @rows_api_test, "ssh command failed: \"$cmd\"\n" );
        push( @rows_api_test_data, "command timed out in eval after : $timeout seconds\n" );
        push( @rows_api_test_data, "ssh command failed: \"$cmd\"\n" );
        #push(@rows_api_test, $out);
        return 0;
      }
    }
    return $out;
  }

  if ( defined $password && $password ne "" ) {

    #$out = ssh_pass_cmd($cmd, $ip_address, $user, $password, $port,$timeout_perf2);
    $out = Xormon::openssh_runcmd( $ssh, $cmd, $timeout_perf2 );
    if ( !defined $out || $out =~ m/^Error|^ERROR/ ) {
      $logger->error($out);
      return 0;
    }
    return $out;
  }
  $out = `$ssh $cmd;echo \$?`;
  if (!defined $out){
    return 0;
  }
  my @parse_output = split("\n",$out);
  my $last = pop(@parse_output);
  if ( $last ne "0" ) {
    $out =~ s/$last/Code: $last/g;
    $logger->error( "command failed: $cmd (code: $last) " . __FILE__ . ":" . __LINE__ );
    $logger->error($out);
    return 0;
  }
  chop $out;
  chop $out;
  return $out;
}

sub get_version {

  my $file    = "$debug_data_performance/version.txt";
  my @data    = get_array_data($file);
  my $version = 5;

  foreach my $version_line (@data) {
    chomp $version_line;
    if ( $version_line eq "" ) { next; }
    $version_line =~ s/Data Domain OS//g;
    $version_line = remove_whitespace($version_line);
    my @chars = split( "", $version_line );
    if ( Xormon::isdigit( $chars[0] ) ) {
      $version = $chars[0];
      $version_original = $version_line;
    }
  }
  return $version;
}

sub get_array_data {
  my $file = shift;
  my $test = test_file_exist($file);
  if ($test) {
    open( FH, "< $file" ) || $logger->error( "Cannot read $file: $!" . __FILE__ . ":" . __LINE__ ) && return ();
    my @file_all = <FH>;
    close(FH);
    return @file_all;
  }
  else {
    if ( $file =~ m/ddboost_file_replication\.txt/ ) {
      my @file_empty = "";
      return @file_empty;
    }
    $logger->error( "File not exist $file: $!" . __FILE__ . ":" . __LINE__ );
    my @file_empty = "";
    return @file_empty;
  }
}

sub get_fc_port{


  my $file     = shift;
  my @fc_ports = ();
  my %hash_position;
  my $active = 0;

  my @data = get_array_data($file);
  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ m/Endpoint/ && $line =~ m/System Address/ ) {
      $line =~ s/ {2,}/  /g;
      my @name_property = split( "  ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $line =~ m/^----/ && $active == 1 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;

    }
    if ( $active == 2 && $line !~ m/^----/ ) {
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( $name ne "System Address" ) { next; }
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          push( @fc_ports, "$string" );
        }
      }
    }
  }

  return @fc_ports;
}

sub write_data_from_cmd {
  my $out      = shift;
  my $file_tmp = shift;
  if (!$enviroment){
    return;
  }
  if ( $out ne "0" ) {
    open( DATA, ">$file_tmp" ) or $logger->error("Cannot open file: $file_tmp : $!") && return 0;
    print DATA $out;
    close(DATA);
  }
}

sub get_mtree {
  my $file      = shift;
  my @data      = get_array_data($file);
  my $active    = 0;
  my @mtree     = ();
  my $index_end = 0;

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line = remove_whitespace($line);
    if ( $line =~ m/^Name/ ) {
      $active = 1;
      next;
    }
    if ( $line =~ m/^----/ && $active == 1 ) {
      my @pom = split( "", $line );
      foreach my $char (@pom) {
        if ( $char eq "-" ) {
          $index_end++;
        }
        else { last; }
      }
      $active++;
      next;
    }
    if ( $line =~ m/^----/ && $active == 2 ) {
      $active++;
      next;
    }
    if ( $active == 2 ) {
      my @data_line   = split( " ", $line );
      my $count_items = scalar @data_line;
      if ( $index_end > 0 ) {
        my $value = substr( $line, 0, $index_end );
        $value = remove_whitespace($value);
        if ( defined $value && $value ne "" ) {
          push( @mtree, "$value" );
          next;
        }
      }

      if ( defined $data_line[0] && $count_items == 3 ) {

        #$data_line[0] =~ s/\&//g;
        push( @mtree, "$data_line[0]" );
        next;
      }
      else {
        my $name = "";
        for ( my $i = 0; $i < $count_items - 2; $i++ ) {
          if ( defined $data_line[$i] ) {
            if ( $name eq "" ) {
              $name = $data_line[$i];
            }
            else {
              $name = $name . " " . $data_line[$i];
            }
          }
        }

        #$name =~ s/\&//g;
        push( @mtree, "$name" );
        next;
      }
    }
  }
  return @mtree;

}

sub get_mtree_index {
  my $file = shift;
  my @data = get_array_data($file);
  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    $line = remove_whitespace($line);
    if ( !Xormon::isdigit($line) ) {
      return 4;
    }
    return $line;
  }
  return 4;
}

sub get_timezone{
  my $date = shift;
  $date =~ s/ {2,}/ /g;
  my @t = split(" ",$date);
  return $t[4];
}


sub get_utc_time{
  my $date = shift;
  my $timezone = shift;
  $date =~ s/ {2,}/ /g;
  if (!$timezone){
    $date =~ s/-/\//g;
    my @t = split(" ",$date);
    if (defined $tzoffset{$TZ}){
      $t[2] = $tzoffset{$TZ};
      my $epoch = str2time( join( " ", @t ) );
      return $epoch;
    }
  }
  my @t = split(" ",$date);
  #print Dumper \@t;
  if (length($t[2]) == 1){
    $t[2] = "0$t[2]";
  }
  my $time = "";
  if (defined $t[5] && $t[5] ne ""){
    $time = "$t[5]/$mon2num{$t[1]}/$t[2] $t[3] $t[4]";
  }
  else{
    $time = "$t[4]/$mon2num{$t[1]}/$t[2] $t[3] $TZ";
  }
  @t = ();
  @t = split( / /, $time );
  if ( defined $t[2] and exists( $tzoffset{ $t[2] } ) ) {
    $t[2] = $tzoffset{ $t[2] };
  }
  elsif (defined $tzoffset{$TZ}){
    $t[2] = $tzoffset{$TZ};
  }
  #print Dumper \@t;
  my $epoch = str2time( join( " ", @t ) );
  #print "$epoch\n";
  return $epoch;

}

sub set_data_perf {

  my $file       = "$debug_data_performance/perf_stat.txt";
  my @data       = get_array_data($file);
  my $active     = 0;
  my $data_items = "";
  my %hash_position;
  my @data_statistics = ();
  my $active_stat     = 0;
  my %hash_split;
  my $data_names       = "";
  my $data_units       = "";
  my $size_data        = scalar @data;
  my $last_line        = $data[-1];
  my $penultimate_line = $data[ $size_data - 2 ];

  foreach my $line (@data) {
    chomp $line;
    if ( $line =~ m/^20/ && ( $version == 6 || $version > 6 ) ) {
      #my $time = $line;
      #$time =~ s/-/:/g;
      #$time =~ s/ /T/g;
      #my $time_s   = str2time($time);
      #my $time_act = $time_s + 300;
      #$time_end   = get_time($time_act);
      #$time_start = get_time($time_s);
      $time_end = get_utc_time($line,0);
      $time_start = $time_end - 300;
      next;
    }
    my $act_moon = get_act_time();
    if ( $line =~ m/^$act_moon/ && $version == 5 ) {
      my $time       = time();
      my $time_human = localtime($time);
      my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
      chomp $year;
      my $time_line = $line;
      $time_line = "$year:" . $time_line;
      $time_line =~ s/\//:/g;
      $time_line =~ s/ /T/g;
      my $time_s   = str2time($time_line);
      my $time_act = $time_s + 300;
      $time_end   = get_time($time_act);
      $time_start = get_time($time_s);
    }
    if ( $line =~ m/CPU/ && $line =~ m/Net/ ) {

    }
    else {
      $line = remove_whitespace($line);
    }
    if ( $line eq "" ) { next; }
    if ( $line =~ m/CPU/ && $line =~ m/Net/ ) {
      my @section_line = split( /\|/, $line );
      $line = $line . "|";
      my @chars         = split( "", $line );
      my $char_index    = -1;
      my $index_section = -1;
      foreach my $char (@chars) {
        $char_index++;
        if ( $char eq "|" ) {
          $index_section++;
          $hash_split{$index_section} = $char_index;
        }
      }
      my $index = -1;
      foreach my $section (@section_line) {
        $index++;
        $section = remove_whitespace($section);
        if ( defined $section && $section ne "" ) {
          $data_structure{$index}{NAME} = $section;
          if ( defined $hash_split{$index} ) {
            $data_structure{$index}{END} = $hash_split{$index};
          }
        }
      }
    }
    if ( $line =~ m/busy/ && $line =~ m/max/ ) {
      $data_items = $line;
      next;
    }
    if ( $line =~ m/eth/ ) {
      $data_names = $line;
      next;
    }
    if ( $line =~ m/MB/ && $line =~ m/ops/ ) {

      #print "$line\n";
      $data_units = $line;
      next;
    }
    if ( $line =~ m/---- ----/ ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 0;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active_stat++;
      next;
    }
    if ( $active_stat == 1 ) {
      push( @data_statistics, "$line" );
    }
  }
  foreach my $section ( sort keys %data_structure ) {
    my $end_section  = $data_structure{$section}{END};
    my $name_section = $data_structure{$section}{NAME};
    if ( !defined $name_section || $name_section eq "" ) { next; }
    if ( !Xormon::isdigit($end_section) )                        { next; }
    foreach my $index ( sort keys %hash_position ) {
      if ( !defined $hash_position{$index}{END} ) {
        my $start = $hash_position{$index}{START};
        if ( Xormon::isdigit($start) ) {
          $hash_position{$index}{END} = $start + 25;
        }
      }
      if ( Xormon::isdigit( $hash_position{$index}{START} ) && Xormon::isdigit( $hash_position{$index}{END} ) && $end_section > $hash_position{$index}{START} ) {
        my $start  = $hash_position{$index}{START};
        my $end    = $hash_position{$index}{END};
        my $length = ( $end - $start );
        $length++;
        my $string            = substr( $data_items,       $start, $length );
        my $name              = substr( $data_names,       $start, $length );
        my $penultimate_value = substr( $penultimate_line, $start, $length );
        my $last_value        = substr( $last_line,        $start, $length );
        $string            = remove_whitespace($string);
        $name              = remove_whitespace($name);
        $penultimate_value = remove_whitespace($penultimate_value);
        $last_value        = remove_whitespace($last_value);

        if ( defined $string && $string ne "" ) {

          #print "$data_names\n";
          #if (!defined || $unit eq ""){
          #  $unit = "";
          #}
          if ( defined $name && $name ne "" && $name =~ m/^eth/ ) {
            $data_structure{$section}{$name}{ITEM}{$string}{NAME} = $string;
            if ( Xormon::isdigit($penultimate_value) ) {
              push @{ $data_structure{$section}{$name}{ITEM}{$string}{VALUE} }, "$penultimate_value";
            }
            if ( Xormon::isdigit($last_value) ) {
              push @{ $data_structure{$section}{$name}{ITEM}{$string}{VALUE} }, "$last_value";
              }
          }
          else {
            $data_structure{$section}{$name_section}{ITEM}{$string}{NAME} = $string;
            if ( Xormon::isdigit($penultimate_value) ) {
              push @{ $data_structure{$section}{$name_section}{ITEM}{$string}{VALUE} }, "$penultimate_value";
            }
            if ( Xormon::isdigit($last_value) ) {
              push @{ $data_structure{$section}{$name_section}{ITEM}{$string}{VALUE} }, "$last_value";
            }
          }
          delete $hash_position{$index};
        }
      }
    }
  }
  #print Dumper \%data_structure;
}

sub set_data_ddboost {

  #print Dumper \%data_structure;
  my $section = "DDBOOST Level Statistics";

  foreach my $id_section ( sort keys %data_structure ) {
    my $section_name = $data_structure{$id_section}{NAME};
    if ( defined $section_name && $section_name eq "DDBOOST" ) {
      foreach my $item ( keys %{ $data_structure{$id_section}{$section_name}{ITEM} } ) {
        if ( $item eq "data out" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
	      $value = Xormon::returnBytes($value,"MB");	    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}   = "ddboost";
            $inventory_data{$section}{"ALL"}{read} = $sum;
          }
        }
        if ( $item eq "data in" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
	      $value = Xormon::returnBytes($value,"MB");	    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}    = "ddboost";
            $inventory_data{$section}{"ALL"}{write} = $sum;
          }
        }
        if ( $item eq "net in" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
	      $value = Xormon::returnBytes($value,"KB");
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + ($value);
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}        = "ddboost";
            $inventory_data{$section}{"ALL"}{write_net} = $sum;
          }
        }
        if ( $item eq "net out" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
	      $value = Xormon::returnBytes($value,"KB");	    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
                }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}       = "ddboost";
            $inventory_data{$section}{"ALL"}{read_net} = $sum;
          }
        }
      }
    }
  }
  $inventory_data{$section}{TIME}{START} = $time_start;
  $inventory_data{$section}{TIME}{END}   = $time_end;
  $inventory_data{$section}{"ALL"}{TIME} = $time_end;
  $inventory_data{$section}{TIME}{STEP}  = 300;

  ### sum

  my $sum_read      = $inventory_data{$section}{"ALL"}{read};
  my $sum_write     = $inventory_data{$section}{"ALL"}{write};
  my $sum_data_rate = "";
  my $sum_read_net  = $inventory_data{$section}{"ALL"}{read_net};
  my $sum_write_net = $inventory_data{$section}{"ALL"}{write_net};
  my $sum_net_rate  = "";
  if ( Xormon::isdigit($sum_read) ) {
    $sum_data_rate = $sum_read;
  }
  if ( Xormon::isdigit($sum_write) ) {
    if ( Xormon::isdigit($sum_data_rate) ) {
      $sum_data_rate = $sum_data_rate + $sum_write;
    }
    else {
      $sum_data_rate = $sum_write;
    }
  }

  if ( Xormon::isdigit($sum_read_net) ) {
    $sum_net_rate = $sum_read_net;
  }
  if ( Xormon::isdigit($sum_write_net) ) {
    if ( Xormon::isdigit($sum_net_rate) ) {
      $sum_net_rate = $sum_net_rate + $sum_write_net;
    }
    else {
      $sum_net_rate = $sum_write_net;
    }
  }

  if ( Xormon::isdigit($sum_data_rate) ) {
    $inventory_data{$section}{"ALL"}{data_rate} = $sum_data_rate;
    }
  if ( Xormon::isdigit($sum_net_rate) ) {
    $inventory_data{$section}{"ALL"}{net_rate} = $sum_net_rate;
  }

  my $file        = "$debug_data_performance/ddboost_file_replication.txt";
  my @data        = get_array_data($file);
  my $active      = 0;
  my $sub_section = "ALL";
  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/---------/ ) {
      $active++;
      next;
    }
    if ( $active == 1 ) {
      my ( $read_pre_comp, $read_network, $write_pre_comp, $write_network ) = split( " ", $line );
      if ( Xormon::isdigit($read_pre_comp) ) {
        $inventory_data{$section}{$sub_section}{"Outbound Pre-comp(KB/s)"} = round(Xormon::returnBytes($read_pre_comp,"KB"),0);
      }
      if ( Xormon::isdigit($write_network) ) {
        $inventory_data{$section}{$sub_section}{"Inbound Network(KB/s)"} = round(Xormon::returnBytes($write_network,"KB"),0);
      }
      if ( Xormon::isdigit($read_network) ) {
        $inventory_data{$section}{$sub_section}{"Outbound Network(KB/s)"} = round(Xormon::returnBytes($read_network,"KB"),0);
      }
      if ( Xormon::isdigit($write_pre_comp) ) {
        $inventory_data{$section}{$sub_section}{"Inbound Pre-comp(KB/s)"} = round(Xormon::returnBytes($write_pre_comp,"KB"),0);
      }
      $active++;
    }
  }
  #print Dumper \$inventory_data{$section};
}

sub set_data_cifs {

  #print Dumper \%data_structure;
  my $section = "FS Level Statistics";

  foreach my $id_section ( sort keys %data_structure ) {
    my $section_name = $data_structure{$id_section}{NAME};
    if ( defined $section_name && $section_name eq "CIFS" ) {
      foreach my $item ( keys %{ $data_structure{$id_section}{$section_name}{ITEM} } ) {
        if ( $item eq "ops/s" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + ($value);
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}       = "0";
            $inventory_data{$section}{"ALL"}{ops_rate} = $sum;
          }
        }
        if ( $item eq "in" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
	      $value = Xormon::returnBytes($value,"MB");	    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}    = "0";
            $inventory_data{$section}{"ALL"}{write} = $sum;
          }
        }
        if ( $item eq "out" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
	      $value = Xormon::returnBytes($value,"MB");	    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}   = "0";
            $inventory_data{$section}{"ALL"}{read} = $sum;
          }
        }
        if ( $item eq "read" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
	      $value = Xormon::returnBytes($value,"KB");	    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + ($value);
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}       = "0";
            $inventory_data{$section}{"ALL"}{read_ops} = $sum;
          }
        }
        if ( $item eq "write" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $value = Xormon::returnBytes($value,"KB");		    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}        = "0";
            $inventory_data{$section}{"ALL"}{write_ops} = $sum;
          }
        }
      }
    }
  }
  $inventory_data{$section}{TIME}{START} = $time_start;
  $inventory_data{$section}{TIME}{END}   = $time_end;
  $inventory_data{$section}{"ALL"}{TIME} = $time_end;
  ### sum

  my $sum_read      = $inventory_data{$section}{"ALL"}{read};
  my $sum_write     = $inventory_data{$section}{"ALL"}{write};
  my $sum_data_rate = "";
  if ( Xormon::isdigit($sum_read) ) {
    $sum_data_rate = $sum_read;
  }
  if ( Xormon::isdigit($sum_write) ) {
    if ( Xormon::isdigit($sum_data_rate) ) {
      $sum_data_rate = $sum_data_rate + $sum_write;
    }
    else {
      $sum_data_rate = $sum_write;
    }
  }

  if ( Xormon::isdigit($sum_data_rate) ) {
    $inventory_data{$section}{"ALL"}{data_rate} = $sum_data_rate;
  }

}

sub set_data_nfs {

  my $section = "FS Level Statistics";
  my $fs_name = "NFS";
  if ( $version == 6 || $version > 6 ) {
    $fs_name = "Protocol Aggr";
  }
  foreach my $id_section ( sort keys %data_structure ) {
    my $section_name = $data_structure{$id_section}{NAME};
    if ( defined $section_name && $section_name eq "$fs_name" ) {
      foreach my $item ( keys %{ $data_structure{$id_section}{$section_name}{ITEM} } ) {
        if ( $item eq "ops/s" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + ($value);
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"1"}{ID}       = "1";
            $inventory_data{$section}{"1"}{ops_rate} = $sum;
          }
        }
        if ( $item eq "data in" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
	      $value = Xormon::returnBytes($value,"MB");
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"1"}{ID}    = "1";
            $inventory_data{$section}{"1"}{write} = $sum;
          }
        }
        if ( $item eq "data out" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
	      $value = Xormon::returnBytes($value,"MB");
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"1"}{ID}   = "0";
            $inventory_data{$section}{"1"}{read} = $sum;
          }
        }
        if ( $item eq "wait in" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = round( $value, 2 );
              }
              else {
                $read = $read + round( $value, 2 );
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 2 );
            $inventory_data{$section}{"1"}{ID}                  = "1";
            $inventory_data{$section}{"1"}{"wait_write(ms/KB)"} = $sum;
          }
        }
        if ( $item eq "wait out" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = round( $value, 2 );
              }
              else {
                $read = $read + round( $value, 2 );
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 2 );
            $inventory_data{$section}{"1"}{ID}                 = "1";
            $inventory_data{$section}{"1"}{"wait_read(ms/KB)"} = $sum;
          }
        }
      }
    }
  }
  $inventory_data{$section}{TIME}{START} = $time_start;
  $inventory_data{$section}{TIME}{END}   = $time_end;
  $inventory_data{$section}{"1"}{TIME}   = $time_end;
  $inventory_data{$section}{TIME}{STEP}  = 300;

  ### sum

  my $sum_read      = $inventory_data{$section}{"1"}{read};
  my $sum_write     = $inventory_data{$section}{"1"}{write};
  my $sum_data_rate = "";
  my $sum_wait      = "";
  my $sum_wait_out  = $inventory_data{$section}{"1"}{"wait_read(ms/KB)"};
  my $sum_wait_in   = $inventory_data{$section}{"1"}{"wait_write(ms/KB)"};
  if ( Xormon::isdigit($sum_read) ) {
    $sum_data_rate = $sum_read;
  }
  if ( Xormon::isdigit($sum_write) ) {
    if ( Xormon::isdigit($sum_data_rate) ) {
      $sum_data_rate = $sum_data_rate + $sum_write;
    }
    else {
      $sum_data_rate = $sum_write;
    }
  }

  if ( Xormon::isdigit($sum_data_rate) ) {
    $inventory_data{$section}{"1"}{data_rate} = $sum_data_rate;
  }

  if ( Xormon::isdigit($sum_wait_in) ) {
    $sum_wait = $sum_wait_in;
  }
  if ( Xormon::isdigit($sum_wait_out) ) {
    if ( Xormon::isdigit($sum_wait) ) {
      $sum_wait = $sum_wait + $sum_wait_out;
    }
    else {
      $sum_wait = $sum_wait_out;
    }
  }

  if ( Xormon::isdigit($sum_data_rate) ) {
    $inventory_data{$section}{"1"}{"wait_rate(ms/KB)"} = $sum_wait;
  }

}

sub set_data_net {

  my $section = "Port Level Statistics";
  foreach my $id_section ( sort keys %data_structure ) {
    foreach my $section_name ( keys %{ $data_structure{$id_section} } ) {
      if ( $section_name !~ m/^eth/ ) { next; }
      foreach my $item ( keys %{ $data_structure{$id_section}{$section_name}{ITEM} } ) {
        if ( $item eq "in" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if (Xormon::isdigit($value) ) {
              $denominator++;
	      $value = Xormon::returnBytes($value,"MB");
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"$section_name"}{ID}    = "$section_name";
            $inventory_data{$section}{"$section_name"}{write} = $sum;
          }
        }
        if ( $item eq "out" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
	      $value = Xormon::returnBytes($value,"MB");	    
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"$section_name"}{ID}   = $section_name;
            $inventory_data{$section}{"$section_name"}{read} = $sum;
          }
        }
      }
      $inventory_data{$section}{$section_name}{TIME} = $time_end;
    }
  }
  $inventory_data{$section}{TIME}{START} = $time_start;
  $inventory_data{$section}{TIME}{END}   = $time_end;
  $inventory_data{$section}{TIME}{STEP}  = 300;

  ### sum

  foreach my $port ( keys %{ $inventory_data{$section} } ) {
    if ( $port eq "TIME" ) { next; }

    my $sum_read      = $inventory_data{$section}{$port}{read};
    my $sum_write     = $inventory_data{$section}{$port}{write};
    my $sum_data_rate = "";
    if ( Xormon::isdigit($sum_read) ) {
      $sum_data_rate = $sum_read;
    }
    if ( Xormon::isdigit($sum_write) ) {
      if ( Xormon::isdigit($sum_data_rate) ) {
        $sum_data_rate = $sum_data_rate + $sum_write;
      }
      else {
        $sum_data_rate = $sum_write;
      }
    }
    if ( Xormon::isdigit($sum_data_rate) ) {
      $inventory_data{$section}{"$port"}{data_rate} = $sum_data_rate;
    }
  }

  ### set ID port
  my $file = "$debug_data_performance/port.txt";
  if ( test_file_exist("$debug_data/port.txt") ) {
    $file = "$debug_data/port.txt";
  }
  my @data = get_array_data($file);
  my %hash_position;
  my $active = 0;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/ && $active == 1 ) { last; }
    if ( $line =~ m/Port/ && $line =~ m/Hardware/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property && $property ne "No." ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
          }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 ) {
      my $id_port         = "";
      my $connection_type = "";
      my $mac             = "";
      my $firmware        = "";
      my $name_port       = "";
      my $link_speed      = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Port" ) {
            $id_port = $string;
          }
          if ( $name eq "Connection" ) {
            $connection_type = $string;
          }
          if ( $name eq "Link" ) {
            $link_speed = $string;
          }
          if ( $name eq "Firmware" ) {
            $firmware = $string;
          }
          if ( $name eq "Hardware" ) {
            $name_port = $string;
            $mac       = $name_port;
            if ( $name_port !~ m/\(/ ) {
              $name_port = $id_port;
            }
            else {
              $name_port =~ s/.*\(//g;
              $name_port =~ s/\)//g;
            }
            $name_port = remove_whitespace($name_port);
            $mac =~ s/\(.*//g;
            $mac = remove_whitespace($mac);
          }
        }
      }
      if ( defined $id_port && $id_port ne "" ) {
        if ( defined $name_port && $name_port ne "" && defined $inventory_data{$section}{$name_port}{ID} ) {
          $inventory_data{$section}{$name_port}{ID} = $id_port;
          if ( defined $connection_type && $connection_type ne "" ) {
            $connection_type = remove_whitespace($connection_type);
            my $type_port = get_type_port($connection_type);
            if ( $type_port ne "" ) {
              $inventory_data{$section}{$name_port}{protocol} = $type_port;
            }
          }
        }
      }
    }
  }
  #print Dumper \$inventory_data{$section};
  ## FC ports
  my @files = <$debug_data_performance/port_fc_*>;
  foreach my $file (@files) {
    @data = get_array_data($file);
    my $name_port = basename($file);
    $name_port =~ s/port_fc_//g;
    $name_port =~ s/\.txt//g;
    $active = 1;
    my $time = "";

    foreach my $index ( keys %hash_position ) {
      delete $hash_position{$index};
    }

    foreach my $line (@data) {
      chomp $line;
      $line = remove_whitespace($line);
      if ( $line eq "" ) {
        next;
      }
      if ( $active == 1 ) {
        $time = $line;
        $time = remove_whitespace($time);
        $active++;
        next;
      }
      if ( $active == 2 && $line =~ m/Endpoint/ && $line =~ m/Read/ ) {
        $line =~ s/ {2,}/  /g;
        my @name_property = split( "  ", $line );
        my $index         = 0;
        foreach my $property (@name_property) {
          $property = remove_whitespace($property);
          if ( defined $property ) {
            $index++;
            $hash_position{$index}{NAME}  = $property;
            $hash_position{$index}{INDEX} = $index;
          }
        }
        $active++;
        next;
        }
      if ( $line =~ m/^----/ && $active == 3 ) {
        my @chars        = split( "", $line );
        my $index        = -1;
        my $num_property = 1;
        foreach my $char (@chars) {
          $index++;
          if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
            $hash_position{$num_property}{END} = $index - 1;
            $num_property++;
            next;
          }
          if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
            $hash_position{$num_property}{START} = $index;
          }
        }
        $active++;
        next;

      }
      if ( $active == 4 && $line !~ m/^----/ && defined $name_port && defined $time ) {
        my $year = get_year();
        if ( $time !~ m/^$year/ ) {
          my $date = "$year/$time";
	  $time = get_utc_time($date,0);
	  #$date =~ s/\//:/g;
	  #$date =~ s/ /T/g;
	  #$time = $date;
        }
	else{
	  $time = get_utc_time($time,0);
	}
        foreach my $index ( sort keys %hash_position ) {
          my $start = $hash_position{$index}{START};
          my $end   = $hash_position{$index}{END};
          my $name  = $hash_position{$index}{NAME};
          if ( !defined $end ) {
            $end = length($line);
          }
          if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
          my $len = length $line;
          if ( $start > $len ) { next; }
          my $length = ( $end - $start );
          $length++;
          my $string = substr( $line, $start, $length );
          $string = remove_whitespace($string);
          if ( Xormon::isdigit($string) ) {
            $inventory_data{$section}{$name_port}{TIME}     = $time;
            $inventory_data{$section}{$name_port}{ID}       = $name_port;
            $inventory_data{$section}{$name_port}{protocol} = "FC";
            if ( $name eq "Ctrl/s" ) {
              $inventory_data{$section}{$name_port}{"Ctrl/s"} = $string;
            }
            if ( $name eq "Read/s" ) {
              $inventory_data{$section}{$name_port}{"read_io"} = $string;
            }
            if ( $name eq "Read MiB/s" ) {
              my $pom = Xormon::returnBytes($string,"MB");
              $inventory_data{$section}{$name_port}{"read"} = round( $pom, 0 );
            }
            if ( $name eq "Write/s" ) {
              $inventory_data{$section}{$name_port}{"write_io"} = $string;
            }
            if ( $name eq "Write MiB/s" ) {
              my $pom = Xormon::returnBytes($string,"MB");
              $inventory_data{$section}{$name_port}{"write"} = round( $pom, 0 );
            }
          }
        }
        my $read      = $inventory_data{$section}{$name_port}{"read"};
        my $write     = $inventory_data{$section}{$name_port}{"write"};
        my $read_io   = $inventory_data{$section}{$name_port}{"read_io"};
        my $write_io  = $inventory_data{$section}{$name_port}{"write_io"};
        my $io_rate   = "";
        my $data_rate = "";
        if ( Xormon::isdigit($read) ) {
          $data_rate = $read;
        }
        if ( Xormon::isdigit($write) ) {
          if ( Xormon::isdigit($data_rate) ) {
            $data_rate = $data_rate + $write;
          }
          else {
            $data_rate = $write;
          }
        }
        if ( Xormon::isdigit($data_rate) ) {
          $inventory_data{$section}{$name_port}{"data_rate"} = $data_rate;
        }

        if ( Xormon::isdigit($read_io) ) {
          $io_rate = $read_io;
        }
        if ( Xormon::isdigit($write_io) ) {
          if ( Xormon::isdigit($io_rate) ) {
            $io_rate = $io_rate + $write_io;
          }
          else {
            $io_rate = $write_io;
          }
        }
        if ( Xormon::isdigit($io_rate) ) {
          $inventory_data{$section}{$name_port}{"io_rate"} = $io_rate;
        }
      }
    }
  }
}

sub set_data_cpu {

  my $section = "CPU-CORE Level Statistics";

  foreach my $id_section ( sort keys %data_structure ) {
    my $section_name = $data_structure{$id_section}{NAME};
    if ( defined $section_name && $section_name eq "CPU" ) {
      foreach my $item ( keys %{ $data_structure{$id_section}{$section_name}{ITEM} } ) {
        if ( $item eq "busy" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + ($value);
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}   = "0";
            $inventory_data{$section}{"ALL"}{busy} = $sum;
          }
        }
      }
    }
  }
  $inventory_data{$section}{TIME}{START} = $time_start;
  $inventory_data{$section}{TIME}{END}   = $time_end;
  $inventory_data{$section}{"ALL"}{TIME} = $time_end;
  $inventory_data{$section}{TIME}{STEP}  = 300;

}

sub basename {
  my $full = shift;
  my $out  = "";

  # basename without direct function
  my @base = split( /\//, $full );
  foreach my $m (@base) {
    $out = $m;
  }

  return $out;
}

sub get_type_port {

  my $text = shift;
  if ( !defined $text || $text eq "" ) {
    return "";
  }
  if ( $text eq "Enet" ) {
    return "ETH";
  }
  if ( $text eq "SAS" ) {
    return $text;
  }
  if ( $text =~ m/^FC/ ) {
    return "FC";
  }

  return $text;

}

sub set_data_disk {

  my $file    = "$debug_data_performance/disk_stat.txt";
  my $section = "Drive Level Statistics";
  my @data    = get_array_data($file);
  my $active  = 0;

  if ( $version == 6 || $version > 6 ) {

    foreach my $line (@data) {
      chomp $line;
      $line = remove_whitespace($line);
      if ( $line eq "" ) { next; }
      if ( $line =~ m/Interval: 5min/ ) {
        my ( $date, $time ) = split( " ", $line );
        if ( defined $date && defined $time ) {
          #print "$line\n";
          $date =~ s/\//-/g;

          #my $time_human = localtime();
          #my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
          #chomp $year;
          my $year     = get_year();
          my $time_all = "$year-$date" . " " . "$time";
          my $time_utc = get_utc_time($time_all,0);
          #print "$time_all\n";
          $inventory_data{$section}{TIME}{END} = $time_utc;
          #my $start_time = $end_time - 300;
          #$inventory_data{$section}{TIME}{START} = get_time($start_time);
          #$inventory_data{$section}{TIME}{STEP}  = 300;
        }
      }
      if ( $line =~ m/^----/ ) {
        $active++;
        next;
      }
      if ( $active == 1 ) {
        my ( $name, $read, $read_io, $resp_t_r, undef, $write, $write_io, $resp_t_w, undef, $data_rate, $io_rate, $resp_t, $busy ) = split( " ", $line );
        $name      = remove_whitespace($name);
        $read      = remove_whitespace($read);
        $read_io   = remove_whitespace($read_io);
        $resp_t_r  = remove_whitespace($resp_t_r);
        $write     = remove_whitespace($write);
        $write_io  = remove_whitespace($write_io);
        $resp_t_w  = remove_whitespace($resp_t_w);
        $data_rate = remove_whitespace($data_rate);
        $io_rate   = remove_whitespace($io_rate);
        $resp_t    = remove_whitespace($resp_t);

        if ( defined $busy ) {
          $busy =~ s/%//g;
        }
        $busy = remove_whitespace($busy);

        if ( defined $name && $name ne "" ) {
          $inventory_data{$section}{$name}{"ID"} = $name;
          if ( Xormon::isdigit($read) ) {
            $inventory_data{$section}{$name}{"read"} = Xormon::returnBytes($read,"KB");
          }
          if ( Xormon::isdigit($read_io) ) {
            $inventory_data{$section}{$name}{"read_io"} = $read_io;
          }
          if ( Xormon::isdigit($resp_t_r) ) {
            $inventory_data{$section}{$name}{"resp_t_r"} = $resp_t_r;
          }
          if ( Xormon::isdigit($write) ) {
            $inventory_data{$section}{$name}{"write"} = Xormon::returnBytes($write,"KB");
          }
          if ( Xormon::isdigit($write_io) ) {
            $inventory_data{$section}{$name}{"write_io"} = $write_io;
          }
          if ( Xormon::isdigit($resp_t_w) ) {
            $inventory_data{$section}{$name}{"resp_t_w"} = $resp_t_w;
          }
          if ( Xormon::isdigit($data_rate) ) {
            my $pom = Xormon::returnBytes($data_rate,"MB");
            $inventory_data{$section}{$name}{"data_rate"} = round( $pom, 0 );
          }
          if ( Xormon::isdigit($io_rate) ) {
            $inventory_data{$section}{$name}{"io_rate"} = $io_rate;
          }
          if ( Xormon::isdigit($resp_t) ) {
            $inventory_data{$section}{$name}{"resp_t"} = $resp_t;
          }
          if ( Xormon::isdigit($busy) ) {
            $inventory_data{$section}{$name}{"busy"} = round( $busy, 3 );
          }
        }

      }
    }
  }

  #### ID and capacity ###

  $file = "$debug_data_performance/disk_capacity.txt";
  if ( test_file_exist("$debug_data/disk_capacity.txt") ) {
    $file = "$debug_data/disk_capacity.txt";
  }
  @data   = ();
  @data   = get_array_data($file);
  $active = 0;
  my %hash_position;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/ && $active == 1 ) { last; }
    if ( $line =~ m/Disk/ && $line =~ m/Firmware/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property && $property ne "No." ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }

      $active++;
      next;
    }
    if ( $active == 1 ) {
      my $name_disk  = "";
      my $slot       = "";
      my $model      = "";
      my $firmware   = "";
      my $serial_num = "";
      my $capacity   = "";
      my $type       = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Disk" ) {
            $name_disk = $string;
          }
          if ( $name eq "Serial" ) {
            $serial_num = $string;
          }
          if ( $name eq "Capacity" ) {
            $capacity = $string;
          }
        }
      }
      if ( defined $name_disk && $name_disk ne "" && defined $inventory_data{$section}{$name_disk} ) {
        if ( defined $serial_num && $serial_num ne "" ) {
          $inventory_data{$section}{$name_disk}{ID} = $serial_num;
        }
        if ( defined $capacity && $capacity ne "" ) {
          my ( $number, $unit ) = split( " ", $capacity );
          $number = remove_whitespace($number);
          $unit   = remove_whitespace($unit);
          if ( Xormon::isdigit($number) && defined $unit ) {
	    $inventory_data{$section}{$name_disk}{"capacity_total"} = Xormon::returnBytes($number,"MiB");	  
	    #my $cap_mb = convert_capacity( $unit, "MB", $number );
	    #if ( Xormon::isdigit($cap_mb) ) {
	    #  $inventory_data{$section}{$name_disk}{"Capacity(MB)"} = $cap_mb;
	    #}
          }
        }
      }
    }
  }
}


sub get_correct_date {
  my $date = shift;

  if ( !defined $date || $date eq "" ) { return; }
  my @t = split( " ", $date );
  #print Dumper \@t;
  if ( defined $t[0] && defined $t[1] && defined $t[2] && defined $t[3] && defined $t[5] ) {
    my $new_date = "$t[0] $t[1] $t[2] $t[3] $t[5]";
    return $new_date;
  }
  return "";

}



sub get_act_time {

  my $timestamp_end = time;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }

  return "$month/$day";

}

sub get_time {
  my $timestamp_end = shift;
  my @time          = localtime($timestamp_end);
  my $time_human    = localtime($timestamp_end);
  my $sec           = $time[0];
  my $minute        = $time[1];
  my $hour          = $time[2];
  my $day           = $time[3];
  my $month         = $time[4] + 1;
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;
  if ( $minute < 10 ) { $minute = "0$minute"; }
  if ( $hour < 10 )   { $hour   = "0$hour"; }
  if ( $day < 10 )    { $day    = "0$day"; }
  if ( $month < 10 )  { $month  = "0$month"; }
  if ( $sec < 10 )    { $sec    = "0$sec"; }
  my $time_end = "$year:$month:$day" . "T" . "$hour:$minute:$sec";
  return $time_end;

}

sub get_year {
  my $time       = time();
  my $time_human = localtime($time);
  my ( undef, undef, undef, undef, $year ) = split( " ", $time_human );
  chomp $year;

  my @data_date    = get_array_data("$debug_data_performance/date_first.txt");
  my $date_line    = "";
  my $year_storage = "";
  foreach my $date (@data_date) {
    chomp $date;
    if ( $date eq "" ) { next; }
    $date_line = $date;
  }
  if ( $date_line ne "" ) {
    my @data = split( " ", $date_line );
    $year_storage = $data[-1];
    $year_storage = remove_whitespace($year_storage);
  }
  if ( Xormon::isdigit($year_storage) ) {

    #my $abs = abs($year - $year_storage);
    my $length = length($year_storage);
    if ( Xormon::isdigit($length) && $length == 4 && $year_storage =~ m/^20/ ) {

      #if (isdigit($abs) && ($abs == 0 || $abs == 1 )){
      $year = $year_storage;
    }
  }

  return $year;

}



sub set_data_mtree {

  my $file        = "$debug_data_performance/mtree_stat.txt";
  my $section     = "Mtree Level Statistics";
  my @data        = get_array_data($file);
  my $active      = 0;
  my $prev_line   = "";
  my @mtree       = get_mtree("$debug_data_performance/mtree.txt");
  my $index       = -1;
  my $count_mtree = scalar @mtree;
  $count_mtree = $count_mtree + 1;
  my $data_index = 0;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { $data_index = 0; next; }
    if ( $line =~ m/^20/ ) {
      if ( $data_index != $mtree_index ) {
        $prev_line = $line;
        $data_index++;
      }

      #my ($date, $time, $read, $write, $rs, $ws, $rr, $wr, $r, $w) = split (" ", $line);
    }
    if ( $prev_line ne "" && $line =~ m/---------/ ) {
      $index++;
      if ( $index > $count_mtree || !defined $mtree[$index] ) { next; }
      if ( $data_index != $mtree_index )                      { next; }
      my $mtree_name = $mtree[$index];
      $mtree_name =~ s/\/data\/col1\///g;
      $mtree_name =~ s/\&//g;
      my ( $date, $time, $read, $write, $other_metrics ) = split( " ", $prev_line );
      my ( $rs, $ws, $rr, $wr, $r, $w ) = split( "/", $other_metrics );

      if ( defined $date && defined $time ) {
        #print "$data_directory ||| $prev_line ||| $mtree_name\n";
        $date =~ s/\//-/g;
        my $end_time        = "$date" . " " . "$time";
        my $utc_time = get_utc_time($end_time,0);
        $inventory_data{$section}{$mtree_name}{TIME} = $utc_time;
        $inventory_data{$section}{$mtree_name}{ID}   = $mtree_name;
        my $data_rate = "";

        if ( Xormon::isdigit($read) ) {
          my $pom = Xormon::returnBytes($read,"MB");
          $inventory_data{$section}{$mtree_name}{read} = round( $pom, 0 );
          $data_rate = $pom;
        }
        if ( Xormon::isdigit($write) ) {
          my $pom = Xormon::returnBytes($write,"MB");
          $inventory_data{$section}{$mtree_name}{write} = round( $pom, 0 );
          if ( Xormon::isdigit($data_rate) ) {
            $data_rate = $data_rate + $pom;
          }
          else {
            $data_rate = $pom;
          }
        }
        $inventory_data{$section}{$mtree_name}{data_rate} = $data_rate;
      }
      $prev_line = "";
    }
  }
}

sub set_data_replication {

  my $section = "Replication Level Statistics";

  foreach my $id_section ( sort keys %data_structure ) {
    my $section_name = $data_structure{$id_section}{NAME};
    if ( defined $section_name && $section_name eq "Repl" ) {
      foreach my $item ( keys %{ $data_structure{$id_section}{$section_name}{ITEM} } ) {
        if ( $item eq "out" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}   = "0";
            $inventory_data{$section}{"ALL"}{read} = $sum;
          }
        }
        if ( $item eq "in" ) {
          my $read        = "";
          my $denominator = 0;
          foreach my $value ( @{ $data_structure{$id_section}{$section_name}{ITEM}{$item}{"VALUE"} } ) {
            if ( Xormon::isdigit($value) ) {
              $denominator++;
              if ( !Xormon::isdigit($read) ) {
                $read = $value;
              }
              else {
                $read = $read + $value;
              }
            }
          }
          if ( Xormon::isdigit($read) && $denominator != 0 ) {
            my $sum = round( $read / $denominator, 0 );
            $inventory_data{$section}{"ALL"}{ID}    = "0";
            $inventory_data{$section}{"ALL"}{write} = $sum;
          }
        }
      }
    }
  }
  $inventory_data{$section}{TIME}{START} = $time_start;
  $inventory_data{$section}{TIME}{END}   = $time_end;
  $inventory_data{$section}{"ALL"}{TIME} = $time_end;
  $inventory_data{$section}{TIME}{STEP}  = 300;

  ### sum

  my $sum_read      = $inventory_data{$section}{"ALL"}{read};
  my $sum_write     = $inventory_data{$section}{"ALL"}{write};
  my $sum_data_rate = "";
  if ( Xormon::isdigit($sum_read) ) {
    $sum_data_rate = $sum_read;
  }
  if ( Xormon::isdigit($sum_write) ) {
    if ( Xormon::isdigit($sum_data_rate) ) {
      $sum_data_rate = $sum_data_rate + $sum_write;
      }
    else {
      $sum_data_rate = $sum_write;
    }
  }

  if ( Xormon::isdigit($sum_data_rate) ) {
    $inventory_data{$section}{"ALL"}{data_rate} = $sum_data_rate;
  }
}

sub set_data_pool {

  my $file      = "$debug_data_performance/system_stat.txt";
  my @data      = get_array_data($file);
  my $active    = 0;
  my $section   = "Pool Level Statistics";
  my $data_line = "";
  my $date_line = "";
  #my @data_date = get_array_data("$debug_data_performance/date.txt");
  #foreach my $date (@data_date) {
  #  chomp $date;
  #  if ( $date eq "" ) { next; }
  #  $date_line = $date;
  #}
  #my $correct_date = get_correct_date($date_line);
  #$date_line = $correct_date;
  #if ( $date_line eq "" ) { return; }
  my $timestamp  = get_utc_time($date_start,1);
  #print "$timestamp\n";
  if (!Xormon::isdigit($timestamp)){return;}
  my $date       = "";
  my $time       = "";
  my $repl_read  = "";
  my $repl_write = "";
  my $io_rate    = "";
  my $read_io    = "";
  my $write_io   = "";
  my $resp_t     = "";
  my $resp_t_r   = "";
  my $resp_t_w   = "";
  my $read       = "";
  my $write      = "";
  my %hash_position;
  my $repl           = "";
  my $repl_pre       = "";
  my $wait           = "";
  my $lcomp          = "";
  my $gcomp          = "";
  my $data           = "";
  my $thra           = "";
  my $unus           = "";
  my $ovhd           = "";
  my $meta           = "";
  my $sys            = "";
  my $name_property  = "";
  my $name_property2 = "";
  my $name_property3 = "";

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ m/Date/ && $line =~ m/Time/ ) {
      $name_property = $line;
      next;
    }
    if ( $line =~ m/read/ && $line =~ m/write/ ) {
      $name_property2 = $line;
      next;
    }
    if ( $line =~ m/thra/ && $line =~ m/meta/ ) {
      $name_property3 = $line;
      next;
    }
    $line = remove_whitespace($line);
    if ( $line =~ m/^----/ && $name_property ne "" ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }

      $active++;
      next;
    }
    if ( $line =~ m/^20/ && ( $version == 6 || $version > 6 ) ) {
      ### BUG DATADOMAIN only version 6 POOL section (replace read_repl for write_repl and vice versa)
      ( $date, $time, $read, $write, $repl_write, $repl_read, $io_rate, $read_io, $write_io, $resp_t, $resp_t_r, $resp_t_w, $sys ) = split( " ", $line );
      if ( !defined $date || !defined $time ) {
        next;
      }
      #$date =~ s/-/:/g;
      #print "$date $time\n";
      my $start_timestamp  = get_utc_time("$date $time",0);
      #my $end_time        = "$date" . "T" . "$time";
      #my $start_timestamp = str2time($end_time);
      if ( $timestamp >= $start_timestamp ) {
        $data_line = $line;
      }
    }
  }
  if ( $data_line ne "" && ( $version == 6 || $version > 6 ) ) {
    ( $date, $time, $read, $write, $repl_write, $repl_read, $io_rate, $read_io, $write_io, $resp_t, $resp_t_r, $resp_t_w, $sys ) = split( " ", $data_line );
    if ( !defined $date || !defined $time ) {
      return;
    }
    #$date =~ s/-/:/g;
    #my $end_time        = "$date" . "T" . "$time";
    #my $start_timestamp = str2time($end_time);
    #$start_timestamp = $start_timestamp - 300;
    #my $start_time = get_time($start_timestamp);
    my $timestamp  = get_utc_time("$date $time",0);
    $inventory_data{$section}{TIME}{END}   = $timestamp;
    $inventory_data{$section}{TIME}{START} = $timestamp - 300;
    $inventory_data{$section}{TIME}{STEP}  = 300;
    $inventory_data{$section}{0}{TIME}     = $timestamp;
    $inventory_data{$section}{0}{ID}       = 0;

    my $data_rate = "";
    my $repl_rate = "";
    if ( Xormon::isdigit($read) ) {
      my $pom = Xormon::returnBytes($read,"MB");
      $inventory_data{$section}{0}{read} = round( $pom, 0 );
      $data_rate = $pom;
    }
    if ( Xormon::isdigit($write) ) {
      my $pom = Xormon::returnBytes($write,"MB");
      $inventory_data{$section}{0}{write} = round( $pom, 0 );
      if ( Xormon::isdigit($data_rate) ) {
        $data_rate = $data_rate + $pom;
      }
      else {
        $data_rate = $write;
      }
    }
    if ( Xormon::isdigit($data_rate) ) {
      $inventory_data{$section}{0}{data_rate} = round( $data_rate, 0 );
    }

    if ( Xormon::isdigit($repl_read) ) {
      my $pom = Xormon::returnBytes($repl_read,"MB");;
      $inventory_data{$section}{0}{repl_read} = round( $pom, 0 );
      $repl_rate = $pom;
    }
    if ( Xormon::isdigit($repl_write) ) {
      my $pom = Xormon::returnBytes($repl_write,"MB");
      $inventory_data{$section}{0}{repl_write} = round( $pom, 0 );
      if ( Xormon::isdigit($repl_rate) ) {
        $repl_rate = $repl_rate + $pom;
      }
      else {
        $repl_rate = $repl_write;
      }
    }
    if ( Xormon::isdigit($repl_rate) ) {
      $inventory_data{$section}{0}{repl_rate} = round( $repl_rate, 0 );
    }
    #io_rate = read_io + write_io
    #if ( Xormon::isdigit($io_rate) ) {
    #  $inventory_data{$section}{0}{io_rate} = round( $io_rate, 0 );
    #}

    if ( $read_io =~ m/\// ) {
      (my $seq, my $random ) = split("/",$read_io);
      $read_io = "";
      if ( Xormon::isdigit($seq) ) {
        $read_io = $seq;
      }
      if (Xormon::isdigit($random)){
        if (Xormon::isdigit($read_io)){
          $read_io += $random; 
        }
        else{
          $read_io = $random;
        }
      }
      $inventory_data{$section}{0}{read_io} = $read_io;
    }
    if ( $write_io =~ m/\// ) {
      (my $seq, my $random ) = split("/",$write_io);
      $write_io = "";
      if ( Xormon::isdigit($seq) ) {
        $write_io = $seq;
      }
      if (Xormon::isdigit($random)){
        if (Xormon::isdigit($write_io)){
          $write_io += $random; 
        }
        else{
          $write_io = $random;
        }
      }
      $inventory_data{$section}{0}{write_io} = $write_io;
    }
    $io_rate = "";
    if ( Xormon::isdigit($read_io) ) {
      $io_rate = $read_io;
    }
    if (Xormon::isdigit($write_io)){
      if (Xormon::isdigit($io_rate)){
        $io_rate += $write_io; 
      }
      else{
        $io_rate = $write_io;
      }
    }
    $inventory_data{$section}{0}{io_rate} = $io_rate;
    if ( Xormon::isdigit($resp_t) ) {
      $inventory_data{$section}{0}{resp_t} = round( $resp_t, 3 );
    }

    if ( $resp_t_r =~ m/\// ) {
      $resp_t_r =~ s/\/.*//g;
      if ( Xormon::isdigit($resp_t_r) ) {
        $inventory_data{$section}{0}{resp_t_r} = round( $resp_t_r, 3 );
      }
    }
    if ( $resp_t_w =~ m/\// ) {
      $resp_t_w =~ s/\/.*//g;
      if ( Xormon::isdigit($resp_t_w) ) {
        $inventory_data{$section}{0}{resp_t_w} = round( $resp_t_w, 3 );
      }
    }
    if ( defined $sys ) {
      $sys =~ s/\/.*//g;
      if ( Xormon::isdigit($sys) ) {
        $inventory_data{"CPU-CORE Level Statistics"}{ALL}{sys} = round($sys);
      }
    }

  }
  ### capacity
  $file = "$debug_data_performance/storage_capacity.txt";
  if ( test_file_exist("$debug_data/storage_capacity.txt") ) {
    $file = "$debug_data/storage_capacity.txt";
  }
  @data = ();
  @data = get_array_data($file);
  foreach my $index ( keys %hash_position ) {
    delete $hash_position{$index};
  }

  $active = 0;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }

    #if ($line =~ m/^----/ && $active == 1){last;}
    if ( $line =~ m/License/ && $line =~ m/Total/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
      $active++;
    }
    if ( $line =~ m/^----/ && $active == 1 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }

      $active++;
      next;

    }
    if ( $active == 2 ) {
      my $total_cap = "";
      my $used_cap  = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Total" ) {
            $total_cap = $string;
          }
          if ( $name eq "Used" ) {
            $used_cap = $string;
          }
        }
      }
      if ( defined $total_cap && $total_cap ne "" ) {
        my ( $number, $unit ) = split( " ", $total_cap );
        $number = remove_whitespace($number);
        $unit   = remove_whitespace($unit);
        if ( Xormon::isdigit($number) && defined $unit ) {
	  $inventory_data{$section}{0}{"capacity_total"} = Xormon::returnBytes($number,$unit);	
	  ##my $cap_mb = convert_capacity( $unit, "TB", $number );
	  ##if ( Xormon::isdigit($cap_mb) ) {
	  #  $inventory_data{$section}{0}{"Capacity(TB)"} = $cap_mb;
	  #}
        }
      }
      if ( defined $used_cap && $used_cap ne "" ) {
        my ( $number, $unit ) = split( " ", $used_cap );
        $number = remove_whitespace($number);
        $unit   = remove_whitespace($unit);
        if ( Xormon::isdigit($number) && defined $unit ) {
          $inventory_data{$section}{0}{"capacity_used"} = Xormon::returnBytes($number,$unit);		
          #my $cap_mb = convert_capacity( $unit, "TB", $number );
	  #if ( Xormon::isdigit($cap_mb) ) {
	  #$inventory_data{$section}{0}{"Used(TB)"} = $cap_mb;
	  #}
        }
      }
      if ( Xormon::isdigit( $inventory_data{$section}{0}{"capacity_used"} ) && Xormon::isdigit( $inventory_data{$section}{0}{"capacity_total"} ) ) {
        my $free = $inventory_data{$section}{0}{"capacity_total"} - $inventory_data{$section}{0}{"capacity_used"};
        $inventory_data{$section}{0}{"capacity_free"} = round( $free, 0 );
      }
    }
  }
}

sub set_data_tier {

  ### capacity

  my $file = "$debug_data_performance/tier_space.txt";
  if ( test_file_exist("$debug_data/tier_space.txt") ) {
    $file = "$debug_data/tier_space.txt";
  }
  my @data    = get_array_data($file);
  my $section = "Tier Level Statistics";
  my $active  = 0;
  my %hash_position;
  my $index     = 0;
  my $date_line = "";
  my @data_date = get_array_data("$debug_data_performance/date.txt");

  foreach my $date (@data_date) {
    chomp $date;
    if ( $date eq "" ) { next; }
    $date_line = $date;
  }
  #my $correct_date = get_correct_date($date_line);
  #$date_line = $correct_date;
  #if ( $date_line eq "" ) { return; }
  #my $timestamp       = str2time($date_line);
  #my $timestamp_start = $timestamp - 300;
  #my $start_time      = get_time($timestamp_start);
  #my $end_time        = get_time($timestamp);
  my $end_time  = get_utc_time($date_start,1);
  if (!Xormon::isdigit($end_time)){return;}
  my $start_time = $end_time - 300;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/ && $active == 1 ) { last; }
    if ( $line =~ m/Resource/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property && $property ne "GiB" && $property ne "GiB*" ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 ) {
      my $resource  = "";
      my $size      = "";
      my $used      = "";
      my $free      = "";
      my $cleanable = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Size" ) {
            $size = $string;
          }
          if ( $name eq "Resource" ) {
            $resource = $string;
          }
          if ( $name eq "Used" ) {
            $used = $string;
          }
          if ( $name eq "Avail" ) {
            $free = $string;
          }
          if ( $name eq "Cleanable" ) {
            $cleanable = $string;
          }
        }
      }
      if ( defined $resource && $resource ne "" ) {
        $index++;
        my $id_element = $resource;
        if ( $id_element eq "/data: pre-comp" ) {
          next;
        }
        if ( $id_element eq "/data: post-comp" ) {
          $id_element = "tier";
        }
        else {
          $id_element =~ s/\//---/g;
          next;
        }
        $inventory_data{$section}{$resource}{ID}   = $id_element;
        $inventory_data{$section}{$resource}{name} = $resource;
        $inventory_data{$section}{$resource}{TIME} = $end_time;
        if ( Xormon::isdigit($size) ) {
          $inventory_data{$section}{$resource}{"capacity_total"} = Xormon::returnBytes($size,"GiB");

          #$inventory_data{$section}{$resource}{"Size(GB)"} = $size;
          $inventory_data{"Pool Level Statistics"}{0}{"capacity_total"} = Xormon::returnBytes($size,"GiB");
        }
        if ( Xormon::isdigit($used) ) {
          $inventory_data{$section}{$resource}{"capacity_used"} = Xormon::returnBytes($used,"GiB");
          $inventory_data{"Pool Level Statistics"}{0}{"capacity_used"} = Xormon::returnBytes($used,"GiB");

          #$inventory_data{$section}{$resource}{"Used(GB)"} = $used;
        }
        if ( Xormon::isdigit($free) ) {
          $inventory_data{$section}{$resource}{"capacity_free"} = Xormon::returnBytes($free,"GiB");
          $inventory_data{"Pool Level Statistics"}{0}{"capacity_free"} = Xormon::returnBytes($free,"GiB");

          #$inventory_data{$section}{$resource}{"Free(GB)"} = $free;
        }
        if ( Xormon::isdigit($cleanable) ) {
          $inventory_data{$section}{$resource}{"cleanable_capacity_total"} =  Xormon::returnBytes($cleanable,"GiB");

          #$inventory_data{$section}{$resource}{"Cleanable(GB)"} = $cleanable;
        }
      }
    }
  }
  $inventory_data{$section}{TIME}{START} = $start_time;
  $inventory_data{$section}{TIME}{END}   = $end_time;
  $inventory_data{$section}{TIME}{STEP}  = 300;
  if ( defined $inventory_data{$section}{"/data: pre-comp"}{"capacity_used"} && defined $inventory_data{$section}{"/data: post-comp"} ) {
    $inventory_data{$section}{"/data: post-comp"}{"precomp_capacity_total"} = $inventory_data{$section}{"/data: pre-comp"}{"capacity_used"};
    delete $inventory_data{$section}{"/data: pre-comp"};
  }
  ### note
  ### if name element ne /data: post-comp then element is partion (inspiration from GUI)

  ### compression

  $file = "$debug_data_performance/tier_compression.txt";
  if ( test_file_exist("$debug_data/tier_compression.txt") ) {
    $file = "$debug_data/tier_compression.txt";
  }
  @data   = get_array_data($file);
  $active = 0;
  foreach my $i ( keys %hash_position ) {
    if ( defined $hash_position{$i} ) {
      delete $hash_position{$i};
    }
  }

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/ && $active == 1 ) { last; }
    if ( $line =~ m/Pre-Comp/ && $line =~ m/Post-Comp/ && $line =~ m/Global-Comp/ && $line =~ m/Total-Comp/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        if ( $index == 0 ) {
          $index++;
          $hash_position{$index}{NAME}  = "NO";
          $hash_position{$index}{INDEX} = $index;
          if ( defined $property ) {
            $index++;
            $hash_position{$index}{NAME}  = $property;
            $hash_position{$index}{INDEX} = $index;
          }
          next;
        }
        $property = remove_whitespace($property);
        if ( defined $property ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 && $line =~ m/Currently Used/ ) {
      my $pre_comp    = "";
      my $post_comp   = "";
      my $global_comp = "";
      my $local_comp  = "";
      my $total_comp  = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Pre-Comp" ) {
            $pre_comp = $string;
          }
          if ( $name eq "Post-Comp" ) {
            $post_comp = $string;
          }
          if ( $name eq "Global-Comp" ) {
            $global_comp = $string;
          }
          if ( $name eq "Local-Comp" ) {
            $local_comp = $string;
          }
          if ( $name eq "Total-Comp" ) {
            $total_comp = $string;
          }
        }
      }
      if ( defined $inventory_data{$section}{"/data: post-comp"}{ID} ) {
        my $mtree_name = "/data: post-comp";
        if ( defined $pre_comp && $pre_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"precomp_capacity_total"} = Xormon::returnBytes($pre_comp,"GiB");
        }
        if ( defined $post_comp && $post_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"postcomp_capacity_total"} = Xormon::returnBytes($post_comp,"GiB");
        }
        if ( defined $global_comp && $global_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Global-comp Factor"} = $global_comp;
        }
        if ( defined $local_comp && $local_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Local-comp Factor"} = $local_comp;
        }
        if ( defined $total_comp && $total_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Total Compression Factor (Reduction %)"} = $total_comp;
          $total_comp =~ s/x//g;
          $total_comp =~ s/\)//g;
          $total_comp =~ s/\(//g;
          my ($total_comp_local, $reduction) = split (/ /,$total_comp);
          if (Xormon::isdigit($total_comp_local)){
            $inventory_data{$section}{$mtree_name}{"Total Compression Factor"} = $total_comp_local;
          }
          if (Xormon::isdigit($reduction)){
            $inventory_data{$section}{$mtree_name}{"Reduction"} = $reduction;
          }
        }
      }
    }
  }
}

sub set_data_clear {

  my $section = "FS Level Statistics";
  my $file    = "$debug_data_performance/filesystem.txt";
  my @data    = get_array_data($file);
  my $active  = 0;
  my %hash_position;
  my $index     = 0;
  my $date_line = "";
  #my @data_date = get_array_data("$debug_data_performance/date2.txt");

  #foreach my $date (@data_date) {
  #  chomp $date;
  #  if ( $date eq "" ) { next; }
  #  $date_line = $date;
  #}
  #my $correct_date = get_correct_date($date_line);
  #$date_line = $correct_date;
  #if ( $date_line eq "" ) { return; }
  my $timestamp  = get_utc_time($date2_start,1);
  if (!Xormon::isdigit($timestamp)){return;}

  my $phase = "";
  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ m/phase/ && $line =~ m/of/ ) {
      $phase = $line;
    }
  }
  if ( !defined $phase || $phase eq "" ) {
    $phase = 0;
  }
  else {
    $phase =~ s/.*phase//g;
    $phase = remove_whitespace($phase);
    my ($num) = split( " ", $phase );
    if ( !Xormon::isdigit($num) ) {
      $phase = 0;
    }
    else {
      $phase = $num;
    }
  }

  #if ( $date_line eq "" ) { return; }
  my $timestamp_start = $timestamp - 300;

  $inventory_data{$section}{TIME}{START}           = $timestamp_start;
  $inventory_data{$section}{TIME}{END}             = $timestamp;
  $inventory_data{$section}{TIME}{STEP}            = 300;
  if (Xormon::isdigit($phase)){
    $inventory_data{$section}{"clean_status"}{phase} = $phase;
    $inventory_data{$section}{"clean_status"}{TIME}  = $timestamp;
    $inventory_data{$section}{"clean_status"}{ID}    = "clean_status";
  }
}

sub get_configurace_statistics {

  my $cmd  = "system show date";
  my $file = "$debug_data/date.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  if ( check_mtree_exclude() ) {

    $cmd  = "mtree list";
    $file = "$debug_data/mtree.txt";
    cmd_to_file( $cmd, $file, $timeout_perf );

    my @mtree = get_mtree($file);
    if ( -e "$debug_data/mtree_capacity.txt" ) {
      unlink "$debug_data/mtree_capacity.txt" or $logger->error("Unable to unlink $debug_data/mtree_capacity.txt: $!");
    }
    my $active_mtree_cap = 0;
    if ( -e "$debug_data/mtree_cap.txt" ) {
      my $file = "$debug_data/mtree_cap_last.txt";
      rename( "$debug_data/mtree_cap.txt", "$file" ) or $logger->error( " Cannot mv $debug_data/mtree_cap.txt $file: $!" . __FILE__ . ":" . __LINE__ );
      if ( -e "$debug_data/mtree_cap.txt" ) {
        unlink "$debug_data/mtree_cap.txt" or $logger->error("Unable to unlink $debug_data/mtree_cap.txt: $!");
      }
    }
    foreach my $mtree_element (@mtree) {
      $mtree_element = remove_whitespace($mtree_element);
      my $cmd  = "mtree show compression '$mtree_element' last 1 hours";
      my $file = "$debug_data/mtree_capacity.txt";
      cmd_to_file( $cmd, $file, $timeout_perf, "save" );

      ### MTREE CAPACITY
      my $active_cap_file = test_mtree_cap_stats();
      if ( !test_file_exist("$debug_data/mtree_cap.touch") || $active_cap_file ) {
        $active_mtree_cap = 1;
        $cmd              = "filesys show compression '$mtree_element'";
        ### timeout 10 minutes
        $file = "$debug_data/mtree_cap.txt";
        `echo '$mtree_element' >> $file`;
        cmd_to_file( $cmd, $file, $timeout_perf2, "save" );
        `echo '--------------------------------------' >> $file`;
      }
      ###
    }
    if ( $active_mtree_cap == 1 ) {
      `touch "$debug_data/mtree_cap.touch"`;
    }
  }

  $cmd  = "disk show hardware";
  $file = "$debug_data/disk_capacity.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "system show ports";
  $file = "$debug_data/port.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "storage show all";
  $file = "$debug_data/storage_capacity.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "filesys show compression last 1 hours";
  $file = "$debug_data/tier_compression.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "filesys show space";
  $file = "$debug_data/tier_space.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "filesys status";
  $file = "$debug_data/filesystem.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "filesys option show";
  $file = "$debug_data/filesystem.txt";
  cmd_to_file( $cmd, $file, $timeout_perf, "save" );

  $cmd  = "filesys clean show config";
  $file = "$debug_data/filesystem.txt";
  cmd_to_file( $cmd, $file, $timeout_perf, "save" );

  $cmd  = "filesys clean status";
  $file = "$debug_data/filesystem.txt";
  cmd_to_file( $cmd, $file, $timeout_perf, "save" );

  $cmd  = "system show version";
  $file = "$debug_data/version.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "system show serialno";
  $file = "$debug_data/serial_number.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "system show modelno";
  $file = "$debug_data/model.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );

  $cmd  = "scsitarget endpoint show list";
  $file = "$debug_data/port_fc.txt";
  cmd_to_file( $cmd, $file, $timeout_perf );
}

sub test_mtree_cap_stats {

  my $mtree_cap_file_touch = "$debug_data/mtree_cap.touch";

  if ( !-e "$mtree_cap_file_touch" ) {
    return 1;
  }
  my $mtime = ( stat("$mtree_cap_file_touch") )[9];
  my $time  = get_time($mtime);
  my ( $date, undef ) = split( "T", $time );
  my ( undef, undef, $day ) = split( ":", $date );
  if ( defined $day ) {
    my $act_time         = time();
    my $act_time_convert = get_time($act_time);
    my ( $date_act, undef ) = split( "T", $act_time_convert );
    my ( undef, undef, $day_act ) = split( ":", $date_act );
    if ( defined $day_act && $day_act ne $day ) {
      return 1;
    }
  }

  return 0;
}

sub set_data_pool_conf {

  ### capacity
  my $file = "$debug_data/storage_capacity.txt";
  my @data = ();
  @data = get_array_data($file);
  my $section = "Pool Level Configuration";

  my $active = 0;
  my %hash_position;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }

    #if ($line =~ m/^----/ && $active == 1){last;}
    if ( $line =~ m/License/ && $line =~ m/Total/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
      $active++;
    }
    if ( $line =~ m/^----/ && $active == 1 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }

      $active++;
      next;

    }
    if ( $active == 2 ) {
      my $total_cap = "";
      my $used_cap  = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Total" ) {
            $total_cap = $string;
          }
          if ( $name eq "Used" ) {
            $used_cap = $string;
          }
        }
      }
      if ( defined $total_cap && $total_cap ne "" ) {
        my ( $number, $unit ) = split( " ", $total_cap );
        $number = remove_whitespace($number);
        $unit   = remove_whitespace($unit);
        if ( Xormon::isdigit($number) && defined $unit ) {
          $inventory_data{$section}{0}{"capacity_total"} = Xormon::returnBytes($number,$unit);		
	  #my $cap_mb = convert_capacity( $unit, "GB", $number );
	  #if ( Xormon::isdigit($cap_mb) ) {
	  #  $inventory_data{$section}{0}{"capacity (GB)"} = $cap_mb;
	  #}
        }
      }
      if ( defined $used_cap && $used_cap ne "" ) {
        my ( $number, $unit ) = split( " ", $used_cap );
        $number = remove_whitespace($number);
        $unit   = remove_whitespace($unit);
        if ( Xormon::isdigit($number) && defined $unit ) {
	  #my $cap_mb = convert_capacity( $unit, "GB", $number );
	  #if ( Xormon::isdigit($cap_mb) ) {
	  #  $inventory_data{$section}{0}{"used_capacity (GB)"} = $cap_mb;
	  #}
	  $inventory_data{$section}{0}{"capacity_used"} = Xormon::returnBytes($number,$unit);
        }
      }
      if ( Xormon::isdigit( $inventory_data{$section}{0}{"capacity_total"} ) && Xormon::isdigit( $inventory_data{$section}{0}{"capacity_used"} ) ) {
        my $free = $inventory_data{$section}{0}{"capacity_total"} - $inventory_data{$section}{0}{"capacity_used"};
        $inventory_data{$section}{0}{"capacity_free"} = round( $free, 0 );
      }
    }
  }
  if ( defined $inventory_data{$section}{0} ) {
    $inventory_data{$section}{0}{"name"} = "array_total";
    $inventory_data{$section}{0}{"id"}   = "array_total";
  }
}

sub set_data_drive_conf {

  #### ID and capacity ###

  my $file    = "$debug_data/disk_capacity.txt";
  my @data    = ();
  my $section = "Drive Level Configuration";
  @data = get_array_data($file);
  my $active = 0;
  my %hash_position;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/ && $active == 1 ) { last; }
    if ( $line =~ m/Disk/  && $line =~ m/Firmware/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property && $property ne "No." ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 ) {
      my $name_disk  = "";
      my $slot       = "";
      my $model      = "";
      my $firmware   = "";
      my $serial_num = "";
      my $capacity   = "";
      my $type       = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end && defined $start && defined $name ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);
        $string = remove_whitespace($string);
        if ( defined $string && $string ne "" ) {
          if ( $name eq "Disk" ) {
            $name_disk = $string;
          }
          if ( $name eq "Serial" ) {
            $serial_num = $string;
          }
          if ( $name eq "Capacity" ) {
            $capacity = $string;
          }
          if ( $name eq "Type" ) {
            $type = $string;
          }
        }
      }
      if ( defined $name_disk && $name_disk ne "" ) {
        if ( defined $serial_num && $serial_num ne "" ) {
          $inventory_data{$section}{$name_disk}{id}   = $serial_num;
          $inventory_data{$section}{$name_disk}{name} = $name_disk;
        }
        if ( defined $capacity && $capacity ne "" ) {
          my ( $number, $unit ) = split( " ", $capacity );
          $number = remove_whitespace($number);
          $unit   = remove_whitespace($unit);
          if ( Xormon::isdigit($number) && defined $unit ) {
            $inventory_data{$section}{$name_disk}{"capacity_total"} = Xormon::returnBytes($number,$unit);		  
	    #my $cap_mb = convert_capacity( $unit, "GB", $number );
	    #if ( Xormon::isdigit($cap_mb) ) {
	    #  $inventory_data{$section}{$name_disk}{"Capacity(GB)"} = $cap_mb;
	    #}
          }
        }
        if ( defined $type && $type ne "" ) {
          $inventory_data{$section}{$name_disk}{"type"} = $type;
        }
      }
    }
  }
}

sub set_data_port_conf {

  my $file    = "$debug_data/port.txt";
  my @data    = get_array_data($file);
  my $section = "Port Level Configuration";
  my %hash_position;
  my $active = 0;

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/ && $active == 1 ) { last; }
    if ( $line =~ m/Port/  && $line =~ m/Hardware/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property && $property ne "No." ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 ) {
      my $id_port         = "";
      my $connection_type = "";
      my $mac             = "";
      my $firmware        = "";
      my $name_port       = "";
      my $link_speed      = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);
        if ( defined $string && $string ne "" ) {
          if ( $name eq "Port" ) {
            $id_port = $string;
          }
          if ( $name eq "Connection" ) {
            $connection_type = $string;
          }
          if ( $name eq "Link" ) {
            $link_speed = $string;
          }
          if ( $name eq "Firmware" ) {
            $firmware = $string;
          }
          if ( $name eq "Hardware" ) {
            $name_port = $string;
            $mac       = $name_port;
            if ( $name_port !~ m/\(/ ) {
              $name_port = $id_port;
            }
            else {
              $name_port =~ s/.*\(//g;
              $name_port =~ s/\)//g;
            }
            $name_port = remove_whitespace($name_port);
            $mac =~ s/\(.*//g;
            $mac = remove_whitespace($mac);
          }
        }
      }
      if ( defined $id_port && $id_port ne "" ) {
        if ( defined $name_port && $name_port ne "" ) {
          $inventory_data{$section}{$id_port}{id}   = $id_port;
          $inventory_data{$section}{$id_port}{name} = $name_port;
        }
        else{
          $name_port = $id_port;
          $inventory_data{$section}{$id_port}{name} = $id_port;
        }
        if ( defined $connection_type && $connection_type ne "" ) {
          $inventory_data{$section}{$id_port}{"connection_type"} = $connection_type;
          my $type_port = get_type_port($connection_type);
          if ( defined $type_port && $type_port ne "" ) {
            $inventory_data{$section}{$id_port}{"Protocol"} = lc($type_port);
          }
        }
        if ( defined $link_speed && $link_speed ne "" ) {
          $inventory_data{$section}{$id_port}{"speed"} = $link_speed;
        }
        if ( defined $firmware && $firmware ne "" ) {
          $inventory_data{$section}{$id_port}{"firmware"} = $firmware;
        }
        if ( defined $mac && $mac ne "" ) {
          $inventory_data{$section}{$id_port}{"MAC"} = $mac;
        }
        #if (defined $mac && $mac ne "" && defined $connection_type && lc(get_type_port($connection_type)) eq "fc"){
        #  $mac =~ s/WWNN//g;
        #  $mac =~ s/://g;
        #  $mac =~ s/^\s+|\s+$//g;
        #  $inventory_data{$section}{$id_port}{name} = "$name_port (" . get_type_port($connection_type). " $mac)";
        #}
        #else{
        #  if (defined $connection_type){
        #    $inventory_data{$section}{$id_port}{name} = "$name_port (" . get_type_port($connection_type) . ")";
        #  }
        #}
      }

    }
  }
}

sub set_data_mtree_conf {
  my $section = shift;
  if (!defined $section || $section eq ""){
    $section = "Mtree Level Configuration";
  }
  my $file    = "$debug_data/mtree_capacity.txt";
  my @data    = get_array_data($file);
  my %hash_position;
  my $active      = 0;
  my @mtree       = get_mtree("$debug_data/mtree.txt");
  my $index_mtree = -1;
  my @status      = get_mtree_status("$debug_data/mtree.txt");

  foreach my $mtree_name (@mtree) {
    $index_mtree++;
    $mtree_name =~ s/\/data\/col1\///g;
    $mtree_name =~ s/\&//g;
    $inventory_data{$section}{$mtree_name}{id}   = $mtree_name;
    $inventory_data{$section}{$mtree_name}{name} = $mtree_name;
    if ( defined $status[$index_mtree] ) {
      $inventory_data{$section}{$mtree_name}{status} = $status[$index_mtree];
    }
  }
  $index_mtree = -1;
  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }

    #if ($line =~ m/^----/ && $active == 1){last;}
    if ( $line =~ m/Pre-Comp/ && $line =~ m/Post-Comp/ && $line =~ m/Global-Comp/ && $line =~ m/Total-Comp/ ) {
      $active = 0;
      $index_mtree++;
      foreach my $i ( keys %hash_position ) {
        if ( defined $hash_position{$i} ) {
          delete $hash_position{$i};
        }
      }
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        if ( $index == 0 ) {
          $index++;
          $hash_position{$index}{NAME}  = "NO";
          $hash_position{$index}{INDEX} = $index;
          if ( defined $property ) {
            $index++;
            $hash_position{$index}{NAME}  = $property;
            $hash_position{$index}{INDEX} = $index;
          }
          next;
        }
        $property = remove_whitespace($property);
        if ( defined $property ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 && $line =~ m/Last 24 hrs/ ) {
      my $pre_comp    = "";
      my $post_comp   = "";
      my $global_comp = "";
      my $local_comp  = "";
      my $total_comp  = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        if ( $start > 3 ) { $start = $start - 2; }
        my $end  = $hash_position{$index}{END};
        my $name = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);
        if ( defined $string && $string ne "" ) {
          if ( $name eq "Pre-Comp" ) {
            $pre_comp = $string;
          }
          if ( $name eq "Post-Comp" ) {
            $post_comp = $string;
          }
          if ( $name eq "Global-Comp" ) {
            $global_comp = $string;
          }
          if ( $name eq "Local-Comp" ) {
            $local_comp = $string;
          }
          if ( $name eq "Total-Comp" ) {
            $total_comp = $string;
          }
        }
      }
      if ( defined $mtree[$index_mtree] && $mtree[$index_mtree] ne "" ) {
        my $mtree_name = $mtree[$index_mtree];
        $mtree_name =~ s/\/data\/col1\///g;
        $mtree_name =~ s/\&//g;
        $inventory_data{$section}{$mtree_name}{id}   = $mtree_name;
        $inventory_data{$section}{$mtree_name}{name} = $mtree_name;
        if ( defined $pre_comp && $pre_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Pre-Comp(B)"} = Xormon::returnBytes($pre_comp,"GiB");
        }
        if ( defined $post_comp && $post_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Post-Comp(B)"} = Xormon::returnBytes($post_comp,"GiB");
        }
        if ( defined $global_comp && $global_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Global-Comp Factor"} = $global_comp;
        }
        if ( defined $local_comp && $local_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Local-Comp Factor"} = $local_comp;
        }
        if ( defined $total_comp && $total_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Total-Comp Factor(Reduction %)"} = $total_comp;
          $total_comp =~ s/x//g;
          $total_comp =~ s/\)//g;
          $total_comp =~ s/\(//g;
          my ($total_comp_local, $reduction) = split (/ /,$total_comp);
          if (Xormon::isdigit($total_comp_local)){
            $inventory_data{$section}{$mtree_name}{"Total Compression Factor"} = $total_comp_local;
          }
          if (Xormon::isdigit($reduction)){
            $inventory_data{$section}{$mtree_name}{"Reduction"} = $reduction;
          }
        }
      }
    }
  }
}

sub get_mtree_status {

  my $file   = shift;
  my @data   = get_array_data($file);
  my $active = 0;
  my %hash_position;
  my @status_short = ();
  my %convert_status;
  my @status_long = ();

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^Name/ ) {
      $active = 0;
      foreach my $i ( keys %hash_position ) {
        if ( defined $hash_position{$i} ) {
          delete $hash_position{$i};
        }
      }
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property ) {
          if ( $property =~ m/\(/ ) { next; }
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 && $line =~ /^----/ ) {
      $active++;
      next;
    }
    if ( $active == 1 ) {
      my $status = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);
        if ( defined $string && $string ne "" ) {
          if ( $name eq "Status" ) {
            $status = $string;
          }
        }
      }
      if ( defined $status && $status ne "" ) {
        push( @status_short, "$status" );
      }
      else {
        push( @status_short, "" );
      }
    }
    if ( $active == 2 ) {
      my ( $short, $long ) = split( " : ", $line );
      $short = remove_whitespace($short);
      $long  = remove_whitespace($long);
      if ( defined $short && defined $long ) {
        $convert_status{$short} = $long;
      }
    }
  }

  #print Dumper \@status_short;
  foreach my $statuses (@status_short) {
    my @status      = split( "/", $statuses );
    my $status_long = "";
    foreach my $stat (@status) {
      if ( defined $convert_status{$stat} ) {
        if ( $status_long eq "" ) {
          $status_long = $convert_status{$stat};
        }
        else {
          $status_long = $status_long . "/" . $convert_status{$stat};
        }
      }
      }
    push( @status_long, "$status_long" );
  }
  return @status_long;
}


sub set_data_fs_conf {

  ### nfs id 1
  ### cifs id 0
  my $section = "FS Level Configuration";
  $inventory_data{$section}{0}{id}   = 0;
  $inventory_data{$section}{0}{name} = "cifs";
  $inventory_data{$section}{1}{id}   = 1;
  $inventory_data{$section}{1}{name} = "nfs";
  #$inventory_data{$section}{"clean_status"}{id}   = "clean_status";
  #$inventory_data{$section}{"clean_status"}{name} = "clean_status";


}


sub set_data_tier_conf {

  ### capacity

  my $file    = "$debug_data/tier_space.txt";
  my @data    = get_array_data($file);
  my $section = "Tier Level Configuration";
  my $active  = 0;
  my %hash_position;
  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/ && $active == 1 ) { last; }
    if ( $line =~ m/Resource/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        $property = remove_whitespace($property);
        if ( defined $property && $property ne "GiB" && $property ne "GiB*" ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 ) {
      my $resource  = "";
      my $size      = "";
      my $used      = "";
      my $free      = "";
      my $cleanable = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Size" ) {
            $size = $string;
          }
          if ( $name eq "Resource" ) {
            $resource = $string;
          }
          if ( $name eq "Used" ) {
            $used = $string;
          }
          if ( $name eq "Avail" ) {
            $free = $string;
          }
          if ( $name eq "Cleanable" ) {
            $cleanable = $string;
          }
        }
      }
      if ( defined $resource && $resource ne "" ) {
        $inventory_data{$section}{$resource}{id}   = $resource;
        $inventory_data{$section}{$resource}{name} = $resource;
        if ( Xormon::isdigit($size) ) {
          $inventory_data{$section}{$resource}{"capacity_total"} = Xormon::returnBytes($size,"GiB");
          if ( $resource eq "/data: post-comp" ) {
            $inventory_data{"Pool Level Configuration"}{0}{"capacity_total"} = Xormon::returnBytes($size,"GiB");
          }
        }
        if ( Xormon::isdigit($used) ) {
          $inventory_data{$section}{$resource}{"capacity_used"} = Xormon::returnBytes($used,"GiB");
          if ( $resource eq "/data: post-comp" ) {
            $inventory_data{"Pool Level Configuration"}{0}{"capacity_used"} = Xormon::returnBytes($used,"GiB");
          }
        }
        if ( Xormon::isdigit($free) ) {
          $inventory_data{$section}{$resource}{"capacity_free"} = Xormon::returnBytes($free,"GiB");
          if ( $resource eq "/data: post-comp" ) {
            $inventory_data{"Pool Level Configuration"}{0}{"capacity_free"} = Xormon::returnBytes($free,"GiB");
          }
        }
        if ( Xormon::isdigit($cleanable) ) {
          $inventory_data{$section}{$resource}{"cleanable_capacity_total"} = Xormon::returnBytes($cleanable,"GiB");
        }
      }
    }
  }
  if ( defined $inventory_data{$section}{"/data: pre-comp"}{"capacity_used"} && defined $inventory_data{$section}{"/data: post-comp"} ) {
    $inventory_data{$section}{"/data: post-comp"}{"precomp_capacity_total"} = $inventory_data{$section}{"/data: pre-comp"}{"capacity_used"};
    delete $inventory_data{$section}{"/data: pre-comp"};
  }
  ### note
  ### if name element ne /data: post-comp then element is partion (inspiration from GUI)

  ### compression
  $file   = "$debug_data/tier_compression.txt";
  @data   = get_array_data($file);
  $active = 0;
  foreach my $i ( keys %hash_position ) {
    if ( defined $hash_position{$i} ) {
      delete $hash_position{$i};
    }
  }

  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $line =~ m/^----/    && $active == 1 ) { last; }
    if ( $line =~ m/Pre-Comp/ && $line =~ m/Post-Comp/ && $line =~ m/Global-Comp/ && $line =~ m/Total-Comp/ ) {
      my @name_property = split( " ", $line );
      my $index         = 0;
      foreach my $property (@name_property) {
        if ( $index == 0 ) {
          $index++;
          $hash_position{$index}{NAME}  = "NO";
          $hash_position{$index}{INDEX} = $index;
          if ( defined $property ) {
            $index++;
            $hash_position{$index}{NAME}  = $property;
            $hash_position{$index}{INDEX} = $index;
          }
          next;
        }
        $property = remove_whitespace($property);
        if ( defined $property ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
    }
    if ( $line =~ m/^----/ && $active == 0 ) {
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }
      $active++;
      next;
    }
    if ( $active == 1 && $line =~ m/Currently Used/ ) {
      my $pre_comp    = "";
      my $post_comp   = "";
      my $global_comp = "";
      my $local_comp  = "";
      my $total_comp  = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( !defined $end ) {
          $end = length($line);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $len = length $line;
        if ( $start > $len ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);

        if ( defined $string && $string ne "" ) {
          if ( $name eq "Pre-Comp" ) {
            $pre_comp = $string;
          }
          if ( $name eq "Post-Comp" ) {
            $post_comp = $string;
          }
          if ( $name eq "Global-Comp" ) {
            $global_comp = $string;
          }
          if ( $name eq "Local-Comp" ) {
            $local_comp = $string;
          }
          if ( $name eq "Total-Comp" ) {
            $total_comp = $string;
          }
        }
      }
      if ( defined $inventory_data{$section}{"/data: post-comp"}{id} ) {
        my $mtree_name = $inventory_data{$section}{"/data: post-comp"}{id};
        if ( defined $pre_comp && $pre_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"precomp_capacity_total"} = Xormon::returnBytes($pre_comp,"GiB");
        }
        if ( defined $post_comp && $post_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"postcomp_capacity_total"} = Xormon::returnBytes($post_comp,"GiB");
        }
        if ( defined $global_comp && $global_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Global-comp Factor"} = $global_comp;
        }
        if ( defined $local_comp && $local_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Local-comp Factor"} = $local_comp;
        }
        if ( defined $total_comp && $total_comp ne "" ) {
          $inventory_data{$section}{$mtree_name}{"Total Compression Factor (Reduction %)"} = $total_comp;
          $total_comp =~ s/x//g;
          $total_comp =~ s/\)//g;
          $total_comp =~ s/\(//g;
          my ($total_comp_local, $reduction) = split (/ /,$total_comp);
          if (Xormon::isdigit($total_comp_local)){
            $inventory_data{$section}{$mtree_name}{"Total Compression Factor"} = $total_comp_local;
          }
          if (Xormon::isdigit($reduction)){
            $inventory_data{$section}{$mtree_name}{"Reduction"} = $reduction;
          }
        }
      }
    }
  }
}

sub set_data_filesystem_conf {

  my $file    = "$debug_data/filesystem.txt";
  my @data    = get_array_data($file);
  my $section = "Filesystem Level Configuration";
  my $active  = 0;
  my %hash_position;
  my @data_out = ();
  push( @data_out, "$section\n" );
  push( @data_out, "----------------------------------\n\n" );
  my @data_section = ();
  my $element      = "";

  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" ) {

      unshift( @data_section, "$element\n\n" );
      push( @data_out, "@data_section\n" );
      @data_section = ();
      $element      = "";
      $active++;
      next;
    }
    $line = remove_whitespace($line);
    if ( $active == 0 ) {
      ### filesystem status
      $element = "  Health status:";
      push( @data_section, "    $line\n" );
      next;
    }
    if ( $active == 1 ) {
      ### filesystem option
      $element = "  Option:";
      push( @data_section, "    $line\n" );
      next;
      }
    if ( $active == 2 ) {
      ### filesystem cleanconfig
      $element = "  Clean config:";
      push( @data_section, "    Throttle: $line\n" );
      $active++;
      next;
    }
    if ( $active == 3 ) {
      ### filesystem clean config
      $element = "  Clean config:";
      push( @data_section, "    Schedule: $line\n" );
      next;
    }
    if ( $active == 4 ) {
      ### filesystem clean status
      $element = "  Clean status:";
      push( @data_section, "    $line\n" );
      next;
    }

  }
  unshift( @data_section, "$element\n\n" );
  push( @data_out, "@data_section\n" );
  push( @data_out, "----------------------------------\n\n" );

  return @data_out;

}

sub set_data_alerting {

  my $hs_error = 0;
  my $file = "$debug_data/alert.txt";
  my @data      = get_array_data($file);
  my $active    = 0;
  my $section   = "Alert Level Statistics";
  my $data_line = "";
  my %hash_position;
  my $hs_reason = "";
  #my $timestamp  = get_utc_time($date_start,1);
  #if (!Xormon::isdigit($end_time)){
  #  next;
  #}
  #my $timestamp_start = $timestamp - 300;

  #my $start_time      = get_time($timestamp_start);
  #my $end_time        = get_time($timestamp);
  my $line_parse      = "";
  foreach my $line (@data) {
    chomp $line;
    $line = remove_whitespace($line);
    if ( $line eq "" ) { next; }
    if ( $active == 3 && $line =~ m/------------/ ) { last; }
    if ( $line =~ m/No active alerts/ ) {
      $inventory_data{$section}{Message} = $line;
      last;
    }
    if ( $line =~ /^Alert/ ) {
      next;
    }
    if ( $line =~ m/Id/ && $line =~ m/Severity/ ) {
      $line =~ s/ {2,}/  /g;
      my @elements = split( "  ", $line );
      my $index    = 0;
      foreach my $property (@elements) {
        $property = remove_whitespace($property);
        if ( defined $property ) {
          $index++;
          $hash_position{$index}{NAME}  = $property;
          $hash_position{$index}{INDEX} = $index;
        }
      }
      $active = 2;
    }
    if ( $active == 2 && $line =~ m/------------/ ) {
      $line_parse = $line;
      my @chars        = split( "", $line );
      my $index        = -1;
      my $num_property = 1;
      foreach my $char (@chars) {
        $index++;
        if ( $char eq " " && defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{END} = $index - 1;
          $num_property++;
          next;
        }
        if ( $char eq "-" && !defined $hash_position{$num_property}{START} ) {
          $hash_position{$num_property}{START} = $index;
        }
      }

      $active++;
      next;

    }
    if ( $active == 3 ) {
      my $id         = "";
      my $post_time  = "";
      my $clean_time = "";
      my $severity   = "";
      my $class      = "";
      my $object     = "";
      my $message    = "";
      foreach my $index ( sort keys %hash_position ) {
        my $start = $hash_position{$index}{START};
        my $end   = $hash_position{$index}{END};
        my $name  = $hash_position{$index}{NAME};
        if ( Xormon::isdigit($start) && !Xormon::isdigit($end) ) {
          $end = length($line_parse);
        }
        if ( !Xormon::isdigit($start) || !Xormon::isdigit($end) || !defined $name ) { next; }
        my $length = ( $end - $start );
        $length++;
        my $string = substr( $line, $start, $length );
        $string = remove_whitespace($string);
        if ( defined $string && $string ne "" ) {
          if ( $name eq "Id" ) {
            $id = $string;
          }
          if ( $name eq "Post Time" ) {
            $post_time = $string;
          }
          if ( $name eq "Clear Time" ) {
            $clean_time = $string;
          }
          if ( $name eq "Severity" ) {
            $severity = $string;
          }
          if ( $name eq "Class" ) {
            $class = $string;
          }
          if ( $name eq "Object" ) {
            $object = $string;
          }
          if ( $name eq "Message" ) {
            $message = $string;
          }
        }
        if ( !defined $id || $id eq "" ) { next; }
        if (defined $post_time && $post_time ne "" && Xormon::isdigit(str2time($post_time))){
          $post_time = get_utc_time($post_time,1);
        }
        if (defined $clean_time && $clean_time eq "" && Xormon::isdigit(str2time($clean_time))){
          $clean_time = "";
          $clean_time = get_utc_time($clean_time,1);
        }
      }
      if ( !defined $severity ) {
        $severity = "";
      }
      if ( !defined $class ) {
        $class = "";
      }
      if ( !defined $object ) {
        $object = "";
      }
      if ( !defined $message ) {
        $message = "";
      }
      if ( !defined $post_time ) {
        $post_time = "";
      }
      if ( !defined $clean_time ) {
        $clean_time = "";
      }
      my %data;
      $data{id} = $id;
      $data{post_time} = $post_time;
      $data{clear_time} = $clean_time;
      $data{severity} = $severity;
      $data{class} = $class;
      $data{object} = $object;
      $data{message} = $message;
      push(@health_status,\%data);
      if ($severity eq "CRITICAL"){
        $hs_error = 1;
        $hs_reason = $message;
      }

    }
  }
  my $status = "ok";
  if ($hs_error){
    $status = "error";
  }
  my %vm_status = (
    "item_id" => $st_serial,
    "status" => $status,
    "data" => $hs_reason
  );
  #my @health_status_global = ();
  push(@health_status_global, \%vm_status);

  #push(@health_status_global, \%vm);
  #my $rc = $xormon->saveStatus(\@health_status_global);
  #my $status_job = get_status($rc, "health status");
  #$logger->info("Task healh status ".$cfg->{hostalias}." is done. Status: $status_job");
  return;

}

sub check_mtree_exclude {

  my $mtree_exclude = $ENV{MTREE_COMPRESSION_STATISTICS_EXCLUDE};
  if ( defined $mtree_exclude ) {
    my @storages = split( " ", $mtree_exclude );
    foreach my $storage (@storages) {
      if ( $storage eq $storage_name ) {
        return 0;
      }
    }
  }
  return 1;

}

sub set_data_ddboost_conf {

  my $section = "DDBOOST Level Configuration";
  my $section2 = "DDBOOST Level Statistics";
  foreach my $id (keys %{$inventory_data{$section2}}){
    if (!defined $inventory_data{$section2}{$id}{ID}){next;}
    $inventory_data{$section}{$id}{id} = $inventory_data{$section2}{$id}{ID};
    $inventory_data{$section}{$id}{name} = $inventory_data{$section2}{$id}{ID};
  }
  #print Dumper \$inventory_data{$section2};
  #print Dumper \$inventory_data{$section};
}

sub set_data_cpu_conf {

  my $section = "CPU-CORE Level Configuration";
  my $section2 = "CPU-CORE Level Statistics";
  foreach my $id (keys %{$inventory_data{$section2}}){
    if (!defined $inventory_data{$section2}{$id}{ID}){next;}
    $inventory_data{$section}{$id}{id} = $inventory_data{$section2}{$id}{ID};
    $inventory_data{$section}{$id}{name} = "total";
  }
  #print Dumper \$inventory_data{$section2};
  #print Dumper \$inventory_data{$section};
}

sub set_data_mtree_cap {
  my $section = shift;

  #my $section = "Mtree Level Statistics";
  my $file    = "$debug_data/mtree_cap.txt";
  if ( !-e "$file" || -z "$file" ) {
    return;
  }
  my @data       = get_array_data($file);
  my $active     = 0;
  my $mtree_name = "";
  my $lcomp      = "";
  my $gcomp      = "";
  my $original   = "";
  my $meta_data  = "";

  my $timestamp  = get_utc_time($date_start,1);
  if (!Xormon::isdigit($timestamp)){return;}
  foreach my $line (@data) {
    chomp $line;
    if ( $line eq "" || $line =~ m/No files matched/ ) { next; }
    $line = remove_whitespace($line);
    if ( $line =~ m/^-------/ ) {

      my $total = "";
      if ( Xormon::isdigit($lcomp) ) {
        $total = $lcomp;
      }
      if ( Xormon::isdigit($meta_data) ) {
        if ( Xormon::isdigit($total) ) {
          $total = $total + $meta_data;
        }
        else {
          $total = $meta_data;
        }
      }
      if ( defined $mtree_name && $mtree_name ne "" ) {
        if ($mtree_name =~ m/Error command:/){
          $active     = 0;
          $mtree_name = "";
          $lcomp      = "";
          $meta_data  = "";
          next;
        }
        if ( Xormon::isdigit($original) || Xormon::isdigit($lcomp) || Xormon::isdigit($gcomp) || Xormon::isdigit($meta_data) ) {
          $inventory_data{$section}{$mtree_name}{ID}   = $mtree_name;
          if (defined $inventory_data{$section}{$mtree_name}{TIME} ){
            if ((abs($inventory_data{$section}{$mtree_name}{TIME} - $timestamp) > 3600 * 24) && $section eq "Mtree Level Statistics"){
              next;
            }
          }
          elsif ($section eq "Mtree Level Statistics"){
            next;
          }
          #$inventory_data{$section}{$mtree_name}{TIME} = $timestamp;
        }
        if ( Xormon::isdigit($total) ) {
          $inventory_data{$section}{$mtree_name}{"capacity(B)"} = $total;
        }
        if ( Xormon::isdigit($original) ) {
          $inventory_data{$section}{$mtree_name}{"original(B)"} = $original;
        }
        if ( Xormon::isdigit($lcomp) ) {
          $inventory_data{$section}{$mtree_name}{"lcomp(B)"} = $lcomp;
        }
        if ( Xormon::isdigit($gcomp) ) {
          $inventory_data{$section}{$mtree_name}{"gcomp(B)"} = $gcomp;
        }
        if ( Xormon::isdigit($meta_data) ) {
          $inventory_data{$section}{$mtree_name}{"meta_data(B)"} = $meta_data;
        }
      }

      $active     = 0;
      $mtree_name = "";
      $lcomp      = "";
      $meta_data  = "";
      next;
    }
    if ( $line =~ m/data\/col1/ ) {
      $mtree_name = $line;
      $mtree_name =~ s/\/data\/col1\///g;
      $mtree_name =~ s/\&//g;
      $mtree_name = remove_whitespace($mtree_name);
      if ( defined $mtree_name && $mtree_name ne "" ) {
        $active++;
        next;
      }
    }
    if ( defined $mtree_name && $mtree_name ne "" && $active == 1 ) {
      if ( $line =~ /Locally Compressed:/ ) {
        ( undef, $lcomp ) = split( ":", $line );
        $lcomp =~ s/,//g;
        $lcomp = remove_whitespace($lcomp);
      }
      if ( $line =~ /Meta-data:/ ) {
        ( undef, $meta_data ) = split( ":", $line );
        $meta_data =~ s/,//g;
        $meta_data = remove_whitespace($meta_data);
      }
      if ( $line =~ /Original Bytes:/ ) {
        ( undef, $original ) = split( ":", $line );
        $original =~ s/,//g;
        $original = remove_whitespace($original);
      }
      if ( $line =~ /Globally Compressed:/ ) {
        ( undef, $gcomp ) = split( ":", $line );
        $gcomp =~ s/,//g;
        $gcomp = remove_whitespace($gcomp);
      }
    }
  }
}

sub set_data_convert{
  my %metrics = get_convert_metrics();
  #print Dumper \%inventory_data;
  #print Dumper \$inventory_data{"Drive Level Configuration"};
  foreach my $section (sort keys %inventory_data){
    #print "$section\n";
    #if (!defined $inventory_data{$section}{TIME}{END}){next;}
    #my $time = $inventory_data{$section}{TIME}{END};
    foreach my $element (keys %{$inventory_data{$section}}){
      if ($element eq "TIME"){next;}
      my $element_new = $element;
      my ($type) = split(" ", $section);
      $type = lc($type);
      if ($section eq "CPU-CORE Level Statistics" || $section eq "CPU-CORE Level Configuration"){
        $type = "cpu";
      }
      if ($element eq "ALL"){
        $element_new = 0;
      }
      if ($type eq "pool"){
        $element_new = "array_total";
      }
      if ($type eq "tier"){
        if ($element ne "/data: post-comp"){next;}
        $element_new = 0;
      }
      if ($type =~ m/port/ && $section =~ m/port Level Statistics/ && defined $inventory_data{$section}{$element}{ID}){
        $element_new = $inventory_data{$section}{$element}{ID};
      }
      #print "$section $type $element_new\n";
      my $uid = Xormon::create_hash_key($st_serial.$type.$element_new);
      foreach my $metric (keys %{$inventory_data{$section}{$element}}){
        my $time = $inventory_data{$section}{$element}{TIME};
        if (!defined $time && defined $inventory_data{$section}{TIME}{END}){
          $time = $inventory_data{$section}{TIME}{END};
        }
        if (!Xormon::isdigit($time) && $section =~ m/Level Statistics$/){last;}
        if ($section =~ m/Level Configuration$/ && defined $metrics{$metric} && defined $inventory_data{$section}{$element}{$metric}){
          #print "$section | $type | $metric\n";
          my $value = $inventory_data{$section}{$element}{$metric};
          if (Xormon::isdigit($value) && $metric ne "name"){
            #if ( $metrics{$metric} =~ m/^data_rate$|^read$|^write$/){
            #  $value = $value * 1000;
            #}
            if ( $metric =~ m/^resp_t/){
              $value = round($value,3);
            }
            if ( $metric eq "Capacity(MB)"){
              $value = round($value * (1024 * 1024),0);
            }
            if ( $metric =~ m/\(TB\)/){
              $value = round($value * (1024 * 1024 * 1024 * 1024),0);
            }
            if ( $metric =~ m/\(GB\)|GiB/){
              $value = round($value * (1024 * 1024 * 1024),0);
            }
            #print "$uid $metric $value\n";
            $configuration_data{$uid}{$metrics{$metric}} = $value;
            if ($type eq "tier" && $metrics{$metric} =~ m/reduction|precomp_capacity_total|_factor|cleanable_capacity_total|postcomp_capacity_total/ ){
              my $pool_id = Xormon::create_hash_key($st_serial."pool"."array_total");
              $configuration_data{$pool_id}{$metrics{$metric}} = $value;
            }
            next;
          }
          if ($inventory_data{$section}{$element}{$metric} eq "-"){$inventory_data{$section}{$element}{$metric} = "";}
          $configuration_data{$uid}{$metrics{$metric}} = $inventory_data{$section}{$element}{$metric};
          if ($section eq "Tier Level Configuration" && $metric eq "name"){
            $configuration_data{$uid}{$metrics{$metric}} = "tier";
          }
          if (!defined $configuration_data{$uid}{TYPE}){
            $configuration_data{$uid}{TYPE} = $type;
          }
          
          if ($metric eq "name" && defined $configuration_data{$uid}{$metrics{$metric}}){
            $configuration_data{$uid}{"label"} = $configuration_data{$uid}{$metrics{$metric}};
          }
          next;
        }
        else{
        }
        if (defined $metrics{$metric} && Xormon::isdigit($inventory_data{$section}{$element}{$metric})){
          my $value = $inventory_data{$section}{$element}{$metric};
          if ( $metric =~ m/^resp_t/){
            $value = round($value ,3);
          }
          if ( $metric eq "Capacity(MB)"){
            $value = round($value * (1024 * 1024),0);
          }
          if ( $metric =~ m/\(TB\)/){
            $value = round($value * (1024 * 1024 * 1024 * 1024),0);
          }
          if ( $metric =~ m/\(GB\)/){
            $value = round($value * (1024 * 1024 * 1024),0);
          }
          $performance_data{$st_type}{$type}{$uid}{$time}{$metrics{$metric}} = $value;
          if ($type eq "tier" && $metrics{$metric} =~ m/reduction|precomp_capacity_total|_factor|cleanable_capacity_total|postcomp_capacity_total/ ){
            my $pool_id = (keys %{$performance_data{$st_type}{pool}})[0];
            $time = (keys %{$performance_data{$st_type}{pool}{$pool_id}})[0];
            if (Xormon::isdigit($time)){
              $performance_data{$st_type}{pool}{$pool_id}{$time}{$metrics{$metric}} = $value;
            }
          }
        }
      }
    }
  }
  #print Dumper \%configuration_data;
  #print Dumper \%performance_data;
}



sub get_convert_metrics {
  my %data;
  $data{read_io} = "read_io";
  $data{write_io} = "write_io";
  $data{io_rate} = "io_rate";
  $data{data_rate} = "data_rate";
  $data{read} = "read";
  $data{write} = "write";
  $data{resp_t} = "resp_t";
  $data{resp_t_r} = "resp_t_r";
  $data{resp_t_w} = "resp_t_w";
  $data{busy} = "cpu_busy";
  $data{"Capacity(MB)"} = "capacity_total";
  $data{"capacity_total"} = "capacity_total";
  $data{sys} = "cpu_system";
  $data{net_rate} = "net_rate";
  $data{read_net} = "read_net";
  $data{write_net} = "write_net";
  $data{"Outbound Pre-comp(KB/s)"} = "outbound_pre_comp";
  $data{"Inbound Pre-comp(KB/s)"} = "inbound_pre_comp";
  $data{"Outbound Network(KB/s)"} = "outbound_net";
  $data{"Inbound Network(KB/s)"} = "inbound_net";
  $data{"original(B)"} = "original_capacity_total";
  $data{"gcomp(B)"} = "gcomp_capacity_total";
  $data{"lcomp(B)"} = "lcomp_capacity_total";
  $data{"meta_data(B)"} = "metadata_capacity_total";
  $data{"Capacity(B)"} = "capacity_total";
  $data{"capacity(B)"} = "capacity_total";
  $data{"repl_read"} = "read_replication";
  $data{"repl_write"} = "write_replication";
  $data{"repl_rate"} = "replication_rate";
  $data{"Capacity(TB)"} = "capacity_total";
  $data{"Used(TB)"} = "capacity_used";
  $data{"Free(TB)"} = "capacity_free";
  $data{"Size(TB)"} = "capacity_total";
  $data{"ops_rate"} = "ops_rate";
  $data{"read_ops"} = "read_ops";
  $data{"write_ops"} = "write_ops";
  $data{id} = "id";
  $data{name} = "name";
  $data{children} = "children";
  $data{"free_capacity (GB)"} = "capacity_free";
  $data{"used_capacity (GB)"} = "capacity_used";
  $data{"capacity (GB)"} = "capacity_total";
  $data{"Size(TB)"} = "capacity_total";
  $data{"Used(TB)"} = "capacity_used";
  $data{"Free(TB)"} = "capacity_free";
  $data{"Cleanable(TB)"} = "cleanable_capacity_total";
  $data{"Pre-comp(TB)"} = "precomp_capacity_total";
  $data{"Post-comp(TB)"} = "postcomp_capacity_total";
  $data{"Global-comp Factor"} = "global_comp_factor";
  $data{"Local-comp Factor"} = "local_comp_factor";
  $data{"Total Compression Factor"} = "total_comp_factor";
  $data{"Reduction"} = "reduction";
  $data{"Used(GiB)"} = "capacity_used";
  $data{"Free(GiB)"} = "capacity_free";
  $data{"Size(GiB)"} = "capacity_total";
  $data{"Cleanable(GiB)"} = "cleanable_capacity_total";
  $data{"Pre-Comp(B)"} = "precomp_capacity_total";
  $data{"Post-Comp(B)"} = "postcomp_capacity_total";
  $data{"Pre-Compression(GiB)"} = "precomp_capacity_total";
  $data{"Post-Compression(GiB)"} = "postcomp_capacity_total";
  $data{"Capacity(GB)"} = "capacity_total";
  $data{"Protocol"} = "protocol";
  $data{"type"} = "type";
  $data{"connection_type"} = "connection_type";
  $data{"speed"} = "speed";
  $data{"firmware"} = "firmware";
  $data{"MAC"} = "mac";
  $data{phase} = "phase";
  $data{capacity_total} = "capacity_total";
  $data{capacity_used} = "capacity_used";
  $data{capacity_free} = "capacity_free";
  $data{"cleanable_capacity_total"} = "cleanable_capacity_total";
  $data{"precomp_capacity_total"} = "precomp_capacity_total";
  $data{"postcomp_capacity_total"} = "postcomp_capacity_total";




  #Size(TB),Used(TB),Free(TB),Cleanable(TB),Pre-comp(TB),Post-comp(TB)
  ### drop in/out for ports does not save

  return %data;

}

sub get_status {
  my $rc = shift;
  my $task = shift;
  if ($rc eq "201" || $rc eq "200") {
    $logger->info("Task for $task ".$cfg->{hostalias}." is done. Status: successful");
    return "successful";
  } elsif ($rc eq "401") {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: unauthorized");
    return "unauthorized";
  } else {
    $logger->error("Task for $task ".$cfg->{hostalias}." is done. Status: error");
    return "error";
  }
}

sub set_data_port_type{
  my $section = "Port Level Statistics";
  foreach my $port (keys %{$inventory_data{$section}}){
    my $port_type = $inventory_data{$section}{$port}{protocol};
    if (!defined $port_type){next;}
    $port_type = lc($port_type) . "port Level Statistics";
    foreach my $metric (keys %{$inventory_data{$section}{$port}}){
      $inventory_data{$port_type}{$port}{$metric} = $inventory_data{$section}{$port}{$metric};
    }
  }
  delete $inventory_data{$section};
  #print Dumper \%inventory_data;
}

sub set_data_port_type_conf{
  my $section = "Port Level Configuration";
  #print Dumper \$inventory_data{$section};
  foreach my $port (keys %{$inventory_data{$section}}){
    my $protocol = $inventory_data{$section}{$port}{Protocol};
    if (!defined $protocol ){next;}
    $protocol = lc($protocol) . "port Level Configuration";
    foreach my $metric (keys %{$inventory_data{$section}{$port}}){
      $inventory_data{$protocol}{$port}{$metric} = $inventory_data{$section}{$port}{$metric};
    }
  }
  delete $inventory_data{$section};
  #print Dumper \%inventory_data;
}



1;
